// Copyright 2017 The Memory Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

//go:build !memory.counters
// +build !memory.counters

package memory // import "modernc.org/memory"

const counters = false
