/*
 * Copyright (c) SAS Institute, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package rpmutils

import "fmt"

// NoSuchTagError is returned when a tag does not exist in the header
type NoSuchTagError struct {
	Tag int
}

func (err NoSuchTagError) Error() string {
	return fmt.Sprintf("No such entry %d", err.Tag)
}

// NewNoSuchTagError creates a NoSuchTagError for a given tag
func NewNoSuchTagError(tag int) NoSuchTagError {
	return NoSuchTagError{Tag: tag}
}
