// DO NOT EDIT: generated by internal/packagemetadata/generate/main.go

package packagemetadata

import "github.com/anchore/syft/syft/pkg"

// AllTypes returns a list of all pkg metadata types that syft supports (that are represented in the pkg.Package.Metadata field).
func AllTypes() []any {
	return []any{
		pkg.AlpmDBEntry{},
		pkg.ApkDBEntry{},
		pkg.BinarySignature{},
		pkg.BitnamiSBOMEntry{},
		pkg.CocoaPodfileLockEntry{},
		pkg.ConanV1LockEntry{},
		pkg.ConanV2LockEntry{},
		pkg.ConanfileEntry{},
		pkg.ConaninfoEntry{},
		pkg.CondaMetaPackage{},
		pkg.DartPubspec{},
		pkg.DartPubspecLockEntry{},
		pkg.DotnetDepsEntry{},
		pkg.DotnetPackagesLockEntry{},
		pkg.DotnetPortableExecutableEntry{},
		pkg.DpkgArchiveEntry{},
		pkg.DpkgDBEntry{},
		pkg.ELFBinaryPackageNoteJSONPayload{},
		pkg.ElixirMixLockEntry{},
		pkg.ErlangRebarLockEntry{},
		pkg.GGUFFileHeader{},
		pkg.GitHubActionsUseStatement{},
		pkg.GolangBinaryBuildinfoEntry{},
		pkg.GolangModuleEntry{},
		pkg.GolangSourceEntry{},
		pkg.HackageStackYamlEntry{},
		pkg.HackageStackYamlLockEntry{},
		pkg.HomebrewFormula{},
		pkg.JavaArchive{},
		pkg.JavaVMInstallation{},
		pkg.LinuxKernel{},
		pkg.LinuxKernelModule{},
		pkg.LuaRocksPackage{},
		pkg.MicrosoftKbPatch{},
		pkg.NixStoreEntry{},
		pkg.NpmPackage{},
		pkg.NpmPackageLockEntry{},
		pkg.OpamPackage{},
		pkg.PEBinary{},
		pkg.PhpComposerInstalledEntry{},
		pkg.PhpComposerLockEntry{},
		pkg.PhpPearEntry{},
		pkg.PhpPeclEntry{},
		pkg.PnpmLockEntry{},
		pkg.PortageEntry{},
		pkg.PythonPackage{},
		pkg.PythonPdmLockEntry{},
		pkg.PythonPipfileLockEntry{},
		pkg.PythonPoetryLockEntry{},
		pkg.PythonRequirementsEntry{},
		pkg.PythonUvLockEntry{},
		pkg.RDescription{},
		pkg.RpmArchive{},
		pkg.RpmDBEntry{},
		pkg.RubyGemspec{},
		pkg.RustBinaryAuditEntry{},
		pkg.RustCargoLockEntry{},
		pkg.SnapEntry{},
		pkg.SwiftPackageManagerResolvedEntry{},
		pkg.SwiplPackEntry{},
		pkg.TerraformLockProviderEntry{},
		pkg.WordpressPluginEntry{},
		pkg.YarnLockEntry{},
	}
}
