// Copyright 2020 The Libc Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

//go:build !libc.nofsync
// +build !libc.nofsync

package libc // import "modernc.org/libc"

const noFsync = false
