// Copyright The OpenTelemetry Authors
// SPDX-License-Identifier: Apache-2.0

package metric // import "go.opentelemetry.io/otel/sdk/metric"

// version is the current release version of the metric SDK in use.
func version() string {
	return "1.37.0"
}
