package options

// Experimental options are opt-in features that are...
// ...not stable
// ...not yet fully supported
// ...not necessarily tested
// ...not ready for production use
// these may go away at any moment, do not depend on them
type Experimental struct {
}
