package spdxlicenses

// Code generated by go-spdx cmd/exceptions.go. DO NOT EDIT.
// Source: https://github.com/spdx/license-list-data specifies official SPDX license list.

// GetExceptions returns a slice of exception license IDs.
func GetExceptions() []string {
	return []string{
		"389-exception",
		"Asterisk-exception",
		"Asterisk-linking-protocols-exception",
		"Autoconf-exception-2.0",
		"Autoconf-exception-3.0",
		"Autoconf-exception-generic",
		"Autoconf-exception-generic-3.0",
		"Autoconf-exception-macro",
		"Bison-exception-1.24",
		"Bison-exception-2.2",
		"Bootloader-exception",
		"CGAL-linking-exception",
		"Classpath-exception-2.0",
		"Classpath-exception-2.0-short",
		"CLISP-exception-2.0",
		"cryptsetup-OpenSSL-exception",
		"Digia-Qt-LGPL-exception-1.1",
		"DigiRule-FOSS-exception",
		"eCos-exception-2.0",
		"erlang-otp-linking-exception",
		"Fawkes-Runtime-exception",
		"FLTK-exception",
		"fmt-exception",
		"Font-exception-2.0",
		"freertos-exception-2.0",
		"GCC-exception-2.0",
		"GCC-exception-2.0-note",
		"GCC-exception-3.1",
		"Gmsh-exception",
		"GNAT-exception",
		"GNOME-examples-exception",
		"GNU-compiler-exception",
		"gnu-javamail-exception",
		"GPL-3.0-389-ds-base-exception",
		"GPL-3.0-interface-exception",
		"GPL-3.0-linking-exception",
		"GPL-3.0-linking-source-exception",
		"GPL-CC-1.0",
		"GStreamer-exception-2005",
		"GStreamer-exception-2008",
		"harbour-exception",
		"i2p-gpl-java-exception",
		"Independent-modules-exception",
		"KiCad-libraries-exception",
		"kvirc-openssl-exception",
		"LGPL-3.0-linking-exception",
		"libpri-OpenH323-exception",
		"Libtool-exception",
		"Linux-syscall-note",
		"LLGPL",
		"LLVM-exception",
		"LZMA-exception",
		"mif-exception",
		"mxml-exception",
		"OCaml-LGPL-linking-exception",
		"OCCT-exception-1.0",
		"OpenJDK-assembly-exception-1.0",
		"openvpn-openssl-exception",
		"PCRE2-exception",
		"polyparse-exception",
		"PS-or-PDF-font-exception-20170817",
		"QPL-1.0-INRIA-2004-exception",
		"Qt-GPL-exception-1.0",
		"Qt-LGPL-exception-1.1",
		"Qwt-exception-1.0",
		"romic-exception",
		"RRDtool-FLOSS-exception-2.0",
		"SANE-exception",
		"SHL-2.0",
		"SHL-2.1",
		"Simple-Library-Usage-exception",
		"stunnel-exception",
		"SWI-exception",
		"Swift-exception",
		"Texinfo-exception",
		"u-boot-exception-2.0",
		"UBDL-exception",
		"Universal-FOSS-exception-1.0",
		"vsftpd-openssl-exception",
		"WxWindows-exception-3.1",
		"x11vnc-openssl-exception",
	}
}
