// SPDX-License-Identifier: Apache-2.0
// Copyright Authors of K9s

package view

import (
	"context"
	"log/slog"

	"github.com/derailed/k9s/internal"
	"github.com/derailed/k9s/internal/client"
	"github.com/derailed/k9s/internal/config/data"
	"github.com/derailed/k9s/internal/slogs"
	"github.com/derailed/k9s/internal/ui"
	"github.com/derailed/tcell/v2"
)

// ScreenDump presents a directory listing viewer.
type ScreenDump struct {
	ResourceViewer
}

// NewScreenDump returns a new viewer.
func NewScreenDump(gvr *client.GVR) ResourceViewer {
	s := ScreenDump{
		ResourceViewer: NewBrowser(gvr),
	}
	s.GetTable().SetBorderFocusColor(tcell.ColorSteelBlue)
	s.GetTable().SetSelectedStyle(tcell.StyleDefault.Foreground(tcell.ColorWhite).Background(tcell.ColorRoyalBlue).Attributes(tcell.AttrNone))
	s.GetTable().SetSortCol(ageCol, true)
	s.GetTable().SelectRow(1, 0, true)
	s.GetTable().SetEnterFn(s.edit)
	s.SetContextFn(s.dirContext)

	return &s
}

func (s *ScreenDump) dirContext(ctx context.Context) context.Context {
	dir := s.App().Config.K9s.ContextScreenDumpDir()
	if err := data.EnsureFullPath(dir, data.DefaultDirMod); err != nil {
		s.App().Flash().Err(err)
		return ctx
	}

	return context.WithValue(ctx, internal.KeyDir, dir)
}

func (s *ScreenDump) edit(app *App, _ ui.Tabular, _ *client.GVR, path string) {
	slog.Debug("ScreenDump selection", slogs.FQN, path)

	s.Stop()
	defer s.Start()
	if !edit(app, &shellOpts{clear: true, args: []string{path}}) {
		app.Flash().Errf("Failed to launch editor")
	}
}
