// Code generated by "stringer -linecomment -type GGUFVersion -output zz_generated.ggufversion.stringer.go -trimprefix GGUFVersion"; DO NOT EDIT.

package gguf_parser

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[GGUFVersionV1-1]
	_ = x[GGUFVersionV2-2]
	_ = x[GGUFVersionV3-3]
}

const _GGUFVersion_name = "V1V2V3"

var _GGUFVersion_index = [...]uint8{0, 2, 4, 6}

func (i GGUFVersion) String() string {
	i -= 1
	if i >= GGUFVersion(len(_GGUFVersion_index)-1) {
		return "GGUFVersion(" + strconv.FormatInt(int64(i+1), 10) + ")"
	}
	return _GGUFVersion_name[_GGUFVersion_index[i]:_GGUFVersion_index[i+1]]
}
