// Code generated by smithy-go-codegen DO NOT EDIT.

package types

type AccessDeniedExceptionReason string

// Enum values for AccessDeniedExceptionReason
const (
	AccessDeniedExceptionReasonKmsAccessDenied AccessDeniedExceptionReason = "KMS_AccessDeniedException"
)

// Values returns all known values for AccessDeniedExceptionReason. Note that this
// can be expanded in the future, and so it is only as up to date as the client.
//
// The ordering of this slice is not guaranteed to be stable across updates.
func (AccessDeniedExceptionReason) Values() []AccessDeniedExceptionReason {
	return []AccessDeniedExceptionReason{
		"KMS_AccessDeniedException",
	}
}

type InvalidRequestExceptionReason string

// Enum values for InvalidRequestExceptionReason
const (
	InvalidRequestExceptionReasonKmsKeyNotFound     InvalidRequestExceptionReason = "KMS_NotFoundException"
	InvalidRequestExceptionReasonKmsInvalidKeyUsage InvalidRequestExceptionReason = "KMS_InvalidKeyUsageException"
	InvalidRequestExceptionReasonKmsInvalidState    InvalidRequestExceptionReason = "KMS_InvalidStateException"
	InvalidRequestExceptionReasonKmsDisabledKey     InvalidRequestExceptionReason = "KMS_DisabledException"
)

// Values returns all known values for InvalidRequestExceptionReason. Note that
// this can be expanded in the future, and so it is only as up to date as the
// client.
//
// The ordering of this slice is not guaranteed to be stable across updates.
func (InvalidRequestExceptionReason) Values() []InvalidRequestExceptionReason {
	return []InvalidRequestExceptionReason{
		"KMS_NotFoundException",
		"KMS_InvalidKeyUsageException",
		"KMS_InvalidStateException",
		"KMS_DisabledException",
	}
}
