// Generated by running
//      maketables --url=http://www.unicode.org/Public/15.0.0/ucd/auxiliary/
// DO NOT EDIT

package textseg

import (
	"unicode"
)

var _GraphemeCR = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0xd, Hi: 0xd, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _GraphemeControl = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x0, Hi: 0x9, Stride: 0x1},
		unicode.Range16{Lo: 0xb, Hi: 0xc, Stride: 0x1},
		unicode.Range16{Lo: 0xe, Hi: 0x1f, Stride: 0x1},
		unicode.Range16{Lo: 0x7f, Hi: 0x9f, Stride: 0x1},
		unicode.Range16{Lo: 0xad, Hi: 0xad, Stride: 0x1},
		unicode.Range16{Lo: 0x61c, Hi: 0x61c, Stride: 0x1},
		unicode.Range16{Lo: 0x180e, Hi: 0x180e, Stride: 0x1},
		unicode.Range16{Lo: 0x200b, Hi: 0x200b, Stride: 0x1},
		unicode.Range16{Lo: 0x200e, Hi: 0x200f, Stride: 0x1},
		unicode.Range16{Lo: 0x2028, Hi: 0x2028, Stride: 0x1},
		unicode.Range16{Lo: 0x2029, Hi: 0x2029, Stride: 0x1},
		unicode.Range16{Lo: 0x202a, Hi: 0x202e, Stride: 0x1},
		unicode.Range16{Lo: 0x2060, Hi: 0x2064, Stride: 0x1},
		unicode.Range16{Lo: 0x2065, Hi: 0x2065, Stride: 0x1},
		unicode.Range16{Lo: 0x2066, Hi: 0x206f, Stride: 0x1},
		unicode.Range16{Lo: 0xfeff, Hi: 0xfeff, Stride: 0x1},
		unicode.Range16{Lo: 0xfff0, Hi: 0xfff8, Stride: 0x1},
		unicode.Range16{Lo: 0xfff9, Hi: 0xfffb, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x13430, Hi: 0x1343f, Stride: 0x1},
		unicode.Range32{Lo: 0x1bca0, Hi: 0x1bca3, Stride: 0x1},
		unicode.Range32{Lo: 0x1d173, Hi: 0x1d17a, Stride: 0x1},
		unicode.Range32{Lo: 0xe0000, Hi: 0xe0000, Stride: 0x1},
		unicode.Range32{Lo: 0xe0001, Hi: 0xe0001, Stride: 0x1},
		unicode.Range32{Lo: 0xe0002, Hi: 0xe001f, Stride: 0x1},
		unicode.Range32{Lo: 0xe0080, Hi: 0xe00ff, Stride: 0x1},
		unicode.Range32{Lo: 0xe01f0, Hi: 0xe0fff, Stride: 0x1},
	},
	LatinOffset: 5,
}

var _GraphemeExtend = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x300, Hi: 0x36f, Stride: 0x1},
		unicode.Range16{Lo: 0x483, Hi: 0x487, Stride: 0x1},
		unicode.Range16{Lo: 0x488, Hi: 0x489, Stride: 0x1},
		unicode.Range16{Lo: 0x591, Hi: 0x5bd, Stride: 0x1},
		unicode.Range16{Lo: 0x5bf, Hi: 0x5bf, Stride: 0x1},
		unicode.Range16{Lo: 0x5c1, Hi: 0x5c2, Stride: 0x1},
		unicode.Range16{Lo: 0x5c4, Hi: 0x5c5, Stride: 0x1},
		unicode.Range16{Lo: 0x5c7, Hi: 0x5c7, Stride: 0x1},
		unicode.Range16{Lo: 0x610, Hi: 0x61a, Stride: 0x1},
		unicode.Range16{Lo: 0x64b, Hi: 0x65f, Stride: 0x1},
		unicode.Range16{Lo: 0x670, Hi: 0x670, Stride: 0x1},
		unicode.Range16{Lo: 0x6d6, Hi: 0x6dc, Stride: 0x1},
		unicode.Range16{Lo: 0x6df, Hi: 0x6e4, Stride: 0x1},
		unicode.Range16{Lo: 0x6e7, Hi: 0x6e8, Stride: 0x1},
		unicode.Range16{Lo: 0x6ea, Hi: 0x6ed, Stride: 0x1},
		unicode.Range16{Lo: 0x711, Hi: 0x711, Stride: 0x1},
		unicode.Range16{Lo: 0x730, Hi: 0x74a, Stride: 0x1},
		unicode.Range16{Lo: 0x7a6, Hi: 0x7b0, Stride: 0x1},
		unicode.Range16{Lo: 0x7eb, Hi: 0x7f3, Stride: 0x1},
		unicode.Range16{Lo: 0x7fd, Hi: 0x7fd, Stride: 0x1},
		unicode.Range16{Lo: 0x816, Hi: 0x819, Stride: 0x1},
		unicode.Range16{Lo: 0x81b, Hi: 0x823, Stride: 0x1},
		unicode.Range16{Lo: 0x825, Hi: 0x827, Stride: 0x1},
		unicode.Range16{Lo: 0x829, Hi: 0x82d, Stride: 0x1},
		unicode.Range16{Lo: 0x859, Hi: 0x85b, Stride: 0x1},
		unicode.Range16{Lo: 0x898, Hi: 0x89f, Stride: 0x1},
		unicode.Range16{Lo: 0x8ca, Hi: 0x8e1, Stride: 0x1},
		unicode.Range16{Lo: 0x8e3, Hi: 0x902, Stride: 0x1},
		unicode.Range16{Lo: 0x93a, Hi: 0x93a, Stride: 0x1},
		unicode.Range16{Lo: 0x93c, Hi: 0x93c, Stride: 0x1},
		unicode.Range16{Lo: 0x941, Hi: 0x948, Stride: 0x1},
		unicode.Range16{Lo: 0x94d, Hi: 0x94d, Stride: 0x1},
		unicode.Range16{Lo: 0x951, Hi: 0x957, Stride: 0x1},
		unicode.Range16{Lo: 0x962, Hi: 0x963, Stride: 0x1},
		unicode.Range16{Lo: 0x981, Hi: 0x981, Stride: 0x1},
		unicode.Range16{Lo: 0x9bc, Hi: 0x9bc, Stride: 0x1},
		unicode.Range16{Lo: 0x9be, Hi: 0x9be, Stride: 0x1},
		unicode.Range16{Lo: 0x9c1, Hi: 0x9c4, Stride: 0x1},
		unicode.Range16{Lo: 0x9cd, Hi: 0x9cd, Stride: 0x1},
		unicode.Range16{Lo: 0x9d7, Hi: 0x9d7, Stride: 0x1},
		unicode.Range16{Lo: 0x9e2, Hi: 0x9e3, Stride: 0x1},
		unicode.Range16{Lo: 0x9fe, Hi: 0x9fe, Stride: 0x1},
		unicode.Range16{Lo: 0xa01, Hi: 0xa02, Stride: 0x1},
		unicode.Range16{Lo: 0xa3c, Hi: 0xa3c, Stride: 0x1},
		unicode.Range16{Lo: 0xa41, Hi: 0xa42, Stride: 0x1},
		unicode.Range16{Lo: 0xa47, Hi: 0xa48, Stride: 0x1},
		unicode.Range16{Lo: 0xa4b, Hi: 0xa4d, Stride: 0x1},
		unicode.Range16{Lo: 0xa51, Hi: 0xa51, Stride: 0x1},
		unicode.Range16{Lo: 0xa70, Hi: 0xa71, Stride: 0x1},
		unicode.Range16{Lo: 0xa75, Hi: 0xa75, Stride: 0x1},
		unicode.Range16{Lo: 0xa81, Hi: 0xa82, Stride: 0x1},
		unicode.Range16{Lo: 0xabc, Hi: 0xabc, Stride: 0x1},
		unicode.Range16{Lo: 0xac1, Hi: 0xac5, Stride: 0x1},
		unicode.Range16{Lo: 0xac7, Hi: 0xac8, Stride: 0x1},
		unicode.Range16{Lo: 0xacd, Hi: 0xacd, Stride: 0x1},
		unicode.Range16{Lo: 0xae2, Hi: 0xae3, Stride: 0x1},
		unicode.Range16{Lo: 0xafa, Hi: 0xaff, Stride: 0x1},
		unicode.Range16{Lo: 0xb01, Hi: 0xb01, Stride: 0x1},
		unicode.Range16{Lo: 0xb3c, Hi: 0xb3c, Stride: 0x1},
		unicode.Range16{Lo: 0xb3e, Hi: 0xb3e, Stride: 0x1},
		unicode.Range16{Lo: 0xb3f, Hi: 0xb3f, Stride: 0x1},
		unicode.Range16{Lo: 0xb41, Hi: 0xb44, Stride: 0x1},
		unicode.Range16{Lo: 0xb4d, Hi: 0xb4d, Stride: 0x1},
		unicode.Range16{Lo: 0xb55, Hi: 0xb56, Stride: 0x1},
		unicode.Range16{Lo: 0xb57, Hi: 0xb57, Stride: 0x1},
		unicode.Range16{Lo: 0xb62, Hi: 0xb63, Stride: 0x1},
		unicode.Range16{Lo: 0xb82, Hi: 0xb82, Stride: 0x1},
		unicode.Range16{Lo: 0xbbe, Hi: 0xbbe, Stride: 0x1},
		unicode.Range16{Lo: 0xbc0, Hi: 0xbc0, Stride: 0x1},
		unicode.Range16{Lo: 0xbcd, Hi: 0xbcd, Stride: 0x1},
		unicode.Range16{Lo: 0xbd7, Hi: 0xbd7, Stride: 0x1},
		unicode.Range16{Lo: 0xc00, Hi: 0xc00, Stride: 0x1},
		unicode.Range16{Lo: 0xc04, Hi: 0xc04, Stride: 0x1},
		unicode.Range16{Lo: 0xc3c, Hi: 0xc3c, Stride: 0x1},
		unicode.Range16{Lo: 0xc3e, Hi: 0xc40, Stride: 0x1},
		unicode.Range16{Lo: 0xc46, Hi: 0xc48, Stride: 0x1},
		unicode.Range16{Lo: 0xc4a, Hi: 0xc4d, Stride: 0x1},
		unicode.Range16{Lo: 0xc55, Hi: 0xc56, Stride: 0x1},
		unicode.Range16{Lo: 0xc62, Hi: 0xc63, Stride: 0x1},
		unicode.Range16{Lo: 0xc81, Hi: 0xc81, Stride: 0x1},
		unicode.Range16{Lo: 0xcbc, Hi: 0xcbc, Stride: 0x1},
		unicode.Range16{Lo: 0xcbf, Hi: 0xcbf, Stride: 0x1},
		unicode.Range16{Lo: 0xcc2, Hi: 0xcc2, Stride: 0x1},
		unicode.Range16{Lo: 0xcc6, Hi: 0xcc6, Stride: 0x1},
		unicode.Range16{Lo: 0xccc, Hi: 0xccd, Stride: 0x1},
		unicode.Range16{Lo: 0xcd5, Hi: 0xcd6, Stride: 0x1},
		unicode.Range16{Lo: 0xce2, Hi: 0xce3, Stride: 0x1},
		unicode.Range16{Lo: 0xd00, Hi: 0xd01, Stride: 0x1},
		unicode.Range16{Lo: 0xd3b, Hi: 0xd3c, Stride: 0x1},
		unicode.Range16{Lo: 0xd3e, Hi: 0xd3e, Stride: 0x1},
		unicode.Range16{Lo: 0xd41, Hi: 0xd44, Stride: 0x1},
		unicode.Range16{Lo: 0xd4d, Hi: 0xd4d, Stride: 0x1},
		unicode.Range16{Lo: 0xd57, Hi: 0xd57, Stride: 0x1},
		unicode.Range16{Lo: 0xd62, Hi: 0xd63, Stride: 0x1},
		unicode.Range16{Lo: 0xd81, Hi: 0xd81, Stride: 0x1},
		unicode.Range16{Lo: 0xdca, Hi: 0xdca, Stride: 0x1},
		unicode.Range16{Lo: 0xdcf, Hi: 0xdcf, Stride: 0x1},
		unicode.Range16{Lo: 0xdd2, Hi: 0xdd4, Stride: 0x1},
		unicode.Range16{Lo: 0xdd6, Hi: 0xdd6, Stride: 0x1},
		unicode.Range16{Lo: 0xddf, Hi: 0xddf, Stride: 0x1},
		unicode.Range16{Lo: 0xe31, Hi: 0xe31, Stride: 0x1},
		unicode.Range16{Lo: 0xe34, Hi: 0xe3a, Stride: 0x1},
		unicode.Range16{Lo: 0xe47, Hi: 0xe4e, Stride: 0x1},
		unicode.Range16{Lo: 0xeb1, Hi: 0xeb1, Stride: 0x1},
		unicode.Range16{Lo: 0xeb4, Hi: 0xebc, Stride: 0x1},
		unicode.Range16{Lo: 0xec8, Hi: 0xece, Stride: 0x1},
		unicode.Range16{Lo: 0xf18, Hi: 0xf19, Stride: 0x1},
		unicode.Range16{Lo: 0xf35, Hi: 0xf35, Stride: 0x1},
		unicode.Range16{Lo: 0xf37, Hi: 0xf37, Stride: 0x1},
		unicode.Range16{Lo: 0xf39, Hi: 0xf39, Stride: 0x1},
		unicode.Range16{Lo: 0xf71, Hi: 0xf7e, Stride: 0x1},
		unicode.Range16{Lo: 0xf80, Hi: 0xf84, Stride: 0x1},
		unicode.Range16{Lo: 0xf86, Hi: 0xf87, Stride: 0x1},
		unicode.Range16{Lo: 0xf8d, Hi: 0xf97, Stride: 0x1},
		unicode.Range16{Lo: 0xf99, Hi: 0xfbc, Stride: 0x1},
		unicode.Range16{Lo: 0xfc6, Hi: 0xfc6, Stride: 0x1},
		unicode.Range16{Lo: 0x102d, Hi: 0x1030, Stride: 0x1},
		unicode.Range16{Lo: 0x1032, Hi: 0x1037, Stride: 0x1},
		unicode.Range16{Lo: 0x1039, Hi: 0x103a, Stride: 0x1},
		unicode.Range16{Lo: 0x103d, Hi: 0x103e, Stride: 0x1},
		unicode.Range16{Lo: 0x1058, Hi: 0x1059, Stride: 0x1},
		unicode.Range16{Lo: 0x105e, Hi: 0x1060, Stride: 0x1},
		unicode.Range16{Lo: 0x1071, Hi: 0x1074, Stride: 0x1},
		unicode.Range16{Lo: 0x1082, Hi: 0x1082, Stride: 0x1},
		unicode.Range16{Lo: 0x1085, Hi: 0x1086, Stride: 0x1},
		unicode.Range16{Lo: 0x108d, Hi: 0x108d, Stride: 0x1},
		unicode.Range16{Lo: 0x109d, Hi: 0x109d, Stride: 0x1},
		unicode.Range16{Lo: 0x135d, Hi: 0x135f, Stride: 0x1},
		unicode.Range16{Lo: 0x1712, Hi: 0x1714, Stride: 0x1},
		unicode.Range16{Lo: 0x1732, Hi: 0x1733, Stride: 0x1},
		unicode.Range16{Lo: 0x1752, Hi: 0x1753, Stride: 0x1},
		unicode.Range16{Lo: 0x1772, Hi: 0x1773, Stride: 0x1},
		unicode.Range16{Lo: 0x17b4, Hi: 0x17b5, Stride: 0x1},
		unicode.Range16{Lo: 0x17b7, Hi: 0x17bd, Stride: 0x1},
		unicode.Range16{Lo: 0x17c6, Hi: 0x17c6, Stride: 0x1},
		unicode.Range16{Lo: 0x17c9, Hi: 0x17d3, Stride: 0x1},
		unicode.Range16{Lo: 0x17dd, Hi: 0x17dd, Stride: 0x1},
		unicode.Range16{Lo: 0x180b, Hi: 0x180d, Stride: 0x1},
		unicode.Range16{Lo: 0x180f, Hi: 0x180f, Stride: 0x1},
		unicode.Range16{Lo: 0x1885, Hi: 0x1886, Stride: 0x1},
		unicode.Range16{Lo: 0x18a9, Hi: 0x18a9, Stride: 0x1},
		unicode.Range16{Lo: 0x1920, Hi: 0x1922, Stride: 0x1},
		unicode.Range16{Lo: 0x1927, Hi: 0x1928, Stride: 0x1},
		unicode.Range16{Lo: 0x1932, Hi: 0x1932, Stride: 0x1},
		unicode.Range16{Lo: 0x1939, Hi: 0x193b, Stride: 0x1},
		unicode.Range16{Lo: 0x1a17, Hi: 0x1a18, Stride: 0x1},
		unicode.Range16{Lo: 0x1a1b, Hi: 0x1a1b, Stride: 0x1},
		unicode.Range16{Lo: 0x1a56, Hi: 0x1a56, Stride: 0x1},
		unicode.Range16{Lo: 0x1a58, Hi: 0x1a5e, Stride: 0x1},
		unicode.Range16{Lo: 0x1a60, Hi: 0x1a60, Stride: 0x1},
		unicode.Range16{Lo: 0x1a62, Hi: 0x1a62, Stride: 0x1},
		unicode.Range16{Lo: 0x1a65, Hi: 0x1a6c, Stride: 0x1},
		unicode.Range16{Lo: 0x1a73, Hi: 0x1a7c, Stride: 0x1},
		unicode.Range16{Lo: 0x1a7f, Hi: 0x1a7f, Stride: 0x1},
		unicode.Range16{Lo: 0x1ab0, Hi: 0x1abd, Stride: 0x1},
		unicode.Range16{Lo: 0x1abe, Hi: 0x1abe, Stride: 0x1},
		unicode.Range16{Lo: 0x1abf, Hi: 0x1ace, Stride: 0x1},
		unicode.Range16{Lo: 0x1b00, Hi: 0x1b03, Stride: 0x1},
		unicode.Range16{Lo: 0x1b34, Hi: 0x1b34, Stride: 0x1},
		unicode.Range16{Lo: 0x1b35, Hi: 0x1b35, Stride: 0x1},
		unicode.Range16{Lo: 0x1b36, Hi: 0x1b3a, Stride: 0x1},
		unicode.Range16{Lo: 0x1b3c, Hi: 0x1b3c, Stride: 0x1},
		unicode.Range16{Lo: 0x1b42, Hi: 0x1b42, Stride: 0x1},
		unicode.Range16{Lo: 0x1b6b, Hi: 0x1b73, Stride: 0x1},
		unicode.Range16{Lo: 0x1b80, Hi: 0x1b81, Stride: 0x1},
		unicode.Range16{Lo: 0x1ba2, Hi: 0x1ba5, Stride: 0x1},
		unicode.Range16{Lo: 0x1ba8, Hi: 0x1ba9, Stride: 0x1},
		unicode.Range16{Lo: 0x1bab, Hi: 0x1bad, Stride: 0x1},
		unicode.Range16{Lo: 0x1be6, Hi: 0x1be6, Stride: 0x1},
		unicode.Range16{Lo: 0x1be8, Hi: 0x1be9, Stride: 0x1},
		unicode.Range16{Lo: 0x1bed, Hi: 0x1bed, Stride: 0x1},
		unicode.Range16{Lo: 0x1bef, Hi: 0x1bf1, Stride: 0x1},
		unicode.Range16{Lo: 0x1c2c, Hi: 0x1c33, Stride: 0x1},
		unicode.Range16{Lo: 0x1c36, Hi: 0x1c37, Stride: 0x1},
		unicode.Range16{Lo: 0x1cd0, Hi: 0x1cd2, Stride: 0x1},
		unicode.Range16{Lo: 0x1cd4, Hi: 0x1ce0, Stride: 0x1},
		unicode.Range16{Lo: 0x1ce2, Hi: 0x1ce8, Stride: 0x1},
		unicode.Range16{Lo: 0x1ced, Hi: 0x1ced, Stride: 0x1},
		unicode.Range16{Lo: 0x1cf4, Hi: 0x1cf4, Stride: 0x1},
		unicode.Range16{Lo: 0x1cf8, Hi: 0x1cf9, Stride: 0x1},
		unicode.Range16{Lo: 0x1dc0, Hi: 0x1dff, Stride: 0x1},
		unicode.Range16{Lo: 0x200c, Hi: 0x200c, Stride: 0x1},
		unicode.Range16{Lo: 0x20d0, Hi: 0x20dc, Stride: 0x1},
		unicode.Range16{Lo: 0x20dd, Hi: 0x20e0, Stride: 0x1},
		unicode.Range16{Lo: 0x20e1, Hi: 0x20e1, Stride: 0x1},
		unicode.Range16{Lo: 0x20e2, Hi: 0x20e4, Stride: 0x1},
		unicode.Range16{Lo: 0x20e5, Hi: 0x20f0, Stride: 0x1},
		unicode.Range16{Lo: 0x2cef, Hi: 0x2cf1, Stride: 0x1},
		unicode.Range16{Lo: 0x2d7f, Hi: 0x2d7f, Stride: 0x1},
		unicode.Range16{Lo: 0x2de0, Hi: 0x2dff, Stride: 0x1},
		unicode.Range16{Lo: 0x302a, Hi: 0x302d, Stride: 0x1},
		unicode.Range16{Lo: 0x302e, Hi: 0x302f, Stride: 0x1},
		unicode.Range16{Lo: 0x3099, Hi: 0x309a, Stride: 0x1},
		unicode.Range16{Lo: 0xa66f, Hi: 0xa66f, Stride: 0x1},
		unicode.Range16{Lo: 0xa670, Hi: 0xa672, Stride: 0x1},
		unicode.Range16{Lo: 0xa674, Hi: 0xa67d, Stride: 0x1},
		unicode.Range16{Lo: 0xa69e, Hi: 0xa69f, Stride: 0x1},
		unicode.Range16{Lo: 0xa6f0, Hi: 0xa6f1, Stride: 0x1},
		unicode.Range16{Lo: 0xa802, Hi: 0xa802, Stride: 0x1},
		unicode.Range16{Lo: 0xa806, Hi: 0xa806, Stride: 0x1},
		unicode.Range16{Lo: 0xa80b, Hi: 0xa80b, Stride: 0x1},
		unicode.Range16{Lo: 0xa825, Hi: 0xa826, Stride: 0x1},
		unicode.Range16{Lo: 0xa82c, Hi: 0xa82c, Stride: 0x1},
		unicode.Range16{Lo: 0xa8c4, Hi: 0xa8c5, Stride: 0x1},
		unicode.Range16{Lo: 0xa8e0, Hi: 0xa8f1, Stride: 0x1},
		unicode.Range16{Lo: 0xa8ff, Hi: 0xa8ff, Stride: 0x1},
		unicode.Range16{Lo: 0xa926, Hi: 0xa92d, Stride: 0x1},
		unicode.Range16{Lo: 0xa947, Hi: 0xa951, Stride: 0x1},
		unicode.Range16{Lo: 0xa980, Hi: 0xa982, Stride: 0x1},
		unicode.Range16{Lo: 0xa9b3, Hi: 0xa9b3, Stride: 0x1},
		unicode.Range16{Lo: 0xa9b6, Hi: 0xa9b9, Stride: 0x1},
		unicode.Range16{Lo: 0xa9bc, Hi: 0xa9bd, Stride: 0x1},
		unicode.Range16{Lo: 0xa9e5, Hi: 0xa9e5, Stride: 0x1},
		unicode.Range16{Lo: 0xaa29, Hi: 0xaa2e, Stride: 0x1},
		unicode.Range16{Lo: 0xaa31, Hi: 0xaa32, Stride: 0x1},
		unicode.Range16{Lo: 0xaa35, Hi: 0xaa36, Stride: 0x1},
		unicode.Range16{Lo: 0xaa43, Hi: 0xaa43, Stride: 0x1},
		unicode.Range16{Lo: 0xaa4c, Hi: 0xaa4c, Stride: 0x1},
		unicode.Range16{Lo: 0xaa7c, Hi: 0xaa7c, Stride: 0x1},
		unicode.Range16{Lo: 0xaab0, Hi: 0xaab0, Stride: 0x1},
		unicode.Range16{Lo: 0xaab2, Hi: 0xaab4, Stride: 0x1},
		unicode.Range16{Lo: 0xaab7, Hi: 0xaab8, Stride: 0x1},
		unicode.Range16{Lo: 0xaabe, Hi: 0xaabf, Stride: 0x1},
		unicode.Range16{Lo: 0xaac1, Hi: 0xaac1, Stride: 0x1},
		unicode.Range16{Lo: 0xaaec, Hi: 0xaaed, Stride: 0x1},
		unicode.Range16{Lo: 0xaaf6, Hi: 0xaaf6, Stride: 0x1},
		unicode.Range16{Lo: 0xabe5, Hi: 0xabe5, Stride: 0x1},
		unicode.Range16{Lo: 0xabe8, Hi: 0xabe8, Stride: 0x1},
		unicode.Range16{Lo: 0xabed, Hi: 0xabed, Stride: 0x1},
		unicode.Range16{Lo: 0xfb1e, Hi: 0xfb1e, Stride: 0x1},
		unicode.Range16{Lo: 0xfe00, Hi: 0xfe0f, Stride: 0x1},
		unicode.Range16{Lo: 0xfe20, Hi: 0xfe2f, Stride: 0x1},
		unicode.Range16{Lo: 0xff9e, Hi: 0xff9f, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x101fd, Hi: 0x101fd, Stride: 0x1},
		unicode.Range32{Lo: 0x102e0, Hi: 0x102e0, Stride: 0x1},
		unicode.Range32{Lo: 0x10376, Hi: 0x1037a, Stride: 0x1},
		unicode.Range32{Lo: 0x10a01, Hi: 0x10a03, Stride: 0x1},
		unicode.Range32{Lo: 0x10a05, Hi: 0x10a06, Stride: 0x1},
		unicode.Range32{Lo: 0x10a0c, Hi: 0x10a0f, Stride: 0x1},
		unicode.Range32{Lo: 0x10a38, Hi: 0x10a3a, Stride: 0x1},
		unicode.Range32{Lo: 0x10a3f, Hi: 0x10a3f, Stride: 0x1},
		unicode.Range32{Lo: 0x10ae5, Hi: 0x10ae6, Stride: 0x1},
		unicode.Range32{Lo: 0x10d24, Hi: 0x10d27, Stride: 0x1},
		unicode.Range32{Lo: 0x10eab, Hi: 0x10eac, Stride: 0x1},
		unicode.Range32{Lo: 0x10efd, Hi: 0x10eff, Stride: 0x1},
		unicode.Range32{Lo: 0x10f46, Hi: 0x10f50, Stride: 0x1},
		unicode.Range32{Lo: 0x10f82, Hi: 0x10f85, Stride: 0x1},
		unicode.Range32{Lo: 0x11001, Hi: 0x11001, Stride: 0x1},
		unicode.Range32{Lo: 0x11038, Hi: 0x11046, Stride: 0x1},
		unicode.Range32{Lo: 0x11070, Hi: 0x11070, Stride: 0x1},
		unicode.Range32{Lo: 0x11073, Hi: 0x11074, Stride: 0x1},
		unicode.Range32{Lo: 0x1107f, Hi: 0x11081, Stride: 0x1},
		unicode.Range32{Lo: 0x110b3, Hi: 0x110b6, Stride: 0x1},
		unicode.Range32{Lo: 0x110b9, Hi: 0x110ba, Stride: 0x1},
		unicode.Range32{Lo: 0x110c2, Hi: 0x110c2, Stride: 0x1},
		unicode.Range32{Lo: 0x11100, Hi: 0x11102, Stride: 0x1},
		unicode.Range32{Lo: 0x11127, Hi: 0x1112b, Stride: 0x1},
		unicode.Range32{Lo: 0x1112d, Hi: 0x11134, Stride: 0x1},
		unicode.Range32{Lo: 0x11173, Hi: 0x11173, Stride: 0x1},
		unicode.Range32{Lo: 0x11180, Hi: 0x11181, Stride: 0x1},
		unicode.Range32{Lo: 0x111b6, Hi: 0x111be, Stride: 0x1},
		unicode.Range32{Lo: 0x111c9, Hi: 0x111cc, Stride: 0x1},
		unicode.Range32{Lo: 0x111cf, Hi: 0x111cf, Stride: 0x1},
		unicode.Range32{Lo: 0x1122f, Hi: 0x11231, Stride: 0x1},
		unicode.Range32{Lo: 0x11234, Hi: 0x11234, Stride: 0x1},
		unicode.Range32{Lo: 0x11236, Hi: 0x11237, Stride: 0x1},
		unicode.Range32{Lo: 0x1123e, Hi: 0x1123e, Stride: 0x1},
		unicode.Range32{Lo: 0x11241, Hi: 0x11241, Stride: 0x1},
		unicode.Range32{Lo: 0x112df, Hi: 0x112df, Stride: 0x1},
		unicode.Range32{Lo: 0x112e3, Hi: 0x112ea, Stride: 0x1},
		unicode.Range32{Lo: 0x11300, Hi: 0x11301, Stride: 0x1},
		unicode.Range32{Lo: 0x1133b, Hi: 0x1133c, Stride: 0x1},
		unicode.Range32{Lo: 0x1133e, Hi: 0x1133e, Stride: 0x1},
		unicode.Range32{Lo: 0x11340, Hi: 0x11340, Stride: 0x1},
		unicode.Range32{Lo: 0x11357, Hi: 0x11357, Stride: 0x1},
		unicode.Range32{Lo: 0x11366, Hi: 0x1136c, Stride: 0x1},
		unicode.Range32{Lo: 0x11370, Hi: 0x11374, Stride: 0x1},
		unicode.Range32{Lo: 0x11438, Hi: 0x1143f, Stride: 0x1},
		unicode.Range32{Lo: 0x11442, Hi: 0x11444, Stride: 0x1},
		unicode.Range32{Lo: 0x11446, Hi: 0x11446, Stride: 0x1},
		unicode.Range32{Lo: 0x1145e, Hi: 0x1145e, Stride: 0x1},
		unicode.Range32{Lo: 0x114b0, Hi: 0x114b0, Stride: 0x1},
		unicode.Range32{Lo: 0x114b3, Hi: 0x114b8, Stride: 0x1},
		unicode.Range32{Lo: 0x114ba, Hi: 0x114ba, Stride: 0x1},
		unicode.Range32{Lo: 0x114bd, Hi: 0x114bd, Stride: 0x1},
		unicode.Range32{Lo: 0x114bf, Hi: 0x114c0, Stride: 0x1},
		unicode.Range32{Lo: 0x114c2, Hi: 0x114c3, Stride: 0x1},
		unicode.Range32{Lo: 0x115af, Hi: 0x115af, Stride: 0x1},
		unicode.Range32{Lo: 0x115b2, Hi: 0x115b5, Stride: 0x1},
		unicode.Range32{Lo: 0x115bc, Hi: 0x115bd, Stride: 0x1},
		unicode.Range32{Lo: 0x115bf, Hi: 0x115c0, Stride: 0x1},
		unicode.Range32{Lo: 0x115dc, Hi: 0x115dd, Stride: 0x1},
		unicode.Range32{Lo: 0x11633, Hi: 0x1163a, Stride: 0x1},
		unicode.Range32{Lo: 0x1163d, Hi: 0x1163d, Stride: 0x1},
		unicode.Range32{Lo: 0x1163f, Hi: 0x11640, Stride: 0x1},
		unicode.Range32{Lo: 0x116ab, Hi: 0x116ab, Stride: 0x1},
		unicode.Range32{Lo: 0x116ad, Hi: 0x116ad, Stride: 0x1},
		unicode.Range32{Lo: 0x116b0, Hi: 0x116b5, Stride: 0x1},
		unicode.Range32{Lo: 0x116b7, Hi: 0x116b7, Stride: 0x1},
		unicode.Range32{Lo: 0x1171d, Hi: 0x1171f, Stride: 0x1},
		unicode.Range32{Lo: 0x11722, Hi: 0x11725, Stride: 0x1},
		unicode.Range32{Lo: 0x11727, Hi: 0x1172b, Stride: 0x1},
		unicode.Range32{Lo: 0x1182f, Hi: 0x11837, Stride: 0x1},
		unicode.Range32{Lo: 0x11839, Hi: 0x1183a, Stride: 0x1},
		unicode.Range32{Lo: 0x11930, Hi: 0x11930, Stride: 0x1},
		unicode.Range32{Lo: 0x1193b, Hi: 0x1193c, Stride: 0x1},
		unicode.Range32{Lo: 0x1193e, Hi: 0x1193e, Stride: 0x1},
		unicode.Range32{Lo: 0x11943, Hi: 0x11943, Stride: 0x1},
		unicode.Range32{Lo: 0x119d4, Hi: 0x119d7, Stride: 0x1},
		unicode.Range32{Lo: 0x119da, Hi: 0x119db, Stride: 0x1},
		unicode.Range32{Lo: 0x119e0, Hi: 0x119e0, Stride: 0x1},
		unicode.Range32{Lo: 0x11a01, Hi: 0x11a0a, Stride: 0x1},
		unicode.Range32{Lo: 0x11a33, Hi: 0x11a38, Stride: 0x1},
		unicode.Range32{Lo: 0x11a3b, Hi: 0x11a3e, Stride: 0x1},
		unicode.Range32{Lo: 0x11a47, Hi: 0x11a47, Stride: 0x1},
		unicode.Range32{Lo: 0x11a51, Hi: 0x11a56, Stride: 0x1},
		unicode.Range32{Lo: 0x11a59, Hi: 0x11a5b, Stride: 0x1},
		unicode.Range32{Lo: 0x11a8a, Hi: 0x11a96, Stride: 0x1},
		unicode.Range32{Lo: 0x11a98, Hi: 0x11a99, Stride: 0x1},
		unicode.Range32{Lo: 0x11c30, Hi: 0x11c36, Stride: 0x1},
		unicode.Range32{Lo: 0x11c38, Hi: 0x11c3d, Stride: 0x1},
		unicode.Range32{Lo: 0x11c3f, Hi: 0x11c3f, Stride: 0x1},
		unicode.Range32{Lo: 0x11c92, Hi: 0x11ca7, Stride: 0x1},
		unicode.Range32{Lo: 0x11caa, Hi: 0x11cb0, Stride: 0x1},
		unicode.Range32{Lo: 0x11cb2, Hi: 0x11cb3, Stride: 0x1},
		unicode.Range32{Lo: 0x11cb5, Hi: 0x11cb6, Stride: 0x1},
		unicode.Range32{Lo: 0x11d31, Hi: 0x11d36, Stride: 0x1},
		unicode.Range32{Lo: 0x11d3a, Hi: 0x11d3a, Stride: 0x1},
		unicode.Range32{Lo: 0x11d3c, Hi: 0x11d3d, Stride: 0x1},
		unicode.Range32{Lo: 0x11d3f, Hi: 0x11d45, Stride: 0x1},
		unicode.Range32{Lo: 0x11d47, Hi: 0x11d47, Stride: 0x1},
		unicode.Range32{Lo: 0x11d90, Hi: 0x11d91, Stride: 0x1},
		unicode.Range32{Lo: 0x11d95, Hi: 0x11d95, Stride: 0x1},
		unicode.Range32{Lo: 0x11d97, Hi: 0x11d97, Stride: 0x1},
		unicode.Range32{Lo: 0x11ef3, Hi: 0x11ef4, Stride: 0x1},
		unicode.Range32{Lo: 0x11f00, Hi: 0x11f01, Stride: 0x1},
		unicode.Range32{Lo: 0x11f36, Hi: 0x11f3a, Stride: 0x1},
		unicode.Range32{Lo: 0x11f40, Hi: 0x11f40, Stride: 0x1},
		unicode.Range32{Lo: 0x11f42, Hi: 0x11f42, Stride: 0x1},
		unicode.Range32{Lo: 0x13440, Hi: 0x13440, Stride: 0x1},
		unicode.Range32{Lo: 0x13447, Hi: 0x13455, Stride: 0x1},
		unicode.Range32{Lo: 0x16af0, Hi: 0x16af4, Stride: 0x1},
		unicode.Range32{Lo: 0x16b30, Hi: 0x16b36, Stride: 0x1},
		unicode.Range32{Lo: 0x16f4f, Hi: 0x16f4f, Stride: 0x1},
		unicode.Range32{Lo: 0x16f8f, Hi: 0x16f92, Stride: 0x1},
		unicode.Range32{Lo: 0x16fe4, Hi: 0x16fe4, Stride: 0x1},
		unicode.Range32{Lo: 0x1bc9d, Hi: 0x1bc9e, Stride: 0x1},
		unicode.Range32{Lo: 0x1cf00, Hi: 0x1cf2d, Stride: 0x1},
		unicode.Range32{Lo: 0x1cf30, Hi: 0x1cf46, Stride: 0x1},
		unicode.Range32{Lo: 0x1d165, Hi: 0x1d165, Stride: 0x1},
		unicode.Range32{Lo: 0x1d167, Hi: 0x1d169, Stride: 0x1},
		unicode.Range32{Lo: 0x1d16e, Hi: 0x1d172, Stride: 0x1},
		unicode.Range32{Lo: 0x1d17b, Hi: 0x1d182, Stride: 0x1},
		unicode.Range32{Lo: 0x1d185, Hi: 0x1d18b, Stride: 0x1},
		unicode.Range32{Lo: 0x1d1aa, Hi: 0x1d1ad, Stride: 0x1},
		unicode.Range32{Lo: 0x1d242, Hi: 0x1d244, Stride: 0x1},
		unicode.Range32{Lo: 0x1da00, Hi: 0x1da36, Stride: 0x1},
		unicode.Range32{Lo: 0x1da3b, Hi: 0x1da6c, Stride: 0x1},
		unicode.Range32{Lo: 0x1da75, Hi: 0x1da75, Stride: 0x1},
		unicode.Range32{Lo: 0x1da84, Hi: 0x1da84, Stride: 0x1},
		unicode.Range32{Lo: 0x1da9b, Hi: 0x1da9f, Stride: 0x1},
		unicode.Range32{Lo: 0x1daa1, Hi: 0x1daaf, Stride: 0x1},
		unicode.Range32{Lo: 0x1e000, Hi: 0x1e006, Stride: 0x1},
		unicode.Range32{Lo: 0x1e008, Hi: 0x1e018, Stride: 0x1},
		unicode.Range32{Lo: 0x1e01b, Hi: 0x1e021, Stride: 0x1},
		unicode.Range32{Lo: 0x1e023, Hi: 0x1e024, Stride: 0x1},
		unicode.Range32{Lo: 0x1e026, Hi: 0x1e02a, Stride: 0x1},
		unicode.Range32{Lo: 0x1e08f, Hi: 0x1e08f, Stride: 0x1},
		unicode.Range32{Lo: 0x1e130, Hi: 0x1e136, Stride: 0x1},
		unicode.Range32{Lo: 0x1e2ae, Hi: 0x1e2ae, Stride: 0x1},
		unicode.Range32{Lo: 0x1e2ec, Hi: 0x1e2ef, Stride: 0x1},
		unicode.Range32{Lo: 0x1e4ec, Hi: 0x1e4ef, Stride: 0x1},
		unicode.Range32{Lo: 0x1e8d0, Hi: 0x1e8d6, Stride: 0x1},
		unicode.Range32{Lo: 0x1e944, Hi: 0x1e94a, Stride: 0x1},
		unicode.Range32{Lo: 0x1f3fb, Hi: 0x1f3ff, Stride: 0x1},
		unicode.Range32{Lo: 0xe0020, Hi: 0xe007f, Stride: 0x1},
		unicode.Range32{Lo: 0xe0100, Hi: 0xe01ef, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _GraphemeL = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x1100, Hi: 0x115f, Stride: 0x1},
		unicode.Range16{Lo: 0xa960, Hi: 0xa97c, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _GraphemeLF = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0xa, Hi: 0xa, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _GraphemeLV = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0xac00, Hi: 0xac00, Stride: 0x1},
		unicode.Range16{Lo: 0xac1c, Hi: 0xac1c, Stride: 0x1},
		unicode.Range16{Lo: 0xac38, Hi: 0xac38, Stride: 0x1},
		unicode.Range16{Lo: 0xac54, Hi: 0xac54, Stride: 0x1},
		unicode.Range16{Lo: 0xac70, Hi: 0xac70, Stride: 0x1},
		unicode.Range16{Lo: 0xac8c, Hi: 0xac8c, Stride: 0x1},
		unicode.Range16{Lo: 0xaca8, Hi: 0xaca8, Stride: 0x1},
		unicode.Range16{Lo: 0xacc4, Hi: 0xacc4, Stride: 0x1},
		unicode.Range16{Lo: 0xace0, Hi: 0xace0, Stride: 0x1},
		unicode.Range16{Lo: 0xacfc, Hi: 0xacfc, Stride: 0x1},
		unicode.Range16{Lo: 0xad18, Hi: 0xad18, Stride: 0x1},
		unicode.Range16{Lo: 0xad34, Hi: 0xad34, Stride: 0x1},
		unicode.Range16{Lo: 0xad50, Hi: 0xad50, Stride: 0x1},
		unicode.Range16{Lo: 0xad6c, Hi: 0xad6c, Stride: 0x1},
		unicode.Range16{Lo: 0xad88, Hi: 0xad88, Stride: 0x1},
		unicode.Range16{Lo: 0xada4, Hi: 0xada4, Stride: 0x1},
		unicode.Range16{Lo: 0xadc0, Hi: 0xadc0, Stride: 0x1},
		unicode.Range16{Lo: 0xaddc, Hi: 0xaddc, Stride: 0x1},
		unicode.Range16{Lo: 0xadf8, Hi: 0xadf8, Stride: 0x1},
		unicode.Range16{Lo: 0xae14, Hi: 0xae14, Stride: 0x1},
		unicode.Range16{Lo: 0xae30, Hi: 0xae30, Stride: 0x1},
		unicode.Range16{Lo: 0xae4c, Hi: 0xae4c, Stride: 0x1},
		unicode.Range16{Lo: 0xae68, Hi: 0xae68, Stride: 0x1},
		unicode.Range16{Lo: 0xae84, Hi: 0xae84, Stride: 0x1},
		unicode.Range16{Lo: 0xaea0, Hi: 0xaea0, Stride: 0x1},
		unicode.Range16{Lo: 0xaebc, Hi: 0xaebc, Stride: 0x1},
		unicode.Range16{Lo: 0xaed8, Hi: 0xaed8, Stride: 0x1},
		unicode.Range16{Lo: 0xaef4, Hi: 0xaef4, Stride: 0x1},
		unicode.Range16{Lo: 0xaf10, Hi: 0xaf10, Stride: 0x1},
		unicode.Range16{Lo: 0xaf2c, Hi: 0xaf2c, Stride: 0x1},
		unicode.Range16{Lo: 0xaf48, Hi: 0xaf48, Stride: 0x1},
		unicode.Range16{Lo: 0xaf64, Hi: 0xaf64, Stride: 0x1},
		unicode.Range16{Lo: 0xaf80, Hi: 0xaf80, Stride: 0x1},
		unicode.Range16{Lo: 0xaf9c, Hi: 0xaf9c, Stride: 0x1},
		unicode.Range16{Lo: 0xafb8, Hi: 0xafb8, Stride: 0x1},
		unicode.Range16{Lo: 0xafd4, Hi: 0xafd4, Stride: 0x1},
		unicode.Range16{Lo: 0xaff0, Hi: 0xaff0, Stride: 0x1},
		unicode.Range16{Lo: 0xb00c, Hi: 0xb00c, Stride: 0x1},
		unicode.Range16{Lo: 0xb028, Hi: 0xb028, Stride: 0x1},
		unicode.Range16{Lo: 0xb044, Hi: 0xb044, Stride: 0x1},
		unicode.Range16{Lo: 0xb060, Hi: 0xb060, Stride: 0x1},
		unicode.Range16{Lo: 0xb07c, Hi: 0xb07c, Stride: 0x1},
		unicode.Range16{Lo: 0xb098, Hi: 0xb098, Stride: 0x1},
		unicode.Range16{Lo: 0xb0b4, Hi: 0xb0b4, Stride: 0x1},
		unicode.Range16{Lo: 0xb0d0, Hi: 0xb0d0, Stride: 0x1},
		unicode.Range16{Lo: 0xb0ec, Hi: 0xb0ec, Stride: 0x1},
		unicode.Range16{Lo: 0xb108, Hi: 0xb108, Stride: 0x1},
		unicode.Range16{Lo: 0xb124, Hi: 0xb124, Stride: 0x1},
		unicode.Range16{Lo: 0xb140, Hi: 0xb140, Stride: 0x1},
		unicode.Range16{Lo: 0xb15c, Hi: 0xb15c, Stride: 0x1},
		unicode.Range16{Lo: 0xb178, Hi: 0xb178, Stride: 0x1},
		unicode.Range16{Lo: 0xb194, Hi: 0xb194, Stride: 0x1},
		unicode.Range16{Lo: 0xb1b0, Hi: 0xb1b0, Stride: 0x1},
		unicode.Range16{Lo: 0xb1cc, Hi: 0xb1cc, Stride: 0x1},
		unicode.Range16{Lo: 0xb1e8, Hi: 0xb1e8, Stride: 0x1},
		unicode.Range16{Lo: 0xb204, Hi: 0xb204, Stride: 0x1},
		unicode.Range16{Lo: 0xb220, Hi: 0xb220, Stride: 0x1},
		unicode.Range16{Lo: 0xb23c, Hi: 0xb23c, Stride: 0x1},
		unicode.Range16{Lo: 0xb258, Hi: 0xb258, Stride: 0x1},
		unicode.Range16{Lo: 0xb274, Hi: 0xb274, Stride: 0x1},
		unicode.Range16{Lo: 0xb290, Hi: 0xb290, Stride: 0x1},
		unicode.Range16{Lo: 0xb2ac, Hi: 0xb2ac, Stride: 0x1},
		unicode.Range16{Lo: 0xb2c8, Hi: 0xb2c8, Stride: 0x1},
		unicode.Range16{Lo: 0xb2e4, Hi: 0xb2e4, Stride: 0x1},
		unicode.Range16{Lo: 0xb300, Hi: 0xb300, Stride: 0x1},
		unicode.Range16{Lo: 0xb31c, Hi: 0xb31c, Stride: 0x1},
		unicode.Range16{Lo: 0xb338, Hi: 0xb338, Stride: 0x1},
		unicode.Range16{Lo: 0xb354, Hi: 0xb354, Stride: 0x1},
		unicode.Range16{Lo: 0xb370, Hi: 0xb370, Stride: 0x1},
		unicode.Range16{Lo: 0xb38c, Hi: 0xb38c, Stride: 0x1},
		unicode.Range16{Lo: 0xb3a8, Hi: 0xb3a8, Stride: 0x1},
		unicode.Range16{Lo: 0xb3c4, Hi: 0xb3c4, Stride: 0x1},
		unicode.Range16{Lo: 0xb3e0, Hi: 0xb3e0, Stride: 0x1},
		unicode.Range16{Lo: 0xb3fc, Hi: 0xb3fc, Stride: 0x1},
		unicode.Range16{Lo: 0xb418, Hi: 0xb418, Stride: 0x1},
		unicode.Range16{Lo: 0xb434, Hi: 0xb434, Stride: 0x1},
		unicode.Range16{Lo: 0xb450, Hi: 0xb450, Stride: 0x1},
		unicode.Range16{Lo: 0xb46c, Hi: 0xb46c, Stride: 0x1},
		unicode.Range16{Lo: 0xb488, Hi: 0xb488, Stride: 0x1},
		unicode.Range16{Lo: 0xb4a4, Hi: 0xb4a4, Stride: 0x1},
		unicode.Range16{Lo: 0xb4c0, Hi: 0xb4c0, Stride: 0x1},
		unicode.Range16{Lo: 0xb4dc, Hi: 0xb4dc, Stride: 0x1},
		unicode.Range16{Lo: 0xb4f8, Hi: 0xb4f8, Stride: 0x1},
		unicode.Range16{Lo: 0xb514, Hi: 0xb514, Stride: 0x1},
		unicode.Range16{Lo: 0xb530, Hi: 0xb530, Stride: 0x1},
		unicode.Range16{Lo: 0xb54c, Hi: 0xb54c, Stride: 0x1},
		unicode.Range16{Lo: 0xb568, Hi: 0xb568, Stride: 0x1},
		unicode.Range16{Lo: 0xb584, Hi: 0xb584, Stride: 0x1},
		unicode.Range16{Lo: 0xb5a0, Hi: 0xb5a0, Stride: 0x1},
		unicode.Range16{Lo: 0xb5bc, Hi: 0xb5bc, Stride: 0x1},
		unicode.Range16{Lo: 0xb5d8, Hi: 0xb5d8, Stride: 0x1},
		unicode.Range16{Lo: 0xb5f4, Hi: 0xb5f4, Stride: 0x1},
		unicode.Range16{Lo: 0xb610, Hi: 0xb610, Stride: 0x1},
		unicode.Range16{Lo: 0xb62c, Hi: 0xb62c, Stride: 0x1},
		unicode.Range16{Lo: 0xb648, Hi: 0xb648, Stride: 0x1},
		unicode.Range16{Lo: 0xb664, Hi: 0xb664, Stride: 0x1},
		unicode.Range16{Lo: 0xb680, Hi: 0xb680, Stride: 0x1},
		unicode.Range16{Lo: 0xb69c, Hi: 0xb69c, Stride: 0x1},
		unicode.Range16{Lo: 0xb6b8, Hi: 0xb6b8, Stride: 0x1},
		unicode.Range16{Lo: 0xb6d4, Hi: 0xb6d4, Stride: 0x1},
		unicode.Range16{Lo: 0xb6f0, Hi: 0xb6f0, Stride: 0x1},
		unicode.Range16{Lo: 0xb70c, Hi: 0xb70c, Stride: 0x1},
		unicode.Range16{Lo: 0xb728, Hi: 0xb728, Stride: 0x1},
		unicode.Range16{Lo: 0xb744, Hi: 0xb744, Stride: 0x1},
		unicode.Range16{Lo: 0xb760, Hi: 0xb760, Stride: 0x1},
		unicode.Range16{Lo: 0xb77c, Hi: 0xb77c, Stride: 0x1},
		unicode.Range16{Lo: 0xb798, Hi: 0xb798, Stride: 0x1},
		unicode.Range16{Lo: 0xb7b4, Hi: 0xb7b4, Stride: 0x1},
		unicode.Range16{Lo: 0xb7d0, Hi: 0xb7d0, Stride: 0x1},
		unicode.Range16{Lo: 0xb7ec, Hi: 0xb7ec, Stride: 0x1},
		unicode.Range16{Lo: 0xb808, Hi: 0xb808, Stride: 0x1},
		unicode.Range16{Lo: 0xb824, Hi: 0xb824, Stride: 0x1},
		unicode.Range16{Lo: 0xb840, Hi: 0xb840, Stride: 0x1},
		unicode.Range16{Lo: 0xb85c, Hi: 0xb85c, Stride: 0x1},
		unicode.Range16{Lo: 0xb878, Hi: 0xb878, Stride: 0x1},
		unicode.Range16{Lo: 0xb894, Hi: 0xb894, Stride: 0x1},
		unicode.Range16{Lo: 0xb8b0, Hi: 0xb8b0, Stride: 0x1},
		unicode.Range16{Lo: 0xb8cc, Hi: 0xb8cc, Stride: 0x1},
		unicode.Range16{Lo: 0xb8e8, Hi: 0xb8e8, Stride: 0x1},
		unicode.Range16{Lo: 0xb904, Hi: 0xb904, Stride: 0x1},
		unicode.Range16{Lo: 0xb920, Hi: 0xb920, Stride: 0x1},
		unicode.Range16{Lo: 0xb93c, Hi: 0xb93c, Stride: 0x1},
		unicode.Range16{Lo: 0xb958, Hi: 0xb958, Stride: 0x1},
		unicode.Range16{Lo: 0xb974, Hi: 0xb974, Stride: 0x1},
		unicode.Range16{Lo: 0xb990, Hi: 0xb990, Stride: 0x1},
		unicode.Range16{Lo: 0xb9ac, Hi: 0xb9ac, Stride: 0x1},
		unicode.Range16{Lo: 0xb9c8, Hi: 0xb9c8, Stride: 0x1},
		unicode.Range16{Lo: 0xb9e4, Hi: 0xb9e4, Stride: 0x1},
		unicode.Range16{Lo: 0xba00, Hi: 0xba00, Stride: 0x1},
		unicode.Range16{Lo: 0xba1c, Hi: 0xba1c, Stride: 0x1},
		unicode.Range16{Lo: 0xba38, Hi: 0xba38, Stride: 0x1},
		unicode.Range16{Lo: 0xba54, Hi: 0xba54, Stride: 0x1},
		unicode.Range16{Lo: 0xba70, Hi: 0xba70, Stride: 0x1},
		unicode.Range16{Lo: 0xba8c, Hi: 0xba8c, Stride: 0x1},
		unicode.Range16{Lo: 0xbaa8, Hi: 0xbaa8, Stride: 0x1},
		unicode.Range16{Lo: 0xbac4, Hi: 0xbac4, Stride: 0x1},
		unicode.Range16{Lo: 0xbae0, Hi: 0xbae0, Stride: 0x1},
		unicode.Range16{Lo: 0xbafc, Hi: 0xbafc, Stride: 0x1},
		unicode.Range16{Lo: 0xbb18, Hi: 0xbb18, Stride: 0x1},
		unicode.Range16{Lo: 0xbb34, Hi: 0xbb34, Stride: 0x1},
		unicode.Range16{Lo: 0xbb50, Hi: 0xbb50, Stride: 0x1},
		unicode.Range16{Lo: 0xbb6c, Hi: 0xbb6c, Stride: 0x1},
		unicode.Range16{Lo: 0xbb88, Hi: 0xbb88, Stride: 0x1},
		unicode.Range16{Lo: 0xbba4, Hi: 0xbba4, Stride: 0x1},
		unicode.Range16{Lo: 0xbbc0, Hi: 0xbbc0, Stride: 0x1},
		unicode.Range16{Lo: 0xbbdc, Hi: 0xbbdc, Stride: 0x1},
		unicode.Range16{Lo: 0xbbf8, Hi: 0xbbf8, Stride: 0x1},
		unicode.Range16{Lo: 0xbc14, Hi: 0xbc14, Stride: 0x1},
		unicode.Range16{Lo: 0xbc30, Hi: 0xbc30, Stride: 0x1},
		unicode.Range16{Lo: 0xbc4c, Hi: 0xbc4c, Stride: 0x1},
		unicode.Range16{Lo: 0xbc68, Hi: 0xbc68, Stride: 0x1},
		unicode.Range16{Lo: 0xbc84, Hi: 0xbc84, Stride: 0x1},
		unicode.Range16{Lo: 0xbca0, Hi: 0xbca0, Stride: 0x1},
		unicode.Range16{Lo: 0xbcbc, Hi: 0xbcbc, Stride: 0x1},
		unicode.Range16{Lo: 0xbcd8, Hi: 0xbcd8, Stride: 0x1},
		unicode.Range16{Lo: 0xbcf4, Hi: 0xbcf4, Stride: 0x1},
		unicode.Range16{Lo: 0xbd10, Hi: 0xbd10, Stride: 0x1},
		unicode.Range16{Lo: 0xbd2c, Hi: 0xbd2c, Stride: 0x1},
		unicode.Range16{Lo: 0xbd48, Hi: 0xbd48, Stride: 0x1},
		unicode.Range16{Lo: 0xbd64, Hi: 0xbd64, Stride: 0x1},
		unicode.Range16{Lo: 0xbd80, Hi: 0xbd80, Stride: 0x1},
		unicode.Range16{Lo: 0xbd9c, Hi: 0xbd9c, Stride: 0x1},
		unicode.Range16{Lo: 0xbdb8, Hi: 0xbdb8, Stride: 0x1},
		unicode.Range16{Lo: 0xbdd4, Hi: 0xbdd4, Stride: 0x1},
		unicode.Range16{Lo: 0xbdf0, Hi: 0xbdf0, Stride: 0x1},
		unicode.Range16{Lo: 0xbe0c, Hi: 0xbe0c, Stride: 0x1},
		unicode.Range16{Lo: 0xbe28, Hi: 0xbe28, Stride: 0x1},
		unicode.Range16{Lo: 0xbe44, Hi: 0xbe44, Stride: 0x1},
		unicode.Range16{Lo: 0xbe60, Hi: 0xbe60, Stride: 0x1},
		unicode.Range16{Lo: 0xbe7c, Hi: 0xbe7c, Stride: 0x1},
		unicode.Range16{Lo: 0xbe98, Hi: 0xbe98, Stride: 0x1},
		unicode.Range16{Lo: 0xbeb4, Hi: 0xbeb4, Stride: 0x1},
		unicode.Range16{Lo: 0xbed0, Hi: 0xbed0, Stride: 0x1},
		unicode.Range16{Lo: 0xbeec, Hi: 0xbeec, Stride: 0x1},
		unicode.Range16{Lo: 0xbf08, Hi: 0xbf08, Stride: 0x1},
		unicode.Range16{Lo: 0xbf24, Hi: 0xbf24, Stride: 0x1},
		unicode.Range16{Lo: 0xbf40, Hi: 0xbf40, Stride: 0x1},
		unicode.Range16{Lo: 0xbf5c, Hi: 0xbf5c, Stride: 0x1},
		unicode.Range16{Lo: 0xbf78, Hi: 0xbf78, Stride: 0x1},
		unicode.Range16{Lo: 0xbf94, Hi: 0xbf94, Stride: 0x1},
		unicode.Range16{Lo: 0xbfb0, Hi: 0xbfb0, Stride: 0x1},
		unicode.Range16{Lo: 0xbfcc, Hi: 0xbfcc, Stride: 0x1},
		unicode.Range16{Lo: 0xbfe8, Hi: 0xbfe8, Stride: 0x1},
		unicode.Range16{Lo: 0xc004, Hi: 0xc004, Stride: 0x1},
		unicode.Range16{Lo: 0xc020, Hi: 0xc020, Stride: 0x1},
		unicode.Range16{Lo: 0xc03c, Hi: 0xc03c, Stride: 0x1},
		unicode.Range16{Lo: 0xc058, Hi: 0xc058, Stride: 0x1},
		unicode.Range16{Lo: 0xc074, Hi: 0xc074, Stride: 0x1},
		unicode.Range16{Lo: 0xc090, Hi: 0xc090, Stride: 0x1},
		unicode.Range16{Lo: 0xc0ac, Hi: 0xc0ac, Stride: 0x1},
		unicode.Range16{Lo: 0xc0c8, Hi: 0xc0c8, Stride: 0x1},
		unicode.Range16{Lo: 0xc0e4, Hi: 0xc0e4, Stride: 0x1},
		unicode.Range16{Lo: 0xc100, Hi: 0xc100, Stride: 0x1},
		unicode.Range16{Lo: 0xc11c, Hi: 0xc11c, Stride: 0x1},
		unicode.Range16{Lo: 0xc138, Hi: 0xc138, Stride: 0x1},
		unicode.Range16{Lo: 0xc154, Hi: 0xc154, Stride: 0x1},
		unicode.Range16{Lo: 0xc170, Hi: 0xc170, Stride: 0x1},
		unicode.Range16{Lo: 0xc18c, Hi: 0xc18c, Stride: 0x1},
		unicode.Range16{Lo: 0xc1a8, Hi: 0xc1a8, Stride: 0x1},
		unicode.Range16{Lo: 0xc1c4, Hi: 0xc1c4, Stride: 0x1},
		unicode.Range16{Lo: 0xc1e0, Hi: 0xc1e0, Stride: 0x1},
		unicode.Range16{Lo: 0xc1fc, Hi: 0xc1fc, Stride: 0x1},
		unicode.Range16{Lo: 0xc218, Hi: 0xc218, Stride: 0x1},
		unicode.Range16{Lo: 0xc234, Hi: 0xc234, Stride: 0x1},
		unicode.Range16{Lo: 0xc250, Hi: 0xc250, Stride: 0x1},
		unicode.Range16{Lo: 0xc26c, Hi: 0xc26c, Stride: 0x1},
		unicode.Range16{Lo: 0xc288, Hi: 0xc288, Stride: 0x1},
		unicode.Range16{Lo: 0xc2a4, Hi: 0xc2a4, Stride: 0x1},
		unicode.Range16{Lo: 0xc2c0, Hi: 0xc2c0, Stride: 0x1},
		unicode.Range16{Lo: 0xc2dc, Hi: 0xc2dc, Stride: 0x1},
		unicode.Range16{Lo: 0xc2f8, Hi: 0xc2f8, Stride: 0x1},
		unicode.Range16{Lo: 0xc314, Hi: 0xc314, Stride: 0x1},
		unicode.Range16{Lo: 0xc330, Hi: 0xc330, Stride: 0x1},
		unicode.Range16{Lo: 0xc34c, Hi: 0xc34c, Stride: 0x1},
		unicode.Range16{Lo: 0xc368, Hi: 0xc368, Stride: 0x1},
		unicode.Range16{Lo: 0xc384, Hi: 0xc384, Stride: 0x1},
		unicode.Range16{Lo: 0xc3a0, Hi: 0xc3a0, Stride: 0x1},
		unicode.Range16{Lo: 0xc3bc, Hi: 0xc3bc, Stride: 0x1},
		unicode.Range16{Lo: 0xc3d8, Hi: 0xc3d8, Stride: 0x1},
		unicode.Range16{Lo: 0xc3f4, Hi: 0xc3f4, Stride: 0x1},
		unicode.Range16{Lo: 0xc410, Hi: 0xc410, Stride: 0x1},
		unicode.Range16{Lo: 0xc42c, Hi: 0xc42c, Stride: 0x1},
		unicode.Range16{Lo: 0xc448, Hi: 0xc448, Stride: 0x1},
		unicode.Range16{Lo: 0xc464, Hi: 0xc464, Stride: 0x1},
		unicode.Range16{Lo: 0xc480, Hi: 0xc480, Stride: 0x1},
		unicode.Range16{Lo: 0xc49c, Hi: 0xc49c, Stride: 0x1},
		unicode.Range16{Lo: 0xc4b8, Hi: 0xc4b8, Stride: 0x1},
		unicode.Range16{Lo: 0xc4d4, Hi: 0xc4d4, Stride: 0x1},
		unicode.Range16{Lo: 0xc4f0, Hi: 0xc4f0, Stride: 0x1},
		unicode.Range16{Lo: 0xc50c, Hi: 0xc50c, Stride: 0x1},
		unicode.Range16{Lo: 0xc528, Hi: 0xc528, Stride: 0x1},
		unicode.Range16{Lo: 0xc544, Hi: 0xc544, Stride: 0x1},
		unicode.Range16{Lo: 0xc560, Hi: 0xc560, Stride: 0x1},
		unicode.Range16{Lo: 0xc57c, Hi: 0xc57c, Stride: 0x1},
		unicode.Range16{Lo: 0xc598, Hi: 0xc598, Stride: 0x1},
		unicode.Range16{Lo: 0xc5b4, Hi: 0xc5b4, Stride: 0x1},
		unicode.Range16{Lo: 0xc5d0, Hi: 0xc5d0, Stride: 0x1},
		unicode.Range16{Lo: 0xc5ec, Hi: 0xc5ec, Stride: 0x1},
		unicode.Range16{Lo: 0xc608, Hi: 0xc608, Stride: 0x1},
		unicode.Range16{Lo: 0xc624, Hi: 0xc624, Stride: 0x1},
		unicode.Range16{Lo: 0xc640, Hi: 0xc640, Stride: 0x1},
		unicode.Range16{Lo: 0xc65c, Hi: 0xc65c, Stride: 0x1},
		unicode.Range16{Lo: 0xc678, Hi: 0xc678, Stride: 0x1},
		unicode.Range16{Lo: 0xc694, Hi: 0xc694, Stride: 0x1},
		unicode.Range16{Lo: 0xc6b0, Hi: 0xc6b0, Stride: 0x1},
		unicode.Range16{Lo: 0xc6cc, Hi: 0xc6cc, Stride: 0x1},
		unicode.Range16{Lo: 0xc6e8, Hi: 0xc6e8, Stride: 0x1},
		unicode.Range16{Lo: 0xc704, Hi: 0xc704, Stride: 0x1},
		unicode.Range16{Lo: 0xc720, Hi: 0xc720, Stride: 0x1},
		unicode.Range16{Lo: 0xc73c, Hi: 0xc73c, Stride: 0x1},
		unicode.Range16{Lo: 0xc758, Hi: 0xc758, Stride: 0x1},
		unicode.Range16{Lo: 0xc774, Hi: 0xc774, Stride: 0x1},
		unicode.Range16{Lo: 0xc790, Hi: 0xc790, Stride: 0x1},
		unicode.Range16{Lo: 0xc7ac, Hi: 0xc7ac, Stride: 0x1},
		unicode.Range16{Lo: 0xc7c8, Hi: 0xc7c8, Stride: 0x1},
		unicode.Range16{Lo: 0xc7e4, Hi: 0xc7e4, Stride: 0x1},
		unicode.Range16{Lo: 0xc800, Hi: 0xc800, Stride: 0x1},
		unicode.Range16{Lo: 0xc81c, Hi: 0xc81c, Stride: 0x1},
		unicode.Range16{Lo: 0xc838, Hi: 0xc838, Stride: 0x1},
		unicode.Range16{Lo: 0xc854, Hi: 0xc854, Stride: 0x1},
		unicode.Range16{Lo: 0xc870, Hi: 0xc870, Stride: 0x1},
		unicode.Range16{Lo: 0xc88c, Hi: 0xc88c, Stride: 0x1},
		unicode.Range16{Lo: 0xc8a8, Hi: 0xc8a8, Stride: 0x1},
		unicode.Range16{Lo: 0xc8c4, Hi: 0xc8c4, Stride: 0x1},
		unicode.Range16{Lo: 0xc8e0, Hi: 0xc8e0, Stride: 0x1},
		unicode.Range16{Lo: 0xc8fc, Hi: 0xc8fc, Stride: 0x1},
		unicode.Range16{Lo: 0xc918, Hi: 0xc918, Stride: 0x1},
		unicode.Range16{Lo: 0xc934, Hi: 0xc934, Stride: 0x1},
		unicode.Range16{Lo: 0xc950, Hi: 0xc950, Stride: 0x1},
		unicode.Range16{Lo: 0xc96c, Hi: 0xc96c, Stride: 0x1},
		unicode.Range16{Lo: 0xc988, Hi: 0xc988, Stride: 0x1},
		unicode.Range16{Lo: 0xc9a4, Hi: 0xc9a4, Stride: 0x1},
		unicode.Range16{Lo: 0xc9c0, Hi: 0xc9c0, Stride: 0x1},
		unicode.Range16{Lo: 0xc9dc, Hi: 0xc9dc, Stride: 0x1},
		unicode.Range16{Lo: 0xc9f8, Hi: 0xc9f8, Stride: 0x1},
		unicode.Range16{Lo: 0xca14, Hi: 0xca14, Stride: 0x1},
		unicode.Range16{Lo: 0xca30, Hi: 0xca30, Stride: 0x1},
		unicode.Range16{Lo: 0xca4c, Hi: 0xca4c, Stride: 0x1},
		unicode.Range16{Lo: 0xca68, Hi: 0xca68, Stride: 0x1},
		unicode.Range16{Lo: 0xca84, Hi: 0xca84, Stride: 0x1},
		unicode.Range16{Lo: 0xcaa0, Hi: 0xcaa0, Stride: 0x1},
		unicode.Range16{Lo: 0xcabc, Hi: 0xcabc, Stride: 0x1},
		unicode.Range16{Lo: 0xcad8, Hi: 0xcad8, Stride: 0x1},
		unicode.Range16{Lo: 0xcaf4, Hi: 0xcaf4, Stride: 0x1},
		unicode.Range16{Lo: 0xcb10, Hi: 0xcb10, Stride: 0x1},
		unicode.Range16{Lo: 0xcb2c, Hi: 0xcb2c, Stride: 0x1},
		unicode.Range16{Lo: 0xcb48, Hi: 0xcb48, Stride: 0x1},
		unicode.Range16{Lo: 0xcb64, Hi: 0xcb64, Stride: 0x1},
		unicode.Range16{Lo: 0xcb80, Hi: 0xcb80, Stride: 0x1},
		unicode.Range16{Lo: 0xcb9c, Hi: 0xcb9c, Stride: 0x1},
		unicode.Range16{Lo: 0xcbb8, Hi: 0xcbb8, Stride: 0x1},
		unicode.Range16{Lo: 0xcbd4, Hi: 0xcbd4, Stride: 0x1},
		unicode.Range16{Lo: 0xcbf0, Hi: 0xcbf0, Stride: 0x1},
		unicode.Range16{Lo: 0xcc0c, Hi: 0xcc0c, Stride: 0x1},
		unicode.Range16{Lo: 0xcc28, Hi: 0xcc28, Stride: 0x1},
		unicode.Range16{Lo: 0xcc44, Hi: 0xcc44, Stride: 0x1},
		unicode.Range16{Lo: 0xcc60, Hi: 0xcc60, Stride: 0x1},
		unicode.Range16{Lo: 0xcc7c, Hi: 0xcc7c, Stride: 0x1},
		unicode.Range16{Lo: 0xcc98, Hi: 0xcc98, Stride: 0x1},
		unicode.Range16{Lo: 0xccb4, Hi: 0xccb4, Stride: 0x1},
		unicode.Range16{Lo: 0xccd0, Hi: 0xccd0, Stride: 0x1},
		unicode.Range16{Lo: 0xccec, Hi: 0xccec, Stride: 0x1},
		unicode.Range16{Lo: 0xcd08, Hi: 0xcd08, Stride: 0x1},
		unicode.Range16{Lo: 0xcd24, Hi: 0xcd24, Stride: 0x1},
		unicode.Range16{Lo: 0xcd40, Hi: 0xcd40, Stride: 0x1},
		unicode.Range16{Lo: 0xcd5c, Hi: 0xcd5c, Stride: 0x1},
		unicode.Range16{Lo: 0xcd78, Hi: 0xcd78, Stride: 0x1},
		unicode.Range16{Lo: 0xcd94, Hi: 0xcd94, Stride: 0x1},
		unicode.Range16{Lo: 0xcdb0, Hi: 0xcdb0, Stride: 0x1},
		unicode.Range16{Lo: 0xcdcc, Hi: 0xcdcc, Stride: 0x1},
		unicode.Range16{Lo: 0xcde8, Hi: 0xcde8, Stride: 0x1},
		unicode.Range16{Lo: 0xce04, Hi: 0xce04, Stride: 0x1},
		unicode.Range16{Lo: 0xce20, Hi: 0xce20, Stride: 0x1},
		unicode.Range16{Lo: 0xce3c, Hi: 0xce3c, Stride: 0x1},
		unicode.Range16{Lo: 0xce58, Hi: 0xce58, Stride: 0x1},
		unicode.Range16{Lo: 0xce74, Hi: 0xce74, Stride: 0x1},
		unicode.Range16{Lo: 0xce90, Hi: 0xce90, Stride: 0x1},
		unicode.Range16{Lo: 0xceac, Hi: 0xceac, Stride: 0x1},
		unicode.Range16{Lo: 0xcec8, Hi: 0xcec8, Stride: 0x1},
		unicode.Range16{Lo: 0xcee4, Hi: 0xcee4, Stride: 0x1},
		unicode.Range16{Lo: 0xcf00, Hi: 0xcf00, Stride: 0x1},
		unicode.Range16{Lo: 0xcf1c, Hi: 0xcf1c, Stride: 0x1},
		unicode.Range16{Lo: 0xcf38, Hi: 0xcf38, Stride: 0x1},
		unicode.Range16{Lo: 0xcf54, Hi: 0xcf54, Stride: 0x1},
		unicode.Range16{Lo: 0xcf70, Hi: 0xcf70, Stride: 0x1},
		unicode.Range16{Lo: 0xcf8c, Hi: 0xcf8c, Stride: 0x1},
		unicode.Range16{Lo: 0xcfa8, Hi: 0xcfa8, Stride: 0x1},
		unicode.Range16{Lo: 0xcfc4, Hi: 0xcfc4, Stride: 0x1},
		unicode.Range16{Lo: 0xcfe0, Hi: 0xcfe0, Stride: 0x1},
		unicode.Range16{Lo: 0xcffc, Hi: 0xcffc, Stride: 0x1},
		unicode.Range16{Lo: 0xd018, Hi: 0xd018, Stride: 0x1},
		unicode.Range16{Lo: 0xd034, Hi: 0xd034, Stride: 0x1},
		unicode.Range16{Lo: 0xd050, Hi: 0xd050, Stride: 0x1},
		unicode.Range16{Lo: 0xd06c, Hi: 0xd06c, Stride: 0x1},
		unicode.Range16{Lo: 0xd088, Hi: 0xd088, Stride: 0x1},
		unicode.Range16{Lo: 0xd0a4, Hi: 0xd0a4, Stride: 0x1},
		unicode.Range16{Lo: 0xd0c0, Hi: 0xd0c0, Stride: 0x1},
		unicode.Range16{Lo: 0xd0dc, Hi: 0xd0dc, Stride: 0x1},
		unicode.Range16{Lo: 0xd0f8, Hi: 0xd0f8, Stride: 0x1},
		unicode.Range16{Lo: 0xd114, Hi: 0xd114, Stride: 0x1},
		unicode.Range16{Lo: 0xd130, Hi: 0xd130, Stride: 0x1},
		unicode.Range16{Lo: 0xd14c, Hi: 0xd14c, Stride: 0x1},
		unicode.Range16{Lo: 0xd168, Hi: 0xd168, Stride: 0x1},
		unicode.Range16{Lo: 0xd184, Hi: 0xd184, Stride: 0x1},
		unicode.Range16{Lo: 0xd1a0, Hi: 0xd1a0, Stride: 0x1},
		unicode.Range16{Lo: 0xd1bc, Hi: 0xd1bc, Stride: 0x1},
		unicode.Range16{Lo: 0xd1d8, Hi: 0xd1d8, Stride: 0x1},
		unicode.Range16{Lo: 0xd1f4, Hi: 0xd1f4, Stride: 0x1},
		unicode.Range16{Lo: 0xd210, Hi: 0xd210, Stride: 0x1},
		unicode.Range16{Lo: 0xd22c, Hi: 0xd22c, Stride: 0x1},
		unicode.Range16{Lo: 0xd248, Hi: 0xd248, Stride: 0x1},
		unicode.Range16{Lo: 0xd264, Hi: 0xd264, Stride: 0x1},
		unicode.Range16{Lo: 0xd280, Hi: 0xd280, Stride: 0x1},
		unicode.Range16{Lo: 0xd29c, Hi: 0xd29c, Stride: 0x1},
		unicode.Range16{Lo: 0xd2b8, Hi: 0xd2b8, Stride: 0x1},
		unicode.Range16{Lo: 0xd2d4, Hi: 0xd2d4, Stride: 0x1},
		unicode.Range16{Lo: 0xd2f0, Hi: 0xd2f0, Stride: 0x1},
		unicode.Range16{Lo: 0xd30c, Hi: 0xd30c, Stride: 0x1},
		unicode.Range16{Lo: 0xd328, Hi: 0xd328, Stride: 0x1},
		unicode.Range16{Lo: 0xd344, Hi: 0xd344, Stride: 0x1},
		unicode.Range16{Lo: 0xd360, Hi: 0xd360, Stride: 0x1},
		unicode.Range16{Lo: 0xd37c, Hi: 0xd37c, Stride: 0x1},
		unicode.Range16{Lo: 0xd398, Hi: 0xd398, Stride: 0x1},
		unicode.Range16{Lo: 0xd3b4, Hi: 0xd3b4, Stride: 0x1},
		unicode.Range16{Lo: 0xd3d0, Hi: 0xd3d0, Stride: 0x1},
		unicode.Range16{Lo: 0xd3ec, Hi: 0xd3ec, Stride: 0x1},
		unicode.Range16{Lo: 0xd408, Hi: 0xd408, Stride: 0x1},
		unicode.Range16{Lo: 0xd424, Hi: 0xd424, Stride: 0x1},
		unicode.Range16{Lo: 0xd440, Hi: 0xd440, Stride: 0x1},
		unicode.Range16{Lo: 0xd45c, Hi: 0xd45c, Stride: 0x1},
		unicode.Range16{Lo: 0xd478, Hi: 0xd478, Stride: 0x1},
		unicode.Range16{Lo: 0xd494, Hi: 0xd494, Stride: 0x1},
		unicode.Range16{Lo: 0xd4b0, Hi: 0xd4b0, Stride: 0x1},
		unicode.Range16{Lo: 0xd4cc, Hi: 0xd4cc, Stride: 0x1},
		unicode.Range16{Lo: 0xd4e8, Hi: 0xd4e8, Stride: 0x1},
		unicode.Range16{Lo: 0xd504, Hi: 0xd504, Stride: 0x1},
		unicode.Range16{Lo: 0xd520, Hi: 0xd520, Stride: 0x1},
		unicode.Range16{Lo: 0xd53c, Hi: 0xd53c, Stride: 0x1},
		unicode.Range16{Lo: 0xd558, Hi: 0xd558, Stride: 0x1},
		unicode.Range16{Lo: 0xd574, Hi: 0xd574, Stride: 0x1},
		unicode.Range16{Lo: 0xd590, Hi: 0xd590, Stride: 0x1},
		unicode.Range16{Lo: 0xd5ac, Hi: 0xd5ac, Stride: 0x1},
		unicode.Range16{Lo: 0xd5c8, Hi: 0xd5c8, Stride: 0x1},
		unicode.Range16{Lo: 0xd5e4, Hi: 0xd5e4, Stride: 0x1},
		unicode.Range16{Lo: 0xd600, Hi: 0xd600, Stride: 0x1},
		unicode.Range16{Lo: 0xd61c, Hi: 0xd61c, Stride: 0x1},
		unicode.Range16{Lo: 0xd638, Hi: 0xd638, Stride: 0x1},
		unicode.Range16{Lo: 0xd654, Hi: 0xd654, Stride: 0x1},
		unicode.Range16{Lo: 0xd670, Hi: 0xd670, Stride: 0x1},
		unicode.Range16{Lo: 0xd68c, Hi: 0xd68c, Stride: 0x1},
		unicode.Range16{Lo: 0xd6a8, Hi: 0xd6a8, Stride: 0x1},
		unicode.Range16{Lo: 0xd6c4, Hi: 0xd6c4, Stride: 0x1},
		unicode.Range16{Lo: 0xd6e0, Hi: 0xd6e0, Stride: 0x1},
		unicode.Range16{Lo: 0xd6fc, Hi: 0xd6fc, Stride: 0x1},
		unicode.Range16{Lo: 0xd718, Hi: 0xd718, Stride: 0x1},
		unicode.Range16{Lo: 0xd734, Hi: 0xd734, Stride: 0x1},
		unicode.Range16{Lo: 0xd750, Hi: 0xd750, Stride: 0x1},
		unicode.Range16{Lo: 0xd76c, Hi: 0xd76c, Stride: 0x1},
		unicode.Range16{Lo: 0xd788, Hi: 0xd788, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _GraphemeLVT = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0xac01, Hi: 0xac1b, Stride: 0x1},
		unicode.Range16{Lo: 0xac1d, Hi: 0xac37, Stride: 0x1},
		unicode.Range16{Lo: 0xac39, Hi: 0xac53, Stride: 0x1},
		unicode.Range16{Lo: 0xac55, Hi: 0xac6f, Stride: 0x1},
		unicode.Range16{Lo: 0xac71, Hi: 0xac8b, Stride: 0x1},
		unicode.Range16{Lo: 0xac8d, Hi: 0xaca7, Stride: 0x1},
		unicode.Range16{Lo: 0xaca9, Hi: 0xacc3, Stride: 0x1},
		unicode.Range16{Lo: 0xacc5, Hi: 0xacdf, Stride: 0x1},
		unicode.Range16{Lo: 0xace1, Hi: 0xacfb, Stride: 0x1},
		unicode.Range16{Lo: 0xacfd, Hi: 0xad17, Stride: 0x1},
		unicode.Range16{Lo: 0xad19, Hi: 0xad33, Stride: 0x1},
		unicode.Range16{Lo: 0xad35, Hi: 0xad4f, Stride: 0x1},
		unicode.Range16{Lo: 0xad51, Hi: 0xad6b, Stride: 0x1},
		unicode.Range16{Lo: 0xad6d, Hi: 0xad87, Stride: 0x1},
		unicode.Range16{Lo: 0xad89, Hi: 0xada3, Stride: 0x1},
		unicode.Range16{Lo: 0xada5, Hi: 0xadbf, Stride: 0x1},
		unicode.Range16{Lo: 0xadc1, Hi: 0xaddb, Stride: 0x1},
		unicode.Range16{Lo: 0xaddd, Hi: 0xadf7, Stride: 0x1},
		unicode.Range16{Lo: 0xadf9, Hi: 0xae13, Stride: 0x1},
		unicode.Range16{Lo: 0xae15, Hi: 0xae2f, Stride: 0x1},
		unicode.Range16{Lo: 0xae31, Hi: 0xae4b, Stride: 0x1},
		unicode.Range16{Lo: 0xae4d, Hi: 0xae67, Stride: 0x1},
		unicode.Range16{Lo: 0xae69, Hi: 0xae83, Stride: 0x1},
		unicode.Range16{Lo: 0xae85, Hi: 0xae9f, Stride: 0x1},
		unicode.Range16{Lo: 0xaea1, Hi: 0xaebb, Stride: 0x1},
		unicode.Range16{Lo: 0xaebd, Hi: 0xaed7, Stride: 0x1},
		unicode.Range16{Lo: 0xaed9, Hi: 0xaef3, Stride: 0x1},
		unicode.Range16{Lo: 0xaef5, Hi: 0xaf0f, Stride: 0x1},
		unicode.Range16{Lo: 0xaf11, Hi: 0xaf2b, Stride: 0x1},
		unicode.Range16{Lo: 0xaf2d, Hi: 0xaf47, Stride: 0x1},
		unicode.Range16{Lo: 0xaf49, Hi: 0xaf63, Stride: 0x1},
		unicode.Range16{Lo: 0xaf65, Hi: 0xaf7f, Stride: 0x1},
		unicode.Range16{Lo: 0xaf81, Hi: 0xaf9b, Stride: 0x1},
		unicode.Range16{Lo: 0xaf9d, Hi: 0xafb7, Stride: 0x1},
		unicode.Range16{Lo: 0xafb9, Hi: 0xafd3, Stride: 0x1},
		unicode.Range16{Lo: 0xafd5, Hi: 0xafef, Stride: 0x1},
		unicode.Range16{Lo: 0xaff1, Hi: 0xb00b, Stride: 0x1},
		unicode.Range16{Lo: 0xb00d, Hi: 0xb027, Stride: 0x1},
		unicode.Range16{Lo: 0xb029, Hi: 0xb043, Stride: 0x1},
		unicode.Range16{Lo: 0xb045, Hi: 0xb05f, Stride: 0x1},
		unicode.Range16{Lo: 0xb061, Hi: 0xb07b, Stride: 0x1},
		unicode.Range16{Lo: 0xb07d, Hi: 0xb097, Stride: 0x1},
		unicode.Range16{Lo: 0xb099, Hi: 0xb0b3, Stride: 0x1},
		unicode.Range16{Lo: 0xb0b5, Hi: 0xb0cf, Stride: 0x1},
		unicode.Range16{Lo: 0xb0d1, Hi: 0xb0eb, Stride: 0x1},
		unicode.Range16{Lo: 0xb0ed, Hi: 0xb107, Stride: 0x1},
		unicode.Range16{Lo: 0xb109, Hi: 0xb123, Stride: 0x1},
		unicode.Range16{Lo: 0xb125, Hi: 0xb13f, Stride: 0x1},
		unicode.Range16{Lo: 0xb141, Hi: 0xb15b, Stride: 0x1},
		unicode.Range16{Lo: 0xb15d, Hi: 0xb177, Stride: 0x1},
		unicode.Range16{Lo: 0xb179, Hi: 0xb193, Stride: 0x1},
		unicode.Range16{Lo: 0xb195, Hi: 0xb1af, Stride: 0x1},
		unicode.Range16{Lo: 0xb1b1, Hi: 0xb1cb, Stride: 0x1},
		unicode.Range16{Lo: 0xb1cd, Hi: 0xb1e7, Stride: 0x1},
		unicode.Range16{Lo: 0xb1e9, Hi: 0xb203, Stride: 0x1},
		unicode.Range16{Lo: 0xb205, Hi: 0xb21f, Stride: 0x1},
		unicode.Range16{Lo: 0xb221, Hi: 0xb23b, Stride: 0x1},
		unicode.Range16{Lo: 0xb23d, Hi: 0xb257, Stride: 0x1},
		unicode.Range16{Lo: 0xb259, Hi: 0xb273, Stride: 0x1},
		unicode.Range16{Lo: 0xb275, Hi: 0xb28f, Stride: 0x1},
		unicode.Range16{Lo: 0xb291, Hi: 0xb2ab, Stride: 0x1},
		unicode.Range16{Lo: 0xb2ad, Hi: 0xb2c7, Stride: 0x1},
		unicode.Range16{Lo: 0xb2c9, Hi: 0xb2e3, Stride: 0x1},
		unicode.Range16{Lo: 0xb2e5, Hi: 0xb2ff, Stride: 0x1},
		unicode.Range16{Lo: 0xb301, Hi: 0xb31b, Stride: 0x1},
		unicode.Range16{Lo: 0xb31d, Hi: 0xb337, Stride: 0x1},
		unicode.Range16{Lo: 0xb339, Hi: 0xb353, Stride: 0x1},
		unicode.Range16{Lo: 0xb355, Hi: 0xb36f, Stride: 0x1},
		unicode.Range16{Lo: 0xb371, Hi: 0xb38b, Stride: 0x1},
		unicode.Range16{Lo: 0xb38d, Hi: 0xb3a7, Stride: 0x1},
		unicode.Range16{Lo: 0xb3a9, Hi: 0xb3c3, Stride: 0x1},
		unicode.Range16{Lo: 0xb3c5, Hi: 0xb3df, Stride: 0x1},
		unicode.Range16{Lo: 0xb3e1, Hi: 0xb3fb, Stride: 0x1},
		unicode.Range16{Lo: 0xb3fd, Hi: 0xb417, Stride: 0x1},
		unicode.Range16{Lo: 0xb419, Hi: 0xb433, Stride: 0x1},
		unicode.Range16{Lo: 0xb435, Hi: 0xb44f, Stride: 0x1},
		unicode.Range16{Lo: 0xb451, Hi: 0xb46b, Stride: 0x1},
		unicode.Range16{Lo: 0xb46d, Hi: 0xb487, Stride: 0x1},
		unicode.Range16{Lo: 0xb489, Hi: 0xb4a3, Stride: 0x1},
		unicode.Range16{Lo: 0xb4a5, Hi: 0xb4bf, Stride: 0x1},
		unicode.Range16{Lo: 0xb4c1, Hi: 0xb4db, Stride: 0x1},
		unicode.Range16{Lo: 0xb4dd, Hi: 0xb4f7, Stride: 0x1},
		unicode.Range16{Lo: 0xb4f9, Hi: 0xb513, Stride: 0x1},
		unicode.Range16{Lo: 0xb515, Hi: 0xb52f, Stride: 0x1},
		unicode.Range16{Lo: 0xb531, Hi: 0xb54b, Stride: 0x1},
		unicode.Range16{Lo: 0xb54d, Hi: 0xb567, Stride: 0x1},
		unicode.Range16{Lo: 0xb569, Hi: 0xb583, Stride: 0x1},
		unicode.Range16{Lo: 0xb585, Hi: 0xb59f, Stride: 0x1},
		unicode.Range16{Lo: 0xb5a1, Hi: 0xb5bb, Stride: 0x1},
		unicode.Range16{Lo: 0xb5bd, Hi: 0xb5d7, Stride: 0x1},
		unicode.Range16{Lo: 0xb5d9, Hi: 0xb5f3, Stride: 0x1},
		unicode.Range16{Lo: 0xb5f5, Hi: 0xb60f, Stride: 0x1},
		unicode.Range16{Lo: 0xb611, Hi: 0xb62b, Stride: 0x1},
		unicode.Range16{Lo: 0xb62d, Hi: 0xb647, Stride: 0x1},
		unicode.Range16{Lo: 0xb649, Hi: 0xb663, Stride: 0x1},
		unicode.Range16{Lo: 0xb665, Hi: 0xb67f, Stride: 0x1},
		unicode.Range16{Lo: 0xb681, Hi: 0xb69b, Stride: 0x1},
		unicode.Range16{Lo: 0xb69d, Hi: 0xb6b7, Stride: 0x1},
		unicode.Range16{Lo: 0xb6b9, Hi: 0xb6d3, Stride: 0x1},
		unicode.Range16{Lo: 0xb6d5, Hi: 0xb6ef, Stride: 0x1},
		unicode.Range16{Lo: 0xb6f1, Hi: 0xb70b, Stride: 0x1},
		unicode.Range16{Lo: 0xb70d, Hi: 0xb727, Stride: 0x1},
		unicode.Range16{Lo: 0xb729, Hi: 0xb743, Stride: 0x1},
		unicode.Range16{Lo: 0xb745, Hi: 0xb75f, Stride: 0x1},
		unicode.Range16{Lo: 0xb761, Hi: 0xb77b, Stride: 0x1},
		unicode.Range16{Lo: 0xb77d, Hi: 0xb797, Stride: 0x1},
		unicode.Range16{Lo: 0xb799, Hi: 0xb7b3, Stride: 0x1},
		unicode.Range16{Lo: 0xb7b5, Hi: 0xb7cf, Stride: 0x1},
		unicode.Range16{Lo: 0xb7d1, Hi: 0xb7eb, Stride: 0x1},
		unicode.Range16{Lo: 0xb7ed, Hi: 0xb807, Stride: 0x1},
		unicode.Range16{Lo: 0xb809, Hi: 0xb823, Stride: 0x1},
		unicode.Range16{Lo: 0xb825, Hi: 0xb83f, Stride: 0x1},
		unicode.Range16{Lo: 0xb841, Hi: 0xb85b, Stride: 0x1},
		unicode.Range16{Lo: 0xb85d, Hi: 0xb877, Stride: 0x1},
		unicode.Range16{Lo: 0xb879, Hi: 0xb893, Stride: 0x1},
		unicode.Range16{Lo: 0xb895, Hi: 0xb8af, Stride: 0x1},
		unicode.Range16{Lo: 0xb8b1, Hi: 0xb8cb, Stride: 0x1},
		unicode.Range16{Lo: 0xb8cd, Hi: 0xb8e7, Stride: 0x1},
		unicode.Range16{Lo: 0xb8e9, Hi: 0xb903, Stride: 0x1},
		unicode.Range16{Lo: 0xb905, Hi: 0xb91f, Stride: 0x1},
		unicode.Range16{Lo: 0xb921, Hi: 0xb93b, Stride: 0x1},
		unicode.Range16{Lo: 0xb93d, Hi: 0xb957, Stride: 0x1},
		unicode.Range16{Lo: 0xb959, Hi: 0xb973, Stride: 0x1},
		unicode.Range16{Lo: 0xb975, Hi: 0xb98f, Stride: 0x1},
		unicode.Range16{Lo: 0xb991, Hi: 0xb9ab, Stride: 0x1},
		unicode.Range16{Lo: 0xb9ad, Hi: 0xb9c7, Stride: 0x1},
		unicode.Range16{Lo: 0xb9c9, Hi: 0xb9e3, Stride: 0x1},
		unicode.Range16{Lo: 0xb9e5, Hi: 0xb9ff, Stride: 0x1},
		unicode.Range16{Lo: 0xba01, Hi: 0xba1b, Stride: 0x1},
		unicode.Range16{Lo: 0xba1d, Hi: 0xba37, Stride: 0x1},
		unicode.Range16{Lo: 0xba39, Hi: 0xba53, Stride: 0x1},
		unicode.Range16{Lo: 0xba55, Hi: 0xba6f, Stride: 0x1},
		unicode.Range16{Lo: 0xba71, Hi: 0xba8b, Stride: 0x1},
		unicode.Range16{Lo: 0xba8d, Hi: 0xbaa7, Stride: 0x1},
		unicode.Range16{Lo: 0xbaa9, Hi: 0xbac3, Stride: 0x1},
		unicode.Range16{Lo: 0xbac5, Hi: 0xbadf, Stride: 0x1},
		unicode.Range16{Lo: 0xbae1, Hi: 0xbafb, Stride: 0x1},
		unicode.Range16{Lo: 0xbafd, Hi: 0xbb17, Stride: 0x1},
		unicode.Range16{Lo: 0xbb19, Hi: 0xbb33, Stride: 0x1},
		unicode.Range16{Lo: 0xbb35, Hi: 0xbb4f, Stride: 0x1},
		unicode.Range16{Lo: 0xbb51, Hi: 0xbb6b, Stride: 0x1},
		unicode.Range16{Lo: 0xbb6d, Hi: 0xbb87, Stride: 0x1},
		unicode.Range16{Lo: 0xbb89, Hi: 0xbba3, Stride: 0x1},
		unicode.Range16{Lo: 0xbba5, Hi: 0xbbbf, Stride: 0x1},
		unicode.Range16{Lo: 0xbbc1, Hi: 0xbbdb, Stride: 0x1},
		unicode.Range16{Lo: 0xbbdd, Hi: 0xbbf7, Stride: 0x1},
		unicode.Range16{Lo: 0xbbf9, Hi: 0xbc13, Stride: 0x1},
		unicode.Range16{Lo: 0xbc15, Hi: 0xbc2f, Stride: 0x1},
		unicode.Range16{Lo: 0xbc31, Hi: 0xbc4b, Stride: 0x1},
		unicode.Range16{Lo: 0xbc4d, Hi: 0xbc67, Stride: 0x1},
		unicode.Range16{Lo: 0xbc69, Hi: 0xbc83, Stride: 0x1},
		unicode.Range16{Lo: 0xbc85, Hi: 0xbc9f, Stride: 0x1},
		unicode.Range16{Lo: 0xbca1, Hi: 0xbcbb, Stride: 0x1},
		unicode.Range16{Lo: 0xbcbd, Hi: 0xbcd7, Stride: 0x1},
		unicode.Range16{Lo: 0xbcd9, Hi: 0xbcf3, Stride: 0x1},
		unicode.Range16{Lo: 0xbcf5, Hi: 0xbd0f, Stride: 0x1},
		unicode.Range16{Lo: 0xbd11, Hi: 0xbd2b, Stride: 0x1},
		unicode.Range16{Lo: 0xbd2d, Hi: 0xbd47, Stride: 0x1},
		unicode.Range16{Lo: 0xbd49, Hi: 0xbd63, Stride: 0x1},
		unicode.Range16{Lo: 0xbd65, Hi: 0xbd7f, Stride: 0x1},
		unicode.Range16{Lo: 0xbd81, Hi: 0xbd9b, Stride: 0x1},
		unicode.Range16{Lo: 0xbd9d, Hi: 0xbdb7, Stride: 0x1},
		unicode.Range16{Lo: 0xbdb9, Hi: 0xbdd3, Stride: 0x1},
		unicode.Range16{Lo: 0xbdd5, Hi: 0xbdef, Stride: 0x1},
		unicode.Range16{Lo: 0xbdf1, Hi: 0xbe0b, Stride: 0x1},
		unicode.Range16{Lo: 0xbe0d, Hi: 0xbe27, Stride: 0x1},
		unicode.Range16{Lo: 0xbe29, Hi: 0xbe43, Stride: 0x1},
		unicode.Range16{Lo: 0xbe45, Hi: 0xbe5f, Stride: 0x1},
		unicode.Range16{Lo: 0xbe61, Hi: 0xbe7b, Stride: 0x1},
		unicode.Range16{Lo: 0xbe7d, Hi: 0xbe97, Stride: 0x1},
		unicode.Range16{Lo: 0xbe99, Hi: 0xbeb3, Stride: 0x1},
		unicode.Range16{Lo: 0xbeb5, Hi: 0xbecf, Stride: 0x1},
		unicode.Range16{Lo: 0xbed1, Hi: 0xbeeb, Stride: 0x1},
		unicode.Range16{Lo: 0xbeed, Hi: 0xbf07, Stride: 0x1},
		unicode.Range16{Lo: 0xbf09, Hi: 0xbf23, Stride: 0x1},
		unicode.Range16{Lo: 0xbf25, Hi: 0xbf3f, Stride: 0x1},
		unicode.Range16{Lo: 0xbf41, Hi: 0xbf5b, Stride: 0x1},
		unicode.Range16{Lo: 0xbf5d, Hi: 0xbf77, Stride: 0x1},
		unicode.Range16{Lo: 0xbf79, Hi: 0xbf93, Stride: 0x1},
		unicode.Range16{Lo: 0xbf95, Hi: 0xbfaf, Stride: 0x1},
		unicode.Range16{Lo: 0xbfb1, Hi: 0xbfcb, Stride: 0x1},
		unicode.Range16{Lo: 0xbfcd, Hi: 0xbfe7, Stride: 0x1},
		unicode.Range16{Lo: 0xbfe9, Hi: 0xc003, Stride: 0x1},
		unicode.Range16{Lo: 0xc005, Hi: 0xc01f, Stride: 0x1},
		unicode.Range16{Lo: 0xc021, Hi: 0xc03b, Stride: 0x1},
		unicode.Range16{Lo: 0xc03d, Hi: 0xc057, Stride: 0x1},
		unicode.Range16{Lo: 0xc059, Hi: 0xc073, Stride: 0x1},
		unicode.Range16{Lo: 0xc075, Hi: 0xc08f, Stride: 0x1},
		unicode.Range16{Lo: 0xc091, Hi: 0xc0ab, Stride: 0x1},
		unicode.Range16{Lo: 0xc0ad, Hi: 0xc0c7, Stride: 0x1},
		unicode.Range16{Lo: 0xc0c9, Hi: 0xc0e3, Stride: 0x1},
		unicode.Range16{Lo: 0xc0e5, Hi: 0xc0ff, Stride: 0x1},
		unicode.Range16{Lo: 0xc101, Hi: 0xc11b, Stride: 0x1},
		unicode.Range16{Lo: 0xc11d, Hi: 0xc137, Stride: 0x1},
		unicode.Range16{Lo: 0xc139, Hi: 0xc153, Stride: 0x1},
		unicode.Range16{Lo: 0xc155, Hi: 0xc16f, Stride: 0x1},
		unicode.Range16{Lo: 0xc171, Hi: 0xc18b, Stride: 0x1},
		unicode.Range16{Lo: 0xc18d, Hi: 0xc1a7, Stride: 0x1},
		unicode.Range16{Lo: 0xc1a9, Hi: 0xc1c3, Stride: 0x1},
		unicode.Range16{Lo: 0xc1c5, Hi: 0xc1df, Stride: 0x1},
		unicode.Range16{Lo: 0xc1e1, Hi: 0xc1fb, Stride: 0x1},
		unicode.Range16{Lo: 0xc1fd, Hi: 0xc217, Stride: 0x1},
		unicode.Range16{Lo: 0xc219, Hi: 0xc233, Stride: 0x1},
		unicode.Range16{Lo: 0xc235, Hi: 0xc24f, Stride: 0x1},
		unicode.Range16{Lo: 0xc251, Hi: 0xc26b, Stride: 0x1},
		unicode.Range16{Lo: 0xc26d, Hi: 0xc287, Stride: 0x1},
		unicode.Range16{Lo: 0xc289, Hi: 0xc2a3, Stride: 0x1},
		unicode.Range16{Lo: 0xc2a5, Hi: 0xc2bf, Stride: 0x1},
		unicode.Range16{Lo: 0xc2c1, Hi: 0xc2db, Stride: 0x1},
		unicode.Range16{Lo: 0xc2dd, Hi: 0xc2f7, Stride: 0x1},
		unicode.Range16{Lo: 0xc2f9, Hi: 0xc313, Stride: 0x1},
		unicode.Range16{Lo: 0xc315, Hi: 0xc32f, Stride: 0x1},
		unicode.Range16{Lo: 0xc331, Hi: 0xc34b, Stride: 0x1},
		unicode.Range16{Lo: 0xc34d, Hi: 0xc367, Stride: 0x1},
		unicode.Range16{Lo: 0xc369, Hi: 0xc383, Stride: 0x1},
		unicode.Range16{Lo: 0xc385, Hi: 0xc39f, Stride: 0x1},
		unicode.Range16{Lo: 0xc3a1, Hi: 0xc3bb, Stride: 0x1},
		unicode.Range16{Lo: 0xc3bd, Hi: 0xc3d7, Stride: 0x1},
		unicode.Range16{Lo: 0xc3d9, Hi: 0xc3f3, Stride: 0x1},
		unicode.Range16{Lo: 0xc3f5, Hi: 0xc40f, Stride: 0x1},
		unicode.Range16{Lo: 0xc411, Hi: 0xc42b, Stride: 0x1},
		unicode.Range16{Lo: 0xc42d, Hi: 0xc447, Stride: 0x1},
		unicode.Range16{Lo: 0xc449, Hi: 0xc463, Stride: 0x1},
		unicode.Range16{Lo: 0xc465, Hi: 0xc47f, Stride: 0x1},
		unicode.Range16{Lo: 0xc481, Hi: 0xc49b, Stride: 0x1},
		unicode.Range16{Lo: 0xc49d, Hi: 0xc4b7, Stride: 0x1},
		unicode.Range16{Lo: 0xc4b9, Hi: 0xc4d3, Stride: 0x1},
		unicode.Range16{Lo: 0xc4d5, Hi: 0xc4ef, Stride: 0x1},
		unicode.Range16{Lo: 0xc4f1, Hi: 0xc50b, Stride: 0x1},
		unicode.Range16{Lo: 0xc50d, Hi: 0xc527, Stride: 0x1},
		unicode.Range16{Lo: 0xc529, Hi: 0xc543, Stride: 0x1},
		unicode.Range16{Lo: 0xc545, Hi: 0xc55f, Stride: 0x1},
		unicode.Range16{Lo: 0xc561, Hi: 0xc57b, Stride: 0x1},
		unicode.Range16{Lo: 0xc57d, Hi: 0xc597, Stride: 0x1},
		unicode.Range16{Lo: 0xc599, Hi: 0xc5b3, Stride: 0x1},
		unicode.Range16{Lo: 0xc5b5, Hi: 0xc5cf, Stride: 0x1},
		unicode.Range16{Lo: 0xc5d1, Hi: 0xc5eb, Stride: 0x1},
		unicode.Range16{Lo: 0xc5ed, Hi: 0xc607, Stride: 0x1},
		unicode.Range16{Lo: 0xc609, Hi: 0xc623, Stride: 0x1},
		unicode.Range16{Lo: 0xc625, Hi: 0xc63f, Stride: 0x1},
		unicode.Range16{Lo: 0xc641, Hi: 0xc65b, Stride: 0x1},
		unicode.Range16{Lo: 0xc65d, Hi: 0xc677, Stride: 0x1},
		unicode.Range16{Lo: 0xc679, Hi: 0xc693, Stride: 0x1},
		unicode.Range16{Lo: 0xc695, Hi: 0xc6af, Stride: 0x1},
		unicode.Range16{Lo: 0xc6b1, Hi: 0xc6cb, Stride: 0x1},
		unicode.Range16{Lo: 0xc6cd, Hi: 0xc6e7, Stride: 0x1},
		unicode.Range16{Lo: 0xc6e9, Hi: 0xc703, Stride: 0x1},
		unicode.Range16{Lo: 0xc705, Hi: 0xc71f, Stride: 0x1},
		unicode.Range16{Lo: 0xc721, Hi: 0xc73b, Stride: 0x1},
		unicode.Range16{Lo: 0xc73d, Hi: 0xc757, Stride: 0x1},
		unicode.Range16{Lo: 0xc759, Hi: 0xc773, Stride: 0x1},
		unicode.Range16{Lo: 0xc775, Hi: 0xc78f, Stride: 0x1},
		unicode.Range16{Lo: 0xc791, Hi: 0xc7ab, Stride: 0x1},
		unicode.Range16{Lo: 0xc7ad, Hi: 0xc7c7, Stride: 0x1},
		unicode.Range16{Lo: 0xc7c9, Hi: 0xc7e3, Stride: 0x1},
		unicode.Range16{Lo: 0xc7e5, Hi: 0xc7ff, Stride: 0x1},
		unicode.Range16{Lo: 0xc801, Hi: 0xc81b, Stride: 0x1},
		unicode.Range16{Lo: 0xc81d, Hi: 0xc837, Stride: 0x1},
		unicode.Range16{Lo: 0xc839, Hi: 0xc853, Stride: 0x1},
		unicode.Range16{Lo: 0xc855, Hi: 0xc86f, Stride: 0x1},
		unicode.Range16{Lo: 0xc871, Hi: 0xc88b, Stride: 0x1},
		unicode.Range16{Lo: 0xc88d, Hi: 0xc8a7, Stride: 0x1},
		unicode.Range16{Lo: 0xc8a9, Hi: 0xc8c3, Stride: 0x1},
		unicode.Range16{Lo: 0xc8c5, Hi: 0xc8df, Stride: 0x1},
		unicode.Range16{Lo: 0xc8e1, Hi: 0xc8fb, Stride: 0x1},
		unicode.Range16{Lo: 0xc8fd, Hi: 0xc917, Stride: 0x1},
		unicode.Range16{Lo: 0xc919, Hi: 0xc933, Stride: 0x1},
		unicode.Range16{Lo: 0xc935, Hi: 0xc94f, Stride: 0x1},
		unicode.Range16{Lo: 0xc951, Hi: 0xc96b, Stride: 0x1},
		unicode.Range16{Lo: 0xc96d, Hi: 0xc987, Stride: 0x1},
		unicode.Range16{Lo: 0xc989, Hi: 0xc9a3, Stride: 0x1},
		unicode.Range16{Lo: 0xc9a5, Hi: 0xc9bf, Stride: 0x1},
		unicode.Range16{Lo: 0xc9c1, Hi: 0xc9db, Stride: 0x1},
		unicode.Range16{Lo: 0xc9dd, Hi: 0xc9f7, Stride: 0x1},
		unicode.Range16{Lo: 0xc9f9, Hi: 0xca13, Stride: 0x1},
		unicode.Range16{Lo: 0xca15, Hi: 0xca2f, Stride: 0x1},
		unicode.Range16{Lo: 0xca31, Hi: 0xca4b, Stride: 0x1},
		unicode.Range16{Lo: 0xca4d, Hi: 0xca67, Stride: 0x1},
		unicode.Range16{Lo: 0xca69, Hi: 0xca83, Stride: 0x1},
		unicode.Range16{Lo: 0xca85, Hi: 0xca9f, Stride: 0x1},
		unicode.Range16{Lo: 0xcaa1, Hi: 0xcabb, Stride: 0x1},
		unicode.Range16{Lo: 0xcabd, Hi: 0xcad7, Stride: 0x1},
		unicode.Range16{Lo: 0xcad9, Hi: 0xcaf3, Stride: 0x1},
		unicode.Range16{Lo: 0xcaf5, Hi: 0xcb0f, Stride: 0x1},
		unicode.Range16{Lo: 0xcb11, Hi: 0xcb2b, Stride: 0x1},
		unicode.Range16{Lo: 0xcb2d, Hi: 0xcb47, Stride: 0x1},
		unicode.Range16{Lo: 0xcb49, Hi: 0xcb63, Stride: 0x1},
		unicode.Range16{Lo: 0xcb65, Hi: 0xcb7f, Stride: 0x1},
		unicode.Range16{Lo: 0xcb81, Hi: 0xcb9b, Stride: 0x1},
		unicode.Range16{Lo: 0xcb9d, Hi: 0xcbb7, Stride: 0x1},
		unicode.Range16{Lo: 0xcbb9, Hi: 0xcbd3, Stride: 0x1},
		unicode.Range16{Lo: 0xcbd5, Hi: 0xcbef, Stride: 0x1},
		unicode.Range16{Lo: 0xcbf1, Hi: 0xcc0b, Stride: 0x1},
		unicode.Range16{Lo: 0xcc0d, Hi: 0xcc27, Stride: 0x1},
		unicode.Range16{Lo: 0xcc29, Hi: 0xcc43, Stride: 0x1},
		unicode.Range16{Lo: 0xcc45, Hi: 0xcc5f, Stride: 0x1},
		unicode.Range16{Lo: 0xcc61, Hi: 0xcc7b, Stride: 0x1},
		unicode.Range16{Lo: 0xcc7d, Hi: 0xcc97, Stride: 0x1},
		unicode.Range16{Lo: 0xcc99, Hi: 0xccb3, Stride: 0x1},
		unicode.Range16{Lo: 0xccb5, Hi: 0xcccf, Stride: 0x1},
		unicode.Range16{Lo: 0xccd1, Hi: 0xcceb, Stride: 0x1},
		unicode.Range16{Lo: 0xcced, Hi: 0xcd07, Stride: 0x1},
		unicode.Range16{Lo: 0xcd09, Hi: 0xcd23, Stride: 0x1},
		unicode.Range16{Lo: 0xcd25, Hi: 0xcd3f, Stride: 0x1},
		unicode.Range16{Lo: 0xcd41, Hi: 0xcd5b, Stride: 0x1},
		unicode.Range16{Lo: 0xcd5d, Hi: 0xcd77, Stride: 0x1},
		unicode.Range16{Lo: 0xcd79, Hi: 0xcd93, Stride: 0x1},
		unicode.Range16{Lo: 0xcd95, Hi: 0xcdaf, Stride: 0x1},
		unicode.Range16{Lo: 0xcdb1, Hi: 0xcdcb, Stride: 0x1},
		unicode.Range16{Lo: 0xcdcd, Hi: 0xcde7, Stride: 0x1},
		unicode.Range16{Lo: 0xcde9, Hi: 0xce03, Stride: 0x1},
		unicode.Range16{Lo: 0xce05, Hi: 0xce1f, Stride: 0x1},
		unicode.Range16{Lo: 0xce21, Hi: 0xce3b, Stride: 0x1},
		unicode.Range16{Lo: 0xce3d, Hi: 0xce57, Stride: 0x1},
		unicode.Range16{Lo: 0xce59, Hi: 0xce73, Stride: 0x1},
		unicode.Range16{Lo: 0xce75, Hi: 0xce8f, Stride: 0x1},
		unicode.Range16{Lo: 0xce91, Hi: 0xceab, Stride: 0x1},
		unicode.Range16{Lo: 0xcead, Hi: 0xcec7, Stride: 0x1},
		unicode.Range16{Lo: 0xcec9, Hi: 0xcee3, Stride: 0x1},
		unicode.Range16{Lo: 0xcee5, Hi: 0xceff, Stride: 0x1},
		unicode.Range16{Lo: 0xcf01, Hi: 0xcf1b, Stride: 0x1},
		unicode.Range16{Lo: 0xcf1d, Hi: 0xcf37, Stride: 0x1},
		unicode.Range16{Lo: 0xcf39, Hi: 0xcf53, Stride: 0x1},
		unicode.Range16{Lo: 0xcf55, Hi: 0xcf6f, Stride: 0x1},
		unicode.Range16{Lo: 0xcf71, Hi: 0xcf8b, Stride: 0x1},
		unicode.Range16{Lo: 0xcf8d, Hi: 0xcfa7, Stride: 0x1},
		unicode.Range16{Lo: 0xcfa9, Hi: 0xcfc3, Stride: 0x1},
		unicode.Range16{Lo: 0xcfc5, Hi: 0xcfdf, Stride: 0x1},
		unicode.Range16{Lo: 0xcfe1, Hi: 0xcffb, Stride: 0x1},
		unicode.Range16{Lo: 0xcffd, Hi: 0xd017, Stride: 0x1},
		unicode.Range16{Lo: 0xd019, Hi: 0xd033, Stride: 0x1},
		unicode.Range16{Lo: 0xd035, Hi: 0xd04f, Stride: 0x1},
		unicode.Range16{Lo: 0xd051, Hi: 0xd06b, Stride: 0x1},
		unicode.Range16{Lo: 0xd06d, Hi: 0xd087, Stride: 0x1},
		unicode.Range16{Lo: 0xd089, Hi: 0xd0a3, Stride: 0x1},
		unicode.Range16{Lo: 0xd0a5, Hi: 0xd0bf, Stride: 0x1},
		unicode.Range16{Lo: 0xd0c1, Hi: 0xd0db, Stride: 0x1},
		unicode.Range16{Lo: 0xd0dd, Hi: 0xd0f7, Stride: 0x1},
		unicode.Range16{Lo: 0xd0f9, Hi: 0xd113, Stride: 0x1},
		unicode.Range16{Lo: 0xd115, Hi: 0xd12f, Stride: 0x1},
		unicode.Range16{Lo: 0xd131, Hi: 0xd14b, Stride: 0x1},
		unicode.Range16{Lo: 0xd14d, Hi: 0xd167, Stride: 0x1},
		unicode.Range16{Lo: 0xd169, Hi: 0xd183, Stride: 0x1},
		unicode.Range16{Lo: 0xd185, Hi: 0xd19f, Stride: 0x1},
		unicode.Range16{Lo: 0xd1a1, Hi: 0xd1bb, Stride: 0x1},
		unicode.Range16{Lo: 0xd1bd, Hi: 0xd1d7, Stride: 0x1},
		unicode.Range16{Lo: 0xd1d9, Hi: 0xd1f3, Stride: 0x1},
		unicode.Range16{Lo: 0xd1f5, Hi: 0xd20f, Stride: 0x1},
		unicode.Range16{Lo: 0xd211, Hi: 0xd22b, Stride: 0x1},
		unicode.Range16{Lo: 0xd22d, Hi: 0xd247, Stride: 0x1},
		unicode.Range16{Lo: 0xd249, Hi: 0xd263, Stride: 0x1},
		unicode.Range16{Lo: 0xd265, Hi: 0xd27f, Stride: 0x1},
		unicode.Range16{Lo: 0xd281, Hi: 0xd29b, Stride: 0x1},
		unicode.Range16{Lo: 0xd29d, Hi: 0xd2b7, Stride: 0x1},
		unicode.Range16{Lo: 0xd2b9, Hi: 0xd2d3, Stride: 0x1},
		unicode.Range16{Lo: 0xd2d5, Hi: 0xd2ef, Stride: 0x1},
		unicode.Range16{Lo: 0xd2f1, Hi: 0xd30b, Stride: 0x1},
		unicode.Range16{Lo: 0xd30d, Hi: 0xd327, Stride: 0x1},
		unicode.Range16{Lo: 0xd329, Hi: 0xd343, Stride: 0x1},
		unicode.Range16{Lo: 0xd345, Hi: 0xd35f, Stride: 0x1},
		unicode.Range16{Lo: 0xd361, Hi: 0xd37b, Stride: 0x1},
		unicode.Range16{Lo: 0xd37d, Hi: 0xd397, Stride: 0x1},
		unicode.Range16{Lo: 0xd399, Hi: 0xd3b3, Stride: 0x1},
		unicode.Range16{Lo: 0xd3b5, Hi: 0xd3cf, Stride: 0x1},
		unicode.Range16{Lo: 0xd3d1, Hi: 0xd3eb, Stride: 0x1},
		unicode.Range16{Lo: 0xd3ed, Hi: 0xd407, Stride: 0x1},
		unicode.Range16{Lo: 0xd409, Hi: 0xd423, Stride: 0x1},
		unicode.Range16{Lo: 0xd425, Hi: 0xd43f, Stride: 0x1},
		unicode.Range16{Lo: 0xd441, Hi: 0xd45b, Stride: 0x1},
		unicode.Range16{Lo: 0xd45d, Hi: 0xd477, Stride: 0x1},
		unicode.Range16{Lo: 0xd479, Hi: 0xd493, Stride: 0x1},
		unicode.Range16{Lo: 0xd495, Hi: 0xd4af, Stride: 0x1},
		unicode.Range16{Lo: 0xd4b1, Hi: 0xd4cb, Stride: 0x1},
		unicode.Range16{Lo: 0xd4cd, Hi: 0xd4e7, Stride: 0x1},
		unicode.Range16{Lo: 0xd4e9, Hi: 0xd503, Stride: 0x1},
		unicode.Range16{Lo: 0xd505, Hi: 0xd51f, Stride: 0x1},
		unicode.Range16{Lo: 0xd521, Hi: 0xd53b, Stride: 0x1},
		unicode.Range16{Lo: 0xd53d, Hi: 0xd557, Stride: 0x1},
		unicode.Range16{Lo: 0xd559, Hi: 0xd573, Stride: 0x1},
		unicode.Range16{Lo: 0xd575, Hi: 0xd58f, Stride: 0x1},
		unicode.Range16{Lo: 0xd591, Hi: 0xd5ab, Stride: 0x1},
		unicode.Range16{Lo: 0xd5ad, Hi: 0xd5c7, Stride: 0x1},
		unicode.Range16{Lo: 0xd5c9, Hi: 0xd5e3, Stride: 0x1},
		unicode.Range16{Lo: 0xd5e5, Hi: 0xd5ff, Stride: 0x1},
		unicode.Range16{Lo: 0xd601, Hi: 0xd61b, Stride: 0x1},
		unicode.Range16{Lo: 0xd61d, Hi: 0xd637, Stride: 0x1},
		unicode.Range16{Lo: 0xd639, Hi: 0xd653, Stride: 0x1},
		unicode.Range16{Lo: 0xd655, Hi: 0xd66f, Stride: 0x1},
		unicode.Range16{Lo: 0xd671, Hi: 0xd68b, Stride: 0x1},
		unicode.Range16{Lo: 0xd68d, Hi: 0xd6a7, Stride: 0x1},
		unicode.Range16{Lo: 0xd6a9, Hi: 0xd6c3, Stride: 0x1},
		unicode.Range16{Lo: 0xd6c5, Hi: 0xd6df, Stride: 0x1},
		unicode.Range16{Lo: 0xd6e1, Hi: 0xd6fb, Stride: 0x1},
		unicode.Range16{Lo: 0xd6fd, Hi: 0xd717, Stride: 0x1},
		unicode.Range16{Lo: 0xd719, Hi: 0xd733, Stride: 0x1},
		unicode.Range16{Lo: 0xd735, Hi: 0xd74f, Stride: 0x1},
		unicode.Range16{Lo: 0xd751, Hi: 0xd76b, Stride: 0x1},
		unicode.Range16{Lo: 0xd76d, Hi: 0xd787, Stride: 0x1},
		unicode.Range16{Lo: 0xd789, Hi: 0xd7a3, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _GraphemePrepend = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x600, Hi: 0x605, Stride: 0x1},
		unicode.Range16{Lo: 0x6dd, Hi: 0x6dd, Stride: 0x1},
		unicode.Range16{Lo: 0x70f, Hi: 0x70f, Stride: 0x1},
		unicode.Range16{Lo: 0x890, Hi: 0x891, Stride: 0x1},
		unicode.Range16{Lo: 0x8e2, Hi: 0x8e2, Stride: 0x1},
		unicode.Range16{Lo: 0xd4e, Hi: 0xd4e, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x110bd, Hi: 0x110bd, Stride: 0x1},
		unicode.Range32{Lo: 0x110cd, Hi: 0x110cd, Stride: 0x1},
		unicode.Range32{Lo: 0x111c2, Hi: 0x111c3, Stride: 0x1},
		unicode.Range32{Lo: 0x1193f, Hi: 0x1193f, Stride: 0x1},
		unicode.Range32{Lo: 0x11941, Hi: 0x11941, Stride: 0x1},
		unicode.Range32{Lo: 0x11a3a, Hi: 0x11a3a, Stride: 0x1},
		unicode.Range32{Lo: 0x11a84, Hi: 0x11a89, Stride: 0x1},
		unicode.Range32{Lo: 0x11d46, Hi: 0x11d46, Stride: 0x1},
		unicode.Range32{Lo: 0x11f02, Hi: 0x11f02, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _GraphemeRegional_Indicator = &unicode.RangeTable{
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x1f1e6, Hi: 0x1f1ff, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _GraphemeSpacingMark = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x903, Hi: 0x903, Stride: 0x1},
		unicode.Range16{Lo: 0x93b, Hi: 0x93b, Stride: 0x1},
		unicode.Range16{Lo: 0x93e, Hi: 0x940, Stride: 0x1},
		unicode.Range16{Lo: 0x949, Hi: 0x94c, Stride: 0x1},
		unicode.Range16{Lo: 0x94e, Hi: 0x94f, Stride: 0x1},
		unicode.Range16{Lo: 0x982, Hi: 0x983, Stride: 0x1},
		unicode.Range16{Lo: 0x9bf, Hi: 0x9c0, Stride: 0x1},
		unicode.Range16{Lo: 0x9c7, Hi: 0x9c8, Stride: 0x1},
		unicode.Range16{Lo: 0x9cb, Hi: 0x9cc, Stride: 0x1},
		unicode.Range16{Lo: 0xa03, Hi: 0xa03, Stride: 0x1},
		unicode.Range16{Lo: 0xa3e, Hi: 0xa40, Stride: 0x1},
		unicode.Range16{Lo: 0xa83, Hi: 0xa83, Stride: 0x1},
		unicode.Range16{Lo: 0xabe, Hi: 0xac0, Stride: 0x1},
		unicode.Range16{Lo: 0xac9, Hi: 0xac9, Stride: 0x1},
		unicode.Range16{Lo: 0xacb, Hi: 0xacc, Stride: 0x1},
		unicode.Range16{Lo: 0xb02, Hi: 0xb03, Stride: 0x1},
		unicode.Range16{Lo: 0xb40, Hi: 0xb40, Stride: 0x1},
		unicode.Range16{Lo: 0xb47, Hi: 0xb48, Stride: 0x1},
		unicode.Range16{Lo: 0xb4b, Hi: 0xb4c, Stride: 0x1},
		unicode.Range16{Lo: 0xbbf, Hi: 0xbbf, Stride: 0x1},
		unicode.Range16{Lo: 0xbc1, Hi: 0xbc2, Stride: 0x1},
		unicode.Range16{Lo: 0xbc6, Hi: 0xbc8, Stride: 0x1},
		unicode.Range16{Lo: 0xbca, Hi: 0xbcc, Stride: 0x1},
		unicode.Range16{Lo: 0xc01, Hi: 0xc03, Stride: 0x1},
		unicode.Range16{Lo: 0xc41, Hi: 0xc44, Stride: 0x1},
		unicode.Range16{Lo: 0xc82, Hi: 0xc83, Stride: 0x1},
		unicode.Range16{Lo: 0xcbe, Hi: 0xcbe, Stride: 0x1},
		unicode.Range16{Lo: 0xcc0, Hi: 0xcc1, Stride: 0x1},
		unicode.Range16{Lo: 0xcc3, Hi: 0xcc4, Stride: 0x1},
		unicode.Range16{Lo: 0xcc7, Hi: 0xcc8, Stride: 0x1},
		unicode.Range16{Lo: 0xcca, Hi: 0xccb, Stride: 0x1},
		unicode.Range16{Lo: 0xcf3, Hi: 0xcf3, Stride: 0x1},
		unicode.Range16{Lo: 0xd02, Hi: 0xd03, Stride: 0x1},
		unicode.Range16{Lo: 0xd3f, Hi: 0xd40, Stride: 0x1},
		unicode.Range16{Lo: 0xd46, Hi: 0xd48, Stride: 0x1},
		unicode.Range16{Lo: 0xd4a, Hi: 0xd4c, Stride: 0x1},
		unicode.Range16{Lo: 0xd82, Hi: 0xd83, Stride: 0x1},
		unicode.Range16{Lo: 0xdd0, Hi: 0xdd1, Stride: 0x1},
		unicode.Range16{Lo: 0xdd8, Hi: 0xdde, Stride: 0x1},
		unicode.Range16{Lo: 0xdf2, Hi: 0xdf3, Stride: 0x1},
		unicode.Range16{Lo: 0xe33, Hi: 0xe33, Stride: 0x1},
		unicode.Range16{Lo: 0xeb3, Hi: 0xeb3, Stride: 0x1},
		unicode.Range16{Lo: 0xf3e, Hi: 0xf3f, Stride: 0x1},
		unicode.Range16{Lo: 0xf7f, Hi: 0xf7f, Stride: 0x1},
		unicode.Range16{Lo: 0x1031, Hi: 0x1031, Stride: 0x1},
		unicode.Range16{Lo: 0x103b, Hi: 0x103c, Stride: 0x1},
		unicode.Range16{Lo: 0x1056, Hi: 0x1057, Stride: 0x1},
		unicode.Range16{Lo: 0x1084, Hi: 0x1084, Stride: 0x1},
		unicode.Range16{Lo: 0x1715, Hi: 0x1715, Stride: 0x1},
		unicode.Range16{Lo: 0x1734, Hi: 0x1734, Stride: 0x1},
		unicode.Range16{Lo: 0x17b6, Hi: 0x17b6, Stride: 0x1},
		unicode.Range16{Lo: 0x17be, Hi: 0x17c5, Stride: 0x1},
		unicode.Range16{Lo: 0x17c7, Hi: 0x17c8, Stride: 0x1},
		unicode.Range16{Lo: 0x1923, Hi: 0x1926, Stride: 0x1},
		unicode.Range16{Lo: 0x1929, Hi: 0x192b, Stride: 0x1},
		unicode.Range16{Lo: 0x1930, Hi: 0x1931, Stride: 0x1},
		unicode.Range16{Lo: 0x1933, Hi: 0x1938, Stride: 0x1},
		unicode.Range16{Lo: 0x1a19, Hi: 0x1a1a, Stride: 0x1},
		unicode.Range16{Lo: 0x1a55, Hi: 0x1a55, Stride: 0x1},
		unicode.Range16{Lo: 0x1a57, Hi: 0x1a57, Stride: 0x1},
		unicode.Range16{Lo: 0x1a6d, Hi: 0x1a72, Stride: 0x1},
		unicode.Range16{Lo: 0x1b04, Hi: 0x1b04, Stride: 0x1},
		unicode.Range16{Lo: 0x1b3b, Hi: 0x1b3b, Stride: 0x1},
		unicode.Range16{Lo: 0x1b3d, Hi: 0x1b41, Stride: 0x1},
		unicode.Range16{Lo: 0x1b43, Hi: 0x1b44, Stride: 0x1},
		unicode.Range16{Lo: 0x1b82, Hi: 0x1b82, Stride: 0x1},
		unicode.Range16{Lo: 0x1ba1, Hi: 0x1ba1, Stride: 0x1},
		unicode.Range16{Lo: 0x1ba6, Hi: 0x1ba7, Stride: 0x1},
		unicode.Range16{Lo: 0x1baa, Hi: 0x1baa, Stride: 0x1},
		unicode.Range16{Lo: 0x1be7, Hi: 0x1be7, Stride: 0x1},
		unicode.Range16{Lo: 0x1bea, Hi: 0x1bec, Stride: 0x1},
		unicode.Range16{Lo: 0x1bee, Hi: 0x1bee, Stride: 0x1},
		unicode.Range16{Lo: 0x1bf2, Hi: 0x1bf3, Stride: 0x1},
		unicode.Range16{Lo: 0x1c24, Hi: 0x1c2b, Stride: 0x1},
		unicode.Range16{Lo: 0x1c34, Hi: 0x1c35, Stride: 0x1},
		unicode.Range16{Lo: 0x1ce1, Hi: 0x1ce1, Stride: 0x1},
		unicode.Range16{Lo: 0x1cf7, Hi: 0x1cf7, Stride: 0x1},
		unicode.Range16{Lo: 0xa823, Hi: 0xa824, Stride: 0x1},
		unicode.Range16{Lo: 0xa827, Hi: 0xa827, Stride: 0x1},
		unicode.Range16{Lo: 0xa880, Hi: 0xa881, Stride: 0x1},
		unicode.Range16{Lo: 0xa8b4, Hi: 0xa8c3, Stride: 0x1},
		unicode.Range16{Lo: 0xa952, Hi: 0xa953, Stride: 0x1},
		unicode.Range16{Lo: 0xa983, Hi: 0xa983, Stride: 0x1},
		unicode.Range16{Lo: 0xa9b4, Hi: 0xa9b5, Stride: 0x1},
		unicode.Range16{Lo: 0xa9ba, Hi: 0xa9bb, Stride: 0x1},
		unicode.Range16{Lo: 0xa9be, Hi: 0xa9c0, Stride: 0x1},
		unicode.Range16{Lo: 0xaa2f, Hi: 0xaa30, Stride: 0x1},
		unicode.Range16{Lo: 0xaa33, Hi: 0xaa34, Stride: 0x1},
		unicode.Range16{Lo: 0xaa4d, Hi: 0xaa4d, Stride: 0x1},
		unicode.Range16{Lo: 0xaaeb, Hi: 0xaaeb, Stride: 0x1},
		unicode.Range16{Lo: 0xaaee, Hi: 0xaaef, Stride: 0x1},
		unicode.Range16{Lo: 0xaaf5, Hi: 0xaaf5, Stride: 0x1},
		unicode.Range16{Lo: 0xabe3, Hi: 0xabe4, Stride: 0x1},
		unicode.Range16{Lo: 0xabe6, Hi: 0xabe7, Stride: 0x1},
		unicode.Range16{Lo: 0xabe9, Hi: 0xabea, Stride: 0x1},
		unicode.Range16{Lo: 0xabec, Hi: 0xabec, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x11000, Hi: 0x11000, Stride: 0x1},
		unicode.Range32{Lo: 0x11002, Hi: 0x11002, Stride: 0x1},
		unicode.Range32{Lo: 0x11082, Hi: 0x11082, Stride: 0x1},
		unicode.Range32{Lo: 0x110b0, Hi: 0x110b2, Stride: 0x1},
		unicode.Range32{Lo: 0x110b7, Hi: 0x110b8, Stride: 0x1},
		unicode.Range32{Lo: 0x1112c, Hi: 0x1112c, Stride: 0x1},
		unicode.Range32{Lo: 0x11145, Hi: 0x11146, Stride: 0x1},
		unicode.Range32{Lo: 0x11182, Hi: 0x11182, Stride: 0x1},
		unicode.Range32{Lo: 0x111b3, Hi: 0x111b5, Stride: 0x1},
		unicode.Range32{Lo: 0x111bf, Hi: 0x111c0, Stride: 0x1},
		unicode.Range32{Lo: 0x111ce, Hi: 0x111ce, Stride: 0x1},
		unicode.Range32{Lo: 0x1122c, Hi: 0x1122e, Stride: 0x1},
		unicode.Range32{Lo: 0x11232, Hi: 0x11233, Stride: 0x1},
		unicode.Range32{Lo: 0x11235, Hi: 0x11235, Stride: 0x1},
		unicode.Range32{Lo: 0x112e0, Hi: 0x112e2, Stride: 0x1},
		unicode.Range32{Lo: 0x11302, Hi: 0x11303, Stride: 0x1},
		unicode.Range32{Lo: 0x1133f, Hi: 0x1133f, Stride: 0x1},
		unicode.Range32{Lo: 0x11341, Hi: 0x11344, Stride: 0x1},
		unicode.Range32{Lo: 0x11347, Hi: 0x11348, Stride: 0x1},
		unicode.Range32{Lo: 0x1134b, Hi: 0x1134d, Stride: 0x1},
		unicode.Range32{Lo: 0x11362, Hi: 0x11363, Stride: 0x1},
		unicode.Range32{Lo: 0x11435, Hi: 0x11437, Stride: 0x1},
		unicode.Range32{Lo: 0x11440, Hi: 0x11441, Stride: 0x1},
		unicode.Range32{Lo: 0x11445, Hi: 0x11445, Stride: 0x1},
		unicode.Range32{Lo: 0x114b1, Hi: 0x114b2, Stride: 0x1},
		unicode.Range32{Lo: 0x114b9, Hi: 0x114b9, Stride: 0x1},
		unicode.Range32{Lo: 0x114bb, Hi: 0x114bc, Stride: 0x1},
		unicode.Range32{Lo: 0x114be, Hi: 0x114be, Stride: 0x1},
		unicode.Range32{Lo: 0x114c1, Hi: 0x114c1, Stride: 0x1},
		unicode.Range32{Lo: 0x115b0, Hi: 0x115b1, Stride: 0x1},
		unicode.Range32{Lo: 0x115b8, Hi: 0x115bb, Stride: 0x1},
		unicode.Range32{Lo: 0x115be, Hi: 0x115be, Stride: 0x1},
		unicode.Range32{Lo: 0x11630, Hi: 0x11632, Stride: 0x1},
		unicode.Range32{Lo: 0x1163b, Hi: 0x1163c, Stride: 0x1},
		unicode.Range32{Lo: 0x1163e, Hi: 0x1163e, Stride: 0x1},
		unicode.Range32{Lo: 0x116ac, Hi: 0x116ac, Stride: 0x1},
		unicode.Range32{Lo: 0x116ae, Hi: 0x116af, Stride: 0x1},
		unicode.Range32{Lo: 0x116b6, Hi: 0x116b6, Stride: 0x1},
		unicode.Range32{Lo: 0x11726, Hi: 0x11726, Stride: 0x1},
		unicode.Range32{Lo: 0x1182c, Hi: 0x1182e, Stride: 0x1},
		unicode.Range32{Lo: 0x11838, Hi: 0x11838, Stride: 0x1},
		unicode.Range32{Lo: 0x11931, Hi: 0x11935, Stride: 0x1},
		unicode.Range32{Lo: 0x11937, Hi: 0x11938, Stride: 0x1},
		unicode.Range32{Lo: 0x1193d, Hi: 0x1193d, Stride: 0x1},
		unicode.Range32{Lo: 0x11940, Hi: 0x11940, Stride: 0x1},
		unicode.Range32{Lo: 0x11942, Hi: 0x11942, Stride: 0x1},
		unicode.Range32{Lo: 0x119d1, Hi: 0x119d3, Stride: 0x1},
		unicode.Range32{Lo: 0x119dc, Hi: 0x119df, Stride: 0x1},
		unicode.Range32{Lo: 0x119e4, Hi: 0x119e4, Stride: 0x1},
		unicode.Range32{Lo: 0x11a39, Hi: 0x11a39, Stride: 0x1},
		unicode.Range32{Lo: 0x11a57, Hi: 0x11a58, Stride: 0x1},
		unicode.Range32{Lo: 0x11a97, Hi: 0x11a97, Stride: 0x1},
		unicode.Range32{Lo: 0x11c2f, Hi: 0x11c2f, Stride: 0x1},
		unicode.Range32{Lo: 0x11c3e, Hi: 0x11c3e, Stride: 0x1},
		unicode.Range32{Lo: 0x11ca9, Hi: 0x11ca9, Stride: 0x1},
		unicode.Range32{Lo: 0x11cb1, Hi: 0x11cb1, Stride: 0x1},
		unicode.Range32{Lo: 0x11cb4, Hi: 0x11cb4, Stride: 0x1},
		unicode.Range32{Lo: 0x11d8a, Hi: 0x11d8e, Stride: 0x1},
		unicode.Range32{Lo: 0x11d93, Hi: 0x11d94, Stride: 0x1},
		unicode.Range32{Lo: 0x11d96, Hi: 0x11d96, Stride: 0x1},
		unicode.Range32{Lo: 0x11ef5, Hi: 0x11ef6, Stride: 0x1},
		unicode.Range32{Lo: 0x11f03, Hi: 0x11f03, Stride: 0x1},
		unicode.Range32{Lo: 0x11f34, Hi: 0x11f35, Stride: 0x1},
		unicode.Range32{Lo: 0x11f3e, Hi: 0x11f3f, Stride: 0x1},
		unicode.Range32{Lo: 0x11f41, Hi: 0x11f41, Stride: 0x1},
		unicode.Range32{Lo: 0x16f51, Hi: 0x16f87, Stride: 0x1},
		unicode.Range32{Lo: 0x16ff0, Hi: 0x16ff1, Stride: 0x1},
		unicode.Range32{Lo: 0x1d166, Hi: 0x1d166, Stride: 0x1},
		unicode.Range32{Lo: 0x1d16d, Hi: 0x1d16d, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _GraphemeT = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x11a8, Hi: 0x11ff, Stride: 0x1},
		unicode.Range16{Lo: 0xd7cb, Hi: 0xd7fb, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _GraphemeV = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x1160, Hi: 0x11a7, Stride: 0x1},
		unicode.Range16{Lo: 0xd7b0, Hi: 0xd7c6, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _GraphemeZWJ = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x200d, Hi: 0x200d, Stride: 0x1},
	},
	LatinOffset: 0,
}

type _GraphemeRuneRange unicode.RangeTable

func _GraphemeRuneType(r rune) *_GraphemeRuneRange {
	switch {
	case unicode.Is(_GraphemeCR, r):
		return (*_GraphemeRuneRange)(_GraphemeCR)
	case unicode.Is(_GraphemeControl, r):
		return (*_GraphemeRuneRange)(_GraphemeControl)
	case unicode.Is(_GraphemeExtend, r):
		return (*_GraphemeRuneRange)(_GraphemeExtend)
	case unicode.Is(_GraphemeL, r):
		return (*_GraphemeRuneRange)(_GraphemeL)
	case unicode.Is(_GraphemeLF, r):
		return (*_GraphemeRuneRange)(_GraphemeLF)
	case unicode.Is(_GraphemeLV, r):
		return (*_GraphemeRuneRange)(_GraphemeLV)
	case unicode.Is(_GraphemeLVT, r):
		return (*_GraphemeRuneRange)(_GraphemeLVT)
	case unicode.Is(_GraphemePrepend, r):
		return (*_GraphemeRuneRange)(_GraphemePrepend)
	case unicode.Is(_GraphemeRegional_Indicator, r):
		return (*_GraphemeRuneRange)(_GraphemeRegional_Indicator)
	case unicode.Is(_GraphemeSpacingMark, r):
		return (*_GraphemeRuneRange)(_GraphemeSpacingMark)
	case unicode.Is(_GraphemeT, r):
		return (*_GraphemeRuneRange)(_GraphemeT)
	case unicode.Is(_GraphemeV, r):
		return (*_GraphemeRuneRange)(_GraphemeV)
	case unicode.Is(_GraphemeZWJ, r):
		return (*_GraphemeRuneRange)(_GraphemeZWJ)
	default:
		return nil
	}
}
func (rng *_GraphemeRuneRange) String() string {
	switch (*unicode.RangeTable)(rng) {
	case _GraphemeCR:
		return "CR"
	case _GraphemeControl:
		return "Control"
	case _GraphemeExtend:
		return "Extend"
	case _GraphemeL:
		return "L"
	case _GraphemeLF:
		return "LF"
	case _GraphemeLV:
		return "LV"
	case _GraphemeLVT:
		return "LVT"
	case _GraphemePrepend:
		return "Prepend"
	case _GraphemeRegional_Indicator:
		return "Regional_Indicator"
	case _GraphemeSpacingMark:
		return "SpacingMark"
	case _GraphemeT:
		return "T"
	case _GraphemeV:
		return "V"
	case _GraphemeZWJ:
		return "ZWJ"
	default:
		return "Other"
	}
}

var _WordALetter = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x41, Hi: 0x5a, Stride: 0x1},
		unicode.Range16{Lo: 0x61, Hi: 0x7a, Stride: 0x1},
		unicode.Range16{Lo: 0xaa, Hi: 0xaa, Stride: 0x1},
		unicode.Range16{Lo: 0xb5, Hi: 0xb5, Stride: 0x1},
		unicode.Range16{Lo: 0xba, Hi: 0xba, Stride: 0x1},
		unicode.Range16{Lo: 0xc0, Hi: 0xd6, Stride: 0x1},
		unicode.Range16{Lo: 0xd8, Hi: 0xf6, Stride: 0x1},
		unicode.Range16{Lo: 0xf8, Hi: 0x1ba, Stride: 0x1},
		unicode.Range16{Lo: 0x1bb, Hi: 0x1bb, Stride: 0x1},
		unicode.Range16{Lo: 0x1bc, Hi: 0x1bf, Stride: 0x1},
		unicode.Range16{Lo: 0x1c0, Hi: 0x1c3, Stride: 0x1},
		unicode.Range16{Lo: 0x1c4, Hi: 0x293, Stride: 0x1},
		unicode.Range16{Lo: 0x294, Hi: 0x294, Stride: 0x1},
		unicode.Range16{Lo: 0x295, Hi: 0x2af, Stride: 0x1},
		unicode.Range16{Lo: 0x2b0, Hi: 0x2c1, Stride: 0x1},
		unicode.Range16{Lo: 0x2c2, Hi: 0x2c5, Stride: 0x1},
		unicode.Range16{Lo: 0x2c6, Hi: 0x2d1, Stride: 0x1},
		unicode.Range16{Lo: 0x2d2, Hi: 0x2d7, Stride: 0x1},
		unicode.Range16{Lo: 0x2de, Hi: 0x2df, Stride: 0x1},
		unicode.Range16{Lo: 0x2e0, Hi: 0x2e4, Stride: 0x1},
		unicode.Range16{Lo: 0x2e5, Hi: 0x2eb, Stride: 0x1},
		unicode.Range16{Lo: 0x2ec, Hi: 0x2ec, Stride: 0x1},
		unicode.Range16{Lo: 0x2ed, Hi: 0x2ed, Stride: 0x1},
		unicode.Range16{Lo: 0x2ee, Hi: 0x2ee, Stride: 0x1},
		unicode.Range16{Lo: 0x2ef, Hi: 0x2ff, Stride: 0x1},
		unicode.Range16{Lo: 0x370, Hi: 0x373, Stride: 0x1},
		unicode.Range16{Lo: 0x374, Hi: 0x374, Stride: 0x1},
		unicode.Range16{Lo: 0x376, Hi: 0x377, Stride: 0x1},
		unicode.Range16{Lo: 0x37a, Hi: 0x37a, Stride: 0x1},
		unicode.Range16{Lo: 0x37b, Hi: 0x37d, Stride: 0x1},
		unicode.Range16{Lo: 0x37f, Hi: 0x37f, Stride: 0x1},
		unicode.Range16{Lo: 0x386, Hi: 0x386, Stride: 0x1},
		unicode.Range16{Lo: 0x388, Hi: 0x38a, Stride: 0x1},
		unicode.Range16{Lo: 0x38c, Hi: 0x38c, Stride: 0x1},
		unicode.Range16{Lo: 0x38e, Hi: 0x3a1, Stride: 0x1},
		unicode.Range16{Lo: 0x3a3, Hi: 0x3f5, Stride: 0x1},
		unicode.Range16{Lo: 0x3f7, Hi: 0x481, Stride: 0x1},
		unicode.Range16{Lo: 0x48a, Hi: 0x52f, Stride: 0x1},
		unicode.Range16{Lo: 0x531, Hi: 0x556, Stride: 0x1},
		unicode.Range16{Lo: 0x559, Hi: 0x559, Stride: 0x1},
		unicode.Range16{Lo: 0x55a, Hi: 0x55c, Stride: 0x1},
		unicode.Range16{Lo: 0x55e, Hi: 0x55e, Stride: 0x1},
		unicode.Range16{Lo: 0x560, Hi: 0x588, Stride: 0x1},
		unicode.Range16{Lo: 0x58a, Hi: 0x58a, Stride: 0x1},
		unicode.Range16{Lo: 0x5f3, Hi: 0x5f3, Stride: 0x1},
		unicode.Range16{Lo: 0x620, Hi: 0x63f, Stride: 0x1},
		unicode.Range16{Lo: 0x640, Hi: 0x640, Stride: 0x1},
		unicode.Range16{Lo: 0x641, Hi: 0x64a, Stride: 0x1},
		unicode.Range16{Lo: 0x66e, Hi: 0x66f, Stride: 0x1},
		unicode.Range16{Lo: 0x671, Hi: 0x6d3, Stride: 0x1},
		unicode.Range16{Lo: 0x6d5, Hi: 0x6d5, Stride: 0x1},
		unicode.Range16{Lo: 0x6e5, Hi: 0x6e6, Stride: 0x1},
		unicode.Range16{Lo: 0x6ee, Hi: 0x6ef, Stride: 0x1},
		unicode.Range16{Lo: 0x6fa, Hi: 0x6fc, Stride: 0x1},
		unicode.Range16{Lo: 0x6ff, Hi: 0x6ff, Stride: 0x1},
		unicode.Range16{Lo: 0x710, Hi: 0x710, Stride: 0x1},
		unicode.Range16{Lo: 0x712, Hi: 0x72f, Stride: 0x1},
		unicode.Range16{Lo: 0x74d, Hi: 0x7a5, Stride: 0x1},
		unicode.Range16{Lo: 0x7b1, Hi: 0x7b1, Stride: 0x1},
		unicode.Range16{Lo: 0x7ca, Hi: 0x7ea, Stride: 0x1},
		unicode.Range16{Lo: 0x7f4, Hi: 0x7f5, Stride: 0x1},
		unicode.Range16{Lo: 0x7fa, Hi: 0x7fa, Stride: 0x1},
		unicode.Range16{Lo: 0x800, Hi: 0x815, Stride: 0x1},
		unicode.Range16{Lo: 0x81a, Hi: 0x81a, Stride: 0x1},
		unicode.Range16{Lo: 0x824, Hi: 0x824, Stride: 0x1},
		unicode.Range16{Lo: 0x828, Hi: 0x828, Stride: 0x1},
		unicode.Range16{Lo: 0x840, Hi: 0x858, Stride: 0x1},
		unicode.Range16{Lo: 0x860, Hi: 0x86a, Stride: 0x1},
		unicode.Range16{Lo: 0x870, Hi: 0x887, Stride: 0x1},
		unicode.Range16{Lo: 0x889, Hi: 0x88e, Stride: 0x1},
		unicode.Range16{Lo: 0x8a0, Hi: 0x8c8, Stride: 0x1},
		unicode.Range16{Lo: 0x8c9, Hi: 0x8c9, Stride: 0x1},
		unicode.Range16{Lo: 0x904, Hi: 0x939, Stride: 0x1},
		unicode.Range16{Lo: 0x93d, Hi: 0x93d, Stride: 0x1},
		unicode.Range16{Lo: 0x950, Hi: 0x950, Stride: 0x1},
		unicode.Range16{Lo: 0x958, Hi: 0x961, Stride: 0x1},
		unicode.Range16{Lo: 0x971, Hi: 0x971, Stride: 0x1},
		unicode.Range16{Lo: 0x972, Hi: 0x980, Stride: 0x1},
		unicode.Range16{Lo: 0x985, Hi: 0x98c, Stride: 0x1},
		unicode.Range16{Lo: 0x98f, Hi: 0x990, Stride: 0x1},
		unicode.Range16{Lo: 0x993, Hi: 0x9a8, Stride: 0x1},
		unicode.Range16{Lo: 0x9aa, Hi: 0x9b0, Stride: 0x1},
		unicode.Range16{Lo: 0x9b2, Hi: 0x9b2, Stride: 0x1},
		unicode.Range16{Lo: 0x9b6, Hi: 0x9b9, Stride: 0x1},
		unicode.Range16{Lo: 0x9bd, Hi: 0x9bd, Stride: 0x1},
		unicode.Range16{Lo: 0x9ce, Hi: 0x9ce, Stride: 0x1},
		unicode.Range16{Lo: 0x9dc, Hi: 0x9dd, Stride: 0x1},
		unicode.Range16{Lo: 0x9df, Hi: 0x9e1, Stride: 0x1},
		unicode.Range16{Lo: 0x9f0, Hi: 0x9f1, Stride: 0x1},
		unicode.Range16{Lo: 0x9fc, Hi: 0x9fc, Stride: 0x1},
		unicode.Range16{Lo: 0xa05, Hi: 0xa0a, Stride: 0x1},
		unicode.Range16{Lo: 0xa0f, Hi: 0xa10, Stride: 0x1},
		unicode.Range16{Lo: 0xa13, Hi: 0xa28, Stride: 0x1},
		unicode.Range16{Lo: 0xa2a, Hi: 0xa30, Stride: 0x1},
		unicode.Range16{Lo: 0xa32, Hi: 0xa33, Stride: 0x1},
		unicode.Range16{Lo: 0xa35, Hi: 0xa36, Stride: 0x1},
		unicode.Range16{Lo: 0xa38, Hi: 0xa39, Stride: 0x1},
		unicode.Range16{Lo: 0xa59, Hi: 0xa5c, Stride: 0x1},
		unicode.Range16{Lo: 0xa5e, Hi: 0xa5e, Stride: 0x1},
		unicode.Range16{Lo: 0xa72, Hi: 0xa74, Stride: 0x1},
		unicode.Range16{Lo: 0xa85, Hi: 0xa8d, Stride: 0x1},
		unicode.Range16{Lo: 0xa8f, Hi: 0xa91, Stride: 0x1},
		unicode.Range16{Lo: 0xa93, Hi: 0xaa8, Stride: 0x1},
		unicode.Range16{Lo: 0xaaa, Hi: 0xab0, Stride: 0x1},
		unicode.Range16{Lo: 0xab2, Hi: 0xab3, Stride: 0x1},
		unicode.Range16{Lo: 0xab5, Hi: 0xab9, Stride: 0x1},
		unicode.Range16{Lo: 0xabd, Hi: 0xabd, Stride: 0x1},
		unicode.Range16{Lo: 0xad0, Hi: 0xad0, Stride: 0x1},
		unicode.Range16{Lo: 0xae0, Hi: 0xae1, Stride: 0x1},
		unicode.Range16{Lo: 0xaf9, Hi: 0xaf9, Stride: 0x1},
		unicode.Range16{Lo: 0xb05, Hi: 0xb0c, Stride: 0x1},
		unicode.Range16{Lo: 0xb0f, Hi: 0xb10, Stride: 0x1},
		unicode.Range16{Lo: 0xb13, Hi: 0xb28, Stride: 0x1},
		unicode.Range16{Lo: 0xb2a, Hi: 0xb30, Stride: 0x1},
		unicode.Range16{Lo: 0xb32, Hi: 0xb33, Stride: 0x1},
		unicode.Range16{Lo: 0xb35, Hi: 0xb39, Stride: 0x1},
		unicode.Range16{Lo: 0xb3d, Hi: 0xb3d, Stride: 0x1},
		unicode.Range16{Lo: 0xb5c, Hi: 0xb5d, Stride: 0x1},
		unicode.Range16{Lo: 0xb5f, Hi: 0xb61, Stride: 0x1},
		unicode.Range16{Lo: 0xb71, Hi: 0xb71, Stride: 0x1},
		unicode.Range16{Lo: 0xb83, Hi: 0xb83, Stride: 0x1},
		unicode.Range16{Lo: 0xb85, Hi: 0xb8a, Stride: 0x1},
		unicode.Range16{Lo: 0xb8e, Hi: 0xb90, Stride: 0x1},
		unicode.Range16{Lo: 0xb92, Hi: 0xb95, Stride: 0x1},
		unicode.Range16{Lo: 0xb99, Hi: 0xb9a, Stride: 0x1},
		unicode.Range16{Lo: 0xb9c, Hi: 0xb9c, Stride: 0x1},
		unicode.Range16{Lo: 0xb9e, Hi: 0xb9f, Stride: 0x1},
		unicode.Range16{Lo: 0xba3, Hi: 0xba4, Stride: 0x1},
		unicode.Range16{Lo: 0xba8, Hi: 0xbaa, Stride: 0x1},
		unicode.Range16{Lo: 0xbae, Hi: 0xbb9, Stride: 0x1},
		unicode.Range16{Lo: 0xbd0, Hi: 0xbd0, Stride: 0x1},
		unicode.Range16{Lo: 0xc05, Hi: 0xc0c, Stride: 0x1},
		unicode.Range16{Lo: 0xc0e, Hi: 0xc10, Stride: 0x1},
		unicode.Range16{Lo: 0xc12, Hi: 0xc28, Stride: 0x1},
		unicode.Range16{Lo: 0xc2a, Hi: 0xc39, Stride: 0x1},
		unicode.Range16{Lo: 0xc3d, Hi: 0xc3d, Stride: 0x1},
		unicode.Range16{Lo: 0xc58, Hi: 0xc5a, Stride: 0x1},
		unicode.Range16{Lo: 0xc5d, Hi: 0xc5d, Stride: 0x1},
		unicode.Range16{Lo: 0xc60, Hi: 0xc61, Stride: 0x1},
		unicode.Range16{Lo: 0xc80, Hi: 0xc80, Stride: 0x1},
		unicode.Range16{Lo: 0xc85, Hi: 0xc8c, Stride: 0x1},
		unicode.Range16{Lo: 0xc8e, Hi: 0xc90, Stride: 0x1},
		unicode.Range16{Lo: 0xc92, Hi: 0xca8, Stride: 0x1},
		unicode.Range16{Lo: 0xcaa, Hi: 0xcb3, Stride: 0x1},
		unicode.Range16{Lo: 0xcb5, Hi: 0xcb9, Stride: 0x1},
		unicode.Range16{Lo: 0xcbd, Hi: 0xcbd, Stride: 0x1},
		unicode.Range16{Lo: 0xcdd, Hi: 0xcde, Stride: 0x1},
		unicode.Range16{Lo: 0xce0, Hi: 0xce1, Stride: 0x1},
		unicode.Range16{Lo: 0xcf1, Hi: 0xcf2, Stride: 0x1},
		unicode.Range16{Lo: 0xd04, Hi: 0xd0c, Stride: 0x1},
		unicode.Range16{Lo: 0xd0e, Hi: 0xd10, Stride: 0x1},
		unicode.Range16{Lo: 0xd12, Hi: 0xd3a, Stride: 0x1},
		unicode.Range16{Lo: 0xd3d, Hi: 0xd3d, Stride: 0x1},
		unicode.Range16{Lo: 0xd4e, Hi: 0xd4e, Stride: 0x1},
		unicode.Range16{Lo: 0xd54, Hi: 0xd56, Stride: 0x1},
		unicode.Range16{Lo: 0xd5f, Hi: 0xd61, Stride: 0x1},
		unicode.Range16{Lo: 0xd7a, Hi: 0xd7f, Stride: 0x1},
		unicode.Range16{Lo: 0xd85, Hi: 0xd96, Stride: 0x1},
		unicode.Range16{Lo: 0xd9a, Hi: 0xdb1, Stride: 0x1},
		unicode.Range16{Lo: 0xdb3, Hi: 0xdbb, Stride: 0x1},
		unicode.Range16{Lo: 0xdbd, Hi: 0xdbd, Stride: 0x1},
		unicode.Range16{Lo: 0xdc0, Hi: 0xdc6, Stride: 0x1},
		unicode.Range16{Lo: 0xf00, Hi: 0xf00, Stride: 0x1},
		unicode.Range16{Lo: 0xf40, Hi: 0xf47, Stride: 0x1},
		unicode.Range16{Lo: 0xf49, Hi: 0xf6c, Stride: 0x1},
		unicode.Range16{Lo: 0xf88, Hi: 0xf8c, Stride: 0x1},
		unicode.Range16{Lo: 0x10a0, Hi: 0x10c5, Stride: 0x1},
		unicode.Range16{Lo: 0x10c7, Hi: 0x10c7, Stride: 0x1},
		unicode.Range16{Lo: 0x10cd, Hi: 0x10cd, Stride: 0x1},
		unicode.Range16{Lo: 0x10d0, Hi: 0x10fa, Stride: 0x1},
		unicode.Range16{Lo: 0x10fc, Hi: 0x10fc, Stride: 0x1},
		unicode.Range16{Lo: 0x10fd, Hi: 0x10ff, Stride: 0x1},
		unicode.Range16{Lo: 0x1100, Hi: 0x1248, Stride: 0x1},
		unicode.Range16{Lo: 0x124a, Hi: 0x124d, Stride: 0x1},
		unicode.Range16{Lo: 0x1250, Hi: 0x1256, Stride: 0x1},
		unicode.Range16{Lo: 0x1258, Hi: 0x1258, Stride: 0x1},
		unicode.Range16{Lo: 0x125a, Hi: 0x125d, Stride: 0x1},
		unicode.Range16{Lo: 0x1260, Hi: 0x1288, Stride: 0x1},
		unicode.Range16{Lo: 0x128a, Hi: 0x128d, Stride: 0x1},
		unicode.Range16{Lo: 0x1290, Hi: 0x12b0, Stride: 0x1},
		unicode.Range16{Lo: 0x12b2, Hi: 0x12b5, Stride: 0x1},
		unicode.Range16{Lo: 0x12b8, Hi: 0x12be, Stride: 0x1},
		unicode.Range16{Lo: 0x12c0, Hi: 0x12c0, Stride: 0x1},
		unicode.Range16{Lo: 0x12c2, Hi: 0x12c5, Stride: 0x1},
		unicode.Range16{Lo: 0x12c8, Hi: 0x12d6, Stride: 0x1},
		unicode.Range16{Lo: 0x12d8, Hi: 0x1310, Stride: 0x1},
		unicode.Range16{Lo: 0x1312, Hi: 0x1315, Stride: 0x1},
		unicode.Range16{Lo: 0x1318, Hi: 0x135a, Stride: 0x1},
		unicode.Range16{Lo: 0x1380, Hi: 0x138f, Stride: 0x1},
		unicode.Range16{Lo: 0x13a0, Hi: 0x13f5, Stride: 0x1},
		unicode.Range16{Lo: 0x13f8, Hi: 0x13fd, Stride: 0x1},
		unicode.Range16{Lo: 0x1401, Hi: 0x166c, Stride: 0x1},
		unicode.Range16{Lo: 0x166f, Hi: 0x167f, Stride: 0x1},
		unicode.Range16{Lo: 0x1681, Hi: 0x169a, Stride: 0x1},
		unicode.Range16{Lo: 0x16a0, Hi: 0x16ea, Stride: 0x1},
		unicode.Range16{Lo: 0x16ee, Hi: 0x16f0, Stride: 0x1},
		unicode.Range16{Lo: 0x16f1, Hi: 0x16f8, Stride: 0x1},
		unicode.Range16{Lo: 0x1700, Hi: 0x1711, Stride: 0x1},
		unicode.Range16{Lo: 0x171f, Hi: 0x1731, Stride: 0x1},
		unicode.Range16{Lo: 0x1740, Hi: 0x1751, Stride: 0x1},
		unicode.Range16{Lo: 0x1760, Hi: 0x176c, Stride: 0x1},
		unicode.Range16{Lo: 0x176e, Hi: 0x1770, Stride: 0x1},
		unicode.Range16{Lo: 0x1820, Hi: 0x1842, Stride: 0x1},
		unicode.Range16{Lo: 0x1843, Hi: 0x1843, Stride: 0x1},
		unicode.Range16{Lo: 0x1844, Hi: 0x1878, Stride: 0x1},
		unicode.Range16{Lo: 0x1880, Hi: 0x1884, Stride: 0x1},
		unicode.Range16{Lo: 0x1887, Hi: 0x18a8, Stride: 0x1},
		unicode.Range16{Lo: 0x18aa, Hi: 0x18aa, Stride: 0x1},
		unicode.Range16{Lo: 0x18b0, Hi: 0x18f5, Stride: 0x1},
		unicode.Range16{Lo: 0x1900, Hi: 0x191e, Stride: 0x1},
		unicode.Range16{Lo: 0x1a00, Hi: 0x1a16, Stride: 0x1},
		unicode.Range16{Lo: 0x1b05, Hi: 0x1b33, Stride: 0x1},
		unicode.Range16{Lo: 0x1b45, Hi: 0x1b4c, Stride: 0x1},
		unicode.Range16{Lo: 0x1b83, Hi: 0x1ba0, Stride: 0x1},
		unicode.Range16{Lo: 0x1bae, Hi: 0x1baf, Stride: 0x1},
		unicode.Range16{Lo: 0x1bba, Hi: 0x1be5, Stride: 0x1},
		unicode.Range16{Lo: 0x1c00, Hi: 0x1c23, Stride: 0x1},
		unicode.Range16{Lo: 0x1c4d, Hi: 0x1c4f, Stride: 0x1},
		unicode.Range16{Lo: 0x1c5a, Hi: 0x1c77, Stride: 0x1},
		unicode.Range16{Lo: 0x1c78, Hi: 0x1c7d, Stride: 0x1},
		unicode.Range16{Lo: 0x1c80, Hi: 0x1c88, Stride: 0x1},
		unicode.Range16{Lo: 0x1c90, Hi: 0x1cba, Stride: 0x1},
		unicode.Range16{Lo: 0x1cbd, Hi: 0x1cbf, Stride: 0x1},
		unicode.Range16{Lo: 0x1ce9, Hi: 0x1cec, Stride: 0x1},
		unicode.Range16{Lo: 0x1cee, Hi: 0x1cf3, Stride: 0x1},
		unicode.Range16{Lo: 0x1cf5, Hi: 0x1cf6, Stride: 0x1},
		unicode.Range16{Lo: 0x1cfa, Hi: 0x1cfa, Stride: 0x1},
		unicode.Range16{Lo: 0x1d00, Hi: 0x1d2b, Stride: 0x1},
		unicode.Range16{Lo: 0x1d2c, Hi: 0x1d6a, Stride: 0x1},
		unicode.Range16{Lo: 0x1d6b, Hi: 0x1d77, Stride: 0x1},
		unicode.Range16{Lo: 0x1d78, Hi: 0x1d78, Stride: 0x1},
		unicode.Range16{Lo: 0x1d79, Hi: 0x1d9a, Stride: 0x1},
		unicode.Range16{Lo: 0x1d9b, Hi: 0x1dbf, Stride: 0x1},
		unicode.Range16{Lo: 0x1e00, Hi: 0x1f15, Stride: 0x1},
		unicode.Range16{Lo: 0x1f18, Hi: 0x1f1d, Stride: 0x1},
		unicode.Range16{Lo: 0x1f20, Hi: 0x1f45, Stride: 0x1},
		unicode.Range16{Lo: 0x1f48, Hi: 0x1f4d, Stride: 0x1},
		unicode.Range16{Lo: 0x1f50, Hi: 0x1f57, Stride: 0x1},
		unicode.Range16{Lo: 0x1f59, Hi: 0x1f59, Stride: 0x1},
		unicode.Range16{Lo: 0x1f5b, Hi: 0x1f5b, Stride: 0x1},
		unicode.Range16{Lo: 0x1f5d, Hi: 0x1f5d, Stride: 0x1},
		unicode.Range16{Lo: 0x1f5f, Hi: 0x1f7d, Stride: 0x1},
		unicode.Range16{Lo: 0x1f80, Hi: 0x1fb4, Stride: 0x1},
		unicode.Range16{Lo: 0x1fb6, Hi: 0x1fbc, Stride: 0x1},
		unicode.Range16{Lo: 0x1fbe, Hi: 0x1fbe, Stride: 0x1},
		unicode.Range16{Lo: 0x1fc2, Hi: 0x1fc4, Stride: 0x1},
		unicode.Range16{Lo: 0x1fc6, Hi: 0x1fcc, Stride: 0x1},
		unicode.Range16{Lo: 0x1fd0, Hi: 0x1fd3, Stride: 0x1},
		unicode.Range16{Lo: 0x1fd6, Hi: 0x1fdb, Stride: 0x1},
		unicode.Range16{Lo: 0x1fe0, Hi: 0x1fec, Stride: 0x1},
		unicode.Range16{Lo: 0x1ff2, Hi: 0x1ff4, Stride: 0x1},
		unicode.Range16{Lo: 0x1ff6, Hi: 0x1ffc, Stride: 0x1},
		unicode.Range16{Lo: 0x2071, Hi: 0x2071, Stride: 0x1},
		unicode.Range16{Lo: 0x207f, Hi: 0x207f, Stride: 0x1},
		unicode.Range16{Lo: 0x2090, Hi: 0x209c, Stride: 0x1},
		unicode.Range16{Lo: 0x2102, Hi: 0x2102, Stride: 0x1},
		unicode.Range16{Lo: 0x2107, Hi: 0x2107, Stride: 0x1},
		unicode.Range16{Lo: 0x210a, Hi: 0x2113, Stride: 0x1},
		unicode.Range16{Lo: 0x2115, Hi: 0x2115, Stride: 0x1},
		unicode.Range16{Lo: 0x2119, Hi: 0x211d, Stride: 0x1},
		unicode.Range16{Lo: 0x2124, Hi: 0x2124, Stride: 0x1},
		unicode.Range16{Lo: 0x2126, Hi: 0x2126, Stride: 0x1},
		unicode.Range16{Lo: 0x2128, Hi: 0x2128, Stride: 0x1},
		unicode.Range16{Lo: 0x212a, Hi: 0x212d, Stride: 0x1},
		unicode.Range16{Lo: 0x212f, Hi: 0x2134, Stride: 0x1},
		unicode.Range16{Lo: 0x2135, Hi: 0x2138, Stride: 0x1},
		unicode.Range16{Lo: 0x2139, Hi: 0x2139, Stride: 0x1},
		unicode.Range16{Lo: 0x213c, Hi: 0x213f, Stride: 0x1},
		unicode.Range16{Lo: 0x2145, Hi: 0x2149, Stride: 0x1},
		unicode.Range16{Lo: 0x214e, Hi: 0x214e, Stride: 0x1},
		unicode.Range16{Lo: 0x2160, Hi: 0x2182, Stride: 0x1},
		unicode.Range16{Lo: 0x2183, Hi: 0x2184, Stride: 0x1},
		unicode.Range16{Lo: 0x2185, Hi: 0x2188, Stride: 0x1},
		unicode.Range16{Lo: 0x24b6, Hi: 0x24e9, Stride: 0x1},
		unicode.Range16{Lo: 0x2c00, Hi: 0x2c7b, Stride: 0x1},
		unicode.Range16{Lo: 0x2c7c, Hi: 0x2c7d, Stride: 0x1},
		unicode.Range16{Lo: 0x2c7e, Hi: 0x2ce4, Stride: 0x1},
		unicode.Range16{Lo: 0x2ceb, Hi: 0x2cee, Stride: 0x1},
		unicode.Range16{Lo: 0x2cf2, Hi: 0x2cf3, Stride: 0x1},
		unicode.Range16{Lo: 0x2d00, Hi: 0x2d25, Stride: 0x1},
		unicode.Range16{Lo: 0x2d27, Hi: 0x2d27, Stride: 0x1},
		unicode.Range16{Lo: 0x2d2d, Hi: 0x2d2d, Stride: 0x1},
		unicode.Range16{Lo: 0x2d30, Hi: 0x2d67, Stride: 0x1},
		unicode.Range16{Lo: 0x2d6f, Hi: 0x2d6f, Stride: 0x1},
		unicode.Range16{Lo: 0x2d80, Hi: 0x2d96, Stride: 0x1},
		unicode.Range16{Lo: 0x2da0, Hi: 0x2da6, Stride: 0x1},
		unicode.Range16{Lo: 0x2da8, Hi: 0x2dae, Stride: 0x1},
		unicode.Range16{Lo: 0x2db0, Hi: 0x2db6, Stride: 0x1},
		unicode.Range16{Lo: 0x2db8, Hi: 0x2dbe, Stride: 0x1},
		unicode.Range16{Lo: 0x2dc0, Hi: 0x2dc6, Stride: 0x1},
		unicode.Range16{Lo: 0x2dc8, Hi: 0x2dce, Stride: 0x1},
		unicode.Range16{Lo: 0x2dd0, Hi: 0x2dd6, Stride: 0x1},
		unicode.Range16{Lo: 0x2dd8, Hi: 0x2dde, Stride: 0x1},
		unicode.Range16{Lo: 0x2e2f, Hi: 0x2e2f, Stride: 0x1},
		unicode.Range16{Lo: 0x3005, Hi: 0x3005, Stride: 0x1},
		unicode.Range16{Lo: 0x303b, Hi: 0x303b, Stride: 0x1},
		unicode.Range16{Lo: 0x303c, Hi: 0x303c, Stride: 0x1},
		unicode.Range16{Lo: 0x3105, Hi: 0x312f, Stride: 0x1},
		unicode.Range16{Lo: 0x3131, Hi: 0x318e, Stride: 0x1},
		unicode.Range16{Lo: 0x31a0, Hi: 0x31bf, Stride: 0x1},
		unicode.Range16{Lo: 0xa000, Hi: 0xa014, Stride: 0x1},
		unicode.Range16{Lo: 0xa015, Hi: 0xa015, Stride: 0x1},
		unicode.Range16{Lo: 0xa016, Hi: 0xa48c, Stride: 0x1},
		unicode.Range16{Lo: 0xa4d0, Hi: 0xa4f7, Stride: 0x1},
		unicode.Range16{Lo: 0xa4f8, Hi: 0xa4fd, Stride: 0x1},
		unicode.Range16{Lo: 0xa500, Hi: 0xa60b, Stride: 0x1},
		unicode.Range16{Lo: 0xa60c, Hi: 0xa60c, Stride: 0x1},
		unicode.Range16{Lo: 0xa610, Hi: 0xa61f, Stride: 0x1},
		unicode.Range16{Lo: 0xa62a, Hi: 0xa62b, Stride: 0x1},
		unicode.Range16{Lo: 0xa640, Hi: 0xa66d, Stride: 0x1},
		unicode.Range16{Lo: 0xa66e, Hi: 0xa66e, Stride: 0x1},
		unicode.Range16{Lo: 0xa67f, Hi: 0xa67f, Stride: 0x1},
		unicode.Range16{Lo: 0xa680, Hi: 0xa69b, Stride: 0x1},
		unicode.Range16{Lo: 0xa69c, Hi: 0xa69d, Stride: 0x1},
		unicode.Range16{Lo: 0xa6a0, Hi: 0xa6e5, Stride: 0x1},
		unicode.Range16{Lo: 0xa6e6, Hi: 0xa6ef, Stride: 0x1},
		unicode.Range16{Lo: 0xa708, Hi: 0xa716, Stride: 0x1},
		unicode.Range16{Lo: 0xa717, Hi: 0xa71f, Stride: 0x1},
		unicode.Range16{Lo: 0xa720, Hi: 0xa721, Stride: 0x1},
		unicode.Range16{Lo: 0xa722, Hi: 0xa76f, Stride: 0x1},
		unicode.Range16{Lo: 0xa770, Hi: 0xa770, Stride: 0x1},
		unicode.Range16{Lo: 0xa771, Hi: 0xa787, Stride: 0x1},
		unicode.Range16{Lo: 0xa788, Hi: 0xa788, Stride: 0x1},
		unicode.Range16{Lo: 0xa789, Hi: 0xa78a, Stride: 0x1},
		unicode.Range16{Lo: 0xa78b, Hi: 0xa78e, Stride: 0x1},
		unicode.Range16{Lo: 0xa78f, Hi: 0xa78f, Stride: 0x1},
		unicode.Range16{Lo: 0xa790, Hi: 0xa7ca, Stride: 0x1},
		unicode.Range16{Lo: 0xa7d0, Hi: 0xa7d1, Stride: 0x1},
		unicode.Range16{Lo: 0xa7d3, Hi: 0xa7d3, Stride: 0x1},
		unicode.Range16{Lo: 0xa7d5, Hi: 0xa7d9, Stride: 0x1},
		unicode.Range16{Lo: 0xa7f2, Hi: 0xa7f4, Stride: 0x1},
		unicode.Range16{Lo: 0xa7f5, Hi: 0xa7f6, Stride: 0x1},
		unicode.Range16{Lo: 0xa7f7, Hi: 0xa7f7, Stride: 0x1},
		unicode.Range16{Lo: 0xa7f8, Hi: 0xa7f9, Stride: 0x1},
		unicode.Range16{Lo: 0xa7fa, Hi: 0xa7fa, Stride: 0x1},
		unicode.Range16{Lo: 0xa7fb, Hi: 0xa801, Stride: 0x1},
		unicode.Range16{Lo: 0xa803, Hi: 0xa805, Stride: 0x1},
		unicode.Range16{Lo: 0xa807, Hi: 0xa80a, Stride: 0x1},
		unicode.Range16{Lo: 0xa80c, Hi: 0xa822, Stride: 0x1},
		unicode.Range16{Lo: 0xa840, Hi: 0xa873, Stride: 0x1},
		unicode.Range16{Lo: 0xa882, Hi: 0xa8b3, Stride: 0x1},
		unicode.Range16{Lo: 0xa8f2, Hi: 0xa8f7, Stride: 0x1},
		unicode.Range16{Lo: 0xa8fb, Hi: 0xa8fb, Stride: 0x1},
		unicode.Range16{Lo: 0xa8fd, Hi: 0xa8fe, Stride: 0x1},
		unicode.Range16{Lo: 0xa90a, Hi: 0xa925, Stride: 0x1},
		unicode.Range16{Lo: 0xa930, Hi: 0xa946, Stride: 0x1},
		unicode.Range16{Lo: 0xa960, Hi: 0xa97c, Stride: 0x1},
		unicode.Range16{Lo: 0xa984, Hi: 0xa9b2, Stride: 0x1},
		unicode.Range16{Lo: 0xa9cf, Hi: 0xa9cf, Stride: 0x1},
		unicode.Range16{Lo: 0xaa00, Hi: 0xaa28, Stride: 0x1},
		unicode.Range16{Lo: 0xaa40, Hi: 0xaa42, Stride: 0x1},
		unicode.Range16{Lo: 0xaa44, Hi: 0xaa4b, Stride: 0x1},
		unicode.Range16{Lo: 0xaae0, Hi: 0xaaea, Stride: 0x1},
		unicode.Range16{Lo: 0xaaf2, Hi: 0xaaf2, Stride: 0x1},
		unicode.Range16{Lo: 0xaaf3, Hi: 0xaaf4, Stride: 0x1},
		unicode.Range16{Lo: 0xab01, Hi: 0xab06, Stride: 0x1},
		unicode.Range16{Lo: 0xab09, Hi: 0xab0e, Stride: 0x1},
		unicode.Range16{Lo: 0xab11, Hi: 0xab16, Stride: 0x1},
		unicode.Range16{Lo: 0xab20, Hi: 0xab26, Stride: 0x1},
		unicode.Range16{Lo: 0xab28, Hi: 0xab2e, Stride: 0x1},
		unicode.Range16{Lo: 0xab30, Hi: 0xab5a, Stride: 0x1},
		unicode.Range16{Lo: 0xab5b, Hi: 0xab5b, Stride: 0x1},
		unicode.Range16{Lo: 0xab5c, Hi: 0xab5f, Stride: 0x1},
		unicode.Range16{Lo: 0xab60, Hi: 0xab68, Stride: 0x1},
		unicode.Range16{Lo: 0xab69, Hi: 0xab69, Stride: 0x1},
		unicode.Range16{Lo: 0xab70, Hi: 0xabbf, Stride: 0x1},
		unicode.Range16{Lo: 0xabc0, Hi: 0xabe2, Stride: 0x1},
		unicode.Range16{Lo: 0xac00, Hi: 0xd7a3, Stride: 0x1},
		unicode.Range16{Lo: 0xd7b0, Hi: 0xd7c6, Stride: 0x1},
		unicode.Range16{Lo: 0xd7cb, Hi: 0xd7fb, Stride: 0x1},
		unicode.Range16{Lo: 0xfb00, Hi: 0xfb06, Stride: 0x1},
		unicode.Range16{Lo: 0xfb13, Hi: 0xfb17, Stride: 0x1},
		unicode.Range16{Lo: 0xfb50, Hi: 0xfbb1, Stride: 0x1},
		unicode.Range16{Lo: 0xfbd3, Hi: 0xfd3d, Stride: 0x1},
		unicode.Range16{Lo: 0xfd50, Hi: 0xfd8f, Stride: 0x1},
		unicode.Range16{Lo: 0xfd92, Hi: 0xfdc7, Stride: 0x1},
		unicode.Range16{Lo: 0xfdf0, Hi: 0xfdfb, Stride: 0x1},
		unicode.Range16{Lo: 0xfe70, Hi: 0xfe74, Stride: 0x1},
		unicode.Range16{Lo: 0xfe76, Hi: 0xfefc, Stride: 0x1},
		unicode.Range16{Lo: 0xff21, Hi: 0xff3a, Stride: 0x1},
		unicode.Range16{Lo: 0xff41, Hi: 0xff5a, Stride: 0x1},
		unicode.Range16{Lo: 0xffa0, Hi: 0xffbe, Stride: 0x1},
		unicode.Range16{Lo: 0xffc2, Hi: 0xffc7, Stride: 0x1},
		unicode.Range16{Lo: 0xffca, Hi: 0xffcf, Stride: 0x1},
		unicode.Range16{Lo: 0xffd2, Hi: 0xffd7, Stride: 0x1},
		unicode.Range16{Lo: 0xffda, Hi: 0xffdc, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x10000, Hi: 0x1000b, Stride: 0x1},
		unicode.Range32{Lo: 0x1000d, Hi: 0x10026, Stride: 0x1},
		unicode.Range32{Lo: 0x10028, Hi: 0x1003a, Stride: 0x1},
		unicode.Range32{Lo: 0x1003c, Hi: 0x1003d, Stride: 0x1},
		unicode.Range32{Lo: 0x1003f, Hi: 0x1004d, Stride: 0x1},
		unicode.Range32{Lo: 0x10050, Hi: 0x1005d, Stride: 0x1},
		unicode.Range32{Lo: 0x10080, Hi: 0x100fa, Stride: 0x1},
		unicode.Range32{Lo: 0x10140, Hi: 0x10174, Stride: 0x1},
		unicode.Range32{Lo: 0x10280, Hi: 0x1029c, Stride: 0x1},
		unicode.Range32{Lo: 0x102a0, Hi: 0x102d0, Stride: 0x1},
		unicode.Range32{Lo: 0x10300, Hi: 0x1031f, Stride: 0x1},
		unicode.Range32{Lo: 0x1032d, Hi: 0x10340, Stride: 0x1},
		unicode.Range32{Lo: 0x10341, Hi: 0x10341, Stride: 0x1},
		unicode.Range32{Lo: 0x10342, Hi: 0x10349, Stride: 0x1},
		unicode.Range32{Lo: 0x1034a, Hi: 0x1034a, Stride: 0x1},
		unicode.Range32{Lo: 0x10350, Hi: 0x10375, Stride: 0x1},
		unicode.Range32{Lo: 0x10380, Hi: 0x1039d, Stride: 0x1},
		unicode.Range32{Lo: 0x103a0, Hi: 0x103c3, Stride: 0x1},
		unicode.Range32{Lo: 0x103c8, Hi: 0x103cf, Stride: 0x1},
		unicode.Range32{Lo: 0x103d1, Hi: 0x103d5, Stride: 0x1},
		unicode.Range32{Lo: 0x10400, Hi: 0x1044f, Stride: 0x1},
		unicode.Range32{Lo: 0x10450, Hi: 0x1049d, Stride: 0x1},
		unicode.Range32{Lo: 0x104b0, Hi: 0x104d3, Stride: 0x1},
		unicode.Range32{Lo: 0x104d8, Hi: 0x104fb, Stride: 0x1},
		unicode.Range32{Lo: 0x10500, Hi: 0x10527, Stride: 0x1},
		unicode.Range32{Lo: 0x10530, Hi: 0x10563, Stride: 0x1},
		unicode.Range32{Lo: 0x10570, Hi: 0x1057a, Stride: 0x1},
		unicode.Range32{Lo: 0x1057c, Hi: 0x1058a, Stride: 0x1},
		unicode.Range32{Lo: 0x1058c, Hi: 0x10592, Stride: 0x1},
		unicode.Range32{Lo: 0x10594, Hi: 0x10595, Stride: 0x1},
		unicode.Range32{Lo: 0x10597, Hi: 0x105a1, Stride: 0x1},
		unicode.Range32{Lo: 0x105a3, Hi: 0x105b1, Stride: 0x1},
		unicode.Range32{Lo: 0x105b3, Hi: 0x105b9, Stride: 0x1},
		unicode.Range32{Lo: 0x105bb, Hi: 0x105bc, Stride: 0x1},
		unicode.Range32{Lo: 0x10600, Hi: 0x10736, Stride: 0x1},
		unicode.Range32{Lo: 0x10740, Hi: 0x10755, Stride: 0x1},
		unicode.Range32{Lo: 0x10760, Hi: 0x10767, Stride: 0x1},
		unicode.Range32{Lo: 0x10780, Hi: 0x10785, Stride: 0x1},
		unicode.Range32{Lo: 0x10787, Hi: 0x107b0, Stride: 0x1},
		unicode.Range32{Lo: 0x107b2, Hi: 0x107ba, Stride: 0x1},
		unicode.Range32{Lo: 0x10800, Hi: 0x10805, Stride: 0x1},
		unicode.Range32{Lo: 0x10808, Hi: 0x10808, Stride: 0x1},
		unicode.Range32{Lo: 0x1080a, Hi: 0x10835, Stride: 0x1},
		unicode.Range32{Lo: 0x10837, Hi: 0x10838, Stride: 0x1},
		unicode.Range32{Lo: 0x1083c, Hi: 0x1083c, Stride: 0x1},
		unicode.Range32{Lo: 0x1083f, Hi: 0x10855, Stride: 0x1},
		unicode.Range32{Lo: 0x10860, Hi: 0x10876, Stride: 0x1},
		unicode.Range32{Lo: 0x10880, Hi: 0x1089e, Stride: 0x1},
		unicode.Range32{Lo: 0x108e0, Hi: 0x108f2, Stride: 0x1},
		unicode.Range32{Lo: 0x108f4, Hi: 0x108f5, Stride: 0x1},
		unicode.Range32{Lo: 0x10900, Hi: 0x10915, Stride: 0x1},
		unicode.Range32{Lo: 0x10920, Hi: 0x10939, Stride: 0x1},
		unicode.Range32{Lo: 0x10980, Hi: 0x109b7, Stride: 0x1},
		unicode.Range32{Lo: 0x109be, Hi: 0x109bf, Stride: 0x1},
		unicode.Range32{Lo: 0x10a00, Hi: 0x10a00, Stride: 0x1},
		unicode.Range32{Lo: 0x10a10, Hi: 0x10a13, Stride: 0x1},
		unicode.Range32{Lo: 0x10a15, Hi: 0x10a17, Stride: 0x1},
		unicode.Range32{Lo: 0x10a19, Hi: 0x10a35, Stride: 0x1},
		unicode.Range32{Lo: 0x10a60, Hi: 0x10a7c, Stride: 0x1},
		unicode.Range32{Lo: 0x10a80, Hi: 0x10a9c, Stride: 0x1},
		unicode.Range32{Lo: 0x10ac0, Hi: 0x10ac7, Stride: 0x1},
		unicode.Range32{Lo: 0x10ac9, Hi: 0x10ae4, Stride: 0x1},
		unicode.Range32{Lo: 0x10b00, Hi: 0x10b35, Stride: 0x1},
		unicode.Range32{Lo: 0x10b40, Hi: 0x10b55, Stride: 0x1},
		unicode.Range32{Lo: 0x10b60, Hi: 0x10b72, Stride: 0x1},
		unicode.Range32{Lo: 0x10b80, Hi: 0x10b91, Stride: 0x1},
		unicode.Range32{Lo: 0x10c00, Hi: 0x10c48, Stride: 0x1},
		unicode.Range32{Lo: 0x10c80, Hi: 0x10cb2, Stride: 0x1},
		unicode.Range32{Lo: 0x10cc0, Hi: 0x10cf2, Stride: 0x1},
		unicode.Range32{Lo: 0x10d00, Hi: 0x10d23, Stride: 0x1},
		unicode.Range32{Lo: 0x10e80, Hi: 0x10ea9, Stride: 0x1},
		unicode.Range32{Lo: 0x10eb0, Hi: 0x10eb1, Stride: 0x1},
		unicode.Range32{Lo: 0x10f00, Hi: 0x10f1c, Stride: 0x1},
		unicode.Range32{Lo: 0x10f27, Hi: 0x10f27, Stride: 0x1},
		unicode.Range32{Lo: 0x10f30, Hi: 0x10f45, Stride: 0x1},
		unicode.Range32{Lo: 0x10f70, Hi: 0x10f81, Stride: 0x1},
		unicode.Range32{Lo: 0x10fb0, Hi: 0x10fc4, Stride: 0x1},
		unicode.Range32{Lo: 0x10fe0, Hi: 0x10ff6, Stride: 0x1},
		unicode.Range32{Lo: 0x11003, Hi: 0x11037, Stride: 0x1},
		unicode.Range32{Lo: 0x11071, Hi: 0x11072, Stride: 0x1},
		unicode.Range32{Lo: 0x11075, Hi: 0x11075, Stride: 0x1},
		unicode.Range32{Lo: 0x11083, Hi: 0x110af, Stride: 0x1},
		unicode.Range32{Lo: 0x110d0, Hi: 0x110e8, Stride: 0x1},
		unicode.Range32{Lo: 0x11103, Hi: 0x11126, Stride: 0x1},
		unicode.Range32{Lo: 0x11144, Hi: 0x11144, Stride: 0x1},
		unicode.Range32{Lo: 0x11147, Hi: 0x11147, Stride: 0x1},
		unicode.Range32{Lo: 0x11150, Hi: 0x11172, Stride: 0x1},
		unicode.Range32{Lo: 0x11176, Hi: 0x11176, Stride: 0x1},
		unicode.Range32{Lo: 0x11183, Hi: 0x111b2, Stride: 0x1},
		unicode.Range32{Lo: 0x111c1, Hi: 0x111c4, Stride: 0x1},
		unicode.Range32{Lo: 0x111da, Hi: 0x111da, Stride: 0x1},
		unicode.Range32{Lo: 0x111dc, Hi: 0x111dc, Stride: 0x1},
		unicode.Range32{Lo: 0x11200, Hi: 0x11211, Stride: 0x1},
		unicode.Range32{Lo: 0x11213, Hi: 0x1122b, Stride: 0x1},
		unicode.Range32{Lo: 0x1123f, Hi: 0x11240, Stride: 0x1},
		unicode.Range32{Lo: 0x11280, Hi: 0x11286, Stride: 0x1},
		unicode.Range32{Lo: 0x11288, Hi: 0x11288, Stride: 0x1},
		unicode.Range32{Lo: 0x1128a, Hi: 0x1128d, Stride: 0x1},
		unicode.Range32{Lo: 0x1128f, Hi: 0x1129d, Stride: 0x1},
		unicode.Range32{Lo: 0x1129f, Hi: 0x112a8, Stride: 0x1},
		unicode.Range32{Lo: 0x112b0, Hi: 0x112de, Stride: 0x1},
		unicode.Range32{Lo: 0x11305, Hi: 0x1130c, Stride: 0x1},
		unicode.Range32{Lo: 0x1130f, Hi: 0x11310, Stride: 0x1},
		unicode.Range32{Lo: 0x11313, Hi: 0x11328, Stride: 0x1},
		unicode.Range32{Lo: 0x1132a, Hi: 0x11330, Stride: 0x1},
		unicode.Range32{Lo: 0x11332, Hi: 0x11333, Stride: 0x1},
		unicode.Range32{Lo: 0x11335, Hi: 0x11339, Stride: 0x1},
		unicode.Range32{Lo: 0x1133d, Hi: 0x1133d, Stride: 0x1},
		unicode.Range32{Lo: 0x11350, Hi: 0x11350, Stride: 0x1},
		unicode.Range32{Lo: 0x1135d, Hi: 0x11361, Stride: 0x1},
		unicode.Range32{Lo: 0x11400, Hi: 0x11434, Stride: 0x1},
		unicode.Range32{Lo: 0x11447, Hi: 0x1144a, Stride: 0x1},
		unicode.Range32{Lo: 0x1145f, Hi: 0x11461, Stride: 0x1},
		unicode.Range32{Lo: 0x11480, Hi: 0x114af, Stride: 0x1},
		unicode.Range32{Lo: 0x114c4, Hi: 0x114c5, Stride: 0x1},
		unicode.Range32{Lo: 0x114c7, Hi: 0x114c7, Stride: 0x1},
		unicode.Range32{Lo: 0x11580, Hi: 0x115ae, Stride: 0x1},
		unicode.Range32{Lo: 0x115d8, Hi: 0x115db, Stride: 0x1},
		unicode.Range32{Lo: 0x11600, Hi: 0x1162f, Stride: 0x1},
		unicode.Range32{Lo: 0x11644, Hi: 0x11644, Stride: 0x1},
		unicode.Range32{Lo: 0x11680, Hi: 0x116aa, Stride: 0x1},
		unicode.Range32{Lo: 0x116b8, Hi: 0x116b8, Stride: 0x1},
		unicode.Range32{Lo: 0x11800, Hi: 0x1182b, Stride: 0x1},
		unicode.Range32{Lo: 0x118a0, Hi: 0x118df, Stride: 0x1},
		unicode.Range32{Lo: 0x118ff, Hi: 0x11906, Stride: 0x1},
		unicode.Range32{Lo: 0x11909, Hi: 0x11909, Stride: 0x1},
		unicode.Range32{Lo: 0x1190c, Hi: 0x11913, Stride: 0x1},
		unicode.Range32{Lo: 0x11915, Hi: 0x11916, Stride: 0x1},
		unicode.Range32{Lo: 0x11918, Hi: 0x1192f, Stride: 0x1},
		unicode.Range32{Lo: 0x1193f, Hi: 0x1193f, Stride: 0x1},
		unicode.Range32{Lo: 0x11941, Hi: 0x11941, Stride: 0x1},
		unicode.Range32{Lo: 0x119a0, Hi: 0x119a7, Stride: 0x1},
		unicode.Range32{Lo: 0x119aa, Hi: 0x119d0, Stride: 0x1},
		unicode.Range32{Lo: 0x119e1, Hi: 0x119e1, Stride: 0x1},
		unicode.Range32{Lo: 0x119e3, Hi: 0x119e3, Stride: 0x1},
		unicode.Range32{Lo: 0x11a00, Hi: 0x11a00, Stride: 0x1},
		unicode.Range32{Lo: 0x11a0b, Hi: 0x11a32, Stride: 0x1},
		unicode.Range32{Lo: 0x11a3a, Hi: 0x11a3a, Stride: 0x1},
		unicode.Range32{Lo: 0x11a50, Hi: 0x11a50, Stride: 0x1},
		unicode.Range32{Lo: 0x11a5c, Hi: 0x11a89, Stride: 0x1},
		unicode.Range32{Lo: 0x11a9d, Hi: 0x11a9d, Stride: 0x1},
		unicode.Range32{Lo: 0x11ab0, Hi: 0x11af8, Stride: 0x1},
		unicode.Range32{Lo: 0x11c00, Hi: 0x11c08, Stride: 0x1},
		unicode.Range32{Lo: 0x11c0a, Hi: 0x11c2e, Stride: 0x1},
		unicode.Range32{Lo: 0x11c40, Hi: 0x11c40, Stride: 0x1},
		unicode.Range32{Lo: 0x11c72, Hi: 0x11c8f, Stride: 0x1},
		unicode.Range32{Lo: 0x11d00, Hi: 0x11d06, Stride: 0x1},
		unicode.Range32{Lo: 0x11d08, Hi: 0x11d09, Stride: 0x1},
		unicode.Range32{Lo: 0x11d0b, Hi: 0x11d30, Stride: 0x1},
		unicode.Range32{Lo: 0x11d46, Hi: 0x11d46, Stride: 0x1},
		unicode.Range32{Lo: 0x11d60, Hi: 0x11d65, Stride: 0x1},
		unicode.Range32{Lo: 0x11d67, Hi: 0x11d68, Stride: 0x1},
		unicode.Range32{Lo: 0x11d6a, Hi: 0x11d89, Stride: 0x1},
		unicode.Range32{Lo: 0x11d98, Hi: 0x11d98, Stride: 0x1},
		unicode.Range32{Lo: 0x11ee0, Hi: 0x11ef2, Stride: 0x1},
		unicode.Range32{Lo: 0x11f02, Hi: 0x11f02, Stride: 0x1},
		unicode.Range32{Lo: 0x11f04, Hi: 0x11f10, Stride: 0x1},
		unicode.Range32{Lo: 0x11f12, Hi: 0x11f33, Stride: 0x1},
		unicode.Range32{Lo: 0x11fb0, Hi: 0x11fb0, Stride: 0x1},
		unicode.Range32{Lo: 0x12000, Hi: 0x12399, Stride: 0x1},
		unicode.Range32{Lo: 0x12400, Hi: 0x1246e, Stride: 0x1},
		unicode.Range32{Lo: 0x12480, Hi: 0x12543, Stride: 0x1},
		unicode.Range32{Lo: 0x12f90, Hi: 0x12ff0, Stride: 0x1},
		unicode.Range32{Lo: 0x13000, Hi: 0x1342f, Stride: 0x1},
		unicode.Range32{Lo: 0x13441, Hi: 0x13446, Stride: 0x1},
		unicode.Range32{Lo: 0x14400, Hi: 0x14646, Stride: 0x1},
		unicode.Range32{Lo: 0x16800, Hi: 0x16a38, Stride: 0x1},
		unicode.Range32{Lo: 0x16a40, Hi: 0x16a5e, Stride: 0x1},
		unicode.Range32{Lo: 0x16a70, Hi: 0x16abe, Stride: 0x1},
		unicode.Range32{Lo: 0x16ad0, Hi: 0x16aed, Stride: 0x1},
		unicode.Range32{Lo: 0x16b00, Hi: 0x16b2f, Stride: 0x1},
		unicode.Range32{Lo: 0x16b40, Hi: 0x16b43, Stride: 0x1},
		unicode.Range32{Lo: 0x16b63, Hi: 0x16b77, Stride: 0x1},
		unicode.Range32{Lo: 0x16b7d, Hi: 0x16b8f, Stride: 0x1},
		unicode.Range32{Lo: 0x16e40, Hi: 0x16e7f, Stride: 0x1},
		unicode.Range32{Lo: 0x16f00, Hi: 0x16f4a, Stride: 0x1},
		unicode.Range32{Lo: 0x16f50, Hi: 0x16f50, Stride: 0x1},
		unicode.Range32{Lo: 0x16f93, Hi: 0x16f9f, Stride: 0x1},
		unicode.Range32{Lo: 0x16fe0, Hi: 0x16fe1, Stride: 0x1},
		unicode.Range32{Lo: 0x16fe3, Hi: 0x16fe3, Stride: 0x1},
		unicode.Range32{Lo: 0x1bc00, Hi: 0x1bc6a, Stride: 0x1},
		unicode.Range32{Lo: 0x1bc70, Hi: 0x1bc7c, Stride: 0x1},
		unicode.Range32{Lo: 0x1bc80, Hi: 0x1bc88, Stride: 0x1},
		unicode.Range32{Lo: 0x1bc90, Hi: 0x1bc99, Stride: 0x1},
		unicode.Range32{Lo: 0x1d400, Hi: 0x1d454, Stride: 0x1},
		unicode.Range32{Lo: 0x1d456, Hi: 0x1d49c, Stride: 0x1},
		unicode.Range32{Lo: 0x1d49e, Hi: 0x1d49f, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4a2, Hi: 0x1d4a2, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4a5, Hi: 0x1d4a6, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4a9, Hi: 0x1d4ac, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4ae, Hi: 0x1d4b9, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4bb, Hi: 0x1d4bb, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4bd, Hi: 0x1d4c3, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4c5, Hi: 0x1d505, Stride: 0x1},
		unicode.Range32{Lo: 0x1d507, Hi: 0x1d50a, Stride: 0x1},
		unicode.Range32{Lo: 0x1d50d, Hi: 0x1d514, Stride: 0x1},
		unicode.Range32{Lo: 0x1d516, Hi: 0x1d51c, Stride: 0x1},
		unicode.Range32{Lo: 0x1d51e, Hi: 0x1d539, Stride: 0x1},
		unicode.Range32{Lo: 0x1d53b, Hi: 0x1d53e, Stride: 0x1},
		unicode.Range32{Lo: 0x1d540, Hi: 0x1d544, Stride: 0x1},
		unicode.Range32{Lo: 0x1d546, Hi: 0x1d546, Stride: 0x1},
		unicode.Range32{Lo: 0x1d54a, Hi: 0x1d550, Stride: 0x1},
		unicode.Range32{Lo: 0x1d552, Hi: 0x1d6a5, Stride: 0x1},
		unicode.Range32{Lo: 0x1d6a8, Hi: 0x1d6c0, Stride: 0x1},
		unicode.Range32{Lo: 0x1d6c2, Hi: 0x1d6da, Stride: 0x1},
		unicode.Range32{Lo: 0x1d6dc, Hi: 0x1d6fa, Stride: 0x1},
		unicode.Range32{Lo: 0x1d6fc, Hi: 0x1d714, Stride: 0x1},
		unicode.Range32{Lo: 0x1d716, Hi: 0x1d734, Stride: 0x1},
		unicode.Range32{Lo: 0x1d736, Hi: 0x1d74e, Stride: 0x1},
		unicode.Range32{Lo: 0x1d750, Hi: 0x1d76e, Stride: 0x1},
		unicode.Range32{Lo: 0x1d770, Hi: 0x1d788, Stride: 0x1},
		unicode.Range32{Lo: 0x1d78a, Hi: 0x1d7a8, Stride: 0x1},
		unicode.Range32{Lo: 0x1d7aa, Hi: 0x1d7c2, Stride: 0x1},
		unicode.Range32{Lo: 0x1d7c4, Hi: 0x1d7cb, Stride: 0x1},
		unicode.Range32{Lo: 0x1df00, Hi: 0x1df09, Stride: 0x1},
		unicode.Range32{Lo: 0x1df0a, Hi: 0x1df0a, Stride: 0x1},
		unicode.Range32{Lo: 0x1df0b, Hi: 0x1df1e, Stride: 0x1},
		unicode.Range32{Lo: 0x1df25, Hi: 0x1df2a, Stride: 0x1},
		unicode.Range32{Lo: 0x1e030, Hi: 0x1e06d, Stride: 0x1},
		unicode.Range32{Lo: 0x1e100, Hi: 0x1e12c, Stride: 0x1},
		unicode.Range32{Lo: 0x1e137, Hi: 0x1e13d, Stride: 0x1},
		unicode.Range32{Lo: 0x1e14e, Hi: 0x1e14e, Stride: 0x1},
		unicode.Range32{Lo: 0x1e290, Hi: 0x1e2ad, Stride: 0x1},
		unicode.Range32{Lo: 0x1e2c0, Hi: 0x1e2eb, Stride: 0x1},
		unicode.Range32{Lo: 0x1e4d0, Hi: 0x1e4ea, Stride: 0x1},
		unicode.Range32{Lo: 0x1e4eb, Hi: 0x1e4eb, Stride: 0x1},
		unicode.Range32{Lo: 0x1e7e0, Hi: 0x1e7e6, Stride: 0x1},
		unicode.Range32{Lo: 0x1e7e8, Hi: 0x1e7eb, Stride: 0x1},
		unicode.Range32{Lo: 0x1e7ed, Hi: 0x1e7ee, Stride: 0x1},
		unicode.Range32{Lo: 0x1e7f0, Hi: 0x1e7fe, Stride: 0x1},
		unicode.Range32{Lo: 0x1e800, Hi: 0x1e8c4, Stride: 0x1},
		unicode.Range32{Lo: 0x1e900, Hi: 0x1e943, Stride: 0x1},
		unicode.Range32{Lo: 0x1e94b, Hi: 0x1e94b, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee00, Hi: 0x1ee03, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee05, Hi: 0x1ee1f, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee21, Hi: 0x1ee22, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee24, Hi: 0x1ee24, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee27, Hi: 0x1ee27, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee29, Hi: 0x1ee32, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee34, Hi: 0x1ee37, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee39, Hi: 0x1ee39, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee3b, Hi: 0x1ee3b, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee42, Hi: 0x1ee42, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee47, Hi: 0x1ee47, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee49, Hi: 0x1ee49, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee4b, Hi: 0x1ee4b, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee4d, Hi: 0x1ee4f, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee51, Hi: 0x1ee52, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee54, Hi: 0x1ee54, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee57, Hi: 0x1ee57, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee59, Hi: 0x1ee59, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee5b, Hi: 0x1ee5b, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee5d, Hi: 0x1ee5d, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee5f, Hi: 0x1ee5f, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee61, Hi: 0x1ee62, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee64, Hi: 0x1ee64, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee67, Hi: 0x1ee6a, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee6c, Hi: 0x1ee72, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee74, Hi: 0x1ee77, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee79, Hi: 0x1ee7c, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee7e, Hi: 0x1ee7e, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee80, Hi: 0x1ee89, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee8b, Hi: 0x1ee9b, Stride: 0x1},
		unicode.Range32{Lo: 0x1eea1, Hi: 0x1eea3, Stride: 0x1},
		unicode.Range32{Lo: 0x1eea5, Hi: 0x1eea9, Stride: 0x1},
		unicode.Range32{Lo: 0x1eeab, Hi: 0x1eebb, Stride: 0x1},
		unicode.Range32{Lo: 0x1f130, Hi: 0x1f149, Stride: 0x1},
		unicode.Range32{Lo: 0x1f150, Hi: 0x1f169, Stride: 0x1},
		unicode.Range32{Lo: 0x1f170, Hi: 0x1f189, Stride: 0x1},
	},
	LatinOffset: 7,
}

var _WordCR = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0xd, Hi: 0xd, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _WordDouble_Quote = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x22, Hi: 0x22, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _WordExtend = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x300, Hi: 0x36f, Stride: 0x1},
		unicode.Range16{Lo: 0x483, Hi: 0x487, Stride: 0x1},
		unicode.Range16{Lo: 0x488, Hi: 0x489, Stride: 0x1},
		unicode.Range16{Lo: 0x591, Hi: 0x5bd, Stride: 0x1},
		unicode.Range16{Lo: 0x5bf, Hi: 0x5bf, Stride: 0x1},
		unicode.Range16{Lo: 0x5c1, Hi: 0x5c2, Stride: 0x1},
		unicode.Range16{Lo: 0x5c4, Hi: 0x5c5, Stride: 0x1},
		unicode.Range16{Lo: 0x5c7, Hi: 0x5c7, Stride: 0x1},
		unicode.Range16{Lo: 0x610, Hi: 0x61a, Stride: 0x1},
		unicode.Range16{Lo: 0x64b, Hi: 0x65f, Stride: 0x1},
		unicode.Range16{Lo: 0x670, Hi: 0x670, Stride: 0x1},
		unicode.Range16{Lo: 0x6d6, Hi: 0x6dc, Stride: 0x1},
		unicode.Range16{Lo: 0x6df, Hi: 0x6e4, Stride: 0x1},
		unicode.Range16{Lo: 0x6e7, Hi: 0x6e8, Stride: 0x1},
		unicode.Range16{Lo: 0x6ea, Hi: 0x6ed, Stride: 0x1},
		unicode.Range16{Lo: 0x711, Hi: 0x711, Stride: 0x1},
		unicode.Range16{Lo: 0x730, Hi: 0x74a, Stride: 0x1},
		unicode.Range16{Lo: 0x7a6, Hi: 0x7b0, Stride: 0x1},
		unicode.Range16{Lo: 0x7eb, Hi: 0x7f3, Stride: 0x1},
		unicode.Range16{Lo: 0x7fd, Hi: 0x7fd, Stride: 0x1},
		unicode.Range16{Lo: 0x816, Hi: 0x819, Stride: 0x1},
		unicode.Range16{Lo: 0x81b, Hi: 0x823, Stride: 0x1},
		unicode.Range16{Lo: 0x825, Hi: 0x827, Stride: 0x1},
		unicode.Range16{Lo: 0x829, Hi: 0x82d, Stride: 0x1},
		unicode.Range16{Lo: 0x859, Hi: 0x85b, Stride: 0x1},
		unicode.Range16{Lo: 0x898, Hi: 0x89f, Stride: 0x1},
		unicode.Range16{Lo: 0x8ca, Hi: 0x8e1, Stride: 0x1},
		unicode.Range16{Lo: 0x8e3, Hi: 0x902, Stride: 0x1},
		unicode.Range16{Lo: 0x903, Hi: 0x903, Stride: 0x1},
		unicode.Range16{Lo: 0x93a, Hi: 0x93a, Stride: 0x1},
		unicode.Range16{Lo: 0x93b, Hi: 0x93b, Stride: 0x1},
		unicode.Range16{Lo: 0x93c, Hi: 0x93c, Stride: 0x1},
		unicode.Range16{Lo: 0x93e, Hi: 0x940, Stride: 0x1},
		unicode.Range16{Lo: 0x941, Hi: 0x948, Stride: 0x1},
		unicode.Range16{Lo: 0x949, Hi: 0x94c, Stride: 0x1},
		unicode.Range16{Lo: 0x94d, Hi: 0x94d, Stride: 0x1},
		unicode.Range16{Lo: 0x94e, Hi: 0x94f, Stride: 0x1},
		unicode.Range16{Lo: 0x951, Hi: 0x957, Stride: 0x1},
		unicode.Range16{Lo: 0x962, Hi: 0x963, Stride: 0x1},
		unicode.Range16{Lo: 0x981, Hi: 0x981, Stride: 0x1},
		unicode.Range16{Lo: 0x982, Hi: 0x983, Stride: 0x1},
		unicode.Range16{Lo: 0x9bc, Hi: 0x9bc, Stride: 0x1},
		unicode.Range16{Lo: 0x9be, Hi: 0x9c0, Stride: 0x1},
		unicode.Range16{Lo: 0x9c1, Hi: 0x9c4, Stride: 0x1},
		unicode.Range16{Lo: 0x9c7, Hi: 0x9c8, Stride: 0x1},
		unicode.Range16{Lo: 0x9cb, Hi: 0x9cc, Stride: 0x1},
		unicode.Range16{Lo: 0x9cd, Hi: 0x9cd, Stride: 0x1},
		unicode.Range16{Lo: 0x9d7, Hi: 0x9d7, Stride: 0x1},
		unicode.Range16{Lo: 0x9e2, Hi: 0x9e3, Stride: 0x1},
		unicode.Range16{Lo: 0x9fe, Hi: 0x9fe, Stride: 0x1},
		unicode.Range16{Lo: 0xa01, Hi: 0xa02, Stride: 0x1},
		unicode.Range16{Lo: 0xa03, Hi: 0xa03, Stride: 0x1},
		unicode.Range16{Lo: 0xa3c, Hi: 0xa3c, Stride: 0x1},
		unicode.Range16{Lo: 0xa3e, Hi: 0xa40, Stride: 0x1},
		unicode.Range16{Lo: 0xa41, Hi: 0xa42, Stride: 0x1},
		unicode.Range16{Lo: 0xa47, Hi: 0xa48, Stride: 0x1},
		unicode.Range16{Lo: 0xa4b, Hi: 0xa4d, Stride: 0x1},
		unicode.Range16{Lo: 0xa51, Hi: 0xa51, Stride: 0x1},
		unicode.Range16{Lo: 0xa70, Hi: 0xa71, Stride: 0x1},
		unicode.Range16{Lo: 0xa75, Hi: 0xa75, Stride: 0x1},
		unicode.Range16{Lo: 0xa81, Hi: 0xa82, Stride: 0x1},
		unicode.Range16{Lo: 0xa83, Hi: 0xa83, Stride: 0x1},
		unicode.Range16{Lo: 0xabc, Hi: 0xabc, Stride: 0x1},
		unicode.Range16{Lo: 0xabe, Hi: 0xac0, Stride: 0x1},
		unicode.Range16{Lo: 0xac1, Hi: 0xac5, Stride: 0x1},
		unicode.Range16{Lo: 0xac7, Hi: 0xac8, Stride: 0x1},
		unicode.Range16{Lo: 0xac9, Hi: 0xac9, Stride: 0x1},
		unicode.Range16{Lo: 0xacb, Hi: 0xacc, Stride: 0x1},
		unicode.Range16{Lo: 0xacd, Hi: 0xacd, Stride: 0x1},
		unicode.Range16{Lo: 0xae2, Hi: 0xae3, Stride: 0x1},
		unicode.Range16{Lo: 0xafa, Hi: 0xaff, Stride: 0x1},
		unicode.Range16{Lo: 0xb01, Hi: 0xb01, Stride: 0x1},
		unicode.Range16{Lo: 0xb02, Hi: 0xb03, Stride: 0x1},
		unicode.Range16{Lo: 0xb3c, Hi: 0xb3c, Stride: 0x1},
		unicode.Range16{Lo: 0xb3e, Hi: 0xb3e, Stride: 0x1},
		unicode.Range16{Lo: 0xb3f, Hi: 0xb3f, Stride: 0x1},
		unicode.Range16{Lo: 0xb40, Hi: 0xb40, Stride: 0x1},
		unicode.Range16{Lo: 0xb41, Hi: 0xb44, Stride: 0x1},
		unicode.Range16{Lo: 0xb47, Hi: 0xb48, Stride: 0x1},
		unicode.Range16{Lo: 0xb4b, Hi: 0xb4c, Stride: 0x1},
		unicode.Range16{Lo: 0xb4d, Hi: 0xb4d, Stride: 0x1},
		unicode.Range16{Lo: 0xb55, Hi: 0xb56, Stride: 0x1},
		unicode.Range16{Lo: 0xb57, Hi: 0xb57, Stride: 0x1},
		unicode.Range16{Lo: 0xb62, Hi: 0xb63, Stride: 0x1},
		unicode.Range16{Lo: 0xb82, Hi: 0xb82, Stride: 0x1},
		unicode.Range16{Lo: 0xbbe, Hi: 0xbbf, Stride: 0x1},
		unicode.Range16{Lo: 0xbc0, Hi: 0xbc0, Stride: 0x1},
		unicode.Range16{Lo: 0xbc1, Hi: 0xbc2, Stride: 0x1},
		unicode.Range16{Lo: 0xbc6, Hi: 0xbc8, Stride: 0x1},
		unicode.Range16{Lo: 0xbca, Hi: 0xbcc, Stride: 0x1},
		unicode.Range16{Lo: 0xbcd, Hi: 0xbcd, Stride: 0x1},
		unicode.Range16{Lo: 0xbd7, Hi: 0xbd7, Stride: 0x1},
		unicode.Range16{Lo: 0xc00, Hi: 0xc00, Stride: 0x1},
		unicode.Range16{Lo: 0xc01, Hi: 0xc03, Stride: 0x1},
		unicode.Range16{Lo: 0xc04, Hi: 0xc04, Stride: 0x1},
		unicode.Range16{Lo: 0xc3c, Hi: 0xc3c, Stride: 0x1},
		unicode.Range16{Lo: 0xc3e, Hi: 0xc40, Stride: 0x1},
		unicode.Range16{Lo: 0xc41, Hi: 0xc44, Stride: 0x1},
		unicode.Range16{Lo: 0xc46, Hi: 0xc48, Stride: 0x1},
		unicode.Range16{Lo: 0xc4a, Hi: 0xc4d, Stride: 0x1},
		unicode.Range16{Lo: 0xc55, Hi: 0xc56, Stride: 0x1},
		unicode.Range16{Lo: 0xc62, Hi: 0xc63, Stride: 0x1},
		unicode.Range16{Lo: 0xc81, Hi: 0xc81, Stride: 0x1},
		unicode.Range16{Lo: 0xc82, Hi: 0xc83, Stride: 0x1},
		unicode.Range16{Lo: 0xcbc, Hi: 0xcbc, Stride: 0x1},
		unicode.Range16{Lo: 0xcbe, Hi: 0xcbe, Stride: 0x1},
		unicode.Range16{Lo: 0xcbf, Hi: 0xcbf, Stride: 0x1},
		unicode.Range16{Lo: 0xcc0, Hi: 0xcc4, Stride: 0x1},
		unicode.Range16{Lo: 0xcc6, Hi: 0xcc6, Stride: 0x1},
		unicode.Range16{Lo: 0xcc7, Hi: 0xcc8, Stride: 0x1},
		unicode.Range16{Lo: 0xcca, Hi: 0xccb, Stride: 0x1},
		unicode.Range16{Lo: 0xccc, Hi: 0xccd, Stride: 0x1},
		unicode.Range16{Lo: 0xcd5, Hi: 0xcd6, Stride: 0x1},
		unicode.Range16{Lo: 0xce2, Hi: 0xce3, Stride: 0x1},
		unicode.Range16{Lo: 0xcf3, Hi: 0xcf3, Stride: 0x1},
		unicode.Range16{Lo: 0xd00, Hi: 0xd01, Stride: 0x1},
		unicode.Range16{Lo: 0xd02, Hi: 0xd03, Stride: 0x1},
		unicode.Range16{Lo: 0xd3b, Hi: 0xd3c, Stride: 0x1},
		unicode.Range16{Lo: 0xd3e, Hi: 0xd40, Stride: 0x1},
		unicode.Range16{Lo: 0xd41, Hi: 0xd44, Stride: 0x1},
		unicode.Range16{Lo: 0xd46, Hi: 0xd48, Stride: 0x1},
		unicode.Range16{Lo: 0xd4a, Hi: 0xd4c, Stride: 0x1},
		unicode.Range16{Lo: 0xd4d, Hi: 0xd4d, Stride: 0x1},
		unicode.Range16{Lo: 0xd57, Hi: 0xd57, Stride: 0x1},
		unicode.Range16{Lo: 0xd62, Hi: 0xd63, Stride: 0x1},
		unicode.Range16{Lo: 0xd81, Hi: 0xd81, Stride: 0x1},
		unicode.Range16{Lo: 0xd82, Hi: 0xd83, Stride: 0x1},
		unicode.Range16{Lo: 0xdca, Hi: 0xdca, Stride: 0x1},
		unicode.Range16{Lo: 0xdcf, Hi: 0xdd1, Stride: 0x1},
		unicode.Range16{Lo: 0xdd2, Hi: 0xdd4, Stride: 0x1},
		unicode.Range16{Lo: 0xdd6, Hi: 0xdd6, Stride: 0x1},
		unicode.Range16{Lo: 0xdd8, Hi: 0xddf, Stride: 0x1},
		unicode.Range16{Lo: 0xdf2, Hi: 0xdf3, Stride: 0x1},
		unicode.Range16{Lo: 0xe31, Hi: 0xe31, Stride: 0x1},
		unicode.Range16{Lo: 0xe34, Hi: 0xe3a, Stride: 0x1},
		unicode.Range16{Lo: 0xe47, Hi: 0xe4e, Stride: 0x1},
		unicode.Range16{Lo: 0xeb1, Hi: 0xeb1, Stride: 0x1},
		unicode.Range16{Lo: 0xeb4, Hi: 0xebc, Stride: 0x1},
		unicode.Range16{Lo: 0xec8, Hi: 0xece, Stride: 0x1},
		unicode.Range16{Lo: 0xf18, Hi: 0xf19, Stride: 0x1},
		unicode.Range16{Lo: 0xf35, Hi: 0xf35, Stride: 0x1},
		unicode.Range16{Lo: 0xf37, Hi: 0xf37, Stride: 0x1},
		unicode.Range16{Lo: 0xf39, Hi: 0xf39, Stride: 0x1},
		unicode.Range16{Lo: 0xf3e, Hi: 0xf3f, Stride: 0x1},
		unicode.Range16{Lo: 0xf71, Hi: 0xf7e, Stride: 0x1},
		unicode.Range16{Lo: 0xf7f, Hi: 0xf7f, Stride: 0x1},
		unicode.Range16{Lo: 0xf80, Hi: 0xf84, Stride: 0x1},
		unicode.Range16{Lo: 0xf86, Hi: 0xf87, Stride: 0x1},
		unicode.Range16{Lo: 0xf8d, Hi: 0xf97, Stride: 0x1},
		unicode.Range16{Lo: 0xf99, Hi: 0xfbc, Stride: 0x1},
		unicode.Range16{Lo: 0xfc6, Hi: 0xfc6, Stride: 0x1},
		unicode.Range16{Lo: 0x102b, Hi: 0x102c, Stride: 0x1},
		unicode.Range16{Lo: 0x102d, Hi: 0x1030, Stride: 0x1},
		unicode.Range16{Lo: 0x1031, Hi: 0x1031, Stride: 0x1},
		unicode.Range16{Lo: 0x1032, Hi: 0x1037, Stride: 0x1},
		unicode.Range16{Lo: 0x1038, Hi: 0x1038, Stride: 0x1},
		unicode.Range16{Lo: 0x1039, Hi: 0x103a, Stride: 0x1},
		unicode.Range16{Lo: 0x103b, Hi: 0x103c, Stride: 0x1},
		unicode.Range16{Lo: 0x103d, Hi: 0x103e, Stride: 0x1},
		unicode.Range16{Lo: 0x1056, Hi: 0x1057, Stride: 0x1},
		unicode.Range16{Lo: 0x1058, Hi: 0x1059, Stride: 0x1},
		unicode.Range16{Lo: 0x105e, Hi: 0x1060, Stride: 0x1},
		unicode.Range16{Lo: 0x1062, Hi: 0x1064, Stride: 0x1},
		unicode.Range16{Lo: 0x1067, Hi: 0x106d, Stride: 0x1},
		unicode.Range16{Lo: 0x1071, Hi: 0x1074, Stride: 0x1},
		unicode.Range16{Lo: 0x1082, Hi: 0x1082, Stride: 0x1},
		unicode.Range16{Lo: 0x1083, Hi: 0x1084, Stride: 0x1},
		unicode.Range16{Lo: 0x1085, Hi: 0x1086, Stride: 0x1},
		unicode.Range16{Lo: 0x1087, Hi: 0x108c, Stride: 0x1},
		unicode.Range16{Lo: 0x108d, Hi: 0x108d, Stride: 0x1},
		unicode.Range16{Lo: 0x108f, Hi: 0x108f, Stride: 0x1},
		unicode.Range16{Lo: 0x109a, Hi: 0x109c, Stride: 0x1},
		unicode.Range16{Lo: 0x109d, Hi: 0x109d, Stride: 0x1},
		unicode.Range16{Lo: 0x135d, Hi: 0x135f, Stride: 0x1},
		unicode.Range16{Lo: 0x1712, Hi: 0x1714, Stride: 0x1},
		unicode.Range16{Lo: 0x1715, Hi: 0x1715, Stride: 0x1},
		unicode.Range16{Lo: 0x1732, Hi: 0x1733, Stride: 0x1},
		unicode.Range16{Lo: 0x1734, Hi: 0x1734, Stride: 0x1},
		unicode.Range16{Lo: 0x1752, Hi: 0x1753, Stride: 0x1},
		unicode.Range16{Lo: 0x1772, Hi: 0x1773, Stride: 0x1},
		unicode.Range16{Lo: 0x17b4, Hi: 0x17b5, Stride: 0x1},
		unicode.Range16{Lo: 0x17b6, Hi: 0x17b6, Stride: 0x1},
		unicode.Range16{Lo: 0x17b7, Hi: 0x17bd, Stride: 0x1},
		unicode.Range16{Lo: 0x17be, Hi: 0x17c5, Stride: 0x1},
		unicode.Range16{Lo: 0x17c6, Hi: 0x17c6, Stride: 0x1},
		unicode.Range16{Lo: 0x17c7, Hi: 0x17c8, Stride: 0x1},
		unicode.Range16{Lo: 0x17c9, Hi: 0x17d3, Stride: 0x1},
		unicode.Range16{Lo: 0x17dd, Hi: 0x17dd, Stride: 0x1},
		unicode.Range16{Lo: 0x180b, Hi: 0x180d, Stride: 0x1},
		unicode.Range16{Lo: 0x180f, Hi: 0x180f, Stride: 0x1},
		unicode.Range16{Lo: 0x1885, Hi: 0x1886, Stride: 0x1},
		unicode.Range16{Lo: 0x18a9, Hi: 0x18a9, Stride: 0x1},
		unicode.Range16{Lo: 0x1920, Hi: 0x1922, Stride: 0x1},
		unicode.Range16{Lo: 0x1923, Hi: 0x1926, Stride: 0x1},
		unicode.Range16{Lo: 0x1927, Hi: 0x1928, Stride: 0x1},
		unicode.Range16{Lo: 0x1929, Hi: 0x192b, Stride: 0x1},
		unicode.Range16{Lo: 0x1930, Hi: 0x1931, Stride: 0x1},
		unicode.Range16{Lo: 0x1932, Hi: 0x1932, Stride: 0x1},
		unicode.Range16{Lo: 0x1933, Hi: 0x1938, Stride: 0x1},
		unicode.Range16{Lo: 0x1939, Hi: 0x193b, Stride: 0x1},
		unicode.Range16{Lo: 0x1a17, Hi: 0x1a18, Stride: 0x1},
		unicode.Range16{Lo: 0x1a19, Hi: 0x1a1a, Stride: 0x1},
		unicode.Range16{Lo: 0x1a1b, Hi: 0x1a1b, Stride: 0x1},
		unicode.Range16{Lo: 0x1a55, Hi: 0x1a55, Stride: 0x1},
		unicode.Range16{Lo: 0x1a56, Hi: 0x1a56, Stride: 0x1},
		unicode.Range16{Lo: 0x1a57, Hi: 0x1a57, Stride: 0x1},
		unicode.Range16{Lo: 0x1a58, Hi: 0x1a5e, Stride: 0x1},
		unicode.Range16{Lo: 0x1a60, Hi: 0x1a60, Stride: 0x1},
		unicode.Range16{Lo: 0x1a61, Hi: 0x1a61, Stride: 0x1},
		unicode.Range16{Lo: 0x1a62, Hi: 0x1a62, Stride: 0x1},
		unicode.Range16{Lo: 0x1a63, Hi: 0x1a64, Stride: 0x1},
		unicode.Range16{Lo: 0x1a65, Hi: 0x1a6c, Stride: 0x1},
		unicode.Range16{Lo: 0x1a6d, Hi: 0x1a72, Stride: 0x1},
		unicode.Range16{Lo: 0x1a73, Hi: 0x1a7c, Stride: 0x1},
		unicode.Range16{Lo: 0x1a7f, Hi: 0x1a7f, Stride: 0x1},
		unicode.Range16{Lo: 0x1ab0, Hi: 0x1abd, Stride: 0x1},
		unicode.Range16{Lo: 0x1abe, Hi: 0x1abe, Stride: 0x1},
		unicode.Range16{Lo: 0x1abf, Hi: 0x1ace, Stride: 0x1},
		unicode.Range16{Lo: 0x1b00, Hi: 0x1b03, Stride: 0x1},
		unicode.Range16{Lo: 0x1b04, Hi: 0x1b04, Stride: 0x1},
		unicode.Range16{Lo: 0x1b34, Hi: 0x1b34, Stride: 0x1},
		unicode.Range16{Lo: 0x1b35, Hi: 0x1b35, Stride: 0x1},
		unicode.Range16{Lo: 0x1b36, Hi: 0x1b3a, Stride: 0x1},
		unicode.Range16{Lo: 0x1b3b, Hi: 0x1b3b, Stride: 0x1},
		unicode.Range16{Lo: 0x1b3c, Hi: 0x1b3c, Stride: 0x1},
		unicode.Range16{Lo: 0x1b3d, Hi: 0x1b41, Stride: 0x1},
		unicode.Range16{Lo: 0x1b42, Hi: 0x1b42, Stride: 0x1},
		unicode.Range16{Lo: 0x1b43, Hi: 0x1b44, Stride: 0x1},
		unicode.Range16{Lo: 0x1b6b, Hi: 0x1b73, Stride: 0x1},
		unicode.Range16{Lo: 0x1b80, Hi: 0x1b81, Stride: 0x1},
		unicode.Range16{Lo: 0x1b82, Hi: 0x1b82, Stride: 0x1},
		unicode.Range16{Lo: 0x1ba1, Hi: 0x1ba1, Stride: 0x1},
		unicode.Range16{Lo: 0x1ba2, Hi: 0x1ba5, Stride: 0x1},
		unicode.Range16{Lo: 0x1ba6, Hi: 0x1ba7, Stride: 0x1},
		unicode.Range16{Lo: 0x1ba8, Hi: 0x1ba9, Stride: 0x1},
		unicode.Range16{Lo: 0x1baa, Hi: 0x1baa, Stride: 0x1},
		unicode.Range16{Lo: 0x1bab, Hi: 0x1bad, Stride: 0x1},
		unicode.Range16{Lo: 0x1be6, Hi: 0x1be6, Stride: 0x1},
		unicode.Range16{Lo: 0x1be7, Hi: 0x1be7, Stride: 0x1},
		unicode.Range16{Lo: 0x1be8, Hi: 0x1be9, Stride: 0x1},
		unicode.Range16{Lo: 0x1bea, Hi: 0x1bec, Stride: 0x1},
		unicode.Range16{Lo: 0x1bed, Hi: 0x1bed, Stride: 0x1},
		unicode.Range16{Lo: 0x1bee, Hi: 0x1bee, Stride: 0x1},
		unicode.Range16{Lo: 0x1bef, Hi: 0x1bf1, Stride: 0x1},
		unicode.Range16{Lo: 0x1bf2, Hi: 0x1bf3, Stride: 0x1},
		unicode.Range16{Lo: 0x1c24, Hi: 0x1c2b, Stride: 0x1},
		unicode.Range16{Lo: 0x1c2c, Hi: 0x1c33, Stride: 0x1},
		unicode.Range16{Lo: 0x1c34, Hi: 0x1c35, Stride: 0x1},
		unicode.Range16{Lo: 0x1c36, Hi: 0x1c37, Stride: 0x1},
		unicode.Range16{Lo: 0x1cd0, Hi: 0x1cd2, Stride: 0x1},
		unicode.Range16{Lo: 0x1cd4, Hi: 0x1ce0, Stride: 0x1},
		unicode.Range16{Lo: 0x1ce1, Hi: 0x1ce1, Stride: 0x1},
		unicode.Range16{Lo: 0x1ce2, Hi: 0x1ce8, Stride: 0x1},
		unicode.Range16{Lo: 0x1ced, Hi: 0x1ced, Stride: 0x1},
		unicode.Range16{Lo: 0x1cf4, Hi: 0x1cf4, Stride: 0x1},
		unicode.Range16{Lo: 0x1cf7, Hi: 0x1cf7, Stride: 0x1},
		unicode.Range16{Lo: 0x1cf8, Hi: 0x1cf9, Stride: 0x1},
		unicode.Range16{Lo: 0x1dc0, Hi: 0x1dff, Stride: 0x1},
		unicode.Range16{Lo: 0x200c, Hi: 0x200c, Stride: 0x1},
		unicode.Range16{Lo: 0x20d0, Hi: 0x20dc, Stride: 0x1},
		unicode.Range16{Lo: 0x20dd, Hi: 0x20e0, Stride: 0x1},
		unicode.Range16{Lo: 0x20e1, Hi: 0x20e1, Stride: 0x1},
		unicode.Range16{Lo: 0x20e2, Hi: 0x20e4, Stride: 0x1},
		unicode.Range16{Lo: 0x20e5, Hi: 0x20f0, Stride: 0x1},
		unicode.Range16{Lo: 0x2cef, Hi: 0x2cf1, Stride: 0x1},
		unicode.Range16{Lo: 0x2d7f, Hi: 0x2d7f, Stride: 0x1},
		unicode.Range16{Lo: 0x2de0, Hi: 0x2dff, Stride: 0x1},
		unicode.Range16{Lo: 0x302a, Hi: 0x302d, Stride: 0x1},
		unicode.Range16{Lo: 0x302e, Hi: 0x302f, Stride: 0x1},
		unicode.Range16{Lo: 0x3099, Hi: 0x309a, Stride: 0x1},
		unicode.Range16{Lo: 0xa66f, Hi: 0xa66f, Stride: 0x1},
		unicode.Range16{Lo: 0xa670, Hi: 0xa672, Stride: 0x1},
		unicode.Range16{Lo: 0xa674, Hi: 0xa67d, Stride: 0x1},
		unicode.Range16{Lo: 0xa69e, Hi: 0xa69f, Stride: 0x1},
		unicode.Range16{Lo: 0xa6f0, Hi: 0xa6f1, Stride: 0x1},
		unicode.Range16{Lo: 0xa802, Hi: 0xa802, Stride: 0x1},
		unicode.Range16{Lo: 0xa806, Hi: 0xa806, Stride: 0x1},
		unicode.Range16{Lo: 0xa80b, Hi: 0xa80b, Stride: 0x1},
		unicode.Range16{Lo: 0xa823, Hi: 0xa824, Stride: 0x1},
		unicode.Range16{Lo: 0xa825, Hi: 0xa826, Stride: 0x1},
		unicode.Range16{Lo: 0xa827, Hi: 0xa827, Stride: 0x1},
		unicode.Range16{Lo: 0xa82c, Hi: 0xa82c, Stride: 0x1},
		unicode.Range16{Lo: 0xa880, Hi: 0xa881, Stride: 0x1},
		unicode.Range16{Lo: 0xa8b4, Hi: 0xa8c3, Stride: 0x1},
		unicode.Range16{Lo: 0xa8c4, Hi: 0xa8c5, Stride: 0x1},
		unicode.Range16{Lo: 0xa8e0, Hi: 0xa8f1, Stride: 0x1},
		unicode.Range16{Lo: 0xa8ff, Hi: 0xa8ff, Stride: 0x1},
		unicode.Range16{Lo: 0xa926, Hi: 0xa92d, Stride: 0x1},
		unicode.Range16{Lo: 0xa947, Hi: 0xa951, Stride: 0x1},
		unicode.Range16{Lo: 0xa952, Hi: 0xa953, Stride: 0x1},
		unicode.Range16{Lo: 0xa980, Hi: 0xa982, Stride: 0x1},
		unicode.Range16{Lo: 0xa983, Hi: 0xa983, Stride: 0x1},
		unicode.Range16{Lo: 0xa9b3, Hi: 0xa9b3, Stride: 0x1},
		unicode.Range16{Lo: 0xa9b4, Hi: 0xa9b5, Stride: 0x1},
		unicode.Range16{Lo: 0xa9b6, Hi: 0xa9b9, Stride: 0x1},
		unicode.Range16{Lo: 0xa9ba, Hi: 0xa9bb, Stride: 0x1},
		unicode.Range16{Lo: 0xa9bc, Hi: 0xa9bd, Stride: 0x1},
		unicode.Range16{Lo: 0xa9be, Hi: 0xa9c0, Stride: 0x1},
		unicode.Range16{Lo: 0xa9e5, Hi: 0xa9e5, Stride: 0x1},
		unicode.Range16{Lo: 0xaa29, Hi: 0xaa2e, Stride: 0x1},
		unicode.Range16{Lo: 0xaa2f, Hi: 0xaa30, Stride: 0x1},
		unicode.Range16{Lo: 0xaa31, Hi: 0xaa32, Stride: 0x1},
		unicode.Range16{Lo: 0xaa33, Hi: 0xaa34, Stride: 0x1},
		unicode.Range16{Lo: 0xaa35, Hi: 0xaa36, Stride: 0x1},
		unicode.Range16{Lo: 0xaa43, Hi: 0xaa43, Stride: 0x1},
		unicode.Range16{Lo: 0xaa4c, Hi: 0xaa4c, Stride: 0x1},
		unicode.Range16{Lo: 0xaa4d, Hi: 0xaa4d, Stride: 0x1},
		unicode.Range16{Lo: 0xaa7b, Hi: 0xaa7b, Stride: 0x1},
		unicode.Range16{Lo: 0xaa7c, Hi: 0xaa7c, Stride: 0x1},
		unicode.Range16{Lo: 0xaa7d, Hi: 0xaa7d, Stride: 0x1},
		unicode.Range16{Lo: 0xaab0, Hi: 0xaab0, Stride: 0x1},
		unicode.Range16{Lo: 0xaab2, Hi: 0xaab4, Stride: 0x1},
		unicode.Range16{Lo: 0xaab7, Hi: 0xaab8, Stride: 0x1},
		unicode.Range16{Lo: 0xaabe, Hi: 0xaabf, Stride: 0x1},
		unicode.Range16{Lo: 0xaac1, Hi: 0xaac1, Stride: 0x1},
		unicode.Range16{Lo: 0xaaeb, Hi: 0xaaeb, Stride: 0x1},
		unicode.Range16{Lo: 0xaaec, Hi: 0xaaed, Stride: 0x1},
		unicode.Range16{Lo: 0xaaee, Hi: 0xaaef, Stride: 0x1},
		unicode.Range16{Lo: 0xaaf5, Hi: 0xaaf5, Stride: 0x1},
		unicode.Range16{Lo: 0xaaf6, Hi: 0xaaf6, Stride: 0x1},
		unicode.Range16{Lo: 0xabe3, Hi: 0xabe4, Stride: 0x1},
		unicode.Range16{Lo: 0xabe5, Hi: 0xabe5, Stride: 0x1},
		unicode.Range16{Lo: 0xabe6, Hi: 0xabe7, Stride: 0x1},
		unicode.Range16{Lo: 0xabe8, Hi: 0xabe8, Stride: 0x1},
		unicode.Range16{Lo: 0xabe9, Hi: 0xabea, Stride: 0x1},
		unicode.Range16{Lo: 0xabec, Hi: 0xabec, Stride: 0x1},
		unicode.Range16{Lo: 0xabed, Hi: 0xabed, Stride: 0x1},
		unicode.Range16{Lo: 0xfb1e, Hi: 0xfb1e, Stride: 0x1},
		unicode.Range16{Lo: 0xfe00, Hi: 0xfe0f, Stride: 0x1},
		unicode.Range16{Lo: 0xfe20, Hi: 0xfe2f, Stride: 0x1},
		unicode.Range16{Lo: 0xff9e, Hi: 0xff9f, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x101fd, Hi: 0x101fd, Stride: 0x1},
		unicode.Range32{Lo: 0x102e0, Hi: 0x102e0, Stride: 0x1},
		unicode.Range32{Lo: 0x10376, Hi: 0x1037a, Stride: 0x1},
		unicode.Range32{Lo: 0x10a01, Hi: 0x10a03, Stride: 0x1},
		unicode.Range32{Lo: 0x10a05, Hi: 0x10a06, Stride: 0x1},
		unicode.Range32{Lo: 0x10a0c, Hi: 0x10a0f, Stride: 0x1},
		unicode.Range32{Lo: 0x10a38, Hi: 0x10a3a, Stride: 0x1},
		unicode.Range32{Lo: 0x10a3f, Hi: 0x10a3f, Stride: 0x1},
		unicode.Range32{Lo: 0x10ae5, Hi: 0x10ae6, Stride: 0x1},
		unicode.Range32{Lo: 0x10d24, Hi: 0x10d27, Stride: 0x1},
		unicode.Range32{Lo: 0x10eab, Hi: 0x10eac, Stride: 0x1},
		unicode.Range32{Lo: 0x10efd, Hi: 0x10eff, Stride: 0x1},
		unicode.Range32{Lo: 0x10f46, Hi: 0x10f50, Stride: 0x1},
		unicode.Range32{Lo: 0x10f82, Hi: 0x10f85, Stride: 0x1},
		unicode.Range32{Lo: 0x11000, Hi: 0x11000, Stride: 0x1},
		unicode.Range32{Lo: 0x11001, Hi: 0x11001, Stride: 0x1},
		unicode.Range32{Lo: 0x11002, Hi: 0x11002, Stride: 0x1},
		unicode.Range32{Lo: 0x11038, Hi: 0x11046, Stride: 0x1},
		unicode.Range32{Lo: 0x11070, Hi: 0x11070, Stride: 0x1},
		unicode.Range32{Lo: 0x11073, Hi: 0x11074, Stride: 0x1},
		unicode.Range32{Lo: 0x1107f, Hi: 0x11081, Stride: 0x1},
		unicode.Range32{Lo: 0x11082, Hi: 0x11082, Stride: 0x1},
		unicode.Range32{Lo: 0x110b0, Hi: 0x110b2, Stride: 0x1},
		unicode.Range32{Lo: 0x110b3, Hi: 0x110b6, Stride: 0x1},
		unicode.Range32{Lo: 0x110b7, Hi: 0x110b8, Stride: 0x1},
		unicode.Range32{Lo: 0x110b9, Hi: 0x110ba, Stride: 0x1},
		unicode.Range32{Lo: 0x110c2, Hi: 0x110c2, Stride: 0x1},
		unicode.Range32{Lo: 0x11100, Hi: 0x11102, Stride: 0x1},
		unicode.Range32{Lo: 0x11127, Hi: 0x1112b, Stride: 0x1},
		unicode.Range32{Lo: 0x1112c, Hi: 0x1112c, Stride: 0x1},
		unicode.Range32{Lo: 0x1112d, Hi: 0x11134, Stride: 0x1},
		unicode.Range32{Lo: 0x11145, Hi: 0x11146, Stride: 0x1},
		unicode.Range32{Lo: 0x11173, Hi: 0x11173, Stride: 0x1},
		unicode.Range32{Lo: 0x11180, Hi: 0x11181, Stride: 0x1},
		unicode.Range32{Lo: 0x11182, Hi: 0x11182, Stride: 0x1},
		unicode.Range32{Lo: 0x111b3, Hi: 0x111b5, Stride: 0x1},
		unicode.Range32{Lo: 0x111b6, Hi: 0x111be, Stride: 0x1},
		unicode.Range32{Lo: 0x111bf, Hi: 0x111c0, Stride: 0x1},
		unicode.Range32{Lo: 0x111c9, Hi: 0x111cc, Stride: 0x1},
		unicode.Range32{Lo: 0x111ce, Hi: 0x111ce, Stride: 0x1},
		unicode.Range32{Lo: 0x111cf, Hi: 0x111cf, Stride: 0x1},
		unicode.Range32{Lo: 0x1122c, Hi: 0x1122e, Stride: 0x1},
		unicode.Range32{Lo: 0x1122f, Hi: 0x11231, Stride: 0x1},
		unicode.Range32{Lo: 0x11232, Hi: 0x11233, Stride: 0x1},
		unicode.Range32{Lo: 0x11234, Hi: 0x11234, Stride: 0x1},
		unicode.Range32{Lo: 0x11235, Hi: 0x11235, Stride: 0x1},
		unicode.Range32{Lo: 0x11236, Hi: 0x11237, Stride: 0x1},
		unicode.Range32{Lo: 0x1123e, Hi: 0x1123e, Stride: 0x1},
		unicode.Range32{Lo: 0x11241, Hi: 0x11241, Stride: 0x1},
		unicode.Range32{Lo: 0x112df, Hi: 0x112df, Stride: 0x1},
		unicode.Range32{Lo: 0x112e0, Hi: 0x112e2, Stride: 0x1},
		unicode.Range32{Lo: 0x112e3, Hi: 0x112ea, Stride: 0x1},
		unicode.Range32{Lo: 0x11300, Hi: 0x11301, Stride: 0x1},
		unicode.Range32{Lo: 0x11302, Hi: 0x11303, Stride: 0x1},
		unicode.Range32{Lo: 0x1133b, Hi: 0x1133c, Stride: 0x1},
		unicode.Range32{Lo: 0x1133e, Hi: 0x1133f, Stride: 0x1},
		unicode.Range32{Lo: 0x11340, Hi: 0x11340, Stride: 0x1},
		unicode.Range32{Lo: 0x11341, Hi: 0x11344, Stride: 0x1},
		unicode.Range32{Lo: 0x11347, Hi: 0x11348, Stride: 0x1},
		unicode.Range32{Lo: 0x1134b, Hi: 0x1134d, Stride: 0x1},
		unicode.Range32{Lo: 0x11357, Hi: 0x11357, Stride: 0x1},
		unicode.Range32{Lo: 0x11362, Hi: 0x11363, Stride: 0x1},
		unicode.Range32{Lo: 0x11366, Hi: 0x1136c, Stride: 0x1},
		unicode.Range32{Lo: 0x11370, Hi: 0x11374, Stride: 0x1},
		unicode.Range32{Lo: 0x11435, Hi: 0x11437, Stride: 0x1},
		unicode.Range32{Lo: 0x11438, Hi: 0x1143f, Stride: 0x1},
		unicode.Range32{Lo: 0x11440, Hi: 0x11441, Stride: 0x1},
		unicode.Range32{Lo: 0x11442, Hi: 0x11444, Stride: 0x1},
		unicode.Range32{Lo: 0x11445, Hi: 0x11445, Stride: 0x1},
		unicode.Range32{Lo: 0x11446, Hi: 0x11446, Stride: 0x1},
		unicode.Range32{Lo: 0x1145e, Hi: 0x1145e, Stride: 0x1},
		unicode.Range32{Lo: 0x114b0, Hi: 0x114b2, Stride: 0x1},
		unicode.Range32{Lo: 0x114b3, Hi: 0x114b8, Stride: 0x1},
		unicode.Range32{Lo: 0x114b9, Hi: 0x114b9, Stride: 0x1},
		unicode.Range32{Lo: 0x114ba, Hi: 0x114ba, Stride: 0x1},
		unicode.Range32{Lo: 0x114bb, Hi: 0x114be, Stride: 0x1},
		unicode.Range32{Lo: 0x114bf, Hi: 0x114c0, Stride: 0x1},
		unicode.Range32{Lo: 0x114c1, Hi: 0x114c1, Stride: 0x1},
		unicode.Range32{Lo: 0x114c2, Hi: 0x114c3, Stride: 0x1},
		unicode.Range32{Lo: 0x115af, Hi: 0x115b1, Stride: 0x1},
		unicode.Range32{Lo: 0x115b2, Hi: 0x115b5, Stride: 0x1},
		unicode.Range32{Lo: 0x115b8, Hi: 0x115bb, Stride: 0x1},
		unicode.Range32{Lo: 0x115bc, Hi: 0x115bd, Stride: 0x1},
		unicode.Range32{Lo: 0x115be, Hi: 0x115be, Stride: 0x1},
		unicode.Range32{Lo: 0x115bf, Hi: 0x115c0, Stride: 0x1},
		unicode.Range32{Lo: 0x115dc, Hi: 0x115dd, Stride: 0x1},
		unicode.Range32{Lo: 0x11630, Hi: 0x11632, Stride: 0x1},
		unicode.Range32{Lo: 0x11633, Hi: 0x1163a, Stride: 0x1},
		unicode.Range32{Lo: 0x1163b, Hi: 0x1163c, Stride: 0x1},
		unicode.Range32{Lo: 0x1163d, Hi: 0x1163d, Stride: 0x1},
		unicode.Range32{Lo: 0x1163e, Hi: 0x1163e, Stride: 0x1},
		unicode.Range32{Lo: 0x1163f, Hi: 0x11640, Stride: 0x1},
		unicode.Range32{Lo: 0x116ab, Hi: 0x116ab, Stride: 0x1},
		unicode.Range32{Lo: 0x116ac, Hi: 0x116ac, Stride: 0x1},
		unicode.Range32{Lo: 0x116ad, Hi: 0x116ad, Stride: 0x1},
		unicode.Range32{Lo: 0x116ae, Hi: 0x116af, Stride: 0x1},
		unicode.Range32{Lo: 0x116b0, Hi: 0x116b5, Stride: 0x1},
		unicode.Range32{Lo: 0x116b6, Hi: 0x116b6, Stride: 0x1},
		unicode.Range32{Lo: 0x116b7, Hi: 0x116b7, Stride: 0x1},
		unicode.Range32{Lo: 0x1171d, Hi: 0x1171f, Stride: 0x1},
		unicode.Range32{Lo: 0x11720, Hi: 0x11721, Stride: 0x1},
		unicode.Range32{Lo: 0x11722, Hi: 0x11725, Stride: 0x1},
		unicode.Range32{Lo: 0x11726, Hi: 0x11726, Stride: 0x1},
		unicode.Range32{Lo: 0x11727, Hi: 0x1172b, Stride: 0x1},
		unicode.Range32{Lo: 0x1182c, Hi: 0x1182e, Stride: 0x1},
		unicode.Range32{Lo: 0x1182f, Hi: 0x11837, Stride: 0x1},
		unicode.Range32{Lo: 0x11838, Hi: 0x11838, Stride: 0x1},
		unicode.Range32{Lo: 0x11839, Hi: 0x1183a, Stride: 0x1},
		unicode.Range32{Lo: 0x11930, Hi: 0x11935, Stride: 0x1},
		unicode.Range32{Lo: 0x11937, Hi: 0x11938, Stride: 0x1},
		unicode.Range32{Lo: 0x1193b, Hi: 0x1193c, Stride: 0x1},
		unicode.Range32{Lo: 0x1193d, Hi: 0x1193d, Stride: 0x1},
		unicode.Range32{Lo: 0x1193e, Hi: 0x1193e, Stride: 0x1},
		unicode.Range32{Lo: 0x11940, Hi: 0x11940, Stride: 0x1},
		unicode.Range32{Lo: 0x11942, Hi: 0x11942, Stride: 0x1},
		unicode.Range32{Lo: 0x11943, Hi: 0x11943, Stride: 0x1},
		unicode.Range32{Lo: 0x119d1, Hi: 0x119d3, Stride: 0x1},
		unicode.Range32{Lo: 0x119d4, Hi: 0x119d7, Stride: 0x1},
		unicode.Range32{Lo: 0x119da, Hi: 0x119db, Stride: 0x1},
		unicode.Range32{Lo: 0x119dc, Hi: 0x119df, Stride: 0x1},
		unicode.Range32{Lo: 0x119e0, Hi: 0x119e0, Stride: 0x1},
		unicode.Range32{Lo: 0x119e4, Hi: 0x119e4, Stride: 0x1},
		unicode.Range32{Lo: 0x11a01, Hi: 0x11a0a, Stride: 0x1},
		unicode.Range32{Lo: 0x11a33, Hi: 0x11a38, Stride: 0x1},
		unicode.Range32{Lo: 0x11a39, Hi: 0x11a39, Stride: 0x1},
		unicode.Range32{Lo: 0x11a3b, Hi: 0x11a3e, Stride: 0x1},
		unicode.Range32{Lo: 0x11a47, Hi: 0x11a47, Stride: 0x1},
		unicode.Range32{Lo: 0x11a51, Hi: 0x11a56, Stride: 0x1},
		unicode.Range32{Lo: 0x11a57, Hi: 0x11a58, Stride: 0x1},
		unicode.Range32{Lo: 0x11a59, Hi: 0x11a5b, Stride: 0x1},
		unicode.Range32{Lo: 0x11a8a, Hi: 0x11a96, Stride: 0x1},
		unicode.Range32{Lo: 0x11a97, Hi: 0x11a97, Stride: 0x1},
		unicode.Range32{Lo: 0x11a98, Hi: 0x11a99, Stride: 0x1},
		unicode.Range32{Lo: 0x11c2f, Hi: 0x11c2f, Stride: 0x1},
		unicode.Range32{Lo: 0x11c30, Hi: 0x11c36, Stride: 0x1},
		unicode.Range32{Lo: 0x11c38, Hi: 0x11c3d, Stride: 0x1},
		unicode.Range32{Lo: 0x11c3e, Hi: 0x11c3e, Stride: 0x1},
		unicode.Range32{Lo: 0x11c3f, Hi: 0x11c3f, Stride: 0x1},
		unicode.Range32{Lo: 0x11c92, Hi: 0x11ca7, Stride: 0x1},
		unicode.Range32{Lo: 0x11ca9, Hi: 0x11ca9, Stride: 0x1},
		unicode.Range32{Lo: 0x11caa, Hi: 0x11cb0, Stride: 0x1},
		unicode.Range32{Lo: 0x11cb1, Hi: 0x11cb1, Stride: 0x1},
		unicode.Range32{Lo: 0x11cb2, Hi: 0x11cb3, Stride: 0x1},
		unicode.Range32{Lo: 0x11cb4, Hi: 0x11cb4, Stride: 0x1},
		unicode.Range32{Lo: 0x11cb5, Hi: 0x11cb6, Stride: 0x1},
		unicode.Range32{Lo: 0x11d31, Hi: 0x11d36, Stride: 0x1},
		unicode.Range32{Lo: 0x11d3a, Hi: 0x11d3a, Stride: 0x1},
		unicode.Range32{Lo: 0x11d3c, Hi: 0x11d3d, Stride: 0x1},
		unicode.Range32{Lo: 0x11d3f, Hi: 0x11d45, Stride: 0x1},
		unicode.Range32{Lo: 0x11d47, Hi: 0x11d47, Stride: 0x1},
		unicode.Range32{Lo: 0x11d8a, Hi: 0x11d8e, Stride: 0x1},
		unicode.Range32{Lo: 0x11d90, Hi: 0x11d91, Stride: 0x1},
		unicode.Range32{Lo: 0x11d93, Hi: 0x11d94, Stride: 0x1},
		unicode.Range32{Lo: 0x11d95, Hi: 0x11d95, Stride: 0x1},
		unicode.Range32{Lo: 0x11d96, Hi: 0x11d96, Stride: 0x1},
		unicode.Range32{Lo: 0x11d97, Hi: 0x11d97, Stride: 0x1},
		unicode.Range32{Lo: 0x11ef3, Hi: 0x11ef4, Stride: 0x1},
		unicode.Range32{Lo: 0x11ef5, Hi: 0x11ef6, Stride: 0x1},
		unicode.Range32{Lo: 0x11f00, Hi: 0x11f01, Stride: 0x1},
		unicode.Range32{Lo: 0x11f03, Hi: 0x11f03, Stride: 0x1},
		unicode.Range32{Lo: 0x11f34, Hi: 0x11f35, Stride: 0x1},
		unicode.Range32{Lo: 0x11f36, Hi: 0x11f3a, Stride: 0x1},
		unicode.Range32{Lo: 0x11f3e, Hi: 0x11f3f, Stride: 0x1},
		unicode.Range32{Lo: 0x11f40, Hi: 0x11f40, Stride: 0x1},
		unicode.Range32{Lo: 0x11f41, Hi: 0x11f41, Stride: 0x1},
		unicode.Range32{Lo: 0x11f42, Hi: 0x11f42, Stride: 0x1},
		unicode.Range32{Lo: 0x13440, Hi: 0x13440, Stride: 0x1},
		unicode.Range32{Lo: 0x13447, Hi: 0x13455, Stride: 0x1},
		unicode.Range32{Lo: 0x16af0, Hi: 0x16af4, Stride: 0x1},
		unicode.Range32{Lo: 0x16b30, Hi: 0x16b36, Stride: 0x1},
		unicode.Range32{Lo: 0x16f4f, Hi: 0x16f4f, Stride: 0x1},
		unicode.Range32{Lo: 0x16f51, Hi: 0x16f87, Stride: 0x1},
		unicode.Range32{Lo: 0x16f8f, Hi: 0x16f92, Stride: 0x1},
		unicode.Range32{Lo: 0x16fe4, Hi: 0x16fe4, Stride: 0x1},
		unicode.Range32{Lo: 0x16ff0, Hi: 0x16ff1, Stride: 0x1},
		unicode.Range32{Lo: 0x1bc9d, Hi: 0x1bc9e, Stride: 0x1},
		unicode.Range32{Lo: 0x1cf00, Hi: 0x1cf2d, Stride: 0x1},
		unicode.Range32{Lo: 0x1cf30, Hi: 0x1cf46, Stride: 0x1},
		unicode.Range32{Lo: 0x1d165, Hi: 0x1d166, Stride: 0x1},
		unicode.Range32{Lo: 0x1d167, Hi: 0x1d169, Stride: 0x1},
		unicode.Range32{Lo: 0x1d16d, Hi: 0x1d172, Stride: 0x1},
		unicode.Range32{Lo: 0x1d17b, Hi: 0x1d182, Stride: 0x1},
		unicode.Range32{Lo: 0x1d185, Hi: 0x1d18b, Stride: 0x1},
		unicode.Range32{Lo: 0x1d1aa, Hi: 0x1d1ad, Stride: 0x1},
		unicode.Range32{Lo: 0x1d242, Hi: 0x1d244, Stride: 0x1},
		unicode.Range32{Lo: 0x1da00, Hi: 0x1da36, Stride: 0x1},
		unicode.Range32{Lo: 0x1da3b, Hi: 0x1da6c, Stride: 0x1},
		unicode.Range32{Lo: 0x1da75, Hi: 0x1da75, Stride: 0x1},
		unicode.Range32{Lo: 0x1da84, Hi: 0x1da84, Stride: 0x1},
		unicode.Range32{Lo: 0x1da9b, Hi: 0x1da9f, Stride: 0x1},
		unicode.Range32{Lo: 0x1daa1, Hi: 0x1daaf, Stride: 0x1},
		unicode.Range32{Lo: 0x1e000, Hi: 0x1e006, Stride: 0x1},
		unicode.Range32{Lo: 0x1e008, Hi: 0x1e018, Stride: 0x1},
		unicode.Range32{Lo: 0x1e01b, Hi: 0x1e021, Stride: 0x1},
		unicode.Range32{Lo: 0x1e023, Hi: 0x1e024, Stride: 0x1},
		unicode.Range32{Lo: 0x1e026, Hi: 0x1e02a, Stride: 0x1},
		unicode.Range32{Lo: 0x1e08f, Hi: 0x1e08f, Stride: 0x1},
		unicode.Range32{Lo: 0x1e130, Hi: 0x1e136, Stride: 0x1},
		unicode.Range32{Lo: 0x1e2ae, Hi: 0x1e2ae, Stride: 0x1},
		unicode.Range32{Lo: 0x1e2ec, Hi: 0x1e2ef, Stride: 0x1},
		unicode.Range32{Lo: 0x1e4ec, Hi: 0x1e4ef, Stride: 0x1},
		unicode.Range32{Lo: 0x1e8d0, Hi: 0x1e8d6, Stride: 0x1},
		unicode.Range32{Lo: 0x1e944, Hi: 0x1e94a, Stride: 0x1},
		unicode.Range32{Lo: 0x1f3fb, Hi: 0x1f3ff, Stride: 0x1},
		unicode.Range32{Lo: 0xe0020, Hi: 0xe007f, Stride: 0x1},
		unicode.Range32{Lo: 0xe0100, Hi: 0xe01ef, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _WordExtendNumLet = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x5f, Hi: 0x5f, Stride: 0x1},
		unicode.Range16{Lo: 0x202f, Hi: 0x202f, Stride: 0x1},
		unicode.Range16{Lo: 0x203f, Hi: 0x2040, Stride: 0x1},
		unicode.Range16{Lo: 0x2054, Hi: 0x2054, Stride: 0x1},
		unicode.Range16{Lo: 0xfe33, Hi: 0xfe34, Stride: 0x1},
		unicode.Range16{Lo: 0xfe4d, Hi: 0xfe4f, Stride: 0x1},
		unicode.Range16{Lo: 0xff3f, Hi: 0xff3f, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _WordFormat = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0xad, Hi: 0xad, Stride: 0x1},
		unicode.Range16{Lo: 0x600, Hi: 0x605, Stride: 0x1},
		unicode.Range16{Lo: 0x61c, Hi: 0x61c, Stride: 0x1},
		unicode.Range16{Lo: 0x6dd, Hi: 0x6dd, Stride: 0x1},
		unicode.Range16{Lo: 0x70f, Hi: 0x70f, Stride: 0x1},
		unicode.Range16{Lo: 0x890, Hi: 0x891, Stride: 0x1},
		unicode.Range16{Lo: 0x8e2, Hi: 0x8e2, Stride: 0x1},
		unicode.Range16{Lo: 0x180e, Hi: 0x180e, Stride: 0x1},
		unicode.Range16{Lo: 0x200e, Hi: 0x200f, Stride: 0x1},
		unicode.Range16{Lo: 0x202a, Hi: 0x202e, Stride: 0x1},
		unicode.Range16{Lo: 0x2060, Hi: 0x2064, Stride: 0x1},
		unicode.Range16{Lo: 0x2066, Hi: 0x206f, Stride: 0x1},
		unicode.Range16{Lo: 0xfeff, Hi: 0xfeff, Stride: 0x1},
		unicode.Range16{Lo: 0xfff9, Hi: 0xfffb, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x110bd, Hi: 0x110bd, Stride: 0x1},
		unicode.Range32{Lo: 0x110cd, Hi: 0x110cd, Stride: 0x1},
		unicode.Range32{Lo: 0x13430, Hi: 0x1343f, Stride: 0x1},
		unicode.Range32{Lo: 0x1bca0, Hi: 0x1bca3, Stride: 0x1},
		unicode.Range32{Lo: 0x1d173, Hi: 0x1d17a, Stride: 0x1},
		unicode.Range32{Lo: 0xe0001, Hi: 0xe0001, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _WordHebrew_Letter = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x5d0, Hi: 0x5ea, Stride: 0x1},
		unicode.Range16{Lo: 0x5ef, Hi: 0x5f2, Stride: 0x1},
		unicode.Range16{Lo: 0xfb1d, Hi: 0xfb1d, Stride: 0x1},
		unicode.Range16{Lo: 0xfb1f, Hi: 0xfb28, Stride: 0x1},
		unicode.Range16{Lo: 0xfb2a, Hi: 0xfb36, Stride: 0x1},
		unicode.Range16{Lo: 0xfb38, Hi: 0xfb3c, Stride: 0x1},
		unicode.Range16{Lo: 0xfb3e, Hi: 0xfb3e, Stride: 0x1},
		unicode.Range16{Lo: 0xfb40, Hi: 0xfb41, Stride: 0x1},
		unicode.Range16{Lo: 0xfb43, Hi: 0xfb44, Stride: 0x1},
		unicode.Range16{Lo: 0xfb46, Hi: 0xfb4f, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _WordKatakana = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x3031, Hi: 0x3035, Stride: 0x1},
		unicode.Range16{Lo: 0x309b, Hi: 0x309c, Stride: 0x1},
		unicode.Range16{Lo: 0x30a0, Hi: 0x30a0, Stride: 0x1},
		unicode.Range16{Lo: 0x30a1, Hi: 0x30fa, Stride: 0x1},
		unicode.Range16{Lo: 0x30fc, Hi: 0x30fe, Stride: 0x1},
		unicode.Range16{Lo: 0x30ff, Hi: 0x30ff, Stride: 0x1},
		unicode.Range16{Lo: 0x31f0, Hi: 0x31ff, Stride: 0x1},
		unicode.Range16{Lo: 0x32d0, Hi: 0x32fe, Stride: 0x1},
		unicode.Range16{Lo: 0x3300, Hi: 0x3357, Stride: 0x1},
		unicode.Range16{Lo: 0xff66, Hi: 0xff6f, Stride: 0x1},
		unicode.Range16{Lo: 0xff70, Hi: 0xff70, Stride: 0x1},
		unicode.Range16{Lo: 0xff71, Hi: 0xff9d, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x1aff0, Hi: 0x1aff3, Stride: 0x1},
		unicode.Range32{Lo: 0x1aff5, Hi: 0x1affb, Stride: 0x1},
		unicode.Range32{Lo: 0x1affd, Hi: 0x1affe, Stride: 0x1},
		unicode.Range32{Lo: 0x1b000, Hi: 0x1b000, Stride: 0x1},
		unicode.Range32{Lo: 0x1b120, Hi: 0x1b122, Stride: 0x1},
		unicode.Range32{Lo: 0x1b155, Hi: 0x1b155, Stride: 0x1},
		unicode.Range32{Lo: 0x1b164, Hi: 0x1b167, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _WordLF = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0xa, Hi: 0xa, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _WordMidLetter = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x3a, Hi: 0x3a, Stride: 0x1},
		unicode.Range16{Lo: 0xb7, Hi: 0xb7, Stride: 0x1},
		unicode.Range16{Lo: 0x387, Hi: 0x387, Stride: 0x1},
		unicode.Range16{Lo: 0x55f, Hi: 0x55f, Stride: 0x1},
		unicode.Range16{Lo: 0x5f4, Hi: 0x5f4, Stride: 0x1},
		unicode.Range16{Lo: 0x2027, Hi: 0x2027, Stride: 0x1},
		unicode.Range16{Lo: 0xfe13, Hi: 0xfe13, Stride: 0x1},
		unicode.Range16{Lo: 0xfe55, Hi: 0xfe55, Stride: 0x1},
		unicode.Range16{Lo: 0xff1a, Hi: 0xff1a, Stride: 0x1},
	},
	LatinOffset: 2,
}

var _WordMidNum = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x2c, Hi: 0x2c, Stride: 0x1},
		unicode.Range16{Lo: 0x3b, Hi: 0x3b, Stride: 0x1},
		unicode.Range16{Lo: 0x37e, Hi: 0x37e, Stride: 0x1},
		unicode.Range16{Lo: 0x589, Hi: 0x589, Stride: 0x1},
		unicode.Range16{Lo: 0x60c, Hi: 0x60d, Stride: 0x1},
		unicode.Range16{Lo: 0x66c, Hi: 0x66c, Stride: 0x1},
		unicode.Range16{Lo: 0x7f8, Hi: 0x7f8, Stride: 0x1},
		unicode.Range16{Lo: 0x2044, Hi: 0x2044, Stride: 0x1},
		unicode.Range16{Lo: 0xfe10, Hi: 0xfe10, Stride: 0x1},
		unicode.Range16{Lo: 0xfe14, Hi: 0xfe14, Stride: 0x1},
		unicode.Range16{Lo: 0xfe50, Hi: 0xfe50, Stride: 0x1},
		unicode.Range16{Lo: 0xfe54, Hi: 0xfe54, Stride: 0x1},
		unicode.Range16{Lo: 0xff0c, Hi: 0xff0c, Stride: 0x1},
		unicode.Range16{Lo: 0xff1b, Hi: 0xff1b, Stride: 0x1},
	},
	LatinOffset: 2,
}

var _WordMidNumLet = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x2e, Hi: 0x2e, Stride: 0x1},
		unicode.Range16{Lo: 0x2018, Hi: 0x2018, Stride: 0x1},
		unicode.Range16{Lo: 0x2019, Hi: 0x2019, Stride: 0x1},
		unicode.Range16{Lo: 0x2024, Hi: 0x2024, Stride: 0x1},
		unicode.Range16{Lo: 0xfe52, Hi: 0xfe52, Stride: 0x1},
		unicode.Range16{Lo: 0xff07, Hi: 0xff07, Stride: 0x1},
		unicode.Range16{Lo: 0xff0e, Hi: 0xff0e, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _WordNewline = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0xb, Hi: 0xc, Stride: 0x1},
		unicode.Range16{Lo: 0x85, Hi: 0x85, Stride: 0x1},
		unicode.Range16{Lo: 0x2028, Hi: 0x2028, Stride: 0x1},
		unicode.Range16{Lo: 0x2029, Hi: 0x2029, Stride: 0x1},
	},
	LatinOffset: 2,
}

var _WordNumeric = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x30, Hi: 0x39, Stride: 0x1},
		unicode.Range16{Lo: 0x660, Hi: 0x669, Stride: 0x1},
		unicode.Range16{Lo: 0x66b, Hi: 0x66b, Stride: 0x1},
		unicode.Range16{Lo: 0x6f0, Hi: 0x6f9, Stride: 0x1},
		unicode.Range16{Lo: 0x7c0, Hi: 0x7c9, Stride: 0x1},
		unicode.Range16{Lo: 0x966, Hi: 0x96f, Stride: 0x1},
		unicode.Range16{Lo: 0x9e6, Hi: 0x9ef, Stride: 0x1},
		unicode.Range16{Lo: 0xa66, Hi: 0xa6f, Stride: 0x1},
		unicode.Range16{Lo: 0xae6, Hi: 0xaef, Stride: 0x1},
		unicode.Range16{Lo: 0xb66, Hi: 0xb6f, Stride: 0x1},
		unicode.Range16{Lo: 0xbe6, Hi: 0xbef, Stride: 0x1},
		unicode.Range16{Lo: 0xc66, Hi: 0xc6f, Stride: 0x1},
		unicode.Range16{Lo: 0xce6, Hi: 0xcef, Stride: 0x1},
		unicode.Range16{Lo: 0xd66, Hi: 0xd6f, Stride: 0x1},
		unicode.Range16{Lo: 0xde6, Hi: 0xdef, Stride: 0x1},
		unicode.Range16{Lo: 0xe50, Hi: 0xe59, Stride: 0x1},
		unicode.Range16{Lo: 0xed0, Hi: 0xed9, Stride: 0x1},
		unicode.Range16{Lo: 0xf20, Hi: 0xf29, Stride: 0x1},
		unicode.Range16{Lo: 0x1040, Hi: 0x1049, Stride: 0x1},
		unicode.Range16{Lo: 0x1090, Hi: 0x1099, Stride: 0x1},
		unicode.Range16{Lo: 0x17e0, Hi: 0x17e9, Stride: 0x1},
		unicode.Range16{Lo: 0x1810, Hi: 0x1819, Stride: 0x1},
		unicode.Range16{Lo: 0x1946, Hi: 0x194f, Stride: 0x1},
		unicode.Range16{Lo: 0x19d0, Hi: 0x19d9, Stride: 0x1},
		unicode.Range16{Lo: 0x1a80, Hi: 0x1a89, Stride: 0x1},
		unicode.Range16{Lo: 0x1a90, Hi: 0x1a99, Stride: 0x1},
		unicode.Range16{Lo: 0x1b50, Hi: 0x1b59, Stride: 0x1},
		unicode.Range16{Lo: 0x1bb0, Hi: 0x1bb9, Stride: 0x1},
		unicode.Range16{Lo: 0x1c40, Hi: 0x1c49, Stride: 0x1},
		unicode.Range16{Lo: 0x1c50, Hi: 0x1c59, Stride: 0x1},
		unicode.Range16{Lo: 0xa620, Hi: 0xa629, Stride: 0x1},
		unicode.Range16{Lo: 0xa8d0, Hi: 0xa8d9, Stride: 0x1},
		unicode.Range16{Lo: 0xa900, Hi: 0xa909, Stride: 0x1},
		unicode.Range16{Lo: 0xa9d0, Hi: 0xa9d9, Stride: 0x1},
		unicode.Range16{Lo: 0xa9f0, Hi: 0xa9f9, Stride: 0x1},
		unicode.Range16{Lo: 0xaa50, Hi: 0xaa59, Stride: 0x1},
		unicode.Range16{Lo: 0xabf0, Hi: 0xabf9, Stride: 0x1},
		unicode.Range16{Lo: 0xff10, Hi: 0xff19, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x104a0, Hi: 0x104a9, Stride: 0x1},
		unicode.Range32{Lo: 0x10d30, Hi: 0x10d39, Stride: 0x1},
		unicode.Range32{Lo: 0x11066, Hi: 0x1106f, Stride: 0x1},
		unicode.Range32{Lo: 0x110f0, Hi: 0x110f9, Stride: 0x1},
		unicode.Range32{Lo: 0x11136, Hi: 0x1113f, Stride: 0x1},
		unicode.Range32{Lo: 0x111d0, Hi: 0x111d9, Stride: 0x1},
		unicode.Range32{Lo: 0x112f0, Hi: 0x112f9, Stride: 0x1},
		unicode.Range32{Lo: 0x11450, Hi: 0x11459, Stride: 0x1},
		unicode.Range32{Lo: 0x114d0, Hi: 0x114d9, Stride: 0x1},
		unicode.Range32{Lo: 0x11650, Hi: 0x11659, Stride: 0x1},
		unicode.Range32{Lo: 0x116c0, Hi: 0x116c9, Stride: 0x1},
		unicode.Range32{Lo: 0x11730, Hi: 0x11739, Stride: 0x1},
		unicode.Range32{Lo: 0x118e0, Hi: 0x118e9, Stride: 0x1},
		unicode.Range32{Lo: 0x11950, Hi: 0x11959, Stride: 0x1},
		unicode.Range32{Lo: 0x11c50, Hi: 0x11c59, Stride: 0x1},
		unicode.Range32{Lo: 0x11d50, Hi: 0x11d59, Stride: 0x1},
		unicode.Range32{Lo: 0x11da0, Hi: 0x11da9, Stride: 0x1},
		unicode.Range32{Lo: 0x11f50, Hi: 0x11f59, Stride: 0x1},
		unicode.Range32{Lo: 0x16a60, Hi: 0x16a69, Stride: 0x1},
		unicode.Range32{Lo: 0x16ac0, Hi: 0x16ac9, Stride: 0x1},
		unicode.Range32{Lo: 0x16b50, Hi: 0x16b59, Stride: 0x1},
		unicode.Range32{Lo: 0x1d7ce, Hi: 0x1d7ff, Stride: 0x1},
		unicode.Range32{Lo: 0x1e140, Hi: 0x1e149, Stride: 0x1},
		unicode.Range32{Lo: 0x1e2f0, Hi: 0x1e2f9, Stride: 0x1},
		unicode.Range32{Lo: 0x1e4f0, Hi: 0x1e4f9, Stride: 0x1},
		unicode.Range32{Lo: 0x1e950, Hi: 0x1e959, Stride: 0x1},
		unicode.Range32{Lo: 0x1fbf0, Hi: 0x1fbf9, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _WordRegional_Indicator = &unicode.RangeTable{
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x1f1e6, Hi: 0x1f1ff, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _WordSingle_Quote = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x27, Hi: 0x27, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _WordWSegSpace = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x20, Hi: 0x20, Stride: 0x1},
		unicode.Range16{Lo: 0x1680, Hi: 0x1680, Stride: 0x1},
		unicode.Range16{Lo: 0x2000, Hi: 0x2006, Stride: 0x1},
		unicode.Range16{Lo: 0x2008, Hi: 0x200a, Stride: 0x1},
		unicode.Range16{Lo: 0x205f, Hi: 0x205f, Stride: 0x1},
		unicode.Range16{Lo: 0x3000, Hi: 0x3000, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _WordZWJ = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x200d, Hi: 0x200d, Stride: 0x1},
	},
	LatinOffset: 0,
}

type _WordRuneRange unicode.RangeTable

func _WordRuneType(r rune) *_WordRuneRange {
	switch {
	case unicode.Is(_WordALetter, r):
		return (*_WordRuneRange)(_WordALetter)
	case unicode.Is(_WordCR, r):
		return (*_WordRuneRange)(_WordCR)
	case unicode.Is(_WordDouble_Quote, r):
		return (*_WordRuneRange)(_WordDouble_Quote)
	case unicode.Is(_WordExtend, r):
		return (*_WordRuneRange)(_WordExtend)
	case unicode.Is(_WordExtendNumLet, r):
		return (*_WordRuneRange)(_WordExtendNumLet)
	case unicode.Is(_WordFormat, r):
		return (*_WordRuneRange)(_WordFormat)
	case unicode.Is(_WordHebrew_Letter, r):
		return (*_WordRuneRange)(_WordHebrew_Letter)
	case unicode.Is(_WordKatakana, r):
		return (*_WordRuneRange)(_WordKatakana)
	case unicode.Is(_WordLF, r):
		return (*_WordRuneRange)(_WordLF)
	case unicode.Is(_WordMidLetter, r):
		return (*_WordRuneRange)(_WordMidLetter)
	case unicode.Is(_WordMidNum, r):
		return (*_WordRuneRange)(_WordMidNum)
	case unicode.Is(_WordMidNumLet, r):
		return (*_WordRuneRange)(_WordMidNumLet)
	case unicode.Is(_WordNewline, r):
		return (*_WordRuneRange)(_WordNewline)
	case unicode.Is(_WordNumeric, r):
		return (*_WordRuneRange)(_WordNumeric)
	case unicode.Is(_WordRegional_Indicator, r):
		return (*_WordRuneRange)(_WordRegional_Indicator)
	case unicode.Is(_WordSingle_Quote, r):
		return (*_WordRuneRange)(_WordSingle_Quote)
	case unicode.Is(_WordWSegSpace, r):
		return (*_WordRuneRange)(_WordWSegSpace)
	case unicode.Is(_WordZWJ, r):
		return (*_WordRuneRange)(_WordZWJ)
	default:
		return nil
	}
}
func (rng *_WordRuneRange) String() string {
	switch (*unicode.RangeTable)(rng) {
	case _WordALetter:
		return "ALetter"
	case _WordCR:
		return "CR"
	case _WordDouble_Quote:
		return "Double_Quote"
	case _WordExtend:
		return "Extend"
	case _WordExtendNumLet:
		return "ExtendNumLet"
	case _WordFormat:
		return "Format"
	case _WordHebrew_Letter:
		return "Hebrew_Letter"
	case _WordKatakana:
		return "Katakana"
	case _WordLF:
		return "LF"
	case _WordMidLetter:
		return "MidLetter"
	case _WordMidNum:
		return "MidNum"
	case _WordMidNumLet:
		return "MidNumLet"
	case _WordNewline:
		return "Newline"
	case _WordNumeric:
		return "Numeric"
	case _WordRegional_Indicator:
		return "Regional_Indicator"
	case _WordSingle_Quote:
		return "Single_Quote"
	case _WordWSegSpace:
		return "WSegSpace"
	case _WordZWJ:
		return "ZWJ"
	default:
		return "Other"
	}
}

var _SentenceATerm = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x2e, Hi: 0x2e, Stride: 0x1},
		unicode.Range16{Lo: 0x2024, Hi: 0x2024, Stride: 0x1},
		unicode.Range16{Lo: 0xfe52, Hi: 0xfe52, Stride: 0x1},
		unicode.Range16{Lo: 0xff0e, Hi: 0xff0e, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _SentenceCR = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0xd, Hi: 0xd, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _SentenceClose = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x22, Hi: 0x22, Stride: 0x1},
		unicode.Range16{Lo: 0x27, Hi: 0x27, Stride: 0x1},
		unicode.Range16{Lo: 0x28, Hi: 0x28, Stride: 0x1},
		unicode.Range16{Lo: 0x29, Hi: 0x29, Stride: 0x1},
		unicode.Range16{Lo: 0x5b, Hi: 0x5b, Stride: 0x1},
		unicode.Range16{Lo: 0x5d, Hi: 0x5d, Stride: 0x1},
		unicode.Range16{Lo: 0x7b, Hi: 0x7b, Stride: 0x1},
		unicode.Range16{Lo: 0x7d, Hi: 0x7d, Stride: 0x1},
		unicode.Range16{Lo: 0xab, Hi: 0xab, Stride: 0x1},
		unicode.Range16{Lo: 0xbb, Hi: 0xbb, Stride: 0x1},
		unicode.Range16{Lo: 0xf3a, Hi: 0xf3a, Stride: 0x1},
		unicode.Range16{Lo: 0xf3b, Hi: 0xf3b, Stride: 0x1},
		unicode.Range16{Lo: 0xf3c, Hi: 0xf3c, Stride: 0x1},
		unicode.Range16{Lo: 0xf3d, Hi: 0xf3d, Stride: 0x1},
		unicode.Range16{Lo: 0x169b, Hi: 0x169b, Stride: 0x1},
		unicode.Range16{Lo: 0x169c, Hi: 0x169c, Stride: 0x1},
		unicode.Range16{Lo: 0x2018, Hi: 0x2018, Stride: 0x1},
		unicode.Range16{Lo: 0x2019, Hi: 0x2019, Stride: 0x1},
		unicode.Range16{Lo: 0x201a, Hi: 0x201a, Stride: 0x1},
		unicode.Range16{Lo: 0x201b, Hi: 0x201c, Stride: 0x1},
		unicode.Range16{Lo: 0x201d, Hi: 0x201d, Stride: 0x1},
		unicode.Range16{Lo: 0x201e, Hi: 0x201e, Stride: 0x1},
		unicode.Range16{Lo: 0x201f, Hi: 0x201f, Stride: 0x1},
		unicode.Range16{Lo: 0x2039, Hi: 0x2039, Stride: 0x1},
		unicode.Range16{Lo: 0x203a, Hi: 0x203a, Stride: 0x1},
		unicode.Range16{Lo: 0x2045, Hi: 0x2045, Stride: 0x1},
		unicode.Range16{Lo: 0x2046, Hi: 0x2046, Stride: 0x1},
		unicode.Range16{Lo: 0x207d, Hi: 0x207d, Stride: 0x1},
		unicode.Range16{Lo: 0x207e, Hi: 0x207e, Stride: 0x1},
		unicode.Range16{Lo: 0x208d, Hi: 0x208d, Stride: 0x1},
		unicode.Range16{Lo: 0x208e, Hi: 0x208e, Stride: 0x1},
		unicode.Range16{Lo: 0x2308, Hi: 0x2308, Stride: 0x1},
		unicode.Range16{Lo: 0x2309, Hi: 0x2309, Stride: 0x1},
		unicode.Range16{Lo: 0x230a, Hi: 0x230a, Stride: 0x1},
		unicode.Range16{Lo: 0x230b, Hi: 0x230b, Stride: 0x1},
		unicode.Range16{Lo: 0x2329, Hi: 0x2329, Stride: 0x1},
		unicode.Range16{Lo: 0x232a, Hi: 0x232a, Stride: 0x1},
		unicode.Range16{Lo: 0x275b, Hi: 0x2760, Stride: 0x1},
		unicode.Range16{Lo: 0x2768, Hi: 0x2768, Stride: 0x1},
		unicode.Range16{Lo: 0x2769, Hi: 0x2769, Stride: 0x1},
		unicode.Range16{Lo: 0x276a, Hi: 0x276a, Stride: 0x1},
		unicode.Range16{Lo: 0x276b, Hi: 0x276b, Stride: 0x1},
		unicode.Range16{Lo: 0x276c, Hi: 0x276c, Stride: 0x1},
		unicode.Range16{Lo: 0x276d, Hi: 0x276d, Stride: 0x1},
		unicode.Range16{Lo: 0x276e, Hi: 0x276e, Stride: 0x1},
		unicode.Range16{Lo: 0x276f, Hi: 0x276f, Stride: 0x1},
		unicode.Range16{Lo: 0x2770, Hi: 0x2770, Stride: 0x1},
		unicode.Range16{Lo: 0x2771, Hi: 0x2771, Stride: 0x1},
		unicode.Range16{Lo: 0x2772, Hi: 0x2772, Stride: 0x1},
		unicode.Range16{Lo: 0x2773, Hi: 0x2773, Stride: 0x1},
		unicode.Range16{Lo: 0x2774, Hi: 0x2774, Stride: 0x1},
		unicode.Range16{Lo: 0x2775, Hi: 0x2775, Stride: 0x1},
		unicode.Range16{Lo: 0x27c5, Hi: 0x27c5, Stride: 0x1},
		unicode.Range16{Lo: 0x27c6, Hi: 0x27c6, Stride: 0x1},
		unicode.Range16{Lo: 0x27e6, Hi: 0x27e6, Stride: 0x1},
		unicode.Range16{Lo: 0x27e7, Hi: 0x27e7, Stride: 0x1},
		unicode.Range16{Lo: 0x27e8, Hi: 0x27e8, Stride: 0x1},
		unicode.Range16{Lo: 0x27e9, Hi: 0x27e9, Stride: 0x1},
		unicode.Range16{Lo: 0x27ea, Hi: 0x27ea, Stride: 0x1},
		unicode.Range16{Lo: 0x27eb, Hi: 0x27eb, Stride: 0x1},
		unicode.Range16{Lo: 0x27ec, Hi: 0x27ec, Stride: 0x1},
		unicode.Range16{Lo: 0x27ed, Hi: 0x27ed, Stride: 0x1},
		unicode.Range16{Lo: 0x27ee, Hi: 0x27ee, Stride: 0x1},
		unicode.Range16{Lo: 0x27ef, Hi: 0x27ef, Stride: 0x1},
		unicode.Range16{Lo: 0x2983, Hi: 0x2983, Stride: 0x1},
		unicode.Range16{Lo: 0x2984, Hi: 0x2984, Stride: 0x1},
		unicode.Range16{Lo: 0x2985, Hi: 0x2985, Stride: 0x1},
		unicode.Range16{Lo: 0x2986, Hi: 0x2986, Stride: 0x1},
		unicode.Range16{Lo: 0x2987, Hi: 0x2987, Stride: 0x1},
		unicode.Range16{Lo: 0x2988, Hi: 0x2988, Stride: 0x1},
		unicode.Range16{Lo: 0x2989, Hi: 0x2989, Stride: 0x1},
		unicode.Range16{Lo: 0x298a, Hi: 0x298a, Stride: 0x1},
		unicode.Range16{Lo: 0x298b, Hi: 0x298b, Stride: 0x1},
		unicode.Range16{Lo: 0x298c, Hi: 0x298c, Stride: 0x1},
		unicode.Range16{Lo: 0x298d, Hi: 0x298d, Stride: 0x1},
		unicode.Range16{Lo: 0x298e, Hi: 0x298e, Stride: 0x1},
		unicode.Range16{Lo: 0x298f, Hi: 0x298f, Stride: 0x1},
		unicode.Range16{Lo: 0x2990, Hi: 0x2990, Stride: 0x1},
		unicode.Range16{Lo: 0x2991, Hi: 0x2991, Stride: 0x1},
		unicode.Range16{Lo: 0x2992, Hi: 0x2992, Stride: 0x1},
		unicode.Range16{Lo: 0x2993, Hi: 0x2993, Stride: 0x1},
		unicode.Range16{Lo: 0x2994, Hi: 0x2994, Stride: 0x1},
		unicode.Range16{Lo: 0x2995, Hi: 0x2995, Stride: 0x1},
		unicode.Range16{Lo: 0x2996, Hi: 0x2996, Stride: 0x1},
		unicode.Range16{Lo: 0x2997, Hi: 0x2997, Stride: 0x1},
		unicode.Range16{Lo: 0x2998, Hi: 0x2998, Stride: 0x1},
		unicode.Range16{Lo: 0x29d8, Hi: 0x29d8, Stride: 0x1},
		unicode.Range16{Lo: 0x29d9, Hi: 0x29d9, Stride: 0x1},
		unicode.Range16{Lo: 0x29da, Hi: 0x29da, Stride: 0x1},
		unicode.Range16{Lo: 0x29db, Hi: 0x29db, Stride: 0x1},
		unicode.Range16{Lo: 0x29fc, Hi: 0x29fc, Stride: 0x1},
		unicode.Range16{Lo: 0x29fd, Hi: 0x29fd, Stride: 0x1},
		unicode.Range16{Lo: 0x2e00, Hi: 0x2e01, Stride: 0x1},
		unicode.Range16{Lo: 0x2e02, Hi: 0x2e02, Stride: 0x1},
		unicode.Range16{Lo: 0x2e03, Hi: 0x2e03, Stride: 0x1},
		unicode.Range16{Lo: 0x2e04, Hi: 0x2e04, Stride: 0x1},
		unicode.Range16{Lo: 0x2e05, Hi: 0x2e05, Stride: 0x1},
		unicode.Range16{Lo: 0x2e06, Hi: 0x2e08, Stride: 0x1},
		unicode.Range16{Lo: 0x2e09, Hi: 0x2e09, Stride: 0x1},
		unicode.Range16{Lo: 0x2e0a, Hi: 0x2e0a, Stride: 0x1},
		unicode.Range16{Lo: 0x2e0b, Hi: 0x2e0b, Stride: 0x1},
		unicode.Range16{Lo: 0x2e0c, Hi: 0x2e0c, Stride: 0x1},
		unicode.Range16{Lo: 0x2e0d, Hi: 0x2e0d, Stride: 0x1},
		unicode.Range16{Lo: 0x2e1c, Hi: 0x2e1c, Stride: 0x1},
		unicode.Range16{Lo: 0x2e1d, Hi: 0x2e1d, Stride: 0x1},
		unicode.Range16{Lo: 0x2e20, Hi: 0x2e20, Stride: 0x1},
		unicode.Range16{Lo: 0x2e21, Hi: 0x2e21, Stride: 0x1},
		unicode.Range16{Lo: 0x2e22, Hi: 0x2e22, Stride: 0x1},
		unicode.Range16{Lo: 0x2e23, Hi: 0x2e23, Stride: 0x1},
		unicode.Range16{Lo: 0x2e24, Hi: 0x2e24, Stride: 0x1},
		unicode.Range16{Lo: 0x2e25, Hi: 0x2e25, Stride: 0x1},
		unicode.Range16{Lo: 0x2e26, Hi: 0x2e26, Stride: 0x1},
		unicode.Range16{Lo: 0x2e27, Hi: 0x2e27, Stride: 0x1},
		unicode.Range16{Lo: 0x2e28, Hi: 0x2e28, Stride: 0x1},
		unicode.Range16{Lo: 0x2e29, Hi: 0x2e29, Stride: 0x1},
		unicode.Range16{Lo: 0x2e42, Hi: 0x2e42, Stride: 0x1},
		unicode.Range16{Lo: 0x2e55, Hi: 0x2e55, Stride: 0x1},
		unicode.Range16{Lo: 0x2e56, Hi: 0x2e56, Stride: 0x1},
		unicode.Range16{Lo: 0x2e57, Hi: 0x2e57, Stride: 0x1},
		unicode.Range16{Lo: 0x2e58, Hi: 0x2e58, Stride: 0x1},
		unicode.Range16{Lo: 0x2e59, Hi: 0x2e59, Stride: 0x1},
		unicode.Range16{Lo: 0x2e5a, Hi: 0x2e5a, Stride: 0x1},
		unicode.Range16{Lo: 0x2e5b, Hi: 0x2e5b, Stride: 0x1},
		unicode.Range16{Lo: 0x2e5c, Hi: 0x2e5c, Stride: 0x1},
		unicode.Range16{Lo: 0x3008, Hi: 0x3008, Stride: 0x1},
		unicode.Range16{Lo: 0x3009, Hi: 0x3009, Stride: 0x1},
		unicode.Range16{Lo: 0x300a, Hi: 0x300a, Stride: 0x1},
		unicode.Range16{Lo: 0x300b, Hi: 0x300b, Stride: 0x1},
		unicode.Range16{Lo: 0x300c, Hi: 0x300c, Stride: 0x1},
		unicode.Range16{Lo: 0x300d, Hi: 0x300d, Stride: 0x1},
		unicode.Range16{Lo: 0x300e, Hi: 0x300e, Stride: 0x1},
		unicode.Range16{Lo: 0x300f, Hi: 0x300f, Stride: 0x1},
		unicode.Range16{Lo: 0x3010, Hi: 0x3010, Stride: 0x1},
		unicode.Range16{Lo: 0x3011, Hi: 0x3011, Stride: 0x1},
		unicode.Range16{Lo: 0x3014, Hi: 0x3014, Stride: 0x1},
		unicode.Range16{Lo: 0x3015, Hi: 0x3015, Stride: 0x1},
		unicode.Range16{Lo: 0x3016, Hi: 0x3016, Stride: 0x1},
		unicode.Range16{Lo: 0x3017, Hi: 0x3017, Stride: 0x1},
		unicode.Range16{Lo: 0x3018, Hi: 0x3018, Stride: 0x1},
		unicode.Range16{Lo: 0x3019, Hi: 0x3019, Stride: 0x1},
		unicode.Range16{Lo: 0x301a, Hi: 0x301a, Stride: 0x1},
		unicode.Range16{Lo: 0x301b, Hi: 0x301b, Stride: 0x1},
		unicode.Range16{Lo: 0x301d, Hi: 0x301d, Stride: 0x1},
		unicode.Range16{Lo: 0x301e, Hi: 0x301f, Stride: 0x1},
		unicode.Range16{Lo: 0xfd3e, Hi: 0xfd3e, Stride: 0x1},
		unicode.Range16{Lo: 0xfd3f, Hi: 0xfd3f, Stride: 0x1},
		unicode.Range16{Lo: 0xfe17, Hi: 0xfe17, Stride: 0x1},
		unicode.Range16{Lo: 0xfe18, Hi: 0xfe18, Stride: 0x1},
		unicode.Range16{Lo: 0xfe35, Hi: 0xfe35, Stride: 0x1},
		unicode.Range16{Lo: 0xfe36, Hi: 0xfe36, Stride: 0x1},
		unicode.Range16{Lo: 0xfe37, Hi: 0xfe37, Stride: 0x1},
		unicode.Range16{Lo: 0xfe38, Hi: 0xfe38, Stride: 0x1},
		unicode.Range16{Lo: 0xfe39, Hi: 0xfe39, Stride: 0x1},
		unicode.Range16{Lo: 0xfe3a, Hi: 0xfe3a, Stride: 0x1},
		unicode.Range16{Lo: 0xfe3b, Hi: 0xfe3b, Stride: 0x1},
		unicode.Range16{Lo: 0xfe3c, Hi: 0xfe3c, Stride: 0x1},
		unicode.Range16{Lo: 0xfe3d, Hi: 0xfe3d, Stride: 0x1},
		unicode.Range16{Lo: 0xfe3e, Hi: 0xfe3e, Stride: 0x1},
		unicode.Range16{Lo: 0xfe3f, Hi: 0xfe3f, Stride: 0x1},
		unicode.Range16{Lo: 0xfe40, Hi: 0xfe40, Stride: 0x1},
		unicode.Range16{Lo: 0xfe41, Hi: 0xfe41, Stride: 0x1},
		unicode.Range16{Lo: 0xfe42, Hi: 0xfe42, Stride: 0x1},
		unicode.Range16{Lo: 0xfe43, Hi: 0xfe43, Stride: 0x1},
		unicode.Range16{Lo: 0xfe44, Hi: 0xfe44, Stride: 0x1},
		unicode.Range16{Lo: 0xfe47, Hi: 0xfe47, Stride: 0x1},
		unicode.Range16{Lo: 0xfe48, Hi: 0xfe48, Stride: 0x1},
		unicode.Range16{Lo: 0xfe59, Hi: 0xfe59, Stride: 0x1},
		unicode.Range16{Lo: 0xfe5a, Hi: 0xfe5a, Stride: 0x1},
		unicode.Range16{Lo: 0xfe5b, Hi: 0xfe5b, Stride: 0x1},
		unicode.Range16{Lo: 0xfe5c, Hi: 0xfe5c, Stride: 0x1},
		unicode.Range16{Lo: 0xfe5d, Hi: 0xfe5d, Stride: 0x1},
		unicode.Range16{Lo: 0xfe5e, Hi: 0xfe5e, Stride: 0x1},
		unicode.Range16{Lo: 0xff08, Hi: 0xff08, Stride: 0x1},
		unicode.Range16{Lo: 0xff09, Hi: 0xff09, Stride: 0x1},
		unicode.Range16{Lo: 0xff3b, Hi: 0xff3b, Stride: 0x1},
		unicode.Range16{Lo: 0xff3d, Hi: 0xff3d, Stride: 0x1},
		unicode.Range16{Lo: 0xff5b, Hi: 0xff5b, Stride: 0x1},
		unicode.Range16{Lo: 0xff5d, Hi: 0xff5d, Stride: 0x1},
		unicode.Range16{Lo: 0xff5f, Hi: 0xff5f, Stride: 0x1},
		unicode.Range16{Lo: 0xff60, Hi: 0xff60, Stride: 0x1},
		unicode.Range16{Lo: 0xff62, Hi: 0xff62, Stride: 0x1},
		unicode.Range16{Lo: 0xff63, Hi: 0xff63, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x1f676, Hi: 0x1f678, Stride: 0x1},
	},
	LatinOffset: 10,
}

var _SentenceExtend = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x300, Hi: 0x36f, Stride: 0x1},
		unicode.Range16{Lo: 0x483, Hi: 0x487, Stride: 0x1},
		unicode.Range16{Lo: 0x488, Hi: 0x489, Stride: 0x1},
		unicode.Range16{Lo: 0x591, Hi: 0x5bd, Stride: 0x1},
		unicode.Range16{Lo: 0x5bf, Hi: 0x5bf, Stride: 0x1},
		unicode.Range16{Lo: 0x5c1, Hi: 0x5c2, Stride: 0x1},
		unicode.Range16{Lo: 0x5c4, Hi: 0x5c5, Stride: 0x1},
		unicode.Range16{Lo: 0x5c7, Hi: 0x5c7, Stride: 0x1},
		unicode.Range16{Lo: 0x610, Hi: 0x61a, Stride: 0x1},
		unicode.Range16{Lo: 0x64b, Hi: 0x65f, Stride: 0x1},
		unicode.Range16{Lo: 0x670, Hi: 0x670, Stride: 0x1},
		unicode.Range16{Lo: 0x6d6, Hi: 0x6dc, Stride: 0x1},
		unicode.Range16{Lo: 0x6df, Hi: 0x6e4, Stride: 0x1},
		unicode.Range16{Lo: 0x6e7, Hi: 0x6e8, Stride: 0x1},
		unicode.Range16{Lo: 0x6ea, Hi: 0x6ed, Stride: 0x1},
		unicode.Range16{Lo: 0x711, Hi: 0x711, Stride: 0x1},
		unicode.Range16{Lo: 0x730, Hi: 0x74a, Stride: 0x1},
		unicode.Range16{Lo: 0x7a6, Hi: 0x7b0, Stride: 0x1},
		unicode.Range16{Lo: 0x7eb, Hi: 0x7f3, Stride: 0x1},
		unicode.Range16{Lo: 0x7fd, Hi: 0x7fd, Stride: 0x1},
		unicode.Range16{Lo: 0x816, Hi: 0x819, Stride: 0x1},
		unicode.Range16{Lo: 0x81b, Hi: 0x823, Stride: 0x1},
		unicode.Range16{Lo: 0x825, Hi: 0x827, Stride: 0x1},
		unicode.Range16{Lo: 0x829, Hi: 0x82d, Stride: 0x1},
		unicode.Range16{Lo: 0x859, Hi: 0x85b, Stride: 0x1},
		unicode.Range16{Lo: 0x898, Hi: 0x89f, Stride: 0x1},
		unicode.Range16{Lo: 0x8ca, Hi: 0x8e1, Stride: 0x1},
		unicode.Range16{Lo: 0x8e3, Hi: 0x902, Stride: 0x1},
		unicode.Range16{Lo: 0x903, Hi: 0x903, Stride: 0x1},
		unicode.Range16{Lo: 0x93a, Hi: 0x93a, Stride: 0x1},
		unicode.Range16{Lo: 0x93b, Hi: 0x93b, Stride: 0x1},
		unicode.Range16{Lo: 0x93c, Hi: 0x93c, Stride: 0x1},
		unicode.Range16{Lo: 0x93e, Hi: 0x940, Stride: 0x1},
		unicode.Range16{Lo: 0x941, Hi: 0x948, Stride: 0x1},
		unicode.Range16{Lo: 0x949, Hi: 0x94c, Stride: 0x1},
		unicode.Range16{Lo: 0x94d, Hi: 0x94d, Stride: 0x1},
		unicode.Range16{Lo: 0x94e, Hi: 0x94f, Stride: 0x1},
		unicode.Range16{Lo: 0x951, Hi: 0x957, Stride: 0x1},
		unicode.Range16{Lo: 0x962, Hi: 0x963, Stride: 0x1},
		unicode.Range16{Lo: 0x981, Hi: 0x981, Stride: 0x1},
		unicode.Range16{Lo: 0x982, Hi: 0x983, Stride: 0x1},
		unicode.Range16{Lo: 0x9bc, Hi: 0x9bc, Stride: 0x1},
		unicode.Range16{Lo: 0x9be, Hi: 0x9c0, Stride: 0x1},
		unicode.Range16{Lo: 0x9c1, Hi: 0x9c4, Stride: 0x1},
		unicode.Range16{Lo: 0x9c7, Hi: 0x9c8, Stride: 0x1},
		unicode.Range16{Lo: 0x9cb, Hi: 0x9cc, Stride: 0x1},
		unicode.Range16{Lo: 0x9cd, Hi: 0x9cd, Stride: 0x1},
		unicode.Range16{Lo: 0x9d7, Hi: 0x9d7, Stride: 0x1},
		unicode.Range16{Lo: 0x9e2, Hi: 0x9e3, Stride: 0x1},
		unicode.Range16{Lo: 0x9fe, Hi: 0x9fe, Stride: 0x1},
		unicode.Range16{Lo: 0xa01, Hi: 0xa02, Stride: 0x1},
		unicode.Range16{Lo: 0xa03, Hi: 0xa03, Stride: 0x1},
		unicode.Range16{Lo: 0xa3c, Hi: 0xa3c, Stride: 0x1},
		unicode.Range16{Lo: 0xa3e, Hi: 0xa40, Stride: 0x1},
		unicode.Range16{Lo: 0xa41, Hi: 0xa42, Stride: 0x1},
		unicode.Range16{Lo: 0xa47, Hi: 0xa48, Stride: 0x1},
		unicode.Range16{Lo: 0xa4b, Hi: 0xa4d, Stride: 0x1},
		unicode.Range16{Lo: 0xa51, Hi: 0xa51, Stride: 0x1},
		unicode.Range16{Lo: 0xa70, Hi: 0xa71, Stride: 0x1},
		unicode.Range16{Lo: 0xa75, Hi: 0xa75, Stride: 0x1},
		unicode.Range16{Lo: 0xa81, Hi: 0xa82, Stride: 0x1},
		unicode.Range16{Lo: 0xa83, Hi: 0xa83, Stride: 0x1},
		unicode.Range16{Lo: 0xabc, Hi: 0xabc, Stride: 0x1},
		unicode.Range16{Lo: 0xabe, Hi: 0xac0, Stride: 0x1},
		unicode.Range16{Lo: 0xac1, Hi: 0xac5, Stride: 0x1},
		unicode.Range16{Lo: 0xac7, Hi: 0xac8, Stride: 0x1},
		unicode.Range16{Lo: 0xac9, Hi: 0xac9, Stride: 0x1},
		unicode.Range16{Lo: 0xacb, Hi: 0xacc, Stride: 0x1},
		unicode.Range16{Lo: 0xacd, Hi: 0xacd, Stride: 0x1},
		unicode.Range16{Lo: 0xae2, Hi: 0xae3, Stride: 0x1},
		unicode.Range16{Lo: 0xafa, Hi: 0xaff, Stride: 0x1},
		unicode.Range16{Lo: 0xb01, Hi: 0xb01, Stride: 0x1},
		unicode.Range16{Lo: 0xb02, Hi: 0xb03, Stride: 0x1},
		unicode.Range16{Lo: 0xb3c, Hi: 0xb3c, Stride: 0x1},
		unicode.Range16{Lo: 0xb3e, Hi: 0xb3e, Stride: 0x1},
		unicode.Range16{Lo: 0xb3f, Hi: 0xb3f, Stride: 0x1},
		unicode.Range16{Lo: 0xb40, Hi: 0xb40, Stride: 0x1},
		unicode.Range16{Lo: 0xb41, Hi: 0xb44, Stride: 0x1},
		unicode.Range16{Lo: 0xb47, Hi: 0xb48, Stride: 0x1},
		unicode.Range16{Lo: 0xb4b, Hi: 0xb4c, Stride: 0x1},
		unicode.Range16{Lo: 0xb4d, Hi: 0xb4d, Stride: 0x1},
		unicode.Range16{Lo: 0xb55, Hi: 0xb56, Stride: 0x1},
		unicode.Range16{Lo: 0xb57, Hi: 0xb57, Stride: 0x1},
		unicode.Range16{Lo: 0xb62, Hi: 0xb63, Stride: 0x1},
		unicode.Range16{Lo: 0xb82, Hi: 0xb82, Stride: 0x1},
		unicode.Range16{Lo: 0xbbe, Hi: 0xbbf, Stride: 0x1},
		unicode.Range16{Lo: 0xbc0, Hi: 0xbc0, Stride: 0x1},
		unicode.Range16{Lo: 0xbc1, Hi: 0xbc2, Stride: 0x1},
		unicode.Range16{Lo: 0xbc6, Hi: 0xbc8, Stride: 0x1},
		unicode.Range16{Lo: 0xbca, Hi: 0xbcc, Stride: 0x1},
		unicode.Range16{Lo: 0xbcd, Hi: 0xbcd, Stride: 0x1},
		unicode.Range16{Lo: 0xbd7, Hi: 0xbd7, Stride: 0x1},
		unicode.Range16{Lo: 0xc00, Hi: 0xc00, Stride: 0x1},
		unicode.Range16{Lo: 0xc01, Hi: 0xc03, Stride: 0x1},
		unicode.Range16{Lo: 0xc04, Hi: 0xc04, Stride: 0x1},
		unicode.Range16{Lo: 0xc3c, Hi: 0xc3c, Stride: 0x1},
		unicode.Range16{Lo: 0xc3e, Hi: 0xc40, Stride: 0x1},
		unicode.Range16{Lo: 0xc41, Hi: 0xc44, Stride: 0x1},
		unicode.Range16{Lo: 0xc46, Hi: 0xc48, Stride: 0x1},
		unicode.Range16{Lo: 0xc4a, Hi: 0xc4d, Stride: 0x1},
		unicode.Range16{Lo: 0xc55, Hi: 0xc56, Stride: 0x1},
		unicode.Range16{Lo: 0xc62, Hi: 0xc63, Stride: 0x1},
		unicode.Range16{Lo: 0xc81, Hi: 0xc81, Stride: 0x1},
		unicode.Range16{Lo: 0xc82, Hi: 0xc83, Stride: 0x1},
		unicode.Range16{Lo: 0xcbc, Hi: 0xcbc, Stride: 0x1},
		unicode.Range16{Lo: 0xcbe, Hi: 0xcbe, Stride: 0x1},
		unicode.Range16{Lo: 0xcbf, Hi: 0xcbf, Stride: 0x1},
		unicode.Range16{Lo: 0xcc0, Hi: 0xcc4, Stride: 0x1},
		unicode.Range16{Lo: 0xcc6, Hi: 0xcc6, Stride: 0x1},
		unicode.Range16{Lo: 0xcc7, Hi: 0xcc8, Stride: 0x1},
		unicode.Range16{Lo: 0xcca, Hi: 0xccb, Stride: 0x1},
		unicode.Range16{Lo: 0xccc, Hi: 0xccd, Stride: 0x1},
		unicode.Range16{Lo: 0xcd5, Hi: 0xcd6, Stride: 0x1},
		unicode.Range16{Lo: 0xce2, Hi: 0xce3, Stride: 0x1},
		unicode.Range16{Lo: 0xcf3, Hi: 0xcf3, Stride: 0x1},
		unicode.Range16{Lo: 0xd00, Hi: 0xd01, Stride: 0x1},
		unicode.Range16{Lo: 0xd02, Hi: 0xd03, Stride: 0x1},
		unicode.Range16{Lo: 0xd3b, Hi: 0xd3c, Stride: 0x1},
		unicode.Range16{Lo: 0xd3e, Hi: 0xd40, Stride: 0x1},
		unicode.Range16{Lo: 0xd41, Hi: 0xd44, Stride: 0x1},
		unicode.Range16{Lo: 0xd46, Hi: 0xd48, Stride: 0x1},
		unicode.Range16{Lo: 0xd4a, Hi: 0xd4c, Stride: 0x1},
		unicode.Range16{Lo: 0xd4d, Hi: 0xd4d, Stride: 0x1},
		unicode.Range16{Lo: 0xd57, Hi: 0xd57, Stride: 0x1},
		unicode.Range16{Lo: 0xd62, Hi: 0xd63, Stride: 0x1},
		unicode.Range16{Lo: 0xd81, Hi: 0xd81, Stride: 0x1},
		unicode.Range16{Lo: 0xd82, Hi: 0xd83, Stride: 0x1},
		unicode.Range16{Lo: 0xdca, Hi: 0xdca, Stride: 0x1},
		unicode.Range16{Lo: 0xdcf, Hi: 0xdd1, Stride: 0x1},
		unicode.Range16{Lo: 0xdd2, Hi: 0xdd4, Stride: 0x1},
		unicode.Range16{Lo: 0xdd6, Hi: 0xdd6, Stride: 0x1},
		unicode.Range16{Lo: 0xdd8, Hi: 0xddf, Stride: 0x1},
		unicode.Range16{Lo: 0xdf2, Hi: 0xdf3, Stride: 0x1},
		unicode.Range16{Lo: 0xe31, Hi: 0xe31, Stride: 0x1},
		unicode.Range16{Lo: 0xe34, Hi: 0xe3a, Stride: 0x1},
		unicode.Range16{Lo: 0xe47, Hi: 0xe4e, Stride: 0x1},
		unicode.Range16{Lo: 0xeb1, Hi: 0xeb1, Stride: 0x1},
		unicode.Range16{Lo: 0xeb4, Hi: 0xebc, Stride: 0x1},
		unicode.Range16{Lo: 0xec8, Hi: 0xece, Stride: 0x1},
		unicode.Range16{Lo: 0xf18, Hi: 0xf19, Stride: 0x1},
		unicode.Range16{Lo: 0xf35, Hi: 0xf35, Stride: 0x1},
		unicode.Range16{Lo: 0xf37, Hi: 0xf37, Stride: 0x1},
		unicode.Range16{Lo: 0xf39, Hi: 0xf39, Stride: 0x1},
		unicode.Range16{Lo: 0xf3e, Hi: 0xf3f, Stride: 0x1},
		unicode.Range16{Lo: 0xf71, Hi: 0xf7e, Stride: 0x1},
		unicode.Range16{Lo: 0xf7f, Hi: 0xf7f, Stride: 0x1},
		unicode.Range16{Lo: 0xf80, Hi: 0xf84, Stride: 0x1},
		unicode.Range16{Lo: 0xf86, Hi: 0xf87, Stride: 0x1},
		unicode.Range16{Lo: 0xf8d, Hi: 0xf97, Stride: 0x1},
		unicode.Range16{Lo: 0xf99, Hi: 0xfbc, Stride: 0x1},
		unicode.Range16{Lo: 0xfc6, Hi: 0xfc6, Stride: 0x1},
		unicode.Range16{Lo: 0x102b, Hi: 0x102c, Stride: 0x1},
		unicode.Range16{Lo: 0x102d, Hi: 0x1030, Stride: 0x1},
		unicode.Range16{Lo: 0x1031, Hi: 0x1031, Stride: 0x1},
		unicode.Range16{Lo: 0x1032, Hi: 0x1037, Stride: 0x1},
		unicode.Range16{Lo: 0x1038, Hi: 0x1038, Stride: 0x1},
		unicode.Range16{Lo: 0x1039, Hi: 0x103a, Stride: 0x1},
		unicode.Range16{Lo: 0x103b, Hi: 0x103c, Stride: 0x1},
		unicode.Range16{Lo: 0x103d, Hi: 0x103e, Stride: 0x1},
		unicode.Range16{Lo: 0x1056, Hi: 0x1057, Stride: 0x1},
		unicode.Range16{Lo: 0x1058, Hi: 0x1059, Stride: 0x1},
		unicode.Range16{Lo: 0x105e, Hi: 0x1060, Stride: 0x1},
		unicode.Range16{Lo: 0x1062, Hi: 0x1064, Stride: 0x1},
		unicode.Range16{Lo: 0x1067, Hi: 0x106d, Stride: 0x1},
		unicode.Range16{Lo: 0x1071, Hi: 0x1074, Stride: 0x1},
		unicode.Range16{Lo: 0x1082, Hi: 0x1082, Stride: 0x1},
		unicode.Range16{Lo: 0x1083, Hi: 0x1084, Stride: 0x1},
		unicode.Range16{Lo: 0x1085, Hi: 0x1086, Stride: 0x1},
		unicode.Range16{Lo: 0x1087, Hi: 0x108c, Stride: 0x1},
		unicode.Range16{Lo: 0x108d, Hi: 0x108d, Stride: 0x1},
		unicode.Range16{Lo: 0x108f, Hi: 0x108f, Stride: 0x1},
		unicode.Range16{Lo: 0x109a, Hi: 0x109c, Stride: 0x1},
		unicode.Range16{Lo: 0x109d, Hi: 0x109d, Stride: 0x1},
		unicode.Range16{Lo: 0x135d, Hi: 0x135f, Stride: 0x1},
		unicode.Range16{Lo: 0x1712, Hi: 0x1714, Stride: 0x1},
		unicode.Range16{Lo: 0x1715, Hi: 0x1715, Stride: 0x1},
		unicode.Range16{Lo: 0x1732, Hi: 0x1733, Stride: 0x1},
		unicode.Range16{Lo: 0x1734, Hi: 0x1734, Stride: 0x1},
		unicode.Range16{Lo: 0x1752, Hi: 0x1753, Stride: 0x1},
		unicode.Range16{Lo: 0x1772, Hi: 0x1773, Stride: 0x1},
		unicode.Range16{Lo: 0x17b4, Hi: 0x17b5, Stride: 0x1},
		unicode.Range16{Lo: 0x17b6, Hi: 0x17b6, Stride: 0x1},
		unicode.Range16{Lo: 0x17b7, Hi: 0x17bd, Stride: 0x1},
		unicode.Range16{Lo: 0x17be, Hi: 0x17c5, Stride: 0x1},
		unicode.Range16{Lo: 0x17c6, Hi: 0x17c6, Stride: 0x1},
		unicode.Range16{Lo: 0x17c7, Hi: 0x17c8, Stride: 0x1},
		unicode.Range16{Lo: 0x17c9, Hi: 0x17d3, Stride: 0x1},
		unicode.Range16{Lo: 0x17dd, Hi: 0x17dd, Stride: 0x1},
		unicode.Range16{Lo: 0x180b, Hi: 0x180d, Stride: 0x1},
		unicode.Range16{Lo: 0x180f, Hi: 0x180f, Stride: 0x1},
		unicode.Range16{Lo: 0x1885, Hi: 0x1886, Stride: 0x1},
		unicode.Range16{Lo: 0x18a9, Hi: 0x18a9, Stride: 0x1},
		unicode.Range16{Lo: 0x1920, Hi: 0x1922, Stride: 0x1},
		unicode.Range16{Lo: 0x1923, Hi: 0x1926, Stride: 0x1},
		unicode.Range16{Lo: 0x1927, Hi: 0x1928, Stride: 0x1},
		unicode.Range16{Lo: 0x1929, Hi: 0x192b, Stride: 0x1},
		unicode.Range16{Lo: 0x1930, Hi: 0x1931, Stride: 0x1},
		unicode.Range16{Lo: 0x1932, Hi: 0x1932, Stride: 0x1},
		unicode.Range16{Lo: 0x1933, Hi: 0x1938, Stride: 0x1},
		unicode.Range16{Lo: 0x1939, Hi: 0x193b, Stride: 0x1},
		unicode.Range16{Lo: 0x1a17, Hi: 0x1a18, Stride: 0x1},
		unicode.Range16{Lo: 0x1a19, Hi: 0x1a1a, Stride: 0x1},
		unicode.Range16{Lo: 0x1a1b, Hi: 0x1a1b, Stride: 0x1},
		unicode.Range16{Lo: 0x1a55, Hi: 0x1a55, Stride: 0x1},
		unicode.Range16{Lo: 0x1a56, Hi: 0x1a56, Stride: 0x1},
		unicode.Range16{Lo: 0x1a57, Hi: 0x1a57, Stride: 0x1},
		unicode.Range16{Lo: 0x1a58, Hi: 0x1a5e, Stride: 0x1},
		unicode.Range16{Lo: 0x1a60, Hi: 0x1a60, Stride: 0x1},
		unicode.Range16{Lo: 0x1a61, Hi: 0x1a61, Stride: 0x1},
		unicode.Range16{Lo: 0x1a62, Hi: 0x1a62, Stride: 0x1},
		unicode.Range16{Lo: 0x1a63, Hi: 0x1a64, Stride: 0x1},
		unicode.Range16{Lo: 0x1a65, Hi: 0x1a6c, Stride: 0x1},
		unicode.Range16{Lo: 0x1a6d, Hi: 0x1a72, Stride: 0x1},
		unicode.Range16{Lo: 0x1a73, Hi: 0x1a7c, Stride: 0x1},
		unicode.Range16{Lo: 0x1a7f, Hi: 0x1a7f, Stride: 0x1},
		unicode.Range16{Lo: 0x1ab0, Hi: 0x1abd, Stride: 0x1},
		unicode.Range16{Lo: 0x1abe, Hi: 0x1abe, Stride: 0x1},
		unicode.Range16{Lo: 0x1abf, Hi: 0x1ace, Stride: 0x1},
		unicode.Range16{Lo: 0x1b00, Hi: 0x1b03, Stride: 0x1},
		unicode.Range16{Lo: 0x1b04, Hi: 0x1b04, Stride: 0x1},
		unicode.Range16{Lo: 0x1b34, Hi: 0x1b34, Stride: 0x1},
		unicode.Range16{Lo: 0x1b35, Hi: 0x1b35, Stride: 0x1},
		unicode.Range16{Lo: 0x1b36, Hi: 0x1b3a, Stride: 0x1},
		unicode.Range16{Lo: 0x1b3b, Hi: 0x1b3b, Stride: 0x1},
		unicode.Range16{Lo: 0x1b3c, Hi: 0x1b3c, Stride: 0x1},
		unicode.Range16{Lo: 0x1b3d, Hi: 0x1b41, Stride: 0x1},
		unicode.Range16{Lo: 0x1b42, Hi: 0x1b42, Stride: 0x1},
		unicode.Range16{Lo: 0x1b43, Hi: 0x1b44, Stride: 0x1},
		unicode.Range16{Lo: 0x1b6b, Hi: 0x1b73, Stride: 0x1},
		unicode.Range16{Lo: 0x1b80, Hi: 0x1b81, Stride: 0x1},
		unicode.Range16{Lo: 0x1b82, Hi: 0x1b82, Stride: 0x1},
		unicode.Range16{Lo: 0x1ba1, Hi: 0x1ba1, Stride: 0x1},
		unicode.Range16{Lo: 0x1ba2, Hi: 0x1ba5, Stride: 0x1},
		unicode.Range16{Lo: 0x1ba6, Hi: 0x1ba7, Stride: 0x1},
		unicode.Range16{Lo: 0x1ba8, Hi: 0x1ba9, Stride: 0x1},
		unicode.Range16{Lo: 0x1baa, Hi: 0x1baa, Stride: 0x1},
		unicode.Range16{Lo: 0x1bab, Hi: 0x1bad, Stride: 0x1},
		unicode.Range16{Lo: 0x1be6, Hi: 0x1be6, Stride: 0x1},
		unicode.Range16{Lo: 0x1be7, Hi: 0x1be7, Stride: 0x1},
		unicode.Range16{Lo: 0x1be8, Hi: 0x1be9, Stride: 0x1},
		unicode.Range16{Lo: 0x1bea, Hi: 0x1bec, Stride: 0x1},
		unicode.Range16{Lo: 0x1bed, Hi: 0x1bed, Stride: 0x1},
		unicode.Range16{Lo: 0x1bee, Hi: 0x1bee, Stride: 0x1},
		unicode.Range16{Lo: 0x1bef, Hi: 0x1bf1, Stride: 0x1},
		unicode.Range16{Lo: 0x1bf2, Hi: 0x1bf3, Stride: 0x1},
		unicode.Range16{Lo: 0x1c24, Hi: 0x1c2b, Stride: 0x1},
		unicode.Range16{Lo: 0x1c2c, Hi: 0x1c33, Stride: 0x1},
		unicode.Range16{Lo: 0x1c34, Hi: 0x1c35, Stride: 0x1},
		unicode.Range16{Lo: 0x1c36, Hi: 0x1c37, Stride: 0x1},
		unicode.Range16{Lo: 0x1cd0, Hi: 0x1cd2, Stride: 0x1},
		unicode.Range16{Lo: 0x1cd4, Hi: 0x1ce0, Stride: 0x1},
		unicode.Range16{Lo: 0x1ce1, Hi: 0x1ce1, Stride: 0x1},
		unicode.Range16{Lo: 0x1ce2, Hi: 0x1ce8, Stride: 0x1},
		unicode.Range16{Lo: 0x1ced, Hi: 0x1ced, Stride: 0x1},
		unicode.Range16{Lo: 0x1cf4, Hi: 0x1cf4, Stride: 0x1},
		unicode.Range16{Lo: 0x1cf7, Hi: 0x1cf7, Stride: 0x1},
		unicode.Range16{Lo: 0x1cf8, Hi: 0x1cf9, Stride: 0x1},
		unicode.Range16{Lo: 0x1dc0, Hi: 0x1dff, Stride: 0x1},
		unicode.Range16{Lo: 0x200c, Hi: 0x200d, Stride: 0x1},
		unicode.Range16{Lo: 0x20d0, Hi: 0x20dc, Stride: 0x1},
		unicode.Range16{Lo: 0x20dd, Hi: 0x20e0, Stride: 0x1},
		unicode.Range16{Lo: 0x20e1, Hi: 0x20e1, Stride: 0x1},
		unicode.Range16{Lo: 0x20e2, Hi: 0x20e4, Stride: 0x1},
		unicode.Range16{Lo: 0x20e5, Hi: 0x20f0, Stride: 0x1},
		unicode.Range16{Lo: 0x2cef, Hi: 0x2cf1, Stride: 0x1},
		unicode.Range16{Lo: 0x2d7f, Hi: 0x2d7f, Stride: 0x1},
		unicode.Range16{Lo: 0x2de0, Hi: 0x2dff, Stride: 0x1},
		unicode.Range16{Lo: 0x302a, Hi: 0x302d, Stride: 0x1},
		unicode.Range16{Lo: 0x302e, Hi: 0x302f, Stride: 0x1},
		unicode.Range16{Lo: 0x3099, Hi: 0x309a, Stride: 0x1},
		unicode.Range16{Lo: 0xa66f, Hi: 0xa66f, Stride: 0x1},
		unicode.Range16{Lo: 0xa670, Hi: 0xa672, Stride: 0x1},
		unicode.Range16{Lo: 0xa674, Hi: 0xa67d, Stride: 0x1},
		unicode.Range16{Lo: 0xa69e, Hi: 0xa69f, Stride: 0x1},
		unicode.Range16{Lo: 0xa6f0, Hi: 0xa6f1, Stride: 0x1},
		unicode.Range16{Lo: 0xa802, Hi: 0xa802, Stride: 0x1},
		unicode.Range16{Lo: 0xa806, Hi: 0xa806, Stride: 0x1},
		unicode.Range16{Lo: 0xa80b, Hi: 0xa80b, Stride: 0x1},
		unicode.Range16{Lo: 0xa823, Hi: 0xa824, Stride: 0x1},
		unicode.Range16{Lo: 0xa825, Hi: 0xa826, Stride: 0x1},
		unicode.Range16{Lo: 0xa827, Hi: 0xa827, Stride: 0x1},
		unicode.Range16{Lo: 0xa82c, Hi: 0xa82c, Stride: 0x1},
		unicode.Range16{Lo: 0xa880, Hi: 0xa881, Stride: 0x1},
		unicode.Range16{Lo: 0xa8b4, Hi: 0xa8c3, Stride: 0x1},
		unicode.Range16{Lo: 0xa8c4, Hi: 0xa8c5, Stride: 0x1},
		unicode.Range16{Lo: 0xa8e0, Hi: 0xa8f1, Stride: 0x1},
		unicode.Range16{Lo: 0xa8ff, Hi: 0xa8ff, Stride: 0x1},
		unicode.Range16{Lo: 0xa926, Hi: 0xa92d, Stride: 0x1},
		unicode.Range16{Lo: 0xa947, Hi: 0xa951, Stride: 0x1},
		unicode.Range16{Lo: 0xa952, Hi: 0xa953, Stride: 0x1},
		unicode.Range16{Lo: 0xa980, Hi: 0xa982, Stride: 0x1},
		unicode.Range16{Lo: 0xa983, Hi: 0xa983, Stride: 0x1},
		unicode.Range16{Lo: 0xa9b3, Hi: 0xa9b3, Stride: 0x1},
		unicode.Range16{Lo: 0xa9b4, Hi: 0xa9b5, Stride: 0x1},
		unicode.Range16{Lo: 0xa9b6, Hi: 0xa9b9, Stride: 0x1},
		unicode.Range16{Lo: 0xa9ba, Hi: 0xa9bb, Stride: 0x1},
		unicode.Range16{Lo: 0xa9bc, Hi: 0xa9bd, Stride: 0x1},
		unicode.Range16{Lo: 0xa9be, Hi: 0xa9c0, Stride: 0x1},
		unicode.Range16{Lo: 0xa9e5, Hi: 0xa9e5, Stride: 0x1},
		unicode.Range16{Lo: 0xaa29, Hi: 0xaa2e, Stride: 0x1},
		unicode.Range16{Lo: 0xaa2f, Hi: 0xaa30, Stride: 0x1},
		unicode.Range16{Lo: 0xaa31, Hi: 0xaa32, Stride: 0x1},
		unicode.Range16{Lo: 0xaa33, Hi: 0xaa34, Stride: 0x1},
		unicode.Range16{Lo: 0xaa35, Hi: 0xaa36, Stride: 0x1},
		unicode.Range16{Lo: 0xaa43, Hi: 0xaa43, Stride: 0x1},
		unicode.Range16{Lo: 0xaa4c, Hi: 0xaa4c, Stride: 0x1},
		unicode.Range16{Lo: 0xaa4d, Hi: 0xaa4d, Stride: 0x1},
		unicode.Range16{Lo: 0xaa7b, Hi: 0xaa7b, Stride: 0x1},
		unicode.Range16{Lo: 0xaa7c, Hi: 0xaa7c, Stride: 0x1},
		unicode.Range16{Lo: 0xaa7d, Hi: 0xaa7d, Stride: 0x1},
		unicode.Range16{Lo: 0xaab0, Hi: 0xaab0, Stride: 0x1},
		unicode.Range16{Lo: 0xaab2, Hi: 0xaab4, Stride: 0x1},
		unicode.Range16{Lo: 0xaab7, Hi: 0xaab8, Stride: 0x1},
		unicode.Range16{Lo: 0xaabe, Hi: 0xaabf, Stride: 0x1},
		unicode.Range16{Lo: 0xaac1, Hi: 0xaac1, Stride: 0x1},
		unicode.Range16{Lo: 0xaaeb, Hi: 0xaaeb, Stride: 0x1},
		unicode.Range16{Lo: 0xaaec, Hi: 0xaaed, Stride: 0x1},
		unicode.Range16{Lo: 0xaaee, Hi: 0xaaef, Stride: 0x1},
		unicode.Range16{Lo: 0xaaf5, Hi: 0xaaf5, Stride: 0x1},
		unicode.Range16{Lo: 0xaaf6, Hi: 0xaaf6, Stride: 0x1},
		unicode.Range16{Lo: 0xabe3, Hi: 0xabe4, Stride: 0x1},
		unicode.Range16{Lo: 0xabe5, Hi: 0xabe5, Stride: 0x1},
		unicode.Range16{Lo: 0xabe6, Hi: 0xabe7, Stride: 0x1},
		unicode.Range16{Lo: 0xabe8, Hi: 0xabe8, Stride: 0x1},
		unicode.Range16{Lo: 0xabe9, Hi: 0xabea, Stride: 0x1},
		unicode.Range16{Lo: 0xabec, Hi: 0xabec, Stride: 0x1},
		unicode.Range16{Lo: 0xabed, Hi: 0xabed, Stride: 0x1},
		unicode.Range16{Lo: 0xfb1e, Hi: 0xfb1e, Stride: 0x1},
		unicode.Range16{Lo: 0xfe00, Hi: 0xfe0f, Stride: 0x1},
		unicode.Range16{Lo: 0xfe20, Hi: 0xfe2f, Stride: 0x1},
		unicode.Range16{Lo: 0xff9e, Hi: 0xff9f, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x101fd, Hi: 0x101fd, Stride: 0x1},
		unicode.Range32{Lo: 0x102e0, Hi: 0x102e0, Stride: 0x1},
		unicode.Range32{Lo: 0x10376, Hi: 0x1037a, Stride: 0x1},
		unicode.Range32{Lo: 0x10a01, Hi: 0x10a03, Stride: 0x1},
		unicode.Range32{Lo: 0x10a05, Hi: 0x10a06, Stride: 0x1},
		unicode.Range32{Lo: 0x10a0c, Hi: 0x10a0f, Stride: 0x1},
		unicode.Range32{Lo: 0x10a38, Hi: 0x10a3a, Stride: 0x1},
		unicode.Range32{Lo: 0x10a3f, Hi: 0x10a3f, Stride: 0x1},
		unicode.Range32{Lo: 0x10ae5, Hi: 0x10ae6, Stride: 0x1},
		unicode.Range32{Lo: 0x10d24, Hi: 0x10d27, Stride: 0x1},
		unicode.Range32{Lo: 0x10eab, Hi: 0x10eac, Stride: 0x1},
		unicode.Range32{Lo: 0x10efd, Hi: 0x10eff, Stride: 0x1},
		unicode.Range32{Lo: 0x10f46, Hi: 0x10f50, Stride: 0x1},
		unicode.Range32{Lo: 0x10f82, Hi: 0x10f85, Stride: 0x1},
		unicode.Range32{Lo: 0x11000, Hi: 0x11000, Stride: 0x1},
		unicode.Range32{Lo: 0x11001, Hi: 0x11001, Stride: 0x1},
		unicode.Range32{Lo: 0x11002, Hi: 0x11002, Stride: 0x1},
		unicode.Range32{Lo: 0x11038, Hi: 0x11046, Stride: 0x1},
		unicode.Range32{Lo: 0x11070, Hi: 0x11070, Stride: 0x1},
		unicode.Range32{Lo: 0x11073, Hi: 0x11074, Stride: 0x1},
		unicode.Range32{Lo: 0x1107f, Hi: 0x11081, Stride: 0x1},
		unicode.Range32{Lo: 0x11082, Hi: 0x11082, Stride: 0x1},
		unicode.Range32{Lo: 0x110b0, Hi: 0x110b2, Stride: 0x1},
		unicode.Range32{Lo: 0x110b3, Hi: 0x110b6, Stride: 0x1},
		unicode.Range32{Lo: 0x110b7, Hi: 0x110b8, Stride: 0x1},
		unicode.Range32{Lo: 0x110b9, Hi: 0x110ba, Stride: 0x1},
		unicode.Range32{Lo: 0x110c2, Hi: 0x110c2, Stride: 0x1},
		unicode.Range32{Lo: 0x11100, Hi: 0x11102, Stride: 0x1},
		unicode.Range32{Lo: 0x11127, Hi: 0x1112b, Stride: 0x1},
		unicode.Range32{Lo: 0x1112c, Hi: 0x1112c, Stride: 0x1},
		unicode.Range32{Lo: 0x1112d, Hi: 0x11134, Stride: 0x1},
		unicode.Range32{Lo: 0x11145, Hi: 0x11146, Stride: 0x1},
		unicode.Range32{Lo: 0x11173, Hi: 0x11173, Stride: 0x1},
		unicode.Range32{Lo: 0x11180, Hi: 0x11181, Stride: 0x1},
		unicode.Range32{Lo: 0x11182, Hi: 0x11182, Stride: 0x1},
		unicode.Range32{Lo: 0x111b3, Hi: 0x111b5, Stride: 0x1},
		unicode.Range32{Lo: 0x111b6, Hi: 0x111be, Stride: 0x1},
		unicode.Range32{Lo: 0x111bf, Hi: 0x111c0, Stride: 0x1},
		unicode.Range32{Lo: 0x111c9, Hi: 0x111cc, Stride: 0x1},
		unicode.Range32{Lo: 0x111ce, Hi: 0x111ce, Stride: 0x1},
		unicode.Range32{Lo: 0x111cf, Hi: 0x111cf, Stride: 0x1},
		unicode.Range32{Lo: 0x1122c, Hi: 0x1122e, Stride: 0x1},
		unicode.Range32{Lo: 0x1122f, Hi: 0x11231, Stride: 0x1},
		unicode.Range32{Lo: 0x11232, Hi: 0x11233, Stride: 0x1},
		unicode.Range32{Lo: 0x11234, Hi: 0x11234, Stride: 0x1},
		unicode.Range32{Lo: 0x11235, Hi: 0x11235, Stride: 0x1},
		unicode.Range32{Lo: 0x11236, Hi: 0x11237, Stride: 0x1},
		unicode.Range32{Lo: 0x1123e, Hi: 0x1123e, Stride: 0x1},
		unicode.Range32{Lo: 0x11241, Hi: 0x11241, Stride: 0x1},
		unicode.Range32{Lo: 0x112df, Hi: 0x112df, Stride: 0x1},
		unicode.Range32{Lo: 0x112e0, Hi: 0x112e2, Stride: 0x1},
		unicode.Range32{Lo: 0x112e3, Hi: 0x112ea, Stride: 0x1},
		unicode.Range32{Lo: 0x11300, Hi: 0x11301, Stride: 0x1},
		unicode.Range32{Lo: 0x11302, Hi: 0x11303, Stride: 0x1},
		unicode.Range32{Lo: 0x1133b, Hi: 0x1133c, Stride: 0x1},
		unicode.Range32{Lo: 0x1133e, Hi: 0x1133f, Stride: 0x1},
		unicode.Range32{Lo: 0x11340, Hi: 0x11340, Stride: 0x1},
		unicode.Range32{Lo: 0x11341, Hi: 0x11344, Stride: 0x1},
		unicode.Range32{Lo: 0x11347, Hi: 0x11348, Stride: 0x1},
		unicode.Range32{Lo: 0x1134b, Hi: 0x1134d, Stride: 0x1},
		unicode.Range32{Lo: 0x11357, Hi: 0x11357, Stride: 0x1},
		unicode.Range32{Lo: 0x11362, Hi: 0x11363, Stride: 0x1},
		unicode.Range32{Lo: 0x11366, Hi: 0x1136c, Stride: 0x1},
		unicode.Range32{Lo: 0x11370, Hi: 0x11374, Stride: 0x1},
		unicode.Range32{Lo: 0x11435, Hi: 0x11437, Stride: 0x1},
		unicode.Range32{Lo: 0x11438, Hi: 0x1143f, Stride: 0x1},
		unicode.Range32{Lo: 0x11440, Hi: 0x11441, Stride: 0x1},
		unicode.Range32{Lo: 0x11442, Hi: 0x11444, Stride: 0x1},
		unicode.Range32{Lo: 0x11445, Hi: 0x11445, Stride: 0x1},
		unicode.Range32{Lo: 0x11446, Hi: 0x11446, Stride: 0x1},
		unicode.Range32{Lo: 0x1145e, Hi: 0x1145e, Stride: 0x1},
		unicode.Range32{Lo: 0x114b0, Hi: 0x114b2, Stride: 0x1},
		unicode.Range32{Lo: 0x114b3, Hi: 0x114b8, Stride: 0x1},
		unicode.Range32{Lo: 0x114b9, Hi: 0x114b9, Stride: 0x1},
		unicode.Range32{Lo: 0x114ba, Hi: 0x114ba, Stride: 0x1},
		unicode.Range32{Lo: 0x114bb, Hi: 0x114be, Stride: 0x1},
		unicode.Range32{Lo: 0x114bf, Hi: 0x114c0, Stride: 0x1},
		unicode.Range32{Lo: 0x114c1, Hi: 0x114c1, Stride: 0x1},
		unicode.Range32{Lo: 0x114c2, Hi: 0x114c3, Stride: 0x1},
		unicode.Range32{Lo: 0x115af, Hi: 0x115b1, Stride: 0x1},
		unicode.Range32{Lo: 0x115b2, Hi: 0x115b5, Stride: 0x1},
		unicode.Range32{Lo: 0x115b8, Hi: 0x115bb, Stride: 0x1},
		unicode.Range32{Lo: 0x115bc, Hi: 0x115bd, Stride: 0x1},
		unicode.Range32{Lo: 0x115be, Hi: 0x115be, Stride: 0x1},
		unicode.Range32{Lo: 0x115bf, Hi: 0x115c0, Stride: 0x1},
		unicode.Range32{Lo: 0x115dc, Hi: 0x115dd, Stride: 0x1},
		unicode.Range32{Lo: 0x11630, Hi: 0x11632, Stride: 0x1},
		unicode.Range32{Lo: 0x11633, Hi: 0x1163a, Stride: 0x1},
		unicode.Range32{Lo: 0x1163b, Hi: 0x1163c, Stride: 0x1},
		unicode.Range32{Lo: 0x1163d, Hi: 0x1163d, Stride: 0x1},
		unicode.Range32{Lo: 0x1163e, Hi: 0x1163e, Stride: 0x1},
		unicode.Range32{Lo: 0x1163f, Hi: 0x11640, Stride: 0x1},
		unicode.Range32{Lo: 0x116ab, Hi: 0x116ab, Stride: 0x1},
		unicode.Range32{Lo: 0x116ac, Hi: 0x116ac, Stride: 0x1},
		unicode.Range32{Lo: 0x116ad, Hi: 0x116ad, Stride: 0x1},
		unicode.Range32{Lo: 0x116ae, Hi: 0x116af, Stride: 0x1},
		unicode.Range32{Lo: 0x116b0, Hi: 0x116b5, Stride: 0x1},
		unicode.Range32{Lo: 0x116b6, Hi: 0x116b6, Stride: 0x1},
		unicode.Range32{Lo: 0x116b7, Hi: 0x116b7, Stride: 0x1},
		unicode.Range32{Lo: 0x1171d, Hi: 0x1171f, Stride: 0x1},
		unicode.Range32{Lo: 0x11720, Hi: 0x11721, Stride: 0x1},
		unicode.Range32{Lo: 0x11722, Hi: 0x11725, Stride: 0x1},
		unicode.Range32{Lo: 0x11726, Hi: 0x11726, Stride: 0x1},
		unicode.Range32{Lo: 0x11727, Hi: 0x1172b, Stride: 0x1},
		unicode.Range32{Lo: 0x1182c, Hi: 0x1182e, Stride: 0x1},
		unicode.Range32{Lo: 0x1182f, Hi: 0x11837, Stride: 0x1},
		unicode.Range32{Lo: 0x11838, Hi: 0x11838, Stride: 0x1},
		unicode.Range32{Lo: 0x11839, Hi: 0x1183a, Stride: 0x1},
		unicode.Range32{Lo: 0x11930, Hi: 0x11935, Stride: 0x1},
		unicode.Range32{Lo: 0x11937, Hi: 0x11938, Stride: 0x1},
		unicode.Range32{Lo: 0x1193b, Hi: 0x1193c, Stride: 0x1},
		unicode.Range32{Lo: 0x1193d, Hi: 0x1193d, Stride: 0x1},
		unicode.Range32{Lo: 0x1193e, Hi: 0x1193e, Stride: 0x1},
		unicode.Range32{Lo: 0x11940, Hi: 0x11940, Stride: 0x1},
		unicode.Range32{Lo: 0x11942, Hi: 0x11942, Stride: 0x1},
		unicode.Range32{Lo: 0x11943, Hi: 0x11943, Stride: 0x1},
		unicode.Range32{Lo: 0x119d1, Hi: 0x119d3, Stride: 0x1},
		unicode.Range32{Lo: 0x119d4, Hi: 0x119d7, Stride: 0x1},
		unicode.Range32{Lo: 0x119da, Hi: 0x119db, Stride: 0x1},
		unicode.Range32{Lo: 0x119dc, Hi: 0x119df, Stride: 0x1},
		unicode.Range32{Lo: 0x119e0, Hi: 0x119e0, Stride: 0x1},
		unicode.Range32{Lo: 0x119e4, Hi: 0x119e4, Stride: 0x1},
		unicode.Range32{Lo: 0x11a01, Hi: 0x11a0a, Stride: 0x1},
		unicode.Range32{Lo: 0x11a33, Hi: 0x11a38, Stride: 0x1},
		unicode.Range32{Lo: 0x11a39, Hi: 0x11a39, Stride: 0x1},
		unicode.Range32{Lo: 0x11a3b, Hi: 0x11a3e, Stride: 0x1},
		unicode.Range32{Lo: 0x11a47, Hi: 0x11a47, Stride: 0x1},
		unicode.Range32{Lo: 0x11a51, Hi: 0x11a56, Stride: 0x1},
		unicode.Range32{Lo: 0x11a57, Hi: 0x11a58, Stride: 0x1},
		unicode.Range32{Lo: 0x11a59, Hi: 0x11a5b, Stride: 0x1},
		unicode.Range32{Lo: 0x11a8a, Hi: 0x11a96, Stride: 0x1},
		unicode.Range32{Lo: 0x11a97, Hi: 0x11a97, Stride: 0x1},
		unicode.Range32{Lo: 0x11a98, Hi: 0x11a99, Stride: 0x1},
		unicode.Range32{Lo: 0x11c2f, Hi: 0x11c2f, Stride: 0x1},
		unicode.Range32{Lo: 0x11c30, Hi: 0x11c36, Stride: 0x1},
		unicode.Range32{Lo: 0x11c38, Hi: 0x11c3d, Stride: 0x1},
		unicode.Range32{Lo: 0x11c3e, Hi: 0x11c3e, Stride: 0x1},
		unicode.Range32{Lo: 0x11c3f, Hi: 0x11c3f, Stride: 0x1},
		unicode.Range32{Lo: 0x11c92, Hi: 0x11ca7, Stride: 0x1},
		unicode.Range32{Lo: 0x11ca9, Hi: 0x11ca9, Stride: 0x1},
		unicode.Range32{Lo: 0x11caa, Hi: 0x11cb0, Stride: 0x1},
		unicode.Range32{Lo: 0x11cb1, Hi: 0x11cb1, Stride: 0x1},
		unicode.Range32{Lo: 0x11cb2, Hi: 0x11cb3, Stride: 0x1},
		unicode.Range32{Lo: 0x11cb4, Hi: 0x11cb4, Stride: 0x1},
		unicode.Range32{Lo: 0x11cb5, Hi: 0x11cb6, Stride: 0x1},
		unicode.Range32{Lo: 0x11d31, Hi: 0x11d36, Stride: 0x1},
		unicode.Range32{Lo: 0x11d3a, Hi: 0x11d3a, Stride: 0x1},
		unicode.Range32{Lo: 0x11d3c, Hi: 0x11d3d, Stride: 0x1},
		unicode.Range32{Lo: 0x11d3f, Hi: 0x11d45, Stride: 0x1},
		unicode.Range32{Lo: 0x11d47, Hi: 0x11d47, Stride: 0x1},
		unicode.Range32{Lo: 0x11d8a, Hi: 0x11d8e, Stride: 0x1},
		unicode.Range32{Lo: 0x11d90, Hi: 0x11d91, Stride: 0x1},
		unicode.Range32{Lo: 0x11d93, Hi: 0x11d94, Stride: 0x1},
		unicode.Range32{Lo: 0x11d95, Hi: 0x11d95, Stride: 0x1},
		unicode.Range32{Lo: 0x11d96, Hi: 0x11d96, Stride: 0x1},
		unicode.Range32{Lo: 0x11d97, Hi: 0x11d97, Stride: 0x1},
		unicode.Range32{Lo: 0x11ef3, Hi: 0x11ef4, Stride: 0x1},
		unicode.Range32{Lo: 0x11ef5, Hi: 0x11ef6, Stride: 0x1},
		unicode.Range32{Lo: 0x11f00, Hi: 0x11f01, Stride: 0x1},
		unicode.Range32{Lo: 0x11f03, Hi: 0x11f03, Stride: 0x1},
		unicode.Range32{Lo: 0x11f34, Hi: 0x11f35, Stride: 0x1},
		unicode.Range32{Lo: 0x11f36, Hi: 0x11f3a, Stride: 0x1},
		unicode.Range32{Lo: 0x11f3e, Hi: 0x11f3f, Stride: 0x1},
		unicode.Range32{Lo: 0x11f40, Hi: 0x11f40, Stride: 0x1},
		unicode.Range32{Lo: 0x11f41, Hi: 0x11f41, Stride: 0x1},
		unicode.Range32{Lo: 0x11f42, Hi: 0x11f42, Stride: 0x1},
		unicode.Range32{Lo: 0x13440, Hi: 0x13440, Stride: 0x1},
		unicode.Range32{Lo: 0x13447, Hi: 0x13455, Stride: 0x1},
		unicode.Range32{Lo: 0x16af0, Hi: 0x16af4, Stride: 0x1},
		unicode.Range32{Lo: 0x16b30, Hi: 0x16b36, Stride: 0x1},
		unicode.Range32{Lo: 0x16f4f, Hi: 0x16f4f, Stride: 0x1},
		unicode.Range32{Lo: 0x16f51, Hi: 0x16f87, Stride: 0x1},
		unicode.Range32{Lo: 0x16f8f, Hi: 0x16f92, Stride: 0x1},
		unicode.Range32{Lo: 0x16fe4, Hi: 0x16fe4, Stride: 0x1},
		unicode.Range32{Lo: 0x16ff0, Hi: 0x16ff1, Stride: 0x1},
		unicode.Range32{Lo: 0x1bc9d, Hi: 0x1bc9e, Stride: 0x1},
		unicode.Range32{Lo: 0x1cf00, Hi: 0x1cf2d, Stride: 0x1},
		unicode.Range32{Lo: 0x1cf30, Hi: 0x1cf46, Stride: 0x1},
		unicode.Range32{Lo: 0x1d165, Hi: 0x1d166, Stride: 0x1},
		unicode.Range32{Lo: 0x1d167, Hi: 0x1d169, Stride: 0x1},
		unicode.Range32{Lo: 0x1d16d, Hi: 0x1d172, Stride: 0x1},
		unicode.Range32{Lo: 0x1d17b, Hi: 0x1d182, Stride: 0x1},
		unicode.Range32{Lo: 0x1d185, Hi: 0x1d18b, Stride: 0x1},
		unicode.Range32{Lo: 0x1d1aa, Hi: 0x1d1ad, Stride: 0x1},
		unicode.Range32{Lo: 0x1d242, Hi: 0x1d244, Stride: 0x1},
		unicode.Range32{Lo: 0x1da00, Hi: 0x1da36, Stride: 0x1},
		unicode.Range32{Lo: 0x1da3b, Hi: 0x1da6c, Stride: 0x1},
		unicode.Range32{Lo: 0x1da75, Hi: 0x1da75, Stride: 0x1},
		unicode.Range32{Lo: 0x1da84, Hi: 0x1da84, Stride: 0x1},
		unicode.Range32{Lo: 0x1da9b, Hi: 0x1da9f, Stride: 0x1},
		unicode.Range32{Lo: 0x1daa1, Hi: 0x1daaf, Stride: 0x1},
		unicode.Range32{Lo: 0x1e000, Hi: 0x1e006, Stride: 0x1},
		unicode.Range32{Lo: 0x1e008, Hi: 0x1e018, Stride: 0x1},
		unicode.Range32{Lo: 0x1e01b, Hi: 0x1e021, Stride: 0x1},
		unicode.Range32{Lo: 0x1e023, Hi: 0x1e024, Stride: 0x1},
		unicode.Range32{Lo: 0x1e026, Hi: 0x1e02a, Stride: 0x1},
		unicode.Range32{Lo: 0x1e08f, Hi: 0x1e08f, Stride: 0x1},
		unicode.Range32{Lo: 0x1e130, Hi: 0x1e136, Stride: 0x1},
		unicode.Range32{Lo: 0x1e2ae, Hi: 0x1e2ae, Stride: 0x1},
		unicode.Range32{Lo: 0x1e2ec, Hi: 0x1e2ef, Stride: 0x1},
		unicode.Range32{Lo: 0x1e4ec, Hi: 0x1e4ef, Stride: 0x1},
		unicode.Range32{Lo: 0x1e8d0, Hi: 0x1e8d6, Stride: 0x1},
		unicode.Range32{Lo: 0x1e944, Hi: 0x1e94a, Stride: 0x1},
		unicode.Range32{Lo: 0xe0020, Hi: 0xe007f, Stride: 0x1},
		unicode.Range32{Lo: 0xe0100, Hi: 0xe01ef, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _SentenceFormat = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0xad, Hi: 0xad, Stride: 0x1},
		unicode.Range16{Lo: 0x600, Hi: 0x605, Stride: 0x1},
		unicode.Range16{Lo: 0x61c, Hi: 0x61c, Stride: 0x1},
		unicode.Range16{Lo: 0x6dd, Hi: 0x6dd, Stride: 0x1},
		unicode.Range16{Lo: 0x70f, Hi: 0x70f, Stride: 0x1},
		unicode.Range16{Lo: 0x890, Hi: 0x891, Stride: 0x1},
		unicode.Range16{Lo: 0x8e2, Hi: 0x8e2, Stride: 0x1},
		unicode.Range16{Lo: 0x180e, Hi: 0x180e, Stride: 0x1},
		unicode.Range16{Lo: 0x200b, Hi: 0x200b, Stride: 0x1},
		unicode.Range16{Lo: 0x200e, Hi: 0x200f, Stride: 0x1},
		unicode.Range16{Lo: 0x202a, Hi: 0x202e, Stride: 0x1},
		unicode.Range16{Lo: 0x2060, Hi: 0x2064, Stride: 0x1},
		unicode.Range16{Lo: 0x2066, Hi: 0x206f, Stride: 0x1},
		unicode.Range16{Lo: 0xfeff, Hi: 0xfeff, Stride: 0x1},
		unicode.Range16{Lo: 0xfff9, Hi: 0xfffb, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x110bd, Hi: 0x110bd, Stride: 0x1},
		unicode.Range32{Lo: 0x110cd, Hi: 0x110cd, Stride: 0x1},
		unicode.Range32{Lo: 0x13430, Hi: 0x1343f, Stride: 0x1},
		unicode.Range32{Lo: 0x1bca0, Hi: 0x1bca3, Stride: 0x1},
		unicode.Range32{Lo: 0x1d173, Hi: 0x1d17a, Stride: 0x1},
		unicode.Range32{Lo: 0xe0001, Hi: 0xe0001, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _SentenceLF = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0xa, Hi: 0xa, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _SentenceLower = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x61, Hi: 0x7a, Stride: 0x1},
		unicode.Range16{Lo: 0xaa, Hi: 0xaa, Stride: 0x1},
		unicode.Range16{Lo: 0xb5, Hi: 0xb5, Stride: 0x1},
		unicode.Range16{Lo: 0xba, Hi: 0xba, Stride: 0x1},
		unicode.Range16{Lo: 0xdf, Hi: 0xf6, Stride: 0x1},
		unicode.Range16{Lo: 0xf8, Hi: 0xff, Stride: 0x1},
		unicode.Range16{Lo: 0x101, Hi: 0x101, Stride: 0x1},
		unicode.Range16{Lo: 0x103, Hi: 0x103, Stride: 0x1},
		unicode.Range16{Lo: 0x105, Hi: 0x105, Stride: 0x1},
		unicode.Range16{Lo: 0x107, Hi: 0x107, Stride: 0x1},
		unicode.Range16{Lo: 0x109, Hi: 0x109, Stride: 0x1},
		unicode.Range16{Lo: 0x10b, Hi: 0x10b, Stride: 0x1},
		unicode.Range16{Lo: 0x10d, Hi: 0x10d, Stride: 0x1},
		unicode.Range16{Lo: 0x10f, Hi: 0x10f, Stride: 0x1},
		unicode.Range16{Lo: 0x111, Hi: 0x111, Stride: 0x1},
		unicode.Range16{Lo: 0x113, Hi: 0x113, Stride: 0x1},
		unicode.Range16{Lo: 0x115, Hi: 0x115, Stride: 0x1},
		unicode.Range16{Lo: 0x117, Hi: 0x117, Stride: 0x1},
		unicode.Range16{Lo: 0x119, Hi: 0x119, Stride: 0x1},
		unicode.Range16{Lo: 0x11b, Hi: 0x11b, Stride: 0x1},
		unicode.Range16{Lo: 0x11d, Hi: 0x11d, Stride: 0x1},
		unicode.Range16{Lo: 0x11f, Hi: 0x11f, Stride: 0x1},
		unicode.Range16{Lo: 0x121, Hi: 0x121, Stride: 0x1},
		unicode.Range16{Lo: 0x123, Hi: 0x123, Stride: 0x1},
		unicode.Range16{Lo: 0x125, Hi: 0x125, Stride: 0x1},
		unicode.Range16{Lo: 0x127, Hi: 0x127, Stride: 0x1},
		unicode.Range16{Lo: 0x129, Hi: 0x129, Stride: 0x1},
		unicode.Range16{Lo: 0x12b, Hi: 0x12b, Stride: 0x1},
		unicode.Range16{Lo: 0x12d, Hi: 0x12d, Stride: 0x1},
		unicode.Range16{Lo: 0x12f, Hi: 0x12f, Stride: 0x1},
		unicode.Range16{Lo: 0x131, Hi: 0x131, Stride: 0x1},
		unicode.Range16{Lo: 0x133, Hi: 0x133, Stride: 0x1},
		unicode.Range16{Lo: 0x135, Hi: 0x135, Stride: 0x1},
		unicode.Range16{Lo: 0x137, Hi: 0x138, Stride: 0x1},
		unicode.Range16{Lo: 0x13a, Hi: 0x13a, Stride: 0x1},
		unicode.Range16{Lo: 0x13c, Hi: 0x13c, Stride: 0x1},
		unicode.Range16{Lo: 0x13e, Hi: 0x13e, Stride: 0x1},
		unicode.Range16{Lo: 0x140, Hi: 0x140, Stride: 0x1},
		unicode.Range16{Lo: 0x142, Hi: 0x142, Stride: 0x1},
		unicode.Range16{Lo: 0x144, Hi: 0x144, Stride: 0x1},
		unicode.Range16{Lo: 0x146, Hi: 0x146, Stride: 0x1},
		unicode.Range16{Lo: 0x148, Hi: 0x149, Stride: 0x1},
		unicode.Range16{Lo: 0x14b, Hi: 0x14b, Stride: 0x1},
		unicode.Range16{Lo: 0x14d, Hi: 0x14d, Stride: 0x1},
		unicode.Range16{Lo: 0x14f, Hi: 0x14f, Stride: 0x1},
		unicode.Range16{Lo: 0x151, Hi: 0x151, Stride: 0x1},
		unicode.Range16{Lo: 0x153, Hi: 0x153, Stride: 0x1},
		unicode.Range16{Lo: 0x155, Hi: 0x155, Stride: 0x1},
		unicode.Range16{Lo: 0x157, Hi: 0x157, Stride: 0x1},
		unicode.Range16{Lo: 0x159, Hi: 0x159, Stride: 0x1},
		unicode.Range16{Lo: 0x15b, Hi: 0x15b, Stride: 0x1},
		unicode.Range16{Lo: 0x15d, Hi: 0x15d, Stride: 0x1},
		unicode.Range16{Lo: 0x15f, Hi: 0x15f, Stride: 0x1},
		unicode.Range16{Lo: 0x161, Hi: 0x161, Stride: 0x1},
		unicode.Range16{Lo: 0x163, Hi: 0x163, Stride: 0x1},
		unicode.Range16{Lo: 0x165, Hi: 0x165, Stride: 0x1},
		unicode.Range16{Lo: 0x167, Hi: 0x167, Stride: 0x1},
		unicode.Range16{Lo: 0x169, Hi: 0x169, Stride: 0x1},
		unicode.Range16{Lo: 0x16b, Hi: 0x16b, Stride: 0x1},
		unicode.Range16{Lo: 0x16d, Hi: 0x16d, Stride: 0x1},
		unicode.Range16{Lo: 0x16f, Hi: 0x16f, Stride: 0x1},
		unicode.Range16{Lo: 0x171, Hi: 0x171, Stride: 0x1},
		unicode.Range16{Lo: 0x173, Hi: 0x173, Stride: 0x1},
		unicode.Range16{Lo: 0x175, Hi: 0x175, Stride: 0x1},
		unicode.Range16{Lo: 0x177, Hi: 0x177, Stride: 0x1},
		unicode.Range16{Lo: 0x17a, Hi: 0x17a, Stride: 0x1},
		unicode.Range16{Lo: 0x17c, Hi: 0x17c, Stride: 0x1},
		unicode.Range16{Lo: 0x17e, Hi: 0x180, Stride: 0x1},
		unicode.Range16{Lo: 0x183, Hi: 0x183, Stride: 0x1},
		unicode.Range16{Lo: 0x185, Hi: 0x185, Stride: 0x1},
		unicode.Range16{Lo: 0x188, Hi: 0x188, Stride: 0x1},
		unicode.Range16{Lo: 0x18c, Hi: 0x18d, Stride: 0x1},
		unicode.Range16{Lo: 0x192, Hi: 0x192, Stride: 0x1},
		unicode.Range16{Lo: 0x195, Hi: 0x195, Stride: 0x1},
		unicode.Range16{Lo: 0x199, Hi: 0x19b, Stride: 0x1},
		unicode.Range16{Lo: 0x19e, Hi: 0x19e, Stride: 0x1},
		unicode.Range16{Lo: 0x1a1, Hi: 0x1a1, Stride: 0x1},
		unicode.Range16{Lo: 0x1a3, Hi: 0x1a3, Stride: 0x1},
		unicode.Range16{Lo: 0x1a5, Hi: 0x1a5, Stride: 0x1},
		unicode.Range16{Lo: 0x1a8, Hi: 0x1a8, Stride: 0x1},
		unicode.Range16{Lo: 0x1aa, Hi: 0x1ab, Stride: 0x1},
		unicode.Range16{Lo: 0x1ad, Hi: 0x1ad, Stride: 0x1},
		unicode.Range16{Lo: 0x1b0, Hi: 0x1b0, Stride: 0x1},
		unicode.Range16{Lo: 0x1b4, Hi: 0x1b4, Stride: 0x1},
		unicode.Range16{Lo: 0x1b6, Hi: 0x1b6, Stride: 0x1},
		unicode.Range16{Lo: 0x1b9, Hi: 0x1ba, Stride: 0x1},
		unicode.Range16{Lo: 0x1bd, Hi: 0x1bf, Stride: 0x1},
		unicode.Range16{Lo: 0x1c6, Hi: 0x1c6, Stride: 0x1},
		unicode.Range16{Lo: 0x1c9, Hi: 0x1c9, Stride: 0x1},
		unicode.Range16{Lo: 0x1cc, Hi: 0x1cc, Stride: 0x1},
		unicode.Range16{Lo: 0x1ce, Hi: 0x1ce, Stride: 0x1},
		unicode.Range16{Lo: 0x1d0, Hi: 0x1d0, Stride: 0x1},
		unicode.Range16{Lo: 0x1d2, Hi: 0x1d2, Stride: 0x1},
		unicode.Range16{Lo: 0x1d4, Hi: 0x1d4, Stride: 0x1},
		unicode.Range16{Lo: 0x1d6, Hi: 0x1d6, Stride: 0x1},
		unicode.Range16{Lo: 0x1d8, Hi: 0x1d8, Stride: 0x1},
		unicode.Range16{Lo: 0x1da, Hi: 0x1da, Stride: 0x1},
		unicode.Range16{Lo: 0x1dc, Hi: 0x1dd, Stride: 0x1},
		unicode.Range16{Lo: 0x1df, Hi: 0x1df, Stride: 0x1},
		unicode.Range16{Lo: 0x1e1, Hi: 0x1e1, Stride: 0x1},
		unicode.Range16{Lo: 0x1e3, Hi: 0x1e3, Stride: 0x1},
		unicode.Range16{Lo: 0x1e5, Hi: 0x1e5, Stride: 0x1},
		unicode.Range16{Lo: 0x1e7, Hi: 0x1e7, Stride: 0x1},
		unicode.Range16{Lo: 0x1e9, Hi: 0x1e9, Stride: 0x1},
		unicode.Range16{Lo: 0x1eb, Hi: 0x1eb, Stride: 0x1},
		unicode.Range16{Lo: 0x1ed, Hi: 0x1ed, Stride: 0x1},
		unicode.Range16{Lo: 0x1ef, Hi: 0x1f0, Stride: 0x1},
		unicode.Range16{Lo: 0x1f3, Hi: 0x1f3, Stride: 0x1},
		unicode.Range16{Lo: 0x1f5, Hi: 0x1f5, Stride: 0x1},
		unicode.Range16{Lo: 0x1f9, Hi: 0x1f9, Stride: 0x1},
		unicode.Range16{Lo: 0x1fb, Hi: 0x1fb, Stride: 0x1},
		unicode.Range16{Lo: 0x1fd, Hi: 0x1fd, Stride: 0x1},
		unicode.Range16{Lo: 0x1ff, Hi: 0x1ff, Stride: 0x1},
		unicode.Range16{Lo: 0x201, Hi: 0x201, Stride: 0x1},
		unicode.Range16{Lo: 0x203, Hi: 0x203, Stride: 0x1},
		unicode.Range16{Lo: 0x205, Hi: 0x205, Stride: 0x1},
		unicode.Range16{Lo: 0x207, Hi: 0x207, Stride: 0x1},
		unicode.Range16{Lo: 0x209, Hi: 0x209, Stride: 0x1},
		unicode.Range16{Lo: 0x20b, Hi: 0x20b, Stride: 0x1},
		unicode.Range16{Lo: 0x20d, Hi: 0x20d, Stride: 0x1},
		unicode.Range16{Lo: 0x20f, Hi: 0x20f, Stride: 0x1},
		unicode.Range16{Lo: 0x211, Hi: 0x211, Stride: 0x1},
		unicode.Range16{Lo: 0x213, Hi: 0x213, Stride: 0x1},
		unicode.Range16{Lo: 0x215, Hi: 0x215, Stride: 0x1},
		unicode.Range16{Lo: 0x217, Hi: 0x217, Stride: 0x1},
		unicode.Range16{Lo: 0x219, Hi: 0x219, Stride: 0x1},
		unicode.Range16{Lo: 0x21b, Hi: 0x21b, Stride: 0x1},
		unicode.Range16{Lo: 0x21d, Hi: 0x21d, Stride: 0x1},
		unicode.Range16{Lo: 0x21f, Hi: 0x21f, Stride: 0x1},
		unicode.Range16{Lo: 0x221, Hi: 0x221, Stride: 0x1},
		unicode.Range16{Lo: 0x223, Hi: 0x223, Stride: 0x1},
		unicode.Range16{Lo: 0x225, Hi: 0x225, Stride: 0x1},
		unicode.Range16{Lo: 0x227, Hi: 0x227, Stride: 0x1},
		unicode.Range16{Lo: 0x229, Hi: 0x229, Stride: 0x1},
		unicode.Range16{Lo: 0x22b, Hi: 0x22b, Stride: 0x1},
		unicode.Range16{Lo: 0x22d, Hi: 0x22d, Stride: 0x1},
		unicode.Range16{Lo: 0x22f, Hi: 0x22f, Stride: 0x1},
		unicode.Range16{Lo: 0x231, Hi: 0x231, Stride: 0x1},
		unicode.Range16{Lo: 0x233, Hi: 0x239, Stride: 0x1},
		unicode.Range16{Lo: 0x23c, Hi: 0x23c, Stride: 0x1},
		unicode.Range16{Lo: 0x23f, Hi: 0x240, Stride: 0x1},
		unicode.Range16{Lo: 0x242, Hi: 0x242, Stride: 0x1},
		unicode.Range16{Lo: 0x247, Hi: 0x247, Stride: 0x1},
		unicode.Range16{Lo: 0x249, Hi: 0x249, Stride: 0x1},
		unicode.Range16{Lo: 0x24b, Hi: 0x24b, Stride: 0x1},
		unicode.Range16{Lo: 0x24d, Hi: 0x24d, Stride: 0x1},
		unicode.Range16{Lo: 0x24f, Hi: 0x293, Stride: 0x1},
		unicode.Range16{Lo: 0x295, Hi: 0x2af, Stride: 0x1},
		unicode.Range16{Lo: 0x2b0, Hi: 0x2b8, Stride: 0x1},
		unicode.Range16{Lo: 0x2c0, Hi: 0x2c1, Stride: 0x1},
		unicode.Range16{Lo: 0x2e0, Hi: 0x2e4, Stride: 0x1},
		unicode.Range16{Lo: 0x371, Hi: 0x371, Stride: 0x1},
		unicode.Range16{Lo: 0x373, Hi: 0x373, Stride: 0x1},
		unicode.Range16{Lo: 0x377, Hi: 0x377, Stride: 0x1},
		unicode.Range16{Lo: 0x37a, Hi: 0x37a, Stride: 0x1},
		unicode.Range16{Lo: 0x37b, Hi: 0x37d, Stride: 0x1},
		unicode.Range16{Lo: 0x390, Hi: 0x390, Stride: 0x1},
		unicode.Range16{Lo: 0x3ac, Hi: 0x3ce, Stride: 0x1},
		unicode.Range16{Lo: 0x3d0, Hi: 0x3d1, Stride: 0x1},
		unicode.Range16{Lo: 0x3d5, Hi: 0x3d7, Stride: 0x1},
		unicode.Range16{Lo: 0x3d9, Hi: 0x3d9, Stride: 0x1},
		unicode.Range16{Lo: 0x3db, Hi: 0x3db, Stride: 0x1},
		unicode.Range16{Lo: 0x3dd, Hi: 0x3dd, Stride: 0x1},
		unicode.Range16{Lo: 0x3df, Hi: 0x3df, Stride: 0x1},
		unicode.Range16{Lo: 0x3e1, Hi: 0x3e1, Stride: 0x1},
		unicode.Range16{Lo: 0x3e3, Hi: 0x3e3, Stride: 0x1},
		unicode.Range16{Lo: 0x3e5, Hi: 0x3e5, Stride: 0x1},
		unicode.Range16{Lo: 0x3e7, Hi: 0x3e7, Stride: 0x1},
		unicode.Range16{Lo: 0x3e9, Hi: 0x3e9, Stride: 0x1},
		unicode.Range16{Lo: 0x3eb, Hi: 0x3eb, Stride: 0x1},
		unicode.Range16{Lo: 0x3ed, Hi: 0x3ed, Stride: 0x1},
		unicode.Range16{Lo: 0x3ef, Hi: 0x3f3, Stride: 0x1},
		unicode.Range16{Lo: 0x3f5, Hi: 0x3f5, Stride: 0x1},
		unicode.Range16{Lo: 0x3f8, Hi: 0x3f8, Stride: 0x1},
		unicode.Range16{Lo: 0x3fb, Hi: 0x3fc, Stride: 0x1},
		unicode.Range16{Lo: 0x430, Hi: 0x45f, Stride: 0x1},
		unicode.Range16{Lo: 0x461, Hi: 0x461, Stride: 0x1},
		unicode.Range16{Lo: 0x463, Hi: 0x463, Stride: 0x1},
		unicode.Range16{Lo: 0x465, Hi: 0x465, Stride: 0x1},
		unicode.Range16{Lo: 0x467, Hi: 0x467, Stride: 0x1},
		unicode.Range16{Lo: 0x469, Hi: 0x469, Stride: 0x1},
		unicode.Range16{Lo: 0x46b, Hi: 0x46b, Stride: 0x1},
		unicode.Range16{Lo: 0x46d, Hi: 0x46d, Stride: 0x1},
		unicode.Range16{Lo: 0x46f, Hi: 0x46f, Stride: 0x1},
		unicode.Range16{Lo: 0x471, Hi: 0x471, Stride: 0x1},
		unicode.Range16{Lo: 0x473, Hi: 0x473, Stride: 0x1},
		unicode.Range16{Lo: 0x475, Hi: 0x475, Stride: 0x1},
		unicode.Range16{Lo: 0x477, Hi: 0x477, Stride: 0x1},
		unicode.Range16{Lo: 0x479, Hi: 0x479, Stride: 0x1},
		unicode.Range16{Lo: 0x47b, Hi: 0x47b, Stride: 0x1},
		unicode.Range16{Lo: 0x47d, Hi: 0x47d, Stride: 0x1},
		unicode.Range16{Lo: 0x47f, Hi: 0x47f, Stride: 0x1},
		unicode.Range16{Lo: 0x481, Hi: 0x481, Stride: 0x1},
		unicode.Range16{Lo: 0x48b, Hi: 0x48b, Stride: 0x1},
		unicode.Range16{Lo: 0x48d, Hi: 0x48d, Stride: 0x1},
		unicode.Range16{Lo: 0x48f, Hi: 0x48f, Stride: 0x1},
		unicode.Range16{Lo: 0x491, Hi: 0x491, Stride: 0x1},
		unicode.Range16{Lo: 0x493, Hi: 0x493, Stride: 0x1},
		unicode.Range16{Lo: 0x495, Hi: 0x495, Stride: 0x1},
		unicode.Range16{Lo: 0x497, Hi: 0x497, Stride: 0x1},
		unicode.Range16{Lo: 0x499, Hi: 0x499, Stride: 0x1},
		unicode.Range16{Lo: 0x49b, Hi: 0x49b, Stride: 0x1},
		unicode.Range16{Lo: 0x49d, Hi: 0x49d, Stride: 0x1},
		unicode.Range16{Lo: 0x49f, Hi: 0x49f, Stride: 0x1},
		unicode.Range16{Lo: 0x4a1, Hi: 0x4a1, Stride: 0x1},
		unicode.Range16{Lo: 0x4a3, Hi: 0x4a3, Stride: 0x1},
		unicode.Range16{Lo: 0x4a5, Hi: 0x4a5, Stride: 0x1},
		unicode.Range16{Lo: 0x4a7, Hi: 0x4a7, Stride: 0x1},
		unicode.Range16{Lo: 0x4a9, Hi: 0x4a9, Stride: 0x1},
		unicode.Range16{Lo: 0x4ab, Hi: 0x4ab, Stride: 0x1},
		unicode.Range16{Lo: 0x4ad, Hi: 0x4ad, Stride: 0x1},
		unicode.Range16{Lo: 0x4af, Hi: 0x4af, Stride: 0x1},
		unicode.Range16{Lo: 0x4b1, Hi: 0x4b1, Stride: 0x1},
		unicode.Range16{Lo: 0x4b3, Hi: 0x4b3, Stride: 0x1},
		unicode.Range16{Lo: 0x4b5, Hi: 0x4b5, Stride: 0x1},
		unicode.Range16{Lo: 0x4b7, Hi: 0x4b7, Stride: 0x1},
		unicode.Range16{Lo: 0x4b9, Hi: 0x4b9, Stride: 0x1},
		unicode.Range16{Lo: 0x4bb, Hi: 0x4bb, Stride: 0x1},
		unicode.Range16{Lo: 0x4bd, Hi: 0x4bd, Stride: 0x1},
		unicode.Range16{Lo: 0x4bf, Hi: 0x4bf, Stride: 0x1},
		unicode.Range16{Lo: 0x4c2, Hi: 0x4c2, Stride: 0x1},
		unicode.Range16{Lo: 0x4c4, Hi: 0x4c4, Stride: 0x1},
		unicode.Range16{Lo: 0x4c6, Hi: 0x4c6, Stride: 0x1},
		unicode.Range16{Lo: 0x4c8, Hi: 0x4c8, Stride: 0x1},
		unicode.Range16{Lo: 0x4ca, Hi: 0x4ca, Stride: 0x1},
		unicode.Range16{Lo: 0x4cc, Hi: 0x4cc, Stride: 0x1},
		unicode.Range16{Lo: 0x4ce, Hi: 0x4cf, Stride: 0x1},
		unicode.Range16{Lo: 0x4d1, Hi: 0x4d1, Stride: 0x1},
		unicode.Range16{Lo: 0x4d3, Hi: 0x4d3, Stride: 0x1},
		unicode.Range16{Lo: 0x4d5, Hi: 0x4d5, Stride: 0x1},
		unicode.Range16{Lo: 0x4d7, Hi: 0x4d7, Stride: 0x1},
		unicode.Range16{Lo: 0x4d9, Hi: 0x4d9, Stride: 0x1},
		unicode.Range16{Lo: 0x4db, Hi: 0x4db, Stride: 0x1},
		unicode.Range16{Lo: 0x4dd, Hi: 0x4dd, Stride: 0x1},
		unicode.Range16{Lo: 0x4df, Hi: 0x4df, Stride: 0x1},
		unicode.Range16{Lo: 0x4e1, Hi: 0x4e1, Stride: 0x1},
		unicode.Range16{Lo: 0x4e3, Hi: 0x4e3, Stride: 0x1},
		unicode.Range16{Lo: 0x4e5, Hi: 0x4e5, Stride: 0x1},
		unicode.Range16{Lo: 0x4e7, Hi: 0x4e7, Stride: 0x1},
		unicode.Range16{Lo: 0x4e9, Hi: 0x4e9, Stride: 0x1},
		unicode.Range16{Lo: 0x4eb, Hi: 0x4eb, Stride: 0x1},
		unicode.Range16{Lo: 0x4ed, Hi: 0x4ed, Stride: 0x1},
		unicode.Range16{Lo: 0x4ef, Hi: 0x4ef, Stride: 0x1},
		unicode.Range16{Lo: 0x4f1, Hi: 0x4f1, Stride: 0x1},
		unicode.Range16{Lo: 0x4f3, Hi: 0x4f3, Stride: 0x1},
		unicode.Range16{Lo: 0x4f5, Hi: 0x4f5, Stride: 0x1},
		unicode.Range16{Lo: 0x4f7, Hi: 0x4f7, Stride: 0x1},
		unicode.Range16{Lo: 0x4f9, Hi: 0x4f9, Stride: 0x1},
		unicode.Range16{Lo: 0x4fb, Hi: 0x4fb, Stride: 0x1},
		unicode.Range16{Lo: 0x4fd, Hi: 0x4fd, Stride: 0x1},
		unicode.Range16{Lo: 0x4ff, Hi: 0x4ff, Stride: 0x1},
		unicode.Range16{Lo: 0x501, Hi: 0x501, Stride: 0x1},
		unicode.Range16{Lo: 0x503, Hi: 0x503, Stride: 0x1},
		unicode.Range16{Lo: 0x505, Hi: 0x505, Stride: 0x1},
		unicode.Range16{Lo: 0x507, Hi: 0x507, Stride: 0x1},
		unicode.Range16{Lo: 0x509, Hi: 0x509, Stride: 0x1},
		unicode.Range16{Lo: 0x50b, Hi: 0x50b, Stride: 0x1},
		unicode.Range16{Lo: 0x50d, Hi: 0x50d, Stride: 0x1},
		unicode.Range16{Lo: 0x50f, Hi: 0x50f, Stride: 0x1},
		unicode.Range16{Lo: 0x511, Hi: 0x511, Stride: 0x1},
		unicode.Range16{Lo: 0x513, Hi: 0x513, Stride: 0x1},
		unicode.Range16{Lo: 0x515, Hi: 0x515, Stride: 0x1},
		unicode.Range16{Lo: 0x517, Hi: 0x517, Stride: 0x1},
		unicode.Range16{Lo: 0x519, Hi: 0x519, Stride: 0x1},
		unicode.Range16{Lo: 0x51b, Hi: 0x51b, Stride: 0x1},
		unicode.Range16{Lo: 0x51d, Hi: 0x51d, Stride: 0x1},
		unicode.Range16{Lo: 0x51f, Hi: 0x51f, Stride: 0x1},
		unicode.Range16{Lo: 0x521, Hi: 0x521, Stride: 0x1},
		unicode.Range16{Lo: 0x523, Hi: 0x523, Stride: 0x1},
		unicode.Range16{Lo: 0x525, Hi: 0x525, Stride: 0x1},
		unicode.Range16{Lo: 0x527, Hi: 0x527, Stride: 0x1},
		unicode.Range16{Lo: 0x529, Hi: 0x529, Stride: 0x1},
		unicode.Range16{Lo: 0x52b, Hi: 0x52b, Stride: 0x1},
		unicode.Range16{Lo: 0x52d, Hi: 0x52d, Stride: 0x1},
		unicode.Range16{Lo: 0x52f, Hi: 0x52f, Stride: 0x1},
		unicode.Range16{Lo: 0x560, Hi: 0x588, Stride: 0x1},
		unicode.Range16{Lo: 0x10fc, Hi: 0x10fc, Stride: 0x1},
		unicode.Range16{Lo: 0x13f8, Hi: 0x13fd, Stride: 0x1},
		unicode.Range16{Lo: 0x1c80, Hi: 0x1c88, Stride: 0x1},
		unicode.Range16{Lo: 0x1d00, Hi: 0x1d2b, Stride: 0x1},
		unicode.Range16{Lo: 0x1d2c, Hi: 0x1d6a, Stride: 0x1},
		unicode.Range16{Lo: 0x1d6b, Hi: 0x1d77, Stride: 0x1},
		unicode.Range16{Lo: 0x1d78, Hi: 0x1d78, Stride: 0x1},
		unicode.Range16{Lo: 0x1d79, Hi: 0x1d9a, Stride: 0x1},
		unicode.Range16{Lo: 0x1d9b, Hi: 0x1dbf, Stride: 0x1},
		unicode.Range16{Lo: 0x1e01, Hi: 0x1e01, Stride: 0x1},
		unicode.Range16{Lo: 0x1e03, Hi: 0x1e03, Stride: 0x1},
		unicode.Range16{Lo: 0x1e05, Hi: 0x1e05, Stride: 0x1},
		unicode.Range16{Lo: 0x1e07, Hi: 0x1e07, Stride: 0x1},
		unicode.Range16{Lo: 0x1e09, Hi: 0x1e09, Stride: 0x1},
		unicode.Range16{Lo: 0x1e0b, Hi: 0x1e0b, Stride: 0x1},
		unicode.Range16{Lo: 0x1e0d, Hi: 0x1e0d, Stride: 0x1},
		unicode.Range16{Lo: 0x1e0f, Hi: 0x1e0f, Stride: 0x1},
		unicode.Range16{Lo: 0x1e11, Hi: 0x1e11, Stride: 0x1},
		unicode.Range16{Lo: 0x1e13, Hi: 0x1e13, Stride: 0x1},
		unicode.Range16{Lo: 0x1e15, Hi: 0x1e15, Stride: 0x1},
		unicode.Range16{Lo: 0x1e17, Hi: 0x1e17, Stride: 0x1},
		unicode.Range16{Lo: 0x1e19, Hi: 0x1e19, Stride: 0x1},
		unicode.Range16{Lo: 0x1e1b, Hi: 0x1e1b, Stride: 0x1},
		unicode.Range16{Lo: 0x1e1d, Hi: 0x1e1d, Stride: 0x1},
		unicode.Range16{Lo: 0x1e1f, Hi: 0x1e1f, Stride: 0x1},
		unicode.Range16{Lo: 0x1e21, Hi: 0x1e21, Stride: 0x1},
		unicode.Range16{Lo: 0x1e23, Hi: 0x1e23, Stride: 0x1},
		unicode.Range16{Lo: 0x1e25, Hi: 0x1e25, Stride: 0x1},
		unicode.Range16{Lo: 0x1e27, Hi: 0x1e27, Stride: 0x1},
		unicode.Range16{Lo: 0x1e29, Hi: 0x1e29, Stride: 0x1},
		unicode.Range16{Lo: 0x1e2b, Hi: 0x1e2b, Stride: 0x1},
		unicode.Range16{Lo: 0x1e2d, Hi: 0x1e2d, Stride: 0x1},
		unicode.Range16{Lo: 0x1e2f, Hi: 0x1e2f, Stride: 0x1},
		unicode.Range16{Lo: 0x1e31, Hi: 0x1e31, Stride: 0x1},
		unicode.Range16{Lo: 0x1e33, Hi: 0x1e33, Stride: 0x1},
		unicode.Range16{Lo: 0x1e35, Hi: 0x1e35, Stride: 0x1},
		unicode.Range16{Lo: 0x1e37, Hi: 0x1e37, Stride: 0x1},
		unicode.Range16{Lo: 0x1e39, Hi: 0x1e39, Stride: 0x1},
		unicode.Range16{Lo: 0x1e3b, Hi: 0x1e3b, Stride: 0x1},
		unicode.Range16{Lo: 0x1e3d, Hi: 0x1e3d, Stride: 0x1},
		unicode.Range16{Lo: 0x1e3f, Hi: 0x1e3f, Stride: 0x1},
		unicode.Range16{Lo: 0x1e41, Hi: 0x1e41, Stride: 0x1},
		unicode.Range16{Lo: 0x1e43, Hi: 0x1e43, Stride: 0x1},
		unicode.Range16{Lo: 0x1e45, Hi: 0x1e45, Stride: 0x1},
		unicode.Range16{Lo: 0x1e47, Hi: 0x1e47, Stride: 0x1},
		unicode.Range16{Lo: 0x1e49, Hi: 0x1e49, Stride: 0x1},
		unicode.Range16{Lo: 0x1e4b, Hi: 0x1e4b, Stride: 0x1},
		unicode.Range16{Lo: 0x1e4d, Hi: 0x1e4d, Stride: 0x1},
		unicode.Range16{Lo: 0x1e4f, Hi: 0x1e4f, Stride: 0x1},
		unicode.Range16{Lo: 0x1e51, Hi: 0x1e51, Stride: 0x1},
		unicode.Range16{Lo: 0x1e53, Hi: 0x1e53, Stride: 0x1},
		unicode.Range16{Lo: 0x1e55, Hi: 0x1e55, Stride: 0x1},
		unicode.Range16{Lo: 0x1e57, Hi: 0x1e57, Stride: 0x1},
		unicode.Range16{Lo: 0x1e59, Hi: 0x1e59, Stride: 0x1},
		unicode.Range16{Lo: 0x1e5b, Hi: 0x1e5b, Stride: 0x1},
		unicode.Range16{Lo: 0x1e5d, Hi: 0x1e5d, Stride: 0x1},
		unicode.Range16{Lo: 0x1e5f, Hi: 0x1e5f, Stride: 0x1},
		unicode.Range16{Lo: 0x1e61, Hi: 0x1e61, Stride: 0x1},
		unicode.Range16{Lo: 0x1e63, Hi: 0x1e63, Stride: 0x1},
		unicode.Range16{Lo: 0x1e65, Hi: 0x1e65, Stride: 0x1},
		unicode.Range16{Lo: 0x1e67, Hi: 0x1e67, Stride: 0x1},
		unicode.Range16{Lo: 0x1e69, Hi: 0x1e69, Stride: 0x1},
		unicode.Range16{Lo: 0x1e6b, Hi: 0x1e6b, Stride: 0x1},
		unicode.Range16{Lo: 0x1e6d, Hi: 0x1e6d, Stride: 0x1},
		unicode.Range16{Lo: 0x1e6f, Hi: 0x1e6f, Stride: 0x1},
		unicode.Range16{Lo: 0x1e71, Hi: 0x1e71, Stride: 0x1},
		unicode.Range16{Lo: 0x1e73, Hi: 0x1e73, Stride: 0x1},
		unicode.Range16{Lo: 0x1e75, Hi: 0x1e75, Stride: 0x1},
		unicode.Range16{Lo: 0x1e77, Hi: 0x1e77, Stride: 0x1},
		unicode.Range16{Lo: 0x1e79, Hi: 0x1e79, Stride: 0x1},
		unicode.Range16{Lo: 0x1e7b, Hi: 0x1e7b, Stride: 0x1},
		unicode.Range16{Lo: 0x1e7d, Hi: 0x1e7d, Stride: 0x1},
		unicode.Range16{Lo: 0x1e7f, Hi: 0x1e7f, Stride: 0x1},
		unicode.Range16{Lo: 0x1e81, Hi: 0x1e81, Stride: 0x1},
		unicode.Range16{Lo: 0x1e83, Hi: 0x1e83, Stride: 0x1},
		unicode.Range16{Lo: 0x1e85, Hi: 0x1e85, Stride: 0x1},
		unicode.Range16{Lo: 0x1e87, Hi: 0x1e87, Stride: 0x1},
		unicode.Range16{Lo: 0x1e89, Hi: 0x1e89, Stride: 0x1},
		unicode.Range16{Lo: 0x1e8b, Hi: 0x1e8b, Stride: 0x1},
		unicode.Range16{Lo: 0x1e8d, Hi: 0x1e8d, Stride: 0x1},
		unicode.Range16{Lo: 0x1e8f, Hi: 0x1e8f, Stride: 0x1},
		unicode.Range16{Lo: 0x1e91, Hi: 0x1e91, Stride: 0x1},
		unicode.Range16{Lo: 0x1e93, Hi: 0x1e93, Stride: 0x1},
		unicode.Range16{Lo: 0x1e95, Hi: 0x1e9d, Stride: 0x1},
		unicode.Range16{Lo: 0x1e9f, Hi: 0x1e9f, Stride: 0x1},
		unicode.Range16{Lo: 0x1ea1, Hi: 0x1ea1, Stride: 0x1},
		unicode.Range16{Lo: 0x1ea3, Hi: 0x1ea3, Stride: 0x1},
		unicode.Range16{Lo: 0x1ea5, Hi: 0x1ea5, Stride: 0x1},
		unicode.Range16{Lo: 0x1ea7, Hi: 0x1ea7, Stride: 0x1},
		unicode.Range16{Lo: 0x1ea9, Hi: 0x1ea9, Stride: 0x1},
		unicode.Range16{Lo: 0x1eab, Hi: 0x1eab, Stride: 0x1},
		unicode.Range16{Lo: 0x1ead, Hi: 0x1ead, Stride: 0x1},
		unicode.Range16{Lo: 0x1eaf, Hi: 0x1eaf, Stride: 0x1},
		unicode.Range16{Lo: 0x1eb1, Hi: 0x1eb1, Stride: 0x1},
		unicode.Range16{Lo: 0x1eb3, Hi: 0x1eb3, Stride: 0x1},
		unicode.Range16{Lo: 0x1eb5, Hi: 0x1eb5, Stride: 0x1},
		unicode.Range16{Lo: 0x1eb7, Hi: 0x1eb7, Stride: 0x1},
		unicode.Range16{Lo: 0x1eb9, Hi: 0x1eb9, Stride: 0x1},
		unicode.Range16{Lo: 0x1ebb, Hi: 0x1ebb, Stride: 0x1},
		unicode.Range16{Lo: 0x1ebd, Hi: 0x1ebd, Stride: 0x1},
		unicode.Range16{Lo: 0x1ebf, Hi: 0x1ebf, Stride: 0x1},
		unicode.Range16{Lo: 0x1ec1, Hi: 0x1ec1, Stride: 0x1},
		unicode.Range16{Lo: 0x1ec3, Hi: 0x1ec3, Stride: 0x1},
		unicode.Range16{Lo: 0x1ec5, Hi: 0x1ec5, Stride: 0x1},
		unicode.Range16{Lo: 0x1ec7, Hi: 0x1ec7, Stride: 0x1},
		unicode.Range16{Lo: 0x1ec9, Hi: 0x1ec9, Stride: 0x1},
		unicode.Range16{Lo: 0x1ecb, Hi: 0x1ecb, Stride: 0x1},
		unicode.Range16{Lo: 0x1ecd, Hi: 0x1ecd, Stride: 0x1},
		unicode.Range16{Lo: 0x1ecf, Hi: 0x1ecf, Stride: 0x1},
		unicode.Range16{Lo: 0x1ed1, Hi: 0x1ed1, Stride: 0x1},
		unicode.Range16{Lo: 0x1ed3, Hi: 0x1ed3, Stride: 0x1},
		unicode.Range16{Lo: 0x1ed5, Hi: 0x1ed5, Stride: 0x1},
		unicode.Range16{Lo: 0x1ed7, Hi: 0x1ed7, Stride: 0x1},
		unicode.Range16{Lo: 0x1ed9, Hi: 0x1ed9, Stride: 0x1},
		unicode.Range16{Lo: 0x1edb, Hi: 0x1edb, Stride: 0x1},
		unicode.Range16{Lo: 0x1edd, Hi: 0x1edd, Stride: 0x1},
		unicode.Range16{Lo: 0x1edf, Hi: 0x1edf, Stride: 0x1},
		unicode.Range16{Lo: 0x1ee1, Hi: 0x1ee1, Stride: 0x1},
		unicode.Range16{Lo: 0x1ee3, Hi: 0x1ee3, Stride: 0x1},
		unicode.Range16{Lo: 0x1ee5, Hi: 0x1ee5, Stride: 0x1},
		unicode.Range16{Lo: 0x1ee7, Hi: 0x1ee7, Stride: 0x1},
		unicode.Range16{Lo: 0x1ee9, Hi: 0x1ee9, Stride: 0x1},
		unicode.Range16{Lo: 0x1eeb, Hi: 0x1eeb, Stride: 0x1},
		unicode.Range16{Lo: 0x1eed, Hi: 0x1eed, Stride: 0x1},
		unicode.Range16{Lo: 0x1eef, Hi: 0x1eef, Stride: 0x1},
		unicode.Range16{Lo: 0x1ef1, Hi: 0x1ef1, Stride: 0x1},
		unicode.Range16{Lo: 0x1ef3, Hi: 0x1ef3, Stride: 0x1},
		unicode.Range16{Lo: 0x1ef5, Hi: 0x1ef5, Stride: 0x1},
		unicode.Range16{Lo: 0x1ef7, Hi: 0x1ef7, Stride: 0x1},
		unicode.Range16{Lo: 0x1ef9, Hi: 0x1ef9, Stride: 0x1},
		unicode.Range16{Lo: 0x1efb, Hi: 0x1efb, Stride: 0x1},
		unicode.Range16{Lo: 0x1efd, Hi: 0x1efd, Stride: 0x1},
		unicode.Range16{Lo: 0x1eff, Hi: 0x1f07, Stride: 0x1},
		unicode.Range16{Lo: 0x1f10, Hi: 0x1f15, Stride: 0x1},
		unicode.Range16{Lo: 0x1f20, Hi: 0x1f27, Stride: 0x1},
		unicode.Range16{Lo: 0x1f30, Hi: 0x1f37, Stride: 0x1},
		unicode.Range16{Lo: 0x1f40, Hi: 0x1f45, Stride: 0x1},
		unicode.Range16{Lo: 0x1f50, Hi: 0x1f57, Stride: 0x1},
		unicode.Range16{Lo: 0x1f60, Hi: 0x1f67, Stride: 0x1},
		unicode.Range16{Lo: 0x1f70, Hi: 0x1f7d, Stride: 0x1},
		unicode.Range16{Lo: 0x1f80, Hi: 0x1f87, Stride: 0x1},
		unicode.Range16{Lo: 0x1f90, Hi: 0x1f97, Stride: 0x1},
		unicode.Range16{Lo: 0x1fa0, Hi: 0x1fa7, Stride: 0x1},
		unicode.Range16{Lo: 0x1fb0, Hi: 0x1fb4, Stride: 0x1},
		unicode.Range16{Lo: 0x1fb6, Hi: 0x1fb7, Stride: 0x1},
		unicode.Range16{Lo: 0x1fbe, Hi: 0x1fbe, Stride: 0x1},
		unicode.Range16{Lo: 0x1fc2, Hi: 0x1fc4, Stride: 0x1},
		unicode.Range16{Lo: 0x1fc6, Hi: 0x1fc7, Stride: 0x1},
		unicode.Range16{Lo: 0x1fd0, Hi: 0x1fd3, Stride: 0x1},
		unicode.Range16{Lo: 0x1fd6, Hi: 0x1fd7, Stride: 0x1},
		unicode.Range16{Lo: 0x1fe0, Hi: 0x1fe7, Stride: 0x1},
		unicode.Range16{Lo: 0x1ff2, Hi: 0x1ff4, Stride: 0x1},
		unicode.Range16{Lo: 0x1ff6, Hi: 0x1ff7, Stride: 0x1},
		unicode.Range16{Lo: 0x2071, Hi: 0x2071, Stride: 0x1},
		unicode.Range16{Lo: 0x207f, Hi: 0x207f, Stride: 0x1},
		unicode.Range16{Lo: 0x2090, Hi: 0x209c, Stride: 0x1},
		unicode.Range16{Lo: 0x210a, Hi: 0x210a, Stride: 0x1},
		unicode.Range16{Lo: 0x210e, Hi: 0x210f, Stride: 0x1},
		unicode.Range16{Lo: 0x2113, Hi: 0x2113, Stride: 0x1},
		unicode.Range16{Lo: 0x212f, Hi: 0x212f, Stride: 0x1},
		unicode.Range16{Lo: 0x2134, Hi: 0x2134, Stride: 0x1},
		unicode.Range16{Lo: 0x2139, Hi: 0x2139, Stride: 0x1},
		unicode.Range16{Lo: 0x213c, Hi: 0x213d, Stride: 0x1},
		unicode.Range16{Lo: 0x2146, Hi: 0x2149, Stride: 0x1},
		unicode.Range16{Lo: 0x214e, Hi: 0x214e, Stride: 0x1},
		unicode.Range16{Lo: 0x2170, Hi: 0x217f, Stride: 0x1},
		unicode.Range16{Lo: 0x2184, Hi: 0x2184, Stride: 0x1},
		unicode.Range16{Lo: 0x24d0, Hi: 0x24e9, Stride: 0x1},
		unicode.Range16{Lo: 0x2c30, Hi: 0x2c5f, Stride: 0x1},
		unicode.Range16{Lo: 0x2c61, Hi: 0x2c61, Stride: 0x1},
		unicode.Range16{Lo: 0x2c65, Hi: 0x2c66, Stride: 0x1},
		unicode.Range16{Lo: 0x2c68, Hi: 0x2c68, Stride: 0x1},
		unicode.Range16{Lo: 0x2c6a, Hi: 0x2c6a, Stride: 0x1},
		unicode.Range16{Lo: 0x2c6c, Hi: 0x2c6c, Stride: 0x1},
		unicode.Range16{Lo: 0x2c71, Hi: 0x2c71, Stride: 0x1},
		unicode.Range16{Lo: 0x2c73, Hi: 0x2c74, Stride: 0x1},
		unicode.Range16{Lo: 0x2c76, Hi: 0x2c7b, Stride: 0x1},
		unicode.Range16{Lo: 0x2c7c, Hi: 0x2c7d, Stride: 0x1},
		unicode.Range16{Lo: 0x2c81, Hi: 0x2c81, Stride: 0x1},
		unicode.Range16{Lo: 0x2c83, Hi: 0x2c83, Stride: 0x1},
		unicode.Range16{Lo: 0x2c85, Hi: 0x2c85, Stride: 0x1},
		unicode.Range16{Lo: 0x2c87, Hi: 0x2c87, Stride: 0x1},
		unicode.Range16{Lo: 0x2c89, Hi: 0x2c89, Stride: 0x1},
		unicode.Range16{Lo: 0x2c8b, Hi: 0x2c8b, Stride: 0x1},
		unicode.Range16{Lo: 0x2c8d, Hi: 0x2c8d, Stride: 0x1},
		unicode.Range16{Lo: 0x2c8f, Hi: 0x2c8f, Stride: 0x1},
		unicode.Range16{Lo: 0x2c91, Hi: 0x2c91, Stride: 0x1},
		unicode.Range16{Lo: 0x2c93, Hi: 0x2c93, Stride: 0x1},
		unicode.Range16{Lo: 0x2c95, Hi: 0x2c95, Stride: 0x1},
		unicode.Range16{Lo: 0x2c97, Hi: 0x2c97, Stride: 0x1},
		unicode.Range16{Lo: 0x2c99, Hi: 0x2c99, Stride: 0x1},
		unicode.Range16{Lo: 0x2c9b, Hi: 0x2c9b, Stride: 0x1},
		unicode.Range16{Lo: 0x2c9d, Hi: 0x2c9d, Stride: 0x1},
		unicode.Range16{Lo: 0x2c9f, Hi: 0x2c9f, Stride: 0x1},
		unicode.Range16{Lo: 0x2ca1, Hi: 0x2ca1, Stride: 0x1},
		unicode.Range16{Lo: 0x2ca3, Hi: 0x2ca3, Stride: 0x1},
		unicode.Range16{Lo: 0x2ca5, Hi: 0x2ca5, Stride: 0x1},
		unicode.Range16{Lo: 0x2ca7, Hi: 0x2ca7, Stride: 0x1},
		unicode.Range16{Lo: 0x2ca9, Hi: 0x2ca9, Stride: 0x1},
		unicode.Range16{Lo: 0x2cab, Hi: 0x2cab, Stride: 0x1},
		unicode.Range16{Lo: 0x2cad, Hi: 0x2cad, Stride: 0x1},
		unicode.Range16{Lo: 0x2caf, Hi: 0x2caf, Stride: 0x1},
		unicode.Range16{Lo: 0x2cb1, Hi: 0x2cb1, Stride: 0x1},
		unicode.Range16{Lo: 0x2cb3, Hi: 0x2cb3, Stride: 0x1},
		unicode.Range16{Lo: 0x2cb5, Hi: 0x2cb5, Stride: 0x1},
		unicode.Range16{Lo: 0x2cb7, Hi: 0x2cb7, Stride: 0x1},
		unicode.Range16{Lo: 0x2cb9, Hi: 0x2cb9, Stride: 0x1},
		unicode.Range16{Lo: 0x2cbb, Hi: 0x2cbb, Stride: 0x1},
		unicode.Range16{Lo: 0x2cbd, Hi: 0x2cbd, Stride: 0x1},
		unicode.Range16{Lo: 0x2cbf, Hi: 0x2cbf, Stride: 0x1},
		unicode.Range16{Lo: 0x2cc1, Hi: 0x2cc1, Stride: 0x1},
		unicode.Range16{Lo: 0x2cc3, Hi: 0x2cc3, Stride: 0x1},
		unicode.Range16{Lo: 0x2cc5, Hi: 0x2cc5, Stride: 0x1},
		unicode.Range16{Lo: 0x2cc7, Hi: 0x2cc7, Stride: 0x1},
		unicode.Range16{Lo: 0x2cc9, Hi: 0x2cc9, Stride: 0x1},
		unicode.Range16{Lo: 0x2ccb, Hi: 0x2ccb, Stride: 0x1},
		unicode.Range16{Lo: 0x2ccd, Hi: 0x2ccd, Stride: 0x1},
		unicode.Range16{Lo: 0x2ccf, Hi: 0x2ccf, Stride: 0x1},
		unicode.Range16{Lo: 0x2cd1, Hi: 0x2cd1, Stride: 0x1},
		unicode.Range16{Lo: 0x2cd3, Hi: 0x2cd3, Stride: 0x1},
		unicode.Range16{Lo: 0x2cd5, Hi: 0x2cd5, Stride: 0x1},
		unicode.Range16{Lo: 0x2cd7, Hi: 0x2cd7, Stride: 0x1},
		unicode.Range16{Lo: 0x2cd9, Hi: 0x2cd9, Stride: 0x1},
		unicode.Range16{Lo: 0x2cdb, Hi: 0x2cdb, Stride: 0x1},
		unicode.Range16{Lo: 0x2cdd, Hi: 0x2cdd, Stride: 0x1},
		unicode.Range16{Lo: 0x2cdf, Hi: 0x2cdf, Stride: 0x1},
		unicode.Range16{Lo: 0x2ce1, Hi: 0x2ce1, Stride: 0x1},
		unicode.Range16{Lo: 0x2ce3, Hi: 0x2ce4, Stride: 0x1},
		unicode.Range16{Lo: 0x2cec, Hi: 0x2cec, Stride: 0x1},
		unicode.Range16{Lo: 0x2cee, Hi: 0x2cee, Stride: 0x1},
		unicode.Range16{Lo: 0x2cf3, Hi: 0x2cf3, Stride: 0x1},
		unicode.Range16{Lo: 0x2d00, Hi: 0x2d25, Stride: 0x1},
		unicode.Range16{Lo: 0x2d27, Hi: 0x2d27, Stride: 0x1},
		unicode.Range16{Lo: 0x2d2d, Hi: 0x2d2d, Stride: 0x1},
		unicode.Range16{Lo: 0xa641, Hi: 0xa641, Stride: 0x1},
		unicode.Range16{Lo: 0xa643, Hi: 0xa643, Stride: 0x1},
		unicode.Range16{Lo: 0xa645, Hi: 0xa645, Stride: 0x1},
		unicode.Range16{Lo: 0xa647, Hi: 0xa647, Stride: 0x1},
		unicode.Range16{Lo: 0xa649, Hi: 0xa649, Stride: 0x1},
		unicode.Range16{Lo: 0xa64b, Hi: 0xa64b, Stride: 0x1},
		unicode.Range16{Lo: 0xa64d, Hi: 0xa64d, Stride: 0x1},
		unicode.Range16{Lo: 0xa64f, Hi: 0xa64f, Stride: 0x1},
		unicode.Range16{Lo: 0xa651, Hi: 0xa651, Stride: 0x1},
		unicode.Range16{Lo: 0xa653, Hi: 0xa653, Stride: 0x1},
		unicode.Range16{Lo: 0xa655, Hi: 0xa655, Stride: 0x1},
		unicode.Range16{Lo: 0xa657, Hi: 0xa657, Stride: 0x1},
		unicode.Range16{Lo: 0xa659, Hi: 0xa659, Stride: 0x1},
		unicode.Range16{Lo: 0xa65b, Hi: 0xa65b, Stride: 0x1},
		unicode.Range16{Lo: 0xa65d, Hi: 0xa65d, Stride: 0x1},
		unicode.Range16{Lo: 0xa65f, Hi: 0xa65f, Stride: 0x1},
		unicode.Range16{Lo: 0xa661, Hi: 0xa661, Stride: 0x1},
		unicode.Range16{Lo: 0xa663, Hi: 0xa663, Stride: 0x1},
		unicode.Range16{Lo: 0xa665, Hi: 0xa665, Stride: 0x1},
		unicode.Range16{Lo: 0xa667, Hi: 0xa667, Stride: 0x1},
		unicode.Range16{Lo: 0xa669, Hi: 0xa669, Stride: 0x1},
		unicode.Range16{Lo: 0xa66b, Hi: 0xa66b, Stride: 0x1},
		unicode.Range16{Lo: 0xa66d, Hi: 0xa66d, Stride: 0x1},
		unicode.Range16{Lo: 0xa681, Hi: 0xa681, Stride: 0x1},
		unicode.Range16{Lo: 0xa683, Hi: 0xa683, Stride: 0x1},
		unicode.Range16{Lo: 0xa685, Hi: 0xa685, Stride: 0x1},
		unicode.Range16{Lo: 0xa687, Hi: 0xa687, Stride: 0x1},
		unicode.Range16{Lo: 0xa689, Hi: 0xa689, Stride: 0x1},
		unicode.Range16{Lo: 0xa68b, Hi: 0xa68b, Stride: 0x1},
		unicode.Range16{Lo: 0xa68d, Hi: 0xa68d, Stride: 0x1},
		unicode.Range16{Lo: 0xa68f, Hi: 0xa68f, Stride: 0x1},
		unicode.Range16{Lo: 0xa691, Hi: 0xa691, Stride: 0x1},
		unicode.Range16{Lo: 0xa693, Hi: 0xa693, Stride: 0x1},
		unicode.Range16{Lo: 0xa695, Hi: 0xa695, Stride: 0x1},
		unicode.Range16{Lo: 0xa697, Hi: 0xa697, Stride: 0x1},
		unicode.Range16{Lo: 0xa699, Hi: 0xa699, Stride: 0x1},
		unicode.Range16{Lo: 0xa69b, Hi: 0xa69b, Stride: 0x1},
		unicode.Range16{Lo: 0xa69c, Hi: 0xa69d, Stride: 0x1},
		unicode.Range16{Lo: 0xa723, Hi: 0xa723, Stride: 0x1},
		unicode.Range16{Lo: 0xa725, Hi: 0xa725, Stride: 0x1},
		unicode.Range16{Lo: 0xa727, Hi: 0xa727, Stride: 0x1},
		unicode.Range16{Lo: 0xa729, Hi: 0xa729, Stride: 0x1},
		unicode.Range16{Lo: 0xa72b, Hi: 0xa72b, Stride: 0x1},
		unicode.Range16{Lo: 0xa72d, Hi: 0xa72d, Stride: 0x1},
		unicode.Range16{Lo: 0xa72f, Hi: 0xa731, Stride: 0x1},
		unicode.Range16{Lo: 0xa733, Hi: 0xa733, Stride: 0x1},
		unicode.Range16{Lo: 0xa735, Hi: 0xa735, Stride: 0x1},
		unicode.Range16{Lo: 0xa737, Hi: 0xa737, Stride: 0x1},
		unicode.Range16{Lo: 0xa739, Hi: 0xa739, Stride: 0x1},
		unicode.Range16{Lo: 0xa73b, Hi: 0xa73b, Stride: 0x1},
		unicode.Range16{Lo: 0xa73d, Hi: 0xa73d, Stride: 0x1},
		unicode.Range16{Lo: 0xa73f, Hi: 0xa73f, Stride: 0x1},
		unicode.Range16{Lo: 0xa741, Hi: 0xa741, Stride: 0x1},
		unicode.Range16{Lo: 0xa743, Hi: 0xa743, Stride: 0x1},
		unicode.Range16{Lo: 0xa745, Hi: 0xa745, Stride: 0x1},
		unicode.Range16{Lo: 0xa747, Hi: 0xa747, Stride: 0x1},
		unicode.Range16{Lo: 0xa749, Hi: 0xa749, Stride: 0x1},
		unicode.Range16{Lo: 0xa74b, Hi: 0xa74b, Stride: 0x1},
		unicode.Range16{Lo: 0xa74d, Hi: 0xa74d, Stride: 0x1},
		unicode.Range16{Lo: 0xa74f, Hi: 0xa74f, Stride: 0x1},
		unicode.Range16{Lo: 0xa751, Hi: 0xa751, Stride: 0x1},
		unicode.Range16{Lo: 0xa753, Hi: 0xa753, Stride: 0x1},
		unicode.Range16{Lo: 0xa755, Hi: 0xa755, Stride: 0x1},
		unicode.Range16{Lo: 0xa757, Hi: 0xa757, Stride: 0x1},
		unicode.Range16{Lo: 0xa759, Hi: 0xa759, Stride: 0x1},
		unicode.Range16{Lo: 0xa75b, Hi: 0xa75b, Stride: 0x1},
		unicode.Range16{Lo: 0xa75d, Hi: 0xa75d, Stride: 0x1},
		unicode.Range16{Lo: 0xa75f, Hi: 0xa75f, Stride: 0x1},
		unicode.Range16{Lo: 0xa761, Hi: 0xa761, Stride: 0x1},
		unicode.Range16{Lo: 0xa763, Hi: 0xa763, Stride: 0x1},
		unicode.Range16{Lo: 0xa765, Hi: 0xa765, Stride: 0x1},
		unicode.Range16{Lo: 0xa767, Hi: 0xa767, Stride: 0x1},
		unicode.Range16{Lo: 0xa769, Hi: 0xa769, Stride: 0x1},
		unicode.Range16{Lo: 0xa76b, Hi: 0xa76b, Stride: 0x1},
		unicode.Range16{Lo: 0xa76d, Hi: 0xa76d, Stride: 0x1},
		unicode.Range16{Lo: 0xa76f, Hi: 0xa76f, Stride: 0x1},
		unicode.Range16{Lo: 0xa770, Hi: 0xa770, Stride: 0x1},
		unicode.Range16{Lo: 0xa771, Hi: 0xa778, Stride: 0x1},
		unicode.Range16{Lo: 0xa77a, Hi: 0xa77a, Stride: 0x1},
		unicode.Range16{Lo: 0xa77c, Hi: 0xa77c, Stride: 0x1},
		unicode.Range16{Lo: 0xa77f, Hi: 0xa77f, Stride: 0x1},
		unicode.Range16{Lo: 0xa781, Hi: 0xa781, Stride: 0x1},
		unicode.Range16{Lo: 0xa783, Hi: 0xa783, Stride: 0x1},
		unicode.Range16{Lo: 0xa785, Hi: 0xa785, Stride: 0x1},
		unicode.Range16{Lo: 0xa787, Hi: 0xa787, Stride: 0x1},
		unicode.Range16{Lo: 0xa78c, Hi: 0xa78c, Stride: 0x1},
		unicode.Range16{Lo: 0xa78e, Hi: 0xa78e, Stride: 0x1},
		unicode.Range16{Lo: 0xa791, Hi: 0xa791, Stride: 0x1},
		unicode.Range16{Lo: 0xa793, Hi: 0xa795, Stride: 0x1},
		unicode.Range16{Lo: 0xa797, Hi: 0xa797, Stride: 0x1},
		unicode.Range16{Lo: 0xa799, Hi: 0xa799, Stride: 0x1},
		unicode.Range16{Lo: 0xa79b, Hi: 0xa79b, Stride: 0x1},
		unicode.Range16{Lo: 0xa79d, Hi: 0xa79d, Stride: 0x1},
		unicode.Range16{Lo: 0xa79f, Hi: 0xa79f, Stride: 0x1},
		unicode.Range16{Lo: 0xa7a1, Hi: 0xa7a1, Stride: 0x1},
		unicode.Range16{Lo: 0xa7a3, Hi: 0xa7a3, Stride: 0x1},
		unicode.Range16{Lo: 0xa7a5, Hi: 0xa7a5, Stride: 0x1},
		unicode.Range16{Lo: 0xa7a7, Hi: 0xa7a7, Stride: 0x1},
		unicode.Range16{Lo: 0xa7a9, Hi: 0xa7a9, Stride: 0x1},
		unicode.Range16{Lo: 0xa7af, Hi: 0xa7af, Stride: 0x1},
		unicode.Range16{Lo: 0xa7b5, Hi: 0xa7b5, Stride: 0x1},
		unicode.Range16{Lo: 0xa7b7, Hi: 0xa7b7, Stride: 0x1},
		unicode.Range16{Lo: 0xa7b9, Hi: 0xa7b9, Stride: 0x1},
		unicode.Range16{Lo: 0xa7bb, Hi: 0xa7bb, Stride: 0x1},
		unicode.Range16{Lo: 0xa7bd, Hi: 0xa7bd, Stride: 0x1},
		unicode.Range16{Lo: 0xa7bf, Hi: 0xa7bf, Stride: 0x1},
		unicode.Range16{Lo: 0xa7c1, Hi: 0xa7c1, Stride: 0x1},
		unicode.Range16{Lo: 0xa7c3, Hi: 0xa7c3, Stride: 0x1},
		unicode.Range16{Lo: 0xa7c8, Hi: 0xa7c8, Stride: 0x1},
		unicode.Range16{Lo: 0xa7ca, Hi: 0xa7ca, Stride: 0x1},
		unicode.Range16{Lo: 0xa7d1, Hi: 0xa7d1, Stride: 0x1},
		unicode.Range16{Lo: 0xa7d3, Hi: 0xa7d3, Stride: 0x1},
		unicode.Range16{Lo: 0xa7d5, Hi: 0xa7d5, Stride: 0x1},
		unicode.Range16{Lo: 0xa7d7, Hi: 0xa7d7, Stride: 0x1},
		unicode.Range16{Lo: 0xa7d9, Hi: 0xa7d9, Stride: 0x1},
		unicode.Range16{Lo: 0xa7f2, Hi: 0xa7f4, Stride: 0x1},
		unicode.Range16{Lo: 0xa7f6, Hi: 0xa7f6, Stride: 0x1},
		unicode.Range16{Lo: 0xa7f8, Hi: 0xa7f9, Stride: 0x1},
		unicode.Range16{Lo: 0xa7fa, Hi: 0xa7fa, Stride: 0x1},
		unicode.Range16{Lo: 0xab30, Hi: 0xab5a, Stride: 0x1},
		unicode.Range16{Lo: 0xab5c, Hi: 0xab5f, Stride: 0x1},
		unicode.Range16{Lo: 0xab60, Hi: 0xab68, Stride: 0x1},
		unicode.Range16{Lo: 0xab69, Hi: 0xab69, Stride: 0x1},
		unicode.Range16{Lo: 0xab70, Hi: 0xabbf, Stride: 0x1},
		unicode.Range16{Lo: 0xfb00, Hi: 0xfb06, Stride: 0x1},
		unicode.Range16{Lo: 0xfb13, Hi: 0xfb17, Stride: 0x1},
		unicode.Range16{Lo: 0xff41, Hi: 0xff5a, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x10428, Hi: 0x1044f, Stride: 0x1},
		unicode.Range32{Lo: 0x104d8, Hi: 0x104fb, Stride: 0x1},
		unicode.Range32{Lo: 0x10597, Hi: 0x105a1, Stride: 0x1},
		unicode.Range32{Lo: 0x105a3, Hi: 0x105b1, Stride: 0x1},
		unicode.Range32{Lo: 0x105b3, Hi: 0x105b9, Stride: 0x1},
		unicode.Range32{Lo: 0x105bb, Hi: 0x105bc, Stride: 0x1},
		unicode.Range32{Lo: 0x10780, Hi: 0x10780, Stride: 0x1},
		unicode.Range32{Lo: 0x10783, Hi: 0x10785, Stride: 0x1},
		unicode.Range32{Lo: 0x10787, Hi: 0x107b0, Stride: 0x1},
		unicode.Range32{Lo: 0x107b2, Hi: 0x107ba, Stride: 0x1},
		unicode.Range32{Lo: 0x10cc0, Hi: 0x10cf2, Stride: 0x1},
		unicode.Range32{Lo: 0x118c0, Hi: 0x118df, Stride: 0x1},
		unicode.Range32{Lo: 0x16e60, Hi: 0x16e7f, Stride: 0x1},
		unicode.Range32{Lo: 0x1d41a, Hi: 0x1d433, Stride: 0x1},
		unicode.Range32{Lo: 0x1d44e, Hi: 0x1d454, Stride: 0x1},
		unicode.Range32{Lo: 0x1d456, Hi: 0x1d467, Stride: 0x1},
		unicode.Range32{Lo: 0x1d482, Hi: 0x1d49b, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4b6, Hi: 0x1d4b9, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4bb, Hi: 0x1d4bb, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4bd, Hi: 0x1d4c3, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4c5, Hi: 0x1d4cf, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4ea, Hi: 0x1d503, Stride: 0x1},
		unicode.Range32{Lo: 0x1d51e, Hi: 0x1d537, Stride: 0x1},
		unicode.Range32{Lo: 0x1d552, Hi: 0x1d56b, Stride: 0x1},
		unicode.Range32{Lo: 0x1d586, Hi: 0x1d59f, Stride: 0x1},
		unicode.Range32{Lo: 0x1d5ba, Hi: 0x1d5d3, Stride: 0x1},
		unicode.Range32{Lo: 0x1d5ee, Hi: 0x1d607, Stride: 0x1},
		unicode.Range32{Lo: 0x1d622, Hi: 0x1d63b, Stride: 0x1},
		unicode.Range32{Lo: 0x1d656, Hi: 0x1d66f, Stride: 0x1},
		unicode.Range32{Lo: 0x1d68a, Hi: 0x1d6a5, Stride: 0x1},
		unicode.Range32{Lo: 0x1d6c2, Hi: 0x1d6da, Stride: 0x1},
		unicode.Range32{Lo: 0x1d6dc, Hi: 0x1d6e1, Stride: 0x1},
		unicode.Range32{Lo: 0x1d6fc, Hi: 0x1d714, Stride: 0x1},
		unicode.Range32{Lo: 0x1d716, Hi: 0x1d71b, Stride: 0x1},
		unicode.Range32{Lo: 0x1d736, Hi: 0x1d74e, Stride: 0x1},
		unicode.Range32{Lo: 0x1d750, Hi: 0x1d755, Stride: 0x1},
		unicode.Range32{Lo: 0x1d770, Hi: 0x1d788, Stride: 0x1},
		unicode.Range32{Lo: 0x1d78a, Hi: 0x1d78f, Stride: 0x1},
		unicode.Range32{Lo: 0x1d7aa, Hi: 0x1d7c2, Stride: 0x1},
		unicode.Range32{Lo: 0x1d7c4, Hi: 0x1d7c9, Stride: 0x1},
		unicode.Range32{Lo: 0x1d7cb, Hi: 0x1d7cb, Stride: 0x1},
		unicode.Range32{Lo: 0x1df00, Hi: 0x1df09, Stride: 0x1},
		unicode.Range32{Lo: 0x1df0b, Hi: 0x1df1e, Stride: 0x1},
		unicode.Range32{Lo: 0x1df25, Hi: 0x1df2a, Stride: 0x1},
		unicode.Range32{Lo: 0x1e030, Hi: 0x1e06d, Stride: 0x1},
		unicode.Range32{Lo: 0x1e922, Hi: 0x1e943, Stride: 0x1},
	},
	LatinOffset: 6,
}

var _SentenceNumeric = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x30, Hi: 0x39, Stride: 0x1},
		unicode.Range16{Lo: 0x660, Hi: 0x669, Stride: 0x1},
		unicode.Range16{Lo: 0x66b, Hi: 0x66c, Stride: 0x1},
		unicode.Range16{Lo: 0x6f0, Hi: 0x6f9, Stride: 0x1},
		unicode.Range16{Lo: 0x7c0, Hi: 0x7c9, Stride: 0x1},
		unicode.Range16{Lo: 0x966, Hi: 0x96f, Stride: 0x1},
		unicode.Range16{Lo: 0x9e6, Hi: 0x9ef, Stride: 0x1},
		unicode.Range16{Lo: 0xa66, Hi: 0xa6f, Stride: 0x1},
		unicode.Range16{Lo: 0xae6, Hi: 0xaef, Stride: 0x1},
		unicode.Range16{Lo: 0xb66, Hi: 0xb6f, Stride: 0x1},
		unicode.Range16{Lo: 0xbe6, Hi: 0xbef, Stride: 0x1},
		unicode.Range16{Lo: 0xc66, Hi: 0xc6f, Stride: 0x1},
		unicode.Range16{Lo: 0xce6, Hi: 0xcef, Stride: 0x1},
		unicode.Range16{Lo: 0xd66, Hi: 0xd6f, Stride: 0x1},
		unicode.Range16{Lo: 0xde6, Hi: 0xdef, Stride: 0x1},
		unicode.Range16{Lo: 0xe50, Hi: 0xe59, Stride: 0x1},
		unicode.Range16{Lo: 0xed0, Hi: 0xed9, Stride: 0x1},
		unicode.Range16{Lo: 0xf20, Hi: 0xf29, Stride: 0x1},
		unicode.Range16{Lo: 0x1040, Hi: 0x1049, Stride: 0x1},
		unicode.Range16{Lo: 0x1090, Hi: 0x1099, Stride: 0x1},
		unicode.Range16{Lo: 0x17e0, Hi: 0x17e9, Stride: 0x1},
		unicode.Range16{Lo: 0x1810, Hi: 0x1819, Stride: 0x1},
		unicode.Range16{Lo: 0x1946, Hi: 0x194f, Stride: 0x1},
		unicode.Range16{Lo: 0x19d0, Hi: 0x19d9, Stride: 0x1},
		unicode.Range16{Lo: 0x1a80, Hi: 0x1a89, Stride: 0x1},
		unicode.Range16{Lo: 0x1a90, Hi: 0x1a99, Stride: 0x1},
		unicode.Range16{Lo: 0x1b50, Hi: 0x1b59, Stride: 0x1},
		unicode.Range16{Lo: 0x1bb0, Hi: 0x1bb9, Stride: 0x1},
		unicode.Range16{Lo: 0x1c40, Hi: 0x1c49, Stride: 0x1},
		unicode.Range16{Lo: 0x1c50, Hi: 0x1c59, Stride: 0x1},
		unicode.Range16{Lo: 0xa620, Hi: 0xa629, Stride: 0x1},
		unicode.Range16{Lo: 0xa8d0, Hi: 0xa8d9, Stride: 0x1},
		unicode.Range16{Lo: 0xa900, Hi: 0xa909, Stride: 0x1},
		unicode.Range16{Lo: 0xa9d0, Hi: 0xa9d9, Stride: 0x1},
		unicode.Range16{Lo: 0xa9f0, Hi: 0xa9f9, Stride: 0x1},
		unicode.Range16{Lo: 0xaa50, Hi: 0xaa59, Stride: 0x1},
		unicode.Range16{Lo: 0xabf0, Hi: 0xabf9, Stride: 0x1},
		unicode.Range16{Lo: 0xff10, Hi: 0xff19, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x104a0, Hi: 0x104a9, Stride: 0x1},
		unicode.Range32{Lo: 0x10d30, Hi: 0x10d39, Stride: 0x1},
		unicode.Range32{Lo: 0x11066, Hi: 0x1106f, Stride: 0x1},
		unicode.Range32{Lo: 0x110f0, Hi: 0x110f9, Stride: 0x1},
		unicode.Range32{Lo: 0x11136, Hi: 0x1113f, Stride: 0x1},
		unicode.Range32{Lo: 0x111d0, Hi: 0x111d9, Stride: 0x1},
		unicode.Range32{Lo: 0x112f0, Hi: 0x112f9, Stride: 0x1},
		unicode.Range32{Lo: 0x11450, Hi: 0x11459, Stride: 0x1},
		unicode.Range32{Lo: 0x114d0, Hi: 0x114d9, Stride: 0x1},
		unicode.Range32{Lo: 0x11650, Hi: 0x11659, Stride: 0x1},
		unicode.Range32{Lo: 0x116c0, Hi: 0x116c9, Stride: 0x1},
		unicode.Range32{Lo: 0x11730, Hi: 0x11739, Stride: 0x1},
		unicode.Range32{Lo: 0x118e0, Hi: 0x118e9, Stride: 0x1},
		unicode.Range32{Lo: 0x11950, Hi: 0x11959, Stride: 0x1},
		unicode.Range32{Lo: 0x11c50, Hi: 0x11c59, Stride: 0x1},
		unicode.Range32{Lo: 0x11d50, Hi: 0x11d59, Stride: 0x1},
		unicode.Range32{Lo: 0x11da0, Hi: 0x11da9, Stride: 0x1},
		unicode.Range32{Lo: 0x11f50, Hi: 0x11f59, Stride: 0x1},
		unicode.Range32{Lo: 0x16a60, Hi: 0x16a69, Stride: 0x1},
		unicode.Range32{Lo: 0x16ac0, Hi: 0x16ac9, Stride: 0x1},
		unicode.Range32{Lo: 0x16b50, Hi: 0x16b59, Stride: 0x1},
		unicode.Range32{Lo: 0x1d7ce, Hi: 0x1d7ff, Stride: 0x1},
		unicode.Range32{Lo: 0x1e140, Hi: 0x1e149, Stride: 0x1},
		unicode.Range32{Lo: 0x1e2f0, Hi: 0x1e2f9, Stride: 0x1},
		unicode.Range32{Lo: 0x1e4f0, Hi: 0x1e4f9, Stride: 0x1},
		unicode.Range32{Lo: 0x1e950, Hi: 0x1e959, Stride: 0x1},
		unicode.Range32{Lo: 0x1fbf0, Hi: 0x1fbf9, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _SentenceOLetter = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x1bb, Hi: 0x1bb, Stride: 0x1},
		unicode.Range16{Lo: 0x1c0, Hi: 0x1c3, Stride: 0x1},
		unicode.Range16{Lo: 0x294, Hi: 0x294, Stride: 0x1},
		unicode.Range16{Lo: 0x2b9, Hi: 0x2bf, Stride: 0x1},
		unicode.Range16{Lo: 0x2c6, Hi: 0x2d1, Stride: 0x1},
		unicode.Range16{Lo: 0x2ec, Hi: 0x2ec, Stride: 0x1},
		unicode.Range16{Lo: 0x2ee, Hi: 0x2ee, Stride: 0x1},
		unicode.Range16{Lo: 0x374, Hi: 0x374, Stride: 0x1},
		unicode.Range16{Lo: 0x559, Hi: 0x559, Stride: 0x1},
		unicode.Range16{Lo: 0x5d0, Hi: 0x5ea, Stride: 0x1},
		unicode.Range16{Lo: 0x5ef, Hi: 0x5f2, Stride: 0x1},
		unicode.Range16{Lo: 0x5f3, Hi: 0x5f3, Stride: 0x1},
		unicode.Range16{Lo: 0x620, Hi: 0x63f, Stride: 0x1},
		unicode.Range16{Lo: 0x640, Hi: 0x640, Stride: 0x1},
		unicode.Range16{Lo: 0x641, Hi: 0x64a, Stride: 0x1},
		unicode.Range16{Lo: 0x66e, Hi: 0x66f, Stride: 0x1},
		unicode.Range16{Lo: 0x671, Hi: 0x6d3, Stride: 0x1},
		unicode.Range16{Lo: 0x6d5, Hi: 0x6d5, Stride: 0x1},
		unicode.Range16{Lo: 0x6e5, Hi: 0x6e6, Stride: 0x1},
		unicode.Range16{Lo: 0x6ee, Hi: 0x6ef, Stride: 0x1},
		unicode.Range16{Lo: 0x6fa, Hi: 0x6fc, Stride: 0x1},
		unicode.Range16{Lo: 0x6ff, Hi: 0x6ff, Stride: 0x1},
		unicode.Range16{Lo: 0x710, Hi: 0x710, Stride: 0x1},
		unicode.Range16{Lo: 0x712, Hi: 0x72f, Stride: 0x1},
		unicode.Range16{Lo: 0x74d, Hi: 0x7a5, Stride: 0x1},
		unicode.Range16{Lo: 0x7b1, Hi: 0x7b1, Stride: 0x1},
		unicode.Range16{Lo: 0x7ca, Hi: 0x7ea, Stride: 0x1},
		unicode.Range16{Lo: 0x7f4, Hi: 0x7f5, Stride: 0x1},
		unicode.Range16{Lo: 0x7fa, Hi: 0x7fa, Stride: 0x1},
		unicode.Range16{Lo: 0x800, Hi: 0x815, Stride: 0x1},
		unicode.Range16{Lo: 0x81a, Hi: 0x81a, Stride: 0x1},
		unicode.Range16{Lo: 0x824, Hi: 0x824, Stride: 0x1},
		unicode.Range16{Lo: 0x828, Hi: 0x828, Stride: 0x1},
		unicode.Range16{Lo: 0x840, Hi: 0x858, Stride: 0x1},
		unicode.Range16{Lo: 0x860, Hi: 0x86a, Stride: 0x1},
		unicode.Range16{Lo: 0x870, Hi: 0x887, Stride: 0x1},
		unicode.Range16{Lo: 0x889, Hi: 0x88e, Stride: 0x1},
		unicode.Range16{Lo: 0x8a0, Hi: 0x8c8, Stride: 0x1},
		unicode.Range16{Lo: 0x8c9, Hi: 0x8c9, Stride: 0x1},
		unicode.Range16{Lo: 0x904, Hi: 0x939, Stride: 0x1},
		unicode.Range16{Lo: 0x93d, Hi: 0x93d, Stride: 0x1},
		unicode.Range16{Lo: 0x950, Hi: 0x950, Stride: 0x1},
		unicode.Range16{Lo: 0x958, Hi: 0x961, Stride: 0x1},
		unicode.Range16{Lo: 0x971, Hi: 0x971, Stride: 0x1},
		unicode.Range16{Lo: 0x972, Hi: 0x980, Stride: 0x1},
		unicode.Range16{Lo: 0x985, Hi: 0x98c, Stride: 0x1},
		unicode.Range16{Lo: 0x98f, Hi: 0x990, Stride: 0x1},
		unicode.Range16{Lo: 0x993, Hi: 0x9a8, Stride: 0x1},
		unicode.Range16{Lo: 0x9aa, Hi: 0x9b0, Stride: 0x1},
		unicode.Range16{Lo: 0x9b2, Hi: 0x9b2, Stride: 0x1},
		unicode.Range16{Lo: 0x9b6, Hi: 0x9b9, Stride: 0x1},
		unicode.Range16{Lo: 0x9bd, Hi: 0x9bd, Stride: 0x1},
		unicode.Range16{Lo: 0x9ce, Hi: 0x9ce, Stride: 0x1},
		unicode.Range16{Lo: 0x9dc, Hi: 0x9dd, Stride: 0x1},
		unicode.Range16{Lo: 0x9df, Hi: 0x9e1, Stride: 0x1},
		unicode.Range16{Lo: 0x9f0, Hi: 0x9f1, Stride: 0x1},
		unicode.Range16{Lo: 0x9fc, Hi: 0x9fc, Stride: 0x1},
		unicode.Range16{Lo: 0xa05, Hi: 0xa0a, Stride: 0x1},
		unicode.Range16{Lo: 0xa0f, Hi: 0xa10, Stride: 0x1},
		unicode.Range16{Lo: 0xa13, Hi: 0xa28, Stride: 0x1},
		unicode.Range16{Lo: 0xa2a, Hi: 0xa30, Stride: 0x1},
		unicode.Range16{Lo: 0xa32, Hi: 0xa33, Stride: 0x1},
		unicode.Range16{Lo: 0xa35, Hi: 0xa36, Stride: 0x1},
		unicode.Range16{Lo: 0xa38, Hi: 0xa39, Stride: 0x1},
		unicode.Range16{Lo: 0xa59, Hi: 0xa5c, Stride: 0x1},
		unicode.Range16{Lo: 0xa5e, Hi: 0xa5e, Stride: 0x1},
		unicode.Range16{Lo: 0xa72, Hi: 0xa74, Stride: 0x1},
		unicode.Range16{Lo: 0xa85, Hi: 0xa8d, Stride: 0x1},
		unicode.Range16{Lo: 0xa8f, Hi: 0xa91, Stride: 0x1},
		unicode.Range16{Lo: 0xa93, Hi: 0xaa8, Stride: 0x1},
		unicode.Range16{Lo: 0xaaa, Hi: 0xab0, Stride: 0x1},
		unicode.Range16{Lo: 0xab2, Hi: 0xab3, Stride: 0x1},
		unicode.Range16{Lo: 0xab5, Hi: 0xab9, Stride: 0x1},
		unicode.Range16{Lo: 0xabd, Hi: 0xabd, Stride: 0x1},
		unicode.Range16{Lo: 0xad0, Hi: 0xad0, Stride: 0x1},
		unicode.Range16{Lo: 0xae0, Hi: 0xae1, Stride: 0x1},
		unicode.Range16{Lo: 0xaf9, Hi: 0xaf9, Stride: 0x1},
		unicode.Range16{Lo: 0xb05, Hi: 0xb0c, Stride: 0x1},
		unicode.Range16{Lo: 0xb0f, Hi: 0xb10, Stride: 0x1},
		unicode.Range16{Lo: 0xb13, Hi: 0xb28, Stride: 0x1},
		unicode.Range16{Lo: 0xb2a, Hi: 0xb30, Stride: 0x1},
		unicode.Range16{Lo: 0xb32, Hi: 0xb33, Stride: 0x1},
		unicode.Range16{Lo: 0xb35, Hi: 0xb39, Stride: 0x1},
		unicode.Range16{Lo: 0xb3d, Hi: 0xb3d, Stride: 0x1},
		unicode.Range16{Lo: 0xb5c, Hi: 0xb5d, Stride: 0x1},
		unicode.Range16{Lo: 0xb5f, Hi: 0xb61, Stride: 0x1},
		unicode.Range16{Lo: 0xb71, Hi: 0xb71, Stride: 0x1},
		unicode.Range16{Lo: 0xb83, Hi: 0xb83, Stride: 0x1},
		unicode.Range16{Lo: 0xb85, Hi: 0xb8a, Stride: 0x1},
		unicode.Range16{Lo: 0xb8e, Hi: 0xb90, Stride: 0x1},
		unicode.Range16{Lo: 0xb92, Hi: 0xb95, Stride: 0x1},
		unicode.Range16{Lo: 0xb99, Hi: 0xb9a, Stride: 0x1},
		unicode.Range16{Lo: 0xb9c, Hi: 0xb9c, Stride: 0x1},
		unicode.Range16{Lo: 0xb9e, Hi: 0xb9f, Stride: 0x1},
		unicode.Range16{Lo: 0xba3, Hi: 0xba4, Stride: 0x1},
		unicode.Range16{Lo: 0xba8, Hi: 0xbaa, Stride: 0x1},
		unicode.Range16{Lo: 0xbae, Hi: 0xbb9, Stride: 0x1},
		unicode.Range16{Lo: 0xbd0, Hi: 0xbd0, Stride: 0x1},
		unicode.Range16{Lo: 0xc05, Hi: 0xc0c, Stride: 0x1},
		unicode.Range16{Lo: 0xc0e, Hi: 0xc10, Stride: 0x1},
		unicode.Range16{Lo: 0xc12, Hi: 0xc28, Stride: 0x1},
		unicode.Range16{Lo: 0xc2a, Hi: 0xc39, Stride: 0x1},
		unicode.Range16{Lo: 0xc3d, Hi: 0xc3d, Stride: 0x1},
		unicode.Range16{Lo: 0xc58, Hi: 0xc5a, Stride: 0x1},
		unicode.Range16{Lo: 0xc5d, Hi: 0xc5d, Stride: 0x1},
		unicode.Range16{Lo: 0xc60, Hi: 0xc61, Stride: 0x1},
		unicode.Range16{Lo: 0xc80, Hi: 0xc80, Stride: 0x1},
		unicode.Range16{Lo: 0xc85, Hi: 0xc8c, Stride: 0x1},
		unicode.Range16{Lo: 0xc8e, Hi: 0xc90, Stride: 0x1},
		unicode.Range16{Lo: 0xc92, Hi: 0xca8, Stride: 0x1},
		unicode.Range16{Lo: 0xcaa, Hi: 0xcb3, Stride: 0x1},
		unicode.Range16{Lo: 0xcb5, Hi: 0xcb9, Stride: 0x1},
		unicode.Range16{Lo: 0xcbd, Hi: 0xcbd, Stride: 0x1},
		unicode.Range16{Lo: 0xcdd, Hi: 0xcde, Stride: 0x1},
		unicode.Range16{Lo: 0xce0, Hi: 0xce1, Stride: 0x1},
		unicode.Range16{Lo: 0xcf1, Hi: 0xcf2, Stride: 0x1},
		unicode.Range16{Lo: 0xd04, Hi: 0xd0c, Stride: 0x1},
		unicode.Range16{Lo: 0xd0e, Hi: 0xd10, Stride: 0x1},
		unicode.Range16{Lo: 0xd12, Hi: 0xd3a, Stride: 0x1},
		unicode.Range16{Lo: 0xd3d, Hi: 0xd3d, Stride: 0x1},
		unicode.Range16{Lo: 0xd4e, Hi: 0xd4e, Stride: 0x1},
		unicode.Range16{Lo: 0xd54, Hi: 0xd56, Stride: 0x1},
		unicode.Range16{Lo: 0xd5f, Hi: 0xd61, Stride: 0x1},
		unicode.Range16{Lo: 0xd7a, Hi: 0xd7f, Stride: 0x1},
		unicode.Range16{Lo: 0xd85, Hi: 0xd96, Stride: 0x1},
		unicode.Range16{Lo: 0xd9a, Hi: 0xdb1, Stride: 0x1},
		unicode.Range16{Lo: 0xdb3, Hi: 0xdbb, Stride: 0x1},
		unicode.Range16{Lo: 0xdbd, Hi: 0xdbd, Stride: 0x1},
		unicode.Range16{Lo: 0xdc0, Hi: 0xdc6, Stride: 0x1},
		unicode.Range16{Lo: 0xe01, Hi: 0xe30, Stride: 0x1},
		unicode.Range16{Lo: 0xe32, Hi: 0xe33, Stride: 0x1},
		unicode.Range16{Lo: 0xe40, Hi: 0xe45, Stride: 0x1},
		unicode.Range16{Lo: 0xe46, Hi: 0xe46, Stride: 0x1},
		unicode.Range16{Lo: 0xe81, Hi: 0xe82, Stride: 0x1},
		unicode.Range16{Lo: 0xe84, Hi: 0xe84, Stride: 0x1},
		unicode.Range16{Lo: 0xe86, Hi: 0xe8a, Stride: 0x1},
		unicode.Range16{Lo: 0xe8c, Hi: 0xea3, Stride: 0x1},
		unicode.Range16{Lo: 0xea5, Hi: 0xea5, Stride: 0x1},
		unicode.Range16{Lo: 0xea7, Hi: 0xeb0, Stride: 0x1},
		unicode.Range16{Lo: 0xeb2, Hi: 0xeb3, Stride: 0x1},
		unicode.Range16{Lo: 0xebd, Hi: 0xebd, Stride: 0x1},
		unicode.Range16{Lo: 0xec0, Hi: 0xec4, Stride: 0x1},
		unicode.Range16{Lo: 0xec6, Hi: 0xec6, Stride: 0x1},
		unicode.Range16{Lo: 0xedc, Hi: 0xedf, Stride: 0x1},
		unicode.Range16{Lo: 0xf00, Hi: 0xf00, Stride: 0x1},
		unicode.Range16{Lo: 0xf40, Hi: 0xf47, Stride: 0x1},
		unicode.Range16{Lo: 0xf49, Hi: 0xf6c, Stride: 0x1},
		unicode.Range16{Lo: 0xf88, Hi: 0xf8c, Stride: 0x1},
		unicode.Range16{Lo: 0x1000, Hi: 0x102a, Stride: 0x1},
		unicode.Range16{Lo: 0x103f, Hi: 0x103f, Stride: 0x1},
		unicode.Range16{Lo: 0x1050, Hi: 0x1055, Stride: 0x1},
		unicode.Range16{Lo: 0x105a, Hi: 0x105d, Stride: 0x1},
		unicode.Range16{Lo: 0x1061, Hi: 0x1061, Stride: 0x1},
		unicode.Range16{Lo: 0x1065, Hi: 0x1066, Stride: 0x1},
		unicode.Range16{Lo: 0x106e, Hi: 0x1070, Stride: 0x1},
		unicode.Range16{Lo: 0x1075, Hi: 0x1081, Stride: 0x1},
		unicode.Range16{Lo: 0x108e, Hi: 0x108e, Stride: 0x1},
		unicode.Range16{Lo: 0x10d0, Hi: 0x10fa, Stride: 0x1},
		unicode.Range16{Lo: 0x10fd, Hi: 0x10ff, Stride: 0x1},
		unicode.Range16{Lo: 0x1100, Hi: 0x1248, Stride: 0x1},
		unicode.Range16{Lo: 0x124a, Hi: 0x124d, Stride: 0x1},
		unicode.Range16{Lo: 0x1250, Hi: 0x1256, Stride: 0x1},
		unicode.Range16{Lo: 0x1258, Hi: 0x1258, Stride: 0x1},
		unicode.Range16{Lo: 0x125a, Hi: 0x125d, Stride: 0x1},
		unicode.Range16{Lo: 0x1260, Hi: 0x1288, Stride: 0x1},
		unicode.Range16{Lo: 0x128a, Hi: 0x128d, Stride: 0x1},
		unicode.Range16{Lo: 0x1290, Hi: 0x12b0, Stride: 0x1},
		unicode.Range16{Lo: 0x12b2, Hi: 0x12b5, Stride: 0x1},
		unicode.Range16{Lo: 0x12b8, Hi: 0x12be, Stride: 0x1},
		unicode.Range16{Lo: 0x12c0, Hi: 0x12c0, Stride: 0x1},
		unicode.Range16{Lo: 0x12c2, Hi: 0x12c5, Stride: 0x1},
		unicode.Range16{Lo: 0x12c8, Hi: 0x12d6, Stride: 0x1},
		unicode.Range16{Lo: 0x12d8, Hi: 0x1310, Stride: 0x1},
		unicode.Range16{Lo: 0x1312, Hi: 0x1315, Stride: 0x1},
		unicode.Range16{Lo: 0x1318, Hi: 0x135a, Stride: 0x1},
		unicode.Range16{Lo: 0x1380, Hi: 0x138f, Stride: 0x1},
		unicode.Range16{Lo: 0x1401, Hi: 0x166c, Stride: 0x1},
		unicode.Range16{Lo: 0x166f, Hi: 0x167f, Stride: 0x1},
		unicode.Range16{Lo: 0x1681, Hi: 0x169a, Stride: 0x1},
		unicode.Range16{Lo: 0x16a0, Hi: 0x16ea, Stride: 0x1},
		unicode.Range16{Lo: 0x16ee, Hi: 0x16f0, Stride: 0x1},
		unicode.Range16{Lo: 0x16f1, Hi: 0x16f8, Stride: 0x1},
		unicode.Range16{Lo: 0x1700, Hi: 0x1711, Stride: 0x1},
		unicode.Range16{Lo: 0x171f, Hi: 0x1731, Stride: 0x1},
		unicode.Range16{Lo: 0x1740, Hi: 0x1751, Stride: 0x1},
		unicode.Range16{Lo: 0x1760, Hi: 0x176c, Stride: 0x1},
		unicode.Range16{Lo: 0x176e, Hi: 0x1770, Stride: 0x1},
		unicode.Range16{Lo: 0x1780, Hi: 0x17b3, Stride: 0x1},
		unicode.Range16{Lo: 0x17d7, Hi: 0x17d7, Stride: 0x1},
		unicode.Range16{Lo: 0x17dc, Hi: 0x17dc, Stride: 0x1},
		unicode.Range16{Lo: 0x1820, Hi: 0x1842, Stride: 0x1},
		unicode.Range16{Lo: 0x1843, Hi: 0x1843, Stride: 0x1},
		unicode.Range16{Lo: 0x1844, Hi: 0x1878, Stride: 0x1},
		unicode.Range16{Lo: 0x1880, Hi: 0x1884, Stride: 0x1},
		unicode.Range16{Lo: 0x1887, Hi: 0x18a8, Stride: 0x1},
		unicode.Range16{Lo: 0x18aa, Hi: 0x18aa, Stride: 0x1},
		unicode.Range16{Lo: 0x18b0, Hi: 0x18f5, Stride: 0x1},
		unicode.Range16{Lo: 0x1900, Hi: 0x191e, Stride: 0x1},
		unicode.Range16{Lo: 0x1950, Hi: 0x196d, Stride: 0x1},
		unicode.Range16{Lo: 0x1970, Hi: 0x1974, Stride: 0x1},
		unicode.Range16{Lo: 0x1980, Hi: 0x19ab, Stride: 0x1},
		unicode.Range16{Lo: 0x19b0, Hi: 0x19c9, Stride: 0x1},
		unicode.Range16{Lo: 0x1a00, Hi: 0x1a16, Stride: 0x1},
		unicode.Range16{Lo: 0x1a20, Hi: 0x1a54, Stride: 0x1},
		unicode.Range16{Lo: 0x1aa7, Hi: 0x1aa7, Stride: 0x1},
		unicode.Range16{Lo: 0x1b05, Hi: 0x1b33, Stride: 0x1},
		unicode.Range16{Lo: 0x1b45, Hi: 0x1b4c, Stride: 0x1},
		unicode.Range16{Lo: 0x1b83, Hi: 0x1ba0, Stride: 0x1},
		unicode.Range16{Lo: 0x1bae, Hi: 0x1baf, Stride: 0x1},
		unicode.Range16{Lo: 0x1bba, Hi: 0x1be5, Stride: 0x1},
		unicode.Range16{Lo: 0x1c00, Hi: 0x1c23, Stride: 0x1},
		unicode.Range16{Lo: 0x1c4d, Hi: 0x1c4f, Stride: 0x1},
		unicode.Range16{Lo: 0x1c5a, Hi: 0x1c77, Stride: 0x1},
		unicode.Range16{Lo: 0x1c78, Hi: 0x1c7d, Stride: 0x1},
		unicode.Range16{Lo: 0x1c90, Hi: 0x1cba, Stride: 0x1},
		unicode.Range16{Lo: 0x1cbd, Hi: 0x1cbf, Stride: 0x1},
		unicode.Range16{Lo: 0x1ce9, Hi: 0x1cec, Stride: 0x1},
		unicode.Range16{Lo: 0x1cee, Hi: 0x1cf3, Stride: 0x1},
		unicode.Range16{Lo: 0x1cf5, Hi: 0x1cf6, Stride: 0x1},
		unicode.Range16{Lo: 0x1cfa, Hi: 0x1cfa, Stride: 0x1},
		unicode.Range16{Lo: 0x2135, Hi: 0x2138, Stride: 0x1},
		unicode.Range16{Lo: 0x2180, Hi: 0x2182, Stride: 0x1},
		unicode.Range16{Lo: 0x2185, Hi: 0x2188, Stride: 0x1},
		unicode.Range16{Lo: 0x2d30, Hi: 0x2d67, Stride: 0x1},
		unicode.Range16{Lo: 0x2d6f, Hi: 0x2d6f, Stride: 0x1},
		unicode.Range16{Lo: 0x2d80, Hi: 0x2d96, Stride: 0x1},
		unicode.Range16{Lo: 0x2da0, Hi: 0x2da6, Stride: 0x1},
		unicode.Range16{Lo: 0x2da8, Hi: 0x2dae, Stride: 0x1},
		unicode.Range16{Lo: 0x2db0, Hi: 0x2db6, Stride: 0x1},
		unicode.Range16{Lo: 0x2db8, Hi: 0x2dbe, Stride: 0x1},
		unicode.Range16{Lo: 0x2dc0, Hi: 0x2dc6, Stride: 0x1},
		unicode.Range16{Lo: 0x2dc8, Hi: 0x2dce, Stride: 0x1},
		unicode.Range16{Lo: 0x2dd0, Hi: 0x2dd6, Stride: 0x1},
		unicode.Range16{Lo: 0x2dd8, Hi: 0x2dde, Stride: 0x1},
		unicode.Range16{Lo: 0x2e2f, Hi: 0x2e2f, Stride: 0x1},
		unicode.Range16{Lo: 0x3005, Hi: 0x3005, Stride: 0x1},
		unicode.Range16{Lo: 0x3006, Hi: 0x3006, Stride: 0x1},
		unicode.Range16{Lo: 0x3007, Hi: 0x3007, Stride: 0x1},
		unicode.Range16{Lo: 0x3021, Hi: 0x3029, Stride: 0x1},
		unicode.Range16{Lo: 0x3031, Hi: 0x3035, Stride: 0x1},
		unicode.Range16{Lo: 0x3038, Hi: 0x303a, Stride: 0x1},
		unicode.Range16{Lo: 0x303b, Hi: 0x303b, Stride: 0x1},
		unicode.Range16{Lo: 0x303c, Hi: 0x303c, Stride: 0x1},
		unicode.Range16{Lo: 0x3041, Hi: 0x3096, Stride: 0x1},
		unicode.Range16{Lo: 0x309d, Hi: 0x309e, Stride: 0x1},
		unicode.Range16{Lo: 0x309f, Hi: 0x309f, Stride: 0x1},
		unicode.Range16{Lo: 0x30a1, Hi: 0x30fa, Stride: 0x1},
		unicode.Range16{Lo: 0x30fc, Hi: 0x30fe, Stride: 0x1},
		unicode.Range16{Lo: 0x30ff, Hi: 0x30ff, Stride: 0x1},
		unicode.Range16{Lo: 0x3105, Hi: 0x312f, Stride: 0x1},
		unicode.Range16{Lo: 0x3131, Hi: 0x318e, Stride: 0x1},
		unicode.Range16{Lo: 0x31a0, Hi: 0x31bf, Stride: 0x1},
		unicode.Range16{Lo: 0x31f0, Hi: 0x31ff, Stride: 0x1},
		unicode.Range16{Lo: 0x3400, Hi: 0x4dbf, Stride: 0x1},
		unicode.Range16{Lo: 0x4e00, Hi: 0xa014, Stride: 0x1},
		unicode.Range16{Lo: 0xa015, Hi: 0xa015, Stride: 0x1},
		unicode.Range16{Lo: 0xa016, Hi: 0xa48c, Stride: 0x1},
		unicode.Range16{Lo: 0xa4d0, Hi: 0xa4f7, Stride: 0x1},
		unicode.Range16{Lo: 0xa4f8, Hi: 0xa4fd, Stride: 0x1},
		unicode.Range16{Lo: 0xa500, Hi: 0xa60b, Stride: 0x1},
		unicode.Range16{Lo: 0xa60c, Hi: 0xa60c, Stride: 0x1},
		unicode.Range16{Lo: 0xa610, Hi: 0xa61f, Stride: 0x1},
		unicode.Range16{Lo: 0xa62a, Hi: 0xa62b, Stride: 0x1},
		unicode.Range16{Lo: 0xa66e, Hi: 0xa66e, Stride: 0x1},
		unicode.Range16{Lo: 0xa67f, Hi: 0xa67f, Stride: 0x1},
		unicode.Range16{Lo: 0xa6a0, Hi: 0xa6e5, Stride: 0x1},
		unicode.Range16{Lo: 0xa6e6, Hi: 0xa6ef, Stride: 0x1},
		unicode.Range16{Lo: 0xa717, Hi: 0xa71f, Stride: 0x1},
		unicode.Range16{Lo: 0xa788, Hi: 0xa788, Stride: 0x1},
		unicode.Range16{Lo: 0xa78f, Hi: 0xa78f, Stride: 0x1},
		unicode.Range16{Lo: 0xa7f7, Hi: 0xa7f7, Stride: 0x1},
		unicode.Range16{Lo: 0xa7fb, Hi: 0xa801, Stride: 0x1},
		unicode.Range16{Lo: 0xa803, Hi: 0xa805, Stride: 0x1},
		unicode.Range16{Lo: 0xa807, Hi: 0xa80a, Stride: 0x1},
		unicode.Range16{Lo: 0xa80c, Hi: 0xa822, Stride: 0x1},
		unicode.Range16{Lo: 0xa840, Hi: 0xa873, Stride: 0x1},
		unicode.Range16{Lo: 0xa882, Hi: 0xa8b3, Stride: 0x1},
		unicode.Range16{Lo: 0xa8f2, Hi: 0xa8f7, Stride: 0x1},
		unicode.Range16{Lo: 0xa8fb, Hi: 0xa8fb, Stride: 0x1},
		unicode.Range16{Lo: 0xa8fd, Hi: 0xa8fe, Stride: 0x1},
		unicode.Range16{Lo: 0xa90a, Hi: 0xa925, Stride: 0x1},
		unicode.Range16{Lo: 0xa930, Hi: 0xa946, Stride: 0x1},
		unicode.Range16{Lo: 0xa960, Hi: 0xa97c, Stride: 0x1},
		unicode.Range16{Lo: 0xa984, Hi: 0xa9b2, Stride: 0x1},
		unicode.Range16{Lo: 0xa9cf, Hi: 0xa9cf, Stride: 0x1},
		unicode.Range16{Lo: 0xa9e0, Hi: 0xa9e4, Stride: 0x1},
		unicode.Range16{Lo: 0xa9e6, Hi: 0xa9e6, Stride: 0x1},
		unicode.Range16{Lo: 0xa9e7, Hi: 0xa9ef, Stride: 0x1},
		unicode.Range16{Lo: 0xa9fa, Hi: 0xa9fe, Stride: 0x1},
		unicode.Range16{Lo: 0xaa00, Hi: 0xaa28, Stride: 0x1},
		unicode.Range16{Lo: 0xaa40, Hi: 0xaa42, Stride: 0x1},
		unicode.Range16{Lo: 0xaa44, Hi: 0xaa4b, Stride: 0x1},
		unicode.Range16{Lo: 0xaa60, Hi: 0xaa6f, Stride: 0x1},
		unicode.Range16{Lo: 0xaa70, Hi: 0xaa70, Stride: 0x1},
		unicode.Range16{Lo: 0xaa71, Hi: 0xaa76, Stride: 0x1},
		unicode.Range16{Lo: 0xaa7a, Hi: 0xaa7a, Stride: 0x1},
		unicode.Range16{Lo: 0xaa7e, Hi: 0xaaaf, Stride: 0x1},
		unicode.Range16{Lo: 0xaab1, Hi: 0xaab1, Stride: 0x1},
		unicode.Range16{Lo: 0xaab5, Hi: 0xaab6, Stride: 0x1},
		unicode.Range16{Lo: 0xaab9, Hi: 0xaabd, Stride: 0x1},
		unicode.Range16{Lo: 0xaac0, Hi: 0xaac0, Stride: 0x1},
		unicode.Range16{Lo: 0xaac2, Hi: 0xaac2, Stride: 0x1},
		unicode.Range16{Lo: 0xaadb, Hi: 0xaadc, Stride: 0x1},
		unicode.Range16{Lo: 0xaadd, Hi: 0xaadd, Stride: 0x1},
		unicode.Range16{Lo: 0xaae0, Hi: 0xaaea, Stride: 0x1},
		unicode.Range16{Lo: 0xaaf2, Hi: 0xaaf2, Stride: 0x1},
		unicode.Range16{Lo: 0xaaf3, Hi: 0xaaf4, Stride: 0x1},
		unicode.Range16{Lo: 0xab01, Hi: 0xab06, Stride: 0x1},
		unicode.Range16{Lo: 0xab09, Hi: 0xab0e, Stride: 0x1},
		unicode.Range16{Lo: 0xab11, Hi: 0xab16, Stride: 0x1},
		unicode.Range16{Lo: 0xab20, Hi: 0xab26, Stride: 0x1},
		unicode.Range16{Lo: 0xab28, Hi: 0xab2e, Stride: 0x1},
		unicode.Range16{Lo: 0xabc0, Hi: 0xabe2, Stride: 0x1},
		unicode.Range16{Lo: 0xac00, Hi: 0xd7a3, Stride: 0x1},
		unicode.Range16{Lo: 0xd7b0, Hi: 0xd7c6, Stride: 0x1},
		unicode.Range16{Lo: 0xd7cb, Hi: 0xd7fb, Stride: 0x1},
		unicode.Range16{Lo: 0xf900, Hi: 0xfa6d, Stride: 0x1},
		unicode.Range16{Lo: 0xfa70, Hi: 0xfad9, Stride: 0x1},
		unicode.Range16{Lo: 0xfb1d, Hi: 0xfb1d, Stride: 0x1},
		unicode.Range16{Lo: 0xfb1f, Hi: 0xfb28, Stride: 0x1},
		unicode.Range16{Lo: 0xfb2a, Hi: 0xfb36, Stride: 0x1},
		unicode.Range16{Lo: 0xfb38, Hi: 0xfb3c, Stride: 0x1},
		unicode.Range16{Lo: 0xfb3e, Hi: 0xfb3e, Stride: 0x1},
		unicode.Range16{Lo: 0xfb40, Hi: 0xfb41, Stride: 0x1},
		unicode.Range16{Lo: 0xfb43, Hi: 0xfb44, Stride: 0x1},
		unicode.Range16{Lo: 0xfb46, Hi: 0xfbb1, Stride: 0x1},
		unicode.Range16{Lo: 0xfbd3, Hi: 0xfd3d, Stride: 0x1},
		unicode.Range16{Lo: 0xfd50, Hi: 0xfd8f, Stride: 0x1},
		unicode.Range16{Lo: 0xfd92, Hi: 0xfdc7, Stride: 0x1},
		unicode.Range16{Lo: 0xfdf0, Hi: 0xfdfb, Stride: 0x1},
		unicode.Range16{Lo: 0xfe70, Hi: 0xfe74, Stride: 0x1},
		unicode.Range16{Lo: 0xfe76, Hi: 0xfefc, Stride: 0x1},
		unicode.Range16{Lo: 0xff66, Hi: 0xff6f, Stride: 0x1},
		unicode.Range16{Lo: 0xff70, Hi: 0xff70, Stride: 0x1},
		unicode.Range16{Lo: 0xff71, Hi: 0xff9d, Stride: 0x1},
		unicode.Range16{Lo: 0xffa0, Hi: 0xffbe, Stride: 0x1},
		unicode.Range16{Lo: 0xffc2, Hi: 0xffc7, Stride: 0x1},
		unicode.Range16{Lo: 0xffca, Hi: 0xffcf, Stride: 0x1},
		unicode.Range16{Lo: 0xffd2, Hi: 0xffd7, Stride: 0x1},
		unicode.Range16{Lo: 0xffda, Hi: 0xffdc, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x10000, Hi: 0x1000b, Stride: 0x1},
		unicode.Range32{Lo: 0x1000d, Hi: 0x10026, Stride: 0x1},
		unicode.Range32{Lo: 0x10028, Hi: 0x1003a, Stride: 0x1},
		unicode.Range32{Lo: 0x1003c, Hi: 0x1003d, Stride: 0x1},
		unicode.Range32{Lo: 0x1003f, Hi: 0x1004d, Stride: 0x1},
		unicode.Range32{Lo: 0x10050, Hi: 0x1005d, Stride: 0x1},
		unicode.Range32{Lo: 0x10080, Hi: 0x100fa, Stride: 0x1},
		unicode.Range32{Lo: 0x10140, Hi: 0x10174, Stride: 0x1},
		unicode.Range32{Lo: 0x10280, Hi: 0x1029c, Stride: 0x1},
		unicode.Range32{Lo: 0x102a0, Hi: 0x102d0, Stride: 0x1},
		unicode.Range32{Lo: 0x10300, Hi: 0x1031f, Stride: 0x1},
		unicode.Range32{Lo: 0x1032d, Hi: 0x10340, Stride: 0x1},
		unicode.Range32{Lo: 0x10341, Hi: 0x10341, Stride: 0x1},
		unicode.Range32{Lo: 0x10342, Hi: 0x10349, Stride: 0x1},
		unicode.Range32{Lo: 0x1034a, Hi: 0x1034a, Stride: 0x1},
		unicode.Range32{Lo: 0x10350, Hi: 0x10375, Stride: 0x1},
		unicode.Range32{Lo: 0x10380, Hi: 0x1039d, Stride: 0x1},
		unicode.Range32{Lo: 0x103a0, Hi: 0x103c3, Stride: 0x1},
		unicode.Range32{Lo: 0x103c8, Hi: 0x103cf, Stride: 0x1},
		unicode.Range32{Lo: 0x103d1, Hi: 0x103d5, Stride: 0x1},
		unicode.Range32{Lo: 0x10450, Hi: 0x1049d, Stride: 0x1},
		unicode.Range32{Lo: 0x10500, Hi: 0x10527, Stride: 0x1},
		unicode.Range32{Lo: 0x10530, Hi: 0x10563, Stride: 0x1},
		unicode.Range32{Lo: 0x10600, Hi: 0x10736, Stride: 0x1},
		unicode.Range32{Lo: 0x10740, Hi: 0x10755, Stride: 0x1},
		unicode.Range32{Lo: 0x10760, Hi: 0x10767, Stride: 0x1},
		unicode.Range32{Lo: 0x10781, Hi: 0x10782, Stride: 0x1},
		unicode.Range32{Lo: 0x10800, Hi: 0x10805, Stride: 0x1},
		unicode.Range32{Lo: 0x10808, Hi: 0x10808, Stride: 0x1},
		unicode.Range32{Lo: 0x1080a, Hi: 0x10835, Stride: 0x1},
		unicode.Range32{Lo: 0x10837, Hi: 0x10838, Stride: 0x1},
		unicode.Range32{Lo: 0x1083c, Hi: 0x1083c, Stride: 0x1},
		unicode.Range32{Lo: 0x1083f, Hi: 0x10855, Stride: 0x1},
		unicode.Range32{Lo: 0x10860, Hi: 0x10876, Stride: 0x1},
		unicode.Range32{Lo: 0x10880, Hi: 0x1089e, Stride: 0x1},
		unicode.Range32{Lo: 0x108e0, Hi: 0x108f2, Stride: 0x1},
		unicode.Range32{Lo: 0x108f4, Hi: 0x108f5, Stride: 0x1},
		unicode.Range32{Lo: 0x10900, Hi: 0x10915, Stride: 0x1},
		unicode.Range32{Lo: 0x10920, Hi: 0x10939, Stride: 0x1},
		unicode.Range32{Lo: 0x10980, Hi: 0x109b7, Stride: 0x1},
		unicode.Range32{Lo: 0x109be, Hi: 0x109bf, Stride: 0x1},
		unicode.Range32{Lo: 0x10a00, Hi: 0x10a00, Stride: 0x1},
		unicode.Range32{Lo: 0x10a10, Hi: 0x10a13, Stride: 0x1},
		unicode.Range32{Lo: 0x10a15, Hi: 0x10a17, Stride: 0x1},
		unicode.Range32{Lo: 0x10a19, Hi: 0x10a35, Stride: 0x1},
		unicode.Range32{Lo: 0x10a60, Hi: 0x10a7c, Stride: 0x1},
		unicode.Range32{Lo: 0x10a80, Hi: 0x10a9c, Stride: 0x1},
		unicode.Range32{Lo: 0x10ac0, Hi: 0x10ac7, Stride: 0x1},
		unicode.Range32{Lo: 0x10ac9, Hi: 0x10ae4, Stride: 0x1},
		unicode.Range32{Lo: 0x10b00, Hi: 0x10b35, Stride: 0x1},
		unicode.Range32{Lo: 0x10b40, Hi: 0x10b55, Stride: 0x1},
		unicode.Range32{Lo: 0x10b60, Hi: 0x10b72, Stride: 0x1},
		unicode.Range32{Lo: 0x10b80, Hi: 0x10b91, Stride: 0x1},
		unicode.Range32{Lo: 0x10c00, Hi: 0x10c48, Stride: 0x1},
		unicode.Range32{Lo: 0x10d00, Hi: 0x10d23, Stride: 0x1},
		unicode.Range32{Lo: 0x10e80, Hi: 0x10ea9, Stride: 0x1},
		unicode.Range32{Lo: 0x10eb0, Hi: 0x10eb1, Stride: 0x1},
		unicode.Range32{Lo: 0x10f00, Hi: 0x10f1c, Stride: 0x1},
		unicode.Range32{Lo: 0x10f27, Hi: 0x10f27, Stride: 0x1},
		unicode.Range32{Lo: 0x10f30, Hi: 0x10f45, Stride: 0x1},
		unicode.Range32{Lo: 0x10f70, Hi: 0x10f81, Stride: 0x1},
		unicode.Range32{Lo: 0x10fb0, Hi: 0x10fc4, Stride: 0x1},
		unicode.Range32{Lo: 0x10fe0, Hi: 0x10ff6, Stride: 0x1},
		unicode.Range32{Lo: 0x11003, Hi: 0x11037, Stride: 0x1},
		unicode.Range32{Lo: 0x11071, Hi: 0x11072, Stride: 0x1},
		unicode.Range32{Lo: 0x11075, Hi: 0x11075, Stride: 0x1},
		unicode.Range32{Lo: 0x11083, Hi: 0x110af, Stride: 0x1},
		unicode.Range32{Lo: 0x110d0, Hi: 0x110e8, Stride: 0x1},
		unicode.Range32{Lo: 0x11103, Hi: 0x11126, Stride: 0x1},
		unicode.Range32{Lo: 0x11144, Hi: 0x11144, Stride: 0x1},
		unicode.Range32{Lo: 0x11147, Hi: 0x11147, Stride: 0x1},
		unicode.Range32{Lo: 0x11150, Hi: 0x11172, Stride: 0x1},
		unicode.Range32{Lo: 0x11176, Hi: 0x11176, Stride: 0x1},
		unicode.Range32{Lo: 0x11183, Hi: 0x111b2, Stride: 0x1},
		unicode.Range32{Lo: 0x111c1, Hi: 0x111c4, Stride: 0x1},
		unicode.Range32{Lo: 0x111da, Hi: 0x111da, Stride: 0x1},
		unicode.Range32{Lo: 0x111dc, Hi: 0x111dc, Stride: 0x1},
		unicode.Range32{Lo: 0x11200, Hi: 0x11211, Stride: 0x1},
		unicode.Range32{Lo: 0x11213, Hi: 0x1122b, Stride: 0x1},
		unicode.Range32{Lo: 0x1123f, Hi: 0x11240, Stride: 0x1},
		unicode.Range32{Lo: 0x11280, Hi: 0x11286, Stride: 0x1},
		unicode.Range32{Lo: 0x11288, Hi: 0x11288, Stride: 0x1},
		unicode.Range32{Lo: 0x1128a, Hi: 0x1128d, Stride: 0x1},
		unicode.Range32{Lo: 0x1128f, Hi: 0x1129d, Stride: 0x1},
		unicode.Range32{Lo: 0x1129f, Hi: 0x112a8, Stride: 0x1},
		unicode.Range32{Lo: 0x112b0, Hi: 0x112de, Stride: 0x1},
		unicode.Range32{Lo: 0x11305, Hi: 0x1130c, Stride: 0x1},
		unicode.Range32{Lo: 0x1130f, Hi: 0x11310, Stride: 0x1},
		unicode.Range32{Lo: 0x11313, Hi: 0x11328, Stride: 0x1},
		unicode.Range32{Lo: 0x1132a, Hi: 0x11330, Stride: 0x1},
		unicode.Range32{Lo: 0x11332, Hi: 0x11333, Stride: 0x1},
		unicode.Range32{Lo: 0x11335, Hi: 0x11339, Stride: 0x1},
		unicode.Range32{Lo: 0x1133d, Hi: 0x1133d, Stride: 0x1},
		unicode.Range32{Lo: 0x11350, Hi: 0x11350, Stride: 0x1},
		unicode.Range32{Lo: 0x1135d, Hi: 0x11361, Stride: 0x1},
		unicode.Range32{Lo: 0x11400, Hi: 0x11434, Stride: 0x1},
		unicode.Range32{Lo: 0x11447, Hi: 0x1144a, Stride: 0x1},
		unicode.Range32{Lo: 0x1145f, Hi: 0x11461, Stride: 0x1},
		unicode.Range32{Lo: 0x11480, Hi: 0x114af, Stride: 0x1},
		unicode.Range32{Lo: 0x114c4, Hi: 0x114c5, Stride: 0x1},
		unicode.Range32{Lo: 0x114c7, Hi: 0x114c7, Stride: 0x1},
		unicode.Range32{Lo: 0x11580, Hi: 0x115ae, Stride: 0x1},
		unicode.Range32{Lo: 0x115d8, Hi: 0x115db, Stride: 0x1},
		unicode.Range32{Lo: 0x11600, Hi: 0x1162f, Stride: 0x1},
		unicode.Range32{Lo: 0x11644, Hi: 0x11644, Stride: 0x1},
		unicode.Range32{Lo: 0x11680, Hi: 0x116aa, Stride: 0x1},
		unicode.Range32{Lo: 0x116b8, Hi: 0x116b8, Stride: 0x1},
		unicode.Range32{Lo: 0x11700, Hi: 0x1171a, Stride: 0x1},
		unicode.Range32{Lo: 0x11740, Hi: 0x11746, Stride: 0x1},
		unicode.Range32{Lo: 0x11800, Hi: 0x1182b, Stride: 0x1},
		unicode.Range32{Lo: 0x118ff, Hi: 0x11906, Stride: 0x1},
		unicode.Range32{Lo: 0x11909, Hi: 0x11909, Stride: 0x1},
		unicode.Range32{Lo: 0x1190c, Hi: 0x11913, Stride: 0x1},
		unicode.Range32{Lo: 0x11915, Hi: 0x11916, Stride: 0x1},
		unicode.Range32{Lo: 0x11918, Hi: 0x1192f, Stride: 0x1},
		unicode.Range32{Lo: 0x1193f, Hi: 0x1193f, Stride: 0x1},
		unicode.Range32{Lo: 0x11941, Hi: 0x11941, Stride: 0x1},
		unicode.Range32{Lo: 0x119a0, Hi: 0x119a7, Stride: 0x1},
		unicode.Range32{Lo: 0x119aa, Hi: 0x119d0, Stride: 0x1},
		unicode.Range32{Lo: 0x119e1, Hi: 0x119e1, Stride: 0x1},
		unicode.Range32{Lo: 0x119e3, Hi: 0x119e3, Stride: 0x1},
		unicode.Range32{Lo: 0x11a00, Hi: 0x11a00, Stride: 0x1},
		unicode.Range32{Lo: 0x11a0b, Hi: 0x11a32, Stride: 0x1},
		unicode.Range32{Lo: 0x11a3a, Hi: 0x11a3a, Stride: 0x1},
		unicode.Range32{Lo: 0x11a50, Hi: 0x11a50, Stride: 0x1},
		unicode.Range32{Lo: 0x11a5c, Hi: 0x11a89, Stride: 0x1},
		unicode.Range32{Lo: 0x11a9d, Hi: 0x11a9d, Stride: 0x1},
		unicode.Range32{Lo: 0x11ab0, Hi: 0x11af8, Stride: 0x1},
		unicode.Range32{Lo: 0x11c00, Hi: 0x11c08, Stride: 0x1},
		unicode.Range32{Lo: 0x11c0a, Hi: 0x11c2e, Stride: 0x1},
		unicode.Range32{Lo: 0x11c40, Hi: 0x11c40, Stride: 0x1},
		unicode.Range32{Lo: 0x11c72, Hi: 0x11c8f, Stride: 0x1},
		unicode.Range32{Lo: 0x11d00, Hi: 0x11d06, Stride: 0x1},
		unicode.Range32{Lo: 0x11d08, Hi: 0x11d09, Stride: 0x1},
		unicode.Range32{Lo: 0x11d0b, Hi: 0x11d30, Stride: 0x1},
		unicode.Range32{Lo: 0x11d46, Hi: 0x11d46, Stride: 0x1},
		unicode.Range32{Lo: 0x11d60, Hi: 0x11d65, Stride: 0x1},
		unicode.Range32{Lo: 0x11d67, Hi: 0x11d68, Stride: 0x1},
		unicode.Range32{Lo: 0x11d6a, Hi: 0x11d89, Stride: 0x1},
		unicode.Range32{Lo: 0x11d98, Hi: 0x11d98, Stride: 0x1},
		unicode.Range32{Lo: 0x11ee0, Hi: 0x11ef2, Stride: 0x1},
		unicode.Range32{Lo: 0x11f02, Hi: 0x11f02, Stride: 0x1},
		unicode.Range32{Lo: 0x11f04, Hi: 0x11f10, Stride: 0x1},
		unicode.Range32{Lo: 0x11f12, Hi: 0x11f33, Stride: 0x1},
		unicode.Range32{Lo: 0x11fb0, Hi: 0x11fb0, Stride: 0x1},
		unicode.Range32{Lo: 0x12000, Hi: 0x12399, Stride: 0x1},
		unicode.Range32{Lo: 0x12400, Hi: 0x1246e, Stride: 0x1},
		unicode.Range32{Lo: 0x12480, Hi: 0x12543, Stride: 0x1},
		unicode.Range32{Lo: 0x12f90, Hi: 0x12ff0, Stride: 0x1},
		unicode.Range32{Lo: 0x13000, Hi: 0x1342f, Stride: 0x1},
		unicode.Range32{Lo: 0x13441, Hi: 0x13446, Stride: 0x1},
		unicode.Range32{Lo: 0x14400, Hi: 0x14646, Stride: 0x1},
		unicode.Range32{Lo: 0x16800, Hi: 0x16a38, Stride: 0x1},
		unicode.Range32{Lo: 0x16a40, Hi: 0x16a5e, Stride: 0x1},
		unicode.Range32{Lo: 0x16a70, Hi: 0x16abe, Stride: 0x1},
		unicode.Range32{Lo: 0x16ad0, Hi: 0x16aed, Stride: 0x1},
		unicode.Range32{Lo: 0x16b00, Hi: 0x16b2f, Stride: 0x1},
		unicode.Range32{Lo: 0x16b40, Hi: 0x16b43, Stride: 0x1},
		unicode.Range32{Lo: 0x16b63, Hi: 0x16b77, Stride: 0x1},
		unicode.Range32{Lo: 0x16b7d, Hi: 0x16b8f, Stride: 0x1},
		unicode.Range32{Lo: 0x16f00, Hi: 0x16f4a, Stride: 0x1},
		unicode.Range32{Lo: 0x16f50, Hi: 0x16f50, Stride: 0x1},
		unicode.Range32{Lo: 0x16f93, Hi: 0x16f9f, Stride: 0x1},
		unicode.Range32{Lo: 0x16fe0, Hi: 0x16fe1, Stride: 0x1},
		unicode.Range32{Lo: 0x16fe3, Hi: 0x16fe3, Stride: 0x1},
		unicode.Range32{Lo: 0x17000, Hi: 0x187f7, Stride: 0x1},
		unicode.Range32{Lo: 0x18800, Hi: 0x18cd5, Stride: 0x1},
		unicode.Range32{Lo: 0x18d00, Hi: 0x18d08, Stride: 0x1},
		unicode.Range32{Lo: 0x1aff0, Hi: 0x1aff3, Stride: 0x1},
		unicode.Range32{Lo: 0x1aff5, Hi: 0x1affb, Stride: 0x1},
		unicode.Range32{Lo: 0x1affd, Hi: 0x1affe, Stride: 0x1},
		unicode.Range32{Lo: 0x1b000, Hi: 0x1b122, Stride: 0x1},
		unicode.Range32{Lo: 0x1b132, Hi: 0x1b132, Stride: 0x1},
		unicode.Range32{Lo: 0x1b150, Hi: 0x1b152, Stride: 0x1},
		unicode.Range32{Lo: 0x1b155, Hi: 0x1b155, Stride: 0x1},
		unicode.Range32{Lo: 0x1b164, Hi: 0x1b167, Stride: 0x1},
		unicode.Range32{Lo: 0x1b170, Hi: 0x1b2fb, Stride: 0x1},
		unicode.Range32{Lo: 0x1bc00, Hi: 0x1bc6a, Stride: 0x1},
		unicode.Range32{Lo: 0x1bc70, Hi: 0x1bc7c, Stride: 0x1},
		unicode.Range32{Lo: 0x1bc80, Hi: 0x1bc88, Stride: 0x1},
		unicode.Range32{Lo: 0x1bc90, Hi: 0x1bc99, Stride: 0x1},
		unicode.Range32{Lo: 0x1df0a, Hi: 0x1df0a, Stride: 0x1},
		unicode.Range32{Lo: 0x1e100, Hi: 0x1e12c, Stride: 0x1},
		unicode.Range32{Lo: 0x1e137, Hi: 0x1e13d, Stride: 0x1},
		unicode.Range32{Lo: 0x1e14e, Hi: 0x1e14e, Stride: 0x1},
		unicode.Range32{Lo: 0x1e290, Hi: 0x1e2ad, Stride: 0x1},
		unicode.Range32{Lo: 0x1e2c0, Hi: 0x1e2eb, Stride: 0x1},
		unicode.Range32{Lo: 0x1e4d0, Hi: 0x1e4ea, Stride: 0x1},
		unicode.Range32{Lo: 0x1e4eb, Hi: 0x1e4eb, Stride: 0x1},
		unicode.Range32{Lo: 0x1e7e0, Hi: 0x1e7e6, Stride: 0x1},
		unicode.Range32{Lo: 0x1e7e8, Hi: 0x1e7eb, Stride: 0x1},
		unicode.Range32{Lo: 0x1e7ed, Hi: 0x1e7ee, Stride: 0x1},
		unicode.Range32{Lo: 0x1e7f0, Hi: 0x1e7fe, Stride: 0x1},
		unicode.Range32{Lo: 0x1e800, Hi: 0x1e8c4, Stride: 0x1},
		unicode.Range32{Lo: 0x1e94b, Hi: 0x1e94b, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee00, Hi: 0x1ee03, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee05, Hi: 0x1ee1f, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee21, Hi: 0x1ee22, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee24, Hi: 0x1ee24, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee27, Hi: 0x1ee27, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee29, Hi: 0x1ee32, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee34, Hi: 0x1ee37, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee39, Hi: 0x1ee39, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee3b, Hi: 0x1ee3b, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee42, Hi: 0x1ee42, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee47, Hi: 0x1ee47, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee49, Hi: 0x1ee49, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee4b, Hi: 0x1ee4b, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee4d, Hi: 0x1ee4f, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee51, Hi: 0x1ee52, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee54, Hi: 0x1ee54, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee57, Hi: 0x1ee57, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee59, Hi: 0x1ee59, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee5b, Hi: 0x1ee5b, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee5d, Hi: 0x1ee5d, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee5f, Hi: 0x1ee5f, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee61, Hi: 0x1ee62, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee64, Hi: 0x1ee64, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee67, Hi: 0x1ee6a, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee6c, Hi: 0x1ee72, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee74, Hi: 0x1ee77, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee79, Hi: 0x1ee7c, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee7e, Hi: 0x1ee7e, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee80, Hi: 0x1ee89, Stride: 0x1},
		unicode.Range32{Lo: 0x1ee8b, Hi: 0x1ee9b, Stride: 0x1},
		unicode.Range32{Lo: 0x1eea1, Hi: 0x1eea3, Stride: 0x1},
		unicode.Range32{Lo: 0x1eea5, Hi: 0x1eea9, Stride: 0x1},
		unicode.Range32{Lo: 0x1eeab, Hi: 0x1eebb, Stride: 0x1},
		unicode.Range32{Lo: 0x20000, Hi: 0x2a6df, Stride: 0x1},
		unicode.Range32{Lo: 0x2a700, Hi: 0x2b739, Stride: 0x1},
		unicode.Range32{Lo: 0x2b740, Hi: 0x2b81d, Stride: 0x1},
		unicode.Range32{Lo: 0x2b820, Hi: 0x2cea1, Stride: 0x1},
		unicode.Range32{Lo: 0x2ceb0, Hi: 0x2ebe0, Stride: 0x1},
		unicode.Range32{Lo: 0x2f800, Hi: 0x2fa1d, Stride: 0x1},
		unicode.Range32{Lo: 0x30000, Hi: 0x3134a, Stride: 0x1},
		unicode.Range32{Lo: 0x31350, Hi: 0x323af, Stride: 0x1},
	},
	LatinOffset: 0,
}

var _SentenceSContinue = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x2c, Hi: 0x2c, Stride: 0x1},
		unicode.Range16{Lo: 0x2d, Hi: 0x2d, Stride: 0x1},
		unicode.Range16{Lo: 0x3a, Hi: 0x3a, Stride: 0x1},
		unicode.Range16{Lo: 0x55d, Hi: 0x55d, Stride: 0x1},
		unicode.Range16{Lo: 0x60c, Hi: 0x60d, Stride: 0x1},
		unicode.Range16{Lo: 0x7f8, Hi: 0x7f8, Stride: 0x1},
		unicode.Range16{Lo: 0x1802, Hi: 0x1802, Stride: 0x1},
		unicode.Range16{Lo: 0x1808, Hi: 0x1808, Stride: 0x1},
		unicode.Range16{Lo: 0x2013, Hi: 0x2014, Stride: 0x1},
		unicode.Range16{Lo: 0x3001, Hi: 0x3001, Stride: 0x1},
		unicode.Range16{Lo: 0xfe10, Hi: 0xfe11, Stride: 0x1},
		unicode.Range16{Lo: 0xfe13, Hi: 0xfe13, Stride: 0x1},
		unicode.Range16{Lo: 0xfe31, Hi: 0xfe32, Stride: 0x1},
		unicode.Range16{Lo: 0xfe50, Hi: 0xfe51, Stride: 0x1},
		unicode.Range16{Lo: 0xfe55, Hi: 0xfe55, Stride: 0x1},
		unicode.Range16{Lo: 0xfe58, Hi: 0xfe58, Stride: 0x1},
		unicode.Range16{Lo: 0xfe63, Hi: 0xfe63, Stride: 0x1},
		unicode.Range16{Lo: 0xff0c, Hi: 0xff0c, Stride: 0x1},
		unicode.Range16{Lo: 0xff0d, Hi: 0xff0d, Stride: 0x1},
		unicode.Range16{Lo: 0xff1a, Hi: 0xff1a, Stride: 0x1},
		unicode.Range16{Lo: 0xff64, Hi: 0xff64, Stride: 0x1},
	},
	LatinOffset: 3,
}

var _SentenceSTerm = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x21, Hi: 0x21, Stride: 0x1},
		unicode.Range16{Lo: 0x3f, Hi: 0x3f, Stride: 0x1},
		unicode.Range16{Lo: 0x589, Hi: 0x589, Stride: 0x1},
		unicode.Range16{Lo: 0x61d, Hi: 0x61f, Stride: 0x1},
		unicode.Range16{Lo: 0x6d4, Hi: 0x6d4, Stride: 0x1},
		unicode.Range16{Lo: 0x700, Hi: 0x702, Stride: 0x1},
		unicode.Range16{Lo: 0x7f9, Hi: 0x7f9, Stride: 0x1},
		unicode.Range16{Lo: 0x837, Hi: 0x837, Stride: 0x1},
		unicode.Range16{Lo: 0x839, Hi: 0x839, Stride: 0x1},
		unicode.Range16{Lo: 0x83d, Hi: 0x83e, Stride: 0x1},
		unicode.Range16{Lo: 0x964, Hi: 0x965, Stride: 0x1},
		unicode.Range16{Lo: 0x104a, Hi: 0x104b, Stride: 0x1},
		unicode.Range16{Lo: 0x1362, Hi: 0x1362, Stride: 0x1},
		unicode.Range16{Lo: 0x1367, Hi: 0x1368, Stride: 0x1},
		unicode.Range16{Lo: 0x166e, Hi: 0x166e, Stride: 0x1},
		unicode.Range16{Lo: 0x1735, Hi: 0x1736, Stride: 0x1},
		unicode.Range16{Lo: 0x1803, Hi: 0x1803, Stride: 0x1},
		unicode.Range16{Lo: 0x1809, Hi: 0x1809, Stride: 0x1},
		unicode.Range16{Lo: 0x1944, Hi: 0x1945, Stride: 0x1},
		unicode.Range16{Lo: 0x1aa8, Hi: 0x1aab, Stride: 0x1},
		unicode.Range16{Lo: 0x1b5a, Hi: 0x1b5b, Stride: 0x1},
		unicode.Range16{Lo: 0x1b5e, Hi: 0x1b5f, Stride: 0x1},
		unicode.Range16{Lo: 0x1b7d, Hi: 0x1b7e, Stride: 0x1},
		unicode.Range16{Lo: 0x1c3b, Hi: 0x1c3c, Stride: 0x1},
		unicode.Range16{Lo: 0x1c7e, Hi: 0x1c7f, Stride: 0x1},
		unicode.Range16{Lo: 0x203c, Hi: 0x203d, Stride: 0x1},
		unicode.Range16{Lo: 0x2047, Hi: 0x2049, Stride: 0x1},
		unicode.Range16{Lo: 0x2e2e, Hi: 0x2e2e, Stride: 0x1},
		unicode.Range16{Lo: 0x2e3c, Hi: 0x2e3c, Stride: 0x1},
		unicode.Range16{Lo: 0x2e53, Hi: 0x2e54, Stride: 0x1},
		unicode.Range16{Lo: 0x3002, Hi: 0x3002, Stride: 0x1},
		unicode.Range16{Lo: 0xa4ff, Hi: 0xa4ff, Stride: 0x1},
		unicode.Range16{Lo: 0xa60e, Hi: 0xa60f, Stride: 0x1},
		unicode.Range16{Lo: 0xa6f3, Hi: 0xa6f3, Stride: 0x1},
		unicode.Range16{Lo: 0xa6f7, Hi: 0xa6f7, Stride: 0x1},
		unicode.Range16{Lo: 0xa876, Hi: 0xa877, Stride: 0x1},
		unicode.Range16{Lo: 0xa8ce, Hi: 0xa8cf, Stride: 0x1},
		unicode.Range16{Lo: 0xa92f, Hi: 0xa92f, Stride: 0x1},
		unicode.Range16{Lo: 0xa9c8, Hi: 0xa9c9, Stride: 0x1},
		unicode.Range16{Lo: 0xaa5d, Hi: 0xaa5f, Stride: 0x1},
		unicode.Range16{Lo: 0xaaf0, Hi: 0xaaf1, Stride: 0x1},
		unicode.Range16{Lo: 0xabeb, Hi: 0xabeb, Stride: 0x1},
		unicode.Range16{Lo: 0xfe56, Hi: 0xfe57, Stride: 0x1},
		unicode.Range16{Lo: 0xff01, Hi: 0xff01, Stride: 0x1},
		unicode.Range16{Lo: 0xff1f, Hi: 0xff1f, Stride: 0x1},
		unicode.Range16{Lo: 0xff61, Hi: 0xff61, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x10a56, Hi: 0x10a57, Stride: 0x1},
		unicode.Range32{Lo: 0x10f55, Hi: 0x10f59, Stride: 0x1},
		unicode.Range32{Lo: 0x10f86, Hi: 0x10f89, Stride: 0x1},
		unicode.Range32{Lo: 0x11047, Hi: 0x11048, Stride: 0x1},
		unicode.Range32{Lo: 0x110be, Hi: 0x110c1, Stride: 0x1},
		unicode.Range32{Lo: 0x11141, Hi: 0x11143, Stride: 0x1},
		unicode.Range32{Lo: 0x111c5, Hi: 0x111c6, Stride: 0x1},
		unicode.Range32{Lo: 0x111cd, Hi: 0x111cd, Stride: 0x1},
		unicode.Range32{Lo: 0x111de, Hi: 0x111df, Stride: 0x1},
		unicode.Range32{Lo: 0x11238, Hi: 0x11239, Stride: 0x1},
		unicode.Range32{Lo: 0x1123b, Hi: 0x1123c, Stride: 0x1},
		unicode.Range32{Lo: 0x112a9, Hi: 0x112a9, Stride: 0x1},
		unicode.Range32{Lo: 0x1144b, Hi: 0x1144c, Stride: 0x1},
		unicode.Range32{Lo: 0x115c2, Hi: 0x115c3, Stride: 0x1},
		unicode.Range32{Lo: 0x115c9, Hi: 0x115d7, Stride: 0x1},
		unicode.Range32{Lo: 0x11641, Hi: 0x11642, Stride: 0x1},
		unicode.Range32{Lo: 0x1173c, Hi: 0x1173e, Stride: 0x1},
		unicode.Range32{Lo: 0x11944, Hi: 0x11944, Stride: 0x1},
		unicode.Range32{Lo: 0x11946, Hi: 0x11946, Stride: 0x1},
		unicode.Range32{Lo: 0x11a42, Hi: 0x11a43, Stride: 0x1},
		unicode.Range32{Lo: 0x11a9b, Hi: 0x11a9c, Stride: 0x1},
		unicode.Range32{Lo: 0x11c41, Hi: 0x11c42, Stride: 0x1},
		unicode.Range32{Lo: 0x11ef7, Hi: 0x11ef8, Stride: 0x1},
		unicode.Range32{Lo: 0x11f43, Hi: 0x11f44, Stride: 0x1},
		unicode.Range32{Lo: 0x16a6e, Hi: 0x16a6f, Stride: 0x1},
		unicode.Range32{Lo: 0x16af5, Hi: 0x16af5, Stride: 0x1},
		unicode.Range32{Lo: 0x16b37, Hi: 0x16b38, Stride: 0x1},
		unicode.Range32{Lo: 0x16b44, Hi: 0x16b44, Stride: 0x1},
		unicode.Range32{Lo: 0x16e98, Hi: 0x16e98, Stride: 0x1},
		unicode.Range32{Lo: 0x1bc9f, Hi: 0x1bc9f, Stride: 0x1},
		unicode.Range32{Lo: 0x1da88, Hi: 0x1da88, Stride: 0x1},
	},
	LatinOffset: 2,
}

var _SentenceSep = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x85, Hi: 0x85, Stride: 0x1},
		unicode.Range16{Lo: 0x2028, Hi: 0x2028, Stride: 0x1},
		unicode.Range16{Lo: 0x2029, Hi: 0x2029, Stride: 0x1},
	},
	LatinOffset: 1,
}

var _SentenceSp = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x9, Hi: 0x9, Stride: 0x1},
		unicode.Range16{Lo: 0xb, Hi: 0xc, Stride: 0x1},
		unicode.Range16{Lo: 0x20, Hi: 0x20, Stride: 0x1},
		unicode.Range16{Lo: 0xa0, Hi: 0xa0, Stride: 0x1},
		unicode.Range16{Lo: 0x1680, Hi: 0x1680, Stride: 0x1},
		unicode.Range16{Lo: 0x2000, Hi: 0x200a, Stride: 0x1},
		unicode.Range16{Lo: 0x202f, Hi: 0x202f, Stride: 0x1},
		unicode.Range16{Lo: 0x205f, Hi: 0x205f, Stride: 0x1},
		unicode.Range16{Lo: 0x3000, Hi: 0x3000, Stride: 0x1},
	},
	LatinOffset: 4,
}

var _SentenceUpper = &unicode.RangeTable{
	R16: []unicode.Range16{
		unicode.Range16{Lo: 0x41, Hi: 0x5a, Stride: 0x1},
		unicode.Range16{Lo: 0xc0, Hi: 0xd6, Stride: 0x1},
		unicode.Range16{Lo: 0xd8, Hi: 0xde, Stride: 0x1},
		unicode.Range16{Lo: 0x100, Hi: 0x100, Stride: 0x1},
		unicode.Range16{Lo: 0x102, Hi: 0x102, Stride: 0x1},
		unicode.Range16{Lo: 0x104, Hi: 0x104, Stride: 0x1},
		unicode.Range16{Lo: 0x106, Hi: 0x106, Stride: 0x1},
		unicode.Range16{Lo: 0x108, Hi: 0x108, Stride: 0x1},
		unicode.Range16{Lo: 0x10a, Hi: 0x10a, Stride: 0x1},
		unicode.Range16{Lo: 0x10c, Hi: 0x10c, Stride: 0x1},
		unicode.Range16{Lo: 0x10e, Hi: 0x10e, Stride: 0x1},
		unicode.Range16{Lo: 0x110, Hi: 0x110, Stride: 0x1},
		unicode.Range16{Lo: 0x112, Hi: 0x112, Stride: 0x1},
		unicode.Range16{Lo: 0x114, Hi: 0x114, Stride: 0x1},
		unicode.Range16{Lo: 0x116, Hi: 0x116, Stride: 0x1},
		unicode.Range16{Lo: 0x118, Hi: 0x118, Stride: 0x1},
		unicode.Range16{Lo: 0x11a, Hi: 0x11a, Stride: 0x1},
		unicode.Range16{Lo: 0x11c, Hi: 0x11c, Stride: 0x1},
		unicode.Range16{Lo: 0x11e, Hi: 0x11e, Stride: 0x1},
		unicode.Range16{Lo: 0x120, Hi: 0x120, Stride: 0x1},
		unicode.Range16{Lo: 0x122, Hi: 0x122, Stride: 0x1},
		unicode.Range16{Lo: 0x124, Hi: 0x124, Stride: 0x1},
		unicode.Range16{Lo: 0x126, Hi: 0x126, Stride: 0x1},
		unicode.Range16{Lo: 0x128, Hi: 0x128, Stride: 0x1},
		unicode.Range16{Lo: 0x12a, Hi: 0x12a, Stride: 0x1},
		unicode.Range16{Lo: 0x12c, Hi: 0x12c, Stride: 0x1},
		unicode.Range16{Lo: 0x12e, Hi: 0x12e, Stride: 0x1},
		unicode.Range16{Lo: 0x130, Hi: 0x130, Stride: 0x1},
		unicode.Range16{Lo: 0x132, Hi: 0x132, Stride: 0x1},
		unicode.Range16{Lo: 0x134, Hi: 0x134, Stride: 0x1},
		unicode.Range16{Lo: 0x136, Hi: 0x136, Stride: 0x1},
		unicode.Range16{Lo: 0x139, Hi: 0x139, Stride: 0x1},
		unicode.Range16{Lo: 0x13b, Hi: 0x13b, Stride: 0x1},
		unicode.Range16{Lo: 0x13d, Hi: 0x13d, Stride: 0x1},
		unicode.Range16{Lo: 0x13f, Hi: 0x13f, Stride: 0x1},
		unicode.Range16{Lo: 0x141, Hi: 0x141, Stride: 0x1},
		unicode.Range16{Lo: 0x143, Hi: 0x143, Stride: 0x1},
		unicode.Range16{Lo: 0x145, Hi: 0x145, Stride: 0x1},
		unicode.Range16{Lo: 0x147, Hi: 0x147, Stride: 0x1},
		unicode.Range16{Lo: 0x14a, Hi: 0x14a, Stride: 0x1},
		unicode.Range16{Lo: 0x14c, Hi: 0x14c, Stride: 0x1},
		unicode.Range16{Lo: 0x14e, Hi: 0x14e, Stride: 0x1},
		unicode.Range16{Lo: 0x150, Hi: 0x150, Stride: 0x1},
		unicode.Range16{Lo: 0x152, Hi: 0x152, Stride: 0x1},
		unicode.Range16{Lo: 0x154, Hi: 0x154, Stride: 0x1},
		unicode.Range16{Lo: 0x156, Hi: 0x156, Stride: 0x1},
		unicode.Range16{Lo: 0x158, Hi: 0x158, Stride: 0x1},
		unicode.Range16{Lo: 0x15a, Hi: 0x15a, Stride: 0x1},
		unicode.Range16{Lo: 0x15c, Hi: 0x15c, Stride: 0x1},
		unicode.Range16{Lo: 0x15e, Hi: 0x15e, Stride: 0x1},
		unicode.Range16{Lo: 0x160, Hi: 0x160, Stride: 0x1},
		unicode.Range16{Lo: 0x162, Hi: 0x162, Stride: 0x1},
		unicode.Range16{Lo: 0x164, Hi: 0x164, Stride: 0x1},
		unicode.Range16{Lo: 0x166, Hi: 0x166, Stride: 0x1},
		unicode.Range16{Lo: 0x168, Hi: 0x168, Stride: 0x1},
		unicode.Range16{Lo: 0x16a, Hi: 0x16a, Stride: 0x1},
		unicode.Range16{Lo: 0x16c, Hi: 0x16c, Stride: 0x1},
		unicode.Range16{Lo: 0x16e, Hi: 0x16e, Stride: 0x1},
		unicode.Range16{Lo: 0x170, Hi: 0x170, Stride: 0x1},
		unicode.Range16{Lo: 0x172, Hi: 0x172, Stride: 0x1},
		unicode.Range16{Lo: 0x174, Hi: 0x174, Stride: 0x1},
		unicode.Range16{Lo: 0x176, Hi: 0x176, Stride: 0x1},
		unicode.Range16{Lo: 0x178, Hi: 0x179, Stride: 0x1},
		unicode.Range16{Lo: 0x17b, Hi: 0x17b, Stride: 0x1},
		unicode.Range16{Lo: 0x17d, Hi: 0x17d, Stride: 0x1},
		unicode.Range16{Lo: 0x181, Hi: 0x182, Stride: 0x1},
		unicode.Range16{Lo: 0x184, Hi: 0x184, Stride: 0x1},
		unicode.Range16{Lo: 0x186, Hi: 0x187, Stride: 0x1},
		unicode.Range16{Lo: 0x189, Hi: 0x18b, Stride: 0x1},
		unicode.Range16{Lo: 0x18e, Hi: 0x191, Stride: 0x1},
		unicode.Range16{Lo: 0x193, Hi: 0x194, Stride: 0x1},
		unicode.Range16{Lo: 0x196, Hi: 0x198, Stride: 0x1},
		unicode.Range16{Lo: 0x19c, Hi: 0x19d, Stride: 0x1},
		unicode.Range16{Lo: 0x19f, Hi: 0x1a0, Stride: 0x1},
		unicode.Range16{Lo: 0x1a2, Hi: 0x1a2, Stride: 0x1},
		unicode.Range16{Lo: 0x1a4, Hi: 0x1a4, Stride: 0x1},
		unicode.Range16{Lo: 0x1a6, Hi: 0x1a7, Stride: 0x1},
		unicode.Range16{Lo: 0x1a9, Hi: 0x1a9, Stride: 0x1},
		unicode.Range16{Lo: 0x1ac, Hi: 0x1ac, Stride: 0x1},
		unicode.Range16{Lo: 0x1ae, Hi: 0x1af, Stride: 0x1},
		unicode.Range16{Lo: 0x1b1, Hi: 0x1b3, Stride: 0x1},
		unicode.Range16{Lo: 0x1b5, Hi: 0x1b5, Stride: 0x1},
		unicode.Range16{Lo: 0x1b7, Hi: 0x1b8, Stride: 0x1},
		unicode.Range16{Lo: 0x1bc, Hi: 0x1bc, Stride: 0x1},
		unicode.Range16{Lo: 0x1c4, Hi: 0x1c5, Stride: 0x1},
		unicode.Range16{Lo: 0x1c7, Hi: 0x1c8, Stride: 0x1},
		unicode.Range16{Lo: 0x1ca, Hi: 0x1cb, Stride: 0x1},
		unicode.Range16{Lo: 0x1cd, Hi: 0x1cd, Stride: 0x1},
		unicode.Range16{Lo: 0x1cf, Hi: 0x1cf, Stride: 0x1},
		unicode.Range16{Lo: 0x1d1, Hi: 0x1d1, Stride: 0x1},
		unicode.Range16{Lo: 0x1d3, Hi: 0x1d3, Stride: 0x1},
		unicode.Range16{Lo: 0x1d5, Hi: 0x1d5, Stride: 0x1},
		unicode.Range16{Lo: 0x1d7, Hi: 0x1d7, Stride: 0x1},
		unicode.Range16{Lo: 0x1d9, Hi: 0x1d9, Stride: 0x1},
		unicode.Range16{Lo: 0x1db, Hi: 0x1db, Stride: 0x1},
		unicode.Range16{Lo: 0x1de, Hi: 0x1de, Stride: 0x1},
		unicode.Range16{Lo: 0x1e0, Hi: 0x1e0, Stride: 0x1},
		unicode.Range16{Lo: 0x1e2, Hi: 0x1e2, Stride: 0x1},
		unicode.Range16{Lo: 0x1e4, Hi: 0x1e4, Stride: 0x1},
		unicode.Range16{Lo: 0x1e6, Hi: 0x1e6, Stride: 0x1},
		unicode.Range16{Lo: 0x1e8, Hi: 0x1e8, Stride: 0x1},
		unicode.Range16{Lo: 0x1ea, Hi: 0x1ea, Stride: 0x1},
		unicode.Range16{Lo: 0x1ec, Hi: 0x1ec, Stride: 0x1},
		unicode.Range16{Lo: 0x1ee, Hi: 0x1ee, Stride: 0x1},
		unicode.Range16{Lo: 0x1f1, Hi: 0x1f2, Stride: 0x1},
		unicode.Range16{Lo: 0x1f4, Hi: 0x1f4, Stride: 0x1},
		unicode.Range16{Lo: 0x1f6, Hi: 0x1f8, Stride: 0x1},
		unicode.Range16{Lo: 0x1fa, Hi: 0x1fa, Stride: 0x1},
		unicode.Range16{Lo: 0x1fc, Hi: 0x1fc, Stride: 0x1},
		unicode.Range16{Lo: 0x1fe, Hi: 0x1fe, Stride: 0x1},
		unicode.Range16{Lo: 0x200, Hi: 0x200, Stride: 0x1},
		unicode.Range16{Lo: 0x202, Hi: 0x202, Stride: 0x1},
		unicode.Range16{Lo: 0x204, Hi: 0x204, Stride: 0x1},
		unicode.Range16{Lo: 0x206, Hi: 0x206, Stride: 0x1},
		unicode.Range16{Lo: 0x208, Hi: 0x208, Stride: 0x1},
		unicode.Range16{Lo: 0x20a, Hi: 0x20a, Stride: 0x1},
		unicode.Range16{Lo: 0x20c, Hi: 0x20c, Stride: 0x1},
		unicode.Range16{Lo: 0x20e, Hi: 0x20e, Stride: 0x1},
		unicode.Range16{Lo: 0x210, Hi: 0x210, Stride: 0x1},
		unicode.Range16{Lo: 0x212, Hi: 0x212, Stride: 0x1},
		unicode.Range16{Lo: 0x214, Hi: 0x214, Stride: 0x1},
		unicode.Range16{Lo: 0x216, Hi: 0x216, Stride: 0x1},
		unicode.Range16{Lo: 0x218, Hi: 0x218, Stride: 0x1},
		unicode.Range16{Lo: 0x21a, Hi: 0x21a, Stride: 0x1},
		unicode.Range16{Lo: 0x21c, Hi: 0x21c, Stride: 0x1},
		unicode.Range16{Lo: 0x21e, Hi: 0x21e, Stride: 0x1},
		unicode.Range16{Lo: 0x220, Hi: 0x220, Stride: 0x1},
		unicode.Range16{Lo: 0x222, Hi: 0x222, Stride: 0x1},
		unicode.Range16{Lo: 0x224, Hi: 0x224, Stride: 0x1},
		unicode.Range16{Lo: 0x226, Hi: 0x226, Stride: 0x1},
		unicode.Range16{Lo: 0x228, Hi: 0x228, Stride: 0x1},
		unicode.Range16{Lo: 0x22a, Hi: 0x22a, Stride: 0x1},
		unicode.Range16{Lo: 0x22c, Hi: 0x22c, Stride: 0x1},
		unicode.Range16{Lo: 0x22e, Hi: 0x22e, Stride: 0x1},
		unicode.Range16{Lo: 0x230, Hi: 0x230, Stride: 0x1},
		unicode.Range16{Lo: 0x232, Hi: 0x232, Stride: 0x1},
		unicode.Range16{Lo: 0x23a, Hi: 0x23b, Stride: 0x1},
		unicode.Range16{Lo: 0x23d, Hi: 0x23e, Stride: 0x1},
		unicode.Range16{Lo: 0x241, Hi: 0x241, Stride: 0x1},
		unicode.Range16{Lo: 0x243, Hi: 0x246, Stride: 0x1},
		unicode.Range16{Lo: 0x248, Hi: 0x248, Stride: 0x1},
		unicode.Range16{Lo: 0x24a, Hi: 0x24a, Stride: 0x1},
		unicode.Range16{Lo: 0x24c, Hi: 0x24c, Stride: 0x1},
		unicode.Range16{Lo: 0x24e, Hi: 0x24e, Stride: 0x1},
		unicode.Range16{Lo: 0x370, Hi: 0x370, Stride: 0x1},
		unicode.Range16{Lo: 0x372, Hi: 0x372, Stride: 0x1},
		unicode.Range16{Lo: 0x376, Hi: 0x376, Stride: 0x1},
		unicode.Range16{Lo: 0x37f, Hi: 0x37f, Stride: 0x1},
		unicode.Range16{Lo: 0x386, Hi: 0x386, Stride: 0x1},
		unicode.Range16{Lo: 0x388, Hi: 0x38a, Stride: 0x1},
		unicode.Range16{Lo: 0x38c, Hi: 0x38c, Stride: 0x1},
		unicode.Range16{Lo: 0x38e, Hi: 0x38f, Stride: 0x1},
		unicode.Range16{Lo: 0x391, Hi: 0x3a1, Stride: 0x1},
		unicode.Range16{Lo: 0x3a3, Hi: 0x3ab, Stride: 0x1},
		unicode.Range16{Lo: 0x3cf, Hi: 0x3cf, Stride: 0x1},
		unicode.Range16{Lo: 0x3d2, Hi: 0x3d4, Stride: 0x1},
		unicode.Range16{Lo: 0x3d8, Hi: 0x3d8, Stride: 0x1},
		unicode.Range16{Lo: 0x3da, Hi: 0x3da, Stride: 0x1},
		unicode.Range16{Lo: 0x3dc, Hi: 0x3dc, Stride: 0x1},
		unicode.Range16{Lo: 0x3de, Hi: 0x3de, Stride: 0x1},
		unicode.Range16{Lo: 0x3e0, Hi: 0x3e0, Stride: 0x1},
		unicode.Range16{Lo: 0x3e2, Hi: 0x3e2, Stride: 0x1},
		unicode.Range16{Lo: 0x3e4, Hi: 0x3e4, Stride: 0x1},
		unicode.Range16{Lo: 0x3e6, Hi: 0x3e6, Stride: 0x1},
		unicode.Range16{Lo: 0x3e8, Hi: 0x3e8, Stride: 0x1},
		unicode.Range16{Lo: 0x3ea, Hi: 0x3ea, Stride: 0x1},
		unicode.Range16{Lo: 0x3ec, Hi: 0x3ec, Stride: 0x1},
		unicode.Range16{Lo: 0x3ee, Hi: 0x3ee, Stride: 0x1},
		unicode.Range16{Lo: 0x3f4, Hi: 0x3f4, Stride: 0x1},
		unicode.Range16{Lo: 0x3f7, Hi: 0x3f7, Stride: 0x1},
		unicode.Range16{Lo: 0x3f9, Hi: 0x3fa, Stride: 0x1},
		unicode.Range16{Lo: 0x3fd, Hi: 0x42f, Stride: 0x1},
		unicode.Range16{Lo: 0x460, Hi: 0x460, Stride: 0x1},
		unicode.Range16{Lo: 0x462, Hi: 0x462, Stride: 0x1},
		unicode.Range16{Lo: 0x464, Hi: 0x464, Stride: 0x1},
		unicode.Range16{Lo: 0x466, Hi: 0x466, Stride: 0x1},
		unicode.Range16{Lo: 0x468, Hi: 0x468, Stride: 0x1},
		unicode.Range16{Lo: 0x46a, Hi: 0x46a, Stride: 0x1},
		unicode.Range16{Lo: 0x46c, Hi: 0x46c, Stride: 0x1},
		unicode.Range16{Lo: 0x46e, Hi: 0x46e, Stride: 0x1},
		unicode.Range16{Lo: 0x470, Hi: 0x470, Stride: 0x1},
		unicode.Range16{Lo: 0x472, Hi: 0x472, Stride: 0x1},
		unicode.Range16{Lo: 0x474, Hi: 0x474, Stride: 0x1},
		unicode.Range16{Lo: 0x476, Hi: 0x476, Stride: 0x1},
		unicode.Range16{Lo: 0x478, Hi: 0x478, Stride: 0x1},
		unicode.Range16{Lo: 0x47a, Hi: 0x47a, Stride: 0x1},
		unicode.Range16{Lo: 0x47c, Hi: 0x47c, Stride: 0x1},
		unicode.Range16{Lo: 0x47e, Hi: 0x47e, Stride: 0x1},
		unicode.Range16{Lo: 0x480, Hi: 0x480, Stride: 0x1},
		unicode.Range16{Lo: 0x48a, Hi: 0x48a, Stride: 0x1},
		unicode.Range16{Lo: 0x48c, Hi: 0x48c, Stride: 0x1},
		unicode.Range16{Lo: 0x48e, Hi: 0x48e, Stride: 0x1},
		unicode.Range16{Lo: 0x490, Hi: 0x490, Stride: 0x1},
		unicode.Range16{Lo: 0x492, Hi: 0x492, Stride: 0x1},
		unicode.Range16{Lo: 0x494, Hi: 0x494, Stride: 0x1},
		unicode.Range16{Lo: 0x496, Hi: 0x496, Stride: 0x1},
		unicode.Range16{Lo: 0x498, Hi: 0x498, Stride: 0x1},
		unicode.Range16{Lo: 0x49a, Hi: 0x49a, Stride: 0x1},
		unicode.Range16{Lo: 0x49c, Hi: 0x49c, Stride: 0x1},
		unicode.Range16{Lo: 0x49e, Hi: 0x49e, Stride: 0x1},
		unicode.Range16{Lo: 0x4a0, Hi: 0x4a0, Stride: 0x1},
		unicode.Range16{Lo: 0x4a2, Hi: 0x4a2, Stride: 0x1},
		unicode.Range16{Lo: 0x4a4, Hi: 0x4a4, Stride: 0x1},
		unicode.Range16{Lo: 0x4a6, Hi: 0x4a6, Stride: 0x1},
		unicode.Range16{Lo: 0x4a8, Hi: 0x4a8, Stride: 0x1},
		unicode.Range16{Lo: 0x4aa, Hi: 0x4aa, Stride: 0x1},
		unicode.Range16{Lo: 0x4ac, Hi: 0x4ac, Stride: 0x1},
		unicode.Range16{Lo: 0x4ae, Hi: 0x4ae, Stride: 0x1},
		unicode.Range16{Lo: 0x4b0, Hi: 0x4b0, Stride: 0x1},
		unicode.Range16{Lo: 0x4b2, Hi: 0x4b2, Stride: 0x1},
		unicode.Range16{Lo: 0x4b4, Hi: 0x4b4, Stride: 0x1},
		unicode.Range16{Lo: 0x4b6, Hi: 0x4b6, Stride: 0x1},
		unicode.Range16{Lo: 0x4b8, Hi: 0x4b8, Stride: 0x1},
		unicode.Range16{Lo: 0x4ba, Hi: 0x4ba, Stride: 0x1},
		unicode.Range16{Lo: 0x4bc, Hi: 0x4bc, Stride: 0x1},
		unicode.Range16{Lo: 0x4be, Hi: 0x4be, Stride: 0x1},
		unicode.Range16{Lo: 0x4c0, Hi: 0x4c1, Stride: 0x1},
		unicode.Range16{Lo: 0x4c3, Hi: 0x4c3, Stride: 0x1},
		unicode.Range16{Lo: 0x4c5, Hi: 0x4c5, Stride: 0x1},
		unicode.Range16{Lo: 0x4c7, Hi: 0x4c7, Stride: 0x1},
		unicode.Range16{Lo: 0x4c9, Hi: 0x4c9, Stride: 0x1},
		unicode.Range16{Lo: 0x4cb, Hi: 0x4cb, Stride: 0x1},
		unicode.Range16{Lo: 0x4cd, Hi: 0x4cd, Stride: 0x1},
		unicode.Range16{Lo: 0x4d0, Hi: 0x4d0, Stride: 0x1},
		unicode.Range16{Lo: 0x4d2, Hi: 0x4d2, Stride: 0x1},
		unicode.Range16{Lo: 0x4d4, Hi: 0x4d4, Stride: 0x1},
		unicode.Range16{Lo: 0x4d6, Hi: 0x4d6, Stride: 0x1},
		unicode.Range16{Lo: 0x4d8, Hi: 0x4d8, Stride: 0x1},
		unicode.Range16{Lo: 0x4da, Hi: 0x4da, Stride: 0x1},
		unicode.Range16{Lo: 0x4dc, Hi: 0x4dc, Stride: 0x1},
		unicode.Range16{Lo: 0x4de, Hi: 0x4de, Stride: 0x1},
		unicode.Range16{Lo: 0x4e0, Hi: 0x4e0, Stride: 0x1},
		unicode.Range16{Lo: 0x4e2, Hi: 0x4e2, Stride: 0x1},
		unicode.Range16{Lo: 0x4e4, Hi: 0x4e4, Stride: 0x1},
		unicode.Range16{Lo: 0x4e6, Hi: 0x4e6, Stride: 0x1},
		unicode.Range16{Lo: 0x4e8, Hi: 0x4e8, Stride: 0x1},
		unicode.Range16{Lo: 0x4ea, Hi: 0x4ea, Stride: 0x1},
		unicode.Range16{Lo: 0x4ec, Hi: 0x4ec, Stride: 0x1},
		unicode.Range16{Lo: 0x4ee, Hi: 0x4ee, Stride: 0x1},
		unicode.Range16{Lo: 0x4f0, Hi: 0x4f0, Stride: 0x1},
		unicode.Range16{Lo: 0x4f2, Hi: 0x4f2, Stride: 0x1},
		unicode.Range16{Lo: 0x4f4, Hi: 0x4f4, Stride: 0x1},
		unicode.Range16{Lo: 0x4f6, Hi: 0x4f6, Stride: 0x1},
		unicode.Range16{Lo: 0x4f8, Hi: 0x4f8, Stride: 0x1},
		unicode.Range16{Lo: 0x4fa, Hi: 0x4fa, Stride: 0x1},
		unicode.Range16{Lo: 0x4fc, Hi: 0x4fc, Stride: 0x1},
		unicode.Range16{Lo: 0x4fe, Hi: 0x4fe, Stride: 0x1},
		unicode.Range16{Lo: 0x500, Hi: 0x500, Stride: 0x1},
		unicode.Range16{Lo: 0x502, Hi: 0x502, Stride: 0x1},
		unicode.Range16{Lo: 0x504, Hi: 0x504, Stride: 0x1},
		unicode.Range16{Lo: 0x506, Hi: 0x506, Stride: 0x1},
		unicode.Range16{Lo: 0x508, Hi: 0x508, Stride: 0x1},
		unicode.Range16{Lo: 0x50a, Hi: 0x50a, Stride: 0x1},
		unicode.Range16{Lo: 0x50c, Hi: 0x50c, Stride: 0x1},
		unicode.Range16{Lo: 0x50e, Hi: 0x50e, Stride: 0x1},
		unicode.Range16{Lo: 0x510, Hi: 0x510, Stride: 0x1},
		unicode.Range16{Lo: 0x512, Hi: 0x512, Stride: 0x1},
		unicode.Range16{Lo: 0x514, Hi: 0x514, Stride: 0x1},
		unicode.Range16{Lo: 0x516, Hi: 0x516, Stride: 0x1},
		unicode.Range16{Lo: 0x518, Hi: 0x518, Stride: 0x1},
		unicode.Range16{Lo: 0x51a, Hi: 0x51a, Stride: 0x1},
		unicode.Range16{Lo: 0x51c, Hi: 0x51c, Stride: 0x1},
		unicode.Range16{Lo: 0x51e, Hi: 0x51e, Stride: 0x1},
		unicode.Range16{Lo: 0x520, Hi: 0x520, Stride: 0x1},
		unicode.Range16{Lo: 0x522, Hi: 0x522, Stride: 0x1},
		unicode.Range16{Lo: 0x524, Hi: 0x524, Stride: 0x1},
		unicode.Range16{Lo: 0x526, Hi: 0x526, Stride: 0x1},
		unicode.Range16{Lo: 0x528, Hi: 0x528, Stride: 0x1},
		unicode.Range16{Lo: 0x52a, Hi: 0x52a, Stride: 0x1},
		unicode.Range16{Lo: 0x52c, Hi: 0x52c, Stride: 0x1},
		unicode.Range16{Lo: 0x52e, Hi: 0x52e, Stride: 0x1},
		unicode.Range16{Lo: 0x531, Hi: 0x556, Stride: 0x1},
		unicode.Range16{Lo: 0x10a0, Hi: 0x10c5, Stride: 0x1},
		unicode.Range16{Lo: 0x10c7, Hi: 0x10c7, Stride: 0x1},
		unicode.Range16{Lo: 0x10cd, Hi: 0x10cd, Stride: 0x1},
		unicode.Range16{Lo: 0x13a0, Hi: 0x13f5, Stride: 0x1},
		unicode.Range16{Lo: 0x1e00, Hi: 0x1e00, Stride: 0x1},
		unicode.Range16{Lo: 0x1e02, Hi: 0x1e02, Stride: 0x1},
		unicode.Range16{Lo: 0x1e04, Hi: 0x1e04, Stride: 0x1},
		unicode.Range16{Lo: 0x1e06, Hi: 0x1e06, Stride: 0x1},
		unicode.Range16{Lo: 0x1e08, Hi: 0x1e08, Stride: 0x1},
		unicode.Range16{Lo: 0x1e0a, Hi: 0x1e0a, Stride: 0x1},
		unicode.Range16{Lo: 0x1e0c, Hi: 0x1e0c, Stride: 0x1},
		unicode.Range16{Lo: 0x1e0e, Hi: 0x1e0e, Stride: 0x1},
		unicode.Range16{Lo: 0x1e10, Hi: 0x1e10, Stride: 0x1},
		unicode.Range16{Lo: 0x1e12, Hi: 0x1e12, Stride: 0x1},
		unicode.Range16{Lo: 0x1e14, Hi: 0x1e14, Stride: 0x1},
		unicode.Range16{Lo: 0x1e16, Hi: 0x1e16, Stride: 0x1},
		unicode.Range16{Lo: 0x1e18, Hi: 0x1e18, Stride: 0x1},
		unicode.Range16{Lo: 0x1e1a, Hi: 0x1e1a, Stride: 0x1},
		unicode.Range16{Lo: 0x1e1c, Hi: 0x1e1c, Stride: 0x1},
		unicode.Range16{Lo: 0x1e1e, Hi: 0x1e1e, Stride: 0x1},
		unicode.Range16{Lo: 0x1e20, Hi: 0x1e20, Stride: 0x1},
		unicode.Range16{Lo: 0x1e22, Hi: 0x1e22, Stride: 0x1},
		unicode.Range16{Lo: 0x1e24, Hi: 0x1e24, Stride: 0x1},
		unicode.Range16{Lo: 0x1e26, Hi: 0x1e26, Stride: 0x1},
		unicode.Range16{Lo: 0x1e28, Hi: 0x1e28, Stride: 0x1},
		unicode.Range16{Lo: 0x1e2a, Hi: 0x1e2a, Stride: 0x1},
		unicode.Range16{Lo: 0x1e2c, Hi: 0x1e2c, Stride: 0x1},
		unicode.Range16{Lo: 0x1e2e, Hi: 0x1e2e, Stride: 0x1},
		unicode.Range16{Lo: 0x1e30, Hi: 0x1e30, Stride: 0x1},
		unicode.Range16{Lo: 0x1e32, Hi: 0x1e32, Stride: 0x1},
		unicode.Range16{Lo: 0x1e34, Hi: 0x1e34, Stride: 0x1},
		unicode.Range16{Lo: 0x1e36, Hi: 0x1e36, Stride: 0x1},
		unicode.Range16{Lo: 0x1e38, Hi: 0x1e38, Stride: 0x1},
		unicode.Range16{Lo: 0x1e3a, Hi: 0x1e3a, Stride: 0x1},
		unicode.Range16{Lo: 0x1e3c, Hi: 0x1e3c, Stride: 0x1},
		unicode.Range16{Lo: 0x1e3e, Hi: 0x1e3e, Stride: 0x1},
		unicode.Range16{Lo: 0x1e40, Hi: 0x1e40, Stride: 0x1},
		unicode.Range16{Lo: 0x1e42, Hi: 0x1e42, Stride: 0x1},
		unicode.Range16{Lo: 0x1e44, Hi: 0x1e44, Stride: 0x1},
		unicode.Range16{Lo: 0x1e46, Hi: 0x1e46, Stride: 0x1},
		unicode.Range16{Lo: 0x1e48, Hi: 0x1e48, Stride: 0x1},
		unicode.Range16{Lo: 0x1e4a, Hi: 0x1e4a, Stride: 0x1},
		unicode.Range16{Lo: 0x1e4c, Hi: 0x1e4c, Stride: 0x1},
		unicode.Range16{Lo: 0x1e4e, Hi: 0x1e4e, Stride: 0x1},
		unicode.Range16{Lo: 0x1e50, Hi: 0x1e50, Stride: 0x1},
		unicode.Range16{Lo: 0x1e52, Hi: 0x1e52, Stride: 0x1},
		unicode.Range16{Lo: 0x1e54, Hi: 0x1e54, Stride: 0x1},
		unicode.Range16{Lo: 0x1e56, Hi: 0x1e56, Stride: 0x1},
		unicode.Range16{Lo: 0x1e58, Hi: 0x1e58, Stride: 0x1},
		unicode.Range16{Lo: 0x1e5a, Hi: 0x1e5a, Stride: 0x1},
		unicode.Range16{Lo: 0x1e5c, Hi: 0x1e5c, Stride: 0x1},
		unicode.Range16{Lo: 0x1e5e, Hi: 0x1e5e, Stride: 0x1},
		unicode.Range16{Lo: 0x1e60, Hi: 0x1e60, Stride: 0x1},
		unicode.Range16{Lo: 0x1e62, Hi: 0x1e62, Stride: 0x1},
		unicode.Range16{Lo: 0x1e64, Hi: 0x1e64, Stride: 0x1},
		unicode.Range16{Lo: 0x1e66, Hi: 0x1e66, Stride: 0x1},
		unicode.Range16{Lo: 0x1e68, Hi: 0x1e68, Stride: 0x1},
		unicode.Range16{Lo: 0x1e6a, Hi: 0x1e6a, Stride: 0x1},
		unicode.Range16{Lo: 0x1e6c, Hi: 0x1e6c, Stride: 0x1},
		unicode.Range16{Lo: 0x1e6e, Hi: 0x1e6e, Stride: 0x1},
		unicode.Range16{Lo: 0x1e70, Hi: 0x1e70, Stride: 0x1},
		unicode.Range16{Lo: 0x1e72, Hi: 0x1e72, Stride: 0x1},
		unicode.Range16{Lo: 0x1e74, Hi: 0x1e74, Stride: 0x1},
		unicode.Range16{Lo: 0x1e76, Hi: 0x1e76, Stride: 0x1},
		unicode.Range16{Lo: 0x1e78, Hi: 0x1e78, Stride: 0x1},
		unicode.Range16{Lo: 0x1e7a, Hi: 0x1e7a, Stride: 0x1},
		unicode.Range16{Lo: 0x1e7c, Hi: 0x1e7c, Stride: 0x1},
		unicode.Range16{Lo: 0x1e7e, Hi: 0x1e7e, Stride: 0x1},
		unicode.Range16{Lo: 0x1e80, Hi: 0x1e80, Stride: 0x1},
		unicode.Range16{Lo: 0x1e82, Hi: 0x1e82, Stride: 0x1},
		unicode.Range16{Lo: 0x1e84, Hi: 0x1e84, Stride: 0x1},
		unicode.Range16{Lo: 0x1e86, Hi: 0x1e86, Stride: 0x1},
		unicode.Range16{Lo: 0x1e88, Hi: 0x1e88, Stride: 0x1},
		unicode.Range16{Lo: 0x1e8a, Hi: 0x1e8a, Stride: 0x1},
		unicode.Range16{Lo: 0x1e8c, Hi: 0x1e8c, Stride: 0x1},
		unicode.Range16{Lo: 0x1e8e, Hi: 0x1e8e, Stride: 0x1},
		unicode.Range16{Lo: 0x1e90, Hi: 0x1e90, Stride: 0x1},
		unicode.Range16{Lo: 0x1e92, Hi: 0x1e92, Stride: 0x1},
		unicode.Range16{Lo: 0x1e94, Hi: 0x1e94, Stride: 0x1},
		unicode.Range16{Lo: 0x1e9e, Hi: 0x1e9e, Stride: 0x1},
		unicode.Range16{Lo: 0x1ea0, Hi: 0x1ea0, Stride: 0x1},
		unicode.Range16{Lo: 0x1ea2, Hi: 0x1ea2, Stride: 0x1},
		unicode.Range16{Lo: 0x1ea4, Hi: 0x1ea4, Stride: 0x1},
		unicode.Range16{Lo: 0x1ea6, Hi: 0x1ea6, Stride: 0x1},
		unicode.Range16{Lo: 0x1ea8, Hi: 0x1ea8, Stride: 0x1},
		unicode.Range16{Lo: 0x1eaa, Hi: 0x1eaa, Stride: 0x1},
		unicode.Range16{Lo: 0x1eac, Hi: 0x1eac, Stride: 0x1},
		unicode.Range16{Lo: 0x1eae, Hi: 0x1eae, Stride: 0x1},
		unicode.Range16{Lo: 0x1eb0, Hi: 0x1eb0, Stride: 0x1},
		unicode.Range16{Lo: 0x1eb2, Hi: 0x1eb2, Stride: 0x1},
		unicode.Range16{Lo: 0x1eb4, Hi: 0x1eb4, Stride: 0x1},
		unicode.Range16{Lo: 0x1eb6, Hi: 0x1eb6, Stride: 0x1},
		unicode.Range16{Lo: 0x1eb8, Hi: 0x1eb8, Stride: 0x1},
		unicode.Range16{Lo: 0x1eba, Hi: 0x1eba, Stride: 0x1},
		unicode.Range16{Lo: 0x1ebc, Hi: 0x1ebc, Stride: 0x1},
		unicode.Range16{Lo: 0x1ebe, Hi: 0x1ebe, Stride: 0x1},
		unicode.Range16{Lo: 0x1ec0, Hi: 0x1ec0, Stride: 0x1},
		unicode.Range16{Lo: 0x1ec2, Hi: 0x1ec2, Stride: 0x1},
		unicode.Range16{Lo: 0x1ec4, Hi: 0x1ec4, Stride: 0x1},
		unicode.Range16{Lo: 0x1ec6, Hi: 0x1ec6, Stride: 0x1},
		unicode.Range16{Lo: 0x1ec8, Hi: 0x1ec8, Stride: 0x1},
		unicode.Range16{Lo: 0x1eca, Hi: 0x1eca, Stride: 0x1},
		unicode.Range16{Lo: 0x1ecc, Hi: 0x1ecc, Stride: 0x1},
		unicode.Range16{Lo: 0x1ece, Hi: 0x1ece, Stride: 0x1},
		unicode.Range16{Lo: 0x1ed0, Hi: 0x1ed0, Stride: 0x1},
		unicode.Range16{Lo: 0x1ed2, Hi: 0x1ed2, Stride: 0x1},
		unicode.Range16{Lo: 0x1ed4, Hi: 0x1ed4, Stride: 0x1},
		unicode.Range16{Lo: 0x1ed6, Hi: 0x1ed6, Stride: 0x1},
		unicode.Range16{Lo: 0x1ed8, Hi: 0x1ed8, Stride: 0x1},
		unicode.Range16{Lo: 0x1eda, Hi: 0x1eda, Stride: 0x1},
		unicode.Range16{Lo: 0x1edc, Hi: 0x1edc, Stride: 0x1},
		unicode.Range16{Lo: 0x1ede, Hi: 0x1ede, Stride: 0x1},
		unicode.Range16{Lo: 0x1ee0, Hi: 0x1ee0, Stride: 0x1},
		unicode.Range16{Lo: 0x1ee2, Hi: 0x1ee2, Stride: 0x1},
		unicode.Range16{Lo: 0x1ee4, Hi: 0x1ee4, Stride: 0x1},
		unicode.Range16{Lo: 0x1ee6, Hi: 0x1ee6, Stride: 0x1},
		unicode.Range16{Lo: 0x1ee8, Hi: 0x1ee8, Stride: 0x1},
		unicode.Range16{Lo: 0x1eea, Hi: 0x1eea, Stride: 0x1},
		unicode.Range16{Lo: 0x1eec, Hi: 0x1eec, Stride: 0x1},
		unicode.Range16{Lo: 0x1eee, Hi: 0x1eee, Stride: 0x1},
		unicode.Range16{Lo: 0x1ef0, Hi: 0x1ef0, Stride: 0x1},
		unicode.Range16{Lo: 0x1ef2, Hi: 0x1ef2, Stride: 0x1},
		unicode.Range16{Lo: 0x1ef4, Hi: 0x1ef4, Stride: 0x1},
		unicode.Range16{Lo: 0x1ef6, Hi: 0x1ef6, Stride: 0x1},
		unicode.Range16{Lo: 0x1ef8, Hi: 0x1ef8, Stride: 0x1},
		unicode.Range16{Lo: 0x1efa, Hi: 0x1efa, Stride: 0x1},
		unicode.Range16{Lo: 0x1efc, Hi: 0x1efc, Stride: 0x1},
		unicode.Range16{Lo: 0x1efe, Hi: 0x1efe, Stride: 0x1},
		unicode.Range16{Lo: 0x1f08, Hi: 0x1f0f, Stride: 0x1},
		unicode.Range16{Lo: 0x1f18, Hi: 0x1f1d, Stride: 0x1},
		unicode.Range16{Lo: 0x1f28, Hi: 0x1f2f, Stride: 0x1},
		unicode.Range16{Lo: 0x1f38, Hi: 0x1f3f, Stride: 0x1},
		unicode.Range16{Lo: 0x1f48, Hi: 0x1f4d, Stride: 0x1},
		unicode.Range16{Lo: 0x1f59, Hi: 0x1f59, Stride: 0x1},
		unicode.Range16{Lo: 0x1f5b, Hi: 0x1f5b, Stride: 0x1},
		unicode.Range16{Lo: 0x1f5d, Hi: 0x1f5d, Stride: 0x1},
		unicode.Range16{Lo: 0x1f5f, Hi: 0x1f5f, Stride: 0x1},
		unicode.Range16{Lo: 0x1f68, Hi: 0x1f6f, Stride: 0x1},
		unicode.Range16{Lo: 0x1f88, Hi: 0x1f8f, Stride: 0x1},
		unicode.Range16{Lo: 0x1f98, Hi: 0x1f9f, Stride: 0x1},
		unicode.Range16{Lo: 0x1fa8, Hi: 0x1faf, Stride: 0x1},
		unicode.Range16{Lo: 0x1fb8, Hi: 0x1fbc, Stride: 0x1},
		unicode.Range16{Lo: 0x1fc8, Hi: 0x1fcc, Stride: 0x1},
		unicode.Range16{Lo: 0x1fd8, Hi: 0x1fdb, Stride: 0x1},
		unicode.Range16{Lo: 0x1fe8, Hi: 0x1fec, Stride: 0x1},
		unicode.Range16{Lo: 0x1ff8, Hi: 0x1ffc, Stride: 0x1},
		unicode.Range16{Lo: 0x2102, Hi: 0x2102, Stride: 0x1},
		unicode.Range16{Lo: 0x2107, Hi: 0x2107, Stride: 0x1},
		unicode.Range16{Lo: 0x210b, Hi: 0x210d, Stride: 0x1},
		unicode.Range16{Lo: 0x2110, Hi: 0x2112, Stride: 0x1},
		unicode.Range16{Lo: 0x2115, Hi: 0x2115, Stride: 0x1},
		unicode.Range16{Lo: 0x2119, Hi: 0x211d, Stride: 0x1},
		unicode.Range16{Lo: 0x2124, Hi: 0x2124, Stride: 0x1},
		unicode.Range16{Lo: 0x2126, Hi: 0x2126, Stride: 0x1},
		unicode.Range16{Lo: 0x2128, Hi: 0x2128, Stride: 0x1},
		unicode.Range16{Lo: 0x212a, Hi: 0x212d, Stride: 0x1},
		unicode.Range16{Lo: 0x2130, Hi: 0x2133, Stride: 0x1},
		unicode.Range16{Lo: 0x213e, Hi: 0x213f, Stride: 0x1},
		unicode.Range16{Lo: 0x2145, Hi: 0x2145, Stride: 0x1},
		unicode.Range16{Lo: 0x2160, Hi: 0x216f, Stride: 0x1},
		unicode.Range16{Lo: 0x2183, Hi: 0x2183, Stride: 0x1},
		unicode.Range16{Lo: 0x24b6, Hi: 0x24cf, Stride: 0x1},
		unicode.Range16{Lo: 0x2c00, Hi: 0x2c2f, Stride: 0x1},
		unicode.Range16{Lo: 0x2c60, Hi: 0x2c60, Stride: 0x1},
		unicode.Range16{Lo: 0x2c62, Hi: 0x2c64, Stride: 0x1},
		unicode.Range16{Lo: 0x2c67, Hi: 0x2c67, Stride: 0x1},
		unicode.Range16{Lo: 0x2c69, Hi: 0x2c69, Stride: 0x1},
		unicode.Range16{Lo: 0x2c6b, Hi: 0x2c6b, Stride: 0x1},
		unicode.Range16{Lo: 0x2c6d, Hi: 0x2c70, Stride: 0x1},
		unicode.Range16{Lo: 0x2c72, Hi: 0x2c72, Stride: 0x1},
		unicode.Range16{Lo: 0x2c75, Hi: 0x2c75, Stride: 0x1},
		unicode.Range16{Lo: 0x2c7e, Hi: 0x2c80, Stride: 0x1},
		unicode.Range16{Lo: 0x2c82, Hi: 0x2c82, Stride: 0x1},
		unicode.Range16{Lo: 0x2c84, Hi: 0x2c84, Stride: 0x1},
		unicode.Range16{Lo: 0x2c86, Hi: 0x2c86, Stride: 0x1},
		unicode.Range16{Lo: 0x2c88, Hi: 0x2c88, Stride: 0x1},
		unicode.Range16{Lo: 0x2c8a, Hi: 0x2c8a, Stride: 0x1},
		unicode.Range16{Lo: 0x2c8c, Hi: 0x2c8c, Stride: 0x1},
		unicode.Range16{Lo: 0x2c8e, Hi: 0x2c8e, Stride: 0x1},
		unicode.Range16{Lo: 0x2c90, Hi: 0x2c90, Stride: 0x1},
		unicode.Range16{Lo: 0x2c92, Hi: 0x2c92, Stride: 0x1},
		unicode.Range16{Lo: 0x2c94, Hi: 0x2c94, Stride: 0x1},
		unicode.Range16{Lo: 0x2c96, Hi: 0x2c96, Stride: 0x1},
		unicode.Range16{Lo: 0x2c98, Hi: 0x2c98, Stride: 0x1},
		unicode.Range16{Lo: 0x2c9a, Hi: 0x2c9a, Stride: 0x1},
		unicode.Range16{Lo: 0x2c9c, Hi: 0x2c9c, Stride: 0x1},
		unicode.Range16{Lo: 0x2c9e, Hi: 0x2c9e, Stride: 0x1},
		unicode.Range16{Lo: 0x2ca0, Hi: 0x2ca0, Stride: 0x1},
		unicode.Range16{Lo: 0x2ca2, Hi: 0x2ca2, Stride: 0x1},
		unicode.Range16{Lo: 0x2ca4, Hi: 0x2ca4, Stride: 0x1},
		unicode.Range16{Lo: 0x2ca6, Hi: 0x2ca6, Stride: 0x1},
		unicode.Range16{Lo: 0x2ca8, Hi: 0x2ca8, Stride: 0x1},
		unicode.Range16{Lo: 0x2caa, Hi: 0x2caa, Stride: 0x1},
		unicode.Range16{Lo: 0x2cac, Hi: 0x2cac, Stride: 0x1},
		unicode.Range16{Lo: 0x2cae, Hi: 0x2cae, Stride: 0x1},
		unicode.Range16{Lo: 0x2cb0, Hi: 0x2cb0, Stride: 0x1},
		unicode.Range16{Lo: 0x2cb2, Hi: 0x2cb2, Stride: 0x1},
		unicode.Range16{Lo: 0x2cb4, Hi: 0x2cb4, Stride: 0x1},
		unicode.Range16{Lo: 0x2cb6, Hi: 0x2cb6, Stride: 0x1},
		unicode.Range16{Lo: 0x2cb8, Hi: 0x2cb8, Stride: 0x1},
		unicode.Range16{Lo: 0x2cba, Hi: 0x2cba, Stride: 0x1},
		unicode.Range16{Lo: 0x2cbc, Hi: 0x2cbc, Stride: 0x1},
		unicode.Range16{Lo: 0x2cbe, Hi: 0x2cbe, Stride: 0x1},
		unicode.Range16{Lo: 0x2cc0, Hi: 0x2cc0, Stride: 0x1},
		unicode.Range16{Lo: 0x2cc2, Hi: 0x2cc2, Stride: 0x1},
		unicode.Range16{Lo: 0x2cc4, Hi: 0x2cc4, Stride: 0x1},
		unicode.Range16{Lo: 0x2cc6, Hi: 0x2cc6, Stride: 0x1},
		unicode.Range16{Lo: 0x2cc8, Hi: 0x2cc8, Stride: 0x1},
		unicode.Range16{Lo: 0x2cca, Hi: 0x2cca, Stride: 0x1},
		unicode.Range16{Lo: 0x2ccc, Hi: 0x2ccc, Stride: 0x1},
		unicode.Range16{Lo: 0x2cce, Hi: 0x2cce, Stride: 0x1},
		unicode.Range16{Lo: 0x2cd0, Hi: 0x2cd0, Stride: 0x1},
		unicode.Range16{Lo: 0x2cd2, Hi: 0x2cd2, Stride: 0x1},
		unicode.Range16{Lo: 0x2cd4, Hi: 0x2cd4, Stride: 0x1},
		unicode.Range16{Lo: 0x2cd6, Hi: 0x2cd6, Stride: 0x1},
		unicode.Range16{Lo: 0x2cd8, Hi: 0x2cd8, Stride: 0x1},
		unicode.Range16{Lo: 0x2cda, Hi: 0x2cda, Stride: 0x1},
		unicode.Range16{Lo: 0x2cdc, Hi: 0x2cdc, Stride: 0x1},
		unicode.Range16{Lo: 0x2cde, Hi: 0x2cde, Stride: 0x1},
		unicode.Range16{Lo: 0x2ce0, Hi: 0x2ce0, Stride: 0x1},
		unicode.Range16{Lo: 0x2ce2, Hi: 0x2ce2, Stride: 0x1},
		unicode.Range16{Lo: 0x2ceb, Hi: 0x2ceb, Stride: 0x1},
		unicode.Range16{Lo: 0x2ced, Hi: 0x2ced, Stride: 0x1},
		unicode.Range16{Lo: 0x2cf2, Hi: 0x2cf2, Stride: 0x1},
		unicode.Range16{Lo: 0xa640, Hi: 0xa640, Stride: 0x1},
		unicode.Range16{Lo: 0xa642, Hi: 0xa642, Stride: 0x1},
		unicode.Range16{Lo: 0xa644, Hi: 0xa644, Stride: 0x1},
		unicode.Range16{Lo: 0xa646, Hi: 0xa646, Stride: 0x1},
		unicode.Range16{Lo: 0xa648, Hi: 0xa648, Stride: 0x1},
		unicode.Range16{Lo: 0xa64a, Hi: 0xa64a, Stride: 0x1},
		unicode.Range16{Lo: 0xa64c, Hi: 0xa64c, Stride: 0x1},
		unicode.Range16{Lo: 0xa64e, Hi: 0xa64e, Stride: 0x1},
		unicode.Range16{Lo: 0xa650, Hi: 0xa650, Stride: 0x1},
		unicode.Range16{Lo: 0xa652, Hi: 0xa652, Stride: 0x1},
		unicode.Range16{Lo: 0xa654, Hi: 0xa654, Stride: 0x1},
		unicode.Range16{Lo: 0xa656, Hi: 0xa656, Stride: 0x1},
		unicode.Range16{Lo: 0xa658, Hi: 0xa658, Stride: 0x1},
		unicode.Range16{Lo: 0xa65a, Hi: 0xa65a, Stride: 0x1},
		unicode.Range16{Lo: 0xa65c, Hi: 0xa65c, Stride: 0x1},
		unicode.Range16{Lo: 0xa65e, Hi: 0xa65e, Stride: 0x1},
		unicode.Range16{Lo: 0xa660, Hi: 0xa660, Stride: 0x1},
		unicode.Range16{Lo: 0xa662, Hi: 0xa662, Stride: 0x1},
		unicode.Range16{Lo: 0xa664, Hi: 0xa664, Stride: 0x1},
		unicode.Range16{Lo: 0xa666, Hi: 0xa666, Stride: 0x1},
		unicode.Range16{Lo: 0xa668, Hi: 0xa668, Stride: 0x1},
		unicode.Range16{Lo: 0xa66a, Hi: 0xa66a, Stride: 0x1},
		unicode.Range16{Lo: 0xa66c, Hi: 0xa66c, Stride: 0x1},
		unicode.Range16{Lo: 0xa680, Hi: 0xa680, Stride: 0x1},
		unicode.Range16{Lo: 0xa682, Hi: 0xa682, Stride: 0x1},
		unicode.Range16{Lo: 0xa684, Hi: 0xa684, Stride: 0x1},
		unicode.Range16{Lo: 0xa686, Hi: 0xa686, Stride: 0x1},
		unicode.Range16{Lo: 0xa688, Hi: 0xa688, Stride: 0x1},
		unicode.Range16{Lo: 0xa68a, Hi: 0xa68a, Stride: 0x1},
		unicode.Range16{Lo: 0xa68c, Hi: 0xa68c, Stride: 0x1},
		unicode.Range16{Lo: 0xa68e, Hi: 0xa68e, Stride: 0x1},
		unicode.Range16{Lo: 0xa690, Hi: 0xa690, Stride: 0x1},
		unicode.Range16{Lo: 0xa692, Hi: 0xa692, Stride: 0x1},
		unicode.Range16{Lo: 0xa694, Hi: 0xa694, Stride: 0x1},
		unicode.Range16{Lo: 0xa696, Hi: 0xa696, Stride: 0x1},
		unicode.Range16{Lo: 0xa698, Hi: 0xa698, Stride: 0x1},
		unicode.Range16{Lo: 0xa69a, Hi: 0xa69a, Stride: 0x1},
		unicode.Range16{Lo: 0xa722, Hi: 0xa722, Stride: 0x1},
		unicode.Range16{Lo: 0xa724, Hi: 0xa724, Stride: 0x1},
		unicode.Range16{Lo: 0xa726, Hi: 0xa726, Stride: 0x1},
		unicode.Range16{Lo: 0xa728, Hi: 0xa728, Stride: 0x1},
		unicode.Range16{Lo: 0xa72a, Hi: 0xa72a, Stride: 0x1},
		unicode.Range16{Lo: 0xa72c, Hi: 0xa72c, Stride: 0x1},
		unicode.Range16{Lo: 0xa72e, Hi: 0xa72e, Stride: 0x1},
		unicode.Range16{Lo: 0xa732, Hi: 0xa732, Stride: 0x1},
		unicode.Range16{Lo: 0xa734, Hi: 0xa734, Stride: 0x1},
		unicode.Range16{Lo: 0xa736, Hi: 0xa736, Stride: 0x1},
		unicode.Range16{Lo: 0xa738, Hi: 0xa738, Stride: 0x1},
		unicode.Range16{Lo: 0xa73a, Hi: 0xa73a, Stride: 0x1},
		unicode.Range16{Lo: 0xa73c, Hi: 0xa73c, Stride: 0x1},
		unicode.Range16{Lo: 0xa73e, Hi: 0xa73e, Stride: 0x1},
		unicode.Range16{Lo: 0xa740, Hi: 0xa740, Stride: 0x1},
		unicode.Range16{Lo: 0xa742, Hi: 0xa742, Stride: 0x1},
		unicode.Range16{Lo: 0xa744, Hi: 0xa744, Stride: 0x1},
		unicode.Range16{Lo: 0xa746, Hi: 0xa746, Stride: 0x1},
		unicode.Range16{Lo: 0xa748, Hi: 0xa748, Stride: 0x1},
		unicode.Range16{Lo: 0xa74a, Hi: 0xa74a, Stride: 0x1},
		unicode.Range16{Lo: 0xa74c, Hi: 0xa74c, Stride: 0x1},
		unicode.Range16{Lo: 0xa74e, Hi: 0xa74e, Stride: 0x1},
		unicode.Range16{Lo: 0xa750, Hi: 0xa750, Stride: 0x1},
		unicode.Range16{Lo: 0xa752, Hi: 0xa752, Stride: 0x1},
		unicode.Range16{Lo: 0xa754, Hi: 0xa754, Stride: 0x1},
		unicode.Range16{Lo: 0xa756, Hi: 0xa756, Stride: 0x1},
		unicode.Range16{Lo: 0xa758, Hi: 0xa758, Stride: 0x1},
		unicode.Range16{Lo: 0xa75a, Hi: 0xa75a, Stride: 0x1},
		unicode.Range16{Lo: 0xa75c, Hi: 0xa75c, Stride: 0x1},
		unicode.Range16{Lo: 0xa75e, Hi: 0xa75e, Stride: 0x1},
		unicode.Range16{Lo: 0xa760, Hi: 0xa760, Stride: 0x1},
		unicode.Range16{Lo: 0xa762, Hi: 0xa762, Stride: 0x1},
		unicode.Range16{Lo: 0xa764, Hi: 0xa764, Stride: 0x1},
		unicode.Range16{Lo: 0xa766, Hi: 0xa766, Stride: 0x1},
		unicode.Range16{Lo: 0xa768, Hi: 0xa768, Stride: 0x1},
		unicode.Range16{Lo: 0xa76a, Hi: 0xa76a, Stride: 0x1},
		unicode.Range16{Lo: 0xa76c, Hi: 0xa76c, Stride: 0x1},
		unicode.Range16{Lo: 0xa76e, Hi: 0xa76e, Stride: 0x1},
		unicode.Range16{Lo: 0xa779, Hi: 0xa779, Stride: 0x1},
		unicode.Range16{Lo: 0xa77b, Hi: 0xa77b, Stride: 0x1},
		unicode.Range16{Lo: 0xa77d, Hi: 0xa77e, Stride: 0x1},
		unicode.Range16{Lo: 0xa780, Hi: 0xa780, Stride: 0x1},
		unicode.Range16{Lo: 0xa782, Hi: 0xa782, Stride: 0x1},
		unicode.Range16{Lo: 0xa784, Hi: 0xa784, Stride: 0x1},
		unicode.Range16{Lo: 0xa786, Hi: 0xa786, Stride: 0x1},
		unicode.Range16{Lo: 0xa78b, Hi: 0xa78b, Stride: 0x1},
		unicode.Range16{Lo: 0xa78d, Hi: 0xa78d, Stride: 0x1},
		unicode.Range16{Lo: 0xa790, Hi: 0xa790, Stride: 0x1},
		unicode.Range16{Lo: 0xa792, Hi: 0xa792, Stride: 0x1},
		unicode.Range16{Lo: 0xa796, Hi: 0xa796, Stride: 0x1},
		unicode.Range16{Lo: 0xa798, Hi: 0xa798, Stride: 0x1},
		unicode.Range16{Lo: 0xa79a, Hi: 0xa79a, Stride: 0x1},
		unicode.Range16{Lo: 0xa79c, Hi: 0xa79c, Stride: 0x1},
		unicode.Range16{Lo: 0xa79e, Hi: 0xa79e, Stride: 0x1},
		unicode.Range16{Lo: 0xa7a0, Hi: 0xa7a0, Stride: 0x1},
		unicode.Range16{Lo: 0xa7a2, Hi: 0xa7a2, Stride: 0x1},
		unicode.Range16{Lo: 0xa7a4, Hi: 0xa7a4, Stride: 0x1},
		unicode.Range16{Lo: 0xa7a6, Hi: 0xa7a6, Stride: 0x1},
		unicode.Range16{Lo: 0xa7a8, Hi: 0xa7a8, Stride: 0x1},
		unicode.Range16{Lo: 0xa7aa, Hi: 0xa7ae, Stride: 0x1},
		unicode.Range16{Lo: 0xa7b0, Hi: 0xa7b4, Stride: 0x1},
		unicode.Range16{Lo: 0xa7b6, Hi: 0xa7b6, Stride: 0x1},
		unicode.Range16{Lo: 0xa7b8, Hi: 0xa7b8, Stride: 0x1},
		unicode.Range16{Lo: 0xa7ba, Hi: 0xa7ba, Stride: 0x1},
		unicode.Range16{Lo: 0xa7bc, Hi: 0xa7bc, Stride: 0x1},
		unicode.Range16{Lo: 0xa7be, Hi: 0xa7be, Stride: 0x1},
		unicode.Range16{Lo: 0xa7c0, Hi: 0xa7c0, Stride: 0x1},
		unicode.Range16{Lo: 0xa7c2, Hi: 0xa7c2, Stride: 0x1},
		unicode.Range16{Lo: 0xa7c4, Hi: 0xa7c7, Stride: 0x1},
		unicode.Range16{Lo: 0xa7c9, Hi: 0xa7c9, Stride: 0x1},
		unicode.Range16{Lo: 0xa7d0, Hi: 0xa7d0, Stride: 0x1},
		unicode.Range16{Lo: 0xa7d6, Hi: 0xa7d6, Stride: 0x1},
		unicode.Range16{Lo: 0xa7d8, Hi: 0xa7d8, Stride: 0x1},
		unicode.Range16{Lo: 0xa7f5, Hi: 0xa7f5, Stride: 0x1},
		unicode.Range16{Lo: 0xff21, Hi: 0xff3a, Stride: 0x1},
	},
	R32: []unicode.Range32{
		unicode.Range32{Lo: 0x10400, Hi: 0x10427, Stride: 0x1},
		unicode.Range32{Lo: 0x104b0, Hi: 0x104d3, Stride: 0x1},
		unicode.Range32{Lo: 0x10570, Hi: 0x1057a, Stride: 0x1},
		unicode.Range32{Lo: 0x1057c, Hi: 0x1058a, Stride: 0x1},
		unicode.Range32{Lo: 0x1058c, Hi: 0x10592, Stride: 0x1},
		unicode.Range32{Lo: 0x10594, Hi: 0x10595, Stride: 0x1},
		unicode.Range32{Lo: 0x10c80, Hi: 0x10cb2, Stride: 0x1},
		unicode.Range32{Lo: 0x118a0, Hi: 0x118bf, Stride: 0x1},
		unicode.Range32{Lo: 0x16e40, Hi: 0x16e5f, Stride: 0x1},
		unicode.Range32{Lo: 0x1d400, Hi: 0x1d419, Stride: 0x1},
		unicode.Range32{Lo: 0x1d434, Hi: 0x1d44d, Stride: 0x1},
		unicode.Range32{Lo: 0x1d468, Hi: 0x1d481, Stride: 0x1},
		unicode.Range32{Lo: 0x1d49c, Hi: 0x1d49c, Stride: 0x1},
		unicode.Range32{Lo: 0x1d49e, Hi: 0x1d49f, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4a2, Hi: 0x1d4a2, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4a5, Hi: 0x1d4a6, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4a9, Hi: 0x1d4ac, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4ae, Hi: 0x1d4b5, Stride: 0x1},
		unicode.Range32{Lo: 0x1d4d0, Hi: 0x1d4e9, Stride: 0x1},
		unicode.Range32{Lo: 0x1d504, Hi: 0x1d505, Stride: 0x1},
		unicode.Range32{Lo: 0x1d507, Hi: 0x1d50a, Stride: 0x1},
		unicode.Range32{Lo: 0x1d50d, Hi: 0x1d514, Stride: 0x1},
		unicode.Range32{Lo: 0x1d516, Hi: 0x1d51c, Stride: 0x1},
		unicode.Range32{Lo: 0x1d538, Hi: 0x1d539, Stride: 0x1},
		unicode.Range32{Lo: 0x1d53b, Hi: 0x1d53e, Stride: 0x1},
		unicode.Range32{Lo: 0x1d540, Hi: 0x1d544, Stride: 0x1},
		unicode.Range32{Lo: 0x1d546, Hi: 0x1d546, Stride: 0x1},
		unicode.Range32{Lo: 0x1d54a, Hi: 0x1d550, Stride: 0x1},
		unicode.Range32{Lo: 0x1d56c, Hi: 0x1d585, Stride: 0x1},
		unicode.Range32{Lo: 0x1d5a0, Hi: 0x1d5b9, Stride: 0x1},
		unicode.Range32{Lo: 0x1d5d4, Hi: 0x1d5ed, Stride: 0x1},
		unicode.Range32{Lo: 0x1d608, Hi: 0x1d621, Stride: 0x1},
		unicode.Range32{Lo: 0x1d63c, Hi: 0x1d655, Stride: 0x1},
		unicode.Range32{Lo: 0x1d670, Hi: 0x1d689, Stride: 0x1},
		unicode.Range32{Lo: 0x1d6a8, Hi: 0x1d6c0, Stride: 0x1},
		unicode.Range32{Lo: 0x1d6e2, Hi: 0x1d6fa, Stride: 0x1},
		unicode.Range32{Lo: 0x1d71c, Hi: 0x1d734, Stride: 0x1},
		unicode.Range32{Lo: 0x1d756, Hi: 0x1d76e, Stride: 0x1},
		unicode.Range32{Lo: 0x1d790, Hi: 0x1d7a8, Stride: 0x1},
		unicode.Range32{Lo: 0x1d7ca, Hi: 0x1d7ca, Stride: 0x1},
		unicode.Range32{Lo: 0x1e900, Hi: 0x1e921, Stride: 0x1},
		unicode.Range32{Lo: 0x1f130, Hi: 0x1f149, Stride: 0x1},
		unicode.Range32{Lo: 0x1f150, Hi: 0x1f169, Stride: 0x1},
		unicode.Range32{Lo: 0x1f170, Hi: 0x1f189, Stride: 0x1},
	},
	LatinOffset: 3,
}

type _SentenceRuneRange unicode.RangeTable

func _SentenceRuneType(r rune) *_SentenceRuneRange {
	switch {
	case unicode.Is(_SentenceATerm, r):
		return (*_SentenceRuneRange)(_SentenceATerm)
	case unicode.Is(_SentenceCR, r):
		return (*_SentenceRuneRange)(_SentenceCR)
	case unicode.Is(_SentenceClose, r):
		return (*_SentenceRuneRange)(_SentenceClose)
	case unicode.Is(_SentenceExtend, r):
		return (*_SentenceRuneRange)(_SentenceExtend)
	case unicode.Is(_SentenceFormat, r):
		return (*_SentenceRuneRange)(_SentenceFormat)
	case unicode.Is(_SentenceLF, r):
		return (*_SentenceRuneRange)(_SentenceLF)
	case unicode.Is(_SentenceLower, r):
		return (*_SentenceRuneRange)(_SentenceLower)
	case unicode.Is(_SentenceNumeric, r):
		return (*_SentenceRuneRange)(_SentenceNumeric)
	case unicode.Is(_SentenceOLetter, r):
		return (*_SentenceRuneRange)(_SentenceOLetter)
	case unicode.Is(_SentenceSContinue, r):
		return (*_SentenceRuneRange)(_SentenceSContinue)
	case unicode.Is(_SentenceSTerm, r):
		return (*_SentenceRuneRange)(_SentenceSTerm)
	case unicode.Is(_SentenceSep, r):
		return (*_SentenceRuneRange)(_SentenceSep)
	case unicode.Is(_SentenceSp, r):
		return (*_SentenceRuneRange)(_SentenceSp)
	case unicode.Is(_SentenceUpper, r):
		return (*_SentenceRuneRange)(_SentenceUpper)
	default:
		return nil
	}
}
func (rng *_SentenceRuneRange) String() string {
	switch (*unicode.RangeTable)(rng) {
	case _SentenceATerm:
		return "ATerm"
	case _SentenceCR:
		return "CR"
	case _SentenceClose:
		return "Close"
	case _SentenceExtend:
		return "Extend"
	case _SentenceFormat:
		return "Format"
	case _SentenceLF:
		return "LF"
	case _SentenceLower:
		return "Lower"
	case _SentenceNumeric:
		return "Numeric"
	case _SentenceOLetter:
		return "OLetter"
	case _SentenceSContinue:
		return "SContinue"
	case _SentenceSTerm:
		return "STerm"
	case _SentenceSep:
		return "Sep"
	case _SentenceSp:
		return "Sp"
	case _SentenceUpper:
		return "Upper"
	default:
		return "Other"
	}
}
