// Code generated by gen_data.go; DO NOT EDIT.

package licensecheck

var builtinLREs = []License{
	{ID: "0BSD", LRE: license_0BSD_lre},
	{ID: "AAL", LRE: license_AAL_lre},
	{ID: "ADSL", LRE: license_ADSL_lre},
	{ID: "AFL-1.1", LRE: license_AFL_1_1_lre},
	{ID: "AFL-1.2", LRE: license_AFL_1_2_lre},
	{ID: "AFL-2.0", LRE: license_AFL_2_0_lre},
	{ID: "AFL-2.1", LRE: license_AFL_2_1_lre},
	{ID: "AFL-3.0", LRE: license_AFL_3_0_lre},
	{ID: "AGPL-1.0", LRE: license_AGPL_1_0_lre},
	{ID: "AGPL-1.0-only", LRE: license_AGPL_1_0_only_lre},
	{ID: "AGPL-1.0-or-later", LRE: license_AGPL_1_0_or_later_lre},
	{ID: "AGPL-3.0", LRE: license_AGPL_3_0_lre},
	{ID: "AGPL-3.0-only", LRE: license_AGPL_3_0_only_lre},
	{ID: "AGPL-3.0-or-later", LRE: license_AGPL_3_0_or_later_lre},
	{ID: "AMDPLPA", LRE: license_AMDPLPA_lre},
	{ID: "AML", LRE: license_AML_lre},
	{ID: "AMPAS", LRE: license_AMPAS_lre},
	{ID: "ANTLR-PD", LRE: license_ANTLR_PD_lre},
	{ID: "APAFML", LRE: license_APAFML_lre},
	{ID: "APL-1.0", LRE: license_APL_1_0_lre},
	{ID: "APSL-1.0", LRE: license_APSL_1_0_lre},
	{ID: "APSL-1.1", LRE: license_APSL_1_1_lre},
	{ID: "APSL-1.2", LRE: license_APSL_1_2_lre},
	{ID: "APSL-2.0", LRE: license_APSL_2_0_lre},
	{ID: "Abstyles", LRE: license_Abstyles_lre},
	{ID: "Adobe-2006", LRE: license_Adobe_2006_lre},
	{ID: "Adobe-Glyph", LRE: license_Adobe_Glyph_lre},
	{ID: "Afmparse", LRE: license_Afmparse_lre},
	{ID: "Aladdin", LRE: license_Aladdin_lre},
	{ID: "Aladdin-9", LRE: license_Aladdin_9_lre},
	{ID: "Anti996", LRE: license_Anti996_lre},
	{ID: "Apache-1.0", LRE: license_Apache_1_0_lre},
	{ID: "Apache-1.1", LRE: license_Apache_1_1_lre},
	{ID: "Apache-2.0", LRE: license_Apache_2_0_lre},
	{ID: "Artistic-1.0", LRE: license_Artistic_1_0_lre},
	{ID: "Artistic-1.0-Perl", LRE: license_Artistic_1_0_Perl_lre},
	{ID: "Artistic-1.0-cl8", LRE: license_Artistic_1_0_cl8_lre},
	{ID: "Artistic-2.0", LRE: license_Artistic_2_0_lre},
	{ID: "BSD-1-Clause", LRE: license_BSD_1_Clause_lre},
	{ID: "BSD-1-Clause-Clear", LRE: license_BSD_1_Clause_Clear_lre},
	{ID: "BSD-2-Clause", LRE: license_BSD_2_Clause_lre},
	{ID: "BSD-2-Clause-Patent", LRE: license_BSD_2_Clause_Patent_lre},
	{ID: "BSD-2-Clause-Views", LRE: license_BSD_2_Clause_Views_lre},
	{ID: "BSD-3-Clause", LRE: license_BSD_3_Clause_lre},
	{ID: "BSD-3-Clause-Attribution", LRE: license_BSD_3_Clause_Attribution_lre},
	{ID: "BSD-3-Clause-Clear", LRE: license_BSD_3_Clause_Clear_lre},
	{ID: "BSD-3-Clause-LBNL", LRE: license_BSD_3_Clause_LBNL_lre},
	{ID: "BSD-3-Clause-No-Nuclear-License", LRE: license_BSD_3_Clause_No_Nuclear_License_lre},
	{ID: "BSD-3-Clause-No-Nuclear-License-2014", LRE: license_BSD_3_Clause_No_Nuclear_License_2014_lre},
	{ID: "BSD-3-Clause-No-Nuclear-Warranty", LRE: license_BSD_3_Clause_No_Nuclear_Warranty_lre},
	{ID: "BSD-3-Clause-NoTrademark", LRE: license_BSD_3_Clause_NoTrademark_lre},
	{ID: "BSD-3-Clause-Open-MPI", LRE: license_BSD_3_Clause_Open_MPI_lre},
	{ID: "BSD-4-Clause-UC", LRE: license_BSD_4_Clause_UC_lre},
	{ID: "BSD-4-Clause", LRE: license_BSD_4_Clause_lre},
	{ID: "BSD-Protection", LRE: license_BSD_Protection_lre},
	{ID: "BSD-Source-Code", LRE: license_BSD_Source_Code_lre},
	{ID: "BSL-1.0", LRE: license_BSL_1_0_lre},
	{ID: "Bahyph", LRE: license_Bahyph_lre},
	{ID: "Barr", LRE: license_Barr_lre},
	{ID: "Beerware", LRE: license_Beerware_lre},
	{ID: "BitTorrent-1.0", LRE: license_BitTorrent_1_0_lre},
	{ID: "BitTorrent-1.1", LRE: license_BitTorrent_1_1_lre},
	{ID: "BlueOak-1.0.0", LRE: license_BlueOak_1_0_0_lre},
	{ID: "Borceux", LRE: license_Borceux_lre},
	{ID: "CAL-1.0", LRE: license_CAL_1_0_lre},
	{ID: "CATOSL-1.1", LRE: license_CATOSL_1_1_lre},
	{ID: "CC-BY-1.0", LRE: license_CC_BY_1_0_lre},
	{ID: "CC-BY-2.0", LRE: license_CC_BY_2_0_lre},
	{ID: "CC-BY-2.5", LRE: license_CC_BY_2_5_lre},
	{ID: "CC-BY-3.0", LRE: license_CC_BY_3_0_lre},
	{ID: "CC-BY-3.0-AT", LRE: license_CC_BY_3_0_AT_lre},
	{ID: "CC-BY-4.0", LRE: license_CC_BY_4_0_lre},
	{ID: "CC-BY-NC-1.0", LRE: license_CC_BY_NC_1_0_lre},
	{ID: "CC-BY-NC-2.0", LRE: license_CC_BY_NC_2_0_lre},
	{ID: "CC-BY-NC-2.5", LRE: license_CC_BY_NC_2_5_lre},
	{ID: "CC-BY-NC-3.0", LRE: license_CC_BY_NC_3_0_lre},
	{ID: "CC-BY-NC-4.0", LRE: license_CC_BY_NC_4_0_lre},
	{ID: "CC-BY-NC-ND-1.0", LRE: license_CC_BY_NC_ND_1_0_lre},
	{ID: "CC-BY-NC-ND-2.0", LRE: license_CC_BY_NC_ND_2_0_lre},
	{ID: "CC-BY-NC-ND-2.5", LRE: license_CC_BY_NC_ND_2_5_lre},
	{ID: "CC-BY-NC-ND-3.0", LRE: license_CC_BY_NC_ND_3_0_lre},
	{ID: "CC-BY-NC-ND-3.0-IGO", LRE: license_CC_BY_NC_ND_3_0_IGO_lre},
	{ID: "CC-BY-NC-ND-4.0", LRE: license_CC_BY_NC_ND_4_0_lre},
	{ID: "CC-BY-NC-SA-1.0", LRE: license_CC_BY_NC_SA_1_0_lre},
	{ID: "CC-BY-NC-SA-2.0", LRE: license_CC_BY_NC_SA_2_0_lre},
	{ID: "CC-BY-NC-SA-2.5", LRE: license_CC_BY_NC_SA_2_5_lre},
	{ID: "CC-BY-NC-SA-3.0", LRE: license_CC_BY_NC_SA_3_0_lre},
	{ID: "CC-BY-NC-SA-3.0-US", LRE: license_CC_BY_NC_SA_3_0_US_lre},
	{ID: "CC-BY-NC-SA-4.0", LRE: license_CC_BY_NC_SA_4_0_lre},
	{ID: "CC-BY-ND-1.0", LRE: license_CC_BY_ND_1_0_lre},
	{ID: "CC-BY-ND-2.0", LRE: license_CC_BY_ND_2_0_lre},
	{ID: "CC-BY-ND-2.5", LRE: license_CC_BY_ND_2_5_lre},
	{ID: "CC-BY-ND-3.0", LRE: license_CC_BY_ND_3_0_lre},
	{ID: "CC-BY-ND-4.0", LRE: license_CC_BY_ND_4_0_lre},
	{ID: "CC-BY-SA-1.0", LRE: license_CC_BY_SA_1_0_lre},
	{ID: "CC-BY-SA-2.0", LRE: license_CC_BY_SA_2_0_lre},
	{ID: "CC-BY-SA-2.5", LRE: license_CC_BY_SA_2_5_lre},
	{ID: "CC-BY-SA-3.0", LRE: license_CC_BY_SA_3_0_lre},
	{ID: "CC-BY-SA-3.0-AT", LRE: license_CC_BY_SA_3_0_AT_lre},
	{ID: "CC-BY-SA-4.0", LRE: license_CC_BY_SA_4_0_lre},
	{ID: "CC-PDDC", LRE: license_CC_PDDC_lre},
	{ID: "CC0-1.0", LRE: license_CC0_1_0_lre},
	{ID: "CDDL-1.0", LRE: license_CDDL_1_0_lre},
	{ID: "CDDL-1.1", LRE: license_CDDL_1_1_lre},
	{ID: "CDLA-Permissive-1.0", LRE: license_CDLA_Permissive_1_0_lre},
	{ID: "CDLA-Sharing-1.0", LRE: license_CDLA_Sharing_1_0_lre},
	{ID: "CECILL-1.0", LRE: license_CECILL_1_0_lre},
	{ID: "CECILL-1.1", LRE: license_CECILL_1_1_lre},
	{ID: "CECILL-2.0", LRE: license_CECILL_2_0_lre},
	{ID: "CECILL-2.1", LRE: license_CECILL_2_1_lre},
	{ID: "CECILL-B", LRE: license_CECILL_B_lre},
	{ID: "CECILL-C", LRE: license_CECILL_C_lre},
	{ID: "CERN-OHL-1.1", LRE: license_CERN_OHL_1_1_lre},
	{ID: "CERN-OHL-1.2", LRE: license_CERN_OHL_1_2_lre},
	{ID: "CERN-OHL-P-2.0", LRE: license_CERN_OHL_P_2_0_lre},
	{ID: "CERN-OHL-S-2.0", LRE: license_CERN_OHL_S_2_0_lre},
	{ID: "CERN-OHL-W-2.0", LRE: license_CERN_OHL_W_2_0_lre},
	{ID: "CNRI-Jython", LRE: license_CNRI_Jython_lre},
	{ID: "CNRI-Python", LRE: license_CNRI_Python_lre},
	{ID: "CNRI-Python-GPL-Compatible", LRE: license_CNRI_Python_GPL_Compatible_lre},
	{ID: "CPAL-1.0", LRE: license_CPAL_1_0_lre},
	{ID: "CPL-1.0", LRE: license_CPL_1_0_lre},
	{ID: "CPOL-1.02", LRE: license_CPOL_1_02_lre},
	{ID: "CUA-OPL-1.0", LRE: license_CUA_OPL_1_0_lre},
	{ID: "Caldera", LRE: license_Caldera_lre},
	{ID: "ClArtistic", LRE: license_ClArtistic_lre},
	{ID: "CommonsClause", LRE: license_CommonsClause_lre},
	{ID: "Condor-1.1", LRE: license_Condor_1_1_lre},
	{ID: "Crossword", LRE: license_Crossword_lre},
	{ID: "CrystalStacker", LRE: license_CrystalStacker_lre},
	{ID: "Cube", LRE: license_Cube_lre},
	{ID: "D-FSL-1.0", LRE: license_D_FSL_1_0_lre},
	{ID: "DOC", LRE: license_DOC_lre},
	{ID: "DSDP", LRE: license_DSDP_lre},
	{ID: "Dotseqn", LRE: license_Dotseqn_lre},
	{ID: "ECL-1.0", LRE: license_ECL_1_0_lre},
	{ID: "ECL-2.0", LRE: license_ECL_2_0_lre},
	{ID: "EFL-1.0", LRE: license_EFL_1_0_lre},
	{ID: "EFL-2.0", LRE: license_EFL_2_0_lre},
	{ID: "EPICS", LRE: license_EPICS_lre},
	{ID: "EPL-1.0", LRE: license_EPL_1_0_lre},
	{ID: "EPL-2.0", LRE: license_EPL_2_0_lre},
	{ID: "EUDatagrid", LRE: license_EUDatagrid_lre},
	{ID: "EUPL-1.0", LRE: license_EUPL_1_0_lre},
	{ID: "EUPL-1.1", LRE: license_EUPL_1_1_lre},
	{ID: "EUPL-1.2", LRE: license_EUPL_1_2_lre},
	{ID: "Entessa", LRE: license_Entessa_lre},
	{ID: "ErlPL-1.1", LRE: license_ErlPL_1_1_lre},
	{ID: "Eurosym", LRE: license_Eurosym_lre},
	{ID: "FSFAP", LRE: license_FSFAP_lre},
	{ID: "FSFUL", LRE: license_FSFUL_lre},
	{ID: "FSFULLR", LRE: license_FSFULLR_lre},
	{ID: "FTL", LRE: license_FTL_lre},
	{ID: "Fair", LRE: license_Fair_lre},
	{ID: "Frameworx-1.0", LRE: license_Frameworx_1_0_lre},
	{ID: "FreeImage", LRE: license_FreeImage_lre},
	{ID: "GFDL-1.3-no-invariants-or-later", LRE: license_GFDL_1_3_no_invariants_or_later_lre},
	{ID: "GFDL-1.3-no-invariants-only", LRE: license_GFDL_1_3_no_invariants_only_lre},
	{ID: "GFDL-1.3-invariants-or-later", LRE: license_GFDL_1_3_invariants_or_later_lre},
	{ID: "GFDL-1.3-invariants-only", LRE: license_GFDL_1_3_invariants_only_lre},
	{ID: "GFDL-1.3", LRE: license_GFDL_1_3_lre},
	{ID: "GFDL-1.2-no-invariants-or-later", LRE: license_GFDL_1_2_no_invariants_or_later_lre},
	{ID: "GFDL-1.2-no-invariants-only", LRE: license_GFDL_1_2_no_invariants_only_lre},
	{ID: "GFDL-1.2-invariants-or-later", LRE: license_GFDL_1_2_invariants_or_later_lre},
	{ID: "GFDL-1.2-invariants-only", LRE: license_GFDL_1_2_invariants_only_lre},
	{ID: "GFDL-1.2", LRE: license_GFDL_1_2_lre},
	{ID: "GFDL-1.1-no-invariants-or-later", LRE: license_GFDL_1_1_no_invariants_or_later_lre},
	{ID: "GFDL-1.1-no-invariants-only", LRE: license_GFDL_1_1_no_invariants_only_lre},
	{ID: "GFDL-1.1-invariants-or-later", LRE: license_GFDL_1_1_invariants_or_later_lre},
	{ID: "GFDL-1.1-invariants-only", LRE: license_GFDL_1_1_invariants_only_lre},
	{ID: "GFDL-1.1", LRE: license_GFDL_1_1_lre},
	{ID: "GL2PS", LRE: license_GL2PS_lre},
	{ID: "GLWTPL", LRE: license_GLWTPL_lre},
	{ID: "GPL-1.0", LRE: license_GPL_1_0_lre},
	{ID: "GPL-1.0-only", LRE: license_GPL_1_0_only_lre},
	{ID: "GPL-1.0-or-later", LRE: license_GPL_1_0_or_later_lre},
	{ID: "GPL-2.0", LRE: license_GPL_2_0_lre},
	{ID: "GPL-2.0-only", LRE: license_GPL_2_0_only_lre},
	{ID: "GPL-2.0-or-3.0", LRE: license_GPL_2_0_or_3_0_lre},
	{ID: "GPL-2.0-or-later", LRE: license_GPL_2_0_or_later_lre},
	{ID: "GPL-3.0", LRE: license_GPL_3_0_lre},
	{ID: "GPL-3.0-only", LRE: license_GPL_3_0_only_lre},
	{ID: "GPL-3.0-or-later", LRE: license_GPL_3_0_or_later_lre},
	{ID: "Giftware", LRE: license_Giftware_lre},
	{ID: "Glide", LRE: license_Glide_lre},
	{ID: "Glulxe", LRE: license_Glulxe_lre},
	{ID: "GooglePatentClause", LRE: license_GooglePatentClause_lre},
	{ID: "GooglePatentsFile", LRE: license_GooglePatentsFile_lre},
	{ID: "HPND", LRE: license_HPND_lre},
	{ID: "HPND-sell-variant", LRE: license_HPND_sell_variant_lre},
	{ID: "HaskellReport", LRE: license_HaskellReport_lre},
	{ID: "Hippocratic-2.1", LRE: license_Hippocratic_2_1_lre},
	{ID: "IBM-pibs", LRE: license_IBM_pibs_lre},
	{ID: "ICU", LRE: license_ICU_lre},
	{ID: "IJG", LRE: license_IJG_lre},
	{ID: "IPA", LRE: license_IPA_lre},
	{ID: "IPL-1.0", LRE: license_IPL_1_0_lre},
	{ID: "ISC", LRE: license_ISC_lre},
	{ID: "ImageMagick", LRE: license_ImageMagick_lre},
	{ID: "Imlib2", LRE: license_Imlib2_lre},
	{ID: "Info-ZIP", LRE: license_Info_ZIP_lre},
	{ID: "Intel", LRE: license_Intel_lre},
	{ID: "Intel-ACPI", LRE: license_Intel_ACPI_lre},
	{ID: "Interbase-1.0", LRE: license_Interbase_1_0_lre},
	{ID: "JPNIC", LRE: license_JPNIC_lre},
	{ID: "JSON", LRE: license_JSON_lre},
	{ID: "JasPer-2.0", LRE: license_JasPer_2_0_lre},
	{ID: "LAL-1.2", LRE: license_LAL_1_2_lre},
	{ID: "LAL-1.3", LRE: license_LAL_1_3_lre},
	{ID: "LGPL-2.0", LRE: license_LGPL_2_0_lre},
	{ID: "LGPL-2.0-only", LRE: license_LGPL_2_0_only_lre},
	{ID: "LGPL-2.0-or-later", LRE: license_LGPL_2_0_or_later_lre},
	{ID: "LGPL-2.1", LRE: license_LGPL_2_1_lre},
	{ID: "LGPL-2.1-only", LRE: license_LGPL_2_1_only_lre},
	{ID: "LGPL-2.1-or-later", LRE: license_LGPL_2_1_or_later_lre},
	{ID: "LGPL-3.0", LRE: license_LGPL_3_0_lre},
	{ID: "LGPL-3.0-only", LRE: license_LGPL_3_0_only_lre},
	{ID: "LGPL-3.0-or-later", LRE: license_LGPL_3_0_or_later_lre},
	{ID: "LGPLLR", LRE: license_LGPLLR_lre},
	{ID: "LPL-1.0", LRE: license_LPL_1_0_lre},
	{ID: "LPL-1.02", LRE: license_LPL_1_02_lre},
	{ID: "LPPL-1.0", LRE: license_LPPL_1_0_lre},
	{ID: "LPPL-1.1", LRE: license_LPPL_1_1_lre},
	{ID: "LPPL-1.2", LRE: license_LPPL_1_2_lre},
	{ID: "LPPL-1.3a", LRE: license_LPPL_1_3a_lre},
	{ID: "LPPL-1.3c", LRE: license_LPPL_1_3c_lre},
	{ID: "Latex2e", LRE: license_Latex2e_lre},
	{ID: "Leptonica", LRE: license_Leptonica_lre},
	{ID: "LiLiQ-P-1.1", LRE: license_LiLiQ_P_1_1_lre},
	{ID: "LiLiQ-R-1.1", LRE: license_LiLiQ_R_1_1_lre},
	{ID: "LiLiQ-Rplus-1.1", LRE: license_LiLiQ_Rplus_1_1_lre},
	{ID: "Libpng", LRE: license_Libpng_lre},
	{ID: "Linux-OpenIB", LRE: license_Linux_OpenIB_lre},
	{ID: "MIT", LRE: license_MIT_lre},
	{ID: "MIT-0", LRE: license_MIT_0_lre},
	{ID: "MIT-CMU", LRE: license_MIT_CMU_lre},
	{ID: "MIT-NoAd", LRE: license_MIT_NoAd_lre},
	{ID: "MIT-advertising", LRE: license_MIT_advertising_lre},
	{ID: "MIT-enna", LRE: license_MIT_enna_lre},
	{ID: "MIT-feh", LRE: license_MIT_feh_lre},
	{ID: "MITNFA", LRE: license_MITNFA_lre},
	{ID: "MPL-1.0", LRE: license_MPL_1_0_lre},
	{ID: "MPL-1.1", LRE: license_MPL_1_1_lre},
	{ID: "MPL-2.0", LRE: license_MPL_2_0_lre},
	{ID: "MPL-2.0-no-copyleft-exception", LRE: license_MPL_2_0_no_copyleft_exception_lre},
	{ID: "MS-PL", LRE: license_MS_PL_lre},
	{ID: "MS-RL", LRE: license_MS_RL_lre},
	{ID: "MTLL", LRE: license_MTLL_lre},
	{ID: "MakeIndex", LRE: license_MakeIndex_lre},
	{ID: "MirOS", LRE: license_MirOS_lre},
	{ID: "Motosoto", LRE: license_Motosoto_lre},
	{ID: "MulanPSL-1.0", LRE: license_MulanPSL_1_0_lre},
	{ID: "MulanPSL-2.0", LRE: license_MulanPSL_2_0_lre},
	{ID: "Multics", LRE: license_Multics_lre},
	{ID: "Mup", LRE: license_Mup_lre},
	{ID: "NASA-1.3", LRE: license_NASA_1_3_lre},
	{ID: "NBPL-1.0", LRE: license_NBPL_1_0_lre},
	{ID: "NCGL-UK-2.0", LRE: license_NCGL_UK_2_0_lre},
	{ID: "NCSA", LRE: license_NCSA_lre},
	{ID: "NGPL", LRE: license_NGPL_lre},
	{ID: "NIST-PD", LRE: license_NIST_PD_lre},
	{ID: "NIST-PD-fallback", LRE: license_NIST_PD_fallback_lre},
	{ID: "NLOD-1.0", LRE: license_NLOD_1_0_lre},
	{ID: "NLPL", LRE: license_NLPL_lre},
	{ID: "NOSL", LRE: license_NOSL_lre},
	{ID: "NPL-1.0", LRE: license_NPL_1_0_lre},
	{ID: "NPL-1.1", LRE: license_NPL_1_1_lre},
	{ID: "NPOSL-3.0", LRE: license_NPOSL_3_0_lre},
	{ID: "NRL", LRE: license_NRL_lre},
	{ID: "NTP", LRE: license_NTP_lre},
	{ID: "NTP-0", LRE: license_NTP_0_lre},
	{ID: "Naumen", LRE: license_Naumen_lre},
	{ID: "Net-SNMP", LRE: license_Net_SNMP_lre},
	{ID: "NetCDF", LRE: license_NetCDF_lre},
	{ID: "Newsletr", LRE: license_Newsletr_lre},
	{ID: "Nokia", LRE: license_Nokia_lre},
	{ID: "Noweb", LRE: license_Noweb_lre},
	{ID: "O-UDA-1.0", LRE: license_O_UDA_1_0_lre},
	{ID: "OCCT-PL", LRE: license_OCCT_PL_lre},
	{ID: "OCLC-2.0", LRE: license_OCLC_2_0_lre},
	{ID: "ODC-By-1.0", LRE: license_ODC_By_1_0_lre},
	{ID: "ODbL-1.0", LRE: license_ODbL_1_0_lre},
	{ID: "OFL-1.0", LRE: license_OFL_1_0_lre},
	{ID: "OFL-1.1", LRE: license_OFL_1_1_lre},
	{ID: "OGC-1.0", LRE: license_OGC_1_0_lre},
	{ID: "OGL-Canada-2.0", LRE: license_OGL_Canada_2_0_lre},
	{ID: "OGL-UK-1.0", LRE: license_OGL_UK_1_0_lre},
	{ID: "OGL-UK-2.0", LRE: license_OGL_UK_2_0_lre},
	{ID: "OGL-UK-3.0", LRE: license_OGL_UK_3_0_lre},
	{ID: "OGTSL", LRE: license_OGTSL_lre},
	{ID: "OLDAP-1.1", LRE: license_OLDAP_1_1_lre},
	{ID: "OLDAP-1.2", LRE: license_OLDAP_1_2_lre},
	{ID: "OLDAP-1.3", LRE: license_OLDAP_1_3_lre},
	{ID: "OLDAP-1.4", LRE: license_OLDAP_1_4_lre},
	{ID: "OLDAP-2.0", LRE: license_OLDAP_2_0_lre},
	{ID: "OLDAP-2.0.1", LRE: license_OLDAP_2_0_1_lre},
	{ID: "OLDAP-2.1", LRE: license_OLDAP_2_1_lre},
	{ID: "OLDAP-2.2", LRE: license_OLDAP_2_2_lre},
	{ID: "OLDAP-2.2.1", LRE: license_OLDAP_2_2_1_lre},
	{ID: "OLDAP-2.2.2", LRE: license_OLDAP_2_2_2_lre},
	{ID: "OLDAP-2.3", LRE: license_OLDAP_2_3_lre},
	{ID: "OLDAP-2.4", LRE: license_OLDAP_2_4_lre},
	{ID: "OLDAP-2.5", LRE: license_OLDAP_2_5_lre},
	{ID: "OLDAP-2.6", LRE: license_OLDAP_2_6_lre},
	{ID: "OLDAP-2.7", LRE: license_OLDAP_2_7_lre},
	{ID: "OLDAP-2.8", LRE: license_OLDAP_2_8_lre},
	{ID: "OML", LRE: license_OML_lre},
	{ID: "OPL-1.0", LRE: license_OPL_1_0_lre},
	{ID: "OSET-PL-2.1", LRE: license_OSET_PL_2_1_lre},
	{ID: "OSL-1.0", LRE: license_OSL_1_0_lre},
	{ID: "OSL-1.1", LRE: license_OSL_1_1_lre},
	{ID: "OSL-2.0", LRE: license_OSL_2_0_lre},
	{ID: "OSL-2.1", LRE: license_OSL_2_1_lre},
	{ID: "OSL-3.0", LRE: license_OSL_3_0_lre},
	{ID: "OpenSSL", LRE: license_OpenSSL_lre},
	{ID: "PDDL-1.0", LRE: license_PDDL_1_0_lre},
	{ID: "PHP-3.0", LRE: license_PHP_3_0_lre},
	{ID: "PHP-3.01", LRE: license_PHP_3_01_lre},
	{ID: "PSF-2.0", LRE: license_PSF_2_0_lre},
	{ID: "Parity-6.0.0", LRE: license_Parity_6_0_0_lre},
	{ID: "Parity-7.0.0", LRE: license_Parity_7_0_0_lre},
	{ID: "Plexus", LRE: license_Plexus_lre},
	{ID: "PolyForm-Noncommercial-1.0.0", LRE: license_PolyForm_Noncommercial_1_0_0_lre},
	{ID: "PolyForm-Small-Business-1.0.0", LRE: license_PolyForm_Small_Business_1_0_0_lre},
	{ID: "PostgreSQL", LRE: license_PostgreSQL_lre},
	{ID: "Prosperity-3.0.0", LRE: license_Prosperity_3_0_0_lre},
	{ID: "Python-2.0", LRE: license_Python_2_0_lre},
	{ID: "QPL-1.0", LRE: license_QPL_1_0_lre},
	{ID: "Qhull", LRE: license_Qhull_lre},
	{ID: "RHeCos-1.1", LRE: license_RHeCos_1_1_lre},
	{ID: "RPL-1.1", LRE: license_RPL_1_1_lre},
	{ID: "RPL-1.5", LRE: license_RPL_1_5_lre},
	{ID: "RPSL-1.0", LRE: license_RPSL_1_0_lre},
	{ID: "RSA-MD", LRE: license_RSA_MD_lre},
	{ID: "RSCPL", LRE: license_RSCPL_lre},
	{ID: "Rdisc", LRE: license_Rdisc_lre},
	{ID: "Ruby", LRE: license_Ruby_lre},
	{ID: "SAX-PD", LRE: license_SAX_PD_lre},
	{ID: "SCEA", LRE: license_SCEA_lre},
	{ID: "SGI-B-1.0", LRE: license_SGI_B_1_0_lre},
	{ID: "SGI-B-1.1", LRE: license_SGI_B_1_1_lre},
	{ID: "SGI-B-2.0", LRE: license_SGI_B_2_0_lre},
	{ID: "SHL-0.5", LRE: license_SHL_0_5_lre},
	{ID: "SHL-0.51", LRE: license_SHL_0_51_lre},
	{ID: "SISSL", LRE: license_SISSL_lre},
	{ID: "SISSL-1.2", LRE: license_SISSL_1_2_lre},
	{ID: "SMLNJ", LRE: license_SMLNJ_lre},
	{ID: "SMPPL", LRE: license_SMPPL_lre},
	{ID: "SNIA", LRE: license_SNIA_lre},
	{ID: "SPL-1.0", LRE: license_SPL_1_0_lre},
	{ID: "SSH-OpenSSH", LRE: license_SSH_OpenSSH_lre},
	{ID: "SSH-short", LRE: license_SSH_short_lre},
	{ID: "SSPL-1.0", LRE: license_SSPL_1_0_lre},
	{ID: "SWL", LRE: license_SWL_lre},
	{ID: "Saxpath", LRE: license_Saxpath_lre},
	{ID: "Sendmail", LRE: license_Sendmail_lre},
	{ID: "Sendmail-8.23", LRE: license_Sendmail_8_23_lre},
	{ID: "SimPL-2.0", LRE: license_SimPL_2_0_lre},
	{ID: "Sleepycat", LRE: license_Sleepycat_lre},
	{ID: "Spencer-86", LRE: license_Spencer_86_lre},
	{ID: "Spencer-94", LRE: license_Spencer_94_lre},
	{ID: "Spencer-99", LRE: license_Spencer_99_lre},
	{ID: "SugarCRM-1.1.3", LRE: license_SugarCRM_1_1_3_lre},
	{ID: "TAPR-OHL-1.0", LRE: license_TAPR_OHL_1_0_lre},
	{ID: "TCL", LRE: license_TCL_lre},
	{ID: "TCP-wrappers", LRE: license_TCP_wrappers_lre},
	{ID: "TMate", LRE: license_TMate_lre},
	{ID: "TORQUE-1.1", LRE: license_TORQUE_1_1_lre},
	{ID: "TOSL", LRE: license_TOSL_lre},
	{ID: "TU-Berlin-1.0", LRE: license_TU_Berlin_1_0_lre},
	{ID: "TU-Berlin-2.0", LRE: license_TU_Berlin_2_0_lre},
	{ID: "UCL-1.0", LRE: license_UCL_1_0_lre},
	{ID: "UPL-1.0", LRE: license_UPL_1_0_lre},
	{ID: "Unicode-DFS-2015", LRE: license_Unicode_DFS_2015_lre},
	{ID: "Unicode-DFS-2016", LRE: license_Unicode_DFS_2016_lre},
	{ID: "Unicode-TOU", LRE: license_Unicode_TOU_lre},
	{ID: "Unlicense", LRE: license_Unlicense_lre},
	{ID: "VOSTROM", LRE: license_VOSTROM_lre},
	{ID: "VSL-1.0", LRE: license_VSL_1_0_lre},
	{ID: "Vim", LRE: license_Vim_lre},
	{ID: "W3C", LRE: license_W3C_lre},
	{ID: "W3C-19980720", LRE: license_W3C_19980720_lre},
	{ID: "W3C-20150513", LRE: license_W3C_20150513_lre},
	{ID: "WTFPL", Type: Discouraged, LRE: license_WTFPL_lre},
	{ID: "Watcom-1.0", LRE: license_Watcom_1_0_lre},
	{ID: "Wsuipa", LRE: license_Wsuipa_lre},
	{ID: "X11", LRE: license_X11_lre},
	{ID: "XFree86-1.1", LRE: license_XFree86_1_1_lre},
	{ID: "XSkat", LRE: license_XSkat_lre},
	{ID: "Xerox", LRE: license_Xerox_lre},
	{ID: "Xnet", LRE: license_Xnet_lre},
	{ID: "YPL-1.0", LRE: license_YPL_1_0_lre},
	{ID: "YPL-1.1", LRE: license_YPL_1_1_lre},
	{ID: "ZPL-1.1", LRE: license_ZPL_1_1_lre},
	{ID: "ZPL-2.0", LRE: license_ZPL_2_0_lre},
	{ID: "ZPL-2.1", LRE: license_ZPL_2_1_lre},
	{ID: "Zed", LRE: license_Zed_lre},
	{ID: "Zend-2.0", LRE: license_Zend_2_0_lre},
	{ID: "Zimbra-1.3", LRE: license_Zimbra_1_3_lre},
	{ID: "Zimbra-1.4", LRE: license_Zimbra_1_4_lre},
	{ID: "Zlib", LRE: license_Zlib_lre},
	{ID: "blessing", LRE: license_blessing_lre},
	{ID: "bzip2-1.0.5", LRE: license_bzip2_1_0_5_lre},
	{ID: "bzip2-1.0.6", LRE: license_bzip2_1_0_6_lre},
	{ID: "copyleft-next-0.3.0", LRE: license_copyleft_next_0_3_0_lre},
	{ID: "copyleft-next-0.3.1", LRE: license_copyleft_next_0_3_1_lre},
	{ID: "curl", LRE: license_curl_lre},
	{ID: "diffmark", LRE: license_diffmark_lre},
	{ID: "dvipdfm", LRE: license_dvipdfm_lre},
	{ID: "eGenix", LRE: license_eGenix_lre},
	{ID: "etalab-2.0", LRE: license_etalab_2_0_lre},
	{ID: "gSOAP-1.3b", LRE: license_gSOAP_1_3b_lre},
	{ID: "gnuplot", LRE: license_gnuplot_lre},
	{ID: "iMatix", LRE: license_iMatix_lre},
	{ID: "libpng-2.0", LRE: license_libpng_2_0_lre},
	{ID: "libselinux-1.0", LRE: license_libselinux_1_0_lre},
	{ID: "libtiff", LRE: license_libtiff_lre},
	{ID: "mpich2", LRE: license_mpich2_lre},
	{ID: "psfrag", LRE: license_psfrag_lre},
	{ID: "psutils", LRE: license_psutils_lre},
	{ID: "xinetd", LRE: license_xinetd_lre},
	{ID: "xpp", LRE: license_xpp_lre},
	{ID: "zlib-acknowledgement", LRE: license_zlib_acknowledgement_lre},
}

const license_0BSD_lre = `//**
BSD Zero Clause License
https://spdx.org/licenses/0BSD.json
http://landley.net/toybox/license.html
**//

//** Copyright **//

Permission to use, copy, modify, and/or distribute this software for any purpose
with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
THIS SOFTWARE.
`
const license_AAL_lre = `//**
Attribution Assurance License
https://spdx.org/licenses/AAL.json
https://opensource.org/licenses/attribution
**//

(( Attribution Assurance License
((Copyright __20__))??
))??

(( ATTRIBUTION ASSURANCE LICENSE (adapted from the original BSD license) ))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the conditions below are met. These conditions
require a modest attribution to
//** <AUTHOR> **//
__5__
(the "Author"), who hopes that its promotional value may help justify the
thousands of dollars in otherwise billable time invested in writing this and
other freely available, open-source software.

   (( 1. ))??
   Redistributions of source code, in whole or part and with or without
   modification (the "Code"), must prominently display this GPG-signed text in
   verifiable form.

   (( 2. ))??
   Redistributions of the Code in binary form must be accompanied by this
   GPG-signed text in any documentation and, each time the resulting executable
   program or a program dependent thereon is launched, a prominent display
   (e.g., splash screen or banner text) of the Author's attribution information,
   which includes:

      __30__

   (( 3. ))??
   Neither the name nor any trademark of the Author may be used to endorse or
   promote products derived from this software without specific prior written
   permission.

   (( 4. ))??
   Users are entirely responsible, to the exclusion of the Author and any other
   persons, for compliance with (1) regulations set by owners or administrators
   of employed equipment, (2) licensing terms of any other software, and (3)
   local regulations regarding use, including those regarding import, export,
   and use of encryption software.

THIS FREE SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR OR ANY CONTRIBUTOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, EFFECTS OF UNAUTHORIZED OR MALICIOUS NETWORK ACCESS; PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
`
const license_ADSL_lre = `//**
Amazon Digital Services License
https://spdx.org/licenses/ADSL.json
https://fedoraproject.org/wiki/Licensing/AmazonDigitalServicesLicense
**//

This software code is made available "AS IS" without warranties of any kind. You
may copy, display, modify and redistribute the software code either by itself or
as incorporated into your code; provided that you do not remove any proprietary
notices. Your use of this software code is at your own risk and you waive any
claim against Amazon Digital Services, Inc. or its affiliates with respect to
your use of this software code. (c) 2006 Amazon Digital Services, Inc. or its
affiliates.
`
const license_AFL_1_1_lre = `//**
Academic Free License v1.1
https://spdx.org/licenses/AFL-1.1.json
http://opensource.linux-mirror.org/licenses/afl-1.1.txt
http://wayback.archive.org/web/20021004124254/http://www.opensource.org/licenses/academic.php
**//

(( Academic Free License

Version 1.1 ))??

(( The Academic Free License applies to any original work of authorship (the
"Original Work") whose owner (the "Licensor") has placed the following notice
immediately following the copyright notice for the Original Work:

Licensed under the Academic Free License version 1.1. ))??

Grant of License. Licensor hereby grants to any person obtaining a copy of the
Original Work ("You") a world-wide, royalty-free, non-exclusive, perpetual,
non-sublicenseable license

   (( (1) ))??
   to use, copy, modify, merge, publish, perform, distribute and/or sell copies
   of the Original Work and derivative works thereof, and

   (( (2) ))??
   under patent claims owned or controlled by the Licensor that are embodied in
   the Original Work as furnished by the Licensor, to make, use, sell and offer
   for sale the Original Work and derivative works thereof, subject to the
   following conditions.

Right of Attribution. Redistributions of the Original Work must reproduce all
copyright notices in the Original Work as furnished by the Licensor, both in the
Original Work itself and in any documentation and/or other materials provided
with the distribution of the Original Work in executable form.

Exclusions from License Grant. Neither the names of Licensor, nor the names of
any contributors to the Original Work, nor any of their trademarks or service
marks, may be used to endorse or promote products derived from this Original
Work without express prior written permission of the Licensor.

WARRANTY AND DISCLAIMERS. LICENSOR WARRANTS THAT THE COPYRIGHT IN AND TO THE
ORIGINAL WORK IS OWNED BY THE LICENSOR OR THAT THE ORIGINAL WORK IS DISTRIBUTED
BY LICENSOR UNDER A VALID CURRENT LICENSE FROM THE COPYRIGHT OWNER. EXCEPT AS
EXPRESSLY STATED IN THE IMMEDIATELY PRECEEDING SENTENCE, THE ORIGINAL WORK IS
PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT WARRANTY, EITHER
EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, THE WARRANTY OF
NON-INFRINGEMENT AND WARRANTIES THAT THE ORIGINAL WORK IS MERCHANTABLE OR FIT
FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY OF THE ORIGINAL WORK
IS WITH YOU. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS
LICENSE. NO LICENSE TO ORIGINAL WORK IS GRANTED HEREUNDER EXCEPT UNDER THIS
DISCLAIMER.

LIMITATION OF LIABILITY. UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY,
WHETHER TORT (INCLUDING NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE LICENSOR
BE LIABLE TO ANY PERSON FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
CONSEQUENTIAL DAMAGES OF ANY CHARACTER ARISING AS A RESULT OF THIS LICENSE OR
THE USE OF THE ORIGINAL WORK INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF
GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER
COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH PERSON SHALL HAVE BEEN INFORMED OF
THE POSSIBILITY OF SUCH DAMAGES. THIS LIMITATION OF LIABILITY SHALL NOT APPLY TO
LIABILITY FOR DEATH OR PERSONAL INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO
THE EXTENT APPLICABLE LAW PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT
ALLOW THE EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO
THIS EXCLUSION AND LIMITATION MAY NOT APPLY TO YOU.

License to Source Code. The term "Source Code" means the preferred form of the
Original Work for making modifications to it and all available documentation
describing how to access and modify the Original Work. Licensor hereby agrees to
provide a machine-readable copy of the Source Code of the Original Work along
with each copy of the Original Work that Licensor distributes. Licensor reserves
the right to satisfy this obligation by placing a machine-readable copy of the
Source Code in an information repository reasonably calculated to permit
inexpensive and convenient access by You for as long as Licensor continues to
distribute the Original Work, and by publishing the address of that information
repository in a notice immediately following the copyright notice that applies
to the Original Work.

Mutual Termination for Patent Action. This License shall terminate automatically
and You may no longer exercise any of the rights granted to You by this License
if You file a lawsuit in any court alleging that any OSI Certified open source
software that is licensed under any license containing this "Mutual Termination
for Patent Action" clause infringes any patent claims that are essential to use
that software.

This license is Copyright (C) 2002 Lawrence E. Rosen. All rights reserved.

Permission is hereby granted to copy and distribute this license without
modification. This license may not be modified without the express written
permission of its copyright owner.
`
const license_AFL_1_2_lre = `//**
Academic Free License v1.2
https://spdx.org/licenses/AFL-1.2.json
http://opensource.linux-mirror.org/licenses/afl-1.2.txt
http://wayback.archive.org/web/20021204204652/http://www.opensource.org/licenses/academic.php
**//

(( Academic Free License

Version 1.2 ))??

(( This Academic Free License applies to any original work of authorship (the
"Original Work") whose owner (the "Licensor") has placed the following notice
immediately following the copyright notice for the Original Work:

Licensed under the Academic Free License version 1.2 ))??

Grant of License. Licensor hereby grants to any person obtaining a copy of the
Original Work ("You") a world-wide, royalty-free, non-exclusive, perpetual,
non-sublicenseable license (1) to use, copy, modify, merge, publish, perform,
distribute and/or sell copies of the Original Work and derivative works thereof,
and (2) under patent claims owned or controlled by the Licensor that are
embodied in the Original Work as furnished by the Licensor, to make, use, sell
and offer for sale the Original Work and derivative works thereof, subject to
the following conditions.

Attribution Rights. You must retain, in the Source Code of any Derivative Works
that You create, all copyright, patent or trademark notices from the Source Code
of the Original Work, as well as any notices of licensing and any descriptive
text identified therein as an "Attribution Notice." You must cause the Source
Code for any Derivative Works that You create to carry a prominent Attribution
Notice reasonably calculated to inform recipients that You have modified the
Original Work.

Exclusions from License Grant. Neither the names of Licensor, nor the names of
any contributors to the Original Work, nor any of their trademarks or service
marks, may be used to endorse or promote products derived from this Original
Work without express prior written permission of the Licensor.

Warranty and Disclaimer of Warranty. Licensor warrants that the copyright in and
to the Original Work is owned by the Licensor or that the Original Work is
distributed by Licensor under a valid current license from the copyright owner.
Except as expressly stated in the immediately proceeding sentence, the Original
Work is provided under this License on an "AS IS" BASIS and WITHOUT WARRANTY,
either express or implied, including, without limitation, the warranties of
NON-INFRINGEMENT, MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY OF THE ORIGINAL WORK IS WITH YOU. This DISCLAIMER
OF WARRANTY constitutes an essential part of this License. No license to
Original Work is granted hereunder except under this disclaimer.

Limitation of Liability. Under no circumstances and under no legal theory,
whether in tort (including negligence), contract, or otherwise, shall the
Licensor be liable to any person for any direct, indirect, special, incidental,
or consequential damages of any character arising as a result of this License or
the use of the Original Work including, without limitation, damages for loss of
goodwill, work stoppage, computer failure or malfunction, or any and all other
commercial damages or losses. This limitation of liability shall not apply to
liability for death or personal injury resulting from Licensor's negligence to
the extent applicable law prohibits such limitation. Some jurisdictions do not
allow the exclusion or limitation of incidental or consequential damages, so
this exclusion and limitation may not apply to You.

License to Source Code. The term "Source Code" means the preferred form of the
Original Work for making modifications to it and all available documentation
describing how to modify the Original Work. Licensor hereby agrees to provide a
machine-readable copy of the Source Code of the Original Work along with each
copy of the Original Work that Licensor distributes. Licensor reserves the right
to satisfy this obligation by placing a machine-readable copy of the Source Code
in an information repository reasonably calculated to permit inexpensive and
convenient access by You for as long as Licensor continues to distribute the
Original Work, and by publishing the address of that information repository in a
notice immediately following the copyright notice that applies to the Original
Work.

Mutual Termination for Patent Action. This License shall terminate automatically
and You may no longer exercise any of the rights granted to You by this License
if You file a lawsuit in any court alleging that any OSI Certified open source
software that is licensed under any license containing this "Mutual Termination
for Patent Action" clause infringes any patent claims that are essential to use
that software.

Right to Use. You may use the Original Work in all ways not otherwise restricted
or conditioned by this License or by law, and Licensor promises not to interfere
with or be responsible for such uses by You.

This license is Copyright (C) 2002 Lawrence E. Rosen. All rights reserved.

Permission is hereby granted to copy and distribute this license without
modification. This license may not be modified without the express written
permission of its copyright owner.
`
const license_AFL_2_0_lre = `//**
Academic Free License v2.0
https://spdx.org/licenses/AFL-2.0.json
http://wayback.archive.org/web/20060924134533/http://www.opensource.org/licenses/afl-2.0.txt
**//

(( The Academic Free License

v. 2.0 ))??

(( This Academic Free License (the "License") applies to any original work of
authorship (the "Original Work") whose owner (the "Licensor") has placed the
following notice immediately following the copyright notice for the Original
Work:

Licensed under the Academic Free License version 2.0 ))??

   (( 1) ))??
   Grant of Copyright License. Licensor hereby grants You a world-wide,
   royalty-free, non-exclusive, perpetual, sublicenseable license to do the
   following:

      (( a) ))??
      to reproduce the Original Work in copies;

      (( b) ))??
      to prepare derivative works ("Derivative Works") based upon the Original
      Work;

      (( c) ))??
      to distribute copies of the Original Work and Derivative Works to the
      public;

      (( d) ))??
      to perform the Original Work publicly; and

      (( e) ))??
      to display the Original Work publicly.

   (( 2) ))??
   Grant of Patent License. Licensor hereby grants You a world-wide,
   royalty-free, non-exclusive, perpetual, sublicenseable license, under patent
   claims owned or controlled by the Licensor that are embodied in the Original
   Work as furnished by the Licensor, to make, use, sell and offer for sale the
   Original Work and Derivative Works.

   (( 3) ))??
   Grant of Source Code License. The term "Source Code" means the preferred form
   of the Original Work for making modifications to it and all available
   documentation describing how to modify the Original Work. Licensor hereby
   agrees to provide a machine-readable copy of the Source Code of the Original
   Work along with each copy of the Original Work that Licensor distributes.
   Licensor reserves the right to satisfy this obligation by placing a
   machine-readable copy of the Source Code in an information repository
   reasonably calculated to permit inexpensive and convenient access by You for
   as long as Licensor continues to distribute the Original Work, and by
   publishing the address of that information repository in a notice immediately
   following the copyright notice that applies to the Original Work.

   (( 4) ))??
   Exclusions From License Grant. Neither the names of Licensor, nor the names
   of any contributors to the Original Work, nor any of their trademarks or
   service marks, may be used to endorse or promote products derived from this
   Original Work without express prior written permission of the Licensor.
   Nothing in this License shall be deemed to grant any rights to trademarks,
   copyrights, patents, trade secrets or any other intellectual property of
   Licensor except as expressly stated herein. No patent license is granted to
   make, use, sell or offer to sell embodiments of any patent claims other than
   the licensed claims defined in Section 2. No right is granted to the
   trademarks of Licensor even if such marks are included in the Original Work.
   Nothing in this License shall be interpreted to prohibit Licensor from
   licensing under different terms from this License any Original Work that
   Licensor otherwise would have a right to license.

   (( 5) ))??
   This section intentionally omitted.

   (( 6) ))??
   Attribution Rights. You must retain, in the Source Code of any Derivative
   Works that You create, all copyright, patent or trademark notices from the
   Source Code of the Original Work, as well as any notices of licensing and any
   descriptive text identified therein as an "Attribution Notice." You must
   cause the Source Code for any Derivative Works that You create to carry a
   prominent Attribution Notice reasonably calculated to inform recipients that
   You have modified the Original Work.

   (( 7) ))??
   Warranty of Provenance and Disclaimer of Warranty. Licensor warrants that the
   copyright in and to the Original Work and the patent rights granted herein by
   Licensor are owned by the Licensor or are sublicensed to You under the terms
   of this License with the permission of the contributor(s) of those copyrights
   and patent rights. Except as expressly stated in the immediately proceeding
   sentence, the Original Work is provided under this License on an "AS IS"
   BASIS and WITHOUT WARRANTY, either express or implied, including, without
   limitation, the warranties of NON-INFRINGEMENT, MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY OF THE ORIGINAL
   WORK IS WITH YOU. This DISCLAIMER OF WARRANTY constitutes an essential part
   of this License. No license to Original Work is granted hereunder except
   under this disclaimer.

   (( 8) ))??
   Limitation of Liability. Under no circumstances and under no legal theory,
   whether in tort (including negligence), contract, or otherwise, shall the
   Licensor be liable to any person for any direct, indirect, special,
   incidental, or consequential damages of any character arising as a result of
   this License or the use of the Original Work including, without limitation,
   damages for loss of goodwill, work stoppage, computer failure or malfunction,
   or any and all other commercial damages or losses. This limitation of
   liability shall not apply to liability for death or personal injury resulting
   from Licensor's negligence to the extent applicable law prohibits such
   limitation. Some jurisdictions do not allow the exclusion or limitation of
   incidental or consequential damages, so this exclusion and limitation may not
   apply to You.

   (( 9) ))??
   Acceptance and Termination. If You distribute copies of the Original Work or
   a Derivative Work, You must make a reasonable effort under the circumstances
   to obtain the express assent of recipients to the terms of this License.
   Nothing else but this License (or another written agreement between Licensor
   and You) grants You permission to create Derivative Works based upon the
   Original Work or to exercise any of the rights granted in Section 1 herein,
   and any attempt to do so except under the terms of this License (or another
   written agreement between Licensor and You) is expressly prohibited by U.S.
   copyright law, the equivalent laws of other countries, and by international
   treaty. Therefore, by exercising any of the rights granted to You in Section
   1 herein, You indicate Your acceptance of this License and all of its terms
   and conditions.

   (( 10) ))??
   Termination for Patent Action. This License shall terminate automatically and
   You may no longer exercise any of the rights granted to You by this License
   as of the date You commence an action, including a cross-claim or
   counterclaim, for patent infringement (i) against Licensor with respect to a
   patent applicable to software or (ii) against any entity with respect to a
   patent applicable to the Original Work (but excluding combinations of the
   Original Work with other software or hardware).

   (( 11) ))??
   Jurisdiction, Venue and Governing Law. Any action or suit relating to this
   License may be brought only in the courts of a jurisdiction wherein the
   Licensor resides or in which Licensor conducts its primary business, and
   under the laws of that jurisdiction excluding its conflict-of-law provisions.
   The application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any use of the Original
   Work outside the scope of this License or after its termination shall be
   subject to the requirements and penalties of the U.S. Copyright Act, 17
   U.S.C. ¤ 101 et seq., the equivalent laws of other countries, and
   international treaty. This section shall survive the termination of this
   License.

   (( 12) ))??
   Attorneys Fees. In any action to enforce the terms of this License or seeking
   damages relating thereto, the prevailing party shall be entitled to recover
   its costs and expenses, including, without limitation, reasonable attorneys'
   fees and costs incurred in connection with such action, including any appeal
   of such action. This section shall survive the termination of this License.

   (( 13) ))??
   Miscellaneous. This License represents the complete agreement concerning the
   subject matter hereof. If any provision of this License is held to be
   unenforceable, such provision shall be reformed only to the extent necessary
   to make it enforceable.

   (( 14) ))??
   Definition of "You" in This License. "You" throughout this License, whether
   in upper or lower case, means an individual or a legal entity exercising
   rights under, and complying with all of the terms of, this License. For legal
   entities, "You" includes any entity that controls, is controlled by, or is
   under common control with you. For purposes of this definition, "control"
   means (i) the power, direct or indirect, to cause the direction or management
   of such entity, whether by contract or otherwise, or (ii) ownership of fifty
   percent (50%) or more of the outstanding shares, or (iii) beneficial
   ownership of such entity.

   (( 15) ))??
   Right to Use. You may use the Original Work in all ways not otherwise
   restricted or conditioned by this License or by law, and Licensor promises
   not to interfere with or be responsible for such uses by You.

This license is Copyright (C) 2003 Lawrence E. Rosen. All rights reserved.

Permission is hereby granted to copy and distribute this license without
modification. This license may not be modified without the express written
permission of its copyright owner.
`
const license_AFL_2_1_lre = `//**
Academic Free License v2.1
https://spdx.org/licenses/AFL-2.1.json
http://opensource.linux-mirror.org/licenses/afl-2.1.txt
**//

(( The Academic Free License

v.2.1 ))??

(( This Academic Free License (the "License") applies to any original work of
authorship (the "Original Work") whose owner (the "Licensor") has placed the
following notice immediately following the copyright notice for the Original
Work:

Licensed under the Academic Free License version 2.1 ))??

   (( 1) ))??
   Grant of Copyright License. Licensor hereby grants You a world-wide,
   royalty-free, non-exclusive, perpetual, sublicenseable license to do the
   following:

      (( a) ))??
      to reproduce the Original Work in copies;

      (( b) ))??
      to prepare derivative works ("Derivative Works") based upon the Original
      Work;

      (( c) ))??
      to distribute copies of the Original Work and Derivative Works to the
      public;

      (( d) ))??
      to perform the Original Work publicly; and

      (( e) ))??
      to display the Original Work publicly.

   (( 2) ))??
   Grant of Patent License. Licensor hereby grants You a world-wide,
   royalty-free, non-exclusive, perpetual, sublicenseable license, under patent
   claims owned or controlled by the Licensor that are embodied in the Original
   Work as furnished by the Licensor, to make, use, sell and offer for sale the
   Original Work and Derivative Works.

   (( 3) ))??
   Grant of Source Code License. The term "Source Code" means the preferred form
   of the Original Work for making modifications to it and all available
   documentation describing how to modify the Original Work. Licensor hereby
   agrees to provide a machine-readable copy of the Source Code of the Original
   Work along with each copy of the Original Work that Licensor distributes.
   Licensor reserves the right to satisfy this obligation by placing a
   machine-readable copy of the Source Code in an information repository
   reasonably calculated to permit inexpensive and convenient access by You for
   as long as Licensor continues to distribute the Original Work, and by
   publishing the address of that information repository in a notice immediately
   following the copyright notice that applies to the Original Work.

   (( 4) ))??
   Exclusions From License Grant. Neither the names of Licensor, nor the names
   of any contributors to the Original Work, nor any of their trademarks or
   service marks, may be used to endorse or promote products derived from this
   Original Work without express prior written permission of the Licensor.
   Nothing in this License shall be deemed to grant any rights to trademarks,
   copyrights, patents, trade secrets or any other intellectual property of
   Licensor except as expressly stated herein. No patent license is granted to
   make, use, sell or offer to sell embodiments of any patent claims other than
   the licensed claims defined in Section 2. No right is granted to the
   trademarks of Licensor even if such marks are included in the Original Work.
   Nothing in this License shall be interpreted to prohibit Licensor from
   licensing under different terms from this License any Original Work that
   Licensor otherwise would have a right to license.

   (( 5) ))??
   This section intentionally omitted.

   (( 6) ))??
   Attribution Rights. You must retain, in the Source Code of any Derivative
   Works that You create, all copyright, patent or trademark notices from the
   Source Code of the Original Work, as well as any notices of licensing and any
   descriptive text identified therein as an "Attribution Notice." You must
   cause the Source Code for any Derivative Works that You create to carry a
   prominent Attribution Notice reasonably calculated to inform recipients that
   You have modified the Original Work.

   (( 7) ))??
   Warranty of Provenance and Disclaimer of Warranty. Licensor warrants that the
   copyright in and to the Original Work and the patent rights granted herein by
   Licensor are owned by the Licensor or are sublicensed to You under the terms
   of this License with the permission of the contributor(s) of those copyrights
   and patent rights. Except as expressly stated in the immediately proceeding
   sentence, the Original Work is provided under this License on an "AS IS"
   BASIS and WITHOUT WARRANTY, either express or implied, including, without
   limitation, the warranties of NON-INFRINGEMENT, MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY OF THE ORIGINAL
   WORK IS WITH YOU. This DISCLAIMER OF WARRANTY constitutes an essential part
   of this License. No license to Original Work is granted hereunder except
   under this disclaimer.

   (( 8) ))??
   Limitation of Liability. Under no circumstances and under no legal theory,
   whether in tort (including negligence), contract, or otherwise, shall the
   Licensor be liable to any person for any direct, indirect, special,
   incidental, or consequential damages of any character arising as a result of
   this License or the use of the Original Work including, without limitation,
   damages for loss of goodwill, work stoppage, computer failure or malfunction,
   or any and all other commercial damages or losses. This limitation of
   liability shall not apply to liability for death or personal injury resulting
   from Licensor's negligence to the extent applicable law prohibits such
   limitation. Some jurisdictions do not allow the exclusion or limitation of
   incidental or consequential damages, so this exclusion and limitation may not
   apply to You.

   (( 9) ))??
   Acceptance and Termination. If You distribute copies of the Original Work or
   a Derivative Work, You must make a reasonable effort under the circumstances
   to obtain the express assent of recipients to the terms of this License.
   Nothing else but this License (or another written agreement between Licensor
   and You) grants You permission to create Derivative Works based upon the
   Original Work or to exercise any of the rights granted in Section 1 herein,
   and any attempt to do so except under the terms of this License (or another
   written agreement between Licensor and You) is expressly prohibited by U.S.
   copyright law, the equivalent laws of other countries, and by international
   treaty. Therefore, by exercising any of the rights granted to You in Section
   1 herein, You indicate Your acceptance of this License and all of its terms
   and conditions.

   (( 10) ))??
   Termination for Patent Action. This License shall terminate automatically and
   You may no longer exercise any of the rights granted to You by this License
   as of the date You commence an action, including a cross-claim or
   counterclaim, against Licensor or any licensee alleging that the Original
   Work infringes a patent. This termination provision shall not apply for an
   action alleging patent infringement by combinations of the Original Work with
   other software or hardware.

   (( 11) ))??
   Jurisdiction, Venue and Governing Law. Any action or suit relating to this
   License may be brought only in the courts of a jurisdiction wherein the
   Licensor resides or in which Licensor conducts its primary business, and
   under the laws of that jurisdiction excluding its conflict-of-law provisions.
   The application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any use of the Original
   Work outside the scope of this License or after its termination shall be
   subject to the requirements and penalties of the U.S. Copyright Act, 17
   U.S.C. § 101 et seq., the equivalent laws of other countries, and
   international treaty. This section shall survive the termination of this
   License.

   (( 12) ))??
   Attorneys Fees. In any action to enforce the terms of this License or seeking
   damages relating thereto, the prevailing party shall be entitled to recover
   its costs and expenses, including, without limitation, reasonable attorneys'
   fees and costs incurred in connection with such action, including any appeal
   of such action. This section shall survive the termination of this License.

   (( 13) ))??
   Miscellaneous. This License represents the complete agreement concerning the
   subject matter hereof. If any provision of this License is held to be
   unenforceable, such provision shall be reformed only to the extent necessary
   to make it enforceable.

   (( 14) ))??
   Definition of "You" in This License. "You" throughout this License, whether
   in upper or lower case, means an individual or a legal entity exercising
   rights under, and complying with all of the terms of, this License. For legal
   entities, "You" includes any entity that controls, is controlled by, or is
   under common control with you. For purposes of this definition, "control"
   means (i) the power, direct or indirect, to cause the direction or management
   of such entity, whether by contract or otherwise, or (ii) ownership of fifty
   percent (50%) or more of the outstanding shares, or (iii) beneficial
   ownership of such entity.

   (( 15) ))??
   Right to Use. You may use the Original Work in all ways not otherwise
   restricted or conditioned by this License or by law, and Licensor promises
   not to interfere with or be responsible for such uses by You.

This license is Copyright (C) 2003-2004 Lawrence E. Rosen. All rights reserved.

Permission is hereby granted to copy and distribute this license without
modification. This license may not be modified without the express written
permission of its copyright owner.
`
const license_AFL_3_0_lre = `//**
Academic Free License v3.0
https://spdx.org/licenses/AFL-3.0.json
http://www.rosenlaw.com/AFL3.0.htm
https://opensource.org/licenses/afl-3.0
**//

(( Academic Free License ("AFL") v. 3.0 ))??

((
This Academic Free License (the "License") applies to any original work of
authorship (the "Original Work") whose owner (the "Licensor") has placed the
following licensing notice adjacent to the copyright notice for the Original
Work:
))??

(( Licensed under the Academic Free License version 3.0 ))??

   (( 1) ))??
   Grant of Copyright License. Licensor grants You a worldwide, royalty-free,
   non-exclusive, sublicensable license, for the duration of the copyright, to
   do the following:

      (( a) || 1. ))??
      to reproduce the Original Work in copies, either alone or as part of a
      collective work;

      (( b) || 2. ))??
      to translate, adapt, alter, transform, modify, or arrange the Original
      Work, thereby creating derivative works ("Derivative Works") based upon
      the Original Work;

      (( c) || 3. ))??
      to distribute or communicate copies of the Original Work and Derivative
      Works to the public, under any license of your choice that does not
      contradict the terms and conditions, including Licensor's reserved rights
      and remedies, in this Academic Free License;

      (( d) || 4. ))??
      to perform the Original Work publicly; and

      (( e) || 5. ))??
      to display the Original Work publicly.

   (( 2) ))??
   Grant of Patent License. Licensor grants You a worldwide, royalty-free,
   non-exclusive, sublicensable license, under patent claims owned or controlled
   by the Licensor that are embodied in the Original Work as furnished by the
   Licensor, for the duration of the patents, to make, use, sell, offer for
   sale, have made, and import the Original Work and Derivative Works.

   (( 3) ))??
   Grant of Source Code License. The term "Source Code" means the preferred form
   of the Original Work for making modifications to it and all available
   documentation describing how to modify the Original Work. Licensor agrees to
   provide a machine-readable copy of the Source Code of the Original Work along
   with each copy of the Original Work that Licensor distributes. Licensor
   reserves the right to satisfy this obligation by placing a machine-readable
   copy of the Source Code in an information repository reasonably calculated to
   permit inexpensive and convenient access by You for as long as Licensor
   continues to distribute the Original Work.

   (( 4) ))??
   Exclusions From License Grant. Neither the names of Licensor, nor the names
   of any contributors to the Original Work, nor any of their trademarks or
   service marks, may be used to endorse or promote products derived from this
   Original Work without express prior permission of the Licensor. Except as
   expressly stated herein, nothing in this License grants any license to
   Licensor's trademarks, copyrights, patents, trade secrets or any other
   intellectual property. No patent license is granted to make, use, sell, offer
   for sale, have made, or import embodiments of any patent claims other than
   the licensed claims defined in Section 2. No license is granted to the
   trademarks of Licensor even if such marks are included in the Original Work.
   Nothing in this License shall be interpreted to prohibit Licensor from
   licensing under terms different from this License any Original Work that
   Licensor otherwise would have a right to license.

   (( 5) ))??
   External Deployment. The term "External Deployment" means the use,
   distribution, or communication of the Original Work or Derivative Works in
   any way such that the Original Work or Derivative Works may be used by anyone
   other than You, whether those works are distributed or communicated to those
   persons or made available as an application intended for use over a network.
   As an express condition for the grants of license hereunder, You must treat
   any External Deployment by You of the Original Work or a Derivative Work as a
   distribution under section 1(c).

   (( 6) ))??
   Attribution Rights. You must retain, in the Source Code of any Derivative
   Works that You create, all copyright, patent, or trademark notices from the
   Source Code of the Original Work, as well as any notices of licensing and any
   descriptive text identified therein as an "Attribution Notice." You must
   cause the Source Code for any Derivative Works that You create to carry a
   prominent Attribution Notice reasonably calculated to inform recipients that
   You have modified the Original Work.

   (( 7) ))??
   Warranty of Provenance and Disclaimer of Warranty. Licensor warrants that the
   copyright in and to the Original Work and the patent rights granted herein by
   Licensor are owned by the Licensor or are sublicensed to You under the terms
   of this License with the permission of the contributor(s) of those copyrights
   and patent rights. Except as expressly stated in the immediately preceding
   sentence, the Original Work is provided under this License on an "AS IS"
   BASIS and WITHOUT WARRANTY, either express or implied, including, without
   limitation, the warranties of non-infringement, merchantability or fitness
   for a particular purpose. THE ENTIRE RISK AS TO THE QUALITY OF THE ORIGINAL
   WORK IS WITH YOU. This DISCLAIMER OF WARRANTY constitutes an essential part
   of this License. No license to the Original Work is granted by this License
   except under this disclaimer.

   (( 8) ))??
   Limitation of Liability. Under no circumstances and under no legal theory,
   whether in tort (including negligence), contract, or otherwise, shall the
   Licensor be liable to anyone for any indirect, special, incidental, or
   consequential damages of any character arising as a result of this License or
   the use of the Original Work including, without limitation, damages for loss
   of goodwill, work stoppage, computer failure or malfunction, or any and all
   other commercial damages or losses. This limitation of liability shall not
   apply to the extent applicable law prohibits such limitation.

   (( 9) ))??
   Acceptance and Termination. If, at any time, You expressly assented to this
   License, that assent indicates your clear and irrevocable acceptance of this
   License and all of its terms and conditions. If You distribute or communicate
   copies of the Original Work or a Derivative Work, You must make a reasonable
   effort under the circumstances to obtain the express assent of recipients to
   the terms of this License. This License conditions your rights to undertake
   the activities listed in Section 1, including your right to create Derivative
   Works based upon the Original Work, and doing so without honoring these terms
   and conditions is prohibited by copyright law and international treaty.
   Nothing in this License is intended to affect copyright exceptions and
   limitations (including "fair use" or "fair dealing"). This License shall
   terminate immediately and You may no longer exercise any of the rights
   granted to You by this License upon your failure to honor the conditions in
   Section 1(c).

   (( 10) ))??
   Termination for Patent Action. This License shall terminate automatically and
   You may no longer exercise any of the rights granted to You by this License
   as of the date You commence an action, including a cross-claim or
   counterclaim, against Licensor or any licensee alleging that the Original
   Work infringes a patent. This termination provision shall not apply for an
   action alleging patent infringement by combinations of the Original Work with
   other software or hardware.

   (( 11) ))??
   Jurisdiction, Venue and Governing Law. Any action or suit relating to this
   License may be brought only in the courts of a jurisdiction wherein the
   Licensor resides or in which Licensor conducts its primary business, and
   under the laws of that jurisdiction excluding its conflict-of-law provisions.
   The application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any use of the Original
   Work outside the scope of this License or after its termination shall be
   subject to the requirements and penalties of copyright or patent law in the
   appropriate jurisdiction. This section shall survive the termination of this
   License.

   (( 12) ))??
   Attorneys' Fees. In any action to enforce the terms of this License or
   seeking damages relating thereto, the prevailing party shall be entitled to
   recover its costs and expenses, including, without limitation, reasonable
   attorneys' fees and costs incurred in connection with such action, including
   any appeal of such action. This section shall survive the termination of this
   License.

   (( 13) ))??
   Miscellaneous. If any provision of this License is held to be unenforceable,
   such provision shall be reformed only to the extent necessary to make it
   enforceable.

   (( 14) ))??
   Definition of "You" in This License. "You" throughout this License, whether
   in upper or lower case, means an individual or a legal entity exercising
   rights under, and complying with all of the terms of, this License. For legal
   entities, "You" includes any entity that controls, is controlled by, or is
   under common control with you. For purposes of this definition, "control"
   means (i) the power, direct or indirect, to cause the direction or management
   of such entity, whether by contract or otherwise, or (ii) ownership of fifty
   percent (50%) or more of the outstanding shares, or (iii) beneficial
   ownership of such entity.

   (( 15) ))??
   Right to Use. You may use the Original Work in all ways not otherwise
   restricted or conditioned by this License or by law, and Licensor promises
   not to interfere with or be responsible for such uses by You.

   (( 16) ))??
   Modification of This License. This License is Copyright © 2005 Lawrence
   Rosen. Permission is granted to copy, distribute, or communicate this License
   without modification. Nothing in this License permits You to modify this
   License as applied to the Original Work or to Derivative Works. However, You
   may modify the text of this License and copy, distribute or communicate your
   modified version (the "Modified License") and apply it to other original
   works of authorship subject to the following conditions: (i) You may not
   indicate in any way that your Modified License is the "Academic Free License"
   or "AFL" and you may not use those names in the name of your Modified
   License; (ii) You must replace the notice specified in the first paragraph
   above with the notice "Licensed under <insert your license name here>" or
   with a notice of your own that is not confusingly similar to the notice in
   this License; and (iii) You may not claim that your original works are open
   source software unless your Modified License has been approved by Open Source
   Initiative (OSI) and You comply with its license review and certification
   process.
`
const license_AGPL_1_0_lre = `//**
Affero General Public License v1.0
http://www.affero.org/oagpl.html
**//

((  AFFERO GENERAL PUBLIC LICENSE

Version 1, March 2002 ))??

Copyright © 2002 Affero Inc.

510 Third Street - Suite 225, San Francisco, CA 94107, USA

This license is a modified version of the GNU General Public License copyright
(C) 1989, 1991 Free Software Foundation, Inc. made with their permission.
Section 2(d) has been added to cover use of software over a computer network.

Everyone is permitted to copy and distribute verbatim copies of this license
document, but changing it is not allowed.

Preamble

The licenses for most software are designed to take away your freedom to share
and change it. By contrast, the Affero General Public License is intended to
guarantee your freedom to share and change free software--to make sure the
software is free for all its users. This Public License applies to most of
Affero's software and to any other program whose authors commit to using it.
(Some other Affero software is covered by the GNU Library General Public License
instead.) You can apply it to your programs, too.

When we speak of free software, we are referring to freedom, not price. This
General Public License is designed to make sure that you have the freedom to
distribute copies of free software (and charge for this service if you wish),
that you receive source code or can get it if you want it, that you can change
the software or use pieces of it in new free programs; and that you know you can
do these things.

To protect your rights, we need to make restrictions that forbid anyone to deny
you these rights or to ask you to surrender the rights. These restrictions
translate to certain responsibilities for you if you distribute copies of the
software, or if you modify it.

For example, if you distribute copies of such a program, whether gratis or for a
fee, you must give the recipients all the rights that you have. You must make
sure that they, too, receive or can get the source code. And you must show them
these terms so they know their rights.

We protect your rights with two steps: (1) copyright the software, and (2) offer
you this license which gives you legal permission to copy, distribute and/or
modify the software.

Also, for each author's protection and ours, we want to make certain that
everyone understands that there is no warranty for this free software. If the
software is modified by someone else and passed on, we want its recipients to
know that what they have is not the original, so that any problems introduced by
others will not reflect on the original authors' reputations.

Finally, any free program is threatened constantly by software patents. We wish
to avoid the danger that redistributors of a free program will individually
obtain patent licenses, in effect making the program proprietary. To prevent
this, we have made it clear that any patent must be licensed for everyone's free
use or not licensed at all.

The precise terms and conditions for copying, distribution and modification
follow.

TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

   __1__ This License applies to any program or other work which contains a
   notice placed by the copyright holder saying it may be distributed under the
   terms of this Affero General Public License. The "Program", below, refers to
   any such program or work, and a "work based on the Program" means either the
   Program or any derivative work under copyright law: that is to say, a work
   containing the Program or a portion of it, either verbatim or with
   modifications and/or translated into another language. (Hereinafter,
   translation is included without limitation in the term "modification".) Each
   licensee is addressed as "you".

   Activities other than copying, distribution and modification are not covered
   by this License; they are outside its scope. The act of running the Program
   is not restricted, and the output from the Program is covered only if its
   contents constitute a work based on the Program (independent of having been
   made by running the Program). Whether that is true depends on what the
   Program does.

   __1__ You may copy and distribute verbatim copies of the Program's source
   code as you receive it, in any medium, provided that you conspicuously and
   appropriately publish on each copy an appropriate copyright notice and
   disclaimer of warranty; keep intact all the notices that refer to this
   License and to the absence of any warranty; and give any other recipients of
   the Program a copy of this License along with the Program.

   You may charge a fee for the physical act of transferring a copy, and you may
   at your option offer warranty protection in exchange for a fee.

   __1__ You may modify your copy or copies of the Program or any portion of it,
   thus forming a work based on the Program, and copy and distribute such
   modifications or work under the terms of Section 1 above, provided that you
   also meet all of these conditions:

      __1__ You must cause the modified files to carry prominent notices stating
      that you changed the files and the date of any change.

      __1__ You must cause any work that you distribute or publish, that in
      whole or in part contains or is derived from the Program or any part
      thereof, to be licensed as a whole at no charge to all third parties under
      the terms of this License.

      __1__ If the modified program normally reads commands interactively when
      run, you must cause it, when started running for such interactive use in
      the most ordinary way, to print or display an announcement including an
      appropriate copyright notice and a notice that there is no warranty (or
      else, saying that you provide a warranty) and that users may redistribute
      the program under these conditions, and telling the user how to view a
      copy of this License. (Exception: if the Program itself is interactive but
      does not normally print such an announcement, your work based on the
      Program is not required to print an announcement.)

      __1__ If the Program as you received it is intended to interact with users
      through a computer network and if, in the version you received, any user
      interacting with the Program was given the opportunity to request
      transmission to that user of the Program's complete source code, you must
      not remove that facility from your modified version of the Program or work
      based on the Program, and must offer an equivalent opportunity for all
      users interacting with your Program through a computer network to request
      immediate transmission by HTTP of the complete source code of your
      modified version or other derivative work.

   These requirements apply to the modified work as a whole. If identifiable
   sections of that work are not derived from the Program, and can be reasonably
   considered independent and separate works in themselves, then this License,
   and its terms, do not apply to those sections when you distribute them as
   separate works. But when you distribute the same sections as part of a whole
   which is a work based on the Program, the distribution of the whole must be
   on the terms of this License, whose permissions for other licensees extend to
   the entire whole, and thus to each and every part regardless of who wrote
   it.

   Thus, it is not the intent of this section to claim rights or contest your
   rights to work written entirely by you; rather, the intent is to exercise the
   right to control the distribution of derivative or collective works based on
   the Program.

   In addition, mere aggregation of another work not based on the Program with
   the Program (or with a work based on the Program) on a volume of a storage or
   distribution medium does not bring the other work under the scope of this
   License.

   __1__ You may copy and distribute the Program (or a work based on it, under
   Section 2) in object code or executable form under the terms of Sections 1
   and 2 above provided that you also do one of the following:

      __1__ Accompany it with the complete corresponding machine-readable source
      code, which must be distributed under the terms of Sections 1 and 2 above
      on a medium customarily used for software interchange; or,

      __1__ Accompany it with a written offer, valid for at least three years,
      to give any third party, for a charge no more than your cost of physically
      performing source distribution, a complete machine-readable copy of the
      corresponding source code, to be distributed under the terms of Sections 1
      and 2 above on a medium customarily used for software interchange; or,

      __1__ Accompany it with the information you received as to the offer to
      distribute corresponding source code. (This alternative is allowed only
      for noncommercial distribution and only if you received the program in
      object code or executable form with such an offer, in accord with
      Subsection b above.)

   The source code for a work means the preferred form of the work for making
   modifications to it. For an executable work, complete source code means all
   the source code for all modules it contains, plus any associated interface
   definition files, plus the scripts used to control compilation and
   installation of the executable. However, as a special exception, the source
   code distributed need not include anything that is normally distributed (in
   either source or binary form) with the major components (compiler, kernel,
   and so on) of the operating system on which the executable runs, unless that
   component itself accompanies the executable.

   If distribution of executable or object code is made by offering access to
   copy from a designated place, then offering equivalent access to copy the
   source code from the same place counts as distribution of the source code,
   even though third parties are not compelled to copy the source along with the
   object code.

   __1__ You may not copy, modify, sublicense, or distribute the Program except
   as expressly provided under this License. Any attempt otherwise to copy,
   modify, sublicense or distribute the Program is void, and will automatically
   terminate your rights under this License. However, parties who have received
   copies, or rights, from you under this License will not have their licenses
   terminated so long as such parties remain in full compliance.

   __1__ You are not required to accept this License, since you have not signed
   it. However, nothing else grants you permission to modify or distribute the
   Program or its derivative works. These actions are prohibited by law if you
   do not accept this License. Therefore, by modifying or distributing the
   Program (or any work based on the Program), you indicate your acceptance of
   this License to do so, and all its terms and conditions for copying,
   distributing or modifying the Program or works based on it.

   __1__ Each time you redistribute the Program (or any work based on the
   Program), the recipient automatically receives a license from the original
   licensor to copy, distribute or modify the Program subject to these terms and
   conditions. You may not impose any further restrictions on the recipients'
   exercise of the rights granted herein. You are not responsible for enforcing
   compliance by third parties to this License.

   __1__ If, as a consequence of a court judgment or allegation of patent
   infringement or for any other reason (not limited to patent issues),
   conditions are imposed on you (whether by court order, agreement or
   otherwise) that contradict the conditions of this License, they do not excuse
   you from the conditions of this License. If you cannot distribute so as to
   satisfy simultaneously your obligations under this License and any other
   pertinent obligations, then as a consequence you may not distribute the
   Program at all. For example, if a patent license would not permit
   royalty-free redistribution of the Program by all those who receive copies
   directly or indirectly through you, then the only way you could satisfy both
   it and this License would be to refrain entirely from distribution of the
   Program.

   If any portion of this section is held invalid or unenforceable under any
   particular circumstance, the balance of the section is intended to apply and
   the section as a whole is intended to apply in other circumstances.

   It is not the purpose of this section to induce you to infringe any patents
   or other property right claims or to contest validity of any such claims;
   this section has the sole purpose of protecting the integrity of the free
   software distribution system, which is implemented by public license
   practices. Many people have made generous contributions to the wide range of
   software distributed through that system in reliance on consistent
   application of that system; it is up to the author/donor to decide if he or
   she is willing to distribute software through any other system and a licensee
   cannot impose that choice.

   This section is intended to make thoroughly clear what is believed to be a
   consequence of the rest of this License.

   __1__ If the distribution and/or use of the Program is restricted in certain
   countries either by patents or by copyrighted interfaces, the original
   copyright holder who places the Program under this License may add an
   explicit geographical distribution limitation excluding those countries, so
   that distribution is permitted only in or among countries not thus excluded.
   In such case, this License incorporates the limitation as if written in the
   body of this License.

   __1__ Affero Inc. may publish revised and/or new versions of the Affero
   General Public License from time to time. Such new versions will be similar
   in spirit to the present version, but may differ in detail to address new
   problems or concerns.

   Each version is given a distinguishing version number. If the Program
   specifies a version number of this License which applies to it and "any later
   version", you have the option of following the terms and conditions either of
   that version or of any later version published by Affero, Inc. If the Program
   does not specify a version number of this License, you may choose any version
   ever published by Affero, Inc.

   You may also choose to redistribute modified versions of this program under
   any version of the Free Software Foundation's GNU General Public License
   version 3 or higher, so long as that version of the GNU GPL includes terms
   and conditions substantially equivalent to those of this license.

   __1__ If you wish to incorporate parts of the Program into other free
   programs whose distribution conditions are different, write to the author to
   ask for permission. For software which is copyrighted by Affero, Inc., write
   to us; we sometimes make exceptions for this. Our decision will be guided by
   the two goals of preserving the free status of all derivatives of our free
   software and of promoting the sharing and reuse of software generally.

   NO WARRANTY

   __1__ BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
   FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
   OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
   PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR
   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO
   THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM
   PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR
   CORRECTION.

   __1__ IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
   WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
   REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
   INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
   OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO
   LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR
   THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
   PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGES.
`
const license_AGPL_1_0_only_lre = `
//**
https://spdx.org/licenses/AGPL-1.0-only.json
This header is an anachronism - AGPL 1.0 did not define a header.
**//

	
	 
	 
	 
	 
	 

	((
		((This
		program
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	GNU Affero
	General Public License
	((AGPL))??
	((as published by the Free Software Foundation))??
	
		((under))??
		version 1
		((AGPLv1))??
		((of the License))??
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		GNU Affero
		General Public License version
		1
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		GNU Affero
		General Public License for more details.

		((
			You should have received a copy of the
			GNU Affero
			General Public License
			((
				((version))??
				1
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_AGPL_1_0_or_later_lre = `
//**
https://spdx.org/licenses/AGPL-1.0-only.json
This header is an anachronism - AGPL 1.0 did not define a header.
**//

	
	 
	 
	 
	 
	 

	((
		((This
		program
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	GNU Affero
	General Public License
	((AGPL))??
	((as published by the Free Software Foundation))??
	
		((
			either version 1
			((of the License))??
			or
			((at your option))??
			any later version.
		||
			version 1 or later
			((of the License))??
		))
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		GNU Affero
		General Public License version
		1
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		GNU Affero
		General Public License for more details.

		((
			You should have received a copy of the
			GNU Affero
			General Public License
			((
				((version))??
				1
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_AGPL_3_0_lre = `//**
GNU Affero General Public License v3.0
https://www.gnu.org/licenses/agpl.txt
https://opensource.org/licenses/AGPL-3.0
**//

((
	GNU AFFERO GENERAL PUBLIC LICENSE Version 3, 19 November 2007
	
	((
		((
			Copyright __20__
			((<https://fsf.org/>))??
			
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))??
		))??

		Everyone is permitted to copy and distribute verbatim copies
		of this license document, but changing it is not allowed.

		((Copyright __20__))??
	))??

))??


Preamble

The GNU Affero General Public License is a free, copyleft license for software
and other kinds of works, specifically designed to ensure cooperation with the
community in the case of network server software.

The licenses for most software and other practical works are designed to take
away your freedom to share and change the works. By contrast, our General Public
Licenses are intended to guarantee your freedom to share and change all versions
of a program--to make sure it remains free software for all its users.

When we speak of free software, we are referring to freedom, not price. Our
General Public Licenses are designed to make sure that you have the freedom to
distribute copies of free software (and charge for them if you wish), that you
receive source code or can get it if you want it, that you can change the
software or use pieces of it in new free programs, and that you know you can do
these things.

Developers that use our General Public Licenses protect your rights with two
steps: (1) assert copyright on the software, and (2) offer you this License
which gives you legal permission to copy, distribute and/or modify the
software.

A secondary benefit of defending all users' freedom is that improvements made in
alternate versions of the program, if they receive widespread use, become
available for other developers to incorporate. Many developers of free software
are heartened and encouraged by the resulting cooperation. However, in the case
of software used on network servers, this result may fail to come about. The GNU
General Public License permits making a modified version and letting the public
access it on a server without ever releasing its source code to the public.

The GNU Affero General Public License is designed specifically to ensure that,
in such cases, the modified source code becomes available to the community. It
requires the operator of a network server to provide the source code of the
modified version running there to the users of that server. Therefore, public
use of a modified version, on a publicly accessible server, gives the public
access to the source code of the modified version.

An older license, called the Affero General Public License and published by
Affero, was designed to accomplish similar goals. This is a different license,
not a version of the Affero GPL, but Affero has released a new version of the
Affero GPL which permits relicensing under this license.

The precise terms and conditions for copying, distribution and modification
follow.

TERMS AND CONDITIONS

   __1__ Definitions.

   "This License" refers to version 3 of the GNU Affero General Public License.

   "Copyright" also means copyright-like laws that apply to other kinds of
   works, such as semiconductor masks.

   "The Program" refers to any copyrightable work licensed under this License.
   Each licensee is addressed as "you". "Licensees" and "recipients" may be
   individuals or organizations.

   To "modify" a work means to copy from or adapt all or part of the work in a
   fashion requiring copyright permission, other than the making of an exact
   copy. The resulting work is called a "modified version" of the earlier work
   or a work "based on" the earlier work.

   A "covered work" means either the unmodified Program or a work based on the
   Program.

   To "propagate" a work means to do anything with it that, without permission,
   would make you directly or secondarily liable for infringement under
   applicable copyright law, except executing it on a computer or modifying a
   private copy. Propagation includes copying, distribution (with or without
   modification), making available to the public, and in some countries other
   activities as well.

   To "convey" a work means any kind of propagation that enables other parties
   to make or receive copies. Mere interaction with a user through a computer
   network, with no transfer of a copy, is not conveying.

   An interactive user interface displays "Appropriate Legal Notices" to the
   extent that it includes a convenient and prominently visible feature that (1)
   displays an appropriate copyright notice, and (2) tells the user that there
   is no warranty for the work (except to the extent that warranties are
   provided), that licensees may convey the work under this License, and how to
   view a copy of this License. If the interface presents a list of user
   commands or options, such as a menu, a prominent item in the list meets this
   criterion.

   __1__ Source Code.

   The "source code" for a work means the preferred form of the work for making
   modifications to it. "Object code" means any non-source form of a work.

   A "Standard Interface" means an interface that either is an official standard
   defined by a recognized standards body, or, in the case of interfaces
   specified for a particular programming language, one that is widely used
   among developers working in that language.

   The "System Libraries" of an executable work include anything, other than the
   work as a whole, that (a) is included in the normal form of packaging a Major
   Component, but which is not part of that Major Component, and (b) serves only
   to enable use of the work with that Major Component, or to implement a
   Standard Interface for which an implementation is available to the public in
   source code form. A "Major Component", in this context, means a major
   essential component (kernel, window system, and so on) of the specific
   operating system (if any) on which the executable work runs, or a compiler
   used to produce the work, or an object code interpreter used to run it.

   The "Corresponding Source" for a work in object code form means all the
   source code needed to generate, install, and (for an executable work) run the
   object code and to modify the work, including scripts to control those
   activities. However, it does not include the work's System Libraries, or
   general-purpose tools or generally available free programs which are used
   unmodified in performing those activities but which are not part of the work.
   For example, Corresponding Source includes interface definition files
   associated with source files for the work, and the source code for shared
   libraries and dynamically linked subprograms that the work is specifically
   designed to require, such as by intimate data communication or control flow
   between those

   subprograms and other parts of the work.

   The Corresponding Source need not include anything that users can regenerate
   automatically from other parts of the Corresponding Source.

   The Corresponding Source for a work in source code form is that same work.

   __1__ Basic Permissions.

   All rights granted under this License are granted for the term of copyright
   on the Program, and are irrevocable provided the stated conditions are met.
   This License explicitly affirms your unlimited permission to run the
   unmodified Program. The output from running a covered work is covered by this
   License only if the output, given its content, constitutes a covered work.
   This License acknowledges your rights of fair use or other equivalent, as
   provided by copyright law.

   You may make, run and propagate covered works that you do not convey, without
   conditions so long as your license otherwise remains in force. You may convey
   covered works to others for the sole purpose of having them make
   modifications exclusively for you, or provide you with facilities for running
   those works, provided that you comply with the terms of this License in
   conveying all material for which you do not control copyright. Those thus
   making or running the covered works for you must do so exclusively on your
   behalf, under your direction and control, on terms that prohibit them from
   making any copies of your copyrighted material outside their relationship
   with you.

   Conveying under any other circumstances is permitted solely under the
   conditions stated below. Sublicensing is not allowed; section 10 makes it
   unnecessary.

   __1__ Protecting Users' Legal Rights From Anti-Circumvention Law.

   No covered work shall be deemed part of an effective technological measure
   under any applicable law fulfilling obligations under article 11 of the WIPO
   copyright treaty adopted on 20 December 1996, or similar laws prohibiting or
   restricting circumvention of such measures.

   When you convey a covered work, you waive any legal power to forbid
   circumvention of technological measures to the extent such circumvention is
   effected by exercising rights under this License with respect to the covered
   work, and you disclaim any intention to limit operation or modification of
   the work as a means of enforcing, against the work's users, your or third
   parties' legal rights to forbid circumvention of technological measures.

   __1__ Conveying Verbatim Copies.

   You may convey verbatim copies of the Program's source code as you receive
   it, in any medium, provided that you conspicuously and appropriately publish
   on each copy an appropriate copyright notice; keep intact all notices stating
   that this License and any non-permissive terms added in accord with section 7
   apply to the code; keep intact all notices of the absence of any warranty;
   and give all recipients a copy of this License along with the Program.

   You may charge any price or no price for each copy that you convey, and you
   may offer support or warranty protection for a fee.

   __1__ Conveying Modified Source Versions.

   You may convey a work based on the Program, or the modifications to produce
   it from the Program, in the form of source code under the terms of section 4,
   provided that you also meet all of these conditions:

      __1__ The work must carry prominent notices stating that you modified it,
      and giving a relevant date.

      __1__ The work must carry prominent notices stating that it is released
      under this License and any conditions added under section 7. This
      requirement modifies the requirement in section 4 to "keep intact all
      notices".

      __1__ You must license the entire work, as a whole, under this License to
      anyone who comes into possession of a copy. This License will therefore
      apply, along with any applicable section 7 additional terms, to the whole
      of the work, and all its parts, regardless of how they are packaged. This
      License gives no permission to license the work in any other way, but it
      does not invalidate such permission if you have separately received it.

      __1__ If the work has interactive user interfaces, each must display
      Appropriate Legal Notices; however, if the Program has interactive
      interfaces that do not display Appropriate Legal Notices, your work need
      not make them do so.

   A compilation of a covered work with other separate and independent works,
   which are not by their nature extensions of the covered work, and which are
   not combined with it such as to form a larger program, in or on a volume of a
   storage or distribution medium, is called an "aggregate" if the compilation
   and its resulting copyright are not used to limit the access or legal rights
   of the compilation's users beyond what the individual works permit. Inclusion
   of a covered work in an aggregate does not cause this License to apply to the
   other parts of the aggregate.

   __1__ Conveying Non-Source Forms.

   You may convey a covered work in object code form under the terms of sections
   4 and 5, provided that you also convey the machine-readable Corresponding
   Source under the terms of this License, in one of these ways:

      __1__ Convey the object code in, or embodied in, a physical product
      (including a physical distribution medium), accompanied by the
      Corresponding Source fixed on a durable physical medium customarily used
      for software interchange.

      __1__ Convey the object code in, or embodied in, a physical product
      (including a physical distribution medium), accompanied by a written
      offer, valid for at least three years and valid for as long as you offer
      spare parts or customer support for that product model, to give anyone who
      possesses the object code either (1) a copy of the Corresponding Source
      for all the software in the product that is covered by this License, on a
      durable physical medium customarily used for software interchange, for a
      price no more than your reasonable cost of physically performing this
      conveying of source, or (2) access to copy the Corresponding Source from a
      network server at no charge.

      __1__ Convey individual copies of the object code with a copy of the
      written offer to provide the Corresponding Source. This alternative is
      allowed only occasionally and noncommercially, and only if you received
      the object code with such an offer, in accord with subsection 6b.

      __1__ Convey the object code by offering access from a designated place
      (gratis or for a charge), and offer equivalent access to the Corresponding
      Source in the same way through the same place at no further charge. You
      need not require recipients to copy the Corresponding Source along with
      the object code. If the place to copy the object code is a network server,
      the Corresponding Source may be on a different server (operated by you or
      a third party) that supports equivalent copying facilities, provided you
      maintain clear directions next to the object code saying where to find the
      Corresponding Source. Regardless of what server hosts the Corresponding
      Source, you remain obligated to ensure that it is available for as long as
      needed to satisfy these requirements.

      __1__ Convey the object code using peer-to-peer transmission, provided you
      inform other peers where the object code and Corresponding Source of the
      work are being offered to the general public at no charge under subsection
      6d.

   A separable portion of the object code, whose source code is excluded from
   the Corresponding Source as a System Library, need not be included in
   conveying the object code work.

   A "User Product" is either (1) a "consumer product", which means any tangible
   personal property which is normally used for personal, family, or household
   purposes, or (2) anything designed or sold for incorporation into a dwelling.
   In determining whether a product is a consumer product, doubtful cases shall
   be resolved in favor of coverage. For a particular product received by a
   particular user, "normally used" refers to a typical or common use of that
   class of product, regardless of the status of the particular user or of the
   way in which the particular user actually uses, or expects or is expected to
   use, the product. A product is a consumer product regardless of whether the
   product has substantial commercial, industrial or non-consumer uses, unless
   such uses represent the only significant mode of use of the product.

   "Installation Information" for a User Product means any methods, procedures,
   authorization keys, or other information required to install and execute
   modified versions of a covered work in that User Product from a modified
   version of its Corresponding Source. The information must suffice to ensure
   that the continued functioning of the modified object code is in no case
   prevented or interfered with solely because modification has been made.

   If you convey an object code work under this section in, or with, or
   specifically for use in, a User Product, and the conveying occurs as part of
   a transaction in which the right of possession and use of the User Product is
   transferred to the recipient in perpetuity or for a fixed term (regardless of
   how the transaction is characterized), the Corresponding Source conveyed
   under this section must be accompanied by the Installation Information. But
   this requirement does not apply if neither you nor any third party retains
   the ability to install modified object code on the User Product (for example,
   the work has been installed in ROM).

   The requirement to provide Installation Information does not include a
   requirement to continue to provide support service, warranty, or updates for
   a work that has been modified or installed by the recipient, or for the User
   Product in which it has been modified or installed. Access to a network may
   be denied when the modification itself materially and adversely affects the
   operation of the network or violates the rules and protocols for
   communication across the network.

   Corresponding Source conveyed, and Installation Information provided, in
   accord with this section must be in a format that is publicly documented (and
   with an implementation available to the public in source code form), and must
   require no special password or key for unpacking, reading or copying.

   __1__ Additional Terms.

   "Additional permissions" are terms that supplement the terms of this License
   by making exceptions from one or more of its conditions. Additional
   permissions that are applicable to the entire Program shall be treated as
   though they were included in this License, to the extent that they are valid
   under applicable law. If additional permissions apply only to part of the
   Program, that part may be used separately under those permissions, but the
   entire Program remains governed by this License without regard to the
   additional permissions.

   When you convey a copy of a covered work, you may at your option remove any
   additional permissions from that copy, or from any part of it. (Additional
   permissions may be written to require their own removal in certain cases when
   you modify the work.) You may place additional permissions on material, added
   by you to a covered work, for which you have or can give appropriate
   copyright permission.

   Notwithstanding any other provision of this License, for material you add to
   a covered work, you may (if authorized by the copyright holders of that
   material) supplement the terms of this License with terms:

      __1__ Disclaiming warranty or limiting liability differently from the
      terms of sections 15 and 16 of this License; or

      __1__ Requiring preservation of specified reasonable legal notices or
      author attributions in that material or in the Appropriate Legal Notices
      displayed by works containing it; or

      __1__ Prohibiting misrepresentation of the origin of that material, or
      requiring that modified versions of such material be marked in reasonable
      ways as different from the original version; or

      __1__ Limiting the use for publicity purposes of names of licensors or
      authors of the material; or

      __1__ Declining to grant rights under trademark law for use of some trade
      names, trademarks, or service marks; or

      __1__ Requiring indemnification of licensors and authors of that material
      by anyone who conveys the material (or modified versions of it) with
      contractual assumptions of liability to the recipient, for any liability
      that these contractual assumptions directly impose on those licensors and
      authors.

   All other non-permissive additional terms are considered "further
   restrictions" within the meaning of section 10. If the Program as you
   received it, or any part of it, contains a notice stating that it is governed
   by this License along with a term that is a further restriction, you may
   remove that term. If a license document contains a further restriction but
   permits relicensing or conveying under this License, you may add to a covered
   work material governed by the terms of that license document, provided that
   the further restriction does not survive such relicensing or conveying.

   If you add terms to a covered work in accord with this section, you must
   place, in the relevant source files, a statement of the additional terms that
   apply to those files, or a notice indicating where to find the applicable
   terms.

   Additional terms, permissive or non-permissive, may be stated in the form of
   a separately written license, or stated as exceptions; the above requirements
   apply either way.

   __1__ Termination.

   You may not propagate or modify a covered work except as expressly provided
   under this License. Any attempt otherwise to propagate or modify it is void,
   and will automatically terminate your rights under this License (including
   any patent licenses granted under the third paragraph of section 11).

   However, if you cease all violation of this License, then your license from a
   particular copyright holder is reinstated (a) provisionally, unless and until
   the copyright holder explicitly and finally terminates your license, and (b)
   permanently, if the copyright holder fails to notify you of the violation by
   some reasonable means prior to 60 days after the cessation.

   Moreover, your license from a particular copyright holder is reinstated
   permanently if the copyright holder notifies you of the violation by some
   reasonable means, this is the first time you have received notice of
   violation of this License (for any work) from that copyright holder, and you
   cure the violation prior to 30 days after your receipt of the notice.

   Termination of your rights under this section does not terminate the licenses
   of parties who have received copies or rights from you under this License. If
   your rights have been terminated and not permanently reinstated, you do not
   qualify to receive new licenses for the same material under section 10.

   __1__ Acceptance Not Required for Having Copies.

   You are not required to accept this License in order to receive or run a copy
   of the Program. Ancillary propagation of a covered work occurring solely as a
   consequence of using peer-to-peer transmission to receive a copy likewise
   does not require acceptance. However, nothing other than this License grants
   you permission to propagate or modify any covered work. These actions
   infringe copyright if you do not accept this License. Therefore, by modifying
   or propagating a covered work, you indicate your acceptance of this License
   to do so.

   __1__ Automatic Licensing of Downstream Recipients.

   Each time you convey a covered work, the recipient automatically receives a
   license from the original licensors, to run, modify and propagate that work,
   subject to this License. You are not responsible for enforcing compliance by
   third parties with this License.

   An "entity transaction" is a transaction transferring control of an
   organization, or substantially all assets of one, or subdividing an
   organization, or merging organizations. If propagation of a covered work
   results from an entity transaction, each party to that transaction who
   receives a copy of the work also receives whatever licenses to the work the
   party's predecessor in interest had or could give under the previous
   paragraph, plus a right to possession of the Corresponding Source of the work
   from the predecessor in interest, if the predecessor has it or can get it
   with reasonable efforts.

   You may not impose any further restrictions on the exercise of the rights
   granted or affirmed under this License. For example, you may not impose a
   license fee, royalty, or other charge for exercise of rights granted under
   this License, and you may not initiate litigation (including a cross-claim or
   counterclaim in a lawsuit) alleging that any patent claim is infringed by
   making, using, selling, offering for sale, or importing the Program or any
   portion of it.

   __1__ Patents.

   A "contributor" is a copyright holder who authorizes use under this License
   of the Program or a work on which the Program is based. The work thus
   licensed is called the contributor's "contributor version".

   A contributor's "essential patent claims" are all patent claims owned or
   controlled by the contributor, whether already acquired or hereafter
   acquired, that would be infringed by some manner, permitted by this License,
   of making, using, or selling its contributor version, but do not include
   claims that would be infringed only as a consequence of further modification
   of the contributor version. For purposes of this definition, "control"
   includes the right to grant patent sublicenses in a manner consistent with
   the requirements of this License.

   Each contributor grants you a non-exclusive, worldwide, royalty-free patent
   license under the contributor's essential patent claims, to make, use, sell,
   offer for sale, import and otherwise run, modify and propagate the contents
   of its contributor version.

   In the following three paragraphs, a "patent license" is any express
   agreement or commitment, however denominated, not to enforce a patent (such
   as an express permission to practice a patent or covenant not to s ue for
   patent infringement). To "grant" such a patent license to a party means to
   make such an agreement or commitment not to enforce a patent against the
   party.

   If you convey a covered work, knowingly relying on a patent license, and the
   Corresponding Source of the work is not available for anyone to copy, free of
   charge and under the terms of this License, through a publicly available
   network server or other readily accessible means, then you must either (1)
   cause the Corresponding Source to be so available, or (2) arrange to deprive
   yourself of the benefit of the patent license for this particular work, or
   (3) arrange, in a manner consistent with the requirements of this License, to
   extend the patent

   license to downstream recipients. "Knowingly relying" means you have actual
   knowledge that, but for the patent license, your conveying the covered work
   in a country, or your recipient's use of the covered work in a country, would
   infringe one or more identifiable patents in that country that you have
   reason to believe are valid.

   If, pursuant to or in connection with a single transaction or arrangement,
   you convey, or propagate by procuring conveyance of, a covered work, and
   grant a patent license to some of the parties receiving the covered work
   authorizing them to use, propagate, modify or convey a specific copy of the
   covered work, then the patent license you grant is automatically extended to
   all recipients of the covered work and works based on it.

   A patent license is "discriminatory" if it does not include within the scope
   of its coverage, prohibits the exercise of, or is conditioned on the
   non-exercise of one or more of the rights that are specifically granted under
   this License. You may not convey a covered work if you are a party to an
   arrangement with a third party that is in the business of distributing
   software, under which you make payment to the third party based on the extent
   of your activity of conveying the work, and under which the third party
   grants, to any of the parties who would receive the covered work from you, a
   discriminatory patent license (a) in connection with copies of the covered
   work conveyed by you (or copies made from those copies), or (b) primarily for
   and in connection with specific products or compilations that contain the
   covered work, unless you entered into that arrangement, or that patent
   license was granted, prior to 28 March 2007.

   Nothing in this License shall be construed as excluding or limiting any
   implied license or other defenses to infringement that may otherwise be
   available to you under applicable patent law.

   __1__ No Surrender of Others' Freedom.

   If conditions are imposed on you (whether by court order, agreement or
   otherwise) that contradict the conditions of this License, they do not excuse
   you from the conditions of this License. If you cannot convey a covered work
   so as to satisfy simultaneously your obligations under this License and any
   other pertinent obligations, then as a consequence you may

   not convey it at all. For example, if you agree to terms that obligate you to
   collect a royalty for further conveying from those to whom you convey the
   Program, the only way you could satisfy both those terms and this License
   would be to refrain entirely from conveying the Program.

   __1__ Remote Network Interaction; Use with the GNU General Public License.

   Notwithstanding any other provision of this License, if you modify the
   Program, your modified version must prominently offer all users interacting
   with it remotely through a computer network (if your version supports such
   interaction) an opportunity to receive the Corresponding Source of your
   version by providing access to the Corresponding Source from a network server
   at no charge, through some standard or customary means of facilitating
   copying of software. This Corresponding Source shall include the
   Corresponding Source for any work covered by version 3 of the GNU General
   Public License that is incorporated pursuant to the following paragraph.

   Notwithstanding any other provision of this License, you have permission to
   link or combine any covered work with a work licensed under version 3 of the
   GNU General Public License into a single combined work, and to convey the
   resulting work. The terms of this License will continue to apply to the part
   which is the covered work, but the work with which it is combined will remain
   governed by version 3 of the GNU General Public License.

   __1__ Revised Versions of this License.

   The Free Software Foundation may publish revised and/or new versions of the
   GNU Affero General Public License from time to time. Such new versions will
   be similar in spirit to the present version, but may differ in detail to
   address new problems or concerns.

   Each version is given a distinguishing version number. If the Program
   specifies that a certain numbered version of the GNU Affero General Public
   License "or any later version" applies to it, you have the option of
   following the terms and conditions either of that numbered version or of any
   later version published by the Free Software Foundation. If the Program does
   not specify a version number of the GNU Affero General Public License, you
   may choose any version ever published by the Free Software Foundation.

   If the Program specifies that a proxy can decide which future versions of the
   GNU Affero General Public License can be used, that proxy's public statement
   of acceptance of a version permanently authorizes you to choose that version
   for the Program.

   Later license versions may give you additional or different permissions.
   However, no additional obligations are imposed on any author or copyright
   holder as a result of your choosing to follow a later version.

   __1__ Disclaimer of Warranty.

   THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE
   LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR
   OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND,
   EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
   ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
   SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY
   SERVICING, REPAIR OR CORRECTION.

   __1__ Limitation of Liability.

   IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
   ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS THE
   PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
   GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE
   OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR
   DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR
   A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH
   HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

   __1__ Interpretation of Sections 15 and 16.

   If the disclaimer of warranty and limitation of liability provided above
   cannot be given local legal effect according to their terms, reviewing courts
   shall apply local law that most closely approximates an absolute waiver of
   all civil liability in connection with the Program, unless a warranty or
   assumption of liability accompanies a copy of the Program in return for a
   fee.

((  END OF TERMS AND CONDITIONS ))??

((

How to Apply These Terms to Your New Programs

If you develop a new program, and you want it to be of the greatest possible use
to the public, the best way to achieve this is to make it free software which
everyone can redistribute and change under these terms.

To do so, attach the following notices to the program. It is safest to attach
them to the start of each source file to most effectively state the exclusion of
warranty; and each file should have at least the "copyright" line and a pointer
to where the full notice is found.

__30__
//**
<one line to give the program's name and a brief idea of what it does.>
Copyright (C) <year> <name of author>
**//

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License as published by the Free
Software Foundation, either version 3 of the License, or
(( (at your option) ))??
any
later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program. If not, see <https:/www.gnu.org/licenses/>.

Also add information on how to contact you by electronic and paper mail.

If your software can interact with users remotely through a computer network,
you should also make sure that it provides a way for users to get its source.
For example, if your program is a web application, its interface could display a
"Source" link that leads users to an archive of the code. There are many ways
you could offer source, and different solutions will be better for different
programs; see section 13 for the specific requirements.

You should also get your employer (if you work as a programmer) or school, if
any, to sign a "copyright disclaimer" for the program, if necessary. For more
information on this, and how to apply and follow the GNU AGPL, see <https:/www.gnu.org/licenses/>. ))??
`
const license_AGPL_3_0_only_lre = `
//**
https://spdx.org/licenses/AGPL-3.0-only.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		program
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	GNU Affero
	General Public License
	((AGPL))??
	((as published by the Free Software Foundation))??
	
		((under))??
		version 3
		((AGPLv3))??
		((of the License))??
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		GNU Affero
		General Public License version
		3
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		GNU Affero
		General Public License for more details.

		((
			You should have received a copy of the
			GNU Affero
			General Public License
			((
				((version))??
				3
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_AGPL_3_0_or_later_lre = `
//**
https://spdx.org/licenses/AGPL-3.0-only.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		program
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	GNU Affero
	General Public License
	((AGPL))??
	((as published by the Free Software Foundation))??
	
		((
			either version 3
			((of the License))??
			or
			((at your option))??
			any later version.
		||
			version 3 or later
			((of the License))??
		))
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		GNU Affero
		General Public License version
		3
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		GNU Affero
		General Public License for more details.

		((
			You should have received a copy of the
			GNU Affero
			General Public License
			((
				((version))??
				3
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_AMDPLPA_lre = `//**
AMD's plpa_map.c License
https://spdx.org/licenses/AMDPLPA.json
https://fedoraproject.org/wiki/Licensing/AMD_plpa_map_License
**//

//** Copyright **//

Redistribution and use in any form of this material and any product thereof
including software in source or binary forms, along with any related
documentation, with or without modification ("this material"), is permitted
provided that the following conditions are met:

Redistributions of source code of any software must retain the above copyright
notice and all terms of this license as part of the code.

Redistributions in binary form of any software must reproduce the above
copyright notice and all terms of this license in any related documentation
and/or other materials.

Neither the names nor trademarks of Advanced Micro Devices, Inc. or any
copyright holders or contributors may be used to endorse or promote products
derived from this material without specific prior written permission.

Notice about U.S. Government restricted rights: This material is provided with
"RESTRICTED RIGHTS." Use, duplication or disclosure by the U.S. Government is
subject to the full extent of restrictions set forth in FAR52.227 and
DFARS252.227 et seq., or any successor or applicable regulations. Use of this
material by the U.S. Government constitutes acknowledgment of the proprietary
rights of Advanced Micro Devices, Inc. and any copyright holders and
contributors.

ANY BREACH OF ANY TERM OF THIS LICENSE SHALL RESULT IN THE IMMEDIATE REVOCATION
OF ALL RIGHTS TO REDISTRIBUTE, ACCESS OR USE THIS MATERIAL.

THIS MATERIAL IS PROVIDED BY ADVANCED MICRO DEVICES, INC. AND ANY COPYRIGHT
HOLDERS AND CONTRIBUTORS "AS IS" IN ITS CURRENT CONDITION AND WITHOUT ANY
REPRESENTATIONS, GUARANTEE, OR WARRANTY OF ANY KIND OR IN ANY WAY RELATED TO
SUPPORT, INDEMNITY, ERROR FREE OR UNINTERRUPTED OPERATION, OR THAT IT IS FREE
FROM DEFECTS OR VIRUSES. ALL OBLIGATIONS ARE HEREBY DISCLAIMED - WHETHER
EXPRESS, IMPLIED, OR STATUTORY - INCLUDING, BUT NOT LIMITED TO, ANY IMPLIED
WARRANTIES OF TITLE, MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
ACCURACY, COMPLETENESS, OPERABILITY, QUALITY OF SERVICE, OR NON-INFRINGEMENT. IN
NO EVENT SHALL ADVANCED MICRO DEVICES, INC. OR ANY COPYRIGHT HOLDERS OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, PUNITIVE,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, REVENUE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED OR BASED ON ANY THEORY OF LIABILITY
ARISING IN ANY WAY RELATED TO THIS MATERIAL, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE. THE ENTIRE AND AGGREGATE LIABILITY OF ADVANCED MICRO DEVICES,
INC. AND ANY COPYRIGHT HOLDERS AND CONTRIBUTORS SHALL NOT EXCEED TEN DOLLARS (US
$10.00). ANYONE REDISTRIBUTING OR ACCESSING OR USING THIS MATERIAL ACCEPTS THIS
ALLOCATION OF RISK AND AGREES TO RELEASE ADVANCED MICRO DEVICES, INC. AND ANY
COPYRIGHT HOLDERS AND CONTRIBUTORS FROM ANY AND ALL LIABILITIES, OBLIGATIONS,
CLAIMS, OR DEMANDS IN EXCESS OF TEN DOLLARS (US $10.00). THE FOREGOING ARE
ESSENTIAL TERMS OF THIS LICENSE AND, IF ANY OF THESE TERMS ARE CONSTRUED AS
UNENFORCEABLE, FAIL IN ESSENTIAL PURPOSE, OR BECOME VOID OR DETRIMENTAL TO
ADVANCED MICRO DEVICES, INC. OR ANY COPYRIGHT HOLDERS OR CONTRIBUTORS FOR ANY
REASON, THEN ALL RIGHTS TO REDISTRIBUTE, ACCESS OR USE THIS MATERIAL SHALL
TERMINATE IMMEDIATELY. MOREOVER, THE FOREGOING SHALL SURVIVE ANY EXPIRATION OR
TERMINATION OF THIS LICENSE OR ANY AGREEMENT OR ACCESS OR USE RELATED TO THIS
MATERIAL.

NOTICE IS HEREBY PROVIDED, AND BY REDISTRIBUTING OR ACCESSING OR USING THIS
MATERIAL SUCH NOTICE IS ACKNOWLEDGED, THAT THIS MATERIAL MAY BE SUBJECT TO
RESTRICTIONS UNDER THE LAWS AND REGULATIONS OF THE UNITED STATES OR OTHER
COUNTRIES, WHICH INCLUDE BUT ARE NOT LIMITED TO, U.S. EXPORT CONTROL LAWS SUCH
AS THE EXPORT ADMINISTRATION REGULATIONS AND NATIONAL SECURITY CONTROLS AS
DEFINED THEREUNDER, AS WELL AS STATE DEPARTMENT CONTROLS UNDER THE U.S.
MUNITIONS LIST. THIS MATERIAL MAY NOT BE USED, RELEASED, TRANSFERRED, IMPORTED,
EXPORTED AND/OR RE- EXPORTED IN ANY MANNER PROHIBITED UNDER ANY APPLICABLE LAWS,
INCLUDING U.S. EXPORT CONTROL LAWS REGARDING SPECIFICALLY DESIGNATED PERSONS,
COUNTRIES AND NATIONALS OF COUNTRIES SUBJECT TO NATIONAL SECURITY CONTROLS.
MOREOVER, THE FOREGOING SHALL SURVIVE ANY EXPIRATION OR TERMINATION OF ANY
LICENSE OR AGREEMENT OR ACCESS OR USE RELATED TO THIS MATERIAL.

This license forms the entire agreement regarding the subject matter hereof and
supersedes all proposals and prior discussions and writings between the parties
with respect thereto. This license does not affect any ownership, rights, title,
or interest in, or relating to, this material. No terms of this license can be
modified or waived, and no breach of this license can be excused, unless done so
in a writing signed by all affected parties. Each term of this license is
separately enforceable. If any term of this license is determined to be or
becomes unenforceable or illegal, such term shall be reformed to the minimum
extent necessary in order for this license to remain in effect in accordance
with its terms as modified by such reformation. This license shall be governed
by and construed in accordance with the laws of the State of Texas without
regard to rules on conflicts of law of any state or jurisdiction or the United
Nations Convention on the International Sale of Goods. All disputes arising out
of this license shall be subject to the jurisdiction of the federal and state
courts in Austin, Texas, and all defenses are hereby waived concerning personal
jurisdiction and venue of these courts.
`
const license_AML_lre = `//**
Apple MIT License
https://spdx.org/licenses/AML.json
https://fedoraproject.org/wiki/Licensing/Apple_MIT_License
**//

//** Copyright **//

IMPORTANT: This Apple software is supplied to you by Apple Computer, Inc.
("Apple") in consideration of your agreement to the following terms, and your
use, installation, modification or redistribution of this Apple software
constitutes acceptance of these terms. If you do not agree with these terms,
please do not use, install, modify or redistribute this Apple software.

In consideration of your agreement to abide by the following terms, and subject
to these terms, Apple grants you a personal, non-exclusive license, under
Apple's copyrights in this original Apple software (the "Apple Software"), to
use, reproduce, modify and redistribute the Apple Software, with or without
modifications, in source and/or binary forms; provided that if you redistribute
the Apple Software in its entirety and without modifications, you must retain
this notice and the following text and disclaimers in all such redistributions
of the Apple Software. Neither the name, trademarks, service marks or logos of
Apple Computer, Inc. may be used to endorse or promote products derived from the
Apple Software without specific prior written permission from Apple. Except as
expressly stated in this notice, no other rights or licenses, express or
implied, are granted by Apple herein, including but not limited to any patent
rights that may be infringed by your derivative works or by other works in which
the Apple Software may be incorporated.

The Apple Software is provided by Apple on an "AS IS" basis. APPLE MAKES NO
WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
COMBINATION WITH YOUR PRODUCTS.

IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR
DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF
CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF
APPLE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
`
const license_AMPAS_lre = `//**
Academy of Motion Picture Arts and Sciences BSD
https://spdx.org/licenses/AMPAS.json
https://fedoraproject.org/wiki/Licensing/BSD#AMPASBSD
**//

//** Copyright **//

A world-wide, royalty-free, non-exclusive right to distribute, copy, modify,
create derivatives, and use, in source and binary forms, is hereby granted,
subject to acceptance of this license. Performance of any of the aforementioned
acts indicates acceptance to be bound by the following terms and conditions:

   *
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the Disclaimer of Warranty.

   *
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the Disclaimer of Warranty in the documentation
   and/or other materials provided with the distribution.

   *
   Nothing in this license shall be deemed to grant any rights to trademarks,
   copyrights, patents, trade secrets or any other intellectual property of
   A.M.P.A.S. or any contributors, except as expressly stated herein, and
   neither the name of A.M.P.A.S. nor of any other contributors to this
   software, may be used to endorse or promote products derived from this
   software without specific prior written permission of A.M.P.A.S. or
   contributor, as appropriate.

This license shall be governed by the laws of the State of California, and
subject to the jurisdiction of the courts therein.

Disclaimer of Warranty: THIS SOFTWARE IS PROVIDED BY A.M.P.A.S. AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL A.M.P.A.S., ANY CONTRIBUTORS
OR DISTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
`
const license_ANTLR_PD_lre = `//**
ANTLR Software Rights Notice
https://spdx.org/licenses/ANTLR-PD.json
http://www.antlr2.org/license.html
**//

(( ANTLR 2 License ))??

We reserve no legal rights to the ANTLR--it is fully in the public domain. An
individual or company may do whatever they wish with source code distributed
with ANTLR or the code generated by ANTLR, including the incorporation of ANTLR,
or its output, into commerical software.

We encourage users to develop software with ANTLR. However, we do ask that
credit is given to us for developing ANTLR. By "credit", we mean that if you use
ANTLR or incorporate any source code into one of your programs (commercial
product, research project, or otherwise) that you acknowledge this fact
somewhere in the documentation, research report, etc... If you like ANTLR and
have developed a nice tool with the output, please mention that you developed it
using ANTLR. In addition, we ask that the headers remain intact in our source
code. As long as these guidelines are kept, we expect to continue enhancing this
system and expect to make other tools available as they are completed.
`
const license_APAFML_lre = `//**
Adobe Postscript AFM License
https://spdx.org/licenses/APAFML.json
https://fedoraproject.org/wiki/Licensing/AdobePostscriptAFM
**//

//** Copyright **//

This file and the 14 PostScript(R) AFM files it accompanies may be used, copied,
and distributed for any purpose and without charge, with or without
modification, provided that all copyright notices are retained; that the AFM
files are not distributed without this file; that all modifications to this file
or any of the AFM files are prominently noted in the modified file(s); and that
this paragraph is not modified. Adobe Systems has no responsibility or
obligation to support the use of the AFM files.
`
const license_APL_1_0_lre = `//**
Adaptive Public License 1.0
https://spdx.org/licenses/APL-1.0.json
https://opensource.org/licenses/APL-1.0
**//

(( ADAPTIVE PUBLIC LICENSE

Version 1.0 ))??

THE LICENSED WORK IS PROVIDED UNDER THE TERMS OF THIS ADAPTIVE PUBLIC LICENSE
("LICENSE"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THE LICENSED WORK
CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS LICENSE AND ITS TERMS, WHETHER OR NOT
SUCH RECIPIENT READS THE TERMS OF THIS LICENSE. "LICENSED WORK" AND "RECIPIENT"
ARE DEFINED BELOW.

IMPORTANT NOTE: This License is "adaptive", and the generic version or another
version of an Adaptive Public License should not be relied upon to determine
your rights and obligations under this License. You must read the specific
Adaptive Public License that you receive with the Licensed Work, as certain
terms are defined at the outset by the Initial Contributor.

See Section 2.2 below, Exhibit A attached, and any Suppfile.txt accompanying
this License to determine the specific adaptive features applicable to this
License. For example, without limiting the foregoing, (a) for selected choice of
law and jurisdiction see Part 3 of Exhibit A; (b) for the selected definition of
Third Party see Part 4 of Exhibit A; and (c) for selected patent licensing terms
(if any) see Section 2.2 below and Part 6 of Exhibit A.

   (( 1. ))??
   DEFINITIONS.

      (( 1.1. ))??
      "CONTRIBUTION" means:

         (( (a) ))??
         In the case of the Initial Contributor, the Initial Work distributed
         under this License by the Initial Contributor; and

         (( (b) ))??
         In the case of each Subsequent Contributor, the Subsequent Work
         originating from and distributed by such Subsequent Contributor.

      (( 1.2. ))??
      "DESIGNATED WEB SITE" means the web site having the URL identified in Part
      1 of Exhibit A, which URL may be changed by the Initial Contributor by
      posting on the current Designated Web Site the new URL for at least sixty
      (60) days.

      (( 1.3. ))??
      "DISTRIBUTOR" means any Person that distributes the Licensed Work or any
      portion thereof to at least one Third Party.

      (( 1.4. ))??
      "ELECTRONIC DISTRIBUTION MECHANISM" means any mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      "EXECUTABLE" means the Licensed Work in any form other than Source Code.

      (( 1.6. ))??
      "GOVERNING JURISDICTION" means the state, province or other legal
      jurisdiction identified in Part 3 of Exhibit A.

      (( 1.7. ))??
      "INDEPENDENT MODULE" means a separate module of software and/or data that
      is not a derivative work of or copied from the Licensed Work or any
      portion thereof. In addition, a module does not qualify as an Independent
      Module but instead forms part of the Licensed Work if the module: (a) is
      embedded in the Licensed Work; (b) is included by reference in the
      Licensed Work other than by a function call or a class reference; or (c)
      must be included or contained, in whole or in part, within a file
      directory or subdirectory actually containing files making up the Licensed
      Work.

      (( 1.8. ))??
      "INITIAL CONTRIBUTOR" means the Person or entity identified as the Initial
      Contributor in the notice required by Part 1 of Exhibit A.

      (( 1.9. ))??
      "INITIAL WORK" means the initial Source Code, object code (if any) and
      documentation for the computer program identified in Part 2 of Exhibit A,
      as such Source Code, object code and documentation is distributed under
      this License by the Initial Contributor.

      (( 1.10. ))??
      "LARGER WORK" means a work that combines the Licensed Work or portions
      thereof with code not governed by this License.

      (( 1.11. ))??
      "LICENSED WORK" means the Initial Work and/or any Subsequent Work, in each
      case including portions thereof.

      (( 1.12. ))??
      "LICENSE NOTICE" has the meaning assigned in Part 5 of Exhibit A.

      (( 1.13. ))??
      "MODIFICATION" or "MODIFICATIONS" means any change to and/or addition to
      the Licensed Work.

      (( 1.14. ))??
      "PERSON" means an individual or other legal entity, including a
      corporation, partnership or other body.

      (( 1.15. ))??
      "RECIPIENT" means any Person who receives or obtains the Licensed Work
      under this License (by way of example, without limiting the foregoing, any
      Subsequent Contributor or Distributor).

      (( 1.16. ))??
      "SOURCE CODE" means the source code for a computer program, including the
      source code for all modules and components of the computer program, plus
      any associated interface definition files, and scripts used to control
      compilation and installation of an executable.

      (( 1.17. ))??
      "SUBSEQUENT CONTRIBUTOR" means any Person that makes or contributes to the
      making of any Subsequent Work and that distributes that Subsequent Work to
      at least one Third Party.

      (( 1.18. ))??
      "SUBSEQUENT WORK" means a work that has resulted or arises from changes to
      and/or additions to:

         (( (a) ))??
         the Initial Work;

         (( (b) ))??
         any other Subsequent Work; or

         (( c ))??
         to any combination of the Initial Work and any such other Subsequent
         Work;

         where such changes and/or additions originate from a Subsequent
         Contributor. A Subsequent Work will "originate" from a Subsequent
         Contributor if the Subsequent Work was a result of efforts by such
         Subsequent Contributor (or anyone acting on such Subsequent
         Contributor's behalf, such as, a contractor or other entity that is
         engaged by or under the direction of the Subsequent Contributor). For
         greater certainty, a Subsequent Work expressly excludes and shall not
         capture within its meaning any Independent Module.

      (( 1.19. ))??
      "SUPPLEMENT FILE" means a file distributed with the Licensed Work having a
      file name "suppfile.txt".

      (( 1.20. ))??
      "THIRD PARTY" has the meaning assigned in Part 4 of Exhibit A.

   (( 2. ))??
   LICENSE.

      (( 2.1. ))??
      COPYRIGHT LICENSE FROM INITIAL AND SUBSEQUENT CONTRIBUTORS.

         (( (a) ))??
         Subject to the terms of this License, the Initial Contributor hereby
         grants each Recipient a world-wide, royalty-free, non-exclusive
         copyright license to:

            (( (i) ))??
            reproduce, prepare derivative works of, publicly display, publicly
            perform, distribute and sublicense the Initial Work; and

            (( (ii) ))??
            reproduce, publicly display, publicly perform, distribute, and
            sublicense any derivative works (if any) prepared by Recipient;

            in Source Code and Executable form, either with other Modifications,
            on an unmodified basis, or as part of a Larger Work.

         (( (b) ))??
         Subject to the terms of this License, each Subsequent Contributor
         hereby grants each Recipient a world-wide, royalty-free, non-exclusive
         copyright license to:

            (( (i) ))??
            reproduce, prepare derivative works of, publicly display, publicly
            perform, distribute and sublicense the Subsequent Work of such
            Subsequent Contributor; and

            (( (ii) ))??
            reproduce, publicly display, publicly perform, distribute, and
            sublicense any derivative works (if any) prepared by Recipient;

            in Source Code and Executable form, either with other Modifications,
            on an unmodified basis, or as part of a Larger Work.

      (( 2.2. ))??
      PATENT LICENSE FROM INITIAL AND SUBSEQUENT CONTRIBUTORS.

         (( (a) ))??
         This License does not include or grant any patent license whatsoever
         from the Initial Contributor, Subsequent Contributor, or any
         Distributor unless, at the time the Initial Work is first distributed
         or made available under this License (as the case may be), the Initial
         Contributor has selected pursuant to Part 6 of Exhibit A the patent
         terms in paragraphs A, B, C, D and E from Part 6 of Exhibit A. If this
         is not done then the Initial Work and any other Subsequent Work is made
         available under the License without any patent license (the
         "PATENTS-EXCLUDED LICENSE").

         (( (b) ))??
         However, the Initial Contributor may subsequently distribute or make
         available (as the case may be) future copies of: (1) the Initial Work;
         or (2) any Licensed Work distributed by the Initial Contributor which
         includes the Initial Work (or any portion thereof) and/or any
         Modification made by the Initial Contributor; available under a License
         which includes a patent license (the "PATENTS-INCLUDED LICENSE") by
         selecting pursuant to Part 6 of Exhibit A the patent terms in
         paragraphs A, B, C, D and E from Part 6 of Exhibit A, when the Initial
         Contributor distributes or makes available (as the case may be) such
         future copies under this License.

         (( c ))??
         If any Recipient receives or obtains one or more copies of the Initial
         Work or any other portion of the Licensed Work under the
         Patents-Included License, then all licensing of such copies under this
         License shall include the terms in paragraphs A, B, C, D and E from
         Part 6 of Exhibit A and that Recipient shall not be able to rely upon
         the Patents-Excluded License for any such copies. However, all
         Recipients that receive one or more copies of the Initial Work or any
         other portion of the Licensed Work under a copy of the License which
         includes the Patents-Excluded License shall have no patent license with
         respect to such copies received under the Patents-Excluded License and
         availability and distribution of such copies, including Modifications
         made by such Recipient to such copies, shall be under a copy of the
         License without any patent license.

         (( (d) ))??
         Where a Recipient uses in combination or combines any copy of the
         Licensed Work (or portion thereof) licensed under a copy of the License
         having a Patents-Excluded License with any copy of the Licensed Work
         (or portion thereof) licensed under a copy of the License having a
         Patents-Included License, the combination (and any portion thereof)
         shall, from the first time such Recipient uses, makes available or
         distributes the combination (as the case may be), be subject to only
         the terms of the License having the Patents-Included License which
         shall include the terms in paragraphs A, B, C, D and E from Part 6 of
         Exhibit A.

      (( 2.3. ))??
      ACKNOWLEDGEMENT AND DISCLAIMER.

      Recipient understands and agrees that although Initial Contributor and
      each Subsequent Contributor grants the licenses to its Contributions set
      forth herein, no representation, warranty, guarantee or assurance is
      provided by any Initial Contributor, Subsequent Contributor, or
      Distributor that the Licensed Work does not infringe the patent or other
      intellectual property rights of any other entity. Initial Contributor,
      Subsequent Contributor, and each Distributor disclaims any liability to
      Recipient for claims brought by any other entity based on infringement of
      intellectual property rights or otherwise, in relation to the Licensed
      Works. As a condition to exercising the rights and licenses granted
      hereunder, each Recipient hereby assumes sole responsibility to secure any
      other intellectual property rights needed, if any. For example, without
      limiting the foregoing disclaimers, if a third party patent license is
      required to allow Recipient to distribute the Licensed Work, it is
      Recipient's responsibility to acquire that license before distributing the
      Licensed Work.

      (( 2.4. ))??
      RESERVATION.

      Nothing in this License shall be deemed to grant any rights to trademarks,
      copyrights, patents, trade secrets or any other intellectual property of
      Initial Contributor, Subsequent Contributor, or Distributor except as
      expressly stated herein.

   (( 3. ))??
   DISTRIBUTION OBLIGATIONS.

      (( 3.1. ))??
      DISTRIBUTION GENERALLY.

         (( (a) ))??
         A Subsequent Contributor shall make that Subsequent Contributor's
         Subsequent Work(s) available to the public via an Electronic
         Distribution Mechanism for a period of at least twelve (12) months. The
         aforesaid twelve (12) month period shall begin within a reasonable time
         after the creation of the Subsequent Work and no later than sixty (60)
         days after first distribution of that Subsequent Contributor's
         Subsequent Work.

         (( (b) ))??
         All Distributors must distribute the Licensed Work in accordance with
         the terms of the License, and must include a copy of this License
         (including without limitation Exhibit A and the accompanying Supplement
         File) with each copy of the Licensed Work distributed. In particular,
         this License must be prominently distributed with the Licensed Work in
         a file called "license.txt." In addition, the License Notice in Part 5
         of Exhibit A must be included at the beginning of all Source Code
         files, and viewable to a user in any executable such that the License
         Notice is reasonably brought to the attention of any party using the
         Licensed Work.

      (( 3.2. ))??
      EXECUTABLE DISTRIBUTIONS OF THE LICENSED WORK.

      A Distributor may choose to distribute the Licensed Work, or any portion
      thereof, in Executable form (an "EXECUTABLE DISTRIBUTION") to any third
      party, under the terms of Section 2 of this License, provided the
      Executable Distribution is made available under and accompanied by a copy
      of this License, AND provided at least ONE of the following conditions is
      fulfilled:

         (( (a) ))??
         The Executable Distribution must be accompanied by the Source Code for
         the Licensed Work making up the Executable Distribution, and the Source
         Code must be distributed on the same media as the Executable
         Distribution or using an Electronic Distribution Mechanism; or

         (( (b) ))??
         The Executable Distribution must be accompanied with a written offer,
         valid for at least thirty six (36) months, to give any third party
         under the terms of this License, for a charge no more than the cost of
         physically performing source distribution, a complete machine-readable
         copy of the Source Code for the Licensed Work making up the Executable
         Distribution, to be available and distributed using an Electronic
         Distribution Mechanism, and such Executable Distribution must remain
         available in Source Code form to any third party via the Electronic
         Distribution Mechanism (or any replacement Electronic Distribution
         Mechanism the particular Distributor may reasonably need to turn to as
         a substitute) for said at least thirty six (36) months.

      For greater certainty, the above-noted requirements apply to any Licensed
      Work or portion thereof distributed to any third party in Executable form,
      whether such distribution is made alone, in combination with a Larger Work
      or Independent Modules, or in some other combination.

      (( 3.3. ))??
      SOURCE CODE DISTRIBUTIONS.

      When a Distributor makes the Licensed Work, or any portion thereof,
      available to any Person in Source Code form, it must be made available
      under this License and a copy of this License must be included with each
      copy of the Source Code, situated so that the copy of the License is
      conspicuously brought to the attention of that Person. For greater
      clarification, this Section 3.3 applies to all distribution of the
      Licensed Work in any Source Code form. A Distributor may charge a fee for
      the physical act of transferring a copy, which charge shall be no more
      than the cost of physically performing source distribution.

      (( 3.4. ))??
      REQUIRED NOTICES IN SOURCE CODE.

      Each Subsequent Contributor must ensure that the notice set out in Part 5
      of Exhibit A is included in each file of the Source Code for each
      Subsequent Work originating from that particular Subsequent Contributor,
      if such notice is not already included in each such file. If it is not
      possible to put such notice in a particular Source Code file due to its
      structure, then the Subsequent Contributor must include such notice in a
      location (such as a relevant directory in which the file is stored) where
      a user would be likely to look for such a notice.

      (( 3.5. ))??
      NO DISTRIBUTION REQUIREMENTS FOR INTERNALLY USED MODIFICATIONS.

      Notwithstanding Sections 3.2, 3.3 and 3.4, Recipient may, internally
      within its own corporation or organization use the Licensed Work,
      including the Initial Work and Subsequent Works, and make Modifications
      for internal use within Recipient's own corporation or organization
      (collectively, "INTERNAL USE MODIFICATIONS"). The Recipient shall have no
      obligation to distribute, in either Source Code or Executable form, any
      such Internal Use Modifications made by Recipient in the course of such
      internal use, except where required below in this Section 3.5. All
      Internal Use Modifications distributed to any Person, whether or not a
      Third Party, shall be distributed pursuant to and be accompanied by the
      terms of this License. If the Recipient chooses to distribute any such
      Internal Use Modifications to any Third Party, then the Recipient shall be
      deemed a Subsequent Contributor, and any such Internal Use Modifications
      distributed to any Third Party shall be deemed a Subsequent Work
      originating from that Subsequent Contributor, and shall from the first
      such instance become part of the Licensed Work that must thereafter be
      distributed and made available to third parties in accordance with the
      terms of Sections 3.1 to 3.4 inclusive.

      (( 3.6. ))??
      INDEPENDENT MODULES.

      This License shall not apply to Independent Modules of any Initial
      Contributor, Subsequent Contributor, Distributor or any Recipient, and
      such Independent Modules may be licensed or made available under one or
      more separate license agreements.

      (( 3.7. ))??
      LARGER WORKS.

      Any Distributor or Recipient may create or contribute to a Larger Work by
      combining any of the Licensed Work with other code not governed by the
      terms of this License, and may distribute the Larger Work as one or more
      products. However, in any such case, Distributor or Recipient (as the case
      may be) must make sure that the requirements of this License are fulfilled
      for the Licensed Work portion of the Larger Work.

      (( 3.8. ))??
      DESCRIPTION OF DISTRIBUTED MODIFICATIONS.

         (( (a) ))??
         Each Subsequent Contributor (including the Initial Contributor where
         the Initial Contributor also qualifies as a Subsequent Contributor)
         must cause each Subsequent Work created or contributed to by that
         Subsequent Contributor to contain a file documenting the changes, in
         accordance with the requirements of Part 1 of the Supplement File, that
         such Subsequent Contributor made in the creation or contribution to
         that Subsequent Work. If no Supplement File exists or no requirements
         are set out in Part 1 of the Supplement File, then there are no
         requirements for Subsequent Contributors to document changes that they
         make resulting in Subsequent Works.

         (( (b) ))??
         The Initial Contributor may at any time introduce requirements or add
         to or change earlier requirements (in each case, the "EARLIER
         DESCRIPTION REQUIREMENTS") for documenting changes resulting in
         Subsequent Works by revising Part 1 of each copy of the Supplement File
         distributed by the Initial Contributor with future copies of the
         Licensed Work so that Part 1 then contains new requirements (the "NEW
         DESCRIPTION REQUIREMENTS") for documenting such changes.

         (( c ))??
         Any Recipient receiving at any time any copy of an Initial Work or any
         Subsequent Work under a copy of this License (in each case, an "Earlier
         LICENSED COPY") having the Earlier Description Requirements may choose,
         with respect to each such Earlier Licensed Copy, to comply with the
         Earlier Description Requirements or the New Description Requirements.
         Where a Recipient chooses to comply with the New Description
         Requirements, that Recipient will, when thereafter distributing any
         copies of any such Earlier Licensed Copy, include a Supplement File
         having a section entitled Part 1 that contains a copy of the New
         Description Requirements.

         (( (d) ))??
         For greater certainty, the intent of Part 1 of the Supplement File is
         to provide a mechanism (if any) by which Subsequent Contributors must
         document changes that they make to the Licensed Work resulting in
         Subsequent Works. Part 1 of any Supplement File shall not be used to
         increase or reduce the scope of the license granted in Article 2 of
         this License or in any other way increase or decrease the rights and
         obligations of any Recipient, and shall at no time serve as the basis
         for terminating the License. Further, a Recipient can be required to
         correct and change its documentation procedures to comply with Part 1
         of the Supplement File, but cannot be penalised with damages. Part 1 of
         any Supplement File is only binding on each Recipient of any Licensed
         Work to the extent Part 1 sets out the requirements for documenting
         changes to the Initial Work or any Subsequent Work.

         (( (e) ))??
         An example of a set of requirements for documenting changes and
         contributions made by Subsequent Contributor is set out in Part 7 of
         Exhibit A of this License. Part 7 is a sample only and is not binding
         on Recipients, unless (subject to the earlier paragraphs of this
         Section 3.8) those are the requirements that the Initial Contributor
         includes in Part 1 of the Supplement File with the copies of the
         Initial Work distributed under this License.

      (( 3.9. ))??
      USE OF DISTRIBUTOR NAME.

      The name of a Distributor may not be used by any other Distributor to
      endorse or promote the Licensed Work or products derived from the Licensed
      Work, without prior written permission.

      (( 3.10. ))??
      LIMITED RECOGNITION OF INITIAL CONTRIBUTOR.

         (( (a) ))??
         As a modest attribution to the Initial Contributor, in the hope that
         its promotional value may help justify the time, money and effort
         invested in writing the Initial Work, the Initial Contributor may
         include in Part 2 of the Supplement File a requirement that each time
         an executable program resulting from the Initial Work or any Subsequent
         Work, or a program dependent thereon, is launched or run, a prominent
         display of the Initial Contributor's attribution information must occur
         (the "ATTRIBUTION INFORMATION"). The Attribution Information must be
         included at the beginning of each Source Code file. For greater
         certainty, the Initial Contributor may specify in the Supplement File
         that the above attribution requirement only applies to an executable
         program resulting from the Initial Work or any Subsequent Work, but not
         a program dependent thereon. The intent is to provide for reasonably
         modest attribution, therefore the Initial Contributor may not require
         Recipients to display, at any time, more than the following Attribution
         Information: (a) a copyright notice including the name of the Initial
         Contributor; (b) a word or one phrase (not exceeding 10 words); (c) one
         digital image or graphic provided with the Initial Work; and (d) a URL
         (collectively, the "ATTRIBUTION LIMITS").

         (( (b) ))??
         If no Supplement File exists, or no Attribution Information is set out
         in Part 2 of the Supplement File, then there are no requirements for
         Recipients to display any Attribution Information of the Initial
         Contributor.

         (( c ))??
         Each Recipient acknowledges that all trademarks, service marks and/or
         trade names contained within Part 2 of the Supplement File distributed
         with the Licensed Work are the exclusive property of the Initial
         Contributor and may only be used with the permission of the Initial
         Contributor, or under circumstances otherwise permitted by law, or as
         expressly set out in this License.

      (( 3.11. ))??
      For greater certainty, any description or attribution provisions contained
      within a Supplement File may only be used to specify the nature of the
      description or attribution requirements, as the case may be. Any provision
      in a Supplement File that otherwise purports to modify, vary, nullify or
      amend any right, obligation or representation contained herein shall be
      deemed void to that extent, and shall be of no force or effect.

   (( 4. ))??
   COMMERCIAL USE AND INDEMNITY.

      (( 4.1. ))??
      COMMERCIAL SERVICES.

      A Recipient ("COMMERCIAL RECIPIENT") may choose to offer, and to charge a
      fee for, warranty, support, indemnity or liability obligations
      (collectively, "SERVICES") to one or more other Recipients or
      Distributors. However, such Commercial Recipient may do so only on that
      Commercial Recipient's own behalf, and not on behalf of any other
      Distributor or Recipient, and Commercial Recipient must make it clear than
      any such warranty, support, indemnity or liability obligation(s) is/are
      offered by Commercial Recipient alone. At no time may Commercial Recipient
      use any Services to deny any party the Licensed Work in Source Code or
      Executable form when so required under any of the other terms of this
      License. For greater certainty, this Section 4.1 does not diminish any of
      the other terms of this License, including without limitation the
      obligation of the Commercial Recipient as a Distributor, when distributing
      any of the Licensed Work in Source Code or Executable form, to make such
      distribution royalty-free (subject to the right to charge a fee of no more
      than the cost of physically performing Source Code or Executable
      distribution (as the case may be).

      (( 4.2. ))??
      INDEMNITY.

      Commercial distributors of software may accept certain responsibilities
      with respect to end users, business partners and the like. While this
      License is intended to facilitate the commercial use of the Licensed Work,
      the Distributor who includes any of the Licensed Work in a commercial
      product offering should do so in a manner which does not create potential
      liability for other Distributors. Therefore, if a Distributor includes the
      Licensed Work in a commercial product offering or offers any Services,
      such Distributor ("COMMERCIAL DISTRIBUTOR") hereby agrees to defend and
      indemnify every other Distributor or Subsequent Contributor (in each case
      an "INDEMNIFIED PARTY") against any losses, damages and costs
      (collectively "LOSSES") arising from claims, lawsuits and other legal
      actions brought by a third party against the Indemnified Party to the
      extent caused by the acts or omissions of such Commercial Distributor in
      connection with its distribution of any of the Licensed Work in a
      commercial product offering or in connection with any Services. The
      obligations in this section do not apply to any claims or Losses relating
      to any actual or alleged intellectual property infringement. In order to
      qualify, an Indemnified Party must: (a) promptly notify the Commercial
      Distributor in writing of such claim; and (b) allow the Commercial
      Distributor to control, and co-operate with the Commercial Distributor in,
      the defense and any related settlement negotiations. The Indemnified Party
      may participate in any such claim at its own expense.

   (( 5. ))??
   VERSIONS OF THE LICENSE.

      (( 5.1. ))??
      NEW VERSIONS.

      The Initial Contributor may publish revised and/or new versions of the
      License from time to time. Each version will be given a distinguishing
      version number.

      (( 5.2. ))??
      EFFECT OF NEW VERSIONS.

      Once the Licensed Work or any portion thereof has been published by
      Initial Contributor under a particular version of the License, Recipient
      may choose to continue to use it under the terms of that version. However,
      if a Recipient chooses to use the Licensed Work under the terms of any
      subsequent version of the License published by the Initial Contributor,
      then from the date of making this choice, the Recipient must comply with
      the terms of that subsequent version with respect to all further
      reproduction, preparation of derivative works, public display of, public
      performance of, distribution and sublicensing by the Recipient in
      connection with the Licensed Work. No one other than the Initial
      Contributor has the right to modify the terms applicable to the Licensed
      Work

   (( 6. ))??
   DISCLAIMER OF WARRANTY.

      (( 6.1. ))??
      GENERAL DISCLAIMER.

      EXCEPT AS EXPRESSLY SET FORTH IN THIS LICENSE, THE LICENSED WORK IS
      PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT ANY
      REPRESENTATION, WARRANTY, GUARANTEE, ASSURANCE OR CONDITION OF ANY KIND,
      EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES OR
      CONDITIONS OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A
      PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF
      THE LICENSED WORK IS WITH RECIPIENT. SHOULD ANY LICENSED WORK PROVE
      DEFECTIVE IN ANY RESPECT, RECIPIENT (NOT THE INITIAL CONTRIBUTOR OR ANY
      SUBSEQUENT CONTRIBUTOR) ASSUMES THE COST OF ANY NECESSARY SERVICING,
      REPAIR OR CORRECTION. THIS CLAUSE CONSTITUTES AN ESSENTIAL PART OF THIS
      LICENSE. NO USE OF ANY LICENSED WORK IS AUTHORIZED HEREUNDER EXCEPT UNDER
      THIS LICENSE INCLUDING WITHOUT LIMITATION THIS DISCLAIMER.

      (( 6.2. ))??
      RESPONSIBILITY OF RECIPIENTS.

      Each Recipient is solely responsible for determining the appropriateness
      of using and distributing the Licensed Work and assumes all risks
      associated with its exercise of rights under this License, including but
      not limited to the risks and costs of program errors, compliance with
      applicable laws, damage to or loss of data, programs or equipment, and
      unavailability or interruption of operations.

   (( 7. ))??
   TERMINATION.

      (( 7.1. ))??
      This License shall continue until terminated in accordance with the
      express terms herein.

      (( 7.2. ))??
      Recipient may choose to terminate this License automatically at any time.

      (( 7.3. ))??
      This License, including without limitation the rights granted hereunder to
      a particular Recipient, will terminate automatically if such Recipient is
      in material breach of any of the terms of this License and fails to cure
      such breach within sixty (60) days of becoming aware of the breach.
      Without limiting the foregoing, any material breach by such Recipient of
      any term of any other License under which such Recipient is granted any
      rights to the Licensed Work shall constitute a material breach of this
      License.

      (( 7.4. ))??
      Upon termination of this License by or with respect to a particular
      Recipient for any reason, all rights granted hereunder and under any other
      License to that Recipient shall terminate. However, all sublicenses to the
      Licensed Work which were previously properly granted by such Recipient
      under a copy of this License (in each case, an "Other License" and in
      plural, "Other Licenses") shall survive any such termination of this
      License, including without limitation the rights and obligations under
      such Other Licenses as set out in their respective Sections 2, 3, 4, 5, 6,
      7 and 8, mutatis mutandis, for so long as the respective sublicensees
      (i.e. other Recipients) remain in compliance with the terms of the copy of
      this License under which such sublicensees received rights to the Licensed
      Work. Any termination of such Other Licenses shall be pursuant to their
      respective Section 7, mutatis mutandis. Provisions which, by their nature,
      must remain in effect beyond the termination of this License shall
      survive.

      (( 7.5. ))??
      Upon any termination of this License by or with respect to a particular
      Recipient, Sections 4.1, 4.2, 6.1, 6.2, 7.4, 7.5, 8.1, and 8.2, together
      with all provisions of this License necessary for the interpretation and
      enforcement of same, shall expressly survive such termination.

   (( 8. ))??
   LIMITATION OF LIABILITY.

      (( 8.1. ))??
      IN NO EVENT SHALL ANY OF INITIAL CONTRIBUTOR, ITS SUBSIDIARIES, OR
      AFFILIATES, OR ANY OF ITS OR THEIR RESPECTIVE OFFICERS, DIRECTORS,
      EMPLOYEES, AND/OR AGENTS (AS THE CASE MAY BE), HAVE ANY LIABILITY FOR ANY
      DIRECT DAMAGES, INDIRECT DAMAGES, PUNITIVE DAMAGES, INCIDENTAL DAMAGES,
      SPECIAL DAMAGES, EXEMPLARY DAMAGES, CONSEQUENTIAL DAMAGES OR ANY OTHER
      DAMAGES WHATSOEVER (INCLUDING WITHOUT LIMITATION LOSS OF USE, DATA OR
      PROFITS, OR ANY OTHER LOSS ARISING OUT OF OR IN ANY WAY RELATED TO THE
      USE, INABILITY TO USE, UNAUTHORIZED USE, PERFORMANCE, OR NON-PERFORMANCE
      OF THE LICENSED WORK OR ANY PART THEREOF OR THE PROVISION OF OR FAILURE TO
      PROVIDE SUPPORT SERVICES, OR THAT RESULT FROM ERRORS, DEFECTS, OMISSIONS,
      DELAYS IN OPERATION OR TRANSMISSION, OR ANY OTHER FAILURE OF PERFORMANCE),
      HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
      LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) IN RELATION TO OR
      ARISING IN ANY WAY OUT OF THIS LICENSE OR THE USE OR DISTRIBUTION OF THE
      LICENSED WORK OR THE EXERCISE OF ANY RIGHTS GRANTED HEREUNDER, EVEN IF
      ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. THIS LIMITATION OF LIABILITY
      SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL INJURY RESULTING FROM
      SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW PROHIBITS SUCH
      LIMITATION. THIS CLAUSE CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE. NO
      USE OF ANY LICENSED WORK IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS LICENSE
      INCLUDING WITHOUT LIMITATION THE LIMITATIONS SET FORTH IN THIS SECTION
      8.1.

      (( 8.2. ))??
      EXCEPT AS EXPRESSLY SET FORTH IN THIS LICENSE, EACH RECIPIENT SHALL NOT
      HAVE ANY LIABILITY FOR ANY EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING
      WITHOUT LIMITATION LOST PROFITS), HOWEVER CAUSED AND ON ANY THEORY OF
      LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
      NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OR DISTRIBUTION
      OF THE LICENSED WORK OR THE EXERCISE OF ANY RIGHTS GRANTED HEREUNDER, EVEN
      IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. THIS LIMITATION OF
      LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL INJURY
      RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
      PROHIBITS SUCH LIMITATION.

   (( 9. ))??
   GOVERNING LAW AND LEGAL ACTION.

      (( 9.1. ))??
      This License shall be governed by and construed in accordance with the
      laws of the Governing Jurisdiction assigned in Part 3 of Exhibit A,
      without regard to its conflict of law provisions. No party may bring a
      legal action under this License more than one year after the cause of the
      action arose. Each party waives its rights (if any) to a jury trial in any
      litigation arising under this License. Note that if the Governing
      Jurisdiction is not assigned in Part 3 of Exhibit A, then the Governing
      Jurisdiction shall be the State of New York.

      (( 9.2. ))??
      The courts of the Governing Jurisdiction shall have jurisdiction, but not
      exclusive jurisdiction, to entertain and determine all disputes and
      claims, whether for specific performance, injunction, damages or
      otherwise, both at law and in equity, arising out of or in any way
      relating to this License, including without limitation, the legality,
      validity, existence and enforceability of this License. Each party to this
      License hereby irrevocably attorns to and accepts the jurisdiction of the
      courts of the Governing Jurisdiction for such purposes.

      (( 9.3. ))??
      Except as expressly set forth elsewhere herein, in the event of any action
      or proceeding brought by any party against another under this License the
      prevailing party shall be entitled to recover all costs and expenses
      including the fees of its attorneys in such action or proceeding in such
      amount as the court may adjudge reasonable.

   (( 10. ))??
   MISCELLANEOUS.

      (( 10.1. ))??
      The obligations imposed by this License are for the benefit of the Initial
      Contributor and any Recipient, and each Recipient acknowledges and agrees
      that the Initial Contributor and/or any other Recipient may enforce the
      terms and conditions of this License against any Recipient.

      (( 10.2. ))??
      This License represents the complete agreement concerning subject matter
      hereof, and supersedes and cancels all previous oral and written
      communications, representations, agreements and understandings between the
      parties with respect to the subject matter hereof.

      (( 10.3. ))??
      The application of the United Nations Convention on Contracts for the
      International Sale of Goods is expressly excluded.

      (( 10.4. ))??
      The language in all parts of this License shall be in all cases construed
      simply according to its fair meaning, and not strictly for or against any
      of the parties hereto. Any law or regulation which provides that the
      language of a contract shall be construed against the drafter shall not
      apply to this License.

      (( 10.5. ))??
      If any provision of this License is invalid or unenforceable under the
      laws of the Governing Jurisdiction, it shall not affect the validity or
      enforceability of the remainder of the terms of this License, and without
      further action by the parties hereto, such provision shall be reformed to
      the minimum extent necessary to make such provision valid and
      enforceable.

      (( 10.6. ))??
      The paragraph headings of this License are for reference and convenience
      only and are not a part of this License, and they shall have no effect
      upon the construction or interpretation of any part hereof.

      (( 10.7. ))??
      Each of the terms "including", "include" and "includes", when used in this
      License, is not limiting whether or not non-limiting language (such as
      "without limitation" or "but not limited to" or words of similar import)
      is used with reference thereto.

      (( 10.8. ))??
      The parties hereto acknowledge they have expressly required that this
      License and notices relating thereto be drafted in the English language.
      (( /***THE LICENSE TERMS END HERE (OTHER THAN AS SET OUT IN EXHIBIT
      A).***/

EXHIBIT A (to the Adaptive Public License)

   (( PART 1: ))??
   INITIAL CONTRIBUTOR AND DESIGNATED WEB SITE

   The Initial Contributor is:

   _

   [Enter full name of Initial Contributor]

   Address of Initial Contributor:

   _

   _

   _

   [Enter address above]

   The Designated Web Site is:

   _

   [Enter URL for Designated Web Site of Initial Contributor]

   NOTE: The Initial Contributor is to complete this Part 1, along with Parts 2,
   3, and 5, and, if applicable, Parts 4 and 6.

   (( PART 2: ))??
   INITIAL WORK

   The Initial Work comprises the computer program(s) distributed by the Initial
   Contributor having the following title(s): _.

   The date on which the Initial Work was first available under this License: _

   (( PART 3: ))??
   GOVERNING JURISDICTION

   For the purposes of this License, the Governing Jurisdiction is _. [Initial
   Contributor to Enter Governing Jurisdiction here]

   (( PART 4: ))??
   THIRD PARTIES

   For the purposes of this License, "Third Party" has the definition set forth
   below in the ONE paragraph selected by the Initial Contributor from
   paragraphs A, B, C, D and E when the Initial Work is distributed or otherwise
   made available by the Initial Contributor. To select one of the following
   paragraphs, the Initial Contributor must place an "X" or "x" in the selection
   box alongside the one respective paragraph selected.

   SELECTION

   BOX PARAGRAPH

      [ ] A. "THIRD PARTY" means any third party.

      [ ] B. "THIRD PARTY" means any third party except for any of the
      following: (a) a wholly owned subsidiary of the Subsequent Contributor in
      question; (b) a legal entity (the "PARENT") that wholly owns the
      Subsequent Contributor in question; or (c) a wholly owned subsidiary of
      the wholly owned subsidiary in (a) or of the Parent in (b).

      [ ] C. "THIRD PARTY" means any third party except for any of the
      following: (a) any Person directly or indirectly owning a majority of the
      voting interest in the Subsequent Contributor or (b) any Person in which
      the Subsequent Contributor directly or indirectly owns a majority voting
      interest.

      [ ] D. "THIRD PARTY" means any third party except for any Person directly
      or indirectly controlled by the Subsequent Contributor. For purposes of
      this definition, "control" shall mean the power to direct or cause the
      direction of, the management and policies of such Person whether through
      the ownership of voting interests, by contract, or otherwise.

      [ ] E. "THIRD PARTY" means any third party except for any Person directly
      or indirectly controlling, controlled by, or under common control with the
      Subsequent Contributor. For purposes of this definition, "control" shall
      mean the power to direct or cause the direction of, the management and
      policies of such Person whether through the ownership of voting interests,
      by contract, or otherwise.

   The default definition of "THIRD PARTY" is the definition set forth in
   paragraph A, if NONE OR MORE THAN ONE of paragraphs A, B, C, D or E in this
   Part 4 are selected by the Initial Contributor.

   (( PART 5: ))??
   NOTICE

   THE LICENSED WORK IS PROVIDED UNDER THE TERMS OF THE ADAPTIVE PUBLIC LICENSE
   ("LICENSE") AS FIRST COMPLETED BY: _ [Insert the name of the Initial
   Contributor here]. ANY USE, PUBLIC DISPLAY, PUBLIC PERFORMANCE, REPRODUCTION
   OR DISTRIBUTION OF, OR PREPARATION OF DERIVATIVE WORKS BASED ON, THE LICENSED
   WORK CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS LICENSE AND ITS TERMS,
   WHETHER OR NOT SUCH RECIPIENT READS THE TERMS OF THE LICENSE. "LICENSED WORK"
   AND "RECIPIENT" ARE DEFINED IN THE LICENSE. A COPY OF THE LICENSE IS LOCATED
   IN THE TEXT FILE ENTITLED "LICENSE.TXT" ACCOMPANYING THE CONTENTS OF THIS
   FILE. IF A COPY OF THE LICENSE DOES NOT ACCOMPANY THIS FILE, A COPY OF THE
   LICENSE MAY ALSO BE OBTAINED AT THE FOLLOWING WEB SITE: _[Insert Initial
   Contributor's Designated Web Site here]

   Software distributed under the License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   (( PART 6: ))??
   PATENT LICENSING TERMS

   For the purposes of this License, paragraphs A, B, C, D and E of this Part 6
   of Exhibit A are only incorporated and form part of the terms of the License
   if the Initial Contributor places an "X" or "x" in the selection box
   alongside the YES answer to the question immediately below.

   Is this a Patents-Included License pursuant to Section 2.2 of the License?

      YES [ ]

      NO [ ]

   By default, if YES is not selected by the Initial Contributor, the answer is
   NO.

      (( A. ))??
      For the purposes of the paragraphs in this Part 6 of Exhibit A,
      "LICENSABLE" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights granted herein.

      (( B. ))??
      The Initial Contributor hereby grants all Recipients a world-wide,
      royalty-free, non-exclusive license, subject to third party intellectual
      property claims, under patent claim(s) Licensable by the Initial
      Contributor that are or would be infringed by the making, using, selling,
      offering for sale, having made, importing, exporting, transfer or disposal
      of such Initial Work or any portion thereof. Notwithstanding the
      foregoing, no patent license is granted under this Paragraph B by the
      Initial Contributor: (1) for any code that the Initial Contributor deletes
      from the Initial Work (or any portion thereof) distributed by the Initial
      Contributor prior to such distribution; (2) for any Modifications made to
      the Initial Work (or any portion thereof) by any other Person; or (3)
      separate from the Initial Work (or portions thereof) distributed or made
      available by the Initial Contributor.

      (( C. ))??
      Effective upon distribution by a Subsequent Contributor to a Third Party
      of any Modifications made by that Subsequent Contributor, such Subsequent
      Contributor hereby grants all Recipients a world-wide, royalty-free,
      non-exclusive license, subject to third party intellectual property
      claims, under patent claim(s) Licensable by such Subsequent Contributor
      that are or would be infringed by the making, using, selling, offering for
      sale, having made, importing, exporting, transfer or disposal of any such
      Modifications made by that Subsequent Contributor alone and/or in
      combination with its Subsequent Work (or portions of such combination) to
      make, use, sell, offer for sale, have made, import, export, transfer and
      otherwise dispose of:

         (( (1) ))??
         Modifications made by that Subsequent Contributor (or portions
         thereof); and

         (( (2) ))??
         the combination of Modifications made by that Subsequent Contributor
         with its Subsequent Work (or portions of such combination);

         (collectively and in each case, the "SUBSEQUENT CONTRIBUTOR VERSION").

         Notwithstanding the foregoing, no patent license is granted under this
         Paragraph C by such Subsequent Contributor: (1) for any code that such
         Subsequent Contributor deletes from the Subsequent Contributor Version
         (or any portion thereof) distributed by the Subsequent Contributor
         prior to such distribution; (2) for any Modifications made to the
         Subsequent Contributor Version (or any portion thereof) by any other
         Person; or (3) separate from the Subsequent Contributor Version (or
         portions thereof) distributed or made available by the Subsequent
         Contributor.

      (( D. ))??
      Effective upon distribution of any Licensed Work by a Distributor to a
      Third Party, such Distributor hereby grants all Recipients a world-wide,
      royalty-free, non-exclusive license, subject to third party intellectual
      property claims, under patent claim(s) Licensable by such Distributor that
      are or would be infringed by the making, using, selling, offering for
      sale, having made, importing, exporting, transfer or disposal of any such
      Licensed Work distributed by such Distributor, to make, use, sell, offer
      for sale, have made, import, export, transfer and otherwise dispose of
      such Licensed Work or portions thereof (collectively and in each case, the
      "DISTRIBUTOR VERSION"). Notwithstanding the foregoing, no patent license
      is granted under this Paragraph D by such Distributor: (1) for any code
      that such Distributor deletes from the Distributor Version (or any portion
      thereof) distributed by the Distributor prior to such distribution; (2)
      for any Modifications made to the Distributor Version (or any portion
      thereof) by any other Person; or (3) separate from the Distributor Version
      (or portions thereof) distributed or made available by the Distributor.

      (( E. ))??
      If Recipient institutes patent litigation against another Recipient (a
      "USER") with respect to a patent applicable to a computer program or
      software (including a cross-claim or counterclaim in a lawsuit, and
      whether or not any of the patent claims are directed to a system, method,
      process, apparatus, device, product, article of manufacture or any other
      form of patent claim), then any patent or copyright license granted by
      that User to such Recipient under this License or any other copy of this
      License shall terminate. The termination shall be effective ninety (90)
      days after notice of termination from User to Recipient, unless the
      Recipient withdraws the patent litigation claim before the end of the
      ninety (90) day period. To be effective, any such notice of license
      termination must include a specific list of applicable patents and/or a
      copy of the copyrighted work of User that User alleges will be infringed
      by Recipient upon License termination. License termination is only
      effective with respect to patents and/or copyrights for which proper
      notice has been given.

   (( PART 7: ))??
   SAMPLE REQUIREMENTS FOR THE DESCRIPTION OF DISTRIBUTED MODIFICATIONS

   Each Subsequent Contributor (including the Initial Contributor where the
   Initial Contributor qualifies as a Subsequent Contributor) is invited (but
   not required) to cause each Subsequent Work created or contributed to by that
   Subsequent Contributor to contain a file documenting the changes such
   Subsequent Contributor made to create that Subsequent Work and the date of
   any change.

/***EXHIBIT A ENDS HERE.***/ ))??
`
const license_APSL_1_0_lre = `//**
Apple Public Source License 1.0
https://spdx.org/licenses/APSL-1.0.json
https://fedoraproject.org/wiki/Licensing/Apple_Public_Source_License_1.0
**//

(( APPLE PUBLIC SOURCE LICENSE

Version 1.0 - March 16, 1999 ))??

(( Please read this License carefully before downloading this software. By
downloading and using this software, you are agreeing to be bound by the terms
of this License. If you do not or cannot agree to the terms of this License,
please do not download or use the software. ))??

   (( 1. ))??
   General; Definitions. This License applies to any program or other work which
   Apple Computer, Inc. ("Apple") publicly announces as subject to this Apple
   Public Source License and which contains a notice placed by Apple identifying
   such program or work as "Original Code" and stating that it is subject to the
   terms of this Apple Public Source License version 1.0 (or subsequent version
   thereof), as it may be revised from time to time by Apple ("License"). As
   used in this License:

      (( 1.1 ))??
      "Applicable Patents" mean: (a) in the case where Apple is the grantor of
      rights, (i) patents or patent applications that are now or hereafter
      acquired, owned by or assigned to Apple and (ii) whose claims cover
      subject matter contained in the Original Code, but only to the extent
      necessary to use, reproduce and/or distribute the Original Code without
      infringement; and (b) in the case where You are the grantor of rights, (i)
      patents and patent applications that are now or hereafter acquired, owned
      by or assigned to You and (ii) whose claims cover subject matter in Your
      Modifications, taken alone or in combination with Original Code.

      (( 1.2 ))??
      "Covered Code" means the Original Code, Modifications, the combination of
      Original Code and any Modifications, and/or any respective portions
      thereof.

      (( 1.3 ))??
      "Deploy" means to use, sublicense or distribute Covered Code other than
      for Your internal research and development (R&D), and includes without
      limitation, any and all internal use or distribution of Covered Code
      within Your business or organization except for R&D use, as well as direct
      or indirect sublicensing or distribution of Covered Code by You to any
      third party in any form or manner.

      (( 1.4 ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.5 ))??
      "Modifications" mean any addition to, deletion from, and/or change to, the
      substance and/or structure of Covered Code. When code is released as a
      series of files, a Modification is: (a) any addition to or deletion from
      the contents of a file containing Covered Code; and/or (b) any new file or
      other representation of computer program statements that contains any part
      of Covered Code.

      (( 1.6 ))??
      "Original Code" means the Source Code of a program or other work as
      originally made available by Apple under this License, including the
      Source Code of any updates or upgrades to such programs or works made
      available by Apple under this License, and that has been expressly
      identified by Apple as such in the header file(s) of such work.

      (( 1.7 ))??
      "Source Code" means the human readable form of a program or other work
      that is suitable for making modifications to it, including all modules it
      contains, plus any associated interface definition files, scripts used to
      control compilation and installation of an executable (object code).

      (( 1.8 ))??
      "You" or "Your" means an individual or a legal entity exercising rights
      under this License. For legal entities, "You" or "Your" includes any
      entity which controls, is controlled by, or is under common control with,
      You, where "control" means (a) the power, direct or indirect, to cause the
      direction or management of such entity, whether by contract or otherwise,
      or (b) ownership of fifty percent (50%) or more of the outstanding shares
      or beneficial ownership of such entity.

   (( 2. ))??
   Permitted Uses; Conditions & Restrictions. Subject to the terms and
   conditions of this License, Apple hereby grants You, effective on the date
   You accept this License and download the Original Code, a world-wide,
   royalty-free, non-exclusive license, to the extent of Apple's Applicable
   Patents and copyrights covering the Original Code, to do the following:

      (( 2.1 ))??
      You may use, copy, modify and distribute Original Code, with or without
      Modifications, solely for Your internal research and development, provided
      that You must in each instance:

         (( (a) ))??
         retain and reproduce in all copies of Original Code the copyright and
         other proprietary notices and disclaimers of Apple as they appear in
         the Original Code, and keep intact all notices in the Original Code
         that refer to this License;

         (( (b) ))??
         include a copy of this License with every copy of Source Code of
         Covered Code and documentation You distribute, and You may not offer or
         impose any terms on such Source Code that alter or restrict this
         License or the recipients' rights hereunder, except as permitted under
         Section 6; and

         (( c ))??
         completely and accurately document all Modifications that you have made
         and the date of each such Modification, designate the version of the
         Original Code you used, prominently include a file carrying such
         information with the Modifications, and duplicate the notice in Exhibit
         A in each file of the Source Code of all such Modifications.

      (( 2.2 ))??
      You may Deploy Covered Code, provided that You must in each instance:

         (( (a) ))??
         satisfy all the conditions of Section 2.1 with respect to the Source
         Code of the Covered Code;

         (( (b) ))??
         make all Your Deployed Modifications publicly available in Source Code
         form via electronic distribution (e.g. download from a web site) under
         the terms of this License and subject to the license grants set forth
         in Section 3 below, and any additional terms You may choose to offer
         under Section 6. You must continue to make the Source Code of Your
         Deployed Modifications available for as long as you Deploy the Covered
         Code or twelve (12) months from the date of initial Deployment,
         whichever is longer;

         (( c ))??
         must notify Apple and other third parties of how to obtain Your
         Deployed Modifications by filling out and submitting the required
         information found at
         http:/www.apple.com/publicsource/modifications.html; and

         (( (d) ))??
         if you Deploy Covered Code in object code, executable form only,
         include a prominent notice, in the code itself as well as in related
         documentation, stating that Source Code of the Covered Code is
         available under the terms of this License with information on how and
         where to obtain such Source Code.

   (( 3. ))??
   Your Grants. In consideration of, and as a condition to, the licenses granted
   to You under this License:

         (( (a) ))??
         You hereby grant to Apple and all third parties a non-exclusive,
         royalty-free license, under Your Applicable Patents and other
         intellectual property rights owned or controlled by You, to use,
         reproduce, modify, distribute and Deploy Your Modifications of the same
         scope and extent as Apple's licenses under Sections 2.1 and 2.2; and

         (( (b) ))??
         You hereby grant to Apple and its subsidiaries a non-exclusive,
         worldwide, royalty-free, perpetual and irrevocable license, under Your
         Applicable Patents and other intellectual property rights owned or
         controlled by You, to use, reproduce, execute, compile, display,
         perform, modify or have modified (for Apple and/or its subsidiaries),
         sublicense and distribute Your Modifications, in any form, through
         multiple tiers of distribution.

   (( 4. ))??
   Larger Works. You may create a Larger Work by combining Covered Code with
   other code not governed by the terms of this License and distribute the
   Larger Work as a single product. In each such instance, You must make sure
   the requirements of this License are fulfilled for the Covered Code or any
   portion thereof.

   (( 5. ))??
   Limitations on Patent License. Except as expressly stated in Section 2, no
   other patent rights, express or implied, are granted by Apple herein.
   Modifications and/or Larger Works may require additional patent licenses from
   Apple which Apple may grant in its sole discretion.

   (( 6. ))??
   Additional Terms. You may choose to offer, and to charge a fee for, warranty,
   support, indemnity or liability obligations and/or other rights consistent
   with the scope of the license granted herein ("Additional Terms") to one or
   more recipients of Covered Code. However, You may do so only on Your own
   behalf and as Your sole responsibility, and not on behalf of Apple. You must
   obtain the recipient's agreement that any such Additional Terms are offered
   by You alone, and You hereby agree to indemnify, defend and hold Apple
   harmless for any liability incurred by or claims asserted against Apple by
   reason of any such Additional Terms.

   (( 7. ))??
   Versions of the License. Apple may publish revised and/or new versions of
   this License from time to time. Each version will be given a distinguishing
   version number. Once Original Code has been published under a particular
   version of this License, You may continue to use it under the terms of that
   version. You may also choose to use such Original Code under the terms of any
   subsequent version of this License published by Apple. No one other than
   Apple has the right to modify the terms applicable to Covered Code created
   under this License.

   (( 8. ))??
   NO WARRANTY OR SUPPORT. The Original Code may contain in whole or in part
   pre-release, untested, or not fully tested works. The Original Code may
   contain errors that could cause failures or loss of data, and may be
   incomplete or contain inaccuracies. You expressly acknowledge and agree that
   use of the Original Code, or any portion thereof, is at Your sole and entire
   risk. THE ORIGINAL CODE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, UPGRADES OR
   SUPPORT OF ANY KIND AND APPLE AND APPLE'S LICENSOR(S) (FOR THE PURPOSES OF
   SECTIONS 8 AND 9, APPLE AND APPLE'S LICENSOR(S) ARE COLLECTIVELY REFERRED TO
   AS "APPLE") EXPRESSLY DISCLAIM ALL WARRANTIES AND/OR CONDITIONS, EXPRESS OR
   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES AND/OR
   CONDITIONS OF MERCHANTABILITY OR SATISFACTORY QUALITY AND FITNESS FOR A
   PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. APPLE DOES NOT
   WARRANT THAT THE FUNCTIONS CONTAINED IN THE ORIGINAL CODE WILL MEET YOUR
   REQUIREMENTS, OR THAT THE OPERATION OF THE ORIGINAL CODE WILL BE
   UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE ORIGINAL CODE WILL BE
   CORRECTED. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY APPLE OR AN
   APPLE AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY
   INCREASE THE SCOPE OF THIS WARRANTY. You acknowledge that the Original Code
   is not intended for use in the operation of nuclear facilities, aircraft
   navigation, communication systems, or air traffic control machines in which
   case the failure of the Original Code could lead to death, personal injury,
   or severe physical or environmental damage.

   (( 9. ))??
   Liability.

      (( 9.1 ))??
      Infringement. If any of the Original Code becomes the subject ofa claim of
      infringement ("Affected Original Code"), Apple may, at its sole discretion
      and option: (a) attempt to procure the rights necessary for You to
      continue using the Affected Original Code; (b) modify the Affected
      Original Code so that it is no longer infringing; or (c) terminate Your
      rights to use the Affected Original Code, effective immediately upon
      Apple's posting of a notice to such effect on the Apple web site that is
      used for implementation of this License.

      (( 9.2 ))??
      LIMITATION OF LIABILITY. UNDER NO CIRCUMSTANCES SHALL APPLE BE LIABLE FOR
      ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES ARISING OUT OF
      OR RELATING TO THIS LICENSE OR YOUR USE OR INABILITY TO USE THE ORIGINAL
      CODE, OR ANY PORTION THEREOF, WHETHER UNDER A THEORY OF CONTRACT,
      WARRANTY, TORT (INCLUDING NEGLIGENCE), PRODUCTS LIABILITY OR OTHERWISE,
      EVEN IF APPLE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES AND
      NOTWITHSTANDING THE FAILURE OF ESSENTIAL PURPOSE OF ANY REMEDY. In no
      event shall Apple's total liability to You for all damages under this
      License exceed the amount of fifty dollars ($50.00).

   (( 10. ))??
   Trademarks. This License does not grant any rights to use the trademarks or
   trade names "Apple", "Apple Computer", "Mac OS X", "Mac OS X Server" or any
   other trademarks or trade names belonging to Apple (collectively "Apple
   Marks") and no Apple Marks may be used to endorse or promote products derived
   from the Original Code

   other than as permitted by and in strict compliance at all times with Apple's
   third party trademark usage guidelines which are posted at
   http:/www.apple.com/legal/guidelinesfor3rdparties.html.

   (( 11. ))??
   Ownership. Apple retains all rights, title and interest in and to the
   Original Code and any Modifications made by or on behalf of Apple ("Apple
   Modifications"), and such Apple Modifications will not be automatically
   subject to this License. Apple may, at its sole discretion, choose to license
   such Apple Modifications under this License, or on different terms from those
   contained in this License or may choose not to license them at all. Apple's
   development, use, reproduction, modification, sublicensing and distribution
   of Covered Code will not be subject to this License.

   (( 12. ))??
   Termination.

      (( 12.1 ))??
      Termination. This License and the rights granted hereunder will
      terminate:

         (( (a) ))??
         automatically without notice from Apple if You fail to comply with any
         term(s) of this License and fail to cure such breach within 30 days of
         becoming aware of such breach;

         (( (b) ))??
         immediately in the event of the circumstances described in Sections 9.1
         and/or 13.6(b); or

         (( c ))??
         automatically without notice from Apple if You, at any time during the
         term of this License, commence an action for patent infringement
         against Apple.

      (( 12.2 ))??
      Effect of Termination. Upon termination, You agree to immediately stop any
      further use, reproduction, modification and distribution of the Covered
      Code, or Affected Original Code in the case of termination under Section
      9.1, and to destroy all copies of the Covered Code or Affected Original
      Code (in the case of

      termination under Section 9.1) that are in your possession or control. All
      sublicenses to the Covered Code which have been properly granted prior to
      termination shall survive any termination of this License. Provisions
      which, by their nature, should remain in effect beyond the termination of
      this License shall survive, including but not limited to Sections 3, 5, 8,
      9, 10, 11, 12.2 and 13. Neither party will be liable to the other for
      compensation, indemnity or damages of any sort solely as a result of
      terminating this License in accordance with its terms, and termination of
      this License will be without prejudice to any other right or remedy of
      either party.

   (( 13. ))??
   Miscellaneous.

      (( 13.1 ))??
      Export Law Assurances. You may not use or otherwise export or re-export
      the Original Code except as authorized by United States law and the laws
      of the jurisdiction in which the Original Code was obtained. In
      particular, but without limitation, the Original Code may not be exported
      or re-exported (a) into (or to a national or resident of) any U.S.
      embargoed country or (b) to anyone on the U.S. Treasury Department's list
      of Specially Designated Nationals or the U.S. Department of Commerce's
      Table of Denial Orders. By using the Original Code, You represent and
      warrant that You are not located in, under control of, or a national or
      resident of any such country or on any such list.

      (( 13.2 ))??
      Government End Users. The Covered Code is a "commercial item" as defined
      in FAR 2.101. Government software and technical data rights in the Covered
      Code include only those rights customarily provided to the public as
      defined in this License. This customary commercial license in technical
      data and software is provided in

      accordance with FAR 12.211 (Technical Data) and 12.212 (Computer Software)
      and, for Department of Defense purchases, DFAR 252.227-7015 (Technical
      Data -- Commercial Items) and 227.7202-3 (Rights in Commercial Computer
      Software or Computer Software Documentation). Accordingly, all U.S.
      Government End Users acquire Covered Code with only those rights set forth
      herein.

      (( 13.3 ))??
      Relationship of Parties. This License will not be construed as creating an
      agency, partnership, joint venture or any other form of legal association
      between You and Apple, and You will not represent to the contrary, whether
      expressly, by implication, appearance or otherwise.

      (( 13.4 ))??
      Independent Development. Nothing in this License will impair Apple's right
      to acquire, license, develop, have others develop for it, market and/or
      distribute technology or products that perform the same or similar
      functions as, or otherwise compete with, Modifications, Larger Works,
      technology or products that You may develop, produce, market or
      distribute.

      (( 13.5 ))??
      Waiver; Construction. Failure by Apple to enforce any provision of this
      License will not be deemed a waiver of future enforcement of that or any
      other provision. Any law or regulation which provides that the language of
      a contract shall be construed against the drafter will not apply to this
      License.

      (( 13.6 ))??
      Severability. (a) If for any reason a court of competent jurisdiction
      finds any provision of this License, or portion thereof, to be
      unenforceable, that provision of the License will be enforced to the
      maximum extent permissible so as to effect the economic benefits and
      intent of the parties, and the remainder of this License will continue in
      full force and effect. (b) Notwithstanding the foregoing, if applicable
      law prohibits or restricts You from fully and/or specifically complying
      with Sections 2 and/or 3 or prevents the enforceability of either of those
      Sections, this License will immediately terminate and You must immediately
      discontinue any use of the Covered Code and destroy all copies of it that
      are in your possession or control.

      (( 13.7 ))??
      Dispute Resolution. Any litigation or other dispute resolution between You
      and Apple relating to this License shall take place in the Northern
      District of California, and You and Apple hereby consent to the personal
      jurisdiction of, and venue in, the state and federal courts within that
      District with respect to this License. The application of the United
      Nations Convention on Contracts for the International Sale of Goods is
      expressly excluded.

      (( 13.8 ))??
      Entire Agreement; Governing Law. This License constitutes the entire
      agreement between the parties with respect to the subject matter hereof.
      This License shall be governed by the laws of the United States and the
      State of California, except that body of California law concerning
      conflicts of law.

      Where You are located in the province of Quebec, Canada, the following
      clause applies: The parties hereby confirm that they have requested that
      this License and all related documents be drafted in English. Les parties
      ont exige que le present contrat et tous les documents connexes soient
      rediges en anglais.
      (( EXHIBIT A.

"Portions Copyright (c) 1999 Apple Computer, Inc. All Rights Reserved. This file
contains Original Code and/or Modifications of Original Code as defined in and
that are subject to the Apple Public Source License Version 1.0 (the 'License').
You may not use this file except in compliance with the License. Please obtain a
copy of the License at http:/www.apple.com/publicsource and read it before using
this file.

The Original Code and all software distributed under the License are distributed
on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION,
ANY WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
NON-INFRINGEMENT. Please see the License for the specific language governing
rights and limitations under the License." ))??
`
const license_APSL_1_1_lre = `//**
Apple Public Source License 1.1
https://spdx.org/licenses/APSL-1.1.json
http://www.opensource.apple.com/source/IOSerialFamily/IOSerialFamily-7/APPLE_LICENSE
**//

(( APPLE PUBLIC SOURCE LICENSE

Version 1.1 - April 19, 1999 ))??

(( Please read this License carefully before downloading this software.

By downloading and using this software, you are agreeing to be bound by the
terms of this License. If you do not or cannot agree to the terms of this
License, please do not download or use the software. ))??

   (( 1. ))??
   General; Definitions. This License applies to any program or other work which
   Apple Computer, Inc. ("Apple") publicly announces as subject to this Apple
   Public Source License and which contains a notice placed by Apple identifying
   such program or work as "Original Code" and stating that it is subject to the
   terms of this Apple Public Source License version 1.1 (or subsequent version
   thereof), as it may be revised from time to time by Apple ("License"). As
   used in this License:

      (( 1.1 ))??
      "Affected Original Code" means only those specific portions of Original
      Code that allegedly infringe upon any party's intellectual property rights
      or are otherwise the subject of a claim of infringement.

      (( 1.2 ))??
      "Applicable Patent Rights" mean: (a) in the case where Apple is the
      grantor of rights, (i) claims of patents that are now or hereafter
      acquired, owned by or assigned to Apple and (ii) that cover subject matter
      contained in the Original Code, but only to the extent necessary to use,
      reproduce and/or distribute the Original Code without infringement; and
      (b) in the case where You are the grantor of rights, (i) claims of patents
      that are now or hereafter acquired, owned by or assigned to You and (ii)
      that cover subject matter in Your Modifications, taken alone or in
      combination with Original Code.

      (( 1.3 ))??
      "Covered Code" means the Original Code, Modifications, the combination of
      Original Code and any Modifications, and/or any respective portions
      thereof.

      (( 1.4 ))??
      "Deploy" means to use, sublicense or distribute Covered Code other than
      for Your internal research and development (R&D), and includes without
      limitation, any and all internal use or distribution of Covered Code
      within Your business or organization except for R&D use, as well as direct
      or indirect sublicensing or distribution of Covered Code by You to any
      third party in any form or manner.

      (( 1.5 ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.6 ))??
      "Modifications" mean any addition to, deletion from, and/or change to, the
      substance and/or structure of Covered Code. When code is released as a
      series of files, a Modification is: (a) any addition to or deletion from
      the contents of a file containing Covered Code; and/or (b) any new file or
      other representation of computer program statements that contains any part
      of Covered Code.

      (( 1.7 ))??
      "Original Code" means (a) the Source Code of a program or other work as
      originally made available by Apple under this License, including the
      Source Code of any updates or upgrades to such programs or works made
      available by Apple under this License, and that has been expressly
      identified by Apple as such in the header file(s) of such work; and (b)
      the object code compiled from such Source Code and originally made
      available by Apple under this License.

      (( 1.8 ))??
      "Source Code" means the human readable form of a program or other work
      that is suitable for making modifications to it, including all modules it
      contains, plus any associated interface definition files, scripts used to
      control compilation and installation of an executable (object code).

      (( 1.9 ))??
      "You" or "Your" means an individual or a legal entity exercising rights
      under this License. For legal entities, "You" or "Your" includes any
      entity which controls, is controlled by, or is under common control with,
      You, where "control" means (a) the power, direct or indirect, to cause the
      direction or management of such entity, whether by contract or otherwise,
      or (b) ownership of fifty percent (50%) or more of the outstanding shares
      or beneficial ownership of such entity.

   (( 2. ))??
   Permitted Uses; Conditions & Restrictions. Subject to the terms and
   conditions of this License, Apple hereby grants You, effective on the date
   You accept this License and download the Original Code, a world-wide,
   royalty-free, non- exclusive license, to the extent of Apple's Applicable
   Patent Rights and copyrights covering the Original Code, to do the
   following:

      (( 2.1 ))??
      You may use, copy, modify and distribute Original Code, with or without
      Modifications, solely for Your internal research and development, provided
      that You must in each instance:

         (( (a) ))??
         retain and reproduce in all copies of Original Code the copyright and
         other proprietary notices and disclaimers of Apple as they appear in
         the Original Code, and keep intact all notices in the Original Code
         that refer to this License;

         (( (b) ))??
         include a copy of this License with every copy of Source Code of
         Covered Code and documentation You distribute, and You may not offer or
         impose any terms on such Source Code that alter or restrict this
         License or the recipients' rights hereunder, except as permitted under
         Section 6; and

         (( c ))??
         completely and accurately document all Modifications that you have made
         and the date of each such Modification, designate the version of the
         Original Code you used, prominently include a file carrying such
         information with the Modifications, and duplicate the notice in Exhibit
         A in each file of the Source Code of all such Modifications.

      (( 2.2 ))??
      You may Deploy Covered Code, provided that You must in each instance:

         (( (a) ))??
         satisfy all the conditions of Section 2.1 with respect to the Source
         Code of the Covered Code;

         (( (b) ))??
         make all Your Deployed Modifications publicly available in Source Code
         form via electronic distribution (e.g. download from a web site) under
         the terms of this License and subject to the license grants set forth
         in Section 3 below, and any additional terms You may choose to offer
         under Section 6. You must continue to make the Source Code of Your
         Deployed Modifications available for as long as you Deploy the Covered
         Code or twelve (12) months from the date of initial Deployment,
         whichever is longer;

         (( c ))??
         if You Deploy Covered Code containing Modifications made by You, inform
         others of how to obtain those Modifications by filling out and
         submitting the information found at
         http:/www.apple.com/publicsource/modifications.html, if available; and

         (( (d) ))??
         if You Deploy Covered Code in object code, executable form only,
         include a prominent notice, in the code itself as well as in related
         documentation, stating that Source Code of the Covered Code is
         available under the terms of this License with information on how and
         where to obtain such Source Code.

   (( 3. ))??
   Your Grants. In consideration of, and as a condition to, the licenses granted
   to You under this License:

      (( (a) ))??
      You hereby grant to Apple and all third parties a non-exclusive,
      royalty-free license, under Your Applicable Patent Rights and other
      intellectual property rights owned or controlled by You, to use,
      reproduce, modify, distribute and Deploy Your Modifications of the same
      scope and extent as Apple's licenses under Sections 2.1 and 2.2; and

      (( (b) ))??
      You hereby grant to Apple and its subsidiaries a non-exclusive, worldwide,
      royalty-free, perpetual and irrevocable license, under Your Applicable
      Patent Rights and other intellectual property rights owned or controlled
      by You, to use, reproduce, execute, compile, display, perform, modify or
      have modified (for Apple and/or its subsidiaries), sublicense and
      distribute Your Modifications, in any form, through multiple tiers of
      distribution.

   (( 4. ))??
   Larger Works. You may create a Larger Work by combining Covered Code with
   other code not governed by the terms of this License and distribute the
   Larger Work as a single product. In each such instance, You must make sure
   the requirements of this License are fulfilled for the Covered Code or any
   portion thereof.

   (( 5. ))??
   Limitations on Patent License. Except as expressly stated in Section 2, no
   other patent rights, express or implied, are granted by Apple herein.
   Modifications and/or Larger Works may require additional patent licenses from
   Apple which Apple may grant in its sole discretion.

   (( 6. ))??
   Additional Terms. You may choose to offer, and to charge a fee for, warranty,
   support, indemnity or liability obligations and/or other rights consistent
   with the scope of the license granted herein ("Additional Terms") to one or
   more recipients of Covered Code. However, You may do so only on Your own
   behalf and as Your sole responsibility, and not on behalf of Apple. You must
   obtain the recipient's agreement that any such Additional Terms are offered
   by You alone, and You hereby agree to indemnify, defend and hold Apple
   harmless for any liability incurred by or claims asserted against Apple by
   reason of any such Additional Terms.

   (( 7. ))??
   Versions of the License. Apple may publish revised and/or new versions of
   this License from time to time. Each version will be given a distinguishing
   version number. Once Original Code has been published under a particular
   version of this License, You may continue to use it under the terms of that
   version. You may also choose to use such Original Code under the terms of any
   subsequent version of this License published by Apple. No one other than
   Apple has the right to modify the terms applicable to Covered Code created
   under this License.

   (( 8. ))??
   NO WARRANTY OR SUPPORT. The Original Code may contain in whole or in part
   pre-release, untested, or not fully tested works. The Original Code may
   contain errors that could cause failures or loss of data, and may be
   incomplete or contain inaccuracies. You expressly acknowledge and agree that
   use of the Original Code, or any portion thereof, is at Your sole and entire
   risk. THE ORIGINAL CODE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, UPGRADES OR
   SUPPORT OF ANY KIND AND APPLE AND APPLE'S LICENSOR(S) (FOR THE PURPOSES OF
   SECTIONS 8 AND 9, APPLE AND APPLE'S LICENSOR(S) ARE COLLECTIVELY REFERRED TO
   AS "APPLE") EXPRESSLY DISCLAIM ALL WARRANTIES AND/OR CONDITIONS, EXPRESS OR
   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES AND/OR
   CONDITIONS OF MERCHANTABILITY OR SATISFACTORY QUALITY AND FITNESS FOR A
   PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. APPLE DOES NOT
   WARRANT THAT THE FUNCTIONS CONTAINED IN THE ORIGINAL CODE WILL MEET YOUR
   REQUIREMENTS, OR THAT THE OPERATION OF THE ORIGINAL CODE WILL BE
   UNINTERRUPTED OR ERROR- FREE, OR THAT DEFECTS IN THE ORIGINAL CODE WILL BE
   CORRECTED. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY APPLE OR AN
   APPLE AUTHORIZED REPRESENTATIVE SHALL CREATE A WARRANTY OR IN ANY WAY
   INCREASE THE SCOPE OF THIS WARRANTY. You acknowledge that the Original Code
   is not intended for use in the operation of nuclear facilities, aircraft
   navigation, communication systems, or air traffic control machines in which
   case the failure of the Original Code could lead to death, personal injury,
   or severe physical or environmental damage.

   (( 9. ))??
   Liability.

      (( 9.1 ))??
      Infringement. If any portion of, or functionality implemented by, the
      Original Code becomes the subject of a claim of infringement, Apple may,
      at its option: (a) attempt to procure the rights necessary for Apple and
      You to continue using the Affected Original Code; (b) modify the Affected
      Original Code so that it is no longer infringing; or (c) suspend Your
      rights to use, reproduce, modify, sublicense and distribute the Affected
      Original Code until a final determination of the claim is made by a court
      or governmental administrative agency of competent jurisdiction and Apple
      lifts the suspension as set forth below. Such suspension of rights will be
      effective immediately upon Apple's posting of a notice to such effect on
      the Apple web site that is used for implementation of this License. Upon
      such final determination being made, if Apple is legally able, without the
      payment of a fee or royalty, to resume use, reproduction, modification,
      sublicensing and distribution of the Affected Original Code, Apple will
      lift the suspension of rights to the Affected Original Code by posting a
      notice to such effect on the Apple web site that is used for
      implementation of this License. If Apple suspends Your rights to Affected
      Original Code, nothing in this License shall be construed to restrict You,
      at Your option and subject to applicable law, from replacing the Affected
      Original Code with non-infringing code or independently negotiating for
      necessary rights from such third party.

      (( 9.2 ))??
      LIMITATION OF LIABILITY. UNDER NO CIRCUMSTANCES SHALL APPLE BE LIABLE FOR
      ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES ARISING OUT OF
      OR RELATING TO THIS LICENSE OR YOUR USE OR INABILITY TO USE THE ORIGINAL
      CODE, OR ANY PORTION THEREOF, WHETHER UNDER A THEORY OF CONTRACT,
      WARRANTY, TORT (INCLUDING NEGLIGENCE), PRODUCTS LIABILITY OR OTHERWISE,
      EVEN IF APPLE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES AND
      NOTWITHSTANDING THE FAILURE OF ESSENTIAL PURPOSE OF ANY REMEDY. In no
      event shall Apple's total liability to You for all damages under this
      License exceed the amount of fifty dollars ($50.00).

   (( 10. ))??
   Trademarks. This License does not grant any rights to use the trademarks or
   trade names "Apple", "Apple Computer", "Mac OS X", "Mac OS X Server" or any
   other trademarks or trade names belonging to Apple (collectively "Apple
   Marks") and no Apple Marks may be used to endorse or promote products derived
   from the Original Code other than as permitted by and in strict compliance at
   all times with Apple's third party trademark usage guidelines which are
   posted at http:/www.apple.com/legal/guidelinesfor3rdparties.html.

   (( 11. ))??
   Ownership. Apple retains all rights, title and interest in and to the
   Original Code and any Modifications made by or on behalf of Apple ("Apple
   Modifications"), and such Apple Modifications will not be automatically
   subject to this License. Apple may, at its sole discretion, choose to license
   such Apple Modifications under this License, or on different terms from those
   contained in this License or may choose not to license them at all. Apple's
   development, use, reproduction, modification, sublicensing and distribution
   of Covered Code will not be subject to this License.

   (( 12. ))??
   Termination.

      (( 12.1 ))??
      Termination. This License and the rights granted hereunder will
      terminate:

         (( (a) ))??
         automatically without notice from Apple if You fail to comply with any
         term(s) of this License and fail to cure such breach within 30 days of
         becoming aware of such breach;

         (( (b) ))??
         immediately in the event of the circumstances described in Section
         13.5(b); or

         (( c ))??
         automatically without notice from Apple if You, at any time during the
         term of this License, commence an action for patent infringement
         against Apple.

      (( 12.2 ))??
      Effect of Termination. Upon termination, You agree to immediately stop any
      further use, reproduction, modification, sublicensing and distribution of
      the Covered Code and to destroy all copies of the Covered Code that are in
      your possession or control. All sublicenses to the Covered Code which have
      been properly granted prior to termination shall survive any termination
      of this License. Provisions which, by their nature, should remain in
      effect beyond the termination of this License shall survive, including but
      not limited to Sections 3, 5, 8, 9, 10, 11, 12.2 and 13. Neither party
      will be liable to the other for compensation, indemnity or damages of any
      sort solely as a result of terminating this License in accordance with its
      terms, and termination of this License will be without prejudice to any
      other right or remedy of either party.

   (( 13. ))??
   Miscellaneous.

      (( 13.1 ))??
      Government End Users. The Covered Code is a "commercial item" as defined
      in FAR 2.101. Government software and technical data rights in the Covered
      Code include only those rights customarily provided to the public as
      defined in this License. This customary commercial license in technical
      data and software is provided in accordance with FAR 12.211 (Technical
      Data) and 12.212 (Computer Software) and, for Department of Defense
      purchases, DFAR 252.227-7015 (Technical Data -- Commercial Items) and
      227.7202-3 (Rights in Commercial Computer Software or Computer Software
      Documentation). Accordingly, all U.S. Government End Users acquire Covered
      Code with only those rights set forth herein.

      (( 13.2 ))??
      Relationship of Parties. This License will not be construed as creating an
      agency, partnership, joint venture or any other form of legal association
      between You and Apple, and You will not represent to the contrary, whether
      expressly, by implication, appearance or otherwise.

      (( 13.3 ))??
      Independent Development. Nothing in this License will impair Apple's right
      to acquire, license, develop, have others develop for it, market and/or
      distribute technology or products that perform the same or similar
      functions as, or otherwise compete with, Modifications, Larger Works,
      technology or products that You may

      develop, produce, market or distribute.

      (( 13.4 ))??
      Waiver; Construction. Failure by Apple to enforce any provision of this
      License will not be deemed a waiver of future enforcement of that or any
      other provision. Any law or regulation which provides that the language of
      a contract shall be construed against the drafter will not apply to this
      License.

      (( 13.5 ))??
      Severability. (a) If for any reason a court of competent jurisdiction
      finds any provision of this License, or portion thereof, to be
      unenforceable, that provision of the License will be enforced to the
      maximum extent permissible so as to effect the economic benefits and
      intent of the parties, and the remainder of this License will continue in
      full force and effect. (b) Notwithstanding the foregoing, if applicable
      law prohibits or restricts You from fully and/or specifically complying
      with Sections 2 and/or 3 or prevents the enforceability of either of those
      Sections, this License will immediately terminate and You must immediately
      discontinue any use of the Covered Code and destroy all copies of it that
      are in your possession or control.

      (( 13.6 ))??
      Dispute Resolution. Any litigation or other dispute resolution between You
      and Apple relating to this License shall take place in the Northern
      District of California, and You and Apple hereby consent to the personal
      jurisdiction of, and venue in, the state and federal courts within that
      District with respect to this License. The application of the United
      Nations Convention on Contracts for the International Sale of Goods is
      expressly excluded.

      (( 13.7 ))??
      Entire Agreement; Governing Law. This License constitutes the entire
      agreement between the parties with respect to the subject matter hereof.
      This License shall be governed by the laws of the United States and the
      State of California, except that body of California law concerning
      conflicts of law.

      Where You are located in the province of Quebec, Canada, the following
      clause applies: The parties hereby confirm that they have requested that
      this License and all related documents be drafted in English. Les parties
      ont exige que le present contrat et tous les documents connexes soient
      rediges en anglais.
      (( EXHIBIT A.

"Portions Copyright (c) 1999-2000 Apple Computer, Inc. All Rights Reserved. This
file contains Original Code and/or Modifications of Original Code as defined in
and that are subject to the Apple Public Source License Version 1.1 (the
"License"). You may not use this file except in compliance with the License.
Please obtain a copy of the License at http:/www.apple.com/publicsource and read
it before using this file.

The Original Code and all software distributed under the License are distributed
on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION,
ANY WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-
INFRINGEMENT. Please see the License for the specific language governing rights
and limitations under the License." ))??
`
const license_APSL_1_2_lre = `//**
Apple Public Source License 1.2
https://spdx.org/licenses/APSL-1.2.json
http://www.samurajdata.se/opensource/mirror/licenses/apsl.php
**//

(( Apple Public Source License Ver. 1.2 ))??

   (( 1. ))??
   General; Definitions. This License applies to any program or other work which
   Apple Computer, Inc. ("Apple") makes publicly available and which contains a
   notice placed by Apple identifying such program or work as "Original Code"
   and stating that it is subject to the terms of this Apple Public Source
   License version 1.2 (or subsequent version thereof) ("License"). As used in
   this License:

      (( 1.1 ))??
      "Applicable Patent Rights" mean: (a) in the case where Apple is the
      grantor of rights, (i) claims of patents that are now or hereafter
      acquired, owned by or assigned to Apple and (ii) that cover subject matter
      contained in the Original Code, but only to the extent necessary to use,
      reproduce and/or distribute the Original Code without infringement; and
      (b) in the case where You are the grantor of rights, (i) claims of patents
      that are now or hereafter acquired, owned by or assigned to You and (ii)
      that cover subject matter in Your Modifications, taken alone or in
      combination with Original Code.

      (( 1.2 ))??
      "Contributor" means any person or entity that creates or contributes to
      the creation of Modifications.

      (( 1.3 ))??
      "Covered Code" means the Original Code, Modifications, the combination of
      Original Code and any Modifications, and/or any respective portions
      thereof.

      (( 1.4 ))??
      "Deploy" means to use, sublicense or distribute Covered Code other than
      for Your internal research and development (R&D) and/or Personal Use, and
      includes without limitation, any and all internal use or distribution of
      Covered Code within Your business or organization except for R&D use
      and/or Personal Use, as well as direct or indirect sublicensing or
      distribution of Covered Code by You to any third party in any form or
      manner.

      (( 1.5 ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.6 ))??
      "Modifications" mean any addition to, deletion from, and/or change to, the
      substance and/or structure of the Original Code, any previous
      Modifications, the combination of Original Code and any previous
      Modifications, and/or any respective portions thereof. When code is
      released as a series of files, a Modification is: (a) any addition to or
      deletion from the contents of a file containing Covered Code; and/or (b)
      any new file or other representation of computer program statements that
      contains any part of Covered Code.

      (( 1.7 ))??
      "Original Code" means (a) the Source Code of a program or other work as
      originally made available by Apple under this License, including the
      Source Code of any updates or upgrades to such programs or works made
      available by Apple under this License, and that has been expressly
      identified by Apple as such in the header file(s) of such work; and (b)
      the object code compiled from such Source Code and originally made
      available by Apple under this License.

      (( 1.8 ))??
      "Personal Use" means use of Covered Code by an individual solely for his
      or her personal, private and non-commercial purposes. An individual's use
      of Covered Code in his or her capacity as an officer, employee, member,
      independent contractor or agent of a corporation, business or organization
      (commercial or non-commercial) does not qualify as Personal Use.

      (( 1.9 ))??
      "Source Code" means the human readable form of a program or other work
      that is suitable for making modifications to it, including all modules it
      contains, plus any associated interface definition files, scripts used to
      control compilation and installation of an executable (object code).

      (( 1.10 ))??
      "You" or "Your" means an individual or a legal entity exercising rights
      under this License. For legal entities, "You" or "Your" includes any
      entity which controls, is controlled by, or is under common control with,
      You, where "control" means (a) the power, direct or indirect, to cause the
      direction or management of such entity, whether by contract or otherwise,
      or (b) ownership of fifty percent (50%) or more of the outstanding shares
      or beneficial ownership of such entity.

   (( 2. ))??
   Permitted Uses; Conditions & Restrictions.Subject to the terms and conditions
   of this License, Apple hereby grants You, effective on the date You accept
   this License and download the Original Code, a world-wide, royalty-free,
   non-exclusive license, to the extent of Apple's Applicable Patent Rights and
   copyrights covering the Original Code, to do the following:

      (( 2.1 ))??
      You may use, reproduce, display, perform, modify and distribute Original
      Code, with or without Modifications, solely for Your internal research and
      development and/or Personal Use, provided that in each instance:

         (( (a) ))??
         You must retain and reproduce in all copies of Original Code the
         copyright and other proprietary notices and disclaimers of Apple as
         they appear in the Original Code, and keep intact all notices in the
         Original Code that refer to this License; and

         (( (b) ))??
         You must include a copy of this License with every copy of Source Code
         of Covered Code and documentation You distribute, and You may not offer
         or impose any terms on such Source Code that alter or restrict this
         License or the recipients' rights hereunder, except as permitted under
         Section 6.

      (( 2.2 ))??
      You may use, reproduce, display, perform, modify and Deploy Covered Code,
      provided that in each instance:

         (( (a) ))??
         You must satisfy all the conditions of Section 2.1 with respect to the
         Source Code of the Covered Code;

         (( (b) ))??
         You must duplicate, to the extent it does not already exist, the notice
         in Exhibit A in each file of the Source Code of all Your Modifications,
         and cause the modified files to carry prominent notices stating that
         You changed the files and the date of any change;

         (( c ))??
         You must make Source Code of all Your Deployed Modifications publicly
         available under the terms of this License, including the license grants
         set forth in Section 3 below, for as long as you Deploy the Covered
         Code or twelve (12) months from the date of initial Deployment,
         whichever is longer. You should preferably distribute the Source Code
         of Your Deployed Modifications electronically (e.g. download from a web
         site); and

         (( (d) ))??
         if You Deploy Covered Code in object code, executable form only, You
         must include a prominent notice, in the code itself as well as in
         related documentation, stating that Source Code of the Covered Code is
         available under the terms of this License with information on how and
         where to obtain such Source Code.

      (( 2.3 ))??
      You expressly acknowledge and agree that although Apple and each
      Contributor grants the licenses to their respective portions of the
      Covered Code set forth herein, no assurances are provided by Apple or any
      Contributor that the Covered Code does not infringe the patent or other
      intellectual property rights of any other entity. Apple and each
      Contributor disclaim any liability to You for claims brought by any other
      entity based on infringement of intellectual property rights or otherwise.
      As a condition to exercising the rights and licenses granted hereunder,
      You hereby assume sole responsibility to secure any other intellectual
      property rights needed, if any. For example, if a third party patent
      license is required to allow You to distribute the Covered Code, it is
      Your responsibility to acquire that license before distributing the
      Covered Code.

   (( 3. ))??
   Your Grants. In consideration of, and as a condition to, the licenses granted
   to You under this License:

      (( (a) ))??
      You hereby grant to Apple and all third parties a non-exclusive,
      royalty-free license, under Your Applicable Patent Rights and other
      intellectual property rights (other than patent) owned or controlled by
      You, to use, reproduce, display, perform, modify, distribute and Deploy
      Your Modifications of the same scope and extent as Apple's licenses under
      Sections 2.1 and 2.2; and

      (( (b) ))??
      You hereby grant to Apple and its subsidiaries a non-exclusive, worldwide,
      royalty-free, perpetual and irrevocable license, under Your Applicable
      Patent Rights and other intellectual property rights (other than patent)
      owned or controlled by You, to use, reproduce, display, perform, modify or
      have modified (for Apple and/or its subsidiaries), sublicense and
      distribute Your Modifications, in any form, through multiple tiers of
      distribution.

   (( 4. ))??
   Larger Works. You may create a Larger Work by combining Covered Code with
   other code not governed by the terms of this License and distribute the
   Larger Work as a single product. In each such instance, You must make sure
   the requirements of this License are fulfilled for the Covered Code or any
   portion thereof.

   (( 5. ))??
   Limitations on Patent License. Except as expressly stated in Section 2, no
   other patent rights, express or implied, are granted by Apple herein.
   Modifications and/or Larger Works may require additional patent licenses from
   Apple which Apple may grant in its sole discretion.

   (( 6. ))??
   Additional Terms. You may choose to offer, and to charge a fee for, warranty,
   support, indemnity or liability obligations and/or other rights consistent
   with the scope of the license granted herein ("Additional Terms") to one or
   more recipients of Covered Code. However, You may do so only on Your own
   behalf and as Your sole responsibility, and not on behalf of Apple or any
   Contributor. You must obtain the recipient's agreement that any such
   Additional Terms are offered by You alone, and You hereby agree to indemnify,
   defend and hold Apple and every Contributor harmless for any liability
   incurred by or claims asserted against Apple or such Contributor by reason of
   any such Additional Terms.

   (( 7. ))??
   Versions of the License. Apple may publish revised and/or new versions of
   this License from time to time. Each version will be given a distinguishing
   version number. Once Original Code has been published under a particular
   version of this License, You may continue to use it under the terms of that
   version. You may also choose to use such Original Code under the terms of any
   subsequent version of this License published by Apple. No one other than
   Apple has the right to modify the terms applicable to Covered Code created
   under this License.

   (( 8. ))??
   NO WARRANTY OR SUPPORT. The Covered Code may contain in whole or in part
   pre-release, untested, or not fully tested works. The Covered Code may
   contain errors that could cause failures or loss of data, and may be
   incomplete or contain inaccuracies. You expressly acknowledge and agree that
   use of the Covered Code, or any portion thereof, is at Your sole and entire
   risk. THE COVERED CODE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, UPGRADES OR
   SUPPORT OF ANY KIND AND APPLE AND APPLE'S LICENSOR(S) (COLLECTIVELY REFERRED
   TO AS "APPLE" FOR THE PURPOSES OF SECTIONS 8 AND 9) AND ALL CONTRIBUTORS
   EXPRESSLY DISCLAIM ALL WARRANTIES AND/OR CONDITIONS, EXPRESS OR IMPLIED,
   INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES AND/OR CONDITIONS OF
   MERCHANTABILITY, OF SATISFACTORY QUALITY, OF FITNESS FOR A PARTICULAR
   PURPOSE, OF ACCURACY, OF QUIET ENJOYMENT, AND NONINFRINGEMENT OF THIRD PARTY
   RIGHTS. APPLE AND EACH CONTRIBUTOR DOES NOT WARRANT AGAINST INTERFERENCE WITH
   YOUR ENJOYMENT OF THE COVERED CODE, THAT THE FUNCTIONS CONTAINED IN THE
   COVERED CODE WILL MEET YOUR REQUIREMENTS, THAT THE OPERATION OF THE COVERED
   CODE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE COVERED CODE
   WILL BE CORRECTED. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY APPLE,
   AN APPLE AUTHORIZED REPRESENTATIVE OR ANY CONTRIBUTOR SHALL CREATE A
   WARRANTY. You acknowledge that the Covered Code is not intended for use in
   the operation of nuclear facilities, aircraft navigation, communication
   systems, or air traffic control machines in which case the failure of the
   Covered Code could lead to death, personal injury, or severe physical or
   environmental damage.

   (( 9. ))??
   LIMITATION OF LIABILITY. TO THE EXTENT NOT PROHIBITED BY LAW, IN NO EVENT
   SHALL APPLE OR ANY CONTRIBUTOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
   INDIRECT OR CONSEQUENTIAL DAMAGES ARISING OUT OF OR RELATING TO THIS LICENSE
   OR YOUR USE OR INABILITY TO USE THE COVERED CODE, OR ANY PORTION THEREOF,
   WHETHER UNDER A THEORY OF CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE),
   PRODUCTS LIABILITY OR OTHERWISE, EVEN IF APPLE OR SUCH CONTRIBUTOR HAS BEEN
   ADVISED OF THE POSSIBILITY OF SUCH DAMAGES AND NOTWITHSTANDING THE FAILURE OF
   ESSENTIAL PURPOSE OF ANY REMEDY. SOME JURISDICTIONS DO NOT ALLOW THE
   LIMITATION OF LIABILITY OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS
   LIMITATION MAY NOT APPLY TO YOU. In no event shall Apple's total liability to
   You for all damages (other than as may be required by applicable law) under
   this License exceed the amount of fifty dollars ($50.00).

   (( 10. ))??
   Trademarks. This License does not grant any rights to use the trademarks or
   trade names "Apple", "Apple Computer", "Mac OS X", "Mac OS X Server",
   "QuickTime", "QuickTime Streaming Server" or any other trademarks or trade
   names belonging to Apple (collectively "Apple Marks") or to any trademark or
   trade name belonging to any Contributor. No Apple Marks may be used to
   endorse or promote products derived from the Original Code other than as
   permitted by and in strict compliance at all times with Apple's third party
   trademark usage guidelines which are posted at
   http:/www.apple.com/legal/guidelinesfor3rdparties.html.

   (( 11. ))??
   Ownership. Subject to the licenses granted under this License, each
   Contributor retains all rights, title and interest in and to any
   Modifications made by such Contributor. Apple retains all rights, title and
   interest in and to the Original Code and any Modifications made by or on
   behalf of Apple ("Apple Modifications"), and such Apple Modifications will
   not be automatically subject to this License. Apple may, at its sole
   discretion, choose to license such Apple Modifications under this License, or
   on different terms from those contained in this License or may choose not to
   license them at all.

   (( 12. ))??
   Termination.

      (( 12.1 ))??
      Termination. This License and the rights granted hereunder will
      terminate:

         (( (a) ))??
         automatically without notice from Apple if You fail to comply with any
         term(s) of this License and fail to cure such breach within 30 days of
         becoming aware of such breach;

         (( (b) ))??
         immediately in the event of the circumstances described in Section
         13.5(b); or

         (( c ))??
         automatically without notice from Apple if You, at any time during the
         term of this License, commence an action for patent infringement
         against Apple.

      (( 12.2 ))??
      Effect of Termination. Upon termination, You agree to immediately stop any
      further use, reproduction, modification, sublicensing and distribution of
      the Covered Code and to destroy all copies of the Covered Code that are in
      your possession or control. All sublicenses to the Covered Code which have
      been properly granted prior to termination shall survive any termination
      of this License. Provisions which, by their nature, should remain in
      effect beyond the termination of this License shall survive, including but
      not limited to Sections 3, 5, 8, 9, 10, 11, 12.2 and 13. No party will be
      liable to any other for compensation, indemnity or damages of any sort
      solely as a result of terminating this License in accordance with its
      terms, and termination of this License will be without prejudice to any
      other right or remedy of any party.

   (( 13. ))??
   Miscellaneous.

      (( 13.1 ))??
      Government End Users. The Covered Code is a "commercial item" as defined
      in FAR 2.101. Government software and technical data rights in the Covered
      Code include only those rights customarily provided to the public as
      defined in this License. This customary commercial license in technical
      data and software is provided in accordance with FAR 12.211 (Technical
      Data) and 12.212 (Computer Software) and, for Department of Defense
      purchases, DFAR 252.227-7015 (Technical Data -- Commercial Items) and
      227.7202-3 (Rights in Commercial Computer Software or Computer Software
      Documentation). Accordingly, all U.S. Government End Users acquire Covered
      Code with only those rights set forth herein.

      (( 13.2 ))??
      Relationship of Parties. This License will not be construed as creating an
      agency, partnership, joint venture or any other form of legal association
      between or amongYou, Apple or any Contributor, and You will not represent
      to the contrary, whether expressly, by implication, appearance or
      otherwise.

      (( 13.3 ))??
      Independent Development. Nothing in this License will impair Apple's right
      to acquire, license, develop, have others develop for it, market and/or
      distribute technology or products that perform the same or similar
      functions as, or otherwise compete with, Modifications, Larger Works,
      technology or products that You may develop, produce, market or
      distribute.

      (( 13.4 ))??
      Waiver; Construction. Failure by Apple or any Contributor to enforce any
      provision of this License will not be deemed a waiver of future
      enforcement of that or any other provision. Any law or regulation which
      provides that the language of a contract shall be construed against the
      drafter will not apply to this License.

      (( 13.5 ))??
      Severability. (a) If for any reason a court of competent jurisdiction
      finds any provision of this License, or portion thereof, to be
      unenforceable, that provision of the License will be enforced to the
      maximum extent permissible so as to effect the economic benefits and
      intent of the parties, and the remainder of this License will continue in
      full force and effect. (b) Notwithstanding the foregoing, if applicable
      law prohibits or restricts You from fully and/or specifically complying
      with Sections 2 and/or 3 or prevents the enforceability of either of those
      Sections, this License will immediately terminate and You must immediately
      discontinue any use of the Covered Code and destroy all copies of it that
      are in your possession or control.

      (( 13.6 ))??
      Dispute Resolution. Any litigation or other dispute resolution between You
      and Apple relating to this License shall take place in the Northern
      District of California, and You and Apple hereby consent to the personal
      jurisdiction of, and venue in, the state and federal courts within that
      District with respect to this License. The application of the United
      Nations Convention on Contracts for the International Sale of Goods is
      expressly excluded.

      (( 13.7 ))??
      Entire Agreement; Governing Law. This License constitutes the entire
      agreement between the parties with respect to the subject matter hereof.
      This License shall be governed by the laws of the United States and the
      State of California, except that body of California law concerning
      conflicts of law.

      Where You are located in the province of Quebec, Canada, the following
      clause applies: The parties hereby confirm that they have requested that
      this License and all related documents be drafted in English. Les parties
      ont exigé que le présent contrat et tous les documents connexes soient
      rédigés en anglais.
      (( EXHIBIT A.

"Portions Copyright (c) 1999-2001 Apple Computer, Inc. All Rights Reserved.

This file contains Original Code and/or Modifications of Original Code as
defined in and that are subject to the Apple Public Source License Version 1.2
(the 'License'). You may not use this file except in compliance with the
License. Please obtain a copy of the License at http:/www.apple.com/publicsource
and read it before using this file.

The Original Code and all software distributed under the License are distributed
on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION,
ANY WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
ENJOYMENT OR NON-INFRINGEMENT. Please see the License for the specific language
governing rights and limitations under the License." ))??
`
const license_APSL_2_0_lre = `//**
Apple Public Source License 2.0
https://spdx.org/licenses/APSL-2.0.json
http://www.opensource.apple.com/license/apsl/
**//

(( APPLE PUBLIC SOURCE LICENSE

Version 2.0 - August 6, 2003 ))??

(( Please read this License carefully before downloading this software. By
downloading or using this software, you are agreeing to be bound by the terms of
this License. If you do not or cannot agree to the terms of this License, please
do not download or use the software. ))??

(( Apple Note: In January 2007, Apple changed its corporate name from "Apple
Computer, Inc." to "Apple Inc." This change has been reflected below and
copyright years updated, but no other changes have been made to the APSL 2.0. ))??

   (( 1. ))??
   General; Definitions. This License applies to any program or other work which
   Apple
   ((Computer))??
   Inc. ("Apple") makes publicly available and which contains a notice
   placed by Apple identifying such program or work as "Original Code" and
   stating that it is subject to the terms of this Apple Public Source License
   version 2.0 ("License"). As used in this License:

      (( 1.1 ))??
      "Applicable Patent Rights" mean: (a) in the case where Apple is the
      grantor of rights, (i) claims of patents that are now or hereafter
      acquired, owned by or assigned to Apple and (ii) that cover subject matter
      contained in the Original Code, but only to the extent necessary to use,
      reproduce and/or distribute the Original Code without infringement; and
      (b) in the case where You are the grantor of rights, (i) claims of patents
      that are now or hereafter acquired, owned by or assigned to You and (ii)
      that cover subject matter in Your Modifications, taken alone or in
      combination with Original Code.

      (( 1.2 ))??
      "Contributor" means any person or entity that creates or contributes to
      the creation of Modifications.

      (( 1.3 ))??
      "Covered Code" means the Original Code, Modifications, the combination of
      Original Code and any Modifications, and/or any respective portions
      thereof.

      (( 1.4 ))??
      "Externally Deploy" means: (a) to sublicense, distribute or otherwise make
      Covered Code available, directly or indirectly, to anyone other than You;
      and/or (b) to use Covered Code, alone or as part of a Larger Work, in any
      way to provide a service, including but not limited to delivery of
      content, through electronic communication with a client other than You.

      (( 1.5 ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.6 ))??
      "Modifications" mean any addition to, deletion from, and/or change to, the
      substance and/or structure of the Original Code, any previous
      Modifications, the combination of Original Code and any previous
      Modifications, and/or any respective portions thereof. When code is
      released as a series of files, a Modification is: (a) any addition to or
      deletion from the contents of a file containing Covered Code; and/or (b)
      any new file or other representation of computer program statements that
      contains any part of Covered Code.

      (( 1.7 ))??
      "Original Code" means (a) the Source Code of a program or other work as
      originally made available by Apple under this License, including the
      Source Code of any updates or upgrades to such programs or works made
      available by Apple under this License, and that has been expressly
      identified by Apple as such in the header file(s) of such work; and (b)
      the object code compiled from such Source Code and originally made
      available by Apple under this License

      (( 1.8 ))??
      "Source Code" means the human readable form of a program or other work
      that is suitable for making modifications to it, including all modules it
      contains, plus any associated interface definition files, scripts used to
      control compilation and installation of an executable (object code).

      (( 1.9 ))??
      "You" or "Your" means an individual or a legal entity exercising rights
      under this License. For legal entities, "You" or "Your" includes any
      entity which controls, is controlled by, or is under common control with,
      You, where "control" means (a) the power, direct or indirect, to cause the
      direction or management of such entity, whether by contract or otherwise,
      or (b) ownership of fifty percent (50%) or more of the outstanding shares
      or beneficial ownership of such entity.

   (( 2. ))??
   Permitted Uses; Conditions & Restrictions. Subject to the terms and
   conditions of this License, Apple hereby grants You, effective on the date
   You accept this License and download the Original Code, a world-wide,
   royalty-free, non-exclusive license, to the extent of Apple's Applicable
   Patent Rights and copyrights covering the Original Code, to do the
   following:

      (( 2.1 ))??
      Unmodified Code. You may use, reproduce, display, perform, internally
      distribute within Your organization, and Externally Deploy verbatim,
      unmodified copies of the Original Code, for commercial or non-commercial
      purposes, provided that in each instance:

         (( (a) ))??
         You must retain and reproduce in all copies of Original Code the
         copyright and other proprietary notices and disclaimers of Apple as
         they appear in the Original Code, and keep intact all notices in the
         Original Code that refer to this License; and

         (( (b) ))??
         You must include a copy of this License with every copy of Source Code
         of Covered Code and documentation You distribute or Externally Deploy,
         and You may not offer or impose any terms on such Source Code that
         alter or restrict this License or the recipients' rights hereunder,
         except as permitted under Section 6.

      (( 2.2 ))??
      Modified Code. You may modify Covered Code and use, reproduce, display,
      perform, internally distribute within Your organization, and Externally
      Deploy Your Modifications and Covered Code, for commercial or
      non-commercial purposes, provided that in each instance You also meet all
      of these conditions:

         (( (a) ))??
         You must satisfy all the conditions of Section 2.1 with respect to the
         Source Code of the Covered Code;

         (( (b) ))??
         You must duplicate, to the extent it does not already exist, the notice
         in Exhibit A in each file of the Source Code of all Your Modifications,
         and cause the modified files to carry prominent notices stating that
         You changed the files and the date of any change; and

         (( c ))??
         If You Externally Deploy Your Modifications, You must make Source Code
         of all Your Externally Deployed Modifications either available to those
         to whom You have Externally Deployed Your Modifications, or publicly
         available. Source Code of Your Externally Deployed Modifications must
         be released under the terms set forth in this License, including the
         license grants set forth in Section 3 below, for as long as you
         Externally Deploy the Covered Code or twelve (12) months from the date
         of initial External Deployment, whichever is longer. You should
         preferably distribute the Source Code of Your Externally Deployed
         Modifications electronically (e.g. download from a web site).

      (( 2.3 ))??
      Distribution of Executable Versions. In addition, if You Externally Deploy
      Covered Code (Original Code and/or Modifications) in object code,
      executable form only, You must include a prominent notice, in the code
      itself as well as in related documentation, stating that Source Code of
      the Covered Code is available under the terms of this License with
      information on how and where to obtain such Source Code.

      (( 2.4 ))??
      Third Party Rights. You expressly acknowledge and agree that although
      Apple and each Contributor grants the licenses to their respective
      portions of the Covered Code set forth herein, no assurances are provided
      by Apple or any Contributor that the Covered Code does not infringe the
      patent or other intellectual property rights of any other entity. Apple
      and each Contributor disclaim any liability to You for claims brought by
      any other entity based on infringement of intellectual property rights or
      otherwise. As a condition to exercising the rights and licenses granted
      hereunder, You hereby assume sole responsibility to secure any other
      intellectual property rights needed, if any. For example, if a third party
      patent license is required to allow You to distribute the Covered Code, it
      is Your responsibility to acquire that license before distributing the
      Covered Code.

   (( 3. ))??
   Your Grants. In consideration of, and as a condition to, the licenses granted
   to You under this License, You hereby grant to any person or entity receiving
   or distributing Covered Code under this License a non-exclusive,
   royalty-free, perpetual, irrevocable license, under Your Applicable Patent
   Rights and other intellectual property rights (other than patent) owned or
   controlled by You, to use, reproduce, display, perform, modify, sublicense,
   distribute and Externally Deploy Your Modifications of the same scope and
   extent as Apple's licenses under Sections 2.1 and 2.2 above.

   (( 4. ))??
   Larger Works. You may create a Larger Work by combining Covered Code with
   other code not governed by the terms of this License and distribute the
   Larger Work as a single product. In each such instance, You must make sure
   the requirements of this License are fulfilled for the Covered Code or any
   portion thereof.

   (( 5. ))??
   Limitations on Patent License. Except as expressly stated in Section 2, no
   other patent rights, express or implied, are granted by Apple herein.
   Modifications and/or Larger Works may require additional patent licenses from
   Apple which Apple may grant in its sole discretion.

   (( 6. ))??
   Additional Terms. You may choose to offer, and to charge a fee for, warranty,
   support, indemnity or liability obligations and/or other rights consistent
   with the scope of the license granted herein ("Additional Terms") to one or
   more recipients of Covered Code. However, You may do so only on Your own
   behalf and as Your sole responsibility, and not on behalf of Apple or any
   Contributor. You must obtain the recipient's agreement that any such
   Additional Terms are offered by You alone, and You hereby agree to indemnify,
   defend and hold Apple and every Contributor harmless for any liability
   incurred by or claims asserted against Apple or such Contributor by reason of
   any such Additional Terms.

   (( 7. ))??
   Versions of the License. Apple may publish revised and/or new versions of
   this License from time to time. Each version will be given a distinguishing
   version number. Once Original Code has been published under a particular
   version of this License, You may continue to use it under the terms of that
   version. You may also choose to use such Original Code under the terms of any
   subsequent version of this License published by Apple. No one other than
   Apple has the right to modify the terms applicable to Covered Code created
   under this License.

   (( 8. ))??
   NO WARRANTY OR SUPPORT. The Covered Code may contain in whole or in part
   pre-release, untested, or not fully tested works. The Covered Code may
   contain errors that could cause failures or loss of data, and may be
   incomplete or contain inaccuracies. You expressly acknowledge and agree that
   use of the Covered Code, or any portion thereof, is at Your sole and entire
   risk. THE COVERED CODE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, UPGRADES OR
   SUPPORT OF ANY KIND AND APPLE AND APPLE'S LICENSOR(S) (COLLECTIVELY REFERRED
   TO AS "APPLE" FOR THE PURPOSES OF SECTIONS 8 AND 9) AND ALL CONTRIBUTORS
   EXPRESSLY DISCLAIM ALL WARRANTIES AND/OR CONDITIONS, EXPRESS OR IMPLIED,
   INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES AND/OR CONDITIONS OF
   MERCHANTABILITY, OF SATISFACTORY QUALITY, OF FITNESS FOR A PARTICULAR
   PURPOSE, OF ACCURACY, OF QUIET ENJOYMENT, AND NONINFRINGEMENT OF THIRD PARTY
   RIGHTS. APPLE AND EACH CONTRIBUTOR DOES NOT WARRANT AGAINST INTERFERENCE WITH
   YOUR ENJOYMENT OF THE COVERED CODE, THAT THE FUNCTIONS CONTAINED IN THE
   COVERED CODE WILL MEET YOUR REQUIREMENTS, THAT THE OPERATION OF THE COVERED
   CODE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN THE COVERED CODE
   WILL BE CORRECTED. NO ORAL OR WRITTEN INFORMATION OR ADVICE GIVEN BY APPLE,
   AN APPLE AUTHORIZED REPRESENTATIVE OR ANY CONTRIBUTOR SHALL CREATE A
   WARRANTY. You acknowledge that the Covered Code is not intended for use in
   the operation of nuclear facilities, aircraft navigation, communication
   systems, or air traffic control machines in which case the failure of the
   Covered Code could lead to death, personal injury, or severe physical or
   environmental damage.

   (( 9. ))??
   LIMITATION OF LIABILITY. TO THE EXTENT NOT PROHIBITED BY LAW, IN NO EVENT
   SHALL APPLE OR ANY CONTRIBUTOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
   INDIRECT OR CONSEQUENTIAL DAMAGES ARISING OUT OF OR RELATING TO THIS LICENSE
   OR YOUR USE OR INABILITY TO USE THE COVERED CODE, OR ANY PORTION THEREOF,
   WHETHER UNDER A THEORY OF CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE),
   PRODUCTS LIABILITY OR OTHERWISE, EVEN IF APPLE OR SUCH CONTRIBUTOR HAS BEEN
   ADVISED OF THE POSSIBILITY OF SUCH DAMAGES AND NOTWITHSTANDING THE FAILURE OF
   ESSENTIAL PURPOSE OF ANY REMEDY. SOME JURISDICTIONS DO NOT ALLOW THE
   LIMITATION OF LIABILITY OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS
   LIMITATION MAY NOT APPLY TO YOU. In no event shall Apple's total liability to
   You for all damages (other than as may be required by applicable law) under
   this License exceed the amount of fifty dollars ($50.00).

   (( 10. ))??
   Trademarks. This License does not grant any rights to use the trademarks or
   trade names "Apple",
   ((Apple Computer))??
   "Mac", "Mac OS", "QuickTime", "QuickTime Streaming
   Server" or any other trademarks, service marks, logos or trade names
   belonging to Apple (collectively "Apple Marks") or to any trademark, service
   mark, logo or trade name belonging to any Contributor. You agree not to use
   any Apple Marks in or as part of the name of products derived from the
   Original Code or to endorse or promote products derived from the Original
   Code other than as expressly permitted by and in strict compliance at all
   times with Apple's third party trademark usage guidelines which are posted at
   http:/www.apple.com/legal/guidelinesfor3rdparties.html.

   (( 11. ))??
   Ownership. Subject to the licenses granted under this License, each
   Contributor retains all rights, title and interest in and to any
   Modifications made by such Contributor. Apple retains all rights, title and
   interest in and to the Original Code and any Modifications made by or on
   behalf of Apple ("Apple Modifications"), and such Apple Modifications will
   not be automatically subject to this License. Apple may, at its sole
   discretion, choose to license such Apple Modifications under this License, or
   on different terms from those contained in this License or may choose not to
   license them at all.

   (( 12. ))??
   Termination.

      (( 12.1 ))??
      Termination. This License and the rights granted hereunder will
      terminate:

         (( (a) ))??
         automatically without notice from Apple if You fail to comply with any
         term(s) of this License and fail to cure such breach within 30 days of
         becoming aware of such breach;

         (( (b) ))??
         immediately in the event of the circumstances described in Section
         13.5(b); or

         (( c ))??
         automatically without notice from Apple if You, at any time during the
         term of this License, commence an action for patent infringement
         against Apple; provided that Apple did not first commence an action for
         patent infringement against You in that instance.

      (( 12.2 ))??
      Effect of Termination. Upon termination, You agree to immediately stop any
      further use, reproduction, modification, sublicensing and distribution of
      the Covered Code. All sublicenses to the Covered Code which have been
      properly granted prior to termination shall survive any termination of
      this License. Provisions which, by their nature, should remain in effect
      beyond the termination of this License shall survive, including but not
      limited to Sections 3, 5, 8, 9, 10, 11, 12.2 and 13. No party will be
      liable to any other for compensation, indemnity or damages of any sort
      solely as a result of terminating this License in accordance with its
      terms, and termination of this License will be without prejudice to any
      other right or remedy of any party.

   (( 13. ))??
   Miscellaneous.

      (( 13.1 ))??
      Government End Users. The Covered Code is a "commercial item" as defined
      in FAR 2.101. Government software and technical data rights in the Covered
      Code include only those rights customarily provided to the public as
      defined in this License. This customary commercial license in technical
      data and software is provided in accordance with FAR 12.211 (Technical
      Data) and 12.212 (Computer Software) and, for Department of Defense
      purchases, DFAR 252.227-7015 (Technical Data -- Commercial Items) and
      227.7202-3 (Rights in Commercial Computer Software or Computer Software
      Documentation). Accordingly, all U.S. Government End Users acquire Covered
      Code with only those rights set forth herein.

      (( 13.2 ))??
      Relationship of Parties. This License will not be construed as creating an
      agency, partnership, joint venture or any other form of legal association
      between or among You, Apple or any Contributor, and You will not represent
      to the contrary, whether expressly, by implication, appearance or
      otherwise.

      (( 13.3 ))??
      Independent Development. Nothing in this License will impair Apple's right
      to acquire, license, develop, have others develop for it, market and/or
      distribute technology or products that perform the same or similar
      functions as, or otherwise compete with, Modifications, Larger Works,
      technology or products that You may develop, produce, market or
      distribute.

      (( 13.4 ))??
      Waiver; Construction. Failure by Apple or any Contributor to enforce any
      provision of this License will not be deemed a waiver of future
      enforcement of that or any other provision. Any law or regulation which
      provides that the language of a contract shall be construed against the
      drafter will not apply to this License.

      (( 13.5 ))??
      Severability. (a) If for any reason a court of competent jurisdiction
      finds any provision of this License, or portion thereof, to be
      unenforceable, that provision of the License will be enforced to the
      maximum extent permissible so as to effect the economic benefits and
      intent of the parties, and the remainder of this License will continue in
      full force and effect. (b) Notwithstanding the foregoing, if applicable
      law prohibits or restricts You from fully and/or specifically complying
      with Sections 2 and/or 3 or prevents the enforceability of either of those
      Sections, this License will immediately terminate and You must immediately
      discontinue any use of the Covered Code and destroy all copies of it that
      are in your possession or control.

      (( 13.6 ))??
      Dispute Resolution. Any litigation or other dispute resolution between You
      and Apple relating to this License shall take place in the Northern
      District of California, and You and Apple hereby consent to the personal
      jurisdiction of, and venue in, the state and federal courts within that
      District with respect to this License. The application of the United
      Nations Convention on Contracts for the International Sale of Goods is
      expressly excluded.

      (( 13.7 ))??
      Entire Agreement; Governing Law. This License constitutes the entire
      agreement between the parties with respect to the subject matter hereof.
      This License shall be governed by the laws of the United States and the
      State of California, except that body of California law concerning
      conflicts of law.

      Where You are located in the province of Quebec, Canada, the following
      clause applies: The parties hereby confirm that they have requested that
      this License and all related documents be drafted in English. Les parties
      ont exigé que le présent contrat et tous les documents connexes soient
      rédigés en anglais.
      (( EXHIBIT A.

"Portions Copyright (c) 1999-2007 Apple
((Computer))??
Inc. All Rights Reserved.

This file contains Original Code and/or Modifications of Original Code as
defined in and that are subject to the Apple Public Source License Version 2.0
(the 'License'). You may not use this file except in compliance with the
License. Please obtain a copy of the License at
http:/www.opensource.apple.com/apsl/ and read it before using this file.

The Original Code and all software distributed under the License are distributed
on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION,
ANY WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
ENJOYMENT OR NON-INFRINGEMENT. Please see the License for the specific language
governing rights and limitations under the License." ))??
`
const license_Abstyles_lre = `//**
Abstyles License
https://spdx.org/licenses/Abstyles.json
https://fedoraproject.org/wiki/Licensing/Abstyles
**//

(( This is APREAMBL.TEX, version 1.10e, written by Hans-Hermann Bode

(HHBODE@DOSUNI1.BITNET), for the BibTeX ` + "`" + `adaptable' family, version 1.10.

See the file APREAMBL.DOC for a detailed documentation. ))??

This program is distributed WITHOUT ANY WARRANTY, express or implied.

Copyright (C) 1991, 1992 Hans-Hermann Bode

Permission is granted to make and distribute verbatim copies of this document
provided that the copyright notice and this permission notice are preserved on
all copies.

Permission is granted to copy and distribute modified versions of this document
under the conditions for verbatim copying, provided that the entire resulting
derived work is distributed under the terms of a permission notice identical to
this one.
`
const license_Adobe_2006_lre = `//**
Adobe Systems Incorporated Source Code License Agreement
https://spdx.org/licenses/Adobe-2006.json
https://fedoraproject.org/wiki/Licensing/AdobeLicense
**//

(( Adobe Systems Incorporated(r) Source Code License Agreement
((Copyright __20__))??
))??

Please read this Source Code License Agreement carefully before using the source
code.

Adobe Systems Incorporated grants to you a perpetual, worldwide, non-exclusive,
no-charge, royalty-free, irrevocable copyright license, to reproduce, prepare
derivative works of, publicly display, publicly perform, and distribute this
source code and such derivative works in source or object code form without any
attribution requirements.

The name "Adobe Systems Incorporated" must not be used to endorse or promote
products derived from the source code without prior written permission.

You agree to indemnify, hold harmless and defend Adobe Systems Incorporated from
and against any loss, damage, claims or lawsuits, including attorney's fees that
arise or result from your use or distribution of the source code.

THIS SOURCE CODE IS PROVIDED "AS IS" AND "WITH ALL FAULTS", WITHOUT ANY
TECHNICAL SUPPORT OR ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. ALSO, THERE IS NO WARRANTY OF
NON-INFRINGEMENT, TITLE OR QUIET ENJOYMENT. IN NO EVENT SHALL MACROMEDIA OR ITS
SUPPLIERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOURCE CODE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
`
const license_Adobe_Glyph_lre = `//**
Adobe Glyph List License
https://spdx.org/licenses/Adobe-Glyph.json
https://fedoraproject.org/wiki/Licensing/MIT#AdobeGlyph
**//

//** Copyright **//

Permission is hereby granted, free of charge, to any person obtaining a copy of
this documentation file to use, copy, publish, distribute, sublicense, and/or
sell copies of the documentation, and to permit others to do the same, provided
that:

   -
   No modification, editing or other alteration of this document is allowed;
   and

   -
   The above copyright notice and this permission notice shall be included in
   all copies of the documentation.

Permission is hereby granted, free of charge, to any person obtaining a copy of
this documentation file, to create their own derivative works from the content
of this document to use, copy, publish, distribute, sublicense, and/or sell the
derivative works, and to permit others to do the same, provided that the derived
work is not represented as being a copy or version of this document.

Adobe shall not be liable to any party for any loss of revenue or profit or for
indirect, incidental, special, consequential, or other similar damages, whether
based on tort (including without limitation negligence or strict liability),
contract or other legal or equitable grounds even if Adobe has been advised or
had reason to know of the possibility of such damages.
__5__
The Adobe materials are provided on an "AS IS" basis.
__5__
Adobe specifically disclaims all express, statutory, or implied warranties
relating to the Adobe materials, including but not limited to those concerning
merchantability or fitness for a particular purpose or non-infringement of any
third party rights regarding the Adobe materials.
`
const license_Afmparse_lre = `//**
Afmparse License
https://spdx.org/licenses/Afmparse.json
https://fedoraproject.org/wiki/Licensing/Afmparse
**//

//** Copyright **//

This file may be freely copied and redistributed as long as:

   (( 1) ))??
   This entire notice continues to be included in the file,

   (( 2) ))??
   If the file has been modified in any way, a notice of such modification is
   conspicuously indicated.

PostScript, Display PostScript, and Adobe are registered trademarks of Adobe
Systems Incorporated.

THE INFORMATION BELOW IS FURNISHED AS IS, IS SUBJECT TO CHANGE WITHOUT NOTICE,
AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ADOBE SYSTEMS INCORPORATED. ADOBE
SYSTEMS INCORPORATED ASSUMES NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
INACCURACIES, MAKES NO WARRANTY OF ANY KIND (EXPRESS, IMPLIED OR STATUTORY) WITH
RESPECT TO THIS INFORMATION, AND EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES OF
MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSES AND NONINFRINGEMENT OF THIRD
PARTY RIGHTS.
`
const license_Aladdin_lre = `//**
Aladdin Free Public License
https://spdx.org/licenses/Aladdin.json
http://pages.cs.wisc.edu/~ghost/doc/AFPL/6.01/Public.htm
**//

(( Aladdin Free Public License (Version 8, November 18, 1999)
((Copyright __20__))??
))??

(( NOTE: This License is not the same as any of the GNU Licenses published by
the Free Software Foundation. Its terms are substantially different from those
of the GNU Licenses. If you are familiar with the GNU Licenses, please read this
license with extra care.

Aladdin Enterprises hereby grants to anyone the permission to apply this License
to their own work, as long as the entire License (including the above notices
and this paragraph) is copied with no changes, additions, or deletions except
for changing the first paragraph of Section 0 to include a suitable description
of the work to which the license is being applied and of the person or entity
that holds the copyright in the work, and, if the License is being applied to a
work created in a country other than the United States, replacing the first
paragraph of Section 6 with an appropriate reference to the laws of the
appropriate country. ))??

   (( 0. ))??
   Subject Matter

   This License applies to the computer program known as "Aladdin Ghostscript."
   The "Program", below, refers to such program. The Program is a copyrighted
   work whose copyright is held by Aladdin Enterprises (the "Licensor"). Please
   note that Aladdin Ghostscript is neither the program known as "GNU
   Ghostscript" nor the version of Ghostscript available for commercial
   licensing from Artifex Software Inc.

   A "work based on the Program" means either the Program or any derivative work
   of the Program, as defined in the United States Copyright Act of 1976, such
   as a translation or a modification.

   BY MODIFYING OR DISTRIBUTING THE PROGRAM (OR ANY WORK BASED ON THE PROGRAM),
   YOU INDICATE YOUR ACCEPTANCE OF THIS LICENSE TO DO SO, AND ALL ITS TERMS AND
   CONDITIONS FOR COPYING, DISTRIBUTING OR MODIFYING THE PROGRAM OR WORKS BASED
   ON IT. NOTHING OTHER THAN THIS LICENSE GRANTS YOU PERMISSION TO MODIFY OR
   DISTRIBUTE THE PROGRAM OR ITS DERIVATIVE WORKS. THESE ACTIONS ARE PROHIBITED
   BY LAW. IF YOU DO NOT ACCEPT THESE TERMS AND CONDITIONS, DO NOT MODIFY OR
   DISTRIBUTE THE PROGRAM.

   (( 1. ))??
   Licenses.

   Licensor hereby grants you the following rights, provided that you comply
   with all of the restrictions set forth in this License and provided, further,
   that you distribute an unmodified copy of this License with the Program:

      (( (a) ))??
      You may copy and distribute literal (i.e., verbatim) copies of the
      Program's source code as you receive it throughout the world, in any
      medium.

      (( (b) ))??
      You may modify the Program, create works based on the Program and
      distribute copies of such throughout the world, in any medium.

   (( 2. ))??
   Restrictions.

   This license is subject to the following restrictions:

      (( (a) ))??
      Distribution of the Program or any work based on the Program by a
      commercial organization to any third party is prohibited if any payment is
      made in connection with such distribution, whether directly (as in payment
      for a copy of the Program) or indirectly (as in payment for some service
      related to the Program, or payment for some product or service that
      includes a copy of the Program "without charge"; these are only examples,
      and not an exhaustive enumeration of prohibited activities). The following
      methods of distribution involving payment shall not in and of themselves
      be a violation of this restriction:

         (( (i) ))??
         Posting the Program on a public access information storage and
         retrieval service for which a fee is received for retrieving
         information (such as an on-line service), provided that the fee is not
         content-dependent (i.e., the fee would be the same for retrieving the
         same volume of information consisting of random data) and that access
         to the service and to the Program is available independent of any other
         product or service. An example of a service that does not fall under
         this section is an on-line service that is operated by a company and
         that is only available to customers of that company. (This is not an
         exhaustive enumeration.)

         (( (ii) ))??
         Distributing the Program on removable computer-readable media, provided
         that the files containing the Program are reproduced entirely and
         verbatim on such media, that all information on such media be
         redistributable for non-commercial purposes without charge, and that
         such media are distributed by themselves (except for accompanying
         documentation) independent of any other product or service. Examples of
         such media include CD-ROM, magnetic tape, and optical storage media.
         (This is not intended to be an exhaustive list.) An example of a
         distribution that does not fall under this section is a CD-ROM included
         in a book or magazine. (This is not an exhaustive enumeration.)

      (( (b) ))??
      Activities other than copying, distribution and modification of the
      Program are not subject to this License and they are outside its scope.
      Functional use (running) of the Program is not restricted, and any output
      produced through the use of the Program is subject to this license only if
      its contents constitute a work based on the Program (independent of having
      been made by running the Program).

      (( c ))??
      You must meet all of the following conditions with respect to any work
      that you distribute or publish that in whole or in part contains or is
      derived from the Program or any part thereof ("the Work"):

         (( (i) ))??
         If you have modified the Program, you must cause the Work to carry
         prominent notices stating that you have modified the Program's files
         and the date of any change. In each source file that you have modified,
         you must include a prominent notice that you have modified the file,
         including your name, your e-mail address (if any), and the date and
         purpose of the change;

         (( (ii) ))??
         You must cause the Work to be licensed as a whole and at no charge to
         all third parties under the terms of this License;

         (( (iii) ))??
         If the Work normally reads commands interactively when run, you must
         cause it, at each time the Work commences operation, to print or
         display an announcement including an appropriate copyright notice and a
         notice that there is no warranty (or else, saying that you provide a
         warranty). Such notice must also state that users may redistribute the
         Work only under the conditions of this License and tell the user how to
         view the copy of this License included with the Work. (Exceptions: if
         the Program is interactive but normally prints or displays such an
         announcement only at the request of a user, such as in an "About box",
         the Work is required to print or display the notice only under the same
         circumstances; if the Program itself is interactive but does not
         normally print such an announcement, the Work is not required to print
         an announcement.);

         (( (iv) ))??
         You must accompany the Work with the complete corresponding
         machine-readable source code, delivered on a medium customarily used
         for software interchange. The source code for a work means the
         preferred form of the work for making modifications to it. For an
         executable work, complete source code means all the source code for all
         modules it contains, plus any associated interface definition files,
         plus the scripts used to control compilation and installation of the
         executable code. If you distribute with the Work any component that is
         normally distributed (in either source or binary form) with the major
         components (compiler, kernel, and so on) of the operating system on
         which the executable runs, you must also distribute the source code of
         that component if you have it and are allowed to do so;

         (( (v) ))??
         If you distribute any written or printed material at all with the Work,
         such material must include either a written copy of this License, or a
         prominent written indication that the Work is covered by this License
         and written instructions for printing and/or displaying the copy of the
         License on the distribution medium;

         (( (vi) ))??
         You may not impose any further restrictions on the recipient's exercise
         of the rights granted herein.

         If distribution of executable or object code is made by offering the
         equivalent ability to copy from a designated place, then offering
         equivalent ability to copy the source code from the same place counts
         as distribution of the source code, even though third parties are not
         compelled to copy the source code along with the object code.

   (( 3. ))??
   Reservation of Rights.

   No rights are granted to the Program except as expressly set forth herein.
   You may not copy, modify, sublicense, or distribute the Program except as
   expressly provided under this License. Any attempt otherwise to copy, modify,
   sublicense or distribute the Program is void, and will automatically
   terminate your rights under this License. However, parties who have received
   copies, or rights, from you under this License will not have their licenses
   terminated so long as such parties remain in full compliance.

   (( 4. ))??
   Other Restrictions.

   If the distribution and/or use of the Program is restricted in certain
   countries for any reason, Licensor may add an explicit geographical
   distribution limitation excluding those countries, so that distribution is
   permitted only in or among countries not thus excluded. In such case, this
   License incorporates the limitation as if written in the body of this
   License.

   (( 5. ))??
   Limitations.

   THE PROGRAM IS PROVIDED TO YOU "AS IS," WITHOUT WARRANTY. THERE IS NO
   WARRANTY FOR THE PROGRAM, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
   PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE
   PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
   REPAIR OR CORRECTION.

   IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
   LICENSOR, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE THE PROGRAM
   AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL,
   SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR
   INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR
   DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR
   A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH
   HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

   (( 6. ))??
   General.

   This License is governed by the laws of the State of California, U.S.A.,
   excluding choice of law rules.

   If any part of this License is found to be in conflict with the law, that
   part shall be interpreted in its broadest meaning consistent with the law,
   and no other parts of the License shall be affected.

   For United States Government users, the Program is provided with RESTRICTED
   RIGHTS. If you are a unit or agency of the United States Government or are
   acquiring the Program for any such unit or agency, the following apply:

   If the unit or agency is the Department of Defense ("DOD"), the Program and
   its documentation are classified as "commercial computer software" and
   "commercial computer software documentation" respectively and, pursuant to
   DFAR Section 227.7202, the Government is acquiring the Program and its
   documentation in accordance with the terms of this License. If the unit or
   agency is other than DOD, the Program and its documentation are classified as
   "commercial computer software" and "commercial computer software
   documentation" respectively and, pursuant to FAR Section 12.212, the
   Government is acquiring the Program and its documentation in accordance with
   the terms of this License.
`
const license_Aladdin_9_lre = `//**
Aladdin Free Public License version 9 (no SPDX identifier)
**//

((Aladdin Free Public License (Version 9, September 18, 2000)
((Copyright __10__ Aladdin Enterprises, Menlo Park, California, U.S.A. All Rights Reserved.))??
))??


((  NOTE: This License is
not the same as any of the GNU Licenses published by the Free Software
Foundation. Its terms are substantially different from those of the GNU
Licenses. If you are familiar with the GNU Licenses, please read this license
with extra care.))??

   Aladdin Enterprises hereby grants to anyone the permission to apply
   this License to their own work, as long as the entire License
   (including the above notices and this paragraph) is copied with no
   changes, additions, or deletions except for changing the first
   paragraph of Section 0 to include a suitable description of the work
   to which the license is being applied and of the person or entity that
   holds the copyright in the work, and, if the License is being applied
   to a work created in a country other than the United States, replacing
   the first paragraph of Section 6 with an appropriate reference to the
   laws of the appropriate country.

   This License is not an Open Source license: among other things, it
   places restrictions on distribution of the Program, specifically
   including sale of the Program. While Aladdin Enterprises respects and
   supports the philosophy of the Open Source Definition, and shares the
   desire of the GNU project to keep licensed software freely
   redistributable in both source and object form, we feel that Open
   Source licenses unfairly prevent developers of useful software from
   being compensated proportionately when others profit financially from
   their work. This License attempts to ensure that those who receive,
   redistribute, and contribute to the licensed Program according to the
   Open Source and Free Software philosophies have the right to do so,
   while retaining for the developer(s) of the Program the power to make
   those who use the Program to enhance the value of commercial products
   pay for the privilege of doing so.

0. Subject Matter

   This License applies to the computer program known as "AFPL
   Ghostscript." The "Program", below, refers to such program. The
   Program is a copyrighted work whose copyright is held by artofcode
   LLC, located in Benicia, California (the "Licensor"). Please note that
   AFPL Ghostscript is neither the program known as "GNU Ghostscript" nor
   the version of Ghostscript available for commercial licensing from
   Artifex Software Inc.

   A "work based on the Program" means either the Program or any
   derivative work of the Program, as defined in the United States
   Copyright Act of 1976, such as a translation or a modification.

   BY MODIFYING OR DISTRIBUTING THE PROGRAM (OR ANY WORK BASED ON THE
   PROGRAM), YOU INDICATE YOUR ACCEPTANCE OF THIS LICENSE TO DO SO, AND
   ALL ITS TERMS AND CONDITIONS FOR COPYING, DISTRIBUTING OR MODIFYING
   THE PROGRAM OR WORKS BASED ON IT. NOTHING OTHER THAN THIS LICENSE
   GRANTS YOU PERMISSION TO MODIFY OR DISTRIBUTE THE PROGRAM OR ITS
   DERIVATIVE WORKS. THESE ACTIONS ARE PROHIBITED BY LAW. IF YOU DO NOT
   ACCEPT THESE TERMS AND CONDITIONS, DO NOT MODIFY OR DISTRIBUTE THE
   PROGRAM.

1. Licenses.

   Licensor hereby grants you the following rights, provided that you
   comply with all of the restrictions set forth in this License and
   provided, further, that you distribute an unmodified copy of this
   License with the Program:
   (a)
          You may copy and distribute literal (i.e., verbatim) copies of
          the Program's source code as you receive it throughout the
          world, in any medium.
   (b)
          You may modify the Program, create works based on the Program
          and distribute copies of such throughout the world, in any
          medium.

2. Restrictions.

   This license is subject to the following restrictions:
   (a)
          Distribution of the Program or any work based on the Program by
          a commercial organization to any third party is prohibited if
          any payment is made in connection with such distribution,
          whether directly (as in payment for a copy of the Program) or
          indirectly (as in payment for some service related to the
          Program, or payment for some product or service that includes a
          copy of the Program "without charge"; these are only examples,
          and not an exhaustive enumeration of prohibited activities).
          The following methods of distribution involving payment shall
          not in and of themselves be a violation of this restriction:
        (i)
               Posting the Program on a public access information storage
               and retrieval service for which a fee is received for
               retrieving information (such as an on-line service),
               provided that the fee is not content-dependent (i.e., the
               fee would be the same for retrieving the same volume of
               information consisting of random data) and that access to
               the service and to the Program is available independent of
               any other product or service. An example of a service that
               does not fall under this section is an on-line service
               that is operated by a company and that is only available
               to customers of that company. (This is not an exhaustive
               enumeration.)
        (ii)
               Distributing the Program on removable computer-readable
               media, provided that the files containing the Program are
               reproduced entirely and verbatim on such media, that all
               information on such media be redistributable for
               non-commercial purposes without charge, and that such
               media are distributed by themselves (except for
               accompanying documentation) independent of any other
               product or service. Examples of such media include CD-ROM,
               magnetic tape, and optical storage media. (This is not
               intended to be an exhaustive list.) An example of a
               distribution that does not fall under this section is a
               CD-ROM included in a book or magazine. (This is not an
               exhaustive enumeration.)
   (b)
          Activities other than copying, distribution and modification of
          the Program are not subject to this License and they are
          outside its scope. Functional use (running) of the Program is
          not restricted, and any output produced through the use of the
          Program is subject to this license only if its contents
          constitute a work based on the Program (independent of having
          been made by running the Program).
   (c)
          You must meet all of the following conditions with respect to
          any work that you distribute or publish that in whole or in
          part contains or is derived from the Program or any part
          thereof ("the Work"):
        (i)
               If you have modified the Program, you must cause the Work
               to carry prominent notices stating that you have modified
               the Program's files and the date of any change. In each
               source file that you have modified, you must include a
               prominent notice that you have modified the file,
               including your name, your e-mail address (if any), and the
               date and purpose of the change;
        (ii)
               You must cause the Work to be licensed as a whole and at
               no charge to all third parties under the terms of this
               License;
        (iii)
               If the Work normally reads commands interactively when
               run, you must cause it, at each time the Work commences
               operation, to print or display an announcement including
               an appropriate copyright notice and a notice that there is
               no warranty (or else, saying that you provide a warranty).
               Such notice must also state that users may redistribute
               the Work only under the conditions of this License and
               tell the user how to view the copy of this License
               included with the Work. (Exceptions: if the Program is
               interactive but normally prints or displays such an
               announcement only at the request of a user, such as in an
               "About box", the Work is required to print or display the
               notice only under the same circumstances; if the Program
               itself is interactive but does not normally print such an
               announcement, the Work is not required to print an
               announcement.);
        (iv)
               You must accompany the Work with the complete
               corresponding machine-readable source code, delivered on a
               medium customarily used for software interchange. The
               source code for a work means the preferred form of the
               work for making modifications to it. For an executable
               work, complete source code means all the source code for
               all modules it contains, plus any associated interface
               definition files, plus the scripts used to control
               compilation and installation of the executable code. If
               you distribute with the Work any component that is
               normally distributed (in either source or binary form)
               with the major components (compiler, kernel, and so on) of
               the operating system on which the executable runs, you
               must also distribute the source code of that component if
               you have it and are allowed to do so;
        (v)
               If you distribute any written or printed material at all
               with the Work, such material must include either a written
               copy of this License, or a prominent written indication
               that the Work is covered by this License and written
               instructions for printing and/or displaying the copy of
               the License on the distribution medium;
        (vi)
               You may not impose any further restrictions on the
               recipient's exercise of the rights granted herein.

   If distribution of executable or object code is made by offering the
   equivalent ability to copy from a designated place, then offering
   equivalent ability to copy the source code from the same place counts
   as distribution of the source code, even though third parties are not
   compelled to copy the source code along with the object code.

  3. Reservation of Rights.

   No rights are granted to the Program except as expressly set forth
   herein. You may not copy, modify, sublicense, or distribute the
   Program except as expressly provided under this License. Any attempt
   otherwise to copy, modify, sublicense or distribute the Program is
   void, and will automatically terminate your rights under this License.
   However, parties who have received copies, or rights, from you under
   this License will not have their licenses terminated so long as such
   parties remain in full compliance.

  4. Other Restrictions.

   If the distribution and/or use of the Program is restricted in certain
   countries for any reason, Licensor may add an explicit geographical
   distribution limitation excluding those countries, so that
   distribution is permitted only in or among countries not thus
   excluded. In such case, this License incorporates the limitation as if
   written in the body of this License.

  5. Limitations.

   THE PROGRAM IS PROVIDED TO YOU "AS IS," WITHOUT WARRANTY. THERE IS NO
   WARRANTY FOR THE PROGRAM, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT
   NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
   THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS
   WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
   ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

   IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
   WILL LICENSOR, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
   THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
   INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
   ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT
   NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR
   LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM
   TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER
   PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

  6. General.

   This License is governed by the laws of the State of California,
   U.S.A., excluding choice of law rules.

   If any part of this License is found to be in conflict with the law,
   that part shall be interpreted in its broadest meaning consistent with
   the law, and no other parts of the License shall be affected.

   For United States Government users, the Program is provided with
   RESTRICTED RIGHTS. If you are a unit or agency of the United States
   Government or are acquiring the Program for any such unit or agency,
   the following apply:

     If the unit or agency is the Department of Defense ("DOD"), the
     Program and its documentation are classified as "commercial
     computer software" and "commercial computer software documentation"
     respectively and, pursuant to DFAR Section 227.7202, the Government
     is acquiring the Program and its documentation in accordance with
     the terms of this License. If the unit or agency is other than DOD,
     the Program and its documentation are classified as "commercial
     computer software" and "commercial computer software documentation"
     respectively and, pursuant to FAR Section 12.212, the Government is
     acquiring the Program and its documentation in accordance with the
     terms of this License.
`
const license_Anti996_lre = `//**
Anti-996 License.
https://github.com/996icu/996.ICU/blob/master/LICENSE
**//

//** Copyright **//

((Anti))??
996 License Version 1.0 (Draft)

Permission is hereby granted to any individual or legal entity
obtaining a copy of this licensed work (including the source code,
documentation and/or related items, hereinafter collectively referred
to as the "licensed work"), free of charge, to deal with the licensed
work for any purpose, including without limitation, the rights to use,
reproduce,
((modify))??
prepare derivative works of,
((publish))??
distribute,
((publish))??
and sublicense the licensed work, subject to the following conditions:

__1__ The individual or the legal entity must conspicuously display,
without modification, this License
((and the notice))??
on each redistributed
or derivative copy of the Licensed Work.

__1__ The individual or the legal entity must strictly comply with all
applicable laws, regulations, rules and standards of the jurisdiction
relating to labor and employment where the individual is physically
located or where the individual was born or naturalized; or where the
legal entity is registered or is operating (whichever is stricter). In
case that the jurisdiction has no such laws, regulations, rules and
standards or its laws, regulations, rules and standards are
unenforceable, the individual or the legal entity are required to
comply with Core International Labor Standards.

__1__ The individual or the legal entity shall not induce,
((suggest))??
((metaphor))??
or force
its employee(s), whether full-time or part-time, or its independent
contractor(s), in any methods, to agree in oral or written form, to
directly or indirectly restrict, weaken or relinquish his or her
rights or remedies under such laws, regulations, rules and standards
relating to labor and employment as mentioned above, no matter whether
such written or oral agreements are enforceable under the laws of the
said jurisdiction, nor shall such individual or the legal entity
limit, in any methods, the rights of its employee(s) or independent
contractor(s) from reporting or complaining to the copyright holder or
relevant authorities monitoring the compliance of the license about
its violation(s) of the said license.

THE LICENSED WORK IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN ANY WAY CONNECTION WITH THE
LICENSED WORK OR THE USE OR OTHER DEALINGS IN THE LICENSED WORK.`
const license_Apache_1_0_lre = `//**
Apache License 1.0
https://spdx.org/licenses/Apache-1.0.json
http://www.apache.org/licenses/LICENSE-1.0
**//

//** Copyright **//

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   All advertising materials mentioning features or use of this software must
   display the following acknowledgment: "This product includes software
   developed by the
   //** Apache Group for use in the Apache HTTP server project (http://www.apache.org/) **//
   __14__

   (( 4. ))??
   The name
   __5__
   must not be used to endorse or promote products derived from this software
   without prior written permission. For written permission, please contact
   //** apache@apache.org **//
   __5__

   (( 5. ))??
   Products derived from this software may not be called
   __5__
   nor may
   __5__
   appear in their name, without prior written permission of
   //** the Apache Group **//
   __5__

   (( 6. ))??
   Redistributions of any form whatsoever must retain the following
   acknowledgment:

   "This product includes software developed by
   //** the Apache Group for use in the Apache HTTP server project (http://www.apache.org/) **//
   __15__

THIS SOFTWARE IS PROVIDED BY
//** THE APACHE GROUP **//
__5__
` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
//** THE APACHE GROUP OR ITS CONTRIBUTORS **//
__6__
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

(( This software consists of voluntary contributions made by many individuals on
behalf of the Apache Group and was originally based on public domain software
written at the National Center for Supercomputing Applications, University of
Illinois, Urbana-Champaign. For more information on the Apache Group and the
Apache HTTP server project, please see <http:/www.apache.org/>. ))??
`
const license_Apache_1_1_lre = `//**
Apache License 1.1
https://spdx.org/licenses/Apache-1.1.json
http://apache.org/licenses/LICENSE-1.1
https://opensource.org/licenses/Apache-1.1
**//

(( Apache License 1.1
((Copyright __20__))??
))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The end-user documentation included with the redistribution, if any, must
   include the following acknowledgment:

   "This product includes software developed by
   //** the Apache Software Foundation (http://www.apache.org/) **//
   __10__

   Alternately, this acknowledgment may appear in the software itself, if and
   wherever such third-party acknowledgments normally appear.

   (( 4. ))??
   The name
   __10__
   must not be used to endorse or promote products derived from this software
   without prior written permission.
   (( For written permission, please contact __10__ ))??

   (( 5. ))??
   Products derived from this software may not be called
   __20__
   without prior written permission of __5__.

THIS SOFTWARE IS PROVIDED ''AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
//** THE APACHE SOFTWARE FOUNDATION OR ITS CONTRIBUTORS **//
__7__
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

(( This software consists of voluntary contributions made by many individuals on
behalf of the Apache Software Foundation. For more information on the Apache
Software Foundation, please see http:/www.apache.org/. Portions of this software
are based upon public domain software originally written at the National Center
for Supercomputing Applications, University of Illinois, Urbana-Champaign. ))??
`
const license_Apache_2_0_lre = `//**
Apache License 2.0
https://spdx.org/licenses/Apache-2.0.json
http://www.apache.org/licenses/LICENSE-2.0
https://opensource.org/licenses/Apache-2.0
**//

((
	((This program is))??
	((Licensed || licenses this __1__))
	((to you))??
	under the Apache License, Version 2.0
	(( (the "License") ))??
	((
		((and))??
		you may not use __5__
		except in compliance with the
		((License || Apache License Version 2.0))
	))??

	((
		((
			A copy of the
			((Apache-2.0))??
			License is located
		||
			You may obtain a copy of the
			((
				((Apache-2.0))??
				License
			||
				Apache License Version 2.0
			))
			((in the LICENSE file or))??
		))
		at
		((the following location))??
	))??

	((
		((http))??
		((www))??
		.apache.org/licenses/LICENSE-2.0
	||
		((http))??
		aws.amazon.com/apache2.0/
	))??

	((or in the license file accompanying this file.))??

	((
		((As well as the file __10__))??
		((
			((Unless required by applicable law or agreed to in writing,))??
			((
				this file
			||
				software distributed under the
				((License || Apache License Version 2.0))
			))
			is distributed on an "AS IS" BASIS,
			WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
			either express or implied.
		||
			
((DISCLAIMER))??
((2.))??

THE
((SOFTWARE || MATERIALS))
IS PROVIDED "AS IS",
WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO
THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE
AND NONINFRINGEMENT.
IN NO EVENT
((SHALL || WILL))
__5__ BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT,
((TORT || FART))
OR OTHERWISE,
ARISING FROM, OUT OF OR
((IN || I))
CONNECTION WITH
((TE || THE))
((SOFTWARE || MATERIALS))
OR THE USE OR OTHER DEALINGS IN
((THE
	((SOFTWARE || MATERIALS))
))??

		||
			THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
			KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
			WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
			MERCHANTABLITY OR NON-INFRINGEMENT.
		))
	))??

	((
		See the
		((License || Apache Version 2.0 License || Apache License Version 2.0))
		for
		((the))??
		specific language governing permissions and limitations
		((thereunder || there under || under the License.))
	))??

||

(( Apache License Version 2.0
  (( January 2004 ))??
  (( http:/www.apache.org/licenses/ ))??
))??

(( TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION ))??

   (( 1. ))??
   Definitions.

      "License" shall mean the terms and conditions for use, reproduction, and
      distribution as defined by Sections 1 through 9 of this document.

      "Licensor" shall mean the copyright owner or entity authorized by the
      copyright owner that is granting the License.

      "Legal Entity" shall mean the union of the acting entity and all other
      entities that control, are controlled by, or are under common control with
      that entity. For the purposes of this definition, "control" means (i) the
      power, direct or indirect, to cause the direction or management of such
      entity, whether by contract or otherwise, or (ii) ownership of fifty
      percent (50%) or more of the outstanding shares, or (iii) beneficial
      ownership of such entity.

      "You" (or "Your") shall mean an individual or Legal Entity exercising
      permissions granted by this License.

      "Source" form shall mean the preferred form for making modifications,
      including but not limited to software source code, documentation source,
      and configuration files.

      "Object" form shall mean any form resulting from mechanical transformation
      or translation of a Source form, including but not limited to compiled
      object code, generated documentation, and conversions to other media
      types.

      "Work" shall mean the work of authorship, whether in Source or Object
      form, made available under the License, as indicated by a copyright notice
      that is included in or attached to the work
      (( (an example is provided in the Appendix below). ))??

      "Derivative Works" shall mean any work, whether in Source or Object form,
      that is based on (or derived from) the Work and for which the editorial
      revisions, annotations, elaborations, or other modifications represent, as
      a whole, an original work of authorship. For the purposes of this License,
      Derivative Works shall not include works that remain separable from, or
      merely link (or bind by name) to the interfaces of, the Work and
      Derivative Works thereof.

      "Contribution" shall mean any work of authorship, including the original
      version of the Work and any modifications or additions to that Work or
      Derivative Works thereof, that is intentionally submitted to Licensor for
      inclusion in the Work by the copyright owner or by an individual or Legal
      Entity authorized to submit on behalf of the copyright owner. For the
      purposes of this definition, "submitted" means any form of electronic,
      verbal, or written communication sent to the Licensor or its
      representatives, including but not limited to communication on
      ((electronic))??
      mailing lists, source code control systems, and issue tracking systems
      that are managed by, or on behalf of, the Licensor for the purpose of
      discussing and improving the Work, but excluding communication that is
      conspicuously marked or otherwise designated in writing by the copyright
      owner as "Not a Contribution."

      "Contributor" shall mean Licensor and any individual or Legal Entity on
      behalf of whom a Contribution has been received by Licensor and
      subsequently incorporated within the Work.

   (( 2. ))??
   Grant of Copyright License. Subject to the terms and conditions of this
   License, each Contributor hereby grants to You a perpetual, worldwide,
   non-exclusive, no-charge, royalty-free, irrevocable copyright license to
   reproduce, prepare Derivative Works of, publicly display, publicly perform,
   sublicense, and distribute the Work and such Derivative Works in Source or
   Object form.

   (( 3. ))??
   Grant of Patent License. Subject to the terms and conditions of this License,
   each Contributor hereby grants to You a perpetual, worldwide, non-exclusive,
   no-charge, royalty-free, irrevocable (except as stated in this section)
   patent license to make, have made, use, offer to sell, sell, import, and
   otherwise transfer the Work, where such license applies only to those patent
   claims licensable by such Contributor that are necessarily infringed by their
   Contribution(s) alone or by combination of their Contribution(s) with the
   Work to which such Contribution(s) was submitted. If You institute patent
   litigation against any entity (including a cross-claim or counterclaim in a
   lawsuit) alleging that the Work or a Contribution incorporated within the
   Work constitutes direct or contributory patent infringement, then any patent
   licenses granted to You under this License for that Work shall terminate as
   of the date such litigation is filed.

   (( 4. ))??
   Redistribution. You may reproduce and distribute copies of the Work or
   Derivative Works thereof in any medium, with or without modifications, and in
   Source or Object form, provided that You meet the following conditions:

      (( 1. || (a) ))??
      You must give any other recipients of the Work or Derivative Works a copy
      of this License; and

      (( 2. || (b) ))??
      You must cause any modified files to carry prominent notices stating that
      You changed the files; and

      (( 3. || c ))??
      You must retain, in the Source form of any Derivative Works that You
      distribute, all copyright, patent, trademark, and attribution notices from
      the Source form of the Work, excluding those notices that do not pertain
      to any part of the Derivative Works; and

      (( 4. || (d) ))??
      If the Work includes a "NOTICE" text file as part of its distribution,
      then any Derivative Works that You distribute must include a readable copy
      of the attribution notices contained within such NOTICE file, excluding
      those notices that do not pertain to any part of the Derivative Works, in
      at least one of the following places: within a NOTICE text file
      distributed as part of the Derivative Works; within the Source form or
      documentation, if provided along with the Derivative Works; or, within a
      display generated by the Derivative Works, if and wherever such
      third-party notices normally appear. The contents of the NOTICE file are
      for informational purposes only and do not modify the License. You may add
      Your own attribution notices within Derivative Works that You distribute,
      alongside or as an addendum to the NOTICE text from the Work, provided
      that such additional attribution notices cannot be construed as modifying
      the License.

      (( 5. ))??
      You may add Your own copyright statement to Your modifications and may
      provide additional or different license terms and conditions for use,
      reproduction, or distribution of Your modifications, or for any such
      Derivative Works as a whole, provided Your use, reproduction, and
      distribution of the Work otherwise complies with the conditions stated in
      this License.

   (( 5. ))??
   Submission of Contributions. Unless You explicitly state otherwise, any
   Contribution intentionally submitted for inclusion in the Work by You to the
   Licensor shall be under the terms and conditions of this License, without any
   additional terms or conditions. Notwithstanding the above, nothing herein
   shall supersede or modify the terms of any separate license agreement you may
   have executed with Licensor regarding such Contributions.

   (( 6. ))??
   Trademarks. This License does not grant permission to use the trade names,
   trademarks, service marks, or product names of the Licensor, except as
   required for reasonable and customary use in describing the origin of the
   Work and reproducing the content of the NOTICE file.

   (( 7. ))??
   Disclaimer of Warranty. Unless required by applicable law or agreed to in
   writing, Licensor provides the Work (and each Contributor provides its
   Contributions) on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
   KIND, either express or implied, including, without limitation, any
   warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or
   FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining
   the appropriateness of using or redistributing the Work and assume any risks
   associated with Your exercise of permissions under this License.

   (( 8. ))??
   Limitation of Liability. In no event and under no legal theory, whether in
   tort (including negligence), contract, or otherwise, unless required by
   applicable law (such as deliberate and grossly negligent acts) or agreed to
   in writing, shall any Contributor be liable to You for damages, including any
   direct, indirect, special, incidental, or consequential damages of any
   character arising as a result of this License or out of the use or inability
   to use the Work (including but not limited to damages for loss of goodwill,
   work stoppage, computer failure or malfunction, or any and all other
   commercial damages or losses), even if such Contributor has been advised of
   the possibility of such damages.

   (( 9. ))??
   Accepting Warranty or Additional Liability. While redistributing the Work or
   Derivative Works thereof, You may choose to offer, and charge a fee for,
   acceptance of support, warranty, indemnity, or other liability obligations
   and/or rights consistent with this License. However, in accepting such
   obligations, You may act only on Your own behalf and on Your sole
   responsibility, not on behalf of any other Contributor, and only if You agree
   to indemnify, defend, and hold each Contributor harmless for any liability
   incurred by, or claims asserted against, such Contributor by reason of your
   accepting any such warranty or additional liability.

(( END OF TERMS AND CONDITIONS ))??

((
APPENDIX: How to apply the Apache License to your work.

To apply the Apache License to your work, attach the following boilerplate
notice, with the fields enclosed by brackets "[]" replaced with your own
identifying information. (Don't include the brackets!) The text should be
enclosed in the appropriate comment syntax for the file format. We also
recommend that a file or class name and description of purpose be included on
the same "printed page" as the copyright notice for easier identification within
third-party archives.

	Copyright __20__

	Licensed under the Apache License, Version 2.0 (the "License");
	you may not use this file except in compliance with the License.
	You may obtain a copy of the License at

	http:/www.apache.org/licenses/LICENSE-2.0

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.

	See the License for the specific language governing permissions and
	limitations under the License.
))??

))
`
const license_Artistic_1_0_lre = `//**
Artistic License 1.0
https://spdx.org/licenses/Artistic-1.0.json
https://opensource.org/licenses/Artistic-1.0
**//

(( The Artistic License ))??

Preamble

The intent of this document is to state the conditions under which a Package may
be copied, such that the Copyright Holder maintains some semblance of artistic
control over the development of the package, while giving the users of the
package the right to use and distribute the Package in a more-or-less customary
fashion, plus the right to make reasonable modifications.

Definitions:

"Package" refers to the collection of files distributed by the Copyright Holder,
and derivatives of that collection of files created through textual
modification.

"Standard Version" refers to such a Package if it has not been modified, or has
been modified in accordance with the wishes of the Copyright Holder.

"Copyright Holder" is whoever is named in the copyright or copyrights for the
package.

"You" is you, if you're thinking about copying or distributing this Package.

"Reasonable copying fee" is whatever you can justify on the basis of media cost,
duplication charges, time of people involved, and so on. (You will not be
required to justify it to the Copyright Holder, but only to the computing
community at large as a market that must bear the fee.)

"Freely Available" means that no fee is charged for the item itself, though
there may be fees involved in handling the item. It also means that recipients
of the item may redistribute it under the same conditions they received it.

   (( 1. ))??
   You may make and give away verbatim copies of the source form of the Standard
   Version of this Package without restriction, provided that you duplicate all
   of the original copyright notices and associated disclaimers.

   (( 2. ))??
   You may apply bug fixes, portability fixes and other modifications derived
   from the Public Domain or from the Copyright Holder. A Package modified in
   such a way shall still be considered the Standard Version.

   (( 3. ))??
   You may otherwise modify your copy of this Package in any way, provided that
   you insert a prominent notice in each changed file stating how and when you
   changed that file, and provided that you do at least ONE of the following:

      (( a) ))??
      place your modifications in the Public Domain or otherwise make them
      Freely Available, such as by posting said modifications to Usenet or an
      equivalent medium, or placing the modifications on a major archive site
      such as ftp.uu.net, or by allowing the Copyright Holder to include your
      modifications in the Standard Version of the Package.

      (( b) ))??
      use the modified Package only within your corporation or organization.

      (( c) ))??
      rename any non-standard executables so the names do not conflict with
      standard executables, which must also be provided, and provide a separate
      manual page for each non-standard executable that clearly documents how it
      differs from the Standard Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 4. ))??
   You may distribute the programs of this Package in object code or executable
   form, provided that you do at least ONE of the following:

      (( a) ))??
      distribute a Standard Version of the executables and library files,
      together with instructions (in the manual page or equivalent) on where to
      get the Standard Version.

      (( b) ))??
      accompany the distribution with the machine-readable source of the Package
      with your modifications.

      (( c) ))??
      accompany any non-standard executables with their corresponding Standard
      Version executables, giving the non-standard executables non-standard
      names, and clearly documenting the differences in manual pages (or
      equivalent), together with instructions on where to get the Standard
      Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 5. ))??
   You may charge a reasonable copying fee for any distribution of this Package.
   You may charge any fee you choose for support of this Package. You may not
   charge a fee for this Package itself. However, you may distribute this
   Package in aggregate with other (possibly commercial) programs as part of a
   larger (possibly commercial) software distribution provided that you do not
   advertise this Package as a product of your own.

   (( 6. ))??
   The scripts and library files supplied as input to or produced as output from
   the programs of this Package do not automatically fall under the copyright of
   this Package, but belong to whomever generated them, and may be sold
   commercially, and may be aggregated with this Package.

   (( 7. ))??
   C or perl subroutines supplied by you and linked into this Package shall not
   be considered part of this Package.

   (( 8. ))??
   The name of the Copyright Holder may not be used to endorse or promote
   products derived from this software without specific prior written
   permission.

   (( 9. ))??
   THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
   (( The End ))??
`
const license_Artistic_1_0_Perl_lre = `//**
Artistic License 1.0 (Perl)
https://spdx.org/licenses/Artistic-1.0-Perl.json
http://dev.perl.org/licenses/artistic.html
**//

(( The "Artistic License" ))??

Preamble

The intent of this document is to state the conditions under which a Package may
be copied, such that the Copyright Holder maintains some semblance of artistic
control over the development of the package, while giving the users of the
package the right to use and distribute the Package in a more-or-less customary
fashion, plus the right to make reasonable modifications.

Definitions:

"Package" refers to the collection of files distributed by the Copyright Holder,
and derivatives of that collection of files created through textual
modification.

"Standard Version" refers to such a Package if it has not been modified, or has
been modified in accordance with the wishes of the Copyright Holder as specified
below.

"Copyright Holder" is whoever is named in the copyright or copyrights for the
package.

"You" is you, if you're thinking about copying or distributing this Package.

"Reasonable copying fee" is whatever you can justify on the basis of media cost,
duplication charges, time of people involved, and so on. (You will not be
required to justify it to the Copyright Holder, but only to the computing
community at large as a market that must bear the fee.)

"Freely Available" means that no fee is charged for the item itself, though
there may be fees involved in handling the item. It also means that recipients
of the item may redistribute it under the same conditions they received it.

   (( 1. ))??
   You may make and give away verbatim copies of the source form of the Standard
   Version of this Package without restriction, provided that you duplicate all
   of the original copyright notices and associated disclaimers.

   (( 2. ))??
   You may apply bug fixes, portability fixes and other modifications derived
   from the Public Domain or from the Copyright Holder. A Package modified in
   such a way shall still be considered the Standard Version.

   (( 3. ))??
   You may otherwise modify your copy of this Package in any way, provided that
   you insert a prominent notice in each changed file stating how and when you
   changed that file, and provided that you do at least ONE of the following:

      (( a) ))??
      place your modifications in the Public Domain or otherwise make them
      Freely Available, such as by posting said modifications to Usenet or an
      equivalent medium, or placing the modifications on a major archive site
      such as uunet.uu.net, or by allowing the Copyright Holder to include your
      modifications in the Standard Version of the Package.

      (( b) ))??
      use the modified Package only within your corporation or organization.

      (( c) ))??
      rename any non-standard executables so the names do not conflict with
      standard executables, which must also be provided, and provide a separate
      manual page for each non-standard executable that clearly documents how it
      differs from the Standard Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 4. ))??
   You may distribute the programs of this Package in object code or executable
   form, provided that you do at least ONE of the following:

      (( a) ))??
      distribute a Standard Version of the executables and library files,
      together with instructions (in the manual page or equivalent) on where to
      get the Standard Version.

      (( b) ))??
      accompany the distribution with the machine-readable source of the Package
      with your modifications.

      (( c) ))??
      give non-standard executables non-standard names, and clearly document the
      differences in manual pages (or equivalent), together with instructions on
      where to get the Standard Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 5. ))??
   You may charge a reasonable copying fee for any distribution of this Package.
   You may charge any fee you choose for support of this Package. You may not
   charge a fee for this Package itself. However, you may distribute this
   Package in aggregate with other (possibly commercial) programs as part of a
   larger (possibly commercial) software distribution provided that you do not
   advertise this Package as a product of your own. You may embed this Package's
   interpreter within an executable of yours (by linking); this shall be
   construed as a mere form of aggregation, provided that the complete Standard
   Version of the interpreter is so embedded.

   (( 6. ))??
   The scripts and library files supplied as input to or produced as output from
   the programs of this Package do not automatically fall under the copyright of
   this Package, but belong to whoever generated them, and may be sold
   commercially, and may be aggregated with this Package. If such scripts or
   library files are aggregated with this Package via the so-called "undump" or
   "unexec" methods of producing a binary executable image, then distribution of
   such an image shall neither be construed as a distribution of this Package
   nor shall it fall under the restrictions of Paragraphs 3 and 4, provided that
   you do not represent such an executable image as a Standard Version of this
   Package.

   (( 7. ))??
   C subroutines (or comparably compiled subroutines in other languages)
   supplied by you and linked into this Package in order to emulate subroutines
   and variables of the language defined by this Package shall not be considered
   part of this Package, but are the equivalent of input as in Paragraph 6,
   provided these subroutines do not change the language in any way that would
   cause it to fail the regression tests for the language.

   (( 8. ))??
   Aggregation of this Package with a commercial distribution is always
   permitted provided that the use of this Package is embedded; that is, when no
   overt attempt is made to make this Package's interfaces visible to the end
   user of the commercial distribution. Such use shall not be construed as a
   distribution of this Package.

   (( 9. ))??
   The name of the Copyright Holder may not be used to endorse or promote
   products derived from this software without specific prior written
   permission.

   (( 10. ))??
   THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
   (( The End ))??
`
const license_Artistic_1_0_cl8_lre = `//**
Artistic License 1.0 w/clause 8
https://spdx.org/licenses/Artistic-1.0-cl8.json
https://opensource.org/licenses/Artistic-1.0
**//

(( The Artistic License ))??

Preamble

The intent of this document is to state the conditions under which a Package may
be copied, such that the Copyright Holder maintains some semblance of artistic
control over the development of the package, while giving the users of the
package the right to use and distribute the Package in a more-or-less customary
fashion, plus the right to make reasonable modifications.

Definitions:

"Package" refers to the collection of files distributed by the Copyright Holder,
and derivatives of that collection of files created through textual
modification.

"Standard Version" refers to such a Package if it has not been modified, or has
been modified in accordance with the wishes of the Copyright Holder.

"Copyright Holder" is whoever is named in the copyright or copyrights for the
package.

"You" is you, if you're thinking about copying or distributing this Package.

"Reasonable copying fee" is whatever you can justify on the basis of media cost,
duplication charges, time of people involved, and so on. (You will not be
required to justify it to the Copyright Holder, but only to the computing
community at large as a market that must bear the fee.)

"Freely Available" means that no fee is charged for the item itself, though
there may be fees involved in handling the item. It also means that recipients
of the item may redistribute it under the same conditions they received it.

   (( 1. ))??
   You may make and give away verbatim copies of the source form of the Standard
   Version of this Package without restriction, provided that you duplicate all
   of the original copyright notices and associated disclaimers.

   (( 2. ))??
   You may apply bug fixes, portability fixes and other modifications derived
   from the Public Domain or from the Copyright Holder. A Package modified in
   such a way shall still be considered the Standard Version.

   (( 3. ))??
   You may otherwise modify your copy of this Package in any way, provided that
   you insert a prominent notice in each changed file stating how and when you
   changed that file, and provided that you do at least ONE of the following:

      (( a) ))??
      place your modifications in the Public Domain or otherwise make them
      Freely Available, such as by posting said modifications to Usenet or an
      equivalent medium, or placing the modifications on a major archive site
      such as ftp.uu.net, or by allowing the Copyright Holder to include your
      modifications in the Standard Version of the Package.

      (( b) ))??
      use the modified Package only within your corporation or organization.

      (( c) ))??
      rename any non-standard executables so the names do not conflict with
      standard executables, which must also be provided, and provide a separate
      manual page for each non-standard executable that clearly documents how it
      differs from the Standard Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 4. ))??
   You may distribute the programs of this Package in object code or executable
   form, provided that you do at least ONE of the following:

      (( a) ))??
      distribute a Standard Version of the executables and library files,
      together with instructions (in the manual page or equivalent) on where to
      get the Standard Version.

      (( b) ))??
      accompany the distribution with the machine-readable source of the Package
      with your modifications.

      (( c) ))??
      accompany any non-standard executables with their corresponding Standard
      Version executables, giving the non-standard executables non-standard
      names, and clearly documenting the differences in manual pages (or
      equivalent), together with instructions on where to get the Standard
      Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 5. ))??
   You may charge a reasonable copying fee for any distribution of this Package.
   You may charge any fee you choose for support of this Package. You may not
   charge a fee for this Package itself. However, you may distribute this
   Package in aggregate with other (possibly commercial) programs as part of a
   larger (possibly commercial) software distribution provided that you do not
   advertise this Package as a product of your own.

   (( 6. ))??
   The scripts and library files supplied as input to or produced as output from
   the programs of this Package do not automatically fall under the copyright of
   this Package, but belong to whomever generated them, and may be sold
   commercially, and may be aggregated with this Package.

   (( 7. ))??
   C or perl subroutines supplied by you and linked into this Package shall not
   be considered part of this Package.

   8.Aggregation of this Package with a commercial distribution is always
   permitted provided that the use of this Package is embedded; that is, when no
   overt attempt is made to make this Package's interfaces visible to the end
   user of the commercial distribution. Such use shall not be construed as a
   distribution of this Package.

   (( 9. ))??
   The name of the Copyright Holder may not be used to endorse or promote
   products derived from this software without specific prior written
   permission.

   (( 10. ))??
   THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
   (( The End ))??
`
const license_Artistic_2_0_lre = `//**
Artistic License 2.0
https://spdx.org/licenses/Artistic-2.0.json
http://www.perlfoundation.org/artistic_license_2_0
https://opensource.org/licenses/artistic-license-2.0
**//

(( The Artistic License 2.0 ))??

((Copyright (c) 2000-2006, The Perl Foundation.))??

Everyone is permitted to copy and distribute verbatim copies of this license
document, but changing it is not allowed.

Preamble

This license establishes the terms under which a given free software Package may
be copied, modified, distributed, and/or redistributed. The intent is that the
Copyright Holder maintains some artistic control over the development of that
Package while still keeping the Package available as open source and free
software.

You are always permitted to make arrangements wholly outside of this license
directly with the Copyright Holder of a given Package. If the terms of this
license do not permit the full use that you propose to make of the Package, you
should contact the Copyright Holder and seek a different licensing arrangement.

Definitions

   "Copyright Holder" means the individual(s) or organization(s) named in the
   copyright notice for the entire Package.

   "Contributor" means any party that has contributed code or other material to
   the Package, in accordance with the Copyright Holder's procedures.

   "You" and "your" means any person who would like to copy, distribute, or
   modify the Package.

   "Package" means the collection of files distributed by the Copyright Holder,
   and derivatives of that collection and/or of those files. A given Package may
   consist of either the Standard Version, or a Modified Version.

   "Distribute" means providing a copy of the Package or making it accessible to
   anyone else, or in the case of a company or organization, to others outside
   of your company or organization.

   "Distributor Fee" means any fee that you charge for Distributing this Package
   or providing support for this Package to another party. It does not mean
   licensing fees.

   "Standard Version" refers to the Package if it has not been modified, or has
   been modified only in ways explicitly requested by the Copyright Holder.

   "Modified Version" means the Package, if it has been changed, and such
   changes were not explicitly requested by the Copyright Holder.

   "Original License" means this Artistic License as Distributed with the
   Standard Version of the Package, in its current version or as it may be
   modified by The Perl Foundation in the future.

   "Source" form means the source code, documentation source, and configuration
   files for the Package.

   "Compiled" form means the compiled bytecode, object code, binary, or any
   other form resulting from mechanical transformation or translation of the
   Source form.

Permission for Use and Modification Without Distribution

   (( (1) ))??
   You are permitted to use the Standard Version and create and use Modified
   Versions for any purpose without restriction, provided that you do not
   Distribute the Modified Version.

Permissions for Redistribution of the Standard Version

   (( (2) ))??
   You may Distribute verbatim copies of the Source form of the Standard Version
   of this Package in any medium without restriction, either gratis or for a
   Distributor Fee, provided that you duplicate all of the original copyright
   notices and associated disclaimers. At your discretion, such verbatim copies
   may or may not include a Compiled form of the Package.

   (( (3) ))??
   You may apply any bug fixes, portability changes, and other modifications
   made available from the Copyright Holder. The resulting Package will still be
   considered the Standard Version, and as such will be subject to the Original
   License.

Distribution of Modified Versions of the Package as Source

   (( (4) ))??
   You may Distribute your Modified Version as Source (either gratis or for a
   Distributor Fee, and with or without a Compiled form of the Modified Version)
   provided that you clearly document how it differs from the Standard Version,
   including, but not limited to, documenting any non-standard features,
   executables, or modules, and provided that you do at least ONE of the
   following:

      (( (a) ))??
      make the Modified Version available to the Copyright Holder of the
      Standard Version, under the Original License, so that the Copyright Holder
      may include your modifications in the Standard Version.

      (( (b) ))??
      ensure that installation of your Modified Version does not prevent the
      user installing or running the Standard Version. In addition, the Modified
      Version must bear a name that is different from the name of the Standard
      Version.

      (( c ))??
      allow anyone who receives a copy of the Modified Version to make the
      Source form of the Modified Version available to others under

         (( (i) ))??
         the Original License or

         (( (ii) ))??
         a license that permits the licensee to freely copy, modify and
         redistribute the Modified Version using the same licensing terms that
         apply to the copy that the licensee received, and requires that the
         Source form of the Modified Version, and of any works derived from it,
         be made freely available in that license fees are prohibited but
         Distributor Fees are allowed.

Distribution of Compiled Forms of the Standard Version or Modified Versions
without the Source

   (( (5) ))??
   You may Distribute Compiled forms of the Standard Version without the Source,
   provided that you include complete instructions on how to get the Source of
   the Standard Version. Such instructions must be valid at the time of your
   distribution. If these instructions, at any time while you are carrying out
   such distribution, become invalid, you must provide new instructions on
   demand or cease further distribution. If you provide valid instructions or
   cease distribution within thirty days after you become aware that the
   instructions are invalid, then you do not forfeit any of your rights under
   this license.

   (( (6) ))??
   You may Distribute a Modified Version in Compiled form without the Source,
   provided that you comply with Section 4 with respect to the Source of the
   Modified Version.

Aggregating or Linking the Package

   (( (7) ))??
   You may aggregate the Package (either the Standard Version or Modified
   Version) with other packages and Distribute the resulting aggregation
   provided that you do not charge a licensing fee for the Package. Distributor
   Fees are permitted, and licensing fees for other components in the
   aggregation are permitted. The terms of this license apply to the use and
   Distribution of the Standard or Modified Versions as included in the
   aggregation.

   (( (8) ))??
   You are permitted to link Modified and Standard Versions with other works, to
   embed the Package in a larger work of your own, or to build stand-alone
   binary or bytecode versions of applications that include the Package, and
   Distribute the result without restriction, provided the result does not
   expose a direct interface to the Package.

Items That are Not Considered Part of a Modified Version

   (( (9) ))??
   Works (including, but not limited to, modules and scripts) that merely extend
   or make use of the Package, do not, by themselves, cause the Package to be a
   Modified Version. In addition, such works are not considered parts of the
   Package itself, and are not subject to the terms of this license.

General Provisions

   (( (10) ))??
   Any use, modification, and distribution of the Standard or Modified Versions
   is governed by this Artistic License. By using, modifying or distributing the
   Package, you accept this license. Do not use, modify, or distribute the
   Package, if you do not accept this license.

   (( (11) ))??
   If your Modified Version has been derived from a Modified Version made by
   someone other than you, you are nevertheless required to ensure that your
   Modified Version complies with the requirements of this license.

   (( (12) ))??
   This license does not grant you the right to use any trademark, service mark,
   tradename, or logo of the Copyright Holder.

   (( (13) ))??
   This license includes the non-exclusive, worldwide, free-of-charge patent
   license to make, have made, use, offer to sell, sell, import and otherwise
   transfer the Package with respect to any patent claims licensable by the
   Copyright Holder that are necessarily infringed by the Package. If you
   institute patent litigation (including a cross-claim or counterclaim) against
   any party alleging that the Package constitutes direct or contributory patent
   infringement, then this Artistic License to you shall terminate on the date
   that such litigation is filed.

   (( (14) ))??
   Disclaimer of Warranty:

   THE PACKAGE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS "AS IS" AND
   WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES. THE IMPLIED WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT ARE
   DISCLAIMED TO THE EXTENT PERMITTED BY YOUR LOCAL LAW. UNLESS REQUIRED BY LAW,
   NO COPYRIGHT HOLDER OR CONTRIBUTOR WILL BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THE
   PACKAGE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
`
const license_BSD_1_Clause_lre = `
//**
BSD 1-Clause License
https://spdx.org/licenses/BSD-1-Clause.json
https://svnweb.freebsd.org/base/head/include/ifaddrs.h?revision=326823
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))


`
const license_BSD_1_Clause_Clear_lre = `
//**
Not known to SPDX - BSD-3-Clause-Clear with only 1 Clause
Example:
	https://github.com/spate/glimage
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S
	PATENT RIGHTS ARE GRANTED BY THIS LICENSE.


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))


`
const license_BSD_2_Clause_lre = `
//**
BSD 2-Clause "Simplified" License
https://spdx.org/licenses/BSD-2-Clause.json
https://opensource.org/licenses/BSD-2-Clause
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))


`
const license_BSD_2_Clause_Patent_lre = `
//**
BSD-2-Clause Plus Patent License
https://spdx.org/licenses/BSD-2-Clause-Patent.json
https://opensource.org/licenses/BSDplusPatent
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	Subject to the terms and conditions of this license, each copyright holder and
	contributor hereby grants to those receiving rights under this license a
	perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable
	(except for failure to satisfy the conditions of this license) patent license to
	make, have made, use, offer to sell, sell, import, and otherwise transfer this
	software, where such license applies only to those patent claims, already
	acquired or hereafter acquired, licensable by such copyright holder or
	contributor that are necessarily infringed by:

	   __1__ their Contribution(s) (the licensed copyrights of copyright holders and
	   non-copyrightable additions of contributors, in source or binary form) alone;
	   or

	   __1__ combination of their Contribution(s) with the work of authorship to
	   which such Contribution(s) was added by such copyright holder or contributor,
	   if, at the time the Contribution is added, such addition causes such
	   combination to be necessarily infringed. The patent license shall not apply
	   to any other combinations which include the Contribution.

	Except as expressly stated above, no rights or licenses from any copyright
	holder or contributor is granted under this license, whether expressly, by
	implication, estoppel or otherwise.


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))


`
const license_BSD_2_Clause_Views_lre = `
//**
BSD 2-Clause Views
https://spdx.org/licenses/BSD-2-Clause-Views.json
http://www.freebsd.org/copyright/freebsd-license.html

This is the same as the BSD-2-Clause plus a paragraph
about the views of the project. It has no effect on use of
the software, so we just merge these two together.
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))



	The views and conclusions contained in the software and documentation are those
	of the authors and should not be interpreted as representing official policies,
	either expressed or implied, of
	((The FreeBSD Project || the copyright holders or contributors))??

`
const license_BSD_3_Clause_lre = `
//**
BSD 3-Clause "New" or "Revised" License
https://spdx.org/licenses/BSD-3-Clause.json
https://opensource.org/licenses/BSD-3-Clause
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	__1__
	((Neither || None || Names || The names || The name))
	__40__ used to endorse
	or promote
	products derived from this
	((software || work))
	without specific
	prior written permission
	((
		((of || from))
		__10__
	))??


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))


`
const license_BSD_3_Clause_Attribution_lre = `
//**
BSD with attribution
https://spdx.org/licenses/BSD-3-Clause-Attribution.json
https://fedoraproject.org/wiki/Licensing/BSD_with_Attribution
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	__1__
	((Neither || None || Names || The names || The name))
	__40__ used to endorse
	or promote
	products derived from this
	((software || work))
	without specific
	prior written permission
	((
		((of || from))
		__10__
	))??


	__1__
	Redistributions of any form whatsoever must retain the following
	acknowledgment:
	This product includes software developed by __40__


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))


`
const license_BSD_3_Clause_Clear_lre = `
//**
BSD 3-Clause Clear License
https://spdx.org/licenses/BSD-3-Clause-Clear.json
http://labs.metacarta.com/license-explanation.html#license
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	__1__
	((Neither || None || Names || The names || The name))
	__40__ used to endorse
	or promote
	products derived from this
	((software || work))
	without specific
	prior written permission
	((
		((of || from))
		__10__
	))??


	NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S
	PATENT RIGHTS ARE GRANTED BY THIS LICENSE.


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))


`
const license_BSD_3_Clause_LBNL_lre = `
//**
Lawrence Berkeley National Labs BSD variant license
https://spdx.org/licenses/BSD-3-Clause-LBNL.json
https://fedoraproject.org/wiki/Licensing/LBNLBSD
**//

//**
BSD 3-Clause "New" or "Revised" License
https://spdx.org/licenses/BSD-3-Clause.json
https://opensource.org/licenses/BSD-3-Clause
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	__1__
	((Neither || None || Names || The names || The name))
	__40__ used to endorse
	or promote
	products derived from this
	((software || work))
	without specific
	prior written permission
	((
		((of || from))
		__10__
	))??


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))




You are under no obligation whatsoever to provide any bug fixes, patches, or
upgrades to the features, functionality or performance of the source code
("Enhancements") to anyone; however, if you choose to make your Enhancements
available either publicly, or directly to __5__ , without imposing a separate
written license agreement for such Enhancements, then you hereby grant the
following license: a non-exclusive, royalty-free perpetual license to install,
use, modify, prepare derivative works, incorporate into other computer software,
distribute, and sublicense such Enhancements or derivative works thereof, in
binary and source code form.
`
const license_BSD_3_Clause_No_Nuclear_License_lre = `
//**
BSD 3-Clause No Nuclear License
https://spdx.org/licenses/BSD-3-Clause-No-Nuclear-License.json
http://download.oracle.com/otn-pub/java/licenses/bsd.txt?AuthParam=1467140197_43d516ce1776bd08a58235a7785be1cc
**//


	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	__1__
	((Neither || None || Names || The names || The name))
	__40__ used to endorse
	or promote
	products derived from this
	((software || work))
	without specific
	prior written permission
	((
		((of || from))
		__10__
	))??


This software is provided "AS IS," without a warranty of any kind. ALL EXPRESS
OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED
WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN MICROSYSTEMS, INC. ("SUN") AND ITS
LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT
OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES. IN NO
EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,
HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE
OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

You acknowledge that this software is not designed, licensed or intended for use
in the design, construction, operation or maintenance of any nuclear facility.

`
const license_BSD_3_Clause_No_Nuclear_License_2014_lre = `
//**
BSD 3-Clause No Nuclear License 2014
https://spdx.org/licenses/BSD-3-Clause-No-Nuclear-License-2014.json
https://java.net/projects/javaeetutorial/pages/BerkeleyLicense
**//

//**
BSD 3-Clause "New" or "Revised" License
https://spdx.org/licenses/BSD-3-Clause.json
https://opensource.org/licenses/BSD-3-Clause
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	__1__
	((Neither || None || Names || The names || The name))
	__40__ used to endorse
	or promote
	products derived from this
	((software || work))
	without specific
	prior written permission
	((
		((of || from))
		__10__
	))??


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))



You acknowledge that this software is not designed, licensed or intended for use
in the design, construction, operation or maintenance of any nuclear facility.
`
const license_BSD_3_Clause_No_Nuclear_Warranty_lre = `
//**
BSD 3-Clause No Nuclear License
https://spdx.org/licenses/BSD-3-Clause-No-Nuclear-License.json
http://download.oracle.com/otn-pub/java/licenses/bsd.txt?AuthParam=1467140197_43d516ce1776bd08a58235a7785be1cc
**//


	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	__1__
	((Neither || None || Names || The names || The name))
	__40__ used to endorse
	or promote
	products derived from this
	((software || work))
	without specific
	prior written permission
	((
		((of || from))
		__10__
	))??


This software is provided "AS IS," without a warranty of any kind. ALL EXPRESS
OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED
WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN MICROSYSTEMS, INC. ("SUN") AND ITS
LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT
OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES. IN NO
EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,
HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE
OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

You acknowledge that this software is not designed,  or intended for use
in the design, construction, operation or maintenance of any nuclear facility.

`
const license_BSD_3_Clause_NoTrademark_lre = `
//**
BSD 3-Clause + no-trademark, like Clear is no-patent.
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	__1__
	((Neither || None || Names || The names || The name))
	__40__ used to endorse
	or promote
	products derived from this
	((software || work))
	without specific
	prior written permission
	((
		((of || from))
		__10__
	))??


	No license is granted to the trademarks of
	the copyright holders even if such marks
	are included in this software.


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))


`
const license_BSD_3_Clause_Open_MPI_lre = `
//**
BSD 3-Clause Open MPI variant
https://spdx.org/licenses/BSD-3-Clause-Open-MPI.json
https://www.open-mpi.org/community/license.php
http://www.netlib.org/lapack/LICENSE.txt
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	__1__
	((Neither || None || Names || The names || The name))
	__40__ used to endorse
	or promote
	products derived from this
	((software || work))
	without specific
	prior written permission
	((
		((of || from))
		__10__
	))??


The copyright holders provide no reassurances that the source code provided does
not infringe any patent, copyright, or any other intellectual property rights of
third parties. The copyright holders disclaim any liability to any recipient for
claims brought against recipient by any third party for infringement of that
parties intellectual property rights.


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))


`
const license_BSD_4_Clause_UC_lre = `
//**
BSD 4-Clause (Univeristy of California-Specific)
https://spdx.org/licenses/BSD-4-Clause-UC.json
http://www.freebsd.org/copyright/license.html
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	__1__
	All advertising materials mentioning features or use of this software
	must display the following acknowledgement:

	This product includes software developed by
	the University of California, Berkeley and its contributors.

	__1__
	Neither the name of the University nor the names of its contributors
	may be used to endorse
	or promote products derived from this software without specific
	prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ''AS IS'' AND ANY
	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
	LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
	OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
	LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
	ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

`
const license_BSD_4_Clause_lre = `
//**
BSD 4-Clause "Original" or "Old" License
https://spdx.org/licenses/BSD-4-Clause.json
http://directory.fsf.org/wiki/License:BSD_4Clause
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Redistribution || Redistributions))
	in binary form must reproduce the
	((above))??
	((original))??
	copyright
	((notice))??
	this list of conditions and the
	following
	((two pargraphs of))??
	disclaimer
	((listed in this license))??
	in the documentation
	((and/or || and || or))
	other materials provided with the distribution.


	__1__
	All advertising materials mentioning features or use of this software
	must display the following acknowledgement:

	This product includes software developed by
	__40__

	be used to endorse
	or promote products derived from this software without specific
	prior written permission.


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))


`
const license_BSD_Protection_lre = `//**
BSD Protection License
https://spdx.org/licenses/BSD-Protection.json
https://fedoraproject.org/wiki/Licensing/BSD_Protection_License
**//

(( BSD Protection License

February 2002))??

(( Preamble

--------

The Berkeley Software Distribution ("BSD") license has proven very effective
over the years at allowing for a wide spread of work throughout both commercial
and non-commercial products. For programmers whose primary intention is to
improve the general quality of available software, it is arguable that there is
no better license than the BSD license, as it permits improvements to be used
wherever they will help, without idealogical or metallic constraint.

This is of particular value to those who produce reference implementations of
proposed standards: The case of TCP/IP clearly illustrates that freely and
universally available implementations leads the rapid acceptance of standards --
often even being used instead of a de jure standard (eg, OSI network models).

With the rapid proliferation of software licensed under the GNU General Public
License, however, the continued success of this role is called into question.
Given that the inclusion of a few lines of "GPL-tainted" work into a larger body
of work will result in restricted distribution -- and given that further work
will likely build upon the "tainted" portions, making them difficult to remove
at a future date -- there are inevitable circumstances where authors would, in
order to protect their goal of providing for the widespread usage of their work,
wish to guard against such "GPL-taint".

In addition, one can imagine that companies which operate by producing and
selling (possibly closed-source) code would wish to protect themselves against
the rise of a GPL-licensed competitor. While under existing licenses this would
mean not releasing their code under any form of open license, if a license
existed under which they could incorporate any improvements back into their own
(commercial) products then they might be far more willing to provide for
non-closed distribution.

For the above reasons, we put forth this "BSD Protection License": A license
designed to retain the freedom granted by the BSD license to use licensed works
in a wide variety of settings, both non-commercial and commercial, while
protecting the work from having future contributors restrict that freedom.

The precise terms and conditions for copying, distribution, and modification
follow.))??


BSD PROTECTION LICENSE TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION, AND
MODIFICATION
(( ----------------------------------------------------------------))??


   __1__ Definitions.

      __1__ "Program", below, refers to any program or work distributed under
      the terms of this license.

      __1__ A "work based on the Program", below, refers to either the Program
      or any derivative work under copyright law.

      __1__ "Modification", below, refers to the act of creating derivative
      works.

      __1__ "You", below, refers to each licensee.

   __1__ Scope.

   This license governs the copying, distribution, and modification of the
   Program. Other activities are outside the scope of this license; The act of
   running the Program is not restricted, and the output from the Program is
   covered only if its contents constitute a work based on the Program.

   __1__ Verbatim copies.

   You may copy and distribute verbatim copies of the Program as you receive it,
   in any medium, provided that you conspicuously and appropriately publish on
   each copy an appropriate copyright notice; keep intact all the notices that
   refer to this License and to the absence of any warranty; and give any other
   recipients of the Program a copy of this License along with the Program.

   __1__ Modification and redistribution under closed license.

   You may modify your copy or copies of the Program, and distribute the
   resulting derivative works, provided that you meet the following conditions:

      __1__ The copyright notice and disclaimer on the Program must be
      reproduced and included in the source code, documentation, and/or other
      materials provided in a manner in which such notices are normally
      distributed.

      __1__ The derivative work must be clearly identified as such, in order
      that it may not be confused with the original work.

      __1__ The license under which the derivative work is distributed must
      expressly prohibit the distribution of further derivative works.

   __1__ Modification and redistribution under open license.

   You may modify your copy or copies of the Program, and distribute the
   resulting derivative works, provided that you meet the following conditions:

      __1__ The copyright notice and disclaimer on the Program must be
      reproduced and included in the source code, documentation, and/or other
      materials provided in a manner in which such notices are normally
      distributed.

      __1__ You must clearly indicate the nature and date of any changes made to
      the Program. The full details need not necessarily be included in the
      individual modified files, provided that each modified file is clearly
      marked as such and instructions are included on where the full details of
      the modifications may be found.

      __1__ You must cause any work that you distribute or publish, that in
      whole or in part contains or is derived from the Program or any part
      thereof, to be licensed as a whole at no charge to all third parties under
      the terms of this License.

   __1__ Implied acceptance.

   You may not copy or distribute the Program or any derivative works except as
   expressly provided under this license. Consequently, any such action will be
   taken as implied acceptance of the terms of this license.

   __1__ NO WARRANTY.

THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE THE PROGRAM AS
PERMITTED ABOVE, BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE PROGRAM (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT, EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
`
const license_BSD_Source_Code_lre = `
//**
BSD 1-Clause License plus non-advertising clause (usual BSD clause #3)
https://spdx.org/licenses/BSD-Source-Code.json

Examples:
	https://github.com/robbiehanson/XMPPFramework/blob/5f24033e/copying.txt
	https://github.com/yapstudios/YapDatabase/blob/e3df69f2/LICENSE.txt
	https://github.com/robbiehanson/CocoaHTTPServer/blob/cb86571a/LICENSE.txt
	https://github.com/skelterjohn/rerun/blob/eb5929af/LICENSE
	https://github.com/skelterjohn/go.matrix/blob/go1/LICENSE
**//

	Redistribution and use
	((of
		((this software || __5__))
	))??
	in source and binary forms
	((of __6__))??
	with or
	((without))??
	modification,
	are permitted
	((subject to the limitations in the disclaimer below))??
	((provided || providing))
	that
	((the))??
	following conditions are met:
	((BSD style license))??


	__1__
	((Redistribution || Redistributions))
	of
	((source code || works))
	must retain the
	((above))??
	((original))??
	copyright
	((notice))??
	((immediately at the beginning of the file, without modification))??
	this
	((list of conditions || condition))
	and the
	((following
		((two paragraphs of))??
		disclaimer
	|| disclaimer that follows))
	((in this position and unchanged))??


	__1__
	((Neither || None || Names || The names || The name))
	__40__ used to endorse
	or promote
	products derived from this
	((software || work))
	without specific
	prior written permission
	((
		((of || from))
		__10__
	))??


	((DISCLAIMER
	))??

	((
		THE
		((SOFTWARE || WORK))
		((AND DOCUMENTATION))??
		IS PROVIDED
		((BY __20__))??
		"AS IS"
		((
			((WITHOUT ANY WARRANTIES WHATSOEVER))??
			((AND))??
			ANY
			((EXPRESS || EXPRESSED))
			OR IMPLIED WARRANTIES,
			INCLUDING, BUT NOT LIMITED TO,
			THE IMPLIED WARRANTIES
			((OF || OR))
			((NONINFRINGEMENT))??
			((MERCHANTABILITY))??
			((AND))??
			FITNESS FOR A PARTICULAR PURPOSE
			((OR NONINFRINGEMENT))??
			ARE
			((EXPRESSLY AND SPECIFICALLY))??
			((HEREBY))??
			DISCLAIMED.
		||
			//** Alternate form in libpcap, which also omits the IN NO EVENT paragraph. **//
			AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
			INCLUDING, WITHOUT LIMITATION,
			THE IMPLIED WARRANTIES OF MERCHANTABILTY
			AND FITNESS FOR A PARTICULAR PURPOSE.
		))

		((
			IN NO EVENT SHALL __20__ BE LIABLE
			FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
			CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
			GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
			HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
			LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
			ARISING IN ANY WAY OUT OF THE USE OF THIS
			((
				((SOFTWARE || WORK))
				((AND DOCUMENTATION))??
				EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
			))??
		))??

	||

		//**
		Alternate form found in some recent University of California releases.
		**//
		IN NO EVENT SHALL __20__ BE LIABLE
		TO ANY PARTY FOR DIRECT, INDIRECT,
		SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
		ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
		__20__ HAS BEEN ADVISED
		OF THE POSSIBILITY OF SUCH DAMAGE.

		__20__ SPECIFICALLY DISCLAIMS
		ANY WARRANTIES, INCLUDING, BUT NOT
		LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
		A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
		ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS"
		__20__ HAS NO OBLIGATION
		TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
		MODIFICATIONS.
	))


`
const license_BSL_1_0_lre = `//**
Boost Software License 1.0
https://spdx.org/licenses/BSL-1.0.json
http://www.boost.org/LICENSE_1_0.txt
https://opensource.org/licenses/BSL-1.0
**//

(( Boost Software License - Version 1.0 - August 17th, 2003 ))??

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software
((and accompanying documentation))??
covered by this license
(( (the "Software") ))??
to use, reproduce, display, distribute, execute, and
transmit the Software, and to prepare derivative works of the Software, and to
permit third-parties to whom the Software is furnished to do so, all subject to
the following:

The copyright notices in the Software and this entire statement, including the
above license grant, this restriction and the following disclaimer, must be
included in all copies of the Software, in whole or in part, and all derivative
works of the Software, unless such copies or derivative works are solely in the
form of machine-executable object code generated by a source language
processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE FOR ANY DAMAGES
OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
`
const license_Bahyph_lre = `//**
Bahyph License
https://spdx.org/licenses/Bahyph.json
https://fedoraproject.org/wiki/Licensing/Bahyph
**//

(( COPYRIGHT NOTICE ))??

These patterns and the generating sh script are Copyright (c) GMV 1991

These patterns were developed for internal GMV use and are made public in the
hope that they will benefit others. Also, spreading these patterns throughout
the Spanish-language TeX community is expected to provide back-benefits to GMV
in that it can help keeping GMV in the mainstream of spanish users.

However, this is given for free and WITHOUT ANY WARRANTY. Under no circumstances
can Julio Sanchez, GMV, Jos'e A. Ma~nas or any agents or representatives thereof
be held responsible for any errors in this software nor for any damages derived
from its use, even in case any of the above has been notified of the possibility
of such damages. If any such situation arises, you responsible for repair. Use
of this software is an explicit acceptance of these conditions.

You can use this software for any purpose. You cannot delete this copyright
notice. If you change this software, you must include comments explaining who,
when and why. You are kindly requested to send any changes to tex@gmv.es. If you
change the generating script, you must include code in it such that any output
is clearly labeled as generated by a modified script. Despite the lack of
warranty, we would like to hear about any problem you find. Please report
problems to tex@gmv.es.
(( END OF COPYRIGHT NOTICE ))??
`
const license_Barr_lre = `//**
Barr License
https://spdx.org/licenses/Barr.json
https://fedoraproject.org/wiki/Licensing/Barr
**//

This is a package of commutative diagram macros built on top of Xy-pic by
Michael Barr (email: barr@barrs.org). Its use is unrestricted. It may be freely
distributed, unchanged, for non-commercial or commercial use. If changed, it
must be renamed. Inclusion in a commercial software package is also permitted,
but I would appreciate receiving a free copy for my personal examination and
use. There are no guarantees that this package is good for anything. I have
tested it with LaTeX 2e, LaTeX 2.09 and Plain TeX. Although I know of no reason
it will not work with AMSTeX, I have not tested it.
`
const license_Beerware_lre = `//**
Beerware License
https://spdx.org/licenses/Beerware.json
https://fedoraproject.org/wiki/Licensing/Beerware
https://people.freebsd.org/~phk/
**//

"THE BEER-WARE LICENSE" (Revision __1__):

(( __10__ wrote
((this file || those files || this code || this software))
. ))??
As long as you retain this notice you can do
whatever you want with this stuff. If we meet some day, and you think this stuff
is worth it, you can buy
((me || us))
((a beer || many beers))
in return
((Poul-Henning Kamp))??

((
__5__
For more information about the BEER-WARE license please see phk's web site:
http://people.freebsd.org/~phk/
))??
`
const license_BitTorrent_1_0_lre = `//**
BitTorrent Open Source License v1.0
https://spdx.org/licenses/BitTorrent-1.0.json
http://sources.gentoo.org/cgi-bin/viewvc.cgi/gentoo-x86/licenses/BitTorrent?r1=1.1&r2=1.1.1.1&diff_format=s
**//

(( BitTorrent Open Source License

Version 1.0 ))??

This BitTorrent Open Source License (the "License") applies to the BitTorrent
client and related software products as well as any updates or maintenance
releases of that software ("BitTorrent Products") that are distributed by
BitTorrent, Inc. ("Licensor"). Any BitTorrent Product licensed pursuant to this
License is a Licensed Product. Licensed Product, in its entirety, is protected
by U.S. copyright law. This License identifies the terms under which you may
use, copy, distribute or modify Licensed Product.

Preamble

This Preamble is intended to describe, in plain English, the nature and scope of
this License. However, this Preamble is not a part of this license. The legal
effect of this License is dependent only upon the terms of the License and not
this Preamble.

This License complies with the Open Source Definition and is derived from the
Jabber Open Source License 1.0 (the "JOSL"), which has been approved by Open
Source Initiative. Sections 4(c) and 4(f)(iii) from the JOSL have been dropped.

This License provides that:

   (( 1. ))??
   You may use, sell or give away the Licensed Product, alone or as a component
   of an aggregate software distribution containing programs from several
   different sources. No royalty or other fee is required.

   (( 2. ))??
   Both Source Code and executable versions of the Licensed Product, including
   Modifications made by previous Contributors, are available for your use. (The
   terms "Licensed Product," "Modifications," "Contributors" and "Source Code"
   are defined in the License.)

   (( 3. ))??
   You are allowed to make Modifications to the Licensed Product, and you can
   create Derivative Works from it. (The term "Derivative Works" is defined in
   the License.)

   (( 4. ))??
   By accepting the Licensed Product under the provisions of this License, you
   agree that any Modifications you make to the Licensed Product and then
   distribute are governed by the provisions of this License. In particular, you
   must make the Source Code of your Modifications available to others.

   (( 5. ))??
   You may use the Licensed Product for any purpose, but the Licensor is not
   providing you any warranty whatsoever, nor is the Licensor accepting any
   liability in the event that the Licensed Product doesn't work properly or
   causes you any injury or damages.

   (( 6. ))??
   If you sublicense the Licensed Product or Derivative Works, you may charge
   fees for warranty or support, or for accepting indemnity or liability
   obligations to your customers. You cannot charge for the Source Code.

   (( 7. ))??
   If you assert any patent claims against the Licensor relating to the Licensed
   Product, or if you breach any terms of the License, your rights to the
   Licensed Product under this License automatically terminate.

You may use this License to distribute your own Derivative Works, in which case
the provisions of this License will apply to your Derivative Works just as they
do to the original Licensed Product.

Alternatively, you may distribute your Derivative Works under any other
OSI-approved Open Source license, or under a proprietary license of your choice.
If you use any license other than this License, however, you must continue to
fulfill the requirements of this License (including the provisions relating to
publishing the Source Code) for those portions of your Derivative Works that
consist of the Licensed Product, including the files containing Modifications.

New versions of this License may be published from time to time. You may choose
to continue to use the license terms in this version of the License or those
from the new version. However, only the Licensor has the right to change the
License terms as they apply to the Licensed Product.

This License relies on precise definitions for certain terms. Those terms are
defined when they are first used, and the definitions are repeated for your
convenience in a Glossary at the end of the License.

License Terms

   (( 1. ))??
   Grant of License From Licensor. Licensor hereby grants you a world-wide,
   royalty-free, non-exclusive license, subject to third party intellectual
   property claims, to do the following:

      (( a. ))??
      Use, reproduce, modify, display, perform, sublicense and distribute any
      Modifications created by such Contributor or portions thereof, in both
      Source Code or as an executable program, either on an unmodified basis or
      as part of Derivative Works.

      (( b. ))??
      Under claims of patents now or hereafter owned or controlled by
      Contributor, to make, use, sell, offer for sale, have made, and/or
      otherwise dispose of Modifications or portions thereof, but solely to the
      extent that any such claim is necessary to enable you to make, use, sell,
      offer for sale, have made, and/or otherwise dispose of Modifications or
      portions thereof or Derivative Works thereof.

   (( 2. ))??
   Grant of License to Modifications From Contributor. "Modifications" means any
   additions to or deletions from the substance or structure of (i) a file
   containing Licensed Product, or (ii) any new file that contains any part of
   Licensed Product. Hereinafter in this License, the term "Licensed Product"
   shall include all previous Modifications that you receive from any
   Contributor. By application of the provisions in Section 4(a) below, each
   person or entity who created or contributed to the creation of, and
   distributed, a Modification (a "Contributor") hereby grants you a world-wide,
   royalty-free, non-exclusive license, subject to third party intellectual
   property claims, to do the following:

      (( a. ))??
      Use, reproduce, modify, display, perform, sublicense and distribute any
      Modifications created by such Contributor or portions thereof, in both
      Source Code or as an executable program, either on an unmodified basis or
      as part of Derivative Works.

      (( b. ))??
      Under claims of patents now or hereafter owned or controlled by
      Contributor, to make, use, sell, offer for sale, have made, and/or
      otherwise dispose of Modifications or portions thereof, but solely to the
      extent that any such claim is necessary to enable you to make, use, sell,
      offer for sale, have made, and/or otherwise dispose of Modifications or
      portions thereof or Derivative Works thereof.

   (( 3. ))??
   Exclusions From License Grant. Nothing in this License shall be deemed to
   grant any rights to trademarks, copyrights, patents, trade secrets or any
   other intellectual property of Licensor or any Contributor except as
   expressly stated herein. No patent license is granted separate from the
   Licensed Product, for code that you delete from the Licensed Product, or for
   combinations of the Licensed Product with other software or hardware. No
   right is granted to the trademarks of Licensor or any Contributor even if
   such marks are included in the Licensed Product. Nothing in this License
   shall be interpreted to prohibit Licensor from licensing under different
   terms from this License any code that Licensor otherwise would have a right
   to license.

   (( 4. ))??
   Your Obligations Regarding Distribution.

      (( a. ))??
      Application of This License to Your Modifications. As an express condition
      for your use of the Licensed Product, you hereby agree that any
      Modifications that you create or to which you contribute, and which you
      distribute, are governed by the terms of this License including, without
      limitation, Section 2. Any Modifications that you create or to which you
      contribute may be distributed only under the terms of this License or a
      future version of this License released under Section 7. You must include
      a copy of this License with every copy of the Modifications you
      distribute. You agree not to offer or impose any terms on any Source Code
      or executable version of the Licensed Product or Modifications that alter
      or restrict the applicable version of this License or the recipients'
      rights hereunder. However, you may include an additional document offering
      the additional rights described in Section 4(d).

      (( b. ))??
      Availability of Source Code. You must make available, under the terms of
      this License, the Source Code of the Licensed Product and any
      Modifications that you distribute, either on the same media as you
      distribute any executable or other form of the Licensed Product, or via a
      mechanism generally accepted in the software development community for the
      electronic transfer of data (an "Electronic Distribution Mechanism"). The
      Source Code for any version of Licensed Product or Modifications that you
      distribute must remain available for at least twelve (12) months after the
      date it initially became available, or at least six (6) months after a
      subsequent version of said Licensed Product or Modifications has been made
      available. You are responsible for ensuring that the Source Code version
      remains available even if the Electronic Distribution Mechanism is
      maintained by a third party.

      (( c. ))??
      Intellectual Property Matters.

         (( i. ))??
         Third Party Claims. If you have knowledge that a license to a third
         party's intellectual property right is required to exercise the rights
         granted by this License, you must include a text file with the Source
         Code distribution titled "LEGAL" that describes the claim and the party
         making the claim in sufficient detail that a recipient will know whom
         to contact. If you obtain such knowledge after you make any
         Modifications available as described in Section 4(b), you shall
         promptly modify the LEGAL file in all copies you make available
         thereafter and shall take other steps (such as notifying appropriate
         mailing lists or newsgroups) reasonably calculated to inform those who
         received the Licensed Product from you that new knowledge has been
         obtained.

         (( ii. ))??
         Contributor APIs. If your Modifications include an application
         programming interface ("API") and you have knowledge of patent licenses
         that are reasonably necessary to implement that API, you must also
         include this information in the LEGAL file.

         (( iii. ))??
         Representations. You represent that, except as disclosed pursuant to
         4(c)(i) above, you believe that any Modifications you distribute are
         your original creations and that you have sufficient rights to grant
         the rights conveyed by this License.

      (( d. ))??
      Required Notices. You must duplicate this License in any documentation you
      provide along with the Source Code of any Modifications you create or to
      which you contribute, and which you distribute, wherever you describe
      recipients' rights relating to Licensed Product. You must duplicate the
      notice contained in Exhibit A (the "Notice") in each file of the Source
      Code of any copy you distribute of the Licensed Product. If you created a
      Modification, you may add your name as a Contributor to the Notice. If it
      is not possible to put the Notice in a particular Source Code file due to
      its structure, then you must include such Notice in a location (such as a
      relevant directory file) where a user would be likely to look for such a
      notice. You may choose to offer, and charge a fee for, warranty, support,
      indemnity or liability obligations to one or more recipients of Licensed
      Product. However, you may do so only on your own behalf, and not on behalf
      of the Licensor or any Contributor. You must make it clear that any such
      warranty, support, indemnity or liability obligation is offered by you
      alone, and you hereby agree to indemnify the Licensor and every
      Contributor for any liability incurred by the Licensor or such Contributor
      as a result of warranty, support, indemnity or liability terms you offer.

      (( e. ))??
      Distribution of Executable Versions. You may distribute Licensed Product
      as an executable program under a license of your choice that may contain
      terms different from this License provided (i) you have satisfied the
      requirements of Sections 4(a) through 4(d) for that distribution, (ii) you
      include a conspicuous notice in the executable version, related
      documentation and collateral materials stating that the Source Code
      version of the Licensed Product is available under the terms of this
      License, including a description of how and where you have fulfilled the
      obligations of Section 4(b), and (iii) you make it clear that any terms
      that differ from this License are offered by you alone, not by Licensor or
      any Contributor. You hereby agree to indemnify the Licensor and every
      Contributor for any liability incurred by Licensor or such Contributor as
      a result of any terms you offer.

      (( f. ))??
      Distribution of Derivative Works. You may create Derivative Works (e.g.,
      combinations of some or all of the Licensed Product with other code) and
      distribute the Derivative Works as products under any other license you
      select, with the proviso that the requirements of this License are
      fulfilled for those portions of the Derivative Works that consist of the
      Licensed Product or any Modifications thereto.

   (( 5. ))??
   Inability to Comply Due to Statute or Regulation. If it is impossible for you
   to comply with any of the terms of this License with respect to some or all
   of the Licensed Product due to statute, judicial order, or regulation, then
   you must (i) comply with the terms of this License to the maximum extent
   possible, (ii) cite the statute or regulation that prohibits you from
   adhering to the License, and (iii) describe the limitations and the code they
   affect. Such description must be included in the LEGAL file described in
   Section 4(d), and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill at computer
   programming to be able to understand it.

   (( 6. ))??
   Application of This License. This License applies to code to which Licensor
   or Contributor has attached the Notice in Exhibit A, which is incorporated
   herein by this reference.

   (( 7. ))??
   Versions of This License.

      (( a. ))??
      New Versions. Licensor may publish from time to time revised and/or new
      versions of the License.

      (( b. ))??
      Effect of New Versions. Once Licensed Product has been published under a
      particular version of the License, you may always continue to use it under
      the terms of that version. You may also choose to use such Licensed
      Product under the terms of any subsequent version of the License published
      by Licensor. No one other than Licensor has the right to modify the terms
      applicable to Licensed Product created under this License.

      (( c. ))??
      Derivative Works of this License. If you create or use a modified version
      of this License, which you may do only in order to apply it to software
      that is not already a Licensed Product under this License, you must rename
      your license so that it is not confusingly similar to this License, and
      must make it clear that your license contains terms that differ from this
      License. In so naming your license, you may not use any trademark of
      Licensor or any Contributor.

   (( 8. ))??
   Disclaimer of Warranty. LICENSED PRODUCT IS PROVIDED UNDER THIS LICENSE ON AN
   AS IS BASIS, WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
   INCLUDING, WITHOUT LIMITATION, WARRANTIES THAT THE LICENSED PRODUCT IS FREE
   OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE
   ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE LICENSED PRODUCT IS WITH
   YOU. SHOULD LICENSED PRODUCT PROVE DEFECTIVE IN ANY RESPECT, YOU (AND NOT THE
   LICENSOR OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY
   SERVICING, REPAIR OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN
   ESSENTIAL PART OF THIS LICENSE. NO USE OF LICENSED PRODUCT IS AUTHORIZED
   HEREUNDER EXCEPT UNDER THIS DISCLAIMER.

   (( 9. ))??
   Termination.

      (( a. ))??
      Automatic Termination Upon Breach. This license and the rights granted
      hereunder will terminate automatically if you fail to comply with the
      terms herein and fail to cure such breach within thirty (30) days of
      becoming aware of the breach. All sublicenses to the Licensed Product that
      are properly granted shall survive any termination of this license.
      Provisions that, by their nature, must remain in effect beyond the
      termination of this License, shall survive.

      (( b. ))??
      Termination Upon Assertion of Patent Infringement. If you initiate
      litigation by asserting a patent infringement claim (excluding declaratory
      judgment actions) against Licensor or a Contributor (Licensor or
      Contributor against whom you file such an action is referred to herein as
      Respondent) alleging that Licensed Product directly or indirectly
      infringes any patent, then any and all rights granted by such Respondent
      to you under Sections 1 or 2 of this License shall terminate prospectively
      upon sixty (60) days notice from Respondent (the "Notice Period") unless
      within that Notice Period you either agree in writing (i) to pay
      Respondent a mutually agreeable reasonably royalty for your past or future
      use of Licensed Product made by such Respondent, or (ii) withdraw your
      litigation claim with respect to Licensed Product against such Respondent.
      If within said Notice Period a reasonable royalty and payment arrangement
      are not mutually agreed upon in writing by the parties or the litigation
      claim is not withdrawn, the rights granted by Licensor to you under
      Sections 1 and 2 automatically terminate at the expiration of said Notice
      Period.

      (( c. ))??
      Reasonable Value of This License. If you assert a patent infringement
      claim against Respondent alleging that Licensed Product directly or
      indirectly infringes any patent where such claim is resolved (such as by
      license or settlement) prior to the initiation of patent infringement
      litigation, then the reasonable value of the licenses granted by said
      Respondent under Sections 1 and 2 shall be taken into account in
      determining the amount or value of any payment or license.

      (( d. ))??
      No Retroactive Effect of Termination. In the event of termination under
      Sections 9(a) or 9(b) above, all end user license agreements (excluding
      licenses to distributors and resellers) that have been validly granted by
      you or any distributor hereunder prior to termination shall survive
      termination.

   (( 10. ))??
   Limitation of Liability. UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY,
   WHETHER TORT (INCLUDING NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE
   LICENSOR, ANY CONTRIBUTOR, OR ANY DISTRIBUTOR OF LICENSED PRODUCT, OR ANY
   SUPPLIER OF ANY OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT,
   SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING,
   WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER
   FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES,
   EVEN IF SUCH PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH
   DAMAGES. THIS LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH
   OR PERSONAL INJURY

   RESULTING FROM SUCH PARTYS NEGLIGENCE TO THE EXTENT APPLICABLE LAW PROHIBITS
   SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR LIMITATION
   OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS EXCLUSION AND LIMITATION MAY
   NOT APPLY TO YOU.

   (( 11. ))??
   Responsibility for Claims. As between Licensor and Contributors, each party
   is responsible for claims and damages arising, directly or indirectly, out of
   its utilization of rights under this License. You agree to work with Licensor
   and Contributors to distribute such responsibility on an equitable basis.
   Nothing herein is intended or

   shall be deemed to constitute any admission of liability.

   (( 12. ))??
   U.S. Government End Users. The Licensed Product is a commercial item, as that
   term is defined in 48 C.F.R. 2.101 (Oct. 1995), consisting of commercial
   computer software and commercial computer software documentation, as such
   terms are used in 48 C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R.
   12.212 and 48 C.F.R. 227.7202-1 through 227.7202-4 (June 1995), all U.S.
   Government End Users acquire Licensed Product with only those rights set
   forth herein.

   (( 13. ))??
   Miscellaneous. This License represents the complete agreement concerning the
   subject matter hereof. If any provision of this License is held to be
   unenforceable, such provision shall be reformed only to the extent necessary
   to make it enforceable. This License shall be governed by California law
   provisions (except to the extent applicable law, if any, provides otherwise),
   excluding its conflict-of-law provisions. You expressly agree that any
   litigation relating to this license shall be subject to the jurisdiction of
   the Federal Courts of the Northern District of California or the Superior
   Court of the County of Santa Clara, California (as appropriate), with venue
   lying in Santa Clara County, California, with the losing party responsible
   for costs including, without limitation, court costs and reasonable attorneys
   fees and expenses. The application of the United Nations Convention on
   Contracts for the International Sale of Goods is expressly excluded. You and
   Licensor expressly waive any rights to a jury trial in any litigation
   concerning Licensed Product or this License. Any law or regulation that
   provides that the language of a contract shall be construed against the
   drafter shall not apply to this License.

   (( 14. ))??
   Definition of You in This License. You throughout this License, whether in
   upper or lower case, means an individual or a legal entity exercising rights
   under, and complying with all of the terms of, this License or a future
   version of this License issued under Section 7. For legal entities, you
   includes any entity that controls, is controlled by, or is under common
   control with you. For purposes of this definition, control means (i) the
   power, direct or indirect, to cause the direction or management of such
   entity, whether by contract or otherwise, or (ii) ownership of fifty percent
   (50%) or more of the outstanding shares, or (iii) beneficial ownership of
   such entity.

   (( 15. ))??
   Glossary. All defined terms in this License that are used in more than one
   Section of this License are repeated here, in alphabetical order, for the
   convenience of the reader. The Section of this License in which each defined
   term is first used is shown in parentheses.

   Contributor: Each person or entity who created or contributed to the creation
   of, and distributed, a Modification. (See Section 2)

   Derivative Works: That term as used in this License is defined under U.S.
   copyright law. (See Section 1(b)

   License: This BitTorrent Open Source License. (See first paragraph of
   License)

   Licensed Product: Any BitTorrent Product licensed pursuant to this License.
   The term "Licensed Product" includes all previous Modifications from any
   Contributor that you receive. (See first paragraph of License and Section 2)

   Licensor: BitTorrent, Inc. (See first paragraph of License)

   Modifications: Any additions to or deletions from the substance or structure
   of (i) a file containing Licensed Product, or (ii) any new file that contains
   any part of Licensed Product. (See Section 2)

   Notice: The notice contained in Exhibit A. (See Section 4(e)

   Source Code: The preferred form for making modifications to the Licensed
   Product, including all modules contained therein, plus any associated
   interface definition files, scripts used to control compilation and
   installation of an executable program, or a list of differential comparisons
   against the Source Code of the Licensed Product. (See

   Section 1(a)

   You: This term is defined in Section 14 of this License.
   (( EXHIBIT A

The Notice below must appear in each file of the Source Code of any copy you
distribute of the Licensed Product or any hereto. Contributors to any
Modifications may add their own copyright notices to identify their own
contributions.

License:

The contents of this file are subject to the BitTorrent Open Source License
Version 1.0 (the License). You may not copy or use this file, in either source
code or executable form, except in compliance with the License. You may obtain a
copy of the License at http:/www.bittorrent.com/license/.

Software distributed under the License is distributed on an AS IS basis, WITHOUT
WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License. ))??
`
const license_BitTorrent_1_1_lre = `//**
BitTorrent Open Source License v1.1
https://spdx.org/licenses/BitTorrent-1.1.json
http://directory.fsf.org/wiki/License:BitTorrentOSL1.1
**//

(( BitTorrent Open Source License

Version 1.1 ))??

This BitTorrent Open Source License (the "License") applies to the BitTorrent
client and related software products as well as any updates or maintenance
releases of that software ("BitTorrent Products") that are distributed by
BitTorrent, Inc. ("Licensor"). Any BitTorrent Product licensed pursuant to this
License is a Licensed Product. Licensed Product, in its entirety, is protected
by U.S. copyright law. This License identifies the terms under which you may
use, copy, distribute or modify Licensed Product.

Preamble

This Preamble is intended to describe, in plain English, the nature and scope of
this License. However, this Preamble is not a part of this license. The legal
effect of this License is dependent only upon the terms of the License and not
this Preamble.

This License complies with the Open Source Definition and is derived from the
Jabber Open Source License 1.0 (the "JOSL"), which has been approved by Open
Source Initiative. Sections 4(c) and 4(f)(iii) from the JOSL have been deleted.

This License provides that:

   (( 1. ))??
   You may use or give away the Licensed Product, alone or as a component of an
   aggregate software distribution containing programs from several different
   sources. No royalty or other fee is required.

   (( 2. ))??
   Both Source Code and executable versions of the Licensed Product, including
   Modifications made by previous Contributors, are available for your use. (The
   terms "Licensed Product," "Modifications," "Contributors" and "Source Code"
   are defined in the License.)

   (( 3. ))??
   You are allowed to make Modifications to the Licensed Product, and you can
   create Derivative Works from it. (The term "Derivative Works" is defined in
   the License.)

   (( 4. ))??
   By accepting the Licensed Product under the provisions of this License, you
   agree that any Modifications you make to the Licensed Product and then
   distribute are governed by the provisions of this License. In particular, you
   must make the Source Code of your Modifications available to others free of
   charge and without a royalty.

   (( 5. ))??
   You may sell, accept donations or otherwise receive compensation for
   executable versions of a Licensed Product, without paying a royalty or other
   fee to the Licensor or any Contributor, provided that such executable
   versions contain your or another Contributor's material Modifications. For
   the avoidance of doubt, to the extent your executable version of a Licensed
   Product does not contain your or another Contributor's material
   Modifications, you may not sell, accept donations or otherwise receive
   compensation for such executable.

   You may use the Licensed Product for any purpose, but the Licensor is not
   providing you any warranty whatsoever, nor is the Licensor accepting any
   liability in the event that the Licensed Product doesn't work properly or
   causes you any injury or damages.

   (( 6. ))??
   If you sublicense the Licensed Product or Derivative Works, you may charge
   fees for warranty or support, or for accepting indemnity or liability
   obligations to your customers. You cannot charge for, sell, accept donations
   or otherwise receive compensation for the Source Code.

   (( 7. ))??
   If you assert any patent claims against the Licensor relating to the Licensed
   Product, or if you breach any terms of the License, your rights to the
   Licensed Product under this License automatically terminate.

You may use this License to distribute your own Derivative Works, in which case
the provisions of this License will apply to your Derivative Works just as they
do to the original Licensed Product.

Alternatively, you may distribute your Derivative Works under any other
OSI-approved Open Source license, or under a proprietary license of your choice.
If you use any license other than this License, however, you must continue to
fulfill the requirements of this License (including the provisions relating to
publishing the Source Code) for those portions of your Derivative Works that
consist of the Licensed Product, including the files containing

Modifications.

New versions of this License may be published from time to time in connection
with new versions of a Licensed Product or otherwise. You may choose to continue
to use the license terms in this version of the License for the Licensed Product
that was originally licensed hereunder, however, the new versions of this
License will at all times apply to new versions of the Licensed Product released
by Licensor after the release of the new version of this License. Only the
Licensor has the right to change the License terms as they apply to the Licensed
Product.

This License relies on precise definitions for certain terms. Those terms are
defined when they are first used, and the definitions are repeated for your
convenience in a Glossary at the end of the License.

License Terms

   (( 1. ))??
   Grant of License From Licensor. Subject to the terms and conditions of this
   License, Licensor hereby grants you a world-wide, royalty-free, non-exclusive
   license, subject to third party intellectual property claims, to do the
   following:

      (( a. ))??
      Use, reproduce, modify, display, perform, sublicense and distribute any
      Modifications created by a Contributor or portions thereof, in both Source
      Code or as an executable program, either on an unmodified basis or as part
      of Derivative Works.

      (( b. ))??
      Under claims of patents now or hereafter owned or controlled by
      Contributor, to make, use, sell, offer for sale, have made, and/or
      otherwise dispose of Modifications or portions thereof, but solely to the
      extent that any such claim is necessary to enable you to make, use, sell,
      offer for sale, have made, and/or otherwise dispose of Modifications or
      portions thereof or Derivative Works thereof.

   (( 2. ))??
   Grant of License to Modifications From Contributor. "Modifications" means any
   additions to or deletions from the substance or structure of (i) a file
   containing a Licensed Product, or (ii) any new file that contains any part of
   a Licensed Product. Hereinafter in this License, the term "Licensed Product"
   shall include all previous Modifications that you receive from any
   Contributor. Subject to the terms and conditions of this License, By
   application of the provisions in Section 4(a) below, each person or entity
   who created or contributed to the creation of, and distributed, a
   Modification (a "Contributor") hereby grants you a world-wide, royalty-free,
   non-exclusive license, subject to third party intellectual property claims,
   to do the following:

      (( a. ))??
      Use, reproduce, modify, display, perform, sublicense and distribute any
      Modifications created by such Contributor or portions thereof, in both
      Source Code or as an executable program, either on an unmodified basis or
      as part of Derivative Works.

      (( b. ))??
      Under claims of patents now or hereafter owned or controlled by
      Contributor, to make, use, sell, offer for sale, have made, and/or
      otherwise dispose of Modifications or portions thereof, but solely to the
      extent that any such claim is necessary to enable you to make, use, sell,
      offer for sale, have made, and/or otherwise dispose of Modifications or
      portions thereof or Derivative Works thereof.

   (( 3. ))??
   Exclusions From License Grant. Nothing in this License shall be deemed to
   grant any rights to trademarks, copyrights, patents, trade secrets or any
   other intellectual property of Licensor or any Contributor except as
   expressly stated herein. No patent license is granted separate from the
   Licensed Product, for code that you delete from the Licensed Product, or for
   combinations of the Licensed Product with other software or hardware. No
   right is granted to the trademarks of Licensor or any Contributor even if
   such marks are included in the Licensed Product. Nothing in this License
   shall be interpreted to prohibit Licensor from licensing under different
   terms from this License any code that Licensor otherwise would have a right
   to license. As an express condition for your use of the Licensed Product, you
   hereby agree that you will not, without the prior written consent of
   Licensor, use any trademarks, copyrights, patents, trade secrets or any other
   intellectual property of Licensor or any Contributor except as expressly
   stated herein. For the avoidance of doubt and without limiting the foregoing,
   you hereby agree that you will not use or display any trademark of Licensor
   or any Contributor in any domain name, directory filepath, advertisement,
   link or other reference to you in any manner or in any media.

   (( 4. ))??
   Your Obligations Regarding Distribution.

      (( a. ))??
      Application of This License to Your Modifications. As an express condition
      for your use of the Licensed Product, you hereby agree that any
      Modifications that you create or to which you contribute, and which you
      distribute, are governed by the terms of this License including, without
      limitation, Section 2. Any Modifications that you create or to which you
      contribute may be distributed only under the terms of this License or a
      future version of this License released under Section 7. You must include
      a copy of this License with every copy of the Modifications you
      distribute. You agree not to offer or impose any terms on any Source Code
      or executable version of the Licensed Product or Modifications that alter
      or restrict the applicable version of this License or the recipients'
      rights hereunder. However, you may include an additional document offering
      the additional rights described in Section 4(d).

      (( b. ))??
      Availability of Source Code. You must make available, without charge,
      under the terms of this License, the Source Code of the Licensed Product
      and any Modifications that you distribute, either on the same media as you
      distribute any executable or other form of the Licensed Product, or via a
      mechanism generally accepted in the software development community for the
      electronic transfer of data (an "Electronic Distribution Mechanism"). The
      Source Code for any version of Licensed Product or Modifications that you
      distribute must remain available for as long as any executable or other
      form of the Licensed Product is distributed by you. You are responsible
      for ensuring that the Source Code version remains available even if the
      Electronic Distribution Mechanism is maintained by a third party.

      (( c. ))??
      Intellectual Property Matters.

         (( i. ))??
         Third Party Claims. If you have knowledge that a license to a third
         party's intellectual property right is required to exercise the rights
         granted by this License, you must include a text file with the Source
         Code distribution titled "LEGAL" that describes the claim and the party
         making the claim in sufficient detail that a recipient will know whom
         to contact. If you obtain such knowledge after you make any
         Modifications available as described in Section 4(b), you shall
         promptly modify the LEGAL file in all copies you make available
         thereafter and shall take other steps (such as notifying appropriate
         mailing lists or newsgroups) reasonably calculated to inform those who
         received the Licensed Product from you that new knowledge has been
         obtained.

         (( ii. ))??
         Contributor APIs. If your Modifications include an application
         programming interface ("API") and you have knowledge of patent licenses
         that are reasonably necessary to implement that API, you must also
         include this information in the LEGAL file.

         (( iii. ))??
         Representations. You represent that, except as disclosed pursuant to
         4(c)(i) above, you believe that any Modifications you distribute are
         your original creations and that you have sufficient rights to grant
         the rights conveyed by this License.

      (( d. ))??
      Required Notices. You must duplicate this License in any documentation you
      provide along with the Source Code of any Modifications you create or to
      which you contribute, and which you distribute, wherever you describe
      recipients' rights relating to Licensed Product. You must duplicate the
      notice contained in Exhibit A (the "Notice") in each file of the Source
      Code of any copy you distribute of the Licensed Product. If you created a
      Modification, you may add your name as a Contributor to the Notice. If it
      is not possible to put the Notice in a particular Source Code file due to
      its structure, then you must include such Notice in a location (such as a
      relevant directory file) where a user would be likely to look for such a
      notice. You may choose to offer, and charge a fee for, warranty, support,
      indemnity or liability obligations to one or more recipients of Licensed
      Product. However, you may do so only on your own behalf, and not on behalf
      of the Licensor or any Contributor. You must make it clear that any such
      warranty, support, indemnity or liability obligation is offered by you
      alone, and you hereby agree to indemnify the Licensor and every
      Contributor for any liability incurred by the Licensor or such Contributor
      as a result of warranty, support, indemnity or liability terms you offer.

      (( e. ))??
      Distribution of Executable Versions. You may distribute Licensed Product
      as an executable program under a license of your choice that may contain
      terms different from this License provided (i) you have satisfied the
      requirements of Sections 4(a) through 4(d) for that distribution, (ii) you
      include a conspicuous notice in the executable version, related
      documentation and collateral materials stating that the Source Code
      version of the Licensed Product is available under the terms of this
      License, including a description of how and where you have fulfilled the
      obligations of Section 4(b), and (iii) you make it clear that any terms
      that differ from this License are offered by you alone, not by Licensor or
      any Contributor. You hereby agree to indemnify the Licensor and every
      Contributor for any liability incurred by Licensor or such Contributor as
      a result of any terms you offer.

      (( f. ))??
      Distribution of Derivative Works. You may create Derivative Works (e.g.,
      combinations of some or all of the Licensed Product with other code) and
      distribute the Derivative Works as products under any other license you
      select, with the proviso that the requirements of this License are
      fulfilled for those portions of the Derivative Works that consist of the
      Licensed Product or any Modifications thereto.

      (( g. ))??
      Compensation for Distribution of Executable Versions of Licensed Products,
      Modifications or Derivative Works. Notwithstanding any provision of this
      License to the contrary, by distributing, selling, licensing, sublicensing
      or otherwise making available any Licensed Product, or Modification or
      Derivative Work thereof, you and Licensor hereby acknowledge and agree
      that you may sell, license or sublicense for a fee, accept donations or
      otherwise receive compensation for executable versions of a Licensed
      Product, without paying a royalty or other fee to the Licensor or any
      other Contributor, provided that such executable versions (i) contain your
      or another Contributor's material Modifications, or (ii) are otherwise
      material Derivative Works. For purposes of this License, an executable
      version of the Licensed Product will be deemed to contain a material
      Modification, or will otherwise be deemed a material Derivative Work, if
      (a) the Licensed Product is modified with your own or a third party's
      software programs or other code, and/or the Licensed Product is combined
      with a number of your own or a third party's software programs or code,
      respectively, and (b) such software programs or code add or contribute
      material value, functionality or features to the License Product. For the
      avoidance of doubt, to the extent your executable version of a Licensed
      Product does not contain your or another Contributor's material
      Modifications or is otherwise not a material Derivative Work, in each case
      as contemplated herein, you may not sell, license or sublicense for a fee,
      accept donations or otherwise receive compensation for such executable.
      Additionally, without limitation of the foregoing and notwithstanding any
      provision of this License to the contrary, you cannot charge for, sell,
      license or sublicense for a fee, accept donations or otherwise receive
      compensation for the Source Code.

   (( 5. ))??
   Inability to Comply Due to Statute or Regulation. If it is impossible for you
   to comply with any of the terms of this License with respect to some or all
   of the Licensed Product due to statute, judicial order, or regulation, then
   you must (i) comply with the terms of this License to the maximum extent
   possible, (ii) cite the statute or regulation that prohibits you from
   adhering to the License, and (iii) describe the limitations and the code they
   affect. Such description must be included in the LEGAL file described in
   Section 4(d), and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill at computer
   programming to be able to understand it.

   (( 6. ))??
   Application of This License. This License applies to code to which Licensor
   or Contributor has attached the Notice in Exhibit A, which is incorporated
   herein by this reference.

   (( 7. ))??
   Versions of This License.

      (( a. ))??
      New Versions. Licensor may publish from time to time revised and/or new
      versions of the License.

      (( b. ))??
      Effect of New Versions. Once Licensed Product has been published under a
      particular version of the License, you may always continue to use it under
      the terms of that version, provided that any such license be in full force
      and effect at the time, and has not been revoked or otherwise terminated.
      You may also choose to use such Licensed Product under the terms of any
      subsequent version (but not any prior version) of the License published by
      Licensor. No one other than Licensor has the right to modify the terms
      applicable to Licensed Product created under this License.

      (( c. ))??
      Derivative Works of this License. If you create or use a modified version
      of this License, which you may do only in order to apply it to software
      that is not already a Licensed Product under this License, you must rename
      your license so that it is not confusingly similar to this License, and
      must make it clear that your license contains terms that differ from this
      License. In so naming your license, you may not use any trademark of
      Licensor or any Contributor.

   (( 8. ))??
   Disclaimer of Warranty. LICENSED PRODUCT IS PROVIDED UNDER THIS LICENSE ON AN
   AS IS BASIS, WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
   INCLUDING, WITHOUT LIMITATION, WARRANTIES THAT THE LICENSED PRODUCT IS FREE
   OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE
   ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE LICENSED PRODUCT IS WITH
   YOU. SHOULD LICENSED PRODUCT PROVE DEFECTIVE IN ANY RESPECT, YOU (AND NOT THE
   LICENSOR OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY
   SERVICING, REPAIR OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN
   ESSENTIAL PART OF THIS LICENSE. NO USE OF LICENSED PRODUCT IS AUTHORIZED
   HEREUNDER EXCEPT UNDER THIS DISCLAIMER.

   (( 9. ))??
   Termination.

      (( a. ))??
      Automatic Termination Upon Breach. This license and the rights granted
      hereunder will terminate automatically if you fail to comply with the
      terms herein and fail to cure such breach within ten (10) days of being
      notified of the breach by the Licensor. For purposes of this provision,
      proof of delivery via email to the address listed in the 'WHOIS' database
      of the registrar for any website through which you distribute or market
      any Licensed Product, or to any alternate email address which you
      designate in writing to the Licensor, shall constitute sufficient
      notification. All sublicenses to the Licensed Product that are properly
      granted shall survive any termination of this license so long as they
      continue to complye with the terms of this License. Provisions that, by
      their nature, must remain in effect beyond the termination of this
      License, shall survive.

      (( b. ))??
      Termination Upon Assertion of Patent Infringement. If you initiate
      litigation by asserting a patent infringement claim (excluding declaratory
      judgment actions) against Licensor or a Contributor (Licensor or
      Contributor against whom you file such an action is referred to herein as
      Respondent) alleging that Licensed Product directly or indirectly
      infringes any patent, then any and all rights granted by such Respondent
      to you under Sections 1 or 2 of this License shall terminate prospectively
      upon sixty (60) days notice from Respondent (the "Notice Period") unless
      within that Notice Period you either agree in writing (i) to pay
      Respondent a mutually agreeable reasonably royalty for your past or future
      use of Licensed Product made by such Respondent, or (ii) withdraw your
      litigation claim with respect to Licensed Product against such Respondent.
      If within said Notice Period a reasonable royalty and payment arrangement
      are not mutually agreed upon in writing by the parties or the litigation
      claim is not withdrawn, the rights granted by Licensor to you under
      Sections 1 and 2 automatically terminate at the expiration of said Notice
      Period.

      (( c. ))??
      Reasonable Value of This License. If you assert a patent infringement
      claim against Respondent alleging that Licensed Product directly or
      indirectly infringes any patent where such claim is resolved (such as by
      license or settlement) prior to the initiation of patent infringement
      litigation, then the reasonable value of the licenses granted by said
      Respondent under Sections 1 and 2 shall be taken into account in
      determining the amount or value of any payment or license.

      (( d. ))??
      No Retroactive Effect of Termination. In the event of termination under
      Sections 9(a) or 9(b) above, all end user license agreements (excluding
      licenses to distributors and resellers) that have been validly granted by
      you or any distributor hereunder prior to termination shall survive
      termination.

   (( 10. ))??
   Limitation of Liability. UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY,
   WHETHER TORT (INCLUDING NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE
   LICENSOR, ANY CONTRIBUTOR, OR ANY DISTRIBUTOR OF LICENSED PRODUCT, OR ANY
   SUPPLIER OF ANY OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT,
   SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING,
   WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER
   FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES,
   EVEN IF SUCH PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH
   DAMAGES. THIS LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH
   OR PERSONAL INJURY RESULTING FROM SUCH PARTYS NEGLIGENCE TO THE EXTENT
   APPLICABLE LAW PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE
   EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS
   EXCLUSION AND LIMITATION MAY NOT APPLY TO YOU.

   (( 11. ))??
   Responsibility for Claims. As between Licensor and Contributors, each party
   is responsible for claims and damages arising, directly or indirectly, out of
   its utilization of rights under this License. You agree to work with Licensor
   and Contributors to distribute such responsibility on an equitable basis.
   Nothing herein is intended or shall be deemed to constitute any admission of
   liability.

   (( 12. ))??
   U.S. Government End Users. The Licensed Product is a commercial item, as that
   term is defined in 48 C.F.R. 2.101 (Oct. 1995), consisting of commercial
   computer software and commercial computer software documentation, as such
   terms are used in 48 C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R.
   12.212 and 48 C.F.R. 227.7202-1 through 227.7202-4 (June 1995), all U.S.
   Government End Users acquire Licensed Product with only those rights set
   forth herein.

   (( 13. ))??
   Miscellaneous. This License represents the complete agreement concerning the
   subject matter hereof. If any provision of this License is held to be
   unenforceable, such provision shall be reformed only to the extent necessary
   to make it enforceable. This License shall be governed by California law
   provisions (except to the extent applicable law, if any, provides otherwise),
   excluding its conflict-of-law provisions. You expressly agree that in any
   litigation relating to this license the losing party shall be responsible for
   costs including, without limitation, court costs and reasonable attorneys
   fees and expenses. The application of the United Nations Convention on
   Contracts for the International Sale of Goods is expressly excluded. Any law
   or regulation that provides that the language of a contract shall be
   construed against the drafter shall not apply to this License.

   (( 14. ))??
   Definition of You in This License. You throughout this License, whether in
   upper or lower case, means an individual or a legal entity exercising rights
   under, and complying with all of the terms of, this License or a future
   version of this License issued under Section 7. For legal entities, you
   includes any entity that controls, is controlled by, is under common control
   with, or affiliated with, you. For purposes of this definition, control means
   (i) the power, direct or indirect, to cause the direction or management of
   such entity, whether by contract or otherwise, or (ii) ownership of fifty
   percent (50%) or more of the outstanding shares, or (iii) beneficial
   ownership of such entity. You are responsible for advising any affiliated
   entity of the terms of this License, and that any rights or privileges
   derived from or obtained by way of this License are subject to the
   restrictions outlined herein.

   (( 15. ))??
   Glossary. All defined terms in this License that are used in more than one
   Section of this License are repeated here, in alphabetical order, for the
   convenience of the reader. The Section of this License in which each defined
   term is first used is shown in parentheses.

   Contributor: Each person or entity who created or contributed to the creation
   of, and distributed, a Modification. (See Section 2)

   Derivative Works: That term as used in this License is defined under U.S.
   copyright law. (See Section 1(b)

   License: This BitTorrent Open Source License. (See first paragraph of
   License)

   Licensed Product: Any BitTorrent Product licensed pursuant to this License.
   The term "Licensed Product" includes all previous Modifications from any
   Contributor that you receive. (See first paragraph of License and Section 2)

   Licensor: BitTorrent, Inc. (See first paragraph of License)

   Modifications: Any additions to or deletions from the substance or structure
   of (i) a file containing Licensed

   Product, or (ii) any new file that contains any part of Licensed Product.
   (See Section 2)

   Notice: The notice contained in Exhibit A. (See Section 4(e)

   Source Code: The preferred form for making modifications to the Licensed
   Product, including all modules contained therein, plus any associated
   interface definition files, scripts used to control compilation and
   installation of an executable program, or a list of differential comparisons
   against the Source Code of the Licensed Product. (See Section 1(a)

   You: This term is defined in Section 14 of this License.
   (( EXHIBIT A

The Notice below must appear in each file of the Source Code of any copy you
distribute of the Licensed Product or any hereto. Contributors to any
Modifications may add their own copyright notices to identify their own
contributions.

License:

The contents of this file are subject to the BitTorrent Open Source License
Version
//** 1.1 **//
__5__
(the License). You may not copy or use this file, in either source code or
executable form, except in compliance with the License. You may obtain a copy of
the License at http:/www.bittorrent.com/license/.

Software distributed under the License is distributed on an AS IS basis, WITHOUT
WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

BitTorrent, Inc. ))??
`
const license_BlueOak_1_0_0_lre = `//**
Blue Oak Model License 1.0.0
https://spdx.org/licenses/BlueOak-1.0.0.json
https://blueoakcouncil.org/license/1.0.0
**//

(( Blue Oak Model License ))??

Version 1.0.0

Purpose

This license gives everyone as much permission to work with this software as
possible, while protecting contributors from liability.

Acceptance

In order to receive this license, you must agree to its rules. The rules of this
license are both obligations under that agreement and conditions to your
license. You must not do anything with this software that triggers a rule that
you cannot or will not follow.

Copyright

Each contributor licenses you to do everything with this software that would
otherwise infringe that contributor's copyright in it.

Notices

You must ensure that everyone who gets a copy of any part of this software from
you, with or without changes, also gets the text of this license or a link to
<https:/blueoakcouncil.org/license/1.0.0>.

Excuse

If anyone notifies you in writing that you have not complied with
//** [Notices](#notices) **//
__5__
, you can keep your license by taking all practical steps to comply within 30
days after the notice. If you do not do so, your license ends immediately.

Patent

Each contributor licenses you to do everything with this software that would
otherwise infringe any patent claims they can license or become able to
license.

Reliability

No contributor can revoke this license.

No Liability

***As far as the law allows, this software comes as is, without any warranty or
condition, and no contributor will be liable to anyone for any damages related
to this software or this license, under any kind of legal claim.***
`
const license_Borceux_lre = `//**
Borceux license
https://spdx.org/licenses/Borceux.json
https://fedoraproject.org/wiki/Licensing/Borceux
**//

//** Copyright **//

You may freely use, modify, and/or distribute each of the files in this package
without limitation. The package consists of the following files:

README

compatibility/OldDiagram

compatibility/OldMaxiDiagram

compatibility/OldMicroDiagram

compatibility/OldMiniDiagram

compatibility/OldMultipleArrows

diagram/Diagram

diagram/MaxiDiagram

diagram/MicroDiagram

diagram/MiniDiagram

diagram/MultipleArrows

user-guides/Diagram_Mode_d_Emploi

user-guides/Diagram_Read_Me

Of course no support is guaranteed, but the author will attempt to assist with
problems. Current email address:

francis dot borceux at uclouvain dot be.
`
const license_CAL_1_0_lre = `//**
Cryptographic Autonomy License 1.0
https://spdx.org/licenses/CAL-1.0.json
http://cryptographicautonomylicense.com/license-text.html
https://opensource.org/licenses/CAL-1.0
**//

(( The Cryptographic Autonomy License, v. 1.0 ))??

(( This ))??
Cryptographic Autonomy License (the "License") applies to any Work whose owner
has marked it with any of the following notices, or a similar demonstration of
intent:

SPDX-License-Identifier: CAL-1.0

Licensed under the Cryptographic Autonomy License version 1.0

((or))??

SPDX-License-Identifier: CAL-1.0-Combined-Work-Exception

Licensed under the Cryptographic Autonomy License version 1.0, with Combined
Work Exception

   (( 1. ))??
   Purpose

   This License gives You unlimited permission to use and modify the software to
   which it applies (the "Work"), either as-is or in modified form, for Your
   private purposes, while protecting the owners and contributors to the
   software from liability.

   This License also strives to protect the freedom and autonomy of third
   parties who receive the Work from you. If any non-affiliated third party
   receives any part, aspect, or element of the Work from You, this License
   requires that You provide that third party all the permissions and materials
   needed to independently use and modify the Work without that third party
   having a loss of data or capability due to your actions.

   The full permissions, conditions, and other terms are laid out below.

   (( 2. ))??
   Receiving a License

   In order to receive this License, You must agree to its rules. The rules of
   this License are both obligations of Your agreement with the Licensor and
   conditions to your License. You must not do anything with the Work that
   triggers a rule You cannot or will not follow.

      (( 2.1. ))??
      Application

      The terms of this License apply to the Work as you receive it from
      Licensor, as well as to any modifications, elaborations, or
      implementations created by You that contain any licensable portion of the
      Work (a "Modified Work"). Unless specified, any reference to the Work also
      applies to a Modified Work.

      (( 2.2. ))??
      Offer and Acceptance

      This License is automatically offered to every person and organization.
      You show that you accept this License and agree to its conditions by
      taking any action with the Work that, absent this License, would infringe
      any intellectual property right held by Licensor.

      (( 2.3. ))??
      Compliance and Remedies

      Any failure to act according to the terms and conditions of this License
      places Your use of the Work outside the scope of the License and infringes
      the intellectual property rights of the Licensor. In the event of
      infringement, the terms and conditions of this License may be enforced by
      Licensor under the intellectual property laws of any jurisdiction to which
      You are subject. You also agree that either the Licensor or a Recipient
      (as an intended third-party beneficiary) may enforce the terms and
      conditions of this License against You via specific performance.

   (( 3. ))??
   Permissions

      (( 3.1. ))??
      Permissions Granted

      Conditioned on compliance with section 4, and subject to the limitations
      of section 3.2, Licensor grants You the world-wide, royalty-free,
      non-exclusive permission to:

         (( a) ))??
         Take any action with the Workthat would infringe the non-patent
         intellectualproperty laws of any jurisdiction to which You aresubject;
         and

         (( b) ))??
         claims that Licensor can licenseor becomes able to license, to the
         extent that thoseclaims are embodied in the Work as distributed
         byLicensor.

      (( 3.2. ))??
      Limitations on Permissions Granted

      The following limitations apply to the permissions granted in section
      3.1:

         (( a) ))??
         Licensor does not grant anypatent license for claims that are only
         infringed dueto modification of the Work as provided by Licensor,or the
         combination of the Work as provided byLicensor, directly or indirectly,
         with any othercomponent, including other software or hardware.

         (( b) ))??
         Licensor does not grant anylicense to the trademarks, service marks, or
         logos ofLicensor, except to the extent necessary to complywith the
         attribution conditions in section 4.1 ofthis License.

   (( 4. ))??
   Conditions

   If You exercise any permission granted by this License, such that the Work,
   or any part, aspect, or element of the Work, is distributed, communicated,
   made available, or made perceptible to a non-Affiliate third party (a
   "Recipient"), either via physical delivery or via a network connection to the
   Recipient, You must comply with the following conditions:

      (( 4.1. ))??
      Provide Access to Source Code

      Subject to the exception in section 4.4, You must provide to each
      Recipient a copy of, or no-charge unrestricted network access to, the
      Source Code corresponding to the Work ("Access").

      The "Source Code" of the Work means the form of the Work preferred for
      making modifications, including any comments, configuration information,
      documentation, help materials, installation instructions, cryptographic
      seeds or keys, and any information reasonably necessary for the Recipient
      to independently compile and use the Source Code and to have full access
      to the functionality contained in the Work.

         (( 4.1.1. ))??
         Providing Network Access to the Source Code

         Network Access to the Notices and Source Code may be provided by You or
         by a third party, such as a public software repository, and must
         persist during the same period in which You exercise any of the
         permissions granted to You under this License and for at least one year
         thereafter.

         (( 4.1.2. ))??
         Source Code for a Modified Work

         Subject to the exception in section 4.5, You must provide to each
         Recipient of a Modified Work Access to Source Code corresponding to
         those portions of the Work remaining in the Modified Work as well as
         the modifications used by You to create the Modified Work. The Source
         Code corresponding to the modifications in the Modified Work must be
         provided to the Recipient either a) under this License, or b) under a
         Compatible Open Source License.

         (( 4.1.3. ))??
         Coordinated Disclosure of Security Vulnerabilities

         You may delay providing the Source Code corresponding to a particular
         modification of the Work for up to ninety (90) days (the "Embargo
         Period") if:

            (( a) ))??
            the modification is intended toaddress a newly-identified
            vulnerability or asecurity flaw in the Work,

            (( b) ))??
            disclosure of the vulnerabilityor security flaw before the end of
            the Embargo Periodwould put the data, identity, or autonomy of one
            ormore Recipients of the Work at significant risk,

            (( c) ))??
            You are participating in acoordinated disclosure of the
            vulnerability orsecurity flaw with one or more additional
            Licensees,and

            (( d) ))??
            Access to the Source Codepertaining to the modification is provided
            to allRecipients at the end of the Embargo Period.

      (( 4.2. ))??
      Maintain User Autonomy

      In addition to providing each Recipient the opportunity to have Access to
      the Source Code, You cannot use the permissions given under this License
      to interfere with a Recipient's ability to fully use an independent copy
      of the Work generated from the Source Code You provide with the
      Recipient's own User Data.

      "User Data" means any data that is an input to or an output from the Work,
      where the presence of the data is necessary for substantially identical
      use of the Work in an equivalent context chosen by the Recipient, and
      where the Recipient has an existing ownership interest, an existing right
      to possess, or where the data has been generated by, for, or has been
      assigned to the Recipient.

         (( 4.2.1. ))??
         No Withholding User Data

         Throughout any period in which You exercise any of the permissions
         granted to You under this License, You must also provide to any
         Recipient to whom you provide services via the Work, a no-charge copy,
         provided in a commonly used electronic form, of the Recipient's User
         Data in your possession, to the extent that such User Data is available
         to You for use in conjunction with the Work.

         (( 4.2.2. ))??
         No Technical Measures that Limit Access

         You may not, by means of the use cryptographic methods applied to
         anything provided to the Recipient, by possession or control of
         cryptographic keys, seeds, hashes, by any other technological
         protection measures, or by any other method, limit a Recipient's
         ability to access any functionality present in Recipient's independent
         copy of the Work, or to deny a Recipient full control of the
         Recipient's User Data.

         (( 4.2.3. ))??
         No Legal or Contractual Measures that Limit Access

         You may not contractually restrict a Recipient's ability to
         independently exercise the permissions granted under this License. You
         waive any legal power to forbid circumvention of technical protection
         measures that include use of the Work, and You waive any claim that the
         capabilities of the Work were limited or modified as a means of
         enforcing the legal rights of third parties against Recipients.

      (( 4.3. ))??
      Provide Notices and Attribution

      You must retain all licensing, authorship, or attribution notices
      contained in the Source Code (the "Notices"), and provide all such Notices
      to each Recipient, together with a statement acknowledging the use of the
      Work. Notices may be provided directly to a Recipient or via an
      easy-to-find hyperlink to an Internet location also providing Access to
      Source Code.

      (( 4.4. ))??
      Scope of Conditions in this License

      You are required to uphold the conditions of this License only relative to
      those who are Recipients of the Work from You. Other than providing
      Recipients with the applicable Notices, Access to Source Code, and a copy
      of and full control of their User Data, nothing in this License requires
      You to provide processing services to or engage in network interactions
      with anyone.

      (( 4.5. ))??
      Combined Work Exception

      As an exception to condition that You provide Recipients Access to Source
      Code, any Source Code files marked by the Licensor as having the "Combined
      Work Exception," or any object code exclusively resulting from Source Code
      files so marked, may be combined with other Software into a "Larger Work."
      So long as you comply with the requirements to provide Recipients the
      applicable Notices and Access to the Source Code provided to You by
      Licensor, and you provide Recipients access to their User Data and do not
      limit Recipient's ability to independently work with their User Data, any
      other Software in the Larger Work as well as the Larger Work as a whole
      may be licensed under the terms of your choice.

   (( 5. ))??
   Term and Termination

   The term of this License begins when You receive the Work, and continues
   until terminated for any of the reasons described herein, or until all
   Licensor's intellectual property rights in the Software expire, whichever
   comes first ("Term"). This License cannot be revoked, only terminated for the
   reasons listed below.

      (( 5.1. ))??
      Effect of Termination

      If this License is terminated for any reason, all permissions granted to
      You under Section 3 by any Licensor automatically terminate. You will
      immediately cease exercising any permissions granted in this License
      relative to the Work, including as part of any Modified Work.

      (( 5.2. ))??
      Termination for Non-Compliance; Reinstatement

      This License terminates automatically if You fail to comply with any of
      the conditions in section 4. As a special exception to termination for
      non-compliance, Your permissions for the Work under this License will
      automatically be reinstated if You come into compliance with all the
      conditions in section 2 within sixty (60) days of being notified by
      Licensor or an intended third-party beneficiary of Your noncompliance. You
      are eligible for reinstatement of permissions for the Work one time only,
      and only for the sixty days immediately after becoming aware of
      noncompliance. Loss of permissions granted for the Work under this License
      due to either a) sustained noncompliance lasting more than sixty days or
      b) subsequent termination for noncompliance after reinstatement, is
      permanent, unless rights are specifically restored by Licensor in
      writing.

      (( 5.3. ))??
      Termination Due to Litigation

      If You initiate litigation against Licensor, or any Recipient of the Work,
      either direct or indirect, asserting that the Work directly or indirectly
      infringes any patent, then all permissions granted to You by this License
      shall terminate. In the event of termination due to litigation, all
      permissions validly granted by You under this License, directly or
      indirectly, shall survive termination. Administrative review procedures,
      declaratory judgment actions, counterclaims in response to patent
      litigation, and enforcement actions against former Licensees terminated
      under this section do not cause termination due to litigation.

   (( 6. ))??
   Disclaimer of Warranty and Limit on Liability

   As far as the law allows, the Work comes AS-IS, without any warranty of any
   kind, and no Licensor or contributor will be liable to anyone for any damages
   related to this software or this license, under any kind of legal claim, or
   for any type of damages, including indirect, special, incidental, or
   consequential damages of any type arising as a result of this License or the
   use of the Work including, without limitation, damages for loss of goodwill,
   work stoppage, computer failure or malfunction, loss of profits, revenue, or
   any and all other commercial damages or losses.

   (( 7. ))??
   Other Provisions

      (( 7.1. ))??
      Affiliates

      An "Affiliate" means any other entity that, directly or indirectly through
      one or more intermediaries, controls, is controlled by, or is under common
      control with, the Licensee. Employees of a Licensee and natural persons
      acting as contractors exclusively providing services to Licensee are also
      Affiliates.

      (( 7.2. ))??
      Choice of Jurisdiction and Governing Law

      A Licensor may require that any action or suit by a Licensee relating to a
      Work provided by Licensor under this License may be brought only in the
      courts of a particular jurisdiction and under the laws of a particular
      jurisdiction (excluding its conflict-of-law provisions), if Licensor
      provides conspicuous notice of the particular jurisdiction to all
      Licensees.

      (( 7.3. ))??
      No Sublicensing

      This License is not sublicensable. Each time You provide the Work or a
      Modified Work to a Recipient, the Recipient automatically receives a
      license under the terms described in this License. You may not impose any
      further reservations, conditions, or other provisions on any Recipients'
      exercise of the permissions granted herein.

      (( 7.4. ))??
      Attorneys' Fees

      In any action to enforce the terms of this License, or seeking damages
      relating thereto, including by an intended third-party beneficiary, the
      prevailing party shall be entitled to recover its costs and expenses,
      including, without limitation, reasonable attorneys' fees and costs
      incurred in connection with such action, including any appeal of such
      action. A "prevailing party" is the party that achieves, or avoids,
      compliance with this License, including through settlement. This section
      shall survive the termination of this License.

      (( 7.5. ))??
      No Waiver

      Any failure by Licensor to enforce any provision of this License will not
      constitute a present or future waiver of such provision nor limit
      Licensor's ability to enforce such provision at a later time.

      (( 7.6. ))??
      Severability

      If any provision of this License is held to be unenforceable, such
      provision shall be reformed only to the extent necessary to make it
      enforceable. Any invalid or unenforceable portion will be interpreted to
      the effect and intent of the original portion. If such a construction is
      not possible, the invalid or unenforceable portion will be severed from
      this License but the rest of this License will remain in full force and
      effect.

      (( 7.7. ))??
      License for the Text of this License

      The text of this license is released under the Creative Commons
      Attribution-ShareAlike 4.0 International License, with the caveat that any
      modifications of this license may not use the name "Cryptographic Autonomy
      License" or any name confusingly similar thereto to describe any derived
      work of this License.
`
const license_CATOSL_1_1_lre = `//**
Computer Associates Trusted Open Source License 1.1
https://spdx.org/licenses/CATOSL-1.1.json
https://opensource.org/licenses/CATOSL-1.1
**//

(( Computer Associates Trusted Open Source License

Version 1.1 ))??

PLEASE READ THIS DOCUMENT CAREFULLY AND IN ITS ENTIRETY. THE ACCOMPANYING
PROGRAM IS PROVIDED UNDER THE TERMS OF THIS COMPUTER ASSOCIATES TRUSTED OPEN
SOURCE LICENSE ("LICENSE"). ANY USE, REPRODUCTION, MODIFICATION OR DISTRIBUTION
OF THE PROGRAM CONSTITUTES THE RECIPIENT'S ACCEPTANCE OF THIS LICENSE.

License Background

Computer Associates International, Inc. (CA) believes in open source. We believe
that the open source development approach can take appropriate software programs
to unprecedented levels of quality, growth, and innovation. To demonstrate our
continuing commitment to open source, we are releasing the Program (as defined
below) under this License.

This License is intended to permit contributors and recipients of the Program to
use the Program, including its source code, freely and without many of the
concerns of some other open source licenses. Although we expect the underlying
Program, and Contributions (as defined below) made to such Program, to remain
open, this License is designed to permit you to maintain your own software
programs free of this License unless you choose to do so. Thus, only your
Contributions to the Program must be distributed under the terms of this
License.

The provisions that follow set forth the terms and conditions under which you
may use the Program.

   (( 1. ))??
   DEFINITIONS

      (( 1.1 ))??
      Contribution means (a) in the case of CA, the Original Program; and (b) in
      the case of each Contributor (including CA), changes and additions to the
      Program, where such changes and/or additions to the Program originate from
      and are distributed by that particular Contributor to unaffiliated third
      parties. A Contribution originates from a Contributor if it was added to
      the Program by such Contributor itself or anyone acting on such
      Contributors behalf. Contributions do not include additions to the Program
      which: (x) are separate modules of software distributed in conjunction
      with the Program under their own license agreement, and (y) are not
      derivative works of the Program.

      (( 1.2 ))??
      Contributor means CA and any other person or entity that distributes the
      Program.

      (( 1.3 ))??
      Contributor Version means as to a Contributor, that version of the Program
      that includes the Contributors Contribution but not any Contributions made
      to the Program thereafter.

      (( 1.4 ))??
      Larger Work means a work that combines the Program or portions thereof
      with code not governed by the terms of this License.

      (( 1.5 ))??
      Licensed Patents mean patents licensable by a Contributor that are
      infringed by the use or sale of its Contribution alone or when combined
      with the Program.

      (( 1.6 ))??
      Original Program means the original version of the software to which this
      License is attached and as released by CA, including source code, object
      code and documentation, if any.

      (( 1.7 ))??
      Program means the Original Program and Contributions.

      (( 1.8 ))??
      Recipient means anyone who modifies, copies, uses or distributes the
      Program.

   (( 2. ))??
   GRANT OF RIGHTS

      (( 2.1 ))??
      Subject to the terms of this License, each Contributor hereby grants
      Recipient an irrevocable, non-exclusive, worldwide, royalty-free license
      to reproduce, prepare derivative works of, publicly display, publicly
      perform, distribute and sublicense the Contribution of such Contributor,
      if any, and such derivative works, in source code and object code form.
      For the avoidance of doubt, the license provided in this Section 2.1 shall
      not include a license to any Licensed Patents of a Contributor.

      (( 2.2 ))??
      Subject to the terms of this License, each Contributor hereby grants
      Recipient an irrevocable, non-exclusive, worldwide, royalty-free license
      to the Licensed Patents to the extent necessary to make, use, sell, offer
      to sell and import the Contribution of such Contributor, if any, in source
      code and object code form. The license granted in this Section 2.2 shall
      apply to the combination of the Contribution and the Program if, at the
      time the Contribution is added by the Contributor, such addition of the
      Contribution causes the Licensed Patents to be infringed by such
      combination. Notwithstanding the foregoing, no license is granted under
      this Section 2.2: (a) for any code or works that do not include the
      Contributor Version, as it exists and is used in accordance with the terms
      hereof; (b) for infringements caused by: (i) third party modifications of
      the Contributor Version; or (ii) the combination of Contributions made by
      each such Contributor with other software (except as part of the
      Contributor Version) or other devices; or (c) with respect to Licensed
      Patents infringed by the Program in the absence of Contributions made by
      that Contributor.

      (( 2.3 ))??
      Recipient understands that although each Contributor grants the licenses
      to its Contributions set forth herein, except as provided in Section 2.4,
      no assurances are provided by any Contributor that the Program does not
      infringe the patent or other intellectual property rights of any other
      person or entity. Each Contributor disclaims any liability to Recipient
      for claims brought by any other person or entity based on infringement of
      intellectual property rights or otherwise. As a condition to exercising
      the rights and licenses granted hereunder, each Recipient hereby assumes
      sole responsibility to secure any other intellectual property rights
      needed, if any.

      (( 2.4 ))??
      Each Contributor represents and warrants that it has all right, title and
      interest in the copyrights in its Contributions, and has the right to
      grant the copyright licenses set forth in this License.

   (( 3. ))??
   DISTRIBUTION REQUIREMENTS

      (( 3.1 ))??
      If the Program is distributed in object code form, then a prominent notice
      must be included in the code itself as well as in any related
      documentation, stating that the source code for the Program is available
      from the Contributor with information on how and where to obtain the
      source code. A Contributor may choose to distribute the Program in object
      code form under its own license agreement, provided that:

         *
         it complies with the terms and conditions of this License; and

         *
         its license agreement:

         *
         effectively disclaims on behalf of all Contributors all warranties and
         conditions, express and implied, including warranties or conditions of
         title and non-infringement, and implied warranties or conditions of
         merchantability and fitness for a particular purpose, to the maximum
         extent permitted by applicable law;

         *
         effectively excludes on behalf of all Contributors all liability for
         damages, including direct, indirect, special, incidental and
         consequential damages, such as lost profits, to the maximum extent
         permitted by applicable law;

         *
         states that any provisions which are inconsistent with this License are
         offered by that Contributor alone and not by any other party; and

         *
         states that source code for the Program is available from such
         Contributor at the cost of distribution, and informs licensees how to
         obtain it in a reasonable manner.

      (( 3.2 ))??
      When the Program is made available in source code form:

         *
         it must be made available under this License; and

         *
         a copy of this License must be included with each copy of the Program.

      (( 3.3 ))??
      This License is intended to facilitate the commercial distribution of the
      Program by any Contributor. However, Contributors may only charge
      Recipients a one-time, upfront fee for the distribution of the Program.
      Contributors may not charge Recipients any recurring charge, license fee,
      or any ongoing royalty for the Recipients exercise of its rights under
      this License to the Program. Contributors shall make the source code for
      the Contributor Version they distribute available at a cost, if any, equal
      to the cost to the Contributor to physically copy and distribute the work.
      It is not the intent of this License to prohibit a Contributor from
      charging fees for any service or maintenance that a Contributor may charge
      to a Recipient, so long as such fees are not an attempt to circumvent the
      foregoing restrictions on charging royalties or other recurring fees for
      the Program itself.

      (( 3.4 ))??
      A Contributor may create a Larger Work by combining the Program with other
      software code not governed by the terms of this License, and distribute
      the Larger Work as a single product. In such a case, the Contributor must
      make sure that the requirements of this License are fulfilled for the
      Program. Any Contributor who includes the Program in a commercial product
      offering, including as part of a Larger Work, may subject itself, but not
      any other Contributor, to additional contractual commitments, including,
      but not limited to, performance warranties and non-infringement
      representations on suchContributors behalf. No Contributor may create any
      additional liability for other Contributors. Therefore, if a Contributor
      includes the Program in a commercial product offering, such Contributor
      (Commercial Contributor) hereby agrees to defend and indemnify every other
      Contributor (Indemnified Contributor) who made Contributions to the
      Program distributed by the Commercial Contributor against any losses,
      damages and costs (collectively Losses) arising from claims, lawsuits and
      other legal actions brought by a third party against the Indemnified
      Contributor to the extent caused by the acts or omissions, including any
      additional contractual commitments, of such Commercial Contributor in
      connection with its distribution of the Program. The obligations in this
      section do not apply to any claims or Losses relating to any actual or
      alleged intellectual property infringement.

      (( 3.5 ))??
      If Contributor has knowledge that a license under a third partys
      intellectual property rights is required to exercise the rights granted by
      such Contributor under Sections 2.1 or 2.2, Contributor must (a) include a
      text file with the Program source code distribution titled ../IP_ISSUES,
      and (b) notify CA in writing at Computer Associates International, Inc.,
      One Computer Associates Plaza, Islandia, New York 11749, Attn: Open Source
      Group or by email at opensource@ca.com, both describing the claim and the
      party making the claim in sufficient detail that a Recipient and CA will
      know whom to contact with regard to such matter. If Contributor obtains
      such knowledge after the Contribution is made available, Contributor shall
      also promptly modify the IP_ISSUES file in all copies Contributor makes
      available thereafter and shall take other steps (such as notifying
      appropriate mailing lists or newsgroups) reasonably calculated to inform
      those who received the Program that such new knowledge has been obtained.

      (( 3.6 ))??
      Recipient shall not remove, obscure, or modify any CA or other Contributor
      copyright or patent proprietary notices appearing in the Program, whether
      in the source code, object code or in any documentation. In addition to
      the obligations set forth in Section 4, each Contributor must identify
      itself as the originator of its Contribution, if any, in a manner that
      reasonably allows subsequent Recipients to identify the originator of the
      Contribution.

   (( 4. ))??
   CONTRIBUTION RESTRICTIONS

      (( 4.1 ))??
      Each Contributor must cause the Program to which the Contributor provides
      a Contribution to contain a file documenting the changes the Contributor
      made to create its version of the Program and the date of any change. Each
      Contributor must also include a prominent statement that the Contribution
      is derived, directly or indirectly, from the Program distributed by a
      prior Contributor, including the name of the prior Contributor from which
      such Contribution was derived, in (a) the Program source code, and (b) in
      any notice in an executable version or related documentation in which the
      Contributor describes the origin or ownership of the Program.

   (( 5. ))??
   NO WARRANTY

      (( 5.1 ))??
      EXCEPT AS EXPRESSLY SET FORTH IN THIS LICENSE, THE PROGRAM IS PROVIDED AS
      IS AND IN ITS PRESENT STATE AND CONDITION. NO WARRANTY, REPRESENTATION,
      CONDITION, UNDERTAKING OR TERM, EXPRESS OR IMPLIED, STATUTORY OR
      OTHERWISE, AS TO THE CONDITION, QUALITY, DURABILITY, PERFORMANCE,
      NON-INFRINGEMENT, MERCHANTABILITY, OR FITNESS FOR A PARTICULAR PURPOSE OR
      USE OF THE PROGRAM IS GIVEN OR ASSUMED BY ANY CONTRIBUTOR AND ALL SUCH
      WARRANTIES, REPRESENTATIONS, CONDITIONS, UNDERTAKINGS AND TERMS ARE HEREBY
      EXCLUDED TO THE FULLEST EXTENT PERMITTED BY LAW.

      (( 5.2 ))??
      Each Recipient is solely responsible for determining the appropriateness
      of using and distributing the Program and assumes all risks associated
      with its exercise of rights under this License, including but not limited
      to the risks and costs of program errors, compliance with applicable laws,
      damage to or loss of data, programs or equipment, and unavailability or
      interruption of operations.

      (( 5.3 ))??
      Each Recipient acknowledges that the Program is not intended for use in
      the operation of nuclear facilities, aircraft navigation, communication
      systems, or air traffic control machines in which case the failure of the
      Program could lead to death, personal injury, or severe physical or
      environmental damage.

   (( 6. ))??
   DISCLAIMER OF LIABILITY

      (( 6.1 ))??
      EXCEPT AS EXPRESSLY SET FORTH IN THIS LICENSE, AND TO THE EXTENT PERMITTED
      BY LAW, NO CONTRIBUTOR SHALL HAVE ANY LIABILITY FOR ANY DIRECT, INDIRECT,
      INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING
      WITHOUT LIMITATION LOST PROFITS), HOWEVER CAUSED AND ON ANY THEORY OF
      LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
      NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OR DISTRIBUTION
      OF THE PROGRAM OR THE EXERCISE OF ANY RIGHTS GRANTED HEREUNDER, EVEN IF
      ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

   (( 7. ))??
   TRADEMARKS AND BRANDING

      (( 7.1 ))??
      This License does not grant any Recipient or any third party any rights to
      use the trademarks or trade names now or subsequently posted at
      http:/www.ca.com/catrdmrk.htm, or any other trademarks, service marks,
      logos or trade names belonging to CA (collectively CA Marks) or to any
      trademark, service mark, logo or trade name belonging to any Contributor.
      Recipient agrees not to use any CA Marks in or as part of the name of
      products derived from the Original Program or to endorse or promote
      products derived from the Original Program.

      (( 7.2 ))??
      Subject to Section 7.1, Recipients may distribute the Program under
      trademarks, logos, and product names belonging to the Recipient provided
      that all copyright and other attribution notices remain in the Program.

   (( 8. ))??
   PATENT LITIGATION

      (( 8.1 ))??
      If Recipient institutes patent litigation against any person or entity
      (including a cross-claim or counterclaim in a lawsuit) alleging that the
      Program itself (excluding combinations of the Program with other software
      or hardware) infringes such Recipients patent(s), then such Recipients
      rights granted under Section 2.2 shall terminate as of the date such
      litigation is filed.

   (( 9. ))??
   OWNERSHIP

      (( 9.1 ))??
      Subject to the licenses granted under this License in Sections 2.1 and 2.2
      above, each Contributor retains all rights, title and interest in and to
      any Contributions made by such Contributor. CA retains all rights, title
      and interest in and to the Original Program and any Contributions made by
      or on behalf of CA (CA Contributions), and such CA Contributions will not
      be automatically subject to this License. CA may, at its sole discretion,
      choose to license such CA Contributions under this License, or on
      different terms from those contained in this License or may choose not to
      license them at all.

   (( 10. ))??
   TERMINATION

      (( 10.1 ))??
      All of Recipients rights under this License shall terminate if it fails to
      comply with any of the material terms or conditions of this License and
      does not cure such failure in a reasonable period of time after becoming
      aware of such noncompliance. If Recipients rights under this License
      terminate, Recipient agrees to cease use and distribution of the Program
      as soon as reasonably practicable. However, Recipients obligations under
      this License and any licenses granted by Recipient as a Contributor
      relating to the Program shall continue and survive termination.

   (( 11. ))??
   GENERAL

      (( 11.1 ))??
      If any provision of this License is invalid or unenforceable under
      applicable law, it shall not affect the validity or enforceability of the
      remainder of the terms of this License, and without further action by the
      parties hereto, such provision shall be reformed to the minimum extent
      necessary to make such provision valid and enforceable.

      (( 11.2 ))??
      CA may publish new versions (including revisions) of this License from
      time to time. Each new version of the License will be given a
      distinguishing version number. The Program (including Contributions) may
      always be distributed subject to the version of the License under which it
      was received. In addition, after a new version of the License is
      published, Contributor may elect to distribute the Program (including its
      Contributions) under the new version. No one other than CA has the right
      to modify this License.

      (( 11.3 ))??
      If it is impossible for Recipient to comply with any of the terms of this
      License with respect to some or all of the Program due to statute,
      judicial order, or regulation, then Recipient must: (a) comply with the
      terms of this License to the maximum extent possible; and (b) describe the
      limitations and the code they affect. Such description must be included in
      the IP_ISSUES file described in Section 3.5 and must be included with all
      distributions of the Program source code. Except to the extent prohibited
      by statute or regulation, such description must be sufficiently detailed
      for a Recipient of ordinary skill to be able to understand it.

      (( 11.4 ))??
      This License is governed by the laws of the State of New York. No
      Recipient will bring a legal action under this License more than one year
      after the cause of action arose. Each Recipient waives its rights to a
      jury trial in any resulting litigation. Any litigation or other dispute
      resolution between a Recipient and CA relating to this License shall take
      place in the State of New York, and Recipient and CA hereby consent to the
      personal jurisdiction of, and venue in, the state and federal courts
      within that district with respect to this License. The application of the
      United Nations Convention on Contracts for the International Sale of Goods
      is expressly excluded.

      (( 11.5 ))??
      Where Recipient is located in the province of Quebec, Canada, the
      following clause applies: The parties hereby confirm that they have
      requested that this License and all related documents be drafted in
      English. Les parties contractantes confirment qu'elles ont exige que le
      present contrat et tous les documents associes soient rediges en anglais.

      (( 11.6 ))??
      The Program is subject to all export and import laws, restrictions and
      regulations of the country in which Recipient receives the Program.
      Recipient is solely responsible for complying with and ensuring that
      Recipient does not export, re-export, or import the Program in violation
      of such laws, restrictions or regulations, or without any necessary
      licenses and authorizations.

      (( 11.7 ))??
      This License constitutes the entire agreement between the parties with
      respect to the subject matter hereof.
`
const license_CC_BY_1_0_lre = `//**
CC-BY-1.0
https://creativecommons.org/licenses/by/1.0
**//

((Creative Commons))??

Creative Commons

Attribution 1.0

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE
LEGAL
SERVICES. DISTRIBUTION OF THIS DRAFT LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION
ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING
FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE").  THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW.  ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE.  THE LICENSOR GRANTS YOU THE RIGHTS
CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole.  A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant.  Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to create and reproduce Derivative Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission Derivative Works;

The above rights may be exercised in all media and formats whether now known or
hereafter devised.  The above rights include the right to make such
modifications as are technically necessary to exercise the rights in other media
and formats.  All rights not expressly granted by Licensor are hereby reserved.

4. Restrictions. The license granted in Section 3 above is expressly made
subject to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform.  You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder.  You may not sublicense the Work.  You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement.  The above applies to the Work as incorporated in a
Collective Work, but this does not require the Collective Work apart from the
Work itself to be made subject to the terms of this License. If You create a
Collective Work, upon notice from any Licensor You must, to the extent
practicable, remove from the Collective Work any reference to such Licensor or
the Original Author, as requested.  If You create a Derivative Work, upon notice
from any Licensor You must, to the extent practicable, remove from the
Derivative Work any reference to such Licensor or the Original Author, as
requested.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or any Derivative Works or Collective Works, You must keep
intact all copyright notices for the Work and give the Original Author credit
reasonable to the medium or means You are utilizing by conveying the name (or
pseudonym if applicable) of the Original Author if supplied; the title of the
Work if supplied; in the case of a Derivative Work, a credit identifying the use
of the Work in the Derivative Work (e.g., "French translation of the Work by
Original Author," or "Screenplay based on original Work by Original Author").
Such credit may be implemented in any reasonable manner; provided, however, that
in the case of a Derivative Work or Collective Work, at a minimum such credit
will appear where any other comparable authorship credit appears and in a manner
at least as prominent as such other comparable authorship credit.

5. Representations, Warranties and Disclaimer

__1__
By offering the Work for public release under this License, Licensor represents
and warrants that, to the best of Licensor's knowledge after reasonable
inquiry:

__1__
Licensor has secured all rights in the Work necessary to grant the license
rights hereunder and to permit the lawful exercise of the rights granted
hereunder without You having any obligation to pay any royalties, compulsory
license fees, residuals or any other payments;

__1__
The Work does not infringe the copyright, trademark, publicity rights, common
law rights or any other right of any third party or constitute defamation,
invasion of privacy or other tortious injury to any third party.

__1__
EXCEPT AS EXPRESSLY STATED IN THIS LICENSE OR OTHERWISE AGREED IN WRITING OR
REQUIRED BY APPLICABLE LAW, THE WORK IS LICENSED ON AN "AS IS" BASIS, WITHOUT
WARRANTIES OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION,
ANY WARRANTIES REGARDING THE CONTENTS OR ACCURACY OF THE WORK.

6. Limitation on Liability.  EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW,
AND EXCEPT FOR DAMAGES ARISING FROM LIABILITY TO A THIRD PARTY RESULTING FROM
BREACH OF THE WARRANTIES IN SECTION 5, IN NO EVENT WILL LICENSOR BE LIABLE TO
YOU ON ANY LEGAL THEORY FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR
EXEMPLARY DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK, EVEN IF
LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Derivative Works or Collective Works from You under this License,
however, will not have their licenses terminated provided such individuals or
entities remain in full compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work).  Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
Each time You distribute or publicly digitally perform a Derivative Work,
Licensor offers to the recipient a license to the original Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here.  There are no understandings, agreements or
representations with respect to the Work not specified here.  Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work.  Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license.  Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons.  Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_2_0_lre = `//**
CC-BY-2.0
https://creativecommons.org/licenses/by/2.0
**//

((Creative Commons))??

Creative Commons

Attribution 2.0

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE LEGAL
SERVICES. DISTRIBUTION OF THIS LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE"). THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE. THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED
HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole. A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License. For
the avoidance of doubt, where the Work is a musical composition or sound
recording, the synchronization of the Work in timed-relation with a moving image
("synching") will be considered a Derivative Work for the purpose of this
License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant. Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to create and reproduce Derivative Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission Derivative Works.

__1__
For the avoidance of doubt, where the work is a musical composition:

__1__ Performance Royalties Under Blanket Licenses. Licensor waives the
exclusive right to collect, whether individually or via a performance rights
society (e.g. ASCAP, BMI, SESAC), royalties for the public performance or public
digital performance (e.g. webcast) of the Work.

__1__ Mechanical Rights and Statutory Royalties. Licensor waives the exclusive
right to collect, whether individually or via a music rights agency or
designated agent (e.g. Harry Fox Agency), royalties for any phonorecord You
create from the Work ("cover version") and distribute, subject to the compulsory
license created by 17 USC Section 115 of the US Copyright Act (or the equivalent
in other jurisdictions).

__1__ Webcasting Rights and Statutory Royalties. For the avoidance of doubt,
where the Work is a sound recording, Licensor waives the exclusive right to
collect, whether individually or via a performance-rights society (e.g.
SoundExchange), royalties for the public digital performance (e.g. webcast) of
the Work, subject to the compulsory license created by 17 USC Section 114 of the
US Copyright Act (or the equivalent in other jurisdictions).

The above rights may be exercised in all media and formats whether now known or
hereafter devised. The above rights include the right to make such modifications
as are technically necessary to exercise the rights in other media and formats.
All rights not expressly granted by Licensor are hereby reserved.

4. Restrictions.The license granted in Section 3 above is expressly made subject
to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform. You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder. You may not sublicense the Work. You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement. The above applies to the Work as incorporated in a Collective
Work, but this does not require the Collective Work apart from the Work itself
to be made subject to the terms of this License. If You create a Collective
Work, upon notice from any Licensor You must, to the extent practicable, remove
from the Collective Work any reference to such Licensor or the Original Author,
as requested. If You create a Derivative Work, upon notice from any Licensor You
must, to the extent practicable, remove from the Derivative Work any reference
to such Licensor or the Original Author, as requested.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or any Derivative Works or Collective Works, You must keep
intact all copyright notices for the Work and give the Original Author credit
reasonable to the medium or means You are utilizing by conveying the name (or
pseudonym if applicable) of the Original Author if supplied; the title of the
Work if supplied; to the extent reasonably practicable, the Uniform Resource
Identifier, if any, that Licensor specifies to be associated with the Work,
unless such URI does not refer to the copyright notice or licensing information
for the Work; and in the case of a Derivative Work, a credit identifying the use
of the Work in the Derivative Work (e.g., "French translation of the Work by
Original Author," or "Screenplay based on original Work by Original Author").
Such credit may be implemented in any reasonable manner; provided, however, that
in the case of a Derivative Work or Collective Work, at a minimum such credit
will appear where any other comparable authorship credit appears and in a manner
at least as prominent as such other comparable authorship credit.

5. Representations, Warranties and Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN WRITING, LICENSOR OFFERS
THE WORK AS-IS AND MAKES NO REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING
THE WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT
LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY, FITNESS FOR A PARTICULAR
PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY,
OR THE PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE. SOME
JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO SUCH
EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability. EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN
NO EVENT WILL LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY SPECIAL,
INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY DAMAGES ARISING OUT OF THIS
LICENSE OR THE USE OF THE WORK, EVEN IF LICENSOR HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Derivative Works or Collective Works from You under this License,
however, will not have their licenses terminated provided such individuals or
entities remain in full compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work). Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
Each time You distribute or publicly digitally perform a Derivative Work,
Licensor offers to the recipient a license to the original Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here. There are no understandings, agreements or
representations with respect to the Work not specified here. Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work. Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license. Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons. Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_2_5_lre = `//**
CC-BY-2.5
https://creativecommons.org/licenses/by/2.5
**//

((Creative Commons))??

Creative Commons

Attribution 2.5

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE LEGAL
SERVICES. DISTRIBUTION OF THIS LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE"). THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE. THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED
HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole. A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License. For
the avoidance of doubt, where the Work is a musical composition or sound
recording, the synchronization of the Work in timed-relation with a moving image
("synching") will be considered a Derivative Work for the purpose of this
License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant. Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to create and reproduce Derivative Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission Derivative Works.

__1__
For the avoidance of doubt, where the work is a musical composition:

__1__ Performance Royalties Under Blanket Licenses. Licensor waives the
exclusive right to collect, whether individually or via a performance rights
society (e.g. ASCAP, BMI, SESAC), royalties for the public performance or public
digital performance (e.g. webcast) of the Work.

__1__ Mechanical Rights and Statutory Royalties. Licensor waives the exclusive
right to collect, whether individually or via a music rights agency or
designated agent (e.g. Harry Fox Agency), royalties for any phonorecord You
create from the Work ("cover version") and distribute, subject to the compulsory
license created by 17 USC Section 115 of the US Copyright Act (or the equivalent
in other jurisdictions).

__1__ Webcasting Rights and Statutory Royalties. For the avoidance of doubt,
where the Work is a sound recording, Licensor waives the exclusive right to
collect, whether individually or via a performance-rights society (e.g.
SoundExchange), royalties for the public digital performance (e.g. webcast) of
the Work, subject to the compulsory license created by 17 USC Section 114 of the
US Copyright Act (or the equivalent in other jurisdictions).

The above rights may be exercised in all media and formats whether now known or
hereafter devised. The above rights include the right to make such modifications
as are technically necessary to exercise the rights in other media and formats.
All rights not expressly granted by Licensor are hereby reserved.

4. Restrictions.The license granted in Section 3 above is expressly made subject
to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform. You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder. You may not sublicense the Work. You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement. The above applies to the Work as incorporated in a Collective
Work, but this does not require the Collective Work apart from the Work itself
to be made subject to the terms of this License. If You create a Collective
Work, upon notice from any Licensor You must, to the extent practicable, remove
from the Collective Work any credit as required by clause 4(b), as requested. If
You create a Derivative Work, upon notice from any Licensor You must, to the
extent practicable, remove from the Derivative Work any credit as required by
clause 4(b), as requested.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or any Derivative Works or Collective Works, You must keep
intact all copyright notices for the Work and provide, reasonable to the medium
or means You are utilizing: (i) the name of the Original Author (or pseudonym,
if applicable) if supplied, and/or (ii) if the Original Author and/or Licensor
designate another party or parties (e.g. a sponsor institute, publishing entity,
journal) for attribution in Licensor's copyright notice, terms of service or by
other reasonable means, the name of such party or parties; the title of the Work
if supplied; to the extent reasonably practicable, the Uniform Resource
Identifier, if any, that Licensor specifies to be associated with the Work,
unless such URI does not refer to the copyright notice or licensing information
for the Work; and in the case of a Derivative Work, a credit identifying the use
of the Work in the Derivative Work (e.g., "French translation of the Work by
Original Author," or "Screenplay based on original Work by Original Author").
Such credit may be implemented in any reasonable manner; provided, however, that
in the case of a Derivative Work or Collective Work, at a minimum such credit
will appear where any other comparable authorship credit appears and in a manner
at least as prominent as such other comparable authorship credit.

5. Representations, Warranties and Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN WRITING, LICENSOR OFFERS
THE WORK AS-IS AND MAKES NO REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING
THE WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT
LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY, FITNESS FOR A PARTICULAR
PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY,
OR THE PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE. SOME
JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO SUCH
EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability. EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN
NO EVENT WILL LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY SPECIAL,
INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY DAMAGES ARISING OUT OF THIS
LICENSE OR THE USE OF THE WORK, EVEN IF LICENSOR HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Derivative Works or Collective Works from You under this License,
however, will not have their licenses terminated provided such individuals or
entities remain in full compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work). Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
Each time You distribute or publicly digitally perform a Derivative Work,
Licensor offers to the recipient a license to the original Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here. There are no understandings, agreements or
representations with respect to the Work not specified here. Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work. Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license. Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons. Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_3_0_lre = `//**
CC-BY-3.0
https://creativecommons.org/licenses/by/3.0
**//

((Creative Commons))??

((
Creative Commons Legal Code
))??
((https://creativecommons.org/licenses/by/3.0
((/legalcode))??
))??

((
Attribution 3.0 Unported
))??
((https://creativecommons.org/licenses/by/3.0
((/legalcode))??
))??

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES
NOT PROVIDE LEGAL SERVICES. DISTRIBUTION OF THIS LICENSE
DOES NOT CREATE AN ATTORNEY-CLIENT RELATIONSHIP. CREATIVE
COMMONS PROVIDES THIS INFORMATION ON AN "AS-IS" BASIS.
CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES
RESULTING FROM ITS USE.
))??

((
((License))??
))??
((https://creativecommons.org/licenses/by/3.0
((/legalcode))??
))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS
OF THIS CREATIVE COMMONS PUBLIC LICENSE ("CCPL" OR
"LICENSE"). THE WORK IS PROTECTED BY COPYRIGHT AND/OR OTHER
APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS
AUTHORIZED UNDER THIS LICENSE OR COPYRIGHT LAW IS
PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU
ACCEPT AND AGREE TO BE BOUND BY THE TERMS OF THIS LICENSE.
TO THE EXTENT THIS LICENSE MAY BE CONSIDERED TO BE A
CONTRACT, THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED HERE
IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND
CONDITIONS.

1. Definitions

__1__ "Adaptation" means a work based upon
the Work, or upon the Work and other pre-existing works,
such as a translation, adaptation, derivative work,
arrangement of music or other alterations of a literary
or artistic work, or phonogram or performance and
includes cinematographic adaptations or any other form in
which the Work may be recast, transformed, or adapted
including in any form recognizably derived from the
original, except that a work that constitutes a
Collection will not be considered an Adaptation for the
purpose of this License. For the avoidance of doubt,
where the Work is a musical work, performance or
phonogram, the synchronization of the Work in
timed-relation with a moving image ("synching") will be
considered an Adaptation for the purpose of this
License.

__1__ "Collection" means a collection of
literary or artistic works, such as encyclopedias and
anthologies, or performances, phonograms or broadcasts,
or other works or subject matter other than works listed
in Section 1(f) below, which, by reason of the selection
and arrangement of their contents, constitute
intellectual creations, in which the Work is included in
its entirety in unmodified form along with one or more
other contributions, each constituting separate and
independent works in themselves, which together are
assembled into a collective whole. A work that
constitutes a Collection will not be considered an
Adaptation (as defined above) for the purposes of this
License.

__1__ "Distribute" means to make available
to the public the original and copies of the Work or
Adaptation, as appropriate, through sale or other
transfer of ownership.

__1__ "Licensor" means the individual,
individuals, entity or entities that offer(s) the Work
under the terms of this License.

__1__ "Original Author" means, in the case
of a literary or artistic work, the individual,
individuals, entity or entities who created the Work or
if no individual or entity can be identified, the
publisher; and in addition (i) in the case of a
performance the actors, singers, musicians, dancers, and
other persons who act, sing, deliver, declaim, play in,
interpret or otherwise perform literary or artistic works
or expressions of folklore; (ii) in the case of a
phonogram the producer being the person or legal entity
who first fixes the sounds of a performance or other
sounds; and, (iii) in the case of broadcasts, the
organization that transmits the broadcast.

__1__ "Work" means the literary and/or
artistic work offered under the terms of this License
including without limitation any production in the
literary, scientific and artistic domain, whatever may be
the mode or form of its expression including digital
form, such as a book, pamphlet and other writing; a
lecture, address, sermon or other work of the same
nature; a dramatic or dramatico-musical work; a
choreographic work or entertainment in dumb show; a
musical composition with or without words; a
cinematographic work to which are assimilated works
expressed by a process analogous to cinematography; a
work of drawing, painting, architecture, sculpture,
engraving or lithography; a photographic work to which
are assimilated works expressed by a process analogous to
photography; a work of applied art; an illustration, map,
plan, sketch or three-dimensional work relative to
geography, topography, architecture or science; a
performance; a broadcast; a phonogram; a compilation of
data to the extent it is protected as a copyrightable
work; or a work performed by a variety or circus
performer to the extent it is not otherwise considered a
literary or artistic work.

__1__ "You" means an individual or entity
exercising rights under this License who has not
previously violated the terms of this License with
respect to the Work, or who has received express
permission from the Licensor to exercise rights under
this License despite a previous violation.

__1__ "Publicly Perform" means to perform
public recitations of the Work and to communicate to the
public those public recitations, by any means or process,
including by wire or wireless means or public digital
performances; to make available to the public Works in
such a way that members of the public may access these
Works from a place and at a place individually chosen by
them; to perform the Work to the public by any means or
process and the communication to the public of the
performances of the Work, including by public digital
performance; to broadcast and rebroadcast the Work by any
means including signs, sounds or images.

__1__ "Reproduce" means to make copies of
the Work by any means including without limitation by
sound or visual recordings and the right of fixation and
reproducing fixations of the Work, including storage of a
protected performance or phonogram in digital form or
other electronic medium.

2. Fair Dealing Rights. Nothing in this
License is intended to reduce, limit, or restrict any uses
free from copyright or rights arising from limitations or
exceptions that are provided for in connection with the
copyright protection under copyright law or other
applicable laws.

3. License Grant. Subject to the terms
and conditions of this License, Licensor hereby grants You
a worldwide, royalty-free, non-exclusive, perpetual (for
the duration of the applicable copyright) license to
exercise the rights in the Work as stated below:

__1__ to Reproduce the Work, to incorporate the Work into
one or more Collections, and to Reproduce the Work as
incorporated in the Collections;

__1__ to create and Reproduce Adaptations provided that any
such Adaptation, including any translation in any medium,
takes reasonable steps to clearly label, demarcate or
otherwise identify that changes were made to the original
Work. For example, a translation could be marked "The
original work was translated from English to Spanish," or
a modification could indicate "The original work has been
modified.";

__1__ to Distribute and Publicly Perform the Work including
as incorporated in Collections; and,

__1__ to Distribute and Publicly Perform Adaptations.

__1__

For the avoidance of doubt:

__1__ Non-waivable Compulsory License
Schemes. In those jurisdictions in which the
right to collect royalties through any statutory or
compulsory licensing scheme cannot be waived, the
Licensor reserves the exclusive right to collect such
royalties for any exercise by You of the rights
granted under this License;

__1__ Waivable Compulsory License
Schemes. In those jurisdictions in which the
right to collect royalties through any statutory or
compulsory licensing scheme can be waived, the
Licensor waives the exclusive right to collect such
royalties for any exercise by You of the rights
granted under this License; and,

__1__ Voluntary License Schemes. The
Licensor waives the right to collect royalties,
whether individually or, in the event that the
Licensor is a member of a collecting society that
administers voluntary licensing schemes, via that
society, from any exercise by You of the rights
granted under this License.

The above rights may be exercised in all media and
formats whether now known or hereafter devised. The above
rights include the right to make such modifications as are
technically necessary to exercise the rights in other media
and formats. Subject to Section 8(f), all rights not
expressly granted by Licensor are hereby reserved.

4. Restrictions. The license granted in
Section 3 above is expressly made subject to and limited by
the following restrictions:

__1__ You may Distribute or Publicly Perform the Work only
under the terms of this License. You must include a copy
of, or the Uniform Resource Identifier (URI) for, this
License with every copy of the Work You Distribute or
Publicly Perform. You may not offer or impose any terms
on the Work that restrict the terms of this License or
the ability of the recipient of the Work to exercise the
rights granted to that recipient under the terms of the
License. You may not sublicense the Work. You must keep
intact all notices that refer to this License and to the
disclaimer of warranties with every copy of the Work You
Distribute or Publicly Perform. When You Distribute or
Publicly Perform the Work, You may not impose any
effective technological measures on the Work that
restrict the ability of a recipient of the Work from You
to exercise the rights granted to that recipient under
the terms of the License. This Section 4(a) applies to
the Work as incorporated in a Collection, but this does
not require the Collection apart from the Work itself to
be made subject to the terms of this License. If You
create a Collection, upon notice from any Licensor You
must, to the extent practicable, remove from the
Collection any credit as required by Section 4(b), as
requested. If You create an Adaptation, upon notice from
any Licensor You must, to the extent practicable, remove
from the Adaptation any credit as required by Section
4(b), as requested.

__1__ If You Distribute, or Publicly Perform the Work or
any Adaptations or Collections, You must, unless a
request has been made pursuant to Section 4(a), keep
intact all copyright notices for the Work and provide,
reasonable to the medium or means You are utilizing: (i)
the name of the Original Author (or pseudonym, if
applicable) if supplied, and/or if the Original Author
and/or Licensor designate another party or parties (e.g.,
a sponsor institute, publishing entity, journal) for
attribution ("Attribution Parties") in Licensor's
copyright notice, terms of service or by other reasonable
means, the name of such party or parties; (ii) the title
of the Work if supplied; (iii) to the extent reasonably
practicable, the URI, if any, that Licensor specifies to
be associated with the Work, unless such URI does not
refer to the copyright notice or licensing information
for the Work; and (iv) , consistent with Section 3(b), in
the case of an Adaptation, a credit identifying the use
of the Work in the Adaptation (e.g., "French translation
of the Work by Original Author," or "Screenplay based on
original Work by Original Author"). The credit required
by this Section 4 (b) may be implemented in any
reasonable manner; provided, however, that in the case of
a Adaptation or Collection, at a minimum such credit will
appear, if a credit for all contributing authors of the
Adaptation or Collection appears, then as part of these
credits and in a manner at least as prominent as the
credits for the other contributing authors. For the
avoidance of doubt, You may only use the credit required
by this Section for the purpose of attribution in the
manner set out above and, by exercising Your rights under
this License, You may not implicitly or explicitly assert
or imply any connection with, sponsorship or endorsement
by the Original Author, Licensor and/or Attribution
Parties, as appropriate, of You or Your use of the Work,
without the separate, express prior written permission of
the Original Author, Licensor and/or Attribution
Parties.

__1__ Except as otherwise agreed in writing by the Licensor
or as may be otherwise permitted by applicable law, if
You Reproduce, Distribute or Publicly Perform the Work
either by itself or as part of any Adaptations or
Collections, You must not distort, mutilate, modify or
take other derogatory action in relation to the Work
which would be prejudicial to the Original Author's honor
or reputation. Licensor agrees that in those
jurisdictions (e.g. Japan), in which any exercise of the
right granted in Section 3(b) of this License (the right
to make Adaptations) would be deemed to be a distortion,
mutilation, modification or other derogatory action
prejudicial to the Original Author's honor and
reputation, the Licensor will waive or not assert, as
appropriate, this Section, to the fullest extent
permitted by the applicable national law, to enable You
to reasonably exercise Your right under Section 3(b) of
this License (right to make Adaptations) but not
otherwise.

5. Representations, Warranties and
Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN
WRITING, LICENSOR OFFERS THE WORK AS-IS AND MAKES NO
REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING THE
WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING,
WITHOUT LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY,
FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE
ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY, OR THE
PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE.
SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
WARRANTIES, SO SUCH EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability. EXCEPT TO
THE EXTENT REQUIRED BY APPLICABLE LAW, IN NO EVENT WILL
LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY
SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY
DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK,
EVEN IF LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

7. Termination

__1__ This License and the rights granted hereunder will
terminate automatically upon any breach by You of the
terms of this License. Individuals or entities who have
received Adaptations or Collections from You under this
License, however, will not have their licenses terminated
provided such individuals or entities remain in full
compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__ Subject to the above terms and conditions, the
license granted here is perpetual (for the duration of
the applicable copyright in the Work). Notwithstanding
the above, Licensor reserves the right to release the
Work under different license terms or to stop
distributing the Work at any time; provided, however that
any such election will not serve to withdraw this License
(or any other license that has been, or is required to
be, granted under the terms of this License), and this
License will continue in full force and effect unless
terminated as stated above.

8. Miscellaneous

__1__ Each time You Distribute or Publicly Perform the Work
or a Collection, the Licensor offers to the recipient a
license to the Work on the same terms and conditions as
the license granted to You under this License.

__1__ Each time You Distribute or Publicly Perform an
Adaptation, Licensor offers to the recipient a license to
the original Work on the same terms and conditions as the
license granted to You under this License.

__1__ If any provision of this License is invalid or
unenforceable under applicable law, it shall not affect
the validity or enforceability of the remainder of the
terms of this License, and without further action by the
parties to this agreement, such provision shall be
reformed to the minimum extent necessary to make such
provision valid and enforceable.

__1__ No term or provision of this License shall be deemed
waived and no breach consented to unless such waiver or
consent shall be in writing and signed by the party to be
charged with such waiver or consent.

__1__ This License constitutes the entire agreement between
the parties with respect to the Work licensed here. There
are no understandings, agreements or representations with
respect to the Work not specified here. Licensor shall
not be bound by any additional provisions that may appear
in any communication from You.

__1__ //** Non-standard paragraph break. **//
This License may not be
modified without the mutual written agreement of the
Licensor and You.

__1__ The rights granted under, and the subject matter
referenced, in this License were drafted utilizing the
terminology of the Berne Convention for the Protection of
Literary and Artistic Works (as amended on September 28,
1979), the Rome Convention of 1961, the WIPO Copyright
Treaty of 1996, the WIPO Performances and Phonograms
Treaty of 1996 and the Universal Copyright Convention (as
revised on July 24, 1971). These rights and subject
matter take effect in the relevant jurisdiction in which
the License terms are sought to be enforced according to
the corresponding provisions of the implementation of
those treaty provisions in the applicable national law.
If the standard suite of rights granted under applicable
copyright law includes additional rights not granted
under this License, such additional rights are deemed to
be included in the License; this License is not intended
to restrict the license of any rights under applicable
law.

((
((
Creative Commons Notice
))??
((https://creativecommons.org/licenses/by/3.0
((/legalcode))??
))??

Creative Commons is not a party to this License, and
makes no warranty whatsoever in connection with the Work.
Creative Commons will not be liable to You or any party
on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or
consequential damages arising in connection to this
license. Notwithstanding the foregoing two (2) sentences,
if Creative Commons has expressly identified itself as
the Licensor hereunder, it shall have all rights and
obligations of Licensor.

Except for the limited purpose of indicating to the
public that the Work is licensed under the CCPL, Creative
Commons does not authorize the use by either party of the
trademark "Creative Commons" or any related trademark or
logo of Creative Commons without the prior written
consent of Creative Commons. Any permitted use will be in
compliance with Creative Commons' then-current trademark
usage guidelines, as may be published on its website or
otherwise made available upon request from time to time.
For the avoidance of doubt, this trademark restriction
does not form part of this License.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_3_0_AT_lre = `//**
Creative Commons Attribution 3.0 Austria
https://spdx.org/licenses/CC-BY-3.0-AT.json
https://creativecommons.org/licenses/by/3.0/at/legalcode
**//

CREATIVE COMMONS IST KEINE RECHTSANWALTSKANZLEI UND LEISTET KEINE
RECHTSBERATUNG. DIE BEREITSTELLUNG DIESER LIZENZ FÜHRT ZU KEINEM
MANDATSVERHÄLTNIS. CREATIVE COMMONS STELLT DIESE INFORMATIONEN OHNE GEWÄHR ZUR
VERFÜGUNG. CREATIVE COMMONS ÜBERNIMMT KEINE GEWÄHRLEISTUNG FÜR DIE
GELIEFERTEN INFORMATIONEN UND SCHLIEßT DIE HAFTUNG FÜR SCHÄDEN AUS, DIE SICH
AUS DEREN GEBRAUCH ERGEBEN.
(( Lizenz))??


DER GEGENSTAND DIESER LIZENZ (WIE UNTER "SCHUTZGEGENSTAND" DEFINIERT) WIRD UNTER
DEN BEDINGUNGEN DIESER CREATIVE COMMONS PUBLIC LICENSE ("CCPL", "LIZENZ" ODER
"LIZENZVERTRAG") ZUR VERFÜGUNG GESTELLT. DER SCHUTZGEGENSTAND IST DURCH DAS
URHEBERRECHT UND/ODER ANDERE GESETZE GESCHÜTZT. JEDE FORM DER NUTZUNG DES
SCHUTZGEGENSTANDES, DIE NICHT AUFGRUND DIESER LIZENZ ODER DURCH GESETZE
GESTATTET IST, IST UNZULÄSSIG.

DURCH DIE AUSÜBUNG EINES DURCH DIESE LIZENZ GEWÄHRTEN RECHTS AN DEM
SCHUTZGEGENSTAND ERKLÄREN SIE SICH MIT DEN LIZENZBEDINGUNGEN RECHTSVERBINDLICH
EINVERSTANDEN. SOWEIT DIESE LIZENZ ALS LIZENZVERTRAG ANZUSEHEN IST, GEWÄHRT
IHNEN DER LIZENZGEBER DIE IN DER LIZENZ GENANNTEN RECHTE UNENTGELTLICH UND IM
AUSTAUSCH DAFÜR, DASS SIE DAS GEBUNDENSEIN AN DIE LIZENZBEDINGUNGEN
AKZEPTIEREN.

   __1__ Definitionen

      __1__ Der Begriff "Bearbeitung" im Sinne dieser Lizenz bezeichnet das
      Ergebnis jeglicher Art von Veränderung des Schutzgegenstandes, solange
      dieses erkennbar vom Schutzgegenstand abgeleitet wurde. Dies kann
      insbesondere auch eine Umgestaltung, Änderung, Anpassung, Übersetzung
      oder Heranziehung des Schutzgegenstandes zur Vertonung von Laufbildern
      sein. Nicht als Bearbeitung des Schutzgegenstandes gelten seine Aufnahme
      in eine Sammlung oder ein Sammelwerk und die freie Nutzung des
      Schutzgegenstandes.

      __1__ Der Begriff "Sammelwerk" im Sinne dieser Lizenz meint eine
      Zusammenstellung von literarischen, künstlerischen oder
      wissenschaftlichen Inhalten zu einem einheitlichen Ganzen, sofern diese
      Zusammenstellung aufgrund von Auswahl und Anordnung der darin enthaltenen
      selbständigen Elemente eine eigentümliche geistige Schöpfung darstellt,
      unabhängig davon, ob die Elemente systematisch oder methodisch angelegt
      und dadurch einzeln zugänglich sind oder nicht.

      __1__ "Verbreiten" im Sinne dieser Lizenz bedeutet, den Schutzgegenstand
      oder Bearbeitungen im Original oder in Form von
      Vervielfältigungsstücken, mithin in körperlich fixierter Form der
      Öffentlichkeit zugänglich zu machen oder in Verkehr zu bringen.

      __1__ Der "Lizenzgeber" im Sinne dieser Lizenz ist diejenige natürliche
      oder juristische Person oder Gruppe, die den Schutzgegenstand unter den
      Bedingungen dieser Lizenz anbietet und insoweit als Rechteinhaberin
      auftritt.

      __1__ "Rechteinhaber" im Sinne dieser Lizenz ist der Urheber des
      Schutzgegenstandes oder jede andere natürliche oder juristische Person,
      die am Schutzgegenstand ein Immaterialgüterrecht erlangt hat, welches die
      in Abschnitt 3 genannten Handlungen erfasst und eine Erteilung,
      Übertragung oder Einräumung von Nutzungsbewilligungen bzw
      Nutzungsrechten an Dritte erlaubt.

      __1__ Der Begriff "Schutzgegenstand" bezeichnet in dieser Lizenz den
      literarischen, künstlerischen oder wissenschaftlichen Inhalt, der unter
      den Bedingungen dieser Lizenz angeboten wird. Das kann insbesondere eine
      eigentümliche geistige Schöpfung jeglicher Art oder ein Werk der kleinen
      Münze, ein nachgelassenes Werk oder auch ein Lichtbild oder anderes
      Objekt eines verwandten Schutzrechts sein, unabhängig von der Art seiner
      Fixierung und unabhängig davon, auf welche Weise jeweils eine Wahrnehmung
      erfolgen kann, gleichviel ob in analoger oder digitaler Form. Soweit
      Datenbanken oder Zusammenstellungen von Daten einen
      immaterialgüterrechtlichen Schutz eigener Art genießen, unterfallen auch
      sie dem Begriff „Schutzgegenstand" im Sinne dieser Lizenz.

      __1__ Mit "Sie" bzw. "Ihnen" ist die natürliche oder juristische Person
      gemeint, die in dieser Lizenz im Abschnitt 3 genannte Nutzungen des
      Schutzgegenstandes vornimmt und zuvor in Hinblick auf den Schutzgegenstand
      nicht gegen Bedingungen dieser Lizenz verstoßen oder aber die
      ausdrückliche Erlaubnis des Lizenzgebers erhalten hat, die durch diese
      Lizenz gewährte Nutzungsbewilligung trotz eines vorherigen Verstoßes
      auszuüben.

      __1__ Unter "Öffentlich Wiedergeben" im Sinne dieser Lizenz sind
      Wahrnehmbarmachungen des Schutzgegenstandes in unkörperlicher Form zu
      verstehen, die für eine Mehrzahl von Mitgliedern der Öffentlichkeit
      bestimmt sind und mittels öffentlicher Wiedergabe in Form von Vortrag,
      Aufführung, Vorführung, Darbietung, Sendung, Weitersendung oder zeit-
      und ortsunabhängiger Zurverfügungstellung erfolgen, unabhängig von den
      zum Einsatz kommenden Techniken und Verfahren, einschließlich
      drahtgebundener oder drahtloser Mittel und Einstellen in das Internet.

      __1__ "Vervielfältigen" im Sinne dieser Lizenz bedeutet, gleichviel in
      welchem Verfahren, auf welchem Träger, in welcher Menge und ob
      vorübergehend oder dauerhaft, Vervielfältigungsstücke des
      Schutzgegenstandes herzustellen, insbesondere durch Ton- oder
      Bildaufzeichnungen, und umfasst auch das erstmalige Festhalten des
      Schutzgegenstandes oder dessen Wahrnehmbarmachung auf Mitteln der
      wiederholbaren Wiedergabe sowie das Herstellen von
      Vervielfältigungsstücken dieser Festhaltung, sowie die Speicherung einer
      geschützten Darbietung oder eines Bild- und/oder Schallträgers in
      digitaler Form oder auf einem anderen elektronischen Medium.

   __1__ Beschränkungen der Verwertungsrechte

   Diese Lizenz ist in keiner Weise darauf gerichtet, Befugnisse zur Nutzung des
   Schutzgegenstandes zu vermindern, zu beschränken oder zu vereiteln, die sich
   aus den Beschränkungen der Verwertungsrechte, anderen Beschränkungen der
   Ausschließlichkeitsrechte des Rechtsinhabers oder anderen entsprechenden
   Rechtsnormen oder sich aus dem Fehlen eines immaterialgüterrechtlichen
   Schutzes ergeben.

   __1__ Lizenzierung

   Unter den Bedingungen dieser Lizenz erteilt Ihnen der Lizenzgeber -
   unbeschadet unverzichtbarer Rechte und vorbehaltlich des Abschnitts 3.e) -
   die vergütungsfreie, räumlich und zeitlich (für die Dauer des
   Urheberrechts oder verwandten Schutzrechts am Schutzgegenstand)
   unbeschränkte Nutzungsbewilligung, den Schutzgegenstand in der folgenden Art
   und Weise zu nutzen:

      __1__ Den Schutzgegenstand in beliebiger Form und Menge zu
      vervielfältigen, ihn in Sammelwerke zu integrieren und ihn als Teil
      solcher Sammelwerke zu vervielfältigen;

      __1__ Den Schutzgegenstand zu bearbeiten, einschließlich Übersetzungen
      unter Nutzung jedweder Medien anzufertigen, sofern deutlich erkennbar
      gemacht wird, dass es sich um eine Bearbeitung handelt;

      __1__ Den Schutzgegenstand, allein oder in Sammelwerke aufgenommen,
      öffentlich wiederzugeben und zu verbreiten; und

      __1__ Bearbeitungen des Schutzgegenstandes zu veröffentlichen,
      öffentlich wiederzugeben und zu verbreiten.

      __1__ Bezüglich der Vergütung für die Nutzung des Schutzgegenstandes
      gilt Folgendes:

         __1__ Unverzichtbare gesetzliche Vergütungsansprüche: Soweit
         unverzichtbare Vergütungsansprüche im Gegenzug für gesetzliche
         Lizenzen vorgesehen oder Pauschalabgabensysteme (zum Beispiel für
         Leermedien) vorhanden sind, behält sich der Lizenzgeber das
         ausschließliche Recht vor, die entsprechenden Vergütungsansprüche
         für jede Ausübung eines Rechts aus dieser Lizenz durch Sie geltend zu
         machen.

         __1__ Vergütung bei Zwangslizenzen: Sofern Zwangslizenzen außerhalb
         dieser Lizenz vorgesehen sind und zustande kommen, verzichtet der
         Lizenzgeber für alle Fälle einer lizenzgerechten Nutzung des
         Schutzgegenstandes durch Sie auf jegliche Vergütung.

         __1__ Vergütung in sonstigen Fällen: Bezüglich lizenzgerechter
         Nutzung des Schutzgegenstandes durch Sie, die nicht unter die beiden
         vorherigen Abschnitte (i) und (ii) fällt, verzichtet der Lizenzgeber
         auf jegliche Vergütung, unabhängig davon, ob eine Geltendmachung der
         Vergütungsansprüche durch ihn selbst oder nur durch eine
         Verwertungsgesellschaft möglich wäre.

   Die vorgenannte Nutzungsbewilligung wird für alle bekannten sowie alle noch
   nicht bekannten Nutzungsarten eingeräumt. Sie beinhaltet auch das Recht,
   solche Änderungen am Schutzgegenstand vorzunehmen, die für bestimmte nach
   dieser Lizenz zulässige Nutzungen technisch erforderlich sind. Alle
   sonstigen Rechte, die über diesen Abschnitt hinaus nicht ausdrücklich vom
   Lizenzgeber eingeräumt werden, bleiben diesem allein vorbehalten. Soweit
   Datenbanken oder Zusammenstellungen von Daten Schutzgegenstand dieser Lizenz
   oder Teil dessen sind und einen immaterialgüterrechtlichen Schutz eigener
   Art genießen, verzichtet der Lizenzgeber auf die Geltendmachung sämtlicher
   daraus resultierender Rechte.

   __1__ Bedingungen

   Die Erteilung der Nutzungsbewilligung gemäß Abschnitt 3 dieser Lizenz
   erfolgt ausdrücklich nur unter den folgenden Bedingungen:

      __1__ Sie dürfen den Schutzgegenstand ausschließlich unter den
      Bedingungen dieser Lizenz verbreiten oder öffentlich wiedergeben. Sie
      müssen dabei stets eine Kopie dieser Lizenz oder deren vollständige
      Internetadresse in Form des Uniform-Resource-Identifier (URI) beifügen.
      Sie dürfen keine Vertrags- oder Nutzungsbedingungen anbieten oder
      fordern, die die Bedingungen dieser Lizenz oder die durch diese Lizenz
      gewährten Rechte beschränken. Sie dürfen den Schutzgegenstand nicht
      unterlizenzieren. Bei jeder Kopie des Schutzgegenstandes, die Sie
      verbreiten oder öffentlich wiedergeben, müssen Sie alle Hinweise
      unverändert lassen, die auf diese Lizenz und den Haftungsausschluss
      hinweisen. Wenn Sie den Schutzgegenstand verbreiten oder öffentlich
      wiedergeben, dürfen Sie (in Bezug auf den Schutzgegenstand) keine
      technischen Maßnahmen ergreifen, die den Nutzer des Schutzgegenstandes in
      der Ausübung der ihm durch diese Lizenz gewährten Rechte behindern
      können. Dasselbe gilt auch für den Fall, dass der Schutzgegenstand einen
      Bestandteil eines Sammelwerkes bildet, was jedoch nicht bedeutet, dass das
      Sammelwerk insgesamt dieser Lizenz unterstellt werden muss. Sofern Sie ein
      Sammelwerk erstellen, müssen Sie - soweit dies praktikabel ist - auf die
      Mitteilung eines Lizenzgebers hin aus dem Sammelwerk die in Abschnitt 4.b)
      aufgezählten Hinweise entfernen. Wenn Sie eine Bearbeitung vornehmen,
      müssen Sie – soweit dies praktikabel ist – auf die Mitteilung eines
      Lizenzgebers hin von der Bearbeitung die in Abschnitt 4.b) aufgezählten
      Hinweise entfernen.

      __1__ Die Verbreitung und die öffentliche Wiedergabe des
      Schutzgegenstandes oder auf ihm aufbauender Inhalte oder ihn enthaltender
      Sammelwerke ist Ihnen nur unter der Bedingung gestattet, dass Sie,
      vorbehaltlich etwaiger Mitteilungen im Sinne von Abschnitt 4.a), alle dazu
      gehörenden Rechtevermerke unberührt lassen. Sie sind verpflichtet, die
      Urheberschaft oder die Rechteinhaberschaft in einer der Nutzung
      entsprechenden, angemessenen Form anzuerkennen, indem Sie selbst –
      soweit bekannt – Folgendes angeben:

         __1__ Den Namen (oder das Pseudonym, falls ein solches verwendet wird)
         Rechteinhabers, und/oder falls der Lizenzgeber im Rechtevermerk, in den
         Nutzungsbedingungen oder auf andere angemessene Weise eine Zuschreibung
         an Dritte vorgenommen hat (z.B. an eine Stiftung, ein Verlagshaus oder
         eine Zeitung) („Zuschreibungsempfänger"), Namen bzw. Bezeichnung
         dieses oder dieser Dritten;

         __1__ den Titel des Inhaltes;

         __1__ in einer praktikablen Form den Uniform-Resource-Identifier (URI,
         z.B. Internetadresse), den der Lizenzgeber zum Schutzgegenstand
         angegeben hat, es sei denn, dieser URI verweist nicht auf den
         Rechtevermerk oder die Lizenzinformationen zum Schutzgegenstand;

         __1__ und im Falle einer Bearbeitung des Schutzgegenstandes in
         Übereinstimmung mit Abschnitt 3.b) einen Hinweis darauf, dass es sich
         um eine Bearbeitung handelt.

      Die nach diesem Abschnitt 4.b) erforderlichen Angaben können in jeder
      angemessenen Form gemacht werden; im Falle einer Bearbeitung des
      Schutzgegenstandes oder eines Sammelwerkes müssen diese Angaben das
      Minimum darstellen und bei gemeinsamer Nennung aller Beitragenden
      dergestalt erfolgen, dass sie zumindest ebenso hervorgehoben sind wie die
      Hinweise auf die übrigen Rechteinhaber. Die Angaben nach diesem Abschnitt
      dürfen Sie ausschließlich zur Angabe der Rechteinhaberschaft in der oben
      bezeichneten Weise verwenden. Durch die Ausübung Ihrer Rechte aus dieser
      Lizenz dürfen Sie ohne eine vorherige, separat und schriftlich
      vorliegende Zustimmung des Urhebers, des Lizenzgebers und/oder des
      Zuschreibungsempfängers weder implizit noch explizit irgendeine
      Verbindung mit dem oder eine Unterstützung oder Billigung durch den
      Urheber, den Lizenzgeber oder den Zuschreibungsempfänger andeuten oder
      erklären.

      __1__ Die oben unter 4.a) und b) genannten Einschränkungen gelten nicht
      für solche Teile des Schutzgegenstandes, die allein deshalb unter den
      Schutzgegenstandsbegriff fallen, weil sie als Datenbanken oder
      Zusammenstellungen von Daten einen immaterialgüterrechtlichen Schutz
      eigener Art genießen.

      __1__ (Urheber)Persönlichkeitsrechte bleiben - soweit sie bestehen - von
      dieser Lizenz unberührt.

   __1__ Gewährleistung

   SOFERN KEINE ANDERS LAUTENDE, SCHRIFTLICHE VEREINBARUNG ZWISCHEN DEM
   LIZENZGEBER UND IHNEN GESCHLOSSEN WURDE UND SOWEIT MÄNGEL NICHT ARGLISTIG
   VERSCHWIEGEN WURDEN, BIETET DER LIZENZGEBER DEN SCHUTZGEGENSTAND UND DIE
   ERTEILUNG DER NUTZUNGSBEWILLIGUNG UNTER AUSSCHLUSS JEGLICHER GEWÄHRLEISTUNG
   AN UND ÜBERNIMMT WEDER AUSDRÜCKLICH NOCH KONKLUDENT GARANTIEN IRGENDEINER
   ART. DIES UMFASST INSBESONDERE DAS FREISEIN VON SACH- UND RECHTSMÄNGELN,
   UNABHÄNGIG VON DEREN ERKENNBARKEIT FÜR DEN LIZENZGEBER, DIE
   VERKEHRSFÄHIGKEIT DES SCHUTZGEGENSTANDES, SEINE VERWENDBARKEIT FÜR EINEN
   BESTIMMTEN ZWECK SOWIE DIE KORREKTHEIT VON BESCHREIBUNGEN.

   __1__ Haftungsbeschränkung

   ÜBER DIE IN ZIFFER 5 GENANNTE GEWÄHRLEISTUNG HINAUS HAFTET DER LIZENZGEBER
   IHNEN GEGENÜBER FÜR SCHÄDEN JEGLICHER ART NUR BEI GROBER FAHRLÄSSIGKEIT
   ODER VORSATZ, UND ÜBERNIMMT DARÜBER HINAUS KEINERLEI FREIWILLIGE HAFTUNG
   FÜR FOLGE- ODER ANDERE SCHÄDEN, AUCH WENN ER ÜBER DIE MÖGLICHKEIT IHRES
   EINTRITTS UNTERRICHTET WURDE.

   __1__ Erlöschen

      __1__ Diese Lizenz und die durch sie erteilte Nutzungsbewilligung
      erlöschen mit Wirkung für die Zukunft im Falle eines Verstoßes gegen
      die Lizenzbedingungen durch Sie, ohne dass es dazu der Kenntnis des
      Lizenzgebers vom Verstoß oder einer weiteren Handlung einer der
      Vertragsparteien bedarf. Mit natürlichen oder juristischen Personen, die
      Bearbeitungen des Schutzgegenstandes oder diesen enthaltende Sammelwerke
      sowie entsprechende Vervielfältigungsstücke unter den Bedingungen dieser
      Lizenz von Ihnen erhalten haben, bestehen nachträglich entstandene
      Lizenzbeziehungen jedoch solange weiter, wie die genannten Personen sich
      ihrerseits an sämtliche Lizenzbedingungen halten. Darüber hinaus gelten
      die Ziffern 1, 2, 5, 6, 7, und 8 auch nach einem Erlöschen dieser Lizenz
      fort.

      __1__ Vorbehaltlich der oben genannten Bedingungen gilt diese Lizenz
      unbefristet bis der rechtliche Schutz für den Schutzgegenstand ausläuft.
      Davon abgesehen behält der Lizenzgeber das Recht, den Schutzgegenstand
      unter anderen Lizenzbedingungen anzubieten oder die eigene Weitergabe des
      Schutzgegenstandes jederzeit einzustellen, solange die Ausübung dieses
      Rechts nicht einer Kündigung oder einem Widerruf dieser Lizenz (oder
      irgendeiner Weiterlizenzierung, die auf Grundlage dieser Lizenz bereits
      erfolgt ist bzw. zukünftig noch erfolgen muss) dient und diese Lizenz
      unter Berücksichtigung der oben zum Erlöschen genannten Bedingungen
      vollumfänglich wirksam bleibt.

   __1__ Sonstige Bestimmungen

      __1__ Jedes Mal wenn Sie den Schutzgegenstand für sich genommen oder als
      Teil eines Sammelwerkes verbreiten oder öffentlich wiedergeben, bietet
      der Lizenzgeber dem Empfänger eine Lizenz zu den gleichen Bedingungen und
      im gleichen Umfang an, wie Ihnen in Form dieser Lizenz.

      __1__ Jedes Mal wenn Sie eine Bearbeitung des Schutzgegenstandes
      verbreiten oder öffentlich wiedergeben, bietet der Lizenzgeber dem
      Empfänger eine Lizenz am ursprünglichen Schutzgegenstand zu den gleichen
      Bedingungen und im gleichen Umfang an, wie Ihnen in Form dieser Lizenz.

      __1__ Sollte eine Bestimmung dieser Lizenz unwirksam sein, so bleibt davon
      die Wirksamkeit der Lizenz im Übrigen unberührt.

      __1__ Keine Bestimmung dieser Lizenz soll als abbedungen und kein Verstoß
      gegen sie als zulässig gelten, solange die von dem Verzicht oder von dem
      Verstoß betroffene Seite nicht schriftlich zugestimmt hat.

      __1__ Diese Lizenz (zusammen mit in ihr ausdrücklich vorgesehenen
      Erlaubnissen, Mitteilungen und Zustimmungen, soweit diese tatsächlich
      vorliegen) stellt die vollständige Vereinbarung zwischen dem Lizenzgeber
      und Ihnen in Bezug auf den Schutzgegenstand dar. Es bestehen keine
      Abreden, Vereinbarungen oder Erklärungen in Bezug auf den
      Schutzgegenstand, die in dieser Lizenz nicht genannt sind.
      Rechtsgeschäftliche Änderungen des Verhältnisses zwischen dem
      Lizenzgeber und Ihnen sind nur über Modifikationen dieser Lizenz
      möglich. Der Lizenzgeber ist an etwaige zusätzliche, einseitig durch Sie
      übermittelte Bestimmungen nicht gebunden. Diese Lizenz kann nur durch
      schriftliche Vereinbarung zwischen Ihnen und dem Lizenzgeber modifiziert
      werden. Derlei Modifikationen wirken ausschließlich zwischen dem
      Lizenzgeber und Ihnen und wirken sich nicht auf die Dritten gemäß 8.a)
      und b) angebotenen Lizenzen aus.

      __1__ Sofern zwischen Ihnen und dem Lizenzgeber keine anderweitige
      Vereinbarung getroffen wurde und soweit Wahlfreiheit besteht, findet auf
      diesen Lizenzvertrag das Recht der Republik Österreich Anwendung.

Creative Commons Notice

Creative Commons ist nicht Partei dieser Lizenz und übernimmt keinerlei Gewähr
oder dergleichen in Bezug auf den Schutzgegenstand. Creative Commons haftet
Ihnen oder einer anderen Partei unter keinem rechtlichen Gesichtspunkt für
irgendwelche Schäden, die - abstrakt oder konkret, zufällig oder vorhersehbar
- im Zusammenhang mit dieser Lizenz entstehen. Unbeschadet der vorangegangen
beiden Sätze, hat Creative Commons alle Rechte und Pflichten eines
Lizenzgebers, wenn es sich ausdrücklich als Lizenzgeber im Sinne dieser Lizenz
bezeichnet.

Creative Commons gewährt den Parteien nur insoweit das Recht, das Logo und die
Marke "Creative Commons" zu nutzen, als dies notwendig ist, um der
Öffentlichkeit gegenüber kenntlich zu machen, dass der Schutzgegenstand unter
einer CCPL steht. Ein darüber hinaus gehender Gebrauch der Marke "Creative
Commons" oder einer verwandten Marke oder eines verwandten Logos bedarf der
vorherigen schriftlichen Zustimmung von Creative Commons. Jeder erlaubte
Gebrauch richtet sich nach der Creative Commons Marken-Nutzungs-Richtlinie in
der jeweils aktuellen Fassung, die von Zeit zu Zeit auf der Website
veröffentlicht oder auf andere Weise auf Anfrage zugänglich gemacht wird. Zur
Klarstellung: Die genannten Einschränkungen der Markennutzung sind nicht
Bestandteil dieser Lizenz.

Creative Commons kann kontaktiert werden über https:/creativecommons.org/.
`
const license_CC_BY_4_0_lre = `//**
CC-BY-4.0
https://creativecommons.org/licenses/by/4.0
**//

((Creative Commons))??

((
Creative Commons Legal Code
))??
((https://creativecommons.org/licenses/by/4.0
((/legalcode))??
))??

((
Attribution 4.0 International
))??
((https://creativecommons.org/licenses/by/4.0
((/legalcode))??
))??

((
Official translations of this license are available
in other languages.
))??

((
Creative Commons Corporation (“Creative Commons”) is not a law firm and does
not provide legal services or legal advice. Distribution of Creative Commons
public licenses does not create a lawyer-client or other relationship. Creative
Commons makes its licenses and related information available on an “as-is”
basis. Creative Commons gives no warranties regarding its licenses, any material
licensed under their terms and conditions, or any related information. Creative
Commons disclaims all liability for damages resulting from their use to the
fullest extent possible.
))??

((
Using Creative Commons Public Licenses

Creative Commons public licenses provide a standard set of terms and conditions
that creators and other rights holders may use to share original works of
authorship and other material subject to copyright and certain other rights
specified in the public license below. The following considerations are for
informational purposes only, are not exhaustive, and do not form part of our
licenses.

Considerations for licensors: Our public licenses are intended for use by those
authorized to give the public permission to use material in ways otherwise
restricted by copyright and certain other rights. Our licenses are irrevocable.
Licensors should read and understand the terms and conditions of the license
they choose before applying it. Licensors should also secure all rights
necessary before applying our licenses so that the public can reuse the material
as expected. Licensors should clearly mark any material not subject to the
license. This includes other CC-licensed material, or material used under an
exception or limitation to copyright. More considerations for licensors.

Considerations for the public: By using one of our public licenses, a licensor
grants the public permission to use the licensed material under specified terms
and conditions. If the licensor’s permission is not necessary for any
reason–for example, because of any applicable exception or limitation to
copyright–then that use is not regulated by the license. Our licenses grant
only permissions under copyright and certain other rights that a licensor has
authority to grant. Use of the licensed material may still be restricted for
other reasons, including because others have copyright or other rights in the
material. A licensor may make special requests, such as asking that all changes
be marked or described. Although not required by our licenses, you are
encouraged to respect those requests where reasonable.
More considerations for the public.
))??

((
Creative Commons Attribution 4.0 International Public License
))??
((https://creativecommons.org/licenses/by/4.0
((/legalcode))??
))??

By exercising the Licensed Rights (defined below), You accept and agree to be
bound by the terms and conditions of this Creative Commons Attribution 4.0
International Public License ("Public License"). To the extent this Public
License may be interpreted as a contract, You are granted the Licensed Rights in
consideration of Your acceptance of these terms and conditions, and the Licensor
grants You such rights in consideration of benefits the Licensor receives from
making the Licensed Material available under these terms and conditions.

Section 1 – Definitions.

__1__ Adapted Material means material subject to Copyright and Similar Rights
that is derived from or based upon the Licensed Material and in which the
Licensed Material is translated, altered, arranged, transformed, or otherwise
modified in a manner requiring permission under the Copyright and Similar Rights
held by the Licensor. For purposes of this Public License, where the Licensed
Material is a musical work, performance, or sound recording, Adapted Material is
always produced where the Licensed Material is synched in timed relation with a
moving image.

__1__ Adapter's License means the license You apply to Your Copyright and
Similar Rights in Your contributions to Adapted Material in accordance with the
terms and conditions of this Public License.

__1__ Copyright and Similar Rights means copyright and/or similar rights closely
related to copyright including, without limitation, performance, broadcast,
sound recording, and Sui Generis Database Rights, without regard to how the
rights are labeled or categorized. For purposes of this Public License, the
rights specified in Section 2(b)(1)-(2) are not Copyright and Similar Rights.

__1__ Effective Technological Measures means those measures that, in the absence
of proper authority, may not
be circumvented under laws fulfilling obligations under Article 11 of the WIPO
Copyright Treaty adopted on December 20, 1996, and/or similar
international agreements.

__1__ Exceptions and Limitations means fair use, fair dealing, and/or any other
exception or limitation to Copyright and Similar Rights that applies to Your use
of the Licensed Material.

__1__ Licensed Material means the artistic or literary work, database, or other
material to which the Licensor applied this Public License.

__1__ Licensed Rights means the rights granted to You subject to the terms and
conditions of this Public License, which are limited to all Copyright and
Similar Rights that apply to Your use of the Licensed Material and that the
Licensor has authority to license.

__1__ Licensor means the individual(s) or entity(ies) granting rights under this
Public License.

__1__ Share means to provide material to the public by any means or process that
requires permission under the Licensed Rights, such as reproduction, public
display, public performance, distribution, dissemination, communication, or
importation, and to make material available to the public including in ways that
members of the public may access the material from a place and at a time
individually chosen by them.

__1__ Sui Generis Database Rights means rights other than copyright resulting
from Directive 96/9/EC of the European Parliament and of the Council of 11 March
1996 on the legal protection of databases, as amended and/or succeeded, as well
as other essentially equivalent rights anywhere in the world.

__1__ You means the individual or entity exercising the Licensed Rights under
this Public License. Your has a corresponding meaning.

Section 2 – Scope.

__1__ License grant.

__1__ Subject to the terms and conditions of this Public License, the Licensor
hereby grants You a worldwide, royalty-free, non-sublicensable, non-exclusive,
irrevocable license to exercise the Licensed Rights in the Licensed Material
to:

__1__ reproduce and Share the Licensed Material, in whole or in part; and

__1__ produce, reproduce, and Share Adapted Material.

__1__ Exceptions and Limitations. For the avoidance of doubt, where Exceptions
and Limitations apply to Your use, this Public License does not apply, and You
do not need to comply with its terms and conditions.

__1__ Term. The term of this Public License is specified in Section 6(a).

__1__ Media and formats; technical modifications allowed. The Licensor
authorizes You to exercise the Licensed Rights in all media and formats whether
now known or hereafter created, and to make technical modifications necessary to
do so. The Licensor waives and/or agrees not to assert any right or authority to
forbid You from making technical modifications necessary to exercise the
Licensed Rights, including technical modifications necessary to circumvent
Effective Technological Measures. For purposes of this Public License, simply
making modifications authorized by this Section 2(a)(4) never produces Adapted
Material.

__1__ Downstream recipients.

__1__ Offer from the Licensor – Licensed Material. Every recipient of the
Licensed Material automatically receives an  offer from the Licensor to exercise
the Licensed Rights under the terms and conditions of this Public License.

__1__ No downstream restrictions. You may not offer or impose any additional or
different terms or  conditions on, or apply any Effective Technological Measures
to, the Licensed Material if doing so restricts exercise of the Licensed Rights
by any recipient of the Licensed Material.

__1__ No endorsement. Nothing in this Public License constitutes or may be
construed as permission to assert or imply that You are, or that Your use of the
Licensed Material is, connected with, or sponsored, endorsed, or granted
official status by, the Licensor or others designated to receive attribution as
provided in Section 3(a)(1)(A)(i).

__1__
Other rights.

__1__ Moral rights, such as the right of integrity, are not licensed under this
Public License, nor are publicity, privacy, and/or other similar personality
rights; however, to the extent possible, the Licensor waives and/or agrees not
to assert any such rights held by the Licensor to the limited extent necessary
to allow You to exercise the Licensed Rights, but not otherwise.

__1__ Patent and trademark rights are not licensed under this Public License.

__1__ To the extent possible, the Licensor waives any right to collect royalties
from You for the exercise of the Licensed Rights, whether directly or through a
collecting society under any voluntary or waivable statutory or compulsory
licensing scheme. In all other cases the Licensor expressly reserves any right
to collect such royalties.

Section 3 – License Conditions.

Your exercise of the Licensed Rights is expressly made subject to the following
conditions.

__1__
Attribution.

__1__
If You Share the Licensed Material (including in modified form), You must:

__1__ retain the following if it is supplied by the Licensor with the Licensed
Material:

__1__ identification of the creator(s) of the Licensed Material and any others
designated to receive attribution, in any reasonable manner requested  by the
Licensor (including by pseudonym if designated);

__1__ a copyright notice;

__1__ a notice that refers to this Public License;

__1__ a notice that refers to the disclaimer of warranties;

__1__ a URI or hyperlink to the Licensed Material to the extent reasonably
practicable;

__1__ indicate if You modified the Licensed Material and retain an indication of
any previous modifications; and

__1__ indicate the Licensed Material is licensed under this Public License,
and include the text of, or the URI or hyperlink to, this Public
License.

__1__ You may satisfy the conditions in Section 3(a)(1) in any reasonable manner
based on the medium, means, and context in which You Share the Licensed
Material. For example, it may be reasonable to satisfy the conditions by
providing a URI or hyperlink to a resource that includes the required
information.

__1__ If requested by the Licensor, You must remove any of the information
required by Section 3(a)(1)(A) to the extent reasonably practicable.

__1__ If You Share Adapted Material You produce, the Adapter's License You apply
must not prevent recipients of the Adapted Material from complying with this
Public License.

Section 4 – Sui Generis Database Rights.

Where the Licensed Rights include Sui Generis Database Rights that apply to Your
use of the Licensed Material:

__1__ for the avoidance of doubt, Section 2(a)(1) grants You the right to
extract, reuse, reproduce, and Share all or a substantial portion of the
contents of the database;

__1__ if You include all or a substantial portion of the database contents in a
database in which You have Sui Generis Database Rights, then the database in
which You have Sui Generis Database Rights (but not its individual contents) is
Adapted Material; and

__1__ You must comply with the conditions in Section 3(a) if You Share all or a
substantial portion of the contents of the database.

__1__ For the avoidance of doubt, this Section 4 supplements and
does not replace Your obligations under this Public License
where the Licensed Rights include other Copyright and Similar
Rights.

Section 5 – Disclaimer of Warranties and Limitation of Liability.

__1__ Unless otherwise separately undertaken by the Licensor, to the extent
possible, the Licensor offers the Licensed Material as-is and as-available, and
makes no representations or warranties of any kind concerning the Licensed
Material, whether express, implied, statutory, or other. This includes, without
limitation, warranties of title, merchantability, fitness for a particular
purpose, non-infringement, absence of latent or other defects, accuracy, or the
presence or absence of errors, whether or not known or discoverable. Where
disclaimers of warranties are not allowed in full or in part, this disclaimer
may not apply to You.

__1__ To the extent possible, in no event will the Licensor be liable to You on
any legal theory (including, without limitation, negligence) or otherwise for
any direct, special, indirect, incidental, consequential, punitive, exemplary,
or other losses, costs, expenses, or damages arising out of this Public License
or use of the Licensed Material, even if the Licensor has been advised of the
possibility of such losses, costs, expenses, or damages. Where a limitation of
liability is not allowed in full or in part, this limitation may not apply to
You.

__1__ The disclaimer of warranties and limitation of liability provided above
shall be interpreted in a manner that, to the extent possible, most closely
approximates an absolute disclaimer and waiver of all liability.

Section 6 – Term and Termination.

__1__ This Public License applies for the term of the Copyright and Similar
Rights licensed here. However, if You fail to comply with this Public License,
then Your rights under this Public License terminate automatically.

__1__

Where Your right to use the Licensed Material has terminated under Section 6(a),
it reinstates:

__1__ automatically as of the date the violation is cured, provided it is cured
within 30 days of Your discovery of the violation; or

__1__ upon express reinstatement by the Licensor.

__1__ For the avoidance of doubt, this Section 6(b) does not
affect any right the Licensor may have to seek remedies for
Your violations of this Public License.

__1__ For the avoidance of doubt, the Licensor may also offer the Licensed
Material under separate terms or conditions or stop distributing the Licensed
Material at any time; however, doing so will not terminate this Public License.

__1__ Sections 1, 5, 6, 7, and 8 survive termination of this Public License.

Section 7 – Other Terms and Conditions.

__1__ The Licensor shall not be bound by any additional or different terms or
conditions communicated by You unless expressly agreed.

__1__ Any arrangements, understandings, or agreements regarding the Licensed
Material not stated herein are separate from and independent of the terms and
conditions of this Public License.

Section 8 – Interpretation.

__1__ For the avoidance of doubt, this Public License does not, and shall not be
interpreted to, reduce, limit, restrict, or impose conditions on any use of the
Licensed Material that could lawfully be made without permission under this
Public License.

__1__ To the extent possible, if any provision of this Public License is deemed
unenforceable, it shall be automatically reformed to the minimum extent
necessary to make it enforceable. If the provision cannot be reformed, it shall
be severed from this Public License without affecting the enforceability of the
remaining terms and conditions.

__1__ No term or condition of this Public License will be waived and no failure
to comply consented to unless expressly agreed to by the Licensor.

__1__ Nothing in this Public License constitutes or may be interpreted as a
limitation upon, or waiver of, any privileges and immunities that apply to the
Licensor or You, including from the legal processes of any jurisdiction or
authority.

((
Creative Commons is not a party to its public licenses. Notwithstanding,
Creative Commons may elect to apply one of its public licenses to material it
publishes and in those instances will be considered the “Licensor.” The text
of the Creative Commons public licenses is dedicated to the public domain under
the CC0 Public Domain Dedication. Except for the limited purpose of indicating
that material is shared under a Creative Commons public license or as otherwise
permitted by the Creative Commons policies published at
creativecommons.org/policies, Creative Commons does not authorize the use of the
trademark “Creative Commons” or any other trademark or logo of Creative
Commons without its prior written consent including, without limitation, in
connection with any unauthorized modifications to any of its public licenses or
any other arrangements, understandings, or agreements concerning use of licensed
material. For the avoidance of doubt, this paragraph does not form part of the
public licenses.
Creative Commons may be contacted at creativecommons.org.
))??

((
Additional languages available:

العربية,
čeština,
Deutsch,
Ελληνικά,
Español,
euskara,
suomeksi,
français,
hrvatski,
Bahasa Indonesia,
italiano,
日本語,
한국어,
Lietuvių,
latviski,
te reo Māori,
Nederlands,
norsk,
polski,
português,
română,
русский,
Slovenščina,
svenska,
Türkçe,
українська,
中文,
華語.

Please read the FAQ for more information about official translations.
))??
`
const license_CC_BY_NC_1_0_lre = `//**
CC-BY-NC-1.0
https://creativecommons.org/licenses/by-nc/1.0
**//

((Creative Commons))??

Creative Commons

Attribution-NonCommercial 1.0

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE
LEGAL
SERVICES. DISTRIBUTION OF THIS DRAFT LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION
ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING
FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE").  THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW.  ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE.  THE LICENSOR GRANTS YOU THE RIGHTS
CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole.  A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant.  Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to create and reproduce Derivative Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission Derivative Works;

The above rights may be exercised in all media and formats whether now known or
hereafter devised.  The above rights include the right to make such
modifications as are technically necessary to exercise the rights in other media
and formats.  All rights not expressly granted by Licensor are hereby reserved.

4. Restrictions. The license granted in Section 3 above is expressly made
subject to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform.  You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder.  You may not sublicense the Work.  You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement.  The above applies to the Work as incorporated in a
Collective Work, but this does not require the Collective Work apart from the
Work itself to be made subject to the terms of this License. If You create a
Collective Work, upon notice from any Licensor You must, to the extent
practicable, remove from the Collective Work any reference to such Licensor or
the Original Author, as requested.  If You create a Derivative Work, upon notice
from any Licensor You must, to the extent practicable, remove from the
Derivative Work any reference to such Licensor or the Original Author, as
requested.

__1__
You may not exercise any of the rights granted to You in Section 3 above in any
manner that is primarily intended for or directed toward commercial advantage or
private monetary compensation.  The exchange of the Work for other copyrighted
works by means of digital file-sharing or otherwise shall not be considered to
be intended for or directed toward commercial advantage or private monetary
compensation, provided there is no payment of any monetary compensation in
connection with the exchange of copyrighted works.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or any Derivative Works or Collective Works, You must keep
intact all copyright notices for the Work and give the Original Author credit
reasonable to the medium or means You are utilizing by conveying the name (or
pseudonym if applicable) of the Original Author if supplied; the title of the
Work if supplied; in the case of a Derivative Work, a credit identifying the use
of the Work in the Derivative Work (e.g., "French translation of the Work by
Original Author," or "Screenplay based on original Work by Original Author").
Such credit may be implemented in any reasonable manner; provided, however, that
in the case of a Derivative Work or Collective Work, at a minimum such credit
will appear where any other comparable authorship credit appears and in a manner
at least as prominent as such other comparable authorship credit.

5. Representations, Warranties and Disclaimer

__1__
By offering the Work for public release under this License, Licensor represents
and warrants that, to the best of Licensor's knowledge after reasonable
inquiry:

__1__
Licensor has secured all rights in the Work necessary to grant the license
rights hereunder and to permit the lawful exercise of the rights granted
hereunder without You having any obligation to pay any royalties, compulsory
license fees, residuals or any other payments;

__1__
The Work does not infringe the copyright, trademark, publicity rights, common
law rights or any other right of any third party or constitute defamation,
invasion of privacy or other tortious injury to any third party.

__1__
EXCEPT AS EXPRESSLY STATED IN THIS LICENSE OR OTHERWISE AGREED IN WRITING OR
REQUIRED BY APPLICABLE LAW, THE WORK IS LICENSED ON AN "AS IS" BASIS, WITHOUT
WARRANTIES OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION,
ANY WARRANTIES REGARDING THE CONTENTS OR ACCURACY OF THE WORK.

6. Limitation on Liability.  EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW,
AND EXCEPT FOR DAMAGES ARISING FROM LIABILITY TO A THIRD PARTY RESULTING FROM
BREACH OF THE WARRANTIES IN SECTION 5, IN NO EVENT WILL LICENSOR BE LIABLE TO
YOU ON ANY LEGAL THEORY FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR
EXEMPLARY DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK, EVEN IF
LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Derivative Works or Collective Works from You under this License,
however, will not have their licenses terminated provided such individuals or
entities remain in full compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work).  Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
Each time You distribute or publicly digitally perform a Derivative Work,
Licensor offers to the recipient a license to the original Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here.  There are no understandings, agreements or
representations with respect to the Work not specified here.  Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work.  Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license.  Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons.  Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_NC_2_0_lre = `//**
CC-BY-NC-2.0
https://creativecommons.org/licenses/by-nc/2.0
**//

((Creative Commons))??

Creative Commons

Attribution-NonCommercial 2.0

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE
LEGAL
SERVICES. DISTRIBUTION OF THIS LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION
ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING
FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE").  THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW.  ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE.  THE LICENSOR GRANTS YOU THE RIGHTS
CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole.  A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License. For
the avoidance of doubt, where the Work is a musical composition or sound
recording, the synchronization of the Work in timed-relation with a moving image
("synching") will be considered a Derivative Work for the purpose of this
License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant.  Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to create and reproduce Derivative Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission Derivative Works;

The above rights may be exercised in all media and formats whether now known or
hereafter devised.  The above rights include the right to make such
modifications as are technically necessary to exercise the rights in other media
and formats.  All rights not expressly granted by Licensor are hereby reserved,
including but not limited to the rights set forth in Sections 4(d) and 4(e).

4. Restrictions.The license granted in Section 3 above is expressly made subject
to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform.  You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder.  You may not sublicense the Work.  You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement.  The above applies to the Work as incorporated in a
Collective Work, but this does not require the Collective Work apart from the
Work itself to be made subject to the terms of this License. If You create a
Collective Work, upon notice from any Licensor You must, to the extent
practicable, remove from the Collective Work any reference to such Licensor or
the Original Author, as requested.  If You create a Derivative Work, upon notice
from any Licensor You must, to the extent practicable, remove from the
Derivative Work any reference to such Licensor or the Original Author, as
requested.

__1__
You may not exercise any of the rights granted to You in Section 3 above in any
manner that is primarily intended for or directed toward commercial advantage or
private monetary compensation.  The exchange of the Work for other copyrighted
works by means of digital file-sharing or otherwise shall not be considered to
be intended for or directed toward commercial advantage or private monetary
compensation, provided there is no payment of any monetary compensation in
connection with the exchange of copyrighted works.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or any Derivative Works or Collective Works, You must keep
intact all copyright notices for the Work and give the Original Author credit
reasonable to the medium or means You are utilizing by conveying the name (or
pseudonym if applicable) of the Original Author if supplied; the title of the
Work if supplied; to the extent reasonably practicable, the Uniform Resource
Identifier, if any, that Licensor specifies to be associated with the Work,
unless such URI does not refer to the copyright notice or licensing information
for the Work; and in the case of a Derivative Work, a credit identifying the use
of the Work in the Derivative Work (e.g., "French translation of the Work by
Original Author," or "Screenplay based on original Work by Original Author").
Such credit may be implemented in any reasonable manner; provided, however, that
in the case of a Derivative Work or Collective Work, at a minimum such credit
will appear where any other comparable authorship credit appears and in a manner
at least as prominent as such other comparable authorship credit.

__1__
For the avoidance of doubt, where the Work is a musical composition:

__1__ Performance Royalties Under Blanket Licenses. Licensor reserves the
exclusive right to collect, whether individually or via a performance rights
society (e.g. ASCAP, BMI, SESAC), royalties for the public performance or public
digital performance (e.g. webcast) of the Work if that performance is primarily
intended for or directed toward commercial advantage or private monetary
compensation.

__1__ Mechanical Rights and Statutory Royalties. Licensor reserves the exclusive
right to collect, whether individually or via a music rights agency or
designated agent (e.g. Harry Fox Agency), royalties for any phonorecord You
create from the Work ("cover version") and distribute, subject to the compulsory
license created by 17 USC Section 115 of the US Copyright Act (or the equivalent
in other jurisdictions), if Your distribution of such cover version is primarily
intended for or directed toward commercial advantage or private monetary
compensation.

__1__ Webcasting Rights and Statutory Royalties.__1__ For the avoidance of
doubt, where the Work is a sound recording, Licensor reserves the exclusive
right to collect, whether individually or via a performance-rights society (e.g.
SoundExchange), royalties for the public digital performance (e.g. webcast) of
the Work, subject to the compulsory license created by 17 USC Section 114 of the
US Copyright Act (or the equivalent in other jurisdictions), if Your public
digital performance is primarily intended for or directed toward commercial
advantage or private monetary compensation.

5. Representations, Warranties and Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN WRITING, LICENSOR OFFERS
THE WORK AS-IS AND MAKES NO REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING
THE WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT
LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY, FITNESS FOR A PARTICULAR
PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY,
OR THE PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE. SOME
JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO SUCH
EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability.  EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN
NO EVENT WILL LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY SPECIAL,
INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY DAMAGES ARISING OUT OF THIS
LICENSE OR THE USE OF THE WORK, EVEN IF LICENSOR HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Derivative Works or Collective Works from You under this License,
however, will not have their licenses terminated provided such individuals or
entities remain in full compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work).  Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
Each time You distribute or publicly digitally perform a Derivative Work,
Licensor offers to the recipient a license to the original Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here.  There are no understandings, agreements or
representations with respect to the Work not specified here.  Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work.  Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license.  Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons.  Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_NC_2_5_lre = `//**
CC-BY-NC-2.5
https://creativecommons.org/licenses/by-nc/2.5
**//

((Creative Commons))??

Creative Commons

Attribution-NonCommercial 2.5

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE
LEGAL
SERVICES. DISTRIBUTION OF THIS LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION
ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING
FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE").  THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW.  ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE.  THE LICENSOR GRANTS YOU THE RIGHTS
CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole.  A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License. For
the avoidance of doubt, where the Work is a musical composition or sound
recording, the synchronization of the Work in timed-relation with a moving image
("synching") will be considered a Derivative Work for the purpose of this
License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant.  Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to create and reproduce Derivative Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission Derivative Works;

The above rights may be exercised in all media and formats whether now known or
hereafter devised.  The above rights include the right to make such
modifications as are technically necessary to exercise the rights in other media
and formats.  All rights not expressly granted by Licensor are hereby reserved,
including but not limited to the rights set forth in Sections 4(d) and 4(e).

4. Restrictions.The license granted in Section 3 above is expressly made subject
to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform.  You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder.

__1__ //** Non-standard paragraph break **//
You may not sublicense the Work.  You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement.  The above applies to the Work as incorporated in a
Collective Work, but this does not require the Collective Work apart from the
Work itself to be made subject to the terms of this License. If You create a
Collective Work, upon notice from any Licensor You must, to the extent
practicable, remove from the Collective Work any credit as required by clause
4(c), as requested.  If You create a Derivative Work, upon notice from any
Licensor You must, to the extent practicable, remove from the Derivative Work
any credit as required by clause 4(c), as requested.

__1__
You may not exercise any of the rights granted to You in Section 3 above in any
manner that is primarily intended for or directed toward commercial advantage or
private monetary compensation.  The exchange of the Work for other copyrighted
works by means of digital file-sharing or otherwise shall not be considered to
be intended for or directed toward commercial advantage or private monetary
compensation, provided there is no payment of any monetary compensation in
connection with the exchange of copyrighted works.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or any Derivative Works or Collective Works, You must keep
intact all copyright notices for the Work and provide, reasonable to the medium
or means You are utilizing: (i) the name of Original Author (or pseudonym, if
applicable) if supplied, and/or (ii) if the Original Author and/or Licensor
designate another party or parties (e.g. a sponsor institute, publishing entity,
journal) for attribution in Licensor's copyright notice, terms of service or by
other reasonable means, the name of such party or parties; the title of the Work
if supplied; to the extent reasonably practicable, the Uniform Resource
Identifier, if any, that Licensor specifies to be associated with the Work,
unless such URI does not refer to the copyright notice or licensing information
for the Work; and in the case of a Derivative Work, a credit identifying the use
of the Work in the Derivative Work (e.g., "French translation of the Work by
Original Author," or "Screenplay based on original Work by Original Author").
Such credit may be implemented in any reasonable manner; provided, however, that
in the case of a Derivative Work or Collective Work, at a minimum such credit
will appear where any other comparable authorship credit appears and in a manner
at least as prominent as such other comparable authorship credit.

__1__
For the avoidance of doubt, where the Work is a musical composition:

__1__ Performance Royalties Under Blanket Licenses. Licensor reserves the
exclusive right to collect, whether individually or via a performance rights
society (e.g. ASCAP, BMI, SESAC), royalties for the public performance or public
digital performance (e.g. webcast) of the Work if that performance is primarily
intended for or directed toward commercial advantage or private monetary
compensation.

__1__ Mechanical Rights and Statutory Royalties. Licensor reserves the exclusive
right to collect, whether individually or via a music rights agency or
designated agent (e.g. Harry Fox Agency), royalties for any phonorecord You
create from the Work ("cover version") and distribute, subject to the compulsory
license created by 17 USC Section 115 of the US Copyright Act (or the equivalent
in other jurisdictions), if Your distribution of such cover version is primarily
intended for or directed toward commercial advantage or private monetary
compensation.

__1__ Webcasting Rights and Statutory Royalties.__1__ For the avoidance of
doubt, where the Work is a sound recording, Licensor reserves the exclusive
right to collect, whether individually or via a performance-rights society (e.g.
SoundExchange), royalties for the public digital performance (e.g. webcast) of
the Work, subject to the compulsory license created by 17 USC Section 114 of the
US Copyright Act (or the equivalent in other jurisdictions), if Your public
digital performance is primarily intended for or directed toward commercial
advantage or private monetary compensation.

5. Representations, Warranties and Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN WRITING, LICENSOR OFFERS
THE WORK AS-IS AND MAKES NO REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING
THE WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT
LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY, FITNESS FOR A PARTICULAR
PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY,
OR THE PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE. SOME
JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO SUCH
EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability.  EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN
NO EVENT WILL LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY SPECIAL,
INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY DAMAGES ARISING OUT OF THIS
LICENSE OR THE USE OF THE WORK, EVEN IF LICENSOR HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Derivative Works or Collective Works from You under this License,
however, will not have their licenses terminated provided such individuals or
entities remain in full compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work).  Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
Each time You distribute or publicly digitally perform a Derivative Work,
Licensor offers to the recipient a license to the original Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here.  There are no understandings, agreements or
representations with respect to the Work not specified here.  Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work.  Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license.  Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons.  Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_NC_3_0_lre = `//**
CC-BY-NC-3.0
https://creativecommons.org/licenses/by-nc/3.0
**//

((Creative Commons))??

((
Creative Commons Legal Code
))??
((https://creativecommons.org/licenses/by-nc/3.0
((/legalcode))??
))??

((
Attribution-NonCommercial 3.0 Unported
))??
((https://creativecommons.org/licenses/by-nc/3.0
((/legalcode))??
))??

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES
NOT PROVIDE LEGAL SERVICES. DISTRIBUTION OF THIS LICENSE
DOES NOT CREATE AN ATTORNEY-CLIENT RELATIONSHIP. CREATIVE
COMMONS PROVIDES THIS INFORMATION ON AN "AS-IS" BASIS.
CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES
RESULTING FROM ITS USE.
))??

((
((License))??
))??
((https://creativecommons.org/licenses/by-nc/3.0
((/legalcode))??
))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS
OF THIS CREATIVE COMMONS PUBLIC LICENSE ("CCPL" OR
"LICENSE"). THE WORK IS PROTECTED BY COPYRIGHT AND/OR OTHER
APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS
AUTHORIZED UNDER THIS LICENSE OR COPYRIGHT LAW IS
PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU
ACCEPT AND AGREE TO BE BOUND BY THE TERMS OF THIS LICENSE.
TO THE EXTENT THIS LICENSE MAY BE CONSIDERED TO BE A
CONTRACT, THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED HERE
IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND
CONDITIONS.

1. Definitions

__1__ "Adaptation" means a work based upon
the Work, or upon the Work and other pre-existing works,
such as a translation, adaptation, derivative work,
arrangement of music or other alterations of a literary
or artistic work, or phonogram or performance and
includes cinematographic adaptations or any other form in
which the Work may be recast, transformed, or adapted
including in any form recognizably derived from the
original, except that a work that constitutes a
Collection will not be considered an Adaptation for the
purpose of this License. For the avoidance of doubt,
where the Work is a musical work, performance or
phonogram, the synchronization of the Work in
timed-relation with a moving image ("synching") will be
considered an Adaptation for the purpose of this
License.

__1__ "Collection" means a collection of
literary or artistic works, such as encyclopedias and
anthologies, or performances, phonograms or broadcasts,
or other works or subject matter other than works listed
in Section 1(f) below, which, by reason of the selection
and arrangement of their contents, constitute
intellectual creations, in which the Work is included in
its entirety in unmodified form along with one or more
other contributions, each constituting separate and
independent works in themselves, which together are
assembled into a collective whole. A work that
constitutes a Collection will not be considered an
Adaptation (as defined above) for the purposes of this
License.

__1__ "Distribute" means to make available
to the public the original and copies of the Work or
Adaptation, as appropriate, through sale or other
transfer of ownership.

__1__ "Licensor" means the individual,
individuals, entity or entities that offer(s) the Work
under the terms of this License.

__1__ "Original Author" means, in the case
of a literary or artistic work, the individual,
individuals, entity or entities who created the Work or
if no individual or entity can be identified, the
publisher; and in addition (i) in the case of a
performance the actors, singers, musicians, dancers, and
other persons who act, sing, deliver, declaim, play in,
interpret or otherwise perform literary or artistic works
or expressions of folklore; (ii) in the case of a
phonogram the producer being the person or legal entity
who first fixes the sounds of a performance or other
sounds; and, (iii) in the case of broadcasts, the
organization that transmits the broadcast.

__1__ "Work" means the literary and/or
artistic work offered under the terms of this License
including without limitation any production in the
literary, scientific and artistic domain, whatever may be
the mode or form of its expression including digital
form, such as a book, pamphlet and other writing; a
lecture, address, sermon or other work of the same
nature; a dramatic or dramatico-musical work; a
choreographic work or entertainment in dumb show; a
musical composition with or without words; a
cinematographic work to which are assimilated works
expressed by a process analogous to cinematography; a
work of drawing, painting, architecture, sculpture,
engraving or lithography; a photographic work to which
are assimilated works expressed by a process analogous to
photography; a work of applied art; an illustration, map,
plan, sketch or three-dimensional work relative to
geography, topography, architecture or science; a
performance; a broadcast; a phonogram; a compilation of
data to the extent it is protected as a copyrightable
work; or a work performed by a variety or circus
performer to the extent it is not otherwise considered a
literary or artistic work.

__1__ "You" means an individual or entity
exercising rights under this License who has not
previously violated the terms of this License with
respect to the Work, or who has received express
permission from the Licensor to exercise rights under
this License despite a previous violation.

__1__ "Publicly Perform" means to perform
public recitations of the Work and to communicate to the
public those public recitations, by any means or process,
including by wire or wireless means or public digital
performances; to make available to the public Works in
such a way that members of the public may access these
Works from a place and at a place individually chosen by
them; to perform the Work to the public by any means or
process and the communication to the public of the
performances of the Work, including by public digital
performance; to broadcast and rebroadcast the Work by any
means including signs, sounds or images.

__1__ "Reproduce" means to make copies of
the Work by any means including without limitation by
sound or visual recordings and the right of fixation and
reproducing fixations of the Work, including storage of a
protected performance or phonogram in digital form or
other electronic medium.

2. Fair Dealing Rights. Nothing in this
License is intended to reduce, limit, or restrict any uses
free from copyright or rights arising from limitations or
exceptions that are provided for in connection with the
copyright protection under copyright law or other
applicable laws.

3. License Grant. Subject to the terms
and conditions of this License, Licensor hereby grants You
a worldwide, royalty-free, non-exclusive, perpetual (for
the duration of the applicable copyright) license to
exercise the rights in the Work as stated below:

__1__ to Reproduce the Work, to incorporate the Work into
one or more Collections, and to Reproduce the Work as
incorporated in the Collections;

__1__ to create and Reproduce Adaptations provided that any
such Adaptation, including any translation in any medium,
takes reasonable steps to clearly label, demarcate or
otherwise identify that changes were made to the original
Work. For example, a translation could be marked "The
original work was translated from English to Spanish," or
a modification could indicate "The original work has been
modified.";

__1__ to Distribute and Publicly Perform the Work including
as incorporated in Collections; and,

__1__ to Distribute and Publicly Perform Adaptations.

The above rights may be exercised in all media and
formats whether now known or hereafter devised. The above
rights include the right to make such modifications as are
technically necessary to exercise the rights in other media
and formats. Subject to Section 8(f), all rights not
expressly granted by Licensor are hereby reserved,
including but not limited to the rights set forth in
Section 4(d).

4. Restrictions. The license granted in
Section 3 above is expressly made subject to and limited by
the following restrictions:

__1__ You may Distribute or Publicly Perform the Work only
under the terms of this License. You must include a copy
of, or the Uniform Resource Identifier (URI) for, this
License with every copy of the Work You Distribute or
Publicly Perform. You may not offer or impose any terms
on the Work that restrict the terms of this License or
the ability of the recipient of the Work to exercise the
rights granted to that recipient under the terms of the
License. You may not sublicense the Work. You must keep
intact all notices that refer to this License and to the
disclaimer of warranties with every copy of the Work You
Distribute or Publicly Perform. When You Distribute or
Publicly Perform the Work, You may not impose any
effective technological measures on the Work that
restrict the ability of a recipient of the Work from You
to exercise the rights granted to that recipient under
the terms of the License. This Section 4(a) applies to
the Work as incorporated in a Collection, but this does
not require the Collection apart from the Work itself to
be made subject to the terms of this License. If You
create a Collection, upon notice from any Licensor You
must, to the extent practicable, remove from the
Collection any credit as required by Section 4(c), as
requested. If You create an Adaptation, upon notice from
any Licensor You must, to the extent practicable, remove
from the Adaptation any credit as required by Section
4(c), as requested.

__1__ You may not exercise any of the rights granted to You
in Section 3 above in any manner that is primarily
intended for or directed toward commercial advantage or
private monetary compensation. The exchange of the Work
for other copyrighted works by means of digital
file-sharing or otherwise shall not be considered to be
intended for or directed toward commercial advantage or
private monetary compensation, provided there is no
payment of any monetary compensation in connection with
the exchange of copyrighted works.

__1__ If You Distribute, or Publicly Perform the Work or
any Adaptations or Collections, You must, unless a
request has been made pursuant to Section 4(a), keep
intact all copyright notices for the Work and provide,
reasonable to the medium or means You are utilizing: (i)
the name of the Original Author (or pseudonym, if
applicable) if supplied, and/or if the Original Author
and/or Licensor designate another party or parties (e.g.,
a sponsor institute, publishing entity, journal) for
attribution ("Attribution Parties") in Licensor's
copyright notice, terms of service or by other reasonable
means, the name of such party or parties; (ii) the title
of the Work if supplied; (iii) to the extent reasonably
practicable, the URI, if any, that Licensor specifies to
be associated with the Work, unless such URI does not
refer to the copyright notice or licensing information
for the Work; and, (iv) consistent with Section 3(b), in
the case of an Adaptation, a credit identifying the use
of the Work in the Adaptation (e.g., "French translation
of the Work by Original Author," or "Screenplay based on
original Work by Original Author"). The credit required
by this Section 4(c) may be implemented in any reasonable
manner; provided, however, that in the case of a
Adaptation or Collection, at a minimum such credit will
appear, if a credit for all contributing authors of the
Adaptation or Collection appears, then as part of these
credits and in a manner at least as prominent as the
credits for the other contributing authors. For the
avoidance of doubt, You may only use the credit required
by this Section for the purpose of attribution in the
manner set out above and, by exercising Your rights under
this License, You may not implicitly or explicitly assert
or imply any connection with, sponsorship or endorsement
by the Original Author, Licensor and/or Attribution
Parties, as appropriate, of You or Your use of the Work,
without the separate, express prior written permission of
the Original Author, Licensor and/or Attribution
Parties.

__1__

For the avoidance of doubt:

__1__ Non-waivable Compulsory License
Schemes. In those jurisdictions in which the
right to collect royalties through any statutory or
compulsory licensing scheme cannot be waived, the
Licensor reserves the exclusive right to collect such
royalties for any exercise by You of the rights
granted under this License;

__1__ Waivable Compulsory License
Schemes. In those jurisdictions in which the
right to collect royalties through any statutory or
compulsory licensing scheme can be waived, the
Licensor reserves the exclusive right to collect such
royalties for any exercise by You of the rights
granted under this License if Your exercise of such
rights is for a purpose or use which is otherwise
than noncommercial as permitted under Section 4(b)
and otherwise waives the right to collect royalties
through any statutory or compulsory licensing scheme;
and,

__1__ Voluntary License Schemes. The
Licensor reserves the right to collect royalties,
whether individually or, in the event that the
Licensor is a member of a collecting society that
administers voluntary licensing schemes, via that
society, from any exercise by You of the rights
granted under this License that is for a purpose or
use which is otherwise than noncommercial as
permitted under Section 4(c).

__1__ Except as otherwise agreed in writing by the Licensor
or as may be otherwise permitted by applicable law, if
You Reproduce, Distribute or Publicly Perform the Work
either by itself or as part of any Adaptations or
Collections, You must not distort, mutilate, modify or
take other derogatory action in relation to the Work
which would be prejudicial to the Original Author's honor
or reputation. Licensor agrees that in those
jurisdictions (e.g. Japan), in which any exercise of the
right granted in Section 3(b) of this License (the right
to make Adaptations) would be deemed to be a distortion,
mutilation, modification or other derogatory action
prejudicial to the Original Author's honor and
reputation, the Licensor will waive or not assert, as
appropriate, this Section, to the fullest extent
permitted by the applicable national law, to enable You
to reasonably exercise Your right under Section 3(b) of
this License (right to make Adaptations) but not
otherwise.

5. Representations, Warranties and
Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN
WRITING, LICENSOR OFFERS THE WORK AS-IS AND MAKES NO
REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING THE
WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING,
WITHOUT LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY,
FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE
ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY, OR THE
PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE.
SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
WARRANTIES, SO SUCH EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability. EXCEPT TO
THE EXTENT REQUIRED BY APPLICABLE LAW, IN NO EVENT WILL
LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY
SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY
DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK,
EVEN IF LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

7. Termination

__1__ This License and the rights granted hereunder will
terminate automatically upon any breach by You of the
terms of this License. Individuals or entities who have
received Adaptations or Collections from You under this
License, however, will not have their licenses terminated
provided such individuals or entities remain in full
compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__ Subject to the above terms and conditions, the
license granted here is perpetual (for the duration of
the applicable copyright in the Work). Notwithstanding
the above, Licensor reserves the right to release the
Work under different license terms or to stop
distributing the Work at any time; provided, however that
any such election will not serve to withdraw this License
(or any other license that has been, or is required to
be, granted under the terms of this License), and this
License will continue in full force and effect unless
terminated as stated above.

8. Miscellaneous

__1__ Each time You Distribute or Publicly Perform the Work
or a Collection, the Licensor offers to the recipient a
license to the Work on the same terms and conditions as
the license granted to You under this License.

__1__ Each time You Distribute or Publicly Perform an
Adaptation, Licensor offers to the recipient a license to
the original Work on the same terms and conditions as the
license granted to You under this License.

__1__ If any provision of this License is invalid or
unenforceable under applicable law, it shall not affect
the validity or enforceability of the remainder of the
terms of this License, and without further action by the
parties to this agreement, such provision shall be
reformed to the minimum extent necessary to make such
provision valid and enforceable.

__1__ No term or provision of this License shall be deemed
waived and no breach consented to unless such waiver or
consent shall be in writing and signed by the party to be
charged with such waiver or consent.

__1__ This License constitutes the entire agreement between
the parties with respect to the Work licensed here. There
are no understandings, agreements or representations with
respect to the Work not specified here. Licensor shall
not be bound by any additional provisions that may appear
in any communication from You. This License may not be
modified without the mutual written agreement of the
Licensor and You.

__1__ The rights granted under, and the subject matter
referenced, in this License were drafted utilizing the
terminology of the Berne Convention for the Protection of
Literary and Artistic Works (as amended on September 28,
1979), the Rome Convention of 1961, the WIPO Copyright
Treaty of 1996, the WIPO Performances and Phonograms
Treaty of 1996 and the Universal Copyright Convention (as
revised on July 24, 1971). These rights and subject
matter take effect in the relevant jurisdiction in which
the License terms are sought to be enforced according to
the corresponding provisions of the implementation of
those treaty provisions in the applicable national law.
If the standard suite of rights granted under applicable
copyright law includes additional rights not granted
under this License, such additional rights are deemed to
be included in the License; this License is not intended
to restrict the license of any rights under applicable
law.

((
((
Creative Commons Notice
))??
((https://creativecommons.org/licenses/by-nc/3.0
((/legalcode))??
))??

Creative Commons is not a party to this License, and
makes no warranty whatsoever in connection with the Work.
Creative Commons will not be liable to You or any party
on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or
consequential damages arising in connection to this
license. Notwithstanding the foregoing two (2) sentences,
if Creative Commons has expressly identified itself as
the Licensor hereunder, it shall have all rights and
obligations of Licensor.

Except for the limited purpose of indicating to the
public that the Work is licensed under the CCPL, Creative
Commons does not authorize the use by either party of the
trademark "Creative Commons" or any related trademark or
logo of Creative Commons without the prior written
consent of Creative Commons. Any permitted use will be in
compliance with Creative Commons' then-current trademark
usage guidelines, as may be published on its website or
otherwise made available upon request from time to time.
For the avoidance of doubt, this trademark restriction
does not form part of the License.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_NC_4_0_lre = `//**
CC-BY-NC-4.0
https://creativecommons.org/licenses/by-nc/4.0
**//

((Creative Commons))??

((
Creative Commons Legal Code
))??
((https://creativecommons.org/licenses/by-nc/4.0
((/legalcode))??
))??

((
Attribution-NonCommercial 4.0 International
))??
((https://creativecommons.org/licenses/by-nc/4.0
((/legalcode))??
))??

((
Official translations of this license are available in
other languages.
))??

((
Creative Commons Corporation (“Creative Commons”) is not a law firm and does
not provide legal services or legal advice. Distribution of Creative Commons
public licenses does not create a lawyer-client or other relationship. Creative
Commons makes its licenses and related information available on an “as-is”
basis. Creative Commons gives no warranties regarding its licenses, any material
licensed under their terms and conditions, or any related information. Creative
Commons disclaims all liability for damages resulting from their use to the
fullest extent possible.
))??

((
Using Creative Commons Public Licenses

Creative Commons public licenses provide a standard set of terms and conditions
that creators and other rights holders may use to share original works of
authorship and other material subject to copyright and certain other rights
specified in the public license below. The following considerations are for
informational purposes only, are not exhaustive, and do not form part of our
licenses.

Considerations for licensors: Our public licenses are intended for use by those
authorized to give the public permission to use material in ways otherwise
restricted by copyright and certain other rights. Our licenses are irrevocable.
Licensors should read and understand the terms and conditions of the license
they choose before applying it. Licensors should also secure all rights
necessary before applying our licenses so that the public can reuse the material
as expected. Licensors should clearly mark any material not subject to the
license. This includes other CC-licensed material, or material used under an
exception or limitation to copyright. More considerations for licensors.

Considerations for the public: By using one of our public licenses, a licensor
grants the public permission to use the licensed material under specified terms
and conditions. If the licensor’s permission is not necessary for any
reason–for example, because of any applicable exception or limitation to
copyright–then that use is not regulated by the license. Our licenses grant
only permissions under copyright and certain other rights that a licensor has
authority to grant. Use of the licensed material may still be restricted for
other reasons, including because others have copyright or other rights in the
material. A licensor may make special requests, such as asking that all changes
be marked or described. Although not required by our licenses, you are
encouraged to respect those requests where reasonable.
More considerations for the public.
))??

((
Creative Commons Attribution-NonCommercial 4.0 International Public License
))??
((https://creativecommons.org/licenses/by-nc/4.0
((/legalcode))??
))??

By exercising the Licensed Rights (defined below), You accept and agree to be
bound by the terms and conditions of this Creative Commons
Attribution-NonCommercial 4.0 International Public License ("Public License").
To the extent this Public License may be interpreted as a contract, You are
granted the Licensed Rights in consideration of Your acceptance of these terms
and conditions, and the Licensor grants You such rights in consideration of
benefits the Licensor receives from making the Licensed Material available under
these terms and conditions.

Section 1 – Definitions.

__1__ Adapted Material means material subject to Copyright and Similar Rights
that is derived from or based upon the Licensed Material and in which the
Licensed Material is translated, altered, arranged, transformed, or otherwise
modified in a manner requiring permission under the Copyright and Similar Rights
held by the Licensor. For purposes of this Public License, where the Licensed
Material is a musical work, performance, or sound recording, Adapted Material is
always produced where the Licensed Material is synched in timed relation with a
moving image.

__1__ Adapter's License means the license You apply to Your Copyright and
Similar Rights in Your contributions to Adapted Material in accordance with the
terms and conditions of this Public License.

__1__ Copyright and Similar Rights means copyright and/or similar rights closely
related to copyright including, without limitation, performance, broadcast,
sound recording, and Sui Generis Database Rights, without regard to how the
rights are labeled or categorized. For purposes of this Public License, the
rights specified in Section 2(b)(1)-(2) are not Copyright and Similar Rights.

__1__ Effective Technological Measures means those measures that, in the absence
of proper authority, may not
be circumvented under laws fulfilling obligations under Article 11 of the WIPO
Copyright Treaty adopted on December 20, 1996, and/or similar
international agreements.

__1__ Exceptions and Limitations means fair use, fair dealing, and/or any other
exception or limitation to Copyright and Similar Rights that applies to Your use
of the Licensed Material.

__1__ Licensed Material means the artistic or literary work, database, or other
material to which the Licensor applied this Public License.

__1__ Licensed Rights means the rights granted to You subject to the terms and
conditions of this Public License, which are limited to all Copyright and
Similar Rights that apply to Your use of the Licensed Material and that the
Licensor has authority to license.

__1__ Licensor means the individual(s) or entity(ies) granting rights under this
Public License.

__1__ NonCommercial means not primarily intended for or directed towards
commercial advantage or monetary compensation. For purposes of this Public
License, the exchange of the Licensed Material for other material subject to
Copyright and Similar Rights by digital file-sharing or similar means is
NonCommercial provided there is no payment of monetary compensation in
connection with the exchange.

__1__ Share means to provide material to the public by any means or process that
requires permission under the Licensed Rights, such as reproduction, public
display, public performance, distribution, dissemination, communication, or
importation, and to make material available to the public including in ways that
members of the public may access the material from a place and at a time
individually chosen by them.

__1__ Sui Generis Database Rights means rights other than copyright resulting
from Directive 96/9/EC of the European Parliament and of the Council of 11 March
1996 on the legal protection of databases, as amended and/or succeeded, as well
as other essentially equivalent rights anywhere in the world.

__1__ You means the individual or entity exercising the Licensed Rights under
this Public License. Your has a corresponding meaning.

Section 2 – Scope.

__1__ License grant.

__1__ Subject to the terms and conditions of this Public License, the Licensor
hereby grants You a worldwide, royalty-free, non-sublicensable, non-exclusive,
irrevocable license to exercise the Licensed Rights in the Licensed Material
to:

__1__ reproduce and Share the Licensed Material, in whole or in part, for
NonCommercial purposes only; and

__1__ produce, reproduce, and Share Adapted Material for NonCommercial purposes
only.

__1__ Exceptions and Limitations. For the avoidance of doubt, where Exceptions
and Limitations apply to Your use, this Public License does not apply, and You
do not need to comply with its terms and conditions.

__1__ Term. The term of this Public License is specified in Section 6(a).

__1__ Media and formats; technical modifications allowed. The Licensor
authorizes You to exercise the Licensed Rights in all media and formats whether
now known or hereafter created, and to make technical modifications necessary to
do so. The Licensor waives and/or agrees not to assert any right or authority to
forbid You from making technical modifications necessary to exercise the
Licensed Rights, including technical modifications necessary to circumvent
Effective Technological Measures. For purposes of this Public License, simply
making modifications authorized by this Section 2(a)(4) never produces Adapted
Material.

__1__ Downstream recipients.

__1__ Offer from the Licensor – Licensed Material. Every recipient of the
Licensed Material automatically receives an  offer from the Licensor to exercise
the Licensed Rights under the terms and conditions of this Public License.

__1__ No downstream restrictions. You may not offer or impose any additional or
different terms or  conditions on, or apply any Effective Technological Measures
to, the Licensed Material if doing so restricts exercise of the Licensed Rights
by any recipient of the Licensed Material.

__1__ No endorsement. Nothing in this Public License constitutes or may be
construed as permission to assert or imply that You are, or that Your use of the
Licensed Material is, connected with, or sponsored, endorsed, or granted
official status by, the Licensor or others designated to receive attribution as
provided in Section 3(a)(1)(A)(i).

__1__
Other rights.

__1__ Moral rights, such as the right of integrity, are not licensed under this
Public License, nor are publicity, privacy, and/or other similar personality
rights; however, to the extent possible, the Licensor waives and/or agrees not
to assert any such rights held by the Licensor to the limited extent necessary
to allow You to exercise the Licensed Rights, but not otherwise.

__1__ Patent and trademark rights are not licensed under this Public License.

__1__ To the extent possible, the Licensor waives any right to collect royalties
from You for the exercise of the Licensed Rights, whether directly or through a
collecting society under any voluntary or waivable statutory or compulsory
licensing scheme. In all other cases the Licensor expressly reserves any right
to collect such royalties, including when the Licensed Material is used other
than for NonCommercial purposes.

Section 3 – License Conditions.

Your exercise of the Licensed Rights is expressly made subject to the following
conditions.

__1__
Attribution.

__1__
If You Share the Licensed Material (including in modified form), You must:

__1__ retain the following if it is supplied by the Licensor with the Licensed
Material:

__1__ identification of the creator(s) of the Licensed Material and any others
designated to receive attribution, in any reasonable manner requested  by the
Licensor (including by pseudonym if designated);

__1__ a copyright notice;

__1__ a notice that refers to this Public License;

__1__ a notice that refers to the disclaimer of warranties;

__1__ a URI or hyperlink to the Licensed Material to the extent reasonably
practicable;

__1__ indicate if You modified the Licensed Material and retain an indication of
any previous modifications; and

__1__ indicate the Licensed Material is licensed under this Public License,
and include the text of, or the URI or hyperlink to, this Public
License.

__1__ You may satisfy the conditions in Section 3(a)(1) in any reasonable manner
based on the medium, means, and context in which You Share the Licensed
Material. For example, it may be reasonable to satisfy the conditions by
providing a URI or hyperlink to a resource that includes the required
information.

__1__ If requested by the Licensor, You must remove any of the information
required by Section 3(a)(1)(A) to the extent reasonably practicable.

__1__ If You Share Adapted Material You produce, the Adapter's License You apply
must not prevent recipients of the Adapted Material from complying with this
Public License.

Section 4 – Sui Generis Database Rights.

Where the Licensed Rights include Sui Generis Database Rights that apply to Your
use of the Licensed Material:

__1__ for the avoidance of doubt, Section 2(a)(1) grants You the right to
extract, reuse, reproduce, and Share all or a substantial portion of the
contents of the database for NonCommercial purposes only;

__1__ if You include all or a substantial portion of the database contents in a
database in which You have Sui Generis Database Rights, then the database in
which You have Sui Generis Database Rights (but not its individual contents) is
Adapted Material; and

__1__ You must comply with the conditions in Section 3(a) if You Share all or a
substantial portion of the contents of the database.

__1__ For the avoidance of doubt, this Section 4 supplements and
does not replace Your obligations under this Public License
where the Licensed Rights include other Copyright and Similar
Rights.

Section 5 – Disclaimer of Warranties and Limitation of Liability.

__1__ Unless otherwise separately undertaken by the Licensor, to the extent
possible, the Licensor offers the Licensed Material as-is and as-available, and
makes no representations or warranties of any kind concerning the Licensed
Material, whether express, implied, statutory, or other. This includes, without
limitation, warranties of title, merchantability, fitness for a particular
purpose, non-infringement, absence of latent or other defects, accuracy, or the
presence or absence of errors, whether or not known or discoverable. Where
disclaimers of warranties are not allowed in full or in part, this disclaimer
may not apply to You.

__1__ To the extent possible, in no event will the Licensor be liable to You on
any legal theory (including, without limitation, negligence) or otherwise for
any direct, special, indirect, incidental, consequential, punitive, exemplary,
or other losses, costs, expenses, or damages arising out of this Public License
or use of the Licensed Material, even if the Licensor has been advised of the
possibility of such losses, costs, expenses, or damages. Where a limitation of
liability is not allowed in full or in part, this limitation may not apply to
You.

__1__ The disclaimer of warranties and limitation of liability provided above
shall be interpreted in a manner that, to the extent possible, most closely
approximates an absolute disclaimer and waiver of all liability.

Section 6 – Term and Termination.

__1__ This Public License applies for the term of the Copyright and Similar
Rights licensed here. However, if You fail to comply with this Public License,
then Your rights under this Public License terminate automatically.

__1__

Where Your right to use the Licensed Material has terminated under Section 6(a),
it reinstates:

__1__ automatically as of the date the violation is cured, provided it is cured
within 30 days of Your discovery of the violation; or

__1__ upon express reinstatement by the Licensor.

__1__ For the avoidance of doubt, this Section 6(b) does not
affect any right the Licensor may have to seek remedies for
Your violations of this Public License.

__1__ For the avoidance of doubt, the Licensor may also offer the Licensed
Material under separate terms or conditions or stop distributing the Licensed
Material at any time; however, doing so will not terminate this Public License.

__1__ Sections 1, 5, 6, 7, and 8 survive termination of this Public License.

Section 7 – Other Terms and Conditions.

__1__ The Licensor shall not be bound by any additional or different terms or
conditions communicated by You unless expressly agreed.

__1__ Any arrangements, understandings, or agreements regarding the Licensed
Material not stated herein are separate from and independent of the terms and
conditions of this Public License.

Section 8 – Interpretation.

__1__ For the avoidance of doubt, this Public License does not, and shall not be
interpreted to, reduce, limit, restrict, or impose conditions on any use of the
Licensed Material that could lawfully be made without permission under this
Public License.

__1__ To the extent possible, if any provision of this Public License is deemed
unenforceable, it shall be automatically reformed to the minimum extent
necessary to make it enforceable. If the provision cannot be reformed, it shall
be severed from this Public License without affecting the enforceability of the
remaining terms and conditions.

__1__ No term or condition of this Public License will be waived and no failure
to comply consented to unless expressly agreed to by the Licensor.

__1__ Nothing in this Public License constitutes or may be interpreted as a
limitation upon, or waiver of, any privileges and immunities that apply to the
Licensor or You, including from the legal processes of any jurisdiction or
authority.

((
Creative Commons is not a party to its public licenses. Notwithstanding,
Creative Commons may elect to apply one of its public licenses to material it
publishes and in those instances will be considered the “Licensor.” The text
of the Creative Commons public licenses is dedicated to the public domain under
the CC0 Public Domain Dedication. Except for the limited purpose of indicating
that material is shared under a Creative Commons public license or as otherwise
permitted by the Creative Commons policies published at
creativecommons.org/policies, Creative Commons does not authorize the use of the
trademark “Creative Commons” or any other trademark or logo of Creative
Commons without its prior written consent including, without limitation, in
connection with any unauthorized modifications to any of its public licenses or
any other arrangements, understandings, or agreements concerning use of licensed
material. For the avoidance of doubt, this paragraph does not form part of the
public licenses.
Creative Commons may be contacted at creativecommons.org.
))??

((
Additional languages available:

العربية,
čeština,
Deutsch,
Ελληνικά,
Español,
euskara,
suomeksi,
français,
hrvatski,
Bahasa Indonesia,
italiano,
日本語,
한국어,
Lietuvių,
latviski,
te reo Māori,
Nederlands,
norsk,
polski,
português,
română,
русский,
Slovenščina,
svenska,
Türkçe,
українська,
中文,
華語.

Please read the FAQ for more information about official translations.
))??
`
const license_CC_BY_NC_ND_1_0_lre = `//**
Creative Commons Attribution Non Commercial No Derivatives 1.0 Generic
https://spdx.org/licenses/CC-BY-NC-ND-1.0.json
https://creativecommons.org/licenses/by-nd-nc/1.0/legalcode
**//

((  Creative Commons Attribution-NoDerivs-NonCommercial 1.0 ))??
((  CREATIVE
COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE LEGAL SERVICES.
DISTRIBUTION OF THIS DRAFT LICENSE DOES NOT CREATE AN ATTORNEY-CLIENT
RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION ON AN "AS-IS" BASIS.
CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE INFORMATION PROVIDED, AND
DISCLAIMS LIABILITY FOR DAMAGES RESULTING FROM ITS USE. ))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE"). THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE. THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED
HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

   __1__ Definitions

      __1__ "Collective Work" means a work, such as a periodical issue,
      anthology or encyclopedia, in which the Work in its entirety in unmodified
      form, along with a number of other contributions, constituting separate
      and independent works in themselves, are assembled into a collective
      whole. A work that constitutes a Collective Work will not be considered a
      Derivative Work (as defined below) for the purposes of this License.

      __1__ "Derivative Work" means a work based upon the Work or upon the Work
      and other pre-existing works, such as a translation, musical arrangement,
      dramatization, fictionalization, motion picture version, sound recording,
      art reproduction, abridgment, condensation, or any other form in which the
      Work may be recast, transformed, or adapted, except that a work that
      constitutes a Collective Work will not be considered a Derivative Work for
      the purpose of this License.

      __1__ "Licensor" means the individual or entity that offers the Work under
      the terms of this License.

      __1__ "Original Author" means the individual or entity who created the
      Work.

      __1__ "Work" means the copyrightable work of authorship offered under the
      terms of this License.

      __1__ "You" means an individual or entity exercising rights under this
      License who has not previously violated the terms of this License with
      respect to the Work, or who has received express permission from the
      Licensor to exercise rights under this License despite a previous
      violation.

   __1__ Fair Use Rights. Nothing in this license is intended to reduce, limit,
   or restrict any rights arising from fair use, first sale or other limitations
   on the exclusive rights of the copyright owner under copyright law or other
   applicable laws.

   __1__ License Grant. Subject to the terms and conditions of this License,
   Licensor hereby grants You a worldwide, royalty-free, non-exclusive,
   perpetual (for the duration of the applicable copyright) license to exercise
   the rights in the Work as stated below:

      __1__ to reproduce the Work, to incorporate the Work into one or more
      Collective Works, and to reproduce the Work as incorporated in the
      Collective Works;

      __1__ to distribute copies or phonorecords of, display publicly, perform
      publicly, and perform publicly by means of a digital audio transmission
      the Work including as incorporated in Collective Works;

      The above rights may be exercised in all media and formats whether now
      known or hereafter devised. The above rights include the right to make
      such modifications as are technically necessary to exercise the rights in
      other media and formats. All rights not expressly granted by Licensor are
      hereby reserved.

   __1__ Restrictions. The license granted in Section 3 above is expressly made
   subject to and limited by the following restrictions:

      __1__ You may distribute, publicly display, publicly perform, or publicly
      digitally perform the Work only under the terms of this License, and You
      must include a copy of, or the Uniform Resource Identifier for, this
      License with every copy or phonorecord of the Work You distribute,
      publicly display, publicly perform, or publicly digitally perform. You may
      not offer or impose any terms on the Work that alter or restrict the terms
      of this License or the recipients' exercise of the rights granted
      hereunder. You may not sublicense the Work. You must keep intact all
      notices that refer to this License and to the disclaimer of warranties.
      You may not distribute, publicly display, publicly perform, or publicly
      digitally perform the Work with any technological measures that control
      access or use of the Work in a manner inconsistent with the terms of this
      License Agreement. The above applies to the Work as incorporated in a
      Collective Work, but this does not require the Collective Work apart from
      the Work itself to be made subject to the terms of this License. If You
      create a Collective Work, upon notice from any Licensor You must, to the
      extent practicable, remove from the Collective Work any reference to such
      Licensor or the Original Author, as requested.

      __1__ You may not exercise any of the rights granted to You in Section 3
      above in any manner that is primarily intended for or directed toward
      commercial advantage or private monetary compensation. The exchange of the
      Work for other copyrighted works by means of digital file-sharing or
      otherwise shall not be considered to be intended for or directed toward
      commercial advantage or private monetary compensation, provided there is
      no payment of any monetary compensation in connection with the exchange of
      copyrighted works.

      __1__ If you distribute, publicly display, publicly perform, or publicly
      digitally perform the Work or any Collective Works, You must keep intact
      all copyright notices for the Work and give the Original Author credit
      reasonable to the medium or means You are utilizing by conveying the name
      (or pseudonym if applicable) of the Original Author if supplied; the title
      of the Work if supplied. Such credit may be implemented in any reasonable
      manner; provided, however, that in the case of a Collective Work, at a
      minimum such credit will appear where any other comparable authorship
      credit appears and in a manner at least as prominent as such other
      comparable authorship credit.

   __1__ Representations, Warranties and Disclaimer

      __1__ By offering the Work for public release under this License, Licensor
      represents and warrants that, to the best of Licensor's knowledge after
      reasonable inquiry:

         __1__ Licensor has secured all rights in the Work necessary to grant
         the license rights hereunder and to permit the lawful exercise of the
         rights granted hereunder without You having any obligation to pay any
         royalties, compulsory license fees, residuals or any other payments;

         __1__ The Work does not infringe the copyright, trademark, publicity
         rights, common law rights or any other right of any third party or
         constitute defamation, invasion of privacy or other tortious injury to
         any third party.

      __1__ EXCEPT AS EXPRESSLY STATED IN THIS LICENSE OR OTHERWISE AGREED IN
      WRITING OR REQUIRED BY APPLICABLE LAW, THE WORK IS LICENSED ON AN "AS IS"
      BASIS, WITHOUT WARRANTIES OF ANY KIND, EITHER EXPRESS OR IMPLIED
      INCLUDING, WITHOUT LIMITATION, ANY WARRANTIES REGARDING THE CONTENTS OR
      ACCURACY OF THE WORK.

   __1__ Limitation on Liability. EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE
   LAW, AND EXCEPT FOR DAMAGES ARISING FROM LIABILITY TO A THIRD PARTY RESULTING
   FROM BREACH OF THE WARRANTIES IN SECTION 5, IN NO EVENT WILL LICENSOR BE
   LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL,
   PUNITIVE OR EXEMPLARY DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE
   WORK, EVEN IF LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

   __1__ Termination

      __1__ This License and the rights granted hereunder will terminate
      automatically upon any breach by You of the terms of this License.
      Individuals or entities who have received Collective Works from You under
      this License, however, will not have their licenses terminated provided
      such individuals or entities remain in full compliance with those
      licenses. Sections 1, 2, 5, 6, 7, and 8 will survive any termination of
      this License.

      __1__ Subject to the above terms and conditions, the license granted here
      is perpetual (for the duration of the applicable copyright in the Work).
      Notwithstanding the above, Licensor reserves the right to release the Work
      under different license terms or to stop distributing the Work at any
      time; provided, however that any such election will not serve to withdraw
      this License (or any other license that has been, or is required to be,
      granted under the terms of this License), and this License will continue
      in full force and effect unless terminated as stated above.

   __1__ Miscellaneous

      __1__ Each time You distribute or publicly digitally perform the Work or a
      Collective Work, the Licensor offers to the recipient a license to the
      Work on the same terms and conditions as the license granted to You under
      this License.

      __1__ If any provision of this License is invalid or unenforceable under
      applicable law, it shall not affect the validity or enforceability of the
      remainder of the terms of this License, and without further action by the
      parties to this agreement, such provision shall be reformed to the minimum
      extent necessary to make such provision valid and enforceable.

      __1__ No term or provision of this License shall be deemed waived and no
      breach consented to unless such waiver or consent shall be in writing and
      signed by the party to be charged with such waiver or consent.

      __1__ This License constitutes the entire agreement between the parties
      with respect to the Work licensed here. There are no understandings,
      agreements or representations with respect to the Work not specified here.
      Licensor shall not be bound by any additional provisions that may appear
      in any communication from You. This License may not be modified without
      the mutual written agreement of the Licensor and You.

(( Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work. Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license. Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons. Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at http:/creativecommons.org/.
))??
`
const license_CC_BY_NC_ND_2_0_lre = `//**
CC-BY-NC-ND-2.0
https://creativecommons.org/licenses/by-nc-nd/2.0
**//

((Creative Commons))??

Creative Commons

Attribution-NonCommercial-NoDerivs 2.0

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE
LEGAL
SERVICES. DISTRIBUTION OF THIS LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION
ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING
FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE").  THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW.  ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE.  THE LICENSOR GRANTS YOU THE RIGHTS
CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole.  A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License. For
the avoidance of doubt, where the Work is a musical composition or sound
recording, the synchronization of the Work in timed-relation with a moving image
("synching") will be considered a Derivative Work for the purpose of this
License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant.  Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

The above rights may be exercised in all media and formats whether now known or
hereafter devised.  The above rights include the right to make such
modifications as are technically necessary to exercise the rights in other media
and formats, but otherwise you have no rights to make Derivative Works.  All
rights not expressly granted by Licensor are hereby reserved, including but not
limited to the rights set forth in Sections 4(d) and 4(e).

4. Restrictions.The license granted in Section 3 above is expressly made subject
to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform.  You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder.  You may not sublicense the Work.  You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement.  The above applies to the Work as incorporated in a
Collective Work, but this does not require the Collective Work apart from the
Work itself to be made subject to the terms of this License. If You create a
Collective Work, upon notice from any Licensor You must, to the extent
practicable, remove from the Collective Work any reference to such Licensor or
the Original Author, as requested.

__1__
You may not exercise any of the rights granted to You in Section 3 above in any
manner that is primarily intended for or directed toward commercial advantage or
private monetary compensation.  The exchange of the Work for other copyrighted
works by means of digital file-sharing or otherwise shall not be considered to
be intended for or directed toward commercial advantage or private monetary
compensation, provided there is no payment of any monetary compensation in
connection with the exchange of copyrighted works.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work, You must keep intact all copyright notices for the Work and
give the Original Author credit reasonable to the medium or means You are
utilizing by conveying the name (or pseudonym if applicable) of the Original
Author if supplied; the title of the Work if supplied; and to the extent
reasonably practicable, the Uniform Resource Identifier, if any, that Licensor
specifies to be associated with the Work, unless such URI does not refer to the
copyright notice or licensing information for the Work.  Such credit may be
implemented in any reasonable manner; provided, however, that in the case of a
Collective Work, at a minimum such credit will appear where any other comparable
authorship credit appears and in a manner at least as prominent as such other
comparable authorship credit.

__1__
For the avoidance of doubt, where the Work is a musical composition:

__1__ Performance Royalties Under Blanket Licenses. Licensor reserves the
exclusive right to collect, whether individually or via a performance rights
society (e.g. ASCAP, BMI, SESAC), royalties for the public performance or public
digital performance (e.g. webcast) of the Work if that performance is primarily
intended for or directed toward commercial advantage or private monetary
compensation.

__1__ Mechanical Rights and Statutory Royalties. Licensor reserves the exclusive
right to collect, whether individually or via a music rights agency or
designated agent (e.g. Harry Fox Agency), royalties for any phonorecord You
create from the Work ("cover version") and distribute, subject to the compulsory
license created by 17 USC Section 115 of the US Copyright Act (or the equivalent
in other jurisdictions), if Your distribution of such cover version is primarily
intended for or directed toward commercial advantage or private monetary
compensation.

__1__ Webcasting Rights and Statutory Royalties.__1__ For the avoidance of
doubt, where the Work is a sound recording, Licensor reserves the exclusive
right to collect, whether individually or via a performance-rights society (e.g.
SoundExchange), royalties for the public digital performance (e.g. webcast) of
the Work, subject to the compulsory license created by 17 USC Section 114 of the
US Copyright Act (or the equivalent in other jurisdictions), if Your public
digital performance is primarily intended for or directed toward commercial
advantage or private monetary compensation.

5. Representations, Warranties and Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED BY THE PARTIES IN WRITING, LICENSOR OFFERS THE
WORK AS-IS AND MAKES NO REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING THE
WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT LIMITATION,
WARRANTIES OF TITLE, MERCHANTIBILITY, FITNESS FOR A PARTICULAR PURPOSE,
NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY, OR THE
PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE.  SOME JURISDICTIONS
DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO SUCH EXCLUSION MAY NOT
APPLY TO YOU.

6. Limitation on Liability.  EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN
NO EVENT WILL LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY SPECIAL,
INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY DAMAGES ARISING OUT OF THIS
LICENSE OR THE USE OF THE WORK, EVEN IF LICENSOR HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Collective Works from You under this License, however, will not have
their licenses terminated provided such individuals or entities remain in full
compliance with those licenses. Sections 1, 2, 5, 6, 7, and 8 will survive any
termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work).  Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here.  There are no understandings, agreements or
representations with respect to the Work not specified here.  Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work.  Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license.  Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons.  Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_NC_ND_2_5_lre = `//**
CC-BY-NC-ND-2.5
https://creativecommons.org/licenses/by-nc-nd/2.5
**//

((Creative Commons))??

Creative Commons

Attribution-NonCommercial-NoDerivs 2.5

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE
LEGAL
SERVICES. DISTRIBUTION OF THIS LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION
ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING
FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE").  THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW.  ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE.  THE LICENSOR GRANTS YOU THE RIGHTS
CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole.  A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License. For
the avoidance of doubt, where the Work is a musical composition or sound
recording, the synchronization of the Work in timed-relation with a moving image
("synching") will be considered a Derivative Work for the purpose of this
License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant.  Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

The above rights may be exercised in all media and formats whether now known or
hereafter devised.  The above rights include the right to make such
modifications as are technically necessary to exercise the rights in other media
and formats, but otherwise you have no rights to make Derivative Works.  All
rights not expressly granted by Licensor are hereby reserved, including but not
limited to the rights set forth in Sections 4(d) and 4(e).

4. Restrictions.The license granted in Section 3 above is expressly made subject
to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform.  You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder.  You may not sublicense the Work.  You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement.  The above applies to the Work as incorporated in a
Collective Work, but this does not require the Collective Work apart from the
Work itself to be made subject to the terms of this License. If You create a
Collective Work, upon notice from any Licensor You must, to the extent
practicable, remove from the Collective Work any credit as required by clause
4(c), as requested.

__1__
You may not exercise any of the rights granted to You in Section 3 above in any
manner that is primarily intended for or directed toward commercial advantage or
private monetary compensation.  The exchange of the Work for other copyrighted
works by means of digital file-sharing or otherwise shall not be considered to
be intended for or directed toward commercial advantage or private monetary
compensation, provided there is no payment of any monetary compensation in
connection with the exchange of copyrighted works.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work, You must keep intact all copyright notices for the Work and
provide, reasonable to the medium or means You are utilizing: (i) the name of
the Original Author (or pseudonym, if applicable) if supplied, and/or (ii) if
the Original Author and/or Licensor designate another party or parties (e.g. a
sponsor institute, publishing entity, journal) for attribution in Licensor's
copyright notice, terms of service or by other reasonable means, the name of
such party or parties; the title of the Work if supplied; and to the extent
reasonably practicable, the Uniform Resource Identifier, if any, that Licensor
specifies to be associated with the Work, unless such URI does not refer to the
copyright notice or licensing information for the Work.  Such credit may be
implemented in any reasonable manner; provided, however, that in the case of a
Collective Work, at a minimum such credit will appear where any other comparable
authorship credit appears and in a manner at least as prominent as such other
comparable authorship credit.

__1__
For the avoidance of doubt, where the Work is a musical composition:

__1__ Performance Royalties Under Blanket Licenses. Licensor reserves the
exclusive right to collect, whether individually or via a performance rights
society (e.g. ASCAP, BMI, SESAC), royalties for the public performance or public
digital performance (e.g. webcast) of the Work if that performance is primarily
intended for or directed toward commercial advantage or private monetary
compensation.

__1__ Mechanical Rights and Statutory Royalties. Licensor reserves the exclusive
right to collect, whether individually or via a music rights agency or
designated agent (e.g. Harry Fox Agency), royalties for any phonorecord You
create from the Work ("cover version") and distribute, subject to the compulsory
license created by 17 USC Section 115 of the US Copyright Act (or the equivalent
in other jurisdictions), if Your distribution of such cover version is primarily
intended for or directed toward commercial advantage or private monetary
compensation.

__1__ Webcasting Rights and Statutory Royalties.__1__ For the avoidance of
doubt, where the Work is a sound recording, Licensor reserves the exclusive
right to collect, whether individually or via a performance-rights society (e.g.
SoundExchange), royalties for the public digital performance (e.g. webcast) of
the Work, subject to the compulsory license created by 17 USC Section 114 of the
US Copyright Act (or the equivalent in other jurisdictions), if Your public
digital performance is primarily intended for or directed toward commercial
advantage or private monetary compensation.

5. Representations, Warranties and Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED BY THE PARTIES IN WRITING, LICENSOR OFFERS THE
WORK AS-IS AND MAKES NO REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING THE
WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT LIMITATION,
WARRANTIES OF TITLE, MERCHANTIBILITY, FITNESS FOR A PARTICULAR PURPOSE,
NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY, OR THE
PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE.  SOME JURISDICTIONS
DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO SUCH EXCLUSION MAY NOT
APPLY TO YOU.

6. Limitation on Liability.  EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN
NO EVENT WILL LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY SPECIAL,
INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY DAMAGES ARISING OUT OF THIS
LICENSE OR THE USE OF THE WORK, EVEN IF LICENSOR HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Collective Works from You under this License, however, will not have
their licenses terminated provided such individuals or entities remain in full
compliance with those licenses. Sections 1, 2, 5, 6, 7, and 8 will survive any
termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work).  Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here.  There are no understandings, agreements or
representations with respect to the Work not specified here.  Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work.  Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license.  Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons.  Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_NC_ND_3_0_lre = `//**
CC-BY-NC-ND-3.0
https://creativecommons.org/licenses/by-nc-nd/3.0
**//

((Creative Commons))??

((
Creative Commons Legal Code
))??
((https://creativecommons.org/licenses/by-nc-nd/3.0
((/legalcode))??
))??

((
Attribution-NonCommercial-NoDerivs 3.0 Unported
))??
((https://creativecommons.org/licenses/by-nc-nd/3.0
((/legalcode))??
))??

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES
NOT PROVIDE LEGAL SERVICES. DISTRIBUTION OF THIS LICENSE
DOES NOT CREATE AN ATTORNEY-CLIENT RELATIONSHIP. CREATIVE
COMMONS PROVIDES THIS INFORMATION ON AN "AS-IS" BASIS.
CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES
RESULTING FROM ITS USE.
))??

((
((License))??
))??
((https://creativecommons.org/licenses/by-nc-nd/3.0
((/legalcode))??
))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS
OF THIS CREATIVE COMMONS PUBLIC LICENSE ("CCPL" OR
"LICENSE"). THE WORK IS PROTECTED BY COPYRIGHT AND/OR OTHER
APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS
AUTHORIZED UNDER THIS LICENSE OR COPYRIGHT LAW IS
PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU
ACCEPT AND AGREE TO BE BOUND BY THE TERMS OF THIS LICENSE.
TO THE EXTENT THIS LICENSE MAY BE CONSIDERED TO BE A
CONTRACT, THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED HERE
IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND
CONDITIONS.

1. Definitions

__1__ "Adaptation" means a work based upon
the Work, or upon the Work and other pre-existing works,
such as a translation, adaptation, derivative work,
arrangement of music or other alterations of a literary
or artistic work, or phonogram or performance and
includes cinematographic adaptations or any other form in
which the Work may be recast, transformed, or adapted
including in any form recognizably derived from the
original, except that a work that constitutes a
Collection will not be considered an Adaptation for the
purpose of this License. For the avoidance of doubt,
where the Work is a musical work, performance or
phonogram, the synchronization of the Work in
timed-relation with a moving image ("synching") will be
considered an Adaptation for the purpose of this
License.

__1__ "Collection" means a collection of
literary or artistic works, such as encyclopedias and
anthologies, or performances, phonograms or broadcasts,
or other works or subject matter other than works listed
in Section 1(f) below, which, by reason of the selection
and arrangement of their contents, constitute
intellectual creations, in which the Work is included in
its entirety in unmodified form along with one or more
other contributions, each constituting separate and
independent works in themselves, which together are
assembled into a collective whole. A work that
constitutes a Collection will not be considered an
Adaptation (as defined above) for the purposes of this
License.

__1__ "Distribute" means to make available
to the public the original and copies of the Work through
sale or other transfer of ownership.

__1__ "Licensor" means the individual,
individuals, entity or entities that offer(s) the Work
under the terms of this License.

__1__ "Original Author" means, in the case
of a literary or artistic work, the individual,
individuals, entity or entities who created the Work or
if no individual or entity can be identified, the
publisher; and in addition (i) in the case of a
performance the actors, singers, musicians, dancers, and
other persons who act, sing, deliver, declaim, play in,
interpret or otherwise perform literary or artistic works
or expressions of folklore; (ii) in the case of a
phonogram the producer being the person or legal entity
who first fixes the sounds of a performance or other
sounds; and, (iii) in the case of broadcasts, the
organization that transmits the broadcast.

__1__ "Work" means the literary and/or
artistic work offered under the terms of this License
including without limitation any production in the
literary, scientific and artistic domain, whatever may be
the mode or form of its expression including digital
form, such as a book, pamphlet and other writing; a
lecture, address, sermon or other work of the same
nature; a dramatic or dramatico-musical work; a
choreographic work or entertainment in dumb show; a
musical composition with or without words; a
cinematographic work to which are assimilated works
expressed by a process analogous to cinematography; a
work of drawing, painting, architecture, sculpture,
engraving or lithography; a photographic work to which
are assimilated works expressed by a process analogous to
photography; a work of applied art; an illustration, map,
plan, sketch or three-dimensional work relative to
geography, topography, architecture or science; a
performance; a broadcast; a phonogram; a compilation of
data to the extent it is protected as a copyrightable
work; or a work performed by a variety or circus
performer to the extent it is not otherwise considered a
literary or artistic work.

__1__ "You" means an individual or entity
exercising rights under this License who has not
previously violated the terms of this License with
respect to the Work, or who has received express
permission from the Licensor to exercise rights under
this License despite a previous violation.

__1__ "Publicly Perform" means to perform
public recitations of the Work and to communicate to the
public those public recitations, by any means or process,
including by wire or wireless means or public digital
performances; to make available to the public Works in
such a way that members of the public may access these
Works from a place and at a place individually chosen by
them; to perform the Work to the public by any means or
process and the communication to the public of the
performances of the Work, including by public digital
performance; to broadcast and rebroadcast the Work by any
means including signs, sounds or images.

__1__ "Reproduce" means to make copies of
the Work by any means including without limitation by
sound or visual recordings and the right of fixation and
reproducing fixations of the Work, including storage of a
protected performance or phonogram in digital form or
other electronic medium.

2. Fair Dealing Rights. Nothing in this
License is intended to reduce, limit, or restrict any uses
free from copyright or rights arising from limitations or
exceptions that are provided for in connection with the
copyright protection under copyright law or other
applicable laws.

3. License Grant. Subject to the terms
and conditions of this License, Licensor hereby grants You
a worldwide, royalty-free, non-exclusive, perpetual (for
the duration of the applicable copyright) license to
exercise the rights in the Work as stated below:

__1__ to Reproduce the Work, to incorporate the Work into
one or more Collections, and to Reproduce the Work as
incorporated in the Collections; and,

__1__ to Distribute and Publicly Perform the Work including
as incorporated in Collections.

The above rights may be exercised in all media and
formats whether now known or hereafter devised. The above
rights include the right to make such modifications as are
technically necessary to exercise the rights in other media
and formats, but otherwise you have no rights to make
Adaptations. Subject to 8(f), all rights not expressly
granted by Licensor are hereby reserved, including but not
limited to the rights set forth in Section 4(d).

4. Restrictions. The license granted in
Section 3 above is expressly made subject to and limited by
the following restrictions:

__1__ You may Distribute or Publicly Perform the Work only
under the terms of this License. You must include a copy
of, or the Uniform Resource Identifier (URI) for, this
License with every copy of the Work You Distribute or
Publicly Perform. You may not offer or impose any terms
on the Work that restrict the terms of this License or
the ability of the recipient of the Work to exercise the
rights granted to that recipient under the terms of the
License. You may not sublicense the Work. You must keep
intact all notices that refer to this License and to the
disclaimer of warranties with every copy of the Work You
Distribute or Publicly Perform. When You Distribute or
Publicly Perform the Work, You may not impose any
effective technological measures on the Work that
restrict the ability of a recipient of the Work from You
to exercise the rights granted to that recipient under
the terms of the License. This Section 4(a) applies to
the Work as incorporated in a Collection, but this does
not require the Collection apart from the Work itself to
be made subject to the terms of this License. If You
create a Collection, upon notice from any Licensor You
must, to the extent practicable, remove from the
Collection any credit as required by Section 4(c), as
requested.

__1__ You may not exercise any of the rights granted to You
in Section 3 above in any manner that is primarily
intended for or directed toward commercial advantage or
private monetary compensation. The exchange of the Work
for other copyrighted works by means of digital
file-sharing or otherwise shall not be considered to be
intended for or directed toward commercial advantage or
private monetary compensation, provided there is no
payment of any monetary compensation in connection with
the exchange of copyrighted works.

__1__ If You Distribute, or Publicly Perform the Work or
Collections, You must, unless a request has been made
pursuant to Section 4(a), keep intact all copyright
notices for the Work and provide, reasonable to the
medium or means You are utilizing: (i) the name of the
Original Author (or pseudonym, if applicable) if
supplied, and/or if the Original Author and/or Licensor
designate another party or parties (e.g., a sponsor
institute, publishing entity, journal) for attribution
("Attribution Parties") in Licensor's copyright notice,
terms of service or by other reasonable means, the name
of such party or parties; (ii) the title of the Work if
supplied; (iii) to the extent reasonably practicable, the
URI, if any, that Licensor specifies to be associated
with the Work, unless such URI does not refer to the
copyright notice or licensing information for the Work.
The credit required by this Section 4(c) may be
implemented in any reasonable manner; provided, however,
that in the case of a Collection, at a minimum such
credit will appear, if a credit for all contributing
authors of Collection appears, then as part of these
credits and in a manner at least as prominent as the
credits for the other contributing authors. For the
avoidance of doubt, You may only use the credit required
by this Section for the purpose of attribution in the
manner set out above and, by exercising Your rights under
this License, You may not implicitly or explicitly assert
or imply any connection with, sponsorship or endorsement
by the Original Author, Licensor and/or Attribution
Parties, as appropriate, of You or Your use of the Work,
without the separate, express prior written permission of
the Original Author, Licensor and/or Attribution
Parties.

__1__

For the avoidance of doubt:

__1__ Non-waivable Compulsory License
Schemes. In those jurisdictions in which the
right to collect royalties through any statutory or
compulsory licensing scheme cannot be waived, the
Licensor reserves the exclusive right to collect such
royalties for any exercise by You of the rights
granted under this License;

__1__ Waivable Compulsory License
Schemes. In those jurisdictions in which the
right to collect royalties through any statutory or
compulsory licensing scheme can be waived, the
Licensor reserves the exclusive right to collect such
royalties for any exercise by You of the rights
granted under this License if Your exercise of such
rights is for a purpose or use which is otherwise
than noncommercial as permitted under Section 4(b)
and otherwise waives the right to collect royalties
through any statutory or compulsory licensing scheme;
and,

__1__ Voluntary License Schemes. The
Licensor reserves the right to collect royalties,
whether individually or, in the event that the
Licensor is a member of a collecting society that
administers voluntary licensing schemes, via that
society, from any exercise by You of the rights
granted under this License that is for a purpose or
use which is otherwise than noncommercial as
permitted under Section 4(b).

__1__ Except as otherwise agreed in writing by the Licensor
or as may be otherwise permitted by applicable law, if
You Reproduce, Distribute or Publicly Perform the Work
either by itself or as part of any Collections, You must
not distort, mutilate, modify or take other derogatory
action in relation to the Work which would be prejudicial
to the Original Author's honor or reputation.

5. Representations, Warranties and
Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED BY THE PARTIES IN
WRITING, LICENSOR OFFERS THE WORK AS-IS AND MAKES NO
REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING THE
WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING,
WITHOUT LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY,
FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE
ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY, OR THE
PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE.
SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
WARRANTIES, SO SUCH EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability. EXCEPT TO
THE EXTENT REQUIRED BY APPLICABLE LAW, IN NO EVENT WILL
LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY
SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY
DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK,
EVEN IF LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

7. Termination

__1__ This License and the rights granted hereunder will
terminate automatically upon any breach by You of the
terms of this License. Individuals or entities who have
received Collections from You under this License,
however, will not have their licenses terminated provided
such individuals or entities remain in full compliance
with those licenses. Sections 1, 2, 5, 6, 7, and 8 will
survive any termination of this License.

__1__ Subject to the above terms and conditions, the
license granted here is perpetual (for the duration of
the applicable copyright in the Work). Notwithstanding
the above, Licensor reserves the right to release the
Work under different license terms or to stop
distributing the Work at any time; provided, however that
any such election will not serve to withdraw this License
(or any other license that has been, or is required to
be, granted under the terms of this License), and this
License will continue in full force and effect unless
terminated as stated above.

8. Miscellaneous

__1__ Each time You Distribute or Publicly Perform the Work
or a Collection, the Licensor offers to the recipient a
license to the Work on the same terms and conditions as
the license granted to You under this License.

__1__ If any provision of this License is invalid or
unenforceable under applicable law, it shall not affect
the validity or enforceability of the remainder of the
terms of this License, and without further action by the
parties to this agreement, such provision shall be
reformed to the minimum extent necessary to make such
provision valid and enforceable.

__1__ No term or provision of this License shall be deemed
waived and no breach consented to unless such waiver or
consent shall be in writing and signed by the party to be
charged with such waiver or consent.

__1__ This License constitutes the entire agreement between
the parties with respect to the Work licensed here. There
are no understandings, agreements or representations with
respect to the Work not specified here. Licensor shall
not be bound by any additional provisions that may appear
in any communication from You. This License may not be
modified without the mutual written agreement of the
Licensor and You.

__1__ The rights granted under, and the subject matter
referenced, in this License were drafted utilizing the
terminology of the Berne Convention for the Protection of
Literary and Artistic Works (as amended on September 28,
1979), the Rome Convention of 1961, the WIPO Copyright
Treaty of 1996, the WIPO Performances and Phonograms
Treaty of 1996 and the Universal Copyright Convention (as
revised on July 24, 1971). These rights and subject
matter take effect in the relevant jurisdiction in which
the License terms are sought to be enforced according to
the corresponding provisions of the implementation of
those treaty provisions in the applicable national law.
If the standard suite of rights granted under applicable
copyright law includes additional rights not granted
under this License, such additional rights are deemed to
be included in the License; this License is not intended
to restrict the license of any rights under applicable
law.

((
((
Creative Commons Notice
))??
((https://creativecommons.org/licenses/by-nc-nd/3.0
((/legalcode))??
))??

Creative Commons is not a party to this License, and
makes no warranty whatsoever in connection with the Work.
Creative Commons will not be liable to You or any party
on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or
consequential damages arising in connection to this
license. Notwithstanding the foregoing two (2) sentences,
if Creative Commons has expressly identified itself as
the Licensor hereunder, it shall have all rights and
obligations of Licensor.

Except for the limited purpose of indicating to the
public that the Work is licensed under the CCPL, Creative
Commons does not authorize the use by either party of the
trademark "Creative Commons" or any related trademark or
logo of Creative Commons without the prior written
consent of Creative Commons. Any permitted use will be in
compliance with Creative Commons' then-current trademark
usage guidelines, as may be published on its website or
otherwise made available upon request from time to time.
For the avoidance of doubt, this trademark restriction
does not form part of this License.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_NC_ND_3_0_IGO_lre = `//**
Creative Commons Attribution Non Commercial No Derivatives 3.0 IGO
https://spdx.org/licenses/CC-BY-NC-ND-3.0-IGO.json
https://creativecommons.org/licenses/by-nc-nd/3.0/igo/legalcode
**//

(( Creative Commons Attribution Non Commercial No Derivatives 3.0 IGO))??


CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE LEGAL
SERVICES. DISTRIBUTION OF THIS LICENSE DOES NOT CREATE AN ATTORNEY-CLIENT
RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION ON AN "AS-IS" BASIS.
CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE INFORMATION PROVIDED, AND
DISCLAIMS LIABILITY FOR DAMAGES RESULTING FROM ITS USE. THE LICENSOR IS NOT
NECESSARILY AN INTERGOVERNMENTAL ORGANIZATION (IGO), AS DEFINED IN THE LICENSE
BELOW.

License

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("LICENSE"). THE LICENSOR (DEFINED BELOW) HOLDS COPYRIGHT AND
OTHER RIGHTS IN THE WORK. ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER
THIS LICENSE IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE. THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED
HERE IN CONSIDERATION FOR YOUR ACCEPTANCE AND AGREEMENT TO THE TERMS OF THE
LICENSE.

   __1__ Definitions

      __1__ "IGO" means, solely and exclusively for purposes of this License, an
      organization established by a treaty or other instrument governed by
      international law and possessing its own international legal personality.
      Other organizations established to carry out activities across national
      borders and that accordingly enjoy immunity from legal process are also
      IGOs for the sole and exclusive purposes of this License. IGOs may include
      as members, in addition to states, other entities.

      __1__ "Work" means the literary and/or artistic work eligible for
      copyright protection, whatever may be the mode or form of its expression
      including digital form, and offered under the terms of this License. It is
      understood that a database, which by reason of the selection and
      arrangement of its contents constitutes an intellectual creation, is
      considered a Work.

      __1__ "Licensor" means the individual, individuals, entity or entities
      that offer(s) the Work under the terms of this License and may be, but is
      not necessarily, an IGO.

      __1__ "You" means an individual or entity exercising rights under this
      License.

      __1__ "Reproduce" means to make a copy of the Work in any manner or form,
      and by any means.

      __1__ "Distribute" means the activity of making publicly available the
      Work (or copies of the Work), as applicable, by sale, rental, public
      lending or any other known form of transfer of ownership or possession of
      the Work or copy of the Work.

      __1__ "Publicly Perform" means to perform public recitations of the Work
      and to communicate to the public those public recitations, by any means or
      process, including by wire or wireless means or public digital
      performances; to make available to the public Works in such a way that
      members of the public may access these Works from a place and at a place
      individually chosen by them; to perform the Work to the public by any
      means or process and the communication to the public of the performances
      of the Work, including by public digital performance; to broadcast and
      rebroadcast the Work by any means including signs, sounds or images.

      __1__ "Adaptation" means a work derived from or based upon the Work, or
      upon the Work and other pre-existing works. Adaptations may include works
      such as translations, derivative works, or any alterations and
      arrangements of any kind involving the Work. For purposes of this License,
      where the Work is a musical work, performance, or phonogram, the
      synchronization of the Work in timed-relation with a moving image is an
      Adaptation. For the avoidance of doubt, including the Work in a Collection
      is not an Adaptation.

      __1__ "Collection" means a collection of literary or artistic works or
      other works or subject matter other than works listed in Section 1(b)
      which by reason of the selection and arrangement of their contents,
      constitute intellectual creations, in which the Work is included in its
      entirety in unmodified form along with one or more other contributions,
      each constituting separate and independent works in themselves, which
      together are assembled into a collective whole. For the avoidance of
      doubt, a Collection will not be considered as an Adaptation.

   __1__ Scope of this License. Nothing in this License is intended to reduce,
   limit, or restrict any uses free from copyright protection.

   __1__ License Grant. Subject to the terms and conditions of this License, the
   Licensor hereby grants You a worldwide, royalty-free, non-exclusive license
   to exercise the rights in the Work as follows:

      __1__ to Reproduce, Distribute and Publicly Perform the Work, to
      incorporate the Work into one or more Collections, and to Reproduce,
      Distribute and Publicly Perform the Work as incorporated in the
      Collections. This License lasts for the duration of the term of the
      copyright in the Work licensed by the Licensor. The above rights may be
      exercised in all media and formats whether now known or hereafter devised.
      The above rights include the right to make such modifications as are
      technically necessary to exercise the rights in other media and formats,
      but otherwise you have no rights to make Adaptations. All rights not
      expressly granted by the Licensor are hereby reserved, including but not
      limited to the rights set forth in Section 4(d).

   __1__ Restrictions. The license granted in Section 3 above is expressly made
   subject to and limited by the following restrictions:

      __1__ You may Distribute or Publicly Perform the Work only under the terms
      of this License. You must include a copy of, or the Uniform Resource
      Identifier (URI) for, this License with every copy of the Work You
      Distribute or Publicly Perform. You may not offer or impose any terms on
      the Work that restrict the terms of this License or the ability of the
      recipient of the Work to exercise the rights granted to that recipient
      under the terms of the License. You may not sublicense the Work (see
      section 8(a). You must keep intact all notices that refer to this License
      and to the disclaimer of warranties with every copy of the Work You
      Distribute or Publicly Perform. When You Distribute or Publicly Perform
      the Work, You may not impose any effective technological measures on the
      Work that restrict the ability of a recipient of the Work from You to
      exercise the rights granted to that recipient under the terms of the
      License. This Section 4(a) applies to the Work as incorporated in a
      Collection, but this does not require the Collection apart from the Work
      itself to be made subject to the terms of this License. If You create a
      Collection, upon notice from a Licensor You must, to the extent
      practicable, remove from the Collection any credit (inclusive of any logo,
      trademark, official mark or official emblem) as required by Section 4(c),
      as requested.

      __1__ You may not exercise any of the rights granted to You in Section 3
      above in any manner that is primarily intended for or directed toward
      commercial advantage or private monetary compensation. The exchange of the
      Work for other copyrighted works by means of digital file-sharing or
      otherwise shall not be considered to be primarily intended for or directed
      toward commercial advantage or private monetary compensation, provided
      there is no payment of any monetary compensation in connection with the
      exchange of copyrighted works.

      __1__ If You Distribute, or Publicly Perform the Work or any Collections,
      You must, unless a request has been made pursuant to Section 4(a), keep
      intact all copyright notices for the Work and provide, reasonable to the
      medium or means You are utilizing: (i) any attributions that the Licensor
      indicates be associated with the Work as indicated in a copyright notice,
      (ii) the title of the Work if supplied; (iii) to the extent reasonably
      practicable, the URI, if any, that the Licensor specifies to be associated
      with the Work, unless such URI does not refer to the copyright notice or
      licensing information for the Work. The credit required by this Section
      4(c) may be implemented in any reasonable manner; provided, however, that
      in the case of a Collection, at a minimum such credit will appear, if a
      credit for all contributors to the Collection appears, then as part of
      these credits and in a manner at least as prominent as the credits for the
      other contributors. For the avoidance of doubt, You may only use the
      credit required by this Section for the purpose of attribution in the
      manner set out above and, by exercising Your rights under this License,
      You may not implicitly or explicitly assert or imply any connection with,
      sponsorship or endorsement by the Licensor or others designated for
      attribution, of You or Your use of the Work, without the separate, express
      prior written permission of the Licensor or such others.

      __1__ For the avoidance of doubt:

         __1__ Non-waivable Compulsory License Schemes. In those jurisdictions
         in which the right to collect royalties through any statutory or
         compulsory licensing scheme cannot be waived, the Licensor reserves the
         exclusive right to collect such royalties for any exercise by You of
         the rights granted under this License;

         __1__ Waivable Compulsory License Schemes. In those jurisdictions in
         which the right to collect royalties through any statutory or
         compulsory licensing scheme can be waived, the Licensor reserves the
         exclusive right to collect such royalties for any exercise by You of
         the rights granted under this License if Your exercise of such rights
         is for a purpose or use which is otherwise than noncommercial as
         permitted under Section 4(b) and otherwise waives the right to collect
         royalties through any statutory or compulsory licensing scheme; and,

         __1__ Voluntary License Schemes. To the extent possible, the Licensor
         waives the right to collect royalties from You for the exercise of the
         Licensed Rights, whether directly or through a collecting society under
         any voluntary licensing scheme. In all other cases the Licensor
         expressly reserves the right to collect such royalties.

      __1__ Except as otherwise agreed in writing by the Licensor, if You
      Reproduce, Distribute or Publicly Perform the Work either by itself or as
      part of any Collections, You must not distort, mutilate, modify or take
      other derogatory action in relation to the Work which would be prejudicial
      to the honor or reputation of the Licensor where moral rights apply.

   __1__ Representations, Warranties and Disclaimer THE LICENSOR OFFERS THE WORK
   AS-IS AND MAKES NO REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING THE
   WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES OF TITLE, MERCHANTABILITY, FITNESS FOR A PARTICULAR
   PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS,
   ACCURACY, OR THE PRESENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE.

   __1__ Limitation on Liability IN NO EVENT WILL THE LICENSOR BE LIABLE TO YOU
   ON ANY LEGAL THEORY FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR
   EXEMPLARY DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK, EVEN IF
   THE LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

   __1__ Termination

      __1__ Subject to the terms and conditions set forth in this License, the
      license granted here lasts for the duration of the term of the copyright
      in the Work licensed by the Licensor as stated in Section 3.
      Notwithstanding the above, the Licensor reserves the right to release the
      Work under different license terms or to stop distributing the Work at any
      time; provided, however that any such election will not serve to withdraw
      this License (or any other license that has been, or is required to be,
      granted under the terms of this License), and this License will continue
      in full force and effect unless terminated as stated below.

      __1__ If You fail to comply with this License, then this License and the
      rights granted hereunder will terminate automatically upon any breach by
      You of the terms of this License. Individuals or entities who have
      received Collections from You under this License, however, will not have
      their licenses terminated provided such individuals or entities remain in
      full compliance with those licenses. Sections 1, 2, 5, 6, 7, and 8 will
      survive any termination of this License. Notwithstanding the foregoing,
      this License reinstates automatically as of the date the violation is
      cured, provided it is cured within 30 days of You discovering the
      violation, or upon express reinstatement by the Licensor. For the
      avoidance of doubt, this Section 7(b) does not affect any rights the
      Licensor may have to seek remedies for violations of this License by You.

   __1__ Miscellaneous

      __1__ Each time You Distribute or Publicly Perform the Work or a
      Collection, the Licensor offers to the recipient a license to the Work on
      the same terms and conditions as the license granted to You under this
      License.

      __1__ If any provision of this License is invalid or unenforceable, it
      shall not affect the validity or enforceability of the remainder of the
      terms of this License, and without further action, such provision shall be
      reformed to the minimum extent necessary to make such provision valid and
      enforceable.

      __1__ No term or provision of this License shall be deemed waived and no
      breach consented to unless such waiver or consent shall be in writing and
      signed by the Licensor.

      __1__ This License constitutes the entire agreement between You and the
      Licensor with respect to the Work licensed here. There are no
      understandings, agreements or representations with respect to the Work not
      specified here. The Licensor shall not be bound by any additional
      provisions that may appear in any communication from You. This License may
      not be modified without the mutual written agreement of the Licensor and
      You.

      __1__ The rights granted under, and the subject matter referenced, in this
      License were drafted utilizing the terminology of the Berne Convention for
      the Protection of Literary and Artistic Works (as amended on September 28,
      1979), the Rome Convention of 1961, the WIPO Copyright Treaty of 1996, the
      WIPO Performances and Phonograms Treaty of 1996 and the Universal
      Copyright Convention (as revised on July 24, 1971). Interpretation of the
      scope of the rights granted by the Licensor and the conditions imposed on
      You under this License, this License, and the rights and conditions set
      forth herein shall be made with reference to copyright as determined in
      accordance with general principles of international law, including the
      above mentioned conventions.

      __1__ Nothing in this License constitutes or may be interpreted as a
      limitation upon or waiver of any privileges and immunities that may apply
      to the Licensor or You, including immunity from the legal processes of any
      jurisdiction, national court or other authority.

      __1__ Where the Licensor is an IGO, any and all disputes arising under
      this License that cannot be settled amicably shall be resolved in
      accordance with the following procedure:

         __1__ Pursuant to a notice of mediation communicated by reasonable
         means by either You or the Licensor to the other, the dispute shall be
         submitted to non-binding mediation conducted in accordance with rules
         designated by the Licensor in the copyright notice published with the
         Work, or if none then in accordance with those communicated in the
         notice of mediation. The language used in the mediation proceedings
         shall be English unless otherwise agreed.

         __1__ If any such dispute has not been settled within 45 days following
         the date on which the notice of mediation is provided, either You or
         the Licensor may, pursuant to a notice of arbitration communicated by
         reasonable means to the other, elect to have the dispute referred to
         and finally determined by arbitration. The arbitration shall be
         conducted in accordance with the rules designated by the Licensor in
         the copyright notice published with the Work, or if none then in
         accordance with the UNCITRAL Arbitration Rules as then in force. The
         arbitral tribunal shall consist of a sole arbitrator and the language
         of the proceedings shall be English unless otherwise agreed. The place
         of arbitration shall be where the Licensor has its headquarters. The
         arbitral proceedings shall be conducted remotely (e.g., via telephone
         conference or written submissions) whenever practicable.

         __1__ Interpretation of this License in any dispute submitted to
         mediation or arbitration shall be as set forth in Section 8(e), above.

Creative Commons Notice

Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work. Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license. Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of the Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, Creative Commons does not authorize the use by either
party of the trademark "Creative Commons" or any related trademark or logo of
Creative Commons without the prior written consent of Creative Commons. Any
permitted use will be in compliance with Creative Commons' then-current
trademark usage guidelines, as may be published on its website or otherwise made
available upon request from time to time. For the avoidance of doubt, this
trademark restriction does not form part of this License.

Creative Commons may be contacted at https:/creativecommons.org/.
`
const license_CC_BY_NC_ND_4_0_lre = `//**
CC-BY-NC-ND-4.0
https://creativecommons.org/licenses/by-nc-nd/4.0
**//

((Creative Commons))??

((
Creative Commons Legal Code
))??
((https://creativecommons.org/licenses/by-nc-nd/4.0
((/legalcode))??
))??

((
Attribution-NonCommercial-NoDerivatives 4.0 International
))??
((https://creativecommons.org/licenses/by-nc-nd/4.0
((/legalcode))??
))??

((
Official translations of this license are available in
other languages.
))??

((
Creative Commons Corporation (“Creative Commons”) is not a law firm and does
not provide legal services or legal advice. Distribution of Creative Commons
public licenses does not create a lawyer-client or other relationship. Creative
Commons makes its licenses and related information available on an “as-is”
basis. Creative Commons gives no warranties regarding its licenses, any material
licensed under their terms and conditions, or any related information. Creative
Commons disclaims all liability for damages resulting from their use to the
fullest extent possible.
))??

((
Using Creative Commons Public Licenses

Creative Commons public licenses provide a standard set of terms and conditions
that creators and other rights holders may use to share original works of
authorship and other material subject to copyright and certain other rights
specified in the public license below. The following considerations are for
informational purposes only, are not exhaustive, and do not form part of our
licenses.

Considerations for licensors: Our public licenses are intended for use by those
authorized to give the public permission to use material in ways otherwise
restricted by copyright and certain other rights. Our licenses are irrevocable.
Licensors should read and understand the terms and conditions of the license
they choose before applying it. Licensors should also secure all rights
necessary before applying our licenses so that the public can reuse the material
as expected. Licensors should clearly mark any material not subject to the
license. This includes other CC-licensed material, or material used under an
exception or limitation to copyright. More considerations for licensors.

Considerations for the public: By using one of our public licenses, a licensor
grants the public permission to use the licensed material under specified terms
and conditions. If the licensor’s permission is not necessary for any
reason–for example, because of any applicable exception or limitation to
copyright–then that use is not regulated by the license. Our licenses grant
only permissions under copyright and certain other rights that a licensor has
authority to grant. Use of the licensed material may still be restricted for
other reasons, including because others have copyright or other rights in the
material. A licensor may make special requests, such as asking that all changes
be marked or described. Although not required by our licenses, you are
encouraged to respect those requests where reasonable.
More considerations for the public.
))??

((
Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International
Public License
))??
((https://creativecommons.org/licenses/by-nc-nd/4.0
((/legalcode))??
))??

By exercising the Licensed Rights (defined below), You accept and agree to be
bound by the terms and conditions of this Creative Commons
Attribution-NonCommercial-NoDerivatives 4.0 International Public License
("Public License"). To the extent this Public License may be interpreted as a
contract, You are granted the Licensed Rights in consideration of Your
acceptance of these terms and conditions, and the Licensor grants You such
rights in consideration of benefits the Licensor receives from making the
Licensed Material available under these terms and conditions.

Section 1 – Definitions.

__1__ Adapted Material means material subject to Copyright and Similar Rights
that is derived from or based upon the Licensed Material and in which the
Licensed Material is translated, altered, arranged, transformed, or otherwise
modified in a manner requiring permission under the Copyright and Similar Rights
held by the Licensor. For purposes of this Public License, where the Licensed
Material is a musical work, performance, or sound recording, Adapted Material is
always produced where the Licensed Material is synched in timed relation with a
moving image.

__1__ Copyright and Similar Rights means copyright and/or similar rights closely
related to copyright including, without limitation, performance, broadcast,
sound recording, and Sui Generis Database Rights, without regard to how the
rights are labeled or categorized. For purposes of this Public License, the
rights specified in Section 2(b)(1)-(2) are not Copyright and Similar Rights.

__1__ Effective Technological Measures means those measures that, in the absence
of proper authority, may not
be circumvented under laws fulfilling obligations under Article 11 of the WIPO
Copyright Treaty adopted on December 20, 1996, and/or similar
international agreements.

__1__ Exceptions and Limitations means fair use, fair dealing, and/or any other
exception or limitation to Copyright and Similar Rights that applies to Your use
of the Licensed Material.

__1__ Licensed Material means the artistic or literary work, database, or other
material to which the Licensor applied this Public License.

__1__ Licensed Rights means the rights granted to You subject to the terms and
conditions of this Public License, which are limited to all Copyright and
Similar Rights that apply to Your use of the Licensed Material and that the
Licensor has authority to license.

__1__ Licensor means the individual(s) or entity(ies) granting rights under this
Public License.

__1__ NonCommercial means not primarily intended for or directed towards
commercial advantage or monetary compensation. For purposes of this Public
License, the exchange of the Licensed Material for other material subject to
Copyright and Similar Rights by digital file-sharing or similar means is
NonCommercial provided there is no payment of monetary compensation in
connection with the exchange.

__1__ Share means to provide material to the public by any means or process that
requires permission under the Licensed Rights, such as reproduction, public
display, public performance, distribution, dissemination, communication, or
importation, and to make material available to the public including in ways that
members of the public may access the material from a place and at a time
individually chosen by them.

__1__ Sui Generis Database Rights means rights other than copyright resulting
from Directive 96/9/EC of the European Parliament and of the Council of 11 March
1996 on the legal protection of databases, as amended and/or succeeded, as well
as other essentially equivalent rights anywhere in the world.

__1__ You means the individual or entity exercising the Licensed Rights under
this Public License. Your has a corresponding meaning.

Section 2 – Scope.

__1__ License grant.

__1__ Subject to the terms and conditions of this Public License, the Licensor
hereby grants You a worldwide, royalty-free, non-sublicensable, non-exclusive,
irrevocable license to exercise the Licensed Rights in the Licensed Material
to:

__1__ reproduce and Share the Licensed Material, in whole or in part, for
NonCommercial purposes only; and

__1__ produce and reproduce, but not Share, Adapted Material for NonCommercial
purposes only.

__1__ Exceptions and Limitations. For the avoidance of doubt, where Exceptions
and Limitations apply to Your use, this Public License does not apply, and You
do not need to comply with its terms and conditions.

__1__ Term. The term of this Public License is specified in Section 6(a).

__1__ Media and formats; technical modifications allowed. The Licensor
authorizes You to exercise the Licensed Rights in all media and formats whether
now known or hereafter created, and to make technical modifications necessary to
do so. The Licensor waives and/or agrees not to assert any right or authority to
forbid You from making technical modifications necessary to exercise the
Licensed Rights, including technical modifications necessary to circumvent
Effective Technological Measures. For purposes of this Public License, simply
making modifications authorized by this Section 2(a)(4) never produces Adapted
Material.

__1__ Downstream recipients.

__1__ Offer from the Licensor – Licensed Material. Every recipient of the
Licensed Material automatically receives an  offer from the Licensor to exercise
the Licensed Rights under the terms and conditions of this Public License.

__1__ No downstream restrictions. You may not offer or impose any additional or
different terms or  conditions on, or apply any Effective Technological Measures
to, the Licensed Material if doing so restricts exercise of the Licensed Rights
by any recipient of the Licensed Material.

__1__ No endorsement. Nothing in this Public License constitutes or may be
construed as permission to assert or imply that You are, or that Your use of the
Licensed Material is, connected with, or sponsored, endorsed, or granted
official status by, the Licensor or others designated to receive attribution as
provided in Section 3(a)(1)(A)(i).

__1__
Other rights.

__1__ Moral rights, such as the right of integrity, are not licensed under this
Public License, nor are publicity, privacy, and/or other similar personality
rights; however, to the extent possible, the Licensor waives and/or agrees not
to assert any such rights held by the Licensor to the limited extent necessary
to allow You to exercise the Licensed Rights, but not otherwise.

__1__ Patent and trademark rights are not licensed under this Public License.

__1__ To the extent possible, the Licensor waives any right to collect royalties
from You for the exercise of the Licensed Rights, whether directly or through a
collecting society under any voluntary or waivable statutory or compulsory
licensing scheme. In all other cases the Licensor expressly reserves any right
to collect such royalties, including when the Licensed Material is used other
than for NonCommercial purposes.

Section 3 – License Conditions.

Your exercise of the Licensed Rights is expressly made subject to the following
conditions.

__1__
Attribution.

__1__
If You Share the Licensed Material, You must:

__1__ retain the following if it is supplied by the Licensor with the Licensed
Material:

__1__ identification of the creator(s) of the Licensed Material and any others
designated to receive attribution, in any reasonable manner requested  by the
Licensor (including by pseudonym if designated);

__1__ a copyright notice;

__1__ a notice that refers to this Public License;

__1__ a notice that refers to the disclaimer of warranties;

__1__ a URI or hyperlink to the Licensed Material to the extent reasonably
practicable;

__1__ indicate if You modified the Licensed Material and retain an indication of
any previous modifications; and

__1__ indicate the Licensed Material is licensed under this Public License,
and include the text of, or the URI or hyperlink to, this Public
License.

__1__ For the avoidance of doubt, You do not have permission under this Public
License to Share Adapted Material.

__1__ You may satisfy the conditions in Section 3(a)(1) in any reasonable manner
based on the medium, means, and context in which You Share the Licensed
Material. For example, it may be reasonable to satisfy the conditions by
providing a URI or hyperlink to a resource that includes the required
information.

__1__ If requested by the Licensor, You must remove any of the information
required by Section 3(a)(1)(A) to the extent reasonably practicable.

Section 4 – Sui Generis Database Rights.

Where the Licensed Rights include Sui Generis Database Rights that apply to Your
use of the Licensed Material:

__1__ for the avoidance of doubt, Section 2(a)(1) grants You the right to
extract, reuse, reproduce, and Share all or a substantial portion of the
contents of the database for NonCommercial purposes only and provided You do not
Share Adapted Material;

__1__ if You include all or a substantial portion of the database contents in a
database in which You have Sui Generis Database Rights, then the database in
which You have Sui Generis Database Rights (but not its individual contents) is
Adapted Material; and

__1__ You must comply with the conditions in Section 3(a) if You Share all or a
substantial portion of the contents of the database.

__1__ For the avoidance of doubt, this Section 4 supplements and
does not replace Your obligations under this Public License
where the Licensed Rights include other Copyright and Similar
Rights.

Section 5 – Disclaimer of Warranties and Limitation of Liability.

__1__ Unless otherwise separately undertaken by the Licensor, to the extent
possible, the Licensor offers the Licensed Material as-is and as-available, and
makes no representations or warranties of any kind concerning the Licensed
Material, whether express, implied, statutory, or other. This includes, without
limitation, warranties of title, merchantability, fitness for a particular
purpose, non-infringement, absence of latent or other defects, accuracy, or the
presence or absence of errors, whether or not known or discoverable. Where
disclaimers of warranties are not allowed in full or in part, this disclaimer
may not apply to You.

__1__ To the extent possible, in no event will the Licensor be liable to You on
any legal theory (including, without limitation, negligence) or otherwise for
any direct, special, indirect, incidental, consequential, punitive, exemplary,
or other losses, costs, expenses, or damages arising out of this Public License
or use of the Licensed Material, even if the Licensor has been advised of the
possibility of such losses, costs, expenses, or damages. Where a limitation of
liability is not allowed in full or in part, this limitation may not apply to
You.

__1__ The disclaimer of warranties and limitation of liability provided above
shall be interpreted in a manner that, to the extent possible, most closely
approximates an absolute disclaimer and waiver of all liability.

Section 6 – Term and Termination.

__1__ This Public License applies for the term of the Copyright and Similar
Rights licensed here. However, if You fail to comply with this Public License,
then Your rights under this Public License terminate automatically.

__1__

Where Your right to use the Licensed Material has terminated under Section 6(a),
it reinstates:

__1__ automatically as of the date the violation is cured, provided it is cured
within 30 days of Your discovery of the violation; or

__1__ upon express reinstatement by the Licensor.

__1__ For the avoidance of doubt, this Section 6(b) does not
affect any right the Licensor may have to seek remedies for
Your violations of this Public License.

__1__ For the avoidance of doubt, the Licensor may also offer the Licensed
Material under separate terms or conditions or stop distributing the Licensed
Material at any time; however, doing so will not terminate this Public License.

__1__ Sections 1, 5, 6, 7, and 8 survive termination of this Public License.

Section 7 – Other Terms and Conditions.

__1__ The Licensor shall not be bound by any additional or different terms or
conditions communicated by You unless expressly agreed.

__1__ Any arrangements, understandings, or agreements regarding the Licensed
Material not stated herein are separate from and independent of the terms and
conditions of this Public License.

Section 8 – Interpretation.

__1__ For the avoidance of doubt, this Public License does not, and shall not be
interpreted to, reduce, limit, restrict, or impose conditions on any use of the
Licensed Material that could lawfully be made without permission under this
Public License.

__1__ To the extent possible, if any provision of this Public License is deemed
unenforceable, it shall be automatically reformed to the minimum extent
necessary to make it enforceable. If the provision cannot be reformed, it shall
be severed from this Public License without affecting the enforceability of the
remaining terms and conditions.

__1__ No term or condition of this Public License will be waived and no failure
to comply consented to unless expressly agreed to by the Licensor.

__1__ Nothing in this Public License constitutes or may be interpreted as a
limitation upon, or waiver of, any privileges and immunities that apply to the
Licensor or You, including from the legal processes of any jurisdiction or
authority.

((
Creative Commons is not a party to its public licenses. Notwithstanding,
Creative Commons may elect to apply one of its public licenses to material it
publishes and in those instances will be considered the “Licensor.” The text
of the Creative Commons public licenses is dedicated to the public domain under
the CC0 Public Domain Dedication. Except for the limited purpose of indicating
that material is shared under a Creative Commons public license or as otherwise
permitted by the Creative Commons policies published at
creativecommons.org/policies, Creative Commons does not authorize the use of the
trademark “Creative Commons” or any other trademark or logo of Creative
Commons without its prior written consent including, without limitation, in
connection with any unauthorized modifications to any of its public licenses or
any other arrangements, understandings, or agreements concerning use of licensed
material. For the avoidance of doubt, this paragraph does not form part of the
public licenses.
Creative Commons may be contacted at creativecommons.org.
))??

((
Additional languages available:

العربية,
čeština,
Deutsch,
Ελληνικά,
Español,
euskara,
suomeksi,
français,
hrvatski,
Bahasa Indonesia,
italiano,
日本語,
한국어,
Lietuvių,
latviski,
te reo Māori,
Nederlands,
norsk,
polski,
português,
română,
русский,
Slovenščina,
svenska,
Türkçe,
українська,
中文,
華語.

Please read the FAQ for more information about official translations.
))??
`
const license_CC_BY_NC_SA_1_0_lre = `//**
CC-BY-NC-SA-1.0
https://creativecommons.org/licenses/by-nc-sa/1.0
**//

((Creative Commons))??

Creative Commons

Attribution-NonCommercial-ShareAlike 1.0

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE
LEGAL
SERVICES. DISTRIBUTION OF THIS DRAFT LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION
ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING
FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE").  THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW.  ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE.  THE LICENSOR GRANTS YOU THE RIGHTS
CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole.  A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant.  Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to create and reproduce Derivative Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission Derivative Works;

The above rights may be exercised in all media and formats whether now known or
hereafter devised.  The above rights include the right to make such
modifications as are technically necessary to exercise the rights in other media
and formats.  All rights not expressly granted by Licensor are hereby reserved.

4. Restrictions. The license granted in Section 3 above is expressly made
subject to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform.  You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder.  You may not sublicense the Work.  You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement.  The above applies to the Work as incorporated in a
Collective Work, but this does not require the Collective Work apart from the
Work itself to be made subject to the terms of this License. If You create a
Collective Work, upon notice from any Licensor You must, to the extent
practicable, remove from the Collective Work any reference to such Licensor or
the Original Author, as requested.  If You create a Derivative Work, upon notice
from any Licensor You must, to the extent practicable, remove from the
Derivative Work any reference to such Licensor or the Original Author, as
requested.

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform a Derivative Work only under the terms of this License, and You must
include a copy of, or the Uniform Resource Identifier for, this License with
every copy or phonorecord of each Derivative Work You distribute, publicly
display, publicly perform, or publicly digitally perform. You may not offer or
impose any terms on the Derivative Works that alter or restrict the terms of
this License or the recipients' exercise of the rights granted hereunder, and
You must keep intact all notices that refer to this License and to the
disclaimer of warranties.  You may not distribute, publicly display, publicly
perform, or publicly digitally perform the Derivative Work with any
technological measures that control access or use of the Work in a manner
inconsistent with the terms of this License Agreement.  The above applies to the
Derivative Work as incorporated in a Collective Work, but this does not require
the Collective Work apart from the Derivative Work itself to be made subject to
the terms of this License.

__1__
You may not exercise any of the rights granted to You in Section 3 above in any
manner that is primarily intended for or directed toward commercial advantage or
private monetary compensation.  The exchange of the Work for other copyrighted
works by means of digital file-sharing or otherwise shall not be considered to
be intended for or directed toward commercial advantage or private monetary
compensation, provided there is no payment of any monetary compensation in
connection with the exchange of copyrighted works.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or any Derivative Works or Collective Works, You must keep
intact all copyright notices for the Work and give the Original Author credit
reasonable to the medium or means You are utilizing by conveying the name (or
pseudonym if applicable) of the Original Author if supplied; the title of the
Work if supplied; in the case of a Derivative Work, a credit identifying the use
of the Work in the Derivative Work (e.g., "French translation of the Work by
Original Author," or "Screenplay based on original Work by Original Author").
Such credit may be implemented in any reasonable manner; provided, however, that
in the case of a Derivative Work or Collective Work, at a minimum such credit
will appear where any other comparable authorship credit appears and in a manner
at least as prominent as such other comparable authorship credit.

5. Representations, Warranties and Disclaimer

__1__
By offering the Work for public release under this License, Licensor represents
and warrants that, to the best of Licensor's knowledge after reasonable
inquiry:

__1__
Licensor has secured all rights in the Work necessary to grant the license
rights hereunder and to permit the lawful exercise of the rights granted
hereunder without You having any obligation to pay any royalties, compulsory
license fees, residuals or any other payments;

__1__
The Work does not infringe the copyright, trademark, publicity rights, common
law rights or any other right of any third party or constitute defamation,
invasion of privacy or other tortious injury to any third party.

__1__
EXCEPT AS EXPRESSLY STATED IN THIS LICENSE OR OTHERWISE AGREED IN WRITING OR
REQUIRED BY APPLICABLE LAW, THE WORK IS LICENSED ON AN "AS IS" BASIS, WITHOUT
WARRANTIES OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION,
ANY WARRANTIES REGARDING THE CONTENTS OR ACCURACY OF THE WORK.

6. Limitation on Liability.  EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW,
AND EXCEPT FOR DAMAGES ARISING FROM LIABILITY TO A THIRD PARTY RESULTING FROM
BREACH OF THE WARRANTIES IN SECTION 5, IN NO EVENT WILL LICENSOR BE LIABLE TO
YOU ON ANY LEGAL THEORY FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR
EXEMPLARY DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK, EVEN IF
LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Derivative Works or Collective Works from You under this License,
however, will not have their licenses terminated provided such individuals or
entities remain in full compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work).  Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
Each time You distribute or publicly digitally perform a Derivative Work,
Licensor offers to the recipient a license to the original Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here.  There are no understandings, agreements or
representations with respect to the Work not specified here.  Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work.  Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license.  Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons.  Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_NC_SA_2_0_lre = `//**
CC-BY-NC-SA-2.0
https://creativecommons.org/licenses/by-nc-sa/2.0
**//

((Creative Commons))??

Creative Commons

Attribution-NonCommercial-ShareAlike 2.0

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE
LEGAL
SERVICES. DISTRIBUTION OF THIS LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION
ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING
FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE").  THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW.  ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE.  THE LICENSOR GRANTS YOU THE RIGHTS
CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole.  A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License. For
the avoidance of doubt, where the Work is a musical composition or sound
recording, the synchronization of the Work in timed-relation with a moving image
("synching") will be considered a Derivative Work for the purpose of this
License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

__1__ "License Elements" means the following high-level license attributes as
selected by Licensor and indicated in the title of this License: Attribution,
Noncommercial, ShareAlike.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant.  Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to create and reproduce Derivative Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission Derivative Works;

The above rights may be exercised in all media and formats whether now known or
hereafter devised.  The above rights include the right to make such
modifications as are technically necessary to exercise the rights in other media
and formats.  All rights not expressly granted by Licensor are hereby reserved,
including but not limited to the rights set forth in Sections 4(e) and 4(f).

4. Restrictions.The license granted in Section 3 above is expressly made subject
to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform.  You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder.  You may not sublicense the Work.  You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement.  The above applies to the Work as incorporated in a
Collective Work, but this does not require the Collective Work apart from the
Work itself to be made subject to the terms of this License. If You create a
Collective Work, upon notice from any Licensor You must, to the extent
practicable, remove from the Collective Work any reference to such Licensor or
the Original Author, as requested.  If You create a Derivative Work, upon notice
from any Licensor You must, to the extent practicable, remove from the
Derivative Work any reference to such Licensor or the Original Author, as
requested.

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform a Derivative Work only under the terms of this License, a later version
of this License with the same License Elements as this License, or a Creative
Commons iCommons license that contains the same License Elements as this License
(e.g. Attribution-NonCommercial-ShareAlike 2.0 Japan). You must include a copy
of, or the Uniform Resource Identifier for, this License or other license
specified in the previous sentence with every copy or phonorecord of each
Derivative Work You distribute, publicly display, publicly perform, or publicly
digitally perform. You may not offer or impose any terms on the Derivative Works
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder, and You must keep intact all notices that refer to
this License and to the disclaimer of warranties. You may not distribute,
publicly display, publicly perform, or publicly digitally perform the Derivative
Work with any technological measures that control access or use of the Work in a
manner inconsistent with the terms of this License Agreement. The above applies
to the Derivative Work as incorporated in a Collective Work, but this does not
require the Collective Work apart from the Derivative Work itself to be made
subject to the terms of this License.

__1__
You may not exercise any of the rights granted to You in Section 3 above in any
manner that is primarily intended for or directed toward commercial advantage or
private monetary compensation.  The exchange of the Work for other copyrighted
works by means of digital file-sharing or otherwise shall not be considered to
be intended for or directed toward commercial advantage or private monetary
compensation, provided there is no payment of any monetary compensation in
connection with the exchange of copyrighted works.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or any Derivative Works or Collective Works, You must keep
intact all copyright notices for the Work and give the Original Author credit
reasonable to the medium or means You are utilizing by conveying the name (or
pseudonym if applicable) of the Original Author if supplied; the title of the
Work if supplied; to the extent reasonably practicable, the Uniform Resource
Identifier, if any, that Licensor specifies to be associated with the Work,
unless such URI does not refer to the copyright notice or licensing information
for the Work; and in the case of a Derivative Work, a credit identifying the use
of the Work in the Derivative Work (e.g., "French translation of the Work by
Original Author," or "Screenplay based on original Work by Original Author").
Such credit may be implemented in any reasonable manner; provided, however, that
in the case of a Derivative Work or Collective Work, at a minimum such credit
will appear where any other comparable authorship credit appears and in a manner
at least as prominent as such other comparable authorship credit.

__1__
For the avoidance of doubt, where the Work is a musical composition:

__1__ Performance Royalties Under Blanket Licenses. Licensor reserves the
exclusive right to collect, whether individually or via a performance rights
society (e.g. ASCAP, BMI, SESAC), royalties for the public performance or public
digital performance (e.g. webcast) of the Work if that performance is primarily
intended for or directed toward commercial advantage or private monetary
compensation.

__1__ Mechanical Rights and Statutory Royalties. Licensor reserves the exclusive
right to collect, whether individually or via a music rights agency or
designated agent (e.g. Harry Fox Agency), royalties for any phonorecord You
create from the Work ("cover version") and distribute, subject to the compulsory
license created by 17 USC Section 115 of the US Copyright Act (or the equivalent
in other jurisdictions), if Your distribution of such cover version is primarily
intended for or directed toward commercial advantage or private monetary
compensation.

__1__ Webcasting Rights and Statutory Royalties. For the avoidance of doubt,
where the Work is a sound recording, Licensor reserves the exclusive right to
collect, whether individually or via a performance-rights society (e.g.
SoundExchange), royalties for the public digital performance (e.g. webcast) of
the Work, subject to the compulsory license created by 17 USC Section 114 of the
US Copyright Act (or the equivalent in other jurisdictions), if Your public
digital performance is primarily intended for or directed toward commercial
advantage or private monetary compensation.

5. Representations, Warranties and Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN WRITING, LICENSOR OFFERS
THE WORK AS-IS AND MAKES NO REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING
THE WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT
LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY, FITNESS FOR A PARTICULAR
PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY,
OR THE PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE. SOME
JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO SUCH
EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability.  EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN
NO EVENT WILL LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY SPECIAL,
INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY DAMAGES ARISING OUT OF THIS
LICENSE OR THE USE OF THE WORK, EVEN IF LICENSOR HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Derivative Works or Collective Works from You under this License,
however, will not have their licenses terminated provided such individuals or
entities remain in full compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work).  Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
Each time You distribute or publicly digitally perform a Derivative Work,
Licensor offers to the recipient a license to the original Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here.  There are no understandings, agreements or
representations with respect to the Work not specified here.  Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work.  Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license.  Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons.  Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_NC_SA_2_5_lre = `//**
CC-BY-NC-SA-2.5
https://creativecommons.org/licenses/by-nc-sa/2.5
**//

((Creative Commons))??

Creative Commons

Attribution-NonCommercial-ShareAlike 2.5

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE
LEGAL
SERVICES. DISTRIBUTION OF THIS LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION
ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING
FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE").  THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW.  ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE.  THE LICENSOR GRANTS YOU THE RIGHTS
CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole.  A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License. For
the avoidance of doubt, where the Work is a musical composition or sound
recording, the synchronization of the Work in timed-relation with a moving image
("synching") will be considered a Derivative Work for the purpose of this
License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

__1__ "License Elements" means the following high-level license attributes as
selected by Licensor and indicated in the title of this License: Attribution,
Noncommercial, ShareAlike.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant.  Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to create and reproduce Derivative Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission Derivative Works;

The above rights may be exercised in all media and formats whether now known or
hereafter devised.  The above rights include the right to make such
modifications as are technically necessary to exercise the rights in other media
and formats.  All rights not expressly granted by Licensor are hereby reserved,
including but not limited to the rights set forth in Sections 4(e) and 4(f).

4. Restrictions.The license granted in Section 3 above is expressly made subject
to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform.  You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder.  You may not sublicense the Work.  You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement.  The above applies to the Work as incorporated in a
Collective Work, but this does not require the Collective Work apart from the
Work itself to be made subject to the terms of this License. If You create a
Collective Work, upon notice from any Licensor You must, to the extent
practicable, remove from the Collective Work any credit as required by clause
4(d), as requested.  If You create a Derivative Work, upon notice from any
Licensor You must, to the extent practicable, remove from the Derivative Work
any credit as required by clause 4(d), as requested.

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform a Derivative Work only under the terms of this License, a later version
of this License with the same License Elements as this License, or a Creative
Commons iCommons license that contains the same License Elements as this License
(e.g. Attribution-NonCommercial-ShareAlike 2.5 Japan). You must include a copy
of, or the Uniform Resource Identifier for, this License or other license
specified in the previous sentence with every copy or phonorecord of each
Derivative Work You distribute, publicly display, publicly perform, or publicly
digitally perform. You may not offer or impose any terms on the Derivative Works
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder, and You must keep intact all notices that refer to
this License and to the disclaimer of warranties. You may not distribute,
publicly display, publicly perform, or publicly digitally perform the Derivative
Work with any technological measures that control access or use of the Work in a
manner inconsistent with the terms of this License Agreement. The above applies
to the Derivative Work as incorporated in a Collective Work, but this does not
require the Collective Work apart from the Derivative Work itself to be made
subject to the terms of this License.

__1__
You may not exercise any of the rights granted to You in Section 3 above in any
manner that is primarily intended for or directed toward commercial advantage or
private monetary compensation.  The exchange of the Work for other copyrighted
works by means of digital file-sharing or otherwise shall not be considered to
be intended for or directed toward commercial advantage or private monetary
compensation, provided there is no payment of any monetary compensation in
connection with the exchange of copyrighted works.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or any Derivative Works or Collective Works, You must keep
intact all copyright notices for the Work and provide, reasonable to the medium
or means You are utilizing: (i) the name of the Original Author (or pseudonym,
if applicable) if supplied, and/or (ii) if the Original Author and/or Licensor
designate another party or parties (e.g. a sponsor institute, publishing entity,
journal) for attribution in Licensor's copyright notice, terms of service or by
other reasonable means, the name of such party or parties; the title of the Work
if supplied; to the extent reasonably practicable, the Uniform Resource
Identifier, if any, that Licensor specifies to be associated with the Work,
unless such URI does not refer to the copyright notice or licensing information
for the Work; and in the case of a Derivative Work, a credit identifying the use
of the Work in the Derivative Work (e.g., "French translation of the Work by
Original Author," or "Screenplay based on original Work by Original Author").
Such credit may be implemented in any reasonable manner; provided, however, that
in the case of a Derivative Work or Collective Work, at a minimum such credit
will appear where any other comparable authorship credit appears and in a manner
at least as prominent as such other comparable authorship credit.

__1__
For the avoidance of doubt, where the Work is a musical composition:

__1__ Performance Royalties Under Blanket Licenses. Licensor reserves the
exclusive right to collect, whether individually or via a performance rights
society (e.g. ASCAP, BMI, SESAC), royalties for the public performance or public
digital performance (e.g. webcast) of the Work if that performance is primarily
intended for or directed toward commercial advantage or private monetary
compensation.

__1__ Mechanical Rights and Statutory Royalties. Licensor reserves the exclusive
right to collect, whether individually or via a music rights agency or
designated agent (e.g. Harry Fox Agency), royalties for any phonorecord You
create from the Work ("cover version") and distribute, subject to the compulsory
license created by 17 USC Section 115 of the US Copyright Act (or the equivalent
in other jurisdictions), if Your distribution of such cover version is primarily
intended for or directed toward commercial advantage or private monetary
compensation.

__1__ Webcasting Rights and Statutory Royalties. For the avoidance of doubt,
where the Work is a sound recording, Licensor reserves the exclusive right to
collect, whether individually or via a performance-rights society (e.g.
SoundExchange), royalties for the public digital performance (e.g. webcast) of
the Work, subject to the compulsory license created by 17 USC Section 114 of the
US Copyright Act (or the equivalent in other jurisdictions), if Your public
digital performance is primarily intended for or directed toward commercial
advantage or private monetary compensation.

5. Representations, Warranties and Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN WRITING, LICENSOR OFFERS
THE WORK AS-IS AND MAKES NO REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING
THE WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT
LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY, FITNESS FOR A PARTICULAR
PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY,
OR THE PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE. SOME
JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO SUCH
EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability.  EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN
NO EVENT WILL LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY SPECIAL,
INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY DAMAGES ARISING OUT OF THIS
LICENSE OR THE USE OF THE WORK, EVEN IF LICENSOR HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Derivative Works or Collective Works from You under this License,
however, will not have their licenses terminated provided such individuals or
entities remain in full compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work).  Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
Each time You distribute or publicly digitally perform a Derivative Work,
Licensor offers to the recipient a license to the original Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here.  There are no understandings, agreements or
representations with respect to the Work not specified here.  Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work.  Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license.  Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons.  Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_NC_SA_3_0_lre = `//**
CC-BY-NC-SA-3.0
https://creativecommons.org/licenses/by-nc-sa/3.0
**//

((Creative Commons))??

((
Creative Commons Legal Code
))??
((https://creativecommons.org/licenses/by-nc-sa/3.0
((/legalcode))??
))??

((
Attribution-NonCommercial-ShareAlike 3.0 Unported
))??
((https://creativecommons.org/licenses/by-nc-sa/3.0
((/legalcode))??
))??

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES
NOT PROVIDE LEGAL SERVICES. DISTRIBUTION OF THIS LICENSE
DOES NOT CREATE AN ATTORNEY-CLIENT RELATIONSHIP. CREATIVE
COMMONS PROVIDES THIS INFORMATION ON AN "AS-IS" BASIS.
CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES
RESULTING FROM ITS USE.
))??

((
((License))??
))??
((https://creativecommons.org/licenses/by-nc-sa/3.0
((/legalcode))??
))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS
OF THIS CREATIVE COMMONS PUBLIC LICENSE ("CCPL" OR
"LICENSE"). THE WORK IS PROTECTED BY COPYRIGHT AND/OR OTHER
APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS
AUTHORIZED UNDER THIS LICENSE OR COPYRIGHT LAW IS
PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU
ACCEPT AND AGREE TO BE BOUND BY THE TERMS OF THIS LICENSE.
TO THE EXTENT THIS LICENSE MAY BE CONSIDERED TO BE A
CONTRACT, THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED HERE
IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND
CONDITIONS.

1. Definitions

__1__ "Adaptation" means a work based upon
the Work, or upon the Work and other pre-existing works,
such as a translation, adaptation, derivative work,
arrangement of music or other alterations of a literary
or artistic work, or phonogram or performance and
includes cinematographic adaptations or any other form in
which the Work may be recast, transformed, or adapted
including in any form recognizably derived from the
original, except that a work that constitutes a
Collection will not be considered an Adaptation for the
purpose of this License. For the avoidance of doubt,
where the Work is a musical work, performance or
phonogram, the synchronization of the Work in
timed-relation with a moving image ("synching") will be
considered an Adaptation for the purpose of this
License.

__1__ "Collection" means a collection of
literary or artistic works, such as encyclopedias and
anthologies, or performances, phonograms or broadcasts,
or other works or subject matter other than works listed
in Section 1(g) below, which, by reason of the selection
and arrangement of their contents, constitute
intellectual creations, in which the Work is included in
its entirety in unmodified form along with one or more
other contributions, each constituting separate and
independent works in themselves, which together are
assembled into a collective whole. A work that
constitutes a Collection will not be considered an
Adaptation (as defined above) for the purposes of this
License.

__1__ "Distribute" means to make available
to the public the original and copies of the Work or
Adaptation, as appropriate, through sale or other
transfer of ownership.

__1__ "License Elements" means the
following high-level license attributes as selected by
Licensor and indicated in the title of this License:
Attribution, Noncommercial, ShareAlike.

__1__ "Licensor" means the individual,
individuals, entity or entities that offer(s) the Work
under the terms of this License.

__1__ "Original Author" means, in the case
of a literary or artistic work, the individual,
individuals, entity or entities who created the Work or
if no individual or entity can be identified, the
publisher; and in addition (i) in the case of a
performance the actors, singers, musicians, dancers, and
other persons who act, sing, deliver, declaim, play in,
interpret or otherwise perform literary or artistic works
or expressions of folklore; (ii) in the case of a
phonogram the producer being the person or legal entity
who first fixes the sounds of a performance or other
sounds; and, (iii) in the case of broadcasts, the
organization that transmits the broadcast.

__1__ "Work" means the literary and/or
artistic work offered under the terms of this License
including without limitation any production in the
literary, scientific and artistic domain, whatever may be
the mode or form of its expression including digital
form, such as a book, pamphlet and other writing; a
lecture, address, sermon or other work of the same
nature; a dramatic or dramatico-musical work; a
choreographic work or entertainment in dumb show; a
musical composition with or without words; a
cinematographic work to which are assimilated works
expressed by a process analogous to cinematography; a
work of drawing, painting, architecture, sculpture,
engraving or lithography; a photographic work to which
are assimilated works expressed by a process analogous to
photography; a work of applied art; an illustration, map,
plan, sketch or three-dimensional work relative to
geography, topography, architecture or science; a
performance; a broadcast; a phonogram; a compilation of
data to the extent it is protected as a copyrightable
work; or a work performed by a variety or circus
performer to the extent it is not otherwise considered a
literary or artistic work.

__1__ "You" means an individual or entity
exercising rights under this License who has not
previously violated the terms of this License with
respect to the Work, or who has received express
permission from the Licensor to exercise rights under
this License despite a previous violation.

__1__ "Publicly Perform" means to perform
public recitations of the Work and to communicate to the
public those public recitations, by any means or process,
including by wire or wireless means or public digital
performances; to make available to the public Works in
such a way that members of the public may access these
Works from a place and at a place individually chosen by
them; to perform the Work to the public by any means or
process and the communication to the public of the
performances of the Work, including by public digital
performance; to broadcast and rebroadcast the Work by any
means including signs, sounds or images.

__1__ "Reproduce" means to make copies of
the Work by any means including without limitation by
sound or visual recordings and the right of fixation and
reproducing fixations of the Work, including storage of a
protected performance or phonogram in digital form or
other electronic medium.

2. Fair Dealing Rights. Nothing in this
License is intended to reduce, limit, or restrict any uses
free from copyright or rights arising from limitations or
exceptions that are provided for in connection with the
copyright protection under copyright law or other
applicable laws.

3. License Grant. Subject to the terms
and conditions of this License, Licensor hereby grants You
a worldwide, royalty-free, non-exclusive, perpetual (for
the duration of the applicable copyright) license to
exercise the rights in the Work as stated below:

__1__ to Reproduce the Work, to incorporate the Work into
one or more Collections, and to Reproduce the Work as
incorporated in the Collections;

__1__ to create and Reproduce Adaptations provided that any
such Adaptation, including any translation in any medium,
takes reasonable steps to clearly label, demarcate or
otherwise identify that changes were made to the original
Work. For example, a translation could be marked "The
original work was translated from English to Spanish," or
a modification could indicate "The original work has been
modified.";

__1__ to Distribute and Publicly Perform the Work including
as incorporated in Collections; and,

__1__ to Distribute and Publicly Perform Adaptations.

The above rights may be exercised in all media and
formats whether now known or hereafter devised. The above
rights include the right to make such modifications as are
technically necessary to exercise the rights in other media
and formats. Subject to Section 8(f), all rights not
expressly granted by Licensor are hereby reserved,
including but not limited to the rights described in
Section 4(e).

4. Restrictions. The license granted in
Section 3 above is expressly made subject to and limited by
the following restrictions:

__1__ You may Distribute or Publicly Perform the Work only
under the terms of this License. You must include a copy
of, or the Uniform Resource Identifier (URI) for, this
License with every copy of the Work You Distribute or
Publicly Perform. You may not offer or impose any terms
on the Work that restrict the terms of this License or
the ability of the recipient of the Work to exercise the
rights granted to that recipient under the terms of the
License. You may not sublicense the Work. You must keep
intact all notices that refer to this License and to the
disclaimer of warranties with every copy of the Work You
Distribute or Publicly Perform. When You Distribute or
Publicly Perform the Work, You may not impose any
effective technological measures on the Work that
restrict the ability of a recipient of the Work from You
to exercise the rights granted to that recipient under
the terms of the License. This Section 4(a) applies to
the Work as incorporated in a Collection, but this does
not require the Collection apart from the Work itself to
be made subject to the terms of this License. If You
create a Collection, upon notice from any Licensor You
must, to the extent practicable, remove from the
Collection any credit as required by Section 4(d), as
requested. If You create an Adaptation, upon notice from
any Licensor You must, to the extent practicable, remove
from the Adaptation any credit as required by Section
4(d), as requested.

__1__ You may Distribute or Publicly Perform an Adaptation
only under: (i) the terms of this License; (ii) a later
version of this License with the same License Elements as
this License; (iii) a Creative Commons jurisdiction
license (either this or a later license version) that
contains the same License Elements as this License (e.g.,
Attribution-NonCommercial-ShareAlike 3.0 US) ("Applicable
License"). You must include a copy of, or the URI, for
Applicable License with every copy of each Adaptation You
Distribute or Publicly Perform. You may not offer or
impose any terms on the Adaptation that restrict the
terms of the Applicable License or the ability of the
recipient of the Adaptation to exercise the rights
granted to that recipient under the terms of the
Applicable License. You must keep intact all notices that
refer to the Applicable License and to the disclaimer of
warranties with every copy of the Work as included in the
Adaptation You Distribute or Publicly Perform. When You
Distribute or Publicly Perform the Adaptation, You may
not impose any effective technological measures on the
Adaptation that restrict the ability of a recipient of
the Adaptation from You to exercise the rights granted to
that recipient under the terms of the Applicable License.
This Section 4(b) applies to the Adaptation as
incorporated in a Collection, but this does not require
the Collection apart from the Adaptation itself to be
made subject to the terms of the Applicable License.

__1__ You may not exercise any of the rights granted to You
in Section 3 above in any manner that is primarily
intended for or directed toward commercial advantage or
private monetary compensation. The exchange of the Work
for other copyrighted works by means of digital
file-sharing or otherwise shall not be considered to be
intended for or directed toward commercial advantage or
private monetary compensation, provided there is no
payment of any monetary compensation in con-nection with
the exchange of copyrighted works.

__1__ If You Distribute, or Publicly Perform the Work or
any Adaptations or Collections, You must, unless a
request has been made pursuant to Section 4(a), keep
intact all copyright notices for the Work and provide,
reasonable to the medium or means You are utilizing: (i)
the name of the Original Author (or pseudonym, if
applicable) if supplied, and/or if the Original Author
and/or Licensor designate another party or parties (e.g.,
a sponsor institute, publishing entity, journal) for
attribution ("Attribution Parties") in Licensor's
copyright notice, terms of service or by other reasonable
means, the name of such party or parties; (ii) the title
of the Work if supplied; (iii) to the extent reasonably
practicable, the URI, if any, that Licensor specifies to
be associated with the Work, unless such URI does not
refer to the copyright notice or licensing information
for the Work; and, (iv) consistent with Section 3(b), in
the case of an Adaptation, a credit identifying the use
of the Work in the Adaptation (e.g., "French translation
of the Work by Original Author," or "Screenplay based on
original Work by Original Author"). The credit required
by this Section 4(d) may be implemented in any reasonable
manner; provided, however, that in the case of a
Adaptation or Collection, at a minimum such credit will
appear, if a credit for all contributing authors of the
Adaptation or Collection appears, then as part of these
credits and in a manner at least as prominent as the
credits for the other contributing authors. For the
avoidance of doubt, You may only use the credit required
by this Section for the purpose of attribution in the
manner set out above and, by exercising Your rights under
this License, You may not implicitly or explicitly assert
or imply any connection with, sponsorship or endorsement
by the Original Author, Licensor and/or Attribution
Parties, as appropriate, of You or Your use of the Work,
without the separate, express prior written permission of
the Original Author, Licensor and/or Attribution
Parties.

__1__

For the avoidance of doubt:

__1__ Non-waivable Compulsory License
Schemes. In those jurisdictions in which the
right to collect royalties through any statutory or
compulsory licensing scheme cannot be waived, the
Licensor reserves the exclusive right to collect such
royalties for any exercise by You of the rights
granted under this License;

__1__ Waivable Compulsory License
Schemes. In those jurisdictions in which the
right to collect royalties through any statutory or
compulsory licensing scheme can be waived, the
Licensor reserves the exclusive right to collect such
royalties for any exercise by You of the rights
granted under this License if Your exercise of such
rights is for a purpose or use which is otherwise
than noncommercial as permitted under Section 4(c)
and otherwise waives the right to collect royalties
through any statutory or compulsory licensing scheme;
and,

__1__ Voluntary License Schemes. The
Licensor reserves the right to collect royalties,
whether individually or, in the event that the
Licensor is a member of a collecting society that
administers voluntary licensing schemes, via that
society, from any exercise by You of the rights
granted under this License that is for a purpose or
use which is otherwise than noncommercial as
permitted under Section 4(c).

__1__ Except as otherwise agreed in writing by the Licensor
or as may be otherwise permitted by applicable law, if
You Reproduce, Distribute or Publicly Perform the Work
either by itself or as part of any Adaptations or
Collections, You must not distort, mutilate, modify or
take other derogatory action in relation to the Work
which would be prejudicial to the Original Author's honor
or reputation. Licensor agrees that in those
jurisdictions (e.g. Japan), in which any exercise of the
right granted in Section 3(b) of this License (the right
to make Adaptations) would be deemed to be a distortion,
mutilation, modification or other derogatory action
prejudicial to the Original Author's honor and
reputation, the Licensor will waive or not assert, as
appropriate, this Section, to the fullest extent
permitted by the applicable national law, to enable You
to reasonably exercise Your right under Section 3(b) of
this License (right to make Adaptations) but not
otherwise.

5. Representations, Warranties and
Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN
WRITING AND TO THE FULLEST EXTENT PERMITTED BY APPLICABLE
LAW, LICENSOR OFFERS THE WORK AS-IS AND MAKES NO
REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING THE
WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING,
WITHOUT LIMITATION, WARRANTIES OF TITLE, MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE
ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY, OR THE
PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE.
SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
WARRANTIES, SO THIS EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability. EXCEPT TO
THE EXTENT REQUIRED BY APPLICABLE LAW, IN NO EVENT WILL
LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY
SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY
DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK,
EVEN IF LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

7. Termination

__1__ This License and the rights granted hereunder will
terminate automatically upon any breach by You of the
terms of this License. Individuals or entities who have
received Adaptations or Collections from You under this
License, however, will not have their licenses terminated
provided such individuals or entities remain in full
compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__ Subject to the above terms and conditions, the
license granted here is perpetual (for the duration of
the applicable copyright in the Work). Notwithstanding
the above, Licensor reserves the right to release the
Work under different license terms or to stop
distributing the Work at any time; provided, however that
any such election will not serve to withdraw this License
(or any other license that has been, or is required to
be, granted under the terms of this License), and this
License will continue in full force and effect unless
terminated as stated above.

8. Miscellaneous

__1__ Each time You Distribute or Publicly Perform the Work
or a Collection, the Licensor offers to the recipient a
license to the Work on the same terms and conditions as
the license granted to You under this License.

__1__ Each time You Distribute or Publicly Perform an
Adaptation, Licensor offers to the recipient a license to
the original Work on the same terms and conditions as the
license granted to You under this License.

__1__ If any provision of this License is invalid or
unenforceable under applicable law, it shall not affect
the validity or enforceability of the remainder of the
terms of this License, and without further action by the
parties to this agreement, such provision shall be
reformed to the minimum extent necessary to make such
provision valid and enforceable.

__1__ No term or provision of this License shall be deemed
waived and no breach consented to unless such waiver or
consent shall be in writing and signed by the party to be
charged with such waiver or consent.

__1__ This License constitutes the entire agreement between
the parties with respect to the Work licensed here. There
are no understandings, agreements or representations with
respect to the Work not specified here. Licensor shall
not be bound by any additional provisions that may appear
in any communication from You. This License may not be
modified without the mutual written agreement of the
Licensor and You.

__1__ The rights granted under, and the subject matter
referenced, in this License were drafted utilizing the
terminology of the Berne Convention for the Protection of
Literary and Artistic Works (as amended on September 28,
1979), the Rome Convention of 1961, the WIPO Copyright
Treaty of 1996, the WIPO Performances and Phonograms
Treaty of 1996 and the Universal Copyright Convention (as
revised on July 24, 1971). These rights and subject
matter take effect in the relevant jurisdiction in which
the License terms are sought to be enforced according to
the corresponding provisions of the implementation of
those treaty provisions in the applicable national law.
If the standard suite of rights granted under applicable
copyright law includes additional rights not granted
under this License, such additional rights are deemed to
be included in the License; this License is not intended
to restrict the license of any rights under applicable
law.

((
((
Creative Commons Notice
))??
((https://creativecommons.org/licenses/by-nc-sa/3.0
((/legalcode))??
))??

Creative Commons is not a party to this License, and
makes no warranty whatsoever in connection with the Work.
Creative Commons will not be liable to You or any party
on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or
consequential damages arising in connection to this
license. Notwithstanding the foregoing two (2) sentences,
if Creative Commons has expressly identified itself as
the Licensor hereunder, it shall have all rights and
obligations of Licensor.

Except for the limited purpose of indicating to the
public that the Work is licensed under the CCPL, Creative
Commons does not authorize the use by either party of the
trademark "Creative Commons" or any related trademark or
logo of Creative Commons without the prior written
consent of Creative Commons. Any permitted use will be in
compliance with Creative Commons' then-current trademark
usage guidelines, as may be published on its website or
otherwise made available upon request from time to time.
For the avoidance of doubt, this trademark restriction
does not form part of this License.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_NC_SA_3_0_US_lre = `//**
CC-BY-NC-SA-3.0-US
https://creativecommons.org/licenses/by-nc-sa/3.0/us
**//

((Creative Commons))??

((
Creative Commons Legal Code
))??
((https://creativecommons.org/licenses/by-nc-sa/3.0/us
((/legalcode))??
))??

((
Attribution-NonCommercial-ShareAlike 3.0 United
States
))??
((https://creativecommons.org/licenses/by-nc-sa/3.0/us
((/legalcode))??
))??

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES
NOT PROVIDE LEGAL SERVICES. DISTRIBUTION OF THIS LICENSE
DOES NOT CREATE AN ATTORNEY-CLIENT RELATIONSHIP. CREATIVE
COMMONS PROVIDES THIS INFORMATION ON AN "AS-IS" BASIS.
CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES
RESULTING FROM ITS USE.
))??

((
((License))??
))??
((https://creativecommons.org/licenses/by-nc-sa/3.0/us
((/legalcode))??
))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS
OF THIS CREATIVE COMMONS PUBLIC LICENSE ("CCPL" OR
"LICENSE"). THE WORK IS PROTECTED BY COPYRIGHT AND/OR OTHER
APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS
AUTHORIZED UNDER THIS LICENSE OR COPYRIGHT LAW IS
PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU
ACCEPT AND AGREE TO BE BOUND BY THE TERMS OF THIS LICENSE.
TO THE EXTENT THIS LICENSE MAY BE CONSIDERED TO BE A
CONTRACT, THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED HERE
IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND
CONDITIONS.

1. Definitions

__1__ "Collective Work" means a work, such
as a periodical issue, anthology or encyclopedia, in
which the Work in its entirety in unmodified form, along
with one or more other contributions, constituting
separate and independent works in themselves, are
assembled into a collective whole. A work that
constitutes a Collective Work will not be considered a
Derivative Work (as defined below) for the purposes of
this License.

__1__ "Derivative Work" means a work based
upon the Work or upon the Work and other pre-existing
works, such as a translation, musical arrangement,
dramatization, fictionalization, motion picture version,
sound recording, art reproduction, abridgment,
condensation, or any other form in which the Work may be
recast, transformed, or adapted, except that a work that
constitutes a Collective Work will not be considered a
Derivative Work for the purpose of this License. For the
avoidance of doubt, where the Work is a musical
composition or sound recording, the synchronization of
the Work in timed-relation with a moving image
("synching") will be considered a Derivative Work for the
purpose of this License.

__1__ "Licensor" means the individual,
individuals, entity or entities that offer(s) the Work
under the terms of this License.

__1__ "Original Author" means the
individual, individuals, entity or entities who created
the Work.

__1__ "Work" means the copyrightable work
of authorship offered under the terms of this
License.

__1__ "You" means an individual or entity
exercising rights under this License who has not
previously violated the terms of this License with
respect to the Work, or who has received express
permission from the Licensor to exercise rights under
this License despite a previous violation.

__1__ "License Elements" means the
following high-level license attributes as selected by
Licensor and indicated in the title of this License:
Attribution, Noncommercial, ShareAlike.

2. Fair Use Rights. Nothing in this
license is intended to reduce, limit, or restrict any
rights arising from fair use, first sale or other
limitations on the exclusive rights of the copyright owner
under copyright law or other applicable laws.

3. License Grant. Subject to the terms
and conditions of this License, Licensor hereby grants You
a worldwide, royalty-free, non-exclusive, perpetual (for
the duration of the applicable copyright) license to
exercise the rights in the Work as stated below:

__1__ to reproduce the Work, to incorporate the Work into
one or more Collective Works, and to reproduce the Work
as incorporated in the Collective Works;

__1__ to create and reproduce Derivative Works provided
that any such Derivative Work, including any translation
in any medium, takes reasonable steps to clearly label,
demarcate or otherwise identify that changes were made to
the original Work. For example, a translation could be
marked "The original work was translated from English to
Spanish," or a modification could indicate "The original
work has been modified.";

__1__ to distribute copies or phonorecords of, display
publicly, perform publicly, and perform publicly by means
of a digital audio transmission the Work including as
incorporated in Collective Works;

__1__ to distribute copies or phonorecords of, display
publicly, perform publicly, and perform publicly by means
of a digital audio transmission Derivative Works;

The above rights may be exercised in all media and
formats whether now known or hereafter devised. The above
rights include the right to make such modifications as are
technically necessary to exercise the rights in other media
and formats. All rights not expressly granted by Licensor
are hereby reserved, including but not limited to the
rights set forth in Sections 4(e) and 4(f).

4. Restrictions. The license granted in
Section 3 above is expressly made subject to and limited by
the following restrictions:

__1__ You may distribute, publicly display, publicly
perform, or publicly digitally perform the Work only
under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this
License with every copy or phonorecord of the Work You
distribute, publicly display, publicly perform, or
publicly digitally perform. You may not offer or impose
any terms on the Work that restrict the terms of this
License or the ability of a recipient of the Work to
exercise the rights granted to that recipient under the
terms of the License. You may not sublicense the Work.
You must keep intact all notices that refer to this
License and to the disclaimer of warranties. When You
distribute, publicly display, publicly perform, or
publicly digitally perform the Work, You may not impose
any technological measures on the Work that restrict the
ability of a recipient of the Work from You to exercise
the rights granted to that recipient under the terms of
the License. This Section 4(a) applies to the Work as
incorporated in a Collective Work, but this does not
require the Collective Work apart from the Work itself to
be made subject to the terms of this License. If You
create a Collective Work, upon notice from any Licensor
You must, to the extent practicable, remove from the
Collective Work any credit as required by Section 4(d),
as requested. If You create a Derivative Work, upon
notice from any Licensor You must, to the extent
practicable, remove from the Derivative Work any credit
as required by Section 4(d), as requested.

__1__ You may distribute, publicly display, publicly
perform, or publicly digitally perform a Derivative Work
only under: (i) the terms of this License; (ii) a later
version of this License with the same License Elements as
this License; or, (iii) either the unported Creative
Commons license or a Creative Commons license for another
jurisdiction (either this or a later license version)
that contains the same License Elements as this License
(e.g. Attribution-NonCommercial-ShareAlike 3.0
(Unported ) )  ("the Applicable License"). You must include
a copy of, or the Uniform Resource Identifier for, the
Applicable License with every copy or phonorecord of each
Derivative Work You distribute, publicly display,
publicly perform, or publicly digitally perform. You may
not offer or impose any terms on the Derivative Works
that restrict the terms of the Applicable License or the
ability of a recipient of the Work to exercise the rights
granted to that recipient under the terms of the
Applicable License. You must keep intact all notices that
refer to the Applicable License and to the disclaimer of
warranties. When You distribute, publicly display,
publicly perform, or publicly digitally perform the
Derivative Work, You may not impose any technological
measures on the Derivative Work that restrict the ability
of a recipient of the Derivative Work from You to
exercise the rights granted to that recipient under the
terms of the Applicable License. This Section 4(b)
applies to the Derivative Work as incorporated in a
Collective Work, but this does not require the Collective
Work apart from the Derivative Work itself to be made
subject to the terms of the Applicable License.

__1__ You may not exercise any of the rights granted to You
in Section 3 above in any manner that is primarily
intended for or directed toward commercial advantage or
private monetary compensation. The exchange of the Work
for other copyrighted works by means of digital
file-sharing or otherwise shall not be considered to be
intended for or directed toward commercial advantage or
private monetary compensation, provided there is no
payment of any monetary compensation in connection with
the exchange of copyrighted works.

__1__ If You distribute, publicly display, publicly
perform, or publicly digitally perform the Work (as
defined in Section 1 above) or any Derivative Works (as
defined in Section 1 above) or Collective Works (as
defined in Section 1 above), You must, unless a request
has been made pursuant to Section 4(a), keep intact all
copyright notices for the Work and provide, reasonable to
the medium or means You are utilizing: (i) the name of
the Original Author (or pseudonym, if applicable) if
supplied, and/or (ii) if the Original Author and/or
Licensor designate another party or parties (e.g. a
sponsor institute, publishing entity, journal) for
attribution ("Attribution Parties") in Licensor's
copyright notice, terms of service or by other reasonable
means, the name of such party or parties; the title of
the Work if supplied; to the extent reasonably
practicable, the Uniform Resource Identifier, if any,
that Licensor specifies to be associated with the Work,
unless such URI does not refer to the copyright notice or
licensing information for the Work; and, consistent with
Section 3(b) in the case of a Derivative Work, a credit
identifying the use of the Work in the Derivative Work
(e.g., "French translation of the Work by Original
Author," or "Screenplay based on original Work by
Original Author"). The credit required by this Section
4(d) may be implemented in any reasonable manner;
provided, however, that in the case of a Derivative Work
or Collective Work, at a minimum such credit will appear,
if a credit for all contributing authors of the
Derivative Work or Collective Work appears, then as part
of these credits and in a manner at least as prominent as
the credits for the other contributing authors. For the
avoidance of doubt, You may only use the credit required
by this Section for the purpose of attribution in the
manner set out above and, by exercising Your rights under
this License, You may not implicitly or explicitly assert
or imply any connection with, sponsorship or endorsement
by the Original Author, Licensor and/or Attribution
Parties, as appropriate, of You or Your use of the Work,
without the separate, express prior written permission of
the Original Author, Licensor and/or Attribution
Parties.

__1__

For the avoidance of doubt, where the Work is a
musical composition:

__1__ Performance Royalties Under Blanket
Licenses. Licensor reserves the exclusive
right to collect whether individually or, in the
event that Licensor is a member of a performance
rights society (e.g. ASCAP, BMI, SESAC), via that
society, royalties for the public performance or
public digital performance (e.g. webcast) of the Work
if that performance is primarily intended for or
directed toward commercial advantage or private
monetary compensation.

__1__ Mechanical Rights and Statutory
Royalties. Licensor reserves the exclusive
right to collect, whether individually or via a music
rights agency or designated agent (e.g. Harry Fox
Agency), royalties for any phonorecord You create
from the Work ("cover version") and distribute,
subject to the compulsory license created by 17 USC
Section 115 of the US Copyright Act (or the
equivalent in other jurisdictions), if Your
distribution of such cover version is primarily
intended for or directed toward commercial advantage
or private monetary compensation.

__1__ Webcasting Rights and Statutory
Royalties.__1__ For the avoidance of doubt, where the
Work is a sound recording, Licensor reserves the
exclusive right to collect, whether individually or via a
performance-rights society (e.g. SoundExchange),
royalties for the public digital performance (e.g.
webcast) of the Work, subject to the compulsory license
created by 17 USC Section 114 of the US Copyright Act (or
the equivalent in other jurisdictions), if Your public
digital performance is primarily intended for or directed
toward commercial advantage or private monetary
compensation.

5. Representations, Warranties and
Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN
WRITING, LICENSOR OFFERS THE WORK AS-IS AND ONLY TO THE
EXTENT OF ANY RIGHTS HELD IN THE LICENSED WORK BY THE
LICENSOR. THE LICENSOR MAKES NO REPRESENTATIONS OR
WARRANTIES OF ANY KIND CONCERNING THE WORK, EXPRESS,
IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT
LIMITATION, WARRANTIES OF TITLE, MARKETABILITY,
MERCHANTIBILITY, FITNESS FOR A PARTICULAR PURPOSE,
NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS,
ACCURACY, OR THE PRESENCE OF ABSENCE OF ERRORS, WHETHER OR
NOT DISCOVERABLE. SOME JURISDICTIONS DO NOT ALLOW THE
EXCLUSION OF IMPLIED WARRANTIES, SO SUCH EXCLUSION MAY NOT
APPLY TO YOU.

6. Limitation on Liability. EXCEPT TO
THE EXTENT REQUIRED BY APPLICABLE LAW, IN NO EVENT WILL
LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY
SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY
DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK,
EVEN IF LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

7. Termination

__1__ This License and the rights granted hereunder will
terminate automatically upon any breach by You of the
terms of this License. Individuals or entities who have
received Derivative Works (as defined in Section 1 above)
or Collective Works (as defined in Section 1 above) from
You under this License, however, will not have their
licenses terminated provided such individuals or entities
remain in full compliance with those licenses. Sections
1, 2, 5, 6, 7, and 8 will survive any termination of this
License.

__1__ Subject to the above terms and conditions, the
license granted here is perpetual (for the duration of
the applicable copyright in the Work). Notwithstanding
the above, Licensor reserves the right to release the
Work under different license terms or to stop
distributing the Work at any time; provided, however that
any such election will not serve to withdraw this License
(or any other license that has been, or is required to
be, granted under the terms of this License), and this
License will continue in full force and effect unless
terminated as stated above.

8. Miscellaneous

__1__ Each time You distribute or publicly digitally
perform the Work (as defined in Section 1 above) or a
Collective Work (as defined in Section 1 above), the
Licensor offers to the recipient a license to the Work on
the same terms and conditions as the license granted to
You under this License.

__1__ Each time You distribute or publicly digitally
perform a Derivative Work, Licensor offers to the
recipient a license to the original Work on the same
terms and conditions as the license granted to You under
this License.

__1__ If any provision of this License is invalid or
unenforceable under applicable law, it shall not affect
the validity or enforceability of the remainder of the
terms of this License, and without further action by the
parties to this agreement, such provision shall be
reformed to the minimum extent necessary to make such
provision valid and enforceable.

__1__ No term or provision of this License shall be deemed
waived and no breach consented to unless such waiver or
consent shall be in writing and signed by the party to be
charged with such waiver or consent.

__1__ This License constitutes the entire agreement between
the parties with respect to the Work licensed here. There
are no understandings, agreements or representations with
respect to the Work not specified here. Licensor shall
not be bound by any additional provisions that may appear
in any communication from You. This License may not be
modified without the mutual written agreement of the
Licensor and You.

((
((
Creative Commons Notice
))??
((https://creativecommons.org/licenses/by-nc-sa/3.0/us
((/legalcode))??
))??

Creative Commons is not a party to this License, and
makes no warranty whatsoever in connection with the Work.
Creative Commons will not be liable to You or any party
on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or
consequential damages arising in connection to this
license. Notwithstanding the foregoing two (2) sentences,
if Creative Commons has expressly identified itself as
the Licensor hereunder, it shall have all rights and
obligations of Licensor.

Except for the limited purpose of indicating to the
public that the Work is licensed under the CCPL, Creative
Commons does not authorize the use by either party of the
trademark "Creative Commons" or any related trademark or
logo of Creative Commons without the prior written
consent of Creative Commons. Any permitted use will be in
compliance with Creative Commons' then-current trademark
usage guidelines, as may be published on its website or
otherwise made available upon request from time to time.
For the avoidance of doubt, this trademark restriction
does not form part of this License.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_NC_SA_4_0_lre = `//**
CC-BY-NC-SA-4.0
https://creativecommons.org/licenses/by-nc-sa/4.0
**//

((Creative Commons))??

((
Creative Commons Legal Code
))??
((https://creativecommons.org/licenses/by-nc-sa/4.0
((/legalcode))??
))??

((
Attribution-NonCommercial-ShareAlike 4.0 International
))??
((https://creativecommons.org/licenses/by-nc-sa/4.0
((/legalcode))??
))??

((
Official translations of this license are available in
other languages.
))??

((
Creative Commons Corporation (“Creative Commons”) is not a law firm and does
not provide legal services or legal advice. Distribution of Creative Commons
public licenses does not create a lawyer-client or other relationship. Creative
Commons makes its licenses and related information available on an “as-is”
basis. Creative Commons gives no warranties regarding its licenses, any material
licensed under their terms and conditions, or any related information. Creative
Commons disclaims all liability for damages resulting from their use to the
fullest extent possible.
))??

((
Using Creative Commons Public Licenses

Creative Commons public licenses provide a standard set of terms and conditions
that creators and other rights holders may use
to share original works of authorship and other material subject to copyright
and certain other rights specified in the public license below. The following
considerations are for informational purposes only, are not exhaustive, and do
not form part of our licenses.

Considerations for licensors: Our public licenses are intended for use by those
authorized to give the public permission to use material in ways otherwise
restricted by copyright and certain other rights. Our licenses are irrevocable.
Licensors should read and understand the terms and conditions of the license
they choose before applying it. Licensors should also secure all rights
necessary before applying our licenses so that the public can reuse the material
as expected. Licensors should clearly mark any material not subject to the
license. This includes other CC-licensed material, or material used under an
exception or limitation to copyright. More considerations for licensors.

Considerations for the public: By using one of our public licenses, a licensor
grants the public permission to use the licensed material under specified terms
and conditions. If the licensor’s permission is not necessary for any
reason–for example, because of any applicable exception or limitation to
copyright–then that use is not regulated by the license. Our licenses grant
only permissions under copyright and certain other rights that a licensor has
authority to grant. Use of the licensed material may still be restricted for
other reasons, including because others have copyright or other rights in the
material. A licensor may make special requests, such as asking that all changes
be marked or described. Although not required by our licenses, you are
encouraged to respect those requests where reasonable.
More considerations for the public.
))??

((
Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International Public
License
))??
((https://creativecommons.org/licenses/by-nc-sa/4.0
((/legalcode))??
))??

By exercising the Licensed Rights (defined below), You accept and agree to be
bound by the terms and conditions of this Creative Commons
Attribution-NonCommercial-ShareAlike 4.0 International Public License ("Public
License"). To the extent this Public License may be interpreted as a contract,
You are granted the Licensed Rights in consideration of Your acceptance of these
terms and conditions, and the Licensor grants You such rights in consideration
of benefits the Licensor receives from making the Licensed Material available
under these terms and conditions.

Section 1 – Definitions.

__1__ Adapted Material means material subject to Copyright and Similar Rights
that is derived from or based upon the Licensed Material and in which the
Licensed Material is translated, altered, arranged, transformed, or otherwise
modified in a manner requiring permission under the Copyright and Similar Rights
held by the Licensor. For purposes of this Public License, where the Licensed
Material is a musical work, performance, or sound recording, Adapted Material is
always produced where the Licensed Material is synched in timed relation with a
moving image.

__1__ Adapter's License means the license You apply to Your Copyright and
Similar Rights in Your contributions to Adapted Material in accordance with the
terms and conditions of this Public License.

__1__ BY-NC-SA Compatible License means a license listed at
creativecommons.org/compatiblelicenses, approved by Creative Commons as
essentially the equivalent of this Public License.

__1__ Copyright and Similar Rights means copyright and/or similar rights closely
related to copyright including, without limitation, performance, broadcast,
sound recording, and Sui Generis Database Rights, without regard to how the
rights are labeled or categorized. For purposes of this Public License, the
rights specified in Section 2(b)(1)-(2) are not Copyright and Similar Rights.

__1__ Effective Technological Measures means those measures that, in the absence
of proper authority, may not
be circumvented under laws fulfilling obligations under Article 11 of the WIPO
Copyright Treaty adopted on December 20, 1996, and/or similar
international agreements.

__1__ Exceptions and Limitations means fair use, fair dealing, and/or any other
exception or limitation to Copyright and Similar Rights that applies to Your use
of the Licensed Material.

__1__ License Elements means the license attributes listed in the name of a
Creative Commons Public License. The License Elements of this Public License are
Attribution, NonCommercial, and ShareAlike.

__1__ Licensed Material means the artistic or literary work, database, or other
material to which the Licensor applied this Public License.

__1__ Licensed Rights means the rights granted to You subject to the terms and
conditions of this Public License, which are limited to all Copyright and
Similar Rights that apply to Your use of the Licensed Material and that the
Licensor has authority to license.

__1__ Licensor means the individual(s) or entity(ies) granting rights under this
Public License.

__1__ NonCommercial means not primarily intended for or directed towards
commercial advantage or monetary compensation. For purposes of this Public
License, the exchange of the Licensed Material for other material subject to
Copyright and Similar Rights by digital file-sharing or similar means is
NonCommercial provided there is no payment of monetary compensation in
connection with the exchange.

__1__ Share means to provide material to the public by any means or process that
requires permission under the Licensed Rights, such as reproduction, public
display, public performance, distribution, dissemination, communication, or
importation, and to make material available to the public including in ways that
members of the public may access the material from a place and at a time
individually chosen by them.

__1__ Sui Generis Database Rights means rights other than copyright resulting
from Directive 96/9/EC of the European Parliament and of the Council of 11 March
1996 on the legal protection of databases, as amended and/or succeeded, as well
as other essentially equivalent rights anywhere in the world.

__1__ You means the individual or entity exercising the Licensed Rights under
this Public License. Your has a corresponding meaning.

Section 2 – Scope.

__1__ License grant.

__1__ Subject to the terms and conditions of this Public License, the Licensor
hereby grants You a worldwide, royalty-free, non-sublicensable, non-exclusive,
irrevocable license to exercise the Licensed Rights in the Licensed Material
to:

__1__ reproduce and Share the Licensed Material, in whole or in part, for
NonCommercial purposes only; and

__1__ produce, reproduce, and Share Adapted Material for NonCommercial purposes
only.

__1__ Exceptions and Limitations. For the avoidance of doubt, where Exceptions
and Limitations apply to Your use, this Public License does not apply, and You
do not need to comply with its terms and conditions.

__1__ Term. The term of this Public License is specified in Section 6(a).

__1__ Media and formats; technical modifications allowed. The Licensor
authorizes You to exercise the Licensed Rights in all media and formats whether
now known or hereafter created, and to make technical modifications necessary to
do so. The Licensor waives and/or agrees not to assert any right or authority to
forbid You from making technical modifications necessary to exercise the
Licensed Rights, including technical modifications necessary to circumvent
Effective Technological Measures. For purposes of this Public License, simply
making modifications authorized by this Section 2(a)(4) never produces Adapted
Material.

__1__ Downstream recipients.

__1__ Offer from the Licensor – Licensed Material. Every recipient of the
Licensed Material automatically receives an  offer from the Licensor to exercise
the Licensed Rights under the terms and conditions of this Public License.

__1__ Additional offer from the Licensor – Adapted Material. Every recipient
of Adapted Material from You automatically receives an  offer from the Licensor
to exercise the Licensed Rights in the Adapted Material under the conditions of
the Adapter’s License You apply.

__1__ No downstream restrictions. You may not offer or impose any additional or
different terms or  conditions on, or apply any Effective Technological Measures
to, the Licensed Material if doing so restricts exercise of the Licensed Rights
by any recipient of the Licensed Material.

__1__ No endorsement. Nothing in this Public License constitutes or may be
construed as permission to assert or imply that You are, or that Your use of the
Licensed Material is, connected with, or sponsored, endorsed, or granted
official status by, the Licensor or others designated to receive attribution as
provided in Section 3(a)(1)(A)(i).

__1__
Other rights.

__1__ Moral rights, such as the right of integrity, are not licensed under this
Public License, nor are publicity, privacy, and/or other similar personality
rights; however, to the extent possible, the Licensor waives and/or agrees not
to assert any such rights held by the Licensor to the limited extent necessary
to allow You to exercise the Licensed Rights, but not otherwise.

__1__ Patent and trademark rights are not licensed under this Public License.

__1__ To the extent possible, the Licensor waives any right to collect royalties
from You for the exercise of the Licensed Rights, whether directly or through a
collecting society under any voluntary or waivable statutory or compulsory
licensing scheme. In all other cases the Licensor expressly reserves any right
to collect such royalties, including when the Licensed Material is used other
than for NonCommercial purposes.

Section 3 – License Conditions.

Your exercise of the Licensed Rights is expressly made subject to the following
conditions.

__1__
Attribution.

__1__
If You Share the Licensed Material (including in modified form), You must:

__1__ retain the following if it is supplied by the Licensor with the Licensed
Material:

__1__ identification of the creator(s) of the Licensed Material and any others
designated to receive attribution, in any reasonable manner requested  by the
Licensor (including by pseudonym if designated);

__1__ a copyright notice;

__1__ a notice that refers to this Public License;

__1__ a notice that refers to the disclaimer of warranties;

__1__ a URI or hyperlink to the Licensed Material to the extent reasonably
practicable;

__1__ indicate if You modified the Licensed Material and retain an indication of
any previous modifications; and

__1__ indicate the Licensed Material is licensed under this Public License,
and include the text of, or the URI or hyperlink to, this Public
License.

__1__ You may satisfy the conditions in Section 3(a)(1) in any reasonable manner
based on the medium, means, and context in which You Share the Licensed
Material. For example, it may be reasonable to satisfy the conditions by
providing a URI or hyperlink to a resource that includes the required
information.

__1__ If requested by the Licensor, You must remove any of the information
required by Section 3(a)(1)(A) to the extent reasonably practicable.

__1__ ShareAlike.

In addition to the conditions in Section 3(a), if You Share Adapted Material You
produce, the following conditions also apply.

__1__ The Adapter’s License You apply must be a Creative Commons license with
the same License Elements, this version or later, or a BY-NC-SA Compatible
License.

__1__ You must include the text of, or the URI or hyperlink to, the Adapter's
License You apply. You may satisfy this condition in any reasonable manner based
on the medium, means, and context in which You Share Adapted Material.

__1__ You may not offer or impose any additional or different terms or
conditions on, or apply any Effective Technological Measures to, Adapted
Material that restrict exercise of the rights granted under the Adapter's
License You apply.

Section 4 – Sui Generis Database Rights.

Where the Licensed Rights include Sui Generis Database Rights that apply to Your
use of the Licensed Material:

__1__ for the avoidance of doubt, Section 2(a)(1) grants You the right to
extract, reuse, reproduce, and Share all or a substantial portion of the
contents of the database for NonCommercial purposes only;

__1__ if You include all or a substantial portion of the database contents in a
database in which You have Sui Generis Database Rights, then the database in
which You have Sui Generis Database Rights (but not its individual contents) is
Adapted Material, including for purposes of Section 3(b); and

__1__ You must comply with the conditions in Section 3(a) if You Share all or a
substantial portion of the contents of the database.

__1__ For the avoidance of doubt, this Section 4 supplements and
does not replace Your obligations under this Public License
where the Licensed Rights include other Copyright and Similar
Rights.

Section 5 – Disclaimer of Warranties and Limitation of Liability.

__1__ Unless otherwise separately undertaken by the Licensor, to the extent
possible, the Licensor offers the Licensed Material as-is and as-available, and
makes no representations or warranties of any kind concerning the Licensed
Material, whether express, implied, statutory, or other. This includes, without
limitation, warranties of title, merchantability, fitness for a particular
purpose, non-infringement, absence of latent or other defects, accuracy, or the
presence or absence of errors, whether or not known or discoverable. Where
disclaimers of warranties are not allowed in full or in part, this disclaimer
may not apply to You.

__1__ To the extent possible, in no event will the Licensor be liable to You on
any legal theory (including, without limitation, negligence) or otherwise for
any direct, special, indirect, incidental, consequential, punitive, exemplary,
or other losses, costs, expenses, or damages arising out of this Public License
or use of the Licensed Material, even if the Licensor has been advised of the
possibility of such losses, costs, expenses, or damages. Where a limitation of
liability is not allowed in full or in part, this limitation may not apply to
You.

__1__ The disclaimer of warranties and limitation of liability provided above
shall be interpreted in a manner that, to the extent possible, most closely
approximates an absolute disclaimer and waiver of all liability.

Section 6 – Term and Termination.

__1__ This Public License applies for the term of the Copyright and Similar
Rights licensed here. However, if You fail to comply with this Public License,
then Your rights under this Public License terminate automatically.

__1__

Where Your right to use the Licensed Material has terminated under Section 6(a),
it reinstates:

__1__ automatically as of the date the violation is cured, provided it is cured
within 30 days of Your discovery of the violation; or

__1__ upon express reinstatement by the Licensor.

__1__ For the avoidance of doubt, this Section 6(b) does not
affect any right the Licensor may have to seek remedies for
Your violations of this Public License.

__1__ For the avoidance of doubt, the Licensor may also offer the Licensed
Material under separate terms or conditions or stop distributing the Licensed
Material at any time; however, doing so will not terminate this Public License.

__1__ Sections 1, 5, 6, 7, and 8 survive termination of this Public License.

Section 7 – Other Terms and Conditions.

__1__ The Licensor shall not be bound by any additional or different terms or
conditions communicated by You unless expressly agreed.

__1__ Any arrangements, understandings, or agreements regarding the Licensed
Material not stated herein are separate from and independent of the terms and
conditions of this Public License.

Section 8 – Interpretation.

__1__ For the avoidance of doubt, this Public License does not, and shall not be
interpreted to, reduce, limit, restrict, or impose conditions on any use of the
Licensed Material that could lawfully be made without permission under this
Public License.

__1__ To the extent possible, if any provision of this Public License is deemed
unenforceable, it shall be automatically reformed to the minimum extent
necessary to make it enforceable. If the provision cannot be reformed, it shall
be severed from this Public License without affecting the enforceability of the
remaining terms and conditions.

__1__ No term or condition of this Public License will be waived and no failure
to comply consented to unless expressly agreed to by the Licensor.

__1__ Nothing in this Public License constitutes or may be interpreted as a
limitation upon, or waiver of, any privileges and immunities that apply to the
Licensor or You, including from the legal processes of any jurisdiction or
authority.

((
Creative Commons is not a party to its public licenses. Notwithstanding,
Creative Commons may elect to apply one of its public licenses to material it
publishes and in those instances will be considered the “Licensor.” The text
of the Creative Commons public licenses is dedicated to the public domain under
the CC0 Public Domain Dedication. Except for the limited purpose of indicating
that material is shared under a Creative Commons public license or as otherwise
permitted by the Creative Commons policies published at
creativecommons.org/policies, Creative Commons does not authorize the use of the
trademark “Creative Commons” or any other trademark or logo of Creative
Commons without its prior written consent including, without limitation, in
connection with any unauthorized modifications to any of its public licenses or
any other arrangements, understandings, or agreements concerning use of licensed
material. For the avoidance of doubt, this paragraph does not form part of the
public licenses.
Creative Commons may be contacted at creativecommons.org.
))??

((
Additional languages available:

العربية,
čeština,
Deutsch,
Ελληνικά,
Español,
euskara,
suomeksi,
français,
hrvatski,
Bahasa Indonesia,
italiano,
日本語,
한국어,
Lietuvių,
latviski,
te reo Māori,
Nederlands,
norsk,
polski,
português,
română,
русский,
Slovenščina,
svenska,
Türkçe,
українська,
中文,
華語.

Please read the FAQ for more information about official translations.
))??
`
const license_CC_BY_ND_1_0_lre = `//**
CC-BY-ND-1.0
https://creativecommons.org/licenses/by-nd/1.0
**//

((Creative Commons))??

Creative Commons

Attribution-NoDerivs 1.0

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE
LEGAL
SERVICES. DISTRIBUTION OF THIS DRAFT LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION
ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING
FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE").  THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW.  ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE.  THE LICENSOR GRANTS YOU THE RIGHTS
CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole.  A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant.  Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

The above rights may be exercised in all media and formats whether now known or
hereafter devised.  The above rights include the right to make such
modifications as are technically necessary to exercise the rights in other media
and formats.  All rights not expressly granted by Licensor are hereby reserved.

4. Restrictions. The license granted in Section 3 above is expressly made
subject to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform.  You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder.  You may not sublicense the Work.  You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement.  The above applies to the Work as incorporated in a
Collective Work, but this does not require the Collective Work apart from the
Work itself to be made subject to the terms of this License. If You create a
Collective Work, upon notice from any Licensor You must, to the extent
practicable, remove from the Collective Work any reference to such Licensor or
the Original Author, as requested.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or any Collective Works, You must keep intact all copyright
notices for the Work and give the Original Author credit reasonable to the
medium or means You are utilizing by conveying the name (or pseudonym if
applicable) of the Original Author if supplied; the title of the Work if
supplied.  Such credit may be implemented in any reasonable manner; provided,
however, that in the case of a Collective Work, at a minimum such credit will
appear where any other comparable authorship credit appears and in a manner at
least as prominent as such other comparable authorship credit.

5. Representations, Warranties and Disclaimer

__1__
By offering the Work for public release under this License, Licensor represents
and warrants that, to the best of Licensor's knowledge after reasonable
inquiry:

__1__
Licensor has secured all rights in the Work necessary to grant the license
rights hereunder and to permit the lawful exercise of the rights granted
hereunder without You having any obligation to pay any royalties, compulsory
license fees, residuals or any other payments;

__1__
The Work does not infringe the copyright, trademark, publicity rights, common
law rights or any other right of any third party or constitute defamation,
invasion of privacy or other tortious injury to any third party.

__1__
EXCEPT AS EXPRESSLY STATED IN THIS LICENSE OR OTHERWISE AGREED IN WRITING OR
REQUIRED BY APPLICABLE LAW, THE WORK IS LICENSED ON AN "AS IS" BASIS, WITHOUT
WARRANTIES OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION,
ANY WARRANTIES REGARDING THE CONTENTS OR ACCURACY OF THE WORK.

6. Limitation on Liability.  EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW,
AND EXCEPT FOR DAMAGES ARISING FROM LIABILITY TO A THIRD PARTY RESULTING FROM
BREACH OF THE WARRANTIES IN SECTION 5, IN NO EVENT WILL LICENSOR BE LIABLE TO
YOU ON ANY LEGAL THEORY FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR
EXEMPLARY DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK, EVEN IF
LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Collective Works from You under this License, however, will not have
their licenses terminated provided such individuals or entities remain in full
compliance with those licenses. Sections 1, 2, 5, 6, 7, and 8 will survive any
termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work).  Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here.  There are no understandings, agreements or
representations with respect to the Work not specified here.  Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work.  Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license.  Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons.  Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_ND_2_0_lre = `//**
CC-BY-ND-2.0
https://creativecommons.org/licenses/by-nd/2.0
**//

((Creative Commons))??

Creative Commons

Attribution-NoDerivs 2.0

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE
LEGAL
SERVICES. DISTRIBUTION OF THIS LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION
ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING
FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE").  THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW.  ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE.  THE LICENSOR GRANTS YOU THE RIGHTS
CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole.  A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License. For
the avoidance of doubt, where the Work is a musical composition or sound
recording, the synchronization of the Work in timed-relation with a moving image
("synching") will be considered a Derivative Work for the purpose of this
License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant.  Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works.

__1__
For the avoidance of doubt, where the work is a musical composition:

__1__ Performance Royalties Under Blanket Licenses. Licensor waives the
exclusive right to collect, whether individually or via a performance rights
society (e.g. ASCAP, BMI, SESAC), royalties for the public performance or public
digital performance (e.g. webcast) of the Work.

__1__ Mechanical Rights and Statutory Royalties. Licensor waives the exclusive
right to collect, whether individually or via a music rights society or
designated agent (e.g. Harry Fox Agency), royalties for any phonorecord You
create from the Work ("cover version") and distribute, subject to the compulsory
license created by 17 USC Section 115 of the US Copyright Act (or the equivalent
in other jurisdictions).

__1__ Webcasting Rights and Statutory Royalties. For the avoidance of doubt,
where the Work is a sound recording, Licensor waives the exclusive right to
collect, whether individually or via a performance-rights society (e.g.
SoundExchange), royalties for the public digital performance (e.g. webcast) of
the Work, subject to the compulsory license created by 17 USC Section 114 of the
US Copyright Act (or the equivalent in other jurisdictions).

The above rights may be exercised in all media and formats whether now known or
hereafter devised.  The above rights include the right to make such
modifications as are technically necessary to exercise the rights in other media
and formats, but otherwise you have no rights to make Derivative Works.  All
rights not expressly granted by Licensor are hereby reserved.

4. Restrictions.The license granted in Section 3 above is expressly made subject
to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform.  You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder.  You may not sublicense the Work.  You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement.  The above applies to the Work as incorporated in a
Collective Work, but this does not require the Collective Work apart from the
Work itself to be made subject to the terms of this License. If You create a
Collective Work, upon notice from any Licensor You must, to the extent
practicable, remove from the Collective Work any reference to such Licensor or
the Original Author, as requested.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or Collective Works, You must keep intact all copyright notices
for the Work and give the Original Author credit reasonable to the medium or
means You are utilizing by conveying the name (or pseudonym if applicable) of
the Original Author if supplied; the title of the Work if supplied; and to the
extent reasonably practicable, the Uniform Resource Identifier, if any, that
Licensor specifies to be associated with the Work, unless such URI does not
refer to the copyright notice or licensing information for the Work. Such credit
may be implemented in any reasonable manner; provided, however, that in the case
of a Collective Work, at a minimum such credit will appear where any other
comparable authorship credit appears and in a manner at least as prominent as
such other comparable authorship credit.

5. Representations, Warranties and Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN WRITING, LICENSOR OFFERS
THE WORK AS-IS AND MAKES NO REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING
THE MATERIALS, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT
LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY, FITNESS FOR A PARTICULAR
PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY,
OR THE PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE. SOME
JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO SUCH
EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability.  EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN
NO EVENT WILL LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY SPECIAL,
INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY DAMAGES ARISING OUT OF THIS
LICENSE OR THE USE OF THE WORK, EVEN IF LICENSOR HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Collective Works from You under this License, however, will not have
their licenses terminated provided such individuals or entities remain in full
compliance with those licenses. Sections 1, 2, 5, 6, 7, and 8 will survive any
termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work).  Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work, the Licensor
offers to the recipient a license to the Work on the same terms and conditions
as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here.  There are no understandings, agreements or
representations with respect to the Work not specified here.  Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work.  Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license.  Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons.  Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_ND_2_5_lre = `//**
CC-BY-ND-2.5
https://creativecommons.org/licenses/by-nd/2.5
**//

((Creative Commons))??

Creative Commons

Attribution-NoDerivs 2.5

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE
LEGAL
SERVICES. DISTRIBUTION OF THIS LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION
ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING
FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE").  THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW.  ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE.  THE LICENSOR GRANTS YOU THE RIGHTS
CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole.  A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License. For
the avoidance of doubt, where the Work is a musical composition or sound
recording, the synchronization of the Work in timed-relation with a moving image
("synching") will be considered a Derivative Work for the purpose of this
License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant.  Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works.

__1__
For the avoidance of doubt, where the work is a musical composition:

__1__ Performance Royalties Under Blanket Licenses. Licensor waives the
exclusive right to collect, whether individually or via a performance rights
society (e.g. ASCAP, BMI, SESAC), royalties for the public performance or public
digital performance (e.g. webcast) of the Work.

__1__ Mechanical Rights and Statutory Royalties. Licensor waives the exclusive
right to collect, whether individually or via a music rights society or
designated agent (e.g. Harry Fox Agency), royalties for any phonorecord You
create from the Work ("cover version") and distribute, subject to the compulsory
license created by 17 USC Section 115 of the US Copyright Act (or the equivalent
in other jurisdictions).

__1__ Webcasting Rights and Statutory Royalties. For the avoidance of doubt,
where the Work is a sound recording, Licensor waives the exclusive right to
collect, whether individually or via a performance-rights society (e.g.
SoundExchange), royalties for the public digital performance (e.g. webcast) of
the Work, subject to the compulsory license created by 17 USC Section 114 of the
US Copyright Act (or the equivalent in other jurisdictions).

The above rights may be exercised in all media and formats whether now known or
hereafter devised.  The above rights include the right to make such
modifications as are technically necessary to exercise the rights in other media
and formats, but otherwise you have no rights to make Derivative Works.  All
rights not expressly granted by Licensor are hereby reserved.

4. Restrictions.The license granted in Section 3 above is expressly made subject
to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform.  You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder.  You may not sublicense the Work.  You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement.  The above applies to the Work as incorporated in a
Collective Work, but this does not require the Collective Work apart from the
Work itself to be made subject to the terms of this License. If You create a
Collective Work, upon notice from any Licensor You must, to the extent
practicable, remove from the Collective Work any credit as required by clause
4(b), as requested.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or Collective Works, You must keep intact all copyright notices
for the Work and provide, reasonable to the medium or means You are utilizing:
(i) the name of the Original Author (or pseudonym, if applicable) if supplied,
and/or (ii) if the Original Author and/or Licensor designate another party or
parties (e.g. a sponsor institute, publishing entity, journal) for attribution
in Licensor's copyright notice, terms of service or by other reasonable means,
the name of such party or parties; the title of the Work if supplied; and to the
extent reasonably practicable, the Uniform Resource Identifier, if any, that
Licensor specifies to be associated with the Work, unless such URI does not
refer to the copyright notice or licensing information for the Work. Such credit
may be implemented in any reasonable manner; provided, however, that in the case
of a Collective Work, at a minimum such credit will appear where any other
comparable authorship credit appears and in a manner at least as prominent as
such other comparable authorship credit.

5. Representations, Warranties and Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN WRITING, LICENSOR OFFERS
THE WORK AS-IS AND MAKES NO REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING
THE MATERIALS, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT
LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY, FITNESS FOR A PARTICULAR
PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY,
OR THE PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE. SOME
JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO SUCH
EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability.  EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN
NO EVENT WILL LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY SPECIAL,
INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY DAMAGES ARISING OUT OF THIS
LICENSE OR THE USE OF THE WORK, EVEN IF LICENSOR HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Collective Works from You under this License, however, will not have
their licenses terminated provided such individuals or entities remain in full
compliance with those licenses. Sections 1, 2, 5, 6, 7, and 8 will survive any
termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work).  Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work, the Licensor
offers to the recipient a license to the Work on the same terms and conditions
as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here.  There are no understandings, agreements or
representations with respect to the Work not specified here.  Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work.  Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license.  Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons.  Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_ND_3_0_lre = `//**
CC-BY-ND-3.0
https://creativecommons.org/licenses/by-nd/3.0
**//

((Creative Commons))??

((
Creative Commons Legal Code
))??
((https://creativecommons.org/licenses/by-nd/3.0
((/legalcode))??
))??

((
Attribution-NoDerivs 3.0 Unported
))??
((https://creativecommons.org/licenses/by-nd/3.0
((/legalcode))??
))??

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES
NOT PROVIDE LEGAL SERVICES. DISTRIBUTION OF THIS LICENSE
DOES NOT CREATE AN ATTORNEY-CLIENT RELATIONSHIP. CREATIVE
COMMONS PROVIDES THIS INFORMATION ON AN "AS-IS" BASIS.
CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES
RESULTING FROM ITS USE.
))??

((
((License))??
))??
((https://creativecommons.org/licenses/by-nd/3.0
((/legalcode))??
))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS
OF THIS CREATIVE COMMONS PUBLIC LICENSE ("CCPL" OR
"LICENSE"). THE WORK IS PROTECTED BY COPYRIGHT AND/OR OTHER
APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS
AUTHORIZED UNDER THIS LICENSE OR COPYRIGHT LAW IS
PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU
ACCEPT AND AGREE TO BE BOUND BY THE TERMS OF THIS LICENSE.
TO THE EXTENT THIS LICENSE MAY BE CONSIDERED TO BE A
CONTRACT, THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED HERE
IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND
CONDITIONS.

1. Definitions

__1__ "Adaptation" means a work based upon
the Work, or upon the Work and other pre-existing works,
such as a translation, adaptation, derivative work,
arrangement of music or other alterations of a literary
or artistic work, or phonogram or performance and
includes cinematographic adaptations or any other form in
which the Work may be recast, transformed, or adapted
including in any form recognizably derived from the
original, except that a work that constitutes a
Collection will not be considered an Adaptation for the
purpose of this License. For the avoidance of doubt,
where the Work is a musical work, performance or
phonogram, the synchronization of the Work in
timed-relation with a moving image ("synching") will be
considered an Adaptation for the purpose of this
License.

__1__ "Collection" means a collection of
literary or artistic works, such as encyclopedias and
anthologies, or performances, phonograms or broadcasts,
or other works or subject matter other than works listed
in Section 1(f) below, which, by reason of the selection
and arrangement of their contents, constitute
intellectual creations, in which the Work is included in
its entirety in unmodified form along with one or more
other contributions, each constituting separate and
independent works in themselves, which together are
assembled into a collective whole. A work that
constitutes a Collection will not be considered an
Adaptation (as defined above) for the purposes of this
License.

__1__ "Distribute" means to make available
to the public the original and copies of the Work through
sale or other transfer of ownership.

__1__ "Licensor" means the individual,
individuals, entity or entities that offer(s) the Work
under the terms of this License.

__1__ "Original Author" means, in the case
of a literary or artistic work, the individual,
individuals, entity or entities who created the Work or
if no individual or entity can be identified, the
publisher; and in addition (i) in the case of a
performance the actors, singers, musicians, dancers, and
other persons who act, sing, deliver, declaim, play in,
interpret or otherwise perform literary or artistic works
or expressions of folklore; (ii) in the case of a
phonogram the producer being the person or legal entity
who first fixes the sounds of a performance or other
sounds; and, (iii) in the case of broadcasts, the
organization that transmits the broadcast.

__1__ "Work" means the literary and/or
artistic work offered under the terms of this License
including without limitation any production in the
literary, scientific and artistic domain, whatever may be
the mode or form of its expression including digital
form, such as a book, pamphlet and other writing; a
lecture, address, sermon or other work of the same
nature; a dramatic or dramatico-musical work; a
choreographic work or entertainment in dumb show; a
musical composition with or without words; a
cinematographic work to which are assimilated works
expressed by a process analogous to cinematography; a
work of drawing, painting, architecture, sculpture,
engraving or lithography; a photographic work to which
are assimilated works expressed by a process analogous to
photography; a work of applied art; an illustration, map,
plan, sketch or three-dimensional work relative to
geography, topography, architecture or science; a
performance; a broadcast; a phonogram; a compilation of
data to the extent it is protected as a copyrightable
work; or a work performed by a variety or circus
performer to the extent it is not otherwise considered a
literary or artistic work.

__1__ "You" means an individual or entity
exercising rights under this License who has not
previously violated the terms of this License with
respect to the Work, or who has received express
permission from the Licensor to exercise rights under
this License despite a previous violation.

__1__ "Publicly Perform" means to perform
public recitations of the Work and to communicate to the
public those public recitations, by any means or process,
including by wire or wireless means or public digital
performances; to make available to the public Works in
such a way that members of the public may access these
Works from a place and at a place individually chosen by
them; to perform the Work to the public by any means or
process and the communication to the public of the
performances of the Work, including by public digital
performance; to broadcast and rebroadcast the Work by any
means including signs, sounds or images.

__1__ "Reproduce" means to make copies of
the Work by any means including without limitation by
sound or visual recordings and the right of fixation and
reproducing fixations of the Work, including storage of a
protected performance or phonogram in digital form or
other electronic medium.

2. Fair Dealing Rights. Nothing in this
License is intended to reduce, limit, or restrict any uses
free from copyright or rights arising from limitations or
exceptions that are provided for in connection with the
copyright protection under copyright law or other
applicable laws.

3. License Grant. Subject to the terms
and conditions of this License, Licensor hereby grants You
a worldwide, royalty-free, non-exclusive, perpetual (for
the duration of the applicable copyright) license to
exercise the rights in the Work as stated below:

__1__ to Reproduce the Work, to incorporate the Work into
one or more Collections, and to Reproduce the Work as
incorporated in the Collections; and,

__1__ to Distribute and Publicly Perform the Work including
as incorporated in Collections.

__1__

For the avoidance of doubt:

__1__ Non-waivable Compulsory License
Schemes. In those jurisdictions in which the
right to collect royalties through any statutory or
compulsory licensing scheme cannot be waived, the
Licensor reserves the exclusive right to collect such
royalties for any exercise by You of the rights
granted under this License;

__1__ Waivable Compulsory License
Schemes. In those jurisdictions in which the
right to collect royalties through any statutory or
compulsory licensing scheme can be waived, the
Licensor waives the exclusive right to collect such
royalties for any exercise by You of the rights
granted under this License; and,

__1__ Voluntary License Schemes. The
Licensor waives the right to collect royalties,
whether individually or, in the event that the
Licensor is a member of a collecting society that
administers voluntary licensing schemes, via that
society, from any exercise by You of the rights
granted under this License.

The above rights may be exercised in all media and
formats whether now known or hereafter devised. The above
rights include the right to make such modifications as are
technically necessary to exercise the rights in other media
and formats, but otherwise you have no rights to make
Adaptations. Subject to Section 8(f), all rights not
expressly granted by Licensor are hereby reserved.

4. Restrictions. The license granted in
Section 3 above is expressly made subject to and limited by
the following restrictions:

__1__ You may Distribute or Publicly Perform the Work only
under the terms of this License. You must include a copy
of, or the Uniform Resource Identifier (URI) for, this
License with every copy of the Work You Distribute or
Publicly Perform. You may not offer or impose any terms
on the Work that restrict the terms of this License or
the ability of the recipient of the Work to exercise the
rights granted to that recipient under the terms of the
License. You may not sublicense the Work. You must keep
intact all notices that refer to this License and to the
disclaimer of warranties with every copy of the Work You
Distribute or Publicly Perform. When You Distribute or
Publicly Perform the Work, You may not impose any
effective technological measures on the Work that
restrict the ability of a recipient of the Work from You
to exercise the rights granted to that recipient under
the terms of the License. This Section 4(a) applies to
the Work as incorporated in a Collection, but this does
not require the Collection apart from the Work itself to
be made subject to the terms of this License. If You
create a Collection, upon notice from any Licensor You
must, to the extent practicable, remove from the
Collection any credit as required by Section 4(b), as
requested.

__1__ If You Distribute, or Publicly Perform the Work or
Collections, You must, unless a request has been made
pursuant to Section 4(a), keep intact all copyright
notices for the Work and provide, reasonable to the
medium or means You are utilizing: (i) the name of the
Original Author (or pseudonym, if applicable) if
supplied, and/or if the Original Author and/or Licensor
designate another party or parties (e.g., a sponsor
institute, publishing entity, journal) for attribution
("Attribution Parties") in Licensor's copyright notice,
terms of service or by other reasonable means, the name
of such party or parties; (ii) the title of the Work if
supplied; (iii) to the extent reasonably practicable, the
URI, if any, that Licensor specifies to be associated
with the Work, unless such URI does not refer to the
copyright notice or licensing information for the Work.
The credit required by this Section 4(b) may be
implemented in any reasonable manner; provided, however,
that in the case of a Collection, at a minimum such
credit will appear, if a credit for all contributing
authors of the Collection appears, then as part of these
credits and in a manner at least as prominent as the
credits for the other contributing authors. For the
avoidance of doubt, You may only use the credit required
by this Section for the purpose of attribution in the
manner set out above and, by exercising Your rights under
this License, You may not implicitly or explicitly assert
or imply any connection with, sponsorship or endorsement
by the Original Author, Licensor and/or Attribution
Parties, as appropriate, of You or Your use of the Work,
without the separate, express prior written permission of
the Original Author, Licensor and/or Attribution
Parties.

__1__ Except as otherwise agreed in writing by the Licensor
or as may be otherwise permitted by applicable law, if
You Reproduce, Distribute or Publicly Perform the Work
either by itself or as part of any Collections, You must
not distort, mutilate, modify or take other derogatory
action in relation to the Work which would be prejudicial
to the Original Author's honor or reputation.

5. Representations, Warranties and
Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN
WRITING, LICENSOR OFFERS THE WORK AS-IS AND MAKES NO
REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING THE
WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING,
WITHOUT LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY,
FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE
ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY, OR THE
PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE.
SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
WARRANTIES, SO SUCH EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability. EXCEPT TO
THE EXTENT REQUIRED BY APPLICABLE LAW, IN NO EVENT WILL
LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY
SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY
DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK,
EVEN IF LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

7. Termination

__1__ This License and the rights granted hereunder will
terminate automatically upon any breach by You of the
terms of this License. Individuals or entities who have
received Collections from You under this License,
however, will not have their licenses terminated provided
such individuals or entities remain in full compliance
with those licenses. Sections 1, 2, 5, 6, 7, and 8 will
survive any termination of this License.

__1__ Subject to the above terms and conditions, the
license granted here is perpetual (for the duration of
the applicable copyright in the Work). Notwithstanding
the above, Licensor reserves the right to release the
Work under different license terms or to stop
distributing the Work at any time; provided, however that
any such election will not serve to withdraw this License
(or any other license that has been, or is required to
be, granted under the terms of this License), and this
License will continue in full force and effect unless
terminated as stated above.

8. Miscellaneous

__1__ Each time You Distribute or Publicly Perform the Work
or a Collection, the Licensor offers to the recipient a
license to the Work on the same terms and conditions as
the license granted to You under this License.

__1__ If any provision of this License is invalid or
unenforceable under applicable law, it shall not affect
the validity or enforceability of the remainder of the
terms of this License, and without further action by the
parties to this agreement, such provision shall be
reformed to the minimum extent necessary to make such
provision valid and enforceable.

__1__ No term or provision of this License shall be deemed
waived and no breach consented to unless such waiver or
consent shall be in writing and signed by the party to be
charged with such waiver or consent.

__1__ This License constitutes the entire agreement between
the parties with respect to the Work licensed here. There
are no understandings, agreements or representations with
respect to the Work not specified here. Licensor shall
not be bound by any additional provisions that may appear
in any communication from You. This License may not be
modified without the mutual written agreement of the
Licensor and You.

__1__ The rights granted under, and the subject matter
referenced, in this License were drafted utilizing the
terminology of the Berne Convention for the Protection of
Literary and Artistic Works (as amended on September 28,
1979), the Rome Convention of 1961, the WIPO Copyright
Treaty of 1996, the WIPO Performances and Phonograms
Treaty of 1996 and the Universal Copyright Convention (as
revised on July 24, 1971). These rights and subject
matter take effect in the relevant jurisdiction in which
the License terms are sought to be enforced according to
the corresponding provisions of the implementation of
those treaty provisions in the applicable national law.
If the standard suite of rights granted under applicable
copyright law includes additional rights not granted
under this License, such additional rights are deemed to
be included in the License; this License is not intended
to restrict the license of any rights under applicable
law.

((
((
Creative Commons Notice
))??
((https://creativecommons.org/licenses/by-nd/3.0
((/legalcode))??
))??

Creative Commons is not a party to this License, and
makes no warranty whatsoever in connection with the Work.
Creative Commons will not be liable to You or any party
on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or
consequential damages arising in connection to this
license. Notwithstanding the foregoing two (2) sentences,
if Creative Commons has expressly identified itself as
the Licensor hereunder, it shall have all rights and
obligations of Licensor.

Except for the limited purpose of indicating to the
public that the Work is licensed under the CCPL, Creative
Commons does not authorize the use by either party of the
trademark "Creative Commons" or any related trademark or
logo of Creative Commons without the prior written
consent of Creative Commons. Any permitted use will be in
compliance with Creative Commons' then-current trademark
usage guidelines, as may be published on its website or
otherwise made available upon request from time to time.
For the avoidance of doubt, this trademark restriction
does not form part of this License.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_ND_4_0_lre = `//**
CC-BY-ND-4.0
https://creativecommons.org/licenses/by-nd/4.0
**//

((Creative Commons))??

((
Creative Commons Legal Code
))??
((https://creativecommons.org/licenses/by-nd/4.0
((/legalcode))??
))??

((
Attribution-NoDerivatives 4.0 International
))??
((https://creativecommons.org/licenses/by-nd/4.0
((/legalcode))??
))??

((
Official translations of this license are available in
other languages.
))??

((
Creative Commons Corporation (“Creative Commons”) is not a law firm and does
not provide legal services or legal advice. Distribution of Creative Commons
public licenses does not create a lawyer-client or other relationship. Creative
Commons makes its licenses and related information available on an “as-is”
basis. Creative Commons gives no warranties regarding its licenses, any material
licensed under their terms and conditions, or any related information. Creative
Commons disclaims all liability for damages resulting from their use to the
fullest extent possible.
))??

((
Using Creative Commons Public Licenses

Creative Commons public licenses provide a standard set of terms and conditions
that creators and other rights holders may use to share original works of
authorship and other material subject to copyright and certain other rights
specified in the public license below. The following considerations are for
informational purposes only, are not exhaustive, and do not form part of our
licenses.

Considerations for licensors: Our public licenses are intended for use by those
authorized to give the public permission to use material in ways otherwise
restricted by copyright and certain other rights. Our licenses are irrevocable.
Licensors should read and understand the terms and conditions of the license
they choose before applying it. Licensors should also secure all rights
necessary before applying our licenses so that the public can reuse the material
as expected. Licensors should clearly mark any material not subject to the
license. This includes other CC-licensed material, or material used under an
exception or limitation to copyright. More considerations for licensors.

Considerations for the public: By using one of our public licenses, a licensor
grants the public permission to use the licensed material under specified terms
and conditions. If the licensor’s permission is not necessary for any
reason–for example, because of any applicable exception or limitation to
copyright–then that use is not regulated by the license. Our licenses grant
only permissions under copyright and certain other rights that a licensor has
authority to grant. Use of the licensed material may still be restricted for
other reasons, including because others have copyright or other rights in the
material. A licensor may make special requests, such as asking that all changes
be marked or described. Although not required by our licenses, you are
encouraged to respect those requests where reasonable.
More considerations for the public.
))??

((
Creative Commons Attribution-NoDerivatives 4.0 International Public License
))??
((https://creativecommons.org/licenses/by-nd/4.0
((/legalcode))??
))??

By exercising the Licensed Rights (defined below), You accept and agree to be
bound by the terms and conditions of this Creative Commons
Attribution-NoDerivatives 4.0 International Public License ("Public License").
To the extent this Public License may be interpreted as a contract, You are
granted the Licensed Rights in consideration of Your acceptance of these terms
and conditions, and the Licensor grants You such rights in consideration of
benefits the Licensor receives from making the Licensed Material available under
these terms and conditions.

Section 1 – Definitions.

__1__ Adapted Material means material subject to Copyright and Similar Rights
that is derived from or based upon the Licensed Material and in which the
Licensed Material is translated, altered, arranged, transformed, or otherwise
modified in a manner requiring permission under the Copyright and Similar Rights
held by the Licensor. For purposes of this Public License, where the Licensed
Material is a musical work, performance, or sound recording, Adapted Material is
always produced where the Licensed Material is synched in timed relation with a
moving image.

__1__ Copyright and Similar Rights means copyright and/or similar rights closely
related to copyright including, without limitation, performance, broadcast,
sound recording, and Sui Generis Database Rights, without regard to how the
rights are labeled or categorized. For purposes of this Public License, the
rights specified in Section 2(b)(1)-(2) are not Copyright and Similar Rights.

__1__ Effective Technological Measures means those measures that, in the absence
of proper authority, may not
be circumvented under laws fulfilling obligations under Article 11 of the WIPO
Copyright Treaty adopted on December 20, 1996, and/or similar
international agreements.

__1__ Exceptions and Limitations means fair use, fair dealing, and/or any other
exception or limitation to Copyright and Similar Rights that applies to Your use
of the Licensed Material.

__1__ Licensed Material means the artistic or literary work, database, or other
material to which the Licensor applied this Public License.

__1__ Licensed Rights means the rights granted to You subject to the terms and
conditions of this Public License, which are limited to all Copyright and
Similar Rights that apply to Your use of the Licensed Material and that the
Licensor has authority to license.

__1__ Licensor means the individual(s) or entity(ies) granting rights under this
Public License.

__1__ Share means to provide material to the public by any means or process that
requires permission under the Licensed Rights, such as reproduction, public
display, public performance, distribution, dissemination, communication, or
importation, and to make material available to the public including in ways that
members of the public may access the material from a place and at a time
individually chosen by them.

__1__ Sui Generis Database Rights means rights other than copyright resulting
from Directive 96/9/EC of the European Parliament and of the Council of 11 March
1996 on the legal protection of databases, as amended and/or succeeded, as well
as other essentially equivalent rights anywhere in the world.

__1__ You means the individual or entity exercising the Licensed Rights under
this Public License. Your has a corresponding meaning.

Section 2 – Scope.

__1__ License grant.

__1__ Subject to the terms and conditions of this Public License, the Licensor
hereby grants You a worldwide, royalty-free, non-sublicensable, non-exclusive,
irrevocable license to exercise the Licensed Rights in the Licensed Material
to:

__1__ reproduce and Share the Licensed Material, in whole or in part; and

__1__ produce and reproduce, but not Share, Adapted Material.

__1__ Exceptions and Limitations. For the avoidance of doubt, where Exceptions
and Limitations apply to Your use, this Public License does not apply, and You
do not need to comply with its terms and conditions.

__1__ Term. The term of this Public License is specified in Section 6(a).

__1__ Media and formats; technical modifications allowed. The Licensor
authorizes You to exercise the Licensed Rights in all media and formats whether
now known or hereafter created, and to make technical modifications necessary to
do so. The Licensor waives and/or agrees not to assert any right or authority to
forbid You from making technical modifications necessary to exercise the
Licensed Rights, including technical modifications necessary to circumvent
Effective Technological Measures. For purposes of this Public License, simply
making modifications authorized by this Section 2(a)(4) never produces Adapted
Material.

__1__ Downstream recipients.

__1__ Offer from the Licensor – Licensed Material. Every recipient of the
Licensed Material automatically receives an  offer from the Licensor to exercise
the Licensed Rights under the terms and conditions of this Public License.

__1__ No downstream restrictions. You may not offer or impose any additional or
different terms or  conditions on, or apply any Effective Technological Measures
to, the Licensed Material if doing so restricts exercise of the Licensed Rights
by any recipient of the Licensed Material.

__1__ No endorsement. Nothing in this Public License constitutes or may be
construed as permission to assert or imply that You are, or that Your use of the
Licensed Material is, connected with, or sponsored, endorsed, or granted
official status by, the Licensor or others designated to receive attribution as
provided in Section 3(a)(1)(A)(i).

__1__
Other rights.

__1__ Moral rights, such as the right of integrity, are not licensed under this
Public License, nor are publicity, privacy, and/or other similar personality
rights; however, to the extent possible, the Licensor waives and/or agrees not
to assert any such rights held by the Licensor to the limited extent necessary
to allow You to exercise the Licensed Rights, but not otherwise.

__1__ Patent and trademark rights are not licensed under this Public License.

__1__ To the extent possible, the Licensor waives any right to collect royalties
from You for the exercise of the Licensed Rights, whether directly or through a
collecting society under any voluntary or waivable statutory or compulsory
licensing scheme. In all other cases the Licensor expressly reserves any right
to collect such royalties.

Section 3 – License Conditions.

Your exercise of the Licensed Rights is expressly made subject to the following
conditions.

__1__
Attribution.

__1__
If You Share the Licensed Material, You must:

__1__ retain the following if it is supplied by the Licensor with the Licensed
Material:

__1__ identification of the creator(s) of the Licensed Material and any others
designated to receive attribution, in any reasonable manner requested  by the
Licensor (including by pseudonym if designated);

__1__ a copyright notice;

__1__ a notice that refers to this Public License;

__1__ a notice that refers to the disclaimer of warranties;

__1__ a URI or hyperlink to the Licensed Material to the extent reasonably
practicable;

__1__ indicate if You modified the Licensed Material and retain an indication of
any previous modifications; and

__1__ indicate the Licensed Material is licensed under this Public License,
and include the text of, or the URI or hyperlink to, this Public
License.

__1__ For the avoidance of doubt, You do not have permission under this Public
License to Share Adapted Material.

__1__ You may satisfy the conditions in Section 3(a)(1) in any reasonable manner
based on the medium, means, and context in which You Share the Licensed
Material. For example, it may be reasonable to satisfy the conditions by
providing a URI or hyperlink to a resource that includes the required
information.

__1__ If requested by the Licensor, You must remove any of the information
required by Section 3(a)(1)(A) to the extent reasonably practicable.

Section 4 – Sui Generis Database Rights.

Where the Licensed Rights include Sui Generis Database Rights that apply to Your
use of the Licensed Material:

__1__ for the avoidance of doubt, Section 2(a)(1) grants You the right to
extract, reuse, reproduce, and Share all or a substantial portion of the
contents of the database, provided You do not Share Adapted Material;

__1__ if You include all or a substantial portion of the database contents in a
database in which You have Sui Generis Database Rights, then the database in
which You have Sui Generis Database Rights (but not its individual contents) is
Adapted Material; and

__1__ You must comply with the conditions in Section 3(a) if You Share all or a
substantial portion of the contents of the database.

__1__ For the avoidance of doubt, this Section 4 supplements and
does not replace Your obligations under this Public License
where the Licensed Rights include other Copyright and Similar
Rights.

Section 5 – Disclaimer of Warranties and Limitation of Liability.

__1__ Unless otherwise separately undertaken by the Licensor, to the extent
possible, the Licensor offers the Licensed Material as-is and as-available, and
makes no representations or warranties of any kind concerning the Licensed
Material, whether express, implied, statutory, or other. This includes, without
limitation, warranties of title, merchantability, fitness for a particular
purpose, non-infringement, absence of latent or other defects, accuracy, or the
presence or absence of errors, whether or not known or discoverable. Where
disclaimers of warranties are not allowed in full or in part, this disclaimer
may not apply to You.

__1__ To the extent possible, in no event will the Licensor be liable to You on
any legal theory (including, without limitation, negligence) or otherwise for
any direct, special, indirect, incidental, consequential, punitive, exemplary,
or other losses, costs, expenses, or damages arising out of this Public License
or use of the Licensed Material, even if the Licensor has been advised of the
possibility of such losses, costs, expenses, or damages. Where a limitation of
liability is not allowed in full or in part, this limitation may not apply to
You.

__1__ The disclaimer of warranties and limitation of liability provided above
shall be interpreted in a manner that, to the extent possible, most closely
approximates an absolute disclaimer and waiver of all liability.

Section 6 – Term and Termination.

__1__ This Public License applies for the term of the Copyright and Similar
Rights licensed here. However, if You fail to comply with this Public License,
then Your rights under this Public License terminate automatically.

__1__

Where Your right to use the Licensed Material has terminated under Section 6(a),
it reinstates:

__1__ automatically as of the date the violation is cured, provided it is cured
within 30 days of Your discovery of the violation; or

__1__ upon express reinstatement by the Licensor.

__1__ For the avoidance of doubt, this Section 6(b) does not
affect any right the Licensor may have to seek remedies for
Your violations of this Public License.

__1__ For the avoidance of doubt, the Licensor may also offer the Licensed
Material under separate terms or conditions or stop distributing the Licensed
Material at any time; however, doing so will not terminate this Public License.

__1__ Sections 1, 5, 6, 7, and 8 survive termination of this Public License.

Section 7 – Other Terms and Conditions.

__1__ The Licensor shall not be bound by any additional or different terms or
conditions communicated by You unless expressly agreed.

__1__ Any arrangements, understandings, or agreements regarding the Licensed
Material not stated herein are separate from and independent of the terms and
conditions of this Public License.

Section 8 – Interpretation.

__1__ For the avoidance of doubt, this Public License does not, and shall not be
interpreted to, reduce, limit, restrict, or impose conditions on any use of the
Licensed Material that could lawfully be made without permission under this
Public License.

__1__ To the extent possible, if any provision of this Public License is deemed
unenforceable, it shall be automatically reformed to the minimum extent
necessary to make it enforceable. If the provision cannot be reformed, it shall
be severed from this Public License without affecting the enforceability of the
remaining terms and conditions.

__1__ No term or condition of this Public License will be waived and no failure
to comply consented to unless expressly agreed to by the Licensor.

__1__ Nothing in this Public License constitutes or may be interpreted as a
limitation upon, or waiver of, any privileges and immunities that apply to the
Licensor or You, including from the legal processes of any jurisdiction or
authority.

((
Creative Commons is not a party to its public licenses. Notwithstanding,
Creative Commons may elect to apply one of its public licenses to material it
publishes and in those instances will be considered the “Licensor.” The text
of the Creative Commons public licenses is dedicated to the public domain under
the CC0 Public Domain Dedication. Except for the limited purpose of indicating
that material is shared under a Creative Commons public license or as otherwise
permitted by the Creative Commons policies published at
creativecommons.org/policies, Creative Commons does not authorize the use of the
trademark “Creative Commons” or any other trademark or logo of Creative
Commons without its prior written consent including, without limitation, in
connection with any unauthorized modifications to any of its public licenses or
any other arrangements, understandings, or agreements concerning use of licensed
material. For the avoidance of doubt, this paragraph does not form part of the
public licenses.
Creative Commons may be contacted at creativecommons.org.
))??

((
Additional languages available:

العربية,
čeština,
Deutsch,
Ελληνικά,
Español,
euskara,
suomeksi,
français,
hrvatski,
Bahasa Indonesia,
italiano,
日本語,
한국어,
Lietuvių,
latviski,
te reo Māori,
Nederlands,
norsk,
polski,
português,
română,
русский,
Slovenščina,
svenska,
Türkçe,
українська,
中文,
華語.

Please read the FAQ for more information about official translations.
))??
`
const license_CC_BY_SA_1_0_lre = `//**
CC-BY-SA-1.0
https://creativecommons.org/licenses/by-sa/1.0
**//

((Creative Commons))??

Creative Commons

Attribution-ShareAlike 1.0

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE
LEGAL
SERVICES. DISTRIBUTION OF THIS DRAFT LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION
ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING
FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE").  THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW.  ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE.  THE LICENSOR GRANTS YOU THE RIGHTS
CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole.  A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant.  Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to create and reproduce Derivative Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission Derivative Works;

The above rights may be exercised in all media and formats whether now known or
hereafter devised.  The above rights include the right to make such
modifications as are technically necessary to exercise the rights in other media
and formats.  All rights not expressly granted by Licensor are hereby reserved.

4. Restrictions. The license granted in Section 3 above is expressly made
subject to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform.  You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder.  You may not sublicense the Work.  You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement.  The above applies to the Work as incorporated in a
Collective Work, but this does not require the Collective Work apart from the
Work itself to be made subject to the terms of this License. If You create a
Collective Work, upon notice from any Licensor You must, to the extent
practicable, remove from the Collective Work any reference to such Licensor or
the Original Author, as requested.  If You create a Derivative Work, upon notice
from any Licensor You must, to the extent practicable, remove from the
Derivative Work any reference to such Licensor or the Original Author, as
requested.

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform a Derivative Work only under the terms of this License, and You must
include a copy of, or the Uniform Resource Identifier for, this License with
every copy or phonorecord of each Derivative Work You distribute, publicly
display, publicly perform, or publicly digitally perform. You may not offer or
impose any terms on the Derivative Works that alter or restrict the terms of
this License or the recipients' exercise of the rights granted hereunder, and
You must keep intact all notices that refer to this License and to the
disclaimer of warranties.  You may not distribute, publicly display, publicly
perform, or publicly digitally perform the Derivative Work with any
technological measures that control access or use of the Work in a manner
inconsistent with the terms of this License Agreement.  The above applies to the
Derivative Work as incorporated in a Collective Work, but this does not require
the Collective Work apart from the Derivative Work itself to be made subject to
the terms of this License.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or any Derivative Works or Collective Works, You must keep
intact all copyright notices for the Work and give the Original Author credit
reasonable to the medium or means You are utilizing by conveying the name (or
pseudonym if applicable) of the Original Author if supplied; the title of the
Work if supplied; in the case of a Derivative Work, a credit identifying the use
of the Work in the Derivative Work (e.g., "French translation of the Work by
Original Author," or "Screenplay based on original Work by Original Author").
Such credit may be implemented in any reasonable manner; provided, however, that
in the case of a Derivative Work or Collective Work, at a minimum such credit
will appear where any other comparable authorship credit appears and in a manner
at least as prominent as such other comparable authorship credit.

5. Representations, Warranties and Disclaimer

__1__
By offering the Work for public release under this License, Licensor represents
and warrants that, to the best of Licensor's knowledge after reasonable
inquiry:

__1__
Licensor has secured all rights in the Work necessary to grant the license
rights hereunder and to permit the lawful exercise of the rights granted
hereunder without You having any obligation to pay any royalties, compulsory
license fees, residuals or any other payments;

__1__
The Work does not infringe the copyright, trademark, publicity rights, common
law rights or any other right of any third party or constitute defamation,
invasion of privacy or other tortious injury to any third party.

__1__
EXCEPT AS EXPRESSLY STATED IN THIS LICENSE OR OTHERWISE AGREED IN WRITING OR
REQUIRED BY APPLICABLE LAW, THE WORK IS LICENSED ON AN "AS IS" BASIS, WITHOUT
WARRANTIES OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION,
ANY WARRANTIES REGARDING THE CONTENTS OR ACCURACY OF THE WORK.

6. Limitation on Liability.  EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW,
AND EXCEPT FOR DAMAGES ARISING FROM LIABILITY TO A THIRD PARTY RESULTING FROM
BREACH OF THE WARRANTIES IN SECTION 5, IN NO EVENT WILL LICENSOR BE LIABLE TO
YOU ON ANY LEGAL THEORY FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR
EXEMPLARY DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK, EVEN IF
LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Derivative Works or Collective Works from You under this License,
however, will not have their licenses terminated provided such individuals or
entities remain in full compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work).  Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
Each time You distribute or publicly digitally perform a Derivative Work,
Licensor offers to the recipient a license to the original Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here.  There are no understandings, agreements or
representations with respect to the Work not specified here.  Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work.  Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license.  Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons.  Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_SA_2_0_lre = `//**
CC-BY-SA-2.0
https://creativecommons.org/licenses/by-sa/2.0
**//

((Creative Commons))??

Creative Commons

Attribution-ShareAlike 2.0

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE LEGAL
SERVICES. DISTRIBUTION OF THIS LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE"). THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE. THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED
HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole. A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License. For
the avoidance of doubt, where the Work is a musical composition or sound
recording, the synchronization of the Work in timed-relation with a moving image
("synching") will be considered a Derivative Work for the purpose of this
License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

__1__ "License Elements" means the following high-level license attributes as
selected by Licensor and indicated in the title of this License: Attribution,
ShareAlike.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant. Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to create and reproduce Derivative Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission Derivative Works.

__1__
For the avoidance of doubt, where the work is a musical composition:

__1__ Performance Royalties Under Blanket Licenses. Licensor waives the
exclusive right to collect, whether individually or via a performance rights
society (e.g. ASCAP, BMI, SESAC), royalties for the public performance or public
digital performance (e.g. webcast) of the Work.

__1__ Mechanical Rights and Statutory Royalties. Licensor waives the exclusive
right to collect, whether individually or via a music rights society or
designated agent (e.g. Harry Fox Agency), royalties for any phonorecord You
create from the Work ("cover version") and distribute, subject to the compulsory
license created by 17 USC Section 115 of the US Copyright Act (or the equivalent
in other jurisdictions).

__1__ Webcasting Rights and Statutory Royalties. For the avoidance of doubt,
where the Work is a sound recording, Licensor waives the exclusive right to
collect, whether individually or via a performance-rights society (e.g.
SoundExchange), royalties for the public digital performance (e.g. webcast) of
the Work, subject to the compulsory license created by 17 USC Section 114 of the
US Copyright Act (or the equivalent in other jurisdictions).

The above rights may be exercised in all media and formats whether now known or
hereafter devised. The above rights include the right to make such modifications
as are technically necessary to exercise the rights in other media and formats.
All rights not expressly granted by Licensor are hereby reserved.

4. Restrictions.The license granted in Section 3 above is expressly made subject
to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform. You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder. You may not sublicense the Work. You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement. The above applies to the Work as incorporated in a Collective
Work, but this does not require the Collective Work apart from the Work itself
to be made subject to the terms of this License. If You create a Collective
Work, upon notice from any Licensor You must, to the extent practicable, remove
from the Collective Work any reference to such Licensor or the Original Author,
as requested. If You create a Derivative Work, upon notice from any Licensor You
must, to the extent practicable, remove from the Derivative Work any reference
to such Licensor or the Original Author, as requested.

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform a Derivative Work only under the terms of this License, a later version
of this License with the same License Elements as this License, or a Creative
Commons iCommons license that contains the same License Elements as this License
(e.g. Attribution-ShareAlike 2.0 Japan). You must include a copy of, or the
Uniform Resource Identifier for, this License or other license specified in the
previous sentence with every copy or phonorecord of each Derivative Work You
distribute, publicly display, publicly perform, or publicly digitally perform.
You may not offer or impose any terms on the Derivative Works that alter or
restrict the terms of this License or the recipients' exercise of the rights
granted hereunder, and You must keep intact all notices that refer to this
License and to the disclaimer of warranties. You may not distribute, publicly
display, publicly perform, or publicly digitally perform the Derivative Work
with any technological measures that control access or use of the Work in a
manner inconsistent with the terms of this License Agreement. The above applies
to the Derivative Work as incorporated in a Collective Work, but this does not
require the Collective Work apart from the Derivative Work itself to be made
subject to the terms of this License.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or any Derivative Works or Collective Works, You must keep
intact all copyright notices for the Work and give the Original Author credit
reasonable to the medium or means You are utilizing by conveying the name (or
pseudonym if applicable) of the Original Author if supplied; the title of the
Work if supplied; to the extent reasonably practicable, the Uniform Resource
Identifier, if any, that Licensor specifies to be associated with the Work,
unless such URI does not refer to the copyright notice or licensing information
for the Work; and in the case of a Derivative Work, a credit identifying the use
of the Work in the Derivative Work (e.g., "French translation of the Work by
Original Author," or "Screenplay based on original Work by Original Author").
Such credit may be implemented in any reasonable manner; provided, however, that
in the case of a Derivative Work or Collective Work, at a minimum such credit
will appear where any other comparable authorship credit appears and in a manner
at least as prominent as such other comparable authorship credit.

5. Representations, Warranties and Disclaimer

UNLESS OTHERWISE AGREED TO BY THE PARTIES IN WRITING, LICENSOR OFFERS THE WORK
AS-IS AND MAKES NO REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING THE
MATERIALS, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT
LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY, FITNESS FOR A PARTICULAR
PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY,
OR THE PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE. SOME
JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO SUCH
EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability. EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN
NO EVENT WILL LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY SPECIAL,
INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY DAMAGES ARISING OUT OF THIS
LICENSE OR THE USE OF THE WORK, EVEN IF LICENSOR HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Derivative Works or Collective Works from You under this License,
however, will not have their licenses terminated provided such individuals or
entities remain in full compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work). Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
Each time You distribute or publicly digitally perform a Derivative Work,
Licensor offers to the recipient a license to the original Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here. There are no understandings, agreements or
representations with respect to the Work not specified here. Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work. Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license. Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons. Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_SA_2_5_lre = `//**
CC-BY-SA-2.5
https://creativecommons.org/licenses/by-sa/2.5
**//

((Creative Commons))??

Creative Commons

Attribution-ShareAlike 2.5

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE LEGAL
SERVICES. DISTRIBUTION OF THIS LICENSE DOES NOT CREATE AN
ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING FROM
ITS USE.
))??

((License))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS
PUBLIC LICENSE ("CCPL" OR "LICENSE"). THE WORK IS PROTECTED BY COPYRIGHT AND/OR
OTHER APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE. THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED
HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

1. Definitions

__1__
"Collective Work" means a work, such as a periodical issue, anthology or
encyclopedia, in which the Work in its entirety in unmodified form, along with a
number of other contributions, constituting separate and independent works in
themselves, are assembled into a collective whole. A work that constitutes a
Collective Work will not be considered a Derivative Work (as defined below) for
the purposes of this License.

__1__
"Derivative Work" means a work based upon the Work or upon the Work and other
pre-existing works, such as a translation, musical arrangement, dramatization,
fictionalization, motion picture version, sound recording, art reproduction,
abridgment, condensation, or any other form in which the Work may be recast,
transformed, or adapted, except that a work that constitutes a Collective Work
will not be considered a Derivative Work for the purpose of this License. For
the avoidance of doubt, where the Work is a musical composition or sound
recording, the synchronization of the Work in timed-relation with a moving image
("synching") will be considered a Derivative Work for the purpose of this
License.

__1__
"Licensor" means the individual or entity that offers the Work under the terms
of this License.

__1__
"Original Author" means the individual or entity who created the Work.

__1__
"Work" means the copyrightable work of authorship offered under the terms of
this License.

__1__
"You" means an individual or entity exercising rights under this License who has
not previously violated the terms of this License with respect to the Work, or
who has received express permission from the Licensor to exercise rights under
this License despite a previous violation.

__1__ "License Elements" means the following high-level license attributes as
selected by Licensor and indicated in the title of this License: Attribution,
ShareAlike.

2. Fair Use Rights. Nothing in this license is intended to reduce, limit, or
restrict any rights arising from fair use, first sale or other limitations on
the exclusive rights of the copyright owner under copyright law or other
applicable laws.

3. License Grant. Subject to the terms and conditions of this License, Licensor
hereby grants You a worldwide, royalty-free, non-exclusive, perpetual (for the
duration of the applicable copyright) license to exercise the rights in the Work
as stated below:

__1__
to reproduce the Work, to incorporate the Work into one or more Collective
Works, and to reproduce the Work as incorporated in the Collective Works;

__1__
to create and reproduce Derivative Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission the Work including as
incorporated in Collective Works;

__1__
to distribute copies or phonorecords of, display publicly, perform publicly, and
perform publicly by means of a digital audio transmission Derivative Works.

__1__
For the avoidance of doubt, where the work is a musical composition:

__1__ Performance Royalties Under Blanket Licenses. Licensor waives the
exclusive right to collect, whether individually or via a performance rights
society (e.g. ASCAP, BMI, SESAC), royalties for the public performance or public
digital performance (e.g. webcast) of the Work.

__1__ Mechanical Rights and Statutory Royalties. Licensor waives the exclusive
right to collect, whether individually or via a music rights society or
designated agent (e.g. Harry Fox Agency), royalties for any phonorecord You
create from the Work ("cover version") and distribute, subject to the compulsory
license created by 17 USC Section 115 of the US Copyright Act (or the equivalent
in other jurisdictions).

__1__ Webcasting Rights and Statutory Royalties. For the avoidance of doubt,
where the Work is a sound recording, Licensor waives the exclusive right to
collect, whether individually or via a performance-rights society (e.g.
SoundExchange), royalties for the public digital performance (e.g. webcast) of
the Work, subject to the compulsory license created by 17 USC Section 114 of the
US Copyright Act (or the equivalent in other jurisdictions).

The above rights may be exercised in all media and formats whether now known or
hereafter devised. The above rights include the right to make such modifications
as are technically necessary to exercise the rights in other media and formats.
All rights not expressly granted by Licensor are hereby reserved.

4. Restrictions.The license granted in Section 3 above is expressly made subject
to and limited by the following restrictions:

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform the Work only under the terms of this License, and You must include a
copy of, or the Uniform Resource Identifier for, this License with every copy or
phonorecord of the Work You distribute, publicly display, publicly perform, or
publicly digitally perform. You may not offer or impose any terms on the Work
that alter or restrict the terms of this License or the recipients' exercise of
the rights granted hereunder. You may not sublicense the Work. You must keep
intact all notices that refer to this License and to the disclaimer of
warranties. You may not distribute, publicly display, publicly perform, or
publicly digitally perform the Work with any technological measures that control
access or use of the Work in a manner inconsistent with the terms of this
License Agreement. The above applies to the Work as incorporated in a Collective
Work, but this does not require the Collective Work apart from the Work itself
to be made subject to the terms of this License. If You create a Collective
Work, upon notice from any Licensor You must, to the extent practicable, remove
from the Collective Work any credit as required by clause 4(c), as requested. If
You create a Derivative Work, upon notice from any Licensor You must, to the
extent practicable, remove from the Derivative Work any credit as required by
clause 4(c), as requested.

__1__
You may distribute, publicly display, publicly perform, or publicly digitally
perform a Derivative Work only under the terms of this License, a later version
of this License with the same License Elements as this License, or a Creative
Commons iCommons license that contains the same License Elements as this License
(e.g. Attribution-ShareAlike 2.5 Japan). You must include a copy of, or the
Uniform Resource Identifier for, this License or other license specified in the
previous sentence with every copy or phonorecord of each Derivative Work You
distribute, publicly display, publicly perform, or publicly digitally perform.
You may not offer or impose any terms on the Derivative Works that alter or
restrict the terms of this License or the recipients' exercise of the rights
granted hereunder, and You must keep intact all notices that refer to this
License and to the disclaimer of warranties. You may not distribute, publicly
display, publicly perform, or publicly digitally perform the Derivative Work
with any technological measures that control access or use of the Work in a
manner inconsistent with the terms of this License Agreement. The above applies
to the Derivative Work as incorporated in a Collective Work, but this does not
require the Collective Work apart from the Derivative Work itself to be made
subject to the terms of this License.

__1__
If you distribute, publicly display, publicly perform, or publicly digitally
perform the Work or any Derivative Works or Collective Works, You must keep
intact all copyright notices for the Work and provide, reasonable to the medium
or means You are utilizing: (i) the name of the Original Author (or pseudonym,
if applicable) if supplied, and/or (ii) if the Original Author and/or Licensor
designate another party or parties (e.g. a sponsor institute, publishing entity,
journal) for attribution in Licensor's copyright notice, terms of service or by
other reasonable means, the name of such party or parties; the title of the Work
if supplied; to the extent reasonably practicable, the Uniform Resource
Identifier, if any, that Licensor specifies to be associated with the Work,
unless such URI does not refer to the copyright notice or licensing information
for the Work; and in the case of a Derivative Work, a credit identifying the use
of the Work in the Derivative Work (e.g., "French translation of the Work by
Original Author," or "Screenplay based on original Work by Original Author").
Such credit may be implemented in any reasonable manner; provided, however, that
in the case of a Derivative Work or Collective Work, at a minimum such credit
will appear where any other comparable authorship credit appears and in a manner
at least as prominent as such other comparable authorship credit.

5. Representations, Warranties and Disclaimer

UNLESS OTHERWISE AGREED TO BY THE PARTIES IN WRITING, LICENSOR OFFERS THE WORK
AS-IS AND MAKES NO REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING THE
MATERIALS, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING, WITHOUT
LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY, FITNESS FOR A PARTICULAR
PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY,
OR THE PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE. SOME
JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO SUCH
EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability. EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN
NO EVENT WILL LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY SPECIAL,
INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY DAMAGES ARISING OUT OF THIS
LICENSE OR THE USE OF THE WORK, EVEN IF LICENSOR HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

7. Termination

__1__
This License and the rights granted hereunder will terminate automatically upon
any breach by You of the terms of this License. Individuals or entities who have
received Derivative Works or Collective Works from You under this License,
however, will not have their licenses terminated provided such individuals or
entities remain in full compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__
Subject to the above terms and conditions, the license granted here is perpetual
(for the duration of the applicable copyright in the Work). Notwithstanding the
above, Licensor reserves the right to release the Work under different license
terms or to stop distributing the Work at any time; provided, however that any
such election will not serve to withdraw this License (or any other license that
has been, or is required to be, granted under the terms of this License), and
this License will continue in full force and effect unless terminated as stated
above.

8. Miscellaneous

__1__
Each time You distribute or publicly digitally perform the Work or a Collective
Work, the Licensor offers to the recipient a license to the Work on the same
terms and conditions as the license granted to You under this License.

__1__
Each time You distribute or publicly digitally perform a Derivative Work,
Licensor offers to the recipient a license to the original Work on the same
terms and conditions as the license granted to You under this License.

__1__
If any provision of this License is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this License, and without further action by the parties to this
agreement, such provision shall be reformed to the minimum extent necessary to
make such provision valid and enforceable.

__1__
No term or provision of this License shall be deemed waived and no breach
consented to unless such waiver or consent shall be in writing and signed by the
party to be charged with such waiver or consent.

__1__
This License constitutes the entire agreement between the parties with respect
to the Work licensed here. There are no understandings, agreements or
representations with respect to the Work not specified here. Licensor shall not
be bound by any additional provisions that may appear in any communication from
You. This License may not be modified without the mutual written agreement of
the Licensor and You.

((
Creative Commons is not a party to this License, and makes no warranty
whatsoever in connection with the Work. Creative Commons will not be liable to
You or any party on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or consequential damages
arising in connection to this license. Notwithstanding the foregoing two (2)
sentences, if Creative Commons has expressly identified itself as the Licensor
hereunder, it shall have all rights and obligations of Licensor.

Except for the limited purpose of indicating to the public that the Work is
licensed under the CCPL, neither party will use the trademark "Creative Commons"
or any related trademark or logo of Creative Commons without the prior written
consent of Creative Commons. Any permitted use will be in compliance with
Creative Commons' then-current trademark usage guidelines, as may be published
on its website or otherwise made available upon request from time to time.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_SA_3_0_lre = `//**
CC-BY-SA-3.0
https://creativecommons.org/licenses/by-sa/3.0
**//

((Creative Commons))??

((
Creative Commons Legal Code
))??
((https://creativecommons.org/licenses/by-sa/3.0
((/legalcode))??
))??

((
Attribution-ShareAlike 3.0 Unported
))??
((https://creativecommons.org/licenses/by-sa/3.0
((/legalcode))??
))??

((
CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES
NOT PROVIDE LEGAL SERVICES. DISTRIBUTION OF THIS LICENSE
DOES NOT CREATE AN ATTORNEY-CLIENT RELATIONSHIP. CREATIVE
COMMONS PROVIDES THIS INFORMATION ON AN "AS-IS" BASIS.
CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE
INFORMATION PROVIDED, AND DISCLAIMS LIABILITY FOR DAMAGES
RESULTING FROM ITS USE.
))??

((
((License))??
))??
((https://creativecommons.org/licenses/by-sa/3.0
((/legalcode))??
))??

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS
OF THIS CREATIVE COMMONS PUBLIC LICENSE ("CCPL" OR
"LICENSE"). THE WORK IS PROTECTED BY COPYRIGHT AND/OR OTHER
APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS
AUTHORIZED UNDER THIS LICENSE OR COPYRIGHT LAW IS
PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU
ACCEPT AND AGREE TO BE BOUND BY THE TERMS OF THIS LICENSE.
TO THE EXTENT THIS LICENSE MAY BE CONSIDERED TO BE A
CONTRACT, THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED HERE
IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND
CONDITIONS.

1. Definitions

__1__ "Adaptation" means a work based upon
the Work, or upon the Work and other pre-existing works,
such as a translation, adaptation, derivative work,
arrangement of music or other alterations of a literary
or artistic work, or phonogram or performance and
includes cinematographic adaptations or any other form in
which the Work may be recast, transformed, or adapted
including in any form recognizably derived from the
original, except that a work that constitutes a
Collection will not be considered an Adaptation for the
purpose of this License. For the avoidance of doubt,
where the Work is a musical work, performance or
phonogram, the synchronization of the Work in
timed-relation with a moving image ("synching") will be
considered an Adaptation for the purpose of this
License.

__1__ "Collection" means a collection of
literary or artistic works, such as encyclopedias and
anthologies, or performances, phonograms or broadcasts,
or other works or subject matter other than works listed
in Section 1(f) below, which, by reason of the selection
and arrangement of their contents, constitute
intellectual creations, in which the Work is included in
its entirety in unmodified form along with one or more
other contributions, each constituting separate and
independent works in themselves, which together are
assembled into a collective whole. A work that
constitutes a Collection will not be considered an
Adaptation (as defined below) for the purposes of this
License.

__1__ "Creative Commons Compatible
License" means a license that is listed at
https://creativecommons.org/compatiblelicenses that has
been approved by Creative Commons as being essentially
equivalent to this License, including, at a minimum,
because that license: (i) contains terms that have the
same purpose, meaning and effect as the License Elements
of this License; and, (ii) explicitly permits the
relicensing of adaptations of works made available under
that license under this License or a Creative Commons
jurisdiction license with the same License Elements as
this License.

__1__ "Distribute" means to make available
to the public the original and copies of the Work or
Adaptation, as appropriate, through sale or other
transfer of ownership.

__1__ "License Elements" means the
following high-level license attributes as selected by
Licensor and indicated in the title of this License:
Attribution, ShareAlike.

__1__ "Licensor" means the individual,
individuals, entity or entities that offer(s) the Work
under the terms of this License.

__1__ "Original Author" means, in the case
of a literary or artistic work, the individual,
individuals, entity or entities who created the Work or
if no individual or entity can be identified, the
publisher; and in addition (i) in the case of a
performance the actors, singers, musicians, dancers, and
other persons who act, sing, deliver, declaim, play in,
interpret or otherwise perform literary or artistic works
or expressions of folklore; (ii) in the case of a
phonogram the producer being the person or legal entity
who first fixes the sounds of a performance or other
sounds; and, (iii) in the case of broadcasts, the
organization that transmits the broadcast.

__1__ "Work" means the literary and/or
artistic work offered under the terms of this License
including without limitation any production in the
literary, scientific and artistic domain, whatever may be
the mode or form of its expression including digital
form, such as a book, pamphlet and other writing; a
lecture, address, sermon or other work of the same
nature; a dramatic or dramatico-musical work; a
choreographic work or entertainment in dumb show; a
musical composition with or without words; a
cinematographic work to which are assimilated works
expressed by a process analogous to cinematography; a
work of drawing, painting, architecture, sculpture,
engraving or lithography; a photographic work to which
are assimilated works expressed by a process analogous to
photography; a work of applied art; an illustration, map,
plan, sketch or three-dimensional work relative to
geography, topography, architecture or science; a
performance; a broadcast; a phonogram; a compilation of
data to the extent it is protected as a copyrightable
work; or a work performed by a variety or circus
performer to the extent it is not otherwise considered a
literary or artistic work.

__1__ "You" means an individual or entity
exercising rights under this License who has not
previously violated the terms of this License with
respect to the Work, or who has received express
permission from the Licensor to exercise rights under
this License despite a previous violation.

__1__ "Publicly Perform" means to perform
public recitations of the Work and to communicate to the
public those public recitations, by any means or process,
including by wire or wireless means or public digital
performances; to make available to the public Works in
such a way that members of the public may access these
Works from a place and at a place individually chosen by
them; to perform the Work to the public by any means or
process and the communication to the public of the
performances of the Work, including by public digital
performance; to broadcast and rebroadcast the Work by any
means including signs, sounds or images.

__1__ "Reproduce" means to make copies of
the Work by any means including without limitation by
sound or visual recordings and the right of fixation and
reproducing fixations of the Work, including storage of a
protected performance or phonogram in digital form or
other electronic medium.

2. Fair Dealing Rights. Nothing in this
License is intended to reduce, limit, or restrict any uses
free from copyright or rights arising from limitations or
exceptions that are provided for in connection with the
copyright protection under copyright law or other
applicable laws.

3. License Grant. Subject to the terms
and conditions of this License, Licensor hereby grants You
a worldwide, royalty-free, non-exclusive, perpetual (for
the duration of the applicable copyright) license to
exercise the rights in the Work as stated below:

__1__ to Reproduce the Work, to incorporate the Work into
one or more Collections, and to Reproduce the Work as
incorporated in the Collections;

__1__ to create and Reproduce Adaptations provided that any
such Adaptation, including any translation in any medium,
takes reasonable steps to clearly label, demarcate or
otherwise identify that changes were made to the original
Work. For example, a translation could be marked "The
original work was translated from English to Spanish," or
a modification could indicate "The original work has been
modified.";

__1__ to Distribute and Publicly Perform the Work including
as incorporated in Collections; and,

__1__ to Distribute and Publicly Perform Adaptations.

__1__

For the avoidance of doubt:

__1__ Non-waivable Compulsory License
Schemes. In those jurisdictions in which the
right to collect royalties through any statutory or
compulsory licensing scheme cannot be waived, the
Licensor reserves the exclusive right to collect such
royalties for any exercise by You of the rights
granted under this License;

__1__ Waivable Compulsory License
Schemes. In those jurisdictions in which the
right to collect royalties through any statutory or
compulsory licensing scheme can be waived, the
Licensor waives the exclusive right to collect such
royalties for any exercise by You of the rights
granted under this License; and,

__1__ Voluntary License Schemes. The
Licensor waives the right to collect royalties,
whether individually or, in the event that the
Licensor is a member of a collecting society that
administers voluntary licensing schemes, via that
society, from any exercise by You of the rights
granted under this License.

The above rights may be exercised in all media and
formats whether now known or hereafter devised. The above
rights include the right to make such modifications as are
technically necessary to exercise the rights in other media
and formats. Subject to Section 8(f), all rights not
expressly granted by Licensor are hereby reserved.

4. Restrictions. The license granted in
Section 3 above is expressly made subject to and limited by
the following restrictions:

__1__ You may Distribute or Publicly Perform the Work only
under the terms of this License. You must include a copy
of, or the Uniform Resource Identifier (URI) for, this
License with every copy of the Work You Distribute or
Publicly Perform. You may not offer or impose any terms
on the Work that restrict the terms of this License or
the ability of the recipient of the Work to exercise the
rights granted to that recipient under the terms of the
License. You may not sublicense the Work. You must keep
intact all notices that refer to this License and to the
disclaimer of warranties with every copy of the Work You
Distribute or Publicly Perform. When You Distribute or
Publicly Perform the Work, You may not impose any
effective technological measures on the Work that
restrict the ability of a recipient of the Work from You
to exercise the rights granted to that recipient under
the terms of the License. This Section 4(a) applies to
the Work as incorporated in a Collection, but this does
not require the Collection apart from the Work itself to
be made subject to the terms of this License. If You
create a Collection, upon notice from any Licensor You
must, to the extent practicable, remove from the
Collection any credit as required by Section 4(c), as
requested. If You create an Adaptation, upon notice from
any Licensor You must, to the extent practicable, remove
from the Adaptation any credit as required by Section
4(c), as requested.

__1__ You may Distribute or Publicly Perform an Adaptation
only under the terms of: (i) this License; (ii) a later
version of this License with the same License Elements as
this License; (iii) a Creative Commons jurisdiction
license (either this or a later license version) that
contains the same License Elements as this License (e.g.,
Attribution-ShareAlike 3.0 US ) ) ; (iv) a Creative Commons
Compatible License. If you license the Adaptation under
one of the licenses mentioned in (iv), you must comply
with the terms of that license. If you license the
Adaptation under the terms of any of the licenses
mentioned in (i), (ii) or (iii) (the "Applicable
License"), you must comply with the terms of the
Applicable License generally and the following
provisions: (I) You must include a copy of, or the URI
for, the Applicable License with every copy of each
Adaptation You Distribute or Publicly Perform; (II) You
may not offer or impose any terms on the Adaptation that
restrict the terms of the Applicable License or the
ability of the recipient of the Adaptation to exercise
the rights granted to that recipient under the terms of
the Applicable License; (III) You must keep intact all
notices that refer to the Applicable License and to the
disclaimer of warranties with every copy of the Work as
included in the Adaptation You Distribute or Publicly
Perform; (IV) when You Distribute or Publicly Perform the
Adaptation, You may not impose any effective
technological measures on the Adaptation that restrict
the ability of a recipient of the Adaptation from You to
exercise the rights granted to that recipient under the
terms of the Applicable License. This Section 4(b)
applies to the Adaptation as incorporated in a
Collection, but this does not require the Collection
apart from the Adaptation itself to be made subject to
the terms of the Applicable License.

__1__ If You Distribute, or Publicly Perform the Work or
any Adaptations or Collections, You must, unless a
request has been made pursuant to Section 4(a), keep
intact all copyright notices for the Work and provide,
reasonable to the medium or means You are utilizing: (i)
the name of the Original Author (or pseudonym, if
applicable) if supplied, and/or if the Original Author
and/or Licensor designate another party or parties (e.g.,
a sponsor institute, publishing entity, journal) for
attribution ("Attribution Parties") in Licensor's
copyright notice, terms of service or by other reasonable
means, the name of such party or parties; (ii) the title
of the Work if supplied; (iii) to the extent reasonably
practicable, the URI, if any, that Licensor specifies to
be associated with the Work, unless such URI does not
refer to the copyright notice or licensing information
for the Work; and (iv) , consistent with Ssection 3(b),
in the case of an Adaptation, a credit identifying the
use of the Work in the Adaptation (e.g., "French
translation of the Work by Original Author," or
"Screenplay based on original Work by Original Author").
The credit required by this Section 4(c) may be
implemented in any reasonable manner; provided, however,
that in the case of a Adaptation or Collection, at a
minimum such credit will appear, if a credit for all
contributing authors of the Adaptation or Collection
appears, then as part of these credits and in a manner at
least as prominent as the credits for the other
contributing authors. For the avoidance of doubt, You may
only use the credit required by this Section for the
purpose of attribution in the manner set out above and,
by exercising Your rights under this License, You may not
implicitly or explicitly assert or imply any connection
with, sponsorship or endorsement by the Original Author,
Licensor and/or Attribution Parties, as appropriate, of
You or Your use of the Work, without the separate,
express prior written permission of the Original Author,
Licensor and/or Attribution Parties.

__1__ Except as otherwise agreed in writing by the Licensor
or as may be otherwise permitted by applicable law, if
You Reproduce, Distribute or Publicly Perform the Work
either by itself or as part of any Adaptations or
Collections, You must not distort, mutilate, modify or
take other derogatory action in relation to the Work
which would be prejudicial to the Original Author's honor
or reputation. Licensor agrees that in those
jurisdictions (e.g. Japan), in which any exercise of the
right granted in Section 3(b) of this License (the right
to make Adaptations) would be deemed to be a distortion,
mutilation, modification or other derogatory action
prejudicial to the Original Author's honor and
reputation, the Licensor will waive or not assert, as
appropriate, this Section, to the fullest extent
permitted by the applicable national law, to enable You
to reasonably exercise Your right under Section 3(b) of
this License (right to make Adaptations) but not
otherwise.

5. Representations, Warranties and
Disclaimer

UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN
WRITING, LICENSOR OFFERS THE WORK AS-IS AND MAKES NO
REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING THE
WORK, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, INCLUDING,
WITHOUT LIMITATION, WARRANTIES OF TITLE, MERCHANTIBILITY,
FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE
ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY, OR THE
PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE.
SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
WARRANTIES, SO SUCH EXCLUSION MAY NOT APPLY TO YOU.

6. Limitation on Liability. EXCEPT TO
THE EXTENT REQUIRED BY APPLICABLE LAW, IN NO EVENT WILL
LICENSOR BE LIABLE TO YOU ON ANY LEGAL THEORY FOR ANY
SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY
DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK,
EVEN IF LICENSOR HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

7. Termination

__1__ This License and the rights granted hereunder will
terminate automatically upon any breach by You of the
terms of this License. Individuals or entities who have
received Adaptations or Collections from You under this
License, however, will not have their licenses terminated
provided such individuals or entities remain in full
compliance with those licenses. Sections 1, 2, 5, 6, 7,
and 8 will survive any termination of this License.

__1__ Subject to the above terms and conditions, the
license granted here is perpetual (for the duration of
the applicable copyright in the Work). Notwithstanding
the above, Licensor reserves the right to release the
Work under different license terms or to stop
distributing the Work at any time; provided, however that
any such election will not serve to withdraw this License
(or any other license that has been, or is required to
be, granted under the terms of this License), and this
License will continue in full force and effect unless
terminated as stated above.

8. Miscellaneous

__1__ Each time You Distribute or Publicly Perform the Work
or a Collection, the Licensor offers to the recipient a
license to the Work on the same terms and conditions as
the license granted to You under this License.

__1__ Each time You Distribute or Publicly Perform an
Adaptation, Licensor offers to the recipient a license to
the original Work on the same terms and conditions as the
license granted to You under this License.

__1__ If any provision of this License is invalid or
unenforceable under applicable law, it shall not affect
the validity or enforceability of the remainder of the
terms of this License, and without further action by the
parties to this agreement, such provision shall be
reformed to the minimum extent necessary to make such
provision valid and enforceable.

__1__ No term or provision of this License shall be deemed
waived and no breach consented to unless such waiver or
consent shall be in writing and signed by the party to be
charged with such waiver or consent.

__1__ This License constitutes the entire agreement between
the parties with respect to the Work licensed here. There
are no understandings, agreements or representations with
respect to the Work not specified here. Licensor shall
not be bound by any additional provisions that may appear
in any communication from You. This License may not be
modified without the mutual written agreement of the
Licensor and You.

__1__ The rights granted under, and the subject matter
referenced, in this License were drafted utilizing the
terminology of the Berne Convention for the Protection of
Literary and Artistic Works (as amended on September 28,
1979), the Rome Convention of 1961, the WIPO Copyright
Treaty of 1996, the WIPO Performances and Phonograms
Treaty of 1996 and the Universal Copyright Convention (as
revised on July 24, 1971). These rights and subject
matter take effect in the relevant jurisdiction in which
the License terms are sought to be enforced according to
the corresponding provisions of the implementation of
those treaty provisions in the applicable national law.
If the standard suite of rights granted under applicable
copyright law includes additional rights not granted
under this License, such additional rights are deemed to
be included in the License; this License is not intended
to restrict the license of any rights under applicable
law.

((
((
Creative Commons Notice
))??
((https://creativecommons.org/licenses/by-sa/3.0
((/legalcode))??
))??

Creative Commons is not a party to this License, and
makes no warranty whatsoever in connection with the Work.
Creative Commons will not be liable to You or any party
on any legal theory for any damages whatsoever, including
without limitation any general, special, incidental or
consequential damages arising in connection to this
license. Notwithstanding the foregoing two (2) sentences,
if Creative Commons has expressly identified itself as
the Licensor hereunder, it shall have all rights and
obligations of Licensor.

Except for the limited purpose of indicating to the
public that the Work is licensed under the CCPL, Creative
Commons does not authorize the use by either party of the
trademark "Creative Commons" or any related trademark or
logo of Creative Commons without the prior written
consent of Creative Commons. Any permitted use will be in
compliance with Creative Commons' then-current trademark
usage guidelines, as may be published on its website or
otherwise made available upon request from time to time.
For the avoidance of doubt, this trademark restriction
does not form part of the License.

Creative Commons may be contacted at https://creativecommons.org/.
))??
`
const license_CC_BY_SA_3_0_AT_lre = `//**
Creative Commons Attribution-Share Alike 3.0 Austria
https://spdx.org/licenses/CC-BY-SA-3.0-AT.json
https://creativecommons.org/licenses/by-sa/3.0/at/legalcode
**//

CREATIVE COMMONS IST KEINE RECHTSANWALTSKANZLEI UND LEISTET KEINE
RECHTSBERATUNG. DIE BEREITSTELLUNG DIESER LIZENZ FÜHRT ZU KEINEM
MANDATSVERHÄLTNIS. CREATIVE COMMONS STELLT DIESE INFORMATIONEN OHNE GEWÄHR ZUR
VERFÜGUNG. CREATIVE COMMONS ÜBERNIMMT KEINE GEWÄHRLEISTUNG FÜR DIE
GELIEFERTEN INFORMATIONEN UND SCHLIEßT DIE HAFTUNG FÜR SCHÄDEN AUS, DIE SICH
AUS DEREN GEBRAUCH ERGEBEN.
(( Lizenz))??


DER GEGENSTAND DIESER LIZENZ (WIE UNTER „SCHUTZGEGENSTAND" DEFINIERT) WIRD
UNTER DEN BEDINGUNGEN DIESER CREATIVE COMMONS PUBLIC LICENSE ("CCPL", „LIZENZ"
ODER "LIZENZVERTRAG") ZUR VERFÜGUNG GESTELLT. DER SCHUTZGEGENSTAND IST DURCH
DAS URHEBERRECHT UND/ODER ANDERE GESETZE GESCHÜTZT. JEDE FORM DER NUTZUNG DES
SCHUTZGEGENSTANDES, DIE NICHT AUFGRUND DIESER LIZENZ ODER DURCH GESETZE
GESTATTET IST, IST UNZULÄSSIG.

DURCH DIE AUSÜBUNG EINES DURCH DIESE LIZENZ GEWÄHRTEN RECHTS AN DEM
SCHUTZGEGENSTAND ERKLÄREN SIE SICH MIT DEN LIZENZBEDINGUNGEN RECHTSVERBINDLICH
EINVERSTANDEN. SOWEIT DIESE LIZENZ ALS LIZENZVERTRAG ANZUSEHEN IST, GEWÄHRT
IHNEN DER LIZENZGEBER DIE IN DER LIZENZ GENANNTEN RECHTE UNENTGELTLICH UND IM
AUSTAUSCH DAFÜR, DASS SIE DAS GEBUNDENSEIN AN DIE LIZENZBEDINGUNGEN
AKZEPTIEREN.

   __1__ Definitionen

      __1__ Der Begriff "Bearbeitung" im Sinne dieser Lizenz bezeichnet das
      Ergebnis jeglicher Art von Veränderung des Schutzgegenstandes, solange
      dieses erkennbar vom Schutzgegenstand abgeleitet wurde. Dies kann
      insbesondere auch eine Umgestaltung, Änderung, Anpassung, Übersetzung
      oder Heranziehung des Schutzgegenstandes zur Vertonung von Laufbildern
      sein. Nicht als Bearbeitung des Schutzgegenstandes gelten seine Aufnahme
      in eine Sammlung oder ein Sammelwerk und die freie Nutzung des
      Schutzgegenstandes.

      __1__ Der Begriff "Sammelwerk" im Sinne dieser Lizenz meint eine
      Zusammenstellung von literarischen, künstlerischen oder
      wissenschaftlichen Inhalten zu einem einheitlichen Ganzen, sofern diese
      Zusammenstellung aufgrund von Auswahl und Anordnung der darin enthaltenen
      selbständigen Elemente eine eigentümliche geistige Schöpfung darstellt,
      unabhängig davon, ob die Elemente systematisch oder methodisch angelegt
      und dadurch einzeln zugänglich sind oder nicht.

      __1__ "Verbreiten" im Sinne dieser Lizenz bedeutet, den Schutzgegenstand
      oder Bearbeitungen im Original oder in Form von
      Vervielfältigungsstücken, mithin in körperlich fixierter Form der
      Öffentlichkeit zugänglich zu machen oder in Verkehr zu bringen.

      __1__ Unter "Lizenzelementen" werden im Sinne dieser Lizenz die folgenden
      übergeordneten Lizenzcharakteristika verstanden, die vom Lizenzgeber
      ausgewählt wurden und in der Bezeichnung der Lizenz zum Ausdruck kommen:
      "Namensnennung", "Weitergabe unter gleichen Bedingungen".

      __1__ Der "Lizenzgeber" im Sinne dieser Lizenz ist diejenige natürliche
      oder juristische Person oder Gruppe, die den Schutzgegenstand unter den
      Bedingungen dieser Lizenz anbietet und insoweit als Rechteinhaberin
      auftritt.

      __1__ "Rechteinhaber" im Sinne dieser Lizenz ist der Urheber des
      Schutzgegenstandes oder jede andere natürliche oder juristische Person,
      die am Schutzgegenstand ein Immaterialgüterrecht erlangt hat, welches die
      in Abschnitt 3 genannten Handlungen erfasst und eine Erteilung,
      Übertragung oder Einräumung von Nutzungsbewilligungen bzw
      Nutzungsrechten an Dritte erlaubt.

      __1__ Der Begriff "Schutzgegenstand" bezeichnet in dieser Lizenz den
      literarischen, künstlerischen oder wissenschaftlichen Inhalt, der unter
      den Bedingungen dieser Lizenz angeboten wird. Das kann insbesondere eine
      eigentümliche geistige Schöpfung jeglicher Art oder ein Werk der kleinen
      Münze, ein nachgelassenes Werk oder auch ein Lichtbild oder anderes
      Objekt eines verwandten Schutzrechts sein, unabhängig von der Art seiner
      Fixierung und unabhängig davon, auf welche Weise jeweils eine Wahrnehmung
      erfolgen kann, gleichviel ob in analoger oder digitaler Form. Soweit
      Datenbanken oder Zusammenstellungen von Daten einen
      immaterialgüterrechtlichen Schutz eigener Art genießen, unterfallen auch
      sie dem Begriff „Schutzgegenstand" im Sinne dieser Lizenz.

      __1__ Mit "Sie" bzw. "Ihnen" ist die natürliche oder juristische Person
      gemeint, die in dieser Lizenz im Abschnitt 3 genannte Nutzungen des
      Schutzgegenstandes vornimmt und zuvor in Hinblick auf den Schutzgegenstand
      nicht gegen Bedingungen dieser Lizenz verstoßen oder aber die
      ausdrückliche Erlaubnis des Lizenzgebers erhalten hat, die durch diese
      Lizenz gewährte Nutzungsbewilligung trotz eines vorherigen Verstoßes
      auszuüben.

      __1__ Unter "Öffentlich Wiedergeben" im Sinne dieser Lizenz sind
      Wahrnehmbarmachungen des Schutzgegenstandes in unkörperlicher Form zu
      verstehen, die für eine Mehrzahl von Mitgliedern der Öffentlichkeit
      bestimmt sind und mittels öffentlicher Wiedergabe in Form von Vortrag,
      Aufführung, Vorführung, Darbietung, Sendung, Weitersendung oder zeit-
      und ortsunabhängiger Zurverfügungstellung erfolgen, unabhängig von den
      zum Einsatz kommenden Techniken und Verfahren, einschließlich
      drahtgebundener oder drahtloser Mittel und Einstellen in das Internet.

      __1__ "Vervielfältigen" im Sinne dieser Lizenz bedeutet, gleichviel in
      welchem Verfahren, auf welchem Träger, in welcher Menge und ob
      vorübergehend oder dauerhaft, Vervielfältigungsstücke des
      Schutzgegenstandes herzustellen, insbesondere durch Ton- oder
      Bildaufzeichnungen, und umfasst auch das erstmalige Festhalten des
      Schutzgegenstandes oder dessen Wahrnehmbarmachung auf Mitteln der
      wiederholbaren Wiedergabe sowie das Herstellen von
      Vervielfältigungsstücken dieser Festhaltung, sowie die Speicherung einer
      geschützten Darbietung oder eines Bild- und/oder Schallträgers in
      digitaler Form oder auf einem anderen elektronischen Medium.

      __1__ "Mit Creative Commons kompatible Lizenz" bezeichnet eine Lizenz, die
      unter https:/creativecommons.org/compatiblelicenses aufgelistet ist und
      die durch Creative Commons als grundsätzlich zur vorliegenden Lizenz
      äquivalent akzeptiert wurde, da zumindest folgende Voraussetzungen
      erfüllt sind:

      Diese mit Creative Commons kompatible Lizenz

         __1__ enthält Bestimmungen, welche die gleichen Ziele verfolgen, die
         gleiche Bedeutung haben und die gleichen Wirkungen erzeugen wie die
         Lizenzelemente der vorliegenden Lizenz; und

         __1__ erlaubt ausdrücklich das Lizenzieren von ihr unterstellten
         Abwandlungen unter vorliegender Lizenz, unter einer anderen
         rechtsordnungsspezifisch angepassten Creative-Commons-Lizenz mit
         denselben Lizenzelementen wie vorliegende Lizenz aufweist oder unter
         der entsprechenden Creative-Commons-Unported-Lizenz.

   __1__ Beschränkungen der Verwertungsrechte

   Diese Lizenz ist in keiner Weise darauf gerichtet, Befugnisse zur Nutzung des
   Schutzgegenstandes zu vermindern, zu beschränken oder zu vereiteln, die sich
   aus den Beschränkungen der Verwertungsrechte, anderen Beschränkungen der
   Ausschließlichkeitsrechte des Rechtsinhabers oder anderen entsprechenden
   Rechtsnormen oder sich aus dem Fehlen eines immaterialgüterrechtlichen
   Schutzes ergeben.

   __1__ Lizenzierung

   Unter den Bedingungen dieser Lizenz erteilt Ihnen der Lizenzgeber -
   unbeschadet unverzichtbarer Rechte und vorbehaltlich des Abschnitts 3.e) -
   die vergütungsfreie, räumlich und zeitlich (für die Dauer des
   Urheberrechts oder verwandten Schutzrechts am Schutzgegenstand)
   unbeschränkte Nutzungsbewilligung, den Schutzgegenstand in der folgenden Art
   und Weise zu nutzen:

      __1__ Den Schutzgegenstand in beliebiger Form und Menge zu
      vervielfältigen, ihn in Sammelwerke zu integrieren und ihn als Teil
      solcher Sammelwerke zu vervielfältigen;

      __1__ Den Schutzgegenstand zu bearbeiten, einschließlich Übersetzungen
      unter Nutzung jedweder Medien anzufertigen, sofern deutlich erkennbar
      gemacht wird, dass es sich um eine Bearbeitung handelt;

      __1__ Den Schutzgegenstand, allein oder in Sammelwerke aufgenommen,
      öffentlich wiederzugeben und zu verbreiten; und

      __1__ Bearbeitungen des Schutzgegenstandes zu veröffentlichen,
      öffentlich wiederzugeben und zu verbreiten.

      __1__ Bezüglich Vergütung für die Nutzung des Schutzgegenstandes gilt
      Folgendes:

         __1__ Unverzichtbare gesetzliche Vergütungsansprüche: Soweit
         unverzichtbare Vergütungsansprüche im Gegenzug für gesetzliche
         Lizenzen vorgesehen oder Pauschalabgabensysteme (zum Beispiel für
         Leermedien) vorhanden sind, behält sich der Lizenzgeber das
         ausschließliche Recht vor, die entsprechenden Vergütungsansprüche
         für jede Ausübung eines Rechts aus dieser Lizenz durch Sie geltend zu
         machen.

         __1__ Vergütung bei Zwangslizenzen: Soweit Zwangslizenzen außerhalb
         dieser Lizenz vorgesehen sind und zustande kommen, verzichtet der
         Lizenzgeber für alle Fälle einer lizenzgerechten Nutzung des
         Schutzgegenstandes durch Sie auf jegliche Vergütung.

         __1__ Vergütung in sonstigen Fällen: Bezüglich lizenzgerechter
         Nutzung des Schutzgegenstandes durch Sie, die nicht unter die beiden
         vorherigen Abschnitte (i) und (ii) fällt, verzichtet der Lizenzgeber
         auf jegliche Vergütung, unabhängig davon, ob eine Geltendmachung der
         Vergütungsansprüche durch ihn selbst oder nur durch eine
         Verwertungsgesellschaft möglich wäre.

   Die vorgenannte Nutzungsbewilligung wird für alle bekannten sowie alle noch
   nicht bekannten Nutzungsarten eingeräumt. Sie beinhaltet auch das Recht,
   solche Änderungen am Schutzgegenstand vorzunehmen, die für bestimmte nach
   dieser Lizenz zulässige Nutzungen technisch erforderlich sind. Alle
   sonstigen Rechte, die über diesen Abschnitt hinaus nicht ausdrücklich vom
   Lizenzgeber eingeräumt werden, bleiben diesem allein vorbehalten. Soweit
   Datenbanken oder Zusammenstellungen von Daten Schutzgegenstand dieser Lizenz
   oder Teil dessen sind und einen immaterialgüterrechtlichen Schutz eigener
   Art genießen, verzichtet der Lizenzgeber auf die Geltendmachung sämtlicher
   daraus resultierender Rechte.

   __1__ Bedingungen

   Die Erteilung der Nutzungsbewilligung gemäß Abschnitt 3 dieser Lizenz
   erfolgt ausdrücklich nur unter den folgenden Bedingungen:

      __1__ Sie dürfen den Schutzgegenstand ausschließlich unter den
      Bedingungen dieser Lizenz verbreiten oder öffentlich wiedergeben. Sie
      müssen dabei stets eine Kopie dieser Lizenz oder deren vollständige
      Internetadresse in Form des Uniform-Resource-Identifier (URI) beifügen.
      Sie dürfen keine Vertrags- oder Nutzungsbedingungen anbieten oder
      fordern, die die Bedingungen dieser Lizenz oder die durch diese Lizenz
      gewährten Rechte beschränken. Sie dürfen den Schutzgegenstand nicht
      unterlizenzieren. Bei jeder Kopie des Schutzgegenstandes, die Sie
      verbreiten oder öffentlich wiedergeben, müssen Sie alle Hinweise
      unverändert lassen, die auf diese Lizenz und den Haftungsausschluss
      hinweisen. Wenn Sie den Schutzgegenstand verbreiten oder öffentlich
      wiedergeben, dürfen Sie (in Bezug auf den Schutzgegenstand) keine
      technischen Maßnahmen ergreifen, die den Nutzer des Schutzgegenstandes in
      der Ausübung der ihm durch diese Lizenz gewährten Rechte behindern
      können. Dasselbe gilt auch für den Fall, dass der Schutzgegenstand einen
      Bestandteil eines Sammelwerkes bildet, was jedoch nicht bedeutet, dass das
      Sammelwerk insgesamt dieser Lizenz unterstellt werden muss. Sofern Sie ein
      Sammelwerk erstellen, müssen Sie - soweit dies praktikabel ist - auf die
      Mitteilung eines Lizenzgebers hin aus dem Sammelwerk die in Abschnitt 4.c)
      aufgezählten Hinweise entfernen. Wenn Sie eine Bearbeitung vornehmen,
      müssen Sie – soweit dies praktikabel ist – auf die Mitteilung eines
      Lizenzgebers hin von der Bearbeitung die in Abschnitt 4.c) aufgezählten
      Hinweise entfernen.

      __1__ Sie dürfen eine Bearbeitung ausschließlich unter den Bedingungen

         __1__ dieser Lizenz,

         __1__ einer späteren Version dieser Lizenz mit denselben
         Lizenzelementen,

         __1__ einer rechtsordnungsspezifischen Creative-Commons-Lizenz mit
         denselben Lizenzelementen ab Version 3.0 aufwärts (z.B. Namensnennung
         - Weitergabe unter gleichen Bedingungen 3.0 US),

         __1__ der Creative-Commons-Unported-Lizenz mit denselben
         Lizenzelementen ab Version 3.0 aufwärts, oder

         __1__ einer mit Creative Commons kompatiblen Lizenz

      verbreiten oder öffentlich wiedergeben.

      Falls Sie die Bearbeitung gemäß Abschnitt b)(v) unter einer mit Creative
      Commons kompatiblen Lizenz lizenzieren, müssen Sie deren
      Lizenzbestimmungen Folge leisten.

      Falls Sie die Bearbeitung unter einer der unter b)(i)-(iv) genannten
      Lizenzen ("Verwendbare Lizenzen") lizenzieren, müssen Sie deren
      Lizenzbestimmungen sowie folgenden Bestimmungen Folge leisten: Sie müssen
      stets eine Kopie der verwendbaren Lizenz oder deren vollständige
      Internetadresse in Form des Uniform-Resource-Identifier (URI) beifügen,
      wenn Sie die Bearbeitung verbreiten oder öffentlich wiedergeben. Sie
      dürfen keine Vertrags- oder Nutzungsbedingungen anbieten oder fordern,
      die die Bedingungen der verwendbaren Lizenz oder die durch sie gewährten
      Rechte beschränken. Bei jeder Bearbeitung, die Sie verbreiten oder
      öffentlich wiedergeben, müssen Sie alle Hinweise auf die verwendbare
      Lizenz und den Haftungsausschluss unverändert lassen. Wenn Sie die
      Bearbeitung verbreiten oder öffentlich wiedergeben, dürfen Sie (in Bezug
      auf die Bearbeitung) keine technischen Maßnahmen ergreifen, die den
      Nutzer der Bearbeitung in der Ausübung der ihm durch die verwendbare
      Lizenz gewährten Rechte behindern können. Dieser Abschnitt 4.b) gilt
      auch für den Fall, dass die Bearbeitung einen Bestandteil eines
      Sammelwerkes bildet; dies bedeutet jedoch nicht, dass das Sammelwerk
      insgesamt der verwendbaren Lizenz unterstellt werden muss.

      __1__ Die Verbreitung und die öffentliche Wiedergabe des
      Schutzgegenstandes oder auf ihm aufbauender Inhalte oder ihn enthaltender
      Sammelwerke ist Ihnen nur unter der Bedingung gestattet, dass Sie,
      vorbehaltlich etwaiger Mitteilungen im Sinne von Abschnitt 4.a), alle dazu
      gehörenden Rechtevermerke unberührt lassen. Sie sind verpflichtet, die
      Urheberschaft oder die Rechteinhaberschaft in einer der Nutzung
      entsprechenden, angemessenen Form anzuerkennen, indem Sie selbst –
      soweit bekannt – Folgendes angeben:

         __1__ Den Namen (oder das Pseudonym, falls ein solches verwendet wird)
         des Rechteinhabers, und/oder falls der Lizenzgeber im Rechtevermerk, in
         den Nutzungsbedingungen oder auf andere angemessene Weise eine
         Zuschreibung an Dritte vorgenommen hat (z.B. an eine Stiftung, ein
         Verlagshaus oder eine Zeitung) („Zuschreibungsempfänger"), Namen
         bzw. Bezeichnung dieses oder dieser Dritten;

         __1__ den Titel des Inhaltes;

         __1__ in einer praktikablen Form den Uniform-Resource-Identifier (URI,
         z.B. Internetadresse), den der Lizenzgeber zum Schutzgegenstand
         angegeben hat, es sei denn, dieser URI verweist nicht auf den
         Rechtevermerk oder die Lizenzinformationen zum Schutzgegenstand;

         __1__ und im Falle einer Bearbeitung des Schutzgegenstandes in
         Übereinstimmung mit Abschnitt 3.b) einen Hinweis darauf, dass es sich
         um eine Bearbeitung handelt.

      Die nach diesem Abschnitt 4.c) erforderlichen Angaben können in jeder
      angemessenen Form gemacht werden; im Falle einer Bearbeitung des
      Schutzgegenstandes oder eines Sammelwerkes müssen diese Angaben das
      Minimum darstellen und bei gemeinsamer Nennung aller Beitragenden
      dergestalt erfolgen, dass sie zumindest ebenso hervorgehoben sind wie die
      Hinweise auf die übrigen Rechteinhaber. Die Angaben nach diesem Abschnitt
      dürfen Sie ausschließlich zur Angabe der Rechteinhaberschaft in der oben
      bezeichneten Weise verwenden. Durch die Ausübung Ihrer Rechte aus dieser
      Lizenz dürfen Sie ohne eine vorherige, separat und schriftlich
      vorliegende Zustimmung des Urhebers, des Lizenzgebers und/oder des
      Zuschreibungsempfängers weder implizit noch explizit irgendeine
      Verbindung mit dem oder eine Unterstützung oder Billigung durch den
      Lizenzgeber oder den Zuschreibungsempfänger andeuten oder erklären.

      __1__ Die oben unter 4.a) bis c) genannten Einschränkungen gelten nicht
      für solche Teile des Schutzgegenstandes, die allein deshalb unter den
      Schutzgegenstandsbegriff fallen, weil sie als Datenbanken oder
      Zusammenstellungen von Daten einen immaterialgüterrechtlichen Schutz
      eigener Art genießen.

      __1__ (Urheber)Persönlichkeitsrechte bleiben - soweit sie bestehen - von
      dieser Lizenz unberührt.

   __1__ Gewährleistung

   SOFERN KEINE ANDERS LAUTENDE, SCHRIFTLICHE VEREINBARUNG ZWISCHEN DEM
   LIZENZGEBER UND IHNEN GESCHLOSSEN WURDE UND SOWEIT MÄNGEL NICHT ARGLISTIG
   VERSCHWIEGEN WURDEN, BIETET DER LIZENZGEBER DEN SCHUTZGEGENSTAND UND DIE
   ERTEILUNG DER NUTZUNGSBEWILLIGUNG UNTER AUSSCHLUSS JEGLICHER GEWÄHRLEISTUNG
   AN UND ÜBERNIMMT WEDER AUSDRÜCKLICH NOCH KONKLUDENT GARANTIEN IRGENDEINER
   ART. DIES UMFASST INSBESONDERE DAS FREISEIN VON SACH- UND RECHTSMÄNGELN,
   UNABHÄNGIG VON DEREN ERKENNBARKEIT FÜR DEN LIZENZGEBER, DIE
   VERKEHRSFÄHIGKEIT DES SCHUTZGEGENSTANDES, SEINE VERWENDBARKEIT FÜR EINEN
   BESTIMMTEN ZWECK SOWIE DIE KORREKTHEIT VON BESCHREIBUNGEN.

   __1__ Haftungsbeschränkung

   ÜBER DIE IN ZIFFER 5 GENANNTE GEWÄHRLEISTUNG HINAUS HAFTET DER LIZENZGEBER
   IHNEN GEGENÜBER FÜR SCHÄDEN JEGLICHER ART NUR BEI GROBER FAHRLÄSSIGKEIT
   ODER VORSATZ, UND ÜBERNIMMT DARÜBER HINAUS KEINERLEI FREIWILLIGE HAFTUNG
   FÜR FOLGE- ODER ANDERE SCHÄDEN, AUCH WENN ER ÜBER DIE MÖGLICHKEIT IHRES
   EINTRITTS UNTERRICHTET WURDE.

   __1__ Erlöschen

      __1__ Diese Lizenz und die durch sie erteilte Nutzungsbewilligung
      erlöschen mit Wirkung für die Zukunft im Falle eines Verstoßes gegen
      die Lizenzbedingungen durch Sie, ohne dass es dazu der Kenntnis des
      Lizenzgebers vom Verstoß oder einer weiteren Handlung einer der
      Vertragsparteien bedarf. Mit natürlichen oder juristischen Personen, die
      Bearbeitungen des Schutzgegenstandes oder diesen enthaltende Sammelwerke
      sowie entsprechende Vervielfältigungsstücke unter den Bedingungen dieser
      Lizenz von Ihnen erhalten haben, bestehen nachträglich entstandene
      Lizenzbeziehungen jedoch solange weiter, wie die genannten Personen sich
      ihrerseits an sämtliche Lizenzbedingungen halten. Darüber hinaus gelten
      die Ziffern 1, 2, 5, 6, 7, und 8 auch nach einem Erlöschen dieser Lizenz
      fort.

      __1__ Vorbehaltlich der oben genannten Bedingungen gilt diese Lizenz
      unbefristet bis der rechtliche Schutz für den Schutzgegenstand ausläuft.
      Davon abgesehen behält der Lizenzgeber das Recht, den Schutzgegenstand
      unter anderen Lizenzbedingungen anzubieten oder die eigene Weitergabe des
      Schutzgegenstandes jederzeit einzustellen, solange die Ausübung dieses
      Rechts nicht einer Kündigung oder einem Widerruf dieser Lizenz (oder
      irgendeiner Weiterlizenzierung, die auf Grundlage dieser Lizenz bereits
      erfolgt ist bzw. zukünftig noch erfolgen muss) dient und diese Lizenz
      unter Berücksichtigung der oben zum Erlöschen genannten Bedingungen
      vollumfänglich wirksam bleibt.

   __1__ Sonstige Bestimmungen

      __1__ Jedes Mal wenn Sie den Schutzgegenstand für sich genommen oder als
      Teil eines Sammelwerkes verbreiten oder öffentlich wiedergeben, bietet
      der Lizenzgeber dem Empfänger eine Lizenz zu den gleichen Bedingungen und
      im gleichen Umfang an, wie Ihnen in Form dieser Lizenz.

      __1__ Jedes Mal wenn Sie eine Bearbeitung des Schutzgegenstandes
      verbreiten oder öffentlich wiedergeben, bietet der Lizenzgeber dem
      Empfänger eine Lizenz am ursprünglichen Schutzgegenstand zu den gleichen
      Bedingungen und im gleichen Umfang an, wie Ihnen in Form dieser Lizenz.

      __1__ Sollte eine Bestimmung dieser Lizenz unwirksam sein, so bleibt davon
      die Wirksamkeit der Lizenz im Übrigen unberührt.

      __1__ Keine Bestimmung dieser Lizenz soll als abbedungen und kein Verstoß
      gegen sie als zulässig gelten, solange die von dem Verzicht oder von dem
      Verstoß betroffene Seite nicht schriftlich zugestimmt hat.

      __1__ Diese Lizenz (zusammen mit in ihr ausdrücklich vorgesehenen
      Erlaubnissen, Mitteilungen und Zustimmungen, soweit diese tatsächlich
      vorliegen) stellt die vollständige Vereinbarung zwischen dem Lizenzgeber
      und Ihnen in Bezug auf den Schutzgegenstand dar. Es bestehen keine
      Abreden, Vereinbarungen oder Erklärungen in Bezug auf den
      Schutzgegenstand, die in dieser Lizenz nicht genannt sind.
      Rechtsgeschäftliche Änderungen des Verhältnisses zwischen dem
      Lizenzgeber und Ihnen sind nur über Modifikationen dieser Lizenz
      möglich. Der Lizenzgeber ist an etwaige zusätzliche, einseitig durch Sie
      übermittelte Bestimmungen nicht gebunden. Diese Lizenz kann nur durch
      schriftliche Vereinbarung zwischen Ihnen und dem Lizenzgeber modifiziert
      werden. Derlei Modifikationen wirken ausschließlich zwischen dem
      Lizenzgeber und Ihnen und wirken sich nicht auf die Dritten gemäß 8.a)
      und b) angebotenen Lizenzen aus.

      __1__ Sofern zwischen Ihnen und dem Lizenzgeber keine anderweitige
      Vereinbarung getroffen wurde und soweit Wahlfreiheit besteht, findet auf
      diesen Lizenzvertrag das Recht der Republik Österreich Anwendung.

Creative Commons Notice

Creative Commons ist nicht Partei dieser Lizenz und übernimmt keinerlei Gewähr
oder dergleichen in Bezug auf den Schutzgegenstand. Creative Commons haftet
Ihnen oder einer anderen Partei unter keinem rechtlichen Gesichtspunkt für
irgendwelche Schäden, die - abstrakt oder konkret, zufällig oder vorhersehbar
- im Zusammenhang mit dieser Lizenz entstehen. Unbeschadet der vorangegangen
beiden Sätze, hat Creative Commons alle Rechte und Pflichten eines
Lizenzgebers, wenn es sich ausdrücklich als Lizenzgeber im Sinne dieser Lizenz
bezeichnet.

Creative Commons gewährt den Parteien nur insoweit das Recht, das Logo und die
Marke "Creative Commons" zu nutzen, als dies notwendig ist, um der
Öffentlichkeit gegenüber kenntlich zu machen, dass der Schutzgegenstand unter
einer CCPL steht. Ein darüber hinaus gehender Gebrauch der Marke "Creative
Commons" oder einer verwandten Marke oder eines verwandten Logos bedarf der
vorherigen schriftlichen Zustimmung von Creative Commons. Jeder erlaubte
Gebrauch richtet sich nach der Creative Commons Marken-Nutzungs-Richtlinie in
der jeweils aktuellen Fassung, die von Zeit zu Zeit auf der Website
veröffentlicht oder auf andere Weise auf Anfrage zugänglich gemacht wird. Zur
Klarstellung: Die genannten Einschränkungen der Markennutzung sind nicht
Bestandteil dieser Lizenz.

Creative Commons kann kontaktiert werden über https:/creativecommons.org/.
`
const license_CC_BY_SA_4_0_lre = `//**
CC-BY-SA-4.0
https://creativecommons.org/licenses/by-sa/4.0
**//

((Creative Commons))??

((
Creative Commons Legal Code
))??
((https://creativecommons.org/licenses/by-sa/4.0
((/legalcode))??
))??

((
Attribution-ShareAlike 4.0 International
))??
((https://creativecommons.org/licenses/by-sa/4.0
((/legalcode))??
))??

((
Official translations of this license are available in
other languages.
))??

((
Creative Commons Corporation (“Creative Commons”) is not a law firm and does
not provide legal services or legal advice. Distribution of Creative Commons
public licenses does not create a lawyer-client or other relationship. Creative
Commons makes its licenses and related information available on an “as-is”
basis. Creative Commons gives no warranties regarding its licenses, any material
licensed under their terms and conditions, or any related information. Creative
Commons disclaims all liability for damages resulting from their use to the
fullest extent possible.
))??

((
Using Creative Commons Public Licenses

Creative Commons public licenses provide a standard set of terms and conditions
that creators and other rights holders may use to share original works of
authorship and other material subject to copyright and certain other rights
specified in the public license below. The following considerations are for
informational purposes only, are not exhaustive, and do not form part of our
licenses.

Considerations for licensors: Our public licenses are intended for use by those
authorized to give the public permission to use material in ways otherwise
restricted by copyright and certain other rights. Our licenses are irrevocable.
Licensors should read and understand the terms and conditions of the license
they choose before applying it. Licensors should also secure all rights
necessary before applying our licenses so that the public can reuse the material
as expected. Licensors should clearly mark any material not subject to the
license. This includes other CC-licensed material, or material used under an
exception or limitation to copyright. More considerations for licensors.

Considerations for the public: By using one of our public licenses, a licensor
grants the public permission to use the licensed material under specified terms
and conditions. If the licensor’s permission is not necessary for any
reason–for example, because of any applicable exception or limitation to
copyright–then that use is not regulated by the license. Our licenses grant
only permissions under copyright and certain other rights that a licensor has
authority to grant. Use of the licensed material may still be restricted for
other reasons, including because others have copyright or other rights in the
material. A licensor may make special requests, such as asking that all changes
be marked or described. Although not required by our licenses, you are
encouraged to respect those requests where reasonable.
More considerations for the public.
))??

((
Creative Commons Attribution-ShareAlike 4.0 International Public License
))??
((https://creativecommons.org/licenses/by-sa/4.0
((/legalcode))??
))??

By exercising the Licensed Rights (defined below), You accept and agree to be
bound by the terms and conditions of this Creative Commons
Attribution-ShareAlike 4.0 International Public License ("Public License"). To
the extent this Public License may be interpreted as a contract, You are granted
the Licensed Rights in consideration of Your acceptance of these terms and
conditions, and the Licensor grants You such rights in consideration of benefits
the Licensor receives from making the Licensed Material available under these
terms and conditions.

Section 1 – Definitions.

__1__ Adapted Material means material subject to Copyright and Similar Rights
that is derived from or based upon the Licensed Material and in which the
Licensed Material is translated, altered, arranged, transformed, or otherwise
modified in a manner requiring permission under the Copyright and Similar Rights
held by the Licensor. For purposes of this Public License, where the Licensed
Material is a musical work, performance, or sound recording, Adapted Material is
always produced where the Licensed Material is synched in timed relation with a
moving image.

__1__ Adapter's License means the license You apply to Your Copyright and
Similar Rights in Your contributions to Adapted Material in accordance with the
terms and conditions of this Public License.

__1__ BY-SA Compatible License means a license listed at
creativecommons.org/compatiblelicenses, approved by Creative Commons as
essentially the equivalent of this Public License.

__1__ Copyright and Similar Rights means copyright and/or similar rights closely
related to copyright including, without limitation, performance, broadcast,
sound recording, and Sui Generis Database Rights, without regard to how the
rights are labeled or categorized. For purposes of this Public License, the
rights specified in Section 2(b)(1)-(2) are not Copyright and Similar Rights.

__1__ Effective Technological Measures means those measures that, in the absence
of proper authority, may not
be circumvented under laws fulfilling obligations under Article 11 of the WIPO
Copyright Treaty adopted on December 20, 1996, and/or similar
international agreements.

__1__ Exceptions and Limitations means fair use, fair dealing, and/or any other
exception or limitation to Copyright and Similar Rights that applies to Your use
of the Licensed Material.

__1__ License Elements means the license attributes listed in the name of a
Creative Commons Public License. The License Elements of this Public License are
Attribution and ShareAlike.

__1__ Licensed Material means the artistic or literary work, database, or other
material to which the Licensor applied this Public License.

__1__ Licensed Rights means the rights granted to You subject to the terms and
conditions of this Public License, which are limited to all Copyright and
Similar Rights that apply to Your use of the Licensed Material and that the
Licensor has authority to license.

__1__ Licensor means the individual(s) or entity(ies) granting rights under this
Public License.

__1__ Share means to provide material to the public by any means or process that
requires permission under the Licensed Rights, such as reproduction, public
display, public performance, distribution, dissemination, communication, or
importation, and to make material available to the public including in ways that
members of the public may access the material from a place and at a time
individually chosen by them.

__1__ Sui Generis Database Rights means rights other than copyright resulting
from Directive 96/9/EC of the European Parliament and of the Council of 11 March
1996 on the legal protection of databases, as amended and/or succeeded, as well
as other essentially equivalent rights anywhere in the world.

__1__ You means the individual or entity exercising the Licensed Rights under
this Public License. Your has a corresponding meaning.

Section 2 – Scope.

__1__ License grant.

__1__ Subject to the terms and conditions of this Public License, the Licensor
hereby grants You a worldwide, royalty-free, non-sublicensable, non-exclusive,
irrevocable license to exercise the Licensed Rights in the Licensed Material
to:

__1__ reproduce and Share the Licensed Material, in whole or in part; and

__1__ produce, reproduce, and Share Adapted Material.

__1__ Exceptions and Limitations. For the avoidance of doubt, where Exceptions
and Limitations apply to Your use, this Public License does not apply, and You
do not need to comply with its terms and conditions.

__1__ Term. The term of this Public License is specified in Section 6(a).

__1__ Media and formats; technical modifications allowed. The Licensor
authorizes You to exercise the Licensed Rights in all media and formats whether
now known or hereafter created, and to make technical modifications necessary to
do so. The Licensor waives and/or agrees not to assert any right or authority to
forbid You from making technical modifications necessary to exercise the
Licensed Rights, including technical modifications necessary to circumvent
Effective Technological Measures. For purposes of this Public License, simply
making modifications authorized by this Section 2(a)(4) never produces Adapted
Material.

__1__ Downstream recipients.

__1__ Offer from the Licensor – Licensed Material. Every recipient of the
Licensed Material automatically receives an  offer from the Licensor to exercise
the Licensed Rights under the terms and conditions of this Public License.

__1__ Additional offer from the Licensor – Adapted Material. Every recipient
of Adapted Material from You automatically receives an  offer from the Licensor
to exercise the Licensed Rights in the Adapted Material under the conditions of
the Adapter’s License You apply.

__1__ No downstream restrictions. You may not offer or impose any additional or
different terms or  conditions on, or apply any Effective Technological Measures
to, the Licensed Material if doing so restricts exercise of the Licensed Rights
by any recipient of the Licensed Material.

__1__ No endorsement. Nothing in this Public License constitutes or may be
construed as permission to assert or imply that You are, or that Your use of the
Licensed Material is, connected with, or sponsored, endorsed, or granted
official status by, the Licensor or others designated to receive attribution as
provided in Section 3(a)(1)(A)(i).

__1__
Other rights.

__1__ Moral rights, such as the right of integrity, are not licensed under this
Public License, nor are publicity, privacy, and/or other similar personality
rights; however, to the extent possible, the Licensor waives and/or agrees not
to assert any such rights held by the Licensor to the limited extent necessary
to allow You to exercise the Licensed Rights, but not otherwise.

__1__ Patent and trademark rights are not licensed under this Public License.

__1__ To the extent possible, the Licensor waives any right to collect royalties
from You for the exercise of the Licensed Rights, whether directly or through a
collecting society under any voluntary or waivable statutory or compulsory
licensing scheme. In all other cases the Licensor expressly reserves any right
to collect such royalties.

Section 3 – License Conditions.

Your exercise of the Licensed Rights is expressly made subject to the following
conditions.

__1__
Attribution.

__1__
If You Share the Licensed Material (including in modified form), You must:

__1__ retain the following if it is supplied by the Licensor with the Licensed
Material:

__1__ identification of the creator(s) of the Licensed Material and any others
designated to receive attribution, in any reasonable manner requested  by the
Licensor (including by pseudonym if designated);

__1__ a copyright notice;

__1__ a notice that refers to this Public License;

__1__ a notice that refers to the disclaimer of warranties;

__1__ a URI or hyperlink to the Licensed Material to the extent reasonably
practicable;

__1__ indicate if You modified the Licensed Material and retain an indication of
any previous modifications; and

__1__ indicate the Licensed Material is licensed under this Public License,
and include the text of, or the URI or hyperlink to, this Public
License.

__1__ You may satisfy the conditions in Section 3(a)(1) in any reasonable manner
based on the medium, means, and context in which You Share the Licensed
Material. For example, it may be reasonable to satisfy the conditions by
providing a URI or hyperlink to a resource that includes the required
information.

__1__ If requested by the Licensor, You must remove any of the information
required by Section 3(a)(1)(A) to the extent reasonably practicable.

__1__ ShareAlike.

In addition to the conditions in Section 3(a), if You Share Adapted Material You
produce, the following conditions also apply.

__1__ The Adapter’s License You apply must be a Creative Commons license with
the same License Elements, this version or later, or a BY-SA Compatible License.

__1__ You must include the text of, or the URI or hyperlink to, the Adapter's
License You apply. You may satisfy this condition in any reasonable manner based
on the medium, means, and context in which You Share Adapted Material.

__1__ You may not offer or impose any additional or different terms or
conditions on, or apply any Effective Technological Measures to, Adapted
Material that restrict exercise of the rights granted under the Adapter's
License You apply.

Section 4 – Sui Generis Database Rights.

Where the Licensed Rights include Sui Generis Database Rights that apply to Your
use of the Licensed Material:

__1__ for the avoidance of doubt, Section 2(a)(1) grants You the right to
extract, reuse, reproduce, and Share all or a substantial portion of the
contents of the database;

__1__ if You include all or a substantial portion of the database contents in a
database in which You have Sui Generis Database Rights, then the database in
which You have Sui Generis Database Rights (but not its individual contents) is
Adapted Material, including for purposes of Section 3(b); and

__1__ You must comply with the conditions in Section 3(a) if You Share all or a
substantial portion of the contents of the database.

__1__ For the avoidance of doubt, this Section 4 supplements and
does not replace Your obligations under this Public License
where the Licensed Rights include other Copyright and Similar
Rights.

Section 5 – Disclaimer of Warranties and Limitation of Liability.

__1__ Unless otherwise separately undertaken by the Licensor, to the extent
possible, the Licensor offers the Licensed Material as-is and as-available, and
makes no representations or warranties of any kind concerning the Licensed
Material, whether express, implied, statutory, or other. This includes, without
limitation, warranties of title, merchantability, fitness for a particular
purpose, non-infringement, absence of latent or other defects, accuracy, or the
presence or absence of errors, whether or not known or discoverable. Where
disclaimers of warranties are not allowed in full or in part, this disclaimer
may not apply to You.

__1__ To the extent possible, in no event will the Licensor be liable to You on
any legal theory (including, without limitation, negligence) or otherwise for
any direct, special, indirect, incidental, consequential, punitive, exemplary,
or other losses, costs, expenses, or damages arising out of this Public License
or use of the Licensed Material, even if the Licensor has been advised of the
possibility of such losses, costs, expenses, or damages. Where a limitation of
liability is not allowed in full or in part, this limitation may not apply to
You.

__1__ The disclaimer of warranties and limitation of liability provided above
shall be interpreted in a manner that, to the extent possible, most closely
approximates an absolute disclaimer and waiver of all liability.

Section 6 – Term and Termination.

__1__ This Public License applies for the term of the Copyright and Similar
Rights licensed here. However, if You fail to comply with this Public License,
then Your rights under this Public License terminate automatically.

__1__

Where Your right to use the Licensed Material has terminated under Section 6(a),
it reinstates:

__1__ automatically as of the date the violation is cured, provided it is cured
within 30 days of Your discovery of the violation; or

__1__ upon express reinstatement by the Licensor.

__1__ For the avoidance of doubt, this Section 6(b) does not
affect any right the Licensor may have to seek remedies for
Your violations of this Public License.

__1__ For the avoidance of doubt, the Licensor may also offer the Licensed
Material under separate terms or conditions or stop distributing the Licensed
Material at any time; however, doing so will not terminate this Public License.

__1__ Sections 1, 5, 6, 7, and 8 survive termination of this Public License.

Section 7 – Other Terms and Conditions.

__1__ The Licensor shall not be bound by any additional or different terms or
conditions communicated by You unless expressly agreed.

__1__ Any arrangements, understandings, or agreements regarding the Licensed
Material not stated herein are separate from and independent of the terms and
conditions of this Public License.

Section 8 – Interpretation.

__1__ For the avoidance of doubt, this Public License does not, and shall not be
interpreted to, reduce, limit, restrict, or impose conditions on any use of the
Licensed Material that could lawfully be made without permission under this
Public License.

__1__ To the extent possible, if any provision of this Public License is deemed
unenforceable, it shall be automatically reformed to the minimum extent
necessary to make it enforceable. If the provision cannot be reformed, it shall
be severed from this Public License without affecting the enforceability of the
remaining terms and conditions.

__1__ No term or condition of this Public License will be waived and no failure
to comply consented to unless expressly agreed to by the Licensor.

__1__ Nothing in this Public License constitutes or may be interpreted as a
limitation upon, or waiver of, any privileges and immunities that apply to the
Licensor or You, including from the legal processes of any jurisdiction or
authority.

((
Creative Commons is not a party to its public licenses. Notwithstanding,
Creative Commons may elect to apply one of its public licenses to material it
publishes and in those instances will be considered the “Licensor.” The text
of the Creative Commons public licenses is dedicated to the public domain under
the CC0 Public Domain Dedication. Except for the limited purpose of indicating
that material is shared under a Creative Commons public license or as otherwise
permitted by the Creative Commons policies published at
creativecommons.org/policies, Creative Commons does not authorize the use of the
trademark “Creative Commons” or any other trademark or logo of Creative
Commons without its prior written consent including, without limitation, in
connection with any unauthorized modifications to any of its public licenses or
any other arrangements, understandings, or agreements concerning use of licensed
material. For the avoidance of doubt, this paragraph does not form part of the
public licenses.
Creative Commons may be contacted at creativecommons.org.
))??

((
Additional languages available:

العربية,
čeština,
Deutsch,
Ελληνικά,
Español,
euskara,
suomeksi,
français,
hrvatski,
Bahasa Indonesia,
italiano,
日本語,
한국어,
Lietuvių,
latviski,
te reo Māori,
Nederlands,
norsk,
polski,
português,
română,
русский,
Slovenščina,
svenska,
Türkçe,
українська,
中文,
華語.

Please read the FAQ for more information about official translations.
))??
`
const license_CC_PDDC_lre = `//**
Creative Commons Public Domain Dedication and Certification
https://spdx.org/licenses/CC-PDDC.json
https://creativecommons.org/licenses/publicdomain/
**//

The person or persons who have associated work with this document (the
"Dedicator" or "Certifier") hereby either (a) certifies that, to the best of his
knowledge, the work of authorship identified is in the public domain of the
country from which the work is published, or (b) hereby dedicates whatever
copyright the dedicators holds in the work of authorship identified below (the
"Work") to the public domain. A certifier, moreover, dedicates any copyright
interest he may have in the associated work, and for these purposes, is
described as a "dedicator" below.

A certifier has taken reasonable steps to verify the copyright status of this
work. Certifier recognizes that his good faith efforts may not shield him from
liability if in fact the work certified is not in the public domain.

Dedicator makes this dedication for the benefit of the public at large and to
the detriment of the Dedicator's heirs and successors. Dedicator intends this
dedication to be an overt act of relinquishment in perpetuity of all present and
future rights under copyright law, whether vested or contingent, in the Work.
Dedicator understands that such relinquishment of all rights includes the
relinquishment of all rights to enforce (by lawsuit or otherwise) those
copyrights in the Work.

Dedicator recognizes that, once placed in the public domain, the Work may be
freely reproduced, distributed, transmitted, used, modified, built upon, or
otherwise exploited by anyone for any purpose, commercial or non-commercial, and
in any way, including by methods that have not yet been invented or conceived.
`
const license_CC0_1_0_lre = `//**
Creative Commons Zero v1.0 Universal
https://spdx.org/licenses/CC0-1.0.json
https://creativecommons.org/publicdomain/zero/1.0/legalcode
**//

((
((  Creative Commons
((  Legal Code ))??
))??

CC0 1.0 Universal ))??

((  CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND
DOES NOT PROVIDE LEGAL SERVICES. DISTRIBUTION OF THIS DOCUMENT DOES NOT CREATE
AN ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS INFORMATION ON
AN "AS-IS" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES REGARDING THE USE OF THIS
DOCUMENT OR THE INFORMATION OR WORKS PROVIDED HEREUNDER, AND DISCLAIMS LIABILITY
FOR DAMAGES RESULTING FROM THE USE OF THIS DOCUMENT OR THE INFORMATION OR WORKS
PROVIDED HEREUNDER. ))??

((Statement of Purpose))??

The laws of most jurisdictions throughout the world automatically confer
exclusive Copyright and Related Rights (defined below) upon the creator and
subsequent owner(s) (each and all, an "owner") of an original work of authorship
and/or a database (each, a "Work").

Certain owners wish to permanently relinquish those rights to a Work for the
purpose of contributing to a commons of creative, cultural and scientific works
("Commons") that the public can reliably and without fear of later claims of
infringement build upon, modify, incorporate in other works, reuse and
redistribute as freely as possible in any form whatsoever and for any purposes,
including without limitation commercial purposes. These owners may contribute to
the Commons to promote the ideal of a free culture and the further production of
creative, cultural and scientific works, or to gain reputation or greater
distribution for their Work in part through the use and efforts of others.

For these and/or other purposes and motivations, and without any expectation of
additional consideration or compensation, the person associating CC0 with a Work
(the "Affirmer"), to the extent that he or she is an owner of Copyright and
Related Rights in the Work, voluntarily elects to apply CC0 to the Work and
publicly distribute the Work under its terms, with knowledge of his or her
Copyright and Related Rights in the Work and the meaning and intended legal
effect of CC0 on those rights.

   __1__ Copyright and Related Rights. A Work made available under CC0 may be
   protected by copyright and related or neighboring rights ("Copyright and
   Related Rights"). Copyright and Related Rights include, but are not limited
   to, the following:

      __1__ the right to reproduce, adapt, distribute, perform, display,
      communicate, and translate a Work;

      __1__ moral rights retained by the original author(s) and/or
      performer(s);

      __1__ publicity and privacy rights pertaining to a person's image or
      likeness depicted in a Work;

      __1__ rights protecting against unfair competition in regards to a Work,
      subject to the limitations in paragraph __2__, below;

      __1__ rights protecting the extraction, dissemination, use and reuse of
      data in a Work;

      __1__ database rights (such as those arising under Directive 96/9/EC of
      the European Parliament and of the Council of 11 March 1996 on the legal
      protection of databases, and under any national implementation thereof,
      including any amended or successor version of such directive); and

      __1__ other similar, equivalent or corresponding rights throughout the
      world based on applicable law or treaty, and any national implementations
      thereof.

   __1__ Waiver. To the greatest extent permitted by, but not in contravention
   of, applicable law, Affirmer hereby overtly, fully, permanently, irrevocably
   and unconditionally waives, abandons, and surrenders all of Affirmer's
   Copyright and Related Rights and associated claims and causes of action,
   whether now known or unknown (including existing as well as future claims and
   causes of action), in the Work (i) in all territories worldwide, (ii) for the
   maximum duration provided by applicable law or treaty (including future time
   extensions), (iii) in any current or future medium and for any number of
   copies, and (iv) for any purpose whatsoever, including without limitation
   commercial, advertising or promotional purposes (the "Waiver"). Affirmer
   makes the Waiver for the benefit of each member of the public at large and to
   the detriment of Affirmer's heirs and successors, fully intending that such
   Waiver shall not be subject to revocation, rescission, cancellation,
   termination, or any other legal or equitable action to disrupt the quiet
   enjoyment of the Work by the public as contemplated by Affirmer's express
   Statement of Purpose.

   __1__ Public License Fallback. Should any part of the Waiver for any reason
   be judged legally invalid or ineffective under applicable law, then the
   Waiver shall be preserved to the maximum extent permitted taking into account
   Affirmer's express Statement of Purpose. In addition, to the extent the
   Waiver is so judged Affirmer hereby grants to each affected person a
   royalty-free, non transferable, non sublicensable, non exclusive, irrevocable
   and unconditional license to exercise Affirmer's Copyright and Related Rights
   in the Work (i) in all territories worldwide, (ii) for the maximum duration
   provided by applicable law or treaty (including future time extensions),
   (iii) in any current or future medium and for any number of copies, and (iv)
   for any purpose whatsoever, including without limitation commercial,
   advertising or promotional purposes (the "License"). The License shall be
   deemed effective as of the date CC0 was applied by Affirmer to the Work.
   Should any part of the License for any reason be judged legally invalid or
   ineffective under applicable law, such partial invalidity or ineffectiveness
   shall not invalidate the remainder of the License, and in such case Affirmer
   hereby affirms that he or she will not (i) exercise any of his or her
   remaining Copyright and Related Rights in the Work or (ii) assert any
   associated claims and causes of action with respect to the Work, in either
   case contrary to Affirmer's express Statement of Purpose.

   __1__ Limitations and Disclaimers.

      __1__ No trademark or patent rights held by Affirmer are waived,
      abandoned, surrendered, licensed or otherwise affected by this document.

      __1__ Affirmer offers the Work as-is and makes no representations or
      warranties of any kind concerning the Work, express, implied, statutory or
      otherwise, including without limitation warranties of title,
      merchantability, fitness for a particular purpose, non infringement, or
      the absence of latent or other defects, accuracy, or the present or
      absence of errors, whether or not discoverable, all to the greatest extent
      permissible under applicable law.

      __1__ Affirmer disclaims responsibility for clearing rights of other
      persons that may apply to the Work or any use thereof, including without
      limitation any person's Copyright and Related Rights in the Work. Further,
      Affirmer disclaims responsibility for obtaining any necessary consents,
      permissions or other rights required for any use of the Work.

      __1__ Affirmer understands and acknowledges that Creative Commons is not a
      party to this document and has no duty or obligation with respect to this
      CC0 or use of the Work.
`
const license_CDDL_1_0_lre = `//**
Common Development and Distribution License 1.0
https://spdx.org/licenses/CDDL-1.0.json
https://opensource.org/licenses/cddl1
**//

((

The accompanying software is licensed under the Common Development and
Distribution License, Version 1.0 (CDDL-1.0, the "License"); you may not use
any part of this software except in compliance with the License.

You may obtain a copy of the License at
	http://opensource.org/licenses/CDDL-1.0
More information about the CDDL can be found at
	http://hub.opensolaris.org/bin/view/Main/licensing_faq

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
License for the specific language governing permissions and limitations under
the License.

||

(( COMMON DEVELOPMENT AND DISTRIBUTION LICENSE (CDDL)

Version 1.0 ))??

   (( 1. ))??
   Definitions.

      (( 1.1. ))??
      "Contributor" means each individual or entity that creates or contributes
      to the creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Software,
      prior Modifications used by a Contributor (if any), and the Modifications
      made by that particular Contributor.

      (( 1.3. ))??
      "Covered Software" means (a) the Original Software, or (b) Modifications,
      or (c) the combination of files containing Original Software with files
      containing Modifications, in each case including portions thereof.

      (( 1.4. ))??
      "Executable" means the Covered Software in any form other than Source
      Code.

      (( 1.5. ))??
      "Initial Developer" means the individual or entity that first makes
      Original Software available under this License.

      (( 1.6. ))??
      "Larger Work" means a work which combines Covered Software or portions
      thereof with code not governed by the terms of this License.

      (( 1.7. ))??
      "License" means this document.

      (( 1.8. ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.9. ))??
      "Modifications" means the Source Code and Executable form of any of the
      following:

         (( A. ))??
         Any file that results from an addition to, deletion from or
         modification of the contents of a file containing Original Software or
         previous Modifications;

         (( B. ))??
         Any new file that contains any part of the Original Software or
         previous Modification; or

         (( C. ))??
         Any new file that is contributed or otherwise made available under the
         terms of this License.

      (( 1.10. ))??
      "Original Software" means the Source Code and Executable form of computer
      software code that is originally released under this License.

      (( 1.11. ))??
      "Patent Claims" means any patent claim(s), now owned or hereafter
      acquired, including without limitation, method, process, and apparatus
      claims, in any patent Licensable by grantor.

      (( 1.12. ))??
      "Source Code" means (a) the common form of computer software code in which
      modifications are made and (b) associated documentation included in or
      with such code.

      (( 1.13. ))??
      "You" (or "Your") means an individual or a legal entity exercising rights
      under, and complying with all of the terms of, this License. For legal
      entities, "You" includes any entity which controls, is controlled by, or
      is under common control with You. For purposes of this definition,
      "control" means (a) the power, direct or indirect, to cause the direction
      or management of such entity, whether by contract or otherwise, or (b)
      ownership of more than fifty percent (50%) of the outstanding shares or
      beneficial ownership of such entity.

   (( 2. ))??
   License Grants.

      (( 2.1. ))??
      The Initial Developer Grant.

      Conditioned upon Your compliance with Section 3.1 below and subject to
      third party intellectual property claims, the Initial Developer hereby
      grants You a world-wide, royalty-free, non-exclusive license:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Initial Developer, to use, reproduce, modify, display,
         perform, sublicense and distribute the Original Software (or portions
         thereof), with or without Modifications, and/or as part of a Larger
         Work; and

         (( (b) ))??
         under Patent Claims infringed by the making, using or selling of
         Original Software, to make, have made, use, practice, sell, and offer
         for sale, and/or otherwise dispose of the Original Software (or
         portions thereof).

         (( c ))??
         The licenses granted in Sections 2.1(a) and (b) are effective on the
         date Initial Developer first distributes or otherwise makes the
         Original Software available to a third party under the terms of this
         License.

         (( (d) ))??
         Notwithstanding Section 2.1(b) above, no patent license is granted: (1)
         for code that You delete from the Original Software, or (2) for
         infringements caused by: (i) the modification of the Original Software,
         or (ii) the combination of the Original Software with other software or
         devices.

      (( 2.2. ))??
      Contributor Grant.

      Conditioned upon Your compliance with Section 3.1 below and subject to
      third party intellectual property claims, each Contributor hereby grants
      You a world-wide, royalty-free, non-exclusive license:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Contributor to use, reproduce, modify, display, perform,
         sublicense and distribute the Modifications created by such Contributor
         (or portions thereof), either on an unmodified basis, with other
         Modifications, as Covered Software and/or as part of a Larger Work;
         and

         (( (b) ))??
         under Patent Claims infringed by the making, using, or selling of
         Modifications made by that Contributor either alone and/or in
         combination with its Contributor Version (or portions of such
         combination), to make, use, sell, offer for sale, have made, and/or
         otherwise dispose of: (1) Modifications made by that Contributor (or
         portions thereof); and (2) the combination of Modifications made by
         that Contributor with its Contributor Version (or portions of such
         combination).

         (( c ))??
         The licenses granted in Sections 2.2(a) and 2.2(b) are effective on the
         date Contributor first distributes or otherwise makes the Modifications
         available to a third party.

         (( (d) ))??
         Notwithstanding Section 2.2(b) above, no patent license is granted: (1)
         for any code that Contributor has deleted from the Contributor Version;
         (2) for infringements caused by: (i) third party modifications of
         Contributor Version, or (ii) the combination of Modifications made by
         that Contributor with other software (except as part of the Contributor
         Version) or other devices; or (3) under Patent Claims infringed by
         Covered Software in the absence of Modifications made by that
         Contributor.

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Availability of Source Code.

      Any Covered Software that You distribute or otherwise make available in
      Executable form must also be made available in Source Code form and that
      Source Code form must be distributed only under the terms of this License.
      You must include a copy of this License with every copy of the Source Code
      form of the Covered Software You distribute or otherwise make available.
      You must inform recipients of any such Covered Software in Executable form
      as to how they can obtain such Covered Software in Source Code form in a
      reasonable manner on or through a medium customarily used for software
      exchange.

      (( 3.2. ))??
      Modifications.

      The Modifications that You create or to which You contribute are governed
      by the terms of this License. You represent that You believe Your
      Modifications are Your original creation(s) and/or You have sufficient
      rights to grant the rights conveyed by this License.

      (( 3.3. ))??
      Required Notices.

      You must include a notice in each of Your Modifications that identifies
      You as the Contributor of the Modification. You may not remove or alter
      any copyright, patent or trademark notices contained within the Covered
      Software, or any notices of licensing or any descriptive text giving
      attribution to any Contributor or the Initial Developer.

      (( 3.4. ))??
      Application of Additional Terms.

      You may not offer or impose any terms on any Covered Software in Source
      Code form that alters or restricts the applicable version of this License
      or the recipients' rights hereunder. You may choose to offer, and to
      charge a fee for, warranty, support, indemnity or liability obligations to
      one or more recipients of Covered Software. However, you may do so only on
      Your own behalf, and not on behalf of the Initial Developer or any
      Contributor. You must make it absolutely clear that any such warranty,
      support, indemnity or liability obligation is offered by You alone, and
      You hereby agree to indemnify the Initial Developer and every Contributor
      for any liability incurred by the Initial Developer or such Contributor as
      a result of warranty, support, indemnity or liability terms You offer.

      (( 3.5. ))??
      Distribution of Executable Versions.

      You may distribute the Executable form of the Covered Software under the
      terms of this License or under the terms of a license of Your choice,
      which may contain terms different from this License, provided that You are
      in compliance with the terms of this License and that the license for the
      Executable form does not attempt to limit or alter the recipient's rights
      in the Source Code form from the rights set forth in this License. If You
      distribute the Covered Software in Executable form under a different
      license, You must make it absolutely clear that any terms which differ
      from this License are offered by You alone, not by the Initial Developer
      or Contributor. You hereby agree to indemnify the Initial Developer and
      every Contributor for any liability incurred by the Initial Developer or
      such Contributor as a result of any such terms You offer.

      (( 3.6. ))??
      Larger Works.

      You may create a Larger Work by combining Covered Software with other code
      not governed by the terms of this License and distribute the Larger Work
      as a single product. In such a case, You must make sure the requirements
      of this License are fulfilled for the Covered Software.

   (( 4. ))??
   Versions of the License.

      (( 4.1. ))??
      New Versions.

      Sun Microsystems, Inc. is the initial license steward and may publish
      revised and/or new versions of this License from time to time. Each
      version will be given a distinguishing version number. Except as provided
      in Section 4.3, no one other than the license steward has the right to
      modify this License.

      (( 4.2. ))??
      Effect of New Versions.

      You may always continue to use, distribute or otherwise make the Covered
      Software available under the terms of the version of the License under
      which You originally received the Covered Software. If the Initial
      Developer includes a notice in the Original Software prohibiting it from
      being distributed or otherwise made available under any subsequent version
      of the License, You must distribute and make the Covered Software
      available under the terms of the version of the License under which You
      originally received the Covered Software. Otherwise, You may also choose
      to use, distribute or otherwise make the Covered Software available under
      the terms of any subsequent version of the License published by the
      license steward.

      (( 4.3. ))??
      Modified Versions.

      When You are an Initial Developer and You want to create a new license for
      Your Original Software, You may create and use a modified version of this
      License if You: (a) rename the license and remove any references to the
      name of the license steward (except to note that the license differs from
      this License); and (b) otherwise make it clear that the license contains
      terms which differ from this License.

   (( 5. ))??
   DISCLAIMER OF WARRANTY.

   COVERED SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED SOFTWARE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED SOFTWARE IS WITH YOU. SHOULD
   ANY COVERED SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL
   DEVELOPER OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY
   SERVICING, REPAIR OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN
   ESSENTIAL PART OF THIS LICENSE. NO USE OF ANY COVERED SOFTWARE IS AUTHORIZED
   HEREUNDER EXCEPT UNDER THIS DISCLAIMER.

   (( 6. ))??
   TERMINATION.

      (( 6.1. ))??
      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. Provisions which, by their
      nature, must remain in effect beyond the termination of this License shall
      survive.

      (( 6.2. ))??
      If You assert a patent infringement claim (excluding declaratory judgment
      actions) against Initial Developer or a Contributor (the Initial Developer
      or Contributor against whom You assert such claim is referred to as
      "Participant") alleging that the Participant Software (meaning the
      Contributor Version where the Participant is a Contributor or the Original
      Software where the Participant is the Initial Developer) directly or
      indirectly infringes any patent, then any and all rights granted directly
      or indirectly to You by such Participant, the Initial Developer (if the
      Initial Developer is not the Participant) and all Contributors under
      Sections 2.1 and/or 2.2 of this License shall, upon 60 days notice from
      Participant terminate prospectively and automatically at the expiration of
      such 60 day notice period, unless if within such 60 day period You
      withdraw Your claim with respect to the Participant Software against such
      Participant either unilaterally or pursuant to a written agreement with
      Participant.

      (( 6.3. ))??
      In the event of termination under Sections 6.1 or 6.2 above, all end user
      licenses that have been validly granted by You or any distributor
      hereunder prior to termination (excluding licenses granted to You by any
      distributor) shall survive termination.

   (( 7. ))??
   LIMITATION OF LIABILITY.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, THE INITIAL DEVELOPER, ANY
   OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED SOFTWARE, OR ANY SUPPLIER OF
   ANY OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOST PROFITS, LOSS OF GOODWILL, WORK STOPPAGE,
   COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR
   LOSSES, EVEN IF SUCH PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF
   SUCH DAMAGES. THIS LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR
   DEATH OR PERSONAL INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT
   APPLICABLE LAW PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE
   EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS
   EXCLUSION AND LIMITATION MAY NOT APPLY TO YOU.

   (( 8. ))??
   U.S. GOVERNMENT END USERS.

   The Covered Software is a "commercial item," as that term is defined in 48
   C.F.R. 2.101 (Oct. 1995), consisting of "commercial computer software" (as
   that term is defined at 48 C.F.R. __1__ 252.227-7014(a)(1) and "commercial
   computer software documentation" as such terms are used in 48 C.F.R. 12.212
   (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R. 227.7202-1
   through 227.7202-4 (June 1995), all U.S. Government End Users acquire Covered
   Software with only those rights set forth herein. This U.S. Government Rights
   clause is in lieu of, and supersedes, any other FAR, DFAR, or other clause or
   provision that addresses Government rights in computer software under this
   License.

   (( 9. ))??
   MISCELLANEOUS.

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by the law of the jurisdiction
   specified in a notice contained within the Original Software (except to the
   extent applicable law, if any, provides otherwise), excluding such
   jurisdiction's conflict-of-law provisions. Any litigation relating to this
   License shall be subject to the jurisdiction of the courts located in the
   jurisdiction and venue specified in a notice contained within the Original
   Software, with the losing party responsible for costs, including, without
   limitation, court costs and reasonable attorneys' fees and expenses. The
   application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any law or regulation
   which provides that the language of a contract shall be construed against the
   drafter shall not apply to this License. You agree that You alone are
   responsible for compliance with the United States export administration
   regulations (and the export control laws and regulation of any other
   countries) when You use, distribute or otherwise make available any Covered
   Software.

   (( 10. ))??
   RESPONSIBILITY FOR CLAIMS.

   As between Initial Developer and the Contributors, each party is responsible
   for claims and damages arising, directly or indirectly, out of its
   utilization of rights under this License and You agree to work with Initial
   Developer and Contributors to distribute such responsibility on an equitable
   basis. Nothing herein is intended or shall be deemed to constitute any
   admission of liability.

))
`
const license_CDDL_1_1_lre = `//**
Common Development and Distribution License 1.1
https://spdx.org/licenses/CDDL-1.1.json
http://glassfish.java.net/public/CDDL+GPL_1_1.html
https://javaee.github.io/glassfish/LICENSE
**//

(( COMMON DEVELOPMENT AND DISTRIBUTION LICENSE (CDDL)

Version 1.1 ))??

   (( 1. ))??
   Definitions.

      (( 1.1. ))??
      "Contributor" means each individual or entity that creates or contributes
      to the creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Software,
      prior Modifications used by a Contributor (if any), and the Modifications
      made by that particular Contributor.

      (( 1.3. ))??
      "Covered Software" means (a) the Original Software, or (b) Modifications,
      or (c) the combination of files containing Original Software with files
      containing Modifications, in each case including portions thereof.

      (( 1.4. ))??
      "Executable" means the Covered Software in any form other than Source
      Code.

      (( 1.5. ))??
      "Initial Developer" means the individual or entity that first makes
      Original Software available under this License.

      (( 1.6. ))??
      "Larger Work" means a work which combines Covered Software or portions
      thereof with code not governed by the terms of this License.

      (( 1.7. ))??
      "License" means this document.

      (( 1.8. ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.9. ))??
      "Modifications" means the Source Code and Executable form of any of the
      following:

         (( A. ))??
         Any file that results from an addition to, deletion from or
         modification of the contents of a file containing Original Software or
         previous Modifications;

         (( B. ))??
         Any new file that contains any part of the Original Software or
         previous Modification; or

         (( C. ))??
         Any new file that is contributed or otherwise made available under the
         terms of this License.

      (( 1.10. ))??
      "Original Software" means the Source Code and Executable form of computer
      software code that is originally released under this License.

      (( 1.11. ))??
      "Patent Claims" means any patent claim(s), now owned or hereafter
      acquired, including without limitation, method, process, and apparatus
      claims, in any patent Licensable by grantor.

      (( 1.12. ))??
      "Source Code" means (a) the common form of computer software code in which
      modifications are made and (b) associated documentation included in or
      with such code.

      (( 1.13. ))??
      "You" (or "Your") means an individual or a legal entity exercising rights
      under, and complying with all of the terms of, this License. For legal
      entities, "You" includes any entity which controls, is controlled by, or
      is under common control with You. For purposes of this definition,
      "control" means (a) the power, direct or indirect, to cause the direction
      or management of such entity, whether by contract or otherwise, or (b)
      ownership of more than fifty percent (50%) of the outstanding shares or
      beneficial ownership of such entity.

   (( 2. ))??
   License Grants.

      (( 2.1. ))??
      The Initial Developer Grant.

      Conditioned upon Your compliance with Section 3.1 below and subject to
      third party intellectual property claims, the Initial Developer hereby
      grants You a world-wide, royalty-free, non-exclusive license:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Initial Developer, to use, reproduce, modify, display,
         perform, sublicense and distribute the Original Software (or portions
         thereof), with or without Modifications, and/or as part of a Larger
         Work; and

         (( (b) ))??
         under Patent Claims infringed by the making, using or selling of
         Original Software, to make, have made, use, practice, sell, and offer
         for sale, and/or otherwise dispose of the Original Software (or
         portions thereof).

         (( c ))??
         The licenses granted in Sections 2.1(a) and (b) are effective on the
         date Initial Developer first distributes or otherwise makes the
         Original Software available to a third party under the terms of this
         License.

         (( (d) ))??
         Notwithstanding Section 2.1(b) above, no patent license is granted: (1)
         for code that You delete from the Original Software, or (2) for
         infringements caused by: (i) the modification of the Original Software,
         or (ii) the combination of the Original Software with other software or
         devices.

      (( 2.2. ))??
      Contributor Grant.

      Conditioned upon Your compliance with Section 3.1 below and subject to
      third party intellectual property claims, each Contributor hereby grants
      You a world-wide, royalty-free, non-exclusive license:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Contributor to use, reproduce, modify, display, perform,
         sublicense and distribute the Modifications created by such Contributor
         (or portions thereof), either on an unmodified basis, with other
         Modifications, as Covered Software and/or as part of a Larger Work;
         and

         (( (b) ))??
         under Patent Claims infringed by the making, using, or selling of
         Modifications made by that Contributor either alone and/or in
         combination with its Contributor Version (or portions of such
         combination), to make, use, sell, offer for sale, have made, and/or
         otherwise dispose of: (1) Modifications made by that Contributor (or
         portions thereof); and (2) the combination of Modifications made by
         that Contributor with its Contributor Version (or portions of such
         combination).

         (( c ))??
         The licenses granted in Sections 2.2(a) and 2.2(b) are effective on the
         date Contributor first distributes or otherwise makes the Modifications
         available to a third party.

         (( (d) ))??
         Notwithstanding Section 2.2(b) above, no patent license is granted: (1)
         for any code that Contributor has deleted from the Contributor Version;
         (2) for infringements caused by: (i) third party modifications of
         Contributor Version, or (ii) the combination of Modifications made by
         that Contributor with other software (except as part of the Contributor
         Version) or other devices; or (3) under Patent Claims infringed by
         Covered Software in the absence of Modifications made by that
         Contributor.

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Availability of Source Code.

      Any Covered Software that You distribute or otherwise make available in
      Executable form must also be made available in Source Code form and that
      Source Code form must be distributed only under the terms of this License.
      You must include a copy of this License with every copy of the Source Code
      form of the Covered Software You distribute or otherwise make available.
      You must inform recipients of any such Covered Software in Executable form
      as to how they can obtain such Covered Software in Source Code form in a
      reasonable manner on or through a medium customarily used for software
      exchange.

      (( 3.2. ))??
      Modifications.

      The Modifications that You create or to which You contribute are governed
      by the terms of this License. You represent that You believe Your
      Modifications are Your original creation(s) and/or You have sufficient
      rights to grant the rights conveyed by this License.

      (( 3.3. ))??
      Required Notices.

      You must include a notice in each of Your Modifications that identifies
      You as the Contributor of the Modification. You may not remove or alter
      any copyright, patent or trademark notices contained within the Covered
      Software, or any notices of licensing or any descriptive text giving
      attribution to any Contributor or the Initial Developer.

      (( 3.4. ))??
      Application of Additional Terms.

      You may not offer or impose any terms on any Covered Software in Source
      Code form that alters or restricts the applicable version of this License
      or the recipients' rights hereunder. You may choose to offer, and to
      charge a fee for, warranty, support, indemnity or liability obligations to
      one or more recipients of Covered Software. However, you may do so only on
      Your own behalf, and not on behalf of the Initial Developer or any
      Contributor. You must make it absolutely clear that any such warranty,
      support, indemnity or liability obligation is offered by You alone, and
      You hereby agree to indemnify the Initial Developer and every Contributor
      for any liability incurred by the Initial Developer or such Contributor as
      a result of warranty, support, indemnity or liability terms You offer.

      (( 3.5. ))??
      Distribution of Executable Versions.

      You may distribute the Executable form of the Covered Software under the
      terms of this License or under the terms of a license of Your choice,
      which may contain terms different from this License, provided that You are
      in compliance with the terms of this License and that the license for the
      Executable form does not attempt to limit or alter the recipient's rights
      in the Source Code form from the rights set forth in this License. If You
      distribute the Covered Software in Executable form under a different
      license, You must make it absolutely clear that any terms which differ
      from this License are offered by You alone, not by the Initial Developer
      or Contributor. You hereby agree to indemnify the Initial Developer and
      every Contributor for any liability incurred by the Initial Developer or
      such Contributor as a result of any such terms You offer.

      (( 3.6. ))??
      Larger Works.

      You may create a Larger Work by combining Covered Software with other code
      not governed by the terms of this License and distribute the Larger Work
      as a single product. In such a case, You must make sure the requirements
      of this License are fulfilled for the Covered Software.

   (( 4. ))??
   Versions of the License.

      (( 4.1. ))??
      New Versions.

      Oracle is the initial license steward and may publish revised and/or new
      versions of this License from time to time. Each version will be given a
      distinguishing version number. Except as provided in Section 4.3, no one
      other than the license steward has the right to modify this License.

      (( 4.2. ))??
      Effect of New Versions.

      You may always continue to use, distribute or otherwise make the Covered
      Software available under the terms of the version of the License under
      which You originally received the Covered Software. If the Initial
      Developer includes a notice in the Original Software prohibiting it from
      being distributed or otherwise made available under any subsequent version
      of the License, You must distribute and make the Covered Software
      available under the terms of the version of the License under which You
      originally received the Covered Software. Otherwise, You may also choose
      to use, distribute or otherwise make the Covered Software available under
      the terms of any subsequent version of the License published by the
      license steward.

      (( 4.3. ))??
      Modified Versions.

      When You are an Initial Developer and You want to create a new license for
      Your Original Software, You may create and use a modified version of this
      License if You: (a) rename the license and remove any references to the
      name of the license steward (except to note that the license differs from
      this License); and (b) otherwise make it clear that the license contains
      terms which differ from this License.

   (( 5. ))??
   DISCLAIMER OF WARRANTY.

   COVERED SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED SOFTWARE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED SOFTWARE IS WITH YOU. SHOULD
   ANY COVERED SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL
   DEVELOPER OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY
   SERVICING, REPAIR OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN
   ESSENTIAL PART OF THIS LICENSE. NO USE OF ANY COVERED SOFTWARE IS AUTHORIZED
   HEREUNDER EXCEPT UNDER THIS DISCLAIMER.

   (( 6. ))??
   TERMINATION.

      (( 6.1. ))??
      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. Provisions which, by their
      nature, must remain in effect beyond the termination of this License shall
      survive.

      (( 6.2. ))??
      If You assert a patent infringement claim (excluding declaratory judgment
      actions) against Initial Developer or a Contributor (the Initial Developer
      or Contributor against whom You assert such claim is referred to as
      "Participant") alleging that the Participant Software (meaning the
      Contributor Version where the Participant is a Contributor or the Original
      Software where the Participant is the Initial Developer) directly or
      indirectly infringes any patent, then any and all rights granted directly
      or indirectly to You by such Participant, the Initial Developer (if the
      Initial Developer is not the Participant) and all Contributors under
      Sections 2.1 and/or 2.2 of this License shall, upon 60 days notice from
      Participant terminate prospectively and automatically at the expiration of
      such 60 day notice period, unless if within such 60 day period You
      withdraw Your claim with respect to the Participant Software against such
      Participant either unilaterally or pursuant to a written agreement with
      Participant.

      (( 6.3. ))??
      If You assert a patent infringement claim against Participant alleging
      that the Participant Software directly or indirectly infringes any patent
      where such claim is resolved (such as by license or settlement) prior to
      the initiation of patent infringement litigation, then the reasonable
      value of the licenses granted by such Participant under Sections 2.1 or
      2.2 shall be taken into account in determining the amount or value of any
      payment or license.

      (( 6.4. ))??
      In the event of termination under Sections 6.1 or 6.2 above, all end user
      licenses that have been validly granted by You or any distributor
      hereunder prior to termination (excluding licenses granted to You by any
      distributor) shall survive termination.

   (( 7. ))??
   LIMITATION OF LIABILITY.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, THE INITIAL DEVELOPER, ANY
   OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED SOFTWARE, OR ANY SUPPLIER OF
   ANY OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 8. ))??
   U.S. GOVERNMENT END USERS.

   The Covered Software is a "commercial item," as that term is defined in 48
   C.F.R. 2.101 (Oct. 1995), consisting of "commercial computer software" (as
   that term is defined at 48 C.F.R. § 252.227-7014(a)(1) and "commercial
   computer software documentation" as such terms are used in 48 C.F.R. 12.212
   (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R. 227.7202-1
   through 227.7202-4 (June 1995), all U.S. Government End Users acquire Covered
   Software with only those rights set forth herein. This U.S. Government Rights
   clause is in lieu of, and supersedes, any other FAR, DFAR, or other clause or
   provision that addresses Government rights in computer software under this
   License.

   (( 9. ))??
   MISCELLANEOUS.

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by the law of the jurisdiction
   specified in a notice contained within the Original Software (except to the
   extent applicable law, if any, provides otherwise), excluding such
   jurisdiction's conflict-of-law provisions. Any litigation relating to this
   License shall be subject to the jurisdiction of the courts located in the
   jurisdiction and venue specified in a notice contained within the Original
   Software, with the losing party responsible for costs, including, without
   limitation, court costs and reasonable attorneys' fees and expenses. The
   application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any law or regulation
   which provides that the language of a contract shall be construed against the
   drafter shall not apply to this License. You agree that You alone are
   responsible for compliance with the United States export administration
   regulations (and the export control laws and regulation of any other
   countries) when You use, distribute or otherwise make available any Covered
   Software.

   (( 10. ))??
   RESPONSIBILITY FOR CLAIMS.

   As between Initial Developer and the Contributors, each party is responsible
   for claims and damages arising, directly or indirectly, out of its
   utilization of rights under this License and You agree to work with Initial
   Developer and Contributors to distribute such responsibility on an equitable
   basis. Nothing herein is intended or shall be deemed to constitute any
   admission of liability.

NOTICE PURSUANT TO SECTION 9 OF THE COMMON DEVELOPMENT AND DISTRIBUTION LICENSE
(CDDL)

The code released under the CDDL shall be governed by the laws of the State of
California (excluding conflict-of-law provisions). Any litigation relating to
this License shall be subject to the jurisdiction of the Federal Courts of the
Northern District of California and the state courts of the State of California,
with venue lying in Santa Clara County, California.
`
const license_CDLA_Permissive_1_0_lre = `//**
Community Data License Agreement Permissive 1.0
https://spdx.org/licenses/CDLA-Permissive-1.0.json
https://cdla.io/permissive-1-0
**//

(( Community Data License Agreement - Permissive - Version 1.0 ))??

This is the Community Data License Agreement - Permissive, Version 1.0
("Agreement"). Data is provided to You under this Agreement by each of the Data
Providers. Your exercise of any of the rights and permissions granted below
constitutes Your acceptance and agreement to be bound by the terms and
conditions of this Agreement.

The benefits that each Data Provider receives from making Data available and
that You receive from Data or otherwise under these terms and conditions shall
be deemed sufficient consideration for the formation of this Agreement.
Accordingly, Data Provider(s) and You (the "Parties") agree as follows:

   (( Section 1. ))??
   Definitions

      (( 1.1 ))??
      "Add" means to supplement Data with Your own or someone else's Data,
      resulting in Your "Additions." Additions do not include Results.

      (( 1.2 ))??
      "Computational Use" means Your analysis (through the use of computational
      devices or otherwise) or other interpretation of Data. By way of example
      and not limitation, "Computational Use" includes the application of any
      computational analytical technique, the purpose of which is the analysis
      of any Data in digital form to generate information about Data such as
      patterns, trends, correlations, inferences, insights and attributes.

      (( 1.3 ))??
      "Data" means the information (including copyrightable information, such as
      images or text), collectively or individually, whether created or gathered
      by a Data Provider or an Entity acting on its behalf, to which rights are
      granted under this Agreement.

      (( 1.4 ))??
      "Data Provider" means any Entity (including any employee or contractor of
      such Entity authorized to Publish Data on behalf of such Entity) that
      Publishes Data under this Agreement prior to Your Receiving it.

      (( 1.5 ))??
      "Enhanced Data" means the subset of Data that You Publish and that is
      composed of (a) Your Additions and/or (b) Modifications to Data You have
      received under this Agreement.

      (( 1.6 ))??
      "Entity" means any natural person or organization that exists under the
      laws of the jurisdiction in which it is organized, together with all other
      entities that control, are controlled by, or are under common control with
      that entity. For the purposes of this definition, "control" means (a) the
      power, directly or indirectly, to cause the direction or management of
      such entity, whether by contract or otherwise, (b) the ownership of more
      than fifty percent (50%) of the outstanding shares or securities, (c) the
      beneficial ownership of such entity or, (d) the ability to appoint,
      whether by agreement or right, the majority of directors of an Entity.

      (( 1.7 ))??
      "Modify" means to delete, erase, correct or re-arrange Data, resulting in
      "Modifications." Modifications do not include Results.

      (( 1.8 ))??
      "Publish" means to make all or a subset of Data (including Your Enhanced
      Data) available in any manner which enables its Use, including by
      providing a copy on physical media or remote access. For any form of
      Entity, that is to make the Data available to any individual who is not
      employed by that Entity or engaged as a contractor or agent to perform
      work on that Entity's behalf. A "Publication" occurs each time You Publish
      Data.

      (( 1.9 ))??
      "Receive" or "Receives" means to have been given access to Data, locally
      or remotely.

      (( 1.10 ))??
      "Results" means the outcomes or outputs that You obtain from Your
      Computational Use of Data. Results shall not include more than a de
      minimis portion of the Data on which the Computational Use is based.

      (( 1.11 ))??
      "Sui Generis Database Rights" means rights, other than copyright,
      resulting from Directive 96/9/EC of the European Parliament and of the
      Council of 11 March 1996 on the legal protection of databases, as amended
      and/or succeeded, as well as other equivalent rights anywhere in the
      world.

      (( 1.12 ))??
      "Use" means using Data (including accessing, copying, studying, reviewing,
      adapting, analyzing, evaluating, or making Computational Use of it),
      either by machines or humans, or a combination of both.

      (( 1.13 ))??
      "You" or "Your" means any Entity that Receives Data under this Agreement.

   (( Section 2. ))??
   Right and License to Use and to Publish

      (( 2.1 ))??
      Subject to the conditions set forth in Section 3 of this Agreement, Data
      Provider(s) hereby grant(s) to You a worldwide, non-exclusive, irrevocable
      (except as provided in Section 5) right to: (a) Use Data; and (b) Publish
      Data.

      (( 2.2 ))??
      To the extent that the Data or the coordination, selection or arrangement
      of Data is protected or protectable under copyright, Sui Generis Database
      Rights, or other law, Data Provider(s) further agree(s) that such Data or
      coordination, selection or arrangement is hereby licensed to You and to
      anyone else who Receives Data under this Agreement for Use and
      Publication, subject to the conditions set forth in Section 3 of this
      Agreement.

      (( 2.3 ))??
      Except for these rights and licenses expressly granted, no other
      intellectual property rights are granted or should be implied.

   (( Section 3. ))??
   Conditions on Rights Granted

      (( 3.1 ))??
      If You Publish Data You Receive or Enhanced Data:

         (( (a) ))??
         You may do so under a license of Your choice provided that You give
         anyone who Receives the Data from You the text of this Agreement, the
         name of this Agreement and/or a hyperlink or other method reasonably
         likely to provide a copy of the text of this Agreement; and

         (( (b) ))??
         You must cause any Data files containing Enhanced Data to carry
         prominent notices that You have changed those files; and

         (( c ))??
         If You Publish Data You Receive, You must preserve all credit or
         attribution to the Data Provider(s). Such retained credit or
         attribution includes any of the following to the extent they exist in
         Data as You have Received it: legal notices or metadata; identification
         of the Data Provider(s); or hyperlinks to Data to the extent it is
         practical to do so.

      (( 3.2 ))??
      You may provide additional or different license terms and conditions for
      use, reproduction, or distribution of that Enhanced Data, or for any
      combination of Data and Enhanced Data as a whole, provided that Your Use
      and Publication of that combined Data otherwise complies with the
      conditions stated in this License.

      (( 3.3 ))??
      You and each Data Provider agree that Enhanced Data shall not be
      considered a work of joint authorship by virtue of its relationship to
      Data licensed under this Agreement and shall not require either any
      obligation of accounting to or the consent of any Data Provider.

      (( 3.4 ))??
      This Agreement imposes no obligations or restrictions on Your Use or
      Publication of Results.

   (( Section 4. ))??
   Data Provider(s)' Representations

      (( 4.1 ))??
      Each Data Provider represents that the Data Provider has exercised
      reasonable care, to assure that: (a) the Data it Publishes was created or
      generated by it or was obtained from others with the right to Publish the
      Data under this Agreement; and (b) Publication of such Data does not
      violate any privacy or confidentiality obligation undertaken by the Data
      Provider.

   (( Section 5. ))??
   Termination

      (( 5.1 ))??
      All of Your rights under this Agreement will terminate, and Your right to
      Receive, Use or Publish the Data will be revoked or modified if You
      materially fail to comply with the terms and conditions of this Agreement
      and You do not cure such failure in a reasonable period of time after
      becoming aware of such noncompliance. If Your rights under this Agreement
      terminate, You agree to cease Receipt, Use and Publication of Data.
      However, Your obligations and any rights and permissions granted by You
      under this Agreement relating to Data that You Published prior to such
      termination will continue and survive.

      (( 5.2 ))??
      If You institute litigation against a Data Provider or anyone else who
      Receives the Data (including a cross-claim in a lawsuit) based on the
      Data, other than a claim asserting breach of this Agreement, then any
      rights previously granted to You to Receive, Use and Publish Data under
      this Agreement will terminate as of the date such litigation is filed.

   (( Section 6. ))??
   Disclaimer of Warranties and Limitation of Liability

      (( 6.1 ))??
      EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, THE DATA (INCLUDING
      ENHANCED DATA) IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OR
      CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT
      LIMITATION, ANY WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT,
      MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

      (( 6.2 ))??
      NEITHER YOU NOR ANY DATA PROVIDERS SHALL HAVE ANY LIABILITY FOR ANY
      DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
      (INCLUDING WITHOUT LIMITATION LOST PROFITS), HOWEVER CAUSED AND ON ANY
      THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
      (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OR
      DISTRIBUTION OF THE DATA OR THE EXERCISE OF ANY RIGHTS GRANTED HEREUNDER,
      EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

   (( Section 7. ))??
   Miscellaneous

      (( 7.1 ))??
      You agree that it is solely Your responsibility to comply with all
      applicable laws with regard to Your Use or Publication of Data, including
      any applicable privacy, data protection, security and export laws. You
      agree to take reasonable steps to assist a Data Provider fulfilling
      responsibilities to comply with applicable laws with regard to Use or
      Publication of Data Received hereunder.

      (( 7.2 ))??
      You and Data Provider(s), collectively and individually, waive and/or
      agree not to assert, to the extent permitted by law, any moral rights You
      or they hold in Data.

      (( 7.3 ))??
      This Agreement confers no rights or remedies upon any person or entity
      other than the Parties and their respective heirs, executors, successors
      and assigns.

      (( 7.4 ))??
      The Data Provider(s) reserve no right or expectation of privacy, data
      protection or confidentiality in any Data that they Publish under this
      Agreement. If You choose to Publish Data under this Agreement, You
      similarly do so with no reservation or expectation of any rights of
      privacy or confidentiality in that Data.

      (( 7.5 ))??
      The Community Data License Agreement workgroup under The Linux Foundation
      is the steward of this Agreement ("Steward"). No one other than the
      Steward has the right to modify or publish new versions of this Agreement.
      Each version will be given a distinguishing version number. You may Use
      and Publish Data Received hereunder under the terms of the version of the
      Agreement under which You originally Received the Data, or under the terms
      of any subsequent version published by the Steward.
`
const license_CDLA_Sharing_1_0_lre = `//**
Community Data License Agreement Sharing 1.0
https://spdx.org/licenses/CDLA-Sharing-1.0.json
https://cdla.io/sharing-1-0
**//

(( Community Data License Agreement - Sharing - Version 1.0 ))??

This is the Community Data License Agreement - Sharing, Version 1.0
("Agreement"). Data is provided to You under this Agreement by each of the Data
Providers. Your exercise of any of the rights and permissions granted below
constitutes Your acceptance and agreement to be bound by the terms and
conditions of this Agreement.

The benefits that each Data Provider receives from making Data available and
that You receive from Data or otherwise under these terms and conditions shall
be deemed sufficient consideration for the formation of this Agreement.
Accordingly, Data Provider(s) and You (the "Parties") agree as follows:

   (( Section 1. ))??
   Definitions

      (( 1.1 ))??
      "Add" means to supplement Data with Your own or someone else's Data,
      resulting in Your "Additions." Additions do not include Results.

      (( 1.2 ))??
      "Computational Use" means Your analysis (through the use of computational
      devices or otherwise) or other interpretation of Data. By way of example
      and not limitation, "Computational Use" includes the application of any
      computational analytical technique, the purpose of which is the analysis
      of any Data in digital form to generate information about Data such as
      patterns, trends, correlations, inferences, insights and attributes.

      (( 1.3 ))??
      "Data" means the information (including copyrightable information, such as
      images or text), collectively or individually, whether created or gathered
      by a Data Provider or an Entity acting on its behalf, to which rights are
      granted under this Agreement.

      (( 1.4 ))??
      "Data Provider" means any Entity (including any employee or contractor of
      such Entity authorized to Publish Data on behalf of such Entity) that
      Publishes Data under this Agreement prior to Your Receiving it.

      (( 1.5 ))??
      "Enhanced Data" means the subset of Data that You Publish and that is
      composed of (a) Your Additions and/or (b) Modifications to Data You have
      received under this Agreement.

      (( 1.6 ))??
      "Entity" means any natural person or organization that exists under the
      laws of the jurisdiction in which it is organized, together with all other
      entities that control, are controlled by, or are under common control with
      that entity. For the purposes of this definition, "control" means (a) the
      power, directly or indirectly, to cause the direction or management of
      such entity, whether by contract or otherwise, (b) the ownership of more
      than fifty percent (50%) of the outstanding shares or securities, (c) the
      beneficial ownership of such entity or, (d) the ability to appoint,
      whether by agreement or right, the majority of directors of an Entity.

      (( 1.7 ))??
      "Ledger" means a digital record of Data or grants of rights in Data
      governed by this Agreement, using any technology having functionality to
      record and store Data or grants, contributions, or licenses to Data
      governed by this Agreement.

      (( 1.8 ))??
      "Modify" means to delete, erase, correct or re-arrange Data, resulting in
      "Modifications." Modifications do not include Results.

      (( 1.9 ))??
      "Publish" means to make all or a subset of Data (including Your Enhanced
      Data) available in any manner which enables its Use, including by
      providing a copy on physical media or remote access. For any form of
      Entity, that is to make the Data available to any individual who is not
      employed by that Entity or engaged as a contractor or agent to perform
      work on that Entity's behalf. A "Publication" occurs each time You Publish
      Data.

      (( 1.10 ))??
      "Receive" or "Receives" means to have been given access to Data, locally
      or remotely.

      (( 1.11 ))??
      "Results" means the outcomes or outputs that You obtain from Your
      Computational Use of Data. Results shall not include more than a de
      minimis portion of the Data on which the Computational Use is based.

      (( 1.12 ))??
      "Sui Generis Database Rights" means rights, other than copyright,
      resulting from Directive 96/9/EC of the European Parliament and of the
      Council of 11 March 1996 on the legal protection of databases, as amended
      and/or succeeded, as well as other equivalent rights anywhere in the
      world.

      (( 1.13 ))??
      "Use" means using Data (including accessing, copying, studying, reviewing,
      adapting, analyzing, evaluating, or making Computational Use of it),
      either by machines or humans, or a combination of both.

      (( 1.14 ))??
      "You" or "Your" means any Entity that Receives Data under this Agreement.

   (( Section 2. ))??
   Right and License to Use and to Publish

      (( 2.1 ))??
      Subject to the conditions set forth in Section 3 of this Agreement, Data
      Provider(s) hereby grant(s) to You a worldwide, non-exclusive, irrevocable
      (except as provided in Section 5) right to: (a) Use Data; and (b) Publish
      Data.

      (( 2.2 ))??
      To the extent that the Data or the coordination, selection or arrangement
      of Data is protected or protectable under copyright, Sui Generis Database
      Rights, or other law, Data Provider(s) further agree(s) that such Data or
      coordination, selection or arrangement is hereby licensed to You and to
      anyone else who Receives Data under this Agreement for Use and
      Publication, subject to the conditions set forth in Section 3 of this
      Agreement.

      (( 2.3 ))??
      Except for these rights and licenses expressly granted, no other
      intellectual property rights are granted or should be implied.

   (( Section 3. ))??
   Conditions on Rights Granted

      (( 3.1 ))??
      If You Publish Data You Receive or Enhanced Data:

         (( (a) ))??
         The Data (including the Enhanced Data) must be Published under this
         Agreement in accordance with this Section 3; and

         (( (b) ))??
         You must cause any Data files containing Enhanced Data to carry
         prominent notices that You have changed those files; and

         (( c ))??
         If You Publish Data You Receive, You must preserve all credit or
         attribution to the Data Provider(s). Such retained credit or
         attribution includes any of the following to the extent they exist in
         Data as You have Received it: legal notices or metadata; identification
         of the Data Provider(s); or hyperlinks to Data to the extent it is
         practical to do so.

      (( 3.2 ))??
      You may not restrict or deter the ability of anyone who Receives the Data
      (a) to Publish the Data in a publicly-accessible manner or (b) if the
      project has designated a Ledger for recording Data or grants of rights in
      Data for purposes of this Agreement, to record the Data or grants of
      rights in Data in the Ledger.

      (( 3.3 ))??
      If You Publish Data You Receive, You must do so under an unmodified form
      of this Agreement and include the text of this Agreement, the name of this
      Agreement and/or a hyperlink or other method reasonably likely to provide
      a copy of the text of this Agreement. You may not modify this Agreement or
      impose any further restrictions on the exercise of the rights granted
      under this Agreement, including by adding any restriction on commercial or
      non-commercial Use of Data (including Your Enhanced Data) or by limiting
      permitted Use of such Data to any particular platform, technology or field
      of endeavor. Notices that purport to modify this Agreement shall be of no
      effect.

      (( 3.4 ))??
      You and each Data Provider agree that Enhanced Data shall not be
      considered a work of joint authorship by virtue of its relationship to
      Data licensed under this Agreement and shall not require either any
      obligation of accounting to or the consent of any Data Provider.

      (( 3.5 ))??
      This Agreement imposes no obligations or restrictions on Your Use or
      Publication of Results.

   (( Section 4. ))??
   Data Provider(s)' Representations

      (( 4.1 ))??
      Each Data Provider represents that the Data Provider has exercised
      reasonable care, to assure that: (a) the Data it Publishes was created or
      generated by it or was obtained from others with the right to Publish the
      Data under this Agreement; and (b) Publication of such Data does not
      violate any privacy or confidentiality obligation undertaken by the Data
      Provider.

   (( Section 5. ))??
   Termination

      (( 5.1 ))??
      All of Your rights under this Agreement will terminate, and Your right to
      Receive, Use or Publish the Data will be revoked or modified if You
      materially fail to comply with the terms and conditions of this Agreement
      and You do not cure such failure in a reasonable period of time after
      becoming aware of such noncompliance. If Your rights under this Agreement
      terminate, You agree to cease Receipt, Use and Publication of Data.
      However, Your obligations and any rights and permissions granted by You
      under this Agreement relating to Data that You Published prior to such
      termination will continue and survive.

      (( 5.2 ))??
      If You institute litigation against a Data Provider or anyone else who
      Receives the Data (including a cross-claim in a lawsuit) based on the
      Data, other than a claim asserting breach of this Agreement, then any
      rights previously granted to You to Receive, Use and Publish Data under
      this Agreement will terminate as of the date such litigation is filed.

   (( Section 6. ))??
   Disclaimer of Warranties and Limitation of Liability

      (( 6.1 ))??
      EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, THE DATA (INCLUDING
      ENHANCED DATA) IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OR
      CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT
      LIMITATION, ANY WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT,
      MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

      (( 6.2 ))??
      NEITHER YOU NOR ANY DATA PROVIDERS SHALL HAVE ANY LIABILITY FOR ANY
      DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
      (INCLUDING WITHOUT LIMITATION LOST PROFITS), HOWEVER CAUSED AND ON ANY
      THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
      (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OR
      DISTRIBUTION OF THE DATA OR THE EXERCISE OF ANY RIGHTS GRANTED HEREUNDER,
      EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

   (( Section 7. ))??
   Miscellaneous

      (( 7.1 ))??
      You agree that it is solely Your responsibility to comply with all
      applicable laws with regard to Your Use or Publication of Data, including
      any applicable privacy, data protection, security and export laws. You
      agree to take reasonable steps to assist a Data Provider fulfilling
      responsibilities to comply with applicable laws with regard to Use or
      Publication of Data Received hereunder.

      (( 7.2 ))??
      You and Data Provider(s), collectively and individually, waive and/or
      agree not to assert, to the extent permitted by law, any moral rights You
      or they hold in Data.

      (( 7.3 ))??
      This Agreement confers no rights or remedies upon any person or entity
      other than the Parties and their respective heirs, executors, successors
      and assigns.

      (( 7.4 ))??
      The Data Provider(s) reserve no right or expectation of privacy, data
      protection or confidentiality in any Data that they Publish under this
      Agreement. If You choose to Publish Data under this Agreement, You
      similarly do so with no reservation or expectation of any rights of
      privacy or confidentiality in that Data.

      (( 7.5 ))??
      The Community Data License Agreement workgroup under The Linux Foundation
      is the steward of this Agreement ("Steward"). No one other than the
      Steward has the right to modify or publish new versions of this Agreement.
      Each version will be given a distinguishing version number. You may Use
      and Publish Data Received hereunder under the terms of the version of the
      Agreement under which You originally Received the Data, or under the terms
      of any subsequent version published by the Steward.
`
const license_CECILL_1_0_lre = `//**
CeCILL Free Software License Agreement v1.0
https://spdx.org/licenses/CECILL-1.0.json
http://www.cecill.info/licences/Licence_CeCILL_V1-fr.html
**//

(( CONTRAT DE LICENCE DE LOGICIEL LIBRE CeCILL ))??

(( Avertissement

Ce contrat est une licence de logiciel libre issue d'une concertation entre ses
auteurs afin que le respect de deux grands principes préside à sa rédaction:

   •
   d'une part, sa conformité au droit français, tant au regard du droit de la
   responsabilité civile que du droit de la propriété intellectuelle et de la
   protection qu'il offre aux auteurs et titulaires des droits patrimoniaux sur
   un logiciel.

   •
   d'autre part, le respect des principes de diffusion des logiciels libres:
   accès au code source, droits étendus conférés aux utilisateurs.

Les auteurs de la licence CeCILL1 sont:

Commissariat à l'Energie Atomique – CEA, établissement public de caractère
scientifique technique et industriel, dont le siège est situé 31-33 rue de la
Fédération, 75752 PARIS cedex 15.

Centre National de la Recherche Scientifique – CNRS, établissement public à
caractère scientifique et technologique, dont le siège est situé 3 rue
Michel-Ange 75794 Paris cedex 16.

Institut National de Recherche en Informatique et en Automatique – INRIA,
établissement public à caractère scientifique et technologique, dont le
siège est situé Domaine de Voluceau, Rocquencourt, BP 105, 78153 Le Chesnay
cedex. ))??

PREAMBULE

Ce contrat est une licence de logiciel libre dont l'objectif est de conférer
aux utilisateurs la liberté de modification et de redistribution du logiciel
régi par cette licence dans le cadre d'un modèle de diffusion «open
source».

L'exercice de ces libertés est assorti de certains devoirs à la charge des
utilisateurs afin de préserver ce statut au cours des redistributions
ultérieures.

L'accessibilité au code source et les droits de copie, de modification et de
redistribution qui en découlent ont pour contrepartie de n'offrir aux
utilisateurs qu'une garantie limitée et de ne faire peser sur l'auteur du
logiciel, le titulaire des droits patrimoniaux et les concédants successifs
qu'une responsabilité restreinte.

A cet égard l'attention de l'utilisateur est attirée sur les risques associés
au chargement, à l'utilisation, à la modification et/ou au développement et
à la reproduction du logiciel par l'utilisateur étant donné sa spécificité
de logiciel libre, qui peut le rendre complexe à manipuler et qui le réserve
donc à des développeurs et des professionnels avertis possédant des
connaissances informatiques approfondies. Les utilisateurs sont donc invités à
charger et tester l'adéquation du Logiciel à leurs besoins dans des conditions
permettant d'assurer la sécurité de leurs systèmes et ou de leurs données
et, plus généralement, à l'utiliser et l'exploiter dans les même conditions
de sécurité. Ce contrat peut être reproduit et diffusé librement, sous
réserve de le conserver en l'état, sans ajout ni suppression de clauses.

Ce contrat est susceptible de s'appliquer à tout logiciel dont le titulaire des
droits patrimoniaux décide de soumettre l'exploitation aux dispositions qu'il
contient.

Article 1er - DEFINITIONS

Dans ce contrat, les termes suivants, lorsqu'ils seront écrits avec une lettre
capitale, auront la signification suivante:

Contrat: désigne le présent contrat de licence, ses éventuelles versions
postérieures et annexes.

Logiciel: désigne le logiciel sous sa forme de Code Objet et/ou de Code Source
et le cas échéant sa documentation, dans leur état au moment de l'acceptation
du

Contrat par le Licencié.

Logiciel Initial: désigne le Logiciel sous sa forme de Code Source et de Code
Objet et le cas échéant sa documentation, dans leur état au moment de leur
première diffusion sous les termes du Contrat.

Logiciel Modifié: désigne le Logiciel modifié par au moins une Contribution.

Code Source: désigne l'ensemble des instructions et des lignes de programme du
Logiciel et auquel l'accès est nécessaire en vue de modifier le Logiciel.

Code Objet: désigne les fichiers binaires issus de la compilation du Code
Source.

Titulaire : désigne le détenteur des droits patrimoniaux d'auteur sur le
Logiciel Initial.

Licencié(s): désigne le ou les utilisateur(s) du Logiciel ayant accepté le
Contrat.

Contributeur: désigne le Licencié auteur d'au moins une Contribution.

Concédant: désigne le Titulaire ou toute personne physique ou morale
distribuant le Logiciel sous le Contrat.

Contributions: désigne l'ensemble des modifications, corrections, traductions,
adaptations et/ou nouvelles fonctionnalités intégrées dans le Logiciel par
tout

Contributeur, ainsi que les Modules Statiques.

Module: désigne un ensemble de fichiers sources y compris leur documentation
qui, une fois compilé sous forme exécutable, permet de réaliser des
fonctionnalités ou

services supplémentaires à ceux fournis par le Logiciel.

Module Dynamique: désigne tout Module, créé par le Contributeur, indépendant
du Logiciel, tel que ce Module et le Logiciel sont sous forme de deux
exécutables indépendants qui s'exécutent dans un espace d'adressage
indépendant, l'un appelant l'autre au moment de leur exécution.

Module Statique: désigne tout Module créé par le Contributeur et lié au
Logiciel par un lien statique rendant leur code objet dépendant l'un de
l'autre. Ce Module et le Logiciel auquel il est lié, sont regroupés en un seul
exécutable.

Parties: désigne collectivement le Licencié et le Concédant.

Ces termes s'entendent au singulier comme au pluriel.

Article 2 - OBJET

Le Contrat a pour objet la concession par le Concédant au Licencié d'une
Licence non exclusive, transférable et mondiale du Logiciel telle que définie
ci-après à l'article 5 pour toute la durée de protection des droits portant
sur ce Logiciel.

Article 3 - ACCEPTATION

   (( 3.1. ))??
   L'acceptation par le Licencié des termes du Contrat est réputée acquise du
   fait du premier des faits suivants:

      •
      (i) le chargement du Logiciel par tout moyen notamment par
      téléchargement à partir d'un serveur distant ou par chargement à
      partir d'un support physique;

      •
      (ii) le premier exercice par le Licencié de l'un quelconque des droits
      concédés par le Contrat.

   (( 3.2. ))??
   Un exemplaire du Contrat, contenant notamment un avertissement relatif aux
   spécificités du Logiciel, à la restriction de garantie et à la limitation
   à un usage par des utilisateurs expérimentés a été mis à disposition du
   Licencié préalablement à son acceptation telle que définie à l'article
   3.1 ci dessus et le Licencié reconnaît en avoir pris connaissances.

Article 4 - ENTREE EN VIGUEUR ET DUREE

   (( 4.1. ))??
   ENTREE EN VIGUEUR

   Le Contrat entre en vigueur à la date de son acceptation par le Licencié
   telle que définie en 3.1.

   (( 4.2. ))??
   DUREE

   Le Contrat produira ses effets pendant toute la durée légale de protection
   des droits patrimoniaux portant sur le Logiciel.

Article 5 - ETENDUE DES DROITS CONCEDES

Le Concédant concède au Licencié, qui accepte, les droits suivants sur le
Logiciel pour toutes destinations et pour la durée du Contrat dans les
conditions ci-après détaillées.

Par ailleurs, le Concédant concède au Licencié à titre gracieux les droits
d'exploitation du ou des brevets qu'il détient sur tout ou partie des
inventions

implémentées dans le Logiciel.

   (( 5.1. ))??
   DROITS D'UTILISATION

   Le Licencié est autorisé à utiliser le Logiciel, sans restriction quant
   aux domaines d'application, étant ci-après précisé que cela comporte:

      (( 1. ))??
      la reproduction permanente ou provisoire du Logiciel en tout ou partie par
      tout moyen et sous toute forme.

      (( 2. ))??
      le chargement, l'affichage, l'exécution, ou le stockage du Logiciel sur
      tout support.

      (( 3. ))??
      la possibilité d'en observer, d'en étudier, ou d'en tester le
      fonctionnement afin de déterminer les idées et principes qui sont à la
      base de n'importe quel élément de ce Logiciel; et ceci, lorsque le
      Licencié effectue toute opération de chargement, d'affichage,
      d'exécution, de transmission ou de stockage du Logiciel qu'il est en
      droit d'effectuer en vertu du Contrat.

   (( 5.2. ))??
   DROIT D'APPORTER DES CONTRIBUTIONS

   Le droit d'apporter des Contributions comporte le droit de traduire,
   d'adapter, d'arranger ou d'apporter toute autre modification du Logiciel et
   le droit de reproduire le Logiciel en résultant.

   Le Licencié est autorisé à apporter toute Contribution au Logiciel sous
   réserve de mentionner, de façon explicite, son nom en tant qu'auteur de
   cette Contribution et la date de création de celle-ci.

   (( 5.3. ))??
   DROITS DE DISTRIBUTION ET DE DIFFUSION

   Le droit de distribution et de diffusion comporte notamment le droit de
   transmettre et de communiquer le Logiciel au public sur tout support et par
   tout moyen ainsi que le droit de mettre sur le marché à titre onéreux ou
   gratuit, un ou des exemplaires du Logiciel par tout procédé.

   Le Licencié est autorisé à redistribuer des copies du Logiciel, modifié
   ou non, à des tiers dans les conditions ci-après détaillées.

      (( 5.3.1. ))??
      REDISTRIBUTION DU LOGICIEL SANS MODIFICATION

      Le Licencié est autorisé à redistribuer des copies conformes du
      Logiciel, sous forme de Code Source ou de Code Objet, à condition que
      cette redistribution respecte les dispositions du Contrat dans leur
      totalité et soit accompagnée:

         (( 1. ))??
         d'un exemplaire du Contrat,

         (( 2. ))??
         d'un avertissement relatif à la restriction de garantie et de
         responsabilité du Concédant telle que prévue aux articles 8 et 9,

         et que, dans le cas où seul le Code Objet du Logiciel est
         redistribué, le Licencié permette aux futurs Licenciés d'accéder
         facilement au Code Source complet du Logiciel en indiquant les
         modalités d'accès, étant entendu que le coût additionnel
         d'acquisition du Code Source ne devra pas excéder le simple coût de
         transfert des données.

      (( 5.3.2. ))??
      REDISTRIBUTION DU LOGICIEL MODIFIE

      Lorsque le Licencié apporte une Contribution au Logiciel, les conditions
      de redistribution du Logiciel Modifié sont alors soumises à
      l'intégralité des dispositions du Contrat.

      Le Licencié est autorisé à redistribuer le Logiciel Modifié, sous
      forme de Code Source ou de Code Objet, à condition que cette
      redistribution respecte les dispositions du Contrat dans leur totalité et
      soit accompagnée:

         (( 1. ))??
         d'un exemplaire du Contrat,

         (( 2. ))??
         d'un avertissement relatif à la restriction de garantie et de
         responsabilité du concédant telle que prévue aux articles 8 et 9,

         et que, dans le cas où seul le Code Objet du Logiciel Modifié est
         redistribué, le Licencié permette aux futurs Licenciés d'accéder
         facilement au Code Source complet du Logiciel Modifié en indiquant les
         modalités d'accès, étant entendu que le coût additionnel
         d'acquisition du Code Source ne devra pas excéder le simple coût de
         transfert des données.

      (( 5.3.3. ))??
      REDISTRIBUTION DES MODULES DYNAMIQUES

      Lorsque le Licencié a développé un Module Dynamique les conditions du
      Contrat ne s'appliquent pas à ce Module Dynamique, qui peut être
      distribué sous un contrat de licence différent.

      (( 5.3.4. ))??
      COMPATIBILITE AVEC LA LICENCE GPL

      Dans le cas où le Logiciel, Modifié ou non, est intégré à un code
      soumis aux dispositions de la licence GPL, le Licencié est autorisé à
      redistribuer l'ensemble sous la licence GPL.

      Dans le cas où le Logiciel Modifié intègre un code soumis aux
      dispositions de la licence GPL, le Licencié est autorisé à redistribuer
      le Logiciel Modifié sous la licence GPL.

Article 6 - PROPRIETE INTELLECTUELLE

   (( 6.1. ))??
   SUR LE LOGICIEL INITIAL

   Le Titulaire est détenteur des droits patrimoniaux sur le Logiciel Initial.
   Toute utilisation du Logiciel Initial est soumise au respect des conditions
   dans lesquelles le Titulaire a choisi de diffuser son œuvre et nul autre n'a
   la faculté de modifier les conditions de diffusion de ce Logiciel Initial.

   Le Titulaire s'engage à maintenir la diffusion du Logiciel initial sous les
   conditions du Contrat et ce, pour la durée visée à l'article 4.2.

   (( 6.2. ))??
   SUR LES CONTRIBUTIONS

   Les droits de propriété intellectuelle sur les Contributions sont attachés
   au titulaire de droits patrimoniaux désigné par la législation
   applicable.

   (( 6.3. ))??
   SUR LES MODULES DYNAMIQUES

   Le Licencié ayant développé un Module Dynamique est titulaire des droits
   de propriété intellectuelle sur ce Module Dynamique et reste libre du choix
   du contrat régissant sa diffusion.

   (( 6.4. ))??
   DISPOSITIONS COMMUNES

      (( 6.4.1. ))??
      Le Licencié s'engage expressément:

         (( 1. ))??
         à ne pas supprimer ou modifier de quelque manière que ce soit les
         mentions de propriété intellectuelle apposées sur le Logiciel;

         (( 2. ))??
         à reproduire à l'identique lesdites mentions de propriété
         intellectuelle sur les copies du Logiciel.

      (( 6.4.2. ))??
      Le Licencié s'engage à ne pas porter atteinte, directement ou
      indirectement, aux droits de propriété intellectuelle du Titulaire et/ou
      des Contributeurs et à prendre, le cas échéant, à l'égard de son
      personnel toutes les mesures nécessaires pour assurer le respect des dits
      droits de propriété intellectuelle du Titulaire et/ou des
      Contributeurs.

Article 7 - SERVICES ASSOCIES

   (( 7.1. ))??
   Le Contrat n'oblige en aucun cas le Concédant à la réalisation de
   prestations d'assistance technique ou de maintenance du Logiciel.

   Cependant le Concédant reste libre de proposer ce type de services. Les
   termes et conditions d'une telle assistance technique et/ou d'une telle
   maintenance seront alors déterminés dans un acte séparé. Ces actes de
   maintenance et/ou assistance technique n'engageront que la seule
   responsabilité du Concédant qui les propose.

   (( 7.2. ))??
   De même, tout Concédant est libre de proposer, sous sa seule
   responsabilité, à ses licenciés une garantie, qui n'engagera que lui, lors
   de la redistribution du Logiciel et/ou du Logiciel Modifié et ce, dans les
   conditions qu'il souhaite. Cette garantie et les modalités financières de
   son application feront l'objet d'un acte séparé entre le Concédant et le
   Licencié.

Article 8 - RESPONSABILITE

   (( 8.1. ))??
   Sous réserve des dispositions de l'article 8.2, si le Concédant n'exécute
   pas tout ou partie des obligations mises à sa charge par le Contrat, le
   Licencié a la faculté, sous réserve de prouver la faute du Concédant
   concerné, de solliciter la réparation du préjudice direct qu'il subit et
   dont il apportera la preuve.

   (( 8.2. ))??
   La responsabilité du Concédant est limitée aux engagements pris en
   application du Contrat et ne saurait être engagée en raison notamment:(i)
   des dommages dus à l'inexécution, totale ou partielle, de ses obligations
   par le Licencié, (ii) des dommages directs ou indirects découlant de
   l'utilisation ou des performances du Logiciel subis par le Licencié
   lorsqu'il s'agit d'un professionnel utilisant le Logiciel à des fins
   professionnelles et (iii) des dommages indirects découlant de l'utilisation
   ou des performances du Logiciel. Les Parties conviennent expressément que
   tout préjudice financier ou commercial (par exemple perte de données, perte
   de bénéfices, perte d'exploitation, perte de clientèle ou de commandes,
   manque à gagner, trouble commercial quelconque) ou toute action dirigée
   contre le Licencié par un tiers, constitue un dommage indirect et n'ouvre
   pas droit à réparation par le Concédant.

Article 9 - GARANTIE

   (( 9.1. ))??
   Le Licencié reconnaît que l'état actuel des connaissances scientifiques et
   techniques au moment de la mise en circulation du Logiciel ne permet pas d'en
   tester et d'en vérifier toutes les utilisations ni de détecter l'existence
   d'éventuels défauts. L'attention du Licencié a été attirée sur ce point
   sur les risques associés au chargement, à l'utilisation, la modification
   et/ou au développement et à la reproduction du Logiciel qui sont réservés
   à des utilisateurs avertis.

   Il relève de la responsabilité du Licencié de contrôler, par tous moyens,
   l'adéquation du produit à ses besoins, son bon fonctionnement et de
   s'assurer qu'il ne causera pas de dommages aux personnes et aux biens.

   (( 9.2. ))??
   Le Concédant déclare de bonne foi être en droit de concéder l'ensemble
   des droits attachés au Logiciel (comprenant notamment les droits visés à
   l'article 5).

   (( 9.3. ))??
   Le Licencié reconnaît que le Logiciel est fourni «en l'état» par le
   Concédant sans autre garantie, expresse ou tacite, que celle prévue à
   l'article 9.2 et notamment sans aucune garantie sur sa valeur commerciale,
   son caractère sécurisé, innovant ou pertinent.

   En particulier, le Concédant ne garantit pas que le Logiciel est exempt
   d'erreur, qu'il fonctionnera sans interruption, qu'il sera compatible avec
   l'équipement du Licencié et sa configuration logicielle ni qu'il remplira
   les besoins du Licencié.

   (( 9.4. ))??
   Le Concédant ne garantit pas, de manière expresse ou tacite, que le
   Logiciel ne porte pas atteinte à un quelconque droit de propriété
   intellectuelle d'un tiers portant sur un brevet, un logiciel ou sur tout
   autre droit de propriété. Ainsi, le Concédant exclut toute garantie au
   profit du Licencié contre les actions en contrefaçon qui pourraient être
   diligentées au titre de l'utilisation, de la modification, et de la
   redistribution du Logiciel. Néanmoins, si de telles actions sont exercées
   contre le Licencié, le Concédant lui apportera son aide technique et
   juridique pour sa défense. Cette aide technique et juridique est
   déterminée au cas par cas entre le Concédant concerné et le Licencié
   dans le cadre d'un protocole d'accord. Le Concédant dégage toute
   responsabilité quant à l'utilisation de la dénomination du Logiciel par le
   Licencié. Aucune garantie n'est apportée quant à l'existence de droits
   antérieurs sur le nom du Logiciel et sur l'existence d'une marque.

Article 10 - RESILIATION

   (( 10.1. ))??
   En cas de manquement par le Licencié aux obligations mises à sa charge par
   le Contrat, le Concédant pourra résilier de plein droit le Contrat trente
   (30) jours après notification adressée au Licencié et restée sans effet.

   (( 10.2. ))??
   Le Licencié dont le Contrat est résilié n'est plus autorisé à utiliser,
   modifier ou distribuer le Logiciel. Cependant, toutes les licences qu'il aura
   concédées antérieurement à la résiliation du Contrat resteront valides
   sous réserve qu'elles aient été effectuées en conformité avec le
   Contrat.

Article 11 - DISPOSITIONS DIVERSES

   (( 11.1. ))??
   CAUSE EXTERIEURE

   Aucune des Parties ne sera responsable d'un retard ou d'une défaillance
   d'exécution du Contrat qui serait dû à un cas de force majeure, un cas
   fortuit ou une cause extérieure, telle que, notamment, le mauvais
   fonctionnement ou les interruptions du réseau électrique ou de
   télécommunication, la paralysie du réseau liée à une attaque
   informatique, l'intervention des autorités gouvernementales, les
   catastrophes naturelles, les dégâts des eaux, les tremblements de terre, le
   feu, les explosions, les grèves et les conflits sociaux, l'état de
   guerre…

   (( 11.2. ))??
   Le fait, par l'une ou l'autre des Parties, d'omettre en une ou plusieurs
   occasions de se prévaloir d'une ou plusieurs dispositions du Contrat, ne
   pourra en aucun cas impliquer renonciation par la Partie intéressée à s'en
   prévaloir ultérieurement.

   (( 11.3. ))??
   Le Contrat annule et remplace toute convention antérieure, écrite ou orale,
   entre les Parties sur le même objet et constitue l'accord entier entre les
   Parties sur cet objet. Aucune addition ou modification aux termes du Contrat
   n'aura d'effet à l'égard des Parties à moins d'être faite par écrit et
   signée par leurs représentants dûment habilités.

   (( 11.4. ))??
   Dans l'hypothèse où une ou plusieurs des dispositions du Contrat
   s'avèrerait contraire à une loi ou à un texte applicable, existants ou
   futurs, cette loi ou ce texte prévaudrait, et les Parties feraient les
   amendements nécessaires pour se conformer à cette loi ou à ce texte.
   Toutes les autres dispositions resteront en vigueur. De même, la nullité,
   pour quelque raison que ce soit, d'une des dispositions du Contrat ne saurait
   entraîner la nullité de l'ensemble du Contrat.

   (( 11.5. ))??
   LANGUE

   Le Contrat est rédigé en langue française et en langue anglaise. En cas de
   divergence d'interprétation, seule la version française fait foi.

Article 12 - NOUVELLES VERSIONS DU CONTRAT

   (( 12.1. ))??
   Toute personne est autorisée à copier et distribuer des copies de ce
   Contrat.

   (( 12.2. ))??
   Afin d'en préserver la cohérence, le texte du Contrat est protégé et ne
   peut être modifié que par les auteurs de la licence, lesquels se réservent
   le droit de publier périodiquement des mises à jour ou de nouvelles
   versions du Contrat, qui possèderont chacune un numéro distinct. Ces
   versions ultérieures seront susceptibles de prendre en compte de nouvelles
   problématiques rencontrées par les logiciels libres.

   (( 12.3. ))??
   Tout Logiciel diffusé sous une version donnée du Contrat ne pourra faire
   l'objet d'une diffusion ultérieure que sous la même version du Contrat ou
   une version postérieure, sous réserve des dispositions de l'article 5.3.4.

Article 13 - LOI APPLICABLE ET COMPETENCE TERRITORIALE

   (( 13.1. ))??
   Le Contrat est régi par la loi française. Les Parties conviennent de tenter
   de régler à l'amiable les différends ou litiges qui viendraient à se
   produire par suite ou à l'occasion du Contrat.

   (( 13.2. ))??
   A défaut d'accord amiable dans un délai de deux (2) mois à compter de leur
   survenance et sauf situation relevant d'une procédure d'urgence, les
   différends ou litiges seront portés par la Partie la plus diligente devant
   les Tribunaux compétents de Paris.

1 Ce: CEA, C: CNRS, I: INRIA, LL: Logiciel Libre
(( Version 1 du 21/06/2004 ))??
`
const license_CECILL_1_1_lre = `//**
CeCILL Free Software License Agreement v1.1
https://spdx.org/licenses/CECILL-1.1.json
http://www.cecill.info/licences/Licence_CeCILL_V1.1-US.html
**//

(( FREE SOFTWARE LICENSING AGREEMENT CeCILL ))??

(( Notice

This Agreement is a free software license that is the result of discussions
between its authors in order to ensure compliance with the two main principles
guiding its drafting:

   -
   firstly, its conformity with French law, both as regards the law of torts and
   intellectual property law, and the protection that it offers to authors and
   the holders of economic rights over software.

   -
   secondly, compliance with the principles for the distribution of free
   software: access to source codes, extended user-rights.

The following bodies are the authors of this license CeCILL (Ce : CEA, C : CNRS,
I : INRIA, LL : Logiciel Libre):

   Commissariat à l'Energie Atomique - CEA, a public scientific, technical and
   industrial establishment, having its principal place of business at 31-33 rue
   de la Fédération, 75752 PARIS cedex 15, France.

   Centre National de la Recherche Scientifique - CNRS, a public scientific and
   technological establishment, having its principal place of business at 3 rue
   Michel-Ange 75794 Paris cedex 16, France.

   Institut National de Recherche en Informatique et en Automatique - INRIA, a
   public scientific and technological establishment, having its principal place
   of business at Domaine de Voluceau, Rocquencourt, BP 105, 78153 Le Chesnay
   cedex. ))??

   PREAMBLE

   The purpose of this Free Software Licensing Agreement is to grant users the
   right to modify and redistribute the software governed by this license within
   the framework of an "open source" distribution model.

   The exercising of these rights is conditional upon certain obligations for
   users so as to ensure that this status is retained for subsequent
   redistribution operations.

   As a counterpart to the access to the source code and rights to copy, modify
   and redistribute granted by the license, users are provided only with a
   limited warranty and the software's author, the holder of the economic
   rights, and the successive licensors only have limited liability.

   In this respect, it is brought to the user's attention that the risks
   associated with loading, using, modifying and/or developing or reproducing
   the software by the user given its nature of Free Software, that may mean
   that it is complicated to manipulate, and that also therefore means that it
   is reserved for developers and experienced professionals having in-depth
   computer knowledge. Users are therefore encouraged to load and test the
   Software's suitability as regards their requirements in conditions enabling
   the security of their systems and/or data to be ensured and, more generally,
   to use and operate it in the same conditions of security. This Agreement may
   be freely reproduced and published, provided it is not altered, and that no
   Articles are either added or removed herefrom.

   This Agreement may apply to any or all software for which the holder of the
   economic rights decides to submit the operation thereof to its provisions.

   (( Article 1 - ))??
   DEFINITIONS

   For the purposes of this Agreement, when the following expressions commence
   with a capital letter, they shall have the following meaning:

      Agreement: means this Licensing Agreement, and any or all of its
      subsequent versions.

      Software: means the software in its Object Code and/or Source Code form
      and, where applicable, its documentation, "as is" at the time when the
      Licensee accepts the Agreement.

      Initial Software: means the Software in its Source Code and/or Object Code
      form and, where applicable, its documentation, "as is" at the time when it
      is distributed for the first time under the terms and conditions of the
      Agreement.

      Modified Software: means the Software modified by at least one
      Contribution.

      Source Code: means all the Software's instructions and program lines to
      which access is required so as to modify the Software.

      Object Code: means the binary files originating from the compilation of
      the Source Code.

      Holder: means the holder of the economic rights over the Initial
      Software.

      Licensee(s): mean(s) the Software user(s) having accepted the Agreement.

      Contributor: means a Licensee having made at least one Contribution.

      Licensor: means the Holder, or any or all other individual or legal
      entity, that distributes the Software under the Agreement.

      Contributions: mean any or all modifications, corrections, translations,
      adaptations and/or new functionalities integrated into the Software by any
      or all Contributor, and the Static Modules.

      Module: means a set of sources files including their documentation that,
      once compiled in executable form, enables supplementary functionalities or
      services to be developed in addition to those offered by the Software.

      Dynamic Module: means any or all module, created by the Contributor, that
      is independent of the Software, so that this module and the Software are
      in two different executable forms that are run in separate address spaces,
      with one calling the other when they are run.

      Static Module: means any or all module, created by the Contributor and
      connected to the Software by a static link that makes their object codes
      interdependent. This module and the Software to which it is connected, are
      combined in a single executable.

      Parties: mean both the Licensee and the Licensor.

   These expressions may be used both in singular and plural form.

   (( Article 2 - ))??
   PURPOSE

   The purpose of the Agreement is to enable the Licensor to grant the Licensee
   a free, non-exclusive, transferable and worldwide License for the Software as
   set forth in Article 5 hereinafter for the whole term of protection of the
   rights over said Software.

   (( Article 3 - ))??
   ACCEPTANCE

      (( 3.1. ))??
      The Licensee shall be deemed as having accepted the terms and conditions
      of this Agreement by the occurrence of the first of the following events:

         (( (i) ))??
         loading the Software by any or all means, notably, by downloading from
         a remote server, or by loading from a physical medium;

         (( (ii) ))??
         the first time the Licensee exercises any of the rights granted
         hereunder.

      (( 3.2. ))??
      One copy of the Agreement, containing a notice relating to the specific
      nature of the Software, to the limited warranty, and to the limitation to
      use by experienced users has been provided to the Licensee prior to its
      acceptance as set forth in Article 3.1 hereinabove, and the Licensee
      hereby acknowledges that it is aware thereof.

   (( Article 4 - ))??
   EFFECTIVE DATE AND TERM

      (( 4.1. ))??
      EFFECTIVE DATE

      The Agreement shall become effective on the date when it is accepted by
      the Licensee as set forth in Article 3.1.

      (( 4.2. ))??
      TERM

      The Agreement shall remain in force during the whole legal term of
      protection of the economic rights over the Software.

   (( Article 5 - ))??
   SCOPE OF THE RIGHTS GRANTED

   The Licensor hereby grants to the Licensee, that accepts such, the following
   rights as regards the Software for any or all use, and for the term of the
   Agreement, on the basis of the terms and conditions set forth hereinafter.

   Otherwise, the Licensor grants to the Licensee free of charge exploitation
   rights on the patents he holds on whole or part of the inventions implemented
   in the Software.

      (( 5.1. ))??
      RIGHTS OF USE

      The Licensee is authorized to use the Software, unrestrictedly, as regards
      the fields of application, with it being hereinafter specified that this
      relates to:

         (( 1. ))??
         permanent or temporary reproduction of all or part of the Software by
         any or all means and in any or all form.

         (( 2. ))??
         loading, displaying, running, or storing the Software on any or all
         medium.

         (( 3. ))??
         entitlement to observe, study or test the operation thereof so as to
         establish the ideas and principles that form the basis for any or all
         constituent elements of said Software. This shall apply when the
         Licensee carries out any or all loading, displaying, running,
         transmission or storage operation as regards the Software, that it is
         entitled to carry out hereunder.

      (( 5.2. ))??
      ENTITLEMENT TO MAKE CONTRIBUTIONS

      The right to make Contributions includes the right to translate, adapt,
      arrange, or make any or all modification to the Software, and the right to
      reproduce the resulting Software.

      The Licensee is authorized to make any or all Contribution to the Software
      provided that it explicitly mentions its name as the author of said
      Contribution and the date of the development thereof.

      (( 5.3. ))??
      DISTRIBUTION AND PUBLICATION RIGHTS

      In particular, the right of distribution and publication includes the
      right to transmit and communicate the Software to the general public on
      any or all medium, and by any or all means, and the right to market,
      either in consideration of a fee, or free of charge, a copy or copies of
      the Software by means of any or all process. The Licensee is further
      authorized to redistribute copies of the modified or unmodified Software
      to third parties according to the terms and conditions set forth
      hereinafter.

         (( 5.3.1. ))??
         REDISTRIBUTION OF SOFTWARE WITHOUT MODIFICATION

         The Licensee is authorized to redistribute true copies of the Software
         in Source Code or Object Code form, provided that said redistribution
         complies with all the provisions of the Agreement and is accompanied
         by:

            (( 1. ))??
            a copy of the Agreement,

            (( 2. ))??
            a notice relating to the limitation of both the Licensor's warranty
            and liability as set forth in Articles 8 and 9,

         and that, in the event that only the Software's Object Code is
         redistributed, the Licensee allows future Licensees unhindered access
         to the Software's full Source Code by providing them with the terms and
         conditions for access thereto, it being understood that the additional
         cost of acquiring the Source Code shall not exceed the cost of
         transferring the data.

         (( 5.3.2. ))??
         REDISTRIBUTION OF MODIFIED SOFTWARE

         When the Licensee makes a Contribution to the Software, the terms and
         conditions for the redistribution of the Modified Software shall then
         be subject to all the provisions hereof.

         The Licensee is authorized to redistribute the Modified Software, in
         Source Code or Object Code form, provided that said redistribution
         complies with all the provisions of the Agreement and is accompanied
         by:

            (( 1. ))??
            a copy of the Agreement,

            (( 2. ))??
            a notice relating to the limitation of both the Licensor's warranty
            and liability as set forth in Articles 8 and 9,

         and that, in the event that only the Modified Software's Object Code is
         redistributed, the Licensee allows future Licensees unhindered access
         to the Modified Software's full Source Code by providing them with the
         terms and conditions for access thereto, it being understood that the
         additional cost of acquiring the Source Code shall not exceed the cost
         of transferring the data.

         (( 5.3.3. ))??
         REDISTRIBUTION OF DYNAMIC MODULES

         When the Licensee has developed a Dynamic Module, the terms and
         conditions hereof do not apply to said Dynamic Module, that may be
         distributed under a separate Licensing Agreement.

         (( 5.3.4. ))??
         COMPATIBILITY WITH THE GPL LICENSE

         In the event that the Modified or unmodified Software is included in a
         code that is subject to the provisions of the GPL License, the Licensee
         is authorized to redistribute the whole under the GPL License.

         In the event that the Modified Software includes a code that is subject
         to the provisions of the GPL License, the Licensee is authorized to
         redistribute the Modified Software under the GPL License.

   (( Article 6 - ))??
   INTELLECTUAL PROPERTY

      (( 6.1. ))??
      OVER THE INITIAL SOFTWARE

      The Holder owns the economic rights over the Initial Software. Any or all
      use of the Initial Software is subject to compliance with the terms and
      conditions under which the Holder has elected to distribute its work and
      no one shall be entitled to and it shall have sole entitlement to modify
      the terms and conditions for the distribution of said Initial Software.

      The Holder undertakes to maintain the distribution of the Initial Software
      under the conditions of the Agreement, for the duration set forth in
      article 4.2..

      (( 6.2. ))??
      OVER THE CONTRIBUTIONS

      The intellectual property rights over the Contributions belong to the
      holder of the economic rights as designated by effective legislation.

      (( 6.3. ))??
      OVER THE DYNAMIC MODULES

      The Licensee having developed a Dynamic Module is the holder of the
      intellectual property rights over said Dynamic Module and is free to
      choose the agreement that shall govern its distribution.

      (( 6.4. ))??
      JOINT PROVISIONS

         6.4.1. The Licensee expressly undertakes:

            (( 1. ))??
            not to remove, or modify, in any or all manner, the intellectual
            property notices affixed to the Software;

            (( 2. ))??
            to reproduce said notices, in an identical manner, in the copies of
            the Software.

         6.4.2. The Licensee undertakes not to directly or indirectly infringe
         the intellectual property rights of the Holder and/or Contributors and
         to take, where applicable, vis-à-vis its staff, any or all measures
         required to ensure respect for said intellectual property rights of the
         Holder and/or Contributors.

   (( Article 7 - ))??
   RELATED SERVICES

      (( 7.1. ))??
      Under no circumstances shall the Agreement oblige the Licensor to provide
      technical assistance or maintenance services for the Software.

      However, the Licensor is entitled to offer this type of service. The terms
      and conditions of such technical assistance, and/or such maintenance,
      shall then be set forth in a separate instrument. Only the Licensor
      offering said maintenance and/or technical assistance services shall incur
      liability therefor.

      (( 7.2. ))??
      Similarly, any or all Licensor shall be entitled to offer to its
      Licensees, under its own responsibility, a warranty, that shall only be
      binding upon itself, for the redistribution of the Software and/or the
      Modified Software, under terms and conditions that it shall decide upon
      itself. Said warranty, and the financial terms and conditions of its
      application, shall be subject to a separate instrument executed between
      the Licensor and the Licensee.

   (( Article 8 - ))??
   LIABILITY

      (( 8.1. ))??
      Subject to the provisions of Article 8.2, should the Licensor fail to
      fulfill all or part of its obligations hereunder, the Licensee shall be
      entitled to claim compensation for the direct loss suffered as a result of
      a fault on the part of the Licensor, subject to providing evidence of it.

      (( 8.2. ))??
      The Licensor's liability is limited to the commitments made under this
      Licensing Agreement and shall not be incurred as a result , in particular:
      (i) of loss due the Licensee's total or partial failure to fulfill its
      obligations, (ii) direct or consequential loss due to the Software's use
      or performance that is suffered by the Licensee, when the latter is a
      professional using said Software for professional purposes and (iii)
      consequential loss due to the Software's use or performance. The Parties
      expressly agree that any or all pecuniary or business loss (i.e. loss of
      data, loss of profits, operating loss, loss of customers or orders,
      opportunity cost, any disturbance to business activities) or any or all
      legal proceedings instituted against the Licensee by a third party, shall
      constitute consequential loss and shall not provide entitlement to any or
      all compensation from the Licensor.

   (( Article 9 - ))??
   WARRANTY

      (( 9.1. ))??
      The Licensee acknowledges that the current situation as regards scientific
      and technical know-how at the time when the Software was distributed did
      not enable all possible uses to be tested and verified, nor for the
      presence of any or all faults to be detected. In this respect, the
      Licensee's attention has been drawn to the risks associated with loading,
      using, modifying and/or developing and reproducing the Software that are
      reserved for experienced users.

      The Licensee shall be responsible for verifying, by any or all means, the
      product's suitability for its requirements, its due and proper
      functioning, and for ensuring that it shall not cause damage to either
      persons or property.

      (( 9.2. ))??
      The Licensor hereby represents, in good faith, that it is entitled to
      grant all the rights on the Software (including in particular the rights
      set forth in Article 5 hereof over the Software).

      (( 9.3. ))??
      The Licensee acknowledges that the Software is supplied "as is" by the
      Licensor without any or all other express or tacit warranty, other than
      that provided for in Article 9.2 and, in particular, without any or all
      warranty as to its market value, its secured, innovative or relevant
      nature.

      Specifically, the Licensor does not warrant that the Software is free from
      any or all error, that it shall operate continuously, that it shall be
      compatible with the Licensee's own equipment and its software
      configuration, nor that it shall meet the Licensee's requirements.

      (( 9.4. ))??
      The Licensor does not either expressly or tacitly warrant that the
      Software does not infringe any or all third party intellectual right
      relating to a patent, software or to any or all other property right.
      Moreover, the Licensor shall not hold the Licensee harmless against any or
      all proceedings for infringement that may be instituted in respect of the
      use, modification and redistribution of the Software. Nevertheless, should
      such proceedings be instituted against the Licensee, the Licensor shall
      provide it with technical and legal assistance for its defense. Such
      technical and legal assistance shall be decided upon on a case-by-case
      basis between the relevant Licensor and the Licensee pursuant to a
      memorandum of understanding. The Licensor disclaims any or all liability
      as regards the Licensee's use of the Software's name. No warranty shall be
      provided as regards the existence of prior rights over the name of the
      Software and as regards the existence of a trademark.

   (( Article 10 - ))??
   TERMINATION

      (( 10.1. ))??
      In the event of a breach by the Licensee of its obligations hereunder, the
      Licensor may automatically terminate this Agreement thirty (30) days after
      notice has been sent to the Licensee and has remained ineffective.

      (( 10.2. ))??
      The Licensee whose Agreement is terminated shall no longer be authorized
      to use, modify or distribute the Software. However, any or all licenses
      that it may have granted prior to termination of the Agreement shall
      remain valid subject to their having been granted in compliance with the
      terms and conditions hereof.

   (( Article 11 - ))??
   MISCELLANEOUS PROVISIONS

      (( 11.1. ))??
      EXCUSABLE EVENTS

      Neither Party shall be liable for any or all delay, or failure to perform
      the Agreement, that may be attributable to an event of force majeure, an
      act of God or an outside cause, such as, notably, defective functioning,
      or interruptions affecting the electricity or telecommunications networks,
      blocking of the network following a virus attack, the intervention of the
      government authorities, natural disasters, water damage, earthquakes,
      fire, explosions, strikes and labor unrest, war, etc.

      (( 11.2. ))??
      The fact that either Party may fail, on one or several occasions, to
      invoke one or several of the provisions hereof, shall under no
      circumstances be interpreted as being a waiver by the interested Party of
      its entitlement to invoke said provision(s) subsequently.

      (( 11.3. ))??
      The Agreement cancels and replaces any or all previous agreement, whether
      written or oral, between the Parties and having the same purpose, and
      constitutes the entirety of the agreement between said Parties concerning
      said purpose. No supplement or modification to the terms and conditions
      hereof shall be effective as regards the Parties unless it is made in
      writing and signed by their duly authorized representatives.

      (( 11.4. ))??
      In the event that one or several of the provisions hereof were to conflict
      with a current or future applicable act or legislative text, said act or
      legislative text shall take precedence, and the Parties shall make the
      necessary amendments so as to be in compliance with said act or
      legislative text. All the other provisions shall remain effective.
      Similarly, the fact that a provision of the Agreement may be null and
      void, for any reason whatsoever, shall not cause the Agreement as a whole
      to be null and void.

      (( 11.5. ))??
      LANGUAGE

      The Agreement is drafted in both French and English. In the event of a
      conflict as regards construction, the French version shall be deemed
      authentic.

   (( Article 12 - ))??
   NEW VERSIONS OF THE AGREEMENT

      (( 12.1. ))??
      Any or all person is authorized to duplicate and distribute copies of this
      Agreement.

      (( 12.2. ))??
      So as to ensure coherence, the wording of this Agreement is protected and
      may only be modified by the authors of the License, that reserve the right
      to periodically publish updates or new versions of the Agreement, each
      with a separate number. These subsequent versions may address new issues
      encountered by Free Software.

      (( 12.3. ))??
      Any or all Software distributed under a given version of the Agreement may
      only be subsequently distributed under the same version of the Agreement,
      or a subsequent version, subject to the provisions of article 5.3.4.

   (( Article 13 - ))??
   GOVERNING LAW AND JURISDICTION

      (( 13.1. ))??
      The Agreement is governed by French law. The Parties agree to endeavor to
      settle the disagreements or disputes that may arise during the performance
      of the Agreement out-of-court.

      (( 13.2. ))??
      In the absence of an out-of-court settlement within two (2) months as from
      their occurrence, and unless emergency proceedings are necessary, the
      disagreements or disputes shall be referred to the Paris Courts having
      jurisdiction, by the first Party to take action.

Version 1.1 of 10/26/2004
`
const license_CECILL_2_0_lre = `//**
CeCILL Free Software License Agreement v2.0
https://spdx.org/licenses/CECILL-2.0.json
http://www.cecill.info/licences/Licence_CeCILL_V2-en.html
**//

(( CeCILL FREE SOFTWARE LICENSE AGREEMENT ))??

(( Notice

This Agreement is a Free Software license agreement that is the result of
discussions between its authors in order to ensure compliance with the two main
principles guiding its drafting:

   *
   firstly, compliance with the principles governing the distribution of Free
   Software: access to source code, broad rights granted to users,

   *
   secondly, the election of a governing law, French law, with which it is
   conformant, both as regards the law of torts and intellectual property law,
   and the protection that it offers to both authors and holders of the economic
   rights over software.

The authors of the
//** CeCILL¹ **//
__5__
license are:

   Commissariat à l'Energie Atomique - CEA, a public scientific, technical and
   industrial research establishment, having its principal place of business at
   25 rue Leblanc, immeuble Le Ponant D, 75015 Paris, France.

   Centre National de la Recherche Scientifique - CNRS, a public scientific and
   technological establishment, having its principal place of business at 3 rue
   Michel-Ange, 75794 Paris cedex 16, France.

   Institut National de Recherche en Informatique et en Automatique - INRIA, a
   public scientific and technological establishment, having its principal place
   of business at Domaine de Voluceau, Rocquencourt, BP 105, 78153 Le Chesnay
   cedex, France. ))??

   (( Preamble ))??
   The purpose of this Free Software license agreement is to grant users the
   right to modify and redistribute the software governed by this license within
   the framework of an open source distribution model.

   The exercising of these rights is conditional upon certain obligations for
   users so as to preserve this status for all subsequent redistributions.

   In consideration of access to the source code and the rights to copy, modify
   and redistribute granted by the license, users are provided only with a
   limited warranty and the software's author, the holder of the economic
   rights, and the successive licensors only have limited liability.

   In this respect, the risks associated with loading, using, modifying and/or
   developing or reproducing the software by the user are brought to the user's
   attention, given its Free Software status, which may make it complicated to
   use, with the result that its use is reserved for developers and experienced
   professionals having in-depth computer knowledge. Users are therefore
   encouraged to load and test the suitability of the software as regards their
   requirements in conditions enabling the security of their systems and/or data
   to be ensured and, more generally, to use and operate it in the same
   conditions of security. This Agreement may be freely reproduced and
   published, provided it is not altered, and that no provisions are either
   added or removed herefrom.

   This Agreement may apply to any or all software for which the holder of the
   economic rights decides to submit the use thereof to its provisions.

   (( Article 1 - ))??
   DEFINITIONS

   For the purpose of this Agreement, when the following expressions commence
   with a capital letter, they shall have the following meaning:

      Agreement: means this license agreement, and its possible subsequent
      versions and annexes.

      Software: means the software in its Object Code and/or Source Code form
      and, where applicable, its documentation, "as is" when the Licensee
      accepts the Agreement.

      Initial Software: means the Software in its Source Code and possibly its
      Object Code form and, where applicable, its documentation, "as is" when it
      is first distributed under the terms and conditions of the Agreement.

      Modified Software: means the Software modified by at least one
      Contribution.

      Source Code: means all the Software's instructions and program lines to
      which access is required so as to modify the Software.

      Object Code: means the binary files originating from the compilation of
      the Source Code.

      Holder: means the holder(s) of the economic rights over the Initial
      Software.

      Licensee: means the Software user(s) having accepted the Agreement.

      Contributor: means a Licensee having made at least one Contribution.

      Licensor: means the Holder, or any other individual or legal entity, who
      distributes the Software under the Agreement.

      Contribution: means any or all modifications, corrections, translations,
      adaptations and/or new functions integrated into the Software by any or
      all Contributors, as well as any or all Internal Modules.

      Module: means a set of sources files including their documentation that
      enables supplementary functions or services in addition to those offered
      by the Software.

      External Module: means any or all Modules, not derived from the Software,
      so that this Module and the Software run in separate address spaces, with
      one calling the other when they are run.

      Internal Module: means any or all Module, connected to the Software so
      that they both execute in the same address space.

      GNU GPL: means the GNU General Public License version 2 or any subsequent
      version, as published by the Free Software Foundation Inc.

      Parties: mean both the Licensee and the Licensor.

   These expressions may be used both in singular and plural form.

   (( Article 2 - ))??
   PURPOSE

   The purpose of the Agreement is the grant by the Licensor to the Licensee of
   a non-exclusive, transferable and worldwide license for the Software as set
   forth in Article 5 hereinafter for the whole term of the protection granted
   by the rights over said Software.

   (( Article 3 - ))??
   ACCEPTANCE

      (( 3.1 ))??
      The Licensee shall be deemed as having accepted the terms and conditions
      of this Agreement upon the occurrence of the first of the following
      events:

         (( (i) ))??
         loading the Software by any or all means, notably, by downloading from
         a remote server, or by loading from a physical medium;

         (( (ii) ))??
         the first time the Licensee exercises any of the rights granted
         hereunder.

      (( 3.2 ))??
      One copy of the Agreement, containing a notice relating to the
      characteristics of the Software, to the limited warranty, and to the fact
      that its use is restricted to experienced users has been provided to the
      Licensee prior to its acceptance as set forth in Article 3.1 hereinabove,
      and the Licensee hereby acknowledges that it has read and understood it.

   (( Article 4 - ))??
   EFFECTIVE DATE AND TERM

      (( 4.1 ))??
      EFFECTIVE DATE

      The Agreement shall become effective on the date when it is accepted by
      the Licensee as set forth in Article 3.1.

      (( 4.2 ))??
      TERM

      The Agreement shall remain in force for the entire legal term of
      protection of the economic rights over the Software.

   (( Article 5 - ))??
   SCOPE OF RIGHTS GRANTED

   The Licensor hereby grants to the Licensee, who accepts, the following rights
   over the Software for any or all use, and for the term of the Agreement, on
   the basis of the terms and conditions set forth hereinafter.

   Besides, if the Licensor owns or comes to own one or more patents protecting
   all or part of the functions of the Software or of its components, the
   Licensor undertakes not to enforce the rights granted by these patents
   against successive Licensees using, exploiting or modifying the Software. If
   these patents are transferred, the Licensor undertakes to have the
   transferees subscribe to the obligations set forth in this paragraph.

      (( 5.1 ))??
      RIGHT OF USE

      The Licensee is authorized to use the Software, without any limitation as
      to its fields of application, with it being hereinafter specified that
      this comprises:

         (( 1. ))??
         permanent or temporary reproduction of all or part of the Software by
         any or all means and in any or all form.

         (( 2. ))??
         loading, displaying, running, or storing the Software on any or all
         medium.

         (( 3. ))??
         entitlement to observe, study or test its operation so as to determine
         the ideas and principles behind any or all constituent elements of said
         Software. This shall apply when the Licensee carries out any or all
         loading, displaying, running, transmission or storage operation as
         regards the Software, that it is entitled to carry out hereunder.

      (( 5.2 ))??
      ENTITLEMENT TO MAKE CONTRIBUTIONS

      The right to make Contributions includes the right to translate, adapt,
      arrange, or make any or all modifications to the Software, and the right
      to reproduce the resulting software.

      The Licensee is authorized to make any or all Contributions to the
      Software provided that it includes an explicit notice that it is the
      author of said Contribution and indicates the date of the creation
      thereof.

      (( 5.3 ))??
      RIGHT OF DISTRIBUTION

      In particular, the right of distribution includes the right to publish,
      transmit and communicate the Software to the general public on any or all
      medium, and by any or all means, and the right to market, either in
      consideration of a fee, or free of charge, one or more copies of the
      Software by any means.

      The Licensee is further authorized to distribute copies of the modified or
      unmodified Software to third parties according to the terms and conditions
      set forth hereinafter.

         (( 5.3.1. ))??
         DISTRIBUTION OF SOFTWARE WITHOUT MODIFICATION

         The Licensee is authorized to distribute true copies of the Software in
         Source Code or Object Code form, provided that said distribution
         complies with all the provisions of the Agreement and is accompanied
         by:

            (( 1. ))??
            a copy of the Agreement,

            (( 2. ))??
            a notice relating to the limitation of both the Licensor's warranty
            and liability as set forth in Articles 8 and 9,

         and that, in the event that only the Object Code of the Software is
         redistributed, the Licensee allows future Licensees unhindered access
         to the full Source Code of the Software by indicating how to access it,
         it being understood that the additional cost of acquiring the Source
         Code shall not exceed the cost of transferring the data.

         (( 5.3.2. ))??
         DISTRIBUTION OF MODIFIED SOFTWARE

         When the Licensee makes a Contribution to the Software, the terms and
         conditions for the distribution of the resulting Modified Software
         become subject to all the provisions of this Agreement.

         The Licensee is authorized to distribute the Modified Software, in
         source code or object code form, provided that said distribution
         complies with all the provisions of the Agreement and is accompanied
         by:

            (( 1. ))??
            a copy of the Agreement,

            (( 2. ))??
            a notice relating to the limitation of both the Licensor's warranty
            and liability as set forth in Articles 8 and 9,

         and that, in the event that only the object code of the Modified
         Software is redistributed, the Licensee allows future Licensees
         unhindered access to the full source code of the Modified Software by
         indicating how to access it, it being understood that the additional
         cost of acquiring the source code shall not exceed the cost of
         transferring the data.

         (( 5.3.3. ))??
         DISTRIBUTION OF EXTERNAL MODULES

         When the Licensee has developed an External Module, the terms and
         conditions of this Agreement do not apply to said External Module, that
         may be distributed under a separate license agreement.

         (( 5.3.4. ))??
         COMPATIBILITY WITH THE GNU GPL

         The Licensee can include a code that is subject to the provisions of
         one of the versions of the GNU GPL in the Modified or unmodified
         Software, and distribute that entire code under the terms of the same
         version of the GNU GPL.

         The Licensee can include the Modified or unmodified Software in a code
         that is subject to the provisions of one of the versions of the GNU
         GPL, and distribute that entire code under the terms of the same
         version of the GNU GPL.

   (( Article 6 - ))??
   INTELLECTUAL PROPERTY

      (( 6.1 ))??
      OVER THE INITIAL SOFTWARE

      The Holder owns the economic rights over the Initial Software. Any or all
      use of the Initial Software is subject to compliance with the terms and
      conditions under which the Holder has elected to distribute its work and
      no one shall be entitled to modify the terms and conditions for the
      distribution of said Initial Software.

      The Holder undertakes that the Initial Software will remain ruled at least
      by this Agreement, for the duration set forth in Article 4.2.

      (( 6.2 ))??
      OVER THE CONTRIBUTIONS

      The Licensee who develops a Contribution is the owner of the intellectual
      property rights over this Contribution as defined by applicable law.

      (( 6.3 ))??
      OVER THE EXTERNAL MODULES

      The Licensee who develops an External Module is the owner of the
      intellectual property rights over this External Module as defined by
      applicable law and is free to choose the type of agreement that shall
      govern its distribution.

      (( 6.4 ))??
      JOINT PROVISIONS

      The Licensee expressly undertakes:

         (( 1. ))??
         not to remove, or modify, in any manner, the intellectual property
         notices attached to the Software;

         (( 2. ))??
         to reproduce said notices, in an identical manner, in the copies of the
         Software modified or not.

      The Licensee undertakes not to directly or indirectly infringe the
      intellectual property rights of the Holder and/or Contributors on the
      Software and to take, where applicable, vis-à-vis its staff, any and all
      measures required to ensure respect of said intellectual property rights
      of the Holder and/or Contributors.

   (( Article 7 - ))??
   RELATED SERVICES

      (( 7.1 ))??
      Under no circumstances shall the Agreement oblige the Licensor to provide
      technical assistance or maintenance services for the Software.

      However, the Licensor is entitled to offer this type of services. The
      terms and conditions of such technical assistance, and/or such
      maintenance, shall be set forth in a separate instrument. Only the
      Licensor offering said maintenance and/or technical assistance services
      shall incur liability therefor.

      (( 7.2 ))??
      Similarly, any Licensor is entitled to offer to its licensees, under its
      sole responsibility, a warranty, that shall only be binding upon itself,
      for the redistribution of the Software and/or the Modified Software, under
      terms and conditions that it is free to decide. Said warranty, and the
      financial terms and conditions of its application, shall be subject of a
      separate instrument executed between the Licensor and the Licensee.

   (( Article 8 - ))??
   LIABILITY

      (( 8.1 ))??
      Subject to the provisions of Article 8.2, the Licensee shall be entitled
      to claim compensation for any direct loss it may have suffered from the
      Software as a result of a fault on the part of the relevant Licensor,
      subject to providing evidence thereof.

      (( 8.2 ))??
      The Licensor's liability is limited to the commitments made under this
      Agreement and shall not be incurred as a result of in particular: (i) loss
      due the Licensee's total or partial failure to fulfill its obligations,
      (ii) direct or consequential loss that is suffered by the Licensee due to
      the use or performance of the Software, and (iii) more generally, any
      consequential loss. In particular the Parties expressly agree that any or
      all pecuniary or business loss (i.e. loss of data, loss of profits,
      operating loss, loss of customers or orders, opportunity cost, any
      disturbance to business activities) or any or all legal proceedings
      instituted against the Licensee by a third party, shall constitute
      consequential loss and shall not provide entitlement to any or all
      compensation from the Licensor.

   (( Article 9 - ))??
   WARRANTY

      (( 9.1 ))??
      The Licensee acknowledges that the scientific and technical
      state-of-the-art when the Software was distributed did not enable all
      possible uses to be tested and verified, nor for the presence of possible
      defects to be detected. In this respect, the Licensee's attention has been
      drawn to the risks associated with loading, using, modifying and/or
      developing and reproducing the Software which are reserved for experienced
      users.

      The Licensee shall be responsible for verifying, by any or all means, the
      suitability of the product for its requirements, its good working order,
      and for ensuring that it shall not cause damage to either persons or
      properties.

      (( 9.2 ))??
      The Licensor hereby represents, in good faith, that it is entitled to
      grant all the rights over the Software (including in particular the rights
      set forth in Article 5).

      (( 9.3 ))??
      The Licensee acknowledges that the Software is supplied "as is" by the
      Licensor without any other express or tacit warranty, other than that
      provided for in Article 9.2 and, in particular, without any warranty as to
      its commercial value, its secured, safe, innovative or relevant nature.

      Specifically, the Licensor does not warrant that the Software is free from
      any error, that it will operate without interruption, that it will be
      compatible with the Licensee's own equipment and software configuration,
      nor that it will meet the Licensee's requirements.

      (( 9.4 ))??
      The Licensor does not either expressly or tacitly warrant that the
      Software does not infringe any third party intellectual property right
      relating to a patent, software or any other property right. Therefore, the
      Licensor disclaims any and all liability towards the Licensee arising out
      of any or all proceedings for infringement that may be instituted in
      respect of the use, modification and redistribution of the Software.
      Nevertheless, should such proceedings be instituted against the Licensee,
      the Licensor shall provide it with technical and legal assistance for its
      defense. Such technical and legal assistance shall be decided on a
      case-by-case basis between the relevant Licensor and the Licensee pursuant
      to a memorandum of understanding. The Licensor disclaims any and all
      liability as regards the Licensee's use of the name of the Software. No
      warranty is given as regards the existence of prior rights over the name
      of the Software or as regards the existence of a trademark.

   (( Article 10 - ))??
   TERMINATION

      (( 10.1 ))??
      In the event of a breach by the Licensee of its obligations hereunder, the
      Licensor may automatically terminate this Agreement thirty (30) days after
      notice has been sent to the Licensee and has remained ineffective.

      (( 10.2 ))??
      A Licensee whose Agreement is terminated shall no longer be authorized to
      use, modify or distribute the Software. However, any licenses that it may
      have granted prior to termination of the Agreement shall remain valid
      subject to their having been granted in compliance with the terms and
      conditions hereof.

   (( Article 11 - ))??
   MISCELLANEOUS

      (( 11.1 ))??
      EXCUSABLE EVENTS

      Neither Party shall be liable for any or all delay, or failure to perform
      the Agreement, that may be attributable to an event of force majeure, an
      act of God or an outside cause, such as defective functioning or
      interruptions of the electricity or telecommunications networks, network
      paralysis following a virus attack, intervention by government
      authorities, natural disasters, water damage, earthquakes, fire,
      explosions, strikes and labor unrest, war, etc.

      (( 11.2 ))??
      Any failure by either Party, on one or more occasions, to invoke one or
      more of the provisions hereof, shall under no circumstances be interpreted
      as being a waiver by the interested Party of its right to invoke said
      provision(s) subsequently.

      (( 11.3 ))??
      The Agreement cancels and replaces any or all previous agreements, whether
      written or oral, between the Parties and having the same purpose, and
      constitutes the entirety of the agreement between said Parties concerning
      said purpose. No supplement or modification to the terms and conditions
      hereof shall be effective as between the Parties unless it is made in
      writing and signed by their duly authorized representatives.

      (( 11.4 ))??
      In the event that one or more of the provisions hereof were to conflict
      with a current or future applicable act or legislative text, said act or
      legislative text shall prevail, and the Parties shall make the necessary
      amendments so as to comply with said act or legislative text. All other
      provisions shall remain effective. Similarly, invalidity of a provision of
      the Agreement, for any reason whatsoever, shall not cause the Agreement as
      a whole to be invalid.

      (( 11.5 ))??
      LANGUAGE

      The Agreement is drafted in both French and English and both versions are
      deemed authentic.

   (( Article 12 - ))??
   NEW VERSIONS OF THE AGREEMENT

      (( 12.1 ))??
      Any person is authorized to duplicate and distribute copies of this
      Agreement.

      (( 12.2 ))??
      So as to ensure coherence, the wording of this Agreement is protected and
      may only be modified by the authors of the License, who reserve the right
      to periodically publish updates or new versions of the Agreement, each
      with a separate number. These subsequent versions may address new issues
      encountered by Free Software.

      (( 12.3 ))??
      Any Software distributed under a given version of the Agreement may only
      be subsequently distributed under the same version of the Agreement or a
      subsequent version, subject to the provisions of Article 5.3.4.

   (( Article 13 - ))??
   GOVERNING LAW AND JURISDICTION

      (( 13.1 ))??
      The Agreement is governed by French law. The Parties agree to endeavor to
      seek an amicable solution to any disagreements or disputes that may arise
      during the performance of the Agreement.

      (( 13.2 ))??
      Failing an amicable solution within two (2) months as from their
      occurrence, and unless emergency proceedings are necessary, the
      disagreements or disputes shall be referred to the Paris Courts having
      jurisdiction, by the more diligent Party.

Version 2.0 dated 2006-09-05.
(( 1 CeCILL stands for Ce(a) C(nrs) I(nria) L(ogiciel) L(ibre) ))??
`
const license_CECILL_2_1_lre = `//**
CeCILL Free Software License Agreement v2.1
https://spdx.org/licenses/CECILL-2.1.json
http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.html
**//

(( CeCILL FREE SOFTWARE LICENSE AGREEMENT ))??

Version 2.1 dated 2013-06-21
(( Notice

This Agreement is a Free Software license agreement that is the result of
discussions between its authors in order to ensure compliance with the two main
principles guiding its drafting:

   *
   firstly, compliance with the principles governing the distribution of Free
   Software: access to source code, broad rights granted to users,

   *
   secondly, the election of a governing law, French law, with which it is
   conformant, both as regards the law of torts and intellectual property law,
   and the protection that it offers to both authors and holders of the economic
   rights over software.

The authors of the
//** CeCILL¹ **//
__5__
license are:

   Commissariat à l'énergie atomique et aux énergies alternatives - CEA, a
   public scientific, technical and industrial research establishment, having
   its principal place of business at 25 rue Leblanc, immeuble Le Ponant D,
   75015 Paris, France.

   Centre National de la Recherche Scientifique - CNRS, a public scientific and
   technological establishment, having its principal place of business at 3 rue
   Michel-Ange, 75794 Paris cedex 16, France.

   Institut National de Recherche en Informatique et en Automatique - Inria, a
   public scientific and technological establishment, having its principal place
   of business at Domaine de Voluceau, Rocquencourt, BP 105, 78153 Le Chesnay
   cedex, France. ))??

   (( Preamble ))??
   The purpose of this Free Software license agreement is to grant users the
   right to modify and redistribute the software governed by this license within
   the framework of an open source distribution model.

   The exercising of this right is conditional upon certain obligations for
   users so as to preserve this status for all subsequent redistributions.

   In consideration of access to the source code and the rights to copy, modify
   and redistribute granted by the license, users are provided only with a
   limited warranty and the software's author, the holder of the economic
   rights, and the successive licensors only have limited liability.

   In this respect, the risks associated with loading, using, modifying and/or
   developing or reproducing the software by the user are brought to the user's
   attention, given its Free Software status, which may make it complicated to
   use, with the result that its use is reserved for developers and experienced
   professionals having in-depth computer knowledge. Users are therefore
   encouraged to load and test the suitability of the software as regards their
   requirements in conditions enabling the security of their systems and/or data
   to be ensured and, more generally, to use and operate it in the same
   conditions of security. This Agreement may be freely reproduced and
   published, provided it is not altered, and that no provisions are either
   added or removed herefrom.

   This Agreement may apply to any or all software for which the holder of the
   economic rights decides to submit the use thereof to its provisions.

   Frequently asked questions can be found on the official website of the CeCILL
   licenses family (http:/www.cecill.info/index.en.html) for any necessary
   clarification.

   (( Article 1 - ))??
   DEFINITIONS

   For the purpose of this Agreement, when the following expressions commence
   with a capital letter, they shall have the following meaning:

      Agreement: means this license agreement, and its possible subsequent
      versions and annexes.

      Software: means the software in its Object Code and/or Source Code form
      and, where applicable, its documentation, "as is" when the Licensee
      accepts the Agreement.

      Initial Software: means the Software in its Source Code and possibly its
      Object Code form and, where applicable, its documentation, "as is" when it
      is first distributed under the terms and conditions of the Agreement.

      Modified Software: means the Software modified by at least one
      Contribution.

      Source Code: means all the Software's instructions and program lines to
      which access is required so as to modify the Software.

      Object Code: means the binary files originating from the compilation of
      the Source Code.

      Holder: means the holder(s) of the economic rights over the Initial
      Software.

      Licensee: means the Software user(s) having accepted the Agreement.

      Contributor: means a Licensee having made at least one Contribution.

      Licensor: means the Holder, or any other individual or legal entity, who
      distributes the Software under the Agreement.

      Contribution: means any or all modifications, corrections, translations,
      adaptations and/or new functions integrated into the Software by any or
      all Contributors, as well as any or all Internal Modules.

      Module: means a set of sources files including their documentation that
      enables supplementary functions or services in addition to those offered
      by the Software.

      External Module: means any or all Modules, not derived from the Software,
      so that this Module and the Software run in separate address spaces, with
      one calling the other when they are run.

      Internal Module: means any or all Module, connected to the Software so
      that they both execute in the same address space.

      GNU GPL: means the GNU General Public License version 2 or any subsequent
      version, as published by the Free Software Foundation Inc.

      GNU Affero GPL: means the GNU Affero General Public License version 3 or
      any subsequent version, as published by the Free Software Foundation Inc.

      EUPL: means the European Union Public License version 1.1 or any
      subsequent version, as published by the European Commission.

      Parties: mean both the Licensee and the Licensor.

   These expressions may be used both in singular and plural form.

   (( Article 2 - ))??
   PURPOSE

   The purpose of the Agreement is the grant by the Licensor to the Licensee of
   a non-exclusive, transferable and worldwide license for the Software as set
   forth in Article 5
   (( <#scope> ))??
   hereinafter for the whole term of the protection granted by the rights over
   said Software.

   (( Article 3 - ))??
   ACCEPTANCE

      (( 3.1 ))??
      The Licensee shall be deemed as having accepted the terms and conditions
      of this Agreement upon the occurrence of the first of the following
      events:

         (( (i) ))??
         loading the Software by any or all means, notably, by downloading from
         a remote server, or by loading from a physical medium;

         (( (ii) ))??
         the first time the Licensee exercises any of the rights granted
         hereunder.

      (( 3.2 ))??
      One copy of the Agreement, containing a notice relating to the
      characteristics of the Software, to the limited warranty, and to the fact
      that its use is restricted to experienced users has been provided to the
      Licensee prior to its acceptance as set forth in Article 3.1
      (( <#accepting> ))??
      hereinabove, and the Licensee hereby acknowledges that it has read and
      understood it.

   (( Article 4 - ))??
   EFFECTIVE DATE AND TERM

      (( 4.1 ))??
      EFFECTIVE DATE

      The Agreement shall become effective on the date when it is accepted by
      the Licensee as set forth in Article 3.1
      (( <#accepting> ))??

      (( 4.2 ))??
      TERM

      The Agreement shall remain in force for the entire legal term of
      protection of the economic rights over the Software.

   (( Article 5 - ))??
   SCOPE OF RIGHTS GRANTED

   The Licensor hereby grants to the Licensee, who accepts, the following rights
   over the Software for any or all use, and for the term of the Agreement, on
   the basis of the terms and conditions set forth hereinafter.

   Besides, if the Licensor owns or comes to own one or more patents protecting
   all or part of the functions of the Software or of its components, the
   Licensor undertakes not to enforce the rights granted by these patents
   against successive Licensees using, exploiting or modifying the Software. If
   these patents are transferred, the Licensor undertakes to have the
   transferees subscribe to the obligations set forth in this paragraph.

      (( 5.1 ))??
      RIGHT OF USE

      The Licensee is authorized to use the Software, without any limitation as
      to its fields of application, with it being hereinafter specified that
      this comprises:

         (( 1. ))??
         permanent or temporary reproduction of all or part of the Software by
         any or all means and in any or all form.

         (( 2. ))??
         loading, displaying, running, or storing the Software on any or all
         medium.

         (( 3. ))??
         entitlement to observe, study or test its operation so as to determine
         the ideas and principles behind any or all constituent elements of said
         Software. This shall apply when the Licensee carries out any or all
         loading, displaying, running, transmission or storage operation as
         regards the Software, that it is entitled to carry out hereunder.

      (( 5.2 ))??
      ENTITLEMENT TO MAKE CONTRIBUTIONS

      The right to make Contributions includes the right to translate, adapt,
      arrange, or make any or all modifications to the Software, and the right
      to reproduce the resulting software.

      The Licensee is authorized to make any or all Contributions to the
      Software provided that it includes an explicit notice that it is the
      author of said Contribution and indicates the date of the creation
      thereof.

      (( 5.3 ))??
      RIGHT OF DISTRIBUTION

      In particular, the right of distribution includes the right to publish,
      transmit and communicate the Software to the general public on any or all
      medium, and by any or all means, and the right to market, either in
      consideration of a fee, or free of charge, one or more copies of the
      Software by any means.

      The Licensee is further authorized to distribute copies of the modified or
      unmodified Software to third parties according to the terms and conditions
      set forth hereinafter.

         (( 5.3.1. ))??
         DISTRIBUTION OF SOFTWARE WITHOUT MODIFICATION

         The Licensee is authorized to distribute true copies of the Software in
         Source Code or Object Code form, provided that said distribution
         complies with all the provisions of the Agreement and is accompanied
         by:

            (( 1. ))??
            a copy of the Agreement,

            (( 2. ))??
            a notice relating to the limitation of both the Licensor's warranty
            and liability as set forth in Articles 8 and 9,

         and that, in the event that only the Object Code of the Software is
         redistributed, the Licensee allows effective access to the full Source
         Code of the Software for a period of at least three years from the
         distribution of the Software, it being understood that the additional
         acquisition cost of the Source Code shall not exceed the cost of the
         data transfer.

         (( 5.3.2. ))??
         DISTRIBUTION OF MODIFIED SOFTWARE

         When the Licensee makes a Contribution to the Software, the terms and
         conditions for the distribution of the resulting Modified Software
         become subject to all the provisions of this Agreement.

         The Licensee is authorized to distribute the Modified Software, in
         source code or object code form, provided that said distribution
         complies with all the provisions of the Agreement and is accompanied
         by:

            (( 1. ))??
            a copy of the Agreement,

            (( 2. ))??
            a notice relating to the limitation of both the Licensor's warranty
            and liability as set forth in Articles 8 and 9,

         and, in the event that only the object code of the Modified Software is
         redistributed,

            (( 3. ))??
            a note stating the conditions of effective access to the full source
            code of the Modified Software for a period of at least three years
            from the distribution of the Modified Software, it being understood
            that the additional acquisition cost of the source code shall not
            exceed the cost of the data transfer.

         (( 5.3.3. ))??
         DISTRIBUTION OF EXTERNAL MODULES

         When the Licensee has developed an External Module, the terms and
         conditions of this Agreement do not apply to said External Module, that
         may be distributed under a separate license agreement.

         (( 5.3.4. ))??
         COMPATIBILITY WITH OTHER LICENSES

         The Licensee can include a code that is subject to the provisions of
         one of the versions of the GNU GPL, GNU Affero GPL and/or EUPL in the
         Modified or unmodified Software, and distribute that entire code under
         the terms of the same version of the GNU GPL, GNU Affero GPL and/or
         EUPL.

         The Licensee can include the Modified or unmodified Software in a code
         that is subject to the provisions of one of the versions of the GNU
         GPL, GNU Affero GPL and/or EUPL and distribute that entire code under
         the terms of the same version of the GNU GPL, GNU Affero GPL and/or
         EUPL.

   (( Article 6 - ))??
   INTELLECTUAL PROPERTY

      (( 6.1 ))??
      OVER THE INITIAL SOFTWARE

      The Holder owns the economic rights over the Initial Software. Any or all
      use of the Initial Software is subject to compliance with the terms and
      conditions under which the Holder has elected to distribute its work and
      no one shall be entitled to modify the terms and conditions for the
      distribution of said Initial Software.

      The Holder undertakes that the Initial Software will remain ruled at least
      by this Agreement, for the duration set forth in Article 4.2
      (( <#term> ))??

      (( 6.2 ))??
      OVER THE CONTRIBUTIONS

      The Licensee who develops a Contribution is the owner of the intellectual
      property rights over this Contribution as defined by applicable law.

      (( 6.3 ))??
      OVER THE EXTERNAL MODULES

      The Licensee who develops an External Module is the owner of the
      intellectual property rights over this External Module as defined by
      applicable law and is free to choose the type of agreement that shall
      govern its distribution.

      (( 6.4 ))??
      JOINT PROVISIONS

      The Licensee expressly undertakes:

         (( 1. ))??
         not to remove, or modify, in any manner, the intellectual property
         notices attached to the Software;

         (( 2. ))??
         to reproduce said notices, in an identical manner, in the copies of the
         Software modified or not.

      The Licensee undertakes not to directly or indirectly infringe the
      intellectual property rights on the Software of the Holder and/or
      Contributors, and to take, where applicable, vis-à-vis its staff, any and
      all measures required to ensure respect of said intellectual property
      rights of the Holder and/or Contributors.

   (( Article 7 - ))??
   RELATED SERVICES

      (( 7.1 ))??
      Under no circumstances shall the Agreement oblige the Licensor to provide
      technical assistance or maintenance services for the Software.

      However, the Licensor is entitled to offer this type of services. The
      terms and conditions of such technical assistance, and/or such
      maintenance, shall be set forth in a separate instrument. Only the
      Licensor offering said maintenance and/or technical assistance services
      shall incur liability therefor.

      (( 7.2 ))??
      Similarly, any Licensor is entitled to offer to its licensees, under its
      sole responsibility, a warranty, that shall only be binding upon itself,
      for the redistribution of the Software and/or the Modified Software, under
      terms and conditions that it is free to decide. Said warranty, and the
      financial terms and conditions of its application, shall be subject of a
      separate instrument executed between the Licensor and the Licensee.

   (( Article 8 - ))??
   LIABILITY

      (( 8.1 ))??
      Subject to the provisions of Article 8.2, the Licensee shall be entitled
      to claim compensation for any direct loss it may have suffered from the
      Software as a result of a fault on the part of the relevant Licensor,
      subject to providing evidence thereof.

      (( 8.2 ))??
      The Licensor's liability is limited to the commitments made under this
      Agreement and shall not be incurred as a result of in particular: (i) loss
      due the Licensee's total or partial failure to fulfill its obligations,
      (ii) direct or consequential loss that is suffered by the Licensee due to
      the use or performance of the Software, and (iii) more generally, any
      consequential loss. In particular the Parties expressly agree that any or
      all pecuniary or business loss (i.e. loss of data, loss of profits,
      operating loss, loss of customers or orders, opportunity cost, any
      disturbance to business activities) or any or all legal proceedings
      instituted against the Licensee by a third party, shall constitute
      consequential loss and shall not provide entitlement to any or all
      compensation from the Licensor.

   (( Article 9 - ))??
   WARRANTY

      (( 9.1 ))??
      The Licensee acknowledges that the scientific and technical
      state-of-the-art when the Software was distributed did not enable all
      possible uses to be tested and verified, nor for the presence of possible
      defects to be detected. In this respect, the Licensee's attention has been
      drawn to the risks associated with loading, using, modifying and/or
      developing and reproducing the Software which are reserved for experienced
      users.

      The Licensee shall be responsible for verifying, by any or all means, the
      suitability of the product for its requirements, its good working order,
      and for ensuring that it shall not cause damage to either persons or
      properties.

      (( 9.2 ))??
      The Licensor hereby represents, in good faith, that it is entitled to
      grant all the rights over the Software (including in particular the rights
      set forth in Article 5
      (( <#scope> ))??
      ).

      (( 9.3 ))??
      The Licensee acknowledges that the Software is supplied "as is" by the
      Licensor without any other express or tacit warranty, other than that
      provided for in Article 9.2
      (( <#good-faith> ))??
      and, in particular, without any warranty as to its commercial value, its
      secured, safe, innovative or relevant nature.

      Specifically, the Licensor does not warrant that the Software is free from
      any error, that it will operate without interruption, that it will be
      compatible with the Licensee's own equipment and software configuration,
      nor that it will meet the Licensee's requirements.

      (( 9.4 ))??
      The Licensor does not either expressly or tacitly warrant that the
      Software does not infringe any third party intellectual property right
      relating to a patent, software or any other property right. Therefore, the
      Licensor disclaims any and all liability towards the Licensee arising out
      of any or all proceedings for infringement that may be instituted in
      respect of the use, modification and redistribution of the Software.
      Nevertheless, should such proceedings be instituted against the Licensee,
      the Licensor shall provide it with technical and legal expertise for its
      defense. Such technical and legal expertise shall be decided on a
      case-by-case basis between the relevant Licensor and the Licensee pursuant
      to a memorandum of understanding. The Licensor disclaims any and all
      liability as regards the Licensee's use of the name of the Software. No
      warranty is given as regards the existence of prior rights over the name
      of the Software or as regards the existence of a trademark.

   (( Article 10 - ))??
   TERMINATION

      (( 10.1 ))??
      In the event of a breach by the Licensee of its obligations hereunder, the
      Licensor may automatically terminate this Agreement thirty (30) days after
      notice has been sent to the Licensee and has remained ineffective.

      (( 10.2 ))??
      A Licensee whose Agreement is terminated shall no longer be authorized to
      use, modify or distribute the Software. However, any licenses that it may
      have granted prior to termination of the Agreement shall remain valid
      subject to their having been granted in compliance with the terms and
      conditions hereof.

   (( Article 11 - ))??
   MISCELLANEOUS

      (( 11.1 ))??
      EXCUSABLE EVENTS

      Neither Party shall be liable for any or all delay, or failure to perform
      the Agreement, that may be attributable to an event of force majeure, an
      act of God or an outside cause, such as defective functioning or
      interruptions of the electricity or telecommunications networks, network
      paralysis following a virus attack, intervention by government
      authorities, natural disasters, water damage, earthquakes, fire,
      explosions, strikes and labor unrest, war, etc.

      (( 11.2 ))??
      Any failure by either Party, on one or more occasions, to invoke one or
      more of the provisions hereof, shall under no circumstances be interpreted
      as being a waiver by the interested Party of its right to invoke said
      provision(s) subsequently.

      (( 11.3 ))??
      The Agreement cancels and replaces any or all previous agreements, whether
      written or oral, between the Parties and having the same purpose, and
      constitutes the entirety of the agreement between said Parties concerning
      said purpose. No supplement or modification to the terms and conditions
      hereof shall be effective as between the Parties unless it is made in
      writing and signed by their duly authorized representatives.

      (( 11.4 ))??
      In the event that one or more of the provisions hereof were to conflict
      with a current or future applicable act or legislative text, said act or
      legislative text shall prevail, and the Parties shall make the necessary
      amendments so as to comply with said act or legislative text. All other
      provisions shall remain effective. Similarly, invalidity of a provision of
      the Agreement, for any reason whatsoever, shall not cause the Agreement as
      a whole to be invalid.

      (( 11.5 ))??
      LANGUAGE

      The Agreement is drafted in both French and English and both versions are
      deemed authentic.

   (( Article 12 - ))??
   NEW VERSIONS OF THE AGREEMENT

      (( 12.1 ))??
      Any person is authorized to duplicate and distribute copies of this
      Agreement.

      (( 12.2 ))??
      So as to ensure coherence, the wording of this Agreement is protected and
      may only be modified by the authors of the License, who reserve the right
      to periodically publish updates or new versions of the Agreement, each
      with a separate number. These subsequent versions may address new issues
      encountered by Free Software.

      (( 12.3 ))??
      Any Software distributed under a given version of the Agreement may only
      be subsequently distributed under the same version of the Agreement or a
      subsequent version, subject to the provisions of Article 5.3.4
      (( <#compatibility> ))??

   (( Article 13 - ))??
   GOVERNING LAW AND JURISDICTION

      (( 13.1 ))??
      The Agreement is governed by French law. The Parties agree to endeavor to
      seek an amicable solution to any disagreements or disputes that may arise
      during the performance of the Agreement.

      (( 13.2 ))??
      Failing an amicable solution within two (2) months as from their
      occurrence, and unless emergency proceedings are necessary, the
      disagreements or disputes shall be referred to the Paris Courts having
      jurisdiction, by the more diligent Party.
      (( 1 CeCILL stands for Ce(a) C(nrs) I(nria) L(ogiciel) L(ibre) ))??
`
const license_CECILL_B_lre = `//**
CeCILL-B Free Software License Agreement
https://spdx.org/licenses/CECILL-B.json
http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
**//

(( CeCILL-B FREE SOFTWARE LICENSE AGREEMENT ))??

(( Notice

This Agreement is a Free Software license agreement that is the result of
discussions between its authors in order to ensure compliance with the two main
principles guiding its drafting:

   *
   firstly, compliance with the principles governing the distribution of Free
   Software: access to source code, broad rights granted to users,

   *
   secondly, the election of a governing law, French law, with which it is
   conformant, both as regards the law of torts and intellectual property law,
   and the protection that it offers to both authors and holders of the economic
   rights over software.

The authors of the
//** CeCILL-B (for Ce[a] C[nrs] I[nria] L[ogiciel] L[ibre]) **//
__20__
license are:

   Commissariat à l'Energie Atomique - CEA, a public scientific, technical and
   industrial research establishment, having its principal place of business at
   25 rue Leblanc, immeuble Le Ponant D, 75015 Paris, France.

   Centre National de la Recherche Scientifique - CNRS, a public scientific and
   technological establishment, having its principal place of business at 3 rue
   Michel-Ange, 75794 Paris cedex 16, France.

   Institut National de Recherche en Informatique et en Automatique - INRIA, a
   public scientific and technological establishment, having its principal place
   of business at Domaine de Voluceau, Rocquencourt, BP 105, 78153 Le Chesnay
   cedex, France. ))??

   (( Preamble ))??
   This Agreement is an open source software license intended to give users
   significant freedom to modify and redistribute the software licensed
   hereunder.

   The exercising of this freedom is conditional upon a strong obligation of
   giving credits for everybody that distributes a software incorporating a
   software ruled by the current license so as all contributions to be properly
   identified and acknowledged.

   In consideration of access to the source code and the rights to copy, modify
   and redistribute granted by the license, users are provided only with a
   limited warranty and the software's author, the holder of the economic
   rights, and the successive licensors only have limited liability.

   In this respect, the risks associated with loading, using, modifying and/or
   developing or reproducing the software by the user are brought to the user's
   attention, given its Free Software status, which may make it complicated to
   use, with the result that its use is reserved for developers and experienced
   professionals having in-depth computer knowledge. Users are therefore
   encouraged to load and test the suitability of the software as regards their
   requirements in conditions enabling the security of their systems and/or data
   to be ensured and, more generally, to use and operate it in the same
   conditions of security. This Agreement may be freely reproduced and
   published, provided it is not altered, and that no provisions are either
   added or removed herefrom.

   This Agreement may apply to any or all software for which the holder of the
   economic rights decides to submit the use thereof to its provisions.

   (( Article 1 - ))??
   DEFINITIONS

   For the purpose of this Agreement, when the following expressions commence
   with a capital letter, they shall have the following meaning:

      Agreement: means this license agreement, and its possible subsequent
      versions and annexes.

      Software: means the software in its Object Code and/or Source Code form
      and, where applicable, its documentation, "as is" when the Licensee
      accepts the Agreement.

      Initial Software: means the Software in its Source Code and possibly its
      Object Code form and, where applicable, its documentation, "as is" when it
      is first distributed under the terms and conditions of the Agreement.

      Modified Software: means the Software modified by at least one
      Contribution.

      Source Code: means all the Software's instructions and program lines to
      which access is required so as to modify the Software.

      Object Code: means the binary files originating from the compilation of
      the Source Code.

      Holder: means the holder(s) of the economic rights over the Initial
      Software.

      Licensee: means the Software user(s) having accepted the Agreement.

      Contributor: means a Licensee having made at least one Contribution.

      Licensor: means the Holder, or any other individual or legal entity, who
      distributes the Software under the Agreement.

      Contribution: means any or all modifications, corrections, translations,
      adaptations and/or new functions integrated into the Software by any or
      all Contributors, as well as any or all Internal Modules.

      Module: means a set of sources files including their documentation that
      enables supplementary functions or services in addition to those offered
      by the Software.

      External Module: means any or all Modules, not derived from the Software,
      so that this Module and the Software run in separate address spaces, with
      one calling the other when they are run.

      Internal Module: means any or all Module, connected to the Software so
      that they both execute in the same address space.

      Parties: mean both the Licensee and the Licensor.

   These expressions may be used both in singular and plural form.

   (( Article 2 - ))??
   PURPOSE

   The purpose of the Agreement is the grant by the Licensor to the Licensee of
   a non-exclusive, transferable and worldwide license for the Software as set
   forth in Article 5 hereinafter for the whole term of the protection granted
   by the rights over said Software.

   (( Article 3 - ))??
   ACCEPTANCE

      (( 3.1 ))??
      The Licensee shall be deemed as having accepted the terms and conditions
      of this Agreement upon the occurrence of the first of the following
      events:

         (( (i) ))??
         loading the Software by any or all means, notably, by downloading from
         a remote server, or by loading from a physical medium;

         (( (ii) ))??
         the first time the Licensee exercises any of the rights granted
         hereunder.

      (( 3.2 ))??
      One copy of the Agreement, containing a notice relating to the
      characteristics of the Software, to the limited warranty, and to the fact
      that its use is restricted to experienced users has been provided to the
      Licensee prior to its acceptance as set forth in Article 3.1 hereinabove,
      and the Licensee hereby acknowledges that it has read and understood it.

   (( Article 4 - ))??
   EFFECTIVE DATE AND TERM

      (( 4.1 ))??
      EFFECTIVE DATE

      The Agreement shall become effective on the date when it is accepted by
      the Licensee as set forth in Article 3.1.

      (( 4.2 ))??
      TERM

      The Agreement shall remain in force for the entire legal term of
      protection of the economic rights over the Software.

   (( Article 5 - ))??
   SCOPE OF RIGHTS GRANTED

   The Licensor hereby grants to the Licensee, who accepts, the following rights
   over the Software for any or all use, and for the term of the Agreement, on
   the basis of the terms and conditions set forth hereinafter.

   Besides, if the Licensor owns or comes to own one or more patents protecting
   all or part of the functions of the Software or of its components, the
   Licensor undertakes not to enforce the rights granted by these patents
   against successive Licensees using, exploiting or modifying the Software. If
   these patents are transferred, the Licensor undertakes to have the
   transferees subscribe to the obligations set forth in this paragraph.

      (( 5.1 ))??
      RIGHT OF USE

      The Licensee is authorized to use the Software, without any limitation as
      to its fields of application, with it being hereinafter specified that
      this comprises:

         (( 1. ))??
         permanent or temporary reproduction of all or part of the Software by
         any or all means and in any or all form.

         (( 2. ))??
         loading, displaying, running, or storing the Software on any or all
         medium.

         (( 3. ))??
         entitlement to observe, study or test its operation so as to determine
         the ideas and principles behind any or all constituent elements of said
         Software. This shall apply when the Licensee carries out any or all
         loading, displaying, running, transmission or storage operation as
         regards the Software, that it is entitled to carry out hereunder.

      (( 5.2 ))??
      ENTITLEMENT TO MAKE CONTRIBUTIONS

      The right to make Contributions includes the right to translate, adapt,
      arrange, or make any or all modifications to the Software, and the right
      to reproduce the resulting software.

      The Licensee is authorized to make any or all Contributions to the
      Software provided that it includes an explicit notice that it is the
      author of said Contribution and indicates the date of the creation
      thereof.

      (( 5.3 ))??
      RIGHT OF DISTRIBUTION

      In particular, the right of distribution includes the right to publish,
      transmit and communicate the Software to the general public on any or all
      medium, and by any or all means, and the right to market, either in
      consideration of a fee, or free of charge, one or more copies of the
      Software by any means.

      The Licensee is further authorized to distribute copies of the modified or
      unmodified Software to third parties according to the terms and conditions
      set forth hereinafter.

         (( 5.3.1. ))??
         DISTRIBUTION OF SOFTWARE WITHOUT MODIFICATION

         The Licensee is authorized to distribute true copies of the Software in
         Source Code or Object Code form, provided that said distribution
         complies with all the provisions of the Agreement and is accompanied
         by:

            (( 1. ))??
            a copy of the Agreement,

            (( 2. ))??
            a notice relating to the limitation of both the Licensor's warranty
            and liability as set forth in Articles 8 and 9,

         and that, in the event that only the Object Code of the Software is
         redistributed, the Licensee allows effective access to the full Source
         Code of the Software at a minimum during the entire period of its
         distribution of the Software, it being understood that the additional
         cost of acquiring the Source Code shall not exceed the cost of
         transferring the data.

         (( 5.3.2. ))??
         DISTRIBUTION OF MODIFIED SOFTWARE

         If the Licensee makes any Contribution to the Software, the resulting
         Modified Software may be distributed under a license agreement other
         than this Agreement subject to compliance with the provisions of
         Article 5.3.4.

         (( 5.3.3. ))??
         DISTRIBUTION OF EXTERNAL MODULES

         When the Licensee has developed an External Module, the terms and
         conditions of this Agreement do not apply to said External Module, that
         may be distributed under a separate license agreement.

         (( 5.3.4. ))??
         CREDITS

         Any Licensee who may distribute a Modified Software hereby expressly
         agrees to:

            (( 1. ))??
            indicate in the related documentation that it is based on the
            Software licensed hereunder, and reproduce the intellectual property
            notice for the Software,

            (( 2. ))??
            ensure that written indications of the Software intended use,
            intellectual property notice and license hereunder are included in
            easily accessible format from the Modified Software interface,

            (( 3. ))??
            mention, on a freely accessible website describing the Modified
            Software, at least throughout the distribution term thereof, that it
            is based on the Software licensed hereunder, and reproduce the
            Software intellectual property notice,

            (( 4. ))??
            where it is distributed to a third party that may distribute a
            Modified Software without having to make its source code available,
            make its best efforts to ensure that said third party agrees to
            comply with the obligations set forth in this Article .

         If the Software, whether or not modified, is distributed with an
         External Module designed for use in connection with the Software, the
         Licensee shall submit said External Module to the foregoing
         obligations.

         (( 5.3.5. ))??
         COMPATIBILITY WITH THE CeCILL AND CeCILL-C LICENSES

         Where a Modified Software contains a Contribution subject to the CeCILL
         license, the provisions set forth in Article 5.3.4 shall be optional.

         A Modified Software may be distributed under the CeCILL-C license. In
         such a case the provisions set forth in Article 5.3.4 shall be
         optional.

   (( Article 6 - ))??
   INTELLECTUAL PROPERTY

      (( 6.1 ))??
      OVER THE INITIAL SOFTWARE

      The Holder owns the economic rights over the Initial Software. Any or all
      use of the Initial Software is subject to compliance with the terms and
      conditions under which the Holder has elected to distribute its work and
      no one shall be entitled to modify the terms and conditions for the
      distribution of said Initial Software.

      The Holder undertakes that the Initial Software will remain ruled at least
      by this Agreement, for the duration set forth in Article 4.2.

      (( 6.2 ))??
      OVER THE CONTRIBUTIONS

      The Licensee who develops a Contribution is the owner of the intellectual
      property rights over this Contribution as defined by applicable law.

      (( 6.3 ))??
      OVER THE EXTERNAL MODULES

      The Licensee who develops an External Module is the owner of the
      intellectual property rights over this External Module as defined by
      applicable law and is free to choose the type of agreement that shall
      govern its distribution.

      (( 6.4 ))??
      JOINT PROVISIONS

      The Licensee expressly undertakes:

         (( 1. ))??
         not to remove, or modify, in any manner, the intellectual property
         notices attached to the Software;

         (( 2. ))??
         to reproduce said notices, in an identical manner, in the copies of the
         Software modified or not.

      The Licensee undertakes not to directly or indirectly infringe the
      intellectual property rights of the Holder and/or Contributors on the
      Software and to take, where applicable, vis-à-vis its staff, any and all
      measures required to ensure respect of said intellectual property rights
      of the Holder and/or Contributors.

   (( Article 7 - ))??
   RELATED SERVICES

      (( 7.1 ))??
      Under no circumstances shall the Agreement oblige the Licensor to provide
      technical assistance or maintenance services for the Software.

      However, the Licensor is entitled to offer this type of services. The
      terms and conditions of such technical assistance, and/or such
      maintenance, shall be set forth in a separate instrument. Only the
      Licensor offering said maintenance and/or technical assistance services
      shall incur liability therefor.

      (( 7.2 ))??
      Similarly, any Licensor is entitled to offer to its licensees, under its
      sole responsibility, a warranty, that shall only be binding upon itself,
      for the redistribution of the Software and/or the Modified Software, under
      terms and conditions that it is free to decide. Said warranty, and the
      financial terms and conditions of its application, shall be subject of a
      separate instrument executed between the Licensor and the Licensee.

   (( Article 8 - ))??
   LIABILITY

      (( 8.1 ))??
      Subject to the provisions of Article 8.2, the Licensee shall be entitled
      to claim compensation for any direct loss it may have suffered from the
      Software as a result of a fault on the part of the relevant Licensor,
      subject to providing evidence thereof.

      (( 8.2 ))??
      The Licensor's liability is limited to the commitments made under this
      Agreement and shall not be incurred as a result of in particular: (i) loss
      due the Licensee's total or partial failure to fulfill its obligations,
      (ii) direct or consequential loss that is suffered by the Licensee due to
      the use or performance of the Software, and (iii) more generally, any
      consequential loss. In particular the Parties expressly agree that any or
      all pecuniary or business loss (i.e. loss of data, loss of profits,
      operating loss, loss of customers or orders, opportunity cost, any
      disturbance to business activities) or any or all legal proceedings
      instituted against the Licensee by a third party, shall constitute
      consequential loss and shall not provide entitlement to any or all
      compensation from the Licensor.

   (( Article 9 - ))??
   WARRANTY

      (( 9.1 ))??
      The Licensee acknowledges that the scientific and technical
      state-of-the-art when the Software was distributed did not enable all
      possible uses to be tested and verified, nor for the presence of possible
      defects to be detected. In this respect, the Licensee's attention has been
      drawn to the risks associated with loading, using, modifying and/or
      developing and reproducing the Software which are reserved for experienced
      users.

      The Licensee shall be responsible for verifying, by any or all means, the
      suitability of the product for its requirements, its good working order,
      and for ensuring that it shall not cause damage to either persons or
      properties.

      (( 9.2 ))??
      The Licensor hereby represents, in good faith, that it is entitled to
      grant all the rights over the Software (including in particular the rights
      set forth in Article 5).

      (( 9.3 ))??
      The Licensee acknowledges that the Software is supplied "as is" by the
      Licensor without any other express or tacit warranty, other than that
      provided for in Article 9.2 and, in particular, without any warranty as to
      its commercial value, its secured, safe, innovative or relevant nature.

      Specifically, the Licensor does not warrant that the Software is free from
      any error, that it will operate without interruption, that it will be
      compatible with the Licensee's own equipment and software configuration,
      nor that it will meet the Licensee's requirements.

      (( 9.4 ))??
      The Licensor does not either expressly or tacitly warrant that the
      Software does not infringe any third party intellectual property right
      relating to a patent, software or any other property right. Therefore, the
      Licensor disclaims any and all liability towards the Licensee arising out
      of any or all proceedings for infringement that may be instituted in
      respect of the use, modification and redistribution of the Software.
      Nevertheless, should such proceedings be instituted against the Licensee,
      the Licensor shall provide it with technical and legal assistance for its
      defense. Such technical and legal assistance shall be decided on a
      case-by-case basis between the relevant Licensor and the Licensee pursuant
      to a memorandum of understanding. The Licensor disclaims any and all
      liability as regards the Licensee's use of the name of the Software. No
      warranty is given as regards the existence of prior rights over the name
      of the Software or as regards the existence of a trademark.

   (( Article 10 - ))??
   TERMINATION

      (( 10.1 ))??
      In the event of a breach by the Licensee of its obligations hereunder, the
      Licensor may automatically terminate this Agreement thirty (30) days after
      notice has been sent to the Licensee and has remained ineffective.

      (( 10.2 ))??
      A Licensee whose Agreement is terminated shall no longer be authorized to
      use, modify or distribute the Software. However, any licenses that it may
      have granted prior to termination of the Agreement shall remain valid
      subject to their having been granted in compliance with the terms and
      conditions hereof.

   (( Article 11 - ))??
   MISCELLANEOUS

      (( 11.1 ))??
      EXCUSABLE EVENTS

      Neither Party shall be liable for any or all delay, or failure to perform
      the Agreement, that may be attributable to an event of force majeure, an
      act of God or an outside cause, such as defective functioning or
      interruptions of the electricity or telecommunications networks, network
      paralysis following a virus attack, intervention by government
      authorities, natural disasters, water damage, earthquakes, fire,
      explosions, strikes and labor unrest, war, etc.

      (( 11.2 ))??
      Any failure by either Party, on one or more occasions, to invoke one or
      more of the provisions hereof, shall under no circumstances be interpreted
      as being a waiver by the interested Party of its right to invoke said
      provision(s) subsequently.

      (( 11.3 ))??
      The Agreement cancels and replaces any or all previous agreements, whether
      written or oral, between the Parties and having the same purpose, and
      constitutes the entirety of the agreement between said Parties concerning
      said purpose. No supplement or modification to the terms and conditions
      hereof shall be effective as between the Parties unless it is made in
      writing and signed by their duly authorized representatives.

      (( 11.4 ))??
      In the event that one or more of the provisions hereof were to conflict
      with a current or future applicable act or legislative text, said act or
      legislative text shall prevail, and the Parties shall make the necessary
      amendments so as to comply with said act or legislative text. All other
      provisions shall remain effective. Similarly, invalidity of a provision of
      the Agreement, for any reason whatsoever, shall not cause the Agreement as
      a whole to be invalid.

      (( 11.5 ))??
      LANGUAGE

      The Agreement is drafted in both French and English and both versions are
      deemed authentic.

   (( Article 12 - ))??
   NEW VERSIONS OF THE AGREEMENT

      (( 12.1 ))??
      Any person is authorized to duplicate and distribute copies of this
      Agreement.

      (( 12.2 ))??
      So as to ensure coherence, the wording of this Agreement is protected and
      may only be modified by the authors of the License, who reserve the right
      to periodically publish updates or new versions of the Agreement, each
      with a separate number. These subsequent versions may address new issues
      encountered by Free Software.

      (( 12.3 ))??
      Any Software distributed under a given version of the Agreement may only
      be subsequently distributed under the same version of the Agreement or a
      subsequent version.

   (( Article 13 - ))??
   GOVERNING LAW AND JURISDICTION

      (( 13.1 ))??
      The Agreement is governed by French law. The Parties agree to endeavor to
      seek an amicable solution to any disagreements or disputes that may arise
      during the performance of the Agreement.

      (( 13.2 ))??
      Failing an amicable solution within two (2) months as from their
      occurrence, and unless emergency proceedings are necessary, the
      disagreements or disputes shall be referred to the Paris Courts having
      jurisdiction, by the more diligent Party.

Version 1.0 dated 2006-09-05.
(( 1 CeCILL stands for Ce(a) C(nrs) I(nria) L(ogiciel) L(ibre) ))??
`
const license_CECILL_C_lre = `//**
CeCILL-C Free Software License Agreement
https://spdx.org/licenses/CECILL-C.json
http://www.cecill.info/licences/Licence_CeCILL-C_V1-en.html
**//

(( CeCILL-C FREE SOFTWARE LICENSE AGREEMENT ))??

(( Notice

This Agreement is a Free Software license agreement that is the result of
discussions between its authors in order to ensure compliance with the two main
principles guiding its drafting:

   *
   firstly, compliance with the principles governing the distribution of Free
   Software: access to source code, broad rights granted to users,

   *
   secondly, the election of a governing law, French law, with which it is
   conformant, both as regards the law of torts and intellectual property law,
   and the protection that it offers to both authors and holders of the economic
   rights over software.

The authors of the
//** CeCILL-B (for Ce[a] C[nrs] I[nria] L[ogiciel] L[ibre]) **//
__20__
license are:

   Commissariat à l'Energie Atomique - CEA, a public scientific, technical and
   industrial research establishment, having its principal place of business at
   25 rue Leblanc, immeuble Le Ponant D, 75015 Paris, France.

   Centre National de la Recherche Scientifique - CNRS, a public scientific and
   technological establishment, having its principal place of business at 3 rue
   Michel-Ange, 75794 Paris cedex 16, France.

   Institut National de Recherche en Informatique et en Automatique - INRIA, a
   public scientific and technological establishment, having its principal place
   of business at Domaine de Voluceau, Rocquencourt, BP 105, 78153 Le Chesnay
   cedex, France. ))??

   (( Preamble ))??
   The purpose of this Free Software license agreement is to grant users the
   right to modify and re-use the software governed by this license.

   The exercising of this right is conditional upon the obligation to make
   available to the community the modifications made to the source code of the
   software so as to contribute to its evolution.

   In consideration of access to the source code and the rights to copy, modify
   and redistribute granted by the license, users are provided only with a
   limited warranty and the software's author, the holder of the economic
   rights, and the successive licensors only have limited liability.

   In this respect, the risks associated with loading, using, modifying and/or
   developing or reproducing the software by the user are brought to the user's
   attention, given its Free Software status, which may make it complicated to
   use, with the result that its use is reserved for developers and experienced
   professionals having in-depth computer knowledge. Users are therefore
   encouraged to load and test the suitability of the software as regards their
   requirements in conditions enabling the security of their systems and/or data
   to be ensured and, more generally, to use and operate it in the same
   conditions of security. This Agreement may be freely reproduced and
   published, provided it is not altered, and that no provisions are either
   added or removed herefrom.

   This Agreement may apply to any or all software for which the holder of the
   economic rights decides to submit the use thereof to its provisions.

   (( Article 1 - ))??
   DEFINITIONS

   For the purpose of this Agreement, when the following expressions commence
   with a capital letter, they shall have the following meaning:

      Agreement: means this license agreement, and its possible subsequent
      versions and annexes.

      Software: means the software in its Object Code and/or Source Code form
      and, where applicable, its documentation, "as is" when the Licensee
      accepts the Agreement.

      Initial Software: means the Software in its Source Code and possibly its
      Object Code form and, where applicable, its documentation, "as is" when it
      is first distributed under the terms and conditions of the Agreement.

      Modified Software: means the Software modified by at least one Integrated
      Contribution.

      Source Code: means all the Software's instructions and program lines to
      which access is required so as to modify the Software.

      Object Code: means the binary files originating from the compilation of
      the Source Code.

      Holder: means the holder(s) of the economic rights over the Initial
      Software.

      Licensee: means the Software user(s) having accepted the Agreement.

      Contributor: means a Licensee having made at least one Integrated
      Contribution.

      Licensor: means the Holder, or any other individual or legal entity, who
      distributes the Software under the Agreement.

      Integrated Contribution: means any or all modifications, corrections,
      translations, adaptations and/or new functions integrated into the Source
      Code by any or all Contributors.

      Related Module: means a set of sources files including their documentation
      that, without modification to the Source Code, enables supplementary
      functions or services in addition to those offered by the Software.

      Derivative Software: means any combination of the Software, modified or
      not, and of a Related Module.

      Parties: mean both the Licensee and the Licensor.

   These expressions may be used both in singular and plural form.

   (( Article 2 - ))??
   PURPOSE

   The purpose of the Agreement is the grant by the Licensor to the Licensee of
   a non-exclusive, transferable and worldwide license for the Software as set
   forth in Article 5 hereinafter for the whole term of the protection granted
   by the rights over said Software.

   (( Article 3 - ))??
   ACCEPTANCE

      (( 3.1 ))??
      The Licensee shall be deemed as having accepted the terms and conditions
      of this Agreement upon the occurrence of the first of the following
      events:

         (( (i) ))??
         loading the Software by any or all means, notably, by downloading from
         a remote server, or by loading from a physical medium;

         (( (ii) ))??
         the first time the Licensee exercises any of the rights granted
         hereunder.

      (( 3.2 ))??
      One copy of the Agreement, containing a notice relating to the
      characteristics of the Software, to the limited warranty, and to the fact
      that its use is restricted to experienced users has been provided to the
      Licensee prior to its acceptance as set forth in Article 3.1 hereinabove,
      and the Licensee hereby acknowledges that it has read and understood it.

   (( Article 4 - ))??
   EFFECTIVE DATE AND TERM

      (( 4.1 ))??
      EFFECTIVE DATE

      The Agreement shall become effective on the date when it is accepted by
      the Licensee as set forth in Article 3.1.

      (( 4.2 ))??
      TERM

      The Agreement shall remain in force for the entire legal term of
      protection of the economic rights over the Software.

   (( Article 5 - ))??
   SCOPE OF RIGHTS GRANTED

   The Licensor hereby grants to the Licensee, who accepts, the following rights
   over the Software for any or all use, and for the term of the Agreement, on
   the basis of the terms and conditions set forth hereinafter.

   Besides, if the Licensor owns or comes to own one or more patents protecting
   all or part of the functions of the Software or of its components, the
   Licensor undertakes not to enforce the rights granted by these patents
   against successive Licensees using, exploiting or modifying the Software. If
   these patents are transferred, the Licensor undertakes to have the
   transferees subscribe to the obligations set forth in this paragraph.

      (( 5.1 ))??
      RIGHT OF USE

      The Licensee is authorized to use the Software, without any limitation as
      to its fields of application, with it being hereinafter specified that
      this comprises:

         (( 1. ))??
         permanent or temporary reproduction of all or part of the Software by
         any or all means and in any or all form.

         (( 2. ))??
         loading, displaying, running, or storing the Software on any or all
         medium.

         (( 3. ))??
         entitlement to observe, study or test its operation so as to determine
         the ideas and principles behind any or all constituent elements of said
         Software. This shall apply when the Licensee carries out any or all
         loading, displaying, running, transmission or storage operation as
         regards the Software, that it is entitled to carry out hereunder.

      (( 5.2 ))??
      RIGHT OF MODIFICATION

      The right of modification includes the right to translate, adapt, arrange,
      or make any or all modifications to the Software, and the right to
      reproduce the resulting software. It includes, in particular, the right to
      create a Derivative Software.

      The Licensee is authorized to make any or all modification to the Software
      provided that it includes an explicit notice that it is the author of said
      modification and indicates the date of the creation thereof.

      (( 5.3 ))??
      RIGHT OF DISTRIBUTION

      In particular, the right of distribution includes the right to publish,
      transmit and communicate the Software to the general public on any or all
      medium, and by any or all means, and the right to market, either in
      consideration of a fee, or free of charge, one or more copies of the
      Software by any means.

      The Licensee is further authorized to distribute copies of the modified or
      unmodified Software to third parties according to the terms and conditions
      set forth hereinafter.

         (( 5.3.1. ))??
         DISTRIBUTION OF SOFTWARE WITHOUT MODIFICATION

         The Licensee is authorized to distribute true copies of the Software in
         Source Code or Object Code form, provided that said distribution
         complies with all the provisions of the Agreement and is accompanied
         by:

            (( 1. ))??
            a copy of the Agreement,

            (( 2. ))??
            a notice relating to the limitation of both the Licensor's warranty
            and liability as set forth in Articles 8 and 9,

         and that, in the event that only the Object Code of the Software is
         redistributed, the Licensee allows effective access to the full Source
         Code of the Software at a minimum during the entire period of its
         distribution of the Software, it being understood that the additional
         cost of acquiring the Source Code shall not exceed the cost of
         transferring the data.

         (( 5.3.2. ))??
         DISTRIBUTION OF MODIFIED SOFTWARE

         When the Licensee makes an Integrated Contribution to the Software, the
         terms and conditions for the distribution of the resulting Modified
         Software become subject to all the provisions of this Agreement.

         The Licensee is authorized to distribute the Modified Software, in
         source code or object code form, provided that said distribution
         complies with all the provisions of the Agreement and is accompanied
         by:

            (( 1. ))??
            a copy of the Agreement,

            (( 2. ))??
            a notice relating to the limitation of both the Licensor's warranty
            and liability as set forth in Articles 8 and 9,

         and that, in the event that only the object code of the Modified
         Software is redistributed, the Licensee allows effective access to the
         full source code of the Modified Software at a minimum during the
         entire period of its distribution of the Modified Software, it being
         understood that the additional cost of acquiring the source code shall
         not exceed the cost of transferring the data.

         (( 5.3.3. ))??
         DISTRIBUTION OF DERIVATIVE SOFTWARE

         When the Licensee creates Derivative Software, this Derivative Software
         may be distributed under a license agreement other than this Agreement,
         subject to compliance with the requirement to include a notice
         concerning the rights over the Software as defined in Article 6.4. In
         the event the creation of the Derivative Software required modification
         of the Source Code, the Licensee undertakes that:

            (( 1. ))??
            the resulting Modified Software will be governed by this Agreement,

            (( 2. ))??
            the Integrated Contributions in the resulting Modified Software will
            be clearly identified and documented,

            (( 3. ))??
            the Licensee will allow effective access to the source code of the
            Modified Software, at a minimum during the entire period of
            distribution of the Derivative Software, such that such
            modifications may be carried over in a subsequent version of the
            Software; it being understood that the additional cost of purchasing
            the source code of the Modified Software shall not exceed the cost
            of transferring the data.

         (( 5.3.4. ))??
         COMPATIBILITY WITH THE CeCILL LICENSE

         When a Modified Software contains an Integrated Contribution subject to
         the CeCILL license agreement, or when a Derivative Software contains a
         Related Module subject to the CeCILL license agreement, the provisions
         set forth in the third item of Article 6.4 are optional.

   (( Article 6 - ))??
   INTELLECTUAL PROPERTY

      (( 6.1 ))??
      OVER THE INITIAL SOFTWARE

      The Holder owns the economic rights over the Initial Software. Any or all
      use of the Initial Software is subject to compliance with the terms and
      conditions under which the Holder has elected to distribute its work and
      no one shall be entitled to modify the terms and conditions for the
      distribution of said Initial Software.

      The Holder undertakes that the Initial Software will remain ruled at least
      by this Agreement, for the duration set forth in Article 4.2.

      (( 6.2 ))??
      OVER THE INTEGRATED CONTRIBUTIONS

      The Licensee who develops an Integrated Contribution is the owner of the
      intellectual property rights over this Contribution as defined by
      applicable law.

      (( 6.3 ))??
      OVER THE RELATED MODULES

      The Licensee who develops a Related Module is the owner of the
      intellectual property rights over this Related Module as defined by
      applicable law and is free to choose the type of agreement that shall
      govern its distribution under the conditions defined in Article 5.3.3.

      (( 6.4 ))??
      NOTICE OF RIGHTS

      The Licensee expressly undertakes:

         (( 1. ))??
         not to remove, or modify, in any manner, the intellectual property
         notices attached to the Software;

         (( 2. ))??
         to reproduce said notices, in an identical manner, in the copies of the
         Software modified or not;

         (( 3. ))??
         to ensure that use of the Software, its intellectual property notices
         and the fact that it is governed by the Agreement is indicated in a
         text that is easily accessible, specifically from the interface of any
         Derivative Software.

      The Licensee undertakes not to directly or indirectly infringe the
      intellectual property rights of the Holder and/or Contributors on the
      Software and to take, where applicable, vis-à-vis its staff, any and all
      measures required to ensure respect of said intellectual property rights
      of the Holder and/or Contributors.

   (( Article 7 - ))??
   RELATED SERVICES

      (( 7.1 ))??
      Under no circumstances shall the Agreement oblige the Licensor to provide
      technical assistance or maintenance services for the Software.

      However, the Licensor is entitled to offer this type of services. The
      terms and conditions of such technical assistance, and/or such
      maintenance, shall be set forth in a separate instrument. Only the
      Licensor offering said maintenance and/or technical assistance services
      shall incur liability therefor.

      (( 7.2 ))??
      Similarly, any Licensor is entitled to offer to its licensees, under its
      sole responsibility, a warranty, that shall only be binding upon itself,
      for the redistribution of the Software and/or the Modified Software, under
      terms and conditions that it is free to decide. Said warranty, and the
      financial terms and conditions of its application, shall be subject of a
      separate instrument executed between the Licensor and the Licensee.

   (( Article 8 - ))??
   LIABILITY

      (( 8.1 ))??
      Subject to the provisions of Article 8.2, the Licensee shall be entitled
      to claim compensation for any direct loss it may have suffered from the
      Software as a result of a fault on the part of the relevant Licensor,
      subject to providing evidence thereof.

      (( 8.2 ))??
      The Licensor's liability is limited to the commitments made under this
      Agreement and shall not be incurred as a result of in particular: (i) loss
      due the Licensee's total or partial failure to fulfill its obligations,
      (ii) direct or consequential loss that is suffered by the Licensee due to
      the use or performance of the Software, and (iii) more generally, any
      consequential loss. In particular the Parties expressly agree that any or
      all pecuniary or business loss (i.e. loss of data, loss of profits,
      operating loss, loss of customers or orders, opportunity cost, any
      disturbance to business activities) or any or all legal proceedings
      instituted against the Licensee by a third party, shall constitute
      consequential loss and shall not provide entitlement to any or all
      compensation from the Licensor.

   (( Article 9 - ))??
   WARRANTY

      (( 9.1 ))??
      The Licensee acknowledges that the scientific and technical
      state-of-the-art when the Software was distributed did not enable all
      possible uses to be tested and verified, nor for the presence of possible
      defects to be detected. In this respect, the Licensee's attention has been
      drawn to the risks associated with loading, using, modifying and/or
      developing and reproducing the Software which are reserved for experienced
      users.

      The Licensee shall be responsible for verifying, by any or all means, the
      suitability of the product for its requirements, its good working order,
      and for ensuring that it shall not cause damage to either persons or
      properties.

      (( 9.2 ))??
      The Licensor hereby represents, in good faith, that it is entitled to
      grant all the rights over the Software (including in particular the rights
      set forth in Article 5).

      (( 9.3 ))??
      The Licensee acknowledges that the Software is supplied "as is" by the
      Licensor without any other express or tacit warranty, other than that
      provided for in Article 9.2 and, in particular, without any warranty as to
      its commercial value, its secured, safe, innovative or relevant nature.

      Specifically, the Licensor does not warrant that the Software is free from
      any error, that it will operate without interruption, that it will be
      compatible with the Licensee's own equipment and software configuration,
      nor that it will meet the Licensee's requirements.

      (( 9.4 ))??
      The Licensor does not either expressly or tacitly warrant that the
      Software does not infringe any third party intellectual property right
      relating to a patent, software or any other property right. Therefore, the
      Licensor disclaims any and all liability towards the Licensee arising out
      of any or all proceedings for infringement that may be instituted in
      respect of the use, modification and redistribution of the Software.
      Nevertheless, should such proceedings be instituted against the Licensee,
      the Licensor shall provide it with technical and legal assistance for its
      defense. Such technical and legal assistance shall be decided on a
      case-by-case basis between the relevant Licensor and the Licensee pursuant
      to a memorandum of understanding. The Licensor disclaims any and all
      liability as regards the Licensee's use of the name of the Software. No
      warranty is given as regards the existence of prior rights over the name
      of the Software or as regards the existence of a trademark.

   (( Article 10 - ))??
   TERMINATION

      (( 10.1 ))??
      In the event of a breach by the Licensee of its obligations hereunder, the
      Licensor may automatically terminate this Agreement thirty (30) days after
      notice has been sent to the Licensee and has remained ineffective.

      (( 10.2 ))??
      A Licensee whose Agreement is terminated shall no longer be authorized to
      use, modify or distribute the Software. However, any licenses that it may
      have granted prior to termination of the Agreement shall remain valid
      subject to their having been granted in compliance with the terms and
      conditions hereof.

   (( Article 11 - ))??
   MISCELLANEOUS

      (( 11.1 ))??
      EXCUSABLE EVENTS

      Neither Party shall be liable for any or all delay, or failure to perform
      the Agreement, that may be attributable to an event of force majeure, an
      act of God or an outside cause, such as defective functioning or
      interruptions of the electricity or telecommunications networks, network
      paralysis following a virus attack, intervention by government
      authorities, natural disasters, water damage, earthquakes, fire,
      explosions, strikes and labor unrest, war, etc.

      (( 11.2 ))??
      Any failure by either Party, on one or more occasions, to invoke one or
      more of the provisions hereof, shall under no circumstances be interpreted
      as being a waiver by the interested Party of its right to invoke said
      provision(s) subsequently.

      (( 11.3 ))??
      The Agreement cancels and replaces any or all previous agreements, whether
      written or oral, between the Parties and having the same purpose, and
      constitutes the entirety of the agreement between said Parties concerning
      said purpose. No supplement or modification to the terms and conditions
      hereof shall be effective as between the Parties unless it is made in
      writing and signed by their duly authorized representatives.

      (( 11.4 ))??
      In the event that one or more of the provisions hereof were to conflict
      with a current or future applicable act or legislative text, said act or
      legislative text shall prevail, and the Parties shall make the necessary
      amendments so as to comply with said act or legislative text. All other
      provisions shall remain effective. Similarly, invalidity of a provision of
      the Agreement, for any reason whatsoever, shall not cause the Agreement as
      a whole to be invalid.

      (( 11.5 ))??
      LANGUAGE

      The Agreement is drafted in both French and English and both versions are
      deemed authentic.

   (( Article 12 - ))??
   NEW VERSIONS OF THE AGREEMENT

      (( 12.1 ))??
      Any person is authorized to duplicate and distribute copies of this
      Agreement.

      (( 12.2 ))??
      So as to ensure coherence, the wording of this Agreement is protected and
      may only be modified by the authors of the License, who reserve the right
      to periodically publish updates or new versions of the Agreement, each
      with a separate number. These subsequent versions may address new issues
      encountered by Free Software.

      (( 12.3 ))??
      Any Software distributed under a given version of the Agreement may only
      be subsequently distributed under the same version of the Agreement or a
      subsequent version.

   (( Article 13 - ))??
   GOVERNING LAW AND JURISDICTION

      (( 13.1 ))??
      The Agreement is governed by French law. The Parties agree to endeavor to
      seek an amicable solution to any disagreements or disputes that may arise
      during the performance of the Agreement.

      (( 13.2 ))??
      Failing an amicable solution within two (2) months as from their
      occurrence, and unless emergency proceedings are necessary, the
      disagreements or disputes shall be referred to the Paris Courts having
      jurisdiction, by the more diligent Party.

Version 1.0 dated 2006-09-05.
(( 1 CeCILL stands for Ce(a) C(nrs) I(nria) L(ogiciel) L(ibre) ))??
`
const license_CERN_OHL_1_1_lre = `//**
CERN Open Hardware Licence v1.1
https://spdx.org/licenses/CERN-OHL-1.1.json
https://www.ohwr.org/project/licenses/wikis/cern-ohl-v1.1
**//

(( CERN OHL v1.1

2011-07-08 - CERN, Geneva, Switzerland

CERN Open Hardware Licence v1.1 ))??

Preamble

Through this CERN Open Hardware Licence ("CERN OHL") version 1.1, the
Organization wishes to disseminate its hardware designs (as published on
http:/www.ohwr.org/) as widely as possible, and generally to foster
collaboration among public research hardware designers. The CERN OHL is
copyright of CERN. Anyone is welcome to use the CERN OHL, in unmodified form
only, for the distribution of his own Open Hardware designs. Any other right is
reserved.

1. Definitions

In this Licence, the following terms have the following meanings:

"Licence" means this CERN OHL.

"Documentation" means schematic diagrams, designs, circuit or circuit board
layouts, mechanical drawings, flow charts and descriptive text, and other
explanatory material that is explicitly stated as being made available under the
conditions of this Licence. The Documentation may be in any medium, including
but not limited to computer files and representations on paper, film, or any
other media.

"Product" means either an entire, or any part of a, device built using the
Documentation or the modified Documentation.

"Licensee" means any natural or legal person exercising rights under this
Licence.

"Licensor" means any natural or legal person that creates or modifies
Documentation and subsequently communicates to the public and/ or distributes
the resulting Documentation under the terms and conditions of this Licence.

A Licensee may at the same time be a Licensor, and vice versa.

2. Applicability

   (( 2.1 ))??
   This Licence governs the use, copying, modification, communication to the
   public and distribution of the Documentation, and the manufacture and
   distribution of Products. By exercising any right granted under this Licence,
   the Licensee irrevocably accepts these terms and conditions.

   (( 2.2 ))??
   This Licence is granted by the Licensor directly to the Licensee, and shall
   apply worldwide and without limitation in time. The Licensee may assign his
   licence rights or grant sub-licences.

   (( 2.3 ))??
   This Licence does not apply to software, firmware, or code loaded into
   programmable devices which may be used in conjunction with the Documentation,
   the modified Documentation or with Products. The use of such software,
   firmware, or code is subject to the applicable licence terms and conditions.

3. Copying, modification, communication to the public and distribution of the
Documentation

   (( 3.1 ))??
   The Licensee shall keep intact all copyright and trademarks notices and all
   notices that refer to this Licence and to the disclaimer of warranties that
   is included in the Documentation. He shall include a copy thereof in every
   copy of the documentation or, as the case may be, modified Documentation,
   that he communicates to the public or distributes.

   (( 3.2 ))??
   The Licensee may use, copy, communicate to the public and distribute verbatim
   copies of the Documentation, in any medium, subject to the requirements
   specified in section 3.1.

   (( 3.3 ))??
   The Licensee may modify the Documentation or any portion thereof. The
   Licensee may communicate to the public and distribute the modified
   Documentation (thereby in addition to being a Licensee also becoming a
   Licensor), always provided that he shall:

      (( a. ))??
      comply with section 3.1;

      (( b. ))??
      cause the modified Documentation to carry prominent notices stating that
      the Licensee has modified the Documentation, with the date and details of
      the modifications;

      (( c. ))??
      license the modified Documentation under the terms and conditions of this
      Licence or, where applicable, a later version of this Licence as may be
      issued by CERN; and

      (( d. ))??
      send a copy of the modified Documentation to all Licensors that
      contributed to the parts of the Documentation that were modified, as well
      as to any other Licensor who has requested to receive a copy of the
      modified Documentation and has provided a means of contact with the
      Documentation.

   (( 3.4 ))??
   The Licence includes a licence to those patents or registered designs that
   are held by the Licensor, to the extent necessary to make use of the rights
   granted under this Licence. The scope of this section 3.4 shall be strictly
   limited to the parts of the Documentation or modified Documentation created
   by the Licensor.

4. Manufacture and distribution of Products

   (( 4.1 ))??
   The Licensee may manufacture or distribute Products always provided that the
   Licensee distributes to each recipient of such Products a copy of the
   Documentation or modified Documentation, as applicable, and complies with
   section 3.

   (( 4.2 ))??
   The Licensee is invited to inform in writing any Licensor who has indicated
   its wish to receive this information about the type, quantity and dates of
   production of Products the Licensee has (had) manufactured.

5. Warranty and liability

   (( 5.1 ))??
   DISCLAIMER – The Documentation and any modified Documentation are provided
   "as is" and any express or implied warranties, including, but not limited to,
   implied warranties of merchantability, of satisfactory quality, and fitness
   for a particular purpose or use are disclaimed in respect of the
   Documentation, the modified Documentation or any Product. The Licensor makes
   no representation that the Documentation, modified Documentation, or any
   Product, does or will not infringe any patent, copyright, trade secret or
   other proprietary right. The entire risk as to the use, quality, and
   performance of a Product shall be with the Licensee and not the Licensor.
   This disclaimer of warranty is an essential part of this Licence and a
   condition for the grant of any rights granted under this Licence. The
   Licensee warrants that it does not act in a consumer capacity.

   (( 5.2 ))??
   LIMITATION OF LIABILITY – The Licensor shall have no liability for direct,
   indirect, special, incidental, consequential, exemplary, punitive or other
   damages of any character including, without limitation, procurement of
   substitute goods or services, loss of use, data or profits, or business
   interruption, however caused and on any theory of contract, warranty, tort
   (including negligence), product liability or otherwise, arising in any way in
   relation to the Documentation, modified Documentation and/or the use,
   manufacture or distribution of a Product, even if advised of the possibility
   of such damages, and the Licensee shall hold the Licensor(s) free and
   harmless from any liability, costs, damages, fees and expenses, including
   claims by third parties, in relation to such use.

6. General

   (( 6.1 ))??
   The rights granted under this Licence do not imply or represent any transfer
   or assignment of intellectual property rights to the Licensee.

   (( 6.2 ))??
   The Licensee shall not use or make reference to any of the names, acronyms,
   images or logos under which the Licensor is known, save in so far as required
   to comply with section 3. Any such permitted use or reference shall be
   factual and shall in no event suggest any kind of endorsement by the Licensor
   or its personnel of the modified Documentation or any Product, or any kind of
   implication by the Licensor or its personnel in the preparation of the
   modified Documentation or Product.

   (( 6.3 ))??
   CERN may publish updated versions of this Licence which retain the same
   general provisions as this version, but differ in detail so far this is
   required and reasonable. New versions will be published with a unique version
   number.

   (( 6.4 ))??
   This Licence shall terminate with immediate effect, upon written notice and
   without involvement of a court if the Licensee fails to comply with any of
   its terms and conditions, or if the Licensee initiates legal action against
   Licensor in relation to this Licence. Section 5 shall continue to apply.

   (( 6.5 ))??
   Except as may be otherwise agreed with the Intergovernmental Organization,
   any dispute with respect to this Licence involving an Intergovernmental
   Organization shall, by virtue of the latter's Intergovernmental status, be
   settled by international arbitration. The arbitration proceedings shall be
   held at the place where the Intergovernmental Organization has its seat. The
   arbitral award shall be final and binding upon the parties, who hereby
   expressly agree to renounce any form of appeal or revision.
`
const license_CERN_OHL_1_2_lre = `//**
CERN Open Hardware Licence v1.2
https://spdx.org/licenses/CERN-OHL-1.2.json
https://www.ohwr.org/project/licenses/wikis/cern-ohl-v1.2
**//

(( CERN OHL v1.2

2013-09-06 - CERN, Geneva, Switzerland

CERN Open Hardware Licence v1.2 ))??

Preamble

Through this CERN Open Hardware Licence ("CERN OHL") version 1.2, CERN wishes to
provide a tool to foster collaboration and sharing among hardware designers. The
CERN OHL is copyright CERN. Anyone is welcome to use the CERN OHL, in unmodified
form only, for the distribution of their own Open Hardware designs. Any other
right is reserved. Release of hardware designs under the CERN OHL does not
constitute an endorsement of the licensor or its designs nor does it imply any
involvement by CERN in the development of such designs.

1. Definitions

In this Licence, the following terms have the following meanings:

"Licence" means this CERN OHL.

"Documentation" means schematic diagrams, designs, circuit or circuit board
layouts, mechanical drawings, flow charts and descriptive text, and other
explanatory material that is explicitly stated as being made available under the
conditions of this Licence. The Documentation may be in any medium, including
but not limited to computer files and representations on paper, film, or any
other media.

"Documentation Location" means a location where the Licensor has placed
Documentation, and which he believes will be publicly accessible for at least
three years from the first communication to the public or distribution of
Documentation.

"Product" means either an entire, or any part of a, device built using the
Documentation or the modified Documentation.

"Licensee" means any natural or legal person exercising rights under this
Licence.

"Licensor" means any natural or legal person that creates or modifies
Documentation and subsequently communicates to the public and/ or distributes
the resulting Documentation under the terms and conditions of this Licence.

A Licensee may at the same time be a Licensor, and vice versa.

Use of the masculine gender includes the feminine and neuter genders and is
employed solely to facilitate reading.

2. Applicability

   (( 2.1. ))??
   This Licence governs the use, copying, modification, communication to the
   public and distribution of the Documentation, and the manufacture and
   distribution of Products. By exercising any right granted under this Licence,
   the Licensee irrevocably accepts these terms and conditions.

   (( 2.2. ))??
   This Licence is granted by the Licensor directly to the Licensee, and shall
   apply worldwide and without limitation in time. The Licensee may assign his
   licence rights or grant sub-licences.

   (( 2.3. ))??
   This Licence does not extend to software, firmware, or code loaded into
   programmable devices which may be used in conjunction with the Documentation,
   the modified Documentation or with Products, unless such software, firmware,
   or code is explicitly expressed to be subject to this Licence. The use of
   such software, firmware, or code is otherwise subject to the applicable
   licence terms and conditions.

3. Copying, modification, communication to the public and distribution of the
Documentation

   (( 3.1. ))??
   The Licensee shall keep intact all copyright and trademarks notices, all
   notices referring to Documentation Location, and all notices that refer to
   this Licence and to the disclaimer of warranties that are included in the
   Documentation. He shall include a copy thereof in every copy of the
   Documentation or, as the case may be, modified Documentation, that he
   communicates to the public or distributes.

   (( 3.2. ))??
   The Licensee may copy, communicate to the public and distribute verbatim
   copies of the Documentation, in any medium, subject to the requirements
   specified in section 3.1.

   (( 3.3. ))??
   The Licensee may modify the Documentation or any portion thereof provided
   that upon modification of the Documentation, the Licensee shall make the
   modified Documentation available from a Documentation Location such that it
   can be easily located by an original Licensor once the Licensee communicates
   to the public or distributes the modified Documentation under section 3.4,
   and, where required by section 4.1, by a recipient of a Product. However, the
   Licensor shall not assert his rights under the foregoing proviso unless or
   until a Product is distributed.

   (( 3.4. ))??
   The Licensee may communicate to the public and distribute the modified
   Documentation (thereby in addition to being a Licensee also becoming a
   Licensor), always provided that he shall:

      (( a) ))??
      comply with section 3.1;

      (( b) ))??
      cause the modified Documentation to carry prominent notices stating that
      the Licensee has modified the Documentation, with the date and description
      of the modifications;

      (( c) ))??
      cause the modified Documentation to carry a new Documentation Location
      notice if the original Documentation provided for one;

      (( d) ))??
      make available the modified Documentation at the same level of abstraction
      as that of the Documentation, in the preferred format for making
      modifications to it (e.g. the native format of the CAD tool as
      applicable), and in the event that format is proprietary, in a format
      viewable with a tool licensed under an OSI-approved license if the
      proprietary tool can create it; and

      (( e) ))??
      license the modified Documentation under the terms and conditions of this
      Licence or, where applicable, a later version of this Licence as may be
      issued by CERN.

   (( 3.5. ))??
   The Licence includes a non-exclusive licence to those patents or registered
   designs that are held by, under the control of, or sub-licensable by the
   Licensor, to the extent necessary to make use of the rights granted under
   this Licence. The scope of this section 3.5 shall be strictly limited to the
   parts of the Documentation or modified Documentation created by the
   Licensor.

4. Manufacture and distribution of Products

   (( 4.1. ))??
   The Licensee may manufacture or distribute Products always provided that,
   where such manufacture or distribution requires a licence under this Licence
   the Licensee provides to each recipient of such Products an easy means of
   accessing a copy of the Documentation or modified Documentation, as
   applicable, as set out in section 3.

   (( 4.2. ))??
   The Licensee is invited to inform any Licensor who has indicated his wish to
   receive this information about the type, quantity and dates of production of
   Products the Licensee has (had) manufactured

5. Warranty and liability

   (( 5.1. ))??
   DISCLAIMER – The Documentation and any modified Documentation are provided
   "as is" and any express or implied warranties, including, but not limited to,
   implied warranties of merchantability, of satisfactory quality,
   non-infringement of third party rights, and fitness for a particular purpose
   or use are disclaimed in respect of the Documentation, the modified
   Documentation or any Product. The Licensor makes no representation that the
   Documentation, modified Documentation, or any Product, does or will not
   infringe any patent, copyright, trade secret or other proprietary right. The
   entire risk as to the use, quality, and performance of a Product shall be
   with the Licensee and not the Licensor. This disclaimer of warranty is an
   essential part of this Licence and a condition for the grant of any rights
   granted under this Licence. The Licensee warrants that it does not act in a
   consumer capacity.

   (( 5.2. ))??
   LIMITATION OF LIABILITY – The Licensor shall have no liability for direct,
   indirect, special, incidental, consequential, exemplary, punitive or other
   damages of any character including, without limitation, procurement of
   substitute goods or services, loss of use, data or profits, or business
   interruption, however caused and on any theory of contract, warranty, tort
   (including negligence), product liability or otherwise, arising in any way in
   relation to the Documentation, modified Documentation and/or the use,
   manufacture or distribution of a Product, even if advised of the possibility
   of such damages, and the Licensee shall hold the Licensor(s) free and
   harmless from any liability, costs, damages, fees and expenses, including
   claims by third parties, in relation to such use.

6. General

   (( 6.1. ))??
   Except for the rights explicitly granted hereunder, this Licence does not
   imply or represent any transfer or assignment of intellectual property rights
   to the Licensee.

   (( 6.2. ))??
   The Licensee shall not use or make reference to any of the names (including
   acronyms and abbreviations), images, or logos under which the Licensor is
   known, save in so far as required to comply with section 3. Any such
   permitted use or reference shall be factual and shall in no event suggest any
   kind of endorsement by the Licensor or its personnel of the modified
   Documentation or any Product, or any kind of implication by the Licensor or
   its personnel in the preparation of the modified Documentation or Product.

   (( 6.3. ))??
   CERN may publish updated versions of this Licence which retain the same
   general provisions as this version, but differ in detail so far this is
   required and reasonable. New versions will be published with a unique version
   number.

   (( 6.4. ))??
   This Licence shall terminate with immediate effect, upon written notice and
   without involvement of a court if the Licensee fails to comply with any of
   its terms and conditions, or if the Licensee initiates legal action against
   Licensor in relation to this Licence. Section 5 shall continue to apply.
`
const license_CERN_OHL_P_2_0_lre = `//**
CERN Open Hardware Licence Version 2 - Permissive
https://spdx.org/licenses/CERN-OHL-P-2.0.json
https://www.ohwr.org/project/cernohl/wikis/Documents/CERN-OHL-version-2
**//

(( CERN Open Hardware Licence Version 2 - Permissive ))??

Preamble

CERN has developed this licence to promote collaboration among hardware
designers and to provide a legal tool which supports the freedom to use, study,
modify, share and distribute hardware designs and products based on those
designs. Version 2 of the CERN Open Hardware Licence comes in three variants:
this licence, CERN-OHL-P (permissive); and two reciprocal licences: CERN- OHL-W
(weakly reciprocal) and CERN-OHL-S (strongly reciprocal).

The CERN-OHL-P is copyright CERN 2020. Anyone is welcome to use it, in
unmodified form only.

Use of this Licence does not imply any endorsement by CERN of any Licensor or
their designs nor does it imply any involvement by CERN in their development.

   (( 1 ))??
   Definitions

      (( 1.1 ))??
      'Licence' means this CERN-OHL-P.

      (( 1.2 ))??
      'Source' means information such as design materials or digital code which
      can be applied to Make or test a Product or to prepare a Product for use,
      Conveyance or sale, regardless of its medium or how it is expressed. It
      may include Notices.

      (( 1.3 ))??
      'Covered Source' means Source that is explicitly made available under this
      Licence.

      (( 1.4 ))??
      'Product' means any device, component, work or physical object, whether in
      finished or intermediate form, arising from the use, application or
      processing of Covered Source.

      (( 1.5 ))??
      'Make' means to create or configure something, whether by manufacture,
      assembly, compiling, loading or applying Covered Source or another Product
      or otherwise.

      (( 1.6 ))??
      'Notice' means copyright, acknowledgement and trademark notices,
      references to the location of any Notices, modification notices
      (subsection 3.3(b) and all notices that refer to this Licence and to the
      disclaimer of warranties that are included in the Covered Source.

      (( 1.7 ))??
      'Licensee' or 'You' means any person exercising rights under this
      Licence.

      (( 1.8 ))??
      'Licensor' means a person who creates Source or modifies Covered Source
      and subsequently Conveys the resulting Covered Source under the terms and
      conditions of this Licence. A person may be a Licensee and a Licensor at
      the same time.

      (( 1.9 ))??
      'Convey' means to communicate to the public or distribute.

   (( 2 ))??
   Applicability

      (( 2.1 ))??
      This Licence governs the use, copying, modification, Conveying of Covered
      Source and Products, and the Making of Products. By exercising any right
      granted under this Licence, You irrevocably accept these terms and
      conditions.

      (( 2.2 ))??
      This Licence is granted by the Licensor directly to You, and shall apply
      worldwide and without limitation in time.

      (( 2.3 ))??
      You shall not attempt to restrict by contract or otherwise the rights
      granted under this Licence to other Licensees.

      (( 2.4 ))??
      This Licence is not intended to restrict fair use, fair dealing, or any
      other similar right.

   (( 3 ))??
   Copying, modifying and Conveying Covered Source

      (( 3.1 ))??
      You may copy and Convey verbatim copies of Covered Source, in any medium,
      provided You retain all Notices.

      (( 3.2 ))??
      You may modify Covered Source, other than Notices.

      You may only delete Notices if they are no longer applicable to the
      corresponding Covered Source as modified by You and You may add additional
      Notices applicable to Your modifications.

      (( 3.3 ))??
      You may Convey modified Covered Source (with the effect that You shall
      also become a Licensor) provided that You:

         (( a) ))??
         retain Notices as required in subsection 3.2; and

         (( b) ))??
         add a Notice to the modified Covered Source stating that You have
         modified it, with the date and brief description of how You have
         modified it.

      (( 3.4 ))??
      You may Convey Covered Source or modified Covered Source under licence
      terms which differ from the terms of this Licence provided that:

         (( a) ))??
         You comply at all times with subsection 3.3; and

         (( b) ))??
         You provide a copy of this Licence to anyone to whom You Convey Covered
         Source or modified Covered Source.

   (( 4 ))??
   Making and Conveying Products

   You may Make Products, and/or Convey them, provided that You ensure that the
   recipient of the Product has access to any Notices applicable to the
   Product.

   (( 5 ))??
   DISCLAIMER AND LIABILITY

      (( 5.1 ))??
      DISCLAIMER OF WARRANTY -- The Covered Source and any Products are provided
      'as is' and any express or implied warranties, including, but not limited
      to, implied warranties of merchantability, of satisfactory quality,
      non-infringement of third party rights, and fitness for a particular
      purpose or use are disclaimed in respect of any Source or Product to the
      maximum extent permitted by law. The Licensor makes no representation that
      any Source or Product does not or will not infringe any patent, copyright,
      trade secret or other proprietary right. The entire risk as to the use,
      quality, and performance of any Source or Product shall be with You and
      not the Licensor. This disclaimer of warranty is an essential part of this
      Licence and a condition for the grant of any rights granted under this
      Licence.

      (( 5.2 ))??
      EXCLUSION AND LIMITATION OF LIABILITY -- The Licensor shall, to the
      maximum extent permitted by law, have no liability for direct, indirect,
      special, incidental, consequential, exemplary, punitive or other damages
      of any character including, without limitation, procurement of substitute
      goods or services, loss of use, data or profits, or business interruption,
      however caused and on any theory of contract, warranty, tort (including
      negligence), product liability or otherwise, arising in any way in
      relation to the Covered Source, modified Covered Source and/or the Making
      or Conveyance of a Product, even if advised of the possibility of such
      damages, and You shall hold the Licensor(s) free and harmless from any
      liability, costs, damages, fees and expenses, including claims by third
      parties, in relation to such use.

   (( 6 ))??
   Patents

      (( 6.1 ))??
      Subject to the terms and conditions of this Licence, each Licensor hereby
      grants to You a perpetual, worldwide, non-exclusive, no-charge,
      royalty-free, irrevocable (except as stated in this section 6, or where
      terminated by the Licensor for cause) patent license to Make, have Made,
      use, offer to sell, sell, import, and otherwise transfer the Covered
      Source and Products, where such licence applies only to those patent
      claims licensable by such Licensor that are necessarily infringed by
      exercising rights under the Covered Source as Conveyed by that Licensor.

      (( 6.2 ))??
      If You institute patent litigation against any entity (including a
      cross-claim or counterclaim in a lawsuit) alleging that the Covered Source
      or a Product constitutes direct or contributory patent infringement, or
      You seek any declaration that a patent licensed to You under this Licence
      is invalid or unenforceable then any rights granted to You under this
      Licence shall terminate as of the date such process is initiated.

   (( 7 ))??
   General

      (( 7.1 ))??
      If any provisions of this Licence are or subsequently become invalid or
      unenforceable for any reason, the remaining provisions shall remain
      effective.

      (( 7.2 ))??
      You shall not use any of the name (including acronyms and abbreviations),
      image, or logo by which the Licensor or CERN is known, except where needed
      to comply with section 3, or where the use is otherwise allowed by law.
      Any such permitted use shall be factual and shall not be made so as to
      suggest any kind of endorsement or implication of involvement by the
      Licensor or its personnel.

      (( 7.3 ))??
      CERN may publish updated versions and variants of this Licence which it
      considers to be in the spirit of this version, but may differ in detail to
      address new problems or concerns. New versions will be published with a
      unique version number and a variant identifier specifying the variant. If
      the Licensor has specified that a given variant applies to the Covered
      Source without specifying a version, You may treat that Covered Source as
      being released under any version of the CERN-OHL with that variant. If no
      variant is specified, the Covered Source shall be treated as being
      released under CERN-OHL-S. The Licensor may also specify that the Covered
      Source is subject to a specific version of the CERN-OHL or any later
      version in which case You may apply this or any later version of CERN-OHL
      with the same variant identifier published by CERN.

      (( 7.4 ))??
      This Licence shall not be enforceable except by a Licensor acting as such,
      and third party beneficiary rights are specifically excluded.
`
const license_CERN_OHL_S_2_0_lre = `//**
CERN Open Hardware Licence Version 2 - Strongly Reciprocal
https://spdx.org/licenses/CERN-OHL-S-2.0.json
https://www.ohwr.org/project/cernohl/wikis/Documents/CERN-OHL-version-2
**//

(( CERN Open Hardware Licence Version 2 - Strongly Reciprocal ))??

Preamble

CERN has developed this licence to promote collaboration among hardware
designers and to provide a legal tool which supports the freedom to use, study,
modify, share and distribute hardware designs and products based on those
designs. Version 2 of the CERN Open Hardware Licence comes in three variants:
CERN-OHL-P (permissive); and two reciprocal licences: CERN-OHL-W (weakly
reciprocal) and this licence, CERN-OHL-S (strongly reciprocal).

The CERN-OHL-S is copyright CERN 2020. Anyone is welcome to use it, in
unmodified form only.

Use of this Licence does not imply any endorsement by CERN of any Licensor or
their designs nor does it imply any involvement by CERN in their development.

   (( 1 ))??
   Definitions

      (( 1.1 ))??
      'Licence' means this CERN-OHL-S.

      (( 1.2 ))??
      'Compatible Licence' means

         (( a) ))??
         any earlier version of the CERN Open Hardware licence, or

         (( b) ))??
         any version of the CERN-OHL-S, or

         (( c) ))??
         any licence which permits You to treat the Source to which it applies
         as licensed under CERN-OHL-S provided that on Conveyance of any such
         Source, or any associated Product You treat the Source in question as
         being licensed under CERN-OHL-S.

      (( 1.3 ))??
      'Source' means information such as design materials or digital code which
      can be applied to Make or test a Product or to prepare a Product for use,
      Conveyance or sale, regardless of its medium or how it is expressed. It
      may include Notices.

      (( 1.4 ))??
      'Covered Source' means Source that is explicitly made available under this
      Licence.

      (( 1.5 ))??
      'Product' means any device, component, work or physical object, whether in
      finished or intermediate form, arising from the use, application or
      processing of Covered Source.

      (( 1.6 ))??
      'Make' means to create or configure something, whether by manufacture,
      assembly, compiling, loading or applying Covered Source or another Product
      or otherwise.

      (( 1.7 ))??
      'Available Component' means any part, sub-assembly, library or code
      which:

         (( a) ))??
         is licensed to You as Complete Source under a Compatible Licence; or

         (( b) ))??
         is available, at the time a Product or the Source containing it is
         first Conveyed, to You and any other prospective licensees

            (( i) ))??
            as a physical part with sufficient rights and information (including
            any configuration and programming files and information about its
            characteristics and interfaces) to enable it either to be Made
            itself, or to be sourced and used to Make the Product; or

            (( ii) ))??
            as part of the normal distribution of a tool used to design or Make
            the Product.

      (( 1.8 ))??
      'Complete Source' means the set of all Source necessary to Make a Product,
      in the preferred form for making modifications, including necessary
      installation and interfacing information both for the Product, and for any
      included Available Components. If the format is proprietary, it must also
      be made available in a format (if the proprietary tool can create it)
      which is viewable with a tool available to potential licensees and
      licensed under a licence approved by the Free Software Foundation or the
      Open Source Initiative. Complete Source need not include the Source of any
      Available Component, provided that You include in the Complete Source
      sufficient information to enable a recipient to Make or source and use the
      Available Component to Make the Product.

      (( 1.9 ))??
      'Source Location' means a location where a Licensor has placed Covered
      Source, and which that Licensor reasonably believes will remain easily
      accessible for at least three years for anyone to obtain a digital copy.

      (( 1.10 ))??
      'Notice' means copyright, acknowledgement and trademark notices, Source
      Location references, modification notices (subsection 3.3(b) and all
      notices that refer to this Licence and to the disclaimer of warranties
      that are included in the Covered Source.

      (( 1.11 ))??
      'Licensee' or 'You' means any person exercising rights under this
      Licence.

      (( 1.12 ))??
      'Licensor' means a natural or legal person who creates or modifies Covered
      Source. A person may be a Licensee and a Licensor at the same time.

      (( 1.13 ))??
      'Convey' means to communicate to the public or distribute.

   (( 2 ))??
   Applicability

      (( 2.1 ))??
      This Licence governs the use, copying, modification, Conveying of Covered
      Source and Products, and the Making of Products. By exercising any right
      granted under this Licence, You irrevocably accept these terms and
      conditions.

      (( 2.2 ))??
      This Licence is granted by the Licensor directly to You, and shall apply
      worldwide and without limitation in time.

      (( 2.3 ))??
      You shall not attempt to restrict by contract or otherwise the rights
      granted under this Licence to other Licensees.

      (( 2.4 ))??
      This Licence is not intended to restrict fair use, fair dealing, or any
      other similar right.

   (( 3 ))??
   Copying, modifying and Conveying Covered Source

      (( 3.1 ))??
      You may copy and Convey verbatim copies of Covered Source, in any medium,
      provided You retain all Notices.

      (( 3.2 ))??
      You may modify Covered Source, other than Notices, provided that You
      irrevocably undertake to make that modified Covered Source available from
      a Source Location should You Convey a Product in circumstances where the
      recipient does not otherwise receive a copy of the modified Covered
      Source. In each case subsection 3.3 shall apply.

      You may only delete Notices if they are no longer applicable to the
      corresponding Covered Source as modified by You and You may add additional
      Notices applicable to Your modifications. Including Covered Source in a
      larger work is modifying the Covered Source, and the larger work becomes
      modified Covered Source.

      (( 3.3 ))??
      You may Convey modified Covered Source (with the effect that You shall
      also become a Licensor) provided that You:

         (( a) ))??
         retain Notices as required in subsection 3.2;

         (( b) ))??
         add a Notice to the modified Covered Source stating that You have
         modified it, with the date and brief description of how You have
         modified it;

         (( c) ))??
         add a Source Location Notice for the modified Covered Source if You
         Convey in circumstances where the recipient does not otherwise receive
         a copy of the modified Covered Source; and

         (( d) ))??
         license the modified Covered Source under the terms and conditions of
         this Licence (or, as set out in subsection 8.3, a later version, if
         permitted by the licence of the original Covered Source). Such modified
         Covered Source must be licensed as a whole, but excluding Available
         Components contained in it, which remain licensed under their own
         applicable licences.

   (( 4 ))??
   Making and Conveying Products

   You may Make Products, and/or Convey them, provided that You either provide
   each recipient with a copy of the Complete Source or ensure that each
   recipient is notified of the Source Location of the Complete Source. That
   Complete Source is Covered Source, and You must accordingly satisfy Your
   obligations set out in subsection 3.3. If specified in a Notice, the Product
   must visibly and securely display the Source Location on it or its packaging
   or documentation in the manner specified in that Notice.

   (( 5 ))??
   Research and Development

   You may Convey Covered Source, modified Covered Source or Products to a legal
   entity carrying out development, testing or quality assurance work on Your
   behalf provided that the work is performed on terms which prevent the entity
   from both using the Source or Products for its own internal purposes and
   Conveying the Source or Products or any modifications to them to any person
   other than You. Any modifications made by the entity shall be deemed to be
   made by You pursuant to subsection 3.2.

   (( 6 ))??
   DISCLAIMER AND LIABILITY

      (( 6.1 ))??
      DISCLAIMER OF WARRANTY -- The Covered Source and any Products are provided
      'as is' and any express or implied warranties, including, but not limited
      to, implied warranties of merchantability, of satisfactory quality,
      non-infringement of third party rights, and fitness for a particular
      purpose or use are disclaimed in respect of any Source or Product to the
      maximum extent permitted by law. The Licensor makes no representation that
      any Source or Product does not or will not infringe any patent, copyright,
      trade secret or other proprietary right. The entire risk as to the use,
      quality, and performance of any Source or Product shall be with You and
      not the Licensor. This disclaimer of warranty is an essential part of this
      Licence and a condition for the grant of any rights granted under this
      Licence.

      (( 6.2 ))??
      EXCLUSION AND LIMITATION OF LIABILITY -- The Licensor shall, to the
      maximum extent permitted by law, have no liability for direct, indirect,
      special, incidental, consequential, exemplary, punitive or other damages
      of any character including, without limitation, procurement of substitute
      goods or services, loss of use, data or profits, or business interruption,
      however caused and on any theory of contract, warranty, tort (including
      negligence), product liability or otherwise, arising in any way in
      relation to the Covered Source, modified Covered Source and/or the Making
      or Conveyance of a Product, even if advised of the possibility of such
      damages, and You shall hold the Licensor(s) free and harmless from any
      liability, costs, damages, fees and expenses, including claims by third
      parties, in relation to such use.

   (( 7 ))??
   Patents

      (( 7.1 ))??
      Subject to the terms and conditions of this Licence, each Licensor hereby
      grants to You a perpetual, worldwide, non-exclusive, no-charge,
      royalty-free, irrevocable (except as stated in subsections 7.2 and 8.4)
      patent license to Make, have Made, use, offer to sell, sell, import, and
      otherwise transfer the Covered Source and Products, where such licence
      applies only to those patent claims licensable by such Licensor that are
      necessarily infringed by exercising rights under the Covered Source as
      Conveyed by that Licensor.

      (( 7.2 ))??
      If You institute patent litigation against any entity (including a
      cross-claim or counterclaim in a lawsuit) alleging that the Covered Source
      or a Product constitutes direct or contributory patent infringement, or
      You seek any declaration that a patent licensed to You under this Licence
      is invalid or unenforceable then any rights granted to You under this
      Licence shall terminate as of the date such process is initiated.

   (( 8 ))??
   General

      (( 8.1 ))??
      If any provisions of this Licence are or subsequently become invalid or
      unenforceable for any reason, the remaining provisions shall remain
      effective.

      (( 8.2 ))??
      You shall not use any of the name (including acronyms and abbreviations),
      image, or logo by which the Licensor or CERN is known, except where needed
      to comply with section 3, or where the use is otherwise allowed by law.
      Any such permitted use shall be factual and shall not be made so as to
      suggest any kind of endorsement or implication of involvement by the
      Licensor or its personnel.

      (( 8.3 ))??
      CERN may publish updated versions and variants of this Licence which it
      considers to be in the spirit of this version, but may differ in detail to
      address new problems or concerns. New versions will be published with a
      unique version number and a variant identifier specifying the variant. If
      the Licensor has specified that a given variant applies to the Covered
      Source without specifying a version, You may treat that Covered Source as
      being released under any version of the CERN-OHL with that variant. If no
      variant is specified, the Covered Source shall be treated as being
      released under CERN-OHL-S. The Licensor may also specify that the Covered
      Source is subject to a specific version of the CERN-OHL or any later
      version in which case You may apply this or any later version of CERN-OHL
      with the same variant identifier published by CERN.

      (( 8.4 ))??
      This Licence shall terminate with immediate effect if You fail to comply
      with any of its terms and conditions.

      (( 8.5 ))??
      However, if You cease all breaches of this Licence, then Your Licence from
      any Licensor is reinstated unless such Licensor has terminated this
      Licence by giving You, while You remain in breach, a notice specifying the
      breach and requiring You to cure it within 30 days, and You have failed to
      come into compliance in all material respects by the end of the 30 day
      period. Should You repeat the breach after receipt of a cure notice and
      subsequent reinstatement, this Licence will terminate immediately and
      permanently. Section 6 shall continue to apply after any termination.

      (( 8.6 ))??
      This Licence shall not be enforceable except by a Licensor acting as such,
      and third party beneficiary rights are specifically excluded.
`
const license_CERN_OHL_W_2_0_lre = `//**
CERN Open Hardware Licence Version 2 - Weakly Reciprocal
https://spdx.org/licenses/CERN-OHL-W-2.0.json
https://www.ohwr.org/project/cernohl/wikis/Documents/CERN-OHL-version-2
**//

(( CERN Open Hardware Licence Version 2 - Weakly Reciprocal ))??

Preamble

CERN has developed this licence to promote collaboration among hardware
designers and to provide a legal tool which supports the freedom to use, study,
modify, share and distribute hardware designs and products based on those
designs. Version 2 of the CERN Open Hardware Licence comes in three variants:
CERN-OHL-P (permissive); and two reciprocal licences: this licence, CERN- OHL-W
(weakly reciprocal) and CERN-OHL-S (strongly reciprocal).

The CERN-OHL-W is copyright CERN 2020. Anyone is welcome to use it, in
unmodified form only.

Use of this Licence does not imply any endorsement by CERN of any Licensor or
their designs nor does it imply any involvement by CERN in their development.

   (( 1 ))??
   Definitions

      (( 1.1 ))??
      'Licence' means this CERN-OHL-W.

      (( 1.2 ))??
      'Compatible Licence' means

         (( a) ))??
         any earlier version of the CERN Open Hardware licence, or

         (( b) ))??
         any version of the CERN-OHL-S or the CERN-OHL-W, or

         (( c) ))??
         any licence which permits You to treat the Source to which it applies
         as licensed under CERN-OHL-S or CERN-OHL-W provided that on Conveyance
         of any such Source, or any associated Product You treat the Source in
         question as being licensed under CERN-OHL-S or CERN-OHL-W as
         appropriate.

      (( 1.3 ))??
      'Source' means information such as design materials or digital code which
      can be applied to Make or test a Product or to prepare a Product for use,
      Conveyance or sale, regardless of its medium or how it is expressed. It
      may include Notices.

      (( 1.4 ))??
      'Covered Source' means Source that is explicitly made available under this
      Licence.

      (( 1.5 ))??
      'Product' means any device, component, work or physical object, whether in
      finished or intermediate form, arising from the use, application or
      processing of Covered Source.

      (( 1.6 ))??
      'Make' means to create or configure something, whether by manufacture,
      assembly, compiling, loading or applying Covered Source or another Product
      or otherwise.

      (( 1.7 ))??
      'Available Component' means any part, sub-assembly, library or code
      which:

         (( a) ))??
         is licensed to You as Complete Source under a Compatible Licence; or

         (( b) ))??
         is available, at the time a Product or the Source containing it is
         first Conveyed, to You and any other prospective licensees

            (( i) ))??
            with sufficient rights and information (including any configuration
            and programming files and information about its characteristics and
            interfaces) to enable it either to be Made itself, or to be sourced
            and used to Make the Product; or

            (( ii) ))??
            as part of the normal distribution of a tool used to design or Make
            the Product.

      (( 1.8 ))??
      'External Material' means anything (including Source) which:

         (( a) ))??
         is only combined with Covered Source in such a way that it interfaces
         with the Covered Source using a documented interface which is described
         in the Covered Source; and

         (( b) ))??
         is not a derivative of or contains Covered Source, or, if it is, it is
         solely to the extent necessary to facilitate such interfacing.

      (( 1.9 ))??
      'Complete Source' means the set of all Source necessary to Make a Product,
      in the preferred form for making modifications, including necessary
      installation and interfacing information both for the Product, and for any
      included Available Components. If the format is proprietary, it must also
      be made available in a format (if the proprietary tool can create it)
      which is viewable with a tool available to potential licensees and
      licensed under a licence approved by the Free Software Foundation or the
      Open Source Initiative. Complete Source need not include the Source of any
      Available Component, provided that You include in the Complete Source
      sufficient information to enable a recipient to Make or source and use the
      Available Component to Make the Product.

      (( 1.10 ))??
      'Source Location' means a location where a Licensor has placed Covered
      Source, and which that Licensor reasonably believes will remain easily
      accessible for at least three years for anyone to obtain a digital copy.

      (( 1.11 ))??
      'Notice' means copyright, acknowledgement and trademark notices, Source
      Location references, modification notices (subsection 3.3(b) and all
      notices that refer to this Licence and to the disclaimer of warranties
      that are included in the Covered Source.

      (( 1.12 ))??
      'Licensee' or 'You' means any person exercising rights under this
      Licence.

      (( 1.13 ))??
      'Licensor' means a natural or legal person who creates or modifies Covered
      Source. A person may be a Licensee and a Licensor at the same time.

      (( 1.14 ))??
      'Convey' means to communicate to the public or distribute.

   (( 2 ))??
   Applicability

      (( 2.1 ))??
      This Licence governs the use, copying, modification, Conveying of Covered
      Source and Products, and the Making of Products. By exercising any right
      granted under this Licence, You irrevocably accept these terms and
      conditions.

      (( 2.2 ))??
      This Licence is granted by the Licensor directly to You, and shall apply
      worldwide and without limitation in time.

      (( 2.3 ))??
      You shall not attempt to restrict by contract or otherwise the rights
      granted under this Licence to other Licensees.

      (( 2.4 ))??
      This Licence is not intended to restrict fair use, fair dealing, or any
      other similar right.

   (( 3 ))??
   Copying, modifying and Conveying Covered Source

      (( 3.1 ))??
      You may copy and Convey verbatim copies of Covered Source, in any medium,
      provided You retain all Notices.

      (( 3.2 ))??
      You may modify Covered Source, other than Notices, provided that You
      irrevocably undertake to make that modified Covered Source available from
      a Source Location should You Convey a Product in circumstances where the
      recipient does not otherwise receive a copy of the modified Covered
      Source. In each case subsection 3.3 shall apply.

      You may only delete Notices if they are no longer applicable to the
      corresponding Covered Source as modified by You and You may add additional
      Notices applicable to Your modifications.

      (( 3.3 ))??
      You may Convey modified Covered Source (with the effect that You shall
      also become a Licensor) provided that You:

         (( a) ))??
         retain Notices as required in subsection 3.2;

         (( b) ))??
         add a Notice to the modified Covered Source stating that You have
         modified it, with the date and brief description of how You have
         modified it;

         (( c) ))??
         add a Source Location Notice for the modified Covered Source if You
         Convey in circumstances where the recipient does not otherwise receive
         a copy of the modified Covered Source; and

         (( d) ))??
         license the modified Covered Source under the terms and conditions of
         this Licence (or, as set out in subsection 8.3, a later version, if
         permitted by the licence of the original Covered Source). Such modified
         Covered Source must be licensed as a whole, but excluding Available
         Components contained in it or External Material to which it is
         interfaced, which remain licensed under their own applicable licences.

   (( 4 ))??
   Making and Conveying Products

      (( 4.1 ))??
      You may Make Products, and/or Convey them, provided that You either
      provide each recipient with a copy of the Complete Source or ensure that
      each recipient is notified of the Source Location of the Complete Source.
      That Complete Source includes Covered Source and You must accordingly
      satisfy Your obligations set out in subsection 3.3. If specified in a
      Notice, the Product must visibly and securely display the Source Location
      on it or its packaging or documentation in the manner specified in that
      Notice.

      (( 4.2 ))??
      Where You Convey a Product which incorporates External Material, the
      Complete Source for that Product which You are required to provide under
      subsection 4.1 need not include any Source for the External Material.

      (( 4.3 ))??
      You may license Products under terms of Your choice, provided that such
      terms do not restrict or attempt to restrict any recipients' rights under
      this Licence to the Covered Source.

   (( 5 ))??
   Research and Development

   You may Convey Covered Source, modified Covered Source or Products to a legal
   entity carrying out development, testing or quality assurance work on Your
   behalf provided that the work is performed on terms which prevent the entity
   from both using the Source or Products for its own internal purposes and
   Conveying the Source or Products or any modifications to them to any person
   other than You. Any modifications made by the entity shall be deemed to be
   made by You pursuant to subsection 3.2.

   (( 6 ))??
   DISCLAIMER AND LIABILITY

      (( 6.1 ))??
      DISCLAIMER OF WARRANTY -- The Covered Source and any Products are provided
      'as is' and any express or implied warranties, including, but not limited
      to, implied warranties of merchantability, of satisfactory quality,
      non-infringement of third party rights, and fitness for a particular
      purpose or use are disclaimed in respect of any Source or Product to the
      maximum extent permitted by law. The Licensor makes no representation that
      any Source or Product does not or will not infringe any patent, copyright,
      trade secret or other proprietary right. The entire risk as to the use,
      quality, and performance of any Source or Product shall be with You and
      not the Licensor. This disclaimer of warranty is an essential part of this
      Licence and a condition for the grant of any rights granted under this
      Licence.

      (( 6.2 ))??
      EXCLUSION AND LIMITATION OF LIABILITY -- The Licensor shall, to the
      maximum extent permitted by law, have no liability for direct, indirect,
      special, incidental, consequential, exemplary, punitive or other damages
      of any character including, without limitation, procurement of substitute
      goods or services, loss of use, data or profits, or business interruption,
      however caused and on any theory of contract, warranty, tort (including
      negligence), product liability or otherwise, arising in any way in
      relation to the Covered Source, modified Covered Source and/or the Making
      or Conveyance of a Product, even if advised of the possibility of such
      damages, and You shall hold the Licensor(s) free and harmless from any
      liability, costs, damages, fees and expenses, including claims by third
      parties, in relation to such use.

   (( 7 ))??
   Patents

      (( 7.1 ))??
      Subject to the terms and conditions of this Licence, each Licensor hereby
      grants to You a perpetual, worldwide, non-exclusive, no-charge,
      royalty-free, irrevocable (except as stated in subsections 7.2 and 8.4)
      patent license to Make, have Made, use, offer to sell, sell, import, and
      otherwise transfer the Covered Source and Products, where such licence
      applies only to those patent claims licensable by such Licensor that are
      necessarily infringed by exercising rights under the Covered Source as
      Conveyed by that Licensor.

      (( 7.2 ))??
      If You institute patent litigation against any entity (including a
      cross-claim or counterclaim in a lawsuit) alleging that the Covered Source
      or a Product constitutes direct or contributory patent infringement, or
      You seek any declaration that a patent licensed to You under this Licence
      is invalid or unenforceable then any rights granted to You under this
      Licence shall terminate as of the date such process is initiated.

   (( 8 ))??
   General

      (( 8.1 ))??
      If any provisions of this Licence are or subsequently become invalid or
      unenforceable for any reason, the remaining provisions shall remain
      effective.

      (( 8.2 ))??
      You shall not use any of the name (including acronyms and abbreviations),
      image, or logo by which the Licensor or CERN is known, except where needed
      to comply with section 3, or where the use is otherwise allowed by law.
      Any such permitted use shall be factual and shall not be made so as to
      suggest any kind of endorsement or implication of involvement by the
      Licensor or its personnel.

      (( 8.3 ))??
      CERN may publish updated versions and variants of this Licence which it
      considers to be in the spirit of this version, but may differ in detail to
      address new problems or concerns. New versions will be published with a
      unique version number and a variant identifier specifying the variant. If
      the Licensor has specified that a given variant applies to the Covered
      Source without specifying a version, You may treat that Covered Source as
      being released under any version of the CERN-OHL with that variant. If no
      variant is specified, the Covered Source shall be treated as being
      released under CERN-OHL-S. The Licensor may also specify that the Covered
      Source is subject to a specific version of the CERN-OHL or any later
      version in which case You may apply this or any later version of CERN-OHL
      with the same variant identifier published by CERN.

      You may treat Covered Source licensed under CERN-OHL-W as licensed under
      CERN-OHL-S if and only if all Available Components referenced in the
      Covered Source comply with the corresponding definition of Available
      Component for CERN-OHL-S.

      (( 8.4 ))??
      This Licence shall terminate with immediate effect if You fail to comply
      with any of its terms and conditions.

      (( 8.5 ))??
      However, if You cease all breaches of this Licence, then Your Licence from
      any Licensor is reinstated unless such Licensor has terminated this
      Licence by giving You, while You remain in breach, a notice specifying the
      breach and requiring You to cure it within 30 days, and You have failed to
      come into compliance in all material respects by the end of the 30 day
      period. Should You repeat the breach after receipt of a cure notice and
      subsequent reinstatement, this Licence will terminate immediately and
      permanently. Section 6 shall continue to apply after any termination.

      (( 8.6 ))??
      This Licence shall not be enforceable except by a Licensor acting as such,
      and third party beneficiary rights are specifically excluded.
`
const license_CNRI_Jython_lre = `//**
CNRI Jython License
https://spdx.org/licenses/CNRI-Jython.json
http://www.jython.org/license.html
**//

   (( 1. ))??
   This LICENSE AGREEMENT is between the Corporation for National Research
   Initiatives, having an office at 1895 Preston White Drive, Reston, VA 20191
   ("CNRI"), and the Individual or Organization ("Licensee") accessing and using
   JPython version 1.1.x in source or binary form and its associated
   documentation as provided herein ("Software").

   (( 2. ))??
   Subject to the terms and conditions of this License Agreement, CNRI hereby
   grants Licensee a non-exclusive, non-transferable, royalty-free, world-wide
   license to reproduce, analyze, test, perform and/or display publicly, prepare
   derivative works, distribute, and otherwise use the Software alone or in any
   derivative version, provided, however, that CNRI's License Agreement and
   CNRI's notice of copyright, i.e., "Copyright (c) 1996-1999 Corporation for
   National Research Initiatives; All Rights Reserved" are both retained in the
   Software, alone or in any derivative version prepared by Licensee.

   Alternatively, in lieu of CNRI's License Agreement, Licensee may substitute
   the following text (omitting the quotes), provided, however, that such text
   is displayed prominently in the Software alone or in any derivative version
   prepared by Licensee: "JPython (Version 1.1.x) is made available subject to
   the terms and conditions in CNRI's License Agreement. This Agreement may be
   located on the Internet using the following unique, persistent identifier
   (known as a handle): 1895.22/1006. The License may also be obtained from a
   proxy server on the Web using the following URL:
   http:/hdl.handle.net/1895.22/1006."

   (( 3. ))??
   In the event Licensee prepares a derivative work that is based on or
   incorporates the Software or any part thereof, and wants to make the
   derivative work available to the public as provided herein, then Licensee
   hereby agrees to indicate in any such work, in a prominently visible way, the
   nature of the modifications made to CNRI's Software.

   (( 4. ))??
   Licensee may not use CNRI trademarks or trade name, including JPython or
   CNRI, in a trademark sense to endorse or promote products or services of
   Licensee, or any third party. Licensee may use the mark JPython in connection
   with Licensee's derivative versions that are based on or incorporate the
   Software, but only in the form "JPython-based _," or equivalent.

   (( 5. ))??
   CNRI is making the Software available to Licensee on an "AS IS" basis. CNRI
   MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED. BY WAY OF
   EXAMPLE, BUT NOT LIMITATION, CNRI MAKES NO AND DISCLAIMS ANY REPRESENTATION
   OR WARRANTY OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT
   THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY THIRD PARTY RIGHTS.

   (( 6. ))??
   CNRI SHALL NOT BE LIABLE TO LICENSEE OR OTHER USERS OF THE SOFTWARE FOR ANY
   INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS AS A RESULT OF USING,
   MODIFYING OR DISTRIBUTING THE SOFTWARE, OR ANY DERIVATIVE THEREOF, EVEN IF
   ADVISED OF THE POSSIBILITY THEREOF. SOME STATES DO NOT ALLOW THE LIMITATION
   OR EXCLUSION OF LIABILITY SO THE ABOVE DISCLAIMER MAY NOT APPLY TO LICENSEE.

   (( 7. ))??
   This License Agreement may be terminated by CNRI (i) immediately upon written
   notice from CNRI of any material breach by the Licensee, if the nature of the
   breach is such that it cannot be promptly remedied; or (ii) sixty (60) days
   following notice from CNRI to Licensee of a material remediable breach, if
   Licensee has not remedied such breach within that sixty-day period.

   (( 8. ))??
   This License Agreement shall be governed by and interpreted in all respects
   by the law of the State of Virginia, excluding conflict of law provisions.
   Nothing in this Agreement shall be deemed to create any relationship of
   agency, partnership, or joint venture between CNRI and Licensee.

   (( 9. ))??
   By clicking on the "ACCEPT" button where indicated, or by installing, copying
   or otherwise using the Software, Licensee agrees to be bound by the terms and
   conditions of this License Agreement.
`
const license_CNRI_Python_lre = `//**
CNRI Python License
https://spdx.org/licenses/CNRI-Python.json
https://opensource.org/licenses/CNRI-Python
**//

(( CNRI OPEN SOURCE LICENSE AGREEMENT ))??

IMPORTANT: PLEASE READ THE FOLLOWING AGREEMENT CAREFULLY.

BY CLICKING ON "ACCEPT" WHERE INDICATED BELOW, OR BY COPYING, INSTALLING OR
OTHERWISE USING PYTHON 1.6, beta 1 SOFTWARE, YOU ARE DEEMED TO HAVE AGREED TO
THE TERMS AND CONDITIONS OF THIS LICENSE AGREEMENT.

   (( 1. ))??
   This LICENSE AGREEMENT is between the Corporation for National Research
   Initiatives, having an office at 1895 Preston White Drive, Reston, VA 20191
   ("CNRI"), and the Individual or Organization ("Licensee") accessing and
   otherwise using Python 1.6, beta 1 software in source or binary form and its
   associated documentation, as released at the www.python.org Internet site on
   August 4, 2000 ("Python 1.6b1").

   (( 2. ))??
   Subject to the terms and conditions of this License Agreement, CNRI hereby
   grants Licensee a non-exclusive, royalty-free, world-wide license to
   reproduce, analyze, test, perform and/or display publicly, prepare derivative
   works, distribute, and otherwise use Python 1.6b1 alone or in any derivative
   version, provided, however, that CNRIs License Agreement is retained in
   Python 1.6b1, alone or in any derivative version prepared by Licensee.

   Alternately, in lieu of CNRIs License Agreement, Licensee may substitute the
   following text (omitting the quotes): "Python 1.6, beta 1, is made available
   subject to the terms and conditions in CNRIs License Agreement. This
   Agreement may be located on the Internet using the following unique,
   persistent identifier (known as a handle): 1895.22/1011. This Agreement may
   also be obtained from a proxy server on the Internet using the
   URL:http:/hdl.handle.net/1895.22/1011".

   (( 3. ))??
   In the event Licensee prepares a derivative work that is based on or
   incorporates Python 1.6b1 or any part thereof, and wants to make the
   derivative work available to the public as provided herein, then Licensee
   hereby agrees to indicate in any such work the nature of the modifications
   made to Python 1.6b1.

   (( 4. ))??
   CNRI is making Python 1.6b1 available to Licensee on an "AS IS" basis. CNRI
   MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED. BY WAY OF
   EXAMPLE, BUT NOT LIMITATION, CNRI MAKES NO AND DISCLAIMS ANY REPRESENTATION
   OR WARRANTY OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT
   THE USE OF PYTHON 1.6b1 WILL NOT INFRINGE ANY THIRD PARTY RIGHTS.

   (( 5. ))??
   CNRI SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF THE SOFTWARE FOR
   ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS A RESULT OF
   USING, MODIFYING OR DISTRIBUTING PYTHON 1.6b1, OR ANY DERIVATIVE THEREOF,
   EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

   (( 6. ))??
   This License Agreement will automatically terminate upon a material breach of
   its terms and conditions.

   (( 7. ))??
   This License Agreement shall be governed by and interpreted in all respects
   by the law of the State of Virginia, excluding conflict of law provisions.
   Nothing in this License Agreement shall be deemed to create any relationship
   of agency, partnership, or joint venture between CNRI and Licensee. This
   License Agreement does not grant permission to use CNRI trademarks or trade
   name in a trademark sense to endorse or promote products or services of
   Licensee, or any third party.

   (( 8. ))??
   By clicking on the "ACCEPT" button where indicated, or by copying, installing
   or otherwise using Python 1.6b1, Licensee agrees to be bound by the terms and
   conditions of this License Agreement.

ACCEPT
`
const license_CNRI_Python_GPL_Compatible_lre = `//**
CNRI Python Open Source GPL Compatible License Agreement
https://spdx.org/licenses/CNRI-Python-GPL-Compatible.json
http://www.python.org/download/releases/1.6.1/download_win/
**//

(( CNRI OPEN SOURCE GPL-COMPATIBLE LICENSE AGREEMENT ))??

IMPORTANT: PLEASE READ THE FOLLOWING AGREEMENT CAREFULLY.

BY CLICKING ON "ACCEPT" WHERE INDICATED BELOW, OR BY COPYING, INSTALLING OR
OTHERWISE USING PYTHON 1.6.1 SOFTWARE, YOU ARE DEEMED TO HAVE AGREED TO THE
TERMS AND CONDITIONS OF THIS LICENSE AGREEMENT.

   (( 1. ))??
   This LICENSE AGREEMENT is between the Corporation for National Research
   Initiatives, having an office at 1895 Preston White Drive, Reston, VA 20191
   ("CNRI"), and the Individual or Organization ("Licensee") accessing and
   otherwise using Python 1.6.1 software in source or binary form and its
   associated documentation.

   (( 2. ))??
   Subject to the terms and conditions of this License Agreement, CNRI hereby
   grants Licensee a nonexclusive, royalty-free, world-wide license to
   reproduce, analyze, test, perform and/or display publicly, prepare derivative
   works, distribute, and otherwise use Python 1.6.1 alone or in any derivative
   version, provided, however, that CNRI's License Agreement and CNRI's notice
   of copyright, i.e., "Copyright © 1995-2001 Corporation for National Research
   Initiatives; All Rights Reserved" are retained in Python 1.6.1 alone or in
   any derivative version prepared by Licensee. Alternately, in lieu of CNRI's
   License Agreement, Licensee may substitute the following text (omitting the
   quotes): "Python 1.6.1 is made available subject to the terms and conditions
   in CNRI's License Agreement. This Agreement together with Python 1.6.1 may be
   located on the Internet using the following unique, persistent identifier
   (known as a handle): 1895.22/1013. This Agreement may also be obtained from a
   proxy server on the Internet using the following URL:
   http:/hdl.handle.net/1895.22/1013".

   (( 3. ))??
   In the event Licensee prepares a derivative work that is based on or
   incorporates Python 1.6.1 or any part thereof, and wants to make the
   derivative work available to others as provided herein, then Licensee hereby
   agrees to include in any such work a brief summary of the changes made to
   Python 1.6.1.

   (( 4. ))??
   CNRI is making Python 1.6.1 available to Licensee on an "AS IS" basis. CNRI
   MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED. BY WAY OF
   EXAMPLE, BUT NOT LIMITATION, CNRI MAKES NO AND DISCLAIMS ANY REPRESENTATION
   OR WARRANTY OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT
   THE USE OF PYTHON 1.6.1 WILL NOT INFRINGE ANY THIRD PARTY RIGHTS.

   (( 5. ))??
   CNRI SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF PYTHON 1.6.1 FOR
   ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS A RESULT OF
   MODIFYING, DISTRIBUTING, OR OTHERWISE USING PYTHON 1.6.1, OR ANY DERIVATIVE
   THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

   (( 6. ))??
   This License Agreement will automatically terminate upon a material breach of
   its terms and conditions.

   (( 7. ))??
   This License Agreement shall be governed by the federal intellectual property
   law of the United States, including without limitation the federal copyright
   law, and, to the extent such U.S. federal law does not apply, by the law of
   the Commonwealth of Virginia, excluding Virginia's conflict of law
   provisions. Notwithstanding the foregoing, with regard to derivative works
   based on Python 1.6.1 that incorporate non-separable material that was
   previously distributed under the GNU General Public License (GPL), the law of
   the Commonwealth of Virginia shall govern this License Agreement only as to
   issues arising under or with respect to Paragraphs 4, 5, and 7 of this
   License Agreement. Nothing in this License Agreement shall be deemed to
   create any relationship of agency, partnership, or joint venture between CNRI
   and Licensee. This License Agreement does not grant permission to use CNRI
   trademarks or trade name in a trademark sense to endorse or promote products
   or services of Licensee, or any third party.

   (( 8. ))??
   By clicking on the "ACCEPT" button where indicated, or by copying, installing
   or otherwise using Python 1.6.1, Licensee agrees to be bound by the terms and
   conditions of this License Agreement.
   (( ACCEPT ))??
`
const license_CPAL_1_0_lre = `//**
Common Public Attribution License 1.0
https://spdx.org/licenses/CPAL-1.0.json
https://opensource.org/licenses/CPAL-1.0
**//

(( Common Public Attribution License Version 1.0 (CPAL) ))??

   (( 1. ))??
   "Definitions"

      (( 1.0.1 ))??
      "Commercial Use" means distribution or otherwise making the Covered Code
      available to a third party.

      (( 1.1 ))??
      "Contributor" means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2 ))??
      "Contributor Version" means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3 ))??
      "Covered Code" means the Original Code or Modifications or the combination
      of the Original Code and Modifications, in each case including portions
      thereof.

      (( 1.4 ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5 ))??
      "Executable" means Covered Code in any form other than Source Code.

      (( 1.6 ))??
      "Initial Developer" means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7 ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.8 ))??
      "License" means this document.

      (( 1.8.1 ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.9 ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10 ))??
      "Original Code" means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      (( 1.10.1 ))??
      "Patent Claims" means any patent claim(s), now owned or hereafter
      acquired, including without limitation, method, process, and apparatus
      claims, in any patent Licensable by grantor.

      (( 1.11 ))??
      "Source Code" means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or source code differential comparisons
      against either the Original Code or another well known, available Covered
      Code of the Contributor's choice. The Source Code can be in a compressed
      or archival form, provided the appropriate decompression or de-archiving
      software is widely available for no charge.

      (( 1.12 ))??
      "You" (or "Your") means an individual or a legal entity exercising rights
      under, and complying with all of the terms of, this License or a future
      version of this License issued under Section 6.1. For legal entities,
      "You" includes any entity which controls, is controlled by, or is under
      common control with You. For purposes of this definition, "control" means
      (a) the power, direct or indirect, to cause the direction or management of
      such entity, whether by contract or otherwise, or (b) ownership of more
      than fifty percent (50%) of the outstanding shares or beneficial ownership
      of such entity.

   (( 2. ))??
   Source Code License.

      (( 2.1 ))??
      The Initial Developer Grant.

      The Initial Developer hereby grants You a world-wide, royalty-free,
      non-exclusive license, subject to third party intellectual property
      claims:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Initial Developer to use, reproduce, modify, display,
         perform, sublicense and distribute the Original Code (or portions
         thereof) with or without Modifications, and/or as part of a Larger
         Work; and

         (( (b) ))??
         under Patents Claims infringed by the making, using or selling of
         Original Code, to make, have made, use, practice, sell, and offer for
         sale, and/or otherwise dispose of the Original Code (or portions
         thereof).

         (( c ))??
         the licenses granted in this Section 2.1(a) and (b) are effective on
         the date Initial Developer first distributes Original Code under the
         terms of this License.

         (( (d) ))??
         Notwithstanding Section 2.1(b) above, no patent license is granted: 1)
         for code that You delete from the Original Code; 2) separate from the
         Original Code; or 3) for infringements caused by: i) the modification
         of the Original Code or ii) the combination of the Original Code with
         other software or devices.

      (( 2.2 ))??
      Contributor Grant.

      Subject to third party intellectual property claims, each Contributor
      hereby grants You a world-wide, royalty-free, non-exclusive license

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Contributor, to use, reproduce, modify, display, perform,
         sublicense and distribute the Modifications created by such Contributor
         (or portions thereof) either on an unmodified basis, with other
         Modifications, as Covered Code and/or as part of a Larger Work; and

         (( (b) ))??
         under Patent Claims infringed by the making, using, or selling of
         Modifications made by that Contributor either alone and/or in
         combination with its Contributor Version (or portions of such
         combination), to make, use, sell, offer for sale, have made, and/or
         otherwise dispose of: 1) Modifications made by that Contributor (or
         portions thereof); and 2) the combination of Modifications made by that
         Contributor with its Contributor Version (or portions of such
         combination).

         (( c ))??
         the licenses granted in Sections 2.2(a) and 2.2(b) are effective on the
         date Contributor first makes Commercial Use of the Covered Code.

         (( (d) ))??
         Notwithstanding Section 2.2(b) above, no patent license is granted: 1)
         for any code that Contributor has deleted from the Contributor Version;
         2) separate from the Contributor Version; 3) for infringements caused
         by: i) third party modifications of Contributor Version or ii) the
         combination of Modifications made by that Contributor with other
         software (except as part of the Contributor Version) or other devices;
         or 4) under Patent Claims infringed by Covered Code in the absence of
         Modifications made by that Contributor.

   (( 3. ))??
   Distribution Obligations.

      (( 3.1 ))??
      Application of License.

      The Modifications which You create or to which You contribute are governed
      by the terms of this License, including without limitation Section 2.2.
      The Source Code version of Covered Code may be distributed only under the
      terms of this License or a future version of this License released under
      Section 6.1, and You must include a copy of this License with every copy
      of the Source Code You distribute. You may not offer or impose any terms
      on any Source Code version that alters or restricts the applicable version
      of this License or the recipients' rights hereunder. However, You may
      include an additional document offering the additional rights described in
      Section 3.5.

      (( 3.2 ))??
      Availability of Source Code.

      Any Modification which You create or to which You contribute must be made
      available in Source Code form under the terms of this License either on
      the same media as an Executable version or via an accepted Electronic
      Distribution Mechanism to anyone to whom you made an Executable version
      available; and if made available via Electronic Distribution Mechanism,
      must remain available for at least twelve (12) months after the date it
      initially became available, or at least six (6) months after a subsequent
      version of that particular Modification has been made available to such
      recipients. You are responsible for ensuring that the Source Code version
      remains available even if the Electronic Distribution Mechanism is
      maintained by a third party.

      (( 3.3 ))??
      Description of Modifications.

      You must cause all Covered Code to which You contribute to contain a file
      documenting the changes You made to create that Covered Code and the date
      of any change. You must include a prominent statement that the
      Modification is derived, directly or indirectly, from Original Code
      provided by the Initial Developer and including the name of the Initial
      Developer in (a) the Source Code, and (b) in any notice in an Executable
      version or related documentation in which You describe the origin or
      ownership of the Covered Code.

      (( 3.4 ))??
      Intellectual Property Matters

         (( (a) ))??
         Third Party Claims. If Contributor has knowledge that a license under a
         third party's intellectual property rights is required to exercise the
         rights granted by such Contributor under Sections 2.1 or 2.2,
         Contributor must include a text file with the Source Code distribution
         titled "LEGAL" which describes the claim and the party making the claim
         in sufficient detail that a recipient will know whom to contact. If
         Contributor obtains such knowledge after the Modification is made
         available as described in Section 3.2, Contributor shall promptly
         modify the LEGAL file in all copies Contributor makes available
         thereafter and shall take other steps (such as notifying appropriate
         mailing lists or newsgroups) reasonably calculated to inform those who
         received the Covered Code that new knowledge has been obtained.

         (( (b) ))??
         Contributor APIs. If Contributor's Modifications include an application
         programming interface and Contributor has knowledge of patent licenses
         which are reasonably necessary to implement that API, Contributor must
         also include this information in the LEGAL file.

         (( c ))??
         Representations. Contributor represents that, except as disclosed
         pursuant to Section 3.4(a) above, Contributor believes that
         Contributor's Modifications are Contributor's original creation(s)
         and/or Contributor has sufficient rights to grant the rights conveyed
         by this License.

      (( 3.5 ))??
      Required Notices.

      You must duplicate the notice in Exhibit A in each file of the Source
      Code. If it is not possible to put such notice in a particular Source Code
      file due to its structure, then You must include such notice in a location
      (such as a relevant directory) where a user would be likely to look for
      such a notice. If You created one or more Modification(s) You may add your
      name as a Contributor to the notice described in Exhibit A. You must also
      duplicate this License in any documentation for the Source Code where You
      describe recipients' rights or ownership rights relating to Covered Code.
      You may choose to offer, and to charge a fee for, warranty, support,
      indemnity or liability obligations to one or more recipients of Covered
      Code. However, You may do so only on Your own behalf, and not on behalf of
      the Initial Developer or any Contributor. You must make it absolutely
      clear than any such warranty, support, indemnity or liability obligation
      is offered by You alone, and You hereby agree to indemnify the Initial
      Developer and every Contributor for any liability incurred by the Initial
      Developer or such Contributor as a result of warranty, support, indemnity
      or liability terms You offer.

      (( 3.6 ))??
      Distribution of Executable Versions.

      You may distribute Covered Code in Executable form only if the
      requirements of Section 3.1-3.5 have been met for that Covered Code, and
      if You include a notice stating that the Source Code version of the
      Covered Code is available under the terms of this License, including a
      description of how and where You have fulfilled the obligations of Section
      3.2. The notice must be conspicuously included in any notice in an
      Executable version, related documentation or collateral in which You
      describe recipients' rights relating to the Covered Code. You may
      distribute the Executable version of Covered Code or ownership rights
      under a license of Your choice, which may contain terms different from
      this License, provided that You are in compliance with the terms of this
      License and that the license for the Executable version does not attempt
      to limit or alter the recipient's rights in the Source Code version from
      the rights set forth in this License. If You distribute the Executable
      version under a different license You must make it absolutely clear that
      any terms which differ from this License are offered by You alone, not by
      the Initial Developer, Original Developer or any Contributor. You hereby
      agree to indemnify the Initial Developer, Original Developer and every
      Contributor for any liability incurred by the Initial Developer, Original
      Developer or such Contributor as a result of any such terms You offer.

      (( 3.7 ))??
      Larger Works.

      You may create a Larger Work by combining Covered Code with other code not
      governed by the terms of this License and distribute the Larger Work as a
      single product. In such a case, You must make sure the requirements of
      this License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute, judicial
   order, or regulation then You must: (a) comply with the terms of this License
   to the maximum extent possible; and (b) describe the limitations and the code
   they affect. Such description must be included in the LEGAL file described in
   Section 3.4 and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Application of this License.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A and to related Covered Code.

   (( 6. ))??
   Versions of the License.

      (( 6.1 ))??
      New Versions.

      Socialtext, Inc. ("Socialtext") may publish revised and/or new versions of
      the License from time to time. Each version will be given a distinguishing
      version number.

      (( 6.2 ))??
      Effect of New Versions.

      Once Covered Code has been published under a particular version of the
      License, You may always continue to use it under the terms of that
      version. You may also choose to use such Covered Code under the terms of
      any subsequent version of the License published by Socialtext. No one
      other than Socialtext has the right to modify the terms applicable to
      Covered Code created under this License.

      (( 6.3 ))??
      Derivative Works.

      If You create or use a modified version of this License (which you may
      only do in order to apply it to code which is not already Covered Code
      governed by this License), You must (a) rename Your license so that the
      phrases "Socialtext", "CPAL" or any confusingly similar phrase do not
      appear in your license (except to note that your license differs from this
      License) and (b) otherwise make it clear that Your version of the license
      contains terms which differ from the CPAL. (Filling in the name of the
      Initial Developer, Original Developer, Original Code or Contributor in the
      notice described in Exhibit A shall not of themselves be deemed to be
      modifications of this License.)

   (( 7. ))??
   DISCLAIMER OF WARRANTY.

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER,
   ORIGINAL DEVELOPER OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY
   SERVICING, REPAIR OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN
   ESSENTIAL PART OF THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED
   HEREUNDER EXCEPT UNDER THIS DISCLAIMER.

   (( 8. ))??
   TERMINATION.

      (( 8.1 ))??
      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. All sublicenses to the
      Covered Code which are properly granted shall survive any termination of
      this License. Provisions which, by their nature, must remain in effect
      beyond the termination of this License shall survive.

      (( 8.2 ))??
      If You initiate litigation by asserting a patent infringement claim
      (excluding declatory judgment actions) against Initial Developer, Original
      Developer or a Contributor (the Initial Developer, Original Developer or
      Contributor against whom You file such action is referred to as
      "Participant") alleging that:

         (( (a) ))??
         such Participant's Contributor Version directly or indirectly infringes
         any patent, then any and all rights granted by such Participant to You
         under Sections 2.1 and/or 2.2 of this License shall, upon 60 days
         notice from Participant terminate prospectively, unless if within 60
         days after receipt of notice You either: (i) agree in writing to pay
         Participant a mutually agreeable reasonable royalty for Your past and
         future use of Modifications made by such Participant, or (ii) withdraw
         Your litigation claim with respect to the Contributor Version against
         such Participant. If within 60 days of notice, a reasonable royalty and
         payment arrangement are not mutually agreed upon in writing by the
         parties or the litigation claim is not withdrawn, the rights granted by
         Participant to You under Sections 2.1 and/or 2.2 automatically
         terminate at the expiration of the 60 day notice period specified
         above.

         (( (b) ))??
         any software, hardware, or device, other than such Participant's
         Contributor Version, directly or indirectly infringes any patent, then
         any rights granted to You by such Participant under Sections 2.1(b) and
         2.2(b) are revoked effective as of the date You first made, used, sold,
         distributed, or had made, Modifications made by that Participant.

      (( 8.3 ))??
      If You assert a patent infringement claim against Participant alleging
      that such Participant's Contributor Version directly or indirectly
      infringes any patent where such claim is resolved (such as by license or
      settlement) prior to the initiation of patent infringement litigation,
      then the reasonable value of the licenses granted by such Participant
      under Sections 2.1 or 2.2 shall be taken into account in determining the
      amount or value of any payment or license.

      (( 8.4 ))??
      In the event of termination under Sections 8.1 or 8.2 above, all end user
      license agreements (excluding distributors and resellers) which have been
      validly granted by You or any distributor hereunder prior to termination
      shall survive termination.

   (( 9. ))??
   LIMITATION OF LIABILITY.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, THE INITIAL DEVELOPER,
   ORIGINAL DEVELOPER, ANY OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED
   CODE, OR ANY SUPPLIER OF ANY OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY
   INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER
   INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE,
   COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR
   LOSSES, EVEN IF SUCH PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF
   SUCH DAMAGES. THIS LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR
   DEATH OR PERSONAL INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT
   APPLICABLE LAW PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE
   EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS
   EXCLUSION AND LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. GOVERNMENT END USERS.

   The Covered Code is a "commercial item," as that term is defined in 48 C.F.R.
   2.101 (Oct. 1995), consisting of "commercial computer software" and
   "commercial computer software documentation," as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Covered Code with only those rights set forth herein.

   (( 11. ))??
   MISCELLANEOUS.

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by California law provisions
   (except to the extent applicable law, if any, provides otherwise), excluding
   its conflict-of-law provisions. With respect to disputes in which at least
   one party is a citizen of, or an entity chartered or registered to do
   business in the United States of America, any litigation relating to this
   License shall be subject to the jurisdiction of the Federal Courts of the
   Northern District of California, with venue lying in Santa Clara County,
   California, with the losing party responsible for costs, including without
   limitation, court costs and reasonable attorneys' fees and expenses. The
   application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any law or regulation
   which provides that the language of a contract shall be construed against the
   drafter shall not apply to this License.

   (( 12. ))??
   RESPONSIBILITY FOR CLAIMS.

   As between Initial Developer, Original Developer and the Contributors, each
   party is responsible for claims and damages arising, directly or indirectly,
   out of its utilization of rights under this License and You agree to work
   with Initial Developer, Original Developer and Contributors to distribute
   such responsibility on an equitable basis. Nothing herein is intended or
   shall be deemed to constitute any admission of liability.

   (( 13. ))??
   MULTIPLE-LICENSED CODE.

   Initial Developer may designate portions of the Covered Code as
   Multiple-Licensed. Multiple-Licensed means that the Initial Developer permits
   you to utilize portions of the Covered Code under Your choice of the CPAL or
   the alternative licenses, if any, specified by the Initial Developer in the
   file described in Exhibit A.

   (( 14. ))??
   ADDITIONAL TERM: ATTRIBUTION

      (( (a) ))??
      As a modest attribution to the organizer of the development of the
      Original Code ("Original Developer"), in the hope that its promotional
      value may help justify the time, money and effort invested in writing the
      Original Code, the Original Developer may include in Exhibit B
      ("Attribution Information") a requirement that each time an Executable and
      Source Code or a Larger Work is launched or initially run (which includes
      initiating a session), a prominent display of the Original Developer's
      Attribution Information (as defined below) must occur on the graphic user
      interface employed by the end user to access such Covered Code (which may
      include display on a splash screen), if any. The size of the graphic image
      should be consistent with the size of the other elements of the
      Attribution Information. If the access by the end user to the Executable
      and Source Code does not create a graphic user interface for access to the
      Covered Code, this obligation shall not apply. If the Original Code
      displays such Attribution Information in a particular form (such as in the
      form of a splash screen, notice at login, an "about" display, or dedicated
      attribution area on user interface screens), continued use of such form
      for that Attribution Information is one way of meeting this requirement
      for notice.

      (( (b) ))??
      Attribution information may only include a copyright notice, a brief
      phrase, graphic image and a URL ("Attribution Information") and is subject
      to the Attribution Limits as defined below. For these purposes, prominent
      shall mean display for sufficient duration to give reasonable notice to
      the user of the identity of the Original Developer and that if You include
      Attribution Information or similar information for other parties, You must
      ensure that the Attribution Information for the Original Developer shall
      be no less prominent than such Attribution Information or similar
      information for the other party. For greater certainty, the Original
      Developer may choose to specify in Exhibit B below that the above
      attribution requirement only applies to an Executable and Source Code
      resulting from the Original Code or any Modification, but not a Larger
      Work. The intent is to provide for reasonably modest attribution,
      therefore the Original Developer cannot require that You display, at any
      time, more than the following information as Attribution Information: (a)
      a copyright notice including the name of the Original Developer; (b) a
      word or one phrase (not exceeding 10 words); (c) one graphic image
      provided by the Original Developer; and (d) a URL (collectively, the
      "Attribution Limits").

      (( c ))??
      If Exhibit B does not include any Attribution Information, then there are
      no requirements for You to display any Attribution Information of the
      Original Developer.

      (( (d) ))??
      You acknowledge that all trademarks, service marks and/or trade names
      contained within the Attribution Information distributed with the Covered
      Code are the exclusive property of their owners and may only be used with
      the permission of their owners, or under circumstances otherwise permitted
      by law or as expressly set out in this License.

   (( 15. ))??
   ADDITIONAL TERM: NETWORK USE.

   The term "External Deployment" means the use, distribution, or communication
   of the Original Code or Modifications in any way such that the Original Code
   or Modifications may be used by anyone other than You, whether those works
   are distributed or communicated to those persons or made available as an
   application intended for use over a network. As an express condition for the
   grants of license hereunder, You must treat any External Deployment by You of
   the Original Code or Modifications as a distribution under section 3.1 and
   make Source Code available under Section 3.2.

(( EXHIBIT A. Common Public Attribution License Version 1.0.

"The contents of this file are subject to the Common Public Attribution License
Version 1.0 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at _____________.

The License is based on the Mozilla Public License Version 1.1 but Sections 14
and 15 have been added to cover use of software over a computer network and
provide for limited attribution for the Original Developer. In addition, Exhibit
A has been modified to be consistent with Exhibit B.

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

The Original Code is ______________________.

The Original Developer is not the Initial Developer and is __________.
If left blank, the Original Developer is the Initial Developer.

The Initial Developer of the Original Code is ____________.
All portions of the code written by ___________
are Copyright (c) ______________.
All Rights Reserved.

Contributor ______________________

Alternatively, the contents of this file may be used under the terms of the
_____ license (the [___] License), in which case the provisions of
[______] License are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
the [____]
License and not to allow others to use your version of this file under the CPAL,
indicate your decision by deleting the provisions above and replace them with
the notice and other provisions required by the
[___] License. If you do not delete the provisions above, a recipient may use your
version of this file under either the CPAL or the
[___] License."

[NOTE: The text of this Exhibit A may differ slightly from the text of the
notices in the Source Code files of the Original Code. You should use the text
of this Exhibit A rather than the text found in the Original Code Source Code
for Your Modifications.]

EXHIBIT B. Attribution Information

Attribution Copyright Notice: _

Attribution Phrase (not exceeding 10 words): _

Attribution URL: _

Graphic Image as provided in the Covered Code, if any.

Display of Attribution Information is [required/not required] in Larger Works
which are defined in the CPAL as a work which combines Covered Code or portions
thereof with code not governed by the terms of the CPAL.
))??
`
const license_CPL_1_0_lre = `//**
Common Public License 1.0
https://spdx.org/licenses/CPL-1.0.json
https://opensource.org/licenses/CPL-1.0
**//

(( Common Public License Version 1.0 ))??

THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS COMMON PUBLIC
LICENSE ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THE PROGRAM
CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.
(( 1. ))??

DEFINITIONS

"Contribution" means:

   (( a) ))??
   in the case of the initial Contributor, the initial code and documentation
   distributed under this Agreement, and

   (( b) ))??
   in the case of each subsequent Contributor:

      (( i) ))??
      changes to the Program, and

      (( ii) ))??
      additions to the Program;

where such changes and/or additions to the Program originate from and are
distributed by that particular Contributor. A Contribution 'originates' from a
Contributor if it was added to the Program by such Contributor itself or anyone
acting on such Contributor's behalf. Contributions do not include additions to
the Program which: (i) are separate modules of software distributed in
conjunction with the Program under their own license agreement, and (ii) are not
derivative works of the Program.

"Contributor" means any person or entity that distributes the Program.

"Licensed Patents " mean patent claims licensable by a Contributor which are
necessarily infringed by the use or sale of its Contribution alone or when
combined with the Program.

"Program" means the Contributions distributed in accordance with this
Agreement.

"Recipient" means anyone who receives the Program under this Agreement,
including all Contributors.
(( 2. ))??

GRANT OF RIGHTS

   (( a) ))??
   Subject to the terms of this Agreement, each Contributor hereby grants
   Recipient a non-exclusive, worldwide, royalty-free copyright license to
   reproduce, prepare derivative works of, publicly display, publicly perform,
   distribute and sublicense the Contribution of such Contributor, if any, and
   such derivative works, in source code and object code form.

   (( b) ))??
   Subject to the terms of this Agreement, each Contributor hereby grants
   Recipient a non-exclusive, worldwide, royalty-free patent license under
   Licensed Patents to make, use, sell, offer to sell, import and otherwise
   transfer the Contribution of such Contributor, if any, in source code and
   object code form. This patent license shall apply to the combination of the
   Contribution and the Program if, at the time the Contribution is added by the
   Contributor, such addition of the Contribution causes such combination to be
   covered by the Licensed Patents. The patent license shall not apply to any
   other combinations which include the Contribution. No hardware per se is
   licensed hereunder.

   (( c) ))??
   Recipient understands that although each Contributor grants the licenses to
   its Contributions set forth herein, no assurances are provided by any
   Contributor that the Program does not infringe the patent or other
   intellectual property rights of any other entity. Each Contributor disclaims
   any liability to Recipient for claims brought by any other entity based on
   infringement of intellectual property rights or otherwise. As a condition to
   exercising the rights and licenses granted hereunder, each Recipient hereby
   assumes sole responsibility to secure any other intellectual property rights
   needed, if any. For example, if a third party patent license is required to
   allow Recipient to distribute the Program, it is Recipient's responsibility
   to acquire that license before distributing the Program.

   (( d) ))??
   Each Contributor represents that to its knowledge it has sufficient copyright
   rights in its Contribution, if any, to grant the copyright license set forth
   in this Agreement.
   (( 3. ))??

REQUIREMENTS

A Contributor may choose to distribute the Program in object code form under its
own license agreement, provided that:

   (( a) ))??
   it complies with the terms and conditions of this Agreement; and

   (( b) ))??
   its license agreement:

      (( i) ))??
      effectively disclaims on behalf of all Contributors all warranties and
      conditions, express and implied, including warranties or conditions of
      title and non-infringement, and implied warranties or conditions of
      merchantability and fitness for a particular purpose;

      (( ii) ))??
      effectively excludes on behalf of all Contributors all liability for
      damages, including direct, indirect, special, incidental and consequential
      damages, such as lost profits;

      (( iii) ))??
      states that any provisions which differ from this Agreement are offered by
      that Contributor alone and not by any other party; and

      (( iv) ))??
      states that source code for the Program is available from such
      Contributor, and informs licensees how to obtain it in a reasonable manner
      on or through a medium customarily used for software exchange.

When the Program is made available in source code form:

   (( a) ))??
   it must be made available under this Agreement; and

   (( b) ))??
   a copy of this Agreement must be included with each copy of the Program.

   Contributors may not remove or alter any copyright notices contained within
   the Program.

   Each Contributor must identify itself as the originator of its Contribution,
   if any, in a manner that reasonably allows subsequent Recipients to identify
   the originator of the Contribution.
   (( 4. ))??

COMMERCIAL DISTRIBUTION

Commercial distributors of software may accept certain responsibilities with
respect to end users, business partners and the like. While this license is
intended to facilitate the commercial use of the Program, the Contributor who
includes the Program in a commercial product offering should do so in a manner
which does not create potential liability for other Contributors. Therefore, if
a Contributor includes the Program in a commercial product offering, such
Contributor ("Commercial Contributor") hereby agrees to defend and indemnify
every other Contributor ("Indemnified Contributor") against any losses, damages
and costs (collectively "Losses") arising from claims, lawsuits and other legal
actions brought by a third party against the Indemnified Contributor to the
extent caused by the acts or omissions of such Commercial Contributor in
connection with its distribution of the Program in a commercial product
offering. The obligations in this section do not apply to any claims or Losses
relating to any actual or alleged intellectual property infringement. In order
to qualify, an Indemnified Contributor must: a) promptly notify the Commercial
Contributor in writing of such claim, and b) allow the Commercial Contributor to
control, and cooperate with the Commercial Contributor in, the defense and any
related settlement negotiations. The Indemnified Contributor may participate in
any such claim at its own expense.

For example, a Contributor might include the Program in a commercial product
offering, Product X. That Contributor is then a Commercial Contributor. If that
Commercial Contributor then makes performance claims, or offers warranties
related to Product X, those performance claims and warranties are such
Commercial Contributor's responsibility alone. Under this section, the
Commercial Contributor would have to defend claims against the other
Contributors related to those performance claims and warranties, and if a court
requires any other Contributor to pay any damages as a result, the Commercial
Contributor must pay those damages.
(( 5. ))??

NO WARRANTY

EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, THE PROGRAM IS PROVIDED ON AN
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR
IMPLIED INCLUDING, WITHOUT LIMITATION, ANY WARRANTIES OR CONDITIONS OF TITLE,
NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Each
Recipient is solely responsible for determining the appropriateness of using and
distributing the Program and assumes all risks associated with its exercise of
rights under this Agreement, including but not limited to the risks and costs of
program errors, compliance with applicable laws, damage to or loss of data,
programs or equipment, and unavailability or interruption of operations.
(( 6. ))??

DISCLAIMER OF LIABILITY

EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, NEITHER RECIPIENT NOR ANY
CONTRIBUTORS SHALL HAVE ANY LIABILITY FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING WITHOUT LIMITATION LOST
PROFITS), HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OR DISTRIBUTION OF THE PROGRAM OR THE EXERCISE OF ANY RIGHTS
GRANTED HEREUNDER, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
(( 7. ))??

GENERAL

If any provision of this Agreement is invalid or unenforceable under applicable
law, it shall not affect the validity or enforceability of the remainder of the
terms of this Agreement, and without further action by the parties hereto, such
provision shall be reformed to the minimum extent necessary to make such
provision valid and enforceable.

If Recipient institutes patent litigation against a Contributor with respect to
a patent applicable to software (including a cross-claim or counterclaim in a
lawsuit), then any patent licenses granted by that Contributor to such Recipient
under this Agreement shall terminate as of the date such litigation is filed. In
addition, if Recipient institutes patent litigation against any entity
(including a cross-claim or counterclaim in a lawsuit) alleging that the Program
itself (excluding combinations of the Program with other software or hardware)
infringes such Recipient's patent(s), then such Recipient's rights granted under
Section 2(b) shall terminate as of the date such litigation is filed.

All Recipient's rights under this Agreement shall terminate if it fails to
comply with any of the material terms or conditions of this Agreement and does
not cure such failure in a reasonable period of time after becoming aware of
such noncompliance. If all Recipient's rights under this Agreement terminate,
Recipient agrees to cease use and distribution of the Program as soon as
reasonably practicable. However, Recipient's obligations under this Agreement
and any licenses granted by Recipient relating to the Program shall continue and
survive.

Everyone is permitted to copy and distribute copies of this Agreement, but in
order to avoid inconsistency the Agreement is copyrighted and may only be
modified in the following manner. The Agreement Steward reserves the right to
publish new versions (including revisions) of this Agreement from time to time.
No one other than the Agreement Steward has the right to modify this Agreement.
IBM is the initial Agreement Steward. IBM may assign the responsibility to serve
as the Agreement Steward to a suitable separate entity. Each new version of the
Agreement will be given a distinguishing version number. The Program (including
Contributions) may always be distributed subject to the version of the Agreement
under which it was received. In addition, after a new version of the Agreement
is published, Contributor may elect to distribute the Program (including its
Contributions) under the new version. Except as expressly stated in Sections
2(a) and 2(b) above, Recipient receives no rights or licenses to the
intellectual property of any Contributor under this Agreement, whether
expressly, by implication, estoppel or otherwise. All rights in the Program not
expressly granted under this Agreement are reserved.

This Agreement is governed by the laws of the State of New York and the
intellectual property laws of the United States of America. No party to this
Agreement will bring a legal action under this Agreement more than one year
after the cause of action arose. Each party waives its rights to a jury trial in
any resulting litigation.
`
const license_CPOL_1_02_lre = `//**
Code Project Open License 1.02
https://spdx.org/licenses/CPOL-1.02.json
http://www.codeproject.com/info/cpol10.aspx
**//

(( The Code Project Open License (CPOL) 1.02 ))??

Preamble

This License governs Your use of the Work. This License is intended to allow
developers to use the Source Code and Executable Files provided as part of the
Work in any application in any form.

The main points subject to the terms of the License are:

   -
   Source Code and Executable Files can be used in commercial applications;

   -
   Source Code and Executable Files can be redistributed; and

   -
   Source Code can be modified to create derivative works.

   -
   No claim of suitability, guarantee, or any warranty whatsoever is provided.
   The software is provided "as-is".

   -
   The Article accompanying the Work may not be distributed or republished
   without the Author's consent

This License is entered between You, the individual or other entity reading or
otherwise making use of the Work licensed pursuant to this License and the
individual or other entity which offers the Work under the terms of this License
("Author").

License

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CODE PROJECT
OPEN LICENSE ("LICENSE"). THE WORK IS PROTECTED BY COPYRIGHT AND/OR OTHER
APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS LICENSE
OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HEREIN, YOU ACCEPT AND AGREE TO BE
BOUND BY THE TERMS OF THIS LICENSE. THE AUTHOR GRANTS YOU THE RIGHTS CONTAINED
HEREIN IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS. IF YOU
DO NOT AGREE TO ACCEPT AND BE BOUND BY THE TERMS OF THIS LICENSE, YOU CANNOT
MAKE ANY USE OF THE WORK.

   (( 1. ))??
   Definitions.

      (( a. ))??
      "Articles" means, collectively, all articles written by Author which
      describes how the Source Code and Executable Files for the Work may be
      used by a user.

      (( b. ))??
      "Author" means the individual or entity that offers the Work under the
      terms of this License.

      (( c. ))??
      "Derivative Work" means a work based upon the Work or upon the Work and
      other pre-existing works.

      (( d. ))??
      "Executable Files" refer to the executables, binary files, configuration
      and any required data files included in the Work.

      (( e. ))??
      "Publisher" means the provider of the website, magazine, CD-ROM, DVD or
      other medium from or by which the Work is obtained by You.

      (( f. ))??
      "Source Code" refers to the collection of source code and configuration
      files used to create the Executable Files.

      (( g. ))??
      "Standard Version" refers to such a Work if it has not been modified, or
      has been modified in accordance with the consent of the Author, such
      consent being in the full discretion of the Author.

      (( h. ))??
      "Work" refers to the collection of files distributed by the Publisher,
      including the Source Code, Executable Files, binaries, data files,
      documentation, whitepapers and the Articles.

      (( i. ))??
      "You" is you, an individual or entity wishing to use the Work and exercise
      your rights under this License.

   (( 2. ))??
   Fair Use/Fair Use Rights. Nothing in this License is intended to reduce,
   limit, or restrict any rights arising from fair use, fair dealing, first sale
   or other limitations on the exclusive rights of the copyright owner under
   copyright law or other applicable laws.

   (( 3. ))??
   License Grant. Subject to the terms and conditions of this License, the
   Author hereby grants You a worldwide, royalty-free, non-exclusive, perpetual
   (for the duration of the applicable copyright) license to exercise the rights
   in the Work as stated below:

      (( a. ))??
      You may use the standard version of the Source Code or Executable Files in
      Your own applications.

      (( b. ))??
      You may apply bug fixes, portability fixes and other modifications
      obtained from the Public Domain or from the Author. A Work modified in
      such a way shall still be considered the standard version and will be
      subject to this License.

      (( c. ))??
      You may otherwise modify Your copy of this Work (excluding the Articles)
      in any way to create a Derivative Work, provided that You insert a
      prominent notice in each changed file stating how, when and where You
      changed that file.

      (( d. ))??
      You may distribute the standard version of the Executable Files and Source
      Code or Derivative Work in aggregate with other (possibly commercial)
      programs as part of a larger (possibly commercial) software distribution.

      (( e. ))??
      The Articles discussing the Work published in any form by the author may
      not be distributed or republished without the Author's consent. The author
      retains copyright to any such Articles. You may use the Executable Files
      and Source Code pursuant to this License but you may not repost or
      republish or otherwise distribute or make available the Articles, without
      the prior written consent of the Author.

   Any subroutines or modules supplied by You and linked into the Source Code or
   Executable Files of this Work shall not be considered part of this Work and
   will not be subject to the terms of this License.

   (( 4. ))??
   Patent License. Subject to the terms and conditions of this License, each
   Author hereby grants to You a perpetual, worldwide, non-exclusive, no-charge,
   royalty-free, irrevocable (except as stated in this section) patent license
   to make, have made, use, import, and otherwise transfer the Work.

   (( 5. ))??
   Restrictions. The license granted in Section 3 above is expressly made
   subject to and limited by the following restrictions:

      (( a. ))??
      You agree not to remove any of the original copyright, patent, trademark,
      and attribution notices and associated disclaimers that may appear in the
      Source Code or Executable Files.

      (( b. ))??
      You agree not to advertise or in any way imply that this Work is a product
      of Your own.

      (( c. ))??
      The name of the Author may not be used to endorse or promote products
      derived from the Work without the prior written consent of the Author.

      (( d. ))??
      You agree not to sell, lease, or rent any part of the Work. This does not
      restrict you from including the Work or any part of the Work inside a
      larger software distribution that itself is being sold. The Work by
      itself, though, cannot be sold, leased or rented.

      (( e. ))??
      You may distribute the Executable Files and Source Code only under the
      terms of this License, and You must include a copy of, or the Uniform
      Resource Identifier for, this License with every copy of the Executable
      Files or Source Code You distribute and ensure that anyone receiving such
      Executable Files and Source Code agrees that the terms of this License
      apply to such Executable Files and/or Source Code. You may not offer or
      impose any terms on the Work that alter or restrict the terms of this
      License or the recipients' exercise of the rights granted hereunder. You
      may not sublicense the Work. You must keep intact all notices that refer
      to this License and to the disclaimer of warranties. You may not
      distribute the Executable Files or Source Code with any technological
      measures that control access or use of the Work in a manner inconsistent
      with the terms of this License.

      (( f. ))??
      You agree not to use the Work for illegal, immoral or improper purposes,
      or on pages containing illegal, immoral or improper material. The Work is
      subject to applicable export laws. You agree to comply with all such laws
      and regulations that may apply to the Work after Your receipt of the
      Work.

   (( 6. ))??
   Representations, Warranties and Disclaimer. THIS WORK IS PROVIDED "AS IS",
   "WHERE IS" AND "AS AVAILABLE", WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OR
   CONDITIONS OR GUARANTEES. YOU, THE USER, ASSUME ALL RISK IN ITS USE,
   INCLUDING COPYRIGHT INFRINGEMENT, PATENT INFRINGEMENT, SUITABILITY, ETC.
   AUTHOR EXPRESSLY DISCLAIMS ALL EXPRESS, IMPLIED OR STATUTORY WARRANTIES OR
   CONDITIONS, INCLUDING WITHOUT LIMITATION, WARRANTIES OR CONDITIONS OF
   MERCHANTABILITY, MERCHANTABLE QUALITY OR FITNESS FOR A PARTICULAR PURPOSE, OR
   ANY WARRANTY OF TITLE OR NON-INFRINGEMENT, OR THAT THE WORK (OR ANY PORTION
   THEREOF) IS CORRECT, USEFUL, BUG-FREE OR FREE OF VIRUSES. YOU MUST PASS THIS
   DISCLAIMER ON WHENEVER YOU DISTRIBUTE THE WORK OR DERIVATIVE WORKS.

   (( 7. ))??
   Indemnity. You agree to defend, indemnify and hold harmless the Author and
   the Publisher from and against any claims, suits, losses, damages,
   liabilities, costs, and expenses (including reasonable legal or attorneys'
   fees) resulting from or relating to any use of the Work by You.

   (( 8. ))??
   Limitation on Liability. EXCEPT TO THE EXTENT REQUIRED BY APPLICABLE LAW, IN
   NO EVENT WILL THE AUTHOR OR THE PUBLISHER BE LIABLE TO YOU ON ANY LEGAL
   THEORY FOR ANY SPECIAL, INCIDENTAL, CONSEQUENTIAL, PUNITIVE OR EXEMPLARY
   DAMAGES ARISING OUT OF THIS LICENSE OR THE USE OF THE WORK OR OTHERWISE, EVEN
   IF THE AUTHOR OR THE PUBLISHER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
   DAMAGES.

   (( 9. ))??
   Termination.

      (( a. ))??
      This License and the rights granted hereunder will terminate automatically
      upon any breach by You of any term of this License. Individuals or
      entities who have received Derivative Works from You under this License,
      however, will not have their licenses terminated provided such individuals
      or entities remain in full compliance with those licenses. Sections 1, 2,
      6, 7, 8, 9, 10 and 11 will survive any termination of this License.

      (( b. ))??
      If You bring a copyright, trademark, patent or any other infringement
      claim against any contributor over infringements You claim are made by the
      Work, your License from such contributor to the Work ends automatically.

      (( c. ))??
      Subject to the above terms and conditions, this License is perpetual (for
      the duration of the applicable copyright in the Work). Notwithstanding the
      above, the Author reserves the right to release the Work under different
      license terms or to stop distributing the Work at any time; provided,
      however that any such election will not serve to withdraw this License (or
      any other license that has been, or is required to be, granted under the
      terms of this License), and this License will continue in full force and
      effect unless terminated as stated above.

   (( 10. ))??
   Publisher. The parties hereby confirm that the Publisher shall not, under any
   circumstances, be responsible for and shall not have any liability in respect
   of the subject matter of this License. The Publisher makes no warranty
   whatsoever in connection with the Work and shall not be liable to You or any
   party on any legal theory for any damages whatsoever, including without
   limitation any general, special, incidental or consequential damages arising
   in connection to this license. The Publisher reserves the right to cease
   making the Work available to You at any time without notice

   (( 11. ))??
   Miscellaneous

      (( a. ))??
      This License shall be governed by the laws of the location of the head
      office of the Author or if the Author is an individual, the laws of
      location of the principal place of residence of the Author.

      (( b. ))??
      If any provision of this License is invalid or unenforceable under
      applicable law, it shall not affect the validity or enforceability of the
      remainder of the terms of this License, and without further action by the
      parties to this License, such provision shall be reformed to the minimum
      extent necessary to make such provision valid and enforceable.

      (( c. ))??
      No term or provision of this License shall be deemed waived and no breach
      consented to unless such waiver or consent shall be in writing and signed
      by the party to be charged with such waiver or consent.

      (( d. ))??
      This License constitutes the entire agreement between the parties with
      respect to the Work licensed herein. There are no understandings,
      agreements or representations with respect to the Work not specified
      herein. The Author shall not be bound by any additional provisions that
      may appear in any communication from You. This License may not be modified
      without the mutual written agreement of the Author and You.
`
const license_CUA_OPL_1_0_lre = `//**
CUA Office Public License v1.0
https://spdx.org/licenses/CUA-OPL-1.0.json
https://opensource.org/licenses/CUA-OPL-1.0
**//

(( CUA Office Public License Version 1.0 ))??

   (( 1. ))??
   Definitions.

      (( 1.0.1. ))??
      "Commercial Use" means distribution or otherwise making the Covered Code
      available to a third party.

      (( 1.1. ))??
      "Contributor" means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      "Covered Code" means the Original Code or Modifications or the combination
      of the Original Code and Modifications, in each case including portions
      thereof.

      (( 1.4. ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      "Executable" means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      "Initial Developer" means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7. ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.8. ))??
      "License" means this document.

      (( 1.8.1. ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.9. ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10. ))??
      "Original Code" means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      (( 1.10.1. ))??
      "Patent Claims" means any patent claim(s), now owned or hereafter
      acquired, including without limitation, method, process, and apparatus
      claims, in any patent Licensable by grantor.

      (( 1.11. ))??
      "Source Code" means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or source code differential comparisons
      against either the Original Code or another well known, available Covered
      Code of the Contributor's choice. The Source Code can be in a compressed
      or archival form, provided the appropriate decompression or de-archiving
      software is widely available for no charge.

      (( 1.12. ))??
      "You" (or "Your") means an individual or a legal entity exercising rights
      under, and complying with all of the terms of, this License or a future
      version of this License issued under Section 6.1. For legal entities,
      "You" includes any entity which controls, is controlled by, or is under
      common control with You. For purposes of this definition, "control" means
      (a) the power, direct or indirect, to cause the direction or management of
      such entity, whether by contract or otherwise, or (b) ownership of more
      than fifty percent (50%) of the outstanding shares or beneficial ownership
      of such entity.

   (( 2. ))??
   Source Code License.

      (( 2.1. ))??
      The Initial Developer Grant. The Initial Developer hereby grants You a
      world-wide, royalty-free, non-exclusive license, subject to third party
      intellectual property claims:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Initial Developer to use, reproduce, modify, display,
         perform, sublicense and distribute the Original Code (or portions
         thereof) with or without Modifications, and/or as part of a Larger
         Work; and

         (( (b) ))??
         under Patents Claims infringed by the making, using or selling of
         Original Code, to make, have made, use, practice, sell, and offer for
         sale, and/or otherwise dispose of the Original Code (or portions
         thereof).

         (( c ))??
         the licenses granted in this Section 2.1(a) and (b) are effective on
         the date Initial Developer first distributes Original Code under the
         terms of this License.

         (( (d) ))??
         Notwithstanding Section 2.1(b) above, no patent license is granted: 1)
         for code that You delete from the Original Code; 2) separate from the
         Original Code; or 3) for infringements caused by: i) the modification
         of the Original Code or ii) the combination of the Original Code with
         other software or devices.

      (( 2.2. ))??
      Contributor Grant. Subject to third party intellectual property claims,
      each Contributor hereby grants You a world-wide, royalty-free,
      non-exclusive license

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Contributor, to use, reproduce, modify, display, perform,
         sublicense and distribute the Modifications created by such Contributor
         (or portions thereof) either on an unmodified basis, with other
         Modifications, as Covered Code and/or as part of a Larger Work; and

         (( (b) ))??
         under Patent Claims infringed by the making, using, or selling of
         Modifications made by that Contributor either alone and/or in
         combination with its Contributor Version (or portions of such
         combination), to make, use, sell, offer for sale, have made, and/or
         otherwise dispose of: 1) Modifications made by that Contributor (or
         portions thereof); and 2) the combination of Modifications made by that
         Contributor with its Contributor Version (or portions of such
         combination).

         (( c ))??
         the licenses granted in Sections 2.2(a) and 2.2(b) are effective on the
         date Contributor first makes Commercial Use of the Covered Code.

         (( (d) ))??
         Notwithstanding Section 2.2(b) above, no patent license is granted: 1)
         for any code that Contributor has deleted from the Contributor Version;
         2) separate from the Contributor Version; 3) for infringements caused
         by: i) third party modifications of Contributor Version or ii) the
         combination of Modifications made by that Contributor with other
         software (except as part of the Contributor Version) or other devices;
         or 4) under Patent Claims infringed by Covered Code in the absence of
         Modifications made by that Contributor.

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Application of License. The Modifications which You create or to which You
      contribute are governed by the terms of this License, including without
      limitation Section 2.2. The Source Code version of Covered Code may be
      distributed only under the terms of this License or a future version of
      this License released under Section 6.1, and You must include a copy of
      this License with every copy of the Source Code You distribute. You may
      not offer or impose any terms on any Source Code version that alters or
      restricts the applicable version of this License or the recipients' rights
      hereunder. However, You may include an additional document offering the
      additional rights described in Section 3.5.

      (( 3.2. ))??
      Availability of Source Code. Any Modification which You create or to which
      You contribute must be made available in Source Code form under the terms
      of this License either on the same media as an Executable version or via
      an accepted Electronic Distribution Mechanism to anyone to whom you made
      an Executable version available; and if made available via Electronic
      Distribution Mechanism, must remain available for at least twelve (12)
      months after the date it initially became available, or at least six (6)
      months after a subsequent version of that particular Modification has been
      made available to such recipients. You are responsible for ensuring that
      the Source Code version remains available even if the Electronic
      Distribution Mechanism is maintained by a third party.

      (( 3.3. ))??
      Description of Modifications. You must cause all Covered Code to which You
      contribute to contain a file documenting the changes You made to create
      that Covered Code and the date of any change. You must include a prominent
      statement that the Modification is derived, directly or indirectly, from
      Original Code provided by the Initial Developer and including the name of
      the Initial Developer in (a) the Source Code, and (b) in any notice in an
      Executable version or related documentation in which You describe the
      origin or ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters

         (( (a) ))??
         Third Party Claims. If Contributor has knowledge that a license under a
         third party's intellectual property rights is required to exercise the
         rights granted by such Contributor under Sections 2.1 or 2.2,
         Contributor must include a text file with the Source Code distribution
         titled "LEGAL" which describes the claim and the party making the claim
         in sufficient detail that a recipient will know whom to contact. If
         Contributor obtains such knowledge after the Modification is made
         available as described in Section 3.2, Contributor shall promptly
         modify the LEGAL file in all copies Contributor makes available
         thereafter and shall take other steps (such as notifying appropriate
         mailing lists or newsgroups) reasonably calculated to inform those who
         received the Covered Code that new knowledge has been obtained.

         (( (b) ))??
         Contributor APIs. If Contributor's Modifications include an application
         programming interface and Contributor has knowledge of patent licenses
         which are reasonably necessary to implement that API, Contributor must
         also include this information in the LEGAL file.

         (( c ))??
         Representations. Contributor represents that, except as disclosed
         pursuant to Section 3.4(a) above, Contributor believes that
         Contributor's Modifications are Contributor's original creation(s)
         and/or Contributor has sufficient rights to grant the rights conveyed
         by this License.

      (( 3.5. ))??
      Required Notices. You must duplicate the notice in Exhibit A in each file
      of the Source Code. If it is not possible to put such notice in a
      particular Source Code file due to its structure, then You must include
      such notice in a location (such as a relevant directory) where a user
      would be likely to look for such a notice. If You created one or more
      Modification(s) You may add your name as a Contributor to the notice
      described in Exhibit A. You must also duplicate this License in any
      documentation for the Source Code where You describe recipients' rights or
      ownership rights relating to Covered Code. You may choose to offer, and to
      charge a fee for, warranty, support, indemnity or liability obligations to
      one or more recipients of Covered Code. However, You may do so only on
      Your own behalf, and not on behalf of the Initial Developer or any
      Contributor. You must make it absolutely clear than any such warranty,
      support, indemnity or liability obligation is offered by You alone, and
      You hereby agree to indemnify the Initial Developer and every Contributor
      for any liability incurred by the Initial Developer or such Contributor as
      a result of warranty, support, indemnity or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Versions. You may distribute Covered Code in
      Executable form only if the requirements of Section 3.1-3.5 have been met
      for that Covered Code, and if You include a notice stating that the Source
      Code version of the Covered Code is available under the terms of this
      License, including a description of how and where You have fulfilled the
      obligations of Section 3.2. The notice must be conspicuously included in
      any notice in an Executable version, related documentation or collateral
      in which You describe recipients' rights relating to the Covered Code. You
      may distribute the Executable version of Covered Code or ownership rights
      under a license of Your choice, which may contain terms different from
      this License, provided that You are in compliance with the terms of this
      License and that the license for the Executable version does not attempt
      to limit or alter the recipient's rights in the Source Code version from
      the rights set forth in this License. If You distribute the Executable
      version under a different license You must make it absolutely clear that
      any terms which differ from this License are offered by You alone, not by
      the Initial Developer or any Contributor. You hereby agree to indemnify
      the Initial Developer and every Contributor for any liability incurred by
      the Initial Developer or such Contributor as a result of any such terms
      You offer.

      (( 3.7. ))??
      Larger Works. You may create a Larger Work by combining Covered Code with
      other code not governed by the terms of this License and distribute the
      Larger Work as a single product. In such a case, You must make sure the
      requirements of this License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute, judicial
   order, or regulation then You must: (a) comply with the terms of this License
   to the maximum extent possible; and (b) describe the limitations and the code
   they affect. Such description must be included in the LEGAL file described in
   Section 3.4 and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Application of this License.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A and to related Covered Code.

   (( 6. ))??
   Versions of the License.

      (( 6.1. ))??
      New Versions. CUA Office Project may publish revised and/or new versions
      of the License from time to time. Each version will be given a
      distinguishing version number.

      (( 6.2. ))??
      Effect of New Versions. Once Covered Code has been published under a
      particular version of the License, You may always continue to use it under
      the terms of that version. You may also choose to use such Covered Code
      under the terms of any subsequent version of the License published by CUA
      Office Project. No one other than CUA Office Project has the right to
      modify the terms applicable to Covered Code created under this License.

      (( 6.3. ))??
      Derivative Works. If You create or use a modified version of this License
      (which you may only do in order to apply it to code which is not already
      Covered Code governed by this License), You must (a) rename Your license
      so that the phrases "CUA Office", "CUA", "CUAPL", or any confusingly
      similar phrase do not appear in your license (except to note that your
      license differs from this License) and (b) otherwise make it clear that
      Your version of the license contains terms which differ from the CUA
      Office Public License. (Filling in the name of the Initial Developer,
      Original Code or Contributor in the notice described in Exhibit A shall
      not of themselves be deemed to be modifications of this License.)

   (( 7. ))??
   DISCLAIMER OF WARRANTY.

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER
   OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR
   OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF
   THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

   (( 8. ))??
   TERMINATION.

      (( 8.1. ))??
      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. All sublicenses to the
      Covered Code which are properly granted shall survive any termination of
      this License. Provisions which, by their nature, must remain in effect
      beyond the termination of this License shall survive.

      (( 8.2. ))??
      If You initiate litigation by asserting a patent infringement claim
      (excluding declatory judgment actions) against Initial Developer or a
      Contributor (the Initial Developer or Contributor against whom You file
      such action is referred to as "Participant") alleging that:

         (( (a) ))??
         such Participant's Contributor Version directly or indirectly infringes
         any patent, then any and all rights granted by such Participant to You
         under Sections 2.1 and/or 2.2 of this License shall, upon 60 days
         notice from Participant terminate prospectively, unless if within 60
         days after receipt of notice You either: (i) agree in writing to pay
         Participant a mutually agreeable reasonable royalty for Your past and
         future use of Modifications made by such Participant, or (ii) withdraw
         Your litigation claim with respect to the Contributor Version against
         such Participant. If within 60 days of notice, a reasonable royalty and
         payment arrangement are not mutually agreed upon in writing by the
         parties or the litigation claim is not withdrawn, the rights granted by
         Participant to You under Sections 2.1 and/or 2.2 automatically
         terminate at the expiration of the 60 day notice period specified
         above.

         (( (b) ))??
         any software, hardware, or device, other than such Participant's
         Contributor Version, directly or indirectly infringes any patent, then
         any rights granted to You by such Participant under Sections 2.1(b) and
         2.2(b) are revoked effective as of the date You first made, used, sold,
         distributed, or had made, Modifications made by that Participant.

      (( 8.3. ))??
      If You assert a patent infringement claim against Participant alleging
      that such Participant's Contributor Version directly or indirectly
      infringes any patent where such claim is resolved (such as by license or
      settlement) prior to the initiation of patent infringement litigation,
      then the reasonable value of the licenses granted by such Participant
      under Sections 2.1 or 2.2 shall be taken into account in determining the
      amount or value of any payment or license.

      (( 8.4. ))??
      In the event of termination under Sections 8.1 or 8.2 above, all end user
      license agreements (excluding distributors and resellers) which have been
      validly granted by You or any distributor hereunder prior to termination
      shall survive termination.

   (( 9. ))??
   LIMITATION OF LIABILITY.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, THE INITIAL DEVELOPER, ANY
   OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE, OR ANY SUPPLIER OF ANY
   OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. GOVERNMENT END USERS.

   The Covered Code is a "commercial item," as that term is defined in 48 C.F.R.
   2.101 (Oct. 1995), consisting of "commercial computer software" and
   "commercial computer software documentation," as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Covered Code with only those rights set forth herein.

   (( 11. ))??
   MISCELLANEOUS.

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by California law provisions
   (except to the extent applicable law, if any, provides otherwise), excluding
   its conflict-of-law provisions. With respect to disputes in which at least
   one party is a citizen of, or an entity chartered or registered to do
   business in the United States of America, any litigation relating to this
   License shall be subject to the jurisdiction of the Federal Courts of the
   Northern District of California, with venue lying in Santa Clara County,
   California, with the losing party responsible for costs, including without
   limitation, court costs and reasonable attorneys' fees and expenses. The
   application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any law or regulation
   which provides that the language of a contract shall be construed against the
   drafter shall not apply to this License.

   (( 12. ))??
   RESPONSIBILITY FOR CLAIMS.

   As between Initial Developer and the Contributors, each party is responsible
   for claims and damages arising, directly or indirectly, out of its
   utilization of rights under this License and You agree to work with Initial
   Developer and Contributors to distribute such responsibility on an equitable
   basis. Nothing herein is intended or shall be deemed to constitute any
   admission of liability.

   (( 13. ))??
   MULTIPLE-LICENSED CODE.

   Initial Developer may designate portions of the Covered Code as
   "Multiple-Licensed". "Multiple-Licensed" means that the Initial Developer
   permits you to utilize portions of the Covered Code under Your choice of the
   NPL or the alternative licenses, if any, specified by the Initial Developer
   in the file described in Exhibit A.

(( EXHIBIT A - CUA Office Public License.

The contents of this file are subject to the CUA Office Public License Version
1.0 (the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at
http:/cuaoffice.sourceforge.net/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

The Original Code is ______________________________________.

The Initial Developer of the Original Code is ________________________.

Portions created by ______________________
are Copyright (C) _________________.
All Rights Reserved.

Contributor(s): ______________________________________.

Alternatively, the contents of this file may be used under the terms of the
_____  license (the "[___] License"), in which case the provisions of
[______]  License are applicable instead of those above.
If you wish to allow use of your
version of this file only under the terms of the [____] License
and not to allow others to use your version of this file under the
CUAPL, indicate your decision by deleting the provisions above and replace them
with the notice and other provisions required by the [___] License.
If you do not delete the provisions above, a recipient may use your
version of this file under either the CUAPL or the
[___] License.

[NOTE: The text of this Exhibit A may differ slightly from the text of the
notices in the Source Code files of the Original Code. You should use the text
of this Exhibit A rather than the text found in the Original Code Source Code
for Your Modifications.]
))??
`
const license_Caldera_lre = `//**
Caldera License
https://spdx.org/licenses/Caldera.json
http://www.lemis.com/grog/UNIX/ancient-source-all.pdf
**//

Caldera International, Inc. hereby grants a fee free license that includes the
rights use, modify and distribute this named source code, including creating
derived binary products created from the source code. The source code for which
Caldera International, Inc. grants rights are limited to the following UNIX
Operating Systems that operate on the 16-Bit PDP-11 CPU and early versions of
the 32-Bit UNIX Operating System, with specific exclusion of UNIX System III and
UNIX System V and successor operating systems:

32-bit 32V UNIX

16 bit UNIX Versions 1, 2, 3, 4, 5, 6, 7

Caldera International, Inc. makes no guarantees or commitments that any source
code is available from Caldera International, Inc.

The following copyright notice applies to the source code files for which this
license is granted.

Copyright(C) Caldera International Inc. 2001-2002. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code and documentation must retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

All advertising materials mentioning features or use of this software must
display the following acknowledgement:

   This product includes software developed or owned by Caldera International,
   Inc.

Neither the name of Caldera International, Inc. nor the names of other
contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

USE OF THE SOFTWARE PROVIDED FOR UNDER THIS LICENSE BY CALDERA INTERNATIONAL,
INC. AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CALDERA
INTERNATIONAL, INC. BE LIABLE FOR ANY DIRECT, INDIRECT INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
`
const license_ClArtistic_lre = `//**
Clarified Artistic License
https://spdx.org/licenses/ClArtistic.json
http://gianluca.dellavedova.org/2011/01/03/clarified-artistic-license/
http://www.ncftp.com/ncftp/doc/LICENSE.txt
**//

(( The Clarified Artistic License ))??

Preamble

The intent of this document is to state the conditions under which a Package may
be copied, such that the Copyright Holder maintains some semblance of artistic
control over the development of the package, while giving the users of the
package the right to use and distribute the Package in a more-or-less customary
fashion, plus the right to make reasonable modifications.

Definitions:

"Package" refers to the collection of files distributed by the Copyright Holder,
and derivatives of that collection of files created through textual
modification.

"Standard Version" refers to such a Package if it has not been modified, or has
been modified in accordance with the wishes of the Copyright Holder as specified
below.

"Copyright Holder" is whoever is named in the copyright or copyrights for the
package.

"You" is you, if you're thinking about copying or distributing this Package.

"Distribution fee" is a fee you charge for providing a copy of this Package to
another party.

"Freely Available" means that no fee is charged for the right to use the item,
though there may be fees involved in handling the item. It also means that
recipients of the item may redistribute it under the same conditions they
received it.

   (( 1. ))??
   You may make and give away verbatim copies of the source form of the Standard
   Version of this Package without restriction, provided that you duplicate all
   of the original copyright notices and associated disclaimers.

   (( 2. ))??
   You may apply bug fixes, portability fixes and other modifications derived
   from the Public Domain, or those made Freely Available, or from the Copyright
   Holder. A Package modified in such a way shall still be considered the
   Standard Version.

   (( 3. ))??
   You may otherwise modify your copy of this Package in any way, provided that
   you insert a prominent notice in each changed file stating how and when you
   changed that file, and provided that you do at least ONE of the following:

      (( a) ))??
      place your modifications in the Public Domain or otherwise make them
      Freely Available, such as by posting said modifications to Usenet or an
      equivalent medium, or placing the modifications on a major network archive
      site allowing unrestricted access to them, or by allowing the Copyright
      Holder to include your modifications in the Standard Version of the
      Package.

      (( b) ))??
      use the modified Package only within your corporation or organization.

      (( c) ))??
      rename any non-standard executables so the names do not conflict with
      standard executables, which must also be provided, and provide a separate
      manual page for each non-standard executable that clearly documents how it
      differs from the Standard Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

      (( e) ))??
      permit and encourge anyone who receives a copy of the modified Package
      permission to make your modifications Freely Available in some specific
      way.

   (( 4. ))??
   You may distribute the programs of this Package in object code or executable
   form, provided that you do at least ONE of the following:

      (( a) ))??
      distribute a Standard Version of the executables and library files,
      together with instructions (in the manual page or equivalent) on where to
      get the Standard Version.

      (( b) ))??
      accompany the distribution with the machine-readable source of the Package
      with your modifications.

      (( c) ))??
      give non-standard executables non-standard names, and clearly document the
      differences in manual pages (or equivalent), together with instructions on
      where to get the Standard Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

      (( e) ))??
      offer the machine-readable source of the Package, with your modifications,
      by mail order.

   (( 5. ))??
   You may charge a distribution fee for any distribution of this Package. If
   you offer support for this Package, you may charge any fee you choose for
   that support. You may not charge a license fee for the right to use this
   Package itself. You may distribute this Package in aggregate with other
   (possibly commercial and possibly nonfree) programs as part of a larger
   (possibly commercial and possibly nonfree) software distribution, and charge
   license fees for other parts of that software distribution, provided that you
   do not advertise this Package as a product of your own. If the Package
   includes an interpreter, You may embed this Package's interpreter within an
   executable of yours (by linking); this shall be construed as a mere form of
   aggregation, provided that the complete Standard Version of the interpreter
   is so embedded.

   (( 6. ))??
   The scripts and library files supplied as input to or produced as output from
   the programs of this Package do not automatically fall under the copyright of
   this Package, but belong to whoever generated them, and may be sold
   commercially, and may be aggregated with this Package. If such scripts or
   library files are aggregated with this Package via the so-called "undump" or
   "unexec" methods of producing a binary executable image, then distribution of
   such an image shall neither be construed as a distribution of this Package
   nor shall it fall under the restrictions of Paragraphs 3 and 4, provided that
   you do not represent such an executable image as a Standard Version of this
   Package.

   (( 7. ))??
   C subroutines (or comparably compiled subroutines in other languages)
   supplied by you and linked into this Package in order to emulate subroutines
   and variables of the language defined by this Package shall not be considered
   part of this Package, but are the equivalent of input as in Paragraph 6,
   provided these subroutines do not change the language in any way that would
   cause it to fail the regression tests for the language.

   (( 8. ))??
   Aggregation of the Standard Version of the Package with a commercial
   distribution is always permitted provided that the use of this Package is
   embedded; that is, when no overt attempt is made to make this Package's
   interfaces visible to the end user of the commercial distribution. Such use
   shall not be construed as a distribution of this Package.

   (( 9. ))??
   The name of the Copyright Holder may not be used to endorse or promote
   products derived from this software without specific prior written
   permission.

   (( 10. ))??
   THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
   (( The End ))??
`
const license_CommonsClause_lre = `//**
CommonsClause addendum
**//

The Software is provided to you by the Licensor under the License, as defined
below, subject to the following condition. Without limiting other conditions in
the License, the grant of rights under the License will not include, and the
License does not grant to you, the right to Sell the Software. For purposes of
the foregoing, “Sell” means practicing any or all of the rights granted to you
under the License to provide to third parties, for a fee or other consideration,
a product or service that consists, entirely or substantially, of the Software
or the functionality of the Software. Any license notice or attribution required
by the License must also include this Commons Cause License Condition notice.
`
const license_Condor_1_1_lre = `//**
Condor Public License v1.1
https://spdx.org/licenses/Condor-1.1.json
http://research.cs.wisc.edu/condor/license.html#condor
http://web.archive.org/web/20111123062036/http://research.cs.wisc.edu/condor/license.html#condor
**//

(( Condor Public License

Version 1.1, October 30, 2003

((Copyright __50__))??
//**
Copyright © 1990-2006 Condor Team, Computer
Sciences Department, University of Wisconsin-Madison, Madison, WI. All Rights
Reserved. For more information contact: Condor Team, Attention: Professor
Miron Livny, Dept of Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
(608) 262-0856 or miron@cs.wisc.edu.
**//
))??


This software referred to as the Condor® Version 6.x software ("Software") was
developed by the Condor Project, Condor Team, Computer Sciences Department,
University of Wisconsin-Madison, under the authority of the Board of Regents of
the University of Wisconsin System and includes voluntary contributions made to
the Condor Project ("Copyright Holders and Contributors and the University").
For more information on the Condor Project, please see
http:/www.condorproject.org/.

Installation, use, reproduction, display, modification and redistribution of
this Software, with or without modification, in source and binary forms, are
permitted. Any exercise of rights under this license including sublicenses by
you is subject to the following conditions:

   (( 1. ))??
   Redistributions of this Software, with or without modification, must
   reproduce this Condor Public License in: (1) the Software, and (2) any user
   documentation or other similar material which is provided with the Software.

   (( 2. ))??
   Any user documentation included with a redistribution must include the
   following notice:

   "This product includes software from the Condor® Project
   (http:/www.condorproject.org/)"

   Alternatively, if that is where third-party acknowledgments normally appear,
   this acknowledgment must be reproduced in the Software itself.

   (( 3. ))??
   Any academic report, publication, or other academic disclosure of results
   obtained with this Software will acknowledge this Software's use by an
   appropriate citation.

   (( 4. ))??
   The name Condor® is a registered trademark of the University of
   Wisconsin-Madison. The trademark may not be used to endorse or promote
   software, or products derived therefrom, and, other than as required by
   section 2 and 3 above, it may not be affixed to modified redistributions of
   this Software without the prior written approval, obtainable via email to
   condor-admin@cs.wisc.edu.

   (( 5. ))??
   To the extent that patent claims licensable by the University of
   Wisconsin-Madison are necessarily infringed by the use or sale of the
   Software, you are granted a non-exclusive, worldwide, royalty-
   __5__
   free perpetual license under such patent claims, with the rights for you to
   make, use, sell, offer to sell, import and otherwise transfer the Software in
   source code and object code form and derivative works. This patent license
   shall apply to the combination of the Software with other software if, at the
   time the Software is added by you, such addition of the Software causes such
   combination to be covered by such patent claims. This patent license shall
   not apply to any other combinations which include the Software. No hardware
   per se is licensed hereunder.If you or any subsequent sub-licensee (a
   ` + "`" + `` + "`" + `Recipient") institutes patent litigation against any entity (including a
   cross-claim or counterclaim in a lawsuit) alleging that the Software
   infringes such Recipient's patent(s), then such Recipient's rights granted
   (directly or indirectly) under the patent license above shall terminate as of
   the date such litigation is filed. All sublicenses to the Software which have
   been properly granted prior to termination shall survive any termination of
   said patent license, if not otherwise terminated pursuant to this section.

   (( 6. ))??
   DISCLAIMER

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AND THE
   UNIVERSITY "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, OF SATISFACTORY
   QUALITY, AND FITNESS FOR A PARTICULAR PURPOSE OR USE ARE DISCLAIMED. THE
   COPYRIGHT HOLDERS AND CONTRIBUTORS AND THE UNIVERSITY MAKE NO REPRESENTATION
   THAT THE SOFTWARE, MODIFICATIONS, ENHANCEMENTS OR DERIVATIVE WORKS THEREOF,
   WILL NOT INFRINGE ANY PATENT, COPYRIGHT, TRADEMARK, TRADE SECRET OR OTHER
   PROPRIETARY RIGHT.

   (( 7. ))??
   LIMITATION OF LIABILITY

   THE COPYRIGHT HOLDERS AND CONTRIBUTORS AND ANY OTHER OFFICER, AGENT, OR
   EMPLOYEE OF THE UNIVERSITY SHALL HAVE NO LIABILITY TO LICENSEE OR OTHER
   PERSONS FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, CONSEQUENTIAL, EXEMPLARY,
   OR PUNITIVE DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT LIMITATION,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES, LOSS OF USE, DATA OR PROFITS, OR
   BUSINESS INTERRUPTION, HOWEVER CAUSED AND ON ANY THEORY OF CONTRACT,
   WARRANTY, TORT (INCLUDING NEGLIGENCE), PRODUCT LIABILITY OR OTHERWISE,
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGES.

   (( 8. ))??
   Certain uses and transfers of the Software or documentation, and/or items or
   software incorporating the Condor Software or documentation, may require a
   license under U.S. Export Control laws. Licensee represents and warrants that
   all uses and transfers of the Condor Software or documentation and/or any
   items or software incorporating Condor shall be in compliance with U.S.
   Export Control laws, and Licensee further understands that failure to comply
   with such export control laws may result in criminal liability to Licensee
   under U.S. laws.

   (( 9. ))??
   The Condor Team may publish revised and/or new versions of this Condor Public
   License (` + "`" + `` + "`" + `this License") from time to time. Each version will be given a
   distinguishing version number. Once Software has been published under a
   particular version of this License, you may always continue to use it under
   the terms of that version. You may also choose to use such Software under the
   terms of any subsequent version of this License published by the Condor Team.
   No one other than the Condor Team has the right to modify the terms of this
   License.
   (( For more information:

Condor Team

Attention: Professor Miron Livny

7367 Computer Sciences

1210 W. Dayton St.

Madison, WI 53706-1685

miron@cs.wisc.edu

http:/pages.cs.wisc.edu/~miron/miron.html ))??
`
const license_Crossword_lre = `//**
Crossword License
https://spdx.org/licenses/Crossword.json
https://fedoraproject.org/wiki/Licensing/Crossword
**//

(( Copyright (C) 1995-2009 Gerd Neugebauer ))??

cwpuzzle.dtx is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY. No author or distributor accepts responsibility to anyone for the
consequences of using it or for whether it serves any particular purpose or
works at all, unless he says so in writing.

Everyone is granted permission to copy, modify and redistribute cwpuzzle.dtx,
provided this copyright notice is preserved and any modifications are indicated.
`
const license_CrystalStacker_lre = `//**
CrystalStacker License
https://spdx.org/licenses/CrystalStacker.json
https://fedoraproject.org/wiki/Licensing:CrystalStacker?rd=Licensing/CrystalStacker
**//

Crystal Stacker is freeware. This means you can pass copies around freely
provided you include this document in it's original form in your distribution.
Please see the "Contacting Us" section of this document if you need to contact
us for any reason.

Disclaimer

NewCreature Design makes no guarantees regarding the Crystal Stacker software.
We are not responsible for damages caused by it, though the software is not
known to cause any problems. If you have trouble with the software, see the
"Contacting Us" section of this document.

The source code is provided as-is and you may do with it whatsoever you please
provided that you include this file in its unmodified form with any new
distribution. NewCreature Design makes no gaurantees regarding the usability of
the source but are willing to help with any problems you might run into. Please
see the "Contacting Us" section of this document if you need to get in touch
with us about any issues you have regarding the source.
`
const license_Cube_lre = `//**
Cube License
https://spdx.org/licenses/Cube.json
https://fedoraproject.org/wiki/Licensing/Cube
**//

(( Cube game engine source code, 20 dec 2003 release.
((Copyright __20__))??
))??

This software is provided 'as-is', without any express or implied warranty. In
no event will the authors be held liable for any damages arising from the use of
this software.

Permission is granted to anyone to use this software for any purpose, including
commercial applications, and to alter it and redistribute it freely, subject to
the following restrictions:

   (( 1. ))??
   The origin of this software must not be misrepresented; you must not claim
   that you wrote the original software. If you use this software in a product,
   an acknowledgment in the product documentation would be appreciated but is
   not required.

   (( 2. ))??
   Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   (( 3. ))??
   This notice may not be removed or altered from any source distribution.

additional clause specific to Cube:

   (( 4. ))??
   Source versions may not be "relicensed" under a different license without my
   explicitly written permission.
`
const license_D_FSL_1_0_lre = `//**
Deutsche Freie Software Lizenz
https://spdx.org/licenses/D-FSL-1.0.json
http://www.dipp.nrw.de/d-fsl/lizenzen/
http://www.dipp.nrw.de/d-fsl/index_html/lizenzen/de/D-FSL-1_0_de.txt
http://www.dipp.nrw.de/d-fsl/index_html/lizenzen/en/D-FSL-1_0_en.txt
https://www.hbz-nrw.de/produkte/open-access/lizenzen/dfsl
https://www.hbz-nrw.de/produkte/open-access/lizenzen/dfsl/deutsche-freie-software-lizenz
https://www.hbz-nrw.de/produkte/open-access/lizenzen/dfsl/german-free-software-license
https://www.hbz-nrw.de/produkte/open-access/lizenzen/dfsl/D-FSL-1_0_de.txt/at_download/file
https://www.hbz-nrw.de/produkte/open-access/lizenzen/dfsl/D-FSL-1_0_en.txt/at_download/file
**//

(( Deutsche Freie Software Lizenz ))??

(c) Ministerium für Wissenschaft und Forschung Nordrhein-Westfalen 2004
(( Erstellt von Axel Metzger und Till Jaeger, Institut für Rechtsfragen der
Freien und Open Source Software -
//** ( **//
__5__
http:/www.ifross.de
//** ) **//
__5__

Präambel

Software ist mehr als ein Wirtschaftsgut. Sie ist die technische Grundlage der
Informationsgesellschaft. Die Frage der Teilhabe der Allgemeinheit ist deswegen
von besonderer Bedeutung. Herkömmlich lizenzierte Programme werden nur im
Object Code vertrieben, der Nutzer darf das Programm weder verändern noch
weitergeben. Das Lizenzmodell der Freien Software (synonym "Open Source
Software") gewährt Ihnen dagegen umfassende Freiheiten im Umgang mit dem
Programm. Die Deutsche Freie Software Lizenz folgt diesem Lizenzmodell. Sie
gewährt Ihnen das Recht, das Programm in umfassender Weise zu nutzen. Es ist
Ihnen gestattet, das Programm nach Ihren Vorstellungen zu verändern, in
veränderter oder unveränderter Form zu vervielfältigen, zu verbreiten und
öffentlich zugänglich zu machen. Diese Rechte werden unentgeltlich
eingeräumt.

Die Deutsche Freie Software Lizenz verbindet die Rechtseinräumung allerdings
mit Pflichten, die dem Zweck dienen, das freie Zirkulieren des Programms und
aller veröffentlichten Fortentwicklungen zu sichern. Wenn Sie das Programm
verbreiten oder öffentlich zugänglich machen, dann müssen Sie jedem, der das
Programm von Ihnen erhält, eine Kopie dieser Lizenz mitliefern und den Zugriff
auf den Source Code ermöglichen. Eine weitere Pflicht betrifft
Fortentwicklungen des Programms. Änderungen am Programm, die Sie öffentlich
verbreiten oder zugänglich machen, müssen nach den Bestimmungen dieser Lizenz
frei gegeben werden.

Die Deutsche Freie Software Lizenz nimmt auf die besonderen Anforderungen des
deutschen und europäischen Rechts Rücksicht. Sie ist zweisprachig gestaltet
und damit auch auf den internationalen Vertrieb ausgerichtet. ))??

   § 0 Definitionen

      Dokumentation: Die Beschreibung des Aufbaus und/oder der Struktur der
      Programmierung und/oder der Funktionalitäten des Programms, unabhängig
      davon, ob sie im Source Code oder gesondert vorgenommen wird.

      Lizenz: Die zwischen dem Lizenzgeber und Ihnen geschlossene Vereinbarung
      mit dem Inhalt der "Deutschen Freien Software Lizenz" bzw. das Angebot
      hierzu.

      Lizenznehmer: Jede natürliche oder juristische Person, die die Lizenz
      angenommen hat.

      Programm: Jedes Computerprogramm, das von den Rechtsinhabern nach den
      Bestimmungen dieser Lizenz verbreitet oder öffentlich zugänglich gemacht
      worden ist.

      Object Code: Die maschinenlesbare, übersetzte Form des Programms.

      Öffentlich: Nicht nur an einen bestimmten Personenkreis gerichtet, der
      persönlich oder durch die Zugehörigkeit zu einer juristischen Person
      oder einem öffentlichen Träger miteinander verbunden ist.

      Öffentlich zugänglich machen: Die öffentliche Weitergabe des Programms
      in unkörperlicher Form, insbesondere das Bereithalten zum Download in
      Datennetzen.

      Rechtsinhaber: Der bzw. die Urheber oder sonstigen Inhaber der
      ausschließlichen Nutzungsrechte an dem Programm.

      Source Code: Die für Menschen lesbare, in Programmiersprache dargestellte
      Form des Programms.

      Verändern: Jede Erweiterung, Kürzung und Bearbeitung des Programms,
      insbesondere Weiterentwicklungen.

      Verbreiten: Die öffentliche Weitergabe körperlicher
      Vervielfältigungsstücke, insbesondere auf Datenträgern oder in
      Verbindung mit Hardware.

      Vollständiger Source Code: Der Source Code in der für die Erstellung
      bzw. die Bearbeitung benutzten Form zusammen mit den zur Übersetzung und
      Installation erforderlichen Konfigurationsdateien und Software-Werkzeugen,
      sofern diese in der benötigten Form nicht allgemein gebräuchlich (z.B.
      Standard-Kompiler) oder für jedermann lizenzgebührenfrei im Internet
      abrufbar sind.

   § 1 Rechte

      (( (1) ))??
      Sie dürfen das Programm in unveränderter Form vervielfältigen,
      verbreiten und öffentlich zugänglich machen.

      (( (2) ))??
      Sie dürfen das Programm verändern und entsprechend veränderte Versionen
      vervielfältigen, verbreiten und öffentlich zugänglich machen. Gestattet
      ist auch die Kombination des Programms oder Teilen hiervon mit anderen
      Programmen.

      (( (3) ))??
      Sie erhalten die Rechte unentgeltlich.

   § 2 Pflichten beim Vertrieb

      (( (1) ))??
      Wenn Sie das Programm verbreiten oder öffentlich zugänglich machen, sei
      es in unveränderter oder veränderter Form, sei es in einer Kombination
      mit anderen Programmen oder in Verbindung mit Hardware, dann müssen sie
      mitliefern:

      (( 1. ))??
      alle Vermerke im Source Code und/oder Object Code, die auf diese Lizenz
      hinweisen;

      (( 2. ))??
      alle Vermerke im Source Code und/oder Object Code, die über die Urheber
      des Programms Auskunft geben;

      (( 3. ))??
      einen für den Empfänger deutlich wahrnehmbaren Hinweis auf diese Lizenz
      und die Internetadressehttp:/www.d-fsl.de;

      (( 4. ))??
      den vollständigen Text dieser Lizenz in deutlich wahrnehmbarer Weise.

      (( (2) ))??
      Wenn bei der Installation des Programms und/oder beim Programmstart
      Lizenz- und/oder Vertragsbedingungen angezeigt werden, dann müssen

      (( 1. ))??
      diese Lizenz,

      (( 2. ))??
      ein Hinweis auf diese Lizenz und

      (( 3. ))??
      ein Hinweis auf den oder die Rechtsinhaber an den ersten unter dieser
      Lizenz nutzbaren Programmbestandteilen

      ebenfalls angezeigt werden.

      (( (3) ))??
      Sie dürfen die Nutzung des Programms nicht von Pflichten oder Bedingungen
      abhängig machen, die nicht in dieser Lizenz vorgesehen sind.

      (( (4) ))??
      Sofern Sie mit dem Programm eine Dokumentation erhalten haben, muss diese
      Dokumentation entsprechend mitgeliefert werden, es sei denn, die freie
      Mitlieferung der Dokumentation ist Ihnen aufgrund der Lizenz für die
      Dokumentation nicht gestattet.

   § 3 Weitere Pflichten beim Vertrieb veränderter Versionen

      (( (1) ))??
      Veränderte Versionen des Programms dürfen Sie nur unter den Bedingungen
      dieser Lizenz verbreiten oder öffentlich zugänglich machen, so dass
      Dritte das veränderte Programm insgesamt unter dieser Lizenz nutzen
      können.

      (( (2) ))??
      Wird das Programm oder ein Teil hiervon mit einem anderen Programm
      kombiniert, gilt auch die Kombination insgesamt als eine veränderte
      Version des Programms, es sei denn, das andere Programm ist formal und
      inhaltlich eigenständig. Ein anderes Programm ist dann als eigenständig
      anzusehen, wenn es die folgenden Voraussetzungen alle erfüllt:

         (( 1. ))??
         Der Source Code der kombinierten Programme muss jeweils in eigenen
         Dateien vorhanden sein, die keine Bestandteile des anderen Teils
         enthalten, die über die zur Programmkombination üblichen und
         erforderlichen Informationen über den anderen Teil hinausgehen, wobei
         der Source Code des anderen Programms nicht mitgeliefert werden muss.

         (( 2. ))??
         Der mit dem Programm kombinierte Teil muss auch dann sinnvoll nutzbar
         sein, wenn er nicht mit dem Programm kombiniert wird, und zwar entweder
         alleine oder mit sonstigen Programmen. Was als "sinnvoll nutzbar"
         anzusehen ist, richtet sich nach der Auffassung der betroffenen
         Fachkreise. Zu den betroffenen Fachkreisen gehören alle Personen, die
         das Programm oder Programme mit vergleichbarer Funktionalität
         entwickeln, benutzen, verbreiten oder öffentlich zugänglich machen.

      (( (3) ))??
      Wenn Sie das Programm oder einen Teil hiervon - verändert oder
      unverändert - zusammen mit einem anderen Programm verbreiten oder
      öffentlich zugänglich machen, das unter der GNU General Public License
      (GPL) lizenziert wird, darf das Programm auch unter den Bedingungen der
      GPL genutzt werden, sofern es mit dem anderen Programm ein "derivative
      work" im Sinne der GPL bildet. Dabei sollen die Hinweise auf diese Lizenz
      entfernt und durch einen Hinweis auf die GPL ersetzt werden. Ob bei der
      Zusammenstellung ein "derivate work" im Sinne der GPL entsteht, beurteilt
      sich nach Ziffer 2 b) der GPL. Diese Bestimmung lautet: "You must cause
      any work that you distribute or publish, that in whole or in part contains
      or is derived from the Program or any part thereof, to be licensed as a
      whole at no charge to all third parties under the terms of this License."
      Die GPL kann abgerufen werden unter http:/www.fsf.org/licenses/gpl.

      (( (4) ))??
      Wenn Sie das Programm in einer veränderten Form verbreiten oder
      öffentlich zugänglich machen, müssen Sie im Source Code einen Hinweis
      mit den Änderungen aufnehmen und mit dem Datum der Änderung versehen.
      Der Hinweis muss erkennen lassen, welche Änderungen vorgenommen wurden
      und bestehende Vermerke, die über die Urheber des Programms Auskunft
      geben, übernehmen. Dies gilt unabhängig davon, ob Sie einen eigenen
      Urhebervermerk hinzufügen. Anstelle eines Hinweises im Source Code
      können Sie auch ein Versionskontrollsystem verwenden oder weiterführen,
      sofern dieses mitverbreitet wird oder öffentlich zugänglich ist.

      (( (5) ))??
      Sie dürfen von Dritten für die Einräumung eines einfachen
      Nutzungsrechts an veränderten Versionen des Programms kein Entgelt
      verlangen.

      (( (6) ))??
      Wenn Sie an der veränderten Version des Programms ein anderes Schutzrecht
      als ein Urheberrecht erwerben, insbesondere ein Patent oder
      Gebrauchsmuster, lizenzieren Sie dieses Schutzrecht für veränderte und
      unveränderte Versionen des Programms in dem Umfang, der erforderlich ist,
      um die Rechte aus dieser Lizenz wahrnehmen zu können.

   § 4 Weitere Pflichten beim Vertrieb im Object Code

      (( (1) ))??
      Wenn Sie das Programm nur im Object Code verbreiten, dann müssen Sie
      zusätzlich zu den in § 2 und § 3 geregelten Pflichten entweder

         (( 1. ))??
         den vollständigen Source Code im Internet öffentlich zugänglich
         machen und bei der Verbreitung des Object Codes deutlich auf die
         vollständige Internetadresse hinweisen, unter der der Source Code
         abgerufen werden kann oder

         (( 2. ))??
         den vollständigen Source Code auf einem hierfür üblichen
         Datenträger unter Beachtung der §§ 2 und 3 mitverbreiten.

      (( (2) ))??
      Wenn Sie das Programm im Object Code öffentlich zugänglich machen, dann
      müssen Sie zusätzlich zu den in § 2 und § 3 geregelten Pflichten den
      vollständigen Source Code im Internet öffentlich zugänglich machen und
      dabei deutlich auf die vollständige Internetadresse hinweisen.

      (( (3) ))??
      Sofern Sie mit dem Programm eine Dokumentation erhalten haben, muss diese
      Dokumentation entsprechend der Absätze 1 und 2 mitgeliefert werden, es
      sei denn, die freie Mitlieferung der Dokumentation ist Ihnen aufgrund der
      Lizenz für die Dokumentation nicht gestattet.

   § 5 Vertragsschluss

      (( (1) ))??
      Mit dieser Lizenz wird Ihnen und jeder anderen Person ein Angebot auf
      Abschluss eines Vertrages über die Nutzung des Programms unter den
      Bedingungen der Deutschen Freien Softwarelizenz unterbreitet.

      (( (2) ))??
      Sie dürfen das Programm nach den jeweils anwendbaren gesetzlichen
      Vorschriften bestimmungsgemäß benutzen, ohne dass es der Annahme dieser
      Lizenz bedarf. Dieses Recht umfasst in der Europäischen Union und in den
      meisten anderen Rechtsordnungen insbesondere die folgenden Befugnisse:

         (( 1. ))??
         das Programm ablaufen zu lassen sowie die Erstellung von hierfür
         erforderlichen Vervielfältigungen im Haupt- und Arbeitsspeicher;

         (( 2. ))??
         das Erstellen einer Sicherungskopie;

         (( 3. ))??
         die Fehlerberichtigung;

         (( 4. ))??
         die Weitergabe einer rechtmäßig erworbenen körperlichen Kopie des
         Programms.

      (( (3) ))??
      Sie erklären Ihre Zustimmung zum Abschluss dieser Lizenz, indem Sie das
      Programm verbreiten, öffentlich zugänglich machen, verändern oder in
      einer Weise vervielfältigen, die über die bestimmungsgemäße Nutzung im
      Sinne von Absatz 2 hinausgeht. Ab diesem Zeitpunkt ist diese Lizenz als
      rechtlich verbindlicher Vertrag zwischen den Rechtsinhabern und Ihnen
      geschlossen, ohne dass es eines Zugangs der Annahmeerklärung bei den
      Rechtsinhabern bedarf.

      (( (4) ))??
      Sie und jeder andere Lizenznehmer erhalten die Rechte aus dieser Lizenz
      direkt von den Rechtsinhabern. Eine Unterlizenzierung oder Übertragung
      der Rechte ist nicht gestattet.

   § 6 Beendigung der Rechte bei Zuwiderhandlung

      (( (1) ))??
      Jede Verletzung Ihrer Verpflichtungen aus dieser Lizenz führt zu einer
      automatischen Beendigung Ihrer Rechte aus dieser Lizenz.

      (( (2) ))??
      Die Rechte Dritter, die das Programm oder Rechte an dem Programm von Ihnen
      erhalten haben, bleiben hiervon unberührt.

   § 7 Haftung und Gewährleistung

      (( (1) ))??
      Für entgegenstehende Rechte Dritter haften die Rechtsinhaber nur, sofern
      sie Kenntnis von diesen Rechten hatten, ohne Sie zu informieren.

      (( (2) ))??
      Die Haftung für Fehler und sonstige Mängel des Programms richtet sich
      nach den außerhalb dieser Lizenz getroffenen Vereinbarungen zwischen
      Ihnen und den Rechtsinhabern oder, wenn eine solche Vereinbarung nicht
      existiert, nach den gesetzlichen Regelungen.

   § 8 Verträge mit Dritten

      (( (1) ))??
      Diese Lizenz regelt nur die Beziehung zwischen Ihnen und den
      Rechtsinhabern. Sie ist nicht Bestandteil der Verträge zwischen Ihnen und
      Dritten.

      (( (2) ))??
      Die Lizenz beschränkt Sie nicht in der Freiheit, mit Dritten, die von
      Ihnen Kopien des Programms erhalten oder Leistungen in Anspruch nehmen,
      die im Zusammenhang mit dem Programm stehen, Verträge beliebigen Inhalts
      zu schließen, sofern Sie dabei Ihren Verpflichtungen aus dieser Lizenz
      nachkommen und die Rechte der Dritten aus dieser Lizenz nicht
      beeinträchtigt werden. Insbesondere dürfen Sie für die Überlassung des
      Programms oder sonstige Leistungen ein Entgelt verlangen.

      (( (3) ))??
      Diese Lizenz verpflichtet Sie nicht, das Programm an Dritte weiterzugeben.
      Es steht Ihnen frei zu entscheiden, wem Sie das Programm zugänglich
      machen. Sie dürfen aber die weitere Nutzung durch Dritte nicht durch den
      Einsatz technischer Schutzmaßnahmen, insbesondere durch den Einsatz von
      Kopierschutzvorrichtungen jeglicher Art, verhindern oder erschweren. Eine
      passwortgeschützte Zugangsbeschränkung oder die Nutzung in einem
      Intranet wird nicht als technische Schutzmaßnahme angesehen.

   § 9 Text der Lizenz

      (( (1) ))??
      Diese Lizenz ist in deutscher und englischer Sprache abgefasst. Beide
      Fassungen sind gleich verbindlich. Es wird unterstellt, dass die in der
      Lizenz verwandten Begriffe in beiden Fassungen dieselbe Bedeutung haben.
      Ergeben sich dennoch Unterschiede, so ist die Bedeutung maßgeblich,
      welche die Fassungen unter Berücksichtigung des Ziels und Zwecks der
      Lizenz am besten miteinander in Einklang bringt.

      (( (2) ))??
      Der Lizenzrat der Deutschen Freien Software Lizenz kann mit verbindlicher
      Wirkung neue Versionen der Lizenz in Kraft setzen, soweit dies
      erforderlich und zumutbar ist. Neue Versionen der Lizenz werden auf der
      Internetseite http:/www.d-fsl.de mit einer eindeutigen Versionsnummer
      veröffentlicht. Die neue Version der Lizenz erlangt für Sie verbindliche
      Wirkung, wenn Sie von deren Veröffentlichung Kenntnis genommen haben.
      Gesetzliche Rechtsbehelfe gegen die Änderung der Lizenz werden durch die
      vorstehenden Bestimmungen nicht beschränkt.

      (( (3) ))??
      Sie dürfen diese Lizenz in unveränderter Form vervielfältigen,
      verbreiten und öffentlich zugänglich machen.

   § 10 Anwendbares Recht

   Auf diese Lizenz findet deutsches Recht Anwendung.

   Anhang: Wie unterstellen Sie ein Programm der Deutschen Freien Software
   Lizenz?

   Um jedermann den Abschluss dieser Lizenz zu ermöglichen, wird empfohlen, das
   Programm mit folgendem Hinweis auf die Lizenz zu versehen:

   "Copyright (C) 20[jj] [Name des Rechtsinhabers].

   Dieses Programm kann durch jedermann gemäß den Bestimmungen der Deutschen
   Freien Software Lizenz genutzt werden.

   Die Lizenz kann unter http:/www.d-fsl.de abgerufen werden."
`
const license_DOC_lre = `//**
DOC License
https://spdx.org/licenses/DOC.json
http://www.cs.wustl.edu/~schmidt/ACE-copying.html
**//

(( Copyright and Licensing Information for
ACE(TM), TAO(TM), CIAO(TM),
DAnCE(TM), and CoSMIC(TM) ))??

ACE(TM), TAO(TM), CIAO(TM), DAnCE>(TM), and CoSMIC(TM) (henceforth referred to
as "DOC software") are copyrighted by Douglas C. Schmidt and his research group
at Washington University, University of California, Irvine, and Vanderbilt
University,
__20__.

Since DOC software is open-source, freely available software, you are free to
use, modify, copy, and distribute--perpetually and irrevocably--the DOC software
source code and object code produced from the source, as well as copy and
distribute modified versions of this software. You must, however, include this
copyright statement along with any code built using DOC software that you
release. No copyright statement needs to be provided if you just ship binary
executables of your software products.

You can use DOC software in commercial and/or binary software releases and are
under no obligation to redistribute any of your source code that is built using
DOC software. Note, however, that you may not misappropriate the DOC software
code, such as copyrighting it yourself or claiming authorship of the DOC
software code, in a way that will prevent DOC software from being distributed
freely using an open-source development model. You needn't inform anyone that
you're using DOC software in your software, though we encourage you to let us
know so we can promote your project in the DOC software success stories.

The ACE, TAO, CIAO, DAnCE, and CoSMIC web sites are maintained by the DOC Group
at the Institute for Software Integrated Systems (ISIS) and the Center for
Distributed Object Computing of Washington University, St. Louis for the
development of open-source software as part of the open-source software
community. Submissions are provided by the submitter ` + "`" + `` + "`" + `as is'' with no
warranties whatsoever, including any warranty of merchantability,
noninfringement of third party intellectual property, or fitness for any
particular purpose. In no event shall the submitter be liable for any direct,
indirect, special, exemplary, punitive, or consequential damages, including
without limitation, lost profits, even if advised of the possibility of such
damages. Likewise, DOC software is provided as is with no warranties of any
kind, including the warranties of design, merchantability, and fitness for a
particular purpose, noninfringement, or arising from a course of dealing, usage
or trade practice. Washington University, UC Irvine, Vanderbilt University,
their employees, and students shall have no liability with respect to the
infringement of copyrights, trade secrets or any patents by DOC software or any
part thereof. Moreover, in no event will Washington University, UC Irvine, or
Vanderbilt University, their employees, or students be liable for any lost
revenue or profits or other special, indirect and consequential damages.

DOC software is provided with no support and without any obligation on the part
of Washington University, UC Irvine, Vanderbilt University, their employees, or
students to assist in its use, correction, modification, or enhancement. A
number of companies around the world provide commercial support for DOC
software, however. DOC software is Y2K-compliant, as long as the underlying OS
platform is Y2K-compliant. Likewise, DOC software is compliant with the new US
daylight savings rule passed by Congress as "The Energy Policy Act of 2005,"
which established new daylight savings times (DST) rules for the United States
that expand DST as of March 2007. Since DOC software obtains time/date and
calendaring information from operating systems users will not be affected by the
new DST rules as long as they upgrade their operating systems accordingly.

The names ACE(TM), TAO(TM), CIAO(TM), DAnCE(TM), CoSMIC(TM), Washington
University, UC Irvine, and Vanderbilt University, may not be used to endorse or
promote products or services derived from this source without express written
permission from Washington University, UC Irvine, or Vanderbilt University. This
license grants no permission to call products or services derived from this
source ACE(TM), TAO(TM), CIAO(TM), DAnCE(TM), or CoSMIC(TM), nor does it grant
permission for the name Washington University, UC Irvine, or Vanderbilt
University to appear in their names.

If you have any suggestions, additions, comments, or questions, please let me
know.

Douglas C. Schmidt
`
const license_DSDP_lre = `//**
DSDP License
https://spdx.org/licenses/DSDP.json
https://fedoraproject.org/wiki/Licensing/DSDP
**//

(( COPYRIGHT NOTIFICATION
((Copyright __20__))??
))??


This program discloses material protectable under copyright laws of the United
States. Permission to copy and modify this software and its documentation is
hereby granted, provided that this notice is retained thereon and on all copies
or modifications. The University of Chicago makes no representations as to the
suitability and operability of this software for any purpose. It is provided "as
is"; without express or implied warranty. Permission is hereby granted to use,
reproduce, prepare derivative works, and to redistribute to others, so long as
this original copyright notice is retained. Any publication resulting from
research that made use of this software should cite this document.

This software was authored by:

Steven J. Benson Mathematics and Computer Science Division Argonne National
Laboratory Argonne IL 60439

Yinyu Ye Department of Management Science and Engineering Stanford University
Stanford, CA U.S.A

Any questions or comments on the software may be directed to benson@mcs.anl.gov
or yinyu-ye@stanford.edu

Argonne National Laboratory with facilities in the states of Illinois and Idaho,
is owned by The United States Government, and operated by the University of
Chicago under provision of a contract with the Department of Energy.

DISCLAIMER

THIS PROGRAM WAS PREPARED AS AN ACCOUNT OF WORK SPONSORED BY AN AGENCY OF THE
UNITED STATES GOVERNMENT. NEITHER THE UNITED STATES GOVERNMENT NOR ANY AGENCY
THEREOF, NOR THE UNIVERSITY OF CHICAGO, NOR ANY OF THEIR EMPLOYEES OR OFFICERS,
MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR ASSUMES ANY LEGAL LIABILITY OR
RESPONSIBILITY FOR THE ACCURACY, COMPLETENESS, OR USEFULNESS OF ANY INFORMATION,
APPARATUS, PRODUCT, OR PROCESS DISCLOSED, OR REPRESENTS THAT ITS USE WOULD NOT
INFRINGE PRIVATELY OWNED RIGHTS. REFERENCE HEREIN TO ANY SPECIFIC COMMERCIAL
PRODUCT, PROCESS, OR SERVICE BY TRADE NAME, TRADEMARK, MANUFACTURER, OR
OTHERWISE, DOES NOT NECESSARILY CONSTITUTE OR IMPLY ITS ENDORSEMENT,
RECOMMENDATION, OR FAVORING BY THE UNITED STATES GOVERNMENT OR ANY AGENCY
THEREOF. THE VIEW AND OPINIONS OF AUTHORS EXPRESSED HEREIN DO NOT NECESSARILY
STATE OR REFLECT THOSE OF THE UNITED STATES GOVERNMENT OR ANY AGENCY THEREOF.
`
const license_Dotseqn_lre = `//**
Dotseqn License
https://spdx.org/licenses/Dotseqn.json
https://fedoraproject.org/wiki/Licensing/Dotseqn
**//

//** Copyright **//

This file may be freely transmitted and reproduced, but it may not be changed
unless the name is changed also (except that you may freely change the
paper-size option for \documentclass).

This notice must be left intact.
`
const license_ECL_1_0_lre = `//**
Educational Community License v1.0
https://spdx.org/licenses/ECL-1.0.json
https://opensource.org/licenses/ECL-1.0
**//

(( The Educational Community License ))??

This Educational Community License (the "License") applies to any original work
of authorship (the "Original Work") whose owner (the "Licensor") has placed the
following notice immediately following the copyright notice for the Original
Work:

Copyright __20__

Licensed under the Educational Community License version 1.0

This Original Work, including software, source code, documents, or other related
items, is being provided by the copyright holder(s) subject to the terms of the
Educational Community License. By obtaining, using and/or copying this Original
Work, you agree that you have read, understand, and will comply with the
following terms and conditions of the Educational Community License:

Permission to use, copy, modify, merge, publish, distribute, and sublicense this
Original Work and its documentation, with or without modification, for any
purpose, and without fee or royalty to the copyright holder(s) is hereby
granted, provided that you include the following on ALL copies of the Original
Work or portions thereof, including modifications or derivatives, that you
make:

The full text of the Educational Community License in a location viewable to
users of the redistributed or derivative work.

Any pre-existing intellectual property disclaimers, notices, or terms and
conditions.

Notice of any changes or modifications to the Original Work, including the date
the changes were made.

Any modifications of the Original Work must be distributed in such a manner as
to avoid any confusion with the Original Work of the copyright holders.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

The name and trademarks of copyright holder(s) may NOT be used in advertising or
publicity pertaining to the Original or Derivative Works without specific,
written prior permission. Title to copyright in the Original Work and any
associated documentation will at all times remain with the copyright holders.
`
const license_ECL_2_0_lre = `//**
Educational Community License v2.0
https://spdx.org/licenses/ECL-2.0.json
https://opensource.org/licenses/ECL-2.0
**//

(( Educational Community License

Version 2.0, April 2007 ))??

(( http:/www.osedu.org/licenses/

The Educational Community License version 2.0 ("ECL") consists of the Apache 2.0
license, modified to change the scope of the patent grant in section 3 to be
specific to the needs of the education communities using this license. The
original Apache 2.0 license can be found at:
http:/www.apache.org/licenses/LICENSE-2.0 ))??

TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION

   (( 1. ))??
   Definitions.

      "License" shall mean the terms and conditions for use, reproduction, and
      distribution as defined by Sections 1 through 9 of this document.

      "Licensor" shall mean the copyright owner or entity authorized by the
      copyright owner that is granting the License.

      "Legal Entity" shall mean the union of the acting entity and all other
      entities that control, are controlled by, or are under common control with
      that entity. For the purposes of this definition, "control" means (i) the
      power, direct or indirect, to cause the direction or management of such
      entity, whether by contract or otherwise, or (ii) ownership of fifty
      percent (50%) or more of the outstanding shares, or (iii) beneficial
      ownership of such entity.

      "You" (or "Your") shall mean an individual or Legal Entity exercising
      permissions granted by this License.

      "Source" form shall mean the preferred form for making modifications,
      including but not limited to software source code, documentation source,
      and configuration files.

      "Object" form shall mean any form resulting from mechanical transformation
      or translation of a Source form, including but not limited to compiled
      object code, generated documentation, and conversions to other media
      types.

      "Work" shall mean the work of authorship, whether in Source or Object
      form, made available under the License, as indicated by a copyright notice
      that is included in or attached to the work (an example is provided in the
      Appendix below).

      "Derivative Works" shall mean any work, whether in Source or Object form,
      that is based on (or derived from) the Work and for which the editorial
      revisions, annotations, elaborations, or other modifications represent, as
      a whole, an original work of authorship. For the purposes of this License,
      Derivative Works shall not include works that remain separable from, or
      merely link (or bind by name) to the interfaces of, the Work and
      Derivative Works thereof.

      "Contribution" shall mean any work of authorship, including the original
      version of the Work and any modifications or additions to that Work or
      Derivative Works thereof, that is intentionally submitted to Licensor for
      inclusion in the Work by the copyright owner or by an individual or Legal
      Entity authorized to submit on behalf of the copyright owner. For the
      purposes of this definition, "submitted" means any form of electronic,
      verbal, or written communication sent to the Licensor or its
      representatives, including but not limited to communication on electronic
      mailing lists, source code control systems, and issue tracking systems
      that are managed by, or on behalf of, the Licensor for the purpose of
      discussing and improving the Work, but excluding communication that is
      conspicuously marked or otherwise designated in writing by the copyright
      owner as "Not a Contribution."

      "Contributor" shall mean Licensor and any individual or Legal Entity on
      behalf of whom a Contribution has been received by Licensor and
      subsequently incorporated within the Work.

   (( 2. ))??
   Grant of Copyright License.

   Subject to the terms and conditions of this License, each Contributor hereby
   grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,
   irrevocable copyright license to reproduce, prepare Derivative Works of,
   publicly display, publicly perform, sublicense, and distribute the Work and
   such Derivative Works in Source or Object form.

   (( 3. ))??
   Grant of Patent License.

   Subject to the terms and conditions of this License, each Contributor hereby
   grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,
   irrevocable (except as stated in this section) patent license to make, have
   made, use, offer to sell, sell, import, and otherwise transfer the Work,
   where such license applies only to those patent claims licensable by such
   Contributor that are necessarily infringed by their Contribution(s) alone or
   by combination of their Contribution(s) with the Work to which such
   Contribution(s) was submitted. If You institute patent litigation against any
   entity (including a cross-claim or counterclaim in a lawsuit) alleging that
   the Work or a Contribution incorporated within the Work constitutes direct or
   contributory patent infringement, then any patent licenses granted to You
   under this License for that Work shall terminate as of the date such
   litigation is filed. Any patent license granted hereby with respect to
   contributions by an individual employed by an institution or organization is
   limited to patent claims where the individual that is the author of the Work
   is also the inventor of the patent claims licensed, and where the
   organization or institution has the right to grant such license under
   applicable grant and research funding agreements. No other express or implied
   licenses are granted.

   (( 4. ))??
   Redistribution.

   You may reproduce and distribute copies of the Work or Derivative Works
   thereof in any medium, with or without modifications, and in Source or Object
   form, provided that You meet the following conditions:

      (( a. ))??
      You must give any other recipients of the Work or Derivative Works a copy
      of this License; and

      (( b. ))??
      You must cause any modified files to carry prominent notices stating that
      You changed the files; and

      (( c. ))??
      You must retain, in the Source form of any Derivative Works that You
      distribute, all copyright, patent, trademark, and attribution notices from
      the Source form of the Work, excluding those notices that do not pertain
      to any part of the Derivative Works; and

      (( d. ))??
      If the Work includes a "NOTICE" text file as part of its distribution,
      then any Derivative Works that You distribute must include a readable copy
      of the attribution notices contained within such NOTICE file, excluding
      those notices that do not pertain to any part of the Derivative Works, in
      at least one of the following places: within a NOTICE text file
      distributed as part of the Derivative Works; within the Source form or
      documentation, if provided along with the Derivative Works; or, within a
      display generated by the Derivative Works, if and wherever such
      third-party notices normally appear. The contents of the NOTICE file are
      for informational purposes only and do not modify the License. You may add
      Your own attribution notices within Derivative Works that You distribute,
      alongside or as an addendum to the NOTICE text from the Work, provided
      that such additional attribution notices cannot be construed as modifying
      the License.

      You may add Your own copyright statement to Your modifications and may
      provide additional or different license terms and conditions for use,
      reproduction, or distribution of Your modifications, or for any such
      Derivative Works as a whole, provided Your use, reproduction, and
      distribution of the Work otherwise complies with the conditions stated in
      this License.

   (( 5. ))??
   Submission of Contributions.

   Unless You explicitly state otherwise, any Contribution intentionally
   submitted for inclusion in the Work by You to the Licensor shall be under the
   terms and conditions of this License, without any additional terms or
   conditions. Notwithstanding the above, nothing herein shall supersede or
   modify the terms of any separate license agreement you may have executed with
   Licensor regarding such Contributions.

   (( 6. ))??
   Trademarks.

   This License does not grant permission to use the trade names, trademarks,
   service marks, or product names of the Licensor, except as required for
   reasonable and customary use in describing the origin of the Work and
   reproducing the content of the NOTICE file.

   (( 7. ))??
   Disclaimer of Warranty.

   Unless required by applicable law or agreed to in writing, Licensor provides
   the Work (and each Contributor provides its Contributions) on an "AS IS"
   BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied, including, without limitation, any warranties or conditions of
   TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR
   PURPOSE. You are solely responsible for determining the appropriateness of
   using or redistributing the Work and assume any risks associated with Your
   exercise of permissions under this License.

   (( 8. ))??
   Limitation of Liability.

   In no event and under no legal theory, whether in tort (including
   negligence), contract, or otherwise, unless required by applicable law (such
   as deliberate and grossly negligent acts) or agreed to in writing, shall any
   Contributor be liable to You for damages, including any direct, indirect,
   special, incidental, or consequential damages of any character arising as a
   result of this License or out of the use or inability to use the Work
   (including but not limited to damages for loss of goodwill, work stoppage,
   computer failure or malfunction, or any and all other commercial damages or
   losses), even if such Contributor has been advised of the possibility of such
   damages.

   (( 9. ))??
   Accepting Warranty or Additional Liability.

While redistributing the Work or Derivative Works thereof, You may choose to
offer, and charge a fee for, acceptance of support, warranty, indemnity, or
other liability obligations and/or rights consistent with this License. However,
in accepting such obligations, You may act only on Your own behalf and on Your
sole responsibility, not on behalf of any other Contributor, and only if You
agree to indemnify, defend, and hold each Contributor harmless for any liability
incurred by, or claims asserted against, such Contributor by reason of your
accepting any such warranty or additional liability.
(( END OF TERMS AND CONDITIONS

APPENDIX: How to apply the Educational Community License to your work

To apply the Educational Community License to your work, attach

the following boilerplate notice, with the fields enclosed by

brackets "[]" replaced with your own identifying information.

(Don't include the brackets!) The text should be enclosed in the

appropriate comment syntax for the file format. We also recommend

that a file or class name and description of purpose be included on

the same "printed page" as the copyright notice for easier

identification within third-party archives.

Copyright __20__

Licensed under the

Educational Community License, Version 2.0 (the "License"); you may

not use this file except in compliance with the License. You may

obtain a copy of the License at

http:/www.osedu.org/licenses/ECL-2.0

Unless required by applicable law or agreed to in writing,

software distributed under the License is distributed on an "AS IS"

BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express

or implied. See the License for the specific language governing

permissions and limitations under the License. ))??
`
const license_EFL_1_0_lre = `//**
Eiffel Forum License v1.0
https://spdx.org/licenses/EFL-1.0.json
http://www.eiffel-nice.org/license/forum.txt
https://opensource.org/licenses/EFL-1.0
**//

(( Eiffel Forum License, version 1 ))??

Permission is hereby granted to use, copy, modify and/or distribute this
package, provided that:

   -
   copyright notices are retained unchanged

   -
   any distribution of this package, whether modified or not, includes this
   file

Permission is hereby also granted to distribute binary programs which depend on
this package, provided that:

   -
   if the binary program depends on a modified version of this package, you must
   publicly release the modified version of this package

THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY. ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE
OF THIS PACKAGE.
`
const license_EFL_2_0_lre = `//**
Eiffel Forum License v2.0
https://spdx.org/licenses/EFL-2.0.json
http://www.eiffel-nice.org/license/eiffel-forum-license-2.html
https://opensource.org/licenses/EFL-2.0
**//

(( Eiffel Forum License, version 2 ))??

   (( 1. ))??
   Permission is hereby granted to use, copy, modify and/or distribute this
   package, provided that:

      -
      copyright notices are retained unchanged,

      -
      any distribution of this package, whether modified or not, includes this
      license text.

   (( 2. ))??
   Permission is hereby also granted to distribute binary programs which depend
   on this package. If the binary program depends on a modified version of this
   package, you are encouraged to publicly release the modified version of this
   package.

THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY. ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE
OF THIS PACKAGE.
`
const license_EPICS_lre = `//**
EPICS Open License
https://spdx.org/licenses/EPICS.json
https://epics.anl.gov/license/open.php
**//

EPICS Open License Terms

The following is the text of the EPICS Open software license agreement which now
applies to EPICS Base and many of the unbundled EPICS extensions and support
modules.

Copyright __20__

... is distributed subject to the following license conditions:

SOFTWARE LICENSE AGREEMENT

Software: __10__

   (( 1. ))??
   The "Software", below, refers to __10__ (in either source code, or binary
   form and accompanying documentation). Each licensee is addressed as "you" or
   "Licensee."

   (( 2. ))??
   The copyright holders shown above and their third-party licensors hereby
   grant Licensee a royalty-free nonexclusive license, subject to the
   limitations stated herein and U.S. Government license rights.

   (( 3. ))??
   You may modify and make a copy or copies of the Software for use within your
   organization, if you meet the following conditions:

      (( a. ))??
      Copies in source code must include the copyright notice and this Software
      License Agreement.

      (( b. ))??
      Copies in binary form must include the copyright notice and this Software
      License Agreement in the documentation and/or other materials provided
      with the copy.

   (( 4. ))??
   You may modify a copy or copies of the Software or any portion of it, thus
   forming a work based on the Software, and distribute copies of such work
   outside your organization, if you meet all of the following conditions:

      (( a. ))??
      Copies in source code must include the copyright notice and this Software
      License Agreement;

      (( b. ))??
      Copies in binary form must include the copyright notice and this Software
      License Agreement in the documentation and/or other materials provided
      with the copy;

      (( c. ))??
      Modified copies and works based on the Software must carry prominent
      notices stating that you changed specified portions of the Software.

   (( 5. ))??
   Portions of the Software resulted from work developed under a U.S. Government
   contract and are subject to the following license: the Government is granted
   for itself and others acting on its behalf a paid-up, nonexclusive,
   irrevocable worldwide license in this computer software to reproduce, prepare
   derivative works, and perform publicly and display publicly.

   (( 6. ))??
   WARRANTY DISCLAIMER. THE SOFTWARE IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY
   KIND. THE COPYRIGHT HOLDERS, THEIR THIRD PARTY LICENSORS, THE UNITED STATES,
   THE UNITED STATES DEPARTMENT OF ENERGY, AND THEIR EMPLOYEES: (1) DISCLAIM ANY
   WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO ANY IMPLIED
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE OR
   NON-INFRINGEMENT, (2) DO NOT ASSUME ANY LEGAL LIABILITY OR RESPONSIBILITY FOR
   THE ACCURACY, COMPLETENESS, OR USEFULNESS OF THE SOFTWARE, (3) DO NOT REPTHAT
   USE OF THE SOFTWARE WOULD NOT INFRINGE PRIVATELY OWNED RIGHTS, (4) DO NOT
   WARRANT THAT THE SOFTWARE WILL FUNCTION UNINTERRUPTED, THAT IT IS ERROR-FREE
   OR THAT ANY ERRORS WILL BE CORRECTED.

   (( 7. ))??
   LIMITATION OF LIABILITY. IN NO EVENT WILL THE COPYRIGHT HOLDERS, THEIR THIRD
   PARTY LICENSORS, THE UNITED STATES, THE UNITED STATES DEPARTMENT OF ENERGY,
   OR THEIR EMPLOYEES: BE LIABLE FOR ANY INDIRECT, INCIDENTAL, CONSEQUENTIAL,
   SPECIAL OR PUNITIVE DAMAGES OF ANY KIND OR NATURE, INCLUDING BUT NOT LIMITED
   TO LOSS OF PROFITS OR LOSS OF DATA, FOR ANY REASON WHATSOEVER, WHETHER SUCH
   LIABILITY IS ASSERTED ON THE BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE OR
   STRICT LIABILITY), OR OTHERWISE, EVEN IF ANY OF SAID PARTIES HAS BEEN WARNED
   OF THE POSSIBILITY OF SUCH LOSS OR DAMAGES.
`
const license_EPL_1_0_lre = `//**
Eclipse Public License 1.0
https://spdx.org/licenses/EPL-1.0.json
http://www.eclipse.org/legal/epl-v10.html
https://opensource.org/licenses/EPL-1.0
**//

(( Eclipse Public License - v 1.0 ))??

((THE ACCOMPANYING PROGRAM))??
IS PROVIDED UNDER THE TERMS OF THIS ECLIPSE PUBLIC
LICENSE ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF
((THE PROGRAM || THIS SOFTWARE))
CONSTITUTES
((THE))??
RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.

   (( 1. ))??
   DEFINITIONS

   "Contribution" means:

      (( a) ))??
      in the case of the initial Contributor, the initial code and documentation
      distributed under this Agreement, and

      (( b) ))??
      in the case of each subsequent Contributor:

         (( i) ))??
         changes to the Program, and

         (( ii) ))??
         additions to the Program;

         where such changes and/or additions to the Program originate from and
         are distributed by that particular Contributor. A Contribution
         'originates' from a Contributor if it was added to the Program by such
         Contributor itself or anyone acting on such Contributor's behalf.
         Contributions do not include additions to the Program which: (i) are
         separate modules of software distributed in conjunction with the
         Program under their own license agreement, and (ii) are not derivative
         works of the Program.

   "Contributor" means any person or entity that distributes the Program.

   "Licensed Patents" mean patent claims licensable by a Contributor which are
   necessarily infringed by the use or sale of its Contribution alone or when
   combined with the Program.

   "Program" means the Contributions distributed in accordance with this
   Agreement.

   "Recipient" means anyone who receives the Program under this Agreement,
   including all Contributors.

   (( 2. ))??
   GRANT OF RIGHTS

      (( a) ))??
      Subject to the terms of this Agreement, each Contributor hereby grants
      Recipient a non-exclusive, worldwide, royalty-free copyright license to
      reproduce, prepare derivative works of, publicly display, publicly
      perform, distribute and sublicense the Contribution of such Contributor,
      if any, and such derivative works, in source code and object code form.

      (( b) ))??
      Subject to the terms of this Agreement, each Contributor hereby grants
      Recipient a non-exclusive, worldwide, royalty-free patent license under
      Licensed Patents to make, use, sell, offer to sell, import and otherwise
      transfer the Contribution of such Contributor, if any, in source code and
      object code form. This patent license shall apply to the combination of
      the Contribution and the Program if, at the time the Contribution is added
      by the Contributor, such addition of the Contribution causes such
      combination to be covered by the Licensed Patents. The patent license
      shall not apply to any other combinations which include the Contribution.
      No hardware per se is licensed hereunder.

      (( c) ))??
      Recipient understands that although each Contributor grants the licenses
      to its Contributions set forth herein, no assurances are provided by any
      Contributor that the Program does not infringe the patent or other
      intellectual property rights of any other entity. Each Contributor
      disclaims any liability to Recipient for claims brought by any other
      entity based on infringement of intellectual property rights or otherwise.
      As a condition to exercising the rights and licenses granted hereunder,
      each Recipient hereby assumes sole responsibility to secure any other
      intellectual property rights needed, if any. For example, if a third party
      patent license is required to allow Recipient to distribute the Program,
      it is Recipient's responsibility to acquire that license before
      distributing the Program.

      (( d) ))??
      Each Contributor represents that to its knowledge it has sufficient
      copyright rights in its Contribution, if any, to grant the copyright
      license set forth in this Agreement.

   (( 3. ))??
   REQUIREMENTS

   A Contributor may choose to distribute the Program in object code form under
   its own license agreement, provided that:

      (( a) ))??
      it complies with the terms and conditions of this Agreement; and

      (( b) ))??
      its license agreement:

         (( i) ))??
         effectively disclaims on behalf of all Contributors all warranties and
         conditions, express and implied, including warranties or conditions of
         title and non-infringement, and implied warranties or conditions of
         merchantability and fitness for a particular purpose;

         (( ii) ))??
         effectively excludes on behalf of all Contributors all liability for
         damages, including direct, indirect, special, incidental and
         consequential damages, such as lost profits;

         (( iii) ))??
         states that any provisions which differ from this Agreement are offered
         by that Contributor alone and not by any other party; and

         (( iv) ))??
         states that source code for the Program is available from such
         Contributor, and informs licensees how to obtain it in a reasonable
         manner on or through a medium customarily used for software exchange.

   When the Program is made available in source code form:

      (( a) ))??
      it must be made available under this Agreement; and

      (( b) ))??
      a copy of this Agreement must be included with each copy of the Program.

      Contributors may not remove or alter any copyright notices contained
      within the Program.

   Each Contributor must identify itself as the originator of its Contribution,
   if any, in a manner that reasonably allows subsequent Recipients to identify
   the originator of the Contribution.

   (( 4. ))??
   COMMERCIAL DISTRIBUTION

   Commercial distributors of software may accept certain responsibilities with
   respect to end users, business partners and the like. While this license is
   intended to facilitate the commercial use of the Program, the Contributor who
   includes the Program in a commercial product offering should do so in a
   manner which does not create potential liability for other Contributors.
   Therefore, if a Contributor includes the Program in a commercial product
   offering, such Contributor ("Commercial Contributor") hereby agrees to defend
   and indemnify every other Contributor ("Indemnified Contributor") against any
   losses, damages and costs (collectively "Losses") arising from claims,
   lawsuits and other legal actions brought by a third party against the
   Indemnified Contributor to the extent caused by the acts or omissions of such
   Commercial Contributor in connection with its distribution of the Program in
   a commercial product offering. The obligations in this section do not apply
   to any claims or Losses relating to any actual or alleged intellectual
   property infringement. In order to qualify, an Indemnified Contributor must:
   a) promptly notify the Commercial Contributor in writing of such claim, and
   b) allow the Commercial Contributor to control, and cooperate with the
   Commercial Contributor in, the defense and any related settlement
   negotiations. The Indemnified Contributor may participate in any such claim
   at its own expense.

   For example, a Contributor might include the Program in a commercial product
   offering, Product X. That Contributor is then a Commercial Contributor. If
   that Commercial Contributor then makes performance claims, or offers
   warranties related to Product X, those performance claims and warranties are
   such Commercial Contributor's responsibility alone. Under this section, the
   Commercial Contributor would have to defend claims against the other
   Contributors related to those performance claims and warranties, and if a
   court requires any other Contributor to pay any damages as a result, the
   Commercial Contributor must pay those damages.

   (( 5. ))??
   NO WARRANTY

   EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, THE PROGRAM IS PROVIDED ON
   AN "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER
   EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION, ANY WARRANTIES OR
   CONDITIONS OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A
   PARTICULAR PURPOSE. Each Recipient is solely responsible for determining the
   appropriateness of using and distributing the Program and assumes all risks
   associated with its exercise of rights under this Agreement, including but
   not limited to the risks and costs of program errors, compliance with
   applicable laws, damage to or loss of data, programs or equipment, and
   unavailability or interruption of operations.

   (( 6. ))??
   DISCLAIMER OF LIABILITY

   EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, NEITHER RECIPIENT NOR ANY
   CONTRIBUTORS SHALL HAVE ANY LIABILITY FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING WITHOUT LIMITATION
   LOST PROFITS), HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OR DISTRIBUTION OF THE PROGRAM OR THE
   EXERCISE OF ANY RIGHTS GRANTED HEREUNDER, EVEN IF ADVISED OF THE POSSIBILITY
   OF SUCH DAMAGES.

   (( 7. ))??
   GENERAL

   If any provision of this Agreement is invalid or unenforceable under
   applicable law, it shall not affect the validity or enforceability of the
   remainder of the terms of this Agreement, and without further action by the
   parties hereto, such provision shall be reformed to the minimum extent
   necessary to make such provision valid and enforceable.

   If Recipient institutes patent litigation against any entity (including a
   cross-claim or counterclaim in a lawsuit) alleging that the Program itself
   (excluding combinations of the Program with other software or hardware)
   infringes such Recipient's patent(s), then such Recipient's rights granted
   under Section 2(b) shall terminate as of the date such litigation is filed.

   All Recipient's rights under this Agreement shall terminate if it fails to
   comply with any of the material terms or conditions of this Agreement and
   does not cure such failure in a reasonable period of time after becoming
   aware of such noncompliance. If all Recipient's rights under this Agreement
   terminate, Recipient agrees to cease use and distribution of the Program as
   soon as reasonably practicable. However, Recipient's obligations under this
   Agreement and any licenses granted by Recipient relating to the Program shall
   continue and survive.

   Everyone is permitted to copy and distribute copies of this Agreement, but in
   order to avoid inconsistency the Agreement is copyrighted and may only be
   modified in the following manner. The Agreement Steward reserves the right to
   publish new versions (including revisions) of this Agreement from time to
   time. No one other than the Agreement Steward has the right to modify this
   Agreement. The Eclipse Foundation is the initial Agreement Steward. The
   Eclipse Foundation may assign the responsibility to serve as the Agreement
   Steward to a suitable separate entity. Each new version of the Agreement will
   be given a distinguishing version number. The Program (including
   Contributions) may always be distributed subject to the version of the
   Agreement under which it was received. In addition, after a new version of
   the Agreement is published, Contributor may elect to distribute the Program
   (including its Contributions) under the new version. Except as expressly
   stated in Sections 2(a) and 2(b) above, Recipient receives no rights or
   licenses to the intellectual property of any Contributor under this
   Agreement, whether expressly, by implication, estoppel or otherwise. All
   rights in the Program not expressly granted under this Agreement are
   reserved.

   This Agreement is governed by the laws of the State of __2__ and the
   intellectual property laws of the United States of America. No party to this
   Agreement will bring a legal action under this Agreement more than one year
   after the cause of action arose. Each party waives its rights to a jury trial
   in any resulting litigation.
`
const license_EPL_2_0_lre = `//**
Eclipse Public License 2.0
https://spdx.org/licenses/EPL-2.0.json
https://www.eclipse.org/legal/epl-2.0
https://www.opensource.org/licenses/EPL-2.0
**//

(( Eclipse Public License - v 2.0 ))??

THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS ECLIPSE PUBLIC
LICENSE ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THE PROGRAM
CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.

   (( 1. ))??
   DEFINITIONS

   "Contribution" means:

      (( a) ))??
      in the case of the initial Contributor, the initial content Distributed
      under this Agreement, and

      (( b) ))??
      in the case of each subsequent Contributor:

         (( i) ))??
         changes to the Program, and

         (( ii) ))??
         additions to the Program;

      where such changes and/or additions to the Program originate from and are
      Distributed by that particular Contributor. A Contribution "originates"
      from a Contributor if it was added to the Program by such Contributor
      itself or anyone acting on such Contributor's behalf. Contributions do not
      include changes or additions to the Program that are not Modified Works.

   "Contributor" means any person or entity that Distributes the Program.

   "Licensed Patents" mean patent claims licensable by a Contributor which are
   necessarily infringed by the use or sale of its Contribution alone or when
   combined with the Program.

   "Program" means the Contributions Distributed in accordance with this
   Agreement.

   "Recipient" means anyone who receives the Program under this Agreement or any
   Secondary License (as applicable), including Contributors.

   "Derivative Works" shall mean any work, whether in Source Code or other form,
   that is based on (or derived from) the Program and for which the editorial
   revisions, annotations, elaborations, or other modifications represent, as a
   whole, an original work of authorship.

   "Modified Works" shall mean any work in Source Code or other form that
   results from an addition to, deletion from, or modification of the contents
   of the Program, including, for purposes of clarity any new file in Source
   Code form that contains any contents of the Program. Modified Works shall not
   include works that contain only declarations, interfaces, types, classes,
   structures, or files of the Program solely in each case in order to link to,
   bind by name, or subclass the Program or Modified Works thereof.

   "Distribute" means the acts of a) distributing or b) making available in any
   manner that enables the transfer of a copy.

   "Source Code" means the form of a Program preferred for making modifications,
   including but not limited to software source code, documentation source, and
   configuration files.

   "Secondary License" means either the GNU General Public License, Version 2.0,
   or any later versions of that license, including any exceptions or additional
   permissions as identified by the initial Contributor.

   (( 2. ))??
   GRANT OF RIGHTS

      (( a) ))??
      Subject to the terms of this Agreement, each Contributor hereby grants
      Recipient a non-exclusive, worldwide, royalty-free copyright license to
      reproduce, prepare Derivative Works of, publicly display, publicly
      perform, Distribute and sublicense the Contribution of such Contributor,
      if any, and such Derivative Works.

      (( b) ))??
      Subject to the terms of this Agreement, each Contributor hereby grants
      Recipient a non-exclusive, worldwide, royalty-free patent license under
      Licensed Patents to make, use, sell, offer to sell, import and otherwise
      transfer the Contribution of such Contributor, if any, in Source Code or
      other form. This patent license shall apply to the combination of the
      Contribution and the Program if, at the time the Contribution is added by
      the Contributor, such addition of the Contribution causes such combination
      to be covered by the Licensed Patents. The patent license shall not apply
      to any other combinations which include the Contribution. No hardware per
      se is licensed hereunder.

      (( c) ))??
      Recipient understands that although each Contributor grants the licenses
      to its Contributions set forth herein, no assurances are provided by any
      Contributor that the Program does not infringe the patent or other
      intellectual property rights of any other entity. Each Contributor
      disclaims any liability to Recipient for claims brought by any other
      entity based on infringement of intellectual property rights or otherwise.
      As a condition to exercising the rights and licenses granted hereunder,
      each Recipient hereby assumes sole responsibility to secure any other
      intellectual property rights needed, if any. For example, if a third party
      patent license is required to allow Recipient to Distribute the Program,
      it is Recipient's responsibility to acquire that license before
      distributing the Program.

      (( d) ))??
      Each Contributor represents that to its knowledge it has sufficient
      copyright rights in its Contribution, if any, to grant the copyright
      license set forth in this Agreement.

      (( e) ))??
      Notwithstanding the terms of any Secondary License, no Contributor makes
      additional grants to any Recipient (other than those set forth in this
      Agreement) as a result of such Recipient's receipt of the Program under
      the terms of a Secondary License (if permitted under the terms of Section
      3).

   (( 3. ))??
   REQUIREMENTS

      (( 3.1 ))??
      If a Contributor Distributes the Program in any form, then:

         (( a) ))??
         the Program must also be made available as Source Code, in accordance
         with section 3.2, and the Contributor must accompany the Program with a
         statement that the Source Code for the Program is available under this
         Agreement, and informs Recipients how to obtain it in a reasonable
         manner on or through a medium customarily used for software exchange;
         and

         (( b) ))??
         the Contributor may Distribute the Program under a license different
         than this Agreement, provided that such license:

            (( i) ))??
            effectively disclaims on behalf of all other Contributors all
            warranties and conditions, express and implied, including warranties
            or conditions of title and non-infringement, and implied warranties
            or conditions of merchantability and fitness for a particular
            purpose;

            (( ii) ))??
            effectively excludes on behalf of all other Contributors all
            liability for damages, including direct, indirect, special,
            incidental and consequential damages, such as lost profits;

            (( iii) ))??
            does not attempt to limit or alter the recipients' rights in the
            Source Code under section 3.2; and

            (( iv) ))??
            requires any subsequent distribution of the Program by any party to
            be under a license that satisfies the requirements of this section
            3.

      (( 3.2 ))??
      When the Program is Distributed as Source Code:

         (( a) ))??
         it must be made available under this Agreement, or if the Program (i)
         is combined with other material in a separate file or files made
         available under a Secondary License, and (ii) the initial Contributor
         attached to the Source Code the notice described in Exhibit A of this
         Agreement, then the Program may be made available under the terms of
         such Secondary Licenses, and

         (( b) ))??
         a copy of this Agreement must be included with each copy of the
         Program.

      (( 3.3 ))??
      Contributors may not remove or alter any copyright, patent, trademark,
      attribution notices, disclaimers of warranty, or limitations of liability
      ("notices") contained within the Program from any copy of the Program
      which they Distribute, provided that Contributors may add their own
      appropriate notices.

   (( 4. ))??
   COMMERCIAL DISTRIBUTION

   Commercial distributors of software may accept certain responsibilities with
   respect to end users, business partners and the like. While this license is
   intended to facilitate the commercial use of the Program, the Contributor who
   includes the Program in a commercial product offering should do so in a
   manner which does not create potential liability for other Contributors.
   Therefore, if a Contributor includes the Program in a commercial product
   offering, such Contributor ("Commercial Contributor") hereby agrees to defend
   and indemnify every other Contributor ("Indemnified Contributor") against any
   losses, damages and costs (collectively "Losses") arising from claims,
   lawsuits and other legal actions brought by a third party against the
   Indemnified Contributor to the extent caused by the acts or omissions of such
   Commercial Contributor in connection with its distribution of the Program in
   a commercial product offering. The obligations in this section do not apply
   to any claims or Losses relating to any actual or alleged intellectual
   property infringement. In order to qualify, an Indemnified Contributor must:
   a) promptly notify the Commercial Contributor in writing of such claim, and
   b) allow the Commercial Contributor to control, and cooperate with the
   Commercial Contributor in, the defense and any related settlement
   negotiations. The Indemnified Contributor may participate in any such claim
   at its own expense.

   For example, a Contributor might include the Program in a commercial product
   offering, Product X. That Contributor is then a Commercial Contributor. If
   that Commercial Contributor then makes performance claims, or offers
   warranties related to Product X, those performance claims and warranties are
   such Commercial Contributor's responsibility alone. Under this section, the
   Commercial Contributor would have to defend claims against the other
   Contributors related to those performance claims and warranties, and if a
   court requires any other Contributor to pay any damages as a result, the
   Commercial Contributor must pay those damages.

   (( 5. ))??
   NO WARRANTY

   EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, AND TO THE EXTENT PERMITTED
   BY APPLICABLE LAW, THE PROGRAM IS PROVIDED ON AN "AS IS" BASIS, WITHOUT
   WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING,
   WITHOUT LIMITATION, ANY WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT,
   MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Each Recipient is solely
   responsible for determining the appropriateness of using and distributing the
   Program and assumes all risks associated with its exercise of rights under
   this Agreement, including but not limited to the risks and costs of program
   errors, compliance with applicable laws, damage to or loss of data, programs
   or equipment, and unavailability or interruption of operations.

   (( 6. ))??
   DISCLAIMER OF LIABILITY

   EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, AND TO THE EXTENT PERMITTED
   BY APPLICABLE LAW, NEITHER RECIPIENT NOR ANY CONTRIBUTORS SHALL HAVE ANY
   LIABILITY FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING WITHOUT LIMITATION LOST PROFITS), HOWEVER
   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OR DISTRIBUTION OF THE PROGRAM OR THE EXERCISE OF ANY RIGHTS GRANTED
   HEREUNDER, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

   (( 7. ))??
   GENERAL

   If any provision of this Agreement is invalid or unenforceable under
   applicable law, it shall not affect the validity or enforceability of the
   remainder of the terms of this Agreement, and without further action by the
   parties hereto, such provision shall be reformed to the minimum extent
   necessary to make such provision valid and enforceable.

   If Recipient institutes patent litigation against any entity (including a
   cross-claim or counterclaim in a lawsuit) alleging that the Program itself
   (excluding combinations of the Program with other software or hardware)
   infringes such Recipient's patent(s), then such Recipient's rights granted
   under Section 2(b) shall terminate as of the date such litigation is filed.

   All Recipient's rights under this Agreement shall terminate if it fails to
   comply with any of the material terms or conditions of this Agreement and
   does not cure such failure in a reasonable period of time after becoming
   aware of such noncompliance. If all Recipient's rights under this Agreement
   terminate, Recipient agrees to cease use and distribution of the Program as
   soon as reasonably practicable. However, Recipient's obligations under this
   Agreement and any licenses granted by Recipient relating to the Program shall
   continue and survive.

   Everyone is permitted to copy and distribute copies of this Agreement, but in
   order to avoid inconsistency the Agreement is copyrighted and may only be
   modified in the following manner. The Agreement Steward reserves the right to
   publish new versions (including revisions) of this Agreement from time to
   time. No one other than the Agreement Steward has the right to modify this
   Agreement. The Eclipse Foundation is the initial Agreement Steward. The
   Eclipse Foundation may assign the responsibility to serve as the Agreement
   Steward to a suitable separate entity. Each new version of the Agreement will
   be given a distinguishing version number. The Program (including
   Contributions) may always be Distributed subject to the version of the
   Agreement under which it was received. In addition, after a new version of
   the Agreement is published, Contributor may elect to Distribute the Program
   (including its Contributions) under the new version.

   Except as expressly stated in Sections 2(a) and 2(b) above, Recipient
   receives no rights or licenses to the intellectual property of any
   Contributor under this Agreement, whether expressly, by implication, estoppel
   or otherwise. All rights in the Program not expressly granted under this
   Agreement are reserved. Nothing in this Agreement is intended to be
   enforceable by any entity that is not a Contributor or Recipient. No
   third-party beneficiary rights are created under this Agreement.

   Exhibit A - Form of Secondary Licenses Notice

   "This Source Code may also be made available under the following Secondary
   Licenses when the conditions for such availability set forth in the Eclipse
   Public License, v. 2.0 are satisfied: {name license(s), version(s), and
   exceptions or additional permissions here}."

   Simply including a copy of this Agreement, including this Exhibit A is not
   sufficient to license the Source Code under Secondary Licenses.

   If it is not possible or desirable to put the notice in a particular file,
   then You may include the notice in a location (such as a LICENSE file in a
   relevant directory) where a recipient would be likely to look for such a
   notice.

   You may add additional accurate notices of copyright ownership.
`
const license_EUDatagrid_lre = `//**
EU DataGrid Software License
https://spdx.org/licenses/EUDatagrid.json
http://eu-datagrid.web.cern.ch/eu-datagrid/license.html
https://opensource.org/licenses/EUDatagrid
**//

(( EU DataGrid Software License
((Copyright __20__))??
))??

This software includes voluntary contributions made to the EU DataGrid. For more
information on the EU DataGrid, please see http:/www.eu-datagrid.org/.

Installation, use, reproduction, display, modification and redistribution of
this software, with or without modification, in source and binary forms, are
permitted. Any exercise of rights under this license by you or your
sub-licensees is subject to the following conditions:

   (( 1. ))??
   Redistributions of this software, with or without modification, must
   reproduce the above copyright notice and the above license statement as well
   as this list of conditions, in the software, the user documentation and any
   other materials provided with the software.

   (( 2. ))??
   The user documentation, if any, included with a redistribution, must include
   the following notice:

   "This product includes software developed by the EU DataGrid
   (http:/www.eu-datagrid.org/)."

   Alternatively, if that is where third-party acknowledgments normally appear,
   this acknowledgment must be reproduced in the software itself.

   (( 3. ))??
   The names "EDG", "EDG Toolkit", "EU DataGrid" and "EU DataGrid Project" may
   not be used to endorse or promote software, or products derived therefrom,
   except with prior written permission by
   hep-project-grid-edg-license@cern.ch.

   (( 4. ))??
   You are under no obligation to provide anyone with any bug fixes, patches,
   upgrades or other modifications, enhancements or derivatives of the features,
   functionality or performance of this software that you may develop. However,
   if you publish or distribute your modifications, enhancements or derivative
   works without contemporaneously requiring users to enter into a separate
   written license agreement, then you are deemed to have granted participants
   in the EU DataGrid a worldwide, non-exclusive, royalty-free, perpetual
   license to install, use, reproduce, display, modify, redistribute and
   sub-license your modifications, enhancements or derivative works, whether in
   binary or source code form, under the license conditions stated in this list
   of conditions.

   (( 5. ))??
   DISCLAIMER

   THIS SOFTWARE IS PROVIDED BY THE EU DATAGRID AND CONTRIBUTORS "AS IS" AND ANY
   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY, OF SATISFACTORY QUALITY, AND FITNESS FOR A
   PARTICULAR PURPOSE OR USE ARE DISCLAIMED. THE EU DATAGRID AND CONTRIBUTORS
   MAKE NO REPRESENTATION THAT THE SOFTWARE, MODIFICATIONS, ENHANCEMENTS OR
   DERIVATIVE WORKS THEREOF, WILL NOT INFRINGE ANY PATENT, COPYRIGHT, TRADE
   SECRET OR OTHER PROPRIETARY RIGHT.

   (( 6. ))??
   LIMITATION OF LIABILITY

THE EU DATAGRID AND CONTRIBUTORS SHALL HAVE NO LIABILITY TO LICENSEE OR OTHER
PERSONS FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, CONSEQUENTIAL, EXEMPLARY, OR
PUNITIVE DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT LIMITATION, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES, LOSS OF USE, DATA OR PROFITS, OR BUSINESS
INTERRUPTION, HOWEVER CAUSED AND ON ANY THEORY OF CONTRACT, WARRANTY, TORT
(INCLUDING NEGLIGENCE), PRODUCT LIABILITY OR OTHERWISE, ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
`
const license_EUPL_1_0_lre = `//**
European Union Public License 1.0
https://spdx.org/licenses/EUPL-1.0.json
http://ec.europa.eu/idabc/en/document/7330.html
http://ec.europa.eu/idabc/servlets/Doc027f.pdf?id=31096
**//

(( European Union Public Licence V.1.0
(( EUPL ))??
(( Copyright __20__ ))??
))??

(( This European Union Public Licence (the "EUPL") applies to the Work or
Software (as defined below) which is provided under the terms of this Licence.
Any use of the Work, other than as authorised under this Licence is prohibited
(to the extent such use is covered by a right of the copyright holder of the
Work).

The Original Work is provided under the terms of this Licence when the Licensor
(as defined below) has placed the following notice immediately following the
copyright notice for the Original Work:

Licensed under the EUPL V.1.0

or has expressed by any other mean his willingness to license under the EUPL. ))??

   (( 1. ))??
   Definitions

   In this Licence, the following terms have the following meaning:

      −
      The Licence: this Licence.

      −
      The Original Work or the Software: the software distributed and/or
      communicated by the Licensor under this Licence, available as Source Code
      and also as Executable Code as the case may be.

      −
      Derivative Works: the works or software that could be created by the
      Licensee, based upon the Original Work or modifications thereof. This
      Licence does not define the extent of modification or dependence on the
      Original Work required in order to classify a work as a Derivative Work;
      this extent is determined by copyright law applicable in the country
      mentioned in Article 15.

      −
      The Work: the Original Work and/or its Derivative Works.

      −
      The Source Code: the human-readable form of the Work which is the most
      convenient for people to study and modify.

      −
      The Executable Code: any code which has generally been compiled and which
      is meant to be interpreted by a computer as a program.

      −
      The Licensor: the natural or legal person that distributes and/or
      communicates the Work under the Licence.

      −
      Contributor(s): any natural or legal person who modifies the Work under
      the Licence, or otherwise contributes to the creation of a Derivative
      Work.

      −
      The Licensee or "You": any natural or legal person who makes any usage of
      the Software under the terms of the Licence.

      −
      Distribution and/or Communication: any act of selling, giving, lending,
      renting, distributing, communicating, transmitting, or otherwise making
      available, on-line or off-line, copies of the Work at the disposal of any
      other natural or legal person.

   (( 2. ))??
   Scope of the rights granted by the Licence

   The Licensor hereby grants You a world-wide, royalty-free, non-exclusive,
   sub-licensable licence to do the following, for the duration of copyright
   vested in the Original Work:

      −
      use the Work in any circumstance and for all usage,

      −
      reproduce the Work,

      −
      modify the Original Work, and make Derivative Works based upon the Work,

      −
      communicate to the public, including the right to make available or
      display the Work or copies thereof to the public and perform publicly, as
      the case may be, the Work,

      −
      distribute the Work or copies thereof,

      −
      lend and rent the Work or copies thereof,

      −
      sub-license rights in the Work or copies thereof.

   Those rights can be exercised on any media, supports and formats, whether now
   known or later invented, as far as the applicable law permits so.

   In the countries where moral rights apply, the Licensor waives his right to
   exercise his moral right to the extent allowed by law in order to make
   effective the licence of the economic rights here above listed.

   The Licensor grants to the Licensee royalty-free, non exclusive usage rights
   to any patents held by the Licensor, to the extent necessary to make use of
   the rights granted on the Work under this Licence.

   (( 3. ))??
   Communication of the Source Code

   The Licensor may provide the Work either in its Source Code form, or as
   Executable Code. If the Work is provided as Executable Code, the Licensor
   provides in addition a machinereadable copy of the Source Code of the Work
   along with each copy of the Work that the Licensor distributes or indicates,
   in a notice following the copyright notice attached to the Work, a repository
   where the Source Code is easily and freely accessible for as long as the
   Licensor continues to distribute and/or communicate the Work.

   (( 4. ))??
   Limitations on copyright

   Nothing in this Licence is intended to deprive the Licensee of the benefits
   from any exception or limitation to the exclusive rights of the rights owners
   in the Original Work or Software, of the exhaustion of those rights or of
   other applicable limitations thereto.

   (( 5. ))??
   Obligations of the Licensee

   The grant of the rights mentioned above is subject to some restrictions and
   obligations imposed on the Licensee. Those obligations are the following:

   Attribution right: the Licensee shall keep intact all copyright, patent or
   trademarks notices and all notices that refer to the Licence and to the
   disclaimer of warranties. The Licensee must include a copy of such notices
   and a copy of the Licence with every copy of the Work he/she distributes
   and/or communicates. The Licensee must cause any Derivative Work to carry
   prominent notices stating that the Work has been modified and the date of
   modification.

   Copyleft clause: If the Licensee distributes and/or communicates copies of
   the Original Works or Derivative Works based upon the Original Work, this
   Distribution and/or Communication will be done under the terms of this
   Licence. The Licensee (becoming Licensor) cannot offer or impose any
   additional terms or conditions on the Work or Derivative Work that alter or
   restrict the terms of the Licence.

   Compatibility clause: If the Licensee Distributes and/or Communicates
   Derivative Works or copies thereof based upon both the Original Work and
   another work licensed under a Compatible Licence, this Distribution and/or
   Communication can be done under the terms of this Compatible Licence. For the
   sake of this clause, "Compatible Licence" refers to the licences listed in
   the appendix attached to this Licence. Should the Licensee's obligations
   under the Compatible Licence conflict with his/her obligations under this
   Licence, the obligations of the Compatible Licence shall prevail.

   Provision of Source Code: When distributing and/or communicating copies of
   the Work, the Licensee will provide a machine-readable copy of the Source
   Code or indicate a repository where this Source will be easily and freely
   available for as long as the Licensee continues to distribute and/or
   communicate the Work.

   Legal Protection: This Licence does not grant permission to use the trade
   names, trademarks, service marks, or names of the Licensor, except as
   required for reasonable and customary use in describing the origin of the
   Work and reproducing the content of the copyright notice.

   (( 6. ))??
   Chain of Authorship

   The original Licensor warrants that the copyright in the Original Work
   granted hereunder is owned by him/her or licensed to him/her and that he/she
   has the power and authority to grant the Licence.

   Each Contributor warrants that the copyright in the modifications he/she
   brings to the Work are owned by him/her or licensed to him/her and that
   he/she has the power and authority to grant the Licence.

   Each time You, as a Licensee, receive the Work, the original Licensor and
   subsequent Contributors grant You a licence to their contributions to the
   Work, under the terms of this Licence.

   (( 7. ))??
   Disclaimer of Warranty

   The Work is a work in progress, which is continuously improved by numerous
   contributors. It is not a finished work and may therefore contain defects or
   "bugs" inherent to this type of software development.

   For the above reason, the Work is provided under the Licence on an "as is"
   basis and without warranties of any kind concerning the Work, including
   without limitation merchantability, fitness for a particular purpose, absence
   of defects or errors, accuracy, non-infringement of intellectual property
   rights other than copyright as stated in Article 6 of this Licence.

   This disclaimer of warranty is an essential part of the Licence and a
   condition for the grant of any rights to the Work.

   (( 8. ))??
   Disclaimer of Liability

   Except in the cases of wilful misconduct or damages directly caused to
   natural persons, the Licensor will in no event be liable for any direct or
   indirect, material or moral, damages of any kind, arising out of the Licence
   or of the use of the Work, including without limitation, damages for loss of
   goodwill, work stoppage, computer failure or malfunction, loss of data or any
   commercial damage, even if the Licensor has been advised of the possibility
   of such damage. However, the Licensor will be liable under statutory product
   liability laws as far such laws apply to the Work.

   (( 9. ))??
   Additional agreements

   While distributing the Original Work or Derivative Works, You may choose to
   conclude an additional agreement to offer, and charge a fee for, acceptance
   of support, warranty, indemnity, or other liability obligations and/or
   services consistent with this Licence. However, in accepting such
   obligations, You may act only on your own behalf and on your sole
   responsibility, not on behalf of the original Licensor or any other
   Contributor, and only if You agree to indemnify, defend, and hold each
   Contributor harmless for any liability incurred by, or claims asserted
   against such Contributor by the fact You have accepted any such warranty or
   additional liability.

   (( 10. ))??
   Acceptance of the Licence

   The provisions of this Licence can be accepted by clicking on an icon "I
   agree" placed under the bottom of a window displaying the text of this
   Licence or by affirming consent in any other similar way, in accordance with
   the rules of applicable law. Clicking on that icon indicates your clear and
   irrevocable acceptance of this Licence and all of its terms and conditions.

   Similarly, you irrevocably accept this Licence and all of its terms and
   conditions by exercising any rights granted to You by Article 2 of this
   Licence, such as the use of the Work, the creation by You of a Derivative
   Work or the Distribution and/or Communication by You of the Work or copies
   thereof.

   (( 11. ))??
   Information to the public

   In case of any Distribution and/or Communication of the Work by means of
   electronic communication by You (for example, by offering to download the
   Work from a remote location) the distribution channel or media (for example,
   a website) must at least provide to the public the information requested by
   the applicable law regarding the identification and address of the Licensor,
   the Licence and the way it may be accessible, concluded, stored and
   reproduced by the Licensee.

   (( 12. ))??
   Termination of the Licence

   The Licence and the rights granted hereunder will terminate automatically
   upon any breach by the Licensee of the terms of the Licence.

   Such a termination will not terminate the licences of any person who has
   received the Work from the Licensee under the Licence, provided such persons
   remain in full compliance with the Licence.

   (( 13. ))??
   Miscellaneous

   Without prejudice of Article 9 above, the Licence represents the complete
   agreement between the Parties as to the Work licensed hereunder.

   If any provision of the Licence is invalid or unenforceable under applicable
   law, this will not affect the validity or enforceability of the Licence as a
   whole. Such provision will be construed and/or reformed so as necessary to
   make it valid and enforceable.

   The European Commission may put into force translations and/or binding new
   versions of this Licence, so far this is required and reasonable. New
   versions of the Licence will be published with a unique version number. The
   new version of the Licence becomes binding for You as soon as You become
   aware of its publication.

   (( 14. ))??
   Jurisdiction

   Any litigation resulting from the interpretation of this License, arising
   between the European Commission, as a Licensor, and any Licensee, will be
   subject to the jurisdiction of the Court of Justice of the European
   Communities, as laid down in article 238 of the Treaty establishing the
   European Community.

   Any litigation arising between Parties, other than the European Commission,
   and resulting from the interpretation of this License, will be subject to the
   exclusive jurisdiction of the competent court where the Licensor resides or
   conducts its primary business.

   (( 15. ))??
   Applicable Law

   This Licence shall be governed by the law of the European Union country where
   the Licensor resides or has his registered office.

   This licence shall be governed by the Belgian law if:

      −
      a litigation arises between the European Commission, as a Licensor, and
      any Licensee;

      −
      the Licensor, other than the European Commission, has no residence or
      registered office inside a European Union country.

Appendix

"Compatible Licences" according to article 5 EUPL are:

   −
   General Public License (GPL) v. 2

   −
   Open Software License (OSL) v. 2.1, v. 3.0

   −
   Common Public License v. 1.0

   −
   Eclipse Public License v. 1.0

   −
   Cecill v. 2.0
`
const license_EUPL_1_1_lre = `//**
European Union Public License 1.1
https://spdx.org/licenses/EUPL-1.1.json
https://joinup.ec.europa.eu/software/page/eupl/licence-eupl
https://joinup.ec.europa.eu/sites/default/files/custom-page/attachment/eupl1.1.-licence-en_0.pdf
https://opensource.org/licenses/EUPL-1.1
**//

(( European Union Public Licence V. 1.1
(( EUPL ))??
(( Copyright __20__ ))??
))??


This European Union Public Licence (the "EUPL") applies to the Work or Software
(as defined below) which is provided under the terms of this Licence. Any use of
the Work, other than as authorised under this Licence is prohibited (to the
extent such use is covered by a right of the copyright holder of the Work).

The Original Work is provided under the terms of this Licence when the Licensor
(as defined below) has placed the following notice immediately following the
copyright notice for the Original Work:

   Licensed under the EUPL V.1.1

   or has expressed by any other mean his willingness to license under the
   EUPL.

   (( 1. ))??
   Definitions

   In this Licence, the following terms have the following meaning:

      -
      The Licence: this Licence.

      -
      The Original Work or the Software: the software distributed and/or
      communicated by the Licensor under this Licence, available as Source Code
      and also as Executable Code as the case may be.

      -
      Derivative Works: the works or software that could be created by the
      Licensee, based upon the Original Work or modifications thereof. This
      Licence does not define the extent of modification or dependence on the
      Original Work required in order to classify a work as a Derivative Work;
      this extent is determined by copyright law applicable in the country
      mentioned in Article 15.

      -
      The Work: the Original Work and/or its Derivative Works.

      -
      The Source Code: the human-readable form of the Work which is the most
      convenient for people to study and modify.

      -
      The Executable Code: any code which has generally been compiled and which
      is meant to be interpreted by a computer as a program.

      -
      The Licensor: the natural or legal person that distributes and/or
      communicates the Work under the Licence.

      -
      Contributor(s): any natural or legal person who modifies the Work under
      the Licence, or otherwise contributes to the creation of a Derivative
      Work.

      -
      The Licensee or "You": any natural or legal person who makes any usage of
      the Software under the terms of the Licence.

      -
      Distribution and/or Communication: any act of selling, giving, lending,
      renting, distributing, communicating, transmitting, or otherwise making
      available, on-line or off-line, copies of the Work or providing access to
      its essential functionalities at the disposal of any other natural or
      legal person.

   (( 2. ))??
   Scope of the rights granted by the Licence

   The Licensor hereby grants You a world-wide, royalty-free, non-exclusive,
   sublicensable licence to do the following, for the duration of copyright
   vested in the Original Work:

      -
      use the Work in any circumstance and for all usage,

      -
      reproduce the Work,

      -
      modify the Original Work, and make Derivative Works based upon the Work,

      -
      communicate to the public, including the right to make available or
      display the Work or copies thereof to the public and perform publicly, as
      the case may be, the Work,

      -
      distribute the Work or copies thereof,

      -
      lend and rent the Work or copies thereof,

      -
      sub-license rights in the Work or copies thereof.

   Those rights can be exercised on any media, supports and formats, whether now
   known or later invented, as far as the applicable law permits so.

   In the countries where moral rights apply, the Licensor waives his right to
   exercise his moral right to the extent allowed by law in order to make
   effective the licence of the economic rights here above listed.

   The Licensor grants to the Licensee royalty-free, non exclusive usage rights
   to any patents held by the Licensor, to the extent necessary to make use of
   the rights granted on the Work under this Licence.

   (( 3. ))??
   Communication of the Source Code

   The Licensor may provide the Work either in its Source Code form, or as
   Executable Code. If the Work is provided as Executable Code, the Licensor
   provides in addition a machine-readable copy of the Source Code of the Work
   along with each copy of the Work that the Licensor distributes or indicates,
   in a notice following the copyright notice attached to the Work, a repository
   where the Source Code is easily and freely accessible for as long as the
   Licensor continues to distribute and/or communicate the Work.

   (( 4. ))??
   Limitations on copyright

   Nothing in this Licence is intended to deprive the Licensee of the benefits
   from any exception or limitation to the exclusive rights of the rights owners
   in the Original Work or Software, of the exhaustion of those rights or of
   other applicable limitations thereto.

   (( 5. ))??
   Obligations of the Licensee

   The grant of the rights mentioned above is subject to some restrictions and
   obligations imposed on the Licensee. Those obligations are the following:

   Attribution right: the Licensee shall keep intact all copyright, patent or
   trademarks notices and all notices that refer to the Licence and to the
   disclaimer of warranties. The Licensee must include a copy of such notices
   and a copy of the Licence with every copy of the Work he/she distributes
   and/or communicates. The Licensee must cause any Derivative Work to carry
   prominent notices stating that the Work has been modified and the date of
   modification.

   Copyleft clause: If the Licensee distributes and/or communicates copies of
   the Original Works or Derivative Works based upon the Original Work, this
   Distribution and/or Communication will be done under the terms of this
   Licence or of a later version of this Licence unless the Original Work is
   expressly distributed only under this version of the Licence. The Licensee
   (becoming Licensor) cannot offer or impose any additional terms or conditions
   on the Work or Derivative Work that alter or restrict the terms of the
   Licence.

   Compatibility clause: If the Licensee Distributes and/or Communicates
   Derivative Works or copies thereof based upon both the Original Work and
   another work licensed under a Compatible Licence, this Distribution and/or
   Communication can be done under the terms of this Compatible Licence. For the
   sake of this clause, "Compatible Licence," refers to the licences listed in
   the appendix attached to this Licence. Should the Licensee's obligations
   under the Compatible Licence conflict with his/her obligations under this
   Licence, the obligations of the Compatible Licence shall prevail.

   Provision of Source Code: When distributing and/or communicating copies of
   the Work, the Licensee will provide a machine-readable copy of the Source
   Code or indicate a repository where this Source will be easily and freely
   available for as long as the Licensee continues to distribute and/or
   communicate the Work.

   Legal Protection: This Licence does not grant permission to use the trade
   names, trademarks, service marks, or names of the Licensor, except as
   required for reasonable and customary use in describing the origin of the
   Work and reproducing the content of the copyright notice.

   (( 6. ))??
   Chain of Authorship

   The original Licensor warrants that the copyright in the Original Work
   granted hereunder is owned by him/her or licensed to him/her and that he/she
   has the power and authority to grant the Licence.

   Each Contributor warrants that the copyright in the modifications he/she
   brings to the Work are owned by him/her or licensed to him/her and that
   he/she has the power and authority to grant the Licence.

   Each time You accept the Licence, the original Licensor and subsequent
   Contributors grant You a licence to their contributions to the Work, under
   the terms of this Licence.

   (( 7. ))??
   Disclaimer of Warranty

   The Work is a work in progress, which is continuously improved by numerous
   contributors. It is not a finished work and may therefore contain defects or
   "bugs" inherent to this type of software development.

   For the above reason, the Work is provided under the Licence on an "as is"
   basis and without warranties of any kind concerning the Work, including
   without limitation merchantability, fitness for a particular purpose, absence
   of defects or errors, accuracy, non-infringement of intellectual property
   rights other than copyright as stated in Article 6 of this Licence.

   This disclaimer of warranty is an essential part of the Licence and a
   condition for the grant of any rights to the Work.

   (( 8. ))??
   Disclaimer of Liability

   Except in the cases of wilful misconduct or damages directly caused to
   natural persons, the Licensor will in no event be liable for any direct or
   indirect, material or moral, damages of any kind, arising out of the Licence
   or of the use of the Work, including without limitation, damages for loss of
   goodwill, work stoppage, computer failure or malfunction, loss of data or any
   commercial damage, even if the Licensor has been advised of the possibility
   of such damage. However, the Licensor will be liable under statutory product
   liability laws as far such laws apply to the Work.

   (( 9. ))??
   Additional agreements

   While distributing the Original Work or Derivative Works, You may choose to
   conclude an additional agreement to offer, and charge a fee for, acceptance
   of support, warranty, indemnity, or other liability obligations and/or
   services consistent with this Licence. However, in accepting such
   obligations, You may act only on your own behalf and on your sole
   responsibility, not on behalf of the original Licensor or any other
   Contributor, and only if You agree to indemnify, defend, and hold each
   Contributor harmless for any liability incurred by, or claims asserted
   against such Contributor by the fact You have accepted any such warranty or
   additional liability.

   (( 10. ))??
   Acceptance of the Licence

   The provisions of this Licence can be accepted by clicking on an icon "I
   agree" placed under the bottom of a window displaying the text of this
   Licence or by affirming consent in any other similar way, in accordance with
   the rules of applicable law. Clicking on that icon indicates your clear and
   irrevocable acceptance of this Licence and all of its terms and conditions.

   Similarly, you irrevocably accept this Licence and all of its terms and
   conditions by exercising any rights granted to You by Article 2 of this
   Licence, such as the use of the Work, the creation by You of a Derivative
   Work or the Distribution and/or Communication by You of the Work or copies
   thereof.

   (( 11. ))??
   Information to the public

   In case of any Distribution and/or Communication of the Work by means of
   electronic communication by You (for example, by offering to download the
   Work from a remote location) the distribution channel or media (for example,
   a website) must at least provide to the public the information requested by
   the applicable law regarding the Licensor, the Licence and the way it may be
   accessible, concluded, stored and reproduced by the Licensee.

   (( 12. ))??
   Termination of the Licence

   The Licence and the rights granted hereunder will terminate automatically
   upon any breach by the Licensee of the terms of the Licence. Such a
   termination will not terminate the licences of any person who has received
   the Work from the Licensee under the Licence, provided such persons remain in
   full compliance with the Licence.

   (( 13. ))??
   Miscellaneous

   Without prejudice of Article 9 above, the Licence represents the complete
   agreement between the Parties as to the Work licensed hereunder.

   If any provision of the Licence is invalid or unenforceable under applicable
   law, this will not affect the validity or enforceability of the Licence as a
   whole. Such provision will be construed and/or reformed so as necessary to
   make it valid and enforceable.

   The European Commission may publish other linguistic versions and/or new
   versions of this Licence, so far this is required and reasonable, without
   reducing the scope of the rights granted by the Licence. New versions of the
   Licence will be published with a unique version number.

   All linguistic versions of this Licence, approved by the European Commission,
   have identical value. Parties can take advantage of the linguistic version of
   their choice.

   (( 14. ))??
   Jurisdiction

   Any litigation resulting from the interpretation of this License, arising
   between the European Commission, as a Licensor, and any Licensee, will be
   subject to the jurisdiction of the Court of Justice of the European
   Communities, as laid down in article 238 of the Treaty establishing the
   European Community.

   Any litigation arising between Parties, other than the European Commission,
   and resulting from the interpretation of this License, will be subject to the
   exclusive jurisdiction of the competent court where the Licensor resides or
   conducts its primary business.

   (( 15. ))??
   Applicable Law

   This Licence shall be governed by the law of the European Union country where
   the Licensor resides or has his registered office.

   This licence shall be governed by the Belgian law if:

      -
      a litigation arises between the European Commission, as a Licensor, and
      any Licensee;

      -
      the Licensor, other than the European Commission, has no residence or
      registered office inside a European Union country.

Appendix

"Compatible Licences" according to article 5 EUPL are:

   -
   GNU General Public License (GNU GPL) v. 2

   -
   Open Software License (OSL) v. 2.1, v. 3.0

   -
   Common Public License v. 1.0

   -
   Eclipse Public License v. 1.0

   -
   Cecill v. 2.0
`
const license_EUPL_1_2_lre = `//**
European Union Public License 1.2
https://spdx.org/licenses/EUPL-1.2.json
https://joinup.ec.europa.eu/page/eupl-text-11-12
https://joinup.ec.europa.eu/sites/default/files/custom-page/attachment/eupl_v1.2_en.pdf
https://joinup.ec.europa.eu/sites/default/files/inline-files/EUPL%20v1_2%20EN(1).txt
http://eur-lex.europa.eu/legal-content/EN/TXT/HTML/?uri=CELEX:32017D0863
https://opensource.org/licenses/EUPL-1.2
**//

(( European Union Public Licence v. 1.2 ))??

EUPL © the European Union 2007, 2016

This European Union Public Licence (the 'EUPL') applies to the Work (as defined
below) which is provided under the terms of this Licence. Any use of the Work,
other than as authorised under this Licence is prohibited (to the extent such
use is covered by a right of the copyright holder of the Work).

The Work is provided under the terms of this Licence when the Licensor (as
defined below) has placed the following notice immediately following the
copyright notice for the Work:

   Licensed under the EUPL

   or has expressed by any other means his willingness to license under the
   EUPL.

   (( 1. ))??
   Definitions

   In this Licence, the following terms have the following meaning:

      —
      'The Licence': this Licence.

      —
      'The Original Work': the work or software distributed or communicated by
      the Licensor under this Licence, available as Source Code and also as
      Executable Code as the case may be.

      —
      'Derivative Works': the works or software that could be created by the
      Licensee, based upon the Original Work or modifications thereof. This
      Licence does not define the extent of modification or dependence on the
      Original Work required in order to classify a work as a Derivative Work;
      this extent is determined by copyright law applicable in the country
      mentioned in Article 15.

      —
      'The Work': the Original Work or its Derivative Works.

      —
      'The Source Code': the human-readable form of the Work which is the most
      convenient for people to study and modify.

      —
      'The Executable Code': any code which has generally been compiled and
      which is meant to be interpreted by a computer as a program.

      —
      'The Licensor': the natural or legal person that distributes or
      communicates the Work under the Licence.

      —
      'Contributor(s)': any natural or legal person who modifies the Work under
      the Licence, or otherwise contributes to the creation of a Derivative
      Work.

      —
      'The Licensee' or 'You': any natural or legal person who makes any usage
      of the Work under the terms of the Licence.

      —
      'Distribution' or 'Communication': any act of selling, giving, lending,
      renting, distributing, communicating, transmitting, or otherwise making
      available, online or offline, copies of the Work or providing access to
      its essential functionalities at the disposal of any other natural or
      legal person.

   (( 2. ))??
   Scope of the rights granted by the Licence

   The Licensor hereby grants You a worldwide, royalty-free, non-exclusive,
   sublicensable licence to do the following, for the duration of copyright
   vested in the Original Work:

      —
      use the Work in any circumstance and for all usage,

      —
      reproduce the Work,

      —
      modify the Work, and make Derivative Works based upon the Work,

      —
      communicate to the public, including the right to make available or
      display the Work or copies thereof to the public and perform publicly, as
      the case may be, the Work,

      —
      distribute the Work or copies thereof,

      —
      lend and rent the Work or copies thereof,

      —
      sublicense rights in the Work or copies thereof.

   Those rights can be exercised on any media, supports and formats, whether now
   known or later invented, as far as the applicable law permits so.

   In the countries where moral rights apply, the Licensor waives his right to
   exercise his moral right to the extent allowed by law in order to make
   effective the licence of the economic rights here above listed.

   The Licensor grants to the Licensee royalty-free, non-exclusive usage rights
   to any patents held by the Licensor, to the extent necessary to make use of
   the rights granted on the Work under this Licence.

   (( 3. ))??
   Communication of the Source Code

   The Licensor may provide the Work either in its Source Code form, or as
   Executable Code. If the Work is provided as Executable Code, the Licensor
   provides in addition a machine-readable copy of the Source Code of the Work
   along with each copy of the Work that the Licensor distributes or indicates,
   in a notice following the copyright notice attached to the Work, a repository
   where the Source Code is easily and freely accessible for as long as the
   Licensor continues to distribute or communicate the Work.

   (( 4. ))??
   Limitations on copyright

   Nothing in this Licence is intended to deprive the Licensee of the benefits
   from any exception or limitation to the exclusive rights of the rights owners
   in the Work, of the exhaustion of those rights or of other applicable
   limitations thereto.

   (( 5. ))??
   Obligations of the Licensee

   The grant of the rights mentioned above is subject to some restrictions and
   obligations imposed on the Licensee. Those obligations are the following:

   Attribution right: The Licensee shall keep intact all copyright, patent or
   trademarks notices and all notices that refer to the Licence and to the
   disclaimer of warranties. The Licensee must include a copy of such notices
   and a copy of the Licence with every copy of the Work he/she distributes or
   communicates. The Licensee must cause any Derivative Work to carry prominent
   notices stating that the Work has been modified and the date of
   modification.

   Copyleft clause: If the Licensee distributes or communicates copies of the
   Original Works or Derivative Works, this Distribution or Communication will
   be done under the terms of this Licence or of a later version of this Licence
   unless the Original Work is expressly distributed only under this version of
   the Licence — for example by communicating 'EUPL v. 1.2 only'. The Licensee
   (becoming Licensor) cannot offer or impose any additional terms or conditions
   on the Work or Derivative Work that alter or restrict the terms of the
   Licence.

   Compatibility clause: If the Licensee Distributes or Communicates Derivative
   Works or copies thereof based upon both the Work and another work licensed
   under a Compatible Licence, this Distribution or Communication can be done
   under the terms of this Compatible Licence. For the sake of this clause,
   'Compatible Licence' refers to the licences listed in the appendix attached
   to this Licence. Should the Licensee's obligations under the Compatible
   Licence conflict with his/her obligations under this Licence, the obligations
   of the Compatible Licence shall prevail.

   Provision of Source Code: When distributing or communicating copies of the
   Work, the Licensee will provide a machine-readable copy of the Source Code or
   indicate a repository where this Source will be easily and freely available
   for as long as the Licensee continues to distribute or communicate the Work.

   Legal Protection: This Licence does not grant permission to use the trade
   names, trademarks, service marks, or names of the Licensor, except as
   required for reasonable and customary use in describing the origin of the
   Work and reproducing the content of the copyright notice.

   (( 6. ))??
   Chain of Authorship

   The original Licensor warrants that the copyright in the Original Work
   granted hereunder is owned by him/her or licensed to him/her and that he/she
   has the power and authority to grant the Licence.

   Each Contributor warrants that the copyright in the modifications he/she
   brings to the Work are owned by him/her or licensed to him/her and that
   he/she has the power and authority to grant the Licence.

   Each time You accept the Licence, the original Licensor and subsequent
   Contributors grant You a licence to their contributions to the Work, under
   the terms of this Licence.

   (( 7. ))??
   Disclaimer of Warranty

   The Work is a work in progress, which is continuously improved by numerous
   Contributors. It is not a finished work and may therefore contain defects or
   'bugs' inherent to this type of development.

   For the above reason, the Work is provided under the Licence on an 'as is'
   basis and without warranties of any kind concerning the Work, including
   without limitation merchantability, fitness for a particular purpose, absence
   of defects or errors, accuracy, non-infringement of intellectual property
   rights other than copyright as stated in Article 6 of this Licence.

   This disclaimer of warranty is an essential part of the Licence and a
   condition for the grant of any rights to the Work.

   (( 8. ))??
   Disclaimer of Liability

   Except in the cases of wilful misconduct or damages directly caused to
   natural persons, the Licensor will in no event be liable for any direct or
   indirect, material or moral, damages of any kind, arising out of the Licence
   or of the use of the Work, including without limitation, damages for loss of
   goodwill, work stoppage, computer failure or malfunction, loss of data or any
   commercial damage, even if the Licensor has been advised of the possibility
   of such damage. However, the Licensor will be liable under statutory product
   liability laws as far such laws apply to the Work.

   (( 9. ))??
   Additional agreements

   While distributing the Work, You may choose to conclude an additional
   agreement, defining obligations or services consistent with this Licence.
   However, if accepting obligations, You may act only on your own behalf and on
   your sole responsibility, not on behalf of the original Licensor or any other
   Contributor, and only if You agree to indemnify, defend, and hold each
   Contributor harmless for any liability incurred by, or claims asserted
   against such Contributor by the fact You have accepted any warranty or
   additional liability.

   (( 10. ))??
   Acceptance of the Licence

   The provisions of this Licence can be accepted by clicking on an icon 'I
   agree' placed under the bottom of a window displaying the text of this
   Licence or by affirming consent in any other similar way, in accordance with
   the rules of applicable law. Clicking on that icon indicates your clear and
   irrevocable acceptance of this Licence and all of its terms and conditions.

   Similarly, you irrevocably accept this Licence and all of its terms and
   conditions by exercising any rights granted to You by Article 2 of this
   Licence, such as the use of the Work, the creation by You of a Derivative
   Work or the Distribution or Communication by You of the Work or copies
   thereof.

   (( 11. ))??
   Information to the public

   In case of any Distribution or Communication of the Work by means of
   electronic communication by You (for example, by offering to download the
   Work from a remote location) the distribution channel or media (for example,
   a website) must at least provide to the public the information requested by
   the applicable law regarding the Licensor, the Licence and the way it may be
   accessible, concluded, stored and reproduced by the Licensee.

   (( 12. ))??
   Termination of the Licence

   The Licence and the rights granted hereunder will terminate automatically
   upon any breach by the Licensee of the terms of the Licence.

   Such a termination will not terminate the licences of any person who has
   received the Work from the Licensee under the Licence, provided such persons
   remain in full compliance with the Licence.

   (( 13. ))??
   Miscellaneous

   Without prejudice of Article 9 above, the Licence represents the complete
   agreement between the Parties as to the Work.

   If any provision of the Licence is invalid or unenforceable under applicable
   law, this will not affect the validity or enforceability of the Licence as a
   whole. Such provision will be construed or reformed so as necessary to make
   it valid and enforceable.

   The European Commission may publish other linguistic versions or new versions
   of this Licence or updated versions of the Appendix, so far this is required
   and reasonable, without reducing the scope of the rights granted by the
   Licence. New versions of the Licence will be published with a unique version
   number.

   All linguistic versions of this Licence, approved by the European Commission,
   have identical value. Parties can take advantage of the linguistic version of
   their choice.

   (( 14. ))??
   Jurisdiction

   Without prejudice to specific agreement between parties,

      —
      any litigation resulting from the interpretation of this License, arising
      between the European Union institutions, bodies, offices or agencies, as a
      Licensor, and any Licensee, will be subject to the jurisdiction of the
      Court of Justice of the European Union, as laid down in article 272 of the
      Treaty on the Functioning of the European Union,

      —
      any litigation arising between other parties and resulting from the
      interpretation of this License, will be subject to the exclusive
      jurisdiction of the competent court where the Licensor resides or conducts
      its primary business.

   (( 15. ))??
   Applicable Law

   Without prejudice to specific agreement between parties,

      —
      this Licence shall be governed by the law of the European Union Member
      State where the Licensor has his seat, resides or has his registered
      office,

      —
      this licence shall be governed by Belgian law if the Licensor has no seat,
      residence or registered office inside a European Union Member State.

Appendix

'Compatible Licences' according to Article 5 EUPL are:

   —
   GNU General Public License (GPL) v. 2, v. 3

   —
   GNU Affero General Public License (AGPL) v. 3

   —
   Open Software License (OSL) v. 2.1, v. 3.0

   —
   Eclipse Public License (EPL) v. 1.0

   —
   CeCILL v. 2.0, v. 2.1

   —
   Mozilla Public Licence (MPL) v. 2

   —
   GNU Lesser General Public Licence (LGPL) v. 2.1, v. 3

   —
   Creative Commons Attribution-ShareAlike v. 3.0 Unported (CC BY-SA 3.0) for
   works other than software

   —
   European Union Public Licence (EUPL) v. 1.1, v. 1.2

   —
   Québec Free and Open-Source Licence — Reciprocity (LiLiQ-R) or Strong
   Reciprocity (LiLiQ-R+).

The European Commission may update this Appendix to later versions of the above
licences without producing a new version of the EUPL, as long as they provide
the rights granted in Article 2 of this Licence and protect the covered Source
Code from exclusive appropriation.

All other changes or additions to this Appendix require the production of a new
EUPL version.
`
const license_Entessa_lre = `//**
Entessa Public License v1.0
https://spdx.org/licenses/Entessa.json
https://opensource.org/licenses/Entessa
**//

(( Entessa Public License Version. 1.0
(( Copyright __20__ ))??
))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The end-user documentation included with the redistribution, if any, must
   include the following acknowledgment:

   "This product includes open source software developed by openSEAL
   (http:/www.openseal.org/)."

   Alternately, this acknowledgment may appear in the software itself, if and
   wherever such third-party acknowledgments normally appear.

   (( 4. ))??
   The names "openSEAL" and "Entessa" must not be used to endorse or promote
   products derived from this software without prior written permission. For
   written permission, please contact epl@entessa.com.

   (( 5. ))??
   Products derived from this software may not be called "openSEAL", nor may
   "openSEAL" appear in their name, without prior written permission of
   Entessa.

THIS SOFTWARE IS PROVIDED ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ENTESSA, LLC,
OPENSEAL OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

This software consists of voluntary contributions made by many individuals on
behalf of openSEAL and was originally based on software contributed by Entessa,
LLC, http:/www.entessa.com. For more information on the openSEAL, please see
<http:/www.openseal.org/>.
`
const license_ErlPL_1_1_lre = `//**
Erlang Public License v1.1
https://spdx.org/licenses/ErlPL-1.1.json
http://www.erlang.org/EPLICENSE
**//

(( ERLANG PUBLIC LICENSE Version 1.1 ))??

   (( 1. ))??
   Definitions.

      (( 1.1. ))??
      ` + "`" + `` + "`" + `Contributor'' means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      ` + "`" + `` + "`" + `Contributor Version'' means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      ` + "`" + `` + "`" + `Covered Code'' means the Original Code or Modifications or the
      combination of the Original Code and Modifications, in each case including
      portions thereof.

      (( 1.4. ))??
      ` + "`" + `` + "`" + `Electronic Distribution Mechanism'' means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      ` + "`" + `` + "`" + `Executable'' means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      ` + "`" + `` + "`" + `Initial Developer'' means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7. ))??
      ` + "`" + `` + "`" + `Larger Work'' means a work which combines Covered Code or portions
      thereof with code not governed by the terms of this License.

      (( 1.8. ))??
      ` + "`" + `` + "`" + `License'' means this document.

      (( 1.9. ))??
      ` + "`" + `` + "`" + `Modifications'' means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10. ))??
      ` + "`" + `` + "`" + `Original Code'' means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      (( 1.11. ))??
      ` + "`" + `` + "`" + `Source Code'' means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or a list of source code differential
      comparisons against either the Original Code or another well known,
      available Covered Code of the Contributor's choice. The Source Code can be
      in a compressed or archival form, provided the appropriate decompression
      or de-archiving software is widely available for no charge.

      (( 1.12. ))??
      ` + "`" + `` + "`" + `You'' means an individual or a legal entity exercising rights under, and
      complying with all of the terms of, this License. For legal entities,
      ` + "`" + `` + "`" + `You'' includes any entity which controls, is controlled by, or is under
      common control with You. For purposes of this definition, ` + "`" + `` + "`" + `control''
      means (a) the power, direct or indirect, to cause the direction or
      management of such entity, whether by contract or otherwise, or (b)
      ownership of fifty percent (50%) or more of the outstanding shares or
      beneficial ownership of such entity.

   (( 2. ))??
   Source Code License.

      (( 2.1. ))??
      The Initial Developer Grant. The Initial Developer hereby grants You a
      world-wide, royalty-free, non-exclusive license, subject to third party
      intellectual property claims:

         (( (a) ))??
         to use, reproduce, modify, display, perform, sublicense and distribute
         the Original Code (or portions thereof) with or without Modifications,
         or as part of a Larger Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Initial
         Developer, to make, have made, use and sell (` + "`" + `` + "`" + `Utilize'') the Original
         Code (or portions thereof), but solely to the extent that any such
         patent is reasonably necessary to enable You to Utilize the Original
         Code (or portions thereof) and not to any greater extent that may be
         necessary to Utilize further Modifications or combinations.

      (( 2.2. ))??
      Contributor Grant. Each Contributor hereby grants You a world-wide,
      royalty-free, non-exclusive license, subject to third party intellectual
      property claims:

         (( (a) ))??
         to use, reproduce, modify, display, perform, sublicense and distribute
         the Modifications created by such Contributor (or portions thereof)
         either on an unmodified basis, with other Modifications, as Covered
         Code or as part of a Larger Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Contributor, to
         Utilize the Contributor Version (or portions thereof), but solely to
         the extent that any such patent is reasonably necessary to enable You
         to Utilize the Contributor Version (or portions thereof), and not to
         any greater extent that may be necessary to Utilize further
         Modifications or combinations.

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Application of License. The Modifications which You contribute are
      governed by the terms of this License, including without limitation
      Section 2.2. The Source Code version of Covered Code may be distributed
      only under the terms of this License, and You must include a copy of this
      License with every copy of the Source Code You distribute. You may not
      offer or impose any terms on any Source Code version that alters or
      restricts the applicable version of this License or the recipients' rights
      hereunder. However, You may include an additional document offering the
      additional rights described in Section 3.5.

      (( 3.2. ))??
      Availability of Source Code. Any Modification which You contribute must be
      made available in Source Code form under the terms of this License either
      on the same media as an Executable version or via an accepted Electronic
      Distribution Mechanism to anyone to whom you made an Executable version
      available; and if made available via Electronic Distribution Mechanism,
      must remain available for at least twelve (12) months after the date it
      initially became available, or at least six (6) months after a subsequent
      version of that particular Modification has been made available to such
      recipients. You are responsible for ensuring that the Source Code version
      remains available even if the Electronic Distribution Mechanism is
      maintained by a third party.

      (( 3.3. ))??
      Description of Modifications. You must cause all Covered Code to which you
      contribute to contain a file documenting the changes You made to create
      that Covered Code and the date of any change. You must include a prominent
      statement that the Modification is derived, directly or indirectly, from
      Original Code provided by the Initial Developer and including the name of
      the Initial Developer in (a) the Source Code, and (b) in any notice in an
      Executable version or related documentation in which You describe the
      origin or ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters

         (( (a) ))??
         Third Party Claims. If You have knowledge that a party claims an
         intellectual property right in particular functionality or code (or its
         utilization under this License), you must include a text file with the
         source code distribution titled ` + "`" + `` + "`" + `LEGAL'' which describes the claim and
         the party making the claim in sufficient detail that a recipient will
         know whom to contact. If you obtain such knowledge after You make Your
         Modification available as described in Section 3.2, You shall promptly
         modify the LEGAL file in all copies You make available thereafter and
         shall take other steps (such as notifying appropriate mailing lists or
         newsgroups) reasonably calculated to inform those who received the
         Covered Code that new knowledge has been obtained.

         (( (b) ))??
         Contributor APIs. If Your Modification is an application programming
         interface and You own or control patents which are reasonably necessary
         to implement that API, you must also include this information in the
         LEGAL file.

      (( 3.5. ))??
      Required Notices. You must duplicate the notice in Exhibit A in each file
      of the Source Code, and this License in any documentation for the Source
      Code, where You describe recipients' rights relating to Covered Code. If
      You created one or more Modification(s), You may add your name as a
      Contributor to the notice described in Exhibit A. If it is not possible to
      put such notice in a particular Source Code file due to its structure,
      then you must include such notice in a location (such as a relevant
      directory file) where a user would be likely to look for such a notice.
      You may choose to offer, and to charge a fee for, warranty, support,
      indemnity or liability obligations to one or more recipients of Covered
      Code. However, You may do so only on Your own behalf, and not on behalf of
      the Initial Developer or any Contributor. You must make it absolutely
      clear than any such warranty, support, indemnity or liability obligation
      is offered by You alone, and You hereby agree to indemnify the Initial
      Developer and every Contributor for any liability incurred by the Initial
      Developer or such Contributor as a result of warranty, support, indemnity
      or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Versions. You may distribute Covered Code in
      Executable form only if the requirements of Section 3.1-3.5 have been met
      for that Covered Code, and if You include a notice stating that the Source
      Code version of the Covered Code is available under the terms of this
      License, including a description of how and where You have fulfilled the
      obligations of Section 3.2. The notice must be conspicuously included in
      any notice in an Executable version, related documentation or collateral
      in which You describe recipients' rights relating to the Covered Code. You
      may distribute the Executable version of Covered Code under a license of
      Your choice, which may contain terms different from this License, provided
      that You are in compliance with the terms of this License and that the
      license for the Executable version does not attempt to limit or alter the
      recipient's rights in the Source Code version from the rights set forth in
      this License. If You distribute the Executable version under a different
      license You must make it absolutely clear that any terms which differ from
      this License are offered by You alone, not by the Initial Developer or any
      Contributor. You hereby agree to indemnify the Initial Developer and every
      Contributor for any liability incurred by the Initial Developer or such
      Contributor as a result of any such terms You offer.

      (( 3.7. ))??
      Larger Works. You may create a Larger Work by combining Covered Code with
      other code not governed by the terms of this License and distribute the
      Larger Work as a single product. In such a case, You must make sure the
      requirements of this License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation. If it is impossible for You
   to comply with any of the terms of this License with respect to some or all
   of the Covered Code due to statute or regulation then You must: (a) comply
   with the terms of this License to the maximum extent possible; and (b)
   describe the limitations and the code they affect. Such description must be
   included in the LEGAL file described in Section 3.4 and must be included with
   all distributions of the Source Code. Except to the extent prohibited by
   statute or regulation, such description must be sufficiently detailed for a
   recipient of ordinary skill to be able to understand it.

   (( 5. ))??
   Application of this License.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A, and to related Covered Code.

   (( 6. ))??
   CONNECTION TO MOZILLA PUBLIC LICENSE

   This Erlang License is a derivative work of the Mozilla Public License,
   Version 1.0. It contains terms which differ from the Mozilla Public License,
   Version 1.0.

   (( 7. ))??
   DISCLAIMER OF WARRANTY.

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN ` + "`" + `` + "`" + `AS IS'' BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER
   OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR
   OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF
   THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

   (( 8. ))??
   TERMINATION.

   This License and the rights granted hereunder will terminate automatically if
   You fail to comply with terms herein and fail to cure such breach within 30
   days of becoming aware of the breach. All sublicenses to the Covered Code
   which are properly granted shall survive any termination of this License.
   Provisions which, by their nature, must remain in effect beyond the
   termination of this License shall survive.

   (( 9. ))??
   DISCLAIMER OF LIABILITY

   Any utilization of Covered Code shall not cause the Initial Developer or any
   Contributor to be liable for any damages (neither direct nor indirect).

   (( 10. ))??
   MISCELLANEOUS

   This License represents the complete agreement concerning the subject matter
   hereof. If any provision is held to be unenforceable, such provision shall be
   reformed only to the extent necessary to make it enforceable. This License
   shall be construed by and in accordance with the substantive laws of Sweden.
   Any dispute, controversy or claim arising out of or relating to this License,
   or the breach, termination or invalidity thereof, shall be subject to the
   exclusive jurisdiction of Swedish courts, with the Stockholm City Court as
   the first instance.

EXHIBIT A.

` + "`" + `` + "`" + `The contents of this file are subject to the Erlang Public License, Version
1.1, (the "License"); you may not use this file except in compliance with the
License. You should have received a copy of the Erlang Public License along with
this software. If not, it can be retrieved via the world wide web at
http:/www.erlang.org/.

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

The Initial Developer of the Original Code is Ericsson Utvecklings AB. Portions
created by Ericsson are Copyright 1999, Ericsson Utvecklings AB. All Rights
Reserved.''
`
const license_Eurosym_lre = `//**
Eurosym License
https://spdx.org/licenses/Eurosym.json
https://fedoraproject.org/wiki/Licensing/Eurosym
**//

//** Copyright **//

(( Licence Version 2 ))??

This software is provided 'as-is', without warranty of any kind, express or
implied. In no event will the authors or copyright holders be held liable for
any damages arising from the use of this software.

Permission is granted to anyone to use this software for any purpose, including
commercial applications, and to alter it and redistribute it freely, subject to
the following restrictions:

   (( 1. ))??
   The origin of this software must not be misrepresented; you must not claim
   that you wrote the original software. If you use this software in a product,
   an acknowledgment in the product documentation would be appreciated.

   (( 2. ))??
   Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   (( 3. ))??
   You must not use any of the names of the authors or copyright holders of the
   original software for advertising or publicity pertaining to distribution
   without specific, written prior permission.

   (( 4. ))??
   If you change this software and redistribute parts or all of it in any form,
   you must make the source code of the altered version of this software
   available.

   (( 5. ))??
   This notice may not be removed or altered from any source distribution.

This licence is governed by the Laws of Germany. Disputes shall be settled by
Saarbruecken City Court.
`
const license_FSFAP_lre = `//**
FSF All Permissive License
https://spdx.org/licenses/FSFAP.json
https://www.gnu.org/prep/maintain/html_node/License-Notices-for-Other-Files.html
**//

Copying and distribution of this file, with or without modification, are
permitted in any medium without royalty provided the copyright notice and this
notice are preserved. This file is offered as-is, without any warranty.
`
const license_FSFUL_lre = `//**
FSF Unlimited License
https://spdx.org/licenses/FSFUL.json
https://fedoraproject.org/wiki/Licensing/FSF_Unlimited_License
**//

//** Copyright **//

This configure script is free software; the Free Software Foundation gives
unlimited permission to copy, distribute and modify it.
`
const license_FSFULLR_lre = `//**
FSF Unlimited License (with License Retention)
https://spdx.org/licenses/FSFULLR.json
https://fedoraproject.org/wiki/Licensing/FSF_Unlimited_License#License_Retention_Variant
**//

//** Copyright **//

This file is free software; the Free Software Foundation gives unlimited
permission to copy and/or distribute it, with or without modifications, as long
as this notice is preserved.
`
const license_FTL_lre = `//**
Freetype Project License
https://spdx.org/licenses/FTL.json
http://freetype.fis.uniroma2.it/FTL.TXT
http://git.savannah.gnu.org/cgit/freetype/freetype2.git/tree/docs/FTL.TXT
**//

(( The FreeType Project LICENSE

2006-Jan-27 ))??

(( Copyright 1996-2002, 2006 by David Turner, Robert Wilhelm, and Werner
Lemberg

Introduction

The FreeType Project is distributed in several archive packages; some of them
may contain, in addition to the FreeType font engine, various tools and
contributions which rely on, or relate to, the FreeType Project.

This license applies to all files found in such packages, and which do not fall
under their own explicit license. The license affects thus the FreeType font
engine, the test programs, documentation and makefiles, at the very least.

This license was inspired by the BSD, Artistic, and IJG (Independent JPEG Group)
licenses, which all encourage inclusion and use of free software in commercial
and freeware products alike. As a consequence, its main points are that:

   (( o ))??
   We don't promise that this software works. However, we will be interested in
   any kind of bug reports. (` + "`" + `as is' distribution)

   (( o ))??
   You can use this software for whatever you want, in parts or full form,
   without having to pay us. (` + "`" + `royalty-free' usage)

   (( o ))??
   You may not pretend that you wrote this software. If you use it, or only
   parts of it, in a program, you must acknowledge somewhere in your
   documentation that you have used the FreeType code. (` + "`" + `credits')

We specifically permit and encourage the inclusion of this software, with or
without modifications, in commercial products. We disclaim all warranties
covering The FreeType Project and assume no liability related to The FreeType
Project.

Finally, many people asked us for a preferred form for a credit/disclaimer to
use in compliance with this license. We thus encourage you to use the following
text:

""" Portions of this software are copyright © <year> The FreeType Project
(www.freetype.org). All rights reserved. """

Please replace <year> with the value from the FreeType version you actually use. ))??

Legal Terms

   (( 0. ))??
   Definitions

   Throughout this license, the terms ` + "`" + `package', ` + "`" + `FreeType Project', and
   ` + "`" + `FreeType archive' refer to the set of files originally distributed by the
   authors (David Turner, Robert Wilhelm, and Werner Lemberg) as the ` + "`" + `FreeType
   Project', be they named as alpha, beta or final release.

   ` + "`" + `You' refers to the licensee, or person using the project, where ` + "`" + `using' is a
   generic term including compiling the project's source code as well as linking
   it to form a ` + "`" + `program' or ` + "`" + `executable'. This program is referred to as ` + "`" + `a
   program using the FreeType engine'.

   This license applies to all files distributed in the original FreeType
   Project, including all source code, binaries and documentation, unless
   otherwise stated in the file in its original, unmodified form as distributed
   in the original archive. If you are unsure whether or not a particular file
   is covered by this license, you must contact us to verify this.

   The FreeType Project is copyright (C) 1996-2000 by David Turner, Robert
   Wilhelm, and Werner Lemberg. All rights reserved except as specified below.

   (( 1. ))??
   No Warranty

   THE FREETYPE PROJECT IS PROVIDED ` + "`" + `AS IS' WITHOUT WARRANTY OF ANY KIND, EITHER
   EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL ANY OF
   THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY DAMAGES CAUSED BY THE USE
   OR THE INABILITY TO USE, OF THE FREETYPE PROJECT.

   (( 2. ))??
   Redistribution

   This license grants a worldwide, royalty-free, perpetual and irrevocable
   right and license to use, execute, perform, compile, display, copy, create
   derivative works of, distribute and sublicense the FreeType Project (in both
   source and object code forms) and derivative works thereof for any purpose;
   and to authorize others to exercise some or all of the rights granted herein,
   subject to the following conditions:

      (( o ))??
      Redistribution of source code must retain this license file (` + "`" + `FTL.TXT')
      unaltered; any additions, deletions or changes to the original files must
      be clearly indicated in accompanying documentation. The copyright notices
      of the unaltered, original files must be preserved in all copies of source
      files.

      (( o ))??
      Redistribution in binary form must provide a disclaimer that states that
      the software is based in part of the work of the FreeType Team, in the
      distribution documentation. We also encourage you to put an URL to the
      FreeType web page in your documentation, though this isn't mandatory.

   These conditions apply to any software derived from or based on the FreeType
   Project, not just the unmodified files. If you use our work, you must
   acknowledge us. However, no fee need be paid to us.

   (( 3. ))??
   Advertising

   Neither the FreeType authors and contributors nor you shall use the name of
   the other for commercial, advertising, or promotional purposes without
   specific prior written permission.

   We suggest, but do not require, that you use one or more of the following
   phrases to refer to this software in your documentation or advertising
   materials: ` + "`" + `FreeType Project', ` + "`" + `FreeType Engine', ` + "`" + `FreeType library', or
   ` + "`" + `FreeType Distribution'.

   As you have not signed this license, you are not required to accept it.
   However, as the FreeType Project is copyrighted material, only this license,
   or another one contracted with the authors, grants you the right to use,
   distribute, and modify it. Therefore, by using, distributing, or modifying
   the FreeType Project, you indicate that you understand and accept all the
   terms of this license.

   (( 4. ))??
   Contacts

   There are two mailing lists related to FreeType:

      (( o ))??
      freetype@nongnu.org

      Discusses general use and applications of FreeType, as well as future and
      wanted additions to the library and distribution. If you are looking for
      support, start in this list if you haven't found anything to help you in
      the documentation.

      (( o ))??
      freetype-devel@nongnu.org

      Discusses bugs, as well as engine internals, design issues, specific
      licenses, porting, etc.

   Our home page can be found at

   http:/www.freetype.org
   (( --- end of FTL.TXT --- ))??
`
const license_Fair_lre = `//**
Fair License
https://spdx.org/licenses/Fair.json
http://fairlicense.org/
https://opensource.org/licenses/Fair
**//

(( Fair License
(( Copyright __20__ ))??
))??

Usage of the works is permitted provided that this instrument is retained with
the works, so that any entity that uses the works is notified of this
instrument.

DISCLAIMER: THE WORKS ARE WITHOUT WARRANTY.
`
const license_Frameworx_1_0_lre = `//**
Frameworx Open License 1.0
https://spdx.org/licenses/Frameworx-1.0.json
https://opensource.org/licenses/Frameworx-1.0
**//

(( THE FRAMEWORX OPEN LICENSE 1.0 ))??

This License Agreement, The Frameworx Open License 1.0, has been entered into
between The Frameworx Company and you, the licensee hereunder, effective as of
Your acceptance of the Frameworx Code Base or an Downstream Distribution (each
as defined below).

AGREEMENT BACKGROUND

The Frameworx Company is committed to the belief that open source software
results in better quality, greater technical and product innovation in the
market place and a more empowered and productive developer and end-user
community. Our objective is to ensure that the Frameworx Code Base, and the
source code for improvements and innovations to it, remain free and open to the
community.To further these beliefs and objectives, we are distributing the
Frameworx Code Base, without royalties and in source code form, to the community
pursuant to this License Agreement.

AGREEMENT TERMS

The Frameworx Company and You have agreed as follows:

   (( 1. ))??
   Definitions.The following terms have the following respective meanings:

      (( (a) ))??
      Frameworx Code Base means the software developed by The Frameworx Company
      and made available under this License Agreement

      (( (b) ))??
      Downstream Distribution means any direct or indirect release, distribution
      or remote availability of software (i) that directly or indirectly
      contains, or depends for its intended functioning on, the Frameworx Code
      Base or any portion or element thereof and (ii) in which rights to use and
      distribute such Frameworx Code Base software depend, directly or
      indirectly, on the License provided in Section 2 below.

      (( c ))??
      "Source Code" to any software means the preferred form for making
      modifications to that software, including any associated documentation,
      interface definition files and compilation or installation scripts, or any
      version thereof that has been compressed or archived, and can be
      reconstituted, using an appropriate and generally available archival or
      compression technology.

      (( (d) ))??
      Value-Added Services means any commercial or fee-based software-related
      service, including without limitation: system or application development
      or consulting; technical or end-user support or training; distribution
      maintenance, configuration or versioning; or outsourced, hosted or
      network-based application services.

   (( 2. ))??
   License Grant. Subject to the terms and conditions hereof, The Frameworx
   Company hereby grants You a non-exclusive license (the License), subject to
   third party intellectual property claims, and for no fee other than a nominal
   charge reflecting the costs of physical distribution, to:

      (( (a) ))??
      use the Frameworx Code Base, in either Source Code or machine-readable
      form;

      (( (b) ))??
      make modifications, additions and deletions to the content or structure of
      the Frameworx Code Base; or

      (( c ))??
      create larger works or derivative works including the Frameworx Code Base
      or any portion or element thereof; and

      (( (d) ))??
      release, distribute or make available, either generally or to any specific
      third-party, any of the foregoing in Source Code or binary form.

   (( 3. ))??
   License Conditions. The grant of the License under Section 1 hereof, and your
   exercise of all rights in connection with this License Agreement, will remain
   subject to the following terms and conditions, as well as to the other
   provisions hereof:

      (( (a) ))??
      Complete Source Code for any Downstream Distribution directly or
      indirectly made by You that contains, or depends for its intended
      functionality on, the Frameworx Code Base, or any portion or element
      thereof, shall be made freely available to all users thereof on terms and
      conditions no more restrictive, and no less favorable for any user
      (including, without limitation, with regard to Source Code availability
      and royalty-free use) than those terms and conditions provided in this
      License Agreement.

      (( (b) ))??
      Any Value-Added Services that you offer or provide, directly or
      indirectly, in relation to any Downstream Distribution shall be offered
      and provided on commercial terms that are reasonably commensurate to the
      fair market value of such Value-Added Services. In addition, the terms and
      conditions on which any such Value Added Services are so offered or
      provided shall be consistent with, and shall fully support, the intent and
      purpose of this License Agreement.

      (( c ))??
      All Downstream Distributions shall:

         (( (i) ))??
         include all portions and elements of the Frameworx Code Base required
         to build the Source Code of such Downstream Distribution into a fully
         functional machine-executable system, or additional build scripts or
         comparable software necessary and sufficient for such purposes;

         (( (ii) ))??
         include, in each file containing any portion or element of the
         Frameworx Code Base, the following identifying legend: This file
         contains software that has been made available under The Frameworx Open
         License 1.0. Use and distribution hereof are subject to the
         restrictions set forth therein.

         (( (iii) ))??
         include all other copyright notices, authorship credits, warranty
         disclaimers (including that provided in Section 6 below), legends,
         documentation, annotations and comments contained in the Frameworx Code
         Base as provided to You hereunder;

         (( (iv) ))??
         contain an unaltered copy of the html file named
         frameworx_community_invitation.html included within the Frameworx Code
         Base that acknowledges new users and provides them with information on
         the Frameworx Code Base community;

         (( (v) ))??
         contain an unaltered copy of the text file named
         the_frameworx_license.txt included within the Frameworx Code Base that
         includes a text copy of the form of this License Agreement; and

         (( (vi) ))??
         prominently display to any viewer or user of the Source Code of such
         Open Downstream Distribution, in the place and manner normally used for
         such displays, the following legend:

   Source code licensed under from The Frameworx Company is contained herein,
   and such source code has been obtained either under The Frameworx Open
   License, or another license granted by The Frameworx Company. Use and
   distribution hereof is subject to the restrictions provided in the relevant
   such license and to the copyrights of the licensor thereunder. A copy of The
   Frameworx Open License is provided in a file named the_frameworx_license.txt
   and included herein, and may also be available for inspection at
   http:/www.frameworx.com.

   (( 4. ))??
   Restrictions on Open Downstream Distributions. Each Downstream Distribution
   made by You, and by any party directly or indirectly obtaining rights to the
   Frameworx Code Base through You, shall be made subject to a license grant or
   agreement to the extent necessary so that each distributee under that
   Downstream Distribution will be subject to the same restrictions on
   re-distribution and use as are binding on You hereunder. You may satisfy this
   licensing requirement either by:

      (( (a) ))??
      requiring as a condition to any Downstream Distribution made by you, or by
      any direct or indirect distributee of Your Downstream Distribution (or any
      portion or element thereof), that each distributee under the relevant
      Downstream Distribution obtain a direct license (on the same terms and
      conditions as those in this License Agreement) from The Frameworx Company;
      or

      (( (b) ))??
      sub-licensing all (and not less than all) of Your rights and obligations
      hereunder to that distributee, including (without limitation) Your
      obligation to require distributees to be bound by license restrictions as
      contemplated by this Section 4 above.

   The Frameworx Company hereby grants to you all rights to sub-license your
   rights hereunder as necessary to fully effect the intent and purpose of this
   Section 4 above, provided, however, that your rights and obligations
   hereunder shall be unaffected by any such sublicensing. In addition, The
   Frameworx Company expressly retains all rights to take all appropriate action
   (including legal action) against any such direct or indirect sub-licensee to
   ensure its full compliance with the intent and purposes of this License
   Agreement.

   (( 5. ))??
   Intellectual Property. Except as expressly provided herein, this License
   Agreement preserves and respects Your and The Frameworx Companys respective
   intellectual property rights, including, in the case of The Frameworx
   Company, its copyrights and patent rights relating to the Frameworx Code
   Base.

   (( 6. ))??
   Warranty Disclaimer. THE SOFTWARE LICENSED HEREUNDER IS PROVIDED ` + "`" + `` + "`" + `AS IS.''
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
   NON-INFRINGEMENT, ARE HEREBY DISCLAIMED. IN NO EVENT SHALL THE LICENSOR OF
   THIS SOFTWARE, BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES INCLUDING (BUT NOT LIMITED TO)
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
   OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
   WHETHER IN CONTRACT, STRICT LIABILITY OR TORT (INCLUDING NEGLIGENCE OR
   OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
   ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

   (( 7. ))??
   License Violation. The License, and all of your rights thereunder, shall be
   deemed automatically terminated and void as of any Downstream Distribution
   directly or indirectly made or facilitated by You that violates the
   provisions of this License Agreement, provided, however, that this License
   Agreement shall survive any such termination in order to remedy the effects
   of such violation. This License Agreement shall be binding on the legal
   successors and assigns of the parties hereto.

   Your agreement to the foregoing as of the date hereof has been evidenced by
   your acceptance of the relevant software distribution hereunder.

   (( (C) THE FRAMEWORX COMPANY 2003 ))??
`
const license_FreeImage_lre = `//**
FreeImage Public License v1.0
https://spdx.org/licenses/FreeImage.json
http://freeimage.sourceforge.net/freeimage-license.txt
**//

(( FreeImage Public License - Version 1.0 ))??

   (( 1. ))??
   Definitions.

      (( 1.1. ))??
      "Contributor" means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      "Covered Code" means the Original Code or Modifications or the combination
      of the Original Code and Modifications, in each case including portions
      thereof.

      (( 1.4. ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      "Executable" means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      "Initial Developer" means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7. ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.8. ))??
      "License" means this document.

      (( 1.9. ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a

      Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10. ))??
      "Original Code" means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      (( 1.11. ))??
      "Source Code" means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or a list of source code differential
      comparisons against either the Original Code or another well known,
      available Covered Code of the Contributor's choice. The Source Code can be
      in a compressed or archival form, provided the appropriate decompression
      or de-archiving software is widely available for no charge.

      (( 1.12. ))??
      "You" means an individual or a legal entity exercising rights under, and
      complying with all of the terms of, this License or a future version of
      this License issued under Section 6.1. For legal entities, "You" includes
      any entity which controls, is controlled by, or is under common control
      with You. For purposes of this definition, "control" means (a) the power,
      direct or indirect, to cause the direction or management of such entity,
      whether by contract or otherwise, or (b) ownership of fifty percent (50%)
      or more of the outstanding shares or beneficial ownership of such entity.

   (( 2. ))??
   Source Code License.

      (( 2.1. ))??
      The Initial Developer Grant.

      The Initial Developer hereby grants You a world-wide, royalty-free,
      non-exclusive license, subject to third party intellectual property
      claims:

         (( (a) ))??
         to use, reproduce, modify, display, perform, sublicense and distribute
         the Original Code (or portions thereof) with or without Modifications,
         or as part of a Larger Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Initial
         Developer, to make, have made, use and sell ("Utilize") the Original
         Code (or portions thereof), but solely to the extent that any such
         patent is reasonably necessary to enable You to Utilize the Original
         Code (or portions thereof) and not to any greater extent that may be
         necessary to Utilize further Modifications or combinations.

      (( 2.2. ))??
      Contributor Grant.

      Each Contributor hereby grants You a world-wide, royalty-free,
      non-exclusive license, subject to third party intellectual property
      claims:

         (( (a) ))??
         to use, reproduce, modify, display, perform, sublicense and distribute
         the Modifications created by such Contributor (or portions thereof)
         either on an unmodified basis, with other Modifications, as Covered
         Code or as part of a Larger Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Contributor, to
         Utilize the Contributor Version (or portions thereof), but solely to
         the extent that any such patent is reasonably necessary to enable You
         to Utilize the Contributor Version (or portions thereof), and not to
         any greater extent that may be necessary to Utilize further
         Modifications or combinations.

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Application of License.

      The Modifications which You create or to which You contribute are governed
      by the terms of this License, including without limitation Section 2.2.
      The Source Code version of Covered Code may be distributed only under the
      terms of this License or a future version of this License released under
      Section 6.1, and You must include a copy of this License with every copy
      of the Source Code You distribute. You may not offer or impose any terms
      on any Source Code version that alters or restricts the applicable version
      of this License or the recipients' rights hereunder. However, You may
      include an additional document offering the additional rights described in
      Section 3.5.

      (( 3.2. ))??
      Availability of Source Code.

      Any Modification which You create or to which You contribute must be made
      available in Source Code form under the terms of this License either on
      the same media as an Executable version or via an accepted Electronic
      Distribution Mechanism to anyone to whom you made an Executable version
      available; and if made available via Electronic Distribution Mechanism,
      must remain available for at least twelve (12) months after the date it
      initially became available, or at least six (6) months after a subsequent
      version of that particular Modification has been made available to such
      recipients. You are responsible for ensuring that the Source Code version
      remains available even if the Electronic Distribution Mechanism is
      maintained by a third party.

      (( 3.3. ))??
      Description of Modifications.

      You must cause all Covered Code to which you contribute to contain a file
      documenting the changes You made to create that Covered Code and the date
      of any change. You must include a prominent statement that the
      Modification is derived, directly or indirectly, from Original Code
      provided by the Initial Developer and including the name of the Initial
      Developer in (a) the Source Code, and (b) in any notice in an Executable
      version or related documentation in which You describe the origin or
      ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters

         (( (a) ))??
         Third Party Claims.

         If You have knowledge that a party claims an intellectual property
         right in particular functionality or code (or its utilization under
         this License), you must include a text file with the source code
         distribution titled "LEGAL" which describes the claim and the party
         making the claim in sufficient detail that a recipient will know whom
         to contact. If you obtain such knowledge after You make Your
         Modification available as described in Section 3.2, You shall promptly
         modify the LEGAL file in all copies You make available thereafter and
         shall take other steps (such as notifying appropriate mailing lists or
         newsgroups) reasonably calculated to inform those who received the
         Covered Code that new knowledge has been obtained.

         (( (b) ))??
         Contributor APIs.

         If Your Modification is an application programming interface and You
         own or control patents which are reasonably necessary to implement that
         API, you must also include this information in the LEGAL file.

      (( 3.5. ))??
      Required Notices.

      You must duplicate the notice in Exhibit A in each file of the Source
      Code, and this License in any documentation for the Source Code, where You
      describe recipients' rights relating to Covered Code. If You created one
      or more Modification(s), You may add your name as a Contributor to the
      notice described in Exhibit A. If it is not possible to put such notice in
      a particular Source Code file due to its structure, then you must include
      such notice in a location (such as a relevant directory file) where a user
      would be likely to look for such a notice. You may choose to offer, and to
      charge a fee for, warranty, support, indemnity or liability obligations to
      one or more recipients of Covered Code. However, You may do so only on
      Your own behalf, and not on behalf of the Initial Developer or any
      Contributor. You must make it absolutely clear than any such warranty,
      support, indemnity or liability obligation is offered by You alone, and
      You hereby agree to indemnify the Initial Developer and every Contributor
      for any liability incurred by the Initial Developer or such Contributor as
      a result of warranty, support, indemnity or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Versions.

      You may distribute Covered Code in Executable form only if the
      requirements of Section 3.1-3.5 have been met for that Covered Code, and
      if You include a notice stating that the Source Code version of the
      Covered Code is available under the terms of this License, including a
      description of how and where You have fulfilled the obligations of Section
      3.2. The notice must be conspicuously included in any notice in an
      Executable version, related documentation or collateral in which You descr
      ibe recipients' rights relating to the Covered Code. You may distribute
      the Executable version of Covered Code under a license of Your choice,
      which may contain terms different from this License, provided that You are
      in compliance with the terms of this License and that the license for the
      Executable version does not attempt to limit or alter the recipient's
      rights in the Source Code version from the rights set forth in this
      License. If You distribute the Executable version under a different
      license You must make it absolutely clear that any terms which differ from
      this License are offered by You alone, not by the Initial Developer or any
      Contributor. You hereby agree to indemnify the Initial Developer and every
      Contributor for any liability incurred by the Initial Developer or such
      Contributor as a result of any such terms You offer.

      (( 3.7. ))??
      Larger Works.

      You may create a Larger Work by combining Covered Code with other code not
      governed by the terms of this License and distribute the Larger Work as a
      single product. In such a case, You must make sure the requirements of
      this License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute or regulation
   then You must: (a) comply with the terms of this License to the maximum
   extent possible; and (b) describe the limitations and the code they affect.
   Such description must be included in the LEGAL file described in Section 3.4
   and must be included with all distributions of the Source Code. Except to the
   extent prohibited by statute or regulation, such description must be
   sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Application of this License.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A, and to related Covered Code.

   (( 6. ))??
   Versions of the License.

      (( 6.1. ))??
      New Versions.

      Floris van den Berg may publish revised and/or new versions of the License
      from time to time. Each version will be given a distinguishing version
      number.

      (( 6.2. ))??
      Effect of New Versions.

      Once Covered Code has been published under a particular version of the
      License, You may always continue to use it under the terms of that
      version. You may also choose to use such Covered Code under the terms of
      any subsequent version of the License published by Floris van den Berg

      No one other than Floris van den Berg has the right to modify the terms
      applicable to Covered Code created under this License.

      (( 6.3. ))??
      Derivative Works.

      If you create or use a modified version of this License (which you may
      only do in order to apply it to code which is not already Covered Code
      governed by this License), you must (a) rename Your license so that the
      phrases "FreeImage", ` + "`" + `FreeImage Public License", "FIPL", or any
      confusingly similar phrase do not appear anywhere in your license and (b)
      otherwise make it clear that your version of the license contains terms
      which differ from the FreeImage Public License. (Filling in the name of
      the Initial Developer, Original Code or Contributor in the notice
      described in Exhibit A shall not of themselves be deemed to be
      modifications of this License.)

   (( 7. ))??
   DISCLAIMER OF WARRANTY.

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER
   OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR
   OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF
   THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

   (( 8. ))??
   TERMINATION.

   This License and the rights granted hereunder will terminate automatically if
   You fail to comply with terms herein and fail to cure such breach within 30
   days of becoming aware of the breach. All sublicenses to the Covered Code
   which are properly granted shall survive any termination of this License.
   Provisions which, by their nature, must remain in effect beyond the
   termination of this License shall survive.

   (( 9. ))??
   LIMITATION OF LIABILITY.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE INITIAL DEVELOPER, ANY OTHER
   CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE, OR ANY SUPPLIER OF ANY OF
   SUCH PARTIES, BE LIABLE TO YOU OR ANY OTHER PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THAT EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. GOVERNMENT END USERS.

   The Covered Code is a "commercial item," as that term is defined in 48 C.F.R.
   2.101 (Oct. 1995), consisting of "commercial computer software" and
   "commercial computer software documentation," as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Covered Code with only those rights set forth herein.

   (( 11. ))??
   MISCELLANEOUS.

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by Dutch law provisions (except
   to the extent applicable law, if any, provides otherwise), excluding its
   conflict-of-law provisions. With respect to disputes in which at least one
   party is a citizen of, or an entity chartered or registered to do business
   in, the The Netherlands: (a) unless otherwise agreed in writing, all disputes
   relating to this License (excepting any dispute relating to intellectual
   property rights) shall be subject to final and binding arbitration, with the
   losing party paying all costs of arbitration; (b) any arbitration relating to
   this Agreement shall be held in Almelo, The Netherlands; and (c) any
   litigation relating to this Agreement shall be subject to the jurisdiction of
   the court of Almelo, The Netherlands with the losing party responsible for
   costs, including without limitation, court costs and reasonable attorneys
   fees and expenses. Any law or regulation which provides that the language of
   a contract shall be construed against the drafter shall not apply to this
   License.

   (( 12. ))??
   RESPONSIBILITY FOR CLAIMS.

   Except in cases where another Contributor has failed to comply with Section
   3.4, You are responsible for damages arising, directly or indirectly, out of
   Your utilization of rights under this License, based on the number of copies
   of Covered Code you made available, the revenues you received from utilizing
   such rights, and other relevant factors. You agree to work with affected
   parties to distribute responsibility on an equitable basis.
   (( EXHIBIT A.

"The contents of this file are subject to the FreeImage Public License Version
1.0 (the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at
http:/home.wxs.nl/~flvdberg/freeimage-license.txt

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License. ))??
`
const license_GFDL_1_3_no_invariants_or_later_lre = ` 
	 
	 
	 

	Permission is granted to copy, distribute and/or
	modify this document under the terms of the GNU Free Documentation License,
	Version 1.3
	
		or any later version published by the Free Software Foundation;
	
	
		with no Invariant Sections,
		with no Front-Cover Texts,
		and with no Back-Cover Texts.
	
	A copy of the license is included in the section entitled
	"GNU Free Documentation License".
 `
const license_GFDL_1_3_no_invariants_only_lre = ` 
	 
	 
	 

	Permission is granted to copy, distribute and/or
	modify this document under the terms of the GNU Free Documentation License,
	Version 1.3
	
		((published by the Free Software Foundation))??
	
	
		with no Invariant Sections,
		with no Front-Cover Texts,
		and with no Back-Cover Texts.
	
	A copy of the license is included in the section entitled
	"GNU Free Documentation License".
 `
const license_GFDL_1_3_invariants_or_later_lre = ` 
	 
	 
	 

	Permission is granted to copy, distribute and/or
	modify this document under the terms of the GNU Free Documentation License,
	Version 1.3
	
		or any later version published by the Free Software Foundation;
	
	
		with
		((no Invariant Sections || the Invariant Sections being __20__))
		with
		((no Front-Cover Texts || the Front-Cover Texts being __20__))
		and with
		((no Back-Cover Texts || the Back-Cover Texts being __20__))
	
	A copy of the license is included in the section entitled
	"GNU Free Documentation License".
 `
const license_GFDL_1_3_invariants_only_lre = ` 
	 
	 
	 

	Permission is granted to copy, distribute and/or
	modify this document under the terms of the GNU Free Documentation License,
	Version 1.3
	
		((published by the Free Software Foundation))??
	
	
		with
		((no Invariant Sections || the Invariant Sections being __20__))
		with
		((no Front-Cover Texts || the Front-Cover Texts being __20__))
		and with
		((no Back-Cover Texts || the Back-Cover Texts being __20__))
	
	A copy of the license is included in the section entitled
	"GNU Free Documentation License".
 `
const license_GFDL_1_3_lre = `//**
GNU Free Documentation License v1.3 or later
https://spdx.org/licenses/GFDL-1.3-or-later.json
https://www.gnu.org/licenses/fdl-1.3.txt
**//

((
	GNU Free Documentation License Version 1.3, 3 November 2008
	
	((
		((
			Copyright __20__
			((<https://fsf.org/>))??
			
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))??
		))??

		Everyone is permitted to copy and distribute verbatim copies
		of this license document, but changing it is not allowed.

		((Copyright __20__))??
	))??

))??

   ((0. || 1.))??
   PREAMBLE

   The purpose of this License is to make a manual, textbook, or other
   functional and useful document "free" in the sense of freedom: to assure
   everyone the effective freedom to copy and redistribute it, with or without
   modifying it, either commercially or noncommercially. Secondarily, this
   License preserves for the author and publisher a way to get credit for their
   work, while not being considered responsible for modifications made by
   others.

   This License is a kind of "copyleft", which means that derivative works of
   the document must themselves be free in the same sense. It complements the
   GNU General Public License, which is a copyleft license designed for free
   software.

   We have designed this License in order to use it for manuals for free
   software, because free software needs free documentation: a free program
   should come with manuals providing the same freedoms that the software does.
   But this License is not limited to software manuals; it can be used for any
   textual work, regardless of subject matter or whether it is published as a
   printed book. We recommend this License principally for works whose purpose
   is instruction or reference.

   __1__ APPLICABILITY AND DEFINITIONS

   This License applies to any manual or other work, in any medium, that
   contains a notice placed by the copyright holder saying it can be distributed
   under the terms of this License. Such a notice grants a world-wide,
   royalty-free license, unlimited in duration, to use that work under the
   conditions stated herein. The "Document", below, refers to any such manual or
   work. Any member of the public is a licensee, and is addressed as "you". You
   accept the license if you copy, modify or distribute the work in a way
   requiring permission under copyright law.

   A "Modified Version" of the Document means any work containing the Document
   or a portion of it, either copied verbatim, or with modifications and/or
   translated into another language.

   A "Secondary Section" is a named appendix or a front-matter section of the
   Document that deals exclusively with the relationship of the publishers or
   authors of the Document to the Document's overall subject (or to related
   matters) and contains nothing that could fall directly within that overall
   subject. (Thus, if the Document is in part a textbook of mathematics, a
   Secondary Section may not explain any mathematics.) The relationship could be
   a matter of historical connection with the subject or with related matters,
   or of legal, commercial, philosophical, ethical or political position
   regarding them.

   The "Invariant Sections" are certain Secondary Sections whose titles are
   designated, as being those of Invariant Sections, in the notice that says
   that the Document is released under this License. If a section does not fit
   the above definition of Secondary then it is not allowed to be designated as
   Invariant. The Document may contain zero Invariant Sections. If the Document
   does not identify any Invariant Sections then there are none.

   The "Cover Texts" are certain short passages of text that are listed, as
   Front-Cover Texts or Back-Cover Texts, in the notice that says that the
   Document is released under this License. A Front-Cover Text may be at most 5
   words, and a Back-Cover Text may be at most 25 words.

   A "Transparent" copy of the Document means a machine-readable copy,
   represented in a format whose specification is available to the general
   public, that is suitable for revising the document straightforwardly with
   generic text editors or (for images composed of pixels) generic paint
   programs or (for drawings) some widely available drawing editor, and that is
   suitable for input to text formatters or for automatic translation to a
   variety of formats suitable for input to text formatters. A copy made in an
   otherwise Transparent file format whose markup, or absence of markup, has
   been arranged to thwart or discourage subsequent modification by readers is
   not Transparent. An image format is not Transparent if used for any
   substantial amount of text. A copy that is not "Transparent" is called
   "Opaque".

   Examples of suitable formats for Transparent copies include plain ASCII
   without markup, Texinfo input format, LaTeX input format, SGML or XML using a
   publicly available DTD, and standard-conforming simple HTML, PostScript or
   PDF designed for human modification. Examples of transparent image formats
   include PNG, XCF and JPG. Opaque formats include proprietary formats that can
   be read and edited only by proprietary word processors, SGML or XML for which
   the DTD and/or processing tools are not generally available, and the
   machine-generated HTML, PostScript or PDF produced by some word processors
   for output purposes only.

   The "Title Page" means, for a printed book, the title page itself, plus such
   following pages as are needed to hold, legibly, the material this License
   requires to appear in the title page. For works in formats which do not have
   any title page as such, "Title Page" means the text near the most prominent
   appearance of the work's title, preceding the beginning of the body of the
   text.

   The "publisher" means any person or entity that distributes copies of the
   Document to the public.

   A section "Entitled XYZ" means a named subunit of the Document whose title
   either is precisely XYZ or contains XYZ in parentheses following text that
   translates XYZ in another language. (Here XYZ stands for a specific section
   name mentioned below, such as "Acknowledgements", "Dedications",
   "Endorsements", or "History".) To "Preserve the Title" of such a section when
   you modify the Document means that it remains a section "Entitled XYZ"
   according to this definition.

   The Document may include Warranty Disclaimers next to the notice which states
   that this License applies to the Document. These Warranty Disclaimers are
   considered to be included by reference in this License, but only as regards
   disclaiming warranties: any other implication that these Warranty Disclaimers
   may have is void and has no effect on the meaning of this License.

   __1__ VERBATIM COPYING

   You may copy and distribute the Document in any medium, either commercially
   or noncommercially, provided that this License, the copyright notices, and
   the license notice saying this License applies to the Document are reproduced
   in all copies, and that you add no other conditions whatsoever to those of
   this License. You may not use technical measures to obstruct or control the
   reading or further copying of the copies you make or distribute. However, you
   may accept compensation in exchange for copies. If you distribute a large
   enough number of copies you must also follow the conditions in section 3.

   You may also lend copies, under the same conditions stated above, and you may
   publicly display copies.

   __1__ COPYING IN QUANTITY

   If you publish printed copies (or copies in media that commonly have printed
   covers) of the Document, numbering more than 100, and the Document's license
   notice requires Cover Texts, you must enclose the copies in covers that
   carry, clearly and legibly, all these Cover Texts: Front-Cover Texts on the
   front cover, and Back-Cover Texts on the back cover. Both covers must also
   clearly and legibly identify you as the publisher of these copies. The front
   cover must present the full title with all words of the title equally
   prominent and visible. You may add other material on the covers in addition.
   Copying with changes limited to the covers, as long as they preserve the
   title of the Document and satisfy these conditions, can be treated as
   verbatim copying in other respects.

   If the required texts for either cover are too voluminous to fit legibly, you
   should put the first ones listed (as many as fit reasonably) on the actual
   cover, and continue the rest onto adjacent pages.

   If you publish or distribute Opaque copies of the Document numbering more
   than 100, you must either include a machine-readable Transparent copy along
   with each Opaque copy, or state in or with each Opaque copy a
   computer-network location from which the general network-using public has
   access to download using public-standard network protocols a complete
   Transparent copy of the Document, free of added material. If you use the
   latter option, you must take reasonably prudent steps, when you begin
   distribution of Opaque copies in quantity, to ensure that this Transparent
   copy will remain thus accessible at the stated location until at least one
   year after the last time you distribute an Opaque copy (directly or through
   your agents or retailers) of that edition to the public.

   It is requested, but not required, that you contact the authors of the
   Document well before redistributing any large number of copies, to give them
   a chance to provide you with an updated version of the Document.

   __1__ MODIFICATIONS

   You may copy and distribute a Modified Version of the Document under the
   conditions of sections 2 and 3 above, provided that you release the Modified
   Version under precisely this License, with the Modified Version filling the
   role of the Document, thus licensing distribution and modification of the
   Modified Version to whoever possesses a copy of it. In addition, you must do
   these things in the Modified Version:

      __1__ Use in the Title Page (and on the covers, if any) a title distinct
      from that of the Document, and from those of previous versions (which
      should, if there were any, be listed in the History section of the
      Document). You may use the same title as a previous version if the
      original publisher of that version gives permission.

      __1__ List on the Title Page, as authors, one or more persons or entities
      responsible for authorship of the modifications in the Modified Version,
      together with at least five of the principal authors of the Document (all
      of its principal authors, if it has fewer than five), unless they release
      you from this requirement.

      __1__ State on the Title page the name of the publisher of the Modified
      Version, as the publisher.

      __1__ Preserve all the copyright notices of the Document.

      __1__ Add an appropriate copyright notice for your modifications adjacent
      to the other copyright notices.

      __1__ Include, immediately after the copyright notices, a license notice
      giving the public permission to use the Modified Version under the terms
      of this License, in the form shown in the Addendum below.

      __1__ Preserve in that license notice the full lists of Invariant Sections
      and required Cover Texts given in the Document's license notice. H.
      Include an unaltered copy of this License.

      __1__ Preserve the section Entitled "History", Preserve its Title, and add
      to it an item stating at least the title, year, new authors, and publisher
      of the Modified Version as given on the Title Page. If there is no section
      Entitled "History" in the Document, create one stating the title, year,
      authors, and publisher of the Document as given on its Title Page, then
      add an item describing the Modified Version as stated in the previous
      sentence.

      __1__ Preserve the network location, if any, given in the Document for
      public access to a Transparent copy of the Document, and likewise the
      network locations given in the Document for previous versions it was based
      on. These may be placed in the "History" section. You may omit a network
      location for a work that was published at least four years before the
      Document itself, or if the original publisher of the version it refers to
      gives permission.

      __1__ For any section Entitled "Acknowledgements" or "Dedications",
      Preserve the Title of the section, and preserve in the section all the
      substance and tone of each of the contributor acknowledgements and/or
      dedications given therein.

      __1__ Preserve all the Invariant Sections of the Document, unaltered in
      their text and in their titles. Section numbers or the equivalent are not
      considered part of the section titles.

      __1__ Delete any section Entitled "Endorsements". Such a section may not
      be included in the Modified Version.

      __1__ Do not retitle any existing section to be Entitled "Endorsements" or
      to conflict in title with any Invariant Section.

      __1__ Preserve any Warranty Disclaimers.

   If the Modified Version includes new front-matter sections or appendices that
   qualify as Secondary Sections and contain no material copied from the
   Document, you may at your option designate some or all of these sections as
   invariant. To do this, add their titles to the list of Invariant Sections in
   the Modified Version's license notice. These titles must be distinct from any
   other section titles.

   You may add a section Entitled "Endorsements", provided it contains nothing
   but endorsements of your Modified Version by various parties--for example,
   statements of peer review or that the text has been approved by an
   organization as the authoritative definition of a standard.

   You may add a passage of up to five words as a Front-Cover Text, and a
   passage of up to 25 words as a Back-Cover Text, to the end of the list of
   Cover Texts in the Modified Version. Only one passage of Front-Cover Text and
   one of Back-Cover Text may be added by (or through arrangements made by) any
   one entity. If the Document already includes a cover text for the same cover,
   previously added by you or by arrangement made by the same entity you are
   acting on behalf of, you may not add another; but you may replace the old
   one, on explicit permission from the previous publisher that added the old
   one.

   The author(s) and publisher(s) of the Document do not by this License give
   permission to use their names for publicity for or to assert or imply
   endorsement of any Modified Version.

   __1__ COMBINING DOCUMENTS

   You may combine the Document with other documents released under this
   License, under the terms defined in section 4 above for modified versions,
   provided that you include in the combination all of the Invariant Sections of
   all of the original documents, unmodified, and list them all as Invariant
   Sections of your combined work in its license notice, and that you preserve
   all their Warranty Disclaimers.

   The combined work need only contain one copy of this License, and multiple
   identical Invariant Sections may be replaced with a single copy. If there are
   multiple Invariant Sections with the same name but different contents, make
   the title of each such section unique by adding at the end of it, in
   parentheses, the name of the original author or publisher of that section if
   known, or else a unique number. Make the same adjustment to the section
   titles in the list of Invariant Sections in the license notice of the
   combined work.

   In the combination, you must combine any sections Entitled "History" in the
   various original documents, forming one section Entitled "History"; likewise
   combine any sections Entitled "Acknowledgements", and any sections Entitled
   "Dedications". You must delete all sections Entitled "Endorsements".

   __1__ COLLECTIONS OF DOCUMENTS

   You may make a collection consisting of the Document and other documents
   released under this License, and replace the individual copies of this
   License in the various documents with a single copy that is included in the
   collection, provided that you follow the rules of this License for verbatim
   copying of each of the documents in all other respects.

   You may extract a single document from such a collection, and distribute it
   individually under this License, provided you insert a copy of this License
   into the extracted document, and follow this License in all other respects
   regarding verbatim copying of that document.

   __1__ AGGREGATION WITH INDEPENDENT WORKS

   A compilation of the Document or its derivatives with other separate and
   independent documents or works, in or on a volume of a storage or
   distribution medium, is called an "aggregate" if the copyright resulting from
   the compilation is not used to limit the legal rights of the compilation's
   users beyond what the individual works permit. When the Document is included
   in an aggregate, this License does not apply to the other works in the
   aggregate which are not themselves derivative works of the Document.

   If the Cover Text requirement of section 3 is applicable to these copies of
   the Document, then if the Document is less than one half of the entire
   aggregate, the Document's Cover Texts may be placed on covers that bracket
   the Document within the aggregate, or the electronic equivalent of covers if
   the Document is in electronic form. Otherwise they must appear on printed
   covers that bracket the whole aggregate.

   __1__ TRANSLATION

   Translation is considered a kind of modification, so you may distribute
   translations of the Document under the terms of section 4. Replacing
   Invariant Sections with translations requires special permission from their
   copyright holders, but you may include translations of some or all Invariant
   Sections in addition to the original versions of these Invariant Sections.
   You may include a translation of this License, and all the license notices in
   the Document, and any Warranty Disclaimers, provided that you also include
   the original English version of this License and the original versions of
   those notices and disclaimers. In case of a disagreement between the
   translation and the original version of this License or a notice or
   disclaimer, the original version will prevail.

   If a section in the Document is Entitled "Acknowledgements", "Dedications",
   or "History", the requirement (section 4) to Preserve its Title (section 1)
   will typically require changing the actual title.

   __1__ TERMINATION

   You may not copy, modify, sublicense, or distribute the Document except as
   expressly provided under this License. Any attempt otherwise to copy, modify,
   sublicense, or distribute it is void, and will automatically terminate your
   rights under this License.

   However, if you cease all violation of this License, then your license from a
   particular copyright holder is reinstated (a) provisionally, unless and until
   the copyright holder explicitly and finally terminates your license, and (b)
   permanently, if the copyright holder fails to notify you of the violation by
   some reasonable means prior to 60 days after the cessation.

   Moreover, your license from a particular copyright holder is reinstated
   permanently if the copyright holder notifies you of the violation by some
   reasonable means, this is the first time you have received notice of
   violation of this License (for any work) from that copyright holder, and you
   cure the violation prior to 30 days after your receipt of the notice.

   Termination of your rights under this section does not terminate the licenses
   of parties who have received copies or rights from you under this License. If
   your rights have been terminated and not permanently reinstated, receipt of a
   copy of some or all of the same material does not give you any rights to use
   it.

   __1__ FUTURE REVISIONS OF THIS LICENSE

   The Free Software Foundation may publish new, revised versions of the GNU
   Free Documentation License from time to time. Such new versions will be
   similar in spirit to the present version, but may differ in detail to address
   new problems or concerns. See http:/www.gnu.org/
((copyleft||licenses))
/.

   Each version of the License is given a distinguishing version number. If the
   Document specifies that a particular numbered version of this License "or any
   later version" applies to it, you have the option of following the terms and
   conditions either of that specified version or of any later version that has
   been published (not as a draft) by the Free Software Foundation. If the
   Document does not specify a version number of this License, you may choose
   any version ever published (not as a draft) by the Free Software Foundation.
   If the Document specifies that a proxy can decide which future versions of
   this License can be used, that proxy's public statement of acceptance of a
   version permanently authorizes you to choose that version for the Document.

   __1__ RELICENSING

   "Massive Multiauthor Collaboration Site" (or "MMC Site") means any World Wide
   Web server that publishes copyrightable works and also provides prominent
   facilities for anybody to edit those works. A public wiki that anybody can
   edit is an example of such a server. A "Massive Multiauthor Collaboration"
   (or "MMC") contained in the site means any set of copyrightable works thus
   published on the MMC site.

   "CC-BY-SA" means the Creative Commons Attribution-Share Alike 3.0 license
   published by Creative Commons Corporation, a not-for-profit corporation with
   a principal place of business in San Francisco, California, as well as future
   copyleft versions of that license published by that same organization.

   "Incorporate" means to publish or republish a Document, in whole or in part,
   as part of another Document.

   An MMC is "eligible for relicensing" if it is licensed under this License,
   and if all works that were first published under this License somewhere other
   than this MMC, and subsequently incorporated in whole or in part into the
   MMC, (1) had no cover texts or invariant sections, and (2) were thus
   incorporated prior to November 1, 2008.

   The operator of an MMC Site may republish an MMC contained in the site under
   CC-BY-SA on the same site at any time before August 1, 2009, provided the MMC
   is eligible for relicensing.
((  ADDENDUM: How to use this License for your
   documents

To use this License in a document you have written, include a copy of the
License in the document and put the following copyright and license notices just
after the title page:

Copyright (c) __10__. Permission is granted to copy, distribute and/or
modify this document under the terms of the GNU Free Documentation License,
Version 1.3 or any later version published by the Free Software Foundation; with
no Invariant Sections, no Front-Cover Texts, and no Back-Cover Texts. A copy of
the license is included in the section entitled "GNU Free Documentation
License".

If you have Invariant Sections, Front-Cover Texts and Back-Cover Texts, replace
the "with...Texts." line with this:

with the Invariant Sections being LIST THEIR TITLES, with the Front-Cover Texts
being LIST, and with the Back-Cover Texts being LIST.

If you have Invariant Sections without Cover Texts, or some other combination of
the three, merge those two alternatives to suit the situation.

If your document contains nontrivial examples of program code, we recommend
releasing these examples in parallel under your choice of free software license,
such as the GNU General Public License, to permit their use in free software.
))??
`
const license_GFDL_1_2_no_invariants_or_later_lre = ` 
	 
	 
	 

	Permission is granted to copy, distribute and/or
	modify this document under the terms of the GNU Free Documentation License,
	Version 1.2
	
		or any later version published by the Free Software Foundation;
	
	
		with no Invariant Sections,
		with no Front-Cover Texts,
		and with no Back-Cover Texts.
	
	A copy of the license is included in the section entitled
	"GNU Free Documentation License".
 `
const license_GFDL_1_2_no_invariants_only_lre = ` 
	 
	 
	 

	Permission is granted to copy, distribute and/or
	modify this document under the terms of the GNU Free Documentation License,
	Version 1.2
	
		((published by the Free Software Foundation))??
	
	
		with no Invariant Sections,
		with no Front-Cover Texts,
		and with no Back-Cover Texts.
	
	A copy of the license is included in the section entitled
	"GNU Free Documentation License".
 `
const license_GFDL_1_2_invariants_or_later_lre = ` 
	 
	 
	 

	Permission is granted to copy, distribute and/or
	modify this document under the terms of the GNU Free Documentation License,
	Version 1.2
	
		or any later version published by the Free Software Foundation;
	
	
		with
		((no Invariant Sections || the Invariant Sections being __20__))
		with
		((no Front-Cover Texts || the Front-Cover Texts being __20__))
		and with
		((no Back-Cover Texts || the Back-Cover Texts being __20__))
	
	A copy of the license is included in the section entitled
	"GNU Free Documentation License".
 `
const license_GFDL_1_2_invariants_only_lre = ` 
	 
	 
	 

	Permission is granted to copy, distribute and/or
	modify this document under the terms of the GNU Free Documentation License,
	Version 1.2
	
		((published by the Free Software Foundation))??
	
	
		with
		((no Invariant Sections || the Invariant Sections being __20__))
		with
		((no Front-Cover Texts || the Front-Cover Texts being __20__))
		and with
		((no Back-Cover Texts || the Back-Cover Texts being __20__))
	
	A copy of the license is included in the section entitled
	"GNU Free Documentation License".
 `
const license_GFDL_1_2_lre = `//**
GNU Free Documentation License v1.2 or later
https://spdx.org/licenses/GFDL-1.2-or-later.json
https://www.gnu.org/licenses/old-licenses/fdl-1.2.txt
**//

((
	GNU Free Documentation License Version 1.2, November 2002
	
	((
		((
			Copyright __20__
			((<https://fsf.org/>))??
			
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))??
		))??

		Everyone is permitted to copy and distribute verbatim copies
		of this license document, but changing it is not allowed.

		((Copyright __20__))??
	))??

))??


   ((0. || 1.))??
   PREAMBLE

   The purpose of this License is to make a manual, textbook, or other
   functional and useful document "free" in the sense of freedom: to assure
   everyone the effective freedom to copy and redistribute it, with or without
   modifying it, either commercially or noncommercially. Secondarily, this
   License preserves for the author and publisher a way to get credit for their
   work, while not being considered responsible for modifications made by
   others.

   This License is a kind of "copyleft", which means that derivative works of
   the document must themselves be free in the same sense. It complements the
   GNU General Public License, which is a copyleft license designed for free
   software.

   We have designed this License in order to use it for manuals for free
   software, because free software needs free documentation: a free program
   should come with manuals providing the same freedoms that the software does.
   But this License is not limited to software manuals; it can be used for any
   textual work, regardless of subject matter or whether it is published as a
   printed book. We recommend this License principally for works whose purpose
   is instruction or reference.

   __1__ APPLICABILITY AND DEFINITIONS

   This License applies to any manual or other work, in any medium, that
   contains a notice placed by the copyright holder saying it can be distributed
   under the terms of this License. Such a notice grants a world-wide,
   royalty-free license, unlimited in duration, to use that work under the
   conditions stated herein. The "Document", below, refers to any such manual or
   work. Any member of the public is a licensee, and is addressed as "you". You
   accept the license if you copy, modify or distribute the work in a way
   requiring permission under copyright law.

   A "Modified Version" of the Document means any work containing the Document
   or a portion of it, either copied verbatim, or with modifications and/or
   translated into another language.

   A "Secondary Section" is a named appendix or a front-matter section of the
   Document that deals exclusively with the relationship of the publishers or
   authors of the Document to the Document's overall subject (or to related
   matters) and contains nothing that could fall directly within that overall
   subject. (Thus, if the Document is in part a textbook of mathematics, a
   Secondary Section may not explain any mathematics.) The relationship could be
   a matter of historical connection with the subject or with related matters,
   or of legal, commercial, philosophical, ethical or political position
   regarding them.

   The "Invariant Sections" are certain Secondary Sections whose titles are
   designated, as being those of Invariant Sections, in the notice that says
   that the Document is released under this License. If a section does not fit
   the above definition of Secondary then it is not allowed to be designated as
   Invariant. The Document may contain zero Invariant Sections. If the Document
   does not identify any Invariant Sections then there are none.

   The "Cover Texts" are certain short passages of text that are listed, as
   Front-Cover Texts or Back-Cover Texts, in the notice that says that the
   Document is released under this License. A Front-Cover Text may be at most 5
   words, and a Back-Cover Text may be at most 25 words.

   A "Transparent" copy of the Document means a machine-readable copy,
   represented in a format whose specification is available to the general
   public, that is suitable for revising the document straightforwardly with
   generic text editors or (for images composed of pixels) generic paint
   programs or (for drawings) some widely available drawing editor, and that is
   suitable for input to text formatters or for automatic translation to a
   variety of formats suitable for input to text formatters. A copy made in an
   otherwise Transparent file format whose markup, or absence of markup, has
   been arranged to thwart or discourage subsequent modification by readers is
   not Transparent. An image format is not Transparent if used for any
   substantial amount of text. A copy that is not "Transparent" is called
   "Opaque".

   Examples of suitable formats for Transparent copies include plain ASCII
   without markup, Texinfo input format, LaTeX input format, SGML or XML using a
   publicly available DTD, and standard-conforming simple HTML, PostScript or
   PDF designed for human modification. Examples of transparent image formats
   include PNG, XCF and JPG. Opaque formats include proprietary formats that can
   be read and edited only by proprietary word processors, SGML or XML for which
   the DTD and/or processing tools are not generally available, and the
   machine-generated HTML, PostScript or PDF produced by some word processors
   for output purposes only.

   The "Title Page" means, for a printed book, the title page itself, plus such
   following pages as are needed to hold, legibly, the material this License
   requires to appear in the title page. For works in formats which do not have
   any title page as such, "Title Page" means the text near the most prominent
   appearance of the work's title, preceding the beginning of the body of the
   text.

   A section "Entitled XYZ" means a named subunit of the Document whose title
   either is precisely XYZ or contains XYZ in parentheses following text that
   translates XYZ in another language. (Here XYZ stands for a specific section
   name mentioned below, such as "Acknowledgements", "Dedications",
   "Endorsements", or "History".) To "Preserve the Title" of such a section when
   you modify the Document means that it remains a section "Entitled XYZ"
   according to this definition.

   The Document may include Warranty Disclaimers next to the notice which states
   that this License applies to the Document. These Warranty Disclaimers are
   considered to be included by reference in this License, but only as regards
   disclaiming warranties: any other implication that these Warranty Disclaimers
   may have is void and has no effect on the meaning of this License.

   __1__ VERBATIM COPYING

   You may copy and distribute the Document in any medium, either commercially
   or noncommercially, provided that this License, the copyright notices, and
   the license notice saying this License applies to the Document are reproduced
   in all copies, and that you add no other conditions whatsoever to those of
   this License. You may not use technical measures to obstruct or control the
   reading or further copying of the copies you make or distribute. However, you
   may accept compensation in exchange for copies. If you distribute a large
   enough number of copies you must also follow the conditions in section 3.

   You may also lend copies, under the same conditions stated above, and you may
   publicly display copies.

   __1__ COPYING IN QUANTITY

   If you publish printed copies (or copies in media that commonly have printed
   covers) of the Document, numbering more than 100, and the Document's license
   notice requires Cover Texts, you must enclose the copies in covers that
   carry, clearly and legibly, all these Cover Texts: Front-Cover Texts on the
   front cover, and Back-Cover Texts on the back cover. Both covers must also
   clearly and legibly identify you as the publisher of these copies. The front
   cover must present the full title with all words of the title equally
   prominent and visible. You may add other material on the covers in addition.
   Copying with changes limited to the covers, as long as they preserve the
   title of the Document and satisfy these conditions, can be treated as
   verbatim copying in other respects.

   If the required texts for either cover are too voluminous to fit legibly, you
   should put the first ones listed (as many as fit reasonably) on the actual
   cover, and continue the rest onto adjacent pages.

   If you publish or distribute Opaque copies of the Document numbering more
   than 100, you must either include a machine-readable Transparent copy along
   with each Opaque copy, or state in or with each Opaque copy a
   computer-network location from which the general network-using public has
   access to download using public-standard network protocols a complete
   Transparent copy of the Document, free of added material. If you use the
   latter option, you must take reasonably prudent steps, when you begin
   distribution of Opaque copies in quantity, to ensure that this Transparent
   copy will remain thus accessible at the stated location until at least one
   year after the last time you distribute an Opaque copy (directly or through
   your agents or retailers) of that edition to the public.

   It is requested, but not required, that you contact the authors of the
   Document well before redistributing any large number of copies, to give them
   a chance to provide you with an updated version of the Document.

   __1__ MODIFICATIONS

   You may copy and distribute a Modified Version of the Document under the
   conditions of sections 2 and 3 above, provided that you release the Modified
   Version under precisely this License, with the Modified Version filling the
   role of the Document, thus licensing distribution and modification of the
   Modified Version to whoever possesses a copy of it. In addition, you must do
   these things in the Modified Version:

      __1__ Use in the Title Page (and on the covers, if any) a title distinct
      from that of the Document, and from those of previous versions (which
      should, if there were any, be listed in the History section of the
      Document). You may use the same title as a previous version if the
      original publisher of that version gives permission.

      __1__ List on the Title Page, as authors, one or more persons or entities
      responsible for authorship of the modifications in the Modified Version,
      together with at least five of the principal authors of the Document (all
      of its principal authors, if it has fewer than five), unless they release
      you from this requirement.

      __1__ State on the Title page the name of the publisher of the Modified
      Version, as the publisher.

      __1__ Preserve all the copyright notices of the Document.

      __1__ Add an appropriate copyright notice for your modifications adjacent
      to the other copyright notices.

      __1__ Include, immediately after the copyright notices, a license notice
      giving the public permission to use the Modified Version under the terms
      of this License, in the form shown in the Addendum below.

      __1__ Preserve in that license notice the full lists of Invariant Sections
      and required Cover Texts given in the Document's license notice.

      __1__ Include an unaltered copy of this License.

      __1__ Preserve the section Entitled "History", Preserve its Title, and add
      to it an item stating at least the title, year, new authors, and publisher
      of the Modified Version as given on the Title Page. If there is no section
      Entitled "History" in the Document, create one stating the title, year,
      authors, and publisher of the Document as given on its Title Page, then
      add an item describing the Modified Version as stated in the previous
      sentence.

      __1__ Preserve the network location, if any, given in the Document for
      public access to a Transparent copy of the Document, and likewise the
      network locations given in the Document for previous versions it was based
      on. These may be placed in the "History" section. You may omit a network
      location for a work that was published at least four years before the
      Document itself, or if the original publisher of the version it refers to
      gives permission.

      __1__ For any section Entitled "Acknowledgements" or "Dedications",
      Preserve the Title of the section, and preserve in the section all the
      substance and tone of each of the contributor acknowledgements and/or
      dedications given therein.

      __1__ Preserve all the Invariant Sections of the Document, unaltered in
      their text and in their titles. Section numbers or the equivalent are not
      considered part of the section titles.

      __1__ Delete any section Entitled "Endorsements". Such a section may not
      be included in the Modified Version.

      __1__ Do not retitle any existing section to be Entitled "Endorsements" or
      to conflict in title with any Invariant Section.

      __1__ Preserve any Warranty Disclaimers.

   If the Modified Version includes new front-matter sections or appendices that
   qualify as Secondary Sections and contain no material copied from the
   Document, you may at your option designate some or all of these sections as
   invariant. To do this, add their titles to the list of Invariant Sections in
   the Modified Version's license notice. These titles must be distinct from any
   other section titles.

   You may add a section Entitled "Endorsements", provided it contains nothing
   but endorsements of your Modified Version by various parties--for example,
   statements of peer review or that the text has been approved by an
   organization as the authoritative definition of a standard.

   You may add a passage of up to five words as a Front-Cover Text, and a
   passage of up to 25 words as a Back-Cover Text, to the end of the list of
   Cover Texts in the Modified Version. Only one passage of Front-Cover Text and
   one of Back-Cover Text may be added by (or through arrangements made by) any
   one entity. If the Document already includes a cover text for the same cover,
   previously added by you or by arrangement made by the same entity you are
   acting on behalf of, you may not add another; but you may replace the old
   one, on explicit permission from the previous publisher that added the old
   one.

   The author(s) and publisher(s) of the Document do not by this License give
   permission to use their names for publicity for or to assert or imply
   endorsement of any Modified Version.

   __1__ COMBINING DOCUMENTS

   You may combine the Document with other documents released under this
   License, under the terms defined in section 4 above for modified versions,
   provided that you include in the combination all of the Invariant Sections of
   all of the original documents, unmodified, and list them all as Invariant
   Sections of your combined work in its license notice, and that you preserve
   all their Warranty Disclaimers.

   The combined work need only contain one copy of this License, and multiple
   identical Invariant Sections may be replaced with a single copy. If there are
   multiple Invariant Sections with the same name but different contents, make
   the title of each such section unique by adding at the end of it, in
   parentheses, the name of the original author or publisher of that section if
   known, or else a unique number. Make the same adjustment to the section
   titles in the list of Invariant Sections in the license notice of the
   combined work.

   In the combination, you must combine any sections Entitled "History" in the
   various original documents, forming one section Entitled "History"; likewise
   combine any sections Entitled "Acknowledgements", and any sections Entitled
   "Dedications". You must delete all sections Entitled "Endorsements".

   __1__ COLLECTIONS OF DOCUMENTS

   You may make a collection consisting of the Document and other documents
   released under this License, and replace the individual copies of this
   License in the various documents with a single copy that is included in the
   collection, provided that you follow the rules of this License for verbatim
   copying of each of the documents in all other respects.

   You may extract a single document from such a collection, and distribute it
   individually under this License, provided you insert a copy of this License
   into the extracted document, and follow this License in all other respects
   regarding verbatim copying of that document.

   __1__ AGGREGATION WITH INDEPENDENT WORKS

   A compilation of the Document or its derivatives with other separate and
   independent documents or works, in or on a volume of a storage or
   distribution medium, is called an "aggregate" if the copyright resulting from
   the compilation is not used to limit the legal rights of the compilation's
   users beyond what the individual works permit. When the Document is included
   in an aggregate, this License does not apply to the other works in the
   aggregate which are not themselves derivative works of the Document.

   If the Cover Text requirement of section 3 is applicable to these copies of
   the Document, then if the Document is less than one half of the entire
   aggregate, the Document's Cover Texts may be placed on covers that bracket
   the Document within the aggregate, or the electronic equivalent of covers if
   the Document is in electronic form. Otherwise they must appear on printed
   covers that bracket the whole aggregate.

   __1__ TRANSLATION

   Translation is considered a kind of modification, so you may distribute
   translations of the Document under the terms of section 4. Replacing
   Invariant Sections with translations requires special permission from their
   copyright holders, but you may include translations of some or all Invariant
   Sections in addition to the original versions of these Invariant Sections.
   You may include a translation of this License, and all the license notices in
   the Document, and any Warranty Disclaimers, provided that you also include
   the original English version of this License and the original versions of
   those notices and disclaimers. In case of a disagreement between the
   translation and the original version of this License or a notice or
   disclaimer, the original version will prevail.

   If a section in the Document is Entitled "Acknowledgements", "Dedications",
   or "History", the requirement (section 4) to Preserve its Title (section 1)
   will typically require changing the actual title.

   __1__ TERMINATION

   You may not copy, modify, sublicense, or distribute the Document except as
   expressly provided for under this License. Any other attempt to copy, modify,
   sublicense or distribute the Document is void, and will automatically
   terminate your rights under this License. However, parties who have received
   copies, or rights, from you under this License will not have their licenses
   terminated so long as such parties remain in full compliance.

   __1__ FUTURE REVISIONS OF THIS LICENSE

   The Free Software Foundation may publish new, revised versions of the GNU
   Free Documentation License from time to time. Such new versions will be
   similar in spirit to the present version, but may differ in detail to address
   new problems or concerns. See http:/www.gnu.org/copyleft/.

   Each version of the License is given a distinguishing version number. If the
   Document specifies that a particular numbered version of this License "or any
   later version" applies to it, you have the option of following the terms and
   conditions either of that specified version or of any later version that has
   been published (not as a draft) by the Free Software Foundation. If the
   Document does not specify a version number of this License, you may choose
   any version ever published (not as a draft) by the Free Software
   Foundation.

(( ADDENDUM: How to use this License for your documents

To use this License in a document you have written, include a copy of the
License in the document and put the following copyright and license notices just
after the title page:

Copyright (c) __10__. Permission is granted to copy, distribute and/or
modify this document under the terms of the GNU Free Documentation License,
Version 1.2 or any later version published by the Free Software Foundation; with
no Invariant Sections, no Front-Cover Texts, and no Back-Cover Texts. A copy of
the license is included in the section entitled "GNU Free Documentation
License".

If you have Invariant Sections, Front-Cover Texts and Back-Cover Texts, replace
the "with...Texts." line with this:

with the Invariant Sections being LIST THEIR TITLES, with the Front-Cover Texts
being LIST, and with the Back-Cover Texts being LIST.

If you have Invariant Sections without Cover Texts, or some other combination of
the three, merge those two alternatives to suit the situation.

If your document contains nontrivial examples of program code, we recommend
releasing these examples in parallel under your choice of free software license,
such as the GNU General Public License, to permit their use in free software.
))??
`
const license_GFDL_1_1_no_invariants_or_later_lre = ` 
	 
	 
	 

	Permission is granted to copy, distribute and/or
	modify this document under the terms of the GNU Free Documentation License,
	Version 1.1
	
		or any later version published by the Free Software Foundation;
	
	
		with no Invariant Sections,
		with no Front-Cover Texts,
		and with no Back-Cover Texts.
	
	A copy of the license is included in the section entitled
	"GNU Free Documentation License".
 `
const license_GFDL_1_1_no_invariants_only_lre = ` 
	 
	 
	 

	Permission is granted to copy, distribute and/or
	modify this document under the terms of the GNU Free Documentation License,
	Version 1.1
	
		((published by the Free Software Foundation))??
	
	
		with no Invariant Sections,
		with no Front-Cover Texts,
		and with no Back-Cover Texts.
	
	A copy of the license is included in the section entitled
	"GNU Free Documentation License".
 `
const license_GFDL_1_1_invariants_or_later_lre = ` 
	 
	 
	 

	Permission is granted to copy, distribute and/or
	modify this document under the terms of the GNU Free Documentation License,
	Version 1.1
	
		or any later version published by the Free Software Foundation;
	
	
		with
		((no Invariant Sections || the Invariant Sections being __20__))
		with
		((no Front-Cover Texts || the Front-Cover Texts being __20__))
		and with
		((no Back-Cover Texts || the Back-Cover Texts being __20__))
	
	A copy of the license is included in the section entitled
	"GNU Free Documentation License".
 `
const license_GFDL_1_1_invariants_only_lre = ` 
	 
	 
	 

	Permission is granted to copy, distribute and/or
	modify this document under the terms of the GNU Free Documentation License,
	Version 1.1
	
		((published by the Free Software Foundation))??
	
	
		with
		((no Invariant Sections || the Invariant Sections being __20__))
		with
		((no Front-Cover Texts || the Front-Cover Texts being __20__))
		and with
		((no Back-Cover Texts || the Back-Cover Texts being __20__))
	
	A copy of the license is included in the section entitled
	"GNU Free Documentation License".
 `
const license_GFDL_1_1_lre = `//**
GNU Free Documentation License v1.1 or later
https://spdx.org/licenses/GFDL-1.1-or-later.json
https://www.gnu.org/licenses/old-licenses/fdl-1.1.txt
**//


















((
	GNU Free Documentation License Version 1.1, March 2000
	
	((
		((
			Copyright __20__
			((<https://fsf.org/>))??
			
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))??
		))??

		Everyone is permitted to copy and distribute verbatim copies
		of this license document, but changing it is not allowed.

		((Copyright __20__))??
	))??

))??

   ((0. || 1.))??
   PREAMBLE

   The purpose of this License is to make a manual, textbook, or other written
   document "free" in the sense of freedom: to assure everyone the effective
   freedom to copy and redistribute it, with or without modifying it, either
   commercially or noncommercially. Secondarily, this License preserves for the
   author and publisher a way to get credit for their work, while not being
   considered responsible for modifications made by others.

   This License is a kind of "copyleft", which means that derivative works of
   the document must themselves be free in the same sense. It complements the
   GNU General Public License, which is a copyleft license designed for free
   software.

   We have designed this License in order to use it for manuals for free
   software, because free software needs free documentation: a free program
   should come with manuals providing the same freedoms that the software does.
   But this License is not limited to software manuals; it can be used for any
   textual work, regardless of subject matter or whether it is published as a
   printed book. We recommend this License principally for works whose purpose
   is instruction or reference.

   __1__ APPLICABILITY AND DEFINITIONS

   This License applies to any manual or other work that contains a notice
   placed by the copyright holder saying it can be distributed under the terms
   of this License. The "Document", below, refers to any such manual or work.
   Any member of the public is a licensee, and is addressed as "you".

   A "Modified Version" of the Document means any work containing the Document
   or a portion of it, either copied verbatim, or with modifications and/or
   translated into another language.

   A "Secondary Section" is a named appendix or a front-matter section of the
   Document that deals exclusively with the relationship of the publishers or
   authors of the Document to the Document's overall subject (or to related
   matters) and contains nothing that could fall directly within that overall
   subject. (For example, if the Document is in part a textbook of mathematics,
   a Secondary Section may not explain any mathematics.) The relationship could
   be a matter of historical connection with the subject or with related
   matters, or of legal, commercial, philosophical, ethical or political
   position regarding them.

   The "Invariant Sections" are certain Secondary Sections whose titles are
   designated, as being those of Invariant Sections, in the notice that says
   that the Document is released under this License.

   The "Cover Texts" are certain short passages of text that are listed, as
   Front-Cover Texts or Back-Cover Texts, in the notice that says that the
   Document is released under this License.

   A "Transparent" copy of the Document means a machine-readable copy,
   represented in a format whose specification is available to the general
   public, whose contents can be viewed and edited directly and
   straightforwardly with generic text editors or (for images composed of
   pixels) generic paint programs or (for drawings) some widely available
   drawing editor, and that is suitable for input to text formatters or for
   automatic translation to a variety of formats suitable for input to text
   formatters. A copy made in an otherwise Transparent file format whose markup
   has been designed to thwart or discourage subsequent modification by readers
   is not Transparent. A copy that is not "Transparent" is called "Opaque".

   Examples of suitable formats for Transparent copies include plain ASCII
   without markup, Texinfo input format, LaTeX input format, SGML or XML using a
   publicly available DTD, and standard-conforming simple HTML designed for
   human modification. Opaque formats include PostScript, PDF, proprietary
   formats that can be read and edited only by proprietary word processors, SGML
   or XML for which the DTD and/or processing tools are not generally available,
   and the machine-generated HTML produced by some word processors for output
   purposes only.

   The "Title Page" means, for a printed book, the title page itself, plus such
   following pages as are needed to hold, legibly, the material this License
   requires to appear in the title page. For works in formats which do not have
   any title page as such, "Title Page" means the text near the most prominent
   appearance of the work's title, preceding the beginning of the body of the
   text.

   __1__ VERBATIM COPYING

   You may copy and distribute the Document in any medium, either commercially
   or noncommercially, provided that this License, the copyright notices, and
   the license notice saying this License applies to the Document are reproduced
   in all copies, and that you add no other conditions whatsoever to those of
   this License. You may not use technical measures to obstruct or control the
   reading or further copying of the copies you make or distribute. However, you
   may accept compensation in exchange for copies. If you distribute a large
   enough number of copies you must also follow the conditions in section 3.

   You may also lend copies, under the same conditions stated above, and you may
   publicly display copies.

   __1__ COPYING IN QUANTITY

   If you publish printed copies of the Document numbering more than 100, and
   the Document's license notice requires Cover Texts, you must enclose the
   copies in covers that carry, clearly and legibly, all these Cover Texts:
   Front-Cover Texts on the front cover, and Back-Cover Texts on the back cover.
   Both covers must also clearly and legibly identify you as the publisher of
   these copies. The front cover must present the full title with all words of
   the title equally prominent and visible. You may add other material on the
   covers in addition. Copying with changes limited to the covers, as long as
   they preserve the title of the Document and satisfy these conditions, can be
   treated as verbatim copying in other respects.

   If the required texts for either cover are too voluminous to fit legibly, you
   should put the first ones listed (as many as fit reasonably) on the actual
   cover, and continue the rest onto adjacent pages.

   If you publish or distribute Opaque copies of the Document numbering more
   than 100, you must either include a machine-readable Transparent copy along
   with each Opaque copy, or state in or with each Opaque copy a
   publicly-accessible computer-network location containing a complete
   Transparent copy of the Document, free of added material, which the general
   network-using public has access to download anonymously at no charge using
   public-standard network protocols. If you use the latter option, you must
   take reasonably prudent steps, when you begin distribution of Opaque copies
   in quantity, to ensure that this Transparent copy will remain thus accessible
   at the stated location until at least one year after the last time you
   distribute an Opaque copy (directly or through your agents or retailers) of
   that edition to the public.

   It is requested, but not required, that you contact the authors of the
   Document well before redistributing any large number of copies, to give them
   a chance to provide you with an updated version of the Document.

   __1__ MODIFICATIONS

   You may copy and distribute a Modified Version of the Document under the
   conditions of sections 2 and 3 above, provided that you release the Modified
   Version under precisely this License, with the Modified Version filling the
   role of the Document, thus licensing distribution and modification of the
   Modified Version to whoever possesses a copy of it. In addition, you must do
   these things in the Modified Version:

      __1__ Use in the Title Page (and on the covers, if any) a title distinct
      from that of the Document, and from those of previous versions (which
      should, if there were any, be listed in the History section of the
      Document). You may use the same title as a previous version if the
      original publisher of that version gives permission.

      __1__ List on the Title Page, as authors, one or more persons or entities
      responsible for authorship of the modifications in the Modified Version,
      together with at least five of the principal authors of the Document (all
      of its principal authors, if it has less than five).

      __1__ State on the Title page the name of the publisher of the Modified
      Version, as the publisher.

      __1__ Preserve all the copyright notices of the Document.

      __1__ Add an appropriate copyright notice for your modifications adjacent
      to the other copyright notices.

      __1__ Include, immediately after the copyright notices, a license notice
      giving the public permission to use the Modified Version under the terms
      of this License, in the form shown in the Addendum below.

      __1__ Preserve in that license notice the full lists of Invariant Sections
      and required Cover Texts given in the Document's license notice.

      __1__ Include an unaltered copy of this License.

      __1__ Preserve the section entitled "History", and its title, and add to
      it an item stating at least the title, year, new authors, and publisher of
      the Modified Version as given on the Title Page. If there is no section
      entitled "History" in the Document, create one stating the title, year,
      authors, and publisher of the Document as given on its Title Page, then
      add an item describing the Modified Version as stated in the previous
      sentence.

      __1__ Preserve the network location, if any, given in the Document for
      public access to a Transparent copy of the Document, and likewise the
      network locations given in the Document for previous versions it was based
      on. These may be placed in the "History" section. You may omit a network
      location for a work that was published at least four years before the
      Document itself, or if the original publisher of the version it refers to
      gives permission.

      __1__ In any section entitled "Acknowledgements" or "Dedications",
      preserve the section's title, and preserve in the section all the
      substance and tone of each of the contributor acknowledgements and/or
      dedications given therein.

      __1__ Preserve all the Invariant Sections of the Document, unaltered in
      their text and in their titles. Section numbers or the equivalent are not
      considered part of the section titles.

      __1__ Delete any section entitled "Endorsements". Such a section may not
      be included in the Modified Version.

      __1__ Do not retitle any existing section as "Endorsements" or to conflict
      in title with any Invariant Section.

   If the Modified Version includes new front-matter sections or appendices that
   qualify as Secondary Sections and contain no material copied from the
   Document, you may at your option designate some or all of these sections as
   invariant. To do this, add their titles to the list of Invariant Sections in
   the Modified Version's license notice. These titles must be distinct from any
   other section titles.

   You may add a section entitled "Endorsements", provided it contains nothing
   but endorsements of your Modified Version by various parties--for example,
   statements of peer review or that the text has been approved by an
   organization as the authoritative definition of a standard.

   You may add a passage of up to five words as a Front-Cover Text, and a
   passage of up to 25 words as a Back-Cover Text, to the end of the list of
   Cover Texts in the Modified Version. Only one passage of Front-Cover Text and
   one of Back-Cover Text may be added by (or through arrangements made by) any
   one entity. If the Document already includes a cover text for the same cover,
   previously added by you or by arrangement made by the same entity you are
   acting on behalf of, you may not add another; but you may replace the old
   one, on explicit permission from the previous publisher that added the old
   one.

   The author(s) and publisher(s) of the Document do not by this License give
   permission to use their names for publicity for or to assert or imply
   endorsement of any Modified Version.

   __1__ COMBINING DOCUMENTS

   You may combine the Document with other documents released under this
   License, under the terms defined in section 4 above for modified versions,
   provided that you include in the combination all of the Invariant Sections of
   all of the original documents, unmodified, and list them all as Invariant
   Sections of your combined work in its license notice.

   The combined work need only contain one copy of this License, and multiple
   identical Invariant Sections may be replaced with a single copy. If there are
   multiple Invariant Sections with the same name but different contents, make
   the title of each such section unique by adding at the end of it, in
   parentheses, the name of the original author or publisher of that section if
   known, or else a unique number. Make the same adjustment to the section
   titles in the list of Invariant Sections in the license notice of the
   combined work.

   In the combination, you must combine any sections entitled "History" in the
   various original documents, forming one section entitled "History"; likewise
   combine any sections entitled "Acknowledgements", and any sections entitled
   "Dedications". You must delete all sections entitled "Endorsements."

   __1__ COLLECTIONS OF DOCUMENTS

   You may make a collection consisting of the Document and other documents
   released under this License, and replace the individual copies of this
   License in the various documents with a single copy that is included in the
   collection, provided that you follow the rules of this License for verbatim
   copying of each of the documents in all other respects.

   You may extract a single document from such a collection, and distribute it
   individually under this License, provided you insert a copy of this License
   into the extracted document, and follow this License in all other respects
   regarding verbatim copying of that document.

   __1__ AGGREGATION WITH INDEPENDENT WORKS

   A compilation of the Document or its derivatives with other separate and
   independent documents or works, in or on a volume of a storage or
   distribution medium, does not as a whole count as a Modified Version of the
   Document, provided no compilation copyright is claimed for the compilation.
   Such a compilation is called an "aggregate", and this License does not apply
   to the other self-contained works thus compiled with the Document, on account
   of their being thus compiled, if they are not themselves derivative works of
   the Document.

   If the Cover Text requirement of section 3 is applicable to these copies of
   the Document, then if the Document is less than one quarter of the entire
   aggregate, the Document's Cover Texts may be placed on covers that surround
   only the Document within the aggregate. Otherwise they must appear on covers
   around the whole aggregate.

   __1__ TRANSLATION

   Translation is considered a kind of modification, so you may distribute
   translations of the Document under the terms of section 4. Replacing
   Invariant Sections with translations requires special permission from their
   copyright holders, but you may include translations of some or all Invariant
   Sections in addition to the original versions of these Invariant Sections.
   You may include a translation of this License provided that you also include
   the original English version of this License. In case of a disagreement
   between the translation and the original English version of this License, the
   original English version will prevail.

   __1__ TERMINATION

   You may not copy, modify, sublicense, or distribute the Document except as
   expressly provided for under this License. Any other attempt to copy, modify,
   sublicense or distribute the Document is void, and will automatically
   terminate your rights under this License. However, parties who have received
   copies, or rights, from you under this License will not have their licenses
   terminated so long as such parties remain in full compliance.

   __1__ FUTURE REVISIONS OF THIS LICENSE

   The Free Software Foundation may publish new, revised versions of the GNU
   Free Documentation License from time to time. Such new versions will be
   similar in spirit to the present version, but may differ in detail to address
   new problems or concerns. See http:/www.gnu.org/copyleft/.

   Each version of the License is given a distinguishing version number. If the
   Document specifies that a particular numbered version of this License "or any
   later version" applies to it, you have the option of following the terms and
   conditions either of that specified version or of any later version that has
   been published (not as a draft) by the Free Software Foundation. If the
   Document does not specify a version number of this License, you may choose
   any version ever published (not as a draft) by the Free Software
   Foundation.

((  ADDENDUM: How to use this License for your documents

To use this License in a document you have written, include a copy of the
License in the document and put the following copyright and license notices just
after the title page:

Copyright (c) __10__.

Permission is granted to copy, distribute and/or
modify this document under the terms of the GNU Free Documentation License,
Version 1.1 or any later version published by the Free Software Foundation; with
the Invariant Sections being LIST THEIR TITLES, with the Front-Cover Texts being
LIST, and with the Back-Cover Texts being LIST. A copy of the license is
included in the section entitled "GNU Free Documentation License".

If you have no Invariant Sections, write "with no Invariant Sections" instead of
saying which ones are invariant. If you have no Front-Cover Texts, write "no
Front-Cover Texts" instead of "Front-Cover Texts being LIST"; likewise for
Back-Cover Texts.

If your document contains nontrivial examples of program code, we recommend
releasing these examples in parallel under your choice of free software license,
such as the GNU General Public License, to permit their use in free software.
))??
`
const license_GL2PS_lre = `//**
GL2PS License
https://spdx.org/licenses/GL2PS.json
http://www.geuz.org/gl2ps/COPYING.GL2PS
**//

(( GL2PS LICENSE Version 2, November 2003
(( Copyright __20__ ))??
))??

Permission to use, copy, and distribute this software and its documentation for
any purpose with or without fee is hereby granted, provided that the copyright
notice appear in all copies and that both that copyright notice and this
permission notice appear in supporting documentation.

Permission to modify and distribute modified versions of this software is
granted, provided that:

   (( 1) ))??
   the modifications are licensed under the same terms as this software;

   (( 2) ))??
   you make available the source code of any modifications that you distribute,
   either on the same media as you distribute any executable or other form of
   this software, or via a mechanism generally accepted in the software
   development community for the electronic transfer of data.

This software is provided "as is" without express or implied warranty.
`
const license_GLWTPL_lre = `//**
Good Luck With That Public License
https://spdx.org/licenses/GLWTPL.json
https://github.com/me-shaon/GLWTPL/commit/da5f6bc734095efbacb442c0b31e33a65b9d6e85
**//

(( GLWT(Good Luck With That) Public License
(( Copyright __20__ ))??
))??

Everyone is permitted to copy, distribute, modify, merge, sell, publish,
sublicense or whatever they want with this software but at their OWN RISK.

Preamble

The author has absolutely no clue what the code in this project does. It might
just work or not, there is no third option.

GOOD LUCK WITH THAT PUBLIC LICENSE

TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION, AND MODIFICATION

   (( 0. ))??
   You just DO WHATEVER YOU WANT TO as long as you NEVER LEAVE A TRACE TO TRACK
   THE AUTHOR of the original product to blame for or hold responsible.

IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

Good luck and Godspeed.
`
const license_GPL_1_0_lre = `//**
GNU General Public License v1.0
https://www.gnu.org/licenses/old-licenses/gpl-1.0-standalone.html
**//

((
	GNU GENERAL PUBLIC LICENSE Version 1, February 1989
	
	((
		((
			Copyright __20__
			((<https://fsf.org/>))??
			
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))??
		))??

		Everyone is permitted to copy and distribute verbatim copies
		of this license document, but changing it is not allowed.

		((Copyright __20__))??
	))??

))??


Preamble

The license agreements of most software companies try to keep users at the mercy
of those companies. By contrast, our General Public License is intended to
guarantee your freedom to share and change free software--to make sure the
software is free for all its users. The General Public License applies to the
Free Software Foundation's software and to any other program whose authors
commit to using it. You can use it for your programs, too.

When we speak of free software, we are referring to freedom, not price.
Specifically, the General Public License is designed to make sure that you have
the freedom to give away or sell copies of free software, that you receive
source code or can get it if you want it, that you can change the software or
use pieces of it in new free programs; and that you know you can do these
things.

To protect your rights, we need to make restrictions that forbid anyone to deny
you these rights or to ask you to surrender the rights. These restrictions
translate to certain responsibilities for you if you distribute copies of the
software, or if you modify it.

For example, if you distribute copies of a such a program, whether gratis or for
a fee, you must give the recipients all the rights that you have. You must make
sure that they, too, receive or can get the source code. And you must tell them
their rights.

We protect your rights with two steps: (1) copyright the software, and (2) offer
you this license which gives you legal permission to copy, distribute and/or
modify the software.

Also, for each author's protection and ours, we want to make certain that
everyone understands that there is no warranty for this free software. If the
software is modified by someone else and passed on, we want its recipients to
know that what they have is not the original, so that any problems introduced by
others will not reflect on the original authors' reputations.

The precise terms and conditions for copying, distribution and modification
follow.

GNU GENERAL PUBLIC LICENSE TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND
MODIFICATION



   __1__ This License Agreement applies to any program or other work which
   contains a notice placed by the copyright holder saying it may be distributed
   under the terms of this General Public License. The "Program", below, refers
   to any such program or work, and a "work based on the Program" means either
   the Program or any work containing the Program or a portion of it, either
   verbatim or with modifications. Each licensee is addressed as "you".



   __1__ You may copy and distribute verbatim copies of the Program's source
   code as you receive it, in any medium, provided that you conspicuously and
   appropriately publish on each copy an appropriate copyright notice and
   disclaimer of warranty; keep intact all the notices that refer to this
   General Public License and to the absence of any warranty; and give any other
   recipients of the Program a copy of this General Public License along with
   the Program. You may charge a fee for the physical act of transferring a
   copy.



   __1__ You may modify your copy or copies of the Program or any portion of it,
   and copy and distribute such modifications under the terms of Paragraph 1
   above, provided that you also do the following:

      __1__ cause the modified files to carry prominent notices stating that you
      changed the files and the date of any change; and

      __1__ cause the whole of any work that you distribute or publish, that in
      whole or in part contains the Program or any part thereof, either with or
      without modifications, to be licensed at no charge to all third parties
      under the terms of this General Public License (except that you may choose
      to grant warranty protection to some or all third parties, at your
      option).

      __1__ If the modified program normally reads commands interactively when
      run, you must cause it, when started running for such interactive use in
      the simplest and most usual way, to print or display an announcement
      including an appropriate copyright notice and a notice that there is no
      warranty (or else, saying that you provide a warranty) and that users may
      redistribute the program under these conditions, and telling the user how
      to view a copy of this General Public License.

      __1__ You may charge a fee for the physical act of transferring a copy,
      and you may at your option offer warranty protection in exchange for a
      fee.

   Mere aggregation of another independent work with the Program (or its
   derivative) on a volume of a storage or distribution medium does not bring
   the other work under the scope of these terms.



   __1__ You may copy and distribute the Program (or a portion or derivative of
   it, under Paragraph 2) in object code or executable form under the terms of
   Paragraphs 1 and 2 above provided that you also do one of the following:

      __1__ accompany it with the complete corresponding machine-readable source
      code, which must be distributed under the terms of Paragraphs 1 and 2
      above; or,

      __1__ accompany it with a written offer, valid for at least three years,
      to give any third party free (except for a nominal charge for the cost of
      distribution) a complete machine-readable copy of the corresponding source
      code, to be distributed under the terms of Paragraphs 1 and 2 above; or,

      __1__ accompany it with the information you received as to where the
      corresponding source code may be obtained. (This alternative is allowed
      only for noncommercial distribution and only if you received the program
      in object code or executable form alone.)

   Source code for a work means the preferred form of the work for making
   modifications to it. For an executable file, complete source code means all
   the source code for all modules it contains; but, as a special exception, it
   need not include source code for modules which are standard libraries that
   accompany the operating system on which the executable file runs, or for
   standard header files or definitions files that accompany that operating
   system.



   __1__ You may not copy, modify, sublicense, distribute or transfer the
   Program except as expressly provided under this General Public License. Any
   attempt otherwise to copy, modify, sublicense, distribute or transfer the
   Program is void, and will automatically terminate your rights to use the
   Program under this License. However, parties who have received copies, or
   rights to use copies, from you under this General Public License will not
   have their licenses terminated so long as such parties remain in full
   compliance.



   __1__ By copying, distributing or modifying the Program (or any work based on
   the Program) you indicate your acceptance of this license to do so, and all
   its terms and conditions.



   __1__ Each time you redistribute the Program (or any work based on the
   Program), the recipient automatically receives a license from the original
   licensor to copy, distribute or modify the Program subject to these terms and
   conditions. You may not impose any further restrictions on the recipients'
   exercise of the rights granted herein.



   __1__ The Free Software Foundation may publish revised and/or new versions of
   the General Public License from time to time. Such new versions will be
   similar in spirit to the present version, but may differ in detail to address
   new problems or concerns.

   Each version is given a distinguishing version number. If the Program
   specifies a version number of the license which applies to it and "any later
   version", you have the option of following the terms and conditions either of
   that version or of any later version published by the Free Software
   Foundation. If the Program does not specify a version number of the license,
   you may choose any version ever published by the Free Software Foundation.



   __1__ If you wish to incorporate parts of the Program into other free
   programs whose distribution conditions are different, write to the author to
   ask for permission. For software which is copyrighted by the Free Software
   Foundation, write to the Free Software Foundation; we sometimes make
   exceptions for this. Our decision will be guided by the two goals of
   preserving the free status of all derivatives of our free software and of
   promoting the sharing and reuse of software generally.



   NO WARRANTY

   __1__ BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
   FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
   OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
   PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR
   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO
   THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM
   PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR
   CORRECTION.



   __1__ IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
   WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
   REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
   INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
   OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO
   LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR
   THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
   PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGES.

((  END OF TERMS AND CONDITIONS))??

((
Appendix: How to Apply These Terms to Your New Programs

If you develop a new program, and you want it to be of the greatest possible use
to humanity, the best way to achieve this is to make it free software which
everyone can redistribute and change under these terms.

To do so, attach the following notices to the program. It is safest to attach
them to the start of each source file to most effectively convey the exclusion
of warranty; and each file should have at least the "copyright" line and a
pointer to where the full notice is found.

__30__
//**
<one line to give the program's name and a brief idea of what it does.>
Copyright (C) <year> <name of author>
**//

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.

Also add information on how to contact you by electronic and paper mail.

If the program is interactive, make it output a short notice like this when it
starts in an interactive mode:

Gnomovision version 69, Copyright (C) 19xx name of author Gnomovision comes with
ABSOLUTELY NO WARRANTY; for details type ` + "`" + `show w'. This is free software, and
you are welcome to redistribute it under certain conditions; type ` + "`" + `show c' for
details.

The hypothetical commands ` + "`" + `show w' and ` + "`" + `show c' should show the appropriate
parts of the General Public License. Of course, the commands you use may be
called something other than ` + "`" + `show w' and ` + "`" + `show c'; they could even be
mouse-clicks or menu items--whatever suits your program.

You should also get your employer (if you work as a programmer) or your school,
if any, to sign a "copyright disclaimer" for the program, if necessary. Here a
sample; alter the names:

Yoyodyne, Inc., hereby disclaims all copyright interest in the program
` + "`" + `Gnomovision' (a program to direct compilers to make passes at assemblers)
written by James Hacker.

__5__ //**<signature of Ty Coon>**//, 1 April 1989 Ty Coon, President of Vice

That's all there is to it! ))??
`
const license_GPL_1_0_only_lre = `
//**
https://spdx.org/licenses/GPL-1.0-only.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		program
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	GNU
	General Public License
	((GPL))??
	((as published by the Free Software Foundation))??
	
		((under))??
		version 1
		((GPLv1))??
		((of the License))??
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		GNU
		General Public License version
		1
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		GNU
		General Public License for more details.

		((
			You should have received a copy of the
			GNU
			General Public License
			((
				((version))??
				1
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_GPL_1_0_or_later_lre = `
//**
https://spdx.org/licenses/GPL-1.0-or-later.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		program
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	GNU
	General Public License
	((GPL))??
	((as published by the Free Software Foundation))??
	
		((
			either version 1
			((of the License))??
			or
			((at your option))??
			any later version.
		||
			version 1 or later
			((of the License))??
		))
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		GNU
		General Public License version
		1
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		GNU
		General Public License for more details.

		((
			You should have received a copy of the
			GNU
			General Public License
			((
				((version))??
				1
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_GPL_2_0_lre = `//**
GNU General Public License v2.0
https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
https://opensource.org/licenses/GPL-2.0
**//

((
	GNU GENERAL PUBLIC LICENSE Version 2, June 1991
	
	((
		((
			Copyright __20__
			((<https://fsf.org/>))??
			
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))??
		))??

		Everyone is permitted to copy and distribute verbatim copies
		of this license document, but changing it is not allowed.

		((Copyright __20__))??
	))??

))??

Preamble

The licenses for most software are designed to take away your freedom to share
and change it. By contrast, the GNU General Public License is intended to
guarantee your freedom to share and change free software--to make sure the
software is free for all its users. This General Public License applies to most
of the Free Software Foundation's software and to any other program whose
authors commit to using it. (Some other Free Software Foundation software is
covered by the GNU
((Lesser||Library))
General Public License instead.) You can apply it to
your programs, too.

When we speak of free software, we are referring to freedom, not price. Our
General Public Licenses are designed to make sure that you have the freedom to
distribute copies of free software (and charge for this service if you wish),
that you receive source code or can get it if you want it, that you can change
the software or use pieces of it in new free programs; and that you know you can
do these things.

To protect your rights, we need to make restrictions that forbid anyone to deny
you these rights or to ask you to surrender the rights. These restrictions
translate to certain responsibilities for you if you distribute copies of the
software, or if you modify it.

For example, if you distribute copies of such a program, whether gratis or for a
fee, you must give the recipients all the rights that you have. You must make
sure that they, too, receive or can get the source code. And you must show them
these terms so they know their rights.

We protect your rights with two steps: (1) copyright the software, and (2) offer
you this license which gives you legal permission to copy, distribute and/or
modify the software.

Also, for each author's protection and ours, we want to make certain that
everyone understands that there is no warranty for this free software. If the
software is modified by someone else and passed on, we want its recipients to
know that what they have is not the original, so that any problems introduced by
others will not reflect on the original authors' reputations.

Finally, any free program is threatened constantly by software patents. We wish
to avoid the danger that redistributors of a free program will individually
obtain patent licenses, in effect making the program proprietary. To prevent
this, we have made it clear that any patent must be licensed for everyone's free
use or not licensed at all.

The precise terms and conditions for copying, distribution and modification
follow.

__5__ TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

   __1__ This License applies to any program or other work which contains a
   notice placed by the copyright holder saying it may be distributed under the
   terms of this General Public License. The "Program", below, refers to any
   such program or work, and a "work based on the Program" means either the
   Program or any derivative work under copyright law: that is to say, a work
   containing the Program or a portion of it, either verbatim or with
   modifications and/or translated into another language. (Hereinafter,
   translation is included without limitation in the term "modification".) Each
   licensee is addressed as "you".

   Activities other than copying, distribution and modification are not covered
   by this License; they are outside its scope. The act of running the Program
   is not restricted, and the output from the Program is covered only if its
   contents constitute a work based on the Program (independent of having been
   made by running the Program). Whether that is true depends on what the
   Program does.

   __1__ You may copy and distribute verbatim copies of the Program's source
   code as you receive it, in any medium, provided that you conspicuously and
   appropriately publish on each copy an appropriate copyright notice and
   disclaimer of warranty; keep intact all the notices that refer to this
   License and to the absence of any warranty; and give any other recipients of
   the Program a copy of this License along with the Program.

   You may charge a fee for the physical act of transferring a copy, and you may
   at your option offer warranty protection in exchange for a fee.

   __1__ You may modify your copy or copies of the Program or any portion of it,
   thus forming a work based on the Program, and copy and distribute such
   modifications or work under the terms of Section 1 above, provided that you
   also meet all of these conditions:

      __1__ You must cause the modified files to carry prominent notices stating
      that you changed the files and the date of any change.

      __1__ You must cause any work that you distribute or publish, that in
      whole or in part contains or is derived from the Program or any part
      thereof, to be licensed as a whole at no charge to all third parties under
      the terms of this License.

      __1__ If the modified program normally reads commands interactively when
      run, you must cause it, when started running for such interactive use in
      the most ordinary way, to print or display an announcement including an
      appropriate copyright notice and a notice that there is no warranty (or
      else, saying that you provide a warranty) and that users may redistribute
      the program under these conditions, and telling the user how to view a
      copy of this License. (Exception: if the Program itself is interactive but
      does not normally print such an announcement, your work based on the
      Program is not required to print an announcement.)

   These requirements apply to the modified work as a whole. If identifiable
   sections of that work are not derived from the Program, and can be reasonably
   considered independent and separate works in themselves, then this License,
   and its terms, do not apply to those sections when you distribute them as
   separate works. But when you distribute the same sections as part of a whole
   which is a work based on the Program, the distribution of the whole must be
   on the terms of this License, whose permissions for other licensees extend to
   the entire whole, and thus to each and every part regardless of who wrote
   it.

   Thus, it is not the intent of this section to claim rights or contest your
   rights to work written entirely by you; rather, the intent is to exercise the
   right to control the distribution of derivative or collective works based on
   the Program.

   In addition, mere aggregation of another work not based on the Program with
   the Program (or with a work based on the Program) on a volume of a storage or
   distribution medium does not bring the other work under the scope of this
   License.

   __1__ You may copy and distribute the Program (or a work based on it, under
   Section 2) in object code or executable form under the terms of Sections 1
   and 2 above provided that you also do one of the following:

      __1__ Accompany it with the complete corresponding machine-readable source
      code, which must be distributed under the terms of Sections 1 and 2 above
      on a medium customarily used for software interchange; or,

      __1__ Accompany it with a written offer, valid for at least three years,
      to give any third party, for a charge no more than your cost of physically
      performing source distribution, a complete machine-readable copy of the
      corresponding source code, to be distributed under the terms of Sections 1
      and 2 above on a medium customarily used for software interchange; or,

      __1__ Accompany it with the information you received as to the offer to
      distribute corresponding source code. (This alternative is allowed only
      for noncommercial distribution and only if you received the program in
      object code or executable form with such an offer, in accord with
      Subsection b above.)

   The source code for a work means the preferred form of the work for making
   modifications to it. For an executable work, complete source code means all
   the source code for all modules it contains, plus any associated interface
   definition files, plus the scripts used to control compilation and
   installation of the executable. However, as a special exception, the source
   code distributed need not include anything that is normally distributed (in
   either source or binary form) with the major components (compiler, kernel,
   and so on) of the operating system on which the executable runs, unless that
   component itself accompanies the executable.

   If distribution of executable or object code is made by offering access to
   copy from a designated place, then offering equivalent access to copy the
   source code from the same place counts as distribution of the source code,
   even though third parties are not compelled to copy the source along with the
   object code.

   __1__ You may not copy, modify, sublicense, or distribute the Program except
   as expressly provided under this License. Any attempt otherwise to copy,
   modify, sublicense or distribute the Program is void, and will automatically
   terminate your rights under this License. However, parties who have received
   copies, or rights, from you under this License will not have their licenses
   terminated so long as such parties remain in full compliance.

   __1__ You are not required to accept this License, since you have not signed
   it. However, nothing else grants you permission to modify or distribute the
   Program or its derivative works. These actions are prohibited by law if you
   do not accept this License. Therefore, by modifying or distributing the
   Program (or any work based on the Program), you indicate your acceptance of
   this License to do so, and all its terms and conditions for copying,
   distributing or modifying the Program or works based on it.

   __1__ Each time you redistribute the Program (or any work based on the
   Program), the recipient automatically receives a license from the original
   licensor to copy, distribute or modify the Program subject to these terms and
   conditions. You may not impose any further restrictions on the recipients'
   exercise of the rights granted herein. You are not responsible for enforcing
   compliance by third parties to this License.

   __1__ If, as a consequence of a court judgment or allegation of patent
   infringement or for any other reason (not limited to patent issues),
   conditions are imposed on you (whether by court order, agreement or
   otherwise) that contradict the conditions of this License, they do not excuse
   you from the conditions of this License. If you cannot distribute so as to
   satisfy simultaneously your obligations under this License and any other
   pertinent obligations, then as a consequence you may not distribute the
   Program at all. For example, if a patent license would not permit
   royalty-free redistribution of the Program by all those who receive copies
   directly or indirectly through you, then the only way you could satisfy both
   it and this License would be to refrain entirely from distribution of the
   Program.

   If any portion of this section is held invalid or unenforceable under any
   particular circumstance, the balance of the section is intended to apply and
   the section as a whole is intended to apply in other circumstances.

   It is not the purpose of this section to induce you to infringe any patents
   or other property right claims or to contest validity of any such claims;
   this section has the sole purpose of protecting the integrity of the free
   software distribution system, which is implemented by public license
   practices. Many people have made generous contributions to the wide range of
   software distributed through that system in reliance on consistent
   application of that system; it is up to the author/donor to decide if he or
   she is willing to distribute software through any other system and a licensee
   cannot impose that choice.

   This section is intended to make thoroughly clear what is believed to be a
   consequence of the rest of this License.

   __1__ If the distribution and/or use of the Program is restricted in certain
   countries either by patents or by copyrighted interfaces, the original
   copyright holder who places the Program under this License may add an
   explicit geographical distribution limitation excluding those countries, so
   that distribution is permitted only in or among countries not thus excluded.
   In such case, this License incorporates the limitation as if written in the
   body of this License.

   __1__ The Free Software Foundation may publish revised and/or new versions of
   the General Public License from time to time. Such new versions will be
   similar in spirit to the present version, but may differ in detail to address
   new problems or concerns.

   Each version is given a distinguishing version number. If the Program
   specifies a version number of this License which applies to it and "any later
   version", you have the option of following the terms and conditions either of
   that version or of any later version published by the Free Software
   Foundation. If the Program does not specify a version number of this License,
   you may choose any version ever published by the Free Software Foundation.

   __1__ If you wish to incorporate parts of the Program into other free
   programs whose distribution conditions are different, write to the author to
   ask for permission. For software which is copyrighted by the Free Software
   Foundation, write to the Free Software Foundation; we sometimes make
   exceptions for this. Our decision will be guided by the two goals of
   preserving the free status of all derivatives of our free software and of
   promoting the sharing and reuse of software generally.

   NO WARRANTY

   __1__ BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
   FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
   OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
   PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR
   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO
   THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM
   PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR
   CORRECTION.

   __1__ IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
   WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
   REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
   INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
   OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO
   LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR
   THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
   PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGES.

((END OF TERMS AND CONDITIONS))??

((
How to Apply These Terms to Your New Programs

If you develop a new program, and you want it to be of the greatest possible use
to the public, the best way to achieve this is to make it free software which
everyone can redistribute and change under these terms.

To do so, attach the following notices to the program. It is safest to attach
them to the start of each source file to most effectively convey the exclusion
of warranty; and each file should have at least the "copyright" line and a
pointer to where the full notice is found.

__30__
//**
<one line to give the program's name and a brief idea of what it does.>
Copyright (C) <year> <name of author>
**//

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc.,

	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))

Also add information on how to contact you by electronic and paper mail.

If the program is interactive, make it output a short notice like this when it
starts in an interactive mode:

Gnomovision version 69, __10__

Gnomovision comes with
ABSOLUTELY NO WARRANTY; for details type ` + "`" + `show w'. This is free software, and
you are welcome to redistribute it under certain conditions; type ` + "`" + `show c' for
details.

The hypothetical commands ` + "`" + `show w' and ` + "`" + `show c' should show the appropriate
parts of the General Public License. Of course, the commands you use may be
called something other than ` + "`" + `show w' and ` + "`" + `show c'; they could even be
mouse-clicks or menu items--whatever suits your program.

You should also get your employer (if you work as a programmer) or your school,
if any, to sign a "copyright disclaimer" for the program, if necessary. Here is
a sample; alter the names:

Yoyodyne, Inc., hereby disclaims all copyright interest in the program
` + "`" + `Gnomovision' (which makes passes at compilers) written by James Hacker.

__5__ //**<signature of Ty Coon>**//, 1 April 1989 Ty Coon, President of Vice
))??

((This General Public License does not permit incorporating your program
into proprietary programs. If your program is a subroutine library, you may
consider it more useful to permit linking proprietary applications with the
library. If this is what you want to do, use the GNU
((Lesser||Library))
General Public
License instead of this License.))??
`
const license_GPL_2_0_only_lre = `
//**
https://spdx.org/licenses/GPL-2.0-only.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		program
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	GNU
	General Public License
	((GPL))??
	((as published by the Free Software Foundation))??
	
		((under))??
		version 2
		((GPLv2))??
		((of the License))??
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		GNU
		General Public License version
		2
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		GNU
		General Public License for more details.

		((
			You should have received a copy of the
			GNU
			General Public License
			((
				((version))??
				2
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_GPL_2_0_or_3_0_lre = `
//** Used by MongoDB, WiredTiger, KeePassX, KeePassXC, maybe others **//
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 or (at your option)
version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
`
const license_GPL_2_0_or_later_lre = `
//**
https://spdx.org/licenses/GPL-2.0-or-later.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		program
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	GNU
	General Public License
	((GPL))??
	((as published by the Free Software Foundation))??
	
		((
			either version 2
			((of the License))??
			or
			((at your option))??
			any later version.
		||
			version 2 or later
			((of the License))??
		))
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		GNU
		General Public License version
		2
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		GNU
		General Public License for more details.

		((
			You should have received a copy of the
			GNU
			General Public License
			((
				((version))??
				2
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_GPL_3_0_lre = `//**
GNU General Public License v3.0
https://www.gnu.org/licenses/gpl-3.0-standalone.html
https://opensource.org/licenses/GPL-3.0
**//

((
	GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
	
	((
		((
			Copyright __20__
			((<https://fsf.org/>))??
			
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))??
		))??

		Everyone is permitted to copy and distribute verbatim copies
		of this license document, but changing it is not allowed.

		((Copyright __20__))??
	))??

))??

Preamble

The GNU General Public License is a free, copyleft license for software and
other kinds of works.

The licenses for most software and other practical works are designed to take
away your freedom to share and change the works. By contrast, the GNU General
Public License is intended to guarantee your freedom to share and change all
versions of a program--to make sure it remains free software for all its users.
We, the Free Software Foundation, use the GNU General Public License for most of
our software; it applies also to any other work released this way by its
authors. You can apply it to your programs, too.

When we speak of free software, we are referring to freedom, not price. Our
General Public Licenses are designed to make sure that you have the freedom to
distribute copies of free software (and charge for them if you wish), that you
receive source code or can get it if you want it, that you can change the
software or use pieces of it in new free programs, and that you know you can do
these things.

To protect your rights, we need to prevent others from denying you these rights
or asking you to surrender the rights. Therefore, you have certain
responsibilities if you distribute copies of the software, or if you modify it:
responsibilities to respect the freedom of others.

For example, if you distribute copies of such a program, whether gratis or for a
fee, you must pass on to the recipients the same freedoms that you received. You
must make sure that they, too, receive or can get the source code. And you must
show them these terms so they know their rights.

Developers that use the GNU GPL protect your rights with two steps: (1) assert
copyright on the software, and (2) offer you this License giving you legal
permission to copy, distribute and/or modify it.

For the developers' and authors' protection, the GPL clearly explains that there
is no warranty for this free software. For both users' and authors' sake, the
GPL requires that modified versions be marked as changed, so that their problems
will not be attributed erroneously to authors of previous versions.

Some devices are designed to deny users access to install or run modified
versions of the software inside them, although the manufacturer can do so. This
is fundamentally incompatible with the aim of protecting users' freedom to
change the software. The systematic pattern of such abuse occurs in the area of
products for individuals to use, which is precisely where it is most
unacceptable. Therefore, we have designed this version of the GPL to prohibit
the practice for those products. If such problems arise substantially in other
domains, we stand ready to extend this provision to those domains in future
versions of the GPL, as needed to protect the freedom of users.

Finally, every program is threatened constantly by software patents. States
should not allow patents to restrict development and use of software on
general-purpose computers, but in those that do, we wish to avoid the special
danger that patents applied to a free program could make it effectively
proprietary. To prevent this, the GPL assures that patents cannot be used to
render the program non-free.

The precise terms and conditions for copying, distribution and modification
follow.

TERMS AND CONDITIONS

   __1__ Definitions.

   "This License" refers to version 3 of the GNU General Public License.

   "Copyright" also means copyright-like laws that apply to other kinds of
   works, such as semiconductor masks.

   "The Program" refers to any copyrightable work licensed under this License.
   Each licensee is addressed as "you". "Licensees" and "recipients" may be
   individuals or organizations.

   To "modify" a work means to copy from or adapt all or part of the work in a
   fashion requiring copyright permission, other than the making of an exact
   copy. The resulting work is called a "modified version" of the earlier work
   or a work "based on" the earlier work.

   A "covered work" means either the unmodified Program or a work based on the
   Program.

   To "propagate" a work means to do anything with it that, without permission,
   would make you directly or secondarily liable for infringement under
   applicable copyright law, except executing it on a computer or modifying a
   private copy. Propagation includes copying, distribution (with or without
   modification), making available to the public, and in some countries other
   activities as well.

   To "convey" a work means any kind of propagation that enables other parties
   to make or receive copies. Mere interaction with a user through a computer
   network, with no transfer of a copy, is not conveying.

   An interactive user interface displays "Appropriate Legal Notices" to the
   extent that it includes a convenient and prominently visible feature that (1)
   displays an appropriate copyright notice, and (2) tells the user that there
   is no warranty for the work (except to the extent that warranties are
   provided), that licensees may convey the work under this License, and how to
   view a copy of this License. If the interface presents a list of user
   commands or options, such as a menu, a prominent item in the list meets this
   criterion.

   __1__ Source Code.

   The "source code" for a work means the preferred form of the work for making
   modifications to it. "Object code" means any non-source form of a work.

   A "Standard Interface" means an interface that either is an official standard
   defined by a recognized standards body, or, in the case of interfaces
   specified for a particular programming language, one that is widely used
   among developers working in that language.

   The "System Libraries" of an executable work include anything, other than the
   work as a whole, that (a) is included in the normal form of packaging a Major
   Component, but which is not part of that Major Component, and (b) serves only
   to enable use of the work with that Major Component, or to implement a
   Standard Interface for which an implementation is available to the public in
   source code form. A "Major Component", in this context, means a major
   essential component (kernel, window system, and so on) of the specific
   operating system (if any) on which the executable work runs, or a compiler
   used to produce the work, or an object code interpreter used to run it.

   The "Corresponding Source" for a work in object code form means all the
   source code needed to generate, install, and (for an executable work) run the
   object code and to modify the work, including scripts to control those
   activities. However, it does not include the work's System Libraries, or
   general-purpose tools or generally available free programs which are used
   unmodified in performing those activities but which are not part of the work.
   For example, Corresponding Source includes interface definition files
   associated with source files for the work, and the source code for shared
   libraries and dynamically linked subprograms that the work is specifically
   designed to require, such as by intimate data communication or control flow
   between those subprograms and other parts of the work.

   The Corresponding Source need not include anything that users can regenerate
   automatically from other parts of the Corresponding Source.

   The Corresponding Source for a work in source code form is that same work.

   __1__ Basic Permissions.

   All rights granted under this License are granted for the term of copyright
   on the Program, and are irrevocable provided the stated conditions are met.
   This License explicitly affirms your unlimited permission to run the
   unmodified Program. The output from running a covered work is covered by this
   License only if the output, given its content, constitutes a covered work.
   This License acknowledges your rights of fair use or other equivalent, as
   provided by copyright law.

   You may make, run and propagate covered works that you do not convey, without
   conditions so long as your license otherwise remains in force. You may convey
   covered works to others for the sole purpose of having them make
   modifications exclusively for you, or provide you with facilities for running
   those works, provided that you comply with the terms of this License in
   conveying all material for which you do not control copyright. Those thus
   making or running the covered works for you must do so exclusively on your
   behalf, under your direction and control, on terms that prohibit them from
   making any copies of your copyrighted material outside their relationship
   with you.

   Conveying under any other circumstances is permitted solely under the
   conditions stated below. Sublicensing is not allowed; section 10 makes it
   unnecessary.

   __1__ Protecting Users' Legal Rights From Anti-Circumvention Law.

   No covered work shall be deemed part of an effective technological measure
   under any applicable law fulfilling obligations under article 11 of the WIPO
   copyright treaty adopted on 20 December 1996, or similar laws prohibiting or
   restricting circumvention of such measures.

   When you convey a covered work, you waive any legal power to forbid
   circumvention of technological measures to the extent such circumvention is
   effected by exercising rights under this License with respect to the covered
   work, and you disclaim any intention to limit operation or modification of
   the work as a means of enforcing, against the work's users, your or third
   parties' legal rights to forbid circumvention of technological measures.

   __1__ Conveying Verbatim Copies.

   You may convey verbatim copies of the Program's source code as you receive
   it, in any medium, provided that you conspicuously and appropriately publish
   on each copy an appropriate copyright notice; keep intact all notices stating
   that this License and any non-permissive terms added in accord with section 7
   apply to the code; keep intact all notices of the absence of any warranty;
   and give all recipients a copy of this License along with the Program.

   You may charge any price or no price for each copy that you convey, and you
   may offer support or warranty protection for a fee.

   __1__ Conveying Modified Source Versions.

   You may convey a work based on the Program, or the modifications to produce
   it from the Program, in the form of source code under the terms of section 4,
   provided that you also meet all of these conditions:

      __1__ The work must carry prominent notices stating that you modified it,
      and giving a relevant date.

      __1__ The work must carry prominent notices stating that it is released
      under this License and any conditions added under section 7. This
      requirement modifies the requirement in section 4 to "keep intact all
      notices".

      __1__ You must license the entire work, as a whole, under this License to
      anyone who comes into possession of a copy. This License will therefore
      apply, along with any applicable section 7 additional terms, to the whole
      of the work, and all its parts, regardless of how they are packaged. This
      License gives no permission to license the work in any other way, but it
      does not invalidate such permission if you have separately received it.

      __1__ If the work has interactive user interfaces, each must display
      Appropriate Legal Notices; however, if the Program has interactive
      interfaces that do not display Appropriate Legal Notices, your work need
      not make them do so.

   A compilation of a covered work with other separate and independent works,
   which are not by their nature extensions of the covered work, and which are
   not combined with it such as to form a larger program, in or on a volume of a
   storage or distribution medium, is called an "aggregate" if the compilation
   and its resulting copyright are not used to limit the access or legal rights
   of the compilation's users beyond what the individual works permit. Inclusion
   of a covered work in an aggregate does not cause this License to apply to the
   other parts of the aggregate.

   __1__ Conveying Non-Source Forms.

   You may convey a covered work in object code form under the terms of sections
   4 and 5, provided that you also convey the machine-readable Corresponding
   Source under the terms of this License, in one of these ways:

      __1__ Convey the object code in, or embodied in, a physical product
      (including a physical distribution medium), accompanied by the
      Corresponding Source fixed on a durable physical medium customarily used
      for software interchange.

      __1__ Convey the object code in, or embodied in, a physical product
      (including a physical distribution medium), accompanied by a written
      offer, valid for at least three years and valid for as long as you offer
      spare parts or customer support for that product model, to give anyone who
      possesses the object code either (1) a copy of the Corresponding Source
      for all the software in the product that is covered by this License, on a
      durable physical medium customarily used for software interchange, for a
      price no more than your reasonable cost of physically performing this
      conveying of source, or (2) access to copy the Corresponding Source from a
      network server at no charge.

      __1__ Convey individual copies of the object code with a copy of the
      written offer to provide the Corresponding Source. This alternative is
      allowed only occasionally and noncommercially, and only if you received
      the object code with such an offer, in accord with subsection 6b.

      __1__ Convey the object code by offering access from a designated place
      (gratis or for a charge), and offer equivalent access to the Corresponding
      Source in the same way through the same place at no further charge. You
      need not require recipients to copy the Corresponding Source along with
      the object code. If the place to copy the object code is a network server,
      the Corresponding Source may be on a different server (operated by you or
      a third party) that supports equivalent copying facilities, provided you
      maintain clear directions next to the object code saying where to find the
      Corresponding Source. Regardless of what server hosts the Corresponding
      Source, you remain obligated to ensure that it is available for as long as
      needed to satisfy these requirements.

      __1__ Convey the object code using peer-to-peer transmission, provided you
      inform other peers where the object code and Corresponding Source of the
      work are being offered to the general public at no charge under subsection
      6d.

   A separable portion of the object code, whose source code is excluded from
   the Corresponding Source as a System Library, need not be included in
   conveying the object code work.

   A "User Product" is either (1) a "consumer product", which means any tangible
   personal property which is normally used for personal, family, or household
   purposes, or (2) anything designed or sold for incorporation into a dwelling.
   In determining whether a product is a consumer product, doubtful cases shall
   be resolved in favor of coverage. For a particular product received by a
   particular user, "normally used" refers to a typical or common use of that
   class of product, regardless of the status of the particular user or of the
   way in which the particular user actually uses, or expects or is expected to
   use, the product. A product is a consumer product regardless of whether the
   product has substantial commercial, industrial or non-consumer uses, unless
   such uses represent the only significant mode of use of the product.

   "Installation Information" for a User Product means any methods, procedures,
   authorization keys, or other information required to install and execute
   modified versions of a covered work in that User Product from a modified
   version of its Corresponding Source. The information must suffice to ensure
   that the continued functioning of the modified object code is in no case
   prevented or interfered with solely because modification has been made.

   If you convey an object code work under this section in, or with, or
   specifically for use in, a User Product, and the conveying occurs as part of
   a transaction in which the right of possession and use of the User Product is
   transferred to the recipient in perpetuity or for a fixed term (regardless of
   how the transaction is characterized), the Corresponding Source conveyed
   under this section must be accompanied by the Installation Information. But
   this requirement does not apply if neither you nor any third party retains
   the ability to install modified object code on the User Product (for example,
   the work has been installed in ROM).

   The requirement to provide Installation Information does not include a
   requirement to continue to provide support service, warranty, or updates for
   a work that has been modified or installed by the recipient, or for the User
   Product in which it has been modified or installed. Access to a network may
   be denied when the modification itself materially and adversely affects the
   operation of the network or violates the rules and protocols for
   communication across the network.

   Corresponding Source conveyed, and Installation Information provided, in
   accord with this section must be in a format that is publicly documented (and
   with an implementation available to the public in source code form), and must
   require no special password or key for unpacking, reading or copying.

   __1__ Additional Terms.

   "Additional permissions" are terms that supplement the terms of this License
   by making exceptions from one or more of its conditions. Additional
   permissions that are applicable to the entire Program shall be treated as
   though they were included in this License, to the extent that they are valid
   under applicable law. If additional permissions apply only to part of the
   Program, that part may be used separately under those permissions, but the
   entire Program remains governed by this License without regard to the
   additional permissions.

   When you convey a copy of a covered work, you may at your option remove any
   additional permissions from that copy, or from any part of it. (Additional
   permissions may be written to require their own removal in certain cases when
   you modify the work.) You may place additional permissions on material, added
   by you to a covered work, for which you have or can give appropriate
   copyright permission.

   Notwithstanding any other provision of this License, for material you add to
   a covered work, you may (if authorized by the copyright holders of that
   material) supplement the terms of this License with terms:

      __1__ Disclaiming warranty or limiting liability differently from the
      terms of sections 15 and 16 of this License; or

      __1__ Requiring preservation of specified reasonable legal notices or
      author attributions in that material or in the Appropriate Legal Notices
      displayed by works containing it; or

      __1__ Prohibiting misrepresentation of the origin of that material, or
      requiring that modified versions of such material be marked in reasonable
      ways as different from the original version; or

      __1__ Limiting the use for publicity purposes of names of licensors or
      authors of the material; or

      __1__ Declining to grant rights under trademark law for use of some trade
      names, trademarks, or service marks; or

      __1__ Requiring indemnification of licensors and authors of that material
      by anyone who conveys the material (or modified versions of it) with
      contractual assumptions of liability to the recipient, for any liability
      that these contractual assumptions directly impose on those licensors and
      authors.

   All other non-permissive additional terms are considered "further
   restrictions" within the meaning of section 10. If the Program as you
   received it, or any part of it, contains a notice stating that it is governed
   by this License along with a term that is a further restriction, you may
   remove that term. If a license document contains a further restriction but
   permits relicensing or conveying under this License, you may add to a covered
   work material governed by the terms of that license document, provided that
   the further restriction does not survive such relicensing or conveying.

   If you add terms to a covered work in accord with this section, you must
   place, in the relevant source files, a statement of the additional terms that
   apply to those files, or a notice indicating where to find the applicable
   terms.

   Additional terms, permissive or non-permissive, may be stated in the form of
   a separately written license, or stated as exceptions; the above requirements
   apply either way.

   __1__ Termination.

   You may not propagate or modify a covered work except as expressly provided
   under this License. Any attempt otherwise to propagate or modify it is void,
   and will automatically terminate your rights under this License (including
   any patent licenses granted under the third paragraph of section 11).

   However, if you cease all violation of this License, then your license from a
   particular copyright holder is reinstated (a) provisionally, unless and until
   the copyright holder explicitly and finally terminates your license, and (b)
   permanently, if the copyright holder fails to notify you of the violation by
   some reasonable means prior to 60 days after the cessation.

   Moreover, your license from a particular copyright holder is reinstated
   permanently if the copyright holder notifies you of the violation by some
   reasonable means, this is the first time you have received notice of
   violation of this License (for any work) from that copyright holder, and you
   cure the violation prior to 30 days after your receipt of the notice.

   Termination of your rights under this section does not terminate the licenses
   of parties who have received copies or rights from you under this License. If
   your rights have been terminated and not permanently reinstated, you do not
   qualify to receive new licenses for the same material under section 10.

   __1__ Acceptance Not Required for Having Copies.

   You are not required to accept this License in order to receive or run a copy
   of the Program. Ancillary propagation of a covered work occurring solely as a
   consequence of using peer-to-peer transmission to receive a copy likewise
   does not require acceptance. However, nothing other than this License grants
   you permission to propagate or modify any covered work. These actions
   infringe copyright if you do not accept this License. Therefore, by modifying
   or propagating a covered work, you indicate your acceptance of this License
   to do so.

   __1__ Automatic Licensing of Downstream Recipients.

   Each time you convey a covered work, the recipient automatically receives a
   license from the original licensors, to run, modify and propagate that work,
   subject to this License. You are not responsible for enforcing compliance by
   third parties with this License.

   An "entity transaction" is a transaction transferring control of an
   organization, or substantially all assets of one, or subdividing an
   organization, or merging organizations. If propagation of a covered work
   results from an entity transaction, each party to that transaction who
   receives a copy of the work also receives whatever licenses to the work the
   party's predecessor in interest had or could give under the previous
   paragraph, plus a right to possession of the Corresponding Source of the work
   from the predecessor in interest, if the predecessor has it or can get it
   with reasonable efforts.

   You may not impose any further restrictions on the exercise of the rights
   granted or affirmed under this License. For example, you may not impose a
   license fee, royalty, or other charge for exercise of rights granted under
   this License, and you may not initiate litigation (including a cross-claim or
   counterclaim in a lawsuit) alleging that any patent claim is infringed by
   making, using, selling, offering for sale, or importing the Program or any
   portion of it.

   __1__ Patents.

   A "contributor" is a copyright holder who authorizes use under this License
   of the Program or a work on which the Program is based. The work thus
   licensed is called the contributor's "contributor version".

   A contributor's "essential patent claims" are all patent claims owned or
   controlled by the contributor, whether already acquired or hereafter
   acquired, that would be infringed by some manner, permitted by this License,
   of making, using, or selling its contributor version, but do not include
   claims that would be infringed only as a consequence of further modification
   of the contributor version. For purposes of this definition, "control"
   includes the right to grant patent sublicenses in a manner consistent with
   the requirements of this License.

   Each contributor grants you a non-exclusive, worldwide, royalty-free patent
   license under the contributor's essential patent claims, to make, use, sell,
   offer for sale, import and otherwise run, modify and propagate the contents
   of its contributor version.

   In the following three paragraphs, a "patent license" is any express
   agreement or commitment, however denominated, not to enforce a patent (such
   as an express permission to practice a patent or covenant not to sue for
   patent infringement). To "grant" such a patent license to a party means to
   make such an agreement or commitment not to enforce a patent against the
   party.

   If you convey a covered work, knowingly relying on a patent license, and the
   Corresponding Source of the work is not available for anyone to copy, free of
   charge and under the terms of this License, through a publicly available
   network server or other readily accessible means, then you must either (1)
   cause the Corresponding Source to be so available, or (2) arrange to deprive
   yourself of the benefit of the patent license for this particular work, or
   (3) arrange, in a manner consistent with the requirements of this License, to
   extend the patent license to downstream recipients. "Knowingly relying" means
   you have actual knowledge that, but for the patent license, your conveying
   the covered work in a country, or your recipient's use of the covered work in
   a country, would infringe one or more identifiable patents in that country
   that you have reason to believe are valid.

   If, pursuant to or in connection with a single transaction or arrangement,
   you convey, or propagate by procuring conveyance of, a covered work, and
   grant a patent license to some of the parties receiving the covered work
   authorizing them to use, propagate, modify or convey a specific copy of the
   covered work, then the patent license you grant is automatically extended to
   all recipients of the covered work and works based on it.

   A patent license is "discriminatory" if it does not include within the scope
   of its coverage, prohibits the exercise of, or is conditioned on the
   non-exercise of one or more of the rights that are specifically granted under
   this License. You may not convey a covered work if you are a party to an
   arrangement with a third party that is in the business of distributing
   software, under which you make payment to the third party based on the extent
   of your activity of conveying the work, and under which the third party
   grants, to any of the parties who would receive the covered work from you, a
   discriminatory patent license (a) in connection with copies of the covered
   work conveyed by you (or copies made from those copies), or (b) primarily for
   and in connection with specific products or compilations that contain the
   covered work, unless you entered into that arrangement, or that patent
   license was granted, prior to 28 March 2007.

   Nothing in this License shall be construed as excluding or limiting any
   implied license or other defenses to infringement that may otherwise be
   available to you under applicable patent law.

   __1__ No Surrender of Others' Freedom.

   If conditions are imposed on you (whether by court order, agreement or
   otherwise) that contradict the conditions of this License, they do not excuse
   you from the conditions of this License. If you cannot convey a covered work
   so as to satisfy simultaneously your obligations under this License and any
   other pertinent obligations, then as a consequence you may not convey it at
   all. For example, if you agree to terms that obligate you to collect a
   royalty for further conveying from those to whom you convey the Program, the
   only way you could satisfy both those terms and this License would be to
   refrain entirely from conveying the Program.

   __1__ Use with the GNU Affero General Public License.

   Notwithstanding any other provision of this License, you have permission to
   link or combine any covered work with a work licensed under version 3 of the
   GNU Affero General Public License into a single combined work, and to convey
   the resulting work. The terms of this License will continue to apply to the
   part which is the covered work, but the special requirements of the GNU
   Affero General Public License, section 13, concerning interaction through a
   network will apply to the combination as such.

   __1__ Revised Versions of this License.

   The Free Software Foundation may publish revised and/or new versions of the
   GNU General Public License from time to time. Such new versions will be
   similar in spirit to the present version, but may differ in detail to address
   new problems or concerns.

   Each version is given a distinguishing version number. If the Program
   specifies that a certain numbered version of the GNU General Public License
   "or any later version" applies to it, you have the option of following the
   terms and conditions either of that numbered version or of any later version
   published by the Free Software Foundation. If the Program does not specify a
   version number of the GNU General Public License, you may choose any version
   ever published by the Free Software Foundation.

   If the Program specifies that a proxy can decide which future versions of the
   GNU General Public License can be used, that proxy's public statement of
   acceptance of a version permanently authorizes you to choose that version for
   the Program.

   Later license versions may give you additional or different permissions.
   However, no additional obligations are imposed on any author or copyright
   holder as a result of your choosing to follow a later version.

   __1__ Disclaimer of Warranty.

   THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE
   LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR
   OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND,
   EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
   ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
   SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY
   SERVICING, REPAIR OR CORRECTION.

   __1__ Limitation of Liability.

   IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
   ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS THE
   PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
   GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE
   OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR
   DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR
   A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH
   HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

   __1__ Interpretation of Sections 15 and 16.

   If the disclaimer of warranty and limitation of liability provided above
   cannot be given local legal effect according to their terms, reviewing courts
   shall apply local law that most closely approximates an absolute waiver of
   all civil liability in connection with the Program, unless a warranty or
   assumption of liability accompanies a copy of the Program in return for a
   fee.

(( END OF TERMS AND CONDITIONS))??

((
How to Apply These Terms to Your New Programs

If you develop a new program, and you want it to be of the greatest possible use
to the public, the best way to achieve this is to make it free software which
everyone can redistribute and change under these terms.



To do so, attach the following notices to the program. It is safest to attach
them to the start of each source file to most effectively state the exclusion of
warranty; and each file should have at least the "copyright" line and a pointer
to where the full notice is found.

__30__
//**
<one line to give the program's name and a brief idea of what it does.>
Copyright (C) <year> <name of author>
**//

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https:/www.gnu.org/licenses/>.

Also add information on how to contact you by electronic and paper mail.

If the program does terminal interaction, make it output a short notice like
this when it starts in an interactive mode:

<program> __10__

This program comes with ABSOLUTELY NO WARRANTY; for details type ` + "`" + `show w'.

This is free software, and you are welcome to redistribute it under certain
conditions; type ` + "`" + `show c' for details.

The hypothetical commands ` + "`" + `show w' and ` + "`" + `show c' should show the appropriate
parts of the General Public License. Of course, your program's commands might be
different; for a GUI interface, you would use an "about box".

You should also get your employer (if you work as a programmer) or school, if
any, to sign a "copyright disclaimer" for the program, if necessary. For more
information on this, and how to apply and follow the GNU GPL, see <https:/www.gnu.org/licenses/>.

The GNU General Public License does not permit incorporating your program into
proprietary programs. If your program is a subroutine library, you may consider
it more useful to permit linking proprietary applications with the library. If
this is what you want to do, use the GNU Lesser General Public License instead
of this License. But first, please read <https://www.gnu.org/licenses/why-not-lgpl.html>.))??
`
const license_GPL_3_0_only_lre = `
//**
https://spdx.org/licenses/GPL-3.0-only.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		program
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	GNU
	General Public License
	((GPL))??
	((as published by the Free Software Foundation))??
	
		((under))??
		version 3
		((GPLv3))??
		((of the License))??
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		GNU
		General Public License version
		3
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		GNU
		General Public License for more details.

		((
			You should have received a copy of the
			GNU
			General Public License
			((
				((version))??
				3
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_GPL_3_0_or_later_lre = `
//**
https://spdx.org/licenses/GPL-3.0-or-later.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		program
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	GNU
	General Public License
	((GPL))??
	((as published by the Free Software Foundation))??
	
		((
			either version 3
			((of the License))??
			or
			((at your option))??
			any later version.
		||
			version 3 or later
			((of the License))??
		))
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		GNU
		General Public License version
		3
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		GNU
		General Public License for more details.

		((
			You should have received a copy of the
			GNU
			General Public License
			((
				((version))??
				3
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_Giftware_lre = `//**
Giftware License
https://spdx.org/licenses/Giftware.json
http://liballeg.org/license.html#allegro-4-the-giftware-license
**//

(( Allegro 4 (the giftware license) ))??

Allegro is gift-ware. It was created by a number of people working in
cooperation, and is given to you freely as a gift. You may use, modify,
redistribute, and generally hack it about in any way you like, and you do not
have to give us anything in return.

However, if you like this product you are encouraged to thank us by making a
return gift to the Allegro community. This could be by writing an add-on
package, providing a useful bug report, making an improvement to the library, or
perhaps just releasing the sources of your program so that other people can
learn from them. If you redistribute parts of this code or make a game using it,
it would be nice if you mentioned Allegro somewhere in the credits, but you are
not required to do this. We trust you not to abuse our generosity.

By Shawn Hargreaves, 18 October 1998.

DISCLAIMER: THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT.
IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE
LIABLE FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
`
const license_Glide_lre = `//**
3dfx Glide License
https://spdx.org/licenses/Glide.json
http://www.users.on.net/~triforce/glidexp/COPYING.txt
**//

(( 3DFX GLIDE Source Code General Public License ))??

   (( 1. ))??
   PREAMBLE

   This license is for software that provides a 3D graphics application program
   interface (API).The license is intended to offer terms similar to some
   standard General Public Licenses designed to foster open standards and
   unrestricted accessibility to source code. Some of these licenses require
   that, as a condition of the license of the software, any derivative works
   (that is, new software which is a work containing the original program or a
   portion of it) must be available for general use, without restriction other
   than for a minor transfer fee, and that the source code for such derivative
   works must likewise be made available. The only restriction is that such
   derivative works must be subject to the same General Public License terms as
   the original work.

   This 3dfx GLIDE Source Code General Public License differs from the standard
   licenses of this type in that it does not require the entire derivative work
   to be made available under the terms of this license nor is the recipient
   required to make available the source code for the entire derivative work.
   Rather, the license is limited to only the identifiable portion of the
   derivative work that is derived from the licensed software. The precise terms
   and conditions for copying, distribution and modification follow.

   (( 2. ))??
   DEFINITIONS

      (( 2.1 ))??
      This License applies to any program (or other "work") which contains a
      notice placed by the copyright holder saying it may be distributed under
      the terms of this 3dfx GLIDE Source Code General Public License.

      (( 2.2 ))??
      The term "Program" as used in this Agreement refers to 3DFX's GLIDE source
      code and object code and any Derivative Work.

      (( 2.3 ))??
      "Derivative Work" means, for the purpose of the License, that portion of
      any work that contains the Program or the identifiable portion of a work
      that is derived from the Program, either verbatim or with modifications
      and/or translated into another language, and that performs 3D graphics API
      operations. It does not include any other portions of a work.

      (( 2.4 ))??
      "Modifications of the Program" means any work, which includes a Derivative
      Work, and includes the whole of such work.

      (( 2.5 ))??
      "License" means this 3dfx GLIDE Source Code General Public License.

      (( 2.6 ))??
      The "Source Code" for a work means the preferred form of the work for
      making modifications to it. For an executable work, complete source code
      means all the source code for all modules it contains, any associated
      interface definition files, and the scripts used to control compilation
      and installation of the executable work.

      (( 2.7 ))??
      "3dfx" means 3dfx Interactive, Inc.

   (( 3. ))??
   LICENSED ACTIVITIES

      (( 3.1 ))??
      COPYING - You may copy and distribute verbatim copies of the Program's
      Source Code as you receive it, in any medium, subject to the provision of
      section 3.3 and provided also that:

         (( (a) ))??
         you conspicuously and appropriately publish on each copy an appropriate
         copyright notice (3dfx Interactive, Inc. 1999), a notice that
         recipients who wish to copy, distribute or modify the Program can only
         do so subject to this License, and a disclaimer of warranty as set
         forth in section 5;

         (( (b) ))??
         keep intact all the notices that refer to this License and to the
         absence of any warranty; and

         (( c ))??
         do not make any use of the GLIDE trademark without the prior written
         permission of 3dfx, and

         (( (d) ))??
         give all recipients of the Program a copy of this License along with
         the Program or instructions on how to easily receive a copy of this
         License.

      (( 3.2 ))??
      MODIFICATION OF THE PROGRAM/DERIVATIVE WORKS - You may modify your copy or
      copies of the Program or any portion of it, and copy and distribute such
      modifications subject to the provisions of section 3.3 and provided that
      you also meet all of the following conditions:

         (( (a) ))??
         you conspicuously and appropriately publish on each copy of a
         Derivative Work an appropriate copyright notice, a notice that
         recipients who wish to copy, distribute or modify the Derivative Work
         can only do so subject to this License, and a disclaimer of warranty as
         set forth in section 5;

         (( (b) ))??
         keep intact all the notices that refer to this License and to the
         absence of any warranty; and (c) give all recipients of the Derivative
         Work a copy of this License along with the Derivative Work or
         instructions on how to easily receive a copy of this License.

         (( (d) ))??
         You must cause the modified files of the Derivative Work to carry
         prominent notices stating that you changed the files and the date of
         any change.

         (( (e) ))??
         You must cause any Derivative Work that you distribute or publish to be
         licensed at no charge to all third parties under the terms of this
         License.

         (( (f) ))??
         You do not make any use of the GLIDE trademark without the prior
         written permission of 3dfx.

         (( (g) ))??
         If the Derivative Work normally reads commands interactively when run,
         you must cause it, when started running for such interactive use, to
         print or display an announcement as follows:

         "COPYRIGHT 3DFX INTERACTIVE, INC. 1999, ALL RIGHTS RESERVED THIS
         SOFTWARE IS FREE AND PROVIDED "AS IS," WITHOUT WARRANTY OF ANY KIND,
         EITHER EXPRESSED OR IMPLIED. THERE IS NO RIGHT TO USE THE GLIDE
         TRADEMARK WITHOUT PRIOR WRITTEN PERMISSION OF 3DFX INTERACTIVE, INC.
         SEE THE 3DFX GLIDE GENERAL PUBLIC LICENSE FOR A FULL TEXT OF THE
         DISTRIBUTION AND NON-WARRANTY PROVISIONS (REQUEST COPY FROM
         INFO@3DFX.COM)."

         (( (h) ))??
         The requirements of this section 3.2 do not apply to the modified work
         as a whole but only to the Derivative Work. It is not the intent of
         this License to claim rights or contest your rights to work written
         entirely by you; rather, the intent is to exercise the right to control
         the distribution of Derivative Works.

      (( 3.3 ))??
      DISTRIBUTION

         (( (a) ))??
         All copies of the Program or Derivative Works which are distributed
         must include in the file headers the following language verbatim:

         "THIS SOFTWARE IS SUBJECT TO COPYRIGHT PROTECTION AND IS OFFERED ONLY
         PURSUANT TO THE 3DFX GLIDE GENERAL PUBLIC LICENSE. THERE IS NO RIGHT TO
         USE THE GLIDE TRADEMARK WITHOUT PRIOR WRITTEN PERMISSION OF 3DFX
         INTERACTIVE, INC. A COPY OF THIS LICENSE MAY BE OBTAINED FROM THE
         DISTRIBUTOR OR BY CONTACTING 3DFX INTERACTIVE INC (info@3dfx.com). THIS
         PROGRAM. IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
         EXPRESSED OR IMPLIED. SEE THE 3DFX GLIDE GENERAL PUBLIC LICENSE FOR A
         FULL TEXT OF THE NON-WARRANTY PROVISIONS.

         USE, DUPLICATION OR DISCLOSURE BY THE GOVERNMENT IS SUBJECT TO
         RESTRICTIONS AS SET FORTH IN SUBDIVISION (C)(1)(II) OF THE RIGHTS IN
         TECHNICAL DATA AND COMPUTER SOFTWARE CLAUSE AT DFARS 252.227-7013,
         AND/OR IN SIMILAR OR SUCCESSOR CLAUSES IN THE FAR, DOD OR NASA FAR
         SUPPLEMENT. UNPUBLISHED RIGHTS RESERVED UNDER THE COPYRIGHT LAWS OF THE
         UNITED STATES.

         COPYRIGHT 3DFX INTERACTIVE, INC. 1999, ALL RIGHTS RESERVED"

         (( (b) ))??
         You may distribute the Program or a Derivative Work in object code or
         executable form under the terms of Sections 3.1 and 3.2 provided that
         you also do one of the following:

            (( (1) ))??
            Accompany it with the complete corresponding machine-readable source
            code, which must be distributed under the terms of Sections 3.1 and
            3.2; or,

            (( (2) ))??
            Accompany it with a written offer, valid for at least three years,
            to give any third party, for a charge no more than your cost of
            physically performing source distribution, a complete
            machine-readable copy of the corresponding source code, to be
            distributed under the terms of Sections 3.1 and 3.2 on a medium
            customarily used for software interchange; or,

            (( (3) ))??
            Accompany it with the information you received as to the offer to
            distribute corresponding source code. (This alternative is allowed
            only for noncommercial distribution and only if you received the
            program in object code or executable form with such an offer, in
            accord with Subsection 3.3(b)(2) above.)

         (( c ))??
         The source code distributed need not include anything that is normally
         distributed (in either source or binary form) with the major components
         (compiler, kernel, and so on) of the operating system on which the
         executable runs, unless that component itself accompanies the
         executable code.

         (( (d) ))??
         If distribution of executable code or object code is made by offering
         access to copy from a designated place, then offering equivalent access
         to copy the source code from the same place counts as distribution of
         the source code, even though third parties are not compelled to copy
         the source along with the object code.

         (( (e) ))??
         Each time you redistribute the Program or any Derivative Work, the
         recipient automatically receives a license from 3dfx and successor
         licensors to copy, distribute or modify the Program and Derivative
         Works subject to the terms and conditions of the License. You may not
         impose any further restrictions on the recipients' exercise of the
         rights granted herein. You are not responsible for enforcing compliance
         by third parties to this License.

         (( (f) ))??
         You may not make any use of the GLIDE trademark without the prior
         written permission of 3dfx.

         (( (g) ))??
         You may not copy, modify, sublicense, or distribute the Program or any
         Derivative Works except as expressly provided under this License. Any
         attempt otherwise to copy, modify, sublicense or distribute the Program
         or any Derivative Works is void, and will automatically terminate your
         rights under this License. However, parties who have received copies,
         or rights, from you under this License will not have their licenses
         terminated so long as such parties remain in full compliance.

   (( 4. ))??
   MISCELLANEOUS

      (( 4.1 ))??
      Acceptance of this License is voluntary. By using, modifying or
      distributing the Program or any Derivative Work, you indicate your
      acceptance of this License to do so, and all its terms and conditions for
      copying, distributing or modifying the Program or works based on it.
      Nothing else grants you permission to modify or distribute the Program or
      Derivative Works and doing so without acceptance of this License is in
      violation of the U.S. and international copyright laws.

      (( 4.2 ))??
      If the distribution and/or use of the Program or Derivative Works is
      restricted in certain countries either by patents or by copyrighted
      interfaces, the original copyright holder who places the Program under
      this License may add an explicit geographical distribution limitation
      excluding those countries, so that distribution is permitted only in or
      among countries not thus excluded. In such case, this License incorporates
      the limitation as if written in the body of this License.

      (( 4.3 ))??
      This License is to be construed according to the laws of the State of
      California and you consent to personal jurisdiction in the State of
      California in the event it is necessary to enforce the provisions of this
      License.

   (( 5. ))??
   NO WARRANTIES

      (( 5.1 ))??
      TO THE EXTENT PERMITTED BY APPLICABLE LAW, THERE IS NO WARRANTY FOR THE
      PROGRAM. OR DERIVATIVE WORKS THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
      PROVIDE THE PROGRAM AND ANY DERIVATIVE WORKS"AS IS" WITHOUT WARRANTY OF
      ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE
      IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
      PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
      AND ANY DERIVATIVE WORK IS WITH YOU. SHOULD THE PROGRAM OR ANY DERIVATIVE
      WORK PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
      REPAIR OR CORRECTION.

      (( 5.2 ))??
      IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL 3DFX INTERACTIVE, INC.,
      OR ANY OTHER COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
      REDISTRIBUTE THE PROGRAM OR DERIVATIVE WORKS AS PERMITTED ABOVE, BE LIABLE
      TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR
      CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE
      PROGRAM OR DERIVATIVE WORKS (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR
      DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES
      OR A FAILURE OF THE PROGRAM OR DERIVATIVE WORKS TO OPERATE WITH ANY OTHER
      PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
      POSSIBILITY OF SUCH DAMAGES.
`
const license_Glulxe_lre = `//**
Glulxe License
https://spdx.org/licenses/Glulxe.json
https://fedoraproject.org/wiki/Licensing/Glulxe
**//

//** Copyright **//

You may copy and distribute it freely, by any means and under any conditions, as
long as the code and documentation is not changed. You may also incorporate this
code into your own program and distribute that, or modify this code and use and
distribute the modified version, as long as you retain a notice in your program
or documentation which mentions my name and the URL shown above.
`
const license_GooglePatentClause_lre = `//**
Patent grant that appeared briefly in the Go LICENSE file,
before moving to a separate PATENTS file and being reworded.
**//

Subject to the terms and conditions of this License, __5__ hereby grants to
You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable
(except as stated in this section) patent license to make, have made, use,
offer to sell, sell, import, and otherwise transfer this implementation __5__
where such license applies only to those patent claims licensable by __5__
that are necessarily infringed by use of this implementation __5__. If You
institute patent litigation against any entity (including a cross-claim or
counterclaim in a lawsuit) alleging that
((this implementation __5__ or a Contribution incorporated within))??
this implementation __5__
constitutes direct or contributory patent infringement,
then any patent licenses granted to You
under this License for this implementation __5__ shall terminate as of the date
such litigation is filed.
`
const license_GooglePatentsFile_lre = `//**
PATENTS file used in WebM, Go, gRPC and other Google open source projects.
Now also used by some other companies.
**//

(( Additional IP Rights Grant (Patents) ))??

"This implementation" means the copyrightable works distributed by
__5__ as part of //** __5__ **//.

__10__ hereby grants to You a perpetual, worldwide, non-exclusive,
no-charge, royalty-free, irrevocable (except as stated in this section)
patent license to make, have made, use, offer to sell, sell, import,
transfer and otherwise run, modify and propagate the contents of this
implementation __5__, where such license applies only to those patent
claims, both currently owned
((or controlled))??
by __5__ and acquired in
the future, licensable by __5__ that are necessarily infringed by this
implementation __5__.  This grant does not include claims that would be
infringed only as a consequence of further modification of this
implementation.  If you or your agent or exclusive licensee institute or
order or agree to the institution of patent litigation against any
entity (including a cross-claim or counterclaim in a lawsuit) alleging
that this implementation __5__ or any code incorporated within this
implementation __5__ constitutes direct or contributory patent
infringement, or inducement of patent infringement, then any patent
rights granted to you under this License for this implementation __5__
shall terminate as of the date such litigation is filed.
`
const license_HPND_lre = `


	//**
	Historical Permission Notice and Disclaimer
	https://spdx.org/licenses/HPND.json
	https://opensource.org/licenses/HPND
	https://fedoraproject.org/wiki/Licensing:MIT#Old_Style
	**//

	(( The files in this directory are subject to the following license. ))??

	(( The author of this software is __10__
	(( Copyright __20__ ))??
	))??

	Permission to use, copy, modify
	and distribute
	this
	(( software || material ))
	(( and its documentation ))??
	for any purpose
	(( and ))??
	(( without fee ))??
	is hereby granted
	(( without fee ))??
	provided that
	(( the above copyright notice
	   (( and this permission notice ))??
	   appear
	|| this entire notice is included
	))
	in all copies
	//**
	Avoid a non-empty match for this next part,
	beause the pattern above matches an ISC license exactly.
	We want to require additional text to avoid matching ISC licenses.
	**//
	((
		//** Traditional Old-style MIT variant **//
		(( and ))??
		that both
		(( that ))??
		(( the ))??
		copyright notice and this permission notice
		appear in supporting documentation
		(( and that the name __10__ not be used
		   in advertising or publicity pertaining to distribution
		   of the software without specific, written prior permission.
		))??
	||
		//** Bellcore variant **//
		(( and that the name __10__ not be used
		   in advertising or publicity pertaining to
		   this material without the specific, prior written permission
		   of an authorized representative of __10__.
		))
	||
		//** AT&T dtoa variant **//
		of any software which is or includes a copy
		or modification of this software and in all copies
		of the supporting documentation for such software.
	))

	((
	(( No representations are made
	|| __10__ makes no representations ))
	about the
	(( accuracy or ))??
	suitability of this
	((software || material))
	for any purpose.
	It is provided "as is" without
	(( any ))??
	express or implied
	((warranty || warranties))
	))??

	((
	__10__ DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
	INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
	IN NO EVENT SHALL __10__ BE LIABLE FOR ANY SPECIAL,
	INDIRECT OR CONSEQUENTIAL DAMAGES
	OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
	DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
	NEGLIGENCE OR OTHER TORTIOUS ACTION,
	ARISING OUT OF OR IN CONNECTION WITH
	THE USE OR PERFORMANCE OF THIS SOFTWARE.

	||

	THE AUTHOR PROVIDES THIS SOFTWARE ''AS IS'' AND ANY EXPRESSED OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
	OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	||

	THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
	WARRANTY.  IN PARTICULAR,
	((NEITHER __10__ MAKES ANY || __10__ MAKES NO))
	REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING THE MERCHANTABILITY
	OF THIS SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR PURPOSE.

	))??



`
const license_HPND_sell_variant_lre = `

	//**
	Historical Permission Notice and Disclaimer
	https://spdx.org/licenses/HPND.json
	https://opensource.org/licenses/HPND
	https://fedoraproject.org/wiki/Licensing:MIT#Old_Style
	**//

	(( The files in this directory are subject to the following license. ))??

	(( The author of this software is __10__
	(( Copyright __20__ ))??
	))??

	Permission to use, copy, modify
	distribute and sell
	this
	(( software || material ))
	(( and its documentation ))??
	for any purpose
	(( and ))??
	(( without fee ))??
	is hereby granted
	(( without fee ))??
	provided that
	(( the above copyright notice
	   (( and this permission notice ))??
	   appear
	|| this entire notice is included
	))
	in all copies
	//**
	Avoid a non-empty match for this next part,
	beause the pattern above matches an ISC license exactly.
	We want to require additional text to avoid matching ISC licenses.
	**//
	((
		//** Traditional Old-style MIT variant **//
		(( and ))??
		that both
		(( that ))??
		(( the ))??
		copyright notice and this permission notice
		appear in supporting documentation
		(( and that the name __10__ not be used
		   in advertising or publicity pertaining to distribution
		   of the software without specific, written prior permission.
		))??
	||
		//** Bellcore variant **//
		(( and that the name __10__ not be used
		   in advertising or publicity pertaining to
		   this material without the specific, prior written permission
		   of an authorized representative of __10__.
		))
	||
		//** AT&T dtoa variant **//
		of any software which is or includes a copy
		or modification of this software and in all copies
		of the supporting documentation for such software.
	))

	((
	(( No representations are made
	|| __10__ makes no representations ))
	about the
	(( accuracy or ))??
	suitability of this
	((software || material))
	for any purpose.
	It is provided "as is" without
	(( any ))??
	express or implied
	((warranty || warranties))
	))??

	((
	__10__ DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
	INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
	IN NO EVENT SHALL __10__ BE LIABLE FOR ANY SPECIAL,
	INDIRECT OR CONSEQUENTIAL DAMAGES
	OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
	DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
	NEGLIGENCE OR OTHER TORTIOUS ACTION,
	ARISING OUT OF OR IN CONNECTION WITH
	THE USE OR PERFORMANCE OF THIS SOFTWARE.

	||

	THE AUTHOR PROVIDES THIS SOFTWARE ''AS IS'' AND ANY EXPRESSED OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
	OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	||

	THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
	WARRANTY.  IN PARTICULAR,
	((NEITHER __10__ MAKES ANY || __10__ MAKES NO))
	REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING THE MERCHANTABILITY
	OF THIS SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR PURPOSE.

	))??

`
const license_HaskellReport_lre = `//**
Haskell Language Report License
https://spdx.org/licenses/HaskellReport.json
https://fedoraproject.org/wiki/Licensing/Haskell_Language_Report_License
**//

Code derived from the document "Report on the Programming Language
Haskell 2010", is distributed under the following license:

Copyright __20__

The authors intend this Report to belong to the entire Haskell community, and so
we grant permission to copy and distribute it for any purpose, provided that it
is reproduced in its entirety, including this Notice. Modified versions of this
Report may also be copied and distributed for any purpose, provided that the
modified version is clearly presented as such, and that it does not claim to be
a definition of the Haskell 2010 Language.
`
const license_Hippocratic_2_1_lre = `//**
Hippocratic License 2.1
https://spdx.org/licenses/Hippocratic-2.1.json
https://firstdonoharm.dev/version/2/1/license.html
https://github.com/EthicalSource/hippocratic-license/blob/58c0e646d64ff6fbee275bfe2b9492f914e3ab2a/LICENSE.txt
**//

//** Copyright **//

(( Hippocratic License Version Number: 2.1. ))??

Purpose. The purpose of this License is for the Licensor named above to permit
the Licensee (as defined below) broad permission, if consistent with Human
Rights Laws and Human Rights Principles (as each is defined below), to use and
work with the Software (as defined below) within the full scope of Licensor's
copyright and patent rights, if any, in the Software, while ensuring attribution
and protecting the Licensor from liability.

Permission and Conditions. The Licensor grants permission by this license
("License"), free of charge, to the extent of Licensor's rights under applicable
copyright and patent law, to any person or entity (the "Licensee") obtaining a
copy of this software and associated documentation files (the "Software"), to do
everything with the Software that would otherwise infringe (i) the Licensor's
copyright in the Software or (ii) any patent claims to the Software that the
Licensor can license or becomes able to license, subject to all of the following
terms and conditions:

   *
   Acceptance. This License is automatically offered to every person and entity
   subject to its terms and conditions. Licensee accepts this License and agrees
   to its terms and conditions by taking any action with the Software that,
   absent this License, would infringe any intellectual property right held by
   Licensor.

   *
   Notice. Licensee must ensure that everyone who gets a copy of any part of
   this Software from Licensee, with or without changes, also receives the
   License and the above copyright notice (and if included by the Licensor,
   patent, trademark and attribution notice). Licensee must cause any modified
   versions of the Software to carry prominent notices stating that Licensee
   changed the Software. For clarity, although Licensee is free to create
   modifications of the Software and distribute only the modified portion
   created by Licensee with additional or different terms, the portion of the
   Software not modified must be distributed pursuant to this License. If anyone
   notifies Licensee in writing that Licensee has not complied with this Notice
   section, Licensee can keep this License by taking all practical steps to
   comply within 30 days after the notice. If Licensee does not do so,
   Licensee's License (and all rights licensed hereunder) shall end
   immediately.

   *
   Compliance with Human Rights Principles and Human Rights Laws.

      (( 1. ))??
      Human Rights Principles.

         (( (a) ))??
         Licensee is advised to consult the articles of the United Nations
         Universal Declaration of Human Rights and the United Nations Global
         Compact that define recognized principles of international human rights
         (the "Human Rights Principles"). Licensee shall use the Software in a
         manner consistent with Human Rights Principles.

         (( (b) ))??
         Unless the Licensor and Licensee agree otherwise, any dispute,
         controversy, or claim arising out of or relating to (i) Section 1(a)
         regarding Human Rights Principles, including the breach of Section
         1(a), termination of this License for breach of the Human Rights
         Principles, or invalidity of Section 1(a) or (ii) a determination of
         whether any Law is consistent or in conflict with Human Rights
         Principles pursuant to Section 2, below, shall be settled by
         arbitration in accordance with the Hague Rules on Business and Human
         Rights Arbitration (the "Rules"); provided, however, that Licensee may
         elect not to participate in such arbitration, in which event this
         License (and all rights licensed hereunder) shall end immediately. The
         number of arbitrators shall be one unless the Rules require otherwise.

         Unless both the Licensor and Licensee agree to the contrary: (1) All
         documents and information concerning the arbitration shall be public
         and may be disclosed by any party; (2) The repository referred to under
         Article 43 of the Rules shall make available to the public in a timely
         manner all documents concerning the arbitration which are communicated
         to it, including all submissions of the parties, all evidence admitted
         into the record of the proceedings, all transcripts or other recordings
         of hearings and all orders, decisions and awards of the arbitral
         tribunal, subject only to the arbitral tribunal's powers to take such
         measures as may be necessary to safeguard the integrity of the arbitral
         process pursuant to Articles 18, 33, 41 and 42 of the Rules; and (3)
         Article 26(6) of the Rules shall not apply.

      (( 2. ))??
      Human Rights Laws. The Software shall not be used by any person or entity
      for any systems, activities, or other uses that violate any Human Rights
      Laws. "Human Rights Laws" means any applicable laws, regulations, or rules
      (collectively, "Laws") that protect human, civil, labor, privacy,
      political, environmental, security, economic, due process, or similar
      rights; provided, however, that such Laws are consistent and not in
      conflict with Human Rights Principles (a dispute over the consistency or a
      conflict between Laws and Human Rights Principles shall be determined by
      arbitration as stated above). Where the Human Rights Laws of more than one
      jurisdiction are applicable or in conflict with respect to the use of the
      Software, the Human Rights Laws that are most protective of the
      individuals or groups harmed shall apply.

      (( 3. ))??
      Indemnity. Licensee shall hold harmless and indemnify Licensor (and any
      other contributor) against all losses, damages, liabilities, deficiencies,
      claims, actions, judgments, settlements, interest, awards, penalties,
      fines, costs, or expenses of whatever kind, including Licensor's
      reasonable attorneys' fees, arising out of or relating to Licensee's use
      of the Software in violation of Human Rights Laws or Human Rights
      Principles.

   *
   Failure to Comply. Any failure of Licensee to act according to the terms and
   conditions of this License is both a breach of the License and an
   infringement of the intellectual property rights of the Licensor (subject to
   exceptions under Laws, e.g., fair use). In the event of a breach or
   infringement, the terms and conditions of this License may be enforced by
   Licensor under the Laws of any jurisdiction to which Licensee is subject.
   Licensee also agrees that the Licensor may enforce the terms and conditions
   of this License against Licensee through specific performance (or similar
   remedy under Laws) to the extent permitted by Laws. For clarity, except in
   the event of a breach of this License, infringement, or as otherwise stated
   in this License, Licensor may not terminate this License with Licensee.

   *
   Enforceability and Interpretation. If any term or provision of this License
   is determined to be invalid, illegal, or unenforceable by a court of
   competent jurisdiction, then such invalidity, illegality, or unenforceability
   shall not affect any other term or provision of this License or invalidate or
   render unenforceable such term or provision in any other jurisdiction;
   provided, however, subject to a court modification pursuant to the
   immediately following sentence, if any term or provision of this License
   pertaining to Human Rights Laws or Human Rights Principles is deemed invalid,
   illegal, or unenforceable against Licensee by a court of competent
   jurisdiction, all rights in the Software granted to Licensee shall be deemed
   null and void as between Licensor and Licensee. Upon a determination that any
   term or provision is invalid, illegal, or unenforceable, to the extent
   permitted by Laws, the court may modify this License to affect the original
   purpose that the Software be used in compliance with Human Rights Principles
   and Human Rights Laws as closely as possible. The language in this License
   shall be interpreted as to its fair meaning and not strictly for or against
   any party.

   *
   Disclaimer. TO THE FULL EXTENT ALLOWED BY LAW, THIS SOFTWARE COMES "AS IS,"
   WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED, AND LICENSOR AND ANY OTHER
   CONTRIBUTOR SHALL NOT BE LIABLE TO ANYONE FOR ANY DAMAGES OR OTHER LIABILITY
   ARISING FROM, OUT OF, OR IN CONNECTION WITH THE SOFTWARE OR THIS LICENSE,
   UNDER ANY KIND OF LEGAL CLAIM.

This Hippocratic License is an Ethical Source license (https:/ethicalsource.dev)
and is offered for use by licensors and licensees at their own risk, on an "AS
IS" basis, and with no warranties express or implied, to the maximum extent
permitted by Laws.
`
const license_IBM_pibs_lre = `//**
IBM PowerPC Initialization and Boot Software
https://spdx.org/licenses/IBM-pibs.json
http://git.denx.de/?p=u-boot.git;a=blob;f=arch/powerpc/cpu/ppc4xx/miiphy.c;h=297155fdafa064b955e53e9832de93bfb0cfb85b;hb=9fab4bf4cc077c21e43941866f3f2c196f28670d
**//

This source code has been made available to you by IBM on an AS-IS basis. Anyone
receiving this source is licensed under IBM copyrights to use it in any way he
or she deems fit, including copying it, modifying it, compiling it, and
redistributing it either with or without modifications. No license under IBM
patents or patent applications is to be implied by the copyright license.

Any user of this software should understand that IBM cannot provide technical
support for this software and will not be responsible for any consequences
resulting from the use of this software.

Any person who transfers this source code or any derivative work must include
the IBM copyright notice, this paragraph, and the preceding two paragraphs in
the transferred software.

COPYRIGHT I B M CORPORATION 2002

LICENSED MATERIAL - PROGRAM PROPERTY OF I B M
`
const license_ICU_lre = `//**
ICU License
https://spdx.org/licenses/ICU.json
http://source.icu-project.org/repos/icu/icu/trunk/license.html
**//

(( ICU License - ICU 1.8.1 and later
COPYRIGHT AND PERMISSION NOTICE
((Copyright __20__))??
))??

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, and/or sell copies of the
Software, and to permit persons to whom the Software is furnished to do so,
provided that the above copyright notice(s) and this permission notice appear in
all copies of the Software and that both the above copyright notice(s) and this
permission notice appear in supporting documentation.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR ANY
CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

Except as contained in this notice, the name of a copyright holder shall not be
used in advertising or otherwise to promote the sale, use or other dealings in
this Software without prior written authorization of the copyright holder.
`
const license_IJG_lre = `//**
Independent JPEG Group License
https://spdx.org/licenses/IJG.json
http://dev.w3.org/cvsweb/Amaya/libjpeg/Attic/README?rev=1.2
**//

(( Independent JPEG Group License ))??

(( LEGAL ISSUES

In plain English:

   (( 1. ))??
   We don't promise that this software works. (But if you find any bugs, please
   let us know!)

   (( 2. ))??
   You can use this software for whatever you want. You don't have to pay us.

   (( 3. ))??
   You may not pretend that you wrote this software. If you use it in a program,
   you must acknowledge somewhere in your documentation that you've used the IJG
   code.

In legalese: ))??

The authors make NO WARRANTY or representation, either express or implied, with
respect to this software, its quality, accuracy, merchantability, or fitness for
a particular purpose. This software is provided "AS IS", and you, its user,
assume the entire risk as to its quality and accuracy.

This software is copyright (C) 1991-1998, Thomas G. Lane. All Rights Reserved
except as specified below.

Permission is hereby granted to use, copy, modify, and distribute this software
(or portions thereof) for any purpose, without fee, subject to these
conditions:

   (( (1) ))??
   If any part of the source code for this software is distributed, then this
   README file must be included, with this copyright and no-warranty notice
   unaltered; and any additions, deletions, or changes to the original files
   must be clearly indicated in accompanying documentation.

   (( (2) ))??
   If only executable code is distributed, then the accompanying documentation
   must state that "this software is based in part on the work of the
   Independent JPEG Group".

   (( (3) ))??
   Permission for use of this software is granted only if the user accepts full
   responsibility for any undesirable consequences; the authors accept NO
   LIABILITY for damages of any kind.

These conditions apply to any software derived from or based on the IJG code,
not just to the unmodified library. If you use our work, you ought to
acknowledge us.

Permission is NOT granted for the use of any IJG author's name or company name
in advertising or publicity relating to this software or products derived from
it. This software may be referred to only as "the Independent JPEG Group's
software".

We specifically permit and encourage the use of this software as the basis of
commercial products, provided that all warranty or liability claims are assumed
by the product vendor.

ansi2knr.c is included in this distribution by permission of L. Peter Deutsch,
sole proprietor of its copyright holder, Aladdin Enterprises of Menlo Park, CA.
ansi2knr.c is NOT covered by the above copyright and conditions, but instead by
the usual distribution terms of the Free Software Foundation; principally, that
you must include source code if you redistribute it. (See the file ansi2knr.c
for full details.) However, since ansi2knr.c is not needed as part of any
program generated from the IJG code, this does not limit you more than the
foregoing paragraphs do.

The Unix configuration script "configure" was produced with GNU Autoconf. It is
copyright by the Free Software Foundation but is freely distributable. The same
holds for its supporting scripts (config.guess, config.sub, ltconfig,
ltmain.sh). Another support script, install-sh, is copyright by M.I.T. but is
also freely distributable.

It appears that the arithmetic coding option of the JPEG spec is covered by
patents owned by IBM, AT&T, and Mitsubishi. Hence arithmetic coding cannot
legally be used without obtaining one or more licenses. For this reason, support
for arithmetic coding has been removed from the free JPEG software. (Since
arithmetic coding provides only a marginal gain over the unpatented Huffman
mode, it is unlikely that very many implementations will support it.) So far as
we are aware, there are no patent restrictions on the remaining code.

The IJG distribution formerly included code to read and write GIF files. To
avoid entanglement with the Unisys LZW patent, GIF reading support has been
removed altogether, and the GIF writer has been simplified to produce
"uncompressed GIFs". This technique does not use the LZW algorithm; the
resulting GIF files are larger than usual, but are readable by all standard GIF
decoders.

We are required to state that

"The Graphics Interchange Format(c) is the Copyright property of CompuServe
Incorporated. GIF(sm) is a Service Mark property of CompuServe Incorporated."
`
const license_IPA_lre = `//**
IPA Font License
https://spdx.org/licenses/IPA.json
https://opensource.org/licenses/IPA
**//

(( IPA Font License Agreement v1.0 ))??

The Licensor provides the Licensed Program (as defined in Article 1 below) under
the terms of this license agreement ("Agreement"). Any use, reproduction or
distribution of the Licensed Program, or any exercise of rights under this
Agreement by a Recipient (as defined in Article 1 below) constitutes the
Recipient's acceptance of this Agreement.

Article 1 (Definitions)

   (( 1. ))??
   "Digital Font Program" shall mean a computer program containing, or used to
   render or display fonts.

   (( 2. ))??
   "Licensed Program" shall mean a Digital Font Program licensed by the Licensor
   under this Agreement.

   (( 3. ))??
   "Derived Program" shall mean a Digital Font Program created as a result of a
   modification, addition, deletion, replacement or any other adaptation to or
   of a part or all of the Licensed Program, and includes a case where a Digital
   Font Program newly created by retrieving font information from a part or all
   of the Licensed Program or Embedded Fonts from a Digital Document File with
   or without modification of the retrieved font information.

   (( 4. ))??
   "Digital Content" shall mean products provided to end users in the form of
   digital data, including video content, motion and/or still pictures, TV
   programs or other broadcasting content and products consisting of character
   text, pictures, photographic images, graphic symbols and/or the like.

   (( 5. ))??
   "Digital Document File" shall mean a PDF file or other Digital Content
   created by various software programs in which a part or all of the Licensed
   Program becomes embedded or contained in the file for the display of the font
   ("Embedded Fonts"). Embedded Fonts are used only in the display of characters
   in the particular Digital Document File within which they are embedded, and
   shall be distinguished from those in any Digital Font Program, which may be
   used for display of characters outside that particular Digital Document
   File.

   (( 6. ))??
   "Computer" shall include a server in this Agreement.

   (( 7. ))??
   "Reproduction and Other Exploitation" shall mean reproduction, transfer,
   distribution, lease, public transmission, presentation, exhibition,
   adaptation and any other exploitation.

   (( 8. ))??
   "Recipient" shall mean anyone who receives the Licensed Program under this
   Agreement, including one that receives the Licensed Program from a
   Recipient.

Article 2 (Grant of License)

The Licensor grants to the Recipient a license to use the Licensed Program in
any and all countries in accordance with each of the provisions set forth in
this Agreement. However, any and all rights underlying in the Licensed Program
shall be held by the Licensor. In no sense is this Agreement intended to
transfer any right relating to the Licensed Program held by the Licensor except
as specifically set forth herein or any right relating to any trademark, trade
name, or service mark to the Recipient.

   (( 1. ))??
   The Recipient may install the Licensed Program on any number of Computers and
   use the same in accordance with the provisions set forth in this Agreement.

   (( 2. ))??
   The Recipient may use the Licensed Program, with or without modification in
   printed materials or in Digital Content as an expression of character texts
   or the like.

   (( 3. ))??
   The Recipient may conduct Reproduction and Other Exploitation of the printed
   materials and Digital Content created in accordance with the preceding
   Paragraph, for commercial or non-commercial purposes and in any form of media
   including but not limited to broadcasting, communication and various
   recording media.

   (( 4. ))??
   If any Recipient extracts Embedded Fonts from a Digital Document File to
   create a Derived Program, such Derived Program shall be subject to the terms
   of this agreement.

   (( 5. ))??
   If any Recipient performs Reproduction or Other Exploitation of a Digital
   Document File in which Embedded Fonts of the Licensed Program are used only
   for rendering the Digital Content within such Digital Document File then such
   Recipient shall have no further obligations under this Agreement in relation
   to such actions.

   (( 6. ))??
   The Recipient may reproduce the Licensed Program as is without modification
   and transfer such copies, publicly transmit or otherwise redistribute the
   Licensed Program to a third party for commercial or non-commercial purposes
   ("Redistribute"), in accordance with the provisions set forth in Article 3
   Paragraph 2.

   (( 7. ))??
   The Recipient may create, use, reproduce and/or Redistribute a Derived
   Program under the terms stated above for the Licensed Program: provided, that
   the Recipient shall follow the provisions set forth in Article 3 Paragraph 1
   when Redistributing the Derived Program.

Article 3 (Restriction)

The license granted in the preceding Article shall be subject to the following
restrictions:

   (( 1. ))??
   If a Derived Program is Redistributed pursuant to Paragraph 4 and 7 of the
   preceding Article, the following conditions must be met :

      (( (1) ))??
      The following must be also Redistributed together with the Derived
      Program, or be made available online or by means of mailing mechanisms in
      exchange for a cost which does not exceed the total costs of postage,
      storage medium and handling fees:

         (( (a) ))??
         a copy of the Derived Program; and

         (( (b) ))??
         any additional file created by the font developing program in the
         course of creating the Derived Program that can be used for further
         modification of the Derived Program, if any.

      (( (2) ))??
      It is required to also Redistribute means to enable recipients of the
      Derived Program to replace the Derived Program with the Licensed Program
      first released under this License (the "Original Program"). Such means may
      be to provide a difference file from the Original Program, or instructions
      setting out a method to replace the Derived Program with the Original
      Program.

      (( (3) ))??
      The Recipient must license the Derived Program under the terms and
      conditions of this Agreement.

      (( (4) ))??
      No one may use or include the name of the Licensed Program as a program
      name, font name or file name of the Derived Program.

      (( (5) ))??
      Any material to be made available online or by means of mailing a medium
      to satisfy the requirements of this paragraph may be provided, verbatim,
      by any party wishing to do so.

   (( 2. ))??
   If the Recipient Redistributes the Licensed Program pursuant to Paragraph 6
   of the preceding Article, the Recipient shall meet all of the following
   conditions:

      (( (1) ))??
      The Recipient may not change the name of the Licensed Program.

      (( (2) ))??
      The Recipient may not alter or otherwise modify the Licensed Program.

      (( (3) ))??
      The Recipient must attach a copy of this Agreement to the Licensed
      Program.

   (( 3. ))??
   THIS LICENSED PROGRAM IS PROVIDED BY THE LICENSOR "AS IS" AND ANY EXPRESSED
   OR IMPLIED WARRANTY AS TO THE LICENSED PROGRAM OR ANY DERIVED PROGRAM,
   INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF TITLE, NON-INFRINGEMENT,
   MERCHANTABILITY, OR FITNESS FOR A PARTICULAR PURPOSE, ARE DISCLAIMED. IN NO
   EVENT SHALL THE LICENSOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXTENDED, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO; PROCUREMENT OF SUBSTITUTED GOODS OR SERVICE; DAMAGES ARISING FROM
   SYSTEM FAILURE; LOSS OR CORRUPTION OF EXISTING DATA OR PROGRAM; LOST
   PROFITS), HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
   STRICT LIABILITY OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
   WAY OUT OF THE INSTALLATION, USE, THE REPRODUCTION OR OTHER EXPLOITATION OF
   THE LICENSED PROGRAM OR ANY DERIVED PROGRAM OR THE EXERCISE OF ANY RIGHTS
   GRANTED HEREUNDER, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

   (( 4. ))??
   The Licensor is under no obligation to respond to any technical questions or
   inquiries, or provide any other user support in connection with the
   installation, use or the Reproduction and Other Exploitation of the Licensed
   Program or Derived Programs thereof.

Article 4 (Termination of Agreement)

   (( 1. ))??
   The term of this Agreement shall begin from the time of receipt of the
   Licensed Program by the Recipient and shall continue as long as the Recipient
   retains any such Licensed Program in any way.

   (( 2. ))??
   Notwithstanding the provision set forth in the preceding Paragraph, in the
   event of the breach of any of the provisions set forth in this Agreement by
   the Recipient, this Agreement shall automatically terminate without any
   notice. In the case of such termination, the Recipient may not use or conduct
   Reproduction and Other Exploitation of the Licensed Program or a Derived
   Program: provided that such termination shall not affect any rights of any
   other Recipient receiving the Licensed Program or the Derived Program from
   such Recipient who breached this Agreement.

Article 5 (Governing Law)

   (( 1. ))??
   IPA may publish revised and/or new versions of this License. In such an
   event, the Recipient may select either this Agreement or any subsequent
   version of the Agreement in using, conducting the Reproduction and Other
   Exploitation of, or Redistributing the Licensed Program or a Derived Program.
   Other matters not specified above shall be subject to the Copyright Law of
   Japan and other related laws and regulations of Japan.

   (( 2. ))??
   This Agreement shall be construed under the laws of Japan.
`
const license_IPL_1_0_lre = `//**
IBM Public License v1.0
https://spdx.org/licenses/IPL-1.0.json
https://opensource.org/licenses/IPL-1.0
**//

(( IBM Public License Version 1.0 ))??

THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS IBM

PUBLIC LICENSE ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION

OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.

   (( 1. ))??
   DEFINITIONS

   "Contribution" means:

      (( a. ))??
      in the case of International Business Machines Corporation ("IBM"), the
      Original Program, and

      (( b. ))??
      in the case of each Contributor,

         (( i. ))??
         changes to the Program, and

         (( ii. ))??
         additions to the Program; where such changes and/or additions to the
         Program originate from and are distributed by that particular
         Contributor. A Contribution 'originates' from a Contributor if it was
         added to the Program by such Contributor itself or anyone acting on
         such Contributor's behalf. Contributions do not include additions to
         the Program which:

            (( (i) ))??
            are separate modules of software distributed in conjunction with

            the Program under their own license agreement, and (ii) are not

            derivative works of the Program.

   "Contributor" means IBM and any other entity that distributes the Program.

   "Licensed Patents " mean patent claims licensable by a Contributor which are
   necessarily infringed by the use or sale of its Contribution alone or when
   combined with the Program.

   "Original Program" means the original version of the software accompanying
   this Agreement as released by IBM, including source code, object code and
   documentation, if any.

   "Program" means the Original Program and Contributions.

   "Recipient" means anyone who receives the Program under this Agreement,
   including all Contributors.

   (( 2. ))??
   GRANT OF RIGHTS

      (( a. ))??
      Subject to the terms of this Agreement, each Contributor hereby grants
      Recipient a non-exclusive, worldwide, royalty-free copyright license to
      reproduce, prepare derivative works of, publicly display, publicly
      perform, distribute and sublicense the Contribution of such Contributor,
      if any, and such derivative works, in source code and object code form.

      (( b. ))??
      Subject to the terms of this Agreement, each Contributor hereby grants
      Recipient a non-exclusive, worldwide, royalty-free patent license under
      Licensed Patents to make, use, sell, offer to sell, import and otherwise
      transfer the Contribution of such Contributor, if any, in source code and
      object code form. This patent license shall apply to the combination of
      the Contribution and the Program if, at the time the Contribution is added
      by the Contributor, such addition of the Contribution causes such
      combination to be covered by the Licensed Patents. The patent license
      shall not apply to any other combinations which include the Contribution.
      No hardware per se is licensed hereunder.

      (( c. ))??
      Recipient understands that although each Contributor grants the

      licenses to its Contributions set forth herein, no assurances are

      provided by any Contributor that the Program does not infringe the

      patent or other intellectual property rights of any other entity.

      Each Contributor disclaims any liability to Recipient for claims

      brought by any other entity based on infringement of intellectual

      property rights or otherwise. As a condition to exercising the

      rights and licenses granted hereunder, each Recipient hereby assumes

      sole responsibility to secure any other intellectual property rights

      needed, if any. For example, if a third party patent license is

      required to allow Recipient to distribute the Program, it is

      Recipient's responsibility to acquire that license before

      distributing the Program.

      (( d. ))??
      Each Contributor represents that to its knowledge it has

      sufficient copyright rights in its Contribution, if any, to grant the

      copyright license set forth in this Agreement.

   (( 3. ))??
   REQUIREMENTS

   A Contributor may choose to distribute the Program in object code form under
   its own license agreement, provided that:

      (( a. ))??
      it complies with the terms and conditions of this Agreement; and

      (( b. ))??
      its license agreement:

         (( i. ))??
         effectively disclaims on behalf of all Contributors all warranties and
         conditions, express and implied, including warranties or conditions of
         title and non-infringement, and implied warranties or conditions of
         merchantability and fitness for a particular purpose;

         (( ii. ))??
         effectively excludes on behalf of all Contributors all liability for
         damages, including direct, indirect, special, incidental and
         consequential damages, such as lost profits;

         (( iii. ))??
         states that any provisions which differ from this Agreement are offered
         by that Contributor alone and not by any other party; and

         (( iv. ))??
         states that source code for the Program is available from such
         Contributor, and informs licensees how to obtain it in a reasonable
         manner on or through a medium customarily used for software exchange.

   When the Program is made available in source code form:

      (( a. ))??
      it must be made available under this Agreement; and

      (( b. ))??
      a copy of this Agreement must be included with each copy of the Program.

   Each Contributor must include the following in a conspicuous location in the
   Program:

   Copyright (C) __5__ International Business Machines Corporation and
   others. All Rights Reserved.

   In addition, each Contributor must identify itself as the originator of its
   Contribution, if any, in a manner that reasonably allows subsequent
   Recipients to identify the originator of the Contribution.

   (( 4. ))??
   COMMERCIAL DISTRIBUTION

   Commercial distributors of software may accept certain responsibilities with
   respect to end users, business partners and the like. While this license is
   intended to facilitate the commercial use of the Program, the Contributor who
   includes the Program in a commercial product offering should do so in a
   manner which does not create potential liability for other Contributors.
   Therefore, if a Contributor includes the Program in a commercial product
   offering, such Contributor ("Commercial Contributor") hereby agrees to defend
   and indemnify every other Contributor ("Indemnified Contributor") against any
   losses, damages and costs (collectively "Losses") arising from claims,
   lawsuits and other legal actions brought by a third party against the
   Indemnified Contributor to the extent caused by the acts or omissions of such
   Commercial Contributor in connection with its distribution of the Program in
   a commercial product offering. The obligations in this section do not apply
   to any claims or Losses relating to any actual or alleged intellectual
   property infringement. In order to qualify, an Indemnified Contributor must:
   a) promptly notify the Commercial Contributor in writing of such claim, and
   b) allow the Commercial Contributor to control, and cooperate with the
   Commercial Contributor in, the defense and any related settlement
   negotiations. The Indemnified Contributor may participate in any such claim
   at its own expense.

   For example, a Contributor might include the Program in a commercial product
   offering, Product X. That Contributor is then a Commercial Contributor. If
   that Commercial Contributor then makes performance claims, or offers
   warranties related to Product X, those performance claims and warranties are
   such Commercial Contributor's responsibility alone. Under this section, the
   Commercial Contributor would have to defend claims against the other
   Contributors related to those performance claims and warranties, and if a
   court requires any other Contributor to pay any damages as a result, the
   Commercial Contributor must pay those damages.

   (( 5. ))??
   NO WARRANTY

   EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, THE PROGRAM IS PROVIDED ON
   AN "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER
   EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION, ANY WARRANTIES OR
   CONDITIONS OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A
   PARTICULAR PURPOSE. Each Recipient is solely responsible for determining the
   appropriateness of using and distributing the Program and assumes all risks
   associated with its exercise of rights under this Agreement, including but
   not limited to the risks and costs of program errors, compliance with
   applicable laws, damage to or loss of data, programs or equipment, and
   unavailability or interruption of operations.

   (( 6. ))??
   DISCLAIMER OF LIABILITY

   EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, NEITHER RECIPIENT NOR ANY
   CONTRIBUTORS SHALL HAVE ANY LIABILITY FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING WITHOUT LIMITATION
   LOST PROFITS), HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OR DISTRIBUTION OF THE PROGRAM OR THE
   EXERCISE OF ANY RIGHTS GRANTED HEREUNDER, EVEN IF ADVISED OF THE POSSIBILITY
   OF SUCH DAMAGES.

   (( 7. ))??
   GENERAL

   If any provision of this Agreement is invalid or unenforceable under
   applicable law, it shall not affect the validity or enforceability of the
   remainder of the terms of this Agreement, and without further action by the
   parties hereto, such provision shall be reformed to the minimum extent
   necessary to make such provision valid and enforceable.

   If Recipient institutes patent litigation against a Contributor with respect
   to a patent applicable to software (including a cross-claim or counterclaim
   in a lawsuit), then any patent licenses granted by that Contributor to such
   Recipient under this Agreement shall terminate as of the date such litigation
   is filed. In addition, if Recipient institutes patent litigation against any
   entity (including a cross-claim or counterclaim in a lawsuit) alleging that
   the Program itself (excluding combinations of the Program with other software
   or hardware) infringes such Recipient's patent(s), then such Recipient's
   rights granted under Section 2(b) shall terminate as of the date such
   litigation is filed.

   All Recipient's rights under this Agreement shall terminate if it fails to
   comply with any of the material terms or conditions of this Agreement and
   does not cure such failure in a reasonable period of time after becoming
   aware of such noncompliance. If all Recipient's rights under this Agreement
   terminate, Recipient agrees to cease use and distribution of the Program as
   soon as reasonably practicable. However, Recipient's obligations under this
   Agreement and any licenses granted by Recipient relating to the Program shall
   continue and survive.

   IBM may publish new versions (including revisions) of this Agreement from
   time to time. Each new version of the Agreement will be given a
   distinguishing version number. The Program (including Contributions) may
   always be distributed subject to the version of the Agreement under which it
   was received. In addition, after a new version of the Agreement is published,
   Contributor may elect to distribute the Program (including its Contributions)
   under the new version. No one other than IBM has the right to modify this
   Agreement. Except as expressly stated in Sections 2(a) and 2(b) above,
   Recipient receives no rights or licenses to the intellectual property of any
   Contributor under this Agreement, whether expressly, by implication, estoppel
   or otherwise. All rights in the Program not expressly granted under this
   Agreement are reserved.

   This Agreement is governed by the laws of the State of New York and the
   intellectual property laws of the United States of America. No party to this
   Agreement will bring a legal action under this Agreement more than one year
   after the cause of action arose. Each party waives its rights to a jury trial
   in any resulting litigation.
`
const license_ISC_lre = `//**
ISC License
https://spdx.org/licenses/ISC.json
https://www.isc.org/downloads/software-support-policy/isc-license/
https://opensource.org/licenses/ISC
**//

((
ISC License
((Copyright __20__))??
))??

Permission to use, copy, modify, and
(( /or ))??
distribute this software for any purpose with or without fee is hereby granted,
provided that the above
(( copyright ))??
notice and this permission notice appear in
all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND
//** ISC **//
__5__
DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
//** ISC **//
__5__
BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
`
const license_ImageMagick_lre = `//**
ImageMagick License
https://spdx.org/licenses/ImageMagick.json
http://www.imagemagick.org/script/license.php
**//

(( Before we get to the text of the license, lets just review what the license
says in simple terms:

It allows you to:

   * freely download and use ImageMagick software, in whole or in part, for
   personal, company internal, or commercial purposes;

   * use ImageMagick software in packages or distributions that you create;

   * link against a library under a different license;

   * link code under a different license against a library under this license;

   * merge code into a work under a different license;

   * extend patent grants to any code using code under this license;

   * and extend patent protection.

It forbids you to:

   * redistribute any piece of ImageMagick-originated software without proper
   attribution;

   * use any marks owned by ImageMagick Studio LLC in any way that might state
   or imply that ImageMagick Studio LLC endorses your distribution;

   * use any marks owned by ImageMagick Studio LLC in any way that might state
   or imply that you created the ImageMagick software in question.

It requires you to:

   * include a copy of the license in any redistribution you may make that
   includes ImageMagick software;

   * provide clear attribution to ImageMagick Studio LLC for any distributions
   that include ImageMagick software.

It does not require you to:

   * include the source of the ImageMagick software itself, or of any
   modifications you may have made to it, in any redistribution you may assemble
   that includes it;

   * submit changes that you make to the software back to the ImageMagick Studio
   LLC (though such feedback is encouraged).

A few other clarifications include:

   * ImageMagick is freely available without charge;

   * you may include ImageMagick on a DVD as long as you comply with the terms
   of the license;

   * you can give modified code away for free or sell it under the terms of the
   ImageMagick license or distribute the result under a different license, but
   you need to acknowledge the use of the ImageMagick software;

   * the license is compatible with the GPL V3.

   * when exporting the ImageMagick software, review its export classification.
   ))??

Terms and Conditions for Use, Reproduction, and Distribution

The legally binding and authoritative terms and conditions for use,
reproduction, and distribution of ImageMagick follow:

Copyright __30__

   (( 1. ))??
   Definitions.

   License shall mean the terms and conditions for use, reproduction, and
   distribution as defined by Sections 1 through 9 of this document.

   Licensor shall mean the copyright owner or entity authorized by the copyright
   owner that is granting the License.

   Legal Entity shall mean the union of the acting entity and all other entities
   that control, are controlled by, or are under common control with that
   entity. For the purposes of this definition, control means (i) the power,
   direct or indirect, to cause the direction or management of such entity,
   whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or
   more of the outstanding shares, or (iii) beneficial ownership of such
   entity.

   You (or Your) shall mean an individual or Legal Entity exercising permissions
   granted by this License.

   Source form shall mean the preferred form for making modifications, including
   but not limited to software source code, documentation source, and
   configuration files.

   Object form shall mean any form resulting from mechanical transformation or
   translation of a Source form, including but not limited to compiled object
   code, generated documentation, and conversions to other media types.

   Work shall mean the work of authorship, whether in Source or Object form,
   made available under the License, as indicated by a copyright notice that is
   included in or attached to the work (an example is provided in the Appendix
   below).

   Derivative Works shall mean any work, whether in Source or Object form, that
   is based on (or derived from) the Work and for which the editorial revisions,
   annotations, elaborations, or other modifications represent, as a whole, an
   original work of authorship. For the purposes of this License, Derivative
   Works shall not include works that remain separable from, or merely link (or
   bind by name) to the interfaces of, the Work and Derivative Works thereof.

   Contribution shall mean any work of authorship, including the original
   version of the Work and any modifications or additions to that Work or
   Derivative Works thereof, that is intentionally submitted to Licensor for
   inclusion in the Work by the copyright owner or by an individual or Legal
   Entity authorized to submit on behalf of the copyright owner. For the
   purposes of this definition, "submitted" means any form of electronic,
   verbal, or written communication sent to the Licensor or its representatives,
   including but not limited to communication on electronic mailing lists,
   source code control systems, and issue tracking systems that are managed by,
   or on behalf of, the Licensor for the purpose of discussing and improving the
   Work, but excluding communication that is conspicuously marked or otherwise
   designated in writing by the copyright owner as Not a Contribution.

   Contributor shall mean Licensor and any individual or Legal Entity on behalf
   of whom a Contribution has been received by Licensor and subsequently
   incorporated within the Work.

   (( 2. ))??
   Grant of Copyright License. Subject to the terms and conditions of this
   License, each Contributor hereby grants to You a perpetual, worldwide,
   non-exclusive, no-charge, royalty-free, irrevocable copyright license to
   reproduce, prepare Derivative Works of, publicly display, publicly perform,
   sublicense, and distribute the Work and such Derivative Works in Source or
   Object form.

   (( 3. ))??
   Grant of Patent License. Subject to the terms and conditions of this License,
   each Contributor hereby grants to You a perpetual, worldwide, non-exclusive,
   no-charge, royalty-free, irrevocable patent license to make, have made, use,
   offer to sell, sell, import, and otherwise transfer the Work, where such
   license applies only to those patent claims licensable by such Contributor
   that are necessarily infringed by their Contribution(s) alone or by
   combination of their Contribution(s) with the Work to which such
   Contribution(s) was submitted. If You institute patent litigation against any
   entity (including a cross-claim or counterclaim in a lawsuit) alleging that
   the Work or a Contribution incorporated within the Work constitutes direct or
   contributory patent infringement, then any patent licenses granted to You
   under this License for that Work shall terminate as of the date such
   litigation is filed.

   (( 4. ))??
   Redistribution. You may reproduce and distribute copies of the Work or
   Derivative Works thereof in any medium, with or without modifications, and in
   Source or Object form, provided that You meet the following conditions:

      (( a. ))??
      You must give any other recipients of the Work or Derivative Works a copy
      of this License; and

      (( b. ))??
      You must cause any modified files to carry prominent notices stating that
      You changed the files; and

      (( c. ))??
      You must retain, in the Source form of any Derivative Works that You
      distribute, all copyright, patent, trademark, and attribution notices from
      the Source form of the Work, excluding those notices that do not pertain
      to any part of the Derivative Works; and

      (( d. ))??
      If the Work includes a "NOTICE" text file as part of its distribution,
      then any Derivative Works that You distribute must include a readable copy
      of the attribution notices contained within such NOTICE file, excluding
      those notices that do not pertain to any part of the Derivative Works, in
      at least one of the following places: within a NOTICE text file
      distributed as part of the Derivative Works; within the Source form or
      documentation, if provided along with the Derivative Works; or, within a
      display generated by the Derivative Works, if and wherever such
      third-party notices normally appear. The contents of the NOTICE file are
      for informational purposes only and do not modify the License. You may add
      Your own attribution notices within Derivative Works that You distribute,
      alongside or as an addendum to the NOTICE text from the Work, provided
      that such additional attribution notices cannot be construed as modifying
      the License.

   You may add Your own copyright statement to Your modifications and may
   provide additional or different license terms and conditions for use,
   reproduction, or distribution of Your modifications, or for any such
   Derivative Works as a whole, provided Your use, reproduction, and
   distribution of the Work otherwise complies with the conditions stated in
   this License.

   (( 5. ))??
   Submission of Contributions. Unless You explicitly state otherwise, any
   Contribution intentionally submitted for inclusion in the Work by You to the
   Licensor shall be under the terms and conditions of this License, without any
   additional terms or conditions. Notwithstanding the above, nothing herein
   shall supersede or modify the terms of any separate license agreement you may
   have executed with Licensor regarding such Contributions.

   (( 6. ))??
   Trademarks. This License does not grant permission to use the trade names,
   trademarks, service marks, or product names of the Licensor, except as
   required for reasonable and customary use in describing the origin of the
   Work and reproducing the content of the NOTICE file.

   (( 7. ))??
   Disclaimer of Warranty. Unless required by applicable law or agreed to in
   writing, Licensor provides the Work (and each Contributor provides its
   Contributions) on an AS IS BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
   KIND, either express or implied, including, without limitation, any
   warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or
   FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining
   the appropriateness of using or redistributing the Work and assume any risks
   associated with Your exercise of permissions under this License.

   (( 8. ))??
   Limitation of Liability. In no event and under no legal theory, whether in
   tort (including negligence), contract, or otherwise, unless required by
   applicable law (such as deliberate and grossly negligent acts) or agreed to
   in writing, shall any Contributor be liable to You for damages, including any
   direct, indirect, special, incidental, or consequential damages of any
   character arising as a result of this License or out of the use or inability
   to use the Work (including but not limited to damages for loss of goodwill,
   work stoppage, computer failure or malfunction, or any and all other
   commercial damages or losses), even if such Contributor has been advised of
   the possibility of such damages.

   (( 9. ))??
   Accepting Warranty or Additional Liability. While redistributing the Work or
   Derivative Works thereof, You may choose to offer, and charge a fee for,
   acceptance of support, warranty, indemnity, or other liability obligations
   and/or rights consistent with this License. However, in accepting such
   obligations, You may act only on Your own behalf and on Your sole
   responsibility, not on behalf of any other Contributor, and only if You agree
   to indemnify, defend, and hold each Contributor harmless for any liability
   incurred by, or claims asserted against, such Contributor by reason of your
   accepting any such warranty or additional liability.
   (( How to Apply the License to your Work

To apply the ImageMagick License to your work, attach the following boilerplate
notice, with the fields enclosed by brackets "[]" replaced with your own
identifying information (don't include the brackets). The text should be
enclosed in the appropriate comment syntax for the file format. We also
recommend that a file or class name and description of purpose be included on
the same "printed page" as the copyright notice for easier identification within
third-party archives.

Copyright [yyyy] [name of copyright owner]

Licensed under the ImageMagick License (the "License"); you may not use this
file except in compliance with the License. You may obtain a copy of the License
at

http:/www.imagemagick.org/script/license.php

Unless required by applicable law or agreed to in writing, software distributed
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
))??
`
const license_Imlib2_lre = `//**
Imlib2 License
https://spdx.org/licenses/Imlib2.json
http://trac.enlightenment.org/e/browser/trunk/imlib2/COPYING
https://git.enlightenment.org/legacy/imlib2.git/tree/COPYING
**//

(( Imlib2 License ))??

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies of the Software and its Copyright notices. In addition publicly
documented acknowledgment must be given that this software has been used if no
source code of this software is made available publicly. Making the source
available publicly means including the source for this software with the
distribution, or a method to get this software via some reasonable mechanism
(electronic transfer via a network or media) as well as making an offer to
supply the source on request. This Copyright notice serves as an offer to supply
the source on on request as well. Instead of this, supplying acknowledgments of
use of this software in either Copyright notices, Manuals, Publicity and
Marketing documents or any documentation provided with any product containing
this software. This License does not apply to any software that links to the
libraries provided by this software (statically or dynamically), but only to the
software provided.

Please see the COPYING-PLAIN for a plain-english explanation of this notice and
its intent.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
`
const license_Info_ZIP_lre = `//**
Info-ZIP License
https://spdx.org/licenses/Info-ZIP.json
http://www.info-zip.org/license.html
**//

(( Info-ZIP License
(( Copyright __20__ ))??
))??

For the purposes of this copyright and license, "Info-ZIP" is defined as the
following set of individuals:

Mark Adler, John Bush, Karl Davis, Harald Denker, Jean-Michel Dubois, Jean-loup
Gailly, Hunter Goatley, Ed Gordon, Ian Gorman, Chris Herborth, Dirk Haase, Greg
Hartwig, Robert Heath, Jonathan Hudson, Paul Kienitz, David Kirschbaum, Johnny
Lee, Onno van der Linden, Igor Mandrichenko, Steve P. Miller, Sergio Monesi,
Keith Owens, George Petrov, Greg Roelofs, Kai Uwe Rommel, Steve Salisbury, Dave
Smith, Steven M. Schweda, Christian Spieler, Cosmin Truta, Antoine Verheijen,
Paul von Behren, Rich Wales, Mike White.

This software is provided "as is," without warranty of any kind, express or
implied. In no event shall Info-ZIP or its contributors be held liable for any
direct, indirect, incidental, special or consequential damages arising out of
the use of or inability to use this software.

Permission is granted to anyone to use this software for any purpose, including
commercial applications, and to alter it and redistribute it freely, subject to
the above disclaimer and the following restrictions:

   *
   Redistributions of source code (in whole or in part) must retain the above
   copyright notice, definition, disclaimer, and this list of conditions.

   *
   Redistributions in binary form (compiled executables and libraries) must
   reproduce the above copyright notice, definition, disclaimer, and this list
   of conditions in documentation and/or other materials provided with the
   distribution. Additional documentation is not needed for executables where a
   command line license option provides these and a note regarding this option
   is in the executable's startup banner. The sole exception to this condition
   is redistribution of a standard UnZipSFX binary (including SFXWiz) as part of
   a self-extracting archive; that is permitted without inclusion of this
   license, as long as the normal SFX banner has not been removed from the
   binary or disabled.

   *
   Altered versions--including, but not limited to, ports to new operating
   systems, existing ports with new graphical interfaces, versions with modified
   or added functionality, and dynamic, shared, or static library versions not
   from Info-ZIP--must be plainly marked as such and must not be misrepresented
   as being the original source or, if binaries, compiled from the original
   source. Such altered versions also must not be misrepresented as being
   Info-ZIP releases--including, but not limited to, labeling of the altered
   versions with the names "Info-ZIP" (or any variation thereof, including, but
   not limited to, different capitalizations), "Pocket UnZip," "WiZ" or "MacZip"
   without the explicit permission of Info-ZIP. Such altered versions are
   further prohibited from misrepresentative use of the Zip-Bugs or Info-ZIP
   e-mail addresses or the Info-ZIP URL(s), such as to imply Info-ZIP will
   provide support for the altered versions.

   *
   Info-ZIP retains the right to use the names "Info-ZIP," "Zip," "UnZip,"
   "UnZipSFX," "WiZ," "Pocket UnZip," "Pocket Zip," and "MacZip" for its own
   source and binary releases.
`
const license_Intel_lre = `//**
Intel Open Source License
https://spdx.org/licenses/Intel.json
https://opensource.org/licenses/Intel
**//

(( Intel Open Source License
(( Copyright __20__ ))??
))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   •
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   •
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   •
   Neither the name of the Intel Corporation nor the names of its contributors
   may be used to endorse or promote products derived from this software without
   specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE INTEL OR CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

EXPORT LAWS: THIS LICENSE ADDS NO RESTRICTIONS TO THE EXPORT LAWS OF YOUR
JURISDICTION. It is licensee's responsibility to comply with any export
regulations applicable in licensee's jurisdiction. Under CURRENT (May 2000) U.S.
export regulations this software is eligible for export from the U.S. and can be
downloaded by or otherwise exported or reexported worldwide EXCEPT to U.S.
embargoed destinations which include Cuba, Iraq, Libya, North Korea, Iran,
Syria, Sudan, Afghanistan and any other country to which the U.S. has embargoed
goods and services.
`
const license_Intel_ACPI_lre = `//**
Intel ACPI Software License Agreement
https://spdx.org/licenses/Intel-ACPI.json
https://fedoraproject.org/wiki/Licensing/Intel_ACPI_Software_License_Agreement
**//

(( ACPI - Software License Agreement ))??

Software License Agreement IMPORTANT - READ BEFORE COPYING, INSTALLING OR
USING.

Do not use or load this software and any associated materials (collectively, the
"Software") until you have carefully read the following terms and conditions. By
loading or using the Software, you agree to the terms of this Agreement. If you
do not wish to so agree, do not install or use the Software.

   (( 1. ))??
   COPYRIGHT NOTICE

   __20__. All rights reserved.

   (( 2. ))??
   LICENSE

      (( 2.1. ))??
      This is your license from Intel Corp. under its intellectual property
      rights. You may have additional license terms from the party that provided
      you this software, covering your right to use that party's intellectual
      property rights.

      (( 2.2. ))??
      Intel grants, free of charge, to any person ("Licensee") obtaining a copy
      of the source code appearing in this file ("Covered Code") an irrevocable,
      perpetual, worldwide license under Intel's copyrights in the base code
      distributed originally by Intel ("Original Intel Code") to copy, make
      derivatives, distribute, use and display any portion of the Covered Code
      in any form, with the right to sublicense such rights; and

      (( 2.3. ))??
      Intel grants Licensee a non-exclusive and non-transferable patent license
      (with the right to sublicense), under only those claims of Intel patents
      that are infringed by the Original Intel Code, to make, use, sell, offer
      to sell, and import the Covered Code and derivative works thereof solely
      to the minimum extent necessary to exercise the above copyright license,
      and in no event shall the patent license extend to any additions to or
      modifications of the Original Intel Code. No other license or right is
      granted directly or by implication, estoppel or otherwise; The above
      copyright and patent license is granted only if the following conditions
      are met:

   (( 3. ))??
   CONDITIONS

      (( 3.1. ))??
      Redistribution of Source with Rights to Further Distribute Source.
      Redistribution of source code of any substantial portion of the Covered
      Code or modification with rights to further distribute source must include
      the above Copyright Notice, the above License, this list of Conditions,
      and the following Disclaimer and Export Compliance provision. In addition,
      Licensee must cause all Covered Code to which Licensee contributes to
      contain a file documenting the changes Licensee made to create that
      Covered Code and the date of any change. Licensee must include in that
      file the documentation of any changes made by any predecessor Licensee.
      Licensee must include a prominent statement that the modification is
      derived, directly or indirectly, from Original Intel Code.

      (( 3.2. ))??
      Redistribution of Source with no Rights to Further Distribute Source.
      Redistribution of source code of any substantial portion of the Covered
      Code or modification without rights to further distribute source must
      include the following Disclaimer and Export Compliance provision in the
      documentation and/or other materials provided with distribution. In
      addition, Licensee may not authorize further sublicense of source of any
      portion of the Covered Code, and must include terms to the effect that the
      license from Licensee to its licensee is limited to the intellectual
      property embodied in the software Licensee provides to its licensee, and
      not to intellectual property embodied in modifications its licensee may
      make.

      (( 3.3. ))??
      Redistribution of Executable. Redistribution in executable form of any
      substantial portion of the Covered Code or modification must reproduce the
      above Copyright Notice, and the following Disclaimer and Export Compliance
      provision in the documentation and/or other materials provided with the
      distribution.

      (( 3.4. ))??
      Intel retains all right, title, and interest in and to the Original Intel
      Code.

      (( 3.5. ))??
      Neither the name Intel nor any other trademark owned or controlled by
      Intel shall be used in advertising or otherwise to promote the sale, use
      or other dealings in products derived from or relating to the Covered Code
      without prior written authorization from Intel.

   (( 4. ))??
   DISCLAIMER AND EXPORT COMPLIANCE

      (( 4.1. ))??
      INTEL MAKES NO WARRANTY OF ANY KIND REGARDING ANY SOFTWARE PROVIDED HERE.
      ANY SOFTWARE ORIGINATING FROM INTEL OR DERIVED FROM INTEL SOFTWARE IS
      PROVIDED "AS IS," AND INTEL WILL NOT PROVIDE ANY SUPPORT, ASSISTANCE,
      INSTALLATION, TRAINING OR OTHER SERVICES. INTEL WILL NOT PROVIDE ANY
      UPDATES, ENHANCEMENTS OR EXTENSIONS. INTEL SPECIFICALLY DISCLAIMS ANY
      IMPLIED WARRANTIES OF MERCHANTABILITY, NONINFRINGEMENT AND FITNESS FOR A
      PARTICULAR PURPOSE.

      (( 4.2. ))??
      IN NO EVENT SHALL INTEL HAVE ANY LIABILITY TO LICENSEE, ITS LICENSEES OR
      ANY OTHER THIRD PARTY, FOR ANY LOST PROFITS, LOST DATA, LOSS OF USE OR
      COSTS OF PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES, OR FOR ANY INDIRECT,
      SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THIS AGREEMENT, UNDER ANY
      CAUSE OF ACTION OR THEORY OF LIABILITY, AND IRRESPECTIVE OF WHETHER INTEL
      HAS ADVANCE NOTICE OF THE POSSIBILITY OF SUCH DAMAGES. THESE LIMITATIONS
      SHALL APPLY NOTWITHSTANDING THE FAILURE OF THE ESSENTIAL PURPOSE OF ANY
      LIMITED REMEDY.

      (( 4.3. ))??
      Licensee shall not export, either directly or indirectly, any of this
      software or system incorporating such software without first obtaining any
      required license or other approval from the U. S. Department of Commerce
      or any other agency or department of the United States Government. In the
      event Licensee exports any such software from the United States or
      re-exports any such software from a foreign destination, Licensee shall
      ensure that the distribution and export/re-export of the software is in
      compliance with all laws, regulations, orders, or other restrictions of
      the U.S. Export Administration Regulations. Licensee agrees that neither
      it nor any of its subsidiaries will export/re-export any technical data,
      process, software, or service, directly or indirectly, to any country for
      which the United States government or any agency thereof requires an
      export license, other governmental approval, or letter of assurance,
      without first obtaining such license, approval or letter.
`
const license_Interbase_1_0_lre = `//**
Interbase Public License v1.0
https://spdx.org/licenses/Interbase-1.0.json
https://web.archive.org/web/20060319014854/http://info.borland.com/devsupport/interbase/opensource/IPL.html
**//

(( INTERBASE PUBLIC LICENSE

Version 1.0 ))??

   (( 1. ))??
   Definitions.

      (( 1.0.1. ))??
      "Commercial Use" means distribution or otherwise making the Covered Code
      available to a third party.

      (( 1.1. ))??
      ''Contributor'' means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      ''Contributor Version'' means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      ''Covered Code'' means the Original Code or Modifications or the
      combination of the Original Code and Modifications, in each case including
      portions thereof.

      (( 1.4. ))??
      ''Electronic Distribution Mechanism'' means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      ''Executable'' means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      ''Initial Developer'' means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7. ))??
      ''Larger Work'' means a work which combines Covered Code or portions
      thereof with code not governed by the terms of this License.

      (( 1.8. ))??
      ''License'' means this document.

      (( 1.8.1. ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.9. ))??
      ''Modifications'' means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10. ))??
      ''Original Code'' means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      (( 1.10.1. ))??
      "Patent Claims" means any patent claim(s), now owned or hereafter
      acquired, including without limitation, method, process, and apparatus
      claims, in any patent Licensable by grantor.

      (( 1.11. ))??
      ''Source Code'' means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or source code differential comparisons
      against either the Original Code or another well known, available Covered
      Code of the Contributor's choice. The Source Code can be in a compressed
      or archival form, provided the appropriate decompression or de-archiving
      software is widely available for no charge.

      (( 1.12. ))??
      "You'' (or "Your") means an individual or a legal entity exercising rights
      under, and complying with all of the terms of, this License or a future
      version of this License issued under Section 6.1. For legal entities,
      "You'' includes any entity which controls, is controlled by, or is under
      common control with You. For purposes of this definition, "control'' means
      (a) the power, direct or indirect, to cause the direction or management of
      such entity, whether by contract or otherwise, or (b) ownership of more
      than fifty percent (50%) of the outstanding shares or beneficial ownership
      of such entity.

   (( 2. ))??
   Source Code License.

      (( 2.1. ))??
      The Initial Developer Grant.

      The Initial Developer hereby grants You a world-wide, royalty-free,
      non-exclusive license, subject to third party intellectual property
      claims:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Initial Developer to use, reproduce, modify, display,
         perform, sublicense and distribute the Original Code (or portions
         thereof) with or without Modifications, and/or as part of a Larger
         Work; and

         (( (b) ))??
         under Patents Claims infringed by the making, using or selling of
         Original Code, to make, have made, use, practice, sell, and offer for
         sale, and/or otherwise dispose of the Original Code (or portions
         thereof).

         (( c ))??
         the licenses granted in this Section 2.1(a) and (b) are effective on
         the date Initial Developer first distributes Original Code under the
         terms of this License.

         (( (d) ))??
         Notwithstanding Section 2.1(b) above, no patent license is granted: 1)
         for code that You delete from the Original Code; 2) separate from the
         Original Code; or 3) for infringements caused by: i) the modification
         of the Original Code or ii) the combination of the Original Code with
         other software or devices.

      (( 2.2. ))??
      Contributor Grant.

      Subject to third party intellectual property claims, each Contributor
      hereby grants You a world-wide, royalty-free, non-exclusive license

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Contributor, to use, reproduce, modify, display, perform,
         sublicense and distribute the Modifications created by such Contributor
         (or portions thereof) either on an unmodified basis, with other
         Modifications, as Covered Code and/or as part of a Larger Work; and

         (( (b) ))??
         under Patent Claims infringed by the making, using, or selling of
         Modifications made by that Contributor either alone and/or in
         combination with its Contributor Version (or portions of such
         combination), to make, use, sell, offer for sale, have made, and/or
         otherwise dispose of: 1) Modifications made by that Contributor (or
         portions thereof); and 2) the combination of Modifications made by that
         Contributor with its Contributor Version (or portions of such
         combination).

         (( c ))??
         the licenses granted in Sections 2.2(a) and 2.2(b) are effective on the
         date Contributor first makes Commercial Use of the Covered Code.

         (( (d) ))??
         Notwithstanding Section 2.2(b) above, no patent license is granted: 1)
         for any code that Contributor has deleted from the Contributor Version;
         2) separate from the Contributor Version; 3) for infringements caused
         by: i) third party modifications of Contributor Version or ii) the
         combination of Modifications made by that Contributor with other
         software (except as part of the Contributor Version) or other devices;
         or 4) under Patent Claims infringed by Covered Code in the absence of
         Modifications made by that Contributor.

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Application of License.

      The Modifications which You create or to which You contribute are governed
      by the terms of this License, including without limitation Section 2.2.
      The Source Code version of Covered Code may be distributed only under the
      terms of this License or a future version of this License released under
      Section 6.1, and You must include a copy of this License with every copy
      of the Source Code You distribute. You may not offer or impose any terms
      on any Source Code version that alters or restricts the applicable version
      of this License or the recipients' rights hereunder. However, You may
      include an additional document offering the additional rights described in
      Section 3.5.

      (( 3.2. ))??
      Availability of Source Code.

      Any Modification which You create or to which You contribute must be made
      available in Source Code form under the terms of this License either on
      the same media as an Executable version or via an accepted Electronic
      Distribution Mechanism to anyone to whom you made an Executable version
      available; and if made available via Electronic Distribution Mechanism,
      must remain available for at least twelve (12) months after the date it
      initially became available, or at least six (6) months after a subsequent
      version of that particular Modification has been made available to such
      recipients. You are responsible for ensuring that the Source Code version
      remains available even if the Electronic Distribution Mechanism is
      maintained by a third party.

      (( 3.3. ))??
      Description of Modifications.

      You must cause all Covered Code to which You contribute to contain a file
      documenting the changes You made to create that Covered Code and the date
      of any change. You must include a prominent statement that the
      Modification is derived, directly or indirectly, from Original Code
      provided by the Initial Developer and including the name of the Initial
      Developer in (a) the Source Code, and (b) in any notice in an Executable
      version or related documentation in which You describe the origin or
      ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters

         (( (a) ))??
         Third Party Claims.

         If Contributor has knowledge that a license under a third party's
         intellectual property rights is required to exercise the rights granted
         by such Contributor under Sections 2.1 or 2.2, Contributor must include
         a text file with the Source Code distribution titled "LEGAL'' which
         describes the claim and the party making the claim in sufficient detail
         that a recipient will know whom to contact. If Contributor obtains such
         knowledge after the Modification is made available as described in
         Section 3.2, Contributor shall promptly modify the LEGAL file in all
         copies Contributor makes available thereafter and shall take other
         steps (such as notifying appropriate mailing lists or newsgroups)
         reasonably calculated to inform those who received the Covered Code
         that new knowledge has been obtained.

         (( (b) ))??
         Contributor APIs.

         If Contributor's Modifications include an application programming
         interface and Contributor has knowledge of patent licenses which are
         reasonably necessary to implement that API, Contributor must also
         include this information in the LEGAL file.

         (( c ))??
         Representations.

         Contributor represents that, except as disclosed pursuant to Section
         3.4(a) above, Contributor believes that Contributor's Modifications are
         Contributor's original creation(s) and/or Contributor has sufficient
         rights to grant the rights conveyed by this License.

      (( 3.5. ))??
      Required Notices.

      You must duplicate the notice in Exhibit A in each file of the Source
      Code. If it is not possible to put such notice in a particular Source Code
      file due to its structure, then You must include such notice in a location
      (such as a relevant directory) where a user would be likely to look for
      such a notice. If You created one or more Modification(s) You may add your
      name as a Contributor to the notice described in Exhibit A. You must also
      duplicate this License in any documentation for the Source Code where You
      describe recipients' rights or ownership rights relating to Covered Code.
      You may choose to offer, and to charge a fee for, warranty, support,
      indemnity or liability obligations to one or more recipients of Covered
      Code. However, You may do so only on Your own behalf, and not on behalf of
      the Initial Developer or any Contributor. You must make it absolutely
      clear than any such warranty, support, indemnity or liability obligation
      is offered by You alone, and You hereby agree to indemnify the Initial
      Developer and every Contributor for any liability incurred by the Initial
      Developer or such Contributor as a result of warranty, support, indemnity
      or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Versions.

      You may distribute Covered Code in Executable form only if the
      requirements of Section 3.1-3.5 have been met for that Covered Code, and
      if You include a notice stating that the Source Code version of the
      Covered Code is available under the terms of this License, including a
      description of how and where You have fulfilled the obligations of Section
      3.2. The notice must be conspicuously included in any notice in an
      Executable version, related documentation or collateral in which You
      describe recipients' rights relating to the Covered Code. You may
      distribute the Executable version of Covered Code or ownership rights
      under a license of Your choice, which may contain terms different from
      this License, provided that You are in compliance with the terms of this
      License and that the license for the Executable version does not attempt
      to limit or alter the recipient's rights in the Source Code version from
      the rights set forth in this License. If You distribute the Executable
      version under a different license You must make it absolutely clear that
      any terms which differ from this License are offered by You alone, not by
      the Initial Developer or any Contributor. You hereby agree to indemnify
      the Initial Developer and every Contributor for any liability incurred by
      the Initial Developer or such Contributor as a result of any such terms
      You offer.

      (( 3.7. ))??
      Larger Works.

      You may create a Larger Work by combining Covered Code with other code not
      governed by the terms of this License and distribute the Larger Work as a
      single product. In such a case, You must make sure the requirements of
      this License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute, judicial
   order, or regulation then You must: (a) comply with the terms of this License
   to the maximum extent possible; and (b) describe the limitations and the code
   they affect. Such description must be included in the LEGAL file described in
   Section 3.4 and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Application of this License.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A and to related Covered Code.

   (( 6. ))??
   Versions of the License.

      (( 6.1. ))??
      New Versions.

      Borland Software Corporation (''Interbase'') may publish revised and/or
      new versions of the License from time to time. Each version will be given
      a distinguishing version number.

      (( 6.2. ))??
      Effect of New Versions.

      Once Covered Code has been published under a particular version of the
      License, You may always continue to use it under the terms of that
      version. You may also choose to use such Covered Code under the terms of
      any subsequent version of the License published by Interbase. No one other
      than Interbase has the right to modify the terms applicable to Covered
      Code created under this License.

      (( 6.3. ))??
      Derivative Works.

      If You create or use a modified version of this License (which you may
      only do in order to apply it to code which is not already Covered Code
      governed by this License), You must (a) rename Your license so that the
      phrases ''Mozilla'', ''MOZILLAPL'', ''MOZPL'', ''Netscape'', "MPL",
      ''NPL", "Interbase", "ISC", "IB'' or any confusingly similar phrase do not
      appear in your license (except to note that your license differs from this
      License) and (b) otherwise make it clear that Your version of the license
      contains terms which differ from the Mozilla Public License and Netscape
      Public License. (Filling in the name of the Initial Developer, Original
      Code or Contributor in the notice described in Exhibit A shall not of
      themselves be deemed to be modifications of this License.)

      (( 6.4 ))??
      Origin of the Interbase Public License.

      The Interbase public license is based on the Mozilla Public License V 1.1
      with the following changes:

      The license is published by Borland Software Corporation. Only Borland
      Software Corporation can modify the terms applicable to Covered Code.

      The license can be modified used for code which is not already governed by
      this license. Modified versions of the license must be renamed to avoid
      confusion with Netscape?s or Interbase Software?s license and must include
      a description of changes from the Interbase Public License.

      The name of the license in Exhibit A is the "Interbase Public License".

      The reference to an alternative license in Exhibit A has been removed.

      Amendments I, II, III, V, and VI have been deleted.

      Exhibit A, Netscape Public License has been deleted

      A new amendment (II) has been added, describing the required and
      restricted rights to use the trademarks of Borland Software Corporation

   (( 7. ))??
   DISCLAIMER OF WARRANTY.

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS'' BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER
   OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR
   OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF
   THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

   (( 8. ))??
   TERMINATION.

      (( 8.1. ))??
      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. All sublicenses to the
      Covered Code which are properly granted shall survive any termination of
      this License. Provisions which, by their nature, must remain in effect
      beyond the termination of this License shall survive.

      (( 8.2. ))??
      If You initiate litigation by asserting a patent infringement claim
      (excluding declatory judgment actions) against Initial Developer or a
      Contributor (the Initial Developer or Contributor against whom You file
      such action is referred to as "Participant") alleging that:

         (( (a) ))??
         such Participant's Contributor Version directly or indirectly infringes
         any patent, then any and all rights granted by such Participant to You
         under Sections 2.1 and/or 2.2 of this License shall, upon 60 days
         notice from Participant terminate prospectively, unless if within 60
         days after receipt of notice You either: (i) agree in writing to pay
         Participant a mutually agreeable reasonable royalty for Your past and
         future use of Modifications made by such Participant, or (ii) withdraw
         Your litigation claim with respect to the Contributor Version against
         such Participant. If within 60 days of notice, a reasonable royalty and
         payment arrangement are not mutually agreed upon in writing by the
         parties or the litigation claim is not withdrawn, the rights granted by
         Participant to You under Sections 2.1 and/or 2.2 automatically
         terminate at the expiration of the 60 day notice period specified
         above.

         (( (b) ))??
         any software, hardware, or device, other than such Participant's
         Contributor Version, directly or indirectly infringes any patent, then
         any rights granted to You by such Participant under Sections 2.1(b) and
         2.2(b) are revoked effective as of the date You first made, used, sold,
         distributed, or had made, Modifications made by that Participant.

      (( 8.3. ))??
      If You assert a patent infringement claim against Participant alleging
      that such Participant's Contributor Version directly or indirectly
      infringes any patent where such claim is resolved (such as by license or
      settlement) prior to the initiation of patent infringement litigation,
      then the reasonable value of the licenses granted by such Participant
      under Sections 2.1 or 2.2 shall be taken into account in determining the
      amount or value of any payment or license.

      (( 8.4. ))??
      In the event of termination under Sections 8.1 or 8.2 above, all end user
      license agreements (excluding distributors and resellers) which have been
      validly granted by You or any distributor hereunder prior to termination
      shall survive termination.

   (( 9. ))??
   LIMITATION OF LIABILITY.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, THE INITIAL DEVELOPER, ANY
   OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE, OR ANY SUPPLIER OF ANY
   OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. GOVERNMENT END USERS.

   The Covered Code is a ''commercial item,'' as that term is defined in 48
   C.F.R. 2.101 (Oct. 1995), consisting of ''commercial computer software'' and
   ''commercial computer software documentation,'' as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Covered Code with only those rights set forth herein.

   (( 11. ))??
   MISCELLANEOUS.

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by California law provisions
   (except to the extent applicable law, if any, provides otherwise), excluding
   its conflict-of-law provisions. With respect to disputes in which at least
   one party is a citizen of, or an entity chartered or registered to do
   business in the United States of America, any litigation relating to this
   License shall be subject to the jurisdiction of the Federal Courts of the
   Northern District of California, with venue lying in Santa Clara County,
   California, with the losing party responsible for costs, including without
   limitation, court costs and reasonable attorneys' fees and expenses. The
   application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any law or regulation
   which provides that the language of a contract shall be construed against the
   drafter shall not apply to this License.

   (( 12. ))??
   RESPONSIBILITY FOR CLAIMS.

   As between Initial Developer and the Contributors, each party is responsible
   for claims and damages arising, directly or indirectly, out of its
   utilization of rights under this License and You agree to work with Initial
   Developer and Contributors to distribute such responsibility on an equitable
   basis. Nothing herein is intended or shall be deemed to constitute any
   admission of liability.

   (( 13. ))??
   MULTIPLE-LICENSED CODE.

   Initial Developer may designate portions of the Covered Code as
   "Multiple-Licensed". "Multiple-Licensed" means that the Initial Developer
   permits you to utilize portions of the Covered Code under Your choice of the
   NPL or the alternative licenses, if any, specified by the Initial Developer
   in the file described in Exhibit A.

   EXHIBIT A - InterBase Public License.

   The contents of this file are subject to the Interbase Public License Version
   1.0 (the "License"); you may not use this file except in compliance with the
   License. You may obtain a copy of the License at
   http:/www.Interbase.com/IPL.html

   Software distributed under the License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   The Original Code was created by InterBase Software Corp and its successors.

   Portions created by Borland/Inprise are Copyright (C) Borland/Inprise. All
   Rights Reserved.

   Contributor(s):
   //** ______________________________________ **//
   __5__

AMENDMENTS

   (( I. ))??
   InterBase and logo. This License does not grant any rights to use the
   trademarks "Interbase'', "Java" or "JavaScript" even if such marks are
   included in the Original Code or Modifications.

   (( II. ))??
   Trademark Usage.

      (( II.1. ))??
      Advertising Materials. All advertising materials mentioning features or
      use of the covered Code must display the following acknowledgement: "This
      product includes software developed by Borland Software Corp. "

      (( II.2. ))??
      Endorsements. The names "InterBase," "ISC," and "IB" must not be used to
      endorse or promote Contributor Versions or Larger Works without the prior
      written permission of Interbase.

      (( II.3. ))??
      Product Names. Contributor Versions and Larger Works may not be called
      "InterBase" or "Interbase" nor may the word "InterBase" appear in their
      names without the prior written permission of Interbase.
`
const license_JPNIC_lre = `//**
Japan Network Information Center License
https://spdx.org/licenses/JPNIC.json
https://gitlab.isc.org/isc-projects/bind9/blob/master/COPYRIGHT#L366
**//

(( Japan Network Information Center License
(( Copyright __20__ ))??
))??

By using this file, you agree to the terms and conditions set forth bellow.

LICENSE TERMS AND CONDITIONS

The following License Terms and Conditions apply, unless a different license is
obtained from
Japan Network Information Center
("JPNIC"),
a Japanese association, Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda, Chiyoda-ku, Tokyo 101-0047, Japan.

   (( 1. ))??
   Use, Modification and Redistribution (including distribution of any modified
   or derived work) in source and/or binary forms is permitted under this
   License Terms and Conditions.

   (( 2. ))??
   Redistribution of source code must retain the copyright notices as they
   appear in each source code file, this License Terms and Conditions.

   (( 3. ))??
   Redistribution in binary form must reproduce the Copyright Notice, this
   License Terms and Conditions, in the documentation and/or other materials
   provided with the distribution. For the purposes of binary distribution the
   "Copyright Notice" refers to the following language:

   "Copyright (c) __20__. All rights reserved."

   (( 4. ))??
   The name of JPNIC
   may not be used to endorse or promote products derived from this Software
   without specific prior written approval of
   JPNIC.

   (( 5. ))??
   Disclaimer/Limitation of Liability: THIS SOFTWARE IS PROVIDED BY JPNIC
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JPNIC
   BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGES.
`
const license_JSON_lre = `//**
JSON License
https://spdx.org/licenses/JSON.json
http://www.json.org/license.html
**//

//**Copyright**//

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

The Software
((shall || should rather))
be used for Good, not Evil.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
`
const license_JasPer_2_0_lre = `//**
JasPer License
https://spdx.org/licenses/JasPer-2.0.json
http://www.ece.uvic.ca/~mdadams/jasper/LICENSE
**//

(( JasPer License Version 2.0
(( Copyright __30__ ))??
))??

Permission is hereby granted, free of charge, to any person (the "User")
obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

   (( 1. ))??
   The above copyright notices and this permission notice (which includes the
   disclaimer below) shall be included in all copies or substantial portions of
   the Software.

   (( 2. ))??
   The name of a copyright holder shall not be used to endorse or promote
   products derived from the Software without specific prior written
   permission.

THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE. NO
USE OF THE SOFTWARE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER. THE
SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS "AS IS", WITHOUT WARRANTY OF ANY
KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD
PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
USE OR PERFORMANCE OF THIS SOFTWARE. NO ASSURANCES ARE PROVIDED BY THE COPYRIGHT
HOLDERS THAT THE SOFTWARE DOES NOT INFRINGE THE PATENT OR OTHER INTELLECTUAL
PROPERTY RIGHTS OF ANY OTHER ENTITY. EACH COPYRIGHT HOLDER DISCLAIMS ANY
LIABILITY TO THE USER FOR CLAIMS BROUGHT BY ANY OTHER ENTITY BASED ON
INFRINGEMENT OF INTELLECTUAL PROPERTY RIGHTS OR OTHERWISE. AS A CONDITION TO
EXERCISING THE RIGHTS GRANTED HEREUNDER, EACH USER HEREBY ASSUMES SOLE
RESPONSIBILITY TO SECURE ANY OTHER INTELLECTUAL PROPERTY RIGHTS NEEDED, IF ANY.
THE SOFTWARE IS NOT FAULT-TOLERANT AND IS NOT INTENDED FOR USE IN
MISSION-CRITICAL SYSTEMS, SUCH AS THOSE USED IN THE OPERATION OF NUCLEAR
FACILITIES, AIRCRAFT NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL
SYSTEMS, DIRECT LIFE SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE
OF THE SOFTWARE OR SYSTEM COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR
SEVERE PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES"). THE COPYRIGHT
HOLDERS SPECIFICALLY DISCLAIM ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
HIGH RISK ACTIVITIES.
`
const license_LAL_1_2_lre = `//**
Licence Art Libre 1.2
https://spdx.org/licenses/LAL-1.2.json
http://artlibre.org/licence/lal/licence-art-libre-12/
**//

(( Licence Art Libre

[ Copyleft Attitude ]

Version 1.2 ))??

Préambule :

Avec cette Licence Art Libre, l'autorisation est donnée de copier, de diffuser
et de transformer librement les oeuvres dans le respect des droits de l'auteur.

Loin d'ignorer les droits de l'auteur, cette licence les reconnaît et les
protège. Elle en reformule le principe en permettant au public de faire un
usage créatif des oeuvres d'art.

Alors que l'usage fait du droit de la propriété littéraire et artistique
conduit à restreindre l'accès du public à l'oeuvre, la Licence Art Libre a
pour but de le favoriser.

L'intention est d'ouvrir l'accès et d'autoriser l'utilisation des ressources
d'une oeuvre par le plus grand nombre. En avoir jouissance pour en multiplier
les réjouissances, créer de nouvelles conditions de création pour amplifier
les possibilités de création. Dans le respect des auteurs avec la
reconnaissance et la défense de leur droit moral.

En effet, avec la venue du numérique, l'invention de l'internet et des
logiciels libres, un nouveau mode de création et de production est apparu. Il
est aussi l'amplification de ce qui a été expérimenté par nombre d'artistes
contemporains.

Le savoir et la création sont des ressources qui doivent demeurer libres pour
être encore véritablement du savoir et de la création. C'est à dire rester
une recherche fondamentale qui ne soit pas directement liée à une application
concrète. Créer c'est découvrir l'inconnu, c'est inventer le réel avant tout
souci de réalisme.

Ainsi, l'objet de l'art n'est pas confondu avec l'objet d'art fini et défini
comme tel.

C'est la raison essentielle de cette Licence Art Libre : promouvoir et protéger
des pratiques artistiques libérées des seules règles de l'économie de
marché.

DÉFINITIONS

– L'oeuvre :

il s'agit d'une oeuvre commune qui comprend l'oeuvre originelle ainsi que toutes
les contributions postérieures (les originaux conséquents et les copies). Elle
est créée à l'initiative de l'auteur originel qui par cette licence définit
les conditions selon lesquelles les contributions sont faites.

– L'oeuvre originelle :

c'est-à-dire l'oeuvre créée par l'initiateur de l'oeuvre commune dont les
copies vont être modifiées par qui le souhaite.

– Les oeuvres conséquentes :

c'est-à-dire les propositions des auteurs qui contribuent à la formation de
l'oeuvre en faisant usage des droits de reproduction, de diffusion et de
modification que leur confère la licence.

– Original (source ou ressource de l'oeuvre) :

exemplaire daté de l'oeuvre, de sa définition, de sa partition ou de son
programme que l'auteur présente comme référence pour toutes actualisations,
interprétations, copies ou reproductions ultérieures.

– Copie :

toute reproduction d'un original au sens de cette licence.

– Auteur de l'oeuvre originelle :

c'est la personne qui a créé l'oeuvre à l'origine d'une arborescence de cette
oeuvre modifiée. Par cette licence, l'auteur détermine les conditions dans
lesquelles ce travail se fait.

– Contributeur :

toute personne qui contribue à la création de l'oeuvre. Il est l'auteur d'une
oeuvre originale résultant de la modification d'une copie de l'oeuvre
originelle ou de la modification d'une copie d'une oeuvre conséquente.

   (( 1. ))??
   OBJET

   Cette licence a pour objet de définir les conditions selon lesquelles vous
   pouvez jouir librement de cette oeuvre.

   (( 2. ))??
   L'ÉTENDUE DE LA JOUISSANCE

   Cette oeuvre est soumise au droit d'auteur, et l'auteur par cette licence
   vous indique quelles sont vos libertés pour la copier, la diffuser et la
   modifier:

   (( 2.1 ))??
   LA LIBERTÉ DE COPIER (OU DE REPRODUCTION)

   Vous avez la liberté de copier cette oeuvre pour un usage personnel, pour
   vos amis, ou toute autre personne et quelque soit la technique employée.

   (( 2.2 ))??
   LA LIBERTÉ DE DIFFUSER, D'INTERPRÉTER (OU DE REPRÉSENTATION)

   Vous pouvez diffuser librement les copies de ces oeuvres, modifiées ou non,
   quel que soit le support, quel que soit le lieu, à titre onéreux ou gratuit
   si vous respectez toutes les conditions suivantes:

   – joindre aux copies, cette licence à l'identique, ou indiquer
   précisément où se trouve la licence,

   – indiquer au destinataire le nom de l'auteur des originaux,

   – indiquer au destinataire où il pourra avoir accès aux originaux

   (originels et/ou conséquents). L'auteur de l'original pourra, s'il le
   souhaite, vous autoriser à diffuser l'original dans les mêmes conditions
   que les copies.

   (( 2.3 ))??
   LA LIBERTÉ DE MODIFIER

   Vous avez la liberté de modifier les copies des originaux (originels et
   conséquents), qui peuvent être partielles ou non, dans le respect des
   conditions prévues à l'article 2.2 en cas de diffusion (ou représentation)
   de la copie modifiée. L'auteur de l'original pourra, s'il le souhaite, vous
   autoriser à modifier l'original dans les mêmes conditions que les copies.

   (( 3. ))??
   L'INCORPORATION DE L'OEUVRE

   Tous les éléments de cette oeuvre doivent demeurer libres, c'est pourquoi
   il ne vous est pas permis d'intégrer les originaux (originels et
   conséquents) dans une autre oeuvre qui ne serait pas soumise à cette
   licence.

   (( 4. ))??
   VOS DROITS D'AUTEUR

   Cette licence n'a pas pour objet de nier vos droits d'auteur sur votre
   contribution. En choisissant de contribuer à l'évolution de cette oeuvre,
   vous acceptez seulement d'offrir aux autres les mêmes droits sur votre
   contribution que ceux qui vous ont été accordés par cette licence.

   (( 5. ))??
   LA DURÉE DE LA LICENCE

   Cette licence prend effet dès votre acceptation de ses dispositions. Le fait
   de copier, de diffuser, ou de modifier l'oeuvre constitue une acception
   tacite.

   Cette licence a pour durée la durée des droits d'auteur attachés à
   l'oeuvre.

   Si vous ne respectez pas les termes de cette licence, vous perdez
   automatiquement les droits qu'elle vous confère.

   Si le régime juridique auquel vous êtes soumis ne vous permet pas de
   respecter les termes de cette licence, vous ne pouvez pas vous prévaloir des
   libertés qu'elle confère.

   (( 6. ))??
   LES DIFFÉRENTES VERSIONS DE LA LICENCE

   Cette licence pourra être modifiée régulièrement, en vue de son
   amélioration, par ses auteurs (les acteurs du mouvement « copyleft
   attitude ») sous la forme de nouvelles versions numérotées.

   Vous avez toujours le choix entre vous contenter des dispositions contenues
   dans la version sous laquelle la copie vous a été communiquée ou alors,
   vous prévaloir des dispositions d'une des versions ultérieures.

   (( 7. ))??
   LES SOUS-LICENCES

   Les sous licences ne sont pas autorisées par la présente. Toute personne
   qui souhaite bénéficier des libertés qu'elle confère sera liée
   directement à l'auteur de l'oeuvre originelle.

   (( 8. ))??
   LA LOI APPLICABLE AU CONTRAT

Cette licence est soumise au droit français.
`
const license_LAL_1_3_lre = `//**
Licence Art Libre 1.3
https://spdx.org/licenses/LAL-1.3.json
https://artlibre.org/
**//

(( Licence Art Libre 1.3 (LAL 1.3) ))??

Préambule :

Avec la Licence Art Libre, l'autorisation est donnée de copier, de diffuser et
de transformer librement les œuvres dans le respect des droits de l'auteur.

Loin d'ignorer ces droits, la Licence Art Libre les reconnaît et les protège.
Elle en reformule l'exercice en permettant à tout un chacun de faire un usage
créatif des productions de l'esprit quels que soient leur genre et leur forme
d'expression.

Si, en règle générale, l'application du droit d'auteur conduit à restreindre
l'accès aux œuvres de l'esprit, la Licence Art Libre, au contraire, le
favorise. L'intention est d'autoriser l'utilisation des ressources d'une
œuvre ; créer de nouvelles conditions de création pour amplifier les
possibilités de création. La Licence Art Libre permet d'avoir jouissance des
œuvres tout en reconnaissant les droits et les responsabilités de chacun.

Avec le développement du numérique, l'invention d'internet et des logiciels
libres, les modalités de création ont évolué : les productions de l'esprit
s'offrent naturellement à la circulation, à l'échange et aux transformations.
Elles se prêtent favorablement à la réalisation d'œuvres communes que chacun
peut augmenter pour l'avantage de tous.

C'est la raison essentielle de la Licence Art Libre : promouvoir et protéger
ces productions de l'esprit selon les principes du copyleft : liberté d'usage,
de copie, de diffusion, de transformation et interdiction d'appropriation
exclusive.

Définitions :

Nous désignons par « œuvre », autant l'œuvre initiale, les œuvres
conséquentes, que l'œuvre commune telles que définies ci-après :

L'œuvre commune : Il s'agit d'une œuvre qui comprend l'œuvre initiale ainsi
que toutes les contributions postérieures (les originaux conséquents et les
copies). Elle est créée à l'initiative de l'auteur initial qui par cette
licence définit les conditions selon lesquelles les contributions sont faites.

L'œuvre initiale : C'est-à-dire l'œuvre créée par l'initiateur de l'œuvre
commune dont les copies vont être modifiées par qui le souhaite.

Les œuvres conséquentes : C'est-à-dire les contributions des auteurs qui
participent à la formation de l'œuvre commune en faisant usage des droits de
reproduction, de diffusion et de modification que leur confère la licence.

Originaux (sources ou ressources de l'œuvre) : Chaque exemplaire daté de
l'œuvre initiale ou conséquente que leurs auteurs présentent comme
référence pour toutes actualisations, interprétations, copies ou
reproductions ultérieures.

Copie : Toute reproduction d'un original au sens de cette licence.

   (( 1- ))??
   OBJET.

   Cette licence a pour objet de définir les conditions selon lesquelles vous
   pouvez jouir librement de l'œuvre.

   (( 2. ))??
   L'ÉTENDUE DE LA JOUISSANCE.

   Cette œuvre est soumise au droit d'auteur, et l'auteur par cette licence
   vous indique quelles sont vos libertés pour la copier, la diffuser et la
   modifier.

      (( 2.1 ))??
      LA LIBERTÉ DE COPIER (OU DE REPRODUCTION).

      Vous avez la liberté de copier cette œuvre pour vous, vos amis ou toute
      autre personne, quelle que soit la technique employée.

      (( 2.2 ))??
      LA LIBERTÉ DE DIFFUSER (INTERPRÉTER, REPRÉSENTER, DISTRIBUER).

      Vous pouvez diffuser librement les copies de ces œuvres, modifiées ou
      non, quel que soit le support, quel que soit le lieu, à titre onéreux ou
      gratuit, si vous respectez toutes les conditions suivantes :

         (( 1. ))??
         joindre aux copies cette licence à l'identique ou indiquer
         précisément où se trouve la licence ;

         (( 2. ))??
         indiquer au destinataire le nom de chaque auteur des originaux, y
         compris le vôtre si vous avez modifié l'œuvre ;

         (( 3. ))??
         indiquer au destinataire où il pourrait avoir accès aux originaux
         (initiaux et/ou conséquents). Les auteurs des originaux pourront,
         s'ils le souhaitent, vous autoriser à diffuser l'original dans les
         mêmes conditions que les copies.

      (( 2.3 ))??
      LA LIBERTÉ DE MODIFIER.

      Vous avez la liberté de modifier les copies des originaux (initiaux et
      conséquents) dans le respect des conditions suivantes :

         (( 1. ))??
         celles prévues à l'article 2.2 en cas de diffusion de la copie
         modifiée ;

         (( 2. ))??
         indiquer qu'il s'agit d'une œuvre modifiée et, si possible, la nature
         de la modification ;

         (( 3. ))??
         diffuser cette œuvre conséquente avec la même licence ou avec toute
         licence compatible ;

         (( 4. ))??
         Les auteurs des originaux pourront, s'ils le souhaitent, vous autoriser
         à modifier l'original dans les mêmes conditions que les copies.

   (( 3. ))??
   DROITS CONNEXES.

   Les actes donnant lieu à des droits d'auteur ou des droits voisins ne
   doivent pas constituer un obstacle aux libertés conférées par cette
   licence.

   C'est pourquoi, par exemple, les interprétations doivent être soumises à
   la même licence ou une licence compatible. De même, l'intégration de
   l'œuvre à une base de données, une compilation ou une anthologie ne doit
   pas faire obstacle à la jouissance de l'œuvre telle que définie par cette
   licence.

   (( 4. ))??
   L' INTÉGRATION DE L'ŒUVRE.

   Toute intégration de cette œuvre à un ensemble non soumis à la LAL doit
   assurer l'exercice des libertés conférées par cette licence.

   Si l'œuvre n'est plus accessible indépendamment de l'ensemble, alors
   l'intégration n'est possible qu'à condition que l'ensemble soit soumis à
   la LAL ou une licence compatible.

   (( 5. ))??
   CRITÈRES DE COMPATIBILITÉ.

   Une licence est compatible avec la LAL si et seulement si :

      (( 1. ))??
      elle accorde l'autorisation de copier, diffuser et modifier des copies de
      l'œuvre, y compris à des fins lucratives, et sans autres restrictions
      que celles qu'impose le respect des autres critères de compatibilité ;

      (( 2. ))??
      elle garantit la paternité de l'œuvre et l'accès aux versions
      antérieures de l'œuvre quand cet accès est possible ;

      (( 3. ))??
      elle reconnaît la LAL également compatible (réciprocité) ;

      (( 4. ))??
      elle impose que les modifications faites sur l'œuvre soient soumises à
      la même licence ou encore à une licence répondant aux critères de
      compatibilité posés par la LAL.

   (( 6. ))??
   VOS DROITS INTELLECTUELS.

   La LAL n'a pas pour objet de nier vos droits d'auteur sur votre contribution
   ni vos droits connexes. En choisissant de contribuer à l'évolution de cette
   œuvre commune, vous acceptez seulement d'offrir aux autres les mêmes
   autorisations sur votre contribution que celles qui vous ont été accordées
   par cette licence. Ces autorisations n'entraînent pas un dessaisissement de
   vos droits intellectuels.

   (( 7. ))??
   VOS RESPONSABILITÉS.

   La liberté de jouir de l'œuvre tel que permis par la LAL (liberté de
   copier, diffuser, modifier) implique pour chacun la responsabilité de ses
   propres faits.

   (( 8. ))??
   LA DURÉE DE LA LICENCE.

   Cette licence prend effet dès votre acceptation de ses dispositions. Le fait
   de copier, de diffuser, ou de modifier l'œuvre constitue une acceptation
   tacite.

   Cette licence a pour durée la durée des droits d'auteur attachés à
   l'œuvre. Si vous ne respectez pas les termes de cette licence, vous perdez
   automatiquement les droits qu'elle vous confère.

   Si le régime juridique auquel vous êtes soumis ne vous permet pas de
   respecter les termes de cette licence, vous ne pouvez pas vous prévaloir des
   libertés qu'elle confère.

   (( 9. ))??
   LES DIFFÉRENTES VERSIONS DE LA LICENCE.

   Cette licence pourra être modifiée régulièrement, en vue de son
   amélioration, par ses auteurs (les acteurs du mouvement Copyleft Attitude)
   sous la forme de nouvelles versions numérotées.

   Vous avez toujours le choix entre vous contenter des dispositions contenues
   dans la version de la LAL sous laquelle la copie vous a été communiquée ou
   alors, vous prévaloir des dispositions d'une des versions ultérieures.

   (( 10. ))??
   LES SOUS-LICENCES.

   Les sous-licences ne sont pas autorisées par la présente. Toute personne
   qui souhaite bénéficier des libertés qu'elle confère sera liée
   directement aux auteurs de l'œuvre commune.

   (( 11. ))??
   LE CONTEXTE JURIDIQUE.

   Cette licence est rédigée en référence au droit français et à la
   Convention de Berne relative au droit d'auteur.
`
const license_LGPL_2_0_lre = `//**
GNU Library General Public License v2
https://www.gnu.org/licenses/old-licenses/lgpl-2.0-standalone.html
**//

((
	GNU LIBRARY GENERAL PUBLIC LICENSE Version 2, June 1991
	
	((
		((
			Copyright __20__
			((<https://fsf.org/>))??
			
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))??
		))??

		Everyone is permitted to copy and distribute verbatim copies
		of this license document, but changing it is not allowed.

		((Copyright __20__))??
	))??

))??

(([This is the first released version of the library GPL. It is numbered 2 because
it goes with version 2 of the ordinary GPL.]))??

Preamble

The licenses for most software are designed to take away your freedom to share
and change it. By contrast, the GNU General Public Licenses are intended to
guarantee your freedom to share and change free software--to make sure the
software is free for all its users.

This license, the Library General Public License, applies to some specially
designated Free Software Foundation software, and to any other libraries whose
authors decide to use it. You can use it for your libraries, too.

When we speak of free software, we are referring to freedom, not price. Our
General Public Licenses are designed to make sure that you have the freedom to
distribute copies of free software (and charge for this service if you wish),
that you receive source code or can get it if you want it, that you can change
the software or use pieces of it in new free programs; and that you know you can
do these things.

To protect your rights, we need to make restrictions that forbid anyone to deny
you these rights or to ask you to surrender the rights. These restrictions
translate to certain responsibilities for you if you distribute copies of the
library, or if you modify it.

For example, if you distribute copies of the library, whether gratis or for a
fee, you must give the recipients all the rights that we gave you. You must make
sure that they, too, receive or can get the source code. If you link a program
with the library, you must provide complete object files to the recipients so
that they can relink them with the library, after making changes to the library
and recompiling it. And you must show them these terms so they know their
rights.

Our method of protecting your rights has two steps: (1) copyright the library,
and (2) offer you this license which gives you legal permission to copy,
distribute and/or modify the library.

Also, for each distributor's protection, we want to make certain that everyone
understands that there is no warranty for this free library. If the library is
modified by someone else and passed on, we want its recipients to know that what
they have is not the original version, so that any problems introduced by others
will not reflect on the original authors' reputations.

Finally, any free program is threatened constantly by software patents. We wish
to avoid the danger that companies distributing free software will individually
obtain patent licenses, thus in effect transforming the program into proprietary
software. To prevent this, we have made it clear that any patent must be
licensed for everyone's free use or not licensed at all.

Most GNU software, including some libraries, is covered by the ordinary GNU
General Public License, which was designed for utility programs. This license,
the GNU Library General Public License, applies to certain designated libraries.
This license is quite different from the ordinary one; be sure to read it in
full, and don't assume that anything in it is the same as in the ordinary
license.

The reason we have a separate public license for some libraries is that they
blur the distinction we usually make between modifying or adding to a program
and simply using it. Linking a program with a library, without changing the
library, is in some sense simply using the library, and is analogous to running
a utility program or application program. However, in a textual and legal sense,
the linked executable is a combined work, a derivative of the original library,
and the ordinary General Public License treats it as such.

Because of this blurred distinction, using the ordinary General Public License
for libraries did not effectively promote software sharing, because most
developers did not use the libraries. We concluded that weaker conditions might
promote sharing better.

However, unrestricted linking of non-free programs would deprive the users of
those programs of all benefit from the free status of the libraries themselves.
This Library General Public License is intended to permit developers of non-free
programs to use free libraries, while preserving your freedom as a user of such
programs to change the free libraries that are incorporated in them. (We have
not seen how to achieve this as regards changes in header files, but we have
achieved it as regards changes in the actual functions of the Library.) The hope
is that this will lead to faster development of free libraries.

The precise terms and conditions for copying, distribution and modification
follow. Pay close attention to the difference between a "work based on the
library" and a "work that uses the library". The former contains code derived
from the library, while the latter only works together with the library.

Note that it is possible for a library to be covered by the ordinary General
Public License rather than by this special one.

((GNU LIBRARY GENERAL PUBLIC LICENSE))??
TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

   __1__ This License Agreement applies to any software library which contains a
   notice placed by the copyright holder or other authorized party saying it may
   be distributed under the terms of this Library General Public License (also
   called "this License"). Each licensee is addressed as "you".

   A "library" means a collection of software functions and/or data prepared so
   as to be conveniently linked with application programs (which use some of
   those functions and data) to form executables.

   The "Library", below, refers to any such software library or work which has
   been distributed under these terms. A "work based on the Library" means
   either the Library or any derivative work under copyright law: that is to
   say, a work containing the Library or a portion of it, either verbatim or
   with modifications and/or translated straightforwardly into another language.
   (Hereinafter, translation is included without limitation in the term
   "modification".)

   "Source code" for a work means the preferred form of the work for making
   modifications to it. For a library, complete source code means all the source
   code for all modules it contains, plus any associated interface definition
   files, plus the scripts used to control compilation and installation of the
   library.

   Activities other than copying, distribution and modification are not covered
   by this License; they are outside its scope. The act of running a program
   using the Library is not restricted, and output from such a program is
   covered only if its contents constitute a work based on the Library
   (independent of the use of the Library in a tool for writing it). Whether
   that is true depends on what the Library does and what the program that uses
   the Library does.

   __1__ You may copy and distribute verbatim copies of the Library's complete
   source code as you receive it, in any medium, provided that you conspicuously
   and appropriately publish on each copy an appropriate copyright notice and
   disclaimer of warranty; keep intact all the notices that refer to this
   License and to the absence of any warranty; and distribute a copy of this
   License along with the Library.

   You may charge a fee for the physical act of transferring a copy, and you may
   at your option offer warranty protection in exchange for a fee.

   __1__ You may modify your copy or copies of the Library or any portion of it,
   thus forming a work based on the Library, and copy and distribute such
   modifications or work under the terms of Section 1 above, provided that you
   also meet all of these conditions:

      __1__ The modified work must itself be a software library.

      __1__ You must cause the files modified to carry prominent notices stating
      that you changed the files and the date of any change.

      __1__ You must cause the whole of the work to be licensed at no charge to
      all third parties under the terms of this License.

      __1__ If a facility in the modified Library refers to a function or a
      table of data to be supplied by an application program that uses the
      facility, other than as an argument passed when the facility is invoked,
      then you must make a good faith effort to ensure that, in the event an
      application does not supply such function or table, the facility still
      operates, and performs whatever part of its purpose remains meaningful.

      (For example, a function in a library to compute square roots has a
      purpose that is entirely well-defined independent of the application.
      Therefore, Subsection 2d requires that any application-supplied function
      or table used by this function must be optional: if the application does
      not supply it, the square root function must still compute square roots.)

   These requirements apply to the modified work as a whole. If identifiable
   sections of that work are not derived from the Library, and can be reasonably
   considered independent and separate works in themselves, then this License,
   and its terms, do not apply to those sections when you distribute them as
   separate works. But when you distribute the same sections as part of a whole
   which is a work based on the Library, the distribution of the whole must be
   on the terms of this License, whose permissions for other licensees extend to
   the entire whole, and thus to each and every part regardless of who wrote
   it.

   Thus, it is not the intent of this section to claim rights or contest your
   rights to work written entirely by you; rather, the intent is to exercise the
   right to control the distribution of derivative or collective works based on
   the Library.

   In addition, mere aggregation of another work not based on the Library with
   the Library (or with a work based on the Library) on a volume of a storage or
   distribution medium does not bring the other work under the scope of this
   License.

   __1__ You may opt to apply the terms of the ordinary GNU General Public
   License instead of this License to a given copy of the Library. To do this,
   you must alter all the notices that refer to this License, so that they refer
   to the ordinary GNU General Public License, version 2, instead of to this
   License. (If a newer version than version 2 of the ordinary GNU General
   Public License has appeared, then you can specify that version instead if you
   wish.) Do not make any other change in these notices.

   Once this change is made in a given copy, it is irreversible for that copy,
   so the ordinary GNU General Public License applies to all subsequent copies
   and derivative works made from that copy.

   This option is useful when you wish to copy part of the code of the Library
   into a program that is not a library.

   __1__ You may copy and distribute the Library (or a portion or derivative of
   it, under Section 2) in object code or executable form under the terms of
   Sections 1 and 2 above provided that you accompany it with the complete
   corresponding machine-readable source code, which must be distributed under
   the terms of Sections 1 and 2 above on a medium customarily used for software
   interchange.

   If distribution of object code is made by offering access to copy from a
   designated place, then offering equivalent access to copy the source code
   from the same place satisfies the requirement to distribute the source code,
   even though third parties are not compelled to copy the source along with the
   object code.

   __1__ A program that contains no derivative of any portion of the Library,
   but is designed to work with the Library by being compiled or linked with it,
   is called a "work that uses the Library". Such a work, in isolation, is not a
   derivative work of the Library, and therefore falls outside the scope of this
   License.

   However, linking a "work that uses the Library" with the Library creates an
   executable that is a derivative of the Library (because it contains portions
   of the Library), rather than a "work that uses the library". The executable
   is therefore covered by this License. Section 6 states terms for distribution
   of such executables.

   When a "work that uses the Library" uses material from a header file that is
   part of the Library, the object code for the work may be a derivative work of
   the Library even though the source code is not. Whether this is true is
   especially significant if the work can be linked without the Library, or if
   the work is itself a library. The threshold for this to be true is not
   precisely defined by law.

   If such an object file uses only numerical parameters, data structure layouts
   and accessors, and small macros and small inline functions (ten lines or less
   in length), then the use of the object file is unrestricted, regardless of
   whether it is legally a derivative work. (Executables containing this object
   code plus portions of the Library will still fall under Section 6.)

   Otherwise, if the work is a derivative of the Library, you may distribute the
   object code for the work under the terms of Section 6. Any executables
   containing that work also fall under Section 6, whether or not they are
   linked directly with the Library itself.

   __1__ As an exception to the Sections above, you may also compile or link a
   "work that uses the Library" with the Library to produce a work containing
   portions of the Library, and distribute that work under terms of your choice,
   provided that the terms permit modification of the work for the customer's
   own use and reverse engineering for debugging such modifications.

   You must give prominent notice with each copy of the work that the Library is
   used in it and that the Library and its use are covered by this License. You
   must supply a copy of this License. If the work during execution displays
   copyright notices, you must include the copyright notice for the Library
   among them, as well as a reference directing the user to the copy of this
   License. Also, you must do one of these things:

      __1__ Accompany the work with the complete corresponding machine-readable
      source code for the Library including whatever changes were used in the
      work (which must be distributed under Sections 1 and 2 above); and, if the
      work is an executable linked with the Library, with the complete
      machine-readable "work that uses the Library", as object code and/or
      source code, so that the user can modify the Library and then relink to
      produce a modified executable containing the modified Library. (It is
      understood that the user who changes the contents of definitions files in
      the Library will not necessarily be able to recompile the application to
      use the modified definitions.)

      __1__ Accompany the work with a written offer, valid for at least three
      years, to give the same user the materials specified in Subsection 6a,
      above, for a charge no more than the cost of performing this
      distribution.

      __1__ If distribution of the work is made by offering access to copy from
      a designated place, offer equivalent access to copy the above specified
      materials from the same place.

      __1__ Verify that the user has already received a copy of these materials
      or that you have already sent this user a copy.

   For an executable, the required form of the "work that uses the Library" must
   include any data and utility programs needed for reproducing the executable
   from it. However, as a special exception, the source code distributed need
   not include anything that is normally distributed (in either source or binary
   form) with the major components (compiler, kernel, and so on) of the
   operating system on which the executable runs, unless that component itself
   accompanies the executable.

   It may happen that this requirement contradicts the license restrictions of
   other proprietary libraries that do not normally accompany the operating
   system. Such a contradiction means you cannot use both them and the Library
   together in an executable that you distribute.

   __1__ You may place library facilities that are a work based on the Library
   side-by-side in a single library together with other library facilities not
   covered by this License, and distribute such a combined library, provided
   that the separate distribution of the work based on the Library and of the
   other library facilities is otherwise permitted, and provided that you do
   these two things:

      __1__ Accompany the combined library with a copy of the same work based on
      the Library, uncombined with any other library facilities. This must be
      distributed under the terms of the Sections above.

      __1__ Give prominent notice with the combined library of the fact that
      part of it is a work based on the Library, and explaining where to find
      the accompanying uncombined form of the same work.

   __1__ You may not copy, modify, sublicense, link with, or distribute the
   Library except as expressly provided under this License. Any attempt
   otherwise to copy, modify, sublicense, link with, or distribute the Library
   is void, and will automatically terminate your rights under this License.
   However, parties who have received copies, or rights, from you under this
   License will not have their licenses terminated so long as such parties
   remain in full compliance.

   __1__ You are not required to accept this License, since you have not signed
   it. However, nothing else grants you permission to modify or distribute the
   Library or its derivative works. These actions are prohibited by law if you
   do not accept this License. Therefore, by modifying or distributing the
   Library (or any work based on the Library), you indicate your acceptance of
   this License to do so, and all its terms and conditions for copying,
   distributing or modifying the Library or works based on it.

   __1__ Each time you redistribute the Library (or any work based on the
   Library), the recipient automatically receives a license from the original
   licensor to copy, distribute, link with or modify the Library subject to
   these terms and conditions. You may not impose any further restrictions on
   the recipients' exercise of the rights granted herein. You are not
   responsible for enforcing compliance by third parties to this License.

   __1__ If, as a consequence of a court judgment or allegation of patent
   infringement or for any other reason (not limited to patent issues),
   conditions are imposed on you (whether by court order, agreement or
   otherwise) that contradict the conditions of this License, they do not excuse
   you from the conditions of this License. If you cannot distribute so as to
   satisfy simultaneously your obligations under this License and any other
   pertinent obligations, then as a consequence you may not distribute the
   Library at all. For example, if a patent license would not permit
   royalty-free redistribution of the Library by all those who receive copies
   directly or indirectly through you, then the only way you could satisfy both
   it and this License would be to refrain entirely from distribution of the
   Library.

   If any portion of this section is held invalid or unenforceable under any
   particular circumstance, the balance of the section is intended to apply, and
   the section as a whole is intended to apply in other circumstances.

   It is not the purpose of this section to induce you to infringe any patents
   or other property right claims or to contest validity of any such claims;
   this section has the sole purpose of protecting the integrity of the free
   software distribution system which is implemented by public license
   practices. Many people have made generous contributions to the wide range of
   software distributed through that system in reliance on consistent
   application of that system; it is up to the author/donor to decide if he or
   she is willing to distribute software through any other system and a licensee
   cannot impose that choice.

   This section is intended to make thoroughly clear what is believed to be a
   consequence of the rest of this License.

   __1__ If the distribution and/or use of the Library is restricted in certain
   countries either by patents or by copyrighted interfaces, the original
   copyright holder who places the Library under this License may add an
   explicit geographical distribution limitation excluding those countries, so
   that distribution is permitted only in or among countries not thus excluded.
   In such case, this License incorporates the limitation as if written in the
   body of this License.

   __1__ The Free Software Foundation may publish revised and/or new versions of
   the Library General Public License from time to time. Such new versions will
   be similar in spirit to the present version, but may differ in detail to
   address new problems or concerns.

   Each version is given a distinguishing version number. If the Library
   specifies a version number of this License which applies to it and "any later
   version", you have the option of following the terms and conditions either of
   that version or of any later version published by the Free Software
   Foundation. If the Library does not specify a license version number, you may
   choose any version ever published by the Free Software Foundation.

   __1__ If you wish to incorporate parts of the Library into other free
   programs whose distribution conditions are incompatible with these, write to
   the author to ask for permission. For software which is copyrighted by the
   Free Software Foundation, write to the Free Software Foundation; we sometimes
   make exceptions for this. Our decision will be guided by the two goals of
   preserving the free status of all derivatives of our free software and of
   promoting the sharing and reuse of software generally.

   NO WARRANTY

   __1__ BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
   FOR THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
   OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
   PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR
   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO
   THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE LIBRARY
   PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR
   CORRECTION.

   __1__ IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
   WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
   REDISTRIBUTE THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
   INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
   OUT OF THE USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO
   LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR
   THIRD PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER
   SOFTWARE), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGES.

((  END OF TERMS AND CONDITIONS))??

(( How to Apply These Terms to Your New Libraries

If you develop a new library, and you want it to be of the greatest possible use
to the public, we recommend making it free software that everyone can
redistribute and change. You can do so by permitting redistribution under these
terms (or, alternatively, under the terms of the ordinary General Public
License).

To apply these terms, attach the following notices to the library. It is safest
to attach them to the start of each source file to most effectively convey the
exclusion of warranty; and each file should have at least the "copyright" line
and a pointer to where the full notice is found.

__30__
//**
<one line to give the program's name and a brief idea of what it does.>
Copyright (C) <year> <name of author>
**//

This library is free software; you can redistribute it and/or modify it under
the terms of the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your option) any
later version.

This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Library General Public License for more
details.

You should have received a copy of the GNU Library General Public License along
with this library; if not, write to the Free Software Foundation, Inc.,

	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))

Also add information on how to contact you by electronic and paper mail.

You should also get your employer (if you work as a programmer) or your school,
if any, to sign a "copyright disclaimer" for the library, if necessary. Here is
a sample; alter the names:

Yoyodyne, Inc., hereby disclaims all copyright interest in

the library ` + "`" + `Frob' (a library for tweaking knobs) written

by James Random Hacker.

__5__ //**<signature of Ty Coon>**//, 1 April 1990

Ty Coon, President of Vice

That's all there is to it! ))??
`
const license_LGPL_2_0_only_lre = `
//**
https://spdx.org/licenses/LGPL-2.0-only.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		library
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
	General Public License
	((LGPL))??
	((as published by the Free Software Foundation))??
	
		((under))??
		version 2
		((LGPLv2))??
		((of the License))??
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
		General Public License version
		2
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
		General Public License for more details.

		((
			You should have received a copy of the
			((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
			General Public License
			((
				((version))??
				2
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_LGPL_2_0_or_later_lre = `
//**
https://spdx.org/licenses/LGPL-2.0-or-later.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		library
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
	General Public License
	((LGPL))??
	((as published by the Free Software Foundation))??
	
		((
			either version 2
			((of the License))??
			or
			((at your option))??
			any later version.
		||
			version 2 or later
			((of the License))??
		))
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
		General Public License version
		2
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
		General Public License for more details.

		((
			You should have received a copy of the
			((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
			General Public License
			((
				((version))??
				2
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_LGPL_2_1_lre = `//**
GNU Lesser General Public License v2.1
https://www.gnu.org/licenses/old-licenses/lgpl-2.1-standalone.html
https://opensource.org/licenses/LGPL-2.1
**//

((
	GNU LESSER GENERAL PUBLIC LICENSE Version 2.1, February 1999
	
	((
		((
			Copyright __20__
			((<https://fsf.org/>))??
			
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))??
		))??

		Everyone is permitted to copy and distribute verbatim copies
		of this license document, but changing it is not allowed.

		((Copyright __20__))??
	))??

))??


((
[This is the first released version of the Lesser GPL. It also counts as the
successor of the GNU Library Public License, version 2, hence the version number
2.1.]
))??

Preamble

The licenses for most software are designed to take away your freedom to share
and change it. By contrast, the GNU General Public Licenses are intended to
guarantee your freedom to share and change free software--to make sure the
software is free for all its users.

This license, the Lesser General Public License, applies to some specially
designated software packages--typically libraries--of the Free Software
Foundation and other authors who decide to use it. You can use it too, but we
suggest you first think carefully about whether this license or the ordinary
General Public License is the better strategy to use in any particular case,
based on the explanations below.

When we speak of free software, we are referring to freedom of use, not price.
Our General Public Licenses are designed to make sure that you have the freedom
to distribute copies of free software (and charge for this service if you wish);
that you receive source code or can get it if you want it; that you can change
the software and use pieces of it in new free programs; and that you are
informed that you can do these things.

To protect your rights, we need to make restrictions that forbid distributors to
deny you these rights or to ask you to surrender these rights. These
restrictions translate to certain responsibilities for you if you distribute
copies of the library or if you modify it.

For example, if you distribute copies of the library, whether gratis or for a
fee, you must give the recipients all the rights that we gave you. You must make
sure that they, too, receive or can get the source code. If you link other code
with the library, you must provide complete object files to the recipients, so
that they can relink them with the library after making changes to the library
and recompiling it. And you must show them these terms so they know their
rights.

We protect your rights with a two-step method: (1) we copyright the library, and
(2) we offer you this license, which gives you legal permission to copy,
distribute and/or modify the library.

To protect each distributor, we want to make it very clear that there is no
warranty for the free library. Also, if the library is modified by someone else
and passed on, the recipients should know that what they have is not the
original version, so that the original author's reputation will not be affected
by problems that might be introduced by others.

Finally, software patents pose a constant threat to the existence of any free
program. We wish to make sure that a company cannot effectively restrict the
users of a free program by obtaining a restrictive license from a patent holder.
Therefore, we insist that any patent license obtained for a version of the
library must be consistent with the full freedom of use specified in this
license.

Most GNU software, including some libraries, is covered by the ordinary GNU
General Public License. This license, the GNU Lesser General Public License,
applies to certain designated libraries, and is quite different from the
ordinary General Public License. We use this license for certain libraries in
order to permit linking those libraries into non-free programs.

When a program is linked with a library, whether statically or using a shared
library, the combination of the two is legally speaking a combined work, a
derivative of the original library. The ordinary General Public License
therefore permits such linking only if the entire combination fits its criteria
of freedom. The Lesser General Public License permits more lax criteria for
linking other code with the library.

We call this license the "Lesser" General Public License because it does Less to
protect the user's freedom than the ordinary General Public License. It also
provides other free software developers Less of an advantage over competing
non-free programs. These disadvantages are the reason we use the ordinary
General Public License for many libraries. However, the Lesser license provides
advantages in certain special circumstances.

For example, on rare occasions, there may be a special need to encourage the
widest possible use of a certain library, so that it becomes a de-facto
standard. To achieve this, non-free programs must be allowed to use the library.
A more frequent case is that a free library does the same job as widely used
non-free libraries. In this case, there is little to gain by limiting the free
library to free software only, so we use the Lesser General Public License.

In other cases, permission to use a particular library in non-free programs
enables a greater number of people to use a large body of free software. For
example, permission to use the GNU C Library in non-free programs enables many
more people to use the whole GNU operating system, as well as its variant, the
GNU/Linux operating system.

Although the Lesser General Public License is Less protective of the users'
freedom, it does ensure that the user of a program that is linked with the
Library has the freedom and the wherewithal to run that program using a modified
version of the Library.

The precise terms and conditions for copying, distribution and modification
follow. Pay close attention to the difference between a "work based on the
library" and a "work that uses the library". The former contains code derived
from the library, whereas the latter must be combined with the library in order
to run.

((GNU LESSER GENERAL PUBLIC LICENSE))??

TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

   __1__ This License Agreement applies to any software library or other program
   which contains a notice placed by the copyright holder or other authorized
   party saying it may be distributed under the terms of this Lesser General
   Public License (also called "this License"). Each licensee is addressed as
   "you".

   A "library" means a collection of software functions and/or data prepared so
   as to be conveniently linked with application programs (which use some of
   those functions and data) to form executables.

   The "Library", below, refers to any such software library or work which has
   been distributed under these terms. A "work based on the Library" means
   either the Library or any derivative work under copyright law: that is to
   say, a work containing the Library or a portion of it, either verbatim or
   with modifications and/or translated straightforwardly into another language.
   (Hereinafter, translation is included without limitation in the term
   "modification".)

   "Source code" for a work means the preferred form of the work for making
   modifications to it. For a library, complete source code means all the source
   code for all modules it contains, plus any associated interface definition
   files, plus the scripts used to control compilation and installation of the
   library.

   Activities other than copying, distribution and modification are not covered
   by this License; they are outside its scope. The act of running a program
   using the Library is not restricted, and output from such a program is
   covered only if its contents constitute a work based on the Library
   (independent of the use of the Library in a tool for writing it). Whether
   that is true depends on what the Library does and what the program that uses
   the Library does.

   __1__ You may copy and distribute verbatim copies of the Library's complete
   source code as you receive it, in any medium, provided that you conspicuously
   and appropriately publish on each copy an appropriate copyright notice and
   disclaimer of warranty; keep intact all the notices that refer to this
   License and to the absence of any warranty; and distribute a copy of this
   License along with the Library.

   You may charge a fee for the physical act of transferring a copy, and you may
   at your option offer warranty protection in exchange for a fee.

   __1__ You may modify your copy or copies of the Library or any portion of it,
   thus forming a work based on the Library, and copy and distribute such
   modifications or work under the terms of Section 1 above, provided that you
   also meet all of these conditions:

      __1__ The modified work must itself be a software library.

      __1__ You must cause the files modified to carry prominent notices stating
      that you changed the files and the date of any change.

      __1__ You must cause the whole of the work to be licensed at no charge to
      all third parties under the terms of this License.

      __1__ If a facility in the modified Library refers to a function or a
      table of data to be supplied by an application program that uses the
      facility, other than as an argument passed when the facility is invoked,
      then you must make a good faith effort to ensure that, in the event an
      application does not supply such function or table, the facility still
      operates, and performs whatever part of its purpose remains meaningful.

   (For example, a function in a library to compute square roots has a purpose
   that is entirely well-defined independent of the application. Therefore,
   Subsection 2d requires that any application-supplied function or table used
   by this function must be optional: if the application does not supply it, the
   square root function must still compute square roots.)

   These requirements apply to the modified work as a whole. If identifiable
   sections of that work are not derived from the Library, and can be reasonably
   considered independent and separate works in themselves, then this License,
   and its terms, do not apply to those sections when you distribute them as
   separate works. But when you distribute the same sections as part of a whole
   which is a work based on the Library, the distribution of the whole must be
   on the terms of this License, whose permissions for other licensees extend to
   the entire whole, and thus to each and every part regardless of who wrote
   it.

   Thus, it is not the intent of this section to claim rights or contest your
   rights to work written entirely by you; rather, the intent is to exercise the
   right to control the distribution of derivative or collective works based on
   the Library.

   In addition, mere aggregation of another work not based on the Library with
   the Library (or with a work based on the Library) on a volume of a storage or
   distribution medium does not bring the other work under the scope of this
   License.

   __1__ You may opt to apply the terms of the ordinary GNU General Public
   License instead of this License to a given copy of the Library. To do this,
   you must alter all the notices that refer to this License, so that they refer
   to the ordinary GNU General Public License, version 2, instead of to this
   License. (If a newer version than version 2 of the ordinary GNU General
   Public License has appeared, then you can specify that version instead if you
   wish.) Do not make any other change in these notices.

   Once this change is made in a given copy, it is irreversible for that copy,
   so the ordinary GNU General Public License applies to all subsequent copies
   and derivative works made from that copy.

   This option is useful when you wish to copy part of the code of the Library
   into a program that is not a library.

   __1__ You may copy and distribute the Library (or a portion or derivative of
   it, under Section 2) in object code or executable form under the terms of
   Sections 1 and 2 above provided that you accompany it with the complete
   corresponding machine-readable source code, which must be distributed under
   the terms of Sections 1 and 2 above on a medium customarily used for software
   interchange.

   If distribution of object code is made by offering access to copy from a
   designated place, then offering equivalent access to copy the source code
   from the same place satisfies the requirement to distribute the source code,
   even though third parties are not compelled to copy the source along with the
   object code.

   __1__ A program that contains no derivative of any portion of the Library,
   but is designed to work with the Library by being compiled or linked with it,
   is called a "work that uses the Library". Such a work, in isolation, is not a
   derivative work of the Library, and therefore falls outside the scope of this
   License.

   However, linking a "work that uses the Library" with the Library creates an
   executable that is a derivative of the Library (because it contains portions
   of the Library), rather than a "work that uses the library". The executable
   is therefore covered by this License. Section 6 states terms for distribution
   of such executables.

   When a "work that uses the Library" uses material from a header file that is
   part of the Library, the object code for the work may be a derivative work of
   the Library even though the source code is not. Whether this is true is
   especially significant if the work can be linked without the Library, or if
   the work is itself a library. The threshold for this to be true is not
   precisely defined by law.

   If such an object file uses only numerical parameters, data structure layouts
   and accessors, and small macros and small inline functions (ten lines or less
   in length), then the use of the object file is unrestricted, regardless of
   whether it is legally a derivative work. (Executables containing this object
   code plus portions of the Library will still fall under Section 6.)

   Otherwise, if the work is a derivative of the Library, you may distribute the
   object code for the work under the terms of Section 6. Any executables
   containing that work also fall under Section 6, whether or not they are
   linked directly with the Library itself.

   __1__ As an exception to the Sections above, you may also combine or link a
   "work that uses the Library" with the Library to produce a work containing
   portions of the Library, and distribute that work under terms of your choice,
   provided that the terms permit modification of the work for the customer's
   own use and reverse engineering for debugging such modifications.

   You must give prominent notice with each copy of the work that the Library is
   used in it and that the Library and its use are covered by this License. You
   must supply a copy of this License. If the work during execution displays
   copyright notices, you must include the copyright notice for the Library
   among them, as well as a reference directing the user to the copy of this
   License. Also, you must do one of these things:

      __1__ Accompany the work with the complete corresponding machine-readable
      source code for the Library including whatever changes were used in the
      work (which must be distributed under Sections 1 and 2 above); and, if the
      work is an executable linked with the Library, with the complete
      machine-readable "work that uses the Library", as object code and/or
      source code, so that the user can modify the Library and then relink to
      produce a modified executable containing the modified Library. (It is
      understood that the user who changes the contents of definitions files in
      the Library will not necessarily be able to recompile the application to
      use the modified definitions.)

      __1__ Use a suitable shared library mechanism for linking with the
      Library. A suitable mechanism is one that (1) uses at run time a copy of
      the library already present on the user's computer system, rather than
      copying library functions into the executable, and (2) will operate
      properly with a modified version of the library, if the user installs one,
      as long as the modified version is interface-compatible with the version
      that the work was made with.

      __1__ Accompany the work with a written offer, valid for at least three
      years, to give the same user the materials specified in Subsection 6a,
      above, for a charge no more than the cost of performing this
      distribution.

      __1__ If distribution of the work is made by offering access to copy from
      a designated place, offer equivalent access to copy the above specified
      materials from the same place.

      __1__ Verify that the user has already received a copy of these materials
      or that you have already sent this user a copy.

   For an executable, the required form of the "work that uses the Library" must
   include any data and utility programs needed for reproducing the executable
   from it. However, as a special exception, the materials to be distributed
   need not include anything that is normally distributed (in either source or
   binary form) with the major components (compiler, kernel, and so on) of the
   operating system on which the executable runs, unless that component itself
   accompanies the executable.

   It may happen that this requirement contradicts the license restrictions of
   other proprietary libraries that do not normally accompany the operating
   system. Such a contradiction means you cannot use both them and the Library
   together in an executable that you distribute.

   __1__ You may place library facilities that are a work based on the Library
   side-by-side in a single library together with other library facilities not
   covered by this License, and distribute such a combined library, provided
   that the separate distribution of the work based on the Library and of the
   other library facilities is otherwise permitted, and provided that you do
   these two things:

      __1__ Accompany the combined library with a copy of the same work based on
      the Library, uncombined with any other library facilities. This must be
      distributed under the terms of the Sections above.

      __1__ Give prominent notice with the combined library of the fact that
      part of it is a work based on the Library, and explaining where to find
      the accompanying uncombined form of the same work.

   __1__ You may not copy, modify, sublicense, link with, or distribute the
   Library except as expressly provided under this License. Any attempt
   otherwise to copy, modify, sublicense, link with, or distribute the Library
   is void, and will automatically terminate your rights under this License.
   However, parties who have received copies, or rights, from you under this
   License will not have their licenses terminated so long as such parties
   remain in full compliance.

   __1__ You are not required to accept this License, since you have not signed
   it. However, nothing else grants you permission to modify or distribute the
   Library or its derivative works. These actions are prohibited by law if you
   do not accept this License. Therefore, by modifying or distributing the
   Library (or any work based on the Library), you indicate your acceptance of
   this License to do so, and all its terms and conditions for copying,
   distributing or modifying the Library or works based on it.

   __1__ Each time you redistribute the Library (or any work based on the
   Library), the recipient automatically receives a license from the original
   licensor to copy, distribute, link with or modify the Library subject to
   these terms and conditions. You may not impose any further restrictions on
   the recipients' exercise of the rights granted herein. You are not
   responsible for enforcing compliance by third parties with this License.

   __1__ If, as a consequence of a court judgment or allegation of patent
   infringement or for any other reason (not limited to patent issues),
   conditions are imposed on you (whether by court order, agreement or
   otherwise) that contradict the conditions of this License, they do not excuse
   you from the conditions of this License. If you cannot distribute so as to
   satisfy simultaneously your obligations under this License and any other
   pertinent obligations, then as a consequence you may not distribute the
   Library at all. For example, if a patent license would not permit
   royalty-free redistribution of the Library by all those who receive copies
   directly or indirectly through you, then the only way you could satisfy both
   it and this License would be to refrain entirely from distribution of the
   Library.

   If any portion of this section is held invalid or unenforceable under any
   particular circumstance, the balance of the section is intended to apply, and
   the section as a whole is intended to apply in other circumstances.

   It is not the purpose of this section to induce you to infringe any patents
   or other property right claims or to contest validity of any such claims;
   this section has the sole purpose of protecting the integrity of the free
   software distribution system which is implemented by public license
   practices. Many people have made generous contributions to the wide range of
   software distributed through that system in reliance on consistent
   application of that system; it is up to the author/donor to decide if he or
   she is willing to distribute software through any other system and a licensee
   cannot impose that choice.

   This section is intended to make thoroughly clear what is believed to be a
   consequence of the rest of this License.

   __1__ If the distribution and/or use of the Library is restricted in certain
   countries either by patents or by copyrighted interfaces, the original
   copyright holder who places the Library under this License may add an
   explicit geographical distribution limitation excluding those countries, so
   that distribution is permitted only in or among countries not thus excluded.
   In such case, this License incorporates the limitation as if written in the
   body of this License.

   __1__ The Free Software Foundation may publish revised and/or new versions of
   the Lesser General Public License from time to time. Such new versions will
   be similar in spirit to the present version, but may differ in detail to
   address new problems or concerns.

   Each version is given a distinguishing version number. If the Library
   specifies a version number of this License which applies to it and "any later
   version", you have the option of following the terms and conditions either of
   that version or of any later version published by the Free Software
   Foundation. If the Library does not specify a license version number, you may
   choose any version ever published by the Free Software Foundation.

   __1__ If you wish to incorporate parts of the Library into other free
   programs whose distribution conditions are incompatible with these, write to
   the author to ask for permission. For software which is copyrighted by the
   Free Software Foundation, write to the Free Software Foundation; we sometimes
   make exceptions for this. Our decision will be guided by the two goals of
   preserving the free status of all derivatives of our free software and of
   promoting the sharing and reuse of software generally.

   NO WARRANTY

   __1__ BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
   FOR THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
   OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
   PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR
   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO
   THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE LIBRARY
   PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR
   CORRECTION.

   __1__ IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
   WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
   REDISTRIBUTE THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
   INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
   OUT OF THE USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO
   LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR
   THIRD PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER
   SOFTWARE), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGES.

((END OF TERMS AND CONDITIONS))??

((How to Apply These Terms to Your New Libraries

If you develop a new library, and you want it to be of the greatest possible use
to the public, we recommend making it free software that everyone can
redistribute and change. You can do so by permitting redistribution under these
terms (or, alternatively, under the terms of the ordinary General Public
License).

To apply these terms, attach the following notices to the library. It is safest
to attach them to the start of each source file to most effectively convey the
exclusion of warranty; and each file should have at least the "copyright" line
and a pointer to where the full notice is found.

__30__
//**
<one line to give the program's name and a brief idea of what it does.>
Copyright (C) <year> <name of author>
**//

This library is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free
Software Foundation; either version 2.1 of the License, or (at your option) any
later version.

This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with this library; if not, write to the Free Software Foundation, Inc.,

	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))

Also add information on
how to contact you by electronic and paper mail.

You should also get your employer (if you work as a programmer) or your school,
if any, to sign a "copyright disclaimer" for the library, if necessary. Here is
a sample; alter the names:

Yoyodyne, Inc., hereby disclaims all copyright interest in

the library ` + "`" + `Frob' (a library for tweaking knobs) written

by James Random Hacker.

__5__ //**<signature of Ty Coon>**//, 1 April 1990

Ty Coon, President of Vice

That's all there is to it! ))??
`
const license_LGPL_2_1_only_lre = `
//**
https://spdx.org/licenses/LGPL-2.1-only.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		library
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
	General Public License
	((LGPL))??
	((as published by the Free Software Foundation))??
	
		((under))??
		version 2.1
		((LGPLv2.1))??
		((of the License))??
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
		General Public License version
		2.1
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
		General Public License for more details.

		((
			You should have received a copy of the
			((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
			General Public License
			((
				((version))??
				2.1
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_LGPL_2_1_or_later_lre = `
//**
https://spdx.org/licenses/LGPL-2.1-or-later.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		library
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
	General Public License
	((LGPL))??
	((as published by the Free Software Foundation))??
	
		((
			either version 2.1
			((of the License))??
			or
			((at your option))??
			any later version.
		||
			version 2.1 or later
			((of the License))??
		))
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
		General Public License version
		2.1
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
		General Public License for more details.

		((
			You should have received a copy of the
			((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
			General Public License
			((
				((version))??
				2.1
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_LGPL_3_0_lre = `//**
GNU Lesser General Public License v3.0
https://www.gnu.org/licenses/lgpl-3.0-standalone.html
https://opensource.org/licenses/LGPL-3.0
**//

((
	GNU LESSER GENERAL PUBLIC LICENSE Version 3, 29 June 2007
	
	((
		((
			Copyright __20__
			((<https://fsf.org/>))??
			
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))??
		))??

		Everyone is permitted to copy and distribute verbatim copies
		of this license document, but changing it is not allowed.

		((Copyright __20__))??
	))??

))??


This version of the GNU Lesser General Public License incorporates the terms and
conditions of version 3 of the GNU General Public License, supplemented by the
additional permissions listed below.

   __1__ Additional Definitions.



      As used herein, "this License" refers to version 3 of the GNU Lesser
      General Public License, and the "GNU GPL" refers to version 3 of the GNU
      General Public License.



      "The Library" refers to a covered work governed by this License, other
      than an Application or a Combined Work as defined below.



      An "Application" is any work that makes use of an interface provided by
      the Library, but which is not otherwise based on the Library. Defining a
      subclass of a class defined by the Library is deemed a mode of using an
      interface provided by the Library.



      A "Combined Work" is a work produced by combining or linking an
      Application with the Library. The particular version of the Library with
      which the Combined Work was made is also called the "Linked Version".



      The "Minimal Corresponding Source" for a Combined Work means the
      Corresponding Source for the Combined Work, excluding any source code for
      portions of the Combined Work that, considered in isolation, are based on
      the Application, and not on the Linked Version.



      The "Corresponding Application Code" for a Combined Work means the object
      code and/or source code for the Application, including any data and
      utility programs needed for reproducing the Combined Work from the
      Application, but excluding the System Libraries of the Combined Work.

   __1__ Exception to Section 3 of the GNU GPL.

   You may convey a covered work under sections 3 and 4 of this License without
   being bound by section 3 of the GNU GPL.

   __1__ Conveying Modified Versions.

   If you modify a copy of the Library, and, in your modifications, a facility
   refers to a function or data to be supplied by an Application that uses the
   facility (other than as an argument passed when the facility is invoked),
   then you may convey a copy of the modified version:

      __1__ under this License, provided that you make a good faith effort to
      ensure that, in the event an Application does not supply the function or
      data, the facility still operates, and performs whatever part of its
      purpose remains meaningful, or

      __1__ under the GNU GPL, with none of the additional permissions of this
      License applicable to that copy.

   __1__ Object Code Incorporating Material from Library Header Files.

   The object code form of an Application may incorporate material from a header
   file that is part of the Library. You may convey such object code under terms
   of your choice, provided that, if the incorporated material is not limited to
   numerical parameters, data structure layouts and accessors, or small macros,
   inline functions and templates (ten or fewer lines in length), you do both of
   the following:

      __1__ Give prominent notice with each copy of the object code that the
      Library is used in it and that the Library and its use are covered by this
      License.

      __1__ Accompany the object code with a copy of the GNU GPL and this
      license document.

   __1__ Combined Works.

   You may convey a Combined Work under terms of your choice that, taken
   together, effectively do not restrict modification of the portions of the
   Library contained in the Combined Work and reverse engineering for debugging
   such modifications, if you also do each of the following:

      __1__ Give prominent notice with each copy of the Combined Work that the
      Library is used in it and that the Library and its use are covered by this
      License.

      __1__ Accompany the Combined Work with a copy of the GNU GPL and this
      license document.

      __1__ For a Combined Work that displays copyright notices during
      execution, include the copyright notice for the Library among these
      notices, as well as a reference directing the user to the copies of the
      GNU GPL and this license document.

      __1__ Do one of the following:

         __1__ Convey the Minimal Corresponding Source under the terms of this
         License, and the Corresponding Application Code in a form suitable for,
         and under terms that permit, the user to recombine or relink the
         Application with a modified version of the Linked Version to produce a
         modified Combined Work, in the manner specified by section 6 of the GNU
         GPL for conveying Corresponding Source.

         __1__ Use a suitable shared library mechanism for linking with the
         Library. A suitable mechanism is one that (a) uses at run time a copy
         of the Library already present on the user's computer system, and (b)
         will operate properly with a modified version of the Library that is
         interface-compatible with the Linked Version.

      __1__ Provide Installation Information, but only if you would otherwise be
      required to provide such information under section 6 of the GNU GPL, and
      only to the extent that such information is necessary to install and
      execute a modified version of the Combined Work produced by recombining or
      relinking the Application with a modified version of the Linked Version.
      (If you use option 4d0, the Installation Information must accompany the
      Minimal Corresponding Source and Corresponding Application Code. If you
      use option 4d1, you must provide the Installation Information in the
      manner specified by section 6 of the GNU GPL for conveying Corresponding
      Source.)

   __1__ Combined Libraries.

   You may place library facilities that are a work based on the Library side by
   side in a single library together with other library facilities that are not
   Applications and are not covered by this License, and convey such a combined
   library under terms of your choice, if you do both of the following:

      __1__ Accompany the combined library with a copy of the same work based on
      the Library, uncombined with any other library facilities, conveyed under
      the terms of this License.

      __1__ Give prominent notice with the combined library that part of it is a
      work based on the Library, and explaining where to find the accompanying
      uncombined form of the same work.

   __1__ Revised Versions of the GNU Lesser General Public License.

   The Free Software Foundation may publish revised and/or new versions of the
   GNU Lesser General Public License from time to time. Such new versions will
   be similar in spirit to the present version, but may differ in detail to
   address new problems or concerns.

   Each version is given a distinguishing version number. If the Library as you
   received it specifies that a certain numbered version of the GNU Lesser
   General Public License "or any later version" applies to it, you have the
   option of following the terms and conditions either of that published version
   or of any later version published by the Free Software Foundation. If the
   Library as you received it does not specify a version number of the GNU
   Lesser General Public License, you may choose any version of the GNU Lesser
   General Public License ever published by the Free Software Foundation.

   If the Library as you received it specifies that a proxy can decide whether
   future versions of the GNU Lesser General Public License shall apply, that
   proxy's public statement of acceptance of any version is permanent
   authorization for you to choose that version for the Library.
`
const license_LGPL_3_0_only_lre = `
//**
https://spdx.org/licenses/LGPL-3.0-only.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		library
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
	General Public License
	((LGPL))??
	((as published by the Free Software Foundation))??
	
		((under))??
		version 3
		((LGPLv3))??
		((of the License))??
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
		General Public License version
		3
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
		General Public License for more details.

		((
			You should have received a copy of the
			((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
			General Public License
			((
				((version))??
				3
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_LGPL_3_0_or_later_lre = `
//**
https://spdx.org/licenses/LGPL-3.0-or-later.json
**//

	
	 
	 
	 
	 
	 

	((
		((This
		library
		))??
		//**__5__**//
		is free software: you can redistribute it
	||
		You can
		((uses))??
		redistribute __5__
	))
	and/or modify
	((it || this code))
	under the terms of the
	((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
	General Public License
	((LGPL))??
	((as published by the Free Software Foundation))??
	
		((
			either version 3
			((of the License))??
			or
			((at your option))??
			any later version.
		||
			version 3 or later
			((of the License))??
		))
	
	((as published by the Free Software Foundation))??

	((
		See the __3__ file for the full terms of the
		((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
		General Public License version
		3
	))??

	((
		__5__ is distributed
		((in the hope that it will be useful, but))??
		WITHOUT ANY WARRANTY;
		without even the implied warranty
		of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
		See the
		((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
		General Public License for more details.

		((
			You should have received a copy of the
			((GNU
((Lesser||Library))
||
((Lesser||Library))
GNU))
			General Public License
			((
				((version))??
				3
			))??
			along with
			((this program))??
			(( __5__; if not,
				((
					write to the Free Software Foundation, Inc.,
					
	((
		51 Franklin
		((Street||St))
		((Fifth Floor||Suite 500,))??
		Boston, MA 02110 __1__ USA
	||
		59 Temple Place, Suite 330, Boston, MA  02111 __1__ USA
	||
		675 Mass Ave, Cambridge, MA 02139, USA
	))
				||
					see <http://www.gnu.org/licenses/>.
				))
			))??
		))??
	))??


`
const license_LGPLLR_lre = `//**
Lesser General Public License For Linguistic Resources
https://spdx.org/licenses/LGPLLR.json
http://www-igm.univ-mlv.fr/~unitex/lgpllr.html
**//

(( Lesser General Public License For Linguistic Resources ))??

Preamble

The licenses for most data are designed to take away your freedom to share and
change it. By contrast, this License is intended to guarantee your freedom to
share and change free data--to make sure the data are free for all their users.

This License, the Lesser General Public License for Linguistic Resources,
applies to some specially designated linguistic resources -- typically lexicons
and grammars.

TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

   (( 0. ))??
   This License Agreement applies to any Linguistic Resource which contains a
   notice placed by the copyright holder or other authorized party saying it may
   be distributed under the terms of this Lesser General Public License for
   Linguistic Resources (also called "this License"). Each licensee is addressed
   as "you".

   A "linguistic resource" means a collection of data about language prepared so
   as to be used with application programs.

   The "Linguistic Resource", below, refers to any such work which has been
   distributed under these terms. A "work based on the Linguistic Resource"
   means either the Linguistic Resource or any derivative work under copyright
   law: that is to say, a work containing the Linguistic Resource or a portion
   of it, either verbatim or with modifications and/or translated
   straightforwardly into another language. (Hereinafter, translation is
   included without limitation in the term "modification".)

   "Legible form" for a linguistic resource means the preferred form of the
   resource for making modifications to it.

   Activities other than copying, distribution and modification are not covered
   by this License; they are outside its scope. The act of running a program
   using the Linguistic Resource is not restricted, and output from such a
   program is covered only if its contents constitute a work based on the
   Linguistic Resource (independent of the use of the Linguistic Resource in a
   tool for writing it). Whether that is true depends on what the program that
   uses the Linguistic Resource does.

   (( 1. ))??
   You may copy and distribute verbatim copies of the Linguistic Resource as you
   receive it, in any medium, provided that you conspicuously and appropriately
   publish on each copy an appropriate copyright notice and disclaimer of
   warranty; keep intact all the notices that refer to this License and to the
   absence of any warranty; and distribute a copy of this License along with the
   Linguistic Resource.

   You may charge a fee for the physical act of transferring a copy, and you may
   at your option offer warranty protection in exchange for a fee.

   (( 2. ))??
   You may modify your copy or copies of the Linguistic Resource or any portion
   of it, thus forming a work based on the Linguistic Resource, and copy and
   distribute such modifications or work under the terms of Section 1 above,
   provided that you also meet all of these conditions:

      (( a) ))??
      The modified work must itself be a linguistic resource.

      (( b) ))??
      You must cause the files modified to carry prominent notices stating that
      you changed the files and the date of any change.

      (( c) ))??
      You must cause the whole of the work to be licensed at no charge to all
      third parties under the terms of this License.

   These requirements apply to the modified work as a whole. If identifiable
   sections of that work are not derived from the Linguistic Resource, and can
   be reasonably considered independent and separate works in themselves, then
   this License, and its terms, do not apply to those sections when you
   distribute them as separate works. But when you distribute the same sections
   as part of a whole which is a work based on the Linguistic Resource, the
   distribution of the whole must be on the terms of this License, whose
   permissions for other licensees extend to the entire whole, and thus to each
   and every part regardless of who wrote it.

   Thus, it is not the intent of this section to claim rights or contest your
   rights to work written entirely by you; rather, the intent is to exercise the
   right to control the distribution of derivative or collective works based on
   the Linguistic Resource.

   In addition, mere aggregation of another work not based on the Linguistic
   Resource with the Linguistic Resource (or with a work based on the Linguistic
   Resource) on a volume of a storage or distribution medium does not bring the
   other work under the scope of this License.

   (( 3. ))??
   A program that contains no derivative of any portion of the Linguistic
   Resource, but is designed to work with the Linguistic Resource (or an
   encrypted form of the Linguistic Resource) by reading it or being compiled or
   linked with it, is called a "work that uses the Linguistic Resource". Such a
   work, in isolation, is not a derivative work of the Linguistic Resource, and
   therefore falls outside the scope of this License.

   However, combining a "work that uses the Linguistic Resource" with the
   Linguistic Resource (or an encrypted form of the Linguistic Resource) creates
   a package that is a derivative of the Linguistic Resource (because it
   contains portions of the Linguistic Resource), rather than a "work that uses
   the Linguistic Resource". If the package is a derivative of the Linguistic
   Resource, you may distribute the package under the terms of Section 4. Any
   works containing that package also fall under Section 4.

   (( 4. ))??
   As an exception to the Sections above, you may also combine a "work that uses
   the Linguistic Resource" with the Linguistic Resource (or an encrypted form
   of the Linguistic Resource) to produce a package containing portions of the
   Linguistic Resource, and distribute that package under terms of your choice,
   provided that the terms permit modification of the package for the customer's
   own use and reverse engineering for debugging such modifications.

   You must give prominent notice with each copy of the package that the
   Linguistic Resource is used in it and that the Linguistic Resource and its
   use are covered by this License. You must supply a copy of this License. If
   the package during execution displays copyright notices, you must include the
   copyright notice for the Linguistic Resource among them, as well as a
   reference directing the user to the copy of this License. Also, you must do
   one of these things:

      (( a) ))??
      Accompany the package with the complete corresponding machine-readable
      legible form of the Linguistic Resource including whatever changes were
      used in the package (which must be distributed under Sections 1 and 2
      above); and, if the package contains an encrypted form of the Linguistic
      Resource, with the complete machine-readable "work that uses the
      Linguistic Resource", as object code and/or source code, so that the user
      can modify the Linguistic Resource and then encrypt it to produce a
      modified package containing the modified Linguistic Resource.

      (( b) ))??
      Use a suitable mechanism for combining with the Linguistic Resource. A
      suitable mechanism is one that will operate properly with a modified
      version of the Linguistic Resource, if the user installs one, as long as
      the modified version is interface-compatible with the version that the
      package was made with.

      (( c) ))??
      Accompany the package with a written offer, valid for at least three
      years, to give the same user the materials specified in Subsection 4a,
      above, for a charge no more than the cost of performing this
      distribution.

      (( d) ))??
      If distribution of the package is made by offering access to copy from a
      designated place, offer equivalent access to copy the above specified
      materials from the same place.

      (( e) ))??
      Verify that the user has already received a copy of these materials or
      that you have already sent this user a copy.

   If the package includes an encrypted form of the Linguistic Resource, the
   required form of the "work that uses the Linguistic Resource" must include
   any data and utility programs needed for reproducing the package from it.
   However, as a special exception, the materials to be distributed need not
   include anything that is normally distributed (in either source or binary
   form) with the major components (compiler, kernel, and so on) of the
   operating system on which the executable runs, unless that component itself
   accompanies the executable.

   It may happen that this requirement contradicts the license restrictions of
   proprietary libraries that do not normally accompany the operating system.
   Such a contradiction means you cannot use both them and the Linguistic
   Resource together in a package that you distribute.

   (( 5. ))??
   You may not copy, modify, sublicense, link with, or distribute the Linguistic
   Resource except as expressly provided under this License. Any attempt
   otherwise to copy, modify, sublicense, link with, or distribute the
   Linguistic Resource is void, and will automatically terminate your rights
   under this License. However, parties who have received copies, or rights,
   from you under this License will not have their licenses terminated so long
   as such parties remain in full compliance.

   (( 6. ))??
   You are not required to accept this License, since you have not signed it.
   However, nothing else grants you permission to modify or distribute the
   Linguistic Resource or its derivative works. These actions are prohibited by
   law if you do not accept this License. Therefore, by modifying or
   distributing the Linguistic Resource (or any work based on the Linguistic
   Resource), you indicate your acceptance of this License to do so, and all its
   terms and conditions for copying, distributing or modifying the Linguistic
   Resource or works based on it.

   (( 7. ))??
   Each time you redistribute the Linguistic Resource (or any work based on the
   Linguistic Resource), the recipient automatically receives a license from the
   original licensor to copy, distribute, link with or modify the Linguistic
   Resource subject to these terms and conditions. You may not impose any
   further restrictions on the recipients' exercise of the rights granted
   herein. You are not responsible for enforcing compliance by third parties
   with this License.

   (( 8. ))??
   If, as a consequence of a court judgment or allegation of patent infringement
   or for any other reason (not limited to patent issues), conditions are
   imposed on you (whether by court order, agreement or otherwise) that
   contradict the conditions of this License, they do not excuse you from the
   conditions of this License. If you cannot distribute so as to satisfy
   simultaneously your obligations under this License and any other pertinent
   obligations, then as a consequence you may not distribute the Linguistic
   Resource at all. For example, if a patent license would not permit
   royalty-free redistribution of the Linguistic Resource by all those who
   receive copies directly or indirectly through you, then the only way you
   could satisfy both it and this License would be to refrain entirely from
   distribution of the Linguistic Resource.

   If any portion of this section is held invalid or unenforceable under any
   particular circumstance, the balance of the section is intended to apply, and
   the section as a whole is intended to apply in other circumstances.

   It is not the purpose of this section to induce you to infringe any patents
   or other property right claims or to contest validity of any such claims;
   this section has the sole purpose of protecting the integrity of the free
   resource distribution system which is implemented by public license
   practices. Many people have made generous contributions to the wide range of
   data distributed through that system in reliance on consistent application of
   that system; it is up to the author/donor to decide if he or she is willing
   to distribute resources through any other system and a licensee cannot impose
   that choice.

   This section is intended to make thoroughly clear what is believed to be a
   consequence of the rest of this License.

   (( 9. ))??
   If the distribution and/or use of the Linguistic Resource is restricted in
   certain countries either by patents or by copyrighted interfaces, the
   original copyright holder who places the Linguistic Resource under this
   License may add an explicit geographical distribution limitation excluding
   those countries, so that distribution is permitted only in or among countries
   not thus excluded. In such case, this License incorporates the limitation as
   if written in the body of this License.

   (( 10. ))??
   The Free Software Foundation may publish revised and/or new versions of the
   Lesser General Public License for Linguistic Resources from time to time.
   Such new versions will be similar in spirit to the present version, but may
   differ in detail to address new problems or concerns.

   Each version is given a distinguishing version number. If the Linguistic
   Resource specifies a version number of this License which applies to it and
   "any later version", you have the option of following the terms and
   conditions either of that version or of any later version published by the
   Free Software Foundation. If the Linguistic Resource does not specify a
   license version number, you may choose any version ever published by the Free
   Software Foundation.

   (( 11. ))??
   If you wish to incorporate parts of the Linguistic Resource into other free
   programs whose distribution conditions are incompatible with these, write to
   the author to ask for permission.

   NO WARRANTY

   (( 12. ))??
   BECAUSE THE LINGUISTIC RESOURCE IS LICENSED FREE OF CHARGE, THERE IS NO
   WARRANTY FOR THE LINGUISTIC RESOURCE, TO THE EXTENT PERMITTED BY APPLICABLE
   LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR
   OTHER PARTIES PROVIDE THE LINGUISTIC RESOURCE "AS IS" WITHOUT WARRANTY OF ANY
   KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
   ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE LINGUISTIC RESOURCE IS
   WITH YOU. SHOULD THE LINGUISTIC RESOURCE PROVE DEFECTIVE, YOU ASSUME THE COST
   OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

   (( 13. ))??
   IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
   ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
   THE LINGUISTIC RESOURCE AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
   INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
   OUT OF THE USE OR INABILITY TO USE THE LINGUISTIC RESOURCE (INCLUDING BUT NOT
   LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED
   BY YOU OR THIRD PARTIES OR A FAILURE OF THE LINGUISTIC RESOURCE TO OPERATE
   WITH ANY OTHER SOFTWARE), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGES.

END OF TERMS AND CONDITIONS
`
const license_LPL_1_0_lre = `//**
Lucent Public License Version 1.0
https://spdx.org/licenses/LPL-1.0.json
https://opensource.org/licenses/LPL-1.0
**//

(( Lucent Public License Version 1.0 ))??

THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS PUBLIC LICENSE
("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THE PROGRAM CONSTITUTES
RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.

   (( 1. ))??
   DEFINITIONS

   "Contribution" means:

      (( a. ))??
      in the case of
      //** <ORGANIZATION> **//
      __5__
      ("
      //** <OWNER> **//
      __5__
      "), the Original Program, and

      (( b. ))??
      in the case of each Contributor,

         (( i. ))??
         changes to the Program, and

         (( ii. ))??
         additions to the Program; where such changes and/or additions to the
         Program originate from and are "Contributed" by that particular
         Contributor.

         A Contribution is "Contributed" by a Contributor only (i) if it was
         added to the Program by such Contributor itself or anyone acting on
         such Contributor's behalf, and (ii) the Contributor explicitly
         consents, in accordance with Section 3C, to characterization of the
         changes and/or additions as Contributions. Contributions do not include
         additions to the Program which: (i) are separate modules of software
         distributed in conjunction with the Program under their own license
         agreement, and (ii) are not derivative works of the Program.

         "Contributor" means
         //** <OWNER> **//
         __5__
         and any other entity that has Contributed a Contribution to the
         Program.

         "Distributor" means a Recipient that distributes the Program,
         modifications to the Program, or any part thereof.

         "Licensed Patents" mean patent claims licensable by a Contributor which
         are necessarily infringed by the use or sale of its Contribution alone
         or when combined with the Program.

         "Original Program" means the original version of the software
         accompanying this Agreement as released by
         //** <OWNER> **//
         __5__
         , including source code, object code and documentation, if any.

         "Program" means the Original Program and Contributions or any part
         thereof

         "Recipient" means anyone who receives the Program under this Agreement,
         including all Contributors.

   (( 2. ))??
   GRANT OF RIGHTS

      (( a. ))??
      Subject to the terms of this Agreement, each Contributor hereby grants
      Recipient a non-exclusive, worldwide, royalty-free copyright license to
      reproduce, prepare derivative works of, publicly display, publicly
      perform, distribute and sublicense the Contribution of such Contributor,
      if any, and such derivative works, in source code and object code form.

      (( b. ))??
      Subject to the terms of this Agreement, each Contributor hereby grants
      Recipient a non-exclusive, worldwide, royalty-free patent license under
      Licensed Patents to make, use, sell, offer to sell, import and otherwise
      transfer the Contribution of such Contributor, if any, in source code and
      object code form. The patent license granted by a Contributor shall also
      apply to the combination of the Contribution of that Contributor and the
      Program if, at the time the Contribution is added by the Contributor, such
      addition of the Contribution causes such combination to be covered by the
      Licensed Patents. The patent license granted by a Contributor shall not
      apply to (i) any other combinations which include the Contribution, nor to
      (ii) Contributions of other Contributors. No hardware per se is licensed
      hereunder.

      (( c. ))??
      Recipient understands that although each Contributor grants the licenses
      to its Contributions set forth herein, no assurances are provided by any
      Contributor that the Program does not infringe the patent or other
      intellectual property rights of any other entity. Each Contributor
      disclaims any liability to Recipient for claims brought by any other
      entity based on infringement of intellectual property rights or otherwise.
      As a condition to exercising the rights and licenses granted hereunder,
      each Recipient hereby assumes sole responsibility to secure any other
      intellectual property rights needed, if any. For example, if a third party
      patent license is required to allow Recipient to distribute the Program,
      it is Recipient's responsibility to acquire that license before
      distributing the Program.

      (( d. ))??
      Each Contributor represents that to its knowledge it has sufficient
      copyright rights in its Contribution, if any, to grant the copyright
      license set forth in this Agreement.

   (( 3. ))??
   REQUIREMENTS

      (( A. ))??
      Distributor may choose to distribute the Program in any form under this
      Agreement or under its own license agreement, provided that:

         (( 1. ))??
         it complies with the terms and conditions of this Agreement;

         (( 2. ))??
         if the Program is distributed in source code or other tangible form, a
         copy of this Agreement or Distributor's own license agreement is
         included with each copy of the Program; and

         (( 3. ))??
         if distributed under Distributor's own license agreement, such license
         agreement:

            (( a. ))??
            effectively disclaims on behalf of all Contributors all warranties
            and conditions, express and implied, including warranties or
            conditions of title and non-infringement, and implied warranties or
            conditions of merchantability and fitness for a particular purpose;

            (( b. ))??
            effectively excludes on behalf of all Contributors all liability for
            damages, including direct, indirect, special, incidental and
            consequential damages, such as lost profits; and

            (( c. ))??
            states that any provisions which differ from this Agreement are
            offered by that Contributor alone and not by any other party.

      (( B. ))??
      Each Distributor must include the following in a conspicuous location in
      the Program:

      Copyright (C) __20__.
      All Rights Reserved.

      (( C. ))??
      In addition, each Contributor must identify itself as the originator of
      its Contribution, if any, and indicate its consent to characterization of
      its additions and/or changes as a Contribution, in a manner that
      reasonably allows subsequent Recipients to identify the originator of the
      Contribution. Once consent is granted, it may not thereafter be revoked.

   (( 4. ))??
   COMMERCIAL DISTRIBUTION

   Commercial distributors of software may accept certain responsibilities with
   respect to end users, business partners and the like. While this license is
   intended to facilitate the commercial use of the Program, the Distributor who
   includes the Program in a commercial product offering should do so in a
   manner which does not create potential liability for Contributors. Therefore,
   if a Distributor includes the Program in a commercial product offering, such
   Distributor ("Commercial Distributor") hereby agrees to defend and indemnify
   every Contributor ("Indemnified Contributor") against any losses, damages and
   costs (collectively "Losses") arising from claims, lawsuits and other legal
   actions brought by a third party against the Indemnified Contributor to the
   extent caused by the acts or omissions of such Commercial Distributor in
   connection with its distribution of the Program in a commercial product
   offering. The obligations in this section do not apply to any claims or
   Losses relating to any actual or alleged intellectual property infringement.
   In order to qualify, an Indemnified Contributor must: a) promptly notify the
   Commercial Distributor in writing of such claim, and b) allow the Commercial
   Distributor to control, and cooperate with the Commercial Distributor in, the
   defense and any related settlement negotiations. The Indemnified Contributor
   may participate in any such claim at its own expense.

   For example, a Distributor might include the Program in a commercial product
   offering, Product X. That Distributor is then a Commercial Distributor. If
   that Commercial Distributor then makes performance claims, or offers
   warranties related to Product X, those performance claims and warranties are
   such Commercial Distributor's responsibility alone. Under this section, the
   Commercial Distributor would have to defend claims against the Contributors
   related to those performance claims and warranties, and if a court requires
   any Contributor to pay any damages as a result, the Commercial Distributor
   must pay those damages.

   (( 5. ))??
   NO WARRANTY

   EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, THE PROGRAM IS PROVIDED ON
   AN "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER
   EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION, ANY WARRANTIES OR
   CONDITIONS OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A
   PARTICULAR PURPOSE. Each Recipient is solely responsible for determining the
   appropriateness of using and distributing the Program and assumes all risks
   associated with its exercise of rights under this Agreement, including but
   not limited to the risks and costs of program errors, compliance with
   applicable laws, damage to or loss of data, programs or equipment, and
   unavailability or interruption of operations.

   (( 6. ))??
   DISCLAIMER OF LIABILITY

   EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, NEITHER RECIPIENT NOR ANY
   CONTRIBUTORS SHALL HAVE ANY LIABILITY FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING WITHOUT LIMITATION
   LOST PROFITS), HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OR DISTRIBUTION OF THE PROGRAM OR THE
   EXERCISE OF ANY RIGHTS GRANTED HEREUNDER, EVEN IF ADVISED OF THE POSSIBILITY
   OF SUCH DAMAGES.

   (( 7. ))??
   GENERAL

   If any provision of this Agreement is invalid or unenforceable under
   applicable law, it shall not affect the validity or enforceability of the
   remainder of the terms of this Agreement, and without further action by the
   parties hereto, such provision shall be reformed to the minimum extent
   necessary to make such provision valid and enforceable.

   If Recipient institutes patent litigation against a Contributor with respect
   to a patent applicable to software (including a cross-claim or counterclaim
   in a lawsuit), then any patent licenses granted by that Contributor to such
   Recipient under this Agreement shall terminate as of the date such litigation
   is filed. In addition, if Recipient institutes patent litigation against any
   entity (including a cross-claim or counterclaim in a lawsuit) alleging that
   the Program itself (excluding combinations of the Program with other software
   or hardware) infringes such Recipient's patent(s), then such Recipient's
   rights granted under Section 2(b) shall terminate as of the date such
   litigation is filed.

   All Recipient's rights under this Agreement shall terminate if it fails to
   comply with any of the material terms or conditions of this Agreement and
   does not cure such failure in a reasonable period of time after becoming
   aware of such noncompliance. If all Recipient's rights under this Agreement
   terminate, Recipient agrees to cease use and distribution of the Program as
   soon as reasonably practicable. However, Recipient's obligations under this
   Agreement and any licenses granted by Recipient relating to the Program shall
   continue and survive.

   //** <OWNER> **//
   __5__
   may publish new versions (including revisions) of this Agreement from time to
   time. Each new version of the Agreement will be given a distinguishing
   version number. The Program (including Contributions) may always be
   distributed subject to the version of the Agreement under which it was
   received. In addition, after a new version of the Agreement is published,
   Contributor may elect to distribute the Program (including its Contributions)
   under the new version. No one other than
   //** <OWNER> **//
   __5__
   has the right to modify this Agreement. Except as expressly stated in
   Sections 2(a) and 2(b) above, Recipient receives no rights or licenses to the
   intellectual property of any Contributor under this Agreement, whether
   expressly, by implication, estoppel or otherwise. All rights in the Program
   not expressly granted under this Agreement are reserved.

   This Agreement is governed by the laws of the State of
   //** <STATE> **//
   __5__
   and the intellectual property laws of the United States of America. No party
   to this Agreement will bring a legal action under this Agreement more than
   one year after the cause of action arose. Each party waives its rights to a
   jury trial in any resulting litigation.
`
const license_LPL_1_02_lre = `//**
Lucent Public License v1.02
https://spdx.org/licenses/LPL-1.02.json
http://plan9.bell-labs.com/plan9/license.html
https://opensource.org/licenses/LPL-1.02
**//

(( Lucent Public License Version 1.02 ))??

THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS PUBLIC LICENSE
("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THE PROGRAM CONSTITUTES
RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.

   (( 1. ))??
   DEFINITIONS

   "Contribution" means:

      (( a. ))??
      in the case of Lucent Technologies Inc. ("LUCENT"), the Original Program,
      and

      (( b. ))??
      in the case of each Contributor,

         (( i. ))??
         changes to the Program, and

         (( ii. ))??
         additions to the Program;

         where such changes and/or additions to the Program were added to the
         Program by such Contributor itself or anyone acting on such
         Contributor's behalf, and the Contributor explicitly consents, in
         accordance with Section 3C, to characterization of the changes and/or
         additions as Contributions.

      "Contributor" means LUCENT and any other entity that has Contributed a
      Contribution to the Program.

      "Distributor" means a Recipient that distributes the Program,
      modifications to the Program, or any part thereof.

      "Licensed Patents" mean patent claims licensable by a Contributor which
      are necessarily infringed by the use or sale of its Contribution alone or
      when combined with the Program.

      "Original Program" means the original version of the software accompanying
      this Agreement as released by LUCENT, including source code, object code
      and documentation, if any.

      "Program" means the Original Program and Contributions or any part
      thereof

      "Recipient" means anyone who receives the Program under this Agreement,
      including all Contributors.

   (( 2. ))??
   GRANT OF RIGHTS

      (( a. ))??
      Subject to the terms of this Agreement, each Contributor hereby grants
      Recipient a non-exclusive, worldwide, royalty-free copyright license to
      reproduce, prepare derivative works of, publicly display, publicly
      perform, distribute and sublicense the Contribution of such Contributor,
      if any, and such derivative works, in source code and object code form.

      (( b. ))??
      Subject to the terms of this Agreement, each Contributor hereby grants
      Recipient a non-exclusive, worldwide, royalty-free patent license under
      Licensed Patents to make, use, sell, offer to sell, import and otherwise
      transfer the Contribution of such Contributor, if any, in source code and
      object code form. The patent license granted by a Contributor shall also
      apply to the combination of the Contribution of that Contributor and the
      Program if, at the time the Contribution is added by the Contributor, such
      addition of the Contribution causes such combination to be covered by the
      Licensed Patents. The patent license granted by a Contributor shall not
      apply to (i) any other combinations which include the Contribution, nor to
      (ii) Contributions of other Contributors. No hardware per se is licensed
      hereunder.

      (( c. ))??
      Recipient understands that although each Contributor grants the licenses
      to its Contributions set forth herein, no assurances are provided by any
      Contributor that the Program does not infringe the patent or other
      intellectual property rights of any other entity. Each Contributor
      disclaims any liability to Recipient for claims brought by any other
      entity based on infringement of intellectual property rights or otherwise.
      As a condition to exercising the rights and licenses granted hereunder,
      each Recipient hereby assumes sole responsibility to secure any other
      intellectual property rights needed, if any. For example, if a third party
      patent license is required to allow Recipient to distribute the Program,
      it is Recipient's responsibility to acquire that license before
      distributing the Program.

      (( d. ))??
      Each Contributor represents that to its knowledge it has sufficient
      copyright rights in its Contribution, if any, to grant the copyright
      license set forth in this Agreement.

   (( 3. ))??
   REQUIREMENTS

      (( A. ))??
      Distributor may choose to distribute the Program in any form under this
      Agreement or under its own license agreement, provided that:

         (( a. || 1. ))??
         it complies with the terms and conditions of this Agreement;

         (( b. || 2. ))??
         if the Program is distributed in source code or other tangible form, a
         copy of this Agreement or Distributor's own license agreement is
         included with each copy of the Program; and

         (( c. || 3. ))??
         if distributed under Distributor's own license agreement, such license
         agreement:

            (( i. || a. ))??
            effectively disclaims on behalf of all Contributors all warranties
            and conditions, express and implied, including warranties or
            conditions of title and non-infringement, and implied warranties or
            conditions of merchantability and fitness for a particular purpose;

            (( ii. || b. ))??
            effectively excludes on behalf of all Contributors all liability for
            damages, including direct, indirect, special, incidental and
            consequential damages, such as lost profits; and

            (( iii. || c. ))??
            states that any provisions which differ from this Agreement are
            offered by that Contributor alone and not by any other party.

      (( B. ))??
      Each Distributor must include the following in a conspicuous location in
      the Program:

      Copyright (C) 2003, Lucent Technologies Inc. and others. All Rights
      Reserved.

      (( C. ))??
      In addition, each Contributor must identify itself as the originator of
      its Contribution in a manner that reasonably allows subsequent Recipients
      to identify the originator of the Contribution. Also, each Contributor
      must agree that the additions and/or changes are intended to be a
      Contribution. Once a Contribution is contributed, it may not thereafter be
      revoked.

   (( 4. ))??
   COMMERCIAL DISTRIBUTION

   Commercial distributors of software may accept certain responsibilities with
   respect to end users, business partners and the like. While this license is
   intended to facilitate the commercial use of the Program, the Distributor who
   includes the Program in a commercial product offering should do so in a
   manner which does not create potential liability for Contributors. Therefore,
   if a Distributor includes the Program in a commercial product offering, such
   Distributor ("Commercial Distributor") hereby agrees to defend and indemnify
   every Contributor ("Indemnified Contributor") against any losses, damages and
   costs (collectively "Losses") arising from claims, lawsuits and other legal
   actions brought by a third party against the Indemnified Contributor to the
   extent caused by the acts or omissions of such Commercial Distributor in
   connection with its distribution of the Program in a commercial product
   offering. The obligations in this section do not apply to any claims or
   Losses relating to any actual or alleged intellectual property infringement.
   In order to qualify, an Indemnified Contributor must: a) promptly notify the
   Commercial Distributor in writing of such claim, and b) allow the Commercial
   Distributor to control, and cooperate with the Commercial Distributor in, the
   defense and any related settlement negotiations. The Indemnified Contributor
   may participate in any such claim at its own expense.

   For example, a Distributor might include the Program in a commercial product
   offering, Product X. That Distributor is then a Commercial Distributor. If
   that Commercial Distributor then makes performance claims, or offers
   warranties related to Product X, those performance claims and warranties are
   such Commercial Distributor's responsibility alone. Under this section, the
   Commercial Distributor would have to defend claims against the Contributors
   related to those performance claims and warranties, and if a court requires
   any Contributor to pay any damages as a result, the Commercial Distributor
   must pay those damages.

   (( 5. ))??
   NO WARRANTY

   EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, THE PROGRAM IS PROVIDED ON
   AN "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER
   EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION, ANY WARRANTIES OR
   CONDITIONS OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A
   PARTICULAR PURPOSE. Each Recipient is solely responsible for determining the
   appropriateness of using and distributing the Program and assumes all risks
   associated with its exercise of rights under this Agreement, including but
   not limited to the risks and costs of program errors, compliance with
   applicable laws, damage to or loss of data, programs or equipment, and
   unavailability or interruption of operations.

   (( 6. ))??
   DISCLAIMER OF LIABILITY

   EXCEPT AS EXPRESSLY SET FORTH IN THIS AGREEMENT, NEITHER RECIPIENT NOR ANY
   CONTRIBUTORS SHALL HAVE ANY LIABILITY FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING WITHOUT LIMITATION
   LOST PROFITS), HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OR DISTRIBUTION OF THE PROGRAM OR THE
   EXERCISE OF ANY RIGHTS GRANTED HEREUNDER, EVEN IF ADVISED OF THE POSSIBILITY
   OF SUCH DAMAGES.

   (( 7. ))??
   EXPORT CONTROL

   Recipient agrees that Recipient alone is responsible for compliance with the
   United States export administration regulations (and the export control laws
   and regulation of any other countries).

   (( 8. ))??
   GENERAL

   If any provision of this Agreement is invalid or unenforceable under
   applicable law, it shall not affect the validity or enforceability of the
   remainder of the terms of this Agreement, and without further action by the
   parties hereto, such provision shall be reformed to the minimum extent
   necessary to make such provision valid and enforceable.

   If Recipient institutes patent litigation against a Contributor with respect
   to a patent applicable to software (including a cross-claim or counterclaim
   in a lawsuit), then any patent licenses granted by that Contributor to such
   Recipient under this Agreement shall terminate as of the date such litigation
   is filed. In addition, if Recipient institutes patent litigation against any
   entity (including a cross-claim or counterclaim in a lawsuit) alleging that
   the Program itself (excluding combinations of the Program with other software
   or hardware) infringes such Recipient's patent(s), then such Recipient's
   rights granted under Section 2(b) shall terminate as of the date such
   litigation is filed.

   All Recipient's rights under this Agreement shall terminate if it fails to
   comply with any of the material terms or conditions of this Agreement and
   does not cure such failure in a reasonable period of time after becoming
   aware of such noncompliance. If all Recipient's rights under this Agreement
   terminate, Recipient agrees to cease use and distribution of the Program as
   soon as reasonably practicable. However, Recipient's obligations under this
   Agreement and any licenses granted by Recipient relating to the Program shall
   continue and survive.

   LUCENT may publish new versions (including revisions) of this Agreement from
   time to time. Each new version of the Agreement will be given a
   distinguishing version number. The Program (including Contributions) may
   always be distributed subject to the version of the Agreement under which it
   was received. In addition, after a new version of the Agreement is published,
   Contributor may elect to distribute the Program (including its Contributions)
   under the new version. No one other than LUCENT has the right to modify this
   Agreement. Except as expressly stated in Sections 2(a) and 2(b) above,
   Recipient receives no rights or licenses to the intellectual property of any
   Contributor under this Agreement, whether expressly, by implication, estoppel
   or otherwise. All rights in the Program not expressly granted under this
   Agreement are reserved.

   This Agreement is governed by the laws of the State of New York and the
   intellectual property laws of the United States of America. No party to this
   Agreement will bring a legal action under this Agreement more than one year
   after the cause of action arose. Each party waives its rights to a jury trial
   in any resulting litigation.
`
const license_LPPL_1_0_lre = `//**
LaTeX Project Public License v1.0
https://spdx.org/licenses/LPPL-1.0.json
http://www.latex-project.org/lppl/lppl-1-0.txt
**//

(( LaTeX Project Public License
LPPL Version 1.0 1999-03-01

((Copyright __20__))??
))??

Everyone is permitted to copy and distribute verbatim copies of this license
document, but modification is not allowed.

Preamble

The LaTeX Project Public License (LPPL) is the license under which the base
LaTeX distribution is distributed. As described below you may use this licence
for any software that you wish to distribute.

It may be particularly suitable if your software is TeX related (such as a LaTeX
package file) but it may be used for any software, even if it is unrelated to
TeX.

To use this license, the files of your distribution should have an explicit
copyright notice giving your name and the year, together with a reference to
this license.

A typical example would be

%% pig.sty
%% Copyright 2001 M. Y. Name
% This program can redistributed and/or modified under the terms
% of the LaTeX Project Public License Distributed from CTAN
% archives in directory macros/latex/base/lppl.txt; either
% version 1 of the License, or (at your option) any later version.

Given such a notice in the file, the conditions of this document would apply,
with:

` + "`" + `The Program' referring to the software ` + "`" + `pig.sty' and ` + "`" + `The Copyright Holder'
referring to the person ` + "`" + `M. Y. Name'.

To see a real example, see the file legal.txt which carries the copyright notice
for the base latex distribution.

This license gives terms under which files of The Program may be distributed and
modified. Individual files may have specific further constraints on
modification, but no file should have restrictions on distribution other than
those specified below.

This is to ensure that a distributor wishing to distribute a complete unmodified
copy of The Program need only check the conditions in this file, and does not
need to check every file in The Program for extra restrictions. If you do need
to modify the distribution terms of some files, do not refer to this license,
instead distribute The Program under a different license. You may use the parts
of the text of LPPL as a model for your own license, but your license should not
directly refer to the LPPL or otherwise give the impression that The Program is
distributed under the LPPL.

The LaTeX Project Public License

================================

Terms And Conditions For Copying, Distribution And Modification

===============================================================

WARRANTY

========

There is no warranty for The Program, to the extent permitted by applicable law.
Except when otherwise stated in writing, The Copyright Holder provides The
Program ` + "`" + `as is' without warranty of any kind, either expressed or implied,
including, but not limited to, the implied warranties of merchantability and
fitness for a particular purpose. The entire risk as to the quality and
performance of the program is with you. Should The Program prove defective, you
assume the cost of all necessary servicing, repair or correction.

In no event unless required by applicable law or agreed to in writing will The
Copyright Holder, or any of the individual authors named in the source for The
Program, be liable to you for damages, including any general, special,
incidental or consequential damages arising out of any use of The Program or out
of inability to use The Program (including but not limited to loss of data or
data being rendered inaccurate or losses sustained by you or by third parties as
a result of a failure of The Program to operate with any other programs), even
if such holder or other party has been advised of the possibility of such
damages.

DISTRIBUTION

============

Redistribution of unchanged files is allowed provided that all files that make
up the distribution of The Program are distributed. In particular this means
that The Program has to be distributed including its documentation if
documentation was part of the original distribution.

The distribution of The Program will contain a prominent file listing all the
files covered by this license.

If you receive only some of these files from someone, complain!

The distribution of changed versions of certain files included in the The
Program, and the reuse of code from The Program, are allowed under the following
restrictions:

   *
   It is allowed only if the legal notice in the file does not expressly forbid
   it. See note below, under "Conditions on individual files".

   *
   You rename the file before you make any changes to it, unless the file
   explicitly says that renaming is not required. Any such changed files must be
   distributed under a license that forbids distribution of those files, and any
   files derived from them, under the names used by the original files in the
   distribution of The Program.

   *
   You change any ` + "`" + `identification string' in The Program to clearly indicate
   that the file is not part of the standard system.

   *
   If The Program includes an ` + "`" + `error report address' so that errors may be
   reported to The Copyright Holder, or other specified addresses, this address
   must be changed in any modified versions of The Program, so that reports for
   files not maintained by the original program maintainers are directed to the
   maintainers of the changed files.

   *
   You acknowledge the source and authorship of the original version in the
   modified file.

   *
   You also distribute the unmodified version of the file or alternatively
   provide sufficient information so that the user of your modified file can be
   reasonably expected to be able to obtain an original, unmodified copy of The
   Program. For example, you may specify a URL to a site that you expect will
   freely provide the user with a copy of The Program (either the version on
   which your modification is based, or perhaps a later version).

   *
   If The Program is intended to be used with, or is based on, LaTeX, then files
   with the following file extensions which have special meaning in LaTeX
   Software, have special modification rules under the license:

      -
      Files with extension ` + "`" + `.ins' (installation files): these files may not be
      modified at all because they contain the legal notices that are placed in
      the generated files.

      -
      Files with extension ` + "`" + `.fd' (LaTeX font definitions files): these files are
      allowed to be modified without changing the name, but only to enable use
      of all available fonts and to prevent attempts to access unavailable
      fonts. However, modified files are not allowed to be distributed in place
      of original files.

      -
      Files with extension ` + "`" + `.cfg' (configuration files): these files can be
      created or modified to enable easy configuration of the system. The
      documentation in cfgguide.tex in the base LaTeX distribution describes
      when it makes sense to modify or generate such files.

The above restrictions are not intended to prohibit, and hence do not apply to,
the updating, by any method, of a file so that it becomes identical to the
latest version of that file in The Program.

========================================================================

NOTES

=====

We believe that these requirements give you the freedom you to make
modifications that conform with whatever technical specifications you wish,
whilst maintaining the availability, integrity and reliability of The Program.
If you do not see how to achieve your goal whilst adhering to these requirements
then read the document cfgguide.tex in the base LaTeX distribution for
suggestions.

Because of the portability and exchangeability aspects of systems like LaTeX,
The LaTeX3 Project deprecates the distribution of non-standard versions of
components of LaTeX or of generally available contributed code for them but such
distributions are permitted under the above restrictions.

The document modguide.tex in the base LaTeX distribution details the reasons for
the legal requirements detailed above. Even if The Program is unrelated to
LaTeX, the argument in modguide.tex may still apply, and should be read before a
modified version of The Program is distributed.

Conditions on individual files

==============================

The individual files may bear additional conditions which supersede the general
conditions on distribution and modification contained in this file. If there are
any such files, the distribution of The Program will contain a prominent file
that lists all the exceptional files.

Typical examples of files with more restrictive modification conditions would be
files that contain the text of copyright notices.

   *
   The conditions on individual files differ only in the extent of
   *modification* that is allowed.

   *
   The conditions on *distribution* are the same for all the files. Thus a
   (re)distributor of a complete, unchanged copy of The Program need meet only
   the conditions in this file; it is not necessary to check the header of every
   file in the distribution to check that a distribution meets these
   requirements.
`
const license_LPPL_1_1_lre = `//**
LaTeX Project Public License v1.1
https://spdx.org/licenses/LPPL-1.1.json
http://www.latex-project.org/lppl/lppl-1-1.txt
**//

(( The LaTeX Project Public License

=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

LPPL Version 1.1 1999-07-10

((Copyright __20__))??
))??

Everyone is allowed to distribute verbatim copies of this license document, but
modification of it is not allowed.

PREAMBLE

========

The LaTeX Project Public License (LPPL) is the license under which the base
LaTeX distribution is distributed.

You may use this license for any program that you have written and wish to
distribute. This license may be particularly suitable if your program is
TeX-related (such as a LaTeX package), but you may use it even if your program
is unrelated to TeX. The section ` + "`" + `WHETHER AND HOW TO DISTRIBUTE PROGRAMS UNDER
THIS LICENSE', below, gives instructions, examples, and recommendations for
authors who are considering distributing their programs under this license.

In this license document, ` + "`" + `The Program' refers to any program distributed under
this license.

This license gives conditions under which The Program may be distributed and
conditions under which modified versions of The Program may be distributed.
Individual files of The Program may bear supplementary and/or superseding
conditions on modification of themselves and on the distribution of modified
versions of themselves, but *no* file of The Program may bear supplementary or
superseding conditions on the distribution of an unmodified copy of the file. A
distributor wishing to distribute a complete, unmodified copy of The Program
therefore needs to check the conditions only in this license and nowhere else.

Activities other than distribution and/or modification of The Program are not
covered by this license; they are outside its scope. In particular, the act of
running The Program is not restricted.

We, the LaTeX3 Project, believe that the conditions below give you the freedom
to make and distribute modified versions of The Program that conform with
whatever technical specifications you wish while maintaining the availability,
integrity, and reliability of The Program. If you do not see how to achieve your
goal while meeting these conditions, then read the document ` + "`" + `cfgguide.tex' in
the base LaTeX distribution for suggestions.

CONDITIONS ON DISTRIBUTION AND MODIFICATION

===========================================

You may distribute a complete, unmodified copy of The Program. Distribution of
only part of The Program is not allowed.

You may not modify in any way a file of The Program that bears a legal notice
forbidding modification of that file.

You may distribute a modified file of The Program if, and only if, the following
eight conditions are met:

   (( 1. ))??
   You must meet any additional conditions borne by the file on the distribution
   of a modified version of the file as described below in the subsection
   ` + "`" + `Additional Conditions on Individual Files of The Program'.

   (( 2. ))??
   If the file is a LaTeX software file, then you must meet any applicable
   additional conditions on the distribution of a modified version of the file
   that are described below in the subsection ` + "`" + `Additional Conditions on LaTeX
   Software Files'.

   (( 3. ))??
   You must not distribute the modified file with the filename of the original
   file.

   (( 4. ))??
   In the modified file, you must acknowledge the authorship and name of the
   original file, and the name (if any) of the program which contains it.

   (( 5. ))??
   You must change any identification string in the file to indicate clearly
   that the modified file is not part of The Program.

   (( 6. ))??
   You must change any addresses in the modified file for the reporting of
   errors in the file or in The Program generally to ensure that reports for
   files no longer maintained by the original maintainers will be directed to
   the maintainers of the modified files.

   (( 7. ))??
   You must distribute the modified file under a license that forbids
   distribution both of the modified file and of any files derived from the
   modified file with the filename of the original file.

   (( 8. ))??
   You must do either (A) or (B):

      (( (A) ))??
      distribute a copy of The Program (that is, a complete, unmodified copy of
      The Program) together with the modified file; if your distribution of the
      modified file is made by offering access to copy the modified file from a
      designated place, then offering equivalent access to copy The Program from
      the same place meets this condition, even though third parties are not
      compelled to copy The Program along with the modified file;

      (( (B) ))??
      provide to those who receive the modified file information that is
      sufficient for them to obtain a copy of The Program; for example, you may
      provide a Uniform Resource Locator (URL) for a site that you expect will
      provide them with a copy of The Program free of charge (either the version
      from which your modification is derived, or perhaps a later version).

Note that in the above, ` + "`" + `distribution' of a file means making the file available
to others by any means. This includes, for instance, installing the file on any
machine in such a way that the file is accessible by users other than yourself.
` + "`" + `Modification' of a file means any procedure that produces a derivative file
under any applicable law -- that is, a file containing the original file or a
significant portion of it, either verbatim or with modifications and/or
translated into another language.

Changing the name of a file is considered to be a modification of the file.

The distribution conditions in this license do not have to be applied to files
that have been modified in accordance with the above conditions. Note, however,
that Condition 7. does apply to any such modified file.

The conditions above are not intended to prohibit, and hence do not apply to,
the updating, by any method, of a file so that it becomes identical to the
latest version of that file of The Program.

A Recommendation on Modification Without Distribution

-----------------------------------------------------

It is wise never to modify a file of The Program, even for your own personal
use, without also meeting the above eight conditions for distributing the
modified file. While you might intend that such modified files will never be
distributed, often this will happen by accident -- you may forget that you have
modified the file; or it may not occur to you when allowing others to access the
modified file that you are thus distributing it and violating the conditions of
this license. It is usually in your best interest to keep your copy of The
Program identical with the public one. Many programs provide ways to control the
behavior of that program without altering its licensed files.

Additional Conditions on Individual Files of The Program

--------------------------------------------------------

An individual file of The Program may bear additional conditions that supplement
and/or supersede the conditions in this license if, and only if, such additional
conditions exclusively concern modification of the file or distribution of a
modified version of the file. The conditions on individual files of The Program
therefore may differ only with respect to the kind and extent of modification of
those files that is allowed, and with respect to the distribution of modified
versions of those files.

Additional Conditions on LaTeX Software Files

---------------------------------------------

If a file of The Program is intended to be used with LaTeX (that is, if it is a
LaTeX software file), then the following additional conditions, which supplement
and/or supersede the conditions above, apply to the file according to its
filename extension:

   -
   You may not modify any file with filename extension ` + "`" + `.ins' since these are
   installation files containing the legal notices that are placed in the files
   they generate.

   -
   You may distribute modified versions of files with filename extension ` + "`" + `.fd'
   (LaTeX font definition files) under the standard conditions of the LPPL as
   described above. You may also distribute such modified LaTeX font definition
   files with their original names provided that:

      (( (1) ))??
      the only changes to the original files either enable use of available
      fonts or prevent attempts to access unavailable fonts;

      (( (2) ))??
      you also distribute the original, unmodified files (TeX input paths can be
      used to control which set of LaTeX font definition files is actually used
      by TeX).

   -
   You may distribute modified versions of files with filename extension ` + "`" + `.cfg'
   (configuration files) with their original names. The Program may (and usually
   will) specify the range of commands that are allowed in a particular
   configuration file.

Because of portability and exchangeability issues in LaTeX software, The LaTeX3
Project deprecates the distribution of modified versions of components of LaTeX
or of generally available contributed code for them, but such distribution can
meet the conditions of this license.

NO WARRANTY

===========

There is no warranty for The Program. Except when otherwise stated in writing,
The Copyright Holder provides The Program ` + "`" + `as is', without warranty of any kind,
either expressed or implied, including, but not limited to, the implied
warranties of merchantability and fitness for a particular purpose. The entire
risk as to the quality and performance of The Program is with you. Should The
Program prove defective, you assume the cost of all necessary servicing, repair,
or correction.

In no event unless agreed to in writing will The Copyright Holder, or any author
named in the files of The Program, or any other party who may distribute and/or
modify The Program as permitted below, be liable to you for damages, including
any general, special, incidental or consequential damages arising out of any use
of The Program or out of inability to use The Program (including, but not
limited to, loss of data, data being rendered inaccurate, or losses sustained by
anyone as a result of any failure of The Program to operate with any other
programs), even if The Copyright Holder or said author or said other party has
been advised of the possibility of such damages.

WHETHER AND HOW TO DISTRIBUTE PROGRAMS UNDER THIS LICENSE

=========================================================

This section contains important instructions, examples, and recommendations for
authors who are considering distributing their programs under this license.
These authors are addressed as ` + "`" + `you' in this section.

Choosing This License or Another License

----------------------------------------

If for any part of your program you want or need to use *distribution*
conditions that differ from those in this license, then do not refer to this
license anywhere in your program but instead distribute your program under a
different license. You may use the text of this license as a model for your own
license, but your license should not refer to the LPPL or otherwise give the
impression that your program is distributed under the LPPL.

The document ` + "`" + `modguide.tex' in the base LaTeX distribution explains the
motivation behind the conditions of this license. It explains, for example, why
distributing LaTeX under the GNU General Public License (GPL) was considered
inappropriate. Even if your program is unrelated to LaTeX, the discussion in
` + "`" + `modguide.tex' may still be relevant, and authors intending to distribute their
programs under any license are encouraged to read it.

How to Use This License

-----------------------

To use this license, place in each of the files of your program both an explicit
copyright notice including your name and the year and also a statement that the
distribution and/or modification of the file is constrained by the conditions in
this license.

Here is an example of such a notice and statement:

%% pig.dtx
%% Copyright 2001 M. Y. Name
%
% This program may be distributed and/or modified under the
% conditions of the LaTeX Project Public License, either version 1.1
% of this license or (at your option) any later version.
% The latest version of this license is in
% http://www.latex-project.org/lppl.txt
% and version 1.1 or later is part of all distributions of LaTeX
% version 1999/06/01 or later.
%
% This program consists of the files pig.dtx and pig.ins

Given such a notice and statement in a file, the conditions given in this
license document would apply, with ` + "`" + `The Program' referring to the two files
` + "`" + `pig.dtx' and ` + "`" + `pig.ins', and ` + "`" + `The Copyright Holder' referring to the person ` + "`" + `M.
Y. Name'.

Important Recommendations

-------------------------

Defining What Constitutes The Program

The LPPL requires that distributions of The Program contain all the files of The
Program. It is therefore important that you provide a way for the licensee to
determine which files constitute The Program. This could, for example, be
achieved by explicitly listing all the files of The Program near the copyright
notice of each file or by using a line like

% This program consists of all files listed in manifest.txt.

in that place. In the absence of an unequivocal list it might be impossible for
the licensee to determine what is considered by you to comprise The Program.

Noting Exceptional Files

If The Program contains any files bearing additional conditions on modification,
or on distribution of modified versions, of those files (other than those listed
in ` + "`" + `Additional Conditions on LaTeX Software Files'), then it is recommended that
The Program contain a prominent file that defines the exceptional conditions,
and either lists the exceptional files or defines one or more categories of
exceptional files.

Files containing the text of a license (such as this file) are often examples of
files bearing more restrictive conditions on modification. LaTeX configuration
files (with filename extension ` + "`" + `.cfg') are examples of files bearing less
restrictive conditions on the distribution of a modified version of the file.
The additional conditions on LaTeX software given above are examples of
declaring a category of files bearing exceptional additional conditions.
`
const license_LPPL_1_2_lre = `//**
LaTeX Project Public License v1.2
https://spdx.org/licenses/LPPL-1.2.json
http://www.latex-project.org/lppl/lppl-1-2.txt
**//

(( The LaTeX Project Public License

=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

LPPL Version 1.2 1999-09-03
((Copyright __20__))??
))??

Everyone is allowed to distribute verbatim copies of this license document, but
modification of it is not allowed.

PREAMBLE

========

The LaTeX Project Public License (LPPL) is the license under which the base
LaTeX distribution is distributed.

You may use this license for any program that you have written and wish to
distribute. This license may be particularly suitable if your program is
TeX-related (such as a LaTeX package), but you may use it even if your program
is unrelated to TeX. The section ` + "`" + `WHETHER AND HOW TO DISTRIBUTE PROGRAMS UNDER
THIS LICENSE', below, gives instructions, examples, and recommendations for
authors who are considering distributing their programs under this license.

In this license document, ` + "`" + `The Program' refers to any program distributed under
this license.

This license gives conditions under which The Program may be distributed and
conditions under which modified versions of The Program may be distributed.
Individual files of The Program may bear supplementary and/or superseding
conditions on modification of themselves and on the distribution of modified
versions of themselves, but *no* file of The Program may bear supplementary or
superseding conditions on the distribution of an unmodified copy of the file. A
distributor wishing to distribute a complete, unmodified copy of The Program
therefore needs to check the conditions only in this license and nowhere else.

Activities other than distribution and/or modification of The Program are not
covered by this license; they are outside its scope. In particular, the act of
running The Program is not restricted.

We, the LaTeX3 Project, believe that the conditions below give you the freedom
to make and distribute modified versions of The Program that conform with
whatever technical specifications you wish while maintaining the availability,
integrity, and reliability of The Program. If you do not see how to achieve your
goal while meeting these conditions, then read the document ` + "`" + `cfgguide.tex' in
the base LaTeX distribution for suggestions.

CONDITIONS ON DISTRIBUTION AND MODIFICATION

===========================================

You may distribute a complete, unmodified copy of The Program. Distribution of
only part of The Program is not allowed.

You may not modify in any way a file of The Program that bears a legal notice
forbidding modification of that file.

You may distribute a modified file of The Program if, and only if, the following
eight conditions are met:

   (( 1. ))??
   You must meet any additional conditions borne by the file on the distribution
   of a modified version of the file as described below in the subsection
   ` + "`" + `Additional Conditions on Individual Files of The Program'.

   (( 2. ))??
   If the file is a LaTeX software file, then you must meet any applicable
   additional conditions on the distribution of a modified version of the file
   that are described below in the subsection ` + "`" + `Additional Conditions on LaTeX
   Software Files'.

   (( 3. ))??
   You must not distribute the modified file with the filename of the original
   file.

   (( 4. ))??
   In the modified file, you must acknowledge the authorship and name of the
   original file, and the name (if any) of the program which contains it.

   (( 5. ))??
   You must change any identification string in the file to indicate clearly
   that the modified file is not part of The Program.

   (( 6. ))??
   You must change any addresses in the modified file for the reporting of
   errors in the file or in The Program generally to ensure that reports for
   files no longer maintained by the original maintainers will be directed to
   the maintainers of the modified files.

   (( 7. ))??
   You must distribute the modified file under a license that forbids
   distribution both of the modified file and of any files derived from the
   modified file with the filename of the original file.

   (( 8. ))??
   You must do either (A) or (B):

      (( (A) ))??
      distribute a copy of The Program (that is, a complete, unmodified copy of
      The Program) together with the modified file; if your distribution of the
      modified file is made by offering access to copy the modified file from a
      designated place, then offering equivalent access to copy The Program from
      the same place meets this condition, even though third parties are not
      compelled to copy The Program along with the modified file;

      (( (B) ))??
      provide to those who receive the modified file information that is
      sufficient for them to obtain a copy of The Program; for example, you may
      provide a Uniform Resource Locator (URL) for a site that you expect will
      provide them with a copy of The Program free of charge (either the version
      from which your modification is derived, or perhaps a later version).

Note that in the above, ` + "`" + `distribution' of a file means making the file available
to others by any means. This includes, for instance, installing the file on any
machine in such a way that the file is accessible by users other than yourself.
` + "`" + `Modification' of a file means any procedure that produces a derivative file
under any applicable law -- that is, a file containing the original file or a
significant portion of it, either verbatim or with modifications and/or
translated into another language.

Changing the name of a file (other than as necessitated by the file conventions
of the target file systems) is considered to be a modification of the file.

The distribution conditions in this license do not have to be applied to files
that have been modified in accordance with the above conditions. Note, however,
that Condition 7. does apply to any such modified file.

The conditions above are not intended to prohibit, and hence do not apply to,
the updating, by any method, of a file so that it becomes identical to the
latest version of that file of The Program.

A Recommendation on Modification Without Distribution
-----------------------------------------------------

It is wise never to modify a file of The Program, even for your own personal
use, without also meeting the above eight conditions for distributing the
modified file. While you might intend that such modified files will never be
distributed, often this will happen by accident -- you may forget that you have
modified the file; or it may not occur to you when allowing others to access the
modified file that you are thus distributing it and violating the conditions of
this license. It is usually in your best interest to keep your copy of The
Program identical with the public one. Many programs provide ways to control the
behavior of that program without altering its licensed files.

Additional Conditions on Individual Files of The Program
--------------------------------------------------------

An individual file of The Program may bear additional conditions that supplement
and/or supersede the conditions in this license if, and only if, such additional
conditions exclusively concern modification of the file or distribution of a
modified version of the file. The conditions on individual files of The Program
therefore may differ only with respect to the kind and extent of modification of
those files that is allowed, and with respect to the distribution of modified
versions of those files.

Additional Conditions on LaTeX Software Files

---------------------------------------------

If a file of The Program is intended to be used with LaTeX (that is, if it is a
LaTeX software file), then the following additional conditions, which supplement
and/or supersede the conditions above, apply to the file according to its
filename extension:

   -
   You may not modify any file with filename extension ` + "`" + `.ins' since these are
   installation files containing the legal notices that are placed in the files
   they generate.

   -
   You may distribute modified versions of files with filename extension ` + "`" + `.fd'
   (LaTeX font definition files) under the standard conditions of the LPPL as
   described above. You may also distribute such modified LaTeX font definition
   files with their original names provided that:

      (( (1) ))??
      the only changes to the original files either enable use of available
      fonts or prevent attempts to access unavailable fonts;

      (( (2) ))??
      you also distribute the original, unmodified files (TeX input paths can be
      used to control which set of LaTeX font definition files is actually used
      by TeX).

   -
   You may distribute modified versions of files with filename extension ` + "`" + `.cfg'
   (configuration files) with their original names. The Program may (and usually
   will) specify the range of commands that are allowed in a particular
   configuration file.

Because of portability and exchangeability issues in LaTeX software, The LaTeX3
Project deprecates the distribution of modified versions of components of LaTeX
or of generally available contributed code for them, but such distribution can
meet the conditions of this license.

NO WARRANTY

===========

There is no warranty for The Program. Except when otherwise stated in writing,
The Copyright Holder provides The Program ` + "`" + `as is', without warranty of any kind,
either expressed or implied, including, but not limited to, the implied
warranties of merchantability and fitness for a particular purpose. The entire
risk as to the quality and performance of The Program is with you. Should The
Program prove defective, you assume the cost of all necessary servicing, repair,
or correction.

In no event unless agreed to in writing will The Copyright Holder, or any author
named in the files of The Program, or any other party who may distribute and/or
modify The Program as permitted above, be liable to you for damages, including
any general, special, incidental or consequential damages arising out of any use
of The Program or out of inability to use The Program (including, but not
limited to, loss of data, data being rendered inaccurate, or losses sustained by
anyone as a result of any failure of The Program to operate with any other
programs), even if The Copyright Holder or said author or said other party has
been advised of the possibility of such damages.

WHETHER AND HOW TO DISTRIBUTE PROGRAMS UNDER THIS LICENSE
=========================================================

This section contains important instructions, examples, and recommendations for
authors who are considering distributing their programs under this license.
These authors are addressed as ` + "`" + `you' in this section.

Choosing This License or Another License

----------------------------------------

If for any part of your program you want or need to use *distribution*
conditions that differ from those in this license, then do not refer to this
license anywhere in your program but instead distribute your program under a
different license. You may use the text of this license as a model for your own
license, but your license should not refer to the LPPL or otherwise give the
impression that your program is distributed under the LPPL.

The document ` + "`" + `modguide.tex' in the base LaTeX distribution explains the
motivation behind the conditions of this license. It explains, for example, why
distributing LaTeX under the GNU General Public License (GPL) was considered
inappropriate. Even if your program is unrelated to LaTeX, the discussion in
` + "`" + `modguide.tex' may still be relevant, and authors intending to distribute their
programs under any license are encouraged to read it.

How to Use This License

-----------------------

To use this license, place in each of the files of your program both an explicit
copyright notice including your name and the year and also a statement that the
distribution and/or modification of the file is constrained by the conditions in
this license.

Here is an example of such a notice and statement:

%% pig.dtx
%% Copyright 2001 M. Y. Name
%
% This program may be distributed and/or modified under the
% conditions of the LaTeX Project Public License, either version 1.2
% of this license or (at your option) any later version.
% The latest version of this license is in
% http://www.latex-project.org/lppl.txt
% and version 1.2 or later is part of all distributions of LaTeX
% version 1999/12/01 or later.
%
% This program consists of the files pig.dtx and pig.ins

Given such a notice and statement in a file, the conditions given in this
license document would apply, with ` + "`" + `The Program' referring to the two files
` + "`" + `pig.dtx' and ` + "`" + `pig.ins', and ` + "`" + `The Copyright Holder' referring to the person ` + "`" + `M.
Y. Name'.

Important Recommendations

-------------------------

Defining What Constitutes The Program

The LPPL requires that distributions of The Program contain all the files of The
Program. It is therefore important that you provide a way for the licensee to
determine which files constitute The Program. This could, for example, be
achieved by explicitly listing all the files of The Program near the copyright
notice of each file or by using a line like

% This program consists of all files listed in manifest.txt.

in that place. In the absence of an unequivocal list it might be impossible for
the licensee to determine what is considered by you to comprise The Program.

Noting Exceptional Files If The Program contains any files bearing additional
conditions on modification, or on distribution of modified versions, of those
files (other than those listed in ` + "`" + `Additional Conditions on LaTeX Software
Files'), then it is recommended that The Program contain a prominent file that
defines the exceptional conditions, and either lists the exceptional files or
defines one or more categories of exceptional files.

Files containing the text of a license (such as this file) are often examples of
files bearing more restrictive conditions on modification. LaTeX configuration
files (with filename extension ` + "`" + `.cfg') are examples of files bearing less
restrictive conditions on the distribution of a modified version of the file.
The additional conditions on LaTeX software given above are examples of
declaring a category of files bearing exceptional additional conditions.
`
const license_LPPL_1_3a_lre = `//**
LaTeX Project Public License v1.3a
https://spdx.org/licenses/LPPL-1.3a.json
http://www.latex-project.org/lppl/lppl-1-3a.txt
**//

(( The LaTeX Project Public License

=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

LPPL Version 1.3a 2004-10-01

((Copyright __20__))??
))??

Everyone is allowed to distribute verbatim copies of this license document, but
modification of it is not allowed.

PREAMBLE

========

The LaTeX Project Public License (LPPL) is the primary license under which the
the LaTeX kernel and the base LaTeX packages are distributed.

You may use this license for any work of which you hold the copyright and which
you wish to distribute. This license may be particularly suitable if your work
is TeX-related (such as a LaTeX package), but you may use it with small
modifications even if your work is unrelated to TeX.

The section ` + "`" + `WHETHER AND HOW TO DISTRIBUTE WORKS UNDER THIS LICENSE', below,
gives instructions, examples, and recommendations for authors who are
considering distributing their works under this license.

This license gives conditions under which a work may be distributed and
modified, as well as conditions under which modified versions of that work may
be distributed.

We, the LaTeX3 Project, believe that the conditions below give you the freedom
to make and distribute modified versions of your work that conform with whatever
technical specifications you wish while maintaining the availability, integrity,
and reliability of that work. If you do not see how to achieve your goal while
meeting these conditions, then read the document ` + "`" + `cfgguide.tex' and
` + "`" + `modguide.tex' in the base LaTeX distribution for suggestions.

DEFINITIONS

===========

In this license document the following terms are used:

` + "`" + `Work' Any work being distributed under this License. ` + "`" + `Derived Work' Any work
that under any applicable law is derived from the Work.

` + "`" + `Modification' Any procedure that produces a Derived Work under any applicable
law -- for example, the production of a file containing an original file
associated with the Work or a significant portion of such a file, either
verbatim or with modifications and/or translated into another language.

` + "`" + `Modify' To apply any procedure that produces a Derived Work under any
applicable law. ` + "`" + `Distribution' Making copies of the Work available from one
person to another, in whole or in part. Distribution includes (but is not
limited to) making any electronic components of the Work accessible by file
transfer protocols such as FTP or HTTP or by shared file systems such as Sun's
Network File System (NFS).

` + "`" + `Compiled Work' A version of the Work that has been processed into a form where
it is directly usable on a computer system. This processing may include using
installation facilities provided by the Work, transformations of the Work,
copying of components of the Work, or other activities. Note that modification
of any installation facilities provided by the Work constitutes modification of
the Work.

` + "`" + `Current Maintainer' A person or persons nominated as such within the Work. If
there is no such explicit nomination then it is the ` + "`" + `Copyright Holder' under any
applicable law.

` + "`" + `Base Interpreter' A program or process that is normally needed for running or
interpreting a part or the whole of the Work. A Base Interpreter may depend on
external components but these are not considered part of the Base Interpreter
provided that each external component clearly identifies itself whenever it is
used interactively. Unless explicitly specified when applying the license to the
Work, the only applicable Base Interpreter is a "LaTeX-Format".

CONDITIONS ON DISTRIBUTION AND MODIFICATION

===========================================

   (( 1. ))??
   Activities other than distribution and/or modification of the Work are not
   covered by this license; they are outside its scope. In particular, the act
   of running the Work is not restricted and no requirements are made concerning
   any offers of support for the Work.

   (( 2. ))??
   You may distribute a complete, unmodified copy of the Work as you received
   it. Distribution of only part of the Work is considered modification of the
   Work, and no right to distribute such a Derived Work may be assumed under the
   terms of this clause.

   (( 3. ))??
   You may distribute a Compiled Work that has been generated from a complete,
   unmodified copy of the Work as distributed under Clause 2 above, as long as
   that Compiled Work is distributed in such a way that the recipients may
   install the Compiled Work on their system exactly as it would have been
   installed if they generated a Compiled Work directly from the Work.

   (( 4. ))??
   If you are the Current Maintainer of the Work, you may, without restriction,
   modify the Work, thus creating a Derived Work. You may also distribute the
   Derived Work without restriction, including Compiled Works generated from the
   Derived Work. Derived Works distributed in this manner by the Current
   Maintainer are considered to be updated versions of the Work.

   (( 5. ))??
   If you are not the Current Maintainer of the Work, you may modify your copy
   of the Work, thus creating a Derived Work based on the Work, and compile this
   Derived Work, thus creating a Compiled Work based on the Derived Work.

   (( 6. ))??
   If you are not the Current Maintainer of the Work, you may distribute a
   Derived Work provided the following conditions are met for every component of
   the Work unless that component clearly states in the copyright notice that it
   is exempt from that condition. Only the Current Maintainer is allowed to add
   such statements of exemption to a component of the Work.

      (( a. ))??
      If a component of this Derived Work can be a direct replacement for a
      component of the Work when that component is used with the Base
      Interpreter, then, wherever this component of the Work identifies itself
      to the user when used interactively with that Base Interpreter, the
      replacement component of this Derived Work clearly and unambiguously
      identifies itself as a modified version of this component to the user when
      used interactively with that Base Interpreter.

      (( b. ))??
      Every component of the Derived Work contains prominent notices detailing
      the nature of the changes to that component, or a prominent reference to
      another file that is distributed as part of the Derived Work and that
      contains a complete and accurate log of the changes.

      (( c. ))??
      No information in the Derived Work implies that any persons, including
      (but not limited to) the authors of the original version of the Work,
      provide any support, including (but not limited to) the reporting and
      handling of errors, to recipients of the Derived Work unless those persons
      have stated explicitly that they do provide such support for the Derived
      Work.

      (( d. ))??
      You distribute at least one of the following with the Derived Work:

         (( 1. ))??
         A complete, unmodified copy of the Work; if your distribution of a
         modified component is made by offering access to copy the modified
         component from a designated place, then offering equivalent access to
         copy the Work from the same or some similar place meets this condition,
         even though third parties are not compelled to copy the Work along with
         the modified component;

         (( 2. ))??
         Information that is sufficient to obtain a complete, unmodified copy of
         the Work.

   (( 7. ))??
   If you are not the Current Maintainer of the Work, you may distribute a
   Compiled Work generated from a Derived Work, as long as the Derived Work is
   distributed to all recipients of the Compiled Work, and as long as the
   conditions of Clause 6, above, are met with regard to the Derived Work.

   (( 8. ))??
   The conditions above are not intended to prohibit, and hence do not apply to,
   the modification, by any method, of any component so that it becomes
   identical to an updated version of that component of the Work as it is
   distributed by the Current Maintainer under Clause 4, above.

   (( 9. ))??
   Distribution of the Work or any Derived Work in an alternative format, where
   the Work or that Derived Work (in whole or in part) is then produced by
   applying some process to that format, does not relax or nullify any sections
   of this license as they pertain to the results of applying that process.

   (( 10. ))??

      (( a. ))??
      A Derived Work may be distributed under a different license provided that
      license itself honors the conditions listed in Clause 6 above, in regard
      to the Work, though it does not have to honor the rest of the conditions
      in this license.

      (( b. ))??
      If a Derived Work is distributed under this license, that Derived Work
      must provide sufficient documentation as part of itself to allow each
      recipient of that Derived Work to honor the restrictions in Clause 6
      above, concerning changes from the Work.

   (( 11. ))??
   This license places no restrictions on works that are unrelated to the Work,
   nor does this license place any restrictions on aggregating such works with
   the Work by any means.

   (( 12. ))??
   Nothing in this license is intended to, or may be used to, prevent complete
   compliance by all parties with all applicable laws.

NO WARRANTY

===========

There is no warranty for the Work. Except when otherwise stated in writing, the
Copyright Holder provides the Work ` + "`" + `as is', without warranty of any kind, either
expressed or implied, including, but not limited to, the implied warranties of
merchantability and fitness for a particular purpose. The entire risk as to the
quality and performance of the Work is with you. Should the Work prove
defective, you assume the cost of all necessary servicing, repair, or
correction.

In no event unless required by applicable law or agreed to in writing will The
Copyright Holder, or any author named in the components of the Work, or any
other party who may distribute and/or modify the Work as permitted above, be
liable to you for damages, including any general, special, incidental or
consequential damages arising out of any use of the Work or out of inability to
use the Work (including, but not limited to, loss of data, data being rendered
inaccurate, or losses sustained by anyone as a result of any failure of the Work
to operate with any other programs), even if the Copyright Holder or said author
or said other party has been advised of the possibility of such damages.

MAINTENANCE OF THE WORK

=======================

The Work has the status ` + "`" + `author-maintained' if the Copyright Holder explicitly
and prominently states near the primary copyright notice in the Work that the
Work can only be maintained by the Copyright Holder or simply that is
` + "`" + `author-maintained'.

The Work has the status ` + "`" + `maintained' if there is a Current Maintainer who has
indicated in the Work that they are willing to receive error reports for the
Work (for example, by supplying a valid e-mail address). It is not required for
the Current Maintainer to acknowledge or act upon these error reports.

The Work changes from status ` + "`" + `maintained' to ` + "`" + `unmaintained' if there is no
Current Maintainer, or the person stated to be Current Maintainer of the work
cannot be reached through the indicated means of communication for a period of
six months, and there are no other significant signs of active maintenance.

You can become the Current Maintainer of the Work by agreement with any existing
Current Maintainer to take over this role.

If the Work is unmaintained, you can become the Current Maintainer of the Work
through the following steps:

   (( 1. ))??
   Make a reasonable attempt to trace the Current Maintainer (and the Copyright
   Holder, if the two differ) through the means of an Internet or similar
   search.

   (( 2. ))??
   If this search is successful, then enquire whether the Work is still
   maintained.

      (( a. ))??
      If it is being maintained, then ask the Current Maintainer to update their
      communication data within one month.

      (( b. ))??
      If the search is unsuccessful or no action to resume active maintenance is
      taken by the Current Maintainer, then announce within the pertinent
      community your intention to take over maintenance. (If the Work is a LaTeX
      work, this could be done, for example, by posting to comp.text.tex.)

      (( 3a. ))??
      If the Current Maintainer is reachable and agrees to pass maintenance of
      the Work to you, then this takes effect immediately upon announcement.

      (( b. ))??
      If the Current Maintainer is not reachable and the Copyright Holder agrees
      that maintenance of the Work be passed to you, then this takes effect
      immediately upon announcement.

   (( 4. ))??
   If you make an ` + "`" + `intention announcement' as described in 2b. above and after
   three months your intention is challenged neither by the Current Maintainer
   nor by the Copyright Holder nor by other people, then you may arrange for the
   Work to be changed so as to name you as the (new) Current Maintainer.

   (( 5. ))??
   If the previously unreachable Current Maintainer becomes reachable once more
   within three months of a change completed under the terms of 3b) or 4), then
   that Current Maintainer must become or remain the Current Maintainer upon
   request provided they then update their communication data within one month.

A change in the Current Maintainer does not, of itself, alter the fact that the
Work is distributed under the LPPL license.

If you become the Current Maintainer of the Work, you should immediately
provide, within the Work, a prominent and unambiguous statement of your status
as Current Maintainer. You should also announce your new status to the same
pertinent community as in 2b) above.

WHETHER AND HOW TO DISTRIBUTE WORKS UNDER THIS LICENSE

======================================================

This section contains important instructions, examples, and recommendations for
authors who are considering distributing their works under this license. These
authors are addressed as ` + "`" + `you' in this section.

Choosing This License or Another License

----------------------------------------

If for any part of your work you want or need to use *distribution* conditions
that differ significantly from those in this license, then do not refer to this
license anywhere in your work but, instead, distribute your work under a
different license. You may use the text of this license as a model for your own
license, but your license should not refer to the LPPL or otherwise give the
impression that your work is distributed under the LPPL.

The document ` + "`" + `modguide.tex' in the base LaTeX distribution explains the
motivation behind the conditions of this license. It explains, for example, why
distributing LaTeX under the GNU General Public License (GPL) was considered
inappropriate. Even if your work is unrelated to LaTeX, the discussion in
` + "`" + `modguide.tex' may still be relevant, and authors intending to distribute their
works under any license are encouraged to read it.

A Recommendation on Modification Without Distribution

-----------------------------------------------------

It is wise never to modify a component of the Work, even for your own personal
use, without also meeting the above conditions for distributing the modified
component. While you might intend that such modifications will never be
distributed, often this will happen by accident -- you may forget that you have
modified that component; or it may not occur to you when allowing others to
access the modified version that you are thus distributing it and violating the
conditions of this license in ways that could have legal implications and,
worse, cause problems for the community. It is therefore usually in your best
interest to keep your copy of the Work identical with the public one. Many works
provide ways to control the behavior of that work without altering any of its
licensed components.

How to Use This License

-----------------------

To use this license, place in each of the components of your work both an
explicit copyright notice including your name and the year the work was authored
and/or last substantially modified. Include also a statement that the
distribution and/or modification of that component is constrained by the
conditions in this license.

Here is an example of such a notice and statement:

%% pig.dtx
%% Copyright 2003 M. Y. Name
%
% This work may be distributed and/or modified under the
% conditions of the LaTeX Project Public License, either version 1.3
% of this license or (at your option) any later version.
% The latest version of this license is in
% http://www.latex-project.org/lppl.txt
% and version 1.3 or later is part of all distributions of LaTeX
% version 2003/12/01 or later.
%
% This work has the LPPL maintenance status " maintained ".
%
% This Current Maintainer of this work is M. Y. Name .
%
% This work consists of the files pig.dtx and pig.ins
% and the derived file pig.sty .

Given such a notice and statement in a file, the conditions given in this
license document would apply, with the ` + "`" + `Work' referring to the three files
` + "`" + `pig.dtx', ` + "`" + `pig.ins', and ` + "`" + `pig.sty' (the last being generated from ` + "`" + `pig.dtx'
using ` + "`" + `pig.ins'), the ` + "`" + `Base Interpreter' referring to any "LaTeX-Format", and
both ` + "`" + `Copyright Holder' and ` + "`" + `Current Maintainer' referring to the person ` + "`" + `M. Y.
Name'.

If you do not want the Maintenance section of LPPL to apply to your Work, change
"maintained" above into "author-maintained". However, we recommend that you use
"maintained" as the Maintenance section was added in order to ensure that your
Work remains useful to the community even when you can no longer maintain and
support it yourself.

Important Recommendations

-------------------------

Defining What Constitutes the Work

The LPPL requires that distributions of the Work contain all the files of the
Work. It is therefore important that you provide a way for the licensee to
determine which files constitute the Work. This could, for example, be achieved
by explicitly listing all the files of the Work near the copyright notice of
each file or by using a line such as:

% This work consists of all files listed in manifest.txt.

in that place. In the absence of an unequivocal list it might be impossible for
the licensee to determine what is considered by you to comprise the Work and, in
such a case, the licensee would be entitled to make reasonable conjectures as to
which files comprise the Work.
`
const license_LPPL_1_3c_lre = `//**
LaTeX Project Public License v1.3c
https://spdx.org/licenses/LPPL-1.3c.json
http://www.latex-project.org/lppl/lppl-1-3c.txt
https://opensource.org/licenses/LPPL-1.3c
**//

(( The LaTeX Project Public License

=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

LPPL Version 1.3c 2008-05-04

(( Copyright __20__ ))??

))??

Everyone is allowed to distribute verbatim copies of this license document, but
modification of it is not allowed.

PREAMBLE

========

The LaTeX Project Public License (LPPL) is the primary license under which the
LaTeX kernel and the base LaTeX packages are distributed.

You may use this license for any work of which you hold the copyright and which
you wish to distribute. This license may be particularly suitable if your work
is TeX-related (such as a LaTeX package), but it is written in such a way that
you can use it even if your work is unrelated to TeX.

The section ` + "`" + `WHETHER AND HOW TO DISTRIBUTE WORKS UNDER THIS LICENSE', below,
gives instructions, examples, and recommendations for authors who are
considering distributing their works under this license.

This license gives conditions under which a work may be distributed and
modified, as well as conditions under which modified versions of that work may
be distributed.

We, the LaTeX3 Project, believe that the conditions below give you the freedom
to make and distribute modified versions of your work that conform with whatever
technical specifications you wish while maintaining the availability, integrity,
and reliability of that work. If you do not see how to achieve your goal while
meeting these conditions, then read the document ` + "`" + `cfgguide.tex' and
` + "`" + `modguide.tex' in the base LaTeX distribution for suggestions.

DEFINITIONS

===========

In this license document the following terms are used:

` + "`" + `Work' Any work being distributed under this License. ` + "`" + `Derived Work' Any work
that under any applicable law is derived from the Work.

` + "`" + `Modification' Any procedure that produces a Derived Work under any applicable
law -- for example, the production of a file containing an original file
associated with the Work or a significant portion of such a file, either
verbatim or with modifications and/or translated into another language.

` + "`" + `Modify' To apply any procedure that produces a Derived Work under any
applicable law. ` + "`" + `Distribution' Making copies of the Work available from one
person to another, in whole or in part. Distribution includes (but is not
limited to) making any electronic components of the Work accessible by file
transfer protocols such as FTP or HTTP or by shared file systems such as Sun's
Network File System (NFS).

` + "`" + `Compiled Work' A version of the Work that has been processed into a form where
it is directly usable on a computer system. This processing may include using
installation facilities provided by the Work, transformations of the Work,
copying of components of the Work, or other activities. Note that modification
of any installation facilities provided by the Work constitutes modification of
the Work.

` + "`" + `Current Maintainer' A person or persons nominated as such within the Work. If
there is no such explicit nomination then it is the ` + "`" + `Copyright Holder' under any
applicable law.

` + "`" + `Base Interpreter' A program or process that is normally needed for running or
interpreting a part or the whole of the Work.

A Base Interpreter may depend on external components but these are not
considered part of the Base Interpreter provided that each external component
clearly identifies itself whenever it is used interactively. Unless explicitly
specified when applying the license to the Work, the only applicable Base
Interpreter is a ` + "`" + `LaTeX-Format' or in the case of files belonging to the
` + "`" + `LaTeX-format' a program implementing the ` + "`" + `TeX language'.

CONDITIONS ON DISTRIBUTION AND MODIFICATION

===========================================

   (( 1. ))??
   Activities other than distribution and/or modification of the Work are not
   covered by this license; they are outside its scope. In particular, the act
   of running the Work is not restricted and no requirements are made concerning
   any offers of support for the Work.

   (( 2. ))??
   You may distribute a complete, unmodified copy of the Work as you received
   it. Distribution of only part of the Work is considered modification of the
   Work, and no right to distribute such a Derived Work may be assumed under the
   terms of this clause.

   (( 3. ))??
   You may distribute a Compiled Work that has been generated from a complete,
   unmodified copy of the Work as distributed under Clause 2 above, as long as
   that Compiled Work is distributed in such a way that the recipients may
   install the Compiled Work on their system exactly as it would have been
   installed if they generated a Compiled Work directly from the Work.

   (( 4. ))??
   If you are the Current Maintainer of the Work, you may, without restriction,
   modify the Work, thus creating a Derived Work. You may also distribute the
   Derived Work without restriction, including Compiled Works generated from the
   Derived Work. Derived Works distributed in this manner by the Current
   Maintainer are considered to be updated versions of the Work.

   (( 5. ))??
   If you are not the Current Maintainer of the Work, you may modify your copy
   of the Work, thus creating a Derived Work based on the Work, and compile this
   Derived Work, thus creating a Compiled Work based on the Derived Work.

   (( 6. ))??
   If you are not the Current Maintainer of the Work, you may distribute a
   Derived Work provided the following conditions are met for every component of
   the Work unless that component clearly states in the copyright notice that it
   is exempt from that condition. Only the Current Maintainer is allowed to add
   such statements of exemption to a component of the Work.

      (( a. ))??
      If a component of this Derived Work can be a direct replacement for a
      component of the Work when that component is used with the Base
      Interpreter, then, wherever this component of the Work identifies itself
      to the user when used interactively with that Base Interpreter, the
      replacement component of this Derived Work clearly and unambiguously
      identifies itself as a modified version of this component to the user when
      used interactively with that Base Interpreter.

      (( b. ))??
      Every component of the Derived Work contains prominent notices detailing
      the nature of the changes to that component, or a prominent reference to
      another file that is distributed as part of the Derived Work and that
      contains a complete and accurate log of the changes.

      (( c. ))??
      No information in the Derived Work implies that any persons, including
      (but not limited to) the authors of the original version of the Work,
      provide any support, including (but not limited to) the reporting and
      handling of errors, to recipients of the Derived Work unless those persons
      have stated explicitly that they do provide such support for the Derived
      Work.

      (( d. ))??
      You distribute at least one of the following with the Derived Work:

         (( 1. ))??
         A complete, unmodified copy of the Work; if your distribution of a
         modified component is made by offering access to copy the modified
         component from a designated place, then offering equivalent access to
         copy the Work from the same or some similar place meets this condition,
         even though third parties are not compelled to copy the Work along with
         the modified component;

         (( 2. ))??
         Information that is sufficient to obtain a complete, unmodified copy of
         the Work.

   (( 7. ))??
   If you are not the Current Maintainer of the Work, you may distribute a
   Compiled Work generated from a Derived Work, as long as the Derived Work is
   distributed to all recipients of the Compiled Work, and as long as the
   conditions of Clause 6, above, are met with regard to the Derived Work.

   (( 8. ))??
   The conditions above are not intended to prohibit, and hence do not apply to,
   the modification, by any method, of any component so that it becomes
   identical to an updated version of that component of the Work as it is
   distributed by the Current Maintainer under Clause 4, above.

   (( 9. ))??
   Distribution of the Work or any Derived Work in an alternative format, where
   the Work or that Derived Work (in whole or in part) is then produced by
   applying some process to that format, does not relax or nullify any sections
   of this license as they pertain to the results of applying that process.

   (( 10. ))??

      (( a. ))??
      A Derived Work may be distributed under a different license provided that
      license itself honors the conditions listed in Clause 6 above, in regard
      to the Work, though it does not have to honor the rest of the conditions
      in this license.

      (( b. ))??
      If a Derived Work is distributed under a different license, that Derived
      Work must provide sufficient documentation as part of itself to allow each
      recipient of that Derived Work to honor the restrictions in Clause 6
      above, concerning changes from the Work.

   (( 11. ))??
   This license places no restrictions on works that are unrelated to the Work,
   nor does this license place any restrictions on aggregating such works with
   the Work by any means.

   (( 12. ))??
   Nothing in this license is intended to, or may be used to, prevent complete
   compliance by all parties with all applicable laws.

NO WARRANTY

===========

There is no warranty for the Work. Except when otherwise stated in writing, the
Copyright Holder provides the Work ` + "`" + `as is', without warranty of any kind, either
expressed or implied, including, but not limited to, the implied warranties of
merchantability and fitness for a particular purpose. The entire risk as to the
quality and performance of the Work is with you. Should the Work prove
defective, you assume the cost of all necessary servicing, repair, or
correction.

In no event unless required by applicable law or agreed to in writing will The
Copyright Holder, or any author named in the components of the Work, or any
other party who may distribute and/or modify the Work as permitted above, be
liable to you for damages, including any general, special, incidental or
consequential damages arising out of any use of the Work or out of inability to
use the Work (including, but not limited to, loss of data, data being rendered
inaccurate, or losses sustained by anyone as a result of any failure of the Work
to operate with any other programs), even if the Copyright Holder or said author
or said other party has been advised of the possibility of such damages.

MAINTENANCE OF THE WORK

=======================

The Work has the status ` + "`" + `author-maintained' if the Copyright Holder explicitly
and prominently states near the primary copyright notice in the Work that the
Work can only be maintained by the Copyright Holder or simply that it is
` + "`" + `author-maintained'.

The Work has the status ` + "`" + `maintained' if there is a Current Maintainer who has
indicated in the Work that they are willing to receive error reports for the
Work (for example, by supplying a valid e-mail address). It is not required for
the Current Maintainer to acknowledge or act upon these error reports.

The Work changes from status ` + "`" + `maintained' to ` + "`" + `unmaintained' if there is no
Current Maintainer, or the person stated to be Current Maintainer of the work
cannot be reached through the indicated means of communication for a period of
six months, and there are no other significant signs of active maintenance.

You can become the Current Maintainer of the Work by agreement with any existing
Current Maintainer to take over this role.

If the Work is unmaintained, you can become the Current Maintainer of the Work
through the following steps:

   (( 1. ))??
   Make a reasonable attempt to trace the Current Maintainer (and the Copyright
   Holder, if the two differ) through the means of an Internet or similar
   search.

   (( 2. ))??
   If this search is successful, then enquire whether the Work is still
   maintained.

      (( a. ))??
      If it is being maintained, then ask the Current Maintainer to update their
      communication data within one month.

      (( b. ))??
      If the search is unsuccessful or no action to resume active maintenance is
      taken by the Current Maintainer, then announce within the pertinent
      community your intention to take over maintenance. (If the Work is a LaTeX
      work, this could be done, for example, by posting to comp.text.tex.)

      (( 3a. ))??
      If the Current Maintainer is reachable and agrees to pass maintenance of
      the Work to you, then this takes effect immediately upon announcement.

      (( b. ))??
      If the Current Maintainer is not reachable and the Copyright Holder agrees
      that maintenance of the Work be passed to you, then this takes effect
      immediately upon announcement.

   (( 4. ))??
   If you make an ` + "`" + `intention announcement' as described in 2b. above and after
   three months your intention is challenged neither by the Current Maintainer
   nor by the Copyright Holder nor by other people, then you may arrange for the
   Work to be changed so as to name you as the (new) Current Maintainer.

   (( 5. ))??
   If the previously unreachable Current Maintainer becomes reachable once more
   within three months of a change completed under the terms of 3b) or 4), then
   that Current Maintainer must become or remain the Current Maintainer upon
   request provided they then update their communication data within one month.

A change in the Current Maintainer does not, of itself, alter the fact that the
Work is distributed under the LPPL license.

If you become the Current Maintainer of the Work, you should immediately
provide, within the Work, a prominent and unambiguous statement of your status
as Current Maintainer. You should also announce your new status to the same
pertinent community as in 2b) above.

WHETHER AND HOW TO DISTRIBUTE WORKS UNDER THIS LICENSE

======================================================

This section contains important instructions, examples, and recommendations for
authors who are considering distributing their works under this license. These
authors are addressed as ` + "`" + `you' in this section.

Choosing This License or Another License

----------------------------------------

If for any part of your work you want or need to use *distribution* conditions
that differ significantly from those in this license, then do not refer to this
license anywhere in your work but, instead, distribute your work under a
different license. You may use the text of this license as a model for your own
license, but your license should not refer to the LPPL or otherwise give the
impression that your work is distributed under the LPPL.

The document ` + "`" + `modguide.tex' in the base LaTeX distribution explains the
motivation behind the conditions of this license. It explains, for example, why
distributing LaTeX under the GNU General Public License (GPL) was considered
inappropriate. Even if your work is unrelated to LaTeX, the discussion in
` + "`" + `modguide.tex' may still be relevant, and authors intending to distribute their
works under any license are encouraged to read it.

A Recommendation on Modification Without Distribution

-----------------------------------------------------

It is wise never to modify a component of the Work, even for your own personal
use, without also meeting the above conditions for distributing the modified
component. While you might intend that such modifications will never be
distributed, often this will happen by accident -- you may forget that you have
modified that component; or it may not occur to you when allowing others to
access the modified version that you are thus distributing it and violating the
conditions of this license in ways that could have legal implications and,
worse, cause problems for the community. It is therefore usually in your best
interest to keep your copy of the Work identical with the public one. Many works
provide ways to control the behavior of that work without altering any of its
licensed components.

How to Use This License

-----------------------

To use this license, place in each of the components of your work both an
explicit copyright notice including your name and the year the work was authored
and/or last substantially modified. Include also a statement that the
distribution and/or modification of that component is constrained by the
conditions in this license.

Here is an example of such a notice and statement:

%% pig.dtx
%% Copyright 2005 M. Y. Name
%
% This work may be distributed and/or modified under the
% conditions of the LaTeX Project Public License, either version 1.3
% of this license or (at your option) any later version.
% The latest version of this license is in
% http://www.latex-project.org/lppl.txt
% and version 1.3 or later is part of all distributions of LaTeX
% version 2005/12/01 or later.
%
% This work has the LPPL maintenance status " maintained ".
%
% The Current Maintainer of this work is M. Y. Name .
%
% This work consists of the files pig.dtx and pig.ins
% and the derived file pig.sty .

Given such a notice and statement in a file, the conditions given in this
license document would apply, with the ` + "`" + `Work' referring to the three files
` + "`" + `pig.dtx', ` + "`" + `pig.ins', and ` + "`" + `pig.sty' (the last being generated from ` + "`" + `pig.dtx'
using ` + "`" + `pig.ins'), the ` + "`" + `Base Interpreter' referring to any ` + "`" + `LaTeX-Format', and
both ` + "`" + `Copyright Holder' and ` + "`" + `Current Maintainer' referring to the person ` + "`" + `M. Y.
Name'.

If you do not want the Maintenance section of LPPL to apply to your Work, change
` + "`" + `maintained' above into ` + "`" + `author-maintained'. However, we recommend that you use
` + "`" + `maintained', as the Maintenance section was added in order to ensure that your
Work remains useful to the community even when you can no longer maintain and
support it yourself.

Derived Works That Are Not Replacements

---------------------------------------

Several clauses of the LPPL specify means to provide reliability and stability
for the user community. They therefore concern themselves with the case that a
Derived Work is intended to be used as a (compatible or incompatible)
replacement of the original Work. If this is not the case (e.g., if a few lines
of code are reused for a completely different task), then clauses 6b and 6d
shall not apply.

Important Recommendations

-------------------------

Defining What Constitutes the Work

The LPPL requires that distributions of the Work contain all the files of the
Work. It is therefore important that you provide a way for the licensee to
determine which files constitute the Work. This could, for example, be achieved
by explicitly listing all the files of the Work near the copyright notice of
each file or by using a line such as:

% This work consists of all files listed in manifest.txt.

in that place. In the absence of an unequivocal list it might be impossible for
the licensee to determine what is considered by you to comprise the Work and, in
such a case, the licensee would be entitled to make reasonable conjectures as to
which files comprise the Work.
`
const license_Latex2e_lre = `//**
Latex2e License
https://spdx.org/licenses/Latex2e.json
https://fedoraproject.org/wiki/Licensing/Latex2e
**//

//** Copyright **//

Permission is granted to make and distribute verbatim copies of this manual
provided the copyright notice and this permission notice are preserved on all
copies.

Permission is granted to copy and distribute modified versions of this manual
under the conditions for verbatim copying, provided that the entire resulting
derived work is distributed under the terms of a permission notice identical to
this one.

Permission is granted to copy and distribute translations of this manual into
another language, under the above conditions for modified versions.
`
const license_Leptonica_lre = `//**
Leptonica License
https://spdx.org/licenses/Leptonica.json
https://fedoraproject.org/wiki/Licensing/Leptonica
**//

//** Copyright **//

This software is distributed in the hope that it will be useful, but with NO
WARRANTY OF ANY KIND.

No author or distributor accepts responsibility to anyone for the consequences
of using this software, or for whether it serves any particular purpose or works
at all, unless he or she says so in writing. Everyone is granted permission to
copy, modify and redistribute this source code, for commercial or non-commercial
purposes, with the following restrictions:

   (( (1) ))??
   the origin of this source code must not be misrepresented;

   (( (2) ))??
   modified versions must be plainly marked as such; and

   (( (3) ))??
   this notice may not be removed or altered from any source or modified source
   distribution.
`
const license_LiLiQ_P_1_1_lre = `//**
Licence Libre du Québec – Permissive version 1.1
https://spdx.org/licenses/LiLiQ-P-1.1.json
https://forge.gouv.qc.ca/licence/fr/liliq-v1-1/
http://opensource.org/licenses/LiLiQ-P-1.1
**//

(( Licence Libre du Québec – Permissive (LiLiQ-P)

Version 1.1 ))??

   (( 1. ))??
   Préambule

   Cette licence s'applique à tout logiciel distribué dont le titulaire du
   droit d'auteur précise qu'il est sujet aux termes de la Licence Libre du
   Québec – Permissive (LiLiQ-P) (ci-après appelée la « licence »).

   (( 2. ))??
   Définitions

   Dans la présente licence, à moins que le contexte n'indique un sens
   différent, on entend par:

   « concédant » : le titulaire du droit d'auteur sur le logiciel, ou
   toute personne dûment autorisée par ce dernier à accorder la présente
   licence;

   « contributeur » : le titulaire du droit d'auteur ou toute personne
   autorisée par ce dernier à soumettre au concédant une contribution. Un
   contributeur dont sa contribution est incorporée au logiciel est considéré
   comme un concédant en regard de sa contribution;

   « contribution » : tout logiciel original, ou partie de logiciel
   original soumis et destiné à être incorporé dans le logiciel;

   « distribution » : le fait de délivrer une copie du logiciel;

   « licencié » : toute personne qui possède une copie du logiciel et qui
   exerce les droits concédés par la licence;

   « logiciel » : une œuvre protégée par le droit d'auteur, telle qu'un
   programme d'ordinateur et sa documentation, pour laquelle le titulaire du
   droit d'auteur a précisé qu'elle est sujette aux termes de la présente
   licence;

   « logiciel dérivé » : tout logiciel original réalisé par un
   licencié, autre que le logiciel ou un logiciel modifié, qui produit ou
   reproduit la totalité ou une partie importante du logiciel;

   « logiciel modifié » : toute modification par un licencié de l'un des
   fichiers source du logiciel ou encore tout nouveau fichier source qui
   incorpore le logiciel ou une partie importante de ce dernier.

   (( 3. ))??
   Licence de droit d'auteur

   Sous réserve des termes de la licence, le concédant accorde au licencié
   une licence non exclusive et libre de redevances lui permettant d'exercer les
   droits suivants sur le logiciel :

      (( 1  ))??
      Produire ou reproduire la totalité ou une partie importante;

      (( 2  ))??
      Exécuter ou représenter la totalité ou une partie importante en
      public;

      (( 3  ))??
      Publier la totalité ou une partie importante;

      (( 4  ))??
      Sous-licencier sous une autre licence libre, approuvée ou certifiée par
      la Free Software Foundation ou l'Open Source Initiative.

   Cette licence est accordée sans limite territoriale et sans limite de
   temps.

   L'exercice complet de ces droits est sujet à la distribution par le
   concédant du code source du logiciel, lequel doit être sous une forme
   permettant d'y apporter des modifications. Le concédant peut aussi
   distribuer le logiciel accompagné d'une offre de distribuer le code source
   du logiciel, sans frais supplémentaires, autres que ceux raisonnables afin
   de permettre la livraison du code source. Cette offre doit être valide
   pendant une durée raisonnable.

   (( 4. ))??
   Distribution

   Le licencié peut distribuer des copies du logiciel, d'un logiciel modifié
   ou dérivé, sous réserve de respecter les conditions suivantes :

      (( 1  ))??
      Le logiciel doit être accompagné d'un exemplaire de cette licence;

      (( 2  ))??
      Si le logiciel a été modifié, le licencié doit en faire la mention, de
      préférence dans chacun des fichiers modifiés dont la nature permet une
      telle mention;

      (( 3  ))??
      Les étiquettes ou mentions faisant état des droits d'auteur, des marques
      de commerce, des garanties ou de la paternité concernant le logiciel ne
      doivent pas être modifiées ou supprimées, à moins que ces étiquettes
      ou mentions ne soient inapplicables à un logiciel modifié ou dérivé
      donné.

   (( 5. ))??

   Contributions

   Sous réserve d'une entente distincte, toute contribution soumise par un
   contributeur au concédant pour inclusion dans le logiciel sera soumise aux
   termes de cette licence.

   (( 6. ))??
   Marques de commerce

   La licence n'accorde aucune permission particulière qui permettrait
   d'utiliser les marques de commerce du concédant, autre que celle requise
   permettant d'identifier la provenance du logiciel.

   (( 7. ))??
   Garanties

   Sauf mention contraire, le concédant distribue le logiciel sans aucune
   garantie, aux risques et périls de l'acquéreur de la copie du logiciel, et
   ce, sans assurer que le logiciel puisse répondre à un besoin particulier ou
   puisse donner un résultat quelconque.

   Sans lier le concédant d'une quelconque manière, rien n'empêche un
   licencié d'offrir ou d'exclure des garanties ou du support.

   (( 8. ))??
   Responsabilité

   Le licencié est responsable de tout préjudice résultant de l'exercice des
   droits accordés par la licence.

   Le concédant ne saurait être tenu responsable de dommages subis par le
   licencié ou par des tiers, pour quelque cause que ce soit en lien avec la
   licence et les droits qui y sont accordés.

   (( 9. ))??
   Résiliation

   La présente licence est automatiquement résiliée dès que les droits qui y
   sont accordés ne sont pas exercés conformément aux termes qui y sont
   stipulés.

   Toutefois, si le défaut est corrigé dans un délai de 30 jours de sa prise
   de connaissance par la personne en défaut, et qu'il s'agit du premier
   défaut, la licence est accordée de nouveau.

   Pour tout défaut subséquent, le consentement exprès du concédant est
   nécessaire afin que la licence soit accordée de nouveau.

   (( 10. ))??
   Version de la licence

   Le Centre de services partagés du Québec, ses ayants cause ou toute
   personne qu'il désigne, peuvent diffuser des versions révisées ou
   modifiées de cette licence. Chaque version recevra un numéro unique. Si un
   logiciel est déjà soumis aux termes d'une version spécifique, c'est
   seulement cette version qui liera les parties à la licence.

   Le concédant peut aussi choisir de concéder la licence sous la version
   actuelle ou toute version ultérieure, auquel cas le licencié peut choisir
   sous quelle version la licence lui est accordée.

   (( 11. ))??
   Divers

   Dans la mesure où le concédant est un ministère, un organisme public ou
   une personne morale de droit public, créés en vertu d'une loi de
   l'Assemblée nationale du Québec, la licence est régie par le droit
   applicable au Québec et en cas de contestation, les tribunaux du Québec
   seront seuls compétents.

   La présente licence peut être distribuée sans conditions particulières.
   Toutefois, une version modifiée doit être distribuée sous un nom
   différent. Toute référence au Centre de services partagés du Québec, et,
   le cas échéant, ses ayant cause, doit être retirée, autre que celle
   permettant d'identifier la provenance de la licence.
`
const license_LiLiQ_R_1_1_lre = `//**
Licence Libre du Québec – Réciprocité version 1.1
https://spdx.org/licenses/LiLiQ-R-1.1.json
https://www.forge.gouv.qc.ca/participez/licence-logicielle/licence-libre-du-quebec-liliq-en-francais/licence-libre-du-quebec-reciprocite-liliq-r-v1-1/
http://opensource.org/licenses/LiLiQ-R-1.1
**//

(( Licence Libre du Québec – Réciprocité (LiLiQ-R)

Version 1.1 ))??

   (( 1. ))??
   Préambule

   Cette licence s'applique à tout logiciel distribué dont le titulaire du
   droit d'auteur précise qu'il est sujet aux termes de la Licence Libre du
   Québec – Réciprocité (LiLiQ-R) (ci-après appelée la « licence »).

   (( 2. ))??
   Définitions

   Dans la présente licence, à moins que le contexte n'indique un sens
   différent, on entend par:

   « concédant » : le titulaire du droit d'auteur sur le logiciel, ou
   toute personne dûment autorisée par ce dernier à accorder la présente
   licence;

   « contributeur » : le titulaire du droit d'auteur ou toute personne
   autorisée par ce dernier à soumettre au concédant une contribution. Un
   contributeur dont sa contribution est incorporée au logiciel est considéré
   comme un concédant en regard de sa contribution;

   « contribution » : tout logiciel original, ou partie de logiciel
   original soumis et destiné à être incorporé dans le logiciel;

   « distribution » : le fait de délivrer une copie du logiciel;

   « licencié » : toute personne qui possède une copie du logiciel et qui
   exerce les droits concédés par la licence;

   « logiciel » : une œuvre protégée par le droit d'auteur, telle qu'un
   programme d'ordinateur et sa documentation, pour laquelle le titulaire du
   droit d'auteur a précisé qu'elle est sujette aux termes de la présente
   licence;

   « logiciel dérivé » : tout logiciel original réalisé par un
   licencié, autre que le logiciel ou un logiciel modifié, qui produit ou
   reproduit la totalité ou une partie importante du logiciel;

   « logiciel modifié » : toute modification par un licencié de l'un des
   fichiers source du logiciel ou encore tout nouveau fichier source qui
   incorpore le logiciel ou une partie importante de ce dernier.

   (( 3. ))??
   Licence de droit d'auteur

   Sous réserve des termes de la licence, le concédant accorde au licencié
   une licence non exclusive et libre de redevances lui permettant d'exercer les
   droits suivants sur le logiciel :

      (( 1  ))??
      Produire ou reproduire la totalité ou une partie importante;

      (( 2  ))??
      Exécuter ou représenter la totalité ou une partie importante en
      public;

      (( 3  ))??
      Publier la totalité ou une partie importante.

   Cette licence est accordée sans limite territoriale et sans limite de
   temps.

   L'exercice complet de ces droits est sujet à la distribution par le
   concédant du code source du logiciel, lequel doit être sous une forme
   permettant d'y apporter des modifications. Le concédant peut aussi
   distribuer le logiciel accompagné d'une offre de distribuer le code source
   du logiciel, sans frais supplémentaires, autres que ceux raisonnables afin
   de permettre la livraison du code source. Cette offre doit être valide
   pendant une durée raisonnable.

   (( 4. ))??
   Distribution

   Le licencié peut distribuer des copies du logiciel, d'un logiciel modifié
   ou dérivé, sous réserve de respecter les conditions suivantes :

      (( 1  ))??
      Le logiciel doit être accompagné d'un exemplaire de cette licence;

      (( 2  ))??
      Si le logiciel a été modifié, le licencié doit en faire la mention, de
      préférence dans chacun des fichiers modifiés dont la nature permet une
      telle mention;

      (( 3  ))??
      Les étiquettes ou mentions faisant état des droits d'auteur, des marques
      de commerce, des garanties ou de la paternité concernant le logiciel ne
      doivent pas être modifiées ou supprimées, à moins que ces étiquettes
      ou mentions ne soient inapplicables à un logiciel modifié ou dérivé
      donné.

   (( 4.1. ))??
   Réciprocité

   Chaque fois que le licencié distribue le logiciel, le concédant offre au
   récipiendaire une concession sur le logiciel selon les termes de la
   présente licence. Le licencié doit offrir une concession selon les termes
   de la présente licence pour tout logiciel modifié qu'il distribue.

   Chaque fois que le licencié distribue le logiciel ou un logiciel modifié,
   ce dernier doit assumer l'obligation d'en distribuer le code source, de la
   manière prévue au troisième alinéa de l'article 3.

   (( 4.2. ))??
   Compatibilité

   Dans la mesure où le licencié souhaite distribuer un logiciel modifié
   combiné à un logiciel assujetti à une licence compatible, mais dont il ne
   serait pas possible d'en respecter les termes, le concédant offre, en plus
   de la présente concession, une concession selon les termes de cette licence
   compatible.

   Un licencié qui est titulaire exclusif du droit d'auteur sur le logiciel
   assujetti à une licence compatible ne peut pas se prévaloir de cette offre.
   Il en est de même pour toute autre personne dûment autorisée à
   sous-licencier par le titulaire exclusif du droit d'auteur sur le logiciel
   assujetti à une licence compatible.

   Est considérée comme une licence compatible toute licence libre approuvée
   ou certifiée par la Free Software Foundation ou l'Open Source Initiative,
   dont le niveau de réciprocité est comparable ou supérieur à celui de la
   présente licence, sans toutefois être moindre, notamment :

      (( 1  ))??
      Common Development and Distribution License (CDDL-1.0)

      (( 2  ))??
      Common Public License Version 1.0 (CPL-1.0)

      (( 3  ))??
      Contrat de licence de logiciel libre CeCILL, version 2.1 (CECILL-2.1)

      (( 4  ))??
      Contrat de licence de logiciel libre CeCILL-C (CECILL-C)

      (( 5  ))??
      Eclipse Public License - v 1.0 (EPL-1.0)

      (( 6  ))??
      European Union Public License, version 1.1 (EUPL v. 1.1)

      (( 7  ))??
      Licence Libre du Québec – Réciprocité forte version 1.1 (LiLiQ-R+
      1.1)

      (( 8  ))??
      GNU General Public License Version 2 (GNU GPLv2)

      (( 9  ))??
      GNU General Public License Version 3 (GNU GPLv3)

      (( 10  ))??
      GNU Lesser General Public License Version 2.1 (GNU LGPLv2.1)

      (( 11  ))??
      GNU Lesser General Public License Version 3 (GNU LGPLv3)

      (( 12  ))??
      Mozilla Public License Version 2.0 (MPL-2.0)

   (( 5. ))??
   Contributions

   Sous réserve d'une entente distincte, toute contribution soumise par un
   contributeur au concédant pour inclusion dans le logiciel sera soumise aux
   termes de cette licence.

   (( 6. ))??
   Marques de commerce

   La licence n'accorde aucune permission particulière qui permettrait
   d'utiliser les marques de commerce du concédant, autre que celle requise
   permettant d'identifier la provenance du logiciel.

   (( 7. ))??
   Garanties

   Sauf mention contraire, le concédant distribue le logiciel sans aucune
   garantie, aux risques et périls de l'acquéreur de la copie du logiciel, et
   ce, sans assurer que le logiciel puisse répondre à un besoin particulier ou
   puisse donner un résultat quelconque.

   Sans lier le concédant d'une quelconque manière, rien n'empêche un
   licencié d'offrir ou d'exclure des garanties ou du support.

   (( 8. ))??
   Responsabilité

   Le licencié est responsable de tout préjudice résultant de l'exercice des
   droits accordés par la licence.

   Le concédant ne saurait être tenu responsable du préjudice subi par le
   licencié ou par des tiers, pour quelque cause que ce soit en lien avec la
   licence et les droits qui y sont accordés.

   (( 9. ))??
   Résiliation

   La présente licence est résiliée de plein droit dès que les droits qui y
   sont accordés ne sont pas exercés conformément aux termes qui y sont
   stipulés.

   Toutefois, si le défaut est corrigé dans un délai de 30 jours de sa prise
   de connaissance par la personne en défaut, et qu'il s'agit du premier
   défaut, la licence est accordée de nouveau.

   Pour tout défaut subséquent, le consentement exprès du concédant est
   nécessaire afin que la licence soit accordée de nouveau.

   (( 10. ))??
   Version de la licence

   Le Centre de services partagés du Québec, ses ayants cause ou toute
   personne qu'il désigne, peuvent diffuser des versions révisées ou
   modifiées de cette licence. Chaque version recevra un numéro unique. Si un
   logiciel est déjà soumis aux termes d'une version spécifique, c'est
   seulement cette version qui liera les parties à la licence.

   Le concédant peut aussi choisir de concéder la licence sous la version
   actuelle ou toute version ultérieure, auquel cas le licencié peut choisir
   sous quelle version la licence lui est accordée.

   (( 11. ))??
   Divers

   Dans la mesure où le concédant est un ministère, un organisme public ou
   une personne morale de droit public, créés en vertu d'une loi de
   l'Assemblée nationale du Québec, la licence est régie par le droit
   applicable au Québec et en cas de contestation, les tribunaux du Québec
   seront seuls compétents.

   La présente licence peut être distribuée sans conditions particulières.
   Toutefois, une version modifiée doit être distribuée sous un nom
   différent. Toute référence au Centre de services partagés du Québec, et,
   le cas échéant, ses ayant droit, doit être retirée, autre que celle
   permettant d'identifier la provenance de la licence.
`
const license_LiLiQ_Rplus_1_1_lre = `//**
Licence Libre du Québec – Réciprocité forte version 1.1
https://spdx.org/licenses/LiLiQ-Rplus-1.1.json
https://www.forge.gouv.qc.ca/participez/licence-logicielle/licence-libre-du-quebec-liliq-en-francais/licence-libre-du-quebec-reciprocite-forte-liliq-r-v1-1/
http://opensource.org/licenses/LiLiQ-Rplus-1.1
**//

(( Licence Libre du Québec – Réciprocité forte (LiLiQ-R+)

Version 1.1 ))??

   (( 1. ))??
   Préambule

   Cette licence s'applique à tout logiciel distribué dont le titulaire du
   droit d'auteur précise qu'il est sujet aux termes de la Licence Libre du
   Québec – Réciprocité forte (LiLiQ-R+) (ci-après appelée la
   « licence »).

   (( 2. ))??
   Définitions

   Dans la présente licence, à moins que le contexte n'indique un sens
   différent, on entend par:

   « concédant » : le titulaire du droit d'auteur sur le logiciel, ou toute
   personne dûment autorisée par ce dernier à accorder la présente licence;

   « contributeur » : le titulaire du droit d'auteur ou toute personne
   autorisée par ce dernier à soumettre au concédant une contribution. Un
   contributeur dont sa contribution est incorporée au logiciel est considéré
   comme un concédant en regard de sa contribution;

   « contribution » : tout logiciel original, ou partie de logiciel original
   soumis et destiné à être incorporé dans le logiciel;

   « distribution » : le fait de délivrer une copie du logiciel;

   « licencié » : toute personne qui possède une copie du logiciel et qui
   exerce les droits concédés par la licence;

   « logiciel » : une œuvre protégée par le droit d'auteur, telle qu'un
   programme d'ordinateur et sa documentation, pour laquelle le titulaire du
   droit d'auteur a précisé qu'elle est sujette aux termes de la présente
   licence;

   « logiciel dérivé » : tout logiciel original réalisé par un licencié,
   autre que le logiciel ou un logiciel modifié, qui produit ou reproduit la
   totalité ou une partie importante du logiciel;

   « logiciel modifié » : toute modification par un licencié de l'un des
   fichiers source du logiciel ou encore tout nouveau fichier source qui
   incorpore le logiciel ou une partie importante de ce dernier.

   (( 3. ))??
   Licence de droit d'auteur

   Sous réserve des termes de la licence, le concédant accorde au licencié
   une licence non exclusive et libre de redevances lui permettant d'exercer les
   droits suivants sur le logiciel:

      (( 1 ))??
      Produire ou reproduire la totalité ou une partie importante;

      (( 2 ))??
      Exécuter ou représenter la totalité ou une partie importante en
      public;

      (( 3 ))??
      Publier la totalité ou une partie importante.

   Cette licence est accordée sans limite territoriale et sans limite de
   temps.

   L'exercice complet de ces droits est sujet à la distribution par le
   concédant du code source du logiciel, lequel doit être sous une forme
   permettant d'y apporter des modifications. Le concédant peut aussi
   distribuer le logiciel accompagné d'une offre de distribuer le code source
   du logiciel, sans frais supplémentaires, autres que ceux raisonnables afin
   de permettre la livraison du code source. Cette offre doit être valide
   pendant une durée raisonnable.

   (( 4. ))??
   Distribution

   Le licencié peut distribuer des copies du logiciel, d'un logiciel modifié
   ou dérivé, sous réserve de respecter les conditions suivantes :

      (( 1 ))??
      Le logiciel doit être accompagné d'un exemplaire de cette licence;

      (( 2 ))??
      Si le logiciel a été modifié, le licencié doit en faire la mention, de
      préférence dans chacun des fichiers modifiés dont la nature permet une
      telle mention;

      (( 3 ))??
      Les étiquettes ou mentions faisant état des droits d'auteur, des marques
      de commerce, des garanties ou de la paternité concernant le logiciel ne
      doivent pas être modifiées ou supprimées, à moins que ces étiquettes
      ou mentions ne soient inapplicables à un logiciel modifié ou dérivé
      donné.

   (( 4.1. ))??
   Réciprocité

   Chaque fois que le licencié distribue le logiciel, le concédant offre au
   récipiendaire une concession sur le logiciel selon les termes de la
   présente licence. Le licencié doit offrir une concession selon les termes
   de la présente licence pour tout logiciel modifié ou dérivé qu'il
   distribue.

   Chaque fois que le licencié distribue le logiciel, un logiciel modifié, ou
   un logiciel dérivé, ce dernier doit assumer l'obligation d'en distribuer le
   code source, de la manière prévue au troisième alinéa de l'article 3.

   (( 4.2. ))??
   Compatibilité

   Dans la mesure où le licencié souhaite distribuer un logiciel modifié ou
   dérivé combiné à un logiciel assujetti à une licence compatible, mais
   dont il ne serait pas possible d'en respecter les termes, le concédant
   offre, en plus de la présente concession, une concession selon les termes de
   cette licence compatible.

   Un licencié qui est titulaire exclusif du droit d'auteur sur le logiciel
   assujetti à une licence compatible ne peut pas se prévaloir de cette offre.
   Il en est de même pour toute autre personne dûment autorisée à
   sous-licencier par le titulaire exclusif du droit d'auteur sur le logiciel
   assujetti à une licence compatible.

   Est considérée comme une licence compatible toute licence libre approuvée
   ou certifiée par la Free Software Foundation ou l'Open Source Initiative,
   dont le niveau de réciprocité est comparable à celui de la présente
   licence, sans toutefois être moindre, notamment:

      (( 1 ))??
      Common Public License Version 1.0 (CPL-1.0)

      (( 2 ))??
      Contrat de licence de logiciel libre CeCILL, version 2.1 (CECILL-2.1)

      (( 3 ))??
      Eclipse Public License - v 1.0 (EPL-1.0)

      (( 4 ))??
      European Union Public License, version 1.1 (EUPL v. 1.1)

      (( 5 ))??
      GNU General Public License Version 2 (GNU GPLv2)

      (( 6 ))??
      GNU General Public License Version 3 (GNU GPLv3)

   (( 5. ))??
   Contributions

   Sous réserve d'une entente distincte, toute contribution soumise par un
   contributeur au concédant pour inclusion dans le logiciel sera soumise aux
   termes de cette licence.

   (( 6. ))??
   Marques de commerce

   La licence n'accorde aucune permission particulière qui permettrait
   d'utiliser les marques de commerce du concédant, autre que celle requise
   permettant d'identifier la provenance du logiciel.

   (( 7. ))??
   Garanties

   Sauf mention contraire, le concédant distribue le logiciel sans aucune
   garantie, aux risques et périls de l'acquéreur de la copie du logiciel, et
   ce, sans assurer que le logiciel puisse répondre à un besoin particulier ou
   puisse donner un résultat quelconque.

   Sans lier le concédant d'une quelconque manière, rien n'empêche un
   licencié d'offrir ou d'exclure des garanties ou du support.

   (( 8. ))??
   Responsabilité

   Le licencié est responsable de tout préjudice résultant de l'exercice des
   droits accordés par la licence.

   Le concédant ne saurait être tenu responsable du préjudice subi par le
   licencié ou par des tiers, pour quelque cause que ce soit en lien avec la
   licence et les droits qui y sont accordés.

   (( 9. ))??
   Résiliation

   La présente licence est résiliée de plein droit dès que les droits qui y
   sont accordés ne sont pas exercés conformément aux termes qui y sont
   stipulés.

   Toutefois, si le défaut est corrigé dans un délai de 30 jours de sa prise
   de connaissance par la personne en défaut, et qu'il s'agit du premier
   défaut, la licence est accordée de nouveau.

   Pour tout défaut subséquent, le consentement exprès du concédant est
   nécessaire afin que la licence soit accordée de nouveau.

   (( 10. ))??
   Version de la licence

   Le Centre de services partagés du Québec, ses ayants cause ou toute
   personne qu'il désigne, peuvent diffuser des versions révisées ou
   modifiées de cette licence. Chaque version recevra un numéro unique. Si un
   logiciel est déjà soumis aux termes d'une version spécifique, c'est
   seulement cette version qui liera les parties à la licence.

   Le concédant peut aussi choisir de concéder la licence sous la version
   actuelle ou toute version ultérieure, auquel cas le licencié peut choisir
   sous quelle version la licence lui est accordée.

   (( 11. ))??
   Divers

   Dans la mesure où le concédant est un ministère, un organisme public ou
   une personne morale de droit public, créés en vertu d'une loi de
   l'Assemblée nationale du Québec, la licence est régie par le droit
   applicable au Québec et en cas de contestation, les tribunaux du Québec
   seront seuls compétents.

   La présente licence peut être distribuée sans conditions particulières.
   Toutefois, une version modifiée doit être distribuée sous un nom
   différent. Toute référence au Centre de services partagés du Québec, et,
   le cas échéant, ses ayant cause, doit être retirée, autre que celle
   permettant d'identifier la provenance de la licence.
`
const license_Libpng_lre = `//**
libpng License
https://spdx.org/licenses/Libpng.json
http://www.libpng.org/pub/png/src/libpng-LICENSE.txt
**//

This copy of the libpng notices is provided for your convenience. In case of any
discrepancy between this copy and the notices in the file png.h that is included
in the libpng distribution, the latter shall prevail.

COPYRIGHT NOTICE, DISCLAIMER, and LICENSE:

If you modify libpng you may insert additional notices immediately following
this sentence.

This code is released under the libpng license.

libpng versions 1.2.6, August 15, 2004, through 1.4.5, December 9, 2010, are
Copyright (c) 2004, 2006-2010 Glenn Randers-Pehrson, and are distributed
according to the same disclaimer and license as libpng-1.2.5 with the following
individual added to the list of Contributing Authors

Cosmin Truta

libpng versions 1.0.7, July 1, 2000, through 1.2.5 - October 3, 2002, are

Copyright (c) 2000-2002 Glenn Randers-Pehrson, and are distributed according to
the same disclaimer and license as libpng-1.0.6 with the following individuals
added to the list of Contributing Authors

Simon-Pierre Cadieux

Eric S. Raymond

Gilles Vollant

and with the following additions to the disclaimer:

There is no warranty against interference with your enjoyment of the library or
against infringement. There is no warranty that our efforts or the library will
fulfill any of your particular purposes or needs. This library is provided with
all faults, and the entire risk of satisfactory quality, performance, accuracy,
and effort is with the user.

libpng versions 0.97, January 1998, through 1.0.6, March 20, 2000, are

Copyright (c) 1998, 1999 Glenn Randers-Pehrson, and are distributed according to
the same disclaimer and license as libpng-0.96, with the following individuals
added to the list of Contributing Authors:

Tom Lane

Glenn Randers-Pehrson

Willem van Schaik

libpng versions 0.89, June 1996, through 0.96, May 1997, are

Copyright (c) 1996, 1997 Andreas Digger

Distributed according to the same disclaimer and license as libpng-0.88, with
the following individuals added to the list of Contributing Authors:

John Bowler

Kevin Bracey

Sam Bushell

Magnus Holmgren

Greg Roelofs

Tom Tanner

libpng versions 0.5, May 1995, through 0.88, January 1996, are

Copyright (c) 1995, 1996 Guy Eric Schalnat, Group 42, Inc.

For the purposes of this copyright and license, "Contributing Authors" is
defined as the following set of individuals:

Andreas Dilger

Dave Martindale

Guy Eric Schalnat

Paul Schmidt

Tim Wegner

The PNG Reference Library is supplied "AS IS". The Contributing Authors and
Group 42, Inc. disclaim all warranties, expressed or implied, including, without
limitation, the warranties of merchantability and of fitness for any purpose.
The Contributing Authors and Group 42, Inc. assume no liability for direct,
indirect, incidental, special, exemplary, or consequential damages, which may
result from the use of the PNG Reference Library, even if advised of the
possibility of such damage.

Permission is hereby granted to use, copy, modify, and distribute this source
code, or portions hereof, for any purpose, without fee, subject to the following
restrictions:

   (( 1. ))??
   The origin of this source code must not be misrepresented.

   (( 2. ))??
   Altered versions must be plainly marked as such and must not be
   misrepresented as being the original source.

   (( 3. ))??
   This Copyright notice may not be removed or altered from any source or
   altered source distribution.

The Contributing Authors and Group 42, Inc. specifically permit, without fee,
and encourage the use of this source code as a component to supporting the PNG
file format in commercial products. If you use this source code in a product,
acknowledgment is not required but would be appreciated.

A "png_get_copyright" function is available, for convenient use in "about" boxes
and the like:

printf("%s",png_get_copyright(NULL);

Also, the PNG logo (in PNG format, of course) is supplied in the files
"pngbar.png" and "pngbar.jpg (88x31) and "pngnow.png" (98x31).

Libpng is OSI Certified Open Source Software. OSI Certified Open Source is a
certification mark of the Open Source Initiative.

Glenn Randers-Pehrson

glennrp at users.sourceforge.net

December 9, 2010
`
const license_Linux_OpenIB_lre = `//**
Linux Kernel Variant of OpenIB.org license
https://spdx.org/licenses/Linux-OpenIB.json
https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git/tree/drivers/infiniband/core/sa.h
**//

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   -
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   -
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
`
const license_MIT_lre = `








//**
MIT License
https://spdx.org/licenses/MIT.json
https://opensource.org/licenses/MIT
https://fedoraproject.org/wiki/Licensing:MIT
**//
(( MIT License))??
//**Copyright**//


Permission is hereby granted,
((free of charge))??
to any person obtaining a copy of
__7__ //** (allow parenthetical descriptions) **//
((and associated documentation files))??
the
((Software || Materials))
to deal in the
((Software || Materials))
((under the copyrights))??
((without restriction))??
including
((without limitation))??
the rights
((to))??
use, copy, modify, merge, publish, distribute,
((sublicense))??
and/or
((sell))??
((modified))??
copies of the
((Software || Materials))
and to permit persons to whom the
((Software is || Materials are))
furnished to do so,

subject to
((the following || all))
conditions:


__1__
((
	The above
	((copyright || authorship))
	notice
	(( and this permission notice
		((including the next paragraph))??
	|| as well as this permission notice
	|| this permission notice, and the below disclaimer
	|| and every other copyright notice found in this software,
		and all the attributions in every file, and this permission notice
	|| and this permission notice (or reference to this permission notice) ))
||
	This permission notice
))
((must || shall))
be included in all
copies
or
((substantial || any))??
portions of the
((Software || Materials))


((DISCLAIMER))??
((2.))??

THE
((SOFTWARE || MATERIALS))
IS PROVIDED "AS IS",
WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO
THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE
AND NONINFRINGEMENT.
IN NO EVENT
((SHALL || WILL))
__5__ BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT,
((TORT || FART))
OR OTHERWISE,
ARISING FROM, OUT OF OR
((IN || I))
CONNECTION WITH
((TE || THE))
((SOFTWARE || MATERIALS))
OR THE USE OR OTHER DEALINGS IN
((THE
	((SOFTWARE || MATERIALS))
))??









`
const license_MIT_0_lre = `
//**
MIT No Attribution
https://spdx.org/licenses/MIT-0.json
https://github.com/aws/mit-0
https://romanrm.net/mit-zero
https://github.com/awsdocs/aws-cloud9-user-guide/blob/master/LICENSE-SAMPLECODE
**//

Permission is hereby granted,
((free of charge))??
to any person obtaining a copy of
__7__ //** (allow parenthetical descriptions) **//
((and associated documentation files))??
the
((Software || Materials))
to deal in the
((Software || Materials))
((under the copyrights))??
((without restriction))??
including
((without limitation))??
the rights
((to))??
use, copy, modify, merge, publish, distribute,
((sublicense))??
and/or
((sell))??
((modified))??
copies of the
((Software || Materials))
and to permit persons to whom the
((Software is || Materials are))
furnished to do so,

((subject to the following conditions))??

((DISCLAIMER))??
((2.))??

THE
((SOFTWARE || MATERIALS))
IS PROVIDED "AS IS",
WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO
THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE
AND NONINFRINGEMENT.
IN NO EVENT
((SHALL || WILL))
__5__ BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT,
((TORT || FART))
OR OTHERWISE,
ARISING FROM, OUT OF OR
((IN || I))
CONNECTION WITH
((TE || THE))
((SOFTWARE || MATERIALS))
OR THE USE OR OTHER DEALINGS IN
((THE
	((SOFTWARE || MATERIALS))
))??

`
const license_MIT_CMU_lre = `//**
CMU License
https://spdx.org/licenses/MIT-CMU.json
https://fedoraproject.org/wiki/Licensing:MIT?rd=Licensing/MIT#CMU_Style
https://github.com/python-pillow/Pillow/blob/fffb426092c8db24a5f4b6df243a8a3c01fb63cd/LICENSE
**//

//** Copyright **//

(( By obtaining, using, and/or copying this software and/or its associated
documentation, you agree that you have read, understood, and will comply with
the following terms and conditions: ))??

Permission to use, copy, modifyand distribute this software and its
(( associated ))??
documentation for any purpose and without fee is hereby granted, provided that
the above copyright notice appears in all copiesand that both that copyright
notice and this permission notice appear in supporting documentation, and that
the name of
//** the copyright holder **//
__5__
not be used in advertising or publicity pertaining to distribution of the
software without specificwritten permission.

//** THE COPYRIGHT HOLDER **//
__5__
DISCLAIMS
ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
//** THE COPYRIGHT HOLDER **//
__5__
BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM
(( THE ))??
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
`
const license_MIT_NoAd_lre = `


Permission is hereby granted,
((free of charge))??
to any person obtaining a copy of
__7__ //** (allow parenthetical descriptions) **//
((and associated documentation files))??
the
((Software || Materials))
to deal in the
((Software || Materials))
((under the copyrights))??
((without restriction))??
including
((without limitation))??
the rights
((to))??
use, copy, modify, merge, publish, distribute,
((sublicense))??
and/or
((sell))??
((modified))??
copies of the
((Software || Materials))
and to permit persons to whom the
((Software is || Materials are))
furnished to do so,

subject to
((the following || all))
conditions:


__1__
((
	The above
	((copyright || authorship))
	notice
	(( and this permission notice
		((including the next paragraph))??
	|| as well as this permission notice
	|| this permission notice, and the below disclaimer
	|| and every other copyright notice found in this software,
		and all the attributions in every file, and this permission notice
	|| and this permission notice (or reference to this permission notice) ))
||
	This permission notice
))
((must || shall))
be included in all
copies
or
((substantial || any))??
portions of the
((Software || Materials))


Except as contained in this notice, the
((name || names))
__10__
shall not be used in advertising or otherwise
to promote the sale, use or other dealings in this Software
without prior written authorization
((
	((of || from))
	__10__
))??


((DISCLAIMER))??
((2.))??

THE
((SOFTWARE || MATERIALS))
IS PROVIDED "AS IS",
WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO
THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE
AND NONINFRINGEMENT.
IN NO EVENT
((SHALL || WILL))
__5__ BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT,
((TORT || FART))
OR OTHERWISE,
ARISING FROM, OUT OF OR
((IN || I))
CONNECTION WITH
((TE || THE))
((SOFTWARE || MATERIALS))
OR THE USE OR OTHER DEALINGS IN
((THE
	((SOFTWARE || MATERIALS))
))??

`
const license_MIT_advertising_lre = `//**
Enlightenment License (e16)
https://spdx.org/licenses/MIT-advertising.json
https://fedoraproject.org/wiki/Licensing/MIT_With_Advertising
**//

//** Copyright **//

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies of the Software, its documentation and marketing & publicity materials,
and acknowledgment shall be given in the documentation, materials and software
packages that this Software was used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
`
const license_MIT_enna_lre = `//**
enna License
https://spdx.org/licenses/MIT-enna.json
https://fedoraproject.org/wiki/Licensing/MIT#enna
**//

//** Copyright **//

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies of the Software and its Copyright notices. In addition publicly
documented acknowledgment must be given that this software has been used if no
source code of this software is made available publicly. This includes
acknowledgments in either Copyright notices, Manuals, Publicity and Marketing
documents or any documentation provided with any product containing this
software. This License does not apply to any software that links to the
libraries provided by this software (statically or dynamically), but only to the
software provided.

Please see the COPYING.PLAIN for a plain-english explanation of this notice and
it's intent.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
`
const license_MIT_feh_lre = `//**
feh License
https://spdx.org/licenses/MIT-feh.json
https://fedoraproject.org/wiki/Licensing/MIT#feh
**//

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies of the Software and its documentation and acknowledgment shall be given
in the documentation and software packages that this Software was used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
`
const license_MITNFA_lre = `
//**
MIT +no-false-attribs license
https://spdx.org/licenses/MITNFA.json
https://fedoraproject.org/wiki/Licensing/MITNFA
**//


Permission is hereby granted,
((free of charge))??
to any person obtaining a copy of
__7__ //** (allow parenthetical descriptions) **//
((and associated documentation files))??
the
((Software || Materials))
to deal in the
((Software || Materials))
((under the copyrights))??
((without restriction))??
including
((without limitation))??
the rights
((to))??
use, copy, modify, merge, publish, distribute,
((sublicense))??
and/or
((sell))??
((modified))??
copies of the
((Software || Materials))
and to permit persons to whom the
((Software is || Materials are))
furnished to do so,

subject to
((the following || all))
conditions:


__1__
((
	The above
	((copyright || authorship))
	notice
	(( and this permission notice
		((including the next paragraph))??
	|| as well as this permission notice
	|| this permission notice, and the below disclaimer
	|| and every other copyright notice found in this software,
		and all the attributions in every file, and this permission notice
	|| and this permission notice (or reference to this permission notice) ))
||
	This permission notice
))
((must || shall))
be included in all
copies
or
((substantial || any))??
portions of the
((Software || Materials))


Distributions of all or part of the Software intended to be used by the
recipients as they would use the unmodified Software, containing modifications
that substantially alter, remove, or disable functionality of the Software,
outside of the documented configuration mechanisms provided by the Software,
shall be modified such that the Original Author's bug reporting email addresses
and urls are either replaced with the contact information of the parties
responsible for the changes, or removed entirely.


((DISCLAIMER))??
((2.))??

THE
((SOFTWARE || MATERIALS))
IS PROVIDED "AS IS",
WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO
THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE
AND NONINFRINGEMENT.
IN NO EVENT
((SHALL || WILL))
__5__ BE LIABLE
FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT,
((TORT || FART))
OR OTHERWISE,
ARISING FROM, OUT OF OR
((IN || I))
CONNECTION WITH
((TE || THE))
((SOFTWARE || MATERIALS))
OR THE USE OR OTHER DEALINGS IN
((THE
	((SOFTWARE || MATERIALS))
))??

`
const license_MPL_1_0_lre = `//**
Mozilla Public License 1.0
https://spdx.org/licenses/MPL-1.0.json
http://www.mozilla.org/MPL/MPL-1.0.html
https://opensource.org/licenses/MPL-1.0
**//

((

The contents of this file are subject to the Mozilla Public License
Version 1.0 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http:/www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

||

(( MOZILLA PUBLIC LICENSE

Version 1.0 ))??

   (( 1. ))??
   Definitions.

      (( 1.1. ))??
      "Contributor" means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      "Covered Code" means the Original Code or Modifications or the combination
      of the Original Code and Modifications, in each case including portions
      thereof.

      (( 1.4. ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      "Executable" means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      "Initial Developer" means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7. ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.8. ))??
      "License" means this document.

      (( 1.9. ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10. ))??
      "Original Code" means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      (( 1.11. ))??
      "Source Code" means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or a list of source code differential
      comparisons against either the Original Code or another well known,
      available Covered Code of the Contributor's choice. The Source Code can be
      in a compressed or archival form, provided the appropriate decompression
      or de-archiving software is widely available for no charge.

      (( 1.12. ))??
      "You" means an individual or a legal entity exercising rights under, and
      complying with all of the terms of, this License or a future version of
      this License issued under Section 6.1. For legal entities, "You" includes
      any entity which controls, is controlled by, or is under common control
      with You. For purposes of this definition, "control" means (a) the power,
      direct or indirect, to cause the direction or management of such entity,
      whether by contract or otherwise, or (b) ownership of fifty percent (50%)
      or more of the outstanding shares or beneficial ownership of such entity.

   (( 2. ))??
   Source Code License.

      (( 2.1. ))??
      The Initial Developer Grant.

      The Initial Developer hereby grants You a world-wide, royalty-free,
      non-exclusive license, subject to third party intellectual property
      claims:

         (( (a) ))??
         to use, reproduce, modify, display, perform, sublicense and distribute
         the Original Code (or portions thereof) with or without Modifications,
         or as part of a Larger Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Initial
         Developer, to make, have made, use and sell ("Utilize") the Original
         Code (or portions thereof), but solely to the extent that any such
         patent is reasonably necessary to enable You to Utilize the Original
         Code (or portions thereof) and not to any greater extent that may be
         necessary to Utilize further Modifications or combinations.

      (( 2.2. ))??
      Contributor Grant.

      Each Contributor hereby grants You a world-wide, royalty-free,
      non-exclusive license, subject to third party intellectual property
      claims:

         (( (a) ))??
         to use, reproduce, modify, display, perform, sublicense and distribute
         the Modifications created by such Contributor (or portions thereof)
         either on an unmodified basis, with other Modifications, as Covered
         Code or as part of a Larger Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Contributor, to
         Utilize the Contributor Version (or portions thereof), but solely to
         the extent that any such patent is reasonably necessary to enable You
         to Utilize the Contributor Version (or portions thereof), and not to
         any greater extent that may be necessary to Utilize further
         Modifications or combinations.

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Application of License.

      The Modifications which You create or to which You contribute are governed
      by the terms of this License, including without limitation Section 2.2.
      The Source Code version of Covered Code may be distributed only under the
      terms of this License or a future version of this License released under
      Section 6.1, and You must include a copy of this License with every copy
      of the Source Code You distribute. You may not offer or impose any terms
      on any Source Code version that alters or restricts the applicable version
      of this License or the recipients' rights hereunder. However, You may
      include an additional document offering the additional rights described in
      Section 3.5.

      (( 3.2. ))??
      Availability of Source Code.

      Any Modification which You create or to which You contribute must be made
      available in Source Code form under the terms of this License either on
      the same media as an Executable version or via an accepted Electronic
      Distribution Mechanism to anyone to whom you made an Executable version
      available; and if made available via Electronic Distribution Mechanism,
      must remain available for at least twelve (12) months after the date it
      initially became available, or at least six (6) months after a subsequent
      version of that particular Modification has been made available to such
      recipients. You are responsible for ensuring that the Source Code version
      remains available even if the Electronic Distribution Mechanism is
      maintained by a third party.

      (( 3.3. ))??
      Description of Modifications.

      You must cause all Covered Code to which you contribute to contain a file
      documenting the changes You made to create that Covered Code and the date
      of any change. You must include a prominent statement that the
      Modification is derived, directly or indirectly, from Original Code
      provided by the Initial Developer and including the name of the Initial
      Developer in (a) the Source Code, and (b) in any notice in an Executable
      version or related documentation in which You describe the origin or
      ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters

         (( (a) ))??
         Third Party Claims.

         If You have knowledge that a party claims an intellectual property
         right in particular functionality or code (or its utilization under
         this License), you must include a text file with the source code
         distribution titled "LEGAL" which describes the claim and the party
         making the claim in sufficient detail that a recipient will know whom
         to contact. If you obtain such knowledge after You make Your
         Modification available as described in Section 3.2, You shall promptly
         modify the LEGAL file in all copies You make available thereafter and
         shall take other steps (such as notifying appropriate mailing lists or
         newsgroups) reasonably calculated to inform those who received the
         Covered Code that new knowledge has been obtained.

         (( (b) ))??
         Contributor APIs.

         If Your Modification is an application programming interface and You
         own or control patents which are reasonably necessary to implement that
         API, you must also include this information in the LEGAL file.

      (( 3.5. ))??
      Required Notices.

      You must duplicate the notice in Exhibit A in each file of the Source
      Code, and this License in any documentation for the Source Code, where You
      describe recipients' rights relating to Covered Code. If You created one
      or more Modification(s), You may add your name as a Contributor to the
      notice described in Exhibit A. If it is not possible to put such notice in
      a particular Source Code file due to its structure, then you must include
      such notice in a location (such as a relevant directory file) where a user
      would be likely to look for such a notice. You may choose to offer, and to
      charge a fee for, warranty, support, indemnity or liability obligations to
      one or more recipients of Covered Code. However, You may do so only on
      Your own behalf, and not on behalf of the Initial Developer or any
      Contributor. You must make it absolutely clear than any such warranty,
      support, indemnity or liability obligation is offered by You alone, and
      You hereby agree to indemnify the Initial Developer and every Contributor
      for any liability incurred by the Initial Developer or such Contributor as
      a result of warranty, support, indemnity or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Versions.

      You may distribute Covered Code in Executable form only if the
      requirements of Section 3.1-3.5 have been met for that Covered Code, and
      if You include a notice stating that the Source Code version of the
      Covered Code is available under the terms of this License, including a
      description of how and where You have fulfilled the obligations of Section
      3.2. The notice must be conspicuously included in any notice in an
      Executable version, related documentation or collateral in which You
      describe recipients' rights relating to the Covered Code. You may
      distribute the Executable version of Covered Code under a license of Your
      choice, which may contain terms different from this License, provided that
      You are in compliance with the terms of this License and that the license
      for the Executable version does not attempt to limit or alter the
      recipient's rights in the Source Code version from the rights set forth in
      this License. If You distribute the Executable version under a different
      license You must make it absolutely clear that any terms which differ from
      this License are offered by You alone, not by the Initial Developer or any
      Contributor. You hereby agree to indemnify the Initial Developer and every
      Contributor for any liability incurred by the Initial Developer or such
      Contributor as a result of any such terms You offer.

      (( 3.7. ))??
      Larger Works.

      You may create a Larger Work by combining Covered Code with other code not
      governed by the terms of this License and distribute the Larger Work as a
      single product. In such a case, You must make sure the requirements of
      this License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute or regulation
   then You must: (a) comply with the terms of this License to the maximum
   extent possible; and (b) describe the limitations and the code they affect.
   Such description must be included in the LEGAL file described in Section 3.4
   and must be included with all distributions of the Source Code. Except to the
   extent prohibited by statute or regulation, such description must be
   sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Application of this License.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A, and to related Covered Code.

   (( 6. ))??
   Versions of the License.

      (( 6.1. ))??
      New Versions.

      Netscape Communications Corporation ("Netscape") may publish revised
      and/or new versions of the License from time to time. Each version will be
      given a distinguishing version number.

      (( 6.2. ))??
      Effect of New Versions.

      Once Covered Code has been published under a particular version of the
      License, You may always continue to use it under the terms of that
      version. You may also choose to use such Covered Code under the terms of
      any subsequent version of the License published by Netscape. No one other
      than Netscape has the right to modify the terms applicable to Covered Code
      created under this License.

      (( 6.3. ))??
      Derivative Works.

      If you create or use a modified version of this License (which you may
      only do in order to apply it to code which is not already Covered Code
      governed by this License), you must (a) rename Your license so that the
      phrases "Mozilla", "MOZILLAPL", "MOZPL", "Netscape", "NPL" or any
      confusingly similar phrase do not appear anywhere in your license and (b)
      otherwise make it clear that your version of the license contains terms
      which differ from the Mozilla Public License and Netscape Public License.
      (Filling in the name of the Initial Developer, Original Code or
      Contributor in the notice described in Exhibit A shall not of themselves
      be deemed to be modifications of this License.)

   (( 7. ))??
   DISCLAIMER OF WARRANTY.

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER
   OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR
   OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF
   THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

   (( 8. ))??
   TERMINATION.

   This License and the rights granted hereunder will terminate automatically if
   You fail to comply with terms herein and fail to cure such breach within 30
   days of becoming aware of the breach. All sublicenses to the Covered Code
   which are properly granted shall survive any termination of this License.
   Provisions which, by their nature, must remain in effect beyond the
   termination of this License shall survive.

   (( 9. ))??
   LIMITATION OF LIABILITY.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE INITIAL DEVELOPER, ANY OTHER
   CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE, OR ANY SUPPLIER OF ANY OF
   SUCH PARTIES, BE LIABLE TO YOU OR ANY OTHER PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THAT EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. GOVERNMENT END USERS.

   The Covered Code is a "commercial item," as that term is defined in 48 C.F.R.
   2.101 (Oct. 1995), consisting of "commercial computer software" and
   "commercial computer software documentation," as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Covered Code with only those rights set forth herein.

   (( 11. ))??
   MISCELLANEOUS.

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by California law provisions
   (except to the extent applicable law, if any, provides otherwise), excluding
   its conflict-of-law provisions. With respect to disputes in which at least
   one party is a citizen of, or an entity chartered or registered to do
   business in, the United States of America: (a) unless otherwise agreed in
   writing, all disputes relating to this License (excepting any dispute
   relating to intellectual property rights) shall be subject to final and
   binding arbitration, with the losing party paying all costs of arbitration;
   (b) any arbitration relating to this Agreement shall be held in Santa Clara
   County, California, under the auspices of JAMS/EndDispute; and (c) any
   litigation relating to this Agreement shall be subject to the jurisdiction of
   the Federal Courts of the Northern District of California, with venue lying
   in Santa Clara County, California, with the losing party responsible for
   costs, including without limitation, court costs and reasonable attorneys
   fees and expenses. The application of the United Nations Convention on
   Contracts for the International Sale of Goods is expressly excluded. Any law
   or regulation which provides that the language of a contract shall be
   construed against the drafter shall not apply to this License.

   (( 12. ))??
   RESPONSIBILITY FOR CLAIMS.

   Except in cases where another Contributor has failed to comply with Section
   3.4, You are responsible for damages arising, directly or indirectly, out of
   Your utilization of rights under this License, based on the number of copies
   of Covered Code you made available, the revenues you received from utilizing
   such rights, and other relevant factors. You agree to work with affected
   parties to distribute responsibility on an equitable basis.

(( EXHIBIT A.

The contents of this file are subject to the Mozilla Public License Version 1.0
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at http:/www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

The Original Code is _____ .
The Initial Developer of the Original Code is _____ .
Portions created by _____ are Copyright (C) _____ . All Rights Reserved.
Contributor(s): _____ ."

))??

))
`
const license_MPL_1_1_lre = `//**
Mozilla Public License 1.1
https://spdx.org/licenses/MPL-1.1.json
http://www.mozilla.org/MPL/MPL-1.1.html
https://opensource.org/licenses/MPL-1.1
**//

((

The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http:/www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

||

(( Mozilla Public License Version 1.1 ))??

   (( 1. ))??
   Definitions.

      (( 1.0.1. ))??
      "Commercial Use" means distribution or otherwise making the Covered Code
      available to a third party.

      (( 1.1. ))??
      "Contributor" means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      "Covered Code" means the Original Code or Modifications or the combination
      of the Original Code and Modifications, in each case including portions
      thereof.

      (( 1.4. ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      "Executable" means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      "Initial Developer" means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7. ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.8. ))??
      "License" means this document.

      (( 1.8.1. ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.9. ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

      ((A.))??
      Any addition to or deletion from the contents of a file containing
      Original Code or previous Modifications.

      ((B.))??
      Any new file that contains any part of the Original Code or previous
      Modifications.

      (( 1.10. ))??
      "Original Code" means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      (( 1.10.1. ))??
      "Patent Claims" means any patent claim(s), now owned or hereafter
      acquired, including without limitation, method, process, and apparatus
      claims, in any patent Licensable by grantor.

      (( 1.11. ))??
      "Source Code" means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or source code differential comparisons
      against either the Original Code or another well known, available Covered
      Code of the Contributor's choice. The Source Code can be in a compressed
      or archival form, provided the appropriate decompression or de-archiving
      software is widely available for no charge.

      (( 1.12. ))??
      "You" (or "Your") means an individual or a legal entity exercising rights
      under, and complying with all of the terms of, this License or a future
      version of this License issued under Section 6.1. For legal entities,
      "You" includes any entity which controls, is controlled by, or is under
      common control with You. For purposes of this definition, "control" means
      (a) the power, direct or indirect, to cause the direction or management of
      such entity, whether by contract or otherwise, or (b) ownership of more
      than fifty percent (50%) of the outstanding shares or beneficial ownership
      of such entity.

   (( 2. ))??
   Source Code License.

      (( 2.1. ))??
      The Initial Developer Grant. The Initial Developer hereby grants You a
      world-wide, royalty-free, non-exclusive license, subject to third party
      intellectual property claims:

         (( a. ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Initial Developer to use, reproduce, modify, display,
         perform, sublicense and distribute the Original Code (or portions
         thereof) with or without Modifications, and/or as part of a Larger
         Work; and

         (( b. ))??
         under Patents Claims infringed by the making, using or selling of
         Original Code, to make, have made, use, practice, sell, and offer for
         sale, and/or otherwise dispose of the Original Code (or portions
         thereof).

         (( c. ))??
         the licenses granted in this Section 2.1 (a) and (b) are effective on
         the date Initial Developer first distributes Original Code under the
         terms of this License.

         (( d. ))??
         Notwithstanding Section 2.1 (b) above, no patent license is granted: 1)
         for code that You delete from the Original Code; 2) separate from the
         Original Code; or 3) for infringements caused by: i) the modification
         of the Original Code or ii) the combination of the Original Code with
         other software or devices.

      (( 2.2. ))??
      Contributor Grant. Subject to third party intellectual property claims,
      each Contributor hereby grants You a world-wide, royalty-free,
      non-exclusive license

         (( a. ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Contributor, to use, reproduce, modify, display, perform,
         sublicense and distribute the Modifications created by such Contributor
         (or portions thereof) either on an unmodified basis, with other
         Modifications, as Covered Code and/or as part of a Larger Work; and

         (( b. ))??
         under Patent Claims infringed by the making, using, or selling of
         Modifications made by that Contributor either alone and/or in
         combination with its Contributor Version (or portions of such
         combination), to make, use, sell, offer for sale, have made, and/or
         otherwise dispose of: 1) Modifications made by that Contributor (or
         portions thereof); and 2) the combination of Modifications made by that
         Contributor with its Contributor Version (or portions of such
         combination).

         (( c. ))??
         the licenses granted in Sections 2.2 (a) and 2.2 (b) are effective on
         the date Contributor first makes Commercial Use of the Covered Code.

         (( d. ))??
         Notwithstanding Section 2.2 (b) above, no patent license is granted: 1)
         for any code that Contributor has deleted from the Contributor Version;
         2) separate from the Contributor Version; 3) for infringements caused
         by: i) third party modifications of Contributor Version or ii) the
         combination of Modifications made by that Contributor with other
         software (except as part of the Contributor Version) or other devices;
         or 4) under Patent Claims infringed by Covered Code in the absence of
         Modifications made by that Contributor.

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Application of License. The Modifications which You create or to which You
      contribute are governed by the terms of this License, including without
      limitation Section 2.2. The Source Code version of Covered Code may be
      distributed only under the terms of this License or a future version of
      this License released under Section 6.1, and You must include a copy of
      this License with every copy of the Source Code You distribute. You may
      not offer or impose any terms on any Source Code version that alters or
      restricts the applicable version of this License or the recipients' rights
      hereunder. However, You may include an additional document offering the
      additional rights described in Section 3.5.

      (( 3.2. ))??
      Availability of Source Code. Any Modification which You create or to which
      You contribute must be made available in Source Code form under the terms
      of this License either on the same media as an Executable version or via
      an accepted Electronic Distribution Mechanism to anyone to whom you made
      an Executable version available; and if made available via Electronic
      Distribution Mechanism, must remain available for at least twelve (12)
      months after the date it initially became available, or at least six (6)
      months after a subsequent version of that particular Modification has been
      made available to such recipients. You are responsible for ensuring that
      the Source Code version remains available even if the Electronic
      Distribution Mechanism is maintained by a third party.

      (( 3.3. ))??
      Description of Modifications. You must cause all Covered Code to which You
      contribute to contain a file documenting the changes You made to create
      that Covered Code and the date of any change. You must include a prominent
      statement that the Modification is derived, directly or indirectly, from
      Original Code provided by the Initial Developer and including the name of
      the Initial Developer in (a) the Source Code, and (b) in any notice in an
      Executable version or related documentation in which You describe the
      origin or ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters

         (( (a) ))??
         Third Party Claims

         If Contributor has knowledge that a license under a third party's
         intellectual property rights is required to exercise the rights granted
         by such Contributor under Sections 2.1 or 2.2, Contributor must include
         a text file with the Source Code distribution titled "LEGAL" which
         describes the claim and the party making the claim in sufficient detail
         that a recipient will know whom to contact. If Contributor obtains such
         knowledge after the Modification is made available as described in
         Section 3.2, Contributor shall promptly modify the LEGAL file in all
         copies Contributor makes available thereafter and shall take other
         steps (such as notifying appropriate mailing lists or newsgroups)
         reasonably calculated to inform those who received the Covered Code
         that new knowledge has been obtained.

         (( (b) ))??
         Contributor APIs

         If Contributor's Modifications include an application programming
         interface and Contributor has knowledge of patent licenses which are
         reasonably necessary to implement that API, Contributor must also
         include this information in the LEGAL file.

         (( c ))??
         Representations.

         Contributor represents that, except as disclosed pursuant to Section
         3.4 (a) above, Contributor believes that Contributor's Modifications
         are Contributor's original creation(s) and/or Contributor has
         sufficient rights to grant the rights conveyed by this License.

      (( 3.5. ))??
      Required Notices. You must duplicate the notice in Exhibit A in each file
      of the Source Code. If it is not possible to put such notice in a
      particular Source Code file due to its structure, then You must include
      such notice in a location (such as a relevant directory) where a user
      would be likely to look for such a notice. If You created one or more
      Modification(s) You may add your name as a Contributor to the notice
      described in Exhibit A. You must also duplicate this License in any
      documentation for the Source Code where You describe recipients' rights or
      ownership rights relating to Covered Code. You may choose to offer, and to
      charge a fee for, warranty, support, indemnity or liability obligations to
      one or more recipients of Covered Code. However, You may do so only on
      Your own behalf, and not on behalf of the Initial Developer or any
      Contributor. You must make it absolutely clear than any such warranty,
      support, indemnity or liability obligation is offered by You alone, and
      You hereby agree to indemnify the Initial Developer and every Contributor
      for any liability incurred by the Initial Developer or such Contributor as
      a result of warranty, support, indemnity or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Versions. You may distribute Covered Code in
      Executable form only if the requirements of Sections
      (( 3.1, 3.2, 3.3, 3.4 and 3.5 || 3.1 - 3.5 ))
      have been met for that Covered Code, and if You include a notice
      stating that the Source Code version of the Covered Code is available
      under the terms of this License, including a description of how and where
      You have fulfilled the obligations of Section 3.2. The notice must be
      conspicuously included in any notice in an Executable version, related
      documentation or collateral in which You describe recipients' rights
      relating to the Covered Code. You may distribute the Executable version of
      Covered Code or ownership rights under a license of Your choice, which may
      contain terms different from this License, provided that You are in
      compliance with the terms of this License and that the license for the
      Executable version does not attempt to limit or alter the recipient's
      rights in the Source Code version from the rights set forth in this
      License. If You distribute the Executable version under a different
      license You must make it absolutely clear that any terms which differ from
      this License are offered by You alone, not by the Initial Developer or any
      Contributor. You hereby agree to indemnify the Initial Developer and every
      Contributor for any liability incurred by the Initial Developer or such
      Contributor as a result of any such terms You offer.

      (( 3.7. ))??
      Larger Works. You may create a Larger Work by combining Covered Code with
      other code not governed by the terms of this License and distribute the
      Larger Work as a single product. In such a case, You must make sure the
      requirements of this License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute, judicial
   order, or regulation then You must: (a) comply with the terms of this License
   to the maximum extent possible; and (b) describe the limitations and the code
   they affect. Such description must be included in the LEGAL file described in
   Section 3.4 and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Application of this License.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A and to related Covered Code.

   (( 6. ))??
   Versions of the License.

      (( 6.1. ))??
      New Versions

      Netscape Communications Corporation ("Netscape") may publish revised
      and/or new versions of the License from time to time. Each version will be
      given a distinguishing version number.

      (( 6.2. ))??
      Effect of New Versions

      Once Covered Code has been published under a particular version of the
      License, You may always continue to use it under the terms of that
      version. You may also choose to use such Covered Code under the terms of
      any subsequent version of the License published by Netscape. No one other
      than Netscape has the right to modify the terms applicable to Covered Code
      created under this License.

      (( 6.3. ))??
      Derivative Works

      If You create or use a modified version of this License (which you may
      only do in order to apply it to code which is not already Covered Code
      governed by this License), You must (a) rename Your license so that the
      phrases "Mozilla", "MOZILLAPL", "MOZPL", "Netscape", "MPL", "NPL" or any
      confusingly similar phrase do not appear in your license (except to note
      that your license differs from this License) and (b) otherwise make it
      clear that Your version of the license contains terms which differ from
      the Mozilla Public License and Netscape Public License. (Filling in the
      name of the Initial Developer, Original Code or Contributor in the notice
      described in Exhibit A shall not of themselves be deemed to be
      modifications of this License.)

   (( 7. ))??
   DISCLAIMER OF WARRANTY

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER
   OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR
   OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF
   THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

   (( 8. ))??
   Termination

      (( 8.1. ))??
      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. All sublicenses to the
      Covered Code which are properly granted shall survive any termination of
      this License. Provisions which, by their nature, must remain in effect
      beyond the termination of this License shall survive.

      (( 8.2. ))??
      If You initiate litigation by asserting a patent infringement claim
      (excluding declatory judgment actions) against Initial Developer or a
      Contributor (the Initial Developer or Contributor against whom You file
      such action is referred to as "Participant") alleging that:

         (( a. ))??
         such Participant's Contributor Version directly or indirectly infringes
         any patent, then any and all rights granted by such Participant to You
         under Sections 2.1 and/or 2.2 of this License shall, upon 60 days
         notice from Participant terminate prospectively, unless if within 60
         days after receipt of notice You either: (i) agree in writing to pay
         Participant a mutually agreeable reasonable royalty for Your past and
         future use of Modifications made by such Participant, or (ii) withdraw
         Your litigation claim with respect to the Contributor Version against
         such Participant. If within 60 days of notice, a reasonable royalty and
         payment arrangement are not mutually agreed upon in writing by the
         parties or the litigation claim is not withdrawn, the rights granted by
         Participant to You under Sections 2.1 and/or 2.2 automatically
         terminate at the expiration of the 60 day notice period specified
         above.

         (( b. ))??
         any software, hardware, or device, other than such Participant's
         Contributor Version, directly or indirectly infringes any patent, then
         any rights granted to You by such Participant under Sections 2.1(b) and
         2.2(b) are revoked effective as of the date You first made, used, sold,
         distributed, or had made, Modifications made by that Participant.

      (( 8.3. ))??
      If You assert a patent infringement claim against Participant alleging
      that such Participant's Contributor Version directly or indirectly
      infringes any patent where such claim is resolved (such as by license or
      settlement) prior to the initiation of patent infringement litigation,
      then the reasonable value of the licenses granted by such Participant
      under Sections 2.1 or 2.2 shall be taken into account in determining the
      amount or value of any payment or license.

      (( 8.4. ))??
      In the event of termination under Sections 8.1 or 8.2 above, all end user
      license agreements (excluding distributors and resellers) which have been
      validly granted by You or any distributor hereunder prior to termination
      shall survive termination.

   (( 9. ))??
   LIMITATION OF LIABILITY

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, THE INITIAL DEVELOPER, ANY
   OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE, OR ANY SUPPLIER OF ANY
   OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. government end users

   The Covered Code is a "commercial item," as that term is defined in 48 C.F.R.
   2.101 (Oct. 1995), consisting of "commercial computer software" and
   "commercial computer software documentation," as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Covered Code with only those rights set forth herein.

   (( 11. ))??
   Miscellaneous

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by California law provisions
   (except to the extent applicable law, if any, provides otherwise), excluding
   its conflict-of-law provisions. With respect to disputes in which at least
   one party is a citizen of, or an entity chartered or registered to do
   business in the United States of America, any litigation relating to this
   License shall be subject to the jurisdiction of the Federal Courts of the
   Northern District of California, with venue lying in Santa Clara County,
   California, with the losing party responsible for costs, including without
   limitation, court costs and reasonable attorneys' fees and expenses. The
   application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any law or regulation
   which provides that the language of a contract shall be construed against the
   drafter shall not apply to this License.

   (( 12. ))??
   Responsibility for claims

   As between Initial Developer and the Contributors, each party is responsible
   for claims and damages arising, directly or indirectly, out of its
   utilization of rights under this License and You agree to work with Initial
   Developer and Contributors to distribute such responsibility on an equitable
   basis. Nothing herein is intended or shall be deemed to constitute any
   admission of liability.

   (( 13. ))??
   Multiple-licensed code

   Initial Developer may designate portions of the Covered Code as
   "Multiple-Licensed". "Multiple-Licensed" means that the Initial Developer
   permits you to utilize portions of the Covered Code under Your choice of the
   MPL or the alternative licenses, if any, specified by the Initial Developer
   in the file described in Exhibit A.

(( Exhibit A - Mozilla Public License.

The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at http:/www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

The Original Code is ______________________________________ .

The Initial Developer of the Original Code is ________________________ .

Portions created by ______________________ are Copyright (C) ______ . All
Rights Reserved.

Contributor(s): ______________________________________ .

Alternatively, the contents of this file may be used under the terms of the
_____ license (the " [___] License"), in which case the provisions of [______]
License are applicable instead of those above. If you wish to allow use of
your version of this file only under the terms of the [____] License and not
to allow others to use your version of this file under the MPL, indicate your
decision by deleting the provisions above and replace them with the notice
and other provisions required by the [___] License. If you do not delete the
provisions above, a recipient may use your version of this file under either
the MPL or the [___] License."

NOTE: The text of this Exhibit A may differ slightly from the text of the
notices in the Source Code files of the Original Code. You should use the
text of this Exhibit A rather than the text found in the Original Code Source
Code for Your Modifications.

))??

))
`
const license_MPL_2_0_lre = `//**
Mozilla Public License 2.0
https://spdx.org/licenses/MPL-2.0.json
http://www.mozilla.org/MPL/2.0/
https://opensource.org/licenses/MPL-2.0
**//





((

This Source Code Form is subject to the terms of the Mozilla Public License, v.
2.0. If a copy of the MPL was not distributed with this
((file || project))
, You can obtain one
at http:/mozilla.org/MPL/2.0/.


||

Licensed under the Mozilla Public License, Version 2.0 (the "License"); you may
not use this software except in compliance with the License. You may obtain a
copy of the License at

http://mozilla.org/MPL/2.0/

Unless required by applicable law or agreed to in writing, software distributed
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.

||

(( Mozilla Public License Version 2.0 ))??

   (( 1. ))??
   Definitions

      (( 1.1. ))??
      "Contributor" means each individual or legal entity that creates,
      contributes to the creation of, or owns Covered Software.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Contributions of others
      (if any) used by a Contributor and that particular Contributor's
      Contribution.

      (( 1.3. ))??
      "Contribution" means Covered Software of a particular Contributor.

      (( 1.4. ))??
      "Covered Software" means Source Code Form to which the initial Contributor
      has attached the notice in Exhibit A, the Executable Form of such Source
      Code Form, and Modifications of such Source Code Form, in each case
      including portions thereof.

      (( 1.5. ))??
      "Incompatible With Secondary Licenses" means

         (( (a) ))??
         that the initial Contributor has attached the notice described in
         Exhibit B to the Covered Software; or

         (( (b) ))??
         that the Covered Software was made available under the terms of version
         1.1 or earlier of the License, but not also under the terms of a
         Secondary License.

      (( 1.6. ))??
      "Executable Form" means any form of the work other than Source Code Form.

      (( 1.7. ))??
      "Larger Work" means a work that combines Covered Software with other
      material, in a separate file or files, that is not Covered Software.

      (( 1.8. ))??
      "License" means this document.

      (( 1.9. ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently, any
      and all of the rights conveyed by this License.

      (( 1.10. ))??
      "Modifications" means any of the following:

         (( (a) ))??
         any file in Source Code Form that results from an addition to, deletion
         from, or modification of the contents of Covered Software; or

         (( (b) ))??
         any new file in Source Code Form that contains any Covered Software.

      (( 1.11. ))??
      "Patent Claims" of a Contributor means any patent claim(s), including
      without limitation, method, process, and apparatus claims, in any patent
      Licensable by such Contributor that would be infringed, but for the grant
      of the License, by the making, using, selling, offering for sale, having
      made, import, or transfer of either its Contributions or its Contributor
      Version.

      (( 1.12. ))??
      "Secondary License" means either the GNU General Public License, Version
      2.0, the GNU Lesser General Public License, Version 2.1, the GNU Affero
      General Public License, Version 3.0, or any later versions of those
      licenses.

      (( 1.13. ))??
      "Source Code Form" means the form of the work preferred for making
      modifications.

      (( 1.14. ))??
      "You" (or "Your") means an individual or a legal entity exercising rights
      under this License. For legal entities, "You" includes any entity that
      controls, is controlled by, or is under common control with You. For
      purposes of this definition, "control" means (a) the power, direct or
      indirect, to cause the direction or management of such entity, whether by
      contract or otherwise, or (b) ownership of more than fifty percent (50%)
      of the outstanding shares or beneficial ownership of such entity.

   (( 2. ))??
   License Grants and Conditions

      (( 2.1. ))??
      Grants

      Each Contributor hereby grants You a world-wide, royalty-free,
      non-exclusive license:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by such Contributor to use, reproduce, make available,
         modify, display, perform, distribute, and otherwise exploit its
         Contributions, either on an unmodified basis, with Modifications, or as
         part of a Larger Work; and

         (( (b) ))??
         under Patent Claims of such Contributor to make, use, sell, offer for
         sale, have made, import, and otherwise transfer either its
         Contributions or its Contributor Version.

      (( 2.2. ))??
      Effective Date

      The licenses granted in Section 2.1 with respect to any Contribution
      become effective for each Contribution on the date the Contributor first
      distributes such Contribution.

      (( 2.3. ))??
      Limitations on Grant Scope

      The licenses granted in this Section 2 are the only rights granted under
      this License. No additional rights or licenses will be implied from the
      distribution or licensing of Covered Software under this License.
      Notwithstanding Section 2.1(b) above, no patent license is granted by a
      Contributor:

         (( (a) ))??
         for any code that a Contributor has removed from Covered Software; or

         (( (b) ))??
         for infringements caused by: (i) Your and any other third party's
         modifications of Covered Software, or (ii) the combination of its
         Contributions with other software (except as part of its Contributor
         Version); or

         (( c ))??
         under Patent Claims infringed by Covered Software in the absence of its
         Contributions.

      This License does not grant any rights in the trademarks, service marks,
      or logos of any Contributor (except as may be necessary to comply with the
      notice requirements in Section 3.4).

      (( 2.4. ))??
      Subsequent Licenses

      No Contributor makes additional grants as a result of Your choice to
      distribute the Covered Software under a subsequent version of this License
      (see Section 10.2) or under the terms of a Secondary License (if permitted
      under the terms of Section 3.3).

      (( 2.5. ))??
      Representation

      Each Contributor represents that the Contributor believes its
      Contributions are its original creation(s) or it has sufficient rights to
      grant the rights to its Contributions conveyed by this License.

      (( 2.6. ))??
      Fair Use

      This License is not intended to limit any rights You have under applicable
      copyright doctrines of fair use, fair dealing, or other equivalents.

      (( 2.7. ))??
      Conditions

      Sections 3.1, 3.2, 3.3, and 3.4 are conditions of the licenses granted in
      Section 2.1.

   (( 3. ))??
   Responsibilities

      (( 3.1. ))??
      Distribution of Source Form

      All distribution of Covered Software in Source Code Form, including any
      Modifications that You create or to which You contribute, must be under
      the terms of this License. You must inform recipients that the Source Code
      Form of the Covered Software is governed by the terms of this License, and
      how they can obtain a copy of this License. You may not attempt to alter
      or restrict the recipients' rights in the Source Code Form.

      (( 3.2. ))??
      Distribution of Executable Form

      If You distribute Covered Software in Executable Form then:

         (( (a) ))??
         such Covered Software must also be made available in Source Code Form,
         as described in Section 3.1, and You must inform recipients of the
         Executable Form how they can obtain a copy of such Source Code Form by
         reasonable means in a timely manner, at a charge no more than the cost
         of distribution to the recipient; and

         (( (b) ))??
         You may distribute such Executable Form under the terms of this
         License, or sublicense it under different terms, provided that the
         license for the Executable Form does not attempt to limit or alter the
         recipients' rights in the Source Code Form under this License.

      (( 3.3. ))??
      Distribution of a Larger Work

      You may create and distribute a Larger Work under terms of Your choice,
      provided that You also comply with the requirements of this License for
      the Covered Software. If the Larger Work is a combination of Covered
      Software with a work governed by one or more Secondary Licenses, and the
      Covered Software is not Incompatible With Secondary Licenses, this License
      permits You to additionally distribute such Covered Software under the
      terms of such Secondary License(s), so that the recipient of the Larger
      Work may, at their option, further distribute the Covered Software under
      the terms of either this License or such Secondary License(s).

      (( 3.4. ))??
      Notices

      You may not remove or alter the substance of any license notices
      (including copyright notices, patent notices, disclaimers of warranty, or
      limitations of liability) contained within the Source Code Form of the
      Covered Software, except that You may alter any license notices to the
      extent required to remedy known factual inaccuracies.

      (( 3.5. ))??
      Application of Additional Terms

      You may choose to offer, and to charge a fee for, warranty, support,
      indemnity or liability obligations to one or more recipients of Covered
      Software. However, You may do so only on Your own behalf, and not on
      behalf of any Contributor. You must make it absolutely clear that any such
      warranty, support, indemnity, or liability obligation is offered by You
      alone, and You hereby agree to indemnify every Contributor for any
      liability incurred by such Contributor as a result of warranty, support,
      indemnity or liability terms You offer. You may include additional
      disclaimers of warranty and limitations of liability specific to any
      jurisdiction.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Software due to statute, judicial
   order, or regulation then You must: (a) comply with the terms of this License
   to the maximum extent possible; and (b) describe the limitations and the code
   they affect. Such description must be placed in a text file included with all
   distributions of the Covered Software under this License. Except to the
   extent prohibited by statute or regulation, such description must be
   sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Termination

      (( 5.1. ))??
      The rights granted under this License will terminate automatically if You
      fail to comply with any of its terms. However, if You become compliant,
      then the rights granted under this License from a particular Contributor
      are reinstated (a) provisionally, unless and until such Contributor
      explicitly and finally terminates Your grants, and (b) on an ongoing
      basis, if such Contributor fails to notify You of the non-compliance by
      some reasonable means prior to 60 days after You have come back into
      compliance. Moreover, Your grants from a particular Contributor are
      reinstated on an ongoing basis if such Contributor notifies You of the
      non-compliance by some reasonable means, this is the first time You have
      received notice of non-compliance with this License from such Contributor,
      and You become compliant prior to 30 days after Your receipt of the
      notice.

      (( 5.2. ))??
      If You initiate litigation against any entity by asserting a patent
      infringement claim (excluding declaratory judgment actions,
      counter-claims, and cross-claims) alleging that a Contributor Version
      directly or indirectly infringes any patent, then the rights granted to
      You by any and all Contributors for the Covered Software under Section 2.1
      of this License shall terminate.

      (( 5.3. ))??
      In the event of termination under Sections 5.1 or 5.2 above, all end user
      license agreements (excluding distributors and resellers) which have been
      validly granted by You or Your distributors under this License prior to
      termination shall survive termination.

   (( 6. ))??
   Disclaimer of Warranty

   Covered Software is provided under this License on an "as is" basis, without
   warranty of any kind, either expressed, implied, or statutory, including,
   without limitation, warranties that the Covered Software is free of defects,
   merchantable, fit for a particular purpose or non-infringing. The entire risk
   as to the quality and performance of the Covered Software is with You. Should
   any Covered Software prove defective in any respect, You (not any
   Contributor) assume the cost of any necessary servicing, repair, or
   correction. This disclaimer of warranty constitutes an essential part of this
   License. No use of any Covered Software is authorized under this License
   except under this disclaimer.

   (( 7. ))??
   Limitation of Liability

   Under no circumstances and under no legal theory, whether tort (including
   negligence), contract, or otherwise, shall any Contributor, or anyone who
   distributes Covered Software as permitted above, be liable to You for any
   direct, indirect, special, incidental, or consequential damages of any
   character including, without limitation, damages for lost profits, loss of
   goodwill, work stoppage, computer failure or malfunction, or any and all
   other commercial damages or losses, even if such party shall have been
   informed of the possibility of such damages. This limitation of liability
   shall not apply to liability for death or personal injury resulting from such
   party's negligence to the extent applicable law prohibits such limitation.
   Some jurisdictions do not allow the exclusion or limitation of incidental or
   consequential damages, so this exclusion and limitation may not apply to
   You.

   (( 8. ))??
   Litigation

   Any litigation relating to this License may be brought only in the courts of
   a jurisdiction where the defendant maintains its principal place of business
   and such litigation shall be governed by laws of that jurisdiction, without
   reference to its conflict-of-law provisions. Nothing in this Section shall
   prevent a party's ability to bring cross-claims or counter-claims.

   (( 9. ))??
   Miscellaneous

   This License represents the complete agreement concerning the subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. Any law or regulation which provides that the language of a
   contract shall be construed against the drafter shall not be used to construe
   this License against a Contributor.

   (( 10. ))??
   Versions of the License

      (( 10.1. ))??
      New Versions

      Mozilla Foundation is the license steward. Except as provided in Section
      10.3, no one other than the license steward has the right to modify or
      publish new versions of this License. Each version will be given a
      distinguishing version number.

      (( 10.2. ))??
      Effect of New Versions

      You may distribute the Covered Software under the terms of the version of
      the License under which You originally received the Covered Software, or
      under the terms of any subsequent version published by the license
      steward.

      (( 10.3. ))??
      Modified Versions

      If you create software not governed by this License, and you want to
      create a new license for such software, you may create and use a modified
      version of this License if you rename the license and remove any
      references to the name of the license steward (except to note that such
      modified license differs from this License).

      (( 10.4. ))??
      Distributing Source Code Form that is Incompatible With Secondary
      Licenses

      If You choose to distribute Source Code Form that is Incompatible With
      Secondary Licenses under the terms of this version of the License, the
      notice described in Exhibit B of this License must be attached.
      (( Exhibit A - Source Code Form License Notice

This Source Code Form is subject to the terms of the Mozilla Public License, v.
2.0. If a copy of the MPL was not distributed with this file, You can obtain one
at http:/mozilla.org/MPL/2.0/.

If it is not possible or desirable to put the notice in a particular file, then
You may include the notice in a location (such as a LICENSE file in a relevant
directory) where a recipient would be likely to look for such a notice.

You may add additional accurate notices of copyright ownership.

Exhibit B - "Incompatible With Secondary Licenses" Notice

This Source Code Form is "Incompatible With Secondary Licenses", as defined by
the Mozilla Public License, v. 2.0. ))??

))
`
const license_MPL_2_0_no_copyleft_exception_lre = `

This Source Code Form is subject to the terms of the Mozilla Public License, v.
2.0. If a copy of the MPL was not distributed with this
((file || project))
, You can obtain one
at http:/mozilla.org/MPL/2.0/.

This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
`
const license_MS_PL_lre = `//**
Microsoft Public License
https://spdx.org/licenses/MS-PL.json
http://www.microsoft.com/opensource/licenses.mspx
https://opensource.org/licenses/MS-PL
**//

(( Microsoft Public License (Ms-PL) ))??

This license governs use of the accompanying software. If you use the software,
you accept this license. If you do not accept the license, do not use the
software.

   (( 1. ))??
   Definitions

   The terms "reproduce," "reproduction," "derivative works," and "distribution"
   have the same meaning here as under U.S. copyright law. A "contribution" is
   the original software, or any additions or changes to the software. A
   "contributor" is any person that distributes its contribution under this
   license. "Licensed patents" are a contributor's patent claims that read
   directly on its contribution.

   (( 2. ))??
   Grant of Rights

      (( (A) ))??
      Copyright Grant- Subject to the terms of this license, including the
      license conditions and limitations in section 3, each contributor grants
      you a non-exclusive, worldwide, royalty-free copyright license to
      reproduce its contribution, prepare derivative works of its contribution,
      and distribute its contribution or any derivative works that you create.

      (( (B) ))??
      Patent Grant- Subject to the terms of this license, including the license
      conditions and limitations in section 3, each contributor grants you a
      non-exclusive, worldwide, royalty-free license under its licensed patents
      to make, have made, use, sell, offer for sale, import, and/or otherwise
      dispose of its contribution in the software or derivative works of the
      contribution in the software.

   (( 3. ))??
   Conditions and Limitations

      (( (A) ))??
      No Trademark License- This license does not grant you rights to use any
      contributors' name, logo, or trademarks.

      (( (B) ))??
      If you bring a patent claim against any contributor over patents that you
      claim are infringed by the software, your patent license from such
      contributor to the software ends automatically.

      (( (C) ))??
      If you distribute any portion of the software, you must retain all
      copyright, patent, trademark, and attribution notices that are present in
      the software.

      (( (D) ))??
      If you distribute any portion of the software in source code form, you may
      do so only under this license by including a complete copy of this license
      with your distribution. If you distribute any portion of the software in
      compiled or object code form, you may only do so under a license that
      complies with this license.

      (( (E) ))??
      The software is licensed "as-is." You bear the risk of using it. The
      contributors give no express warranties, guarantees, or conditions. You
      may have additional consumer rights under your local laws which this
      license cannot change. To the extent permitted under your local laws, the
      contributors exclude the implied warranties of merchantability, fitness
      for a particular purpose and non-infringement.
`
const license_MS_RL_lre = `//**
Microsoft Reciprocal License
https://spdx.org/licenses/MS-RL.json
http://www.microsoft.com/opensource/licenses.mspx
https://opensource.org/licenses/MS-RL
**//

(( Microsoft Reciprocal License (Ms-RL) ))??

This license governs use of the accompanying software. If you use the software,
you accept this license. If you do not accept the license, do not use the
software.

   (( 1. ))??
   Definitions

   The terms "reproduce," "reproduction," "derivative works," and "distribution"
   have the same meaning here as under U.S. copyright law.

   A "contribution" is the original software, or any additions or changes to the
   software.

   A "contributor" is any person that distributes its contribution under this
   license.

   "Licensed patents" are a contributor's patent claims that read directly on
   its contribution.

   (( 2. ))??
   Grant of Rights

      (( (A) ))??
      Copyright Grant- Subject to the terms of this license, including the
      license conditions and limitations in section 3, each contributor grants
      you a non-exclusive, worldwide, royalty-free copyright license to
      reproduce its contribution, prepare derivative works of its contribution,
      and distribute its contribution or any derivative works that you create.

      (( (B) ))??
      Patent Grant- Subject to the terms of this license, including the license
      conditions and limitations in section 3, each contributor grants you a
      non-exclusive, worldwide, royalty-free license under its licensed patents
      to make, have made, use, sell, offer for sale, import, and/or otherwise
      dispose of its contribution in the software or derivative works of the
      contribution in the software.

   (( 3. ))??
   Conditions and Limitations

      (( (A) ))??
      Reciprocal Grants- For any file you distribute that contains code from the
      software (in source code or binary format), you must provide recipients
      the source code to that file along with a copy of this license, which
      license will govern that file. You may license other files that are
      entirely your own work and do not contain code from the software under any
      terms you choose.

      (( (B) ))??
      No Trademark License- This license does not grant you rights to use any
      contributors' name, logo, or trademarks.

      (( (C) ))??
      If you bring a patent claim against any contributor over patents that you
      claim are infringed by the software, your patent license from such
      contributor to the software ends automatically.

      (( (D) ))??
      If you distribute any portion of the software, you must retain all
      copyright, patent, trademark, and attribution notices that are present in
      the software.

      (( (E) ))??
      If you distribute any portion of the software in source code form, you may
      do so only under this license by including a complete copy of this license
      with your distribution. If you distribute any portion of the software in
      compiled or object code form, you may only do so under a license that
      complies with this license.

      (( (F) ))??
      The software is licensed "as-is." You bear the risk of using it. The
      contributors give no express warranties, guarantees, or conditions. You
      may have additional consumer rights under your local laws which this
      license cannot change. To the extent permitted under your local laws, the
      contributors exclude the implied warranties of merchantability, fitness
      for a particular purpose and non-infringement.
`
const license_MTLL_lre = `//**
Matrix Template Library License
https://spdx.org/licenses/MTLL.json
https://fedoraproject.org/wiki/Licensing/Matrix_Template_Library_License
**//

(( Software License for MTL
(( Copyright __40__ ))??
))??

This file is part of the Matrix Template Library

Dresden University of Technology -- short TUD -- and Indiana University -- short
IU -- have the exclusive rights to license this product under the following
license.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   All redistributions of source code must retain the above copyright notice,
   the list of authors in the original source code, this list of conditions and
   the disclaimer listed in this license;

   (( 2. ))??
   All redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the disclaimer listed in this license in the
   documentation and/or other materials provided with the distribution;

   (( 3. ))??
   Any documentation included with all redistributions must include the
   following acknowledgement:

   "This product includes software developed at the University of Notre Dame,
   the Pervasive Technology Labs at Indiana University, and Dresden University
   of Technology. For technical information contact Andrew Lumsdaine at the
   Pervasive Technology Labs at Indiana University. For administrative and
   license questions contact the Advanced Research and Technology Institute at
   1100 Waterway Blvd. Indianapolis, Indiana 46202, phone 317-274-5905, fax
   317-274-5902."

   Alternatively, this acknowledgement may appear in the software itself, and
   wherever such third-party acknowledgments normally appear.

   (( 4. ))??
   The name "MTL" shall not be used to endorse or promote products derived from
   this software without prior written permission from IU or TUD. For written
   permission, please contact Indiana University Advanced Research & Technology
   Institute.

   (( 5. ))??
   Products derived from this software may not be called "MTL", nor may "MTL"
   appear in their name, without prior written permission of Indiana University
   Advanced Research & Technology Institute.

TUD and IU provide no reassurances that the source code provided does not
infringe the patent or any other intellectual property rights of any other
entity. TUD and IU disclaim any liability to any recipient for claims brought by
any other entity based on infringement of intellectual property rights or
otherwise.

LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO WARRANTIES
AS TO CAPABILITIES OR ACCURACY ARE MADE. DRESDEN UNIVERSITY OF TECHNOLOGY AND
INDIANA UNIVERSITY GIVE NO WARRANTIES AND MAKE NO REPRESENTATION THAT SOFTWARE
IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY
RIGHTS. DRESDEN UNIVERSITY OF TECHNOLOGY AND INDIANA UNIVERSITY MAKE NO
WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
DOORS", "WORMS", OR OTHER HARMFUL CODE. LICENSEE ASSUMES THE ENTIRE RISK AS TO
THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS, AND TO THE PERFORMANCE
AND VALIDITY OF INFORMATION GENERATED USING SOFTWARE.
`
const license_MakeIndex_lre = `//**
MakeIndex License
https://spdx.org/licenses/MakeIndex.json
https://fedoraproject.org/wiki/Licensing/MakeIndex
**//

(( MakeIndex Distribution Notice 11/11/1989
(( Copyright __50__ ))??
))??

Permission is hereby granted to make and distribute original copies of this
program provided that the copyright notice and this permission notice are
preserved and provided that the recipient is not asked to waive or limit his
right to redistribute copies as allowed by this permission notice and provided
that anyone who receives an executable form of this program is granted access to
a machine-readable form of the source code for this program at a cost not
greater than reasonable reproduction, shipping, and handling costs. Executable
forms of this program distributed without the source code must be accompanied by
a conspicuous copy of this permission notice and a statement that tells the
recipient how to obtain the source code.

Permission is granted to distribute modified versions of all or part of this
program under the conditions above with the additional requirement that the
entire modified work must be covered by a permission notice identical to this
permission notice. Anything distributed with and usable only in conjunction with
something derived from this program, whose useful purpose is to extend or adapt
or add capabilities to this program, is to be considered a modified version of
this program under the requirement above. Ports of this program to other systems
not supported in the distribution are also considered modified versions. All
modified versions should be reported back to the author.

This program is distributed with no warranty of any sort. No contributor accepts
responsibility for the consequences of using this program or for whether it
serves any particular purpose.
`
const license_MirOS_lre = `//**
The MirOS Licence
https://spdx.org/licenses/MirOS.json
https://opensource.org/licenses/MirOS
**//

(( The MirOS Licence
(( Copyright __20__ ))??
))??

Provided that these terms and disclaimer and all copyright notices are retained
or reproduced in an accompanying document, permission is granted to deal in this
work without restriction, including unlimited rights to use, publicly perform,
distribute, sell, modify, merge, give away, or sublicence.

This work is provided "AS IS" and WITHOUT WARRANTY of any kind, to the utmost
extent permitted by applicable law, neither express nor implied; without
malicious intent or gross negligence. In no event may a licensor, author or
contributor be held liable for indirect, direct, other damage, loss, or other
issues arising in any way out of dealing in the work, even if advised of the
possibility of such damage or existence of a defect, except proven that it
results out of said person's immediate fault when using the work as intended.
`
const license_Motosoto_lre = `//**
Motosoto License
https://spdx.org/licenses/Motosoto.json
https://opensource.org/licenses/Motosoto
**//

(( MOTOSOTO OPEN SOURCE LICENSE - Version 0.9.1 ))??

This Motosoto Open Source License (the "License") applies to "Community Portal
Server" and related software products as well as any updatesor maintenance
releases of that software ("Motosoto Products") that are distributed by
Motosoto.Com B.V. ("Licensor"). Any Motosoto Product licensed pursuant to this
License is a "Licensed Product." Licensed Product, in its entirety, is protected
by Dutch copyright law. This License identifies the terms under which you may
use, copy, distribute or modify Licensed Product and has been submitted to the
Open Software Initiative (OSI) for approval.
(( Preamble

This Preamble is intended to describe, in plain English, the nature and scope of
this License. However, this Preamble is not a part of this license. The legal
effect of this License is dependent only upon the terms of the License and not
this Preamble. This License complies with the Open Source Definition and has
been approved by Open Source Initiative. Software distributed under this License
may be marked as "OSI Certified Open Source Software."

This License provides that:

   (( 1. ))??
   You may use, sell or give away the Licensed Product, alone or as a component
   of an aggregate software distribution containing programs from several
   different sources. No royalty or other fee is required.

   (( 2. ))??
   Both Source Code and executable versions of the Licensed Product, including
   Modifications made by previous Contributors, are available for your use. (The
   terms "Licensed Product," "Modifications," "Contributors" and "Source Code"
   are defined in the License.)

   (( 3. ))??
   You are allowed to make Modifications to the Licensed Product, and you can
   create Derivative Works from it. (The term "Derivative Works" is defined in
   the License.)

   (( 4. ))??
   By accepting the Licensed Product under the provisions of this License, you
   agree that any Modifications you make to the Licensed Product and then
   distribute are governed by the provisions of this License. In particular, you
   must make the Source Code of your Modifications available to others.

   (( 5. ))??
   You may use the Licensed Product for any purpose, but the Licensor is not
   providing you any warranty whatsoever, nor is the Licensor accepting any
   liability in the event that the Licensed Product doesn't work properly or
   causes you any injury or damages.

   (( 6. ))??
   If you sublicense the Licensed Product or Derivative Works, you may charge
   fees for warranty or support, or for accepting indemnity or liability
   obligations to your customers. You cannot charge for the Source Code.

   (( 7. ))??
   If you assert any patent claims against the Licensor relating to the Licensed
   Product, or if you breach any terms of the License, your rights to the
   Licensed Product under this License automatically terminate.

You may use this License to distribute your own Derivative Works, in which case
the provisions of this License will apply to your Derivative Works just as they
do to the original Licensed Product.

Alternatively, you may distribute your Derivative Works under any other
OSI-approved Open Source license, or under a proprietary license of your choice.
If you use any license other than this License, however, you must continue to
fulfill the requirements of this License (including the provisions relating to
publishing the Source Code) for those portions of your Derivative Works that
consist of the Licensed Product, including the files containing Modifications.

New versions of this License may be published from time to time. You may choose
to continue to use the license terms in this version of the License or those
from the new version. However, only the Licensor has the right to change the
License terms as they apply to the Licensed Product. This License relies on
precise definitions for certain terms. Those terms are defined when they are
first used, and the definitions are repeated for your convenience in a Glossary
at the end of the License.

License Terms ))??

   (( 1. ))??
   Grant of License From Licensor.

   Licensor hereby grants you a world-wide, royalty-free, non-exclusive license,
   subject to third party intellectual property claims, to do the following:

      (( a. ))??
      Use, reproduce, modify, display, perform, sublicense and distribute
      Licensed Product or portions thereof (including Modifications as
      hereinafter defined), in both Source Code or as an executable program.
      "Source Code" means the preferred form for making modifications to the
      Licensed Product, including all modules contained therein, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an executable program, or a list of differential
      comparisons against the Source Code of the Licensed Product.

      (( b. ))??
      Create Derivative Works (as that term is defined under Dutch copyright
      law) of Licensed Product by adding to or deleting from the substance or
      structure of said Licensed Product.

      (( c. ))??
      Under claims of patents now or hereafter owned or controlled by Licensor,
      to make, use, sell, offer for sale, have made, and/or otherwise dispose of
      Licensed Product or portions thereof, but solely to the extent that any
      such claim is necessary to enable you to make, use, sell, offer for sale,
      have made, and/or otherwise dispose of Licensed Product or portions
      thereof or Derivative Works thereof.

   (( 2. ))??
   Grant of License to Modifications From Contributor.

   "Modifications" means any additions to or deletions from the substance or
   structure of (i) a file containing Licensed Product, or (ii) any new file
   that contains any part of Licensed Product. Hereinafter in this License, the
   term "Licensed Product" shall include all previous Modifications that you
   receive from any Contributor. By application of the provisions in Section
   4(a) below, each person or entity who created or contributed to the creation
   of, and distributed, a Modification (a "Contributor") hereby grants you a
   world-wide, royalty-free, non-exclusive license, subject to third party
   intellectual property claims, to do the following:

      (( a. ))??
      Use, reproduce, modify, display, perform, sublicense and distribute any
      Modifications created by such Contributor or portions thereof, in both
      Source Code or as an executable program, either on an unmodified basis or
      as part of Derivative Works.

      (( b. ))??
      Under claims of patents now or hereafter owned or controlled by
      Contributor, to make, use, sell, offer for sale, have made, and/or
      otherwise dispose of Modifications or portions thereof, but solely to the
      extent that any such claim is necessary to enable you to make, use, sell,
      offer for sale, have made, and/or otherwise dispose of Modifications or
      portions thereof or Derivative Works thereof.

   (( 3. ))??
   Exclusions From License Grant.

   Nothing in this License shall be deemed to grant any rights to trademarks,
   copyrights, patents, trade secrets or any other intellectual property of
   Licensor or any Contributor except as expressly stated herein. No patent
   license is granted separate from the Licensed Product, for code that you
   delete from the Licensed Product, or for combinations of the Licensed Product
   with other software or hardware. No right is granted to the trademarks of
   Licensor or any Contributor even if such marks are included in the Licensed
   Product. Nothing in this License shall be interpreted to prohibit Licensor
   from licensing under different terms from this License any code that Licensor
   otherwise would have a right to license.

   (( 4. ))??
   Your Obligations Regarding Distribution.

      (( a. ))??
      Application of This License to Your Modifications. As an express condition
      for your use of the Licensed Product, you hereby agree that any
      Modifications that you create or to which you contribute, and which you
      distribute, are governed by the terms of this License including, without
      limitation, Section 2. Any Modifications that you create or to which you
      contribute may be distributed only under the terms of this License or a
      future version of this License released under Section 7. You must include
      a copy of this License with every copy of the Modifications you
      distribute. You agree not to offer or impose any terms on any Source Code
      or executable version of the Licensed Product or Modifications that alter
      or restrict the applicable version of this License or the recipients'
      rights hereunder. However, you may include an additional document offering
      the additional rights described in Section 4(e).

      (( b. ))??
      Availability of Source Code. You must make available, under the terms of
      this License, the Source Code of the Licensed Product and any
      Modifications that you distribute, either on the same media as you
      distribute any executable or other form of the Licensed Product, or via a
      mechanism generally accepted in the software development community for the
      electronic transfer of data (an "Electronic Distribution Mechanism"). The
      Source Code for any version of Licensed Product or Modifications that you
      distribute must remain available for at least twelve (12) months after the
      date it initially became available, or at least six (6) months after a
      subsequent version of said Licensed Product or Modifications has been made
      available. You are responsible for ensuring that the Source Code version
      remains available even if the Electronic Distribution Mechanism is
      maintained by a third party.

      (( c. ))??
      Description of Modifications. You must cause any Modifications that you
      create or to which you contribute, and which you distribute, to contain a
      file documenting the additions, changes or deletions you made to create or
      contribute to those Modifications, and the dates of any such additions,
      changes or deletions. You must include a prominent statement that the
      Modifications are derived, directly or indirectly, from the Licensed
      Product and include the names of the Licensor and any Contributor to the
      Licensed Product in (i) the Source Code and (ii) in any notice displayed
      by a version of the Licensed Product you distribute or in related
      documentation in which you describe the origin or ownership of the
      Licensed Product. You may not modify or delete any preexisting copyright
      notices in the Licensed Product.

      (( d. ))??
      Intellectual Property Matters.

         (( i. ))??
         Third Party Claims. If you have knowledge that a license to a third
         party's intellectual property right is required to exercise the rights
         granted by this License, you must include a text file with the Source
         Code distribution titled "LEGAL" that describes the claim and the party
         making the claim in sufficient detail that a recipient will know whom
         to contact. If you obtain such knowledge after you make any
         Modifications available as described in Section 4(b), you shall
         promptly modify the LEGAL file in all copies you make available
         thereafter and shall take other steps (such as notifying appropriate
         mailing lists or newsgroups) reasonably calculated to inform those who
         received the Licensed Product from you that new knowledge has been
         obtained.

         (( ii. ))??
         Contributor APIs. If your Modifications include an application
         programming interface ("API") and you have knowledge of patent licenses
         that are reasonably necessary to implement that API, you must also
         include this information in the LEGAL file.

         (( iii. ))??
         Representations. You represent that, except as disclosed pursuant to
         4(d)(i) above, you believe that any Modifications you distribute are
         your original creations and that you have sufficient rights to grant
         the rights conveyed by this License.

      (( e. ))??
      Required Notices. You must duplicate this License in any documentation you
      provide along with the Source Code of any Modifications you create or to
      which you contribute, and which you distribute, wherever you describe
      recipients' rights relating to Licensed Product. You must duplicate the
      notice contained in Exhibit A (the "Notice") in each file of the Source
      Code of any copy you distribute of the Licensed Product. If you created a
      Modification, you may add your name as a Contributor to the Notice. If it
      is not possible to put the Notice in a particular Source Code file due to
      its structure, then you must include such Notice in a location (such as a
      relevant directory file) where a user would be likely to look for such a
      notice. You may choose to offer, and charge a fee for, warranty, support,
      indemnity or liability obligations to one or more recipients of Licensed
      Product. However, you may do so only on your own behalf, and not on behalf
      of the Licensor or any Contributor. You must make it clear that any such
      warranty, support, indemnity or liability obligation is offered by you
      alone, and you hereby agree to indemnify the Licensor and every
      Contributor for any liability incurred by the Licensor or such Contributor
      as a result of warranty, support, indemnity or liability terms you offer.

      (( f. ))??
      Distribution of Executable Versions. You may distribute Licensed Product
      as an executable program under a license of your choice that may contain
      terms different from this License provided (i) you have satisfied the
      requirements of Sections 4(a) through 4(e) for that distribution, (ii) you
      include a conspicuous notice in the executable version, related
      documentation and collateral materials stating that the Source Code
      version of the Licensed Product is available under the terms of this
      License, including a description of how and where you have fulfilled the
      obligations of Section 4(b), (iii) you retain all existing copyright
      notices in the Licensed Product, and (iv) you make it clear that any terms
      that differ from this License are offered by you alone, not by Licensor or
      any Contributor. You hereby agree to indemnify the Licensor and every
      Contributor for any liability incurred by Licensor or such Contributor as
      a result of any terms you offer.

      (( g. ))??
      Distribution of Derivative Works. You may create Derivative Works (e.g.,
      combinations of some or all of the Licensed Product with other code) and
      distribute the Derivative Works as products under any other license you
      select, with the proviso that the requirements of this License are
      fulfilled for those portions of the Derivative Works that consist of the
      Licensed Product or any Modifications thereto.

   (( 5. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for you to comply with any of the terms of this License
   with respect to some or all of the Licensed Product due to statute, judicial
   order, or regulation, then you must (i) comply with the terms of this License
   to the maximum extent possible, (ii) cite the statute or regulation that
   prohibits you from adhering to the License, and (iii) describe the
   limitations and the code they affect. Such description must be included in
   the LEGAL file described in Section 4(d), and must be included with all
   distributions of the Source Code. Except to the extent prohibited by statute
   or regulation, such description must be sufficiently detailed for a recipient
   of ordinary skill at computer programming to be able to understand it.

   (( 6. ))??
   Application of This License.

   This License applies to code to which Licensor or Contributor has attached
   the Notice in Exhibit A, which is incorporated herein by this reference.

   (( 7. ))??
   Versions of This License.

      (( a. ))??
      Version. The Motosoto Open Source License is derived from the Jabber Open
      Source License. All changes are related to applicable law and the location
      of court.

      (( b. ))??
      New Versions. Licensor may publish from time to time revised and/or new
      versions of the License.

      (( c. ))??
      Effect of New Versions. Once Licensed Product has been published under a
      particular version of the License, you may always continue to use it under
      the terms of that version. You may also choose to use such Licensed
      Product under the terms of any subsequent version of the License published
      by Licensor. No one other than Lic ensor has the right to modify the terms
      applicable to Licensed Product created under this License.

      (( d. ))??
      Derivative Works of this License. If you create or use a modified version
      of this License, which you may do only in order to apply it to software
      that is not already a Licensed Product under this License, you must rename
      your license so that it is not confusingly similar to this License, and
      must make it clear that your license contains terms that differ from this
      License. In so naming your license, you may not use any trademark of
      Licensor or any Contributor.

   (( 8. ))??
   Disclaimer of Warranty.

   LICENSED PRODUCT IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE LICENSED PRODUCT IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE LICENSED PRODUCT IS WITH YOU. SHOULD
   LICENSED PRODUCT PROVE DEFECTIVE IN ANY RESPECT, YOU (AND NOT THE LICENSOR OR
   ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR OR
   CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS
   LICENSE. NO USE OF LICENSED PRODUCT IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS
   DISCLAIMER.

   (( 9. ))??
   Termination.

      (( a. ))??
      Automatic Termination Upon Breach. This license and the rights granted
      hereunder will terminate automatically if you fail to comply with the
      terms herein and fail to cure such breach within thirty (30) days of
      becoming aware of the breach. All sublicenses to the Licensed Product that
      are properly granted shall survive any termination of this license.
      Provisions that, by their nature, must remain in effect beyond the
      termination of this License, shall survive.

      (( b. ))??
      Termination Upon Assertion of Patent Infringement. If you initiate
      litigation by asserting a patent infringement claim (excluding declaratory
      judgment actions) against Licensor or a Contributor (Licensor or
      Contributor against whom you file such an action is referred to herein as
      "Respondent") alleging that Licensed Product directly or indirectly
      infringes any patent, then any and all rights granted by such Respondent
      to you under Sections 1 or 2 of this License shall terminate prospectively
      upon sixty (60) days notice from Respondent (the "Notice Period") unless
      within that Notice Period you either agree in writing (i) to pay
      Respondent a mutually agreeable reasonably royalty for your past or future
      use of Licensed Product made by such Respondent, or (ii) withdraw your
      litigation claim with respect to Licensed Product against such Respondent.
      If within said Notice Period a reasonable royalty and payment arrangement
      are not mutually agreed upon in writing by the parties or the litigation
      claim is not withdrawn, the rights granted by Licensor to you under
      Sections 1 and 2 automatically terminate at the expiration of said Notice
      Period.

      (( c. ))??
      Reasonable Value of This License. If you assert a patent infringement
      claim against Respondent alleging that Licensed Product directly or
      indirectly infringes any patent where such claim is resolved (such as by
      license or settlement) prior to the initiation of patent infringement
      litigation, then the reasonable value of the licenses granted by said
      Respondent under Sections 1 and 2 shall be taken into account in
      determining the amount or value of any payment or license.

      (( d. ))??
      No Retroactive Effect of Termination. In the event of termination under
      Sections 9(a) or 9(b) above, all end user license agreements (excluding
      licenses to distributors and reselle rs) that have been validly granted by
      you or any distributor hereunder prior to termination shall survive
      termination.

   (( 10. ))??
   Limitation of Liability.

UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE LICENSOR, ANY CONTRIBUTOR, OR ANY
DISTRIBUTOR OF LICENSED PRODUCT, OR ANY SUPPLIER OF ANY OF SUCH PARTIES, BE
LIABLE TO ANY PERSON FOR ANY INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF
GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER
COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH PARTY SHALL HAVE BEEN INFORMED OF THE
POSSIBILITY OF SUCH DAMAGES. THIS LIMITATION OF LIABILITY SHALL NOT APPLY TO
LIABILITY FOR DEATH OR PERSONAL INJURY RESULTING FROM SUCH PARTY
`
const license_MulanPSL_1_0_lre = `//**
Mulan Permissive Software License, Version 1
https://spdx.org/licenses/MulanPSL-1.0.json
https://license.coscl.org.cn/MulanPSL/
https://github.com/yuwenlong/longphp/blob/25dfb70cc2a466dc4bb55ba30901cbce08d164b5/LICENSE
**//

(( 木兰宽松许可证, 第1版 ))??

(( 木兰宽松许可证， 第1版

2019年8月 http:/license.coscl.org.cn/MulanPSL ))??

您对"软件"的复制、使用、修改及分发受木兰宽松许可证，第1版（"本许可证"）的如下条款的约束：

   (( 0. ))??
   定义

   "软件"是指由"贡献"构成的许可在"本许可证"下的程序和相关文档的集合。

   "贡献者"是指将受版权法保护的作品许可在"本许可证"下的自然人或"法人实体"。

   "法人实体"是指提交贡献的机构及其"关联实体"。

   "关联实体"是指，对"本许可证"下的一方而言，控制、受控制或与其共同受控制的机构，此处的控制是指有受控方或共同受控方至少50%直接或间接的投票权、资金或其他有价证券。

   "贡献"是指由任一"贡献者"许可在"本许可证"下的受版权法保护的作品。

   (( 1. ))??
   授予版权许可

   每个"贡献者"根据"本许可证"授予您永久性的、全球性的、免费的、非独占的、不可撤销的版权许可，您可以复制、使用、修改、分发其"贡献"，不论修改与否。

   (( 2. ))??
   授予专利许可

   每个"贡献者"根据"本许可证"授予您永久性的、全球性的、免费的、非独占的、不可撤销的（根据本条规定撤销除外）专利许可，供您制造、委托制造、使用、许诺销售、销售、进口其"贡献"或以其他方式转移其"贡献"。前述专利许可仅限于"贡献者"现在或将来拥有或控制的其"贡献"本身或其"贡献"与许可"贡献"时的"软件"结合而将必然会侵犯的专利权利要求，不包括仅因您或他人修改"贡献"或其他结合而将必然会侵犯到的专利权利要求。如您或您的"关联实体"直接或间接地（包括通过代理、专利被许可人或受让人），就"软件"或其中的"贡献"对任何人发起专利侵权诉讼（包括反诉或交叉诉讼）或其他专利维权行动，指控其侵犯专利权，则"本许可证"授予您对"软件"的专利许可自您提起诉讼或发起维权行动之日终止。

   (( 3. ))??
   无商标许可

   "本许可证"不提供对"贡献者"的商品名称、商标、服务标志或产品名称的商标许可，但您为满足第4条规定的声明义务而必须使用除外。

   (( 4. ))??
   分发限制

   您可以在任何媒介中将"软件"以源程序形式或可执行形式重新分发，不论修改与否，但您必须向接收者提供"本许可证"的副本，并保留"软件"中的版权、商标、专利及免责声明。

   (( 5. ))??
   免责声明与责任限制

   "软件"及其中的"贡献"在提供时不带任何明示或默示的担保。在任何情况下，"贡献者"或版权所有者不对任何人因使用"软件"或其中的"贡献"而引发的任何直接或间接损失承担责任，不论因何种原因导致或者基于何种法律理论,即使其曾被建议有此种损失的可能性。

条款结束

如何将木兰宽松许可证，第1版，应用到您的软件

如果您希望将木兰宽松许可证，第1版，应用到您的新软件，为了方便接收者查阅，建议您完成如下三步：

   (( 1， ))??
   请您补充如下声明中的空白，包括软件名、软件的首次发表年份以及您作为版权人的名字；

   (( 2， ))??
   请您在软件包的一级目录下创建以"LICENSE"为名的文件，将整个许可证文本放入该文件中；

   (( 3， ))??
   请将如下声明文本放入每个源文件的头部注释中。

Copyright (c) [2019] [name of copyright holder]

[Software Name] is licensed under the Mulan PSL v1.

You can use this software according to the terms and conditions of the Mulan PSL
v1.

You may obtain a copy of Mulan PSL v1 at:

http:/license.coscl.org.cn/MulanPSL

THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.

See the Mulan PSL v1 for more details.
(( Mulan Permissive Software License，Version 1 ))??

(( Mulan Permissive Software License，Version 1 (Mulan PSL v1)

August 2019 http:/license.coscl.org.cn/MulanPSL ))??

Your reproduction, use, modification and distribution of the Software shall be
subject to Mulan PSL v1 (this License) with following terms and conditions:

   (( 0. ))??
   Definition

   Software means the program and related documents which are comprised of those
   Contribution and licensed under this License.

   Contributor means the Individual or Legal Entity who licenses its
   copyrightable work under this License.

   Legal Entity means the entity making a Contribution and all its Affiliates.

   Affiliates means entities that control, or are controlled by, or are under
   common control with a party to this License, 'control' means direct or
   indirect ownership of at least fifty percent (50%) of the voting power,
   capital or other securities of controlled or commonly controlled entity.

   Contribution means the copyrightable work licensed by a particular
   Contributor under this License.

   (( 1. ))??
   Grant of Copyright License

   Subject to the terms and conditions of this License, each Contributor hereby
   grants to you a perpetual, worldwide, royalty-free, non-exclusive,
   irrevocable copyright license to reproduce, use, modify, or distribute its
   Contribution, with modification or not.

   (( 2. ))??
   Grant of Patent License

   Subject to the terms and conditions of this License, each Contributor hereby
   grants to you a perpetual, worldwide, royalty-free, non-exclusive,
   irrevocable (except for revocation under this Section) patent license to
   make, have made, use, offer for sale, sell, import or otherwise transfer its
   Contribution where such patent license is only limited to the patent claims
   owned or controlled by such Contributor now or in future which will be
   necessarily infringed by its Contribution alone, or by combination of the
   Contribution with the Software to which the Contribution was contributed,
   excluding of any patent claims solely be infringed by your or others'
   modification or other combinations. If you or your Affiliates directly or
   indirectly (including through an agent, patent licensee or assignee）,
   institute patent litigation (including a cross claim or counterclaim in a
   litigation) or other patent enforcement activities against any individual or
   entity by alleging that the Software or any Contribution in it infringes
   patents, then any patent license granted to you under this License for the
   Software shall terminate as of the date such litigation or activity is filed
   or taken.

   (( 3. ))??
   No Trademark License

   No trademark license is granted to use the trade names, trademarks, service
   marks, or product names of Contributor, except as required to fulfill notice
   requirements in section 4.

   (( 4. ))??
   Distribution Restriction

   You may distribute the Software in any medium with or without modification,
   whether in source or executable forms, provided that you provide recipients
   with a copy of this License and retain copyright, patent, trademark and
   disclaimer statements in the Software.

   (( 5. ))??
   Disclaimer of Warranty and Limitation of Liability

   The Software and Contribution in it are provided without warranties of any
   kind, either express or implied. In no event shall any Contributor or
   copyright holder be liable to you for any damages,including, but not limited
   to any direct, or indirect, special or consequential damages arising from
   your use or inability to use the Software or the Contribution in it, no
   matter how it's caused or based on which legal theory, even if advised of the
   possibility of such damages.

End of the Terms and Conditions

How to apply the Mulan Permissive Software License，Version 1 (Mulan PSL v1) to
your software

To apply the Mulan PSL v1 to your work, for easy identification by recipients,
you are suggested to complete following three steps:

   (( i. ))??
   Fill in the blanks in following statement, including insert your software
   name, the year of the first publication of your software, and your name
   identified as the copyright owner;

   (( ii. ))??
   Create a file named "LICENSE" which contains the whole context of this
   License in the first directory of your software package;

   (( iii. ))??
   Attach the statement to the appropriate annotated syntax at the beginning of
   each source file.

Copyright (c) [2019] [name of copyright holder]

[Software Name] is licensed under the Mulan PSL v1.

You can use this software according to the terms and conditions of the Mulan PSL
v1.

You may obtain a copy of Mulan PSL v1 at:

http:/license.coscl.org.cn/MulanPSL

THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.

See the Mulan PSL v1 for more details.
`
const license_MulanPSL_2_0_lre = `//**
Mulan Permissive Software License, Version 2
https://spdx.org/licenses/MulanPSL-2.0.json
https://license.coscl.org.cn/MulanPSL2/
**//

(( 木兰宽松许可证, 第2版 ))??

(( 木兰宽松许可证， 第2版

2020年1月 http:/license.coscl.org.cn/MulanPSL2 ))??

您对"软件"的复制、使用、修改及分发受木兰宽松许可证，第2版（"本许可证"）的如下条款的约束：

   (( 0. ))??
   定义

   "软件"
   是指由"贡献"构成的许可在"本许可证"下的程序和相关文档的集合。

   "贡献"
   是指由任一"贡献者"许可在"本许可证"下的受版权法保护的作品。

   "贡献者"
   是指将受版权法保护的作品许可在"本许可证"下的自然人或"法人实体"。

   "法人实体" 是指提交贡献的机构及其"关联实体"。

   "关联实体"
   是指，对"本许可证"下的行为方而言，控制、受控制或与其共同受控制的机构，此处的控制是指有受控方或共同受控方至少50%直接或间接的投票权、资金或其他有价证券。

   (( 1. ))??
   授予版权许可

   每个"贡献者"根据"本许可证"授予您永久性的、全球性的、免费的、非独占的、不可撤销的版权许可，您可以复制、使用、修改、分发其"贡献"，不论修改与否。

   (( 2. ))??
   授予专利许可

   每个"贡献者"根据"本许可证"授予您永久性的、全球性的、免费的、非独占的、不可撤销的（根据本条规定撤销除外）专利许可，供您制造、委托制造、使用、许诺销售、销售、进口其"贡献"或以其他方式转移其"贡献"。前述专利许可仅限于"贡献者"现在或将来拥有或控制的其"贡献"本身或其"贡献"与许可"贡献"时的"软件"结合而将必然会侵犯的专利权利要求，不包括对"贡献"的修改或包含"贡献"的其他结合。如果您或您的"关联实体"直接或间接地，就"软件"或其中的"贡献"对任何人发起专利侵权诉讼（包括反诉或交叉诉讼）或其他专利维权行动，指控其侵犯专利权，则"本许可证"授予您对"软件"的专利许可自您提起诉讼或发起维权行动之日终止。

   (( 3. ))??
   无商标许可

   "本许可证"不提供对"贡献者"的商品名称、商标、服务标志或产品名称的商标许可，但您为满足第4条规定的声明义务而必须使用除外。

   (( 4. ))??
   分发限制

   您可以在任何媒介中将"软件"以源程序形式或可执行形式重新分发，不论修改与否，但您必须向接收者提供"本许可证"的副本，并保留"软件"中的版权、商标、专利及免责声明。

   (( 5. ))??
   免责声明与责任限制

   "软件"及其中的"贡献"在提供时不带任何明示或默示的担保。在任何情况下，"贡献者"或版权所有者不对任何人因使用"软件"或其中的"贡献"而引发的任何直接或间接损失承担责任，不论因何种原因导致或者基于何种法律理论，即使其曾被建议有此种损失的可能性。

   (( 6. ))??
   语言

   "本许可证"以中英文双语表述，中英文版本具有同等法律效力。如果中英文版本存在任何冲突不一致，以中文版为准。

条款结束

如何将木兰宽松许可证，第2版，应用到您的软件

如果您希望将木兰宽松许可证，第2版，应用到您的新软件，为了方便接收者查阅，建议您完成如下三步：

   (( 1， ))??
   请您补充如下声明中的空白，包括软件名、软件的首次发表年份以及您作为版权人的名字；

   (( 2， ))??
   请您在软件包的一级目录下创建以"LICENSE"为名的文件，将整个许可证文本放入该文件中；

   (( 3， ))??
   请将如下声明文本放入每个源文件的头部注释中。

Copyright (c) [Year] [name of copyright holder]

[Software Name] is licensed under Mulan PSL v2.

You can use this software according to the terms and conditions of the Mulan PSL
v2.

You may obtain a copy of Mulan PSL v2 at:

http:/license.coscl.org.cn/MulanPSL2

THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,

EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,

MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.

See the Mulan PSL v2 for more details.
(( Mulan Permissive Software License，Version 2 ))??

(( Mulan Permissive Software License，Version 2 (Mulan PSL v2)

January 2020 http:/license.coscl.org.cn/MulanPSL2 ))??

Your reproduction, use, modification and distribution of the Software shall be
subject to Mulan PSL v2 (this License) with the following terms and conditions:

   (( 0. ))??
   Definition

   Software means the program and related documents which are licensed under
   this License and comprise all Contribution(s).

   Contribution means the copyrightable work licensed by a particular
   Contributor under this License.

   Contributor means the Individual or Legal Entity who licenses its
   copyrightable work under this License.

   Legal Entity means the entity making a Contribution and all its Affiliates.

   Affiliates means entities that control, are controlled by, or are under
   common control with the acting entity under this License, 'control' means
   direct or indirect ownership of at least fifty percent (50%) of the voting
   power, capital or other securities of controlled or commonly controlled
   entity.

   (( 1. ))??
   Grant of Copyright License

   Subject to the terms and conditions of this License, each Contributor hereby
   grants to you a perpetual, worldwide, royalty-free, non-exclusive,
   irrevocable copyright license to reproduce, use, modify, or distribute its
   Contribution, with modification or not.

   (( 2. ))??
   Grant of Patent License

   Subject to the terms and conditions of this License, each Contributor hereby
   grants to you a perpetual, worldwide, royalty-free, non-exclusive,
   irrevocable (except for revocation under this Section) patent license to
   make, have made, use, offer for sale, sell, import or otherwise transfer its
   Contribution, where such patent license is only limited to the patent claims
   owned or controlled by such Contributor now or in future which will be
   necessarily infringed by its Contribution alone, or by combination of the
   Contribution with the Software to which the Contribution was contributed. The
   patent license shall not apply to any modification of the Contribution, and
   any other combination which includes the Contribution. If you or your
   Affiliates directly or indirectly institute patent litigation (including a
   cross claim or counterclaim in a litigation) or other patent enforcement
   activities against any individual or entity by alleging that the Software or
   any Contribution in it infringes patents, then any patent license granted to
   you under this License for the Software shall terminate as of the date such
   litigation or activity is filed or taken.

   (( 3. ))??
   No Trademark License

   No trademark license is granted to use the trade names, trademarks, service
   marks, or product names of Contributor, except as required to fulfill notice
   requirements in section 4.

   (( 4. ))??
   Distribution Restriction

   You may distribute the Software in any medium with or without modification,
   whether in source or executable forms, provided that you provide recipients
   with a copy of this License and retain copyright, patent, trademark and
   disclaimer statements in the Software.

   (( 5. ))??
   Disclaimer of Warranty and Limitation of Liability

   THE SOFTWARE AND CONTRIBUTION IN IT ARE PROVIDED WITHOUT WARRANTIES OF ANY
   KIND, EITHER EXPRESS OR IMPLIED. IN NO EVENT SHALL ANY CONTRIBUTOR OR
   COPYRIGHT HOLDER BE LIABLE TO YOU FOR ANY DAMAGES, INCLUDING, BUT NOT LIMITED
   TO ANY DIRECT, OR INDIRECT, SPECIAL OR CONSEQUENTIAL DAMAGES ARISING FROM
   YOUR USE OR INABILITY TO USE THE SOFTWARE OR THE CONTRIBUTION IN IT, NO
   MATTER HOW IT'S CAUSED OR BASED ON WHICH LEGAL THEORY, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGES.

   (( 6. ))??
   Language

   THIS LICENSE IS WRITTEN IN BOTH CHINESE AND ENGLISH, AND THE CHINESE VERSION
   AND ENGLISH VERSION SHALL HAVE THE SAME LEGAL EFFECT. IN THE CASE OF
   DIVERGENCE BETWEEN THE CHINESE AND ENGLISH VERSIONS, THE CHINESE VERSION
   SHALL PREVAIL.

END OF THE TERMS AND CONDITIONS

How to Apply the Mulan Permissive Software License，Version 2 (Mulan PSL v2) to
Your Software

To apply the Mulan PSL v2 to your work, for easy identification by recipients,
you are suggested to complete following three steps:

   (( i. ))??
   Fill in the blanks in following statement, including insert your software
   name, the year of the first publication of your software, and your name
   identified as the copyright owner;

   (( ii. ))??
   Create a file named "LICENSE" which contains the whole context of this
   License in the first directory of your software package;

   (( iii. ))??
   Attach the statement to the appropriate annotated syntax at the beginning of
   each source file.

Copyright (c) [Year] [name of copyright holder]

[Software Name] is licensed under Mulan PSL v2.

You can use this software according to the terms and conditions of the Mulan PSL
v2.

You may obtain a copy of Mulan PSL v2 at:

http:/license.coscl.org.cn/MulanPSL2

THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,

EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,

MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.

See the Mulan PSL v2 for more details.
`
const license_Multics_lre = `//**
Multics License
https://spdx.org/licenses/Multics.json
https://opensource.org/licenses/Multics
**//

(( Multics License ))??

(( Historical Background

This edition of the Multics software materials and documentation is provided and
donated to Massachusetts Institute of Technology by Group BULL including BULL HN
Information Systems Inc. as a contribution to computer science knowledge. This
donation is made also to give evidence of the common contributions of
Massachusetts Institute of Technology, Bell Laboratories, General Electric,
Honeywell Information Systems Inc., Honeywell BULL Inc., Groupe BULL and BULL HN
Information Systems Inc. to the development of this operating system. Multics
development was initiated by Massachusetts Institute of Technology Project MAC
(1963-1970), renamed the MIT Laboratory for Computer Science and Artificial
Intelligence in the mid 1970s, under the leadership of Professor Fernando Jose
Corbato. Users consider that Multics provided the best software architecture for
managing computer hardware properly and for executing programs. Many subsequent
operating systems incorporated Multics principles. Multics was distributed in
1975 to 2000 by Group Bull in Europe, and in the U.S. by Bull HN Information
Systems Inc., as successor in interest by change in name only to Honeywell Bull
Inc. and Honeywell Information Systems Inc. ))??

Permission to use, copy, modify, and distribute these programs and their
documentation for any purpose and without fee is hereby granted, provided that
the below copyright notice and historical background appear in all copies and
that both the copyright notice and historical background and this permission
notice appear in supporting documentation, and that the names of MIT, HIS, BULL
or BULL HN not be used in advertising or publicity pertaining to distribution of
the programs without specific prior written permission.

((
Copyright 1972 by Massachusetts Institute of Technology and Honeywell Information Systems Inc.
Copyright 2006 by BULL HN Information Systems Inc.
Copyright 2006 by Bull SAS All Rights Reserved
))??

`
const license_Mup_lre = `//**
Mup License
https://spdx.org/licenses/Mup.json
https://fedoraproject.org/wiki/Licensing/Mup
**//

//** Copyright **//

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following DISCLAIMER.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following DISCLAIMER in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   Any additions, deletions, or changes to the original files must be clearly
   indicated in accompanying documentation. including the reasons for the
   changes, and the names of those who made the modifications.

DISCLAIMER

THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
`
const license_NASA_1_3_lre = `//**
NASA Open Source Agreement 1.3
https://spdx.org/licenses/NASA-1.3.json
http://ti.arc.nasa.gov/opensource/nosa/
https://opensource.org/licenses/NASA-1.3
**//

(( NASA OPEN SOURCE AGREEMENT VERSION 1.3 ))??

THIS OPEN SOURCE AGREEMENT ("AGREEMENT") DEFINES THE RIGHTS OF USE,
REPRODUCTION, DISTRIBUTION, MODIFICATION AND REDISTRIBUTION OF CERTAIN COMPUTER
SOFTWARE ORIGINALLY RELEASED BY THE UNITED STATES GOVERNMENT AS REPRESENTED BY
THE GOVERNMENT AGENCY LISTED BELOW ("GOVERNMENT AGENCY"). THE UNITED STATES
GOVERNMENT, AS REPRESENTED BY GOVERNMENT AGENCY, IS AN INTENDED THIRD-PARTY
BENEFICIARY OF ALL SUBSEQUENT DISTRIBUTIONS OR REDISTRIBUTIONS OF THE SUBJECT
SOFTWARE. ANYONE WHO USES, REPRODUCES, DISTRIBUTES, MODIFIES OR REDISTRIBUTES
THE SUBJECT SOFTWARE, AS DEFINED HEREIN, OR ANY PART THEREOF, IS, BY THAT
ACTION, ACCEPTING IN FULL THE RESPONSIBILITIES AND OBLIGATIONS CONTAINED IN THIS
AGREEMENT.

Government Agency: __10__
Government Agency Original Software Designation: __10__
Government Agency Original Software Title: __10__
User Registration Requested. Please Visit __10__
Government Agency Point of Contact for Original Software: __10__

((1.))??
   DEFINITIONS

      (( A. ))??
      "Contributor" means Government Agency, as the developer of the Original
      Software, and any entity that makes a Modification.

      (( B. ))??
      "Covered Patents" mean patent claims licensable by a Contributor that are
      necessarily infringed by the use or sale of its Modification alone or when
      combined with the Subject Software.

      (( C. ))??
      "Display" means the showing of a copy of the Subject Software, either
      directly or by means of an image, or any other device.

      (( D. ))??
      "Distribution" means conveyance or transfer of the Subject Software,
      regardless of means, to another.

      (( E. ))??
      "Larger Work" means computer software that combines Subject Software, or
      portions thereof, with software separate from the Subject Software that is
      not governed by the terms of this Agreement.

      (( F. ))??
      "Modification" means any alteration of, including addition to or deletion
      from, the substance or structure of either the Original Software or
      Subject Software, and includes derivative works, as that term is defined
      in the Copyright Statute, 17 USC 101. However, the act of including
      Subject Software as part of a Larger Work does not in and of itself
      constitute a Modification.

      (( G. ))??
      "Original Software" means the computer software first released under this
      Agreement by Government Agency with Government Agency designation
      __10__ and entitled __10__
      including source code, object code and accompanying documentation, if
      any.

      (( H. ))??
      "Recipient" means anyone who acquires the Subject Software under this
      Agreement, including all Contributors.

      (( I. ))??
      "Redistribution" means Distribution of the Subject Software after a
      Modification has been made.

      (( J. ))??
      "Reproduction" means the making of a counterpart, image or copy of the
      Subject Software.

      (( K. ))??
      "Sale" means the exchange of the Subject Software for money or equivalent
      value.

      (( L. ))??
      "Subject Software" means the Original Software, Modifications, or any
      respective parts thereof.

      (( M. ))??
      "Use" means the application or employment of the Subject Software for any
      purpose.

((2.))??
   GRANT OF RIGHTS

      (( A. ))??
      Under Non-Patent Rights: Subject to the terms and conditions of this
      Agreement, each Contributor, with respect to its own contribution to the
      Subject Software, hereby grants to each Recipient a non-exclusive,
      world-wide, royalty-free license to engage in the following activities
      pertaining to the Subject Software:

         (( 1. ))??
         Use

         (( 2. ))??
         Distribution

         (( 3. ))??
         Reproduction

         (( 4. ))??
         Modification

         (( 5. ))??
         Redistribution

         (( 6. ))??
         Display

      (( B. ))??
      Under Patent Rights: Subject to the terms and conditions of this
      Agreement, each Contributor, with respect to its own contribution to the
      Subject Software, hereby grants to each Recipient under Covered Patents a
      non-exclusive, world-wide, royalty-free license to engage in the following
      activities pertaining to the Subject Software:

         (( 1. ))??
         Use

         (( 2. ))??
         Distribution

         (( 3. ))??
         Reproduction

         (( 4. ))??
         Sale

         (( 5. ))??
         Offer for Sale

      (( C. ))??
      The rights granted under Paragraph B. also apply to the combination of a
      Contributor's Modification and the Subject Software if, at the time the
      Modification is added by the Contributor, the addition of such
      Modification causes the combination to be covered by the Covered Patents.
      It does not apply to any other combinations that include a Modification.

      (( D. ))??
      The rights granted in Paragraphs A. and B. allow the Recipient to
      sublicense those same rights. Such sublicense must be under the same terms
      and conditions of this Agreement.

((3.))??
   OBLIGATIONS OF RECIPIENT

      (( A. ))??
      Distribution or Redistribution of the Subject Software must be made under
      this Agreement except for additions covered under paragraph 3H.

         (( 1. ))??
         Whenever a Recipient distributes or redistributes the Subject Software,
         a copy of this Agreement must be included with each copy of the Subject
         Software; and

         (( 2. ))??
         If Recipient distributes or redistributes the Subject Software in any
         form other than source code, Recipient must also make the source code
         freely available, and must provide with each copy of the Subject
         Software information on how to obtain the source code in a reasonable
         manner on or through a medium customarily used for software exchange.

      (( B. ))??
      Each Recipient must ensure that the following copyright notice appears
      prominently in the Subject Software:

      Copyright (c) __5__ United States Government as represented by __10__.
      (( All Rights Reserved.
      || No copyright is claimed in the United States under Title 17, U.S.Code.
         All Other Rights Reserved.
      ))

      (( C. ))??
      Each Contributor must characterize its alteration of the Subject Software
      as a Modification and must identify itself as the originator of its
      Modification in a manner that reasonably allows subsequent Recipients to
      identify the originator of the Modification. In fulfillment of these
      requirements, Contributor must include a file (e.g., a change log file)
      that describes the alterations made and the date of the alterations,
      identifies Contributor as originator of the alterations, and consents to
      characterization of the alterations as a Modification, for example, by
      including a statement that the Modification is derived, directly or
      indirectly, from Original Software provided by Government Agency. Once
      consent is granted, it may not thereafter be revoked.

      (( D. ))??
      A Contributor may add its own copyright notice to the Subject Software.
      Once a copyright notice has been added to the Subject Software, a
      Recipient may not remove it without the express permission of the
      Contributor who added the notice.

      (( E. ))??
      A Recipient may not make any representation in the Subject Software or in
      any promotional, advertising or other material that may be construed as an
      endorsement by Government Agency or by any prior Recipient of any product
      or service provided by Recipient, or that may seek to obtain commercial
      advantage by the fact of Government Agency's or a prior Recipient's
      participation in this Agreement.

      (( F. ))??
      ((
      In an effort to track usage and maintain accurate records of the Subject
      Software, each Recipient, upon receipt of the Subject Software, is
      requested to register with Government Agency by visiting the following
      website: __10__
      Recipient's name and personal information shall be used for statistical
      purposes only. Once a Recipient makes a Modification available, it is
      requested that the Recipient inform Government Agency at the web site
      provided above how to access the Modification.
      ||
      In an effort to track usage and maintain accurate records of the Subject
      Software, each Recipient, upon receipt of the Subject Software, is
      requested to provide Government Agency, by e-mail to the Government Agency
      Point of Contact listed in clause 5.F., the following information: __10__
      Recipient's name and personal information shall be used for statistical
      purposes only. Once a Recipient makes a Modification available, it is
      requested that the Recipient inform Government Agency, by e-mail to the
      Government Agency Point of Contact listed in clause 5.F., how to access
      the Modification.
      ))

      (( G. ))??
      Each Contributor represents that that its Modification is believed to be
      Contributor's original creation and does not violate any existing
      agreements, regulations, statutes or rules, and further that Contributor
      has sufficient rights to grant the rights conveyed by this Agreement.

      (( H. ))??
      A Recipient may choose to offer, and to charge a fee for, warranty,
      support, indemnity and/or liability obligations to one or more other
      Recipients of the Subject Software. A Recipient may do so, however, only
      on its own behalf and not on behalf of Government Agency or any other
      Recipient. Such a Recipient must make it absolutely clear that any such
      warranty, support, indemnity and/or liability obligation is offered by
      that Recipient alone. Further, such Recipient agrees to indemnify
      Government Agency and every other Recipient for any liability incurred by
      them as a result of warranty, support, indemnity and/or liability offered
      by such Recipient.

      (( I. ))??
      A Recipient may create a Larger Work by combining Subject Software with
      separate software not governed by the terms of this agreement and
      distribute the Larger Work as a single product. In such case, the
      Recipient must make sure Subject Software, or portions thereof, included
      in the Larger Work is subject to this Agreement.

      (( J. ))??
      Notwithstanding any provisions contained herein, Recipient is hereby put
      on notice that export of any goods or technical data from the United
      States may require some form of export license from the U.S. Government.
      Failure to obtain necessary export licenses may result in criminal
      liability under U.S. laws. Government Agency neither represents that a
      license shall not be required nor that, if required, it shall be issued.
      Nothing granted herein provides any such export license.

((4.))??
   DISCLAIMER OF WARRANTIES AND LIABILITIES; WAIVER AND INDEMNIFICATION

      (( A. ))??
      No Warranty: THE SUBJECT SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY
      OF ANY KIND, EITHER EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT
      LIMITED TO, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL CONFORM TO
      SPECIFICATIONS, ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
      PARTICULAR PURPOSE, OR FREEDOM FROM INFRINGEMENT, ANY WARRANTY THAT THE
      SUBJECT SOFTWARE WILL BE ERROR FREE, OR ANY WARRANTY THAT DOCUMENTATION,
      IF PROVIDED, WILL CONFORM TO THE SUBJECT SOFTWARE. THIS AGREEMENT DOES
      NOT, IN ANY MANNER, CONSTITUTE AN ENDORSEMENT BY GOVERNMENT AGENCY OR ANY
      PRIOR RECIPIENT OF ANY RESULTS, RESULTING DESIGNS, HARDWARE, SOFTWARE
      PRODUCTS OR ANY OTHER APPLICATIONS RESULTING FROM USE OF THE SUBJECT
      SOFTWARE. FURTHER, GOVERNMENT AGENCY DISCLAIMS ALL WARRANTIES AND
      LIABILITIES REGARDING THIRD-PARTY SOFTWARE, IF PRESENT IN THE ORIGINAL
      SOFTWARE, AND DISTRIBUTES IT "AS IS."

      (( B. ))??
      Waiver and Indemnity: RECIPIENT AGREES TO WAIVE ANY AND ALL CLAIMS AGAINST
      THE UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS, AS WELL
      AS ANY PRIOR RECIPIENT. IF RECIPIENT'S USE OF THE SUBJECT SOFTWARE RESULTS
      IN ANY LIABILITIES, DEMANDS, DAMAGES, EXPENSES OR LOSSES ARISING FROM SUCH
      USE, INCLUDING ANY DAMAGES FROM PRODUCTS BASED ON, OR RESULTING FROM,
      RECIPIENT'S USE OF THE SUBJECT SOFTWARE, RECIPIENT SHALL INDEMNIFY AND
      HOLD HARMLESS THE UNITED STATES GOVERNMENT, ITS CONTRACTORS AND
      SUBCONTRACTORS, AS WELL AS ANY PRIOR RECIPIENT, TO THE EXTENT PERMITTED BY
      LAW. RECIPIENT'S SOLE REMEDY FOR ANY SUCH MATTER SHALL BE THE IMMEDIATE,
      UNILATERAL TERMINATION OF THIS AGREEMENT.

((5.))??
   GENERAL TERMS

      (( A. ))??
      Termination: This Agreement and the rights granted hereunder will
      terminate automatically if a Recipient fails to comply with these terms
      and conditions, and fails to cure such noncompliance within thirty (30)
      days of becoming aware of such noncompliance. Upon termination, a
      Recipient agrees to immediately cease use and distribution of the Subject
      Software. All sublicenses to the Subject Software properly granted by the
      breaching Recipient shall survive any such termination of this Agreement.

      (( B. ))??
      Severability: If any provision of this Agreement is invalid or
      unenforceable under applicable law, it shall not affect the validity or
      enforceability of the remainder of the terms of this Agreement.

      (( C. ))??
      Applicable Law: This Agreement shall be subject to United States federal
      law only for all purposes, including, but not limited to, determining the
      validity of this Agreement, the meaning of its provisions and the rights,
      obligations and remedies of the parties.

      (( D. ))??
      Entire Understanding: This Agreement constitutes the entire understanding
      and agreement of the parties relating to release of the Subject Software
      and may not be superseded, modified or amended except by further written
      agreement duly executed by the parties.

      (( E. ))??
      Binding Authority: By accepting and using the Subject Software under this
      Agreement, a Recipient affirms its authority to bind the Recipient to all
      terms and conditions of this Agreement and that that Recipient hereby
      agrees to all terms and conditions herein.

      (( F. ))??
      Point of Contact: Any Recipient contact with Government Agency is to be
      directed to the designated representative as follows:
`
const license_NBPL_1_0_lre = `//**
Net Boolean Public License v1
https://spdx.org/licenses/NBPL-1.0.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=37b4b3f6cc4bf34e1d3dec61e69914b9819d8894
**//

(( The Net Boolean Public License
(( Version 1, 22 August 1998 ))??
(( Copyright __20__ ))??
))??

(( Note: This license is derived from the "Artistic License" as distributed with
the Perl Programming Language. Its terms are different from those of the
"Artistic License."

PREAMBLE

The intent of this document is to state the conditions under which a Package may
be copied, such that the Copyright Holder maintains some semblance of artistic
control over the development of the package, while giving the users of the
package the right to use and distribute the Package in a more-or-less customary
fashion, plus the right to make reasonable modifications. ))??

Definitions:

"Package" refers to the collection of files distributed by the Copyright Holder,
and derivatives of that collection of files created through textual
modification.

"Standard Version" refers to such a Package if it has not been modified, or has
been modified in accordance with the wishes of the Copyright Holder.

"Copyright Holder" is whoever is named in the copyright or copyrights for the
package.

"You" is you, if you're thinking about copying or distributing this Package.

"Reasonable copying fee" is whatever you can justify on the basis of media cost,
duplication charges, time of people involved, and so on. (You will not be
required to justify it to the Copyright Holder, but only to the computing
community at large as a market that must bear the fee.)

"Freely Available" means that no fee is charged for the item itself, though
there may be fees involved in handling the item. It also means that recipients
of the item may redistribute it under the same conditions they received it.

   (( 1. ))??
   You may make and give away verbatim copies of the source form of the Standard
   Version of this Package without restriction, provided that you duplicate all
   of the original copyright notices and associated disclaimers.

   (( 2. ))??
   You may apply bug fixes, portability fixes and other modifications derived
   from the Public Domain or from the Copyright Holder. A Package modified in
   such a way shall still be considered the Standard Version.

   (( 3. ))??
   You may otherwise modify your copy of this Package in any way, provided that
   you insert a prominent notice in each changed file stating how and when you
   changed that file, and provided that you do at least ONE of the following:

      (( a) ))??
      place your modifications in the Public Domain or otherwise make them
      Freely Available, such as by posting said modifications to Usenet or an
      equivalent medium, or placing the modifications on a major archive site
      such as uunet.uu.net, or by allowing the Copyright Holder to include your
      modifications in the Standard Version of the Package.

      (( b) ))??
      use the modified Package only within your corporation or organization.

      (( c) ))??
      rename any non-standard executables so the names do not conflict with
      standard executables, which must also be provided, and provide a separate
      manual page for each non-standard executable that clearly documents how it
      differs from the Standard Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 4. ))??
   You may distribute the programs of this Package in object code or executable
   form, provided that you do at least ONE of the following:

      (( a) ))??
      distribute a Standard Version of the executables and library files,
      together with instructions (in the manual page or equivalent) on where to
      get the Standard Version.

      (( b) ))??
      accompany the distribution with the machine-readable source of the Package
      with your modifications.

      (( c) ))??
      accompany any non-standard executables with their corresponding Standard
      Version executables, giving the non-standard executables non-standard
      names, and clearly documenting the differences in manual pages (or
      equivalent), together with instructions on where to get the Standard
      Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 5. ))??
   You may charge a reasonable copying fee for any distribution of this Package.
   You may charge any fee you choose for support of this Package. You may not
   charge a fee for this Package itself. However, you may distribute this
   Package in aggregate with other (possibly commercial) programs as part of a
   larger (possibly commercial) software distribution provided that you do not
   advertise this Package as a product of your own.

   (( 6. ))??
   The scripts and library files supplied as input to or produced as output from
   the programs of this Package do not automatically fall under the copyright of
   this Package, but belong to whomever generated them, and may be sold
   commercially, and may be aggregated with this Package.

   (( 7. ))??
   C subroutines supplied by you and linked into this Package in order to
   emulate subroutines and variables of the language defined by this Package
   shall not be considered part of this Package, but are the equivalent of input
   as in Paragraph 6, provided these subroutines do not change the language in
   any way that would cause it to fail the regression tests for the language.

   (( 8. ))??
   The name of the Copyright Holder may not be used to endorse or promote
   products derived from this software without specific prior written
   permission.

   (( 9. ))??
   THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
   (( The End ))??
`
const license_NCGL_UK_2_0_lre = `//**
Non-Commercial Government Licence
https://spdx.org/licenses/NCGL-UK-2.0.json
https://github.com/spdx/license-list-XML/blob/master/src/Apache-2.0.xml
**//

(( Non-Commercial Government Licence

for public sector information ))??

You are encouraged to use and re-use the Information that is available under
this licence freely and flexibly, with only a few conditions.

Using information under this licence

Use of copyright and database right material expressly made available under this
licence (the 'Information') indicates your acceptance of the terms and
conditions below.

The Licensor grants you a worldwide, royalty-free, perpetual, non-exclusive
licence to use the Information for Non-Commercial purposes only subject to the
conditions below.

This licence does not affect your freedom under fair dealing or fair use or any
other copyright or database right exceptions and limitations.

You are free to:

   copy, publish, distribute and transmit the Information;

   adapt the Information;

   exploit the Information for Non-Commercial purposes for example, by combining
   it with other information in your own product or application.

You are not permitted to:

   exercise any of the rights granted to you by this licence in any manner that
   is primarily intended for or directed toward commercial advantage or private
   monetary compensation.

You must, where you do any of the above:

   acknowledge the source of the Information by including any attribution
   statement specified by the Information Provider(s) and, where possible,
   provide a link to this licence;

   If the Information Provider does not provide a specific attribution
   statement, you must use the following:

   Contains information licensed under the Non-Commercial Government Licence
   v2.0.

   If you are using Information from several Information Providers and listing
   multiple attributions is not practical in your product or application, you
   may include a URI or hyperlink to a resource that contains the required
   attribution statements.

   ensure that any onward licensing of the Information – for example when
   combined with other information – is for Non-Commercial purposes only.

   These are important conditions of this licence and if you fail to comply with
   them or use the Information other than for Non-Commercial purposes the rights
   granted to you under this licence, or any similar licence granted by the
   Licensor, will end automatically.

Exemptions

This licence does not cover the use of:

   •
   personal data in the Information;

   •
   Information that has not been accessed by way of publication or disclosure
   under information access legislation (including the Freedom of Information
   Acts for the UK and Scotland) by or with the consent of the Information
   Provider;

   •
   departmental or public sector organisation logos, crests, military insignia
   and the Royal Arms except where they form an integral part of a document or
   dataset;

   •
   military insignia

   •
   third party rights the Information Provider is not authorised to license;

   •
   other intellectual property rights, including patents, trade marks, and
   design rights; and

   •
   identity documents such as the British Passport.

Non-endorsement

This licence does not grant you any right to use the Information in a way that
suggests any official status or that the Information Provider and/or Licensor
endorse you or your use of the Information.

No warranty

The Information is licensed 'as is' and the Information Provider excludes all
representations, warranties, obligations and liabilities in relation to the
Information to the maximum extent permitted by law.

The Information Provider is not liable for any errors or omissions in the
Information and shall not be liable for any loss, injury or damage of any kind
caused by its use. The Information Provider does not guarantee the continued
supply of the Information.

Governing Law

This licence is governed by the laws of the jurisdiction in which the
Information Provider has its principal place of business, unless otherwise
specified by the Information Provider.

Definitions

In this licence the terms below have the following meanings:

'Information' means information protected by copyright or by database right (for
example, literary and artistic works, content, data and source code) offered for
use under the terms of this licence.

'Information Provider' means the person or organisation providing the
Information under this licence.

'Licensor' means any Information Provider which has the authority to offer
Information under the terms of this licence or the Keeper of the Public Records,
who has the authority to offer Information subject to Crown copyright and Crown
database rights and Information subject to copyright and database right that has
been assigned to or acquired by the Crown, under the terms of this licence.

'Non-Commercial purposes' means not intended for or directed toward commercial
advantage or private monetary compensation. For the purposes of this licence,
'private monetary compensation' does not include the exchange of the Information
for other copyrighted works by means of digital file-sharing or otherwise
provided there is no payment of any monetary compensation in connection with the
exchange of the Information.

'Use' as a verb, means doing any act which is restricted by copyright or
database right, whether in the original medium or in any other medium, and
includes without limitation distributing, copying, adapting, modifying as may be
technically necessary to use it in a different mode or format.

'You' means the natural or legal person, or body of persons corporate or
incorporate, acquiring rights under this licence.
`
const license_NCSA_lre = `//**
University of Illinois/NCSA Open Source License
https://spdx.org/licenses/NCSA.json
http://otm.illinois.edu/uiuc_openSource
https://opensource.org/licenses/NCSA
**//

(( University of Illinois/NCSA Open Source License
(( Copyright __20__ ))??
))??

((Developed by: __30__))??

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal with
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

   *
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimers.

   *
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimers in the documentation
   and/or other materials provided with the distribution.

   *
   Neither the names of __10__
   nor the names of its contributors may be used to endorse or promote
   products derived from this Software without specific prior written
   permission.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE CONTRIBUTORS
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
`
const license_NGPL_lre = `//**
Nethack General Public License
https://spdx.org/licenses/NGPL.json
https://opensource.org/licenses/NGPL
**//

(( NETHACK GENERAL PUBLIC LICENSE
(( Copyright __20__ ))??
))??

(Based on the BISON general public license, copyright 1988 Richard M. Stallman)

Everyone is permitted to copy and distribute verbatim copies of this license,
but changing it is not allowed. You can also use this wording to make the terms
for other programs.

The license agreements of most software companies keep you at the mercy of those
companies. By contrast, our general public license is intended to give everyone
the right to share NetHack. To make sure that you get the rights we want you to
have, we need to make restrictions that forbid anyone to deny you these rights
or to ask you to surrender the rights. Hence this license agreement.

Specifically, we want to make sure that you have the right to give away copies
of NetHack, that you receive source code or else can get it if you want it, that
you can change NetHack or use pieces of it in new free programs, and that you
know you can do these things.

To make sure that everyone has such rights, we have to forbid you to deprive
anyone else of these rights. For example, if you distribute copies of NetHack,
you must give the recipients all the rights that you have. You must make sure
that they, too, receive or can get the source code. And you must tell them their
rights.

Also, for our own protection, we must make certain that everyone finds out that
there is no warranty for NetHack. If NetHack is modified by someone else and
passed on, we want its recipients to know that what they have is not what we
distributed.

Therefore we (Mike Stephenson and other holders of NetHack copyrights) make the
following terms which say what you must do to be allowed to distribute or change
NetHack.

COPYING POLICIES

   (( 1. ))??
   You may copy and distribute verbatim copies of NetHack source code as you
   receive it, in any medium, provided that you keep intact the notices on all
   files that refer to copyrights, to this License Agreement, and to the absence
   of any warranty; and give any other recipients of the NetHack program a copy
   of this License Agreement along with the program.

   (( 2. ))??
   You may modify your copy or copies of NetHack or any portion of it, and copy
   and distribute such modifications under the terms of Paragraph 1 above
   (including distributing this License Agreement), provided that you also do
   the following:

      (( a) ))??
      cause the modified files to carry prominent notices stating that you
      changed the files and the date of any change; and

      (( b) ))??
      cause the whole of any work that you distribute or publish, that in whole
      or in part contains or is a derivative of NetHack or any part thereof, to
      be licensed at no charge to all third parties on terms identical to those
      contained in this License Agreement (except that you may choose to grant
      more extensive warranty protection to some or all third parties, at your
      option)

      (( c) ))??
      You may charge a distribution fee for the physical act of transferring a
      copy, and you may at your option offer warranty protection in exchange for
      a fee.

   (( 3. ))??
   You may copy and distribute NetHack (or a portion or derivative of it, under
   Paragraph 2) in object code or executable form under the terms of Paragraphs
   1 and 2 above provided that you also do one of the following:

      (( a) ))??
      accompany it with the complete machine-readable source code, which must be
      distributed under the terms of Paragraphs 1 and 2 above; or,

      (( b) ))??
      accompany it with full information as to how to obtain the complete
      machine-readable source code from an appropriate archive site. (This
      alternative is allowed only for noncommercial distribution.)

   For these purposes, complete source code means either the full source
   distribution as originally released over Usenet or updated copies of the
   files in this distribution used to create the object code or executable.

   (( 4. ))??
   You may not copy, sublicense, distribute or transfer NetHack except as
   expressly provided under this License Agreement. Any attempt otherwise to
   copy, sublicense, distribute or transfer NetHack is void and your rights to
   use the program under this License agreement shall be automatically
   terminated. However, parties who have received computer software programs
   from you with this License Agreement will not have their licenses terminated
   so long as such parties remain in full compliance.

Stated plainly: You are permitted to modify NetHack, or otherwise use parts of
NetHack, provided that you comply with the conditions specified above; in
particular, your modified NetHack or program containing parts of NetHack must
remain freely available as provided in this License Agreement. In other words,
go ahead and share NetHack, but don't try to stop anyone else from sharing it
farther.
`
const license_NIST_PD_lre = `//**
NIST Public Domain Notice
https://spdx.org/licenses/NIST-PD.json
https://github.com/tcheneau/simpleRPL/blob/e645e69e38dd4e3ccfeceb2db8cba05b7c2e0cd3/LICENSE.txt
https://github.com/tcheneau/Routing/blob/f09f46fcfe636107f22f2c98348188a65a135d98/README.md
**//

(( Terms of Use ))??

This software was developed by employees of the National Institute of Standards
and Technology (NIST), and others. This software has been contributed to the
public domain. Pursuant to title 15 Untied States Code Section 105, works of
NIST employees are not subject to copyright protection in the United States and
are considered to be in the public domain. As a result, a formal license is not
needed to use this software.

This software is provided "AS IS." NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS,
IMPLIED OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA
ACCURACY. NIST does not warrant or make any representations regarding the use of
the software or the results thereof, including but not limited to the
correctness, accuracy, reliability or usefulness of this software.
`
const license_NIST_PD_fallback_lre = `//**
NIST Public Domain Notice with license fallback
https://spdx.org/licenses/NIST-PD-fallback.json
https://github.com/usnistgov/jsip/blob/59700e6926cbe96c5cdae897d9a7d2656b42abe3/LICENSE
https://github.com/usnistgov/fipy/blob/86aaa5c2ba2c6f1be19593c5986071cf6568cc34/LICENSE.rst
**//

(( Conditions of Use ))??

This software was developed by employees of the National Institute of Standards
and Technology (NIST), an agency of the Federal Government and is being made
available as a public service. Pursuant to title 17 United States Code Section
105, works of NIST employees are not subject to copyright protection in the
United States. This software may be subject to foreign copyright. Permission in
the United States and in foreign countries, to the extent that NIST may hold
copyright, to use, copy, modify, create derivative works, and distribute this
software and its documentation without fee is hereby granted on a non-exclusive
basis, provided that this notice and disclaimer of warranty appears in all
copies.

THE SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY OF ANY KIND, EITHER
EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, ANY WARRANTY
THAT THE SOFTWARE WILL CONFORM TO SPECIFICATIONS, ANY IMPLIED WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND FREEDOM FROM
INFRINGEMENT, AND ANY WARRANTY THAT THE DOCUMENTATION WILL CONFORM TO THE
SOFTWARE, OR ANY WARRANTY THAT THE SOFTWARE WILL BE ERROR FREE. IN NO EVENT
SHALL NIST BE LIABLE FOR ANY DAMAGES, INCLUDING, BUT NOT LIMITED TO, DIRECT,
INDIRECT, SPECIAL OR CONSEQUENTIAL DAMAGES, ARISING OUT OF, RESULTING FROM, OR
IN ANY WAY CONNECTED WITH THIS SOFTWARE, WHETHER OR NOT BASED UPON WARRANTY,
CONTRACT, TORT, OR OTHERWISE, WHETHER OR NOT INJURY WAS SUSTAINED BY PERSONS OR
PROPERTY OR OTHERWISE, AND WHETHER OR NOT LOSS WAS SUSTAINED FROM, OR AROSE OUT
OF THE RESULTS OF, OR USE OF, THE SOFTWARE OR SERVICES PROVIDED HEREUNDER.
`
const license_NLOD_1_0_lre = `//**
Norwegian Licence for Open Government Data
https://spdx.org/licenses/NLOD-1.0.json
http://data.norge.no/nlod/en/1.0
**//

(( Norwegian Licence for Open Government Data (NLOD) ))??

Preface of licence

This licence grants you the right to copy, use and distribute information,
provided you acknowledge the contributors and comply with the terms and
conditions stipulated in this licence. By using information made available under
this licence, you accept the terms and conditions set forth in this licence. As
set out in Section 7, the licensor disclaims any and all liability for the
quality of the information and what the information is used for.

This licence shall not impose any limitations on the rights or freedoms of the
licensee under the Norwegian Freedom of Information Act or any other legislation
granting the general public a right of access to public sector information, or
that follow from exemptions or limitations stipulated in the Norwegian Copyright
Act. Further, the licence shall not impose any limitations on the licensee's
freedom of expression recognized by law.

   (( 1. ))??
   Definitions

   «Database» shall mean a database or similar protected under Section 43 of
   the Norwegian Copyright Act.

   «Information» shall mean texts, images, recordings, data sets or other
   works protected under Section 1 of the Norwegian Copyright Act, or which are
   protected under provisions addressing what is referred to as «neighbouring
   rights» in Chapter 5 of the Norwegian Copyright Act (including databases and
   photographs), and which are distributed under this licence.

   «Copy» shall mean reproduction in any form.

   «Licensee» and «you» shall mean natural or legal persons using
   information under this licence.

   «Licensor» shall mean the natural or legal person that makes information
   available under this licence.

   «Distribute» shall mean any actions whereby information is made available,
   including to distribute, transfer, communicate, disperse, show, perform,
   sell, lend and rent.

   «Use» shall mean one or more actions relevant to copyright law requiring
   permission from the owner of the copyright.

   (( 2. ))??
   Licence

   The licensee, subject to the limitations that follow from this licence, may
   use the information for any purpose and in all contexts, by:

   *
   copying the information and distributing the information to others,

   *
   modifying the information and/or combining the information with other
   information, and

   *
   copying and distributing such changed or combined information.

   *
   This is a non-exclusive, free, perpetual and worldwide licence. The
   information may be used in any medium and format known today and/or which
   will become known in the future. The Licensee shall not sub-license or
   transfer this licence.

   (( 3. ))??
   Exemptions

   The licence does not apply to and therefore does not grant a right to use:

   *
   information which contains personal data covered by the Norwegian Personal
   Data Act unless there is a legitimate basis for the disclosure and further
   processing of the personal data

   *
   information distributed in violation of a statutory obligation to observe
   confidentiality

   *
   information excluded from public disclosure pursuant to law, including
   information deemed sensitive under the Norwegian National Security Act

   *
   information subject to third party rights which the licensor is not
   authorised to license to the licensee

   *
   information protected by intellectual property rights other than copyright
   and neighbouring rights in accordance with Chapter 5 of the Norwegian
   Copyright Act, such as trademarks, patents and design rights, but this does
   not entail an impediment to use information where the licensor's logo has
   been permanently integrated into the information or to attribute the origin
   of the information in accordance with the article below relating to
   attribution.

   If the licensor has made available information not covered by the licence
   according to the above list, the licensee must cease all use of the
   information under the licence, and erase the information as soon as he or she
   becomes aware of or should have understood that the information is not
   covered by the licence.

   (( 4. ))??
   Effects of breach of the licence

   The licence is subject to the licensee's compliance with the terms and
   conditions of this licence. In the event that the licensee commits a breach
   of this licence, this will entail that the licensee's right to use the
   information will be revoked immediately without further notice. In case of
   such a breach, the licensee must immediately and without further notice take
   measures to cause the infringement to end. Because the right to use the
   information has been terminated, the licensee must cease all use of the
   information by virtue of the licence.

   (( 5. ))??
   Attribution

   The licensee shall attribute the licensor as specified by the licensor and
   include a reference to this licence. To the extent practically possible, the
   licensee shall provide a link to both this licence and the source of the
   information.

   If the licensor has not specified how attributions shall be made, the
   licensee shall normally state the following: «Contains data under the
   Norwegian licence for Open Government data (NLOD) distributed by [name of
   licensor]».

   If the licensor has specified that the information shall only be available
   under a specific version of this licence, cf. Section 10, the licensee shall
   also state this.

   If the information has been changed, the licensee must clearly indicate that
   changes have been made by the licensee.

   (( 6. ))??
   Proper use

   The licensee shall not use the information in a manner that appears
   misleading nor present the information in a distorted or incorrect manner.

   Neither the licensor's nor other contributors' names or trademarks must be
   used to support, recommend or market the licensee or any products or services
   using the information.

   (( 7. ))??
   Disclaimer of liability

   The information is licensed «as is». The information may contain errors and
   omissions. The licensor provides no warranties, including relating to the
   content and relevance of the information.

   The licensor disclaims any liability for errors and defects associated with
   the information to the maximum extent permitted by law.

   The licensor shall not be liable for direct or indirect losses as a result of
   use of the information or in connection with copying or further distribution
   of the information.

   (( 8. ))??
   Guarantees regarding data quality and accessibility

   This licence does not prevent the licensor from issuing supplementary
   statements regarding expected or intended data quality and accessibility.
   Such statements shall be regarded as indicative in nature and not binding on
   the part of the licensor. The disclaimers in Section 7 also apply in full for
   such indicative statements. Based on separate agreement, the licensor may
   provide guarantees and distribute the information on terms and conditions
   different from those set forth in this licence.

   (( 9. ))??
   Licence compatibility

   If the licensee is to distribute an adapted or combined work based on
   information covered by this licence and some other work licensed under a
   licence compatible by contract, such distribution may be based on an
   appropriate licence compatible by contract, cf. the list below.

   A licence compatible by contract shall mean the following licences:

   *
   for all information: Open Government Licence (version 1.0),

   *
   for those parts of the information which do not constitute databases:
   Creative Commons Attribution Licence (generic version 1.0, 2.0, 2.5 and
   unported version 3.0) and Creative Commons Navngivelse 3.0 Norge,

   *
   for those parts of the information which constitute databases: Open Data
   Commons Attribution License (version 1.0).

   This provision does not prevent other licences from being compatible with
   this licence based on their content.

   (( 10. ))??
   New versions of the licence

   The licensee may choose to use the information covered by this licence under
   any new versions of the Norwegian licence for Open Government data (NLOD)
   issued by the responsible ministry (currently the Ministry of Government
   Administration, Reform and Church Affairs) when these versions are final and
   official, unless the licensor when making the information available under
   this licence specifically has stated that solely version 1.0 of this licence
   may be used.

   (( 11. ))??
   Governing law and legal venue

This licence, including its formation, and any disputes and claims arising in
connection with or relating to this licence, shall be regulated by Norwegian
law. The legal venue shall be the licensor's ordinary legal venue. The licensor
may, with regard to intellectual proprietary rights, choose to pursue a claim at
other competent legal venues and/or based on the laws of the country where the
intellectual property rights are sought enforced.
`
const license_NLPL_lre = `//**
No Limit Public License
https://spdx.org/licenses/NLPL.json
https://fedoraproject.org/wiki/Licensing/NLPL
**//

(( NO LIMIT PUBLIC LICENSE

Version 0, June 2012 ))??

(( Gilles LAMIRAL

La Billais

35580 Baulon

France ))??

(( NO LIMIT PUBLIC LICENSE ))??

Terms and conditions for copying, distribution, modification or anything else.

   (( 0. ))??
   No limit to do anything with this work and this license.
`
const license_NOSL_lre = `//**
Netizen Open Source License
https://spdx.org/licenses/NOSL.json
http://bits.netizen.com.au/licenses/NOSL/nosl.txt
**//

(( NETIZEN OPEN SOURCE LICENSE

Version 1.0 ))??

   (( 1. ))??
   Definitions.

      (( 1.0.1. ))??
      "Commercial Use" means distribution or otherwise making the Covered Code
      available to a third party.

      (( 1.1. ))??
      "Contributor" means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      "Covered Code" means the Original Code or Modifications or the combination
      of the Original Code and Modifications, in each case including portions
      thereof.

      (( 1.4. ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      "Executable" means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      "Initial Developer" means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7. ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.8. ))??
      "License" means this document.

      (( 1.8.1. ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.9. ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10. ))??
      "Original Code" means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      (( 1.10.1. ))??
      "Patent Claims" means any patent claim(s), now owned or hereafter
      acquired, including without limitation, method, process, and apparatus
      claims, in any patent Licensable by grantor.

      (( 1.11. ))??
      "Source Code" means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or source code differential comparisons
      against either the Original Code or another well known, available Covered
      Code of the Contributor's choice. The Source Code can be in a compressed
      or archival form, provided the appropriate decompression or de-archiving
      software is widely available for no charge.

      (( 1.12. ))??
      "You" (or "Your") means an individual or a legal entity exercising rights
      under, and complying with all of the terms of, this License or a future
      version of this License issued under Section 6.1. For legal entities,
      "You" includes any entity which controls, is controlled by, or is under
      common control with You. For purposes of this definition, "control" means
      (a) the power, direct or indirect, to cause the direction or management of
      such entity, whether by contract or otherwise, or (b) ownership of more
      than fifty percent (50%) of the outstanding shares or beneficial ownership
      of such entity.

   (( 2. ))??
   Source Code License.

      (( 2.1. ))??
      The Initial Developer Grant. The Initial Developer hereby grants You a
      world-wide, royalty-free, non-exclusive license, subject to third party
      intellectual property claims:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Initial Developer to use, reproduce, modify, display,
         perform, sublicense and distribute the Original Code (or portions
         thereof) with or without Modifications, and/or as part of a Larger
         Work; and

         (( (b) ))??
         under Patents Claims infringed by the making, using or selling of
         Original Code, to make, have made, use, practice, sell, and offer for
         sale, and/or otherwise dispose of the Original Code (or portions
         thereof).

         (( c ))??
         the licenses granted in this Section 2.1(a) and (b) are effective on
         the date Initial Developer first distributes Original Code under the
         terms of this License.

         (( (d) ))??
         Notwithstanding Section 2.1(b) above, no patent license is granted: 1)
         for code that You delete from the Original Code; 2) separate from the
         Original Code; or 3) for infringements caused by: i) the modification
         of the Original Code or ii) the combination of the Original Code with
         other software or devices.

      (( 2.2. ))??
      Contributor Grant.

      Subject to third party intellectual property claims, each Contributor
      hereby grants You a world-wide, royalty-free, non-exclusive license

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Contributor, to use, reproduce, modify, display, perform,
         sublicense and distribute the Modifications created by such Contributor
         (or portions thereof) either on an unmodified basis, with other
         Modifications, as Covered Code and/or as part of a Larger Work; and

         (( (b) ))??
         under Patent Claims infringed by the making, using, or selling of
         Modifications made by that Contributor either alone and/or in
         combination with its Contributor Version (or portions of such
         combination), to make, use, sell, offer for sale, have made, and/or
         otherwise dispose of: 1) Modifications made by that Contributor (or
         portions thereof); and 2) the combination of Modifications made by that
         Contributor with its Contributor Version (or portions of such
         combination).

         (( c ))??
         the licenses granted in Sections 2.2(a) and 2.2(b) are effective on the
         date Contributor first makes Commercial Use of the Covered Code.

         (( (d) ))??
         Notwithstanding Section 2.2(b) above, no patent license is granted: 1)
         for any code that Contributor has deleted from the Contributor Version;
         2) separate from the Contributor Version; 3) for infringements caused
         by: i) third party modifications of Contributor Version or ii) the
         combination of Modifications made by that Contributor with other
         software (except as part of the Contributor Version) or other devices;
         or 4) under Patent Claims infringed by Covered Code in the absence of
         Modifications made by that Contributor.

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Application of License. The Modifications which You create or to which You
      contribute are governed by the terms of this License, including without
      limitation Section 2.2. The Source Code version of Covered Code may be
      distributed only under the terms of this License or a future version of
      this License released under Section 6.1, and You must include a copy of
      this License with every copy of the Source Code You distribute. You may
      not offer or impose any terms on any Source Code version that alters or
      restricts the applicable version of this License or the recipients' rights
      hereunder. However, You may include an additional document offering the
      additional rights described in Section 3.5.

      (( 3.2. ))??
      Availability of Source Code. Any Modification which You create or to which
      You contribute must be made available in Source Code form under the terms
      of this License either on the same media as an Executable version or via
      an accepted Electronic Distribution Mechanism to anyone to whom you made
      an Executable version available; and if made available via Electronic
      Distribution Mechanism, must remain available for at least twelve (12)
      months after the date it initially became available, or at least six (6)
      months after a subsequent version of that particular Modification has been
      made available to such recipients. You are responsible for ensuring that
      the Source Code version remains available even if the Electronic
      Distribution Mechanism is maintained by a third party.

      (( 3.3. ))??
      Description of Modifications. You must cause all Covered Code to which You
      contribute to contain a file documenting the changes You made to create
      that Covered Code and the date of any change. You must include a prominent
      statement that the Modification is derived, directly or indirectly, from
      Original Code provided by the Initial Developer and including the name of
      the Initial Developer in (a) the Source Code, and (b) in any notice in an
      Executable version or related documentation in which You describe the
      origin or ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters

         (( (a) ))??
         Third Party Claims.

         If Contributor has knowledge that a license under a third party's
         intellectual property rights is required to exercise the rights granted
         by such Contributor under Sections 2.1 or 2.2, Contributor must include
         a text file with the Source Code distribution titled "LEGAL" which
         describes the claim and the party making the claim in sufficient detail
         that a recipient will know whom to contact. If Contributor obtains such
         knowledge after the Modification is made available as described in
         Section 3.2, Contributor shall promptly modify the LEGAL file in all
         copies Contributor makes available thereafter and shall take other
         steps (such as notifying appropriate mailing lists or newsgroups)
         reasonably calculated to inform those who received the Covered Code
         that new knowledge has been obtained.

         (( (b) ))??
         Contributor APIs.

         If Contributor's Modifications include an application programming
         interface and Contributor has knowledge of patent licenses which are
         reasonably necessary to implement that API, Contributor must also
         include this information in the LEGAL file.

         (( c ))??
         Representations.

         Contributor represents that, except as disclosed pursuant to Section
         3.4(a) above, Contributor believes that Contributor's Modifications are
         Contributor's original creation(s) and/or Contributor has sufficient
         rights to grant the rights conveyed by this License.

      (( 3.5. ))??
      Required Notices.

      You must duplicate the notice in Exhibit A in each file of the Source
      Code. If it is not possible to put such notice in a particular Source Code
      file due to its structure, then You must include such notice in a location
      (such as a relevant directory) where a user would be likely to look for
      such a notice. If You created one or more Modification(s) You may add your
      name as a Contributor to the notice described in Exhibit A. You must also
      duplicate this License in any documentation for the Source Code where You
      describe recipients' rights or ownership rights relating to Covered Code.
      You may choose to offer, and to charge a fee for, warranty, support,
      indemnity or liability obligations to one or more recipients of Covered
      Code. However, You may do so only on Your own behalf, and not on behalf of
      the Initial Developer or any Contributor. You must make it absolutely
      clear than any such warranty, support, indemnity or liability obligation
      is offered by You alone, and You hereby agree to indemnify the Initial
      Developer and every Contributor for any liability incurred by the Initial
      Developer or such Contributor as a result of warranty, support, indemnity
      or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Versions.

      You may distribute Covered Code in Executable form only if the
      requirements of Section 3.1-3.5 have been met for that Covered Code, and
      if You include a notice stating that the Source Code version of the
      Covered Code is available under the terms of this License, including a
      description of how and where You have fulfilled the obligations of Section
      3.2. The notice must be conspicuously included in any notice in an
      Executable version, related documentation or collateral in which You
      describe recipients' rights relating to the Covered Code. You may
      distribute the Executable version of Covered Code or ownership rights
      under a license of Your choice, which may contain terms different from
      this License, provided that You are in compliance with the terms of this
      License and that the license for the Executable version does not attempt
      to limit or alter the recipient's rights in the Source Code version from
      the rights set forth in this License. If You distribute the Executable
      version under a different license You must make it absolutely clear that
      any terms which differ from this License are offered by You alone, not by
      the Initial Developer or any Contributor. You hereby agree to indemnify
      the Initial Developer and every Contributor for any liability incurred by
      the Initial Developer or such Contributor as a result of any such terms
      You offer.

      (( 3.7. ))??
      Larger Works.

      You may create a Larger Work by combining Covered Code with other code not
      governed by the terms of this License and distribute the Larger Work as a
      single
      (( LEDs ))??
      product. In such a case, You must make sure the requirements of this
      License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute, judicial
   order, or regulation then You must: (a) comply with the terms of this License
   to the maximum extent possible; and (b) describe the limitations and the code
   they affect. Such description must be included in the LEGAL file described in
   Section 3.4 and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Application of this License.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A and to related Covered Code.

   (( 6. ))??
   Versions of the License.

      (( 6.1. ))??
      New Versions.

      Netizen Pty Ltd ("Netizen ") may publish revised and/or new versions of
      the License from time to time. Each version will be given a distinguishing
      version number.

      (( 6.2. ))??
      Effect of New Versions.

      Once Covered Code has been published under a particular version of the
      License, You may always continue to use it under the terms of that
      version. You may also choose to use such Covered Code under the terms of
      any subsequent version of the License published by Netizen. No one other
      than Netizen has the right to modify the terms applicable to Covered Code
      created under this License.

      (( 6.3. ))??
      Derivative Works.

      If You create or use a modified version of this License (which you may
      only do in order to apply it to code which is not already Covered Code
      governed by this License), You must (a) rename Your license so that the
      phrases "Netizen", "NOSL" or any confusingly similar phrase do not appear
      in your license (except to note that your license differs from this
      License) and (b) otherwise make it clear that Your version of the license
      contains terms which differ from the Netizen Open Source License and Xen
      Open Source License. (Filling in the name of the Initial Developer,
      Original Code or Contributor in the notice described in Exhibit A shall
      not of themselves be deemed to be modifications of this License.)

   (( 7. ))??
   DISCLAIMER OF WARRANTY.

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER
   OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR
   OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF
   THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

      (( 7.1 ))??
      To the extent permitted by law and except as expressly provided to the
      contrary in this Agreement, all warranties whether express, implied,
      statutory or otherwise, relating in any way to the subject matter of this
      Agreement or to this Agreement generally, are excluded. Where legislation
      implies in this Agreement any condition or warranty and that legislation
      avoids or prohibits provisions in a contract excluding or modifying the
      application of or the exercise of or liability under such term, such term
      shall be deemed to be included in this Agreement. However, the liability
      of Supplier for any breach of such term shall be limited, at the option of
      Supplier, to any one or more of the following: if the breach related to
      goods: the replacement of the goods or the supply of equivalent goods; the
      repair of such goods; the payment of the cost of replacing the goods or of
      acquiring equivalent goods; or the payment of the cost of having the goods
      repaired; and if the breach relates to services the supplying of the
      services again; or the payment of the cost of having the services supplied
      again.

   (( 8. ))??
   TERMINATION.

      (( 8.1. ))??
      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. All sublicenses to the
      Covered Code which are properly granted shall survive any termination of
      this License. Provisions which, by their nature, must remain in effect
      beyond the termination of this License shall survive.

      (( 8.2. ))??
      If You initiate litigation by asserting a patent infringement claim
      (excluding declatory judgment actions) against Initial Developer or a
      Contributor (the Initial Developer or Contributor against whom You file
      such action is referred to as "Participant") alleging that:

         (( (a) ))??
         such Participant's Contributor Version directly or indirectly infringes
         any patent, then any and all rights granted by such Participant to You
         under Sections 2.1 and/or 2.2 of this License shall, upon 60 days
         notice from Participant terminate prospectively, unless if within 60
         days after receipt of notice You either: (i) agree in writing to pay
         Participant a mutually agreeable reasonable royalty for Your past and
         future use of Modifications made by such Participant, or (ii) withdraw
         Your litigation claim with respect to the Contributor Version against
         such Participant. If within 60 days of notice, a reasonable royalty and
         payment arrangement are not mutually agreed upon in writing by the
         parties or the litigation claim is not withdrawn, the rights granted by
         Participant to You under Sections 2.1 and/or 2.2 automatically
         terminate at the expiration of the 60 day notice period specified
         above.

         (( (b) ))??
         any software, hardware, or device, other than such Participant's
         Contributor Version, directly or indirectly infringes any patent, then
         any rights granted to You by such Participant under Sections 2.1(b) and
         2.2(b) are revoked effective as of the date You first made, used, sold,
         distributed, or had made, Modifications made by that Participant.

      (( 8.3. ))??
      If You assert a patent infringement claim against Participant alleging
      that such Participant's Contributor Version directly or indirectly
      infringes any patent where such claim is resolved (such as by license or
      settlement) prior to the initiation of patent infringement litigation,
      then the reasonable value of the licenses granted by such Participant
      under Sections 2.1 or 2.2 shall be taken into account in determining the
      amount or value of any payment or license.

      (( 8.4. ))??
      In the event of termination under Sections 8.1 or 8.2 above, all end user
      license agreements (excluding distributors and resellers) which have been
      validly granted by You or any distributor hereunder prior to termination
      shall survive termination.

   (( 9. ))??
   LIMITATION OF LIABILITY.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, THE INITIAL DEVELOPER, ANY
   OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE, OR ANY SUPPLIER OF ANY
   OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. GOVERNMENT END USERS.

   The Covered Code is a "commercial item," as that term is defined in 48 C.F.R.
   2.101 (Oct. 1995), consisting of "commercial computer software" and
   "commercial computer software documentation," as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Covered Code with only those rights set forth herein.

   (( 11. ))??
   MISCELLANEOUS.

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable.

   This Agreement shall be governed by and construed according to the law of the
   State of Victoria. The parties irrevocably submit to the exclusive
   jurisdiction of the Courts of Victoria and Australia and any Courts hearing
   appeals from such Courts. This Agreement is deemed to have been made in
   Victoria.

   The application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any law or regulation
   which provides that the language of a contract shall be construed against the
   drafter shall not apply to this License.

   (( 12. ))??
   RESPONSIBILITY FOR CLAIMS.

   As between Initial Developer and the Contributors, each party is responsible
   for claims and damages arising, directly or indirectly, out of its
   utilization of rights under this License and You agree to work with Initial
   Developer and Contributors to distribute such responsibility on an equitable
   basis. Nothing herein is intended or shall be deemed to constitute any
   admission of liability.

   (( 13. ))??
   MULTIPLE-LICENSED CODE.

   Initial Developer may designate portions of the Covered Code as
   "Multiple-Licensed". "Multiple-Licensed" means that the Initial Developer
   permits you to utilize portions of the Covered Code under Your choice of the
   NPL or the alternative licenses, if any, specified by the Initial Developer
   in the file described in Exhibit A.
   (( EXHIBIT A - Netizen Open Source License

` + "`" + `` + "`" + `The contents of this file are subject to the Netizen Open Source License
Version 1.0 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http:/netizen.com.au/licenses/NOPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

The Original Code is _.

The Initial Developer of the Original Code is _. Portions created by _ are
Copyright (C) _ _. All Rights Reserved.

Contributor(s): _.

Alternatively, the contents of this file may be used under the terms of the _
license (the "[_] License"), in which case the provisions of [_] License are
applicable instead of those above. If you wish to allow use of your version of
this file only under the terms of the [_] License and not to allow others to use
your version of this file under the NOSL, indicate your decision by deleting the
provisions above and replace them with the notice and other provisions required
by the [_] License. If you do not delete the provisions above, a recipient may
use your version of this file under either the NOSL or the [_] License."

[NOTE: The text of this Exhibit A may differ slightly from the text of the
notices in the Source Code files of the Original Code. You should use the text
of this Exhibit A rather than the text found in the Original Code Source Code
for Your Modifications.] ))??
`
const license_NPL_1_0_lre = `//**
Netscape Public License v1.0
https://spdx.org/licenses/NPL-1.0.json
http://www.mozilla.org/MPL/NPL/1.0/
**//

(( NETSCAPE PUBLIC LICENSE

Version 1.0 ))??

   (( 1. ))??
   Definitions.

      (( 1.1. ))??
      ` + "`" + `` + "`" + `Contributor'' means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      ` + "`" + `` + "`" + `Contributor Version'' means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      ` + "`" + `` + "`" + `Covered Code'' means the Original Code or Modifications or the
      combination of the Original Code and Modifications, in each case including
      portions thereof.

      (( 1.4. ))??
      ` + "`" + `` + "`" + `Electronic Distribution Mechanism'' means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      ` + "`" + `` + "`" + `Executable'' means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      ` + "`" + `` + "`" + `Initial Developer'' means the individual or entity identified as the
      Initial Developer in the Source Code notice required byExhibit A.

      (( 1.7. ))??
      ` + "`" + `` + "`" + `Larger Work'' means a work which combines Covered Code or portions
      thereof with code not governed by the terms of this License.

      (( 1.8. ))??
      ` + "`" + `` + "`" + `License'' means this document.

      (( 1.9. ))??
      ` + "`" + `` + "`" + `Modifications'' means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10. ))??
      ` + "`" + `` + "`" + `Original Code'' means Source Code of computer software code which is
      described in the Source Code notice required byExhibit A as Original Code,
      and which, at the time of its release under this License is not already
      Covered Code governed by this License.

      (( 1.11. ))??
      ` + "`" + `` + "`" + `Source Code'' means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or a list of source code differential
      comparisons against either the Original Code or another well known,
      available Covered Code of the Contributor's choice. The Source Code can be
      in a compressed or archival form, provided the appropriate decompression
      or de-archiving software is widely available for no charge.

      (( 1.12. ))??
      ` + "`" + `` + "`" + `You'' means an individual or a legal entity exercising rights under, and
      complying with all of the terms of, this License or a future version of
      this License issued under Section 6.1. For legal entities, ` + "`" + `` + "`" + `You''
      includes any entity which controls, is controlled by, or is under common
      control with You. For purposes of this definition, ` + "`" + `` + "`" + `control'' means (a)
      the power, direct or indirect, to cause the direction or management of
      such entity, whether by contract or otherwise, or (b) ownership of fifty
      percent (50%) or more of the outstanding shares or beneficial ownership of
      such entity.

   (( 2. ))??
   Source Code License.

      (( 2.1. ))??
      The Initial Developer Grant. The Initial Developer hereby grants You a
      world-wide, royalty-free, non-exclusive license, subject to third party
      intellectual property claims:

         (( a) ))??
         to use, reproduce, modify, display, perform, sublicense and distribute
         the Original Code (or portions thereof) with or without Modifications,
         or as part of a Larger Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Initial
         Developer, to make, have made, use and sell (` + "`" + `` + "`" + `Utilize'') the Original
         Code (or portions thereof), but solely to the extent that any such
         patent is reasonably necessary to enable You to Utilize the Original
         Code (or portions thereof) and not to any greater extent that may be
         necessary to Utilize further Modifications or combinations.

      (( 2.2. ))??
      Contributor Grant.

      Each Contributor hereby grants You a world-wide, royalty-free,
      non-exclusive license, subject to third party intellectual property
      claims:

         (( (a) ))??
         to use, reproduce, modify, display, perform, sublicense and distribute
         the Modifications created by such Contributor (or portions thereof)
         either on an unmodified basis, with other Modifications, as Covered
         Code or as part of a Larger Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Contributor, to
         Utilize the Contributor Version (or portions thereof), but solely to
         the extent that any such patent is reasonably necessary to enable You
         to Utilize the Contributor Version (or portions thereof), and not to
         any greater extent that may be necessary to Utilize further
         Modifications or combinations.

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Application of License. The Modifications which You create or to which You
      contribute are governed by the terms of this License, including without
      limitation Section 2.2. The Source Code version of Covered Code may be
      distributed only under the terms of this License or a future version of
      this License released under Section 6.1, and You must include a copy of
      this License with every copy of the Source Code You distribute. You may
      not offer or impose any terms on any Source Code version that alters or
      restricts the applicable version of this License or the recipients' rights
      hereunder. However, You may include an additional document offering the
      additional rights described in Section 3.5.

      (( 3.2. ))??
      Availability of Source Code. Any Modification which You create or to which
      You contribute must be made available in Source Code form under the terms
      of this License either on the same media as an Executable version or via
      an accepted Electronic Distribution Mechanism to anyone to whom you made
      an Executable version available; and if made available via Electronic
      Distribution Mechanism, must remain available for at least twelve (12)
      months after the date it initially became available, or at least six (6)
      months after a subsequent version of that particular Modification has been
      made available to such recipients. You are responsible for ensuring that
      the Source Code version remains available even if the Electronic
      Distribution Mechanism is maintained by a third party.

      (( 3.3. ))??
      Description of Modifications. You must cause all Covered Code to which you
      contribute to contain a file documenting the changes You made to create
      that Covered Code and the date of any change. You must include a prominent
      statement that the Modification is derived, directly or indirectly, from
      Original Code provided by the Initial Developer and including the name of
      the Initial Developer in (a) the Source Code, and (b) in any notice in an
      Executable version or related documentation in which You describe the
      origin or ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters

         (( (a) ))??
         Third Party Claims. If You have knowledge that a party claims an
         intellectual property right in particular functionality or code (or its
         utilization under this License), you must include a text file with the
         source code distribution titled ` + "`" + `` + "`" + `LEGAL'' which describes the claim and
         the party making the claim in sufficient detail that a recipient will
         know whom to contact. If you obtain such knowledge after You make Your
         Modification available as described in Section 3.2, You shall promptly
         modify the LEGAL file in all copies You make available thereafter and
         shall take other steps (such as notifying appropriate mailing lists or
         newsgroups) reasonably calculated to inform those who received the
         Covered Code that new knowledge has been obtained.

         (( (b) ))??
         Contributor APIs. If Your Modification is an application programming
         interface and You own or control patents which are reasonably necessary
         to implement that API, you must also include this information in the
         LEGAL file.

      (( 3.5. ))??
      Required Notices. You must duplicate the notice in Exhibit A in each file
      of the Source Code, and this License in any documentation for the Source
      Code, where You describe recipients' rights relating to Covered Code. If
      You created one or more Modification(s), You may add your name as a
      Contributor to the notice described in Exhibit A. If it is not possible to
      put such notice in a particular Source Code file due to its structure,
      then you must include such notice in a location (such as a relevant
      directory file) where a user would be likely to look for such a notice.
      You may choose to offer, and to charge a fee for, warranty, support,
      indemnity or liability obligations to one or more recipients of Covered
      Code. However, You may do so only on Your own behalf, and not on behalf of
      the Initial Developer or any Contributor. You must make it absolutely
      clear than any such warranty, support, indemnity or liability obligation
      is offered by You alone, and You hereby agree to indemnify the Initial
      Developer and every Contributor for any liability incurred by the Initial
      Developer or such Contributor as a result of warranty, support, indemnity
      or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Versions. You may distribute Covered Code in
      Executable form only if the requirements of Section 3.1-3.5 have been met
      for that Covered Code, and if You include a notice stating that the Source
      Code version of the Covered Code is available under the terms of this
      License, including a description of how and where You have fulfilled the
      obligations of Section 3.2. The notice must be conspicuously included in
      any notice in an Executable version, related documentation or collateral
      in which You describe recipients' rights relating to the Covered Code. You
      may distribute the Executable version of Covered Code under a license of
      Your choice, which may contain terms different from this License, provided
      that You are in compliance with the terms of this License and that the
      license for the Executable version does not attempt to limit or alter the
      recipient's rights in the Source Code version from the rights set forth in
      this License. If You distribute the Executable version under a different
      license You must make it absolutely clear that any terms which differ from
      this License are offered by You alone, not by the Initial Developer or any
      Contributor. You hereby agree to indemnify the Initial Developer and every
      Contributor for any liability incurred by the Initial Developer or such
      Contributor as a result of any such terms You offer.

      (( 3.7. ))??
      Larger Works. You may create a Larger Work by combining Covered Code with
      other code not governed by the terms of this License and distribute the
      Larger Work as a single product. In such a case, You must make sure the
      requirements of this License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute or regulation
   then You must: (a) comply with the terms of this License to the maximum
   extent possible; and (b) describe the limitations and the code they affect.
   Such description must be included in the LEGAL file described in Section 3.4
   and must be included with all distributions of the Source Code. Except to the
   extent prohibited by statute or regulation, such description must be
   sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Application of this License.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A, and to related Covered Code.

   (( 6. ))??
   Versions of the License.

      (( 6.1. ))??
      New Versions. Netscape Communications Corporation (` + "`" + `` + "`" + `Netscape'') may
      publish revised and/or new versions of the License from time to time. Each
      version will be given a distinguishing version number.

      (( 6.2. ))??
      Effect of New Versions. Once Covered Code has been published under a
      particular version of the License, You may always continue to use it under
      the terms of that version. You may also choose to use such Covered Code
      under the terms of any subsequent version of the License published by
      Netscape. No one other than Netscape has the right to modify the terms
      applicable to Covered Code created under this License.

      (( 6.3. ))??
      Derivative Works. If you create or use a modified version of this License
      (which you may only do in order to apply it to code which is not already
      Covered Code governed by this License), you must (a) rename Your license
      so that the phrases ` + "`" + `` + "`" + `Mozilla'', ` + "`" + `` + "`" + `MOZILLAPL'', ` + "`" + `` + "`" + `MOZPL'', ` + "`" + `` + "`" + `Netscape'',
      ` + "`" + `` + "`" + `NPL'' or any confusingly similar phrase do not appear anywhere in your
      license and (b) otherwise make it clear that your version of the license
      contains terms which differ from the Mozilla Public License and Netscape
      Public License. (Filling in the name of the Initial Developer, Original
      Code or Contributor in the notice described in Exhibit A shall not of
      themselves be deemed to be modifications of this License.)

   (( 7. ))??
   DISCLAIMER OF WARRANTY.

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN ` + "`" + `` + "`" + `AS IS'' BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER
   OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR
   OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF
   THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

   (( 8. ))??
   TERMINATION.

   This License and the rights granted hereunder will terminate automatically if
   You fail to comply with terms herein and fail to cure such breach within 30
   days of becoming aware of the breach. All sublicenses to the Covered Code
   which are properly granted shall survive any termination of this License.
   Provisions which, by their nature, must remain in effect beyond the
   termination of this License shall survive.

   (( 9. ))??
   LIMITATION OF LIABILITY.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE INITIAL DEVELOPER, ANY OTHER
   CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE, OR ANY SUPPLIER OF ANY OF
   SUCH PARTIES, BE LIABLE TO YOU OR ANY OTHER PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THAT EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. GOVERNMENT END USERS.

   The Covered Code is a ` + "`" + `` + "`" + `commercial item,'' as that term is defined in 48
   C.F.R. 2.101 (Oct. 1995), consisting of ` + "`" + `` + "`" + `commercial computer software'' and
   ` + "`" + `` + "`" + `commercial computer software documentation,'' as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Covered Code with only those rights set forth herein.

   (( 11. ))??
   MISCELLANEOUS.

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by California law provisions
   (except to the extent applicable law, if any, provides otherwise), excluding
   its conflict-of-law provisions. With respect to disputes in which at least
   one party is a citizen of, or an entity chartered or registered to do
   business in, the United States of America: (a) unless otherwise agreed in
   writing, all disputes relating to this License (excepting any dispute
   relating to intellectual property rights) shall be subject to final and
   binding arbitration, with the losing party paying all costs of arbitration;
   (b) any arbitration relating to this Agreement shall be held in Santa Clara
   County, California, under the auspices of JAMS/EndDispute; and (c) any
   litigation relating to this Agreement shall be subject to the jurisdiction of
   the Federal Courts of the Northern District of California, with venue lying
   in Santa Clara County, California, with the losing party responsible for
   costs, including without limitation, court costs and reasonable attorneys
   fees and expenses. The application of the United Nations Convention on
   Contracts for the International Sale of Goods is expressly excluded. Any law
   or regulation which provides that the language of a contract shall be
   construed against the drafter shall not apply to this License.

   (( 12. ))??
   RESPONSIBILITY FOR CLAIMS.

   Except in cases where another Contributor has failed to comply with Section
   3.4, You are responsible for damages arising, directly or indirectly, out of
   Your utilization of rights under this License, based on the number of copies
   of Covered Code you made available, the revenues you received from utilizing
   such rights, and other relevant factors. You agree to work with affected
   parties to distribute responsibility on an equitable basis.

AMENDMENTS

Additional Terms applicable to the Netscape Public License.

   (( I. ))??
   Effect.

   These additional terms described in this Netscape Public License --
   Amendments shall apply to the Mozilla Communicator client code and to all
   Covered Code under this License.

   (( II. ))??
   ` + "`" + `` + "`" + `Netscape's Branded Code'' means Covered Code that Netscape distributes
   and/or permits others to distribute under one or more trademark(s) which are
   controlled by Netscape but which are not licensed for use under this
   License.

   (( III. ))??
   Netscape and logo.

   This License does not grant any rights to use the trademark ` + "`" + `` + "`" + `Netscape'', the
   ` + "`" + `` + "`" + `Netscape N and horizon'' logo or the Netscape lighthouse logo, even if such
   marks are included in the Original Code.

   (( IV. ))??
   Inability to Comply Due to Contractual Obligation.

   Prior to licensing the Original Code under this License, Netscape has
   licensed third party code for use in Netscape's Branded Code. To the extent
   that Netscape is limited contractually from making such third party code
   available under this License, Netscape may choose to reintegrate such code
   into Covered Code without being required to distribute such code in Source
   Code form, even if such code would otherwise be considered ` + "`" + `` + "`" + `Modifications''
   under this License.

   (( V. ))??
   Use of Modifications and Covered Code by Initial Developer.

   V.1. In General. The obligations of Section 3 apply to Netscape, except to
   the extent specified in this Amendment, Section V.2 and V.3.

   V.2. Other Products. Netscape may include Covered Code in products other than
   the Netscape's Branded Code which are released by Netscape during the two (2)
   years following the release date of the Original Code, without such
   additional products becoming subject to the terms of this License, and may
   license such additional products on different terms from those contained in
   this License. V.3. Alternative Licensing. Netscape may license the Source
   Code of Netscape's Branded Code, including Modifications incorporated
   therein, without such additional products becoming subject to the terms of
   this License, and may license such additional products on different terms
   from those contained in this License.

   (( VI. ))??
   Arbitration and Litigation.

   Notwithstanding the limitations of Section 11 above, the provisions regarding
   arbitration and litigation in Section 11(a), (b) and (c) of the License shall
   apply to all disputes relating to this License.
   (( EXHIBIT A.

"The contents of this file are subject to the Netscape Public License Version
1.0 (the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at http:/www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

The Original Code is Mozilla Communicator client code, released March 31, 1998.

The Initial Developer of the Original Code is Netscape Communications
Corporation. Portions created by Netscape are Copyright (C) 1998 Netscape
Communications Corporation. All Rights Reserved.

Contributor(s): _."

[NOTE: The text of this Exhibit A may differ slightly from the text of the
notices in the Source Code files of the Original Code. This is due to time
constraints encountered in simultaneously finalizing the License and in
preparing the Original Code for release. You should use the text of this Exhibit
A rather than the text found in the Original Code Source Code for Your
Modifications.] ))??
`
const license_NPL_1_1_lre = `//**
Netscape Public License v1.1
https://spdx.org/licenses/NPL-1.1.json
http://www.mozilla.org/MPL/NPL/1.1/
**//

(( Netscape Public LIcense version 1.1 ))??

AMENDMENTS

The Netscape Public License Version 1.1 ("NPL") consists of the Mozilla Public
License Version 1.1 with the following Amendments, including Exhibit A-Netscape
Public License. Files identified with "Exhibit A-Netscape Public License" are
governed by the Netscape Public License Version 1.1.

Additional Terms applicable to the Netscape Public License.

   (( I. ))??
   Effect.

   These additional terms described in this Netscape Public License --
   Amendments shall apply to the Mozilla Communicator client code and to all
   Covered Code under this License.

   (( II. ))??
   "Netscape's Branded Code" means Covered Code that Netscape distributes and/or
   permits others to distribute under one or more trademark(s) which are
   controlled by Netscape but which are not licensed for use under this
   License.

   (( III. ))??
   Netscape and logo.

   This License does not grant any rights to use the trademarks "Netscape", the
   "Netscape N and horizon" logo or the "Netscape lighthouse" logo, "Netcenter",
   "Gecko", "Java" or "JavaScript", "Smart Browsing" even if such marks are
   included in the Original Code or Modifications.

   (( IV. ))??
   Inability to Comply Due to Contractual Obligation.

   Prior to licensing the Original Code under this License, Netscape has
   licensed third party code for use in Netscape's Branded Code. To the extent
   that Netscape is limited contractually from making such third party code
   available under this License, Netscape may choose to reintegrate such code
   into Covered Code without being required to distribute such code in Source
   Code form, even if such code would otherwise be considered "Modifications"
   under this License.

   (( V. ))??
   Use of Modifications and Covered Code by Initial Developer.

      (( V.1. ))??
      In General.

      The obligations of Section 3 apply to Netscape, except to the extent
      specified in this Amendment, Section V.2 and V.3.

      (( V.2. ))??
      Other Products.

      Netscape may include Covered Code in products other than the Netscape's
      Branded Code which are released by Netscape during the two (2) years
      following the release date of the Original Code, without such additional
      products becoming subject to the terms of this License, and may license
      such additional products on different terms from those contained in this
      License.

      (( V.3. ))??
      Alternative Licensing.

      Netscape may license the Source Code of Netscape's Branded Code, including
      Modifications incorporated therein, without such Netscape Branded Code
      becoming subject to the terms of this License, and may license such
      Netscape Branded Code on different terms from those contained in this
      License.

   (( VI. ))??
   Litigation.

   Notwithstanding the limitations of Section 11 above, the provisions regarding
   litigation in Section 11(a), (b) and (c) of the License shall apply to all
   disputes relating to this License.

   EXHIBIT A-Netscape Public License.

   "The contents of this file are subject to the Netscape Public License Version
   1.1 (the "License"); you may not use this file except in compliance with the
   License. You may obtain a copy of the License at http:/www.mozilla.org/NPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

The Original Code is Mozilla Communicator client code, released March 31, 1998.

The Initial Developer of the Original Code is Netscape Communications
Corporation. Portions created by Netscape are Copyright (C) 1998-1999 Netscape
Communications Corporation. All Rights Reserved.

Contributor(s): _.

Alternatively, the contents of this file may be used under the terms of the _
license (the "[_] License"), in which case the provisions of [_] License are
applicable instead of those above. If you wish to allow use of your version of
this file only under the terms of the [_] License and not to allow others to use
your version of this file under the NPL, indicate your decision by deleting the
provisions above and replace them with the notice and other provisions required
by the [_] License. If you do not delete the provisions above, a recipient may
use your version of this file under either the NPL or the [_] License."

Mozilla Public License Version 1.1

   (( 1. ))??
   Definitions.

   1.0.1. "Commercial Use" means distribution or otherwise making the Covered
   Code available to a third party.

      (( 1.1. ))??
      "Contributor" means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      "Covered Code" means the Original Code or Modifications or the combination
      of the Original Code and Modifications, in each case including portions
      thereof.

      (( 1.4. ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      "Executable" means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      "Initial Developer" means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7. ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.8. ))??
      "License" means this document.

      1.8.1. "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.9. ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

      Any addition to or deletion from the contents of a file containing
      Original Code or previous Modifications.

      Any new file that contains any part of the Original Code or previous
      Modifications.

      (( 1.10. ))??
      "Original Code" means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      1.10.1. "Patent Claims" means any patent claim(s), now owned or hereafter
      acquired, including without limitation, method, process, and apparatus
      claims, in any patent Licensable by grantor.

      (( 1.11. ))??
      "Source Code" means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or source code differential comparisons
      against either the Original Code or another well known, available Covered
      Code of the Contributor's choice. The Source Code can be in a compressed
      or archival form, provided the appropriate decompression or de-archiving
      software is widely available for no charge.

      (( 1.12. ))??
      "You" (or "Your") means an individual or a legal entity exercising rights
      under, and complying with all of the terms of, this License or a future
      version of this License issued under Section 6.1. For legal entities,
      "You" includes any entity which controls, is controlled by, or is under
      common control with You. For purposes of this definition, "control" means
      (a) the power, direct or indirect, to cause the direction or management of
      such entity, whether by contract or otherwise, or (b) ownership of more
      than fifty percent (50%) of the outstanding shares or beneficial ownership
      of such entity.

   (( 2. ))??
   Source Code License.

      (( 2.1. ))??
      The Initial Developer Grant. The Initial Developer hereby grants You a
      world-wide, royalty-free, non-exclusive license, subject to third party
      intellectual property claims:

         (( a. ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Initial Developer to use, reproduce, modify, display,
         perform, sublicense and distribute the Original Code (or portions
         thereof) with or without Modifications, and/or as part of a Larger
         Work; and

         (( b. ))??
         under Patents Claims infringed by the making, using or selling of
         Original Code, to make, have made, use, practice, sell, and offer for
         sale, and/or otherwise dispose of the Original Code (or portions
         thereof).

         (( c. ))??
         the licenses granted in this Section 2.1 (a) and (b) are effective on
         the date Initial Developer first distributes Original Code under the
         terms of this License.

         (( d. ))??
         Notwithstanding Section 2.1 (b) above, no patent license is granted: 1)
         for code that You delete from the Original Code; 2) separate from the
         Original Code; or 3) for infringements caused by: i) the modification
         of the Original Code or ii) the combination of the Original Code with
         other software or devices.

      (( 2.2. ))??
      Contributor Grant. Subject to third party intellectual property claims,
      each Contributor hereby grants You a world-wide, royalty-free,
      non-exclusive license

         (( a. ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Contributor, to use, reproduce, modify, display, perform,
         sublicense and distribute the Modifications created by such Contributor
         (or portions thereof) either on an unmodified basis, with other
         Modifications, as Covered Code and/or as part of a Larger Work; and

         (( b. ))??
         under Patent Claims infringed by the making, using, or selling of
         Modifications made by that Contributor either alone and/or in
         combination with its Contributor Version (or portions of such
         combination), to make, use, sell, offer for sale, have made, and/or
         otherwise dispose of: 1) Modifications made by that Contributor (or
         portions thereof); and 2) the combination of Modifications made by that
         Contributor with its Contributor Version (or portions of such
         combination).

         (( c. ))??
         the licenses granted in Sections 2.2 (a) and 2.2 (b) are effective on
         the date Contributor first makes Commercial Use of the Covered Code.

         (( d. ))??
         Notwithstanding Section 2.2 (b) above, no patent license is granted: 1)
         for any code that Contributor has deleted from the Contributor Version;
         2) separate from the Contributor Version; 3) for infringements caused
         by: i) third party modifications of Contributor Version or ii) the
         combination of Modifications made by that Contributor with other
         software (except as part of the Contributor Version) or other devices;
         or 4) under Patent Claims infringed by Covered Code in the absence of
         Modifications made by that Contributor.

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Application of License. The Modifications which You create or to which You
      contribute are governed by the terms of this License, including without
      limitation Section 2.2. The Source Code version of Covered Code may be
      distributed only under the terms of this License or a future version of
      this License released under Section 6.1, and You must include a copy of
      this License with every copy of the Source Code You distribute. You may
      not offer or impose any terms on any Source Code version that alters or
      restricts the applicable version of this License or the recipients' rights
      hereunder. However, You may include an additional document offering the
      additional rights described in Section 3.5.

      (( 3.2. ))??
      Availability of Source Code. Any Modification which You create or to which
      You contribute must be made available in Source Code form under the terms
      of this License either on the same media as an Executable version or via
      an accepted Electronic Distribution Mechanism to anyone to whom you made
      an Executable version available; and if made available via Electronic
      Distribution Mechanism, must remain available for at least twelve (12)
      months after the date it initially became available, or at least six (6)
      months after a subsequent version of that particular Modification has been
      made available to such recipients. You are responsible for ensuring that
      the Source Code version remains available even if the Electronic
      Distribution Mechanism is maintained by a third party.

      (( 3.3. ))??
      Description of Modifications. You must cause all Covered Code to which You
      contribute to contain a file documenting the changes You made to create
      that Covered Code and the date of any change. You must include a prominent
      statement that the Modification is derived, directly or indirectly, from
      Original Code provided by the Initial Developer and including the name of
      the Initial Developer in (a) the Source Code, and (b) in any notice in an
      Executable version or related documentation in which You describe the
      origin or ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters

         (( (a) ))??
         Third Party Claims

         If Contributor has knowledge that a license under a third party's
         intellectual property rights is required to exercise the rights granted
         by such Contributor under Sections 2.1 or 2.2, Contributor must include
         a text file with the Source Code distribution titled "LEGAL" which
         describes the claim and the party making the claim in sufficient detail
         that a recipient will know whom to contact. If Contributor obtains such
         knowledge after the Modification is made available as described in
         Section 3.2, Contributor shall promptly modify the LEGAL file in all
         copies Contributor makes available thereafter and shall take other
         steps (such as notifying appropriate mailing lists or newsgroups)
         reasonably calculated to inform those who received the Covered Code
         that new knowledge has been obtained.

         (( (b) ))??
         Contributor APIs

         If Contributor's Modifications include an application programming
         interface and Contributor has knowledge of patent licenses which are
         reasonably necessary to implement that API, Contributor must also
         include this information in the LEGAL file.

         (( c ))??
         Representations.

         Contributor represents that, except as disclosed pursuant to Section
         3.4 (a) above, Contributor believes that Contributor's Modifications
         are Contributor's original creation(s) and/or Contributor has
         sufficient rights to grant the rights conveyed by this License.

      (( 3.5. ))??
      Required Notices. You must duplicate the notice in Exhibit A in each file
      of the Source Code. If it is not possible to put such notice in a
      particular Source Code file due to its structure, then You must include
      such notice in a location (such as a relevant directory) where a user
      would be likely to look for such a notice. If You created one or more
      Modification(s) You may add your name as a Contributor to the notice
      described in Exhibit A. You must also duplicate this License in any
      documentation for the Source Code where You describe recipients' rights or
      ownership rights relating to Covered Code. You may choose to offer, and to
      charge a fee for, warranty, support, indemnity or liability obligations to
      one or more recipients of Covered Code. However, You may do so only on
      Your own behalf, and not on behalf of the Initial Developer or any
      Contributor. You must make it absolutely clear than any such warranty,
      support, indemnity or liability obligation is offered by You alone, and
      You hereby agree to indemnify the Initial Developer and every Contributor
      for any liability incurred by the Initial Developer or such Contributor as
      a result of warranty, support, indemnity or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Versions. You may distribute Covered Code in
      Executable form only if the requirements of Sections 3.1, 3.2, 3.3, 3.4
      and 3.5 have been met for that Covered Code, and if You include a notice
      stating that the Source Code version of the Covered Code is available
      under the terms of this License, including a description of how and where
      You have fulfilled the obligations of Section 3.2. The notice must be
      conspicuously included in any notice in an Executable version, related
      documentation or collateral in which You describe recipients' rights
      relating to the Covered Code. You may distribute the Executable version of
      Covered Code or ownership rights under a license of Your choice, which may
      contain terms different from this License, provided that You are in
      compliance with the terms of this License and that the license for the
      Executable version does not attempt to limit or alter the recipient's
      rights in the Source Code version from the rights set forth in this
      License. If You distribute the Executable version under a different
      license You must make it absolutely clear that any terms which differ from
      this License are offered by You alone, not by the Initial Developer or any
      Contributor. You hereby agree to indemnify the Initial Developer and every
      Contributor for any liability incurred by the Initial Developer or such
      Contributor as a result of any such terms You offer.

      (( 3.7. ))??
      Larger Works. You may create a Larger Work by combining Covered Code with
      other code not governed by the terms of this License and distribute the
      Larger Work as a single product. In such a case, You must make sure the
      requirements of this License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute, judicial
   order, or regulation then You must: (a) comply with the terms of this License
   to the maximum extent possible; and (b) describe the limitations and the code
   they affect. Such description must be included in the LEGAL file described in
   Section 3.4 and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Application of this License.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A and to related Covered Code.

   (( 6. ))??
   Versions of the License.

      (( 6.1. ))??
      New Versions

      Netscape Communications Corporation ("Netscape") may publish revised
      and/or new versions of the License from time to time. Each version will be
      given a distinguishing version number.

      (( 6.2. ))??
      Effect of New Versions

      Once Covered Code has been published under a particular version of the
      License, You may always continue to use it under the terms of that
      version. You may also choose to use such Covered Code under the terms of
      any subsequent version of the License published by Netscape. No one other
      than Netscape has the right to modify the terms applicable to Covered Code
      created under this License.

      (( 6.3. ))??
      Derivative Works

      If You create or use a modified version of this License (which you may
      only do in order to apply it to code which is not already Covered Code
      governed by this License), You must (a) rename Your license so that the
      phrases "Mozilla", "MOZILLAPL", "MOZPL", "Netscape", "MPL", "NPL" or any
      confusingly similar phrase do not appear in your license (except to note
      that your license differs from this License) and (b) otherwise make it
      clear that Your version of the license contains terms which differ from
      the Mozilla Public License and Netscape Public License. (Filling in the
      name of the Initial Developer, Original Code or Contributor in the notice
      described in Exhibit A shall not of themselves be deemed to be
      modifications of this License.)

   (( 7. ))??
   DISCLAIMER OF WARRANTY

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER
   OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR
   OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF
   THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

   (( 8. ))??
   Termination

      (( 8.1. ))??
      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. All sublicenses to the
      Covered Code which are properly granted shall survive any termination of
      this License. Provisions which, by their nature, must remain in effect
      beyond the termination of this License shall survive.

      (( 8.2. ))??
      If You initiate litigation by asserting a patent infringement claim
      (excluding declatory judgment actions) against Initial Developer or a
      Contributor (the Initial Developer or Contributor against whom You file
      such action is referred to as "Participant") alleging that:

         (( a. ))??
         such Participant's Contributor Version directly or indirectly infringes
         any patent, then any and all rights granted by such Participant to You
         under Sections 2.1 and/or 2.2 of this License shall, upon 60 days
         notice from Participant terminate prospectively, unless if within 60
         days after receipt of notice You either: (i) agree in writing to pay
         Participant a mutually agreeable reasonable royalty for Your past and
         future use of Modifications made by such Participant, or (ii) withdraw
         Your litigation claim with respect to the Contributor Version against
         such Participant. If within 60 days of notice, a reasonable royalty and
         payment arrangement are not mutually agreed upon in writing by the
         parties or the litigation claim is not withdrawn, the rights granted by
         Participant to You under Sections 2.1 and/or 2.2 automatically
         terminate at the expiration of the 60 day notice period specified
         above.

         (( b. ))??
         any software, hardware, or device, other than such Participant's
         Contributor Version, directly or indirectly infringes any patent, then
         any rights granted to You by such Participant under Sections 2.1(b) and
         2.2(b) are revoked effective as of the date You first made, used, sold,
         distributed, or had made, Modifications made by that Participant.

      (( 8.3. ))??
      If You assert a patent infringement claim against Participant alleging
      that such Participant's Contributor Version directly or indirectly
      infringes any patent where such claim is resolved (such as by license or
      settlement) prior to the initiation of patent infringement litigation,
      then the reasonable value of the licenses granted by such Participant
      under Sections 2.1 or 2.2 shall be taken into account in determining the
      amount or value of any payment or license.

      (( 8.4. ))??
      In the event of termination under Sections 8.1 or 8.2 above, all end user
      license agreements (excluding distributors and resellers) which have been
      validly granted by You or any distributor hereunder prior to termination
      shall survive termination.

   (( 9. ))??
   LIMITATION OF LIABILITY

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, THE INITIAL DEVELOPER, ANY
   OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE, OR ANY SUPPLIER OF ANY
   OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. government end users

   The Covered Code is a "commercial item," as that term is defined in 48 C.F.R.
   2.101 (Oct. 1995), consisting of "commercial computer software" and
   "commercial computer software documentation," as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Covered Code with only those rights set forth herein.

   (( 11. ))??
   Miscellaneous

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by California law provisions
   (except to the extent applicable law, if any, provides otherwise), excluding
   its conflict-of-law provisions. With respect to disputes in which at least
   one party is a citizen of, or an entity chartered or registered to do
   business in the United States of America, any litigation relating to this
   License shall be subject to the jurisdiction of the Federal Courts of the
   Northern District of California, with venue lying in Santa Clara County,
   California, with the losing party responsible for costs, including without
   limitation, court costs and reasonable attorneys' fees and expenses. The
   application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any law or regulation
   which provides that the language of a contract shall be construed against the
   drafter shall not apply to this License.

   (( 12. ))??
   Responsibility for claims

   As between Initial Developer and the Contributors, each party is responsible
   for claims and damages arising, directly or indirectly, out of its
   utilization of rights under this License and You agree to work with Initial
   Developer and Contributors to distribute such responsibility on an equitable
   basis. Nothing herein is intended or shall be deemed to constitute any
   admission of liability.

   (( 13. ))??
   Multiple-licensed code

   Initial Developer may designate portions of the Covered Code as
   "Multiple-Licensed". "Multiple-Licensed" means that the Initial Developer
   permits you to utilize portions of the Covered Code under Your choice of the
   MPL or the alternative licenses, if any, specified by the Initial Developer
   in the file described in Exhibit A.
   (( Exhibit A - Mozilla Public License.

"The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at http:/www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

The Original Code is _.

The Initial Developer of the Original Code is _.

Portions created by _ are Copyright (C) _

_. All Rights Reserved.

Contributor(s): _.

Alternatively, the contents of this file may be used under the terms of the _
license (the "[_] License"), in which case the provisions of [_] License are
applicable instead of those above. If you wish to allow use of your version of
this file only under the terms of the [_] License and not to allow others to use
your version of this file under the MPL, indicate your decision by deleting the
provisions above and replace them with the notice and other provisions required
by the [_] License. If you do not delete the provisions above, a recipient may
use your version of this file under either the MPL or the [_] License."

NOTE: The text of this Exhibit A may differ slightly from the text of the
notices in the Source Code files of the Original Code. You should use the text
of this Exhibit A rather than the text found in the Original Code Source Code
for Your Modifications. ))??
`
const license_NPOSL_3_0_lre = `//**
Non-Profit Open Software License 3.0
https://spdx.org/licenses/NPOSL-3.0.json
https://opensource.org/licenses/NOSL3.0
**//

(( Non-Profit Open Software License 3.0 ))??

This Non-Profit Open Software License ("Non-Profit OSL") version 3.0 (the
"License") applies to any original work of authorship (the "Original Work")
whose owner (the "Licensor") has placed the following licensing notice adjacent
to the copyright notice for the Original Work:

Licensed under the Non-Profit Open Software License version 3.0

   (( 1) ))??
   Grant of Copyright License. Licensor grants You a worldwide, royalty-free,
   non-exclusive, sublicensable license, for the duration of the copyright, to
   do the following:

      (( a) ))??
      to reproduce the Original Work in copies, either alone or as part of a
      collective work;

      (( b) ))??
      to translate, adapt, alter, transform, modify, or arrange the Original
      Work, thereby creating derivative works ("Derivative Works") based upon
      the Original Work;

      (( c) ))??
      to distribute or communicate copies of the Original Work and Derivative
      Works to the public, with the proviso that copies of Original Work or
      Derivative Works that You distribute or communicate shall be licensed
      under this Non-Profit Open Software License or as provided in section
      17(d);

      (( d) ))??
      to perform the Original Work publicly; and

      (( e) ))??
      to display the Original Work publicly.

   (( 2) ))??
   Grant of Patent License. Licensor grants You a worldwide, royalty-free,
   non-exclusive, sublicensable license, under patent claims owned or controlled
   by the Licensor that are embodied in the Original Work as furnished by the
   Licensor, for the duration of the patents, to make, use, sell, offer for
   sale, have made, and import the Original Work and Derivative Works.

   (( 3) ))??
   Grant of Source Code License. The term "Source Code" means the preferred form
   of the Original Work for making modifications to it and all available
   documentation describing how to modify the Original Work. Licensor agrees to
   provide a machine-readable copy of the Source Code of the Original Work along
   with each copy of the Original Work that Licensor distributes. Licensor
   reserves the right to satisfy this obligation by placing a machine-readable
   copy of the Source Code in an information repository reasonably calculated to
   permit inexpensive and convenient access by You for as long as Licensor
   continues to distribute the Original Work.

   (( 4) ))??
   Exclusions From License Grant. Neither the names of Licensor, nor the names
   of any contributors to the Original Work, nor any of their trademarks or
   service marks, may be used to endorse or promote products derived from this
   Original Work without express prior permission of the Licensor. Except as
   expressly stated herein, nothing in this License grants any license to
   Licensor's trademarks, copyrights, patents, trade secrets or any other
   intellectual property. No patent license is granted to make, use, sell, offer
   for sale, have made, or import embodiments of any patent claims other than
   the licensed claims defined in Section 2. No license is granted to the
   trademarks of Licensor even if such marks are included in the Original Work.
   Nothing in this License shall be interpreted to prohibit Licensor from
   licensing under terms different from this License any Original Work that
   Licensor otherwise would have a right to license.

   (( 5) ))??
   External Deployment. The term "External Deployment" means the use,
   distribution, or communication of the Original Work or Derivative Works in
   any way such that the Original Work or Derivative Works may be used by anyone
   other than You, whether those works are distributed or communicated to those
   persons or made available as an application intended for use over a network.
   As an express condition for the grants of license hereunder, You must treat
   any External Deployment by You of the Original Work or a Derivative Work as a
   distribution under section 1(c).

   (( 6) ))??
   Attribution Rights. You must retain, in the Source Code of any Derivative
   Works that You create, all copyright, patent, or trademark notices from the
   Source Code of the Original Work, as well as any notices of licensing and any
   descriptive text identified therein as an "Attribution Notice." You must
   cause the Source Code for any Derivative Works that You create to carry a
   prominent Attribution Notice reasonably calculated to inform recipients that
   You have modified the Original Work.

   (( 7) ))??
   Warranty of Provenance and Disclaimer of Warranty. The Original Work is
   provided under this License on an "AS IS" BASIS and WITHOUT WARRANTY, either
   express or implied, including, without limitation, the warranties of
   non-infringement, merchantability or fitness for a particular purpose. THE
   ENTIRE RISK AS TO THE QUALITY OF THE ORIGINAL WORK IS WITH YOU. This
   DISCLAIMER OF WARRANTY constitutes an essential part of this License. No
   license to the Original Work is granted by this License except under this
   disclaimer.

   (( 8) ))??
   Limitation of Liability. Under no circumstances and under no legal theory,
   whether in tort (including negligence), contract, or otherwise, shall the
   Licensor be liable to anyone for any direct, indirect, special, incidental,
   or consequential damages of any character arising as a result of this License
   or the use of the Original Work including, without limitation, damages for
   loss of goodwill, work stoppage, computer failure or malfunction, or any and
   all other commercial damages or losses. This limitation of liability shall
   not apply to the extent applicable law prohibits such limitation.

   (( 9) ))??
   Acceptance and Termination. If, at any time, You expressly assented to this
   License, that assent indicates your clear and irrevocable acceptance of this
   License and all of its terms and conditions. If You distribute or communicate
   copies of the Original Work or a Derivative Work, You must make a reasonable
   effort under the circumstances to obtain the express assent of recipients to
   the terms of this License. This License conditions your rights to undertake
   the activities listed in Section 1, including your right to create Derivative
   Works based upon the Original Work, and doing so without honoring these terms
   and conditions is prohibited by copyright law and international treaty.
   Nothing in this License is intended to affect copyright exceptions and
   limitations (including "fair use" or "fair dealing"). This License shall
   terminate immediately and You may no longer exercise any of the rights
   granted to You by this License upon your failure to honor the conditions in
   Section 1(c).

   (( 10) ))??
   Termination for Patent Action. This License shall terminate automatically and
   You may no longer exercise any of the rights granted to You by this License
   as of the date You commence an action, including a cross-claim or
   counterclaim, against Licensor or any licensee alleging that the Original
   Work infringes a patent. This termination provision shall not apply for an
   action alleging patent infringement by combinations of the Original Work with
   other software or hardware.

   (( 11) ))??
   Jurisdiction, Venue and Governing Law. Any action or suit relating to this
   License may be brought only in the courts of a jurisdiction wherein the
   Licensor resides or in which Licensor conducts its primary business, and
   under the laws of that jurisdiction excluding its conflict-of-law provisions.
   The application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any use of the Original
   Work outside the scope of this License or after its termination shall be
   subject to the requirements and penalties of copyright or patent law in the
   appropriate jurisdiction. This section shall survive the termination of this
   License.

   (( 12) ))??
   Attorneys' Fees. In any action to enforce the terms of this License or
   seeking damages relating thereto, the prevailing party shall be entitled to
   recover its costs and expenses, including, without limitation, reasonable
   attorneys' fees and costs incurred in connection with such action, including
   any appeal of such action. This section shall survive the termination of this
   License.

   (( 13) ))??
   Miscellaneous. If any provision of this License is held to be unenforceable,
   such provision shall be reformed only to the extent necessary to make it
   enforceable.

   (( 14) ))??
   Definition of "You" in This License. "You" throughout this License, whether
   in upper or lower case, means an individual or a legal entity exercising
   rights under, and complying with all of the terms of, this License. For legal
   entities, "You" includes any entity that controls, is controlled by, or is
   under common control with you. For purposes of this definition, "control"
   means (i) the power, direct or indirect, to cause the direction or management
   of such entity, whether by contract or otherwise, or (ii) ownership of fifty
   percent (50%) or more of the outstanding shares, or (iii) beneficial
   ownership of such entity.

   (( 15) ))??
   Right to Use. You may use the Original Work in all ways not otherwise
   restricted or conditioned by this License or by law, and Licensor promises
   not to interfere with or be responsible for such uses by You.

   (( 16) ))??
   Modification of This License. This License is Copyright © 2005 Lawrence
   Rosen. Permission is granted to copy, distribute, or communicate this License
   without modification. Nothing in this License permits You to modify this
   License as applied to the Original Work or to Derivative Works. However, You
   may modify the text of this License and copy, distribute or communicate your
   modified version (the "Modified License") and apply it to other original
   works of authorship subject to the following conditions: (i) You may not
   indicate in any way that your Modified License is the "Open Software License"
   or "OSL" and you may not use those names in the name of your Modified
   License; (ii) You must replace the notice specified in the first paragraph
   above with the notice "Licensed under <insert your license name here>" or
   with a notice of your own that is not confusingly similar to the notice in
   this License; and (iii) You may not claim that your original works are open
   source software unless your Modified License has been approved by Open Source
   Initiative (OSI) and You comply with its license review and certification
   process.

   (( 17) ))??
   Non-Profit Amendment. The name of this amended version of the Open Software
   License ("OSL 3.0") is "Non-Profit Open Software License 3.0". The original
   OSL 3.0 license has been amended as follows:

      (( (a) ))??
      Licensor represents and declares that it is a not-for-profit organization
      that derives no revenue whatsoever from the distribution of the Original
      Work or Derivative Works thereof, or from support or services relating
      thereto.

      (( (b) ))??
      The first sentence of Section 7 ["Warranty of Provenance"] of OSL 3.0 has
      been stricken. For Original Works licensed under this Non-Profit OSL 3.0,
      LICENSOR OFFERS NO WARRANTIES WHATSOEVER.

      (( c ))??
      In the first sentence of Section 8 ["Limitation of Liability"] of this
      Non-Profit OSL 3.0, the list of damages for which LIABILITY IS LIMITED now
      includes "direct" damages.

      (( (d) ))??
      The proviso in Section 1(c) of this License now refers to this "Non-Profit
      Open Software License" rather than the "Open Software License". You may
      distribute or communicate the Original Work or Derivative Works thereof
      under this Non-Profit OSL 3.0 license only if You make the representation
      and declaration in paragraph (a) of this Section 17. Otherwise, You shall
      distribute or communicate the Original Work or Derivative Works thereof
      only under the OSL 3.0 license and You shall publish clear licensing
      notices so stating. Also by way of clarification, this License does not
      authorize You to distribute or communicate works under this Non-Profit OSL
      3.0 if You received them under the original OSL 3.0 license.

      (( (e) ))??
      Original Works licensed under this license shall reference "Non-Profit OSL
      3.0" in licensing notices to distinguish them from works licensed under
      the original OSL 3.0 license.
`
const license_NRL_lre = `//**
NRL License
https://spdx.org/licenses/NRL.json
http://web.mit.edu/network/isakmp/nrllicense.html
**//

(( NRL License ))??

(( COPYRIGHT NOTICE ))??

All of the documentation and software included in this software distribution
from the US Naval Research Laboratory (NRL) are copyrighted by their respective
developers.

Portions of the software are derived from the Net/2 and 4.4-Lite Berkeley
Software Distributions (BSD) of the University of California at Berkeley and
those portions are copyright by The Regents of the University of California. All
Rights Reserved. The UC Berkeley Copyright and License agreement is binding on
those portions of the software. In all cases, the NRL developers have retained
the original UC Berkeley copyright and license notices in the respective files
in accordance with the UC Berkeley copyrights and license.

Portions of this software and documentation were developed at NRL by various
people. Those developers have each copyrighted the portions that they developed
at NRL and have assigned All Rights for those portions to NRL. Outside the USA,
NRL has copyright on some of the software developed at NRL. The affected files
all contain specific copyright notices and those notices must be retained in any
derived work.
(( NRL LICENSE ))??

NRL grants permission for redistribution and use in source and binary forms,
with or without modification, of the software and documentation created at NRL
provided that the following conditions are met:

   (( 1. ))??
   All terms of the UC Berkeley copyright and license must be followed.

   (( 2. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   (( 3. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 4. ))??
   All advertising materials mentioning features or use of this software must
   display the following acknowledgements:

      This product includes software developed by the University of California,
      Berkeley and its contributors.

      This product includes software developed at the Information Technology
      Division, US Naval Research Laboratory.

   (( 5. ))??
   Neither the name of the NRL nor the names of its contributors may be used to
   endorse or promote products derived from this software without specific prior
   written permission.

THE SOFTWARE PROVIDED BY NRL IS PROVIDED BY NRL AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL NRL OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the US Naval Research Laboratory (NRL).
`
const license_NTP_lre = `//**
NTP License
https://spdx.org/licenses/NTP.json
https://opensource.org/licenses/NTP
**//

(( NTP License (NTP)
(( Copyright __20__ ))??
))??

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose with or without fee is hereby granted, provided
that the above copyright notice appears in all copies and that both the
copyright notice and this permission notice appear in supporting documentation,
and that the name
//** (TrademarkedName) **//
__5__
not be used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.
//** (TrademarkedName) **//
__5__
makes no representations about the suitability this software for any purpose. It
is provided "as is" without express or implied warranty.
`
const license_NTP_0_lre = `//**
NTP No Attribution
https://spdx.org/licenses/NTP-0.json
https://github.com/tytso/e2fsprogs/blob/master/lib/et/et_name.c
**//

(( NTP No Attribution (NTP-0)
(( Copyright __20__ ))??
))??

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose is hereby granted, provided that the names of
//** (TrademarkedName) **//
__5__
not be used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.
//** (TrademarkedName) **//
__5__
makes no representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.
`
const license_Naumen_lre = `//**
Naumen Public License
https://spdx.org/licenses/Naumen.json
https://opensource.org/licenses/Naumen
**//

(( NAUMEN Public License
(( This software is ))??
(( Copyright __20__ ))??
))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions in source code must retain the above copyright notice, this
   list of conditions, and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions, and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The name NAUMEN (tm) must not be used to endorse or promote products derived
   from this software without prior written permission from NAUMEN.

   (( 4. ))??
   The right to distribute this software or to use it for any purpose does not
   give you the right to use Servicemarks (sm) or Trademarks (tm) of NAUMEN.

   (( 5. ))??
   If any files originating from NAUMEN or Contributors are modified, you must
   cause the modified files to carry prominent notices stating that you changed
   the files and the date of any change.

Disclaimer:

THIS SOFTWARE IS PROVIDED BY NAUMEN "AS IS" AND ANY EXPRESSED OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL NAUMEN OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

This software consists of contributions made by NAUMEN and Contributors.
Specific attributions are listed in the accompanying credits file.
`
const license_Net_SNMP_lre = `//**
Net-SNMP License
https://spdx.org/licenses/Net-SNMP.json
http://net-snmp.sourceforge.net/about/license.html
**//

---- Part 1: CMU/UCD copyright notice: (BSD like) -----
Copyright 1989, 1991, 1992 by Carnegie Mellon University

Derivative Work - 1996, 1998-2000 Copyright 1996, 1998-2000 The Regents of
the University of California

All Rights Reserved

Permission to use, copy, modify and distribute this software and its
documentation for any purpose and without fee is hereby granted, provided that
the above copyright notice appears in all copies and that both that copyright
notice and this permission notice appear in supporting documentation, and that
the name of CMU and The Regents of the University of California not be used in
advertising or publicity pertaining to distribution of the software without
specific written permission.

CMU AND THE REGENTS OF THE UNIVERSITY OF CALIFORNIA DISCLAIM ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS. IN NO EVENT SHALL CMU OR THE REGENTS OF THE UNIVERSITY OF CALIFORNIA BE
LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM THE LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

---- Part 2: Networks Associates Technology, Inc copyright notice (BSD) -----

Copyright (c) 2001-2003, Networks Associates Technology, Inc All rights
reserved. Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

   *
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   *
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   *
   Neither the name of the Networks Associates Technology, Inc nor the names of
   its contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.

   ---- Part 3: Cambridge Broadband Ltd. copyright notice (BSD) -----

   Portions of this code are copyright (c) 2001-2003, Cambridge Broadband Ltd.
   All rights reserved. Redistribution and use in source and binary forms, with
   or without modification, are permitted provided that the following conditions
   are met:

   *
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   *
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   *
   The name of Cambridge Broadband Ltd. may not be used to endorse or promote
   products derived from this software without specific prior written
   permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

---- Part 4: Sun Microsystems, Inc. copyright notice (BSD) -----
Copyright © 2003 Sun Microsystems, Inc., 4150 Network Circle, Santa Clara, California
95054, U.S.A. All rights reserved.

Use is subject to license terms below.

This distribution may include materials developed by third parties.

Sun, Sun Microsystems, the Sun logo and Solaris are trademarks or registered
trademarks of Sun Microsystems, Inc. in the U.S. and other countries.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   *
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   *
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   *
   Neither the name of the Sun Microsystems, Inc. nor the names of its
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.

   ---- Part 5: Sparta, Inc copyright notice (BSD) -----

   Copyright (c) 2003-2009, Sparta, Inc All rights reserved. Redistribution and
   use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:

   *
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   *
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   *
   Neither the name of Sparta, Inc nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.

   ---- Part 6: Cisco/BUPTNIC copyright notice (BSD) -----

   Copyright (c) 2004, Cisco, Inc and Information Network Center of Beijing
   University of Posts and Telecommunications. All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   *
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   *
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   *
   Neither the name of Cisco, Inc, Beijing University of Posts and
   Telecommunications, nor the names of their contributors may be used to
   endorse or promote products derived from this software without specific prior
   written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

---- Part 7: Fabasoft R&D Software GmbH & Co KG copyright notice (BSD) -----
Copyright (c) Fabasoft R&D Software GmbH & Co KG, 2003 oss@fabasoft.com Author:
Bernhard Penz

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   *
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   *
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   *
   The name of Fabasoft R&D Software GmbH & Co KG or any of its subsidiaries,
   brand or product names may not be used to endorse or promote products derived
   from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

---- Part 8: Apple Inc. copyright notice (BSD) -----
Copyright (c) 2007 Apple Inc. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   Neither the name of Apple Inc. ("Apple") nor the names of its contributors
   may be used to endorse or promote products derived from this software without
   specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

   ---- Part 9: ScienceLogic, LLC copyright notice (BSD) -----

   Copyright (c) 2009, ScienceLogic, LLC All rights reserved. Redistribution and
   use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:

   *
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   *
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   *
   Neither the name of ScienceLogic, LLC nor the names of its contributors may
   be used to endorse or promote products derived from this software without
   specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
`
const license_NetCDF_lre = `//**
NetCDF license
https://spdx.org/licenses/NetCDF.json
http://www.unidata.ucar.edu/software/netcdf/copyright.html
**//

//** Copyright **//

Portions of this software were developed by the Unidata Program at the
University Corporation for Atmospheric Research.

Access and use of this software shall impose the following obligations and
understandings on the user. The user is granted the right, without any fee or
cost, to use, copy, modify, alter, enhance and distribute this software, and any
derivative works thereof, and its supporting documentation for any purpose
whatsoever, provided that this entire notice appears in all copies of the
software, derivative works and supporting documentation. Further, UCAR requests
that the user credit UCAR/Unidata in any publications that result from the use
of this software or in any product that includes this software, although this is
not an obligation. The names UCAR and/or Unidata, however, may not be used in
any advertising or publicity to endorse or promote any products or commercial
entity unless specific written permission is obtained from UCAR/Unidata. The
user also understands that UCAR/Unidata is not obligated to provide the user
with any support, consulting, training or assistance of any kind with regard to
the use, operation and performance of this software nor to provide the user with
any updates, revisions, new versions or "bug fixes."

THIS SOFTWARE IS PROVIDED BY UCAR/UNIDATA "AS IS" AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL UCAR/UNIDATA BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES
OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.
`
const license_Newsletr_lre = `//**
Newsletr License
https://spdx.org/licenses/Newsletr.json
https://fedoraproject.org/wiki/Licensing/Newsletr
**//

//** Copyright **//

Permission is granted to anyone to use this software for any purpose on any
computer system, and to redistribute it freely, subject to the following
restrictions:

   (( 1. ))??
   This software is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.

   (( 2. ))??
   Altered versions must be plainly marked as such, and must not be
   misrepresented as being the original software.
`
const license_Nokia_lre = `//**
Nokia Open Source License
https://spdx.org/licenses/Nokia.json
https://opensource.org/licenses/nokia
**//

(( Nokia Open Source License (NOKOS License)

Version 1.0a ))??

(( 1. ))??
DEFINITIONS.

"Affiliates" of a party shall mean an entity

   (( a) ))??
   which is directly or indirectly controlling such party;

   (( b) ))??
   which is under the same direct or indirect ownership or control as such
   party; or

   (( c) ))??
   which is directly or indirectly owned or controlled by such party.

For these purposes, an entity shall be treated as being controlled by another if
that other entity has fifty percent (50%) or more of the votes in such entity,
is able to direct its affairs and/or to control the composition of its board of
directors or equivalent body.

"Commercial Use" shall mean distribution or otherwise making the Covered
Software available to a third party.

"Contributor" shall mean each entity that creates or contributes to the creation
of Modifications.

"Contributor Version" shall mean in case of any Contributor the combination of
the Original Software, prior Modifications used by a Contributor, and the
Modifications made by that particular Contributor and in case of Nokia in
addition the Original Software in any form, including the form as Exceutable.

"Covered Software" shall mean the Original Software or Modifications or the
combination of the Original Software and Modifications, in each case including
portions thereof.

"Electronic Distribution Mechanism" shall mean a mechanism generally accepted in
the software development community for the electronic transfer of data.

"Executable" shall mean Covered Software in any form other than Source Code.

"Nokia" shall mean Nokia Corporation and its Affiliates.

"Larger Work" shall mean a work, which combines Covered Software or portions
thereof with code not governed by the terms of this License.

"License" shall mean this document.

"Licensable" shall mean having the right to grant, to the maximum extent
possible, whether at the time of the initial grant or subsequently acquired, any
and all of the rights conveyed herein.

"Modifications" shall mean any addition to or deletion from the substance or
structure of either the Original Software or any previous Modifications. When
Covered Software is released as a series of files, a Modification is:

   (( a) ))??
   Any addition to or deletion from the contents of a file containing Original
   Software or previous Modifications.

   (( b) ))??
   Any new file that contains any part of the Original Software or previous
   Modifications.

"Original Software" shall mean the Source Code of computer software code which
is described in the Source Code notice required by Exhibit A as Original
Software, and which, at the time of its release under this License is not
already Covered Software governed by this License.

"Patent Claims" shall mean any patent claim(s), now owned or hereafter acquired,
including without limitation, method, process, and apparatus claims, in any
patent Licensable by grantor.

"Source Code" shall mean the preferred form of the Covered Software for making
modifications to it, including all modules it contains, plus any associated
interface definition files, scripts used to control compilation and installation
of an Executable, or source code differential comparisons against either the
Original Software or another well known, available Covered Software of the
Contributor's choice. The Source Code can be in a compressed or archival form,
provided the appropriate decompression or de-archiving software is widely
available for no charge.

"You" (or "Your") shall mean an individual or a legal entity exercising rights
under, and complying with all of the terms of, this License or a future version
of this License issued under Section 6.1. For legal entities, "You" includes
Affiliates of such entity.
(( 2. ))??
SOURCE CODE LICENSE.

   (( 2.1 ))??
   Nokia Grant.

   Subject to the terms of this License, Nokia hereby grants You a world-wide,
   royalty-free, non-exclusive license, subject to third party intellectual
   property claims:

      (( a) ))??
      under copyrights Licensable by Nokia to use, reproduce, modify, display,
      perform, sublicense and distribute the Original Software (or portions
      thereof) with or without Modifications, and/or as part of a Larger Work;

      (( b) ))??
      and under Patents Claims necessarily infringed by the making, using or
      selling of Original Software, to make, have made, use, practice, sell, and
      offer for sale, and/or otherwise dispose of the Original Software (or
      portions thereof).

      (( c) ))??
      The licenses granted in this Section 2.1(a) and (b) are effective on the
      date Nokia first distributes Original Software under the terms of this
      License.

      (( d) ))??
      Notwithstanding Section 2.1(b) above, no patent license is granted: 1) for
      code that You delete from the Original Software; 2) separate from the
      Original Software; or 3) for infringements caused by: i) the modification
      of the Original Software or ii) the combination of the Original Software
      with other software or devices.

   (( 2.2 ))??
   Contributor Grant.

   Subject to the terms of this License and subject to third party intellectual
   property claims, each Contributor hereby grants You a world-wide,
   royalty-free, non-exclusive license

      (( a) ))??
      under copyrights Licensable by Contributor, to use, reproduce, modify,
      display, perform, sublicense and distribute the Modifications created by
      such Contributor (or portions thereof) either on an unmodified basis, with
      other Modifications, as Covered Software and/or as part of a Larger Work;
      and

      (( b) ))??
      under Patent Claims necessarily infringed by the making, using, or selling
      of Modifications made by that Contributor either alone and/or in
      combination with its Contributor Version (or portions of such
      combination), to make, use, sell, offer for sale, have made, and/or
      otherwise dispose of: 1) Modifications made by that Contributor (or
      portions thereof); and 2) the combination of Modifications made by that
      Contributor with its Contributor Version (or portions of such
      combination).

      (( c) ))??
      The licenses granted in Sections 2.2(a) and 2.2(b) are effective on the
      date Contributor first makes Commercial Use of the Covered Software.

      (( d) ))??
      Notwithstanding Section 2.2(b) above, no patent license is granted: 1) for
      any code that Contributor has deleted from the Contributor Version; 2)
      separate from the Contributor Version; 3) for infringements caused by: i)
      third party modifications of Contributor Version or ii) the combination of
      Modifications made by that Contributor with other software (except as part
      of the Contributor Version) or other devices; or 4) under Patent Claims
      infringed by Covered Software in the absence of Modifications made by that
      Contributor.
      (( 3. ))??
      DISTRIBUTION OBLIGATIONS.

   (( 3.1 ))??
   Application of License.

   The Modifications which You create or to which You contribute are governed by
   the terms of this License, including without limitation Section 2.2. The
   Source Code version of Covered Software may be distributed only under the
   terms of this License or a future version of this License released under
   Section 6.1, and You must include a copy of this License with every copy of
   the Source Code You distribute. You may not offer or impose any terms on any
   Source Code version that alters or restricts the applicable version of this
   License or the recipients' rights hereunder. However, You may include an
   additional document offering the additional rights described in Section 3.5.

   (( 3.2 ))??
   Availability of Source Code.

   Any Modification which You create or to which You contribute must be made
   available in Source Code form under the terms of this License either on the
   same media as an Executable version or via an accepted Electronic
   Distribution Mechanism to anyone to whom you made an Executable version
   available; and if made available via Electronic Distribution Mechanism, must
   remain available for at least twelve (12) months after the date it initially
   became available, or at least six (6) months after a subsequent version of
   that particular Modification has been made available to such recipients. You
   are responsible for ensuring that the Source Code version remains available
   even if the Electronic Distribution Mechanism is maintained by a third
   party.

   (( 3.3 ))??
   Description of Modifications.

   You must cause all Covered Software to which You contribute to contain a file
   documenting the changes You made to create that Covered Software and the date
   of any change. You must include a prominent statement that the Modification
   is derived, directly or indirectly, from Original Software provided by Nokia
   and including the name of Nokia in (a) the Source Code, and (b) in any notice
   in an Executable version or related documentation in which You describe the
   origin or ownership of the Covered Software.

   (( 3.4 ))??
   Intellectual Property Matters

      (( (a) ))??
      Third Party Claims. If Contributor has knowledge that a license under a
      third party's intellectual property rights is required to exercise the
      rights granted by such Contributor under Sections 2.1 or 2.2, Contributor
      must include a text file with the Source Code distribution titled "LEGAL"
      which describes the claim and the party making the claim in sufficient
      detail that a recipient will know whom to contact. If Contributor obtains
      such knowledge after the Modification is made available as described in
      Section 3.2, Contributor shall promptly modify the LEGAL file in all
      copies Contributor makes available thereafter and shall take other steps
      (such as notifying appropriate mailing lists or newsgroups) reasonably
      calculated to inform those who received the Covered Software that new
      knowledge has been obtained.

      (( (b) ))??
      Contributor APIs. If Contributor's Modifications include an application
      programming interface and Contributor has knowledge of patent licenses
      which are reasonably necessary to implement that API, Contributor must
      also include this information in the LEGAL file.

      (( c ))??
      Representations. Contributor represents that, except as disclosed pursuant
      to Section 3.4(a) above, Contributor believes that Contributor's
      Modifications are Contributor's original creation(s) and/or Contributor
      has sufficient rights to grant the rights conveyed by this License.

   (( 3.5 ))??
   Required Notices.

   You must duplicate the notice in Exhibit A in each file of the Source Code.
   If it is not possible to put such notice in a particular Source Code file due
   to its structure, then You must include such notice in a location (such as a
   relevant directory) where a user would be likely to look for such a notice.
   If You created one or more Modification(s) You may add your name as a
   Contributor to the notice described in Exhibit A. You must also duplicate
   this License in any documentation for the Source Code where You describe
   recipients' rights or ownership rights relating to Covered Software. You may
   choose to offer, and to charge a fee for, warranty, support, indemnity or
   liability obligations to one or more recipients of Covered Software. However,
   You may do so only on Your own behalf, and not on behalf of Nokia or any
   Contributor. You must make it absolutely clear that any such warranty,
   support, indemnity or liability obligation is offered by You alone, and You
   hereby agree to indemnify Nokia and every Contributor for any liability
   incurred by Nokia or such Contributor as a result of warranty, support,
   indemnity or liability terms You offer.

   (( 3.6 ))??
   Distribution of Executable Versions.

   You may distribute Covered Software in Executable form only if the
   requirements of Section 3.1-3.5 have been met for that Covered Software, and
   if You include a notice stating that the Source Code version of the Covered
   Software is available under the terms of this License, including a
   description of how and where You have fulfilled the obligations of Section
   3.2. The notice must be conspicuously included in any notice in an Executable
   version, related documentation or collateral in which You describe
   recipients' rights relating to the Covered Software. You may distribute the
   Executable version of Covered Software or ownership rights under a license of
   Your choice, which may contain terms different from this License, provided
   that You are in compliance with the terms of this License and that the
   license for the Executable version does not attempt to limit or alter the
   recipient's rights in the Source Code version from the rights set forth in
   this License. If You distribute the Executable version under a different
   license You must make it absolutely clear that any terms which differ from
   this License are offered by You alone, not by Nokia or any Contributor. You
   hereby agree to indemnify Nokia and every Contributor for any liability
   incurred by Nokia or such Contributor as a result of any such terms You
   offer.

   (( 3.7 ))??
   Larger Works.

   You may create a Larger Work by combining Covered Software with other
   software not governed by the terms of this License and distribute the Larger
   Work as a single product. In such a case, You must make sure the requirements
   of this License are fulfilled for the Covered Software.
   (( 4. ))??
   INABILITY TO COMPLY DUE TO STATUTE OR REGULATION.

If it is impossible for You to comply with any of the terms of this License with
respect to some or all of the Covered Software due to statute, judicial order,
or regulation then You must: (a) comply with the terms of this License to the
maximum extent possible; and (b) describe the limitations and the code they
affect. Such description must be included in the LEGAL file described in Section
3.4 and must be included with all distributions of the Source Code.

Except to the extent prohibited by statute or regulation, such description must
be sufficiently detailed for a recipient of ordinary skill to be able to
understand it.
(( 5. ))??
APPLICATION OF THIS LICENSE.

This License applies to code to which Nokia has attached the notice in Exhibit A
and to related Covered Software.
(( 6. ))??
VERSIONS OF THE LICENSE.

   (( 6.1 ))??
   New Versions.

   Nokia may publish revised and/or new versions of the License from time to
   time. Each version will be given a distinguishing version number.

   (( 6.2 ))??
   Effect of New Versions.

   Once Covered Software has been published under a particular version of the
   License, You may always continue to use it under the terms of that version.
   You may also choose to use such Covered Software under the terms of any
   subsequent version of the License published by Nokia. No one other than Nokia
   has the right to modify the terms applicable to Covered Software created
   under this License.
   (( 7. ))??
   DISCLAIMER OF WARRANTY.

COVERED SOFTWARE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
LIMITATION, WARRANTIES THAT THE COVERED SOFTWARE IS FREE OF DEFECTS,
MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK AS
TO THE QUALITY AND PERFORMANCE OF THE COVERED SOFTWARE IS WITH YOU. SHOULD ANY
COVERED SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT NOKIA, ITS LICENSORS
OR AFFILIATES OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY
SERVICING, REPAIR OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN
ESSENTIAL PART OF THIS LICENSE. NO USE OF ANY COVERED SOFTWARE IS AUTHORIZED
HEREUNDER EXCEPT UNDER THIS DISCLAIMER.
(( 8. ))??
TERMINATION.

   (( 8.1 ))??
   This License and the rights granted hereunder will terminate automatically if
   You fail to comply with terms herein and fail to cure such breach within 30
   days of becoming aware of the breach. All sublicenses to the Covered Software
   which are properly granted shall survive any termination of this License.
   Provisions which, by their nature, must remain in effect beyond the
   termination of this License shall survive.

   (( 8.2 ))??
   If You initiate litigation by asserting a patent infringement claim
   (excluding declatory judgment actions) against Nokia or a Contributor (Nokia
   or Contributor against whom You file such action is referred to as
   "Participant") alleging that:

      (( a) ))??
      such Participant's Contributor Version directly or indirectly infringes
      any patent, then any and all rights granted by such Participant to You
      under Sections 2.1 and/or 2.2 of this License shall, upon 60 days notice
      from Participant terminate prospectively, unless if within 60 days after
      receipt of notice You either: (i) agree in writing to pay Participant a
      mutually agreeable reasonable royalty for Your past and future use of
      Modifications made by such Participant, or (ii) withdraw Your litigation
      claim with respect to the Contributor Version against such Participant. If
      within 60 days of notice, a reasonable royalty and payment arrangement are
      not mutually agreed upon in writing by the parties or the litigation claim
      is not withdrawn, the rights granted by Participant to You under Sections
      2.1 and/or 2.2 automatically terminate at the expiration of the 60 day
      notice period specified above.

      (( b) ))??
      any software, hardware, or device, other than such Participant's
      Contributor Version, directly or indirectly infringes any patent, then any
      rights granted to You by such Participant under Sections 2.1(b) and 2.2(b)
      are revoked effective as of the date You first made, used, sold,
      distributed, or had made, Modifications made by that Participant.

   (( 8.3 ))??
   If You assert a patent infringement claim against Participant alleging that
   such Participant's Contributor Version directly or indirectly infringes any
   patent where such claim is resolved (such as by license or settlement) prior
   to the initiation of patent infringement litigation, then the reasonable
   value of the licenses granted by such Participant under Sections 2.1 or 2.2
   shall be taken into account in determining the amount or value of any payment
   or license.

   (( 8.4 ))??
   In the event of termination under Sections 8.1 or 8.2 above, all end user
   license agreements (excluding distributors and resellers) which have been
   validly granted by You or any distributor hereunder prior to termination
   shall survive termination.
   (( 9. ))??
   LIMITATION OF LIABILITY.

UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, NOKIA, ANY OTHER CONTRIBUTOR, OR
ANY DISTRIBUTOR OF COVERED SOFTWARE, OR ANY SUPPLIER OF ANY OF SUCH PARTIES, BE
LIABLE TO ANY PERSON FOR ANY INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF
GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER
COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH PARTY SHALL HAVE BEEN INFORMED OF THE
POSSIBILITY OF SUCH DAMAGES. THIS LIMITATION OF LIABILITY SHALL NOT APPLY TO
LIABILITY FOR DEATH OR PERSONAL INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO
THE EXTENT APPLICABLE LAW PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT
ALLOW THE EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, BUT
MAY ALLOW LIABILITY TO BE LIMITED; IN SUCH CASES, A PARTY's, ITS EMPLOYEES,
LICENSORS OR AFFILIATES' LIABILITY SHALL BE LIMITED TO U.S. $50. Nothing
contained in this License shall prejudice the statutory rights of any party
dealing as a consumer.
(( 10. ))??
MISCELLANEOUS.

This License represents the complete agreement concerning subject matter hereof.
All rights in the Covered Software not expressly granted under this License are
reserved. Nothing in this License shall grant You any rights to use any of the
trademarks of Nokia or any of its Affiliates, even if any of such trademarks are
included in any part of Covered Software and/or documentation to it.

This License is governed by the laws of Finland excluding its conflict-of-law
provisions. All disputes arising from or relating to this Agreement shall be
settled by a single arbitrator appointed by the Central Chamber of Commerce of
Finland. The arbitration procedure shall take place in Helsinki, Finland in the
English language. If any part of this Agreement is found void and unenforceable,
it will not affect the validity of the balance of the Agreement, which shall
remain valid and enforceable according to its terms.
(( 11. ))??
RESPONSIBILITY FOR CLAIMS.

As between Nokia and the Contributors, each party is responsible for claims and
damages arising, directly or indirectly, out of its utilization of rights under
this License and You agree to work with Nokia and Contributors to distribute
such responsibility on an equitable basis. Nothing herein is intended or shall
be deemed to constitute any admission of liability.
(( EXHIBIT A

The contents of this file are subject to the NOKOS License Version 1.0 (the
"License"); you may not use this file except in compliance with the License.

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

The Original Software is

_.

Copyright © <year> Nokia and others. All Rights Reserved.

Contributor(s): _. ))??
`
const license_Noweb_lre = `//**
Noweb License
https://spdx.org/licenses/Noweb.json
https://fedoraproject.org/wiki/Licensing/Noweb
**//

//** Copyright **//

((Noweb is Copyright __20__))??

Noweb is protected by copyright. It is not public-domain software or shareware,
and it is not protected by a ` + "`" + `` + "`" + `copyleft'' agreement like the one used by the
Free Software Foundation.

Noweb is available free for any use in any field of endeavor. You may
redistribute noweb in whole or in part provided you acknowledge its source and
include this COPYRIGHT file. You may modify noweb and create derived works,
provided you retain this copyright notice, but the result may not be called
noweb without my written consent.

You may sell noweb if you wish. For example, you may sell a CD-ROM including
noweb.

You may sell a derived work, provided that all source code for your derived work
is available, at no additional charge, to anyone who buys your derived work in
any form. You must give permisson for said source code to be used and modified
under the terms of this license. You must state clearly that your work uses or
is based on noweb and that noweb is available free of change. You must also
request that bug reports on your work be reported to you.
`
const license_O_UDA_1_0_lre = `//**
Open Use of Data Agreement v1.0
https://spdx.org/licenses/O-UDA-1.0.json
https://github.com/microsoft/Open-Use-of-Data-Agreement/blob/v1.0/O-UDA-1.0.md
**//

(( Open Use of Data Agreement v1.0 ))??

This is the Open Use of Data Agreement, Version 1.0 (the "O-UDA"). Capitalized
terms are defined in Section 5. Data Provider and you agree as follows:

   (( 1. ))??
   Provision of the Data

      (( 1.1. ))??
      You may use, modify, and distribute the Data made available to you by the
      Data Provider under this O-UDA if you follow the O-UDA's terms.

      (( 1.2. ))??
      Data Provider will not sue you or any Downstream Recipient for any claim
      arising out of the use, modification, or distribution of the Data provided
      you meet the terms of the O-UDA.

      (( 1.3. ))??
      This O-UDA does not restrict your use, modification, or distribution of
      any portions of the Data that are in the public domain or that may be
      used, modified, or distributed under any other legal exception or
      limitation.

   (( 2. ))??
   No Restrictions on Use or Results

      (( 2.1. ))??
      The O-UDA does not impose any restriction with respect to:

         (( 2.1.1. ))??
         the use or modification of Data; or

         (( 2.1.2. ))??
         the use, modification, or distribution of Results.

   (( 3. ))??
   Redistribution of Data

      (( 3.1. ))??
      You may redistribute the Data under terms of your choice, so long as:

         (( 3.1.1. ))??
         You include with any Data you redistribute all credit or attribution
         information that you received with the Data, and your terms require any
         Downstream Recipient to do the same; and

         (( 3.1.2. ))??
         Your terms include a warranty disclaimer and limitation of liability
         for Upstream Data Providers at least as broad as those contained in
         Section 4.2 and 4.3 of the O-UDA.

   (( 4. ))??
   No Warranty, Limitation of Liability

      (( 4.1. ))??
      Data Provider does not represent or warrant that it has any rights
      whatsoever in the Data.

      (( 4.2. ))??
      THE DATA IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
      OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION, ANY
      WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR
      FITNESS FOR A PARTICULAR PURPOSE.

      (( 4.3. ))??
      NEITHER DATA PROVIDER NOR ANY UPSTREAM DATA PROVIDER SHALL HAVE ANY
      LIABILITY FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
      CONSEQUENTIAL DAMAGES (INCLUDING WITHOUT LIMITATION LOST PROFITS), HOWEVER
      CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
      LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
      OUT OF THE DATA OR RESULTS, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
      DAMAGES.

   (( 5. ))??
   Definitions

      (( 5.1. ))??
      "Data" means the material you receive under the O-UDA in modified or
      unmodified form, but not including Results.

      (( 5.2. ))??
      "Data Provider" means the source from which you receive the Data and with
      whom you enter into the O-UDA.

      (( 5.3. ))??
      "Downstream Recipient" means any person or persons who receives the Data
      directly or indirectly from you in accordance with the O-UDA.

      (( 5.4. ))??
      "Result" means anything that you develop or improve from your use of Data
      that does not include more than a de minimis portion of the Data on which
      the use is based. Results may include de minimis portions of the Data
      necessary to report on or explain use that has been conducted with the
      Data, such as figures in scientific papers, but do not include more.
      Artificial intelligence models trained on Data (and which do not include
      more than a de minimis portion of Data) are Results.

      (( 5.5. ))??
      "Upstream Data Providers" means the source or sources from which the Data
      Provider directly or indirectly received, under the terms of the O-UDA,
      material that is included in the Data.
`
const license_OCCT_PL_lre = `//**
Open CASCADE Technology Public License
https://spdx.org/licenses/OCCT-PL.json
http://www.opencascade.com/content/occt-public-license
**//

(( Open CASCADE Technology Public License

Version 6.6, April 2013 ))??

OPEN CASCADE releases and makes publicly available the source code of the
software Open CASCADE Technology to the free software development community
under the terms and conditions of this license.

It is not the purpose of this license to induce you to infringe any patents or
other property right claims or to contest validity of any such claims; this
license has the sole purpose of protecting the integrity of the free software
distribution system, which is implemented by public license practices. Many
people have made generous contributions to the wide range of software
distributed through that system in reliance on consistent application of that
system; it is up to the author/donor to decide if he or she is willing to
distribute software through any other system and a licensee cannot impose that
choice.

Please read this license carefully and completely before downloading this
software. By downloading, using, modifying, distributing and sublicensing this
software, you indicate your acceptance to be bound by the terms and conditions
of this license. If you do not want to accept or cannot accept for any reasons
the terms and conditions of this license, please do not download or use in any
manner this software.

   (( 1. ))??
   Definitions

   Unless there is something in the subject matter or in the context
   inconsistent therewith, the capitalized terms used in this License shall have
   the following meaning.

   "Applicable Intellectual Property Rights" means (a) with respect to the
   Initial Developer, any rights under patents or patents applications or other
   intellectual property rights that are now or hereafter acquired, owned by or
   assigned to the Initial Developer and that cover subject matter contained in
   the Original Code, but only to the extent necessary to use, reproduce,
   modify, distribute or sublicense the Original Code without infringement; and
   (b) with respect to You or any Contributor, any rights under patents or
   patents applications or other intellectual property rights that are now or
   hereafter acquired, owned by or assigned to You or to such Contributor and
   that cover subject matter contained in Your Modifications or in such
   Contributor's Modifications, taken alone or in combination with Original
   Code.

   "Contributor" means each individual or legal entity that creates or
   contributes to the creation of any Modification, including the Initial
   Developer.

   "Derivative Program": means a new program combining the Software or portions
   thereof with other source code not governed by the terms of this License.

   "Initial Developer": means OPEN CASCADE, with main offices at 1, place des
   Frères Montgolfier, 78280, Guyancourt, France.

   "Modifications": mean any addition to, deletion from or change to the
   substance or the structure of the Software. When source code of the Software
   is released as a series of files, a Modification is: (a) any addition to,
   deletion from or change to the contents of a file containing the Software or
   (b) any new file or other representation of computer program statements that
   contains any part of the Software. By way of example, Modifications include
   any debug of, or improvement to, the Original Code or any of its components
   or portions as well as its next versions or releases thereof.

   "Original Code": means (a) the source code of the software Open CASCADE
   Technology originally made available by the Initial Developer under this
   License, including the source code of any updates or upgrades of the Original
   Code and (b) the object code compiled from such source code and originally
   made available by Initial Developer under this License.

   "Software": means the Original Code, the Modifications, the combination of
   Original Code and any Modifications or any respective portions thereof.

   "You" or "Your": means an individual or a legal entity exercising rights
   under this License

   (( 2. ))??
   Acceptance of license

   By using, reproducing, modifying, distributing or sublicensing the Software
   or any portion thereof, You expressly indicate Your acceptance of the terms
   and conditions of this License and undertake to act in accordance with all
   the provisions of this License applicable to You.

   (( 3. ))??
   Scope and purpose

   This License applies to the Software and You may not use, reproduce, modify,
   distribute, sublicense or circulate the Software, or any portion thereof,
   except as expressly provided under this License. Any attempt to otherwise
   use, reproduce, modify, distribute or sublicense the Software is void and
   will automatically terminate Your rights under this License.

   (( 4. ))??
   Contributor license

   Subject to the terms and conditions of this License, the Initial Developer
   and each of the Contributors hereby grant You a world-wide, royalty-free,
   irrevocable and non-exclusive license under the Applicable Intellectual
   Property Rights they own or control, to use, reproduce, modify, distribute
   and sublicense the Software provided that:

   You reproduce in all copies of the Software the copyright and other
   proprietary notices and disclaimers of the Initial Developer as they appear
   in the Original Code and attached hereto as Schedule "A" and any other
   notices or disclaimers attached to the Software and keep intact all notices
   in the Original Code that refer to this License and to the absence of any
   warranty;

   You include a copy of this License with every copy of the Software You
   distribute;

   If you distribute or sublicense the Software (as modified by You or on Your
   behalf as the case may be), You cause such Software to be licensed as a
   whole, at no charge, to all third parties, under the terms and conditions of
   the License, making in particular available to all third parties the source
   code of the Software;

   You document all Your Modifications, indicate the date of each such
   Modification, designate the version of the Software You used, prominently
   include a file carrying such information with respect to the Modifications
   and duplicate the copyright and other proprietary notices and disclaimers
   attached hereto as Schedule "B" or any other notices or disclaimers attached
   to the Software with your Modifications.

   For greater certainty, it is expressly understood that You may freely create
   Derivative Programs (without any obligation to publish such Derivative
   Program) and distribute same as a single product. In such case, You must
   ensure that all the requirements of this License are fulfilled for the
   Software or any portion thereof.

   (( 5. ))??
   Your license

   You hereby grant all Contributors and anyone who becomes a party under this
   License a world-wide, non-exclusive, royalty-free and irrevocable license
   under the Applicable Intellectual Property Rights owned or controlled by You,
   to use, reproduce, modify, distribute and sublicense all Your Modifications
   under the terms and conditions of this License.

   (( 6. ))??
   Software subject to license

   Your Modifications shall be governed by the terms and conditions of this
   License. You are not authorized to impose any other terms or conditions than
   those prevailing under this License when You distribute and/or sublicense the
   Software, save and except as permitted under Section 7 hereof.

   (( 7. ))??
   Additional terms

   You may choose to offer, on a non-exclusive basis, and to charge a fee for
   any warranty, support, maintenance, liability obligations or other rights
   consistent with the scope of this License with respect to the Software (the
   "Additional Terms") to the recipients of the Software. However, You may do so
   only on Your own behalf and on Your sole and exclusive responsibility. You
   must obtain the recipient's agreement that any such Additional Terms are
   offered by You alone, and You hereby agree to indemnify, defend and hold the
   Initial Developer and any Contributor harmless for any liability incurred by
   or claims asserted against the Initial Developer or any Contributors with
   respect to any such Additional Terms.

   (( 8. ))??
   Disclaimer of warranty

   The Software is provided under this License on an "as is" basis, without
   warranty of any kind, including without limitation, warranties that the
   Software is free of defects, merchantable, fit for a particular purpose or
   non-infringing. The entire risk as to the quality and performance of the
   Software is with You.

   (( 9. ))??
   Liability

   Under no circumstances shall You, the Initial Developer or any Contributor be
   liable to any person for any direct or indirect damages of any kind
   including, without limitation, damages for loss of goodwill, loss of data,
   work stoppage, computer failure or malfunction or any and all other
   commercial damages or losses resulting from or relating to this License or
   indirectly to the use of the Software.

   (( 10. ))??
   Trademark

   This License does not grant any rights to use the trademarks, trade names and
   domain names "MATRA", "EADS Matra Datavision", "CAS.CADE", "Open CASCADE",
   "opencascade.com" and "opencascade.org" or any other trademarks, trade names
   or domain names used or owned by the Initial Developer.

   (( 11. ))??
   Copyright

   The Initial Developer retains all rights, title and interest in and to the
   Original Code. You may not remove the copyright © notice which appears when
   You download the Software.

   (( 12. ))??
   Term

   This License is granted to You for a term equal to the remaining period of
   protection covered by the intellectual property rights applicable to the
   Original Code.

   (( 13. ))??
   Termination

   In case of termination, as provided in Section 3 above, You agree to
   immediately stop any further use, reproduction, modification, distribution
   and sublicensing of the Software and to destroy all copies of the Software
   that are in Your possession or control. All sublicenses of the Software which
   have been properly granted prior to termination shall survive any termination
   of this License. In addition, Sections 5, 8 to 11, 13.2 and 15.2 of this
   License, in reason of their nature, shall survive the termination of this
   License for a period of fifteen (15) years.

   (( 14. ))??
   Versions of the license

   The Initial Developer may publish new versions of this License from time to
   time. Once Original Code has been published under a particular version of
   this License, You may choose to continue to use it under the terms and
   conditions of that version or use the Original Code under the terms of any
   subsequent version of this License published by the Initial Developer.

   (( 15. ))??
   Miscellaneous

      (( 15.1 ))??
      Relationship of the Parties This License will not be construed as creating
      an agency, partnership, joint venture or any other form of legal
      association between You and the Initial Developer, and You will not
      represent to the contrary, whether expressly, by implication or
      otherwise.

      (( 15.2 ))??
      Independent Development Nothing in this License will impair the Initial
      Developer's right to acquire, license, develop, have others develop for
      it, market or distribute technology or products that perform the same or
      similar functions as, or otherwise compete with, Modifications, Derivative
      Programs, technology or products that You may develop, produce, market or
      distribute.

      (( 15.3 ))??
      Severability If for any reason a court of competent jurisdiction finds any
      provision of this License, or portion thereof, to be unenforceable, that
      provision of the License will be enforced to the maximum extent
      permissible so as to effect the economic benefits and intent of the
      parties, and the remainder of this License will continue in full force and
      extent.
      (( END OF THE TERMS AND CONDITIONS OF THIS LICENSE

OPEN CASCADE is a French société par actions simplifiée having its registered
head office at 1, place des Frères Montgolfier, 78280, Guyancourt, France and
main offices at 1, place des Frères Montgolfier, 78280, Guyancourt, France. Its
web site is located at the following address opencascade.com

Open CASCADE Technology Public License

Schedule "A"

The content of this file is subject to the Open CASCADE Technology Public
License (the "License"). You may not use the content of this file except in
compliance with the License. Please obtain a copy of the License at
opencascade.com and read it completely before using this file.

The Initial Developer of the Original Code is OPEN CASCADE, with main offices at
1, place des Frères Montgolfier, 78280, Guyancourt, France. The Original Code
is copyright © OPEN CASCADE SAS, 2001. All rights reserved. "The Original Code
and all software distributed under the License are distributed on an "AS IS"
basis, without warranty of any kind, and the Initial Developer hereby disclaims
all such warranties, including without limitation, any warranties of
merchantability, fitness for a particular purpose or non-infringement.

Please see the License for the specific terms and conditions governing rights
and limitations under the License".

End of Schedule "A"

Open CASCADE Technology Public License

Schedule "B"

"The content of this file is subject to the Open CASCADE Technology Public
License (the "License"). You may not use the content of this file except in
compliance with the License. Please obtain a copy of the License at
opencascade.com and read it completely before using this file.

The Initial Developer of the Original Code is OPEN CASCADE, with main offices at
1, place des Frères Montgolfier, 78280, Guyancourt, France. The Original Code
is copyright © Open CASCADE SAS, 2001. All rights reserved.

Modifications to the Original Code have been made by _. Modifications are
copyright © [Year to be included]. All rights reserved.

The software Open CASCADE Technology and all software distributed under the
License are distributed on an "AS IS" basis, without warranty of any kind, and
the Initial Developer hereby disclaims all such warranties, including without
limitation, any warranties of merchantability, fitness for a particular purpose
or non-infringement.

Please see the License for the specific terms and conditions governing rights
and limitations under the License"

End of Schedule "B" ))??
`
const license_OCLC_2_0_lre = `//**
OCLC Research Public License 2.0
https://spdx.org/licenses/OCLC-2.0.json
http://www.oclc.org/research/activities/software/license/v2final.htm
https://opensource.org/licenses/OCLC-2.0
**//

(( OCLC Research Public License 2.0
Terms & Conditions Of Use
May, 2002

(( Copyright __20__ ))??
))??

PLEASE READ THIS DOCUMENT CAREFULLY. BY DOWNLOADING OR USING THE CODE BASE
AND/OR DOCUMENTATION ACCOMPANYING THIS LICENSE (THE "License"), YOU AGREE TO THE
FOLLOWING TERMS AND CONDITIONS OF THIS LICENSE.

Section 1. Your Rights

Subject to these terms and conditions of this License, the OCLC Office of
Research (the "Original Contributor") and each subsequent contributor
(collectively with the Original Contributor, the "Contributors") hereby grant
you a non-exclusive, worldwide, no-charge, transferable license to execute,
prepare derivative works of, and distribute (internally and externally), for
commercial and noncommercial purposes, the original code contributed by Original
Contributor and all Modifications (collectively called the "Program").

Section 2. Definitions

A "Modification" to the Program is any addition to or deletion from the contents
of any file of the Program and any new file that contains any part of the
Program. If you make a Modification and distribute the Program externally you
are a "Contributor." The distribution of the Program must be under the terms of
this license including those in Section 3 below.

A "Combined Work" results from combining and integrating all or parts of the
Program with other code. A Combined Work may be thought of as having multiple
parents or being result of multiple lines of code development.

Section 3. Distribution Licensing Terms

   (( A. ))??
   General Requirements

   Except as necessary to recognize third-party rights or third-party
   restriction (see below), a distribution of the Program in any of the forms
   listed below must not put any further restrictions on the recipient's
   exercise of the rights granted herein.

   As a Contributor, you represent that your Modification(s) are your original
   creation(s) and, to the best of your knowledge, no third party has any claim
   (including but not limited to intellectual property claims) relating to your
   Modification(s). You represent that each of your Modifications includes
   complete details of any third-party right or other third-party restriction
   associated with any part of your Modification (including a copy of any
   applicable license agreement).

   The Program must be distributed without charge beyond the costs of physically
   transferring the files to the recipient.

   This Warranty Disclaimer/Limitation of Liability must be prominently
   displayed with every distribution of the Program in any form:

   YOU AGREE THAT THE PROGRAM IS PROVIDED AS-IS, WITHOUT WARRANTY OF ANY KIND
   (EITHER EXPRESS OR IMPLIED). ACCORDINGLY, OCLC MAKES NO WARRANTIES,
   REPRESENTATIONS OR GUARANTEES, EITHER EXPRESS OR IMPLIED, AND DISCLAIMS ALL
   SUCH WARRANTIES, REPRESENTATIONS OR GUARANTEES, INCLUDING, WITHOUT
   LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
   PARTICULAR PURPOSE, AS TO: (A) THE FUNCTIONALITY OR NONINFRINGEMENT OF
   PROGRAM, ANY MODIFICATION, A COMBINED WORK OR AN AGGREGATE WORK; OR (B) THE
   RESULTS OF ANY PROJECT UNDERTAKEN USING THE PROGRAM, ANY MODIFICATION, A
   COMBINED WORK OR AN AGGREGATE WORK. IN NO EVENT SHALL THE CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
   CONSEQUENTIAL OR ANY OTHER DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
   OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
   OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THE PROGRAM, EVEN IF ADVISED
   OF THE POSSIBILITY OF SUCH DAMAGES. YOU HEREBY WAIVE ANY CLAIMS FOR DAMAGES
   OF ANY KIND AGAINST CONTRIBUTORS WHICH MAY RESULT FROM YOUR USE OF THE
   PROGRAM.

   (( B. ))??
   Requirements for a Distribution of Modifiable Code

   If you distribute the Program in a form to which the recipient can make
   Modifications (e.g. source code), the terms of this license apply to use by
   recipient. In addition, each source and data file of the Program and any
   Modification you distribute must contain the following notice:

   Copyright (c) __20__
   All rights reserved.

   The contents of this file, as updated from time to time
   by the OCLC Office of Research, are subject to OCLC Research Public License
   Version 2.0 (the "License"); you may not use this file except in compliance with
   the License. You may obtain a current copy of the License at
   http:/purl.oclc.org/oclc/research/ORPL/. Software distributed under the License
   is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
   or implied. See the License for the specific language governing rights and
   limitations under the License. This software consists of voluntary contributions
   made by many individuals on behalf of OCLC Research. For more information on
   OCLC Research, please see http:/www.oclc.org/research/.

   The Original Code is ____.
   The Initial Developer of the Original Code is ____.
   Portions created by ____ are Copyright (C) ____. All Rights Reserved.
   Contributor(s): ____.

   (( C. ))??
   Requirements for a Distribution of Non-modifiable Code

   If you distribute the Program in a form to which the recipient cannot make
   Modifications (e.g. object code), the terms of this license apply to use by
   recipient and you must include the following statement in appropriate and
   conspicuous locations:

   "Copyright (c) 2000- (insert then current year) OCLC Online Computer Library
   Center, Inc. and other contributors. All rights reserved."

   In addition, the source code must be included with the object code
   distribution or the distributor must provide the source code to the recipient
   upon request.

   (( D. ))??
   Requirements for a Combined Work Distribution

   Distributions of Combined Works are subject to the terms of this license and
   must be made at no charge to the recipient beyond the costs of physically
   transferring the files to recipient.

   A Combined Work may be distributed as either modifiable or non-modifiable
   code. The requirements of Section 3.B or 3.C above (as appropriate) apply to
   such distributions.

   An "Aggregate Work" is when the Program exists, without integration, with
   other programs on a storage medium. This License does not apply to portions
   of an Aggregate Work which are not covered by the definition of "Program"
   provided in this License. You are not forbidden from selling an Aggregate
   Work. However, the Program contained in an Aggregate Work is subject to this
   License. Also, should the Program be extracted from an Aggregate Work, this
   License applies to any use of the Program apart from the Aggregate Work.

   Section 4. License Grant

   For purposes of permitting use of your Modifications by OCLC and other
   licensees hereunder, you hereby grant to OCLC and such other licensees the
   non-exclusive, worldwide, royalty-free, transferable, sublicenseable license
   to execute, copy, alter, delete, modify, adapt, change, revise, enhance,
   develop, publicly display, distribute (internally and externally) and/or
   create derivative works based on your Modifications (and derivative works
   thereof) in accordance with these Terms. This Section 4 shall survive
   termination of this License for any reason.

Section 5. Termination of Rights

This non-exclusive license (with respect to the grant from a particular
Contributor) automatically terminates for any entity that initiates legal action
for intellectual property infringement (with respect to the Program) against
such Contributor as of the initiation of such action.

If you fail to comply with this License, your rights (but not your obligations)
under this License shall terminate automatically unless you cure such breach
within thirty (30) days of becoming aware of the noncompliance. All sublicenses
granted by you which preexist such termination and are properly granted shall
survive such termination.

Section 6. Other Terms

Except for the copyright notices required above, you may not use any trademark
of any of the Contributors without the prior written consent of the relevant
Contributor. You agree not to remove, alter or obscure any copyright or other
proprietary rights notice contained in the Program.

All transfers of the Program or any part thereof shall be made in compliance
with U.S. import/export regulations or other restrictions of the U.S. Department
of Commerce, as well as other similar trade or commerce restrictions which might
apply.

Any patent obtained by any party covering the Program or any part thereof must
include a provision providing for the free, perpetual and unrestricted
commercial and noncommercial use by any third party.

If, as a consequence of a court judgment or settlement relating to intellectual
property infringement or any other cause of action, conditions are imposed on
you that contradict the conditions of this License, such conditions do not
excuse you from compliance with this License. If you cannot distribute the
Program so as to simultaneously satisfy your obligations under this License and
such other conditions, you may not distribute the Program at all. For example,
if a patent license would not permit royalty-free redistribution of the Program
by all those who receive copies directly or indirectly through you, you could
not satisfy both the patent license and this License, and you would be required
to refrain entirely from distribution of the Program.

If you learn of a third party claim or other restriction relating to a Program
you have already distributed you shall promptly redo your Program to address the
issue and take all reasonable steps to inform those who may have received the
Program at issue. An example of an appropriate reasonable step to inform would
be posting an announcement on an appropriate web bulletin board.

The provisions of this License are deemed to be severable, and the invalidity or
unenforceability of any provision shall not affect or impair the remaining
provisions which shall continue in full force and effect. In substitution for
any provision held unlawful, there shall be substituted a provision of similar
import reflecting the original intent of the parties hereto to the extent
permissible under law.

The Original Contributor from time to time may change this License, and the
amended license will apply to all copies of the Program downloaded after the new
license is posted. This License grants only the rights expressly stated herein
and provides you with no implied rights or licenses to the intellectual property
of any Contributor.

This License is the complete and exclusive statement of the agreement between
the parties concerning the subject matter hereof and may not be amended except
by the written agreement of the parties. This License shall be governed by and
construed in accordance with the laws of the State of Ohio and the United States
of America, without regard to principles of conflicts of law.
`
const license_ODC_By_1_0_lre = `//**
Open Data Commons Attribution License v1.0
https://spdx.org/licenses/ODC-By-1.0.json
https://opendatacommons.org/licenses/by/1.0/
**//

((

ODC Attribution License (ODC-By) ))??

Preamble

The Open Data Commons Attribution License is a license agreement intended to
allow users to freely share, modify, and use this Database subject only to the
attribution requirements set out in Section 4.

Databases can contain a wide variety of types of content (images, audiovisual
material, and sounds all in the same database, for example), and so this license
only governs the rights over the Database, and not the contents of the Database
individually. Licensors may therefore wish to use this license together with
another license for the contents.

Sometimes the contents of a database, or the database itself, can be covered by
other rights not addressed here (such as private contracts, trademark over the
name, or privacy rights / data protection rights over information in the
contents), and so you are advised that you may have to consult other documents
or clear other rights before doing activities not covered by this License.

------

The Licensor (as defined below)

and

You (as defined below)

agree as follows:

   (( 1.0 ))??
   Definitions of Capitalised Words

   "Collective Database" - Means this Database in unmodified form as part of a
   collection of independent databases in themselves that together are assembled
   into a collective whole. A work that constitutes a Collective Database will
   not be considered a Derivative Database.

   "Convey" - As a verb, means Using the Database, a Derivative Database, or the
   Database as part of a Collective Database in any way that enables a Person to
   make or receive copies of the Database or a Derivative Database. Conveying
   does not include interaction with a user through a computer network, or
   creating and Using a Produced Work, where no transfer of a copy of the
   Database or a Derivative Database occurs.

   "Contents" - The contents of this Database, which includes the information,
   independent works, or other material collected into the Database. For
   example, the contents of the Database could be factual data or works such as
   images, audiovisual material, text, or sounds.

   "Database" - A collection of material (the Contents) arranged in a systematic
   or methodical way and individually accessible by electronic or other means
   offered under the terms of this License.

   "Database Directive" - Means Directive 96/9/EC of the European Parliament and
   of the Council of 11 March 1996 on the legal protection of databases, as
   amended or succeeded.

   "Database Right" - Means rights resulting from the Chapter III ("sui
   generis") rights in the Database Directive (as amended and as transposed by
   member states), which includes the Extraction and Re-utilisation of the whole
   or a Substantial part of the Contents, as well as any similar rights
   available in the relevant jurisdiction under Section 10.4.

   "Derivative Database" - Means a database based upon the Database, and
   includes any translation, adaptation, arrangement, modification, or any other
   alteration of the Database or of a Substantial part of the Contents. This
   includes, but is not limited to, Extracting or Re-utilising the whole or a
   Substantial part of the Contents in a new Database.

   "Extraction" - Means the permanent or temporary transfer of all or a
   Substantial part of the Contents to another medium by any means or in any
   form.

   "License" - Means this license agreement and is both a license of rights such
   as copyright and Database Rights and an agreement in contract.

   "Licensor" - Means the Person that offers the Database under the terms of
   this License.

   "Person" - Means a natural or legal person or a body of persons corporate or
   incorporate.

   "Produced Work" - a work (such as an image, audiovisual material, text, or
   sounds) resulting from using the whole or a Substantial part of the Contents
   (via a search or other query) from this Database, a Derivative Database, or
   this Database as part of a Collective Database.

   "Publicly" - means to Persons other than You or under Your control by either
   more than 50% ownership or by the power to direct their activities (such as
   contracting with an independent consultant).

   "Re-utilisation" - means any form of making available to the public all or a
   Substantial part of the Contents by the distribution of copies, by renting,
   by online or other forms of transmission.

   "Substantial" - Means substantial in terms of quantity or quality or a
   combination of both. The repeated and systematic Extraction or Re-utilisation
   of insubstantial parts of the Contents may amount to the Extraction or
   Re-utilisation of a Substantial part of the Contents.

   "Use" - As a verb, means doing any act that is restricted by copyright or
   Database Rights whether in the original medium or any other; and includes
   without limitation distributing, copying, publicly performing, publicly
   displaying, and preparing derivative works of the Database, as well as
   modifying the Database as may be technically necessary to use it in a
   different mode or format.

   "You" - Means a Person exercising rights under this License who has not
   previously violated the terms of this License with respect to the Database,
   or who has received express permission from the Licensor to exercise rights
   under this License despite a previous violation.

   Words in the singular include the plural and vice versa.

   (( 2.0 ))??
   What this License covers

   (( 2.1. ))??
   Legal effect of this document. This License is:

      (( a. ))??
      A license of applicable copyright and neighbouring rights;

      (( b. ))??
      A license of the Database Right; and

      (( c. ))??
      An agreement in contract between You and the Licensor.

   (( 2.2 ))??
   Legal rights covered. This License covers the legal rights in the Database,
   including:

      (( a. ))??
      Copyright. Any copyright or neighbouring rights in the Database. The
      copyright licensed includes any individual elements of the Database, but
      does not cover the copyright over the Contents independent of this
      Database. See Section 2.4 for details. Copyright law varies between
      jurisdictions, but is likely to cover: the Database model or schema, which
      is the structure, arrangement, and organisation of the Database, and can
      also include the Database tables and table indexes; the data entry and
      output sheets; and the Field names of Contents stored in the Database;

      (( b. ))??
      Database Rights. Database Rights only extend to the Extraction and
      Re-utilisation of the whole or a Substantial part of the Contents.
      Database Rights can apply even when there is no copyright over the
      Database. Database Rights can also apply when the Contents are removed
      from the Database and are selected and arranged in a way that would not
      infringe any applicable copyright; and

      (( c. ))??
      Contract. This is an agreement between You and the Licensor for access to
      the Database. In return you agree to certain conditions of use on this
      access as outlined in this License.

   (( 2.3 ))??
   Rights not covered.

      (( a. ))??
      This License does not apply to computer programs used in the making or
      operation of the Database;

      (( b. ))??
      This License does not cover any patents over the Contents or the Database;
      and

      (( c. ))??
      This License does not cover any trademarks associated with the Database.

   (( 2.4 ))??
   Relationship to Contents in the Database. The individual items of the
   Contents contained in this Database may be covered by other rights, including
   copyright, patent, data protection, privacy, or personality rights, and this
   License does not cover any rights (other than Database Rights or in contract)
   in individual Contents contained in the Database. For example, if used on a
   Database of images (the Contents), this License would not apply to copyright
   over individual images, which could have their own separate licenses, or one
   single license covering all of the rights over the images.

   (( 3.0 ))??
   Rights granted

   (( 3.1 ))??
   Subject to the terms and conditions of this License, the Licensor grants to
   You a worldwide, royalty-free, non-exclusive, terminable (but only under
   Section 9) license to Use the Database for the duration of any applicable
   copyright and Database Rights. These rights explicitly include commercial
   use, and do not exclude any field of endeavour. To the extent possible in the
   relevant jurisdiction, these rights may be exercised in all media and formats
   whether now known or created in the future.

   The rights granted cover, for example:

      (( a. ))??
      Extraction and Re-utilisation of the whole or a Substantial part of the
      Contents;

      (( b. ))??
      Creation of Derivative Databases;

      (( c. ))??
      Creation of Collective Databases;

      (( d. ))??
      Creation of temporary or permanent reproductions by any means and in any
      form, in whole or in part, including of any Derivative Databases or as a
      part of Collective Databases; and

      (( e. ))??
      Distribution, communication, display, lending, making available, or
      performance to the public by any means and in any form, in whole or in
      part, including of any Derivative Database or as a part of Collective
      Databases.

   (( 3.2 ))??
   Compulsory license schemes. For the avoidance of doubt:

      (( a. ))??
      Non-waivable compulsory license schemes. In those jurisdictions in which
      the right to collect royalties through any statutory or compulsory
      licensing scheme cannot be waived, the Licensor reserves the exclusive
      right to collect such royalties for any exercise by You of the rights
      granted under this License;

      (( b. ))??
      Waivable compulsory license schemes. In those jurisdictions in which the
      right to collect royalties through any statutory or compulsory licensing
      scheme can be waived, the Licensor waives the exclusive right to collect
      such royalties for any exercise by You of the rights granted under this
      License; and,

      (( c. ))??
      Voluntary license schemes. The Licensor waives the right to collect
      royalties, whether individually or, in the event that the Licensor is a
      member of a collecting society that administers voluntary licensing
      schemes, via that society, from any exercise by You of the rights granted
      under this License.

   (( 3.3 ))??
   The right to release the Database under different terms, or to stop
   distributing or making available the Database, is reserved. Note that this
   Database may be multiple-licensed, and so You may have the choice of using
   alternative licenses for this Database. Subject to Section 10.4, all other
   rights not expressly granted by Licensor are reserved.

   (( 4.0 ))??
   Conditions of Use

   (( 4.1 ))??
   The rights granted in Section 3 above are expressly made subject to Your
   complying with the following conditions of use. These are important
   conditions of this License, and if You fail to follow them, You will be in
   material breach of its terms.

   (( 4.2 ))??
   Notices. If You Publicly Convey this Database, any Derivative Database, or
   the Database as part of a Collective Database, then You must:

      (( a. ))??
      Do so only under the terms of this License;

      (( b. ))??
      Include a copy of this License or its Uniform Resource Identifier (URI)
      with the Database or Derivative Database, including both in the Database
      or Derivative Database and in any relevant documentation;

      (( c. ))??
      Keep intact any copyright or Database Right notices and notices that refer
      to this License; and

      (( d. ))??
      If it is not possible to put the required notices in a particular file due
      to its structure, then You must include the notices in a location (such as
      a relevant directory) where users would be likely to look for it.

   (( 4.3 ))??
   Notice for using output (Contents). Creating and Using a Produced Work does
   not require the notice in Section 4.2. However, if you Publicly Use a
   Produced Work, You must include a notice associated with the Produced Work
   reasonably calculated to make any Person that uses, views, accesses,
   interacts with, or is otherwise exposed to the Produced Work aware that
   Content was obtained from the Database, Derivative Database, or the Database
   as part of a Collective Database, and that it is available under this
   License.

      (( a. ))??
      Example notice. The following text will satisfy notice under Section 4.3:

      Contains information from DATABASE NAME which is made available under the
      ODC Attribution License.

      DATABASE NAME should be replaced with the name of the Database and a
      hyperlink to the location of the Database. "ODC Attribution License"
      should contain a hyperlink to the URI of the text of this License. If
      hyperlinks are not possible, You should include the plain text of the
      required URI's with the above notice.

   (( 4.4 ))??
   Licensing of others. You may not sublicense the Database. Each time You
   communicate the Database, the whole or Substantial part of the Contents, or
   any Derivative Database to anyone else in any way, the Licensor offers to the
   recipient a license to the Database on the same terms and conditions as this
   License. You are not responsible for enforcing compliance by third parties
   with this License, but You may enforce any rights that You have over a
   Derivative Database. You are solely responsible for any modifications of a
   Derivative Database made by You or another Person at Your direction. You may
   not impose any further restrictions on the exercise of the rights granted or
   affirmed under this License.

   (( 5.0 ))??
   Moral rights

   (( 5.1 ))??
   Moral rights. This section covers moral rights, including any rights to be
   identified as the author of the Database or to object to treatment that would
   otherwise prejudice the author's honour and reputation, or any other
   derogatory treatment:

      (( a. ))??
      For jurisdictions allowing waiver of moral rights, Licensor waives all
      moral rights that Licensor may have in the Database to the fullest extent
      possible by the law of the relevant jurisdiction under Section 10.4;

      (( b. ))??
      If waiver of moral rights under Section 5.1 a in the relevant jurisdiction
      is not possible, Licensor agrees not to assert any moral rights over the
      Database and waives all claims in moral rights to the fullest extent
      possible by the law of the relevant jurisdiction under Section 10.4; and

      (( c. ))??
      For jurisdictions not allowing waiver or an agreement not to assert moral
      rights under Section 5.1 a and b, the author may retain their moral rights
      over certain aspects of the Database.

      Please note that some jurisdictions do not allow for the waiver of moral
      rights, and so moral rights may still subsist over the Database in some
      jurisdictions.

   (( 6.0 ))??
   Fair dealing, Database exceptions, and other rights not affected

   (( 6.1 ))??
   This License does not affect any rights that You or anyone else may
   independently have under any applicable law to make any use of this Database,
   including without limitation:

      (( a. ))??
      Exceptions to the Database Right including: Extraction of Contents from
      non-electronic Databases for private purposes, Extraction for purposes of
      illustration for teaching or scientific research, and Extraction or
      Re-utilisation for public security or an administrative or judicial
      procedure.

      (( b. ))??
      Fair dealing, fair use, or any other legally recognised limitation or
      exception to infringement of copyright or other applicable laws.

   (( 6.2 ))??
   This License does not affect any rights of lawful users to Extract and
   Re-utilise insubstantial parts of the Contents, evaluated quantitatively or
   qualitatively, for any purposes whatsoever, including creating a Derivative
   Database (subject to other rights over the Contents, see Section 2.4). The
   repeated and systematic Extraction or Re-utilisation of insubstantial parts
   of the Contents may however amount to the Extraction or Re-utilisation of a
   Substantial part of the Contents.

   (( 7.0 ))??
   Warranties and Disclaimer

   (( 7.1 ))??
   The Database is licensed by the Licensor "as is" and without any warranty of
   any kind, either express, implied, or arising by statute, custom, course of
   dealing, or trade usage. Licensor specifically disclaims any and all implied
   warranties or conditions of title, non-infringement, accuracy or
   completeness, the presence or absence of errors, fitness for a particular
   purpose, merchantability, or otherwise. Some jurisdictions do not allow the
   exclusion of implied warranties, so this exclusion may not apply to You.

   (( 8.0 ))??
   Limitation of liability

   (( 8.1 ))??
   Subject to any liability that may not be excluded or limited by law, the
   Licensor is not liable for, and expressly excludes, all liability for loss or
   damage however and whenever caused to anyone by any use under this License,
   whether by You or by anyone else, and whether caused by any fault on the part
   of the Licensor or not. This exclusion of liability includes, but is not
   limited to, any special, incidental, consequential, punitive, or exemplary
   damages such as loss of revenue, data, anticipated profits, and lost
   business. This exclusion applies even if the Licensor has been advised of the
   possibility of such damages.

   (( 8.2 ))??
   If liability may not be excluded by law, it is limited to actual and direct
   financial loss to the extent it is caused by proved negligence on the part of
   the Licensor.

   (( 9.0 ))??
   Termination of Your rights under this License

   (( 9.1 ))??
   Any breach by You of the terms and conditions of this License automatically
   terminates this License with immediate effect and without notice to You. For
   the avoidance of doubt, Persons who have received the Database, the whole or
   a Substantial part of the Contents, Derivative Databases, or the Database as
   part of a Collective Database from You under this License will not have their
   licenses terminated provided their use is in full compliance with this
   License or a license granted under Section 4.8 of this License. Sections 1,
   2, 7, 8, 9 and 10 will survive any termination of this License.

   (( 9.2 ))??
   If You are not in breach of the terms of this License, the Licensor will not
   terminate Your rights under it.

   (( 9.3 ))??
   Unless terminated under Section 9.1, this License is granted to You for the
   duration of applicable rights in the Database.

   (( 9.4 ))??
   Reinstatement of rights. If you cease any breach of the terms and conditions
   of this License, then your full rights under this License will be
   reinstated:

      (( a. ))??
      Provisionally and subject to permanent termination until the 60th day
      after cessation of breach;

      (( b. ))??
      Permanently on the 60th day after cessation of breach unless otherwise
      reasonably notified by the Licensor; or

      (( c. ))??
      Permanently if reasonably notified by the Licensor of the violation, this
      is the first time You have received notice of violation of this License
      from the Licensor, and You cure the violation prior to 30 days after your
      receipt of the notice.

   (( 9.5 ))??
   Notwithstanding the above, Licensor reserves the right to release the
   Database under different license terms or to stop distributing or making
   available the Database. Releasing the Database under different license terms
   or stopping the distribution of the Database will not withdraw this License
   (or any other license that has been, or is required to be, granted under the
   terms of this License), and this License will continue in full force and
   effect unless terminated as stated above.

   (( 10.0 ))??
   General

   (( 10.1 ))??
   If any provision of this License is held to be invalid or unenforceable, that
   must not affect the validity or enforceability of the remainder of the terms
   and conditions of this License and each remaining provision of this License
   shall be valid and enforced to the fullest extent permitted by law.

   (( 10.2 ))??
   This License is the entire agreement between the parties with respect to the
   rights granted here over the Database. It replaces any earlier
   understandings, agreements or representations with respect to the Database.

   (( 10.3 ))??
   If You are in breach of the terms of this License, You will not be entitled
   to rely on the terms of this License or to complain of any breach by the
   Licensor.

   (( 10.4 ))??
   Choice of law. This License takes effect in and will be governed by the laws
   of the relevant jurisdiction in which the License terms are sought to be
   enforced. If the standard suite of rights granted under applicable copyright
   law and Database Rights in the relevant jurisdiction includes additional
   rights not granted under this License, these additional rights are granted in
   this License in order to meet the terms of this License.
`
const license_ODbL_1_0_lre = `//**
ODC Open Database License v1.0
https://spdx.org/licenses/ODbL-1.0.json
http://www.opendatacommons.org/licenses/odbl/1.0/
**//

(( ODC Open Database License (ODbL) ))??

(( Preamble

The Open Database License (ODbL) is a license agreement intended to allow users
to freely share, modify, and use this Database while maintaining this same
freedom for others. Many databases are covered by copyright, and therefore this
document licenses these rights. Some jurisdictions, mainly in the European
Union, have specific rights that cover databases, and so the ODbL addresses
these rights, too. Finally, the ODbL is also an agreement in contract for users
of this Database to act in certain ways in return for accessing this Database.

Databases can contain a wide variety of types of content (images, audiovisual
material, and sounds all in the same database, for example), and so the ODbL
only governs the rights over the Database, and not the contents of the Database
individually. Licensors should use the ODbL together with another license for
the contents, if the contents have a single set of rights that uniformly covers
all of the contents. If the contents have multiple sets of different rights,
Licensors should describe what rights govern what contents together in the
individual record or in some other way that clarifies what rights apply.

Sometimes the contents of a database, or the database itself, can be covered by
other rights not addressed here (such as private contracts, trade mark over the
name, or privacy rights / data protection rights over information in the
contents), and so you are advised that you may have to consult other documents
or clear other rights before doing activities not covered by this License.
))??

The Licensor (as defined below)

and

You (as defined below)

agree as follows:

   (( 1.0 ))??
   Definitions of Capitalised Words

   "Collective Database" - Means this Database in unmodified form as part of a
   collection of independent databases in themselves that together are assembled
   into a collective whole. A work that constitutes a Collective Database will
   not be considered a Derivative Database.

   "Convey" - As a verb, means Using the Database, a Derivative Database, or the
   Database as part of a Collective Database in any way that enables a Person to
   make or receive copies of the Database or a Derivative Database. Conveying
   does not include interaction with a user through a computer network, or
   creating and Using a Produced Work, where no transfer of a copy of the
   Database or a Derivative Database occurs.

   "Contents" - The contents of this Database, which includes the information,
   independent works, or other material collected into the Database. For
   example, the contents of the Database could be factual data or works such as
   images, audiovisual material, text, or sounds.

   "Database" - A collection of material (the Contents) arranged in a systematic
   or methodical way and individually accessible by electronic or other means
   offered under the terms of this License.

   "Database Directive" - Means Directive 96/9/EC of the European Parliament and
   of the Council of 11 March 1996 on the legal protection of databases, as
   amended or succeeded.

   "Database Right" - Means rights resulting from the Chapter III ("sui
   generis") rights in the Database Directive (as amended and as transposed by
   member states), which includes the Extraction and Re-utilisation of the whole
   or a Substantial part of the Contents, as well as any similar rights
   available in the relevant jurisdiction under Section 10.4.

   "Derivative Database" - Means a database based upon the Database, and
   includes any translation, adaptation, arrangement, modification, or any other
   alteration of the Database or of a Substantial part of the Contents. This
   includes, but is not limited to, Extracting or Re-utilising the whole or a
   Substantial part of the Contents in a new Database.

   "Extraction" - Means the permanent or temporary transfer of all or a
   Substantial part of the Contents to another medium by any means or in any
   form.

   "License" - Means this license agreement and is both a license of rights such
   as copyright and Database Rights and an agreement in contract.

   "Licensor" - Means the Person that offers the Database under the terms of
   this License.

   "Person" - Means a natural or legal person or a body of persons corporate or
   incorporate.

   "Produced Work" - a work (such as an image, audiovisual material, text, or
   sounds) resulting from using the whole or a Substantial part of the Contents
   (via a search or other query) from this Database, a Derivative Database, or
   this Database as part of a Collective Database.

   "Publicly" - means to Persons other than You or under Your control by either
   more than 50% ownership or by the power to direct their activities (such as
   contracting with an independent consultant).

   "Re-utilisation" - means any form of making available to the public all or a
   Substantial part of the Contents by the distribution of copies, by renting,
   by online or other forms of transmission.

   "Substantial" - Means substantial in terms of quantity or quality or a
   combination of both. The repeated and systematic Extraction or Re-utilisation
   of insubstantial parts of the Contents may amount to the Extraction or
   Re-utilisation of a Substantial part of the Contents.

   "Use" - As a verb, means doing any act that is restricted by copyright or
   Database Rights whether in the original medium or any other; and includes
   without limitation distributing, copying, publicly performing, publicly
   displaying, and preparing derivative works of the Database, as well as
   modifying the Database as may be technically necessary to use it in a
   different mode or format.

   "You" - Means a Person exercising rights under this License who has not
   previously violated the terms of this License with respect to the Database,
   or who has received express permission from the Licensor to exercise rights
   under this License despite a previous violation.

   Words in the singular include the plural and vice versa.

   (( 2.0 ))??
   What this License covers

   (( 2.1. ))??
   Legal effect of this document. This License is:

      (( a. ))??
      A license of applicable copyright and neighbouring rights;

      (( b. ))??
      A license of the Database Right; and

      (( c. ))??
      An agreement in contract between You and the Licensor.

   (( 2.2 ))??
   Legal rights covered. This License covers the legal rights in the Database,
   including:

      (( a. ))??
      Copyright. Any copyright or neighbouring rights in the Database. The
      copyright licensed includes any individual elements of the Database, but
      does not cover the copyright over the Contents independent of this
      Database. See Section 2.4 for details. Copyright law varies between
      jurisdictions, but is likely to cover: the Database model or schema, which
      is the structure, arrangement, and organisation of the Database, and can
      also include the Database tables and table indexes; the data entry and
      output sheets; and the Field names of Contents stored in the Database;

      (( b. ))??
      Database Rights. Database Rights only extend to the Extraction and
      Re-utilisation of the whole or a Substantial part of the Contents.
      Database Rights can apply even when there is no copyright over the
      Database. Database Rights can also apply when the Contents are removed
      from the Database and are selected and arranged in a way that would not
      infringe any applicable copyright; and

      (( c. ))??
      Contract. This is an agreement between You and the Licensor for access to
      the Database. In return you agree to certain conditions of use on this
      access as outlined in this License.

   (( 2.3 ))??
   Rights not covered.

      (( a. ))??
      This License does not apply to computer programs used in the making or
      operation of the Database;

      (( b. ))??
      This License does not cover any patents over the Contents or the Database;
      and

      (( c. ))??
      This License does not cover any trademarks associated with the Database.

   (( 2.4 ))??
   Relationship to Contents in the Database. The individual items of the
   Contents contained in this Database may be covered by other rights, including
   copyright, patent, data protection, privacy, or personality rights, and this
   License does not cover any rights (other than Database Rights or in contract)
   in individual Contents contained in the Database. For example, if used on a
   Database of images (the Contents), this License would not apply to copyright
   over individual images, which could have their own separate licenses, or one
   single license covering all of the rights over the images.

   (( 3.0 ))??
   Rights granted

   (( 3.1 ))??
   Subject to the terms and conditions of this License, the Licensor grants to
   You a worldwide, royalty-free, non-exclusive, terminable (but only under
   Section 9) license to Use the Database for the duration of any applicable
   copyright and Database Rights. These rights explicitly include commercial
   use, and do not exclude any field of endeavour. To the extent possible in the
   relevant jurisdiction, these rights may be exercised in all media and formats
   whether now known or created in the future.

   The rights granted cover, for example:

      (( a. ))??
      Extraction and Re-utilisation of the whole or a Substantial part of the
      Contents;

      (( b. ))??
      Creation of Derivative Databases;

      (( c. ))??
      Creation of Collective Databases;

      (( d. ))??
      Creation of temporary or permanent reproductions by any means and in any
      form, in whole or in part, including of any Derivative Databases or as a
      part of Collective Databases; and

      (( e. ))??
      Distribution, communication, display, lending, making available, or
      performance to the public by any means and in any form, in whole or in
      part, including of any Derivative Database or as a part of Collective
      Databases.

   (( 3.2 ))??
   Compulsory license schemes. For the avoidance of doubt:

      (( a. ))??
      Non-waivable compulsory license schemes. In those jurisdictions in which
      the right to collect royalties through any statutory or compulsory
      licensing scheme cannot be waived, the Licensor reserves the exclusive
      right to collect such royalties for any exercise by You of the rights
      granted under this License;

      (( b. ))??
      Waivable compulsory license schemes. In those jurisdictions in which the
      right to collect royalties through any statutory or compulsory licensing
      scheme can be waived, the Licensor waives the exclusive right to collect
      such royalties for any exercise by You of the rights granted under this
      License; and,

      (( c. ))??
      Voluntary license schemes. The Licensor waives the right to collect
      royalties, whether individually or, in the event that the Licensor is a
      member of a collecting society that administers voluntary licensing
      schemes, via that society, from any exercise by You of the rights granted
      under this License.

   (( 3.3 ))??
   The right to release the Database under different terms, or to stop
   distributing or making available the Database, is reserved. Note that this
   Database may be multiple-licensed, and so You may have the choice of using
   alternative licenses for this Database. Subject to Section 10.4, all other
   rights not expressly granted by Licensor are reserved.

   (( 4.0 ))??
   Conditions of Use

   (( 4.1 ))??
   The rights granted in Section 3 above are expressly made subject to Your
   complying with the following conditions of use. These are important
   conditions of this License, and if You fail to follow them, You will be in
   material breach of its terms.

   (( 4.2 ))??
   Notices. If You Publicly Convey this Database, any Derivative Database, or
   the Database as part of a Collective Database, then You must:

      (( a. ))??
      Do so only under the terms of this License or another license permitted
      under Section 4.4;

      (( b. ))??
      Include a copy of this License (or, as applicable, a license permitted
      under Section 4.4) or its Uniform Resource Identifier (URI) with the
      Database or Derivative Database, including both in the Database or
      Derivative Database and in any relevant documentation; and

      (( c. ))??
      Keep intact any copyright or Database Right notices and notices that refer
      to this License.

      (( d. ))??
      If it is not possible to put the required notices in a particular file due
      to its structure, then You must include the notices in a location (such as
      a relevant directory) where users would be likely to look for it.

   (( 4.3 ))??
   Notice for using output (Contents). Creating and Using a Produced Work does
   not require the notice in Section 4.2. However, if you Publicly Use a
   Produced Work, You must include a notice associated with the Produced Work
   reasonably calculated to make any Person that uses, views, accesses,
   interacts with, or is otherwise exposed to the Produced Work aware that
   Content was obtained from the Database, Derivative Database, or the Database
   as part of a Collective Database, and that it is available under this
   License.

      (( a. ))??
      Example notice. The following text will satisfy notice under Section 4.3:

      Contains information from DATABASE NAME, which is made available here
      under the Open Database License (ODbL).

      DATABASE NAME should be replaced with the name of the Database and a
      hyperlink to the URI of the Database. "Open Database License" should
      contain a hyperlink to the URI of the text of this License. If hyperlinks
      are not possible, You should include the plain text of the required URI's
      with the above notice.

   (( 4.4 ))??
   Share alike.

      (( a. ))??
      Any Derivative Database that You Publicly Use must be only under the terms
      of:

         (( i. ))??
         This License;

         (( ii. ))??
         A later version of this License similar in spirit to this License; or

         (( iii. ))??
         A compatible license.

         If You license the Derivative Database under one of the licenses
         mentioned in (iii), You must comply with the terms of that license.

      (( b. ))??
      For the avoidance of doubt, Extraction or Re-utilisation of the whole or a
      Substantial part of the Contents into a new database is a Derivative
      Database and must comply with Section 4.4.

      (( c. ))??
      Derivative Databases and Produced Works. A Derivative Database is Publicly
      Used and so must comply with Section 4.4. if a Produced Work created from
      the Derivative Database is Publicly Used.

      (( d. ))??
      Share Alike and additional Contents. For the avoidance of doubt, You must
      not add Contents to Derivative Databases under Section 4.4 a that are
      incompatible with the rights granted under this License.

      (( e. ))??
      Compatible licenses. Licensors may authorise a proxy to determine
      compatible licenses under Section 4.4 a iii. If they do so, the authorised
      proxy's public statement of acceptance of a compatible license grants You
      permission to use the compatible license.

   (( 4.5 ))??
   Limits of Share Alike. The requirements of Section 4.4 do not apply in the
   following:

      (( a. ))??
      For the avoidance of doubt, You are not required to license Collective
      Databases under this License if You incorporate this Database or a
      Derivative Database in the collection, but this License still applies to
      this Database or a Derivative Database as a part of the Collective
      Database;

      (( b. ))??
      Using this Database, a Derivative Database, or this Database as part of a
      Collective Database to create a Produced Work does not create a Derivative
      Database for purposes of Section 4.4; and

      (( c. ))??
      Use of a Derivative Database internally within an organisation is not to
      the public and therefore does not fall under the requirements of Section
      4.4.

   (( 4.6 ))??
   Access to Derivative Databases. If You Publicly Use a Derivative Database or
   a Produced Work from a Derivative Database, You must also offer to recipients
   of the Derivative Database or Produced Work a copy in a machine readable form
   of:

      (( a. ))??
      The entire Derivative Database; or

      (( b. ))??
      A file containing all of the alterations made to the Database or the
      method of making the alterations to the Database (such as an algorithm),
      including any additional Contents, that make up all the differences
      between the Database and the Derivative Database.

      The Derivative Database (under a.) or alteration file (under b.) must be
      available at no more than a reasonable production cost for physical
      distributions and free of charge if distributed over the internet.

   (( 4.7 ))??
   Technological measures and additional terms

      (( a. ))??
      This License does not allow You to impose (except subject to Section 4.7
      b.) any terms or any technological measures on the Database, a Derivative
      Database, or the whole or a Substantial part of the Contents that alter or
      restrict the terms of this License, or any rights granted under it, or
      have the effect or intent of restricting the ability of any person to
      exercise those rights.

      (( b. ))??
      Parallel distribution. You may impose terms or technological measures on
      the Database, a Derivative Database, or the whole or a Substantial part of
      the Contents (a "Restricted Database") in contravention of Section 4.74 a.
      only if You also make a copy of the Database or a Derivative Database
      available to the recipient of the Restricted Database:

         (( i. ))??
         That is available without additional fee;

         (( ii. ))??
         That is available in a medium that does not alter or restrict the terms
         of this License, or any rights granted under it, or have the effect or
         intent of restricting the ability of any person to exercise those
         rights (an "Unrestricted Database"); and

         (( iii. ))??
         The Unrestricted Database is at least as accessible to the recipient as
         a practical matter as the Restricted Database.

      (( c. ))??
      For the avoidance of doubt, You may place this Database or a Derivative
      Database in an authenticated environment, behind a password, or within a
      similar access control scheme provided that You do not alter or restrict
      the terms of this License or any rights granted under it or have the
      effect or intent of restricting the ability of any person to exercise
      those rights.

   (( 4.8 ))??
   Licensing of others. You may not sublicense the Database. Each time You
   communicate the Database, the whole or Substantial part of the Contents, or
   any Derivative Database to anyone else in any way, the Licensor offers to the
   recipient a license to the Database on the same terms and conditions as this
   License. You are not responsible for enforcing compliance by third parties
   with this License, but You may enforce any rights that You have over a
   Derivative Database. You are solely responsible for any modifications of a
   Derivative Database made by You or another Person at Your direction. You may
   not impose any further restrictions on the exercise of the rights granted or
   affirmed under this License.

   (( 5.0 ))??
   Moral rights

   (( 5.1 ))??
   Moral rights. This section covers moral rights, including any rights to be
   identified as the author of the Database or to object to treatment that would
   otherwise prejudice the author's honour and reputation, or any other
   derogatory treatment:

      (( a. ))??
      For jurisdictions allowing waiver of moral rights, Licensor waives all
      moral rights that Licensor may have in the Database to the fullest extent
      possible by the law of the relevant jurisdiction under Section 10.4;

      (( b. ))??
      If waiver of moral rights under Section 5.1 a in the relevant jurisdiction
      is not possible, Licensor agrees not to assert any moral rights over the
      Database and waives all claims in moral rights to the fullest extent
      possible by the law of the relevant jurisdiction under Section 10.4; and

      (( c. ))??
      For jurisdictions not allowing waiver or an agreement not to assert moral
      rights under Section 5.1 a and b, the author may retain their moral rights
      over certain aspects of the Database.

      Please note that some jurisdictions do not allow for the waiver of moral
      rights, and so moral rights may still subsist over the Database in some
      jurisdictions.

   (( 6.0 ))??
   Fair dealing, Database exceptions, and other rights not affected

   (( 6.1 ))??
   This License does not affect any rights that You or anyone else may
   independently have under any applicable law to make any use of this Database,
   including without limitation:

      (( a. ))??
      Exceptions to the Database Right including: Extraction of Contents from
      non-electronic Databases for private purposes, Extraction for purposes of
      illustration for teaching or scientific research, and Extraction or
      Re-utilisation for public security or an administrative or judicial
      procedure.

      (( b. ))??
      Fair dealing, fair use, or any other legally recognised limitation or
      exception to infringement of copyright or other applicable laws.

   (( 6.2 ))??
   This License does not affect any rights of lawful users to Extract and
   Re-utilise insubstantial parts of the Contents, evaluated quantitatively or
   qualitatively, for any purposes whatsoever, including creating a Derivative
   Database (subject to other rights over the Contents, see Section 2.4). The
   repeated and systematic Extraction or Re-utilisation of insubstantial parts
   of the Contents may however amount to the Extraction or Re-utilisation of a
   Substantial part of the Contents.

   (( 7.0 ))??
   Warranties and Disclaimer

   (( 7.1 ))??
   The Database is licensed by the Licensor "as is" and without any warranty of
   any kind, either express, implied, or arising by statute, custom, course of
   dealing, or trade usage. Licensor specifically disclaims any and all implied
   warranties or conditions of title, non-infringement, accuracy or
   completeness, the presence or absence of errors, fitness for a particular
   purpose, merchantability, or otherwise. Some jurisdictions do not allow the
   exclusion of implied warranties, so this exclusion may not apply to You.

   (( 8.0 ))??
   Limitation of liability

   (( 8.1 ))??
   Subject to any liability that may not be excluded or limited by law, the
   Licensor is not liable for, and expressly excludes, all liability for loss or
   damage however and whenever caused to anyone by any use under this License,
   whether by You or by anyone else, and whether caused by any fault on the part
   of the Licensor or not. This exclusion of liability includes, but is not
   limited to, any special, incidental, consequential, punitive, or exemplary
   damages such as loss of revenue, data, anticipated profits, and lost
   business. This exclusion applies even if the Licensor has been advised of the
   possibility of such damages.

   (( 8.2 ))??
   If liability may not be excluded by law, it is limited to actual and direct
   financial loss to the extent it is caused by proved negligence on the part of
   the Licensor.

   (( 9.0 ))??
   Termination of Your rights under this License

   (( 9.1 ))??
   Any breach by You of the terms and conditions of this License automatically
   terminates this License with immediate effect and without notice to You. For
   the avoidance of doubt, Persons who have received the Database, the whole or
   a Substantial part of the Contents, Derivative Databases, or the Database as
   part of a Collective Database from You under this License will not have their
   licenses terminated provided their use is in full compliance with this
   License or a license granted under Section 4.8 of this License. Sections 1,
   2, 7, 8, 9 and 10 will survive any termination of this License.

   (( 9.2 ))??
   If You are not in breach of the terms of this License, the Licensor will not
   terminate Your rights under it.

   (( 9.3 ))??
   Unless terminated under Section 9.1, this License is granted to You for the
   duration of applicable rights in the Database.

   (( 9.4 ))??
   Reinstatement of rights. If you cease any breach of the terms and conditions
   of this License, then your full rights under this License will be
   reinstated:

      (( a. ))??
      Provisionally and subject to permanent termination until the 60th day
      after cessation of breach;

      (( b. ))??
      Permanently on the 60th day after cessation of breach unless otherwise
      reasonably notified by the Licensor; or

      (( c. ))??
      Permanently if reasonably notified by the Licensor of the violation, this
      is the first time You have received notice of violation of this License
      from the Licensor, and You cure the violation prior to 30 days after your
      receipt of the notice.

      Persons subject to permanent termination of rights are not eligible to be
      a recipient and receive a license under Section 4.8.

   (( 9.5 ))??
   Notwithstanding the above, Licensor reserves the right to release the
   Database under different license terms or to stop distributing or making
   available the Database. Releasing the Database under different license terms
   or stopping the distribution of the Database will not withdraw this License
   (or any other license that has been, or is required to be, granted under the
   terms of this License), and this License will continue in full force and
   effect unless terminated as stated above.

   (( 10.0 ))??
   General

   (( 10.1 ))??
   If any provision of this License is held to be invalid or unenforceable, that
   must not affect the validity or enforceability of the remainder of the terms
   and conditions of this License and each remaining provision of this License
   shall be valid and enforced to the fullest extent permitted by law.

   (( 10.2 ))??
   This License is the entire agreement between the parties with respect to the
   rights granted here over the Database. It replaces any earlier
   understandings, agreements or representations with respect to the Database.

   (( 10.3 ))??
   If You are in breach of the terms of this License, You will not be entitled
   to rely on the terms of this License or to complain of any breach by the
   Licensor.

   (( 10.4 ))??
   Choice of law. This License takes effect in and will be governed by the laws
   of the relevant jurisdiction in which the License terms are sought to be
   enforced. If the standard suite of rights granted under applicable copyright
   law and Database Rights in the relevant jurisdiction includes additional
   rights not granted under this License, these additional rights are granted in
   this License in order to meet the terms of this License.
`
const license_OFL_1_0_lre = `//**
SIL Open Font License 1.0
https://spdx.org/licenses/OFL-1.0.json
http://scripts.sil.org/cms/scripts/page.php?item_id=OFL10_web
**//

(( SIL OPEN FONT LICENSE

Version 1.0 - 22 November 2005 ))??

PREAMBLE

The goals of the Open Font License (OFL) are to stimulate worldwide development
of cooperative font projects, to support the font creation efforts of academic
and linguistic communities, and to provide an open framework in which fonts may
be shared and improved in partnership with others.

The OFL allows the licensed fonts to be used, studied, modified and
redistributed freely as long as they are not sold by themselves. The fonts,
including any derivative works, can be bundled, embedded, redistributed and sold
with any software provided that the font names of derivative works are changed.
The fonts and derivatives, however, cannot be released under any other type of
license.

DEFINITIONS

"Font Software" refers to any and all of the following:

   -
   font files

   -
   data files

   -
   source code

   -
   build scripts

   -
   documentation

"Reserved Font Name" refers to the Font Software name as seen by users and any
other names as specified after the copyright statement.

"Standard Version" refers to the collection of Font Software components as
distributed by the Copyright Holder.

"Modified Version" refers to any derivative font software made by adding to,
deleting, or substituting — in part or in whole -- any of the components of
the Standard Version, by changing formats or by porting the Font Software to a
new environment.

"Author" refers to any designer, engineer, programmer, technical writer or other
person who contributed to the Font Software.

PERMISSION & CONDITIONS

Permission is hereby granted, free of charge, to any person obtaining a copy of
the Font Software, to use, study, copy, merge, embed, modify, redistribute, and
sell modified and unmodified copies of the Font Software, subject to the
following conditions:

   (( 1) ))??
   Neither the Font Software nor any of its individual components, in Standard
   or Modified Versions, may be sold by itself.

   (( 2) ))??
   Standard or Modified Versions of the Font Software may be bundled,
   redistributed and sold with any software, provided that each copy contains
   the above copyright notice and this license. These can be included either as
   stand-alone text files, human-readable headers or in the appropriate
   machine-readable metadata fields within text or binary files as long as those
   fields can be easily viewed by the user.

   (( 3) ))??
   No Modified Version of the Font Software may use the Reserved Font Name(s),
   in part or in whole, unless explicit written permission is granted by the
   Copyright Holder. This restriction applies to all references stored in the
   Font Software, such as the font menu name and other font description fields,
   which are used to differentiate the font from others.

   (( 4) ))??
   The name(s) of the Copyright Holder or the Author(s) of the Font Software
   shall not be used to promote, endorse or advertise any Modified Version,
   except to acknowledge the contribution(s) of the Copyright Holder and the
   Author(s) or with their explicit written permission.

   (( 5) ))??
   The Font Software, modified or unmodified, in part or in whole, must be
   distributed using this license, and may not be distributed under any other
   license.

TERMINATION

This license becomes null and void if any of the above conditions are not met.

DISCLAIMER

THE FONT SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF COPYRIGHT, PATENT, TRADEMARK, OR
OTHER RIGHT. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, INCLUDING ANY GENERAL, SPECIAL, INDIRECT,
INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF THE USE OR INABILITY TO USE THE FONT SOFTWARE OR
FROM OTHER DEALINGS IN THE FONT SOFTWARE.
`
const license_OFL_1_1_lre = `//**
SIL Open Font License 1.1
https://spdx.org/licenses/OFL-1.1.json
http://scripts.sil.org/cms/scripts/page.php?item_id=OFL_web
https://opensource.org/licenses/OFL-1.1
**//

//** Copyright **//

(( This Font Software is licensed under the SIL Open Font License, Version 1.1.

This license is copied below, and is also available with a FAQ at:
http:/scripts.sil.org/OFL ))??

(( SIL OPEN FONT LICENSE

Version 1.1 - 26 February 2007 ))??

PREAMBLE

The goals of the Open Font License (OFL) are to stimulate worldwide development
of collaborative font projects, to support the font creation efforts of academic
and linguistic communities, and to provide a free and open framework in which
fonts may be shared and improved in partnership with others.

The OFL allows the licensed fonts to be used, studied, modified and
redistributed freely as long as they are not sold by themselves. The fonts,
including any derivative works, can be bundled, embedded, redistributed and/or
sold with any software provided that any reserved names are not used by
derivative works. The fonts and derivatives, however, cannot be released under
any other type of license. The requirement for fonts to remain under this
license does not apply to any document created using the fonts or their
derivatives.

DEFINITIONS

"Font Software" refers to the set of files released by the Copyright Holder(s)
under this license and clearly marked as such. This may include source files,
build scripts and documentation.

"Reserved Font Name" refers to any names specified as such after the copyright
statement(s).

"Original Version" refers to the collection of Font Software components as
distributed by the Copyright Holder(s).

"Modified Version" refers to any derivative made by adding to, deleting, or
substituting — in part or in whole — any of the components of the Original
Version, by changing formats or by porting the Font Software to a new
environment.

"Author" refers to any designer, engineer, programmer, technical writer or other
person who contributed to the Font Software.

PERMISSION
((AND || &))
CONDITIONS

Permission is hereby granted, free of charge, to any person obtaining a copy of
the Font Software, to use, study, copy, merge, embed, modify, redistribute, and
sell modified and unmodified copies of the Font Software, subject to the
following conditions:

   (( 1) ))??
   Neither the Font Software nor any of its individual components, in Original
   or Modified Versions, may be sold by itself.

   (( 2) ))??
   Original or Modified Versions of the Font Software may be bundled,
   redistributed and/or sold with any software, provided that each copy contains
   the above copyright notice and this license. These can be included either as
   stand-alone text files, human-readable headers or in the appropriate
   machine-readable metadata fields within text or binary files as long as those
   fields can be easily viewed by the user.

   (( 3) ))??
   No Modified Version of the Font Software may use the Reserved Font Name(s)
   unless explicit written permission is granted by the corresponding Copyright
   Holder. This restriction only applies to the primary font name as presented
   to the users.

   (( 4) ))??
   The name(s) of the Copyright Holder(s) or the Author(s) of the Font Software
   shall not be used to promote, endorse or advertise any Modified Version,
   except to acknowledge the contribution(s) of the Copyright Holder(s) and the
   Author(s) or with their explicit written permission.

   (( 5) ))??
   The Font Software, modified or unmodified, in part or in whole, must be
   distributed entirely under this license, and must not be distributed under
   any other license. The requirement for fonts to remain under this license
   does not apply to any document created using the Font Software.

TERMINATION

This license becomes null and void if any of the above conditions are not met.

DISCLAIMER

THE FONT SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF COPYRIGHT, PATENT, TRADEMARK, OR
OTHER RIGHT. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, INCLUDING ANY GENERAL, SPECIAL, INDIRECT,
INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF THE USE OR INABILITY TO USE THE FONT SOFTWARE OR
FROM OTHER DEALINGS IN THE FONT SOFTWARE.
`
const license_OGC_1_0_lre = `//**
OGC Software License, Version 1.0
https://spdx.org/licenses/OGC-1.0.json
https://www.ogc.org/ogc/software/1.0
**//

(( OGC Software License, Version 1.0 ))??

This OGC work (including software, documents, or other related items) is being
provided by the copyright holders under the following license. By obtaining,
using and/or copying this work, you (the licensee) agree that you have read,
understood, and will comply with the following terms and conditions:

Permission to use, copy, and modify this software and its documentation, with or
without modification, for any purpose and without fee or royalty is hereby
granted, provided that you include the following on ALL copies of the software
and documentation or portions thereof, including modifications, that you make:

   (( 1. ))??
   The full text of this NOTICE in a location viewable to users of the
   redistributed or derivative work.

   (( 2. ))??
   Any pre-existing intellectual property disclaimers, notices, or terms and
   conditions. If none exist, a short notice of the following form (hypertext is
   preferred, text is permitted) should be used within the body of any
   redistributed or derivative code: "Copyright © [$date-of-document] Open
   Geospatial Consortium, Inc. All Rights Reserved. http:/www.
   //** ogc **//
   __5__
   .org/ogc/legal (Hypertext is preferred, but a textual representation is
   permitted.)

   (( 3. ))??
   Notice of any changes or modifications to the OGC files, including the date
   changes were made. (We recommend you provide URIs to the location from which
   the code is derived.)

THIS SOFTWARE AND DOCUMENTATION IS PROVIDED "AS IS," AND COPYRIGHT HOLDERS MAKE
NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO, WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT
THE USE OF THE SOFTWARE OR DOCUMENTATION WILL NOT INFRINGE ANY THIRD PARTY
PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS.

COPYRIGHT HOLDERS WILL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL OR
CONSEQUENTIAL DAMAGES ARISING OUT OF ANY USE OF THE SOFTWARE OR DOCUMENTATION.

The name and trademarks of copyright holders may NOT be used in advertising or
publicity pertaining to the software without specific, written prior permission.
Title to copyright in this software and any associated documentation will at all
times remain with copyright holders.
`
const license_OGL_Canada_2_0_lre = `//**
Open Government Licence - Canada
https://spdx.org/licenses/OGL-Canada-2.0.json
https://open.canada.ca/en/open-government-licence-canada
**//

Open Government Licence - Canada

You are encouraged to use the Information that is available under this licence
with only a few conditions.

Using Information under this licence

   *
   Use of any Information indicates your acceptance of the terms below.

   *
   The Information Provider grants you a worldwide, royalty-free, perpetual,
   non-exclusive licence to use the Information, including for commercial
   purposes, subject to the terms below.

You are free to:

   *
   Copy, modify, publish, translate, adapt, distribute or otherwise use the
   Information in any medium, mode or format for any lawful purpose.

You must, where you do any of the above:

   *
   Acknowledge the source of the Information by including any attribution
   statement specified by the Information Provider(s) and, where possible,
   provide a link to this licence.

   *
   If the Information Provider does not provide a specific attribution
   statement, or if you are using Information from several information providers
   and multiple attributions are not practical for your product or application,
   you must use the following attribution statement:

   Contains information licensed under the Open Government Licence – Canada.

The terms of this licence are important, and if you fail to comply with any of
them, the rights granted to you under this licence, or any similar licence
granted by the Information Provider, will end automatically.

Exemptions

This licence does not grant you any right to use:

   *
   Personal Information;

   *
   third party rights the Information Provider is not authorized to license;

   *
   the names, crests, logos, or other official symbols of the Information
   Provider; and

   *
   Information subject to other intellectual property rights, including patents,
   trade-marks and official marks.

Non-endorsement

This licence does not grant you any right to use the Information in a way that
suggests any official status or that the Information Provider endorses you or
your use of the Information.

No Warranty

The Information is licensed "as is", and the Information Provider excludes all
representations, warranties, obligations, and liabilities, whether express or
implied, to the maximum extent permitted by law.

The Information Provider is not liable for any errors or omissions in the
Information, and will not under any circumstances be liable for any direct,
indirect, special, incidental, consequential, or other loss, injury or damage
caused by its use or otherwise arising in connection with this licence or the
Information, even if specifically advised of the possibility of such loss,
injury or damage.

Governing Law

This licence is governed by the laws of the province of Ontario and the
applicable laws of Canada.

Legal proceedings related to this licence may only be brought in the courts of
Ontario or the Federal Court of Canada.

Definitions

In this licence, the terms below have the following meanings:

"Information"

means information resources protected by copyright or other information that is
offered for use under the terms of this licence.

"Information Provider"

means Her Majesty the Queen in right of Canada.

"Personal Information"

means "personal information" as defined in section 3 of the Privacy Act, R.S.C.
1985, c. P-21.

"You"

means the natural or legal person, or body of persons corporate or incorporate,
acquiring rights under this licence.

Versioning

This is version 2.0 of the Open Government Licence – Canada. The Information
Provider may make changes to the terms of this licence from time to time and
issue a new version of the licence. Your use of the Information will be governed
by the terms of the licence in force as of the date you accessed the
information.
`
const license_OGL_UK_1_0_lre = `//**
Open Government Licence v1.0
https://spdx.org/licenses/OGL-UK-1.0.json
http://www.nationalarchives.gov.uk/doc/open-government-licence/version/1/
**//

(( Open Government Licence v1.0 ))??

You are encouraged to use and re-use the Information that is available under
this licence, the Open Government Licence, freely and flexibly, with only a few
conditions.

Using Information under this licence

Use of copyright and database right material expressly made available under this
licence (the 'Information') indicates your acceptance of the terms and
conditions below.

The Licensor grants you a worldwide, royalty-free, perpetual, non-exclusive
licence to use the Information subject to the conditions below.

This licence does not affect your freedom under fair dealing or fair use or any
other copyright or database right exceptions and limitations.

You are free to:

   copy, publish, distribute and transmit the Information;

   adapt the Information;

   exploit the Information commercially for example, by combining it with other
   Information, or by including it in your own product or application.

You must, where you do any of the above:

   acknowledge the source of the Information by including any attribution
   statement specified by the Information Provider(s) and, where possible,
   provide a link to this licence;

   If the Information Provider does not provide a specific attribution
   statement, or if you are using Information from several Information Providers
   and multiple attributions are not practical in your product or application,
   you may consider using the following:

   Contains public sector information licensed under the Open Government Licence
   v1.0.

   ensure that you do not use the Information in a way that suggests any
   official status or that the Information Provider endorses you or your use of
   the Information;

   ensure that you do not mislead others or misrepresent the Information or its
   source;

   ensure that your use of the Information does not breach the Data Protection
   Act 1998 or the Privacy and Electronic Communications (EC Directive)
   Regulations 2003.

These are important conditions of this licence and if you fail to comply with
them the rights granted to you under this licence, or any similar licence
granted by the Licensor, will end automatically.

Exemptions

This licence does not cover the use of:

   -
   personal data in the Information;

   -
   Information that has neither been published nor disclosed under information
   access legislation (including the Freedom of Information Acts for the UK and
   Scotland) by or with the consent of the Information Provider;

   -
   departmental or public sector organisation logos, crests and the Royal Arms
   except where they form an integral part of a document or dataset;

   -
   military insignia;

   -
   third party rights the Information Provider is not authorised to license;

   -
   Information subject to other intellectual property rights, including patents,
   trademarks, and design rights; and

   -
   identity documents such as the British Passport.

No warranty

The Information is licensed 'as is' and the Information Provider excludes all
representations, warranties, obligations and liabilities in relation to the
Information to the maximum extent permitted by law.

The Information Provider is not liable for any errors or omissions in the
Information and shall not be liable for any loss, injury or damage of any kind
caused by its use. The Information Provider does not guarantee the continued
supply of the Information.

Governing Law

This licence is governed by the laws of the jurisdiction in which the
Information Provider has its principal place of business, unless otherwise
specified by the Information Provider.

Definitions

In this licence, the terms below have the following meanings:

   'Information' means information protected by copyright or by database right
   (for example, literary and artistic works, content, data and source code)
   offered for use under the terms of this licence.

   'Information Provider' means the person or organisation providing the
   Information under this licence.

   'Licensor' means any Information Provider which has the authority to offer
   Information under the terms of this licence or the Controller of Her
   Majesty's Stationery Office, who has the authority to offer Information
   subject to Crown copyright and Crown database rights and Information subject
   to copyright and database right that has been assigned to or acquired by the
   Crown, under the terms of this licence.

   'Use' as a verb, means doing any act which is restricted by copyright or
   database right, whether in the original medium or in any other medium, and
   includes without limitation distributing, copying, adapting, modifying as may
   be technically necessary to use it in a different mode or format.

   'You' means the natural or legal person, or body of persons corporate or
   incorporate, acquiring rights under this licence.

About the Open Government Licence

The Controller of Her Majesty's Stationery Office (HMSO) has developed this
licence as a tool to enable Information Providers in the public sector to
license the use and re-use of their Information under a common open licence. The
Controller invites public sector bodies owning their own copyright and database
rights to permit the use of their Information under this licence.

The Controller of HMSO has authority to license Information subject to copyright
and database right owned by the Crown. The extent of the Controller's offer to
license this Information under the terms of this licence is set out in the UK
Government Licensing Framework.

This is version 1.0 of the Open Government Licence. The Controller of HMSO may,
from time to time, issue new versions of the Open Government Licence. However,
you may continue to use Information licensed under this version should you wish
to do so.

These terms have been aligned to be interoperable with any Creative Commons
Attribution Licence, which covers copyright, and Open Data Commons Attribution
License, which covers database rights and applicable copyrights.

Further context, best practice and guidance can be found in the UK Government
Licensing Framework section on The National Archives website.
`
const license_OGL_UK_2_0_lre = `//**
Open Government Licence v2.0
https://spdx.org/licenses/OGL-UK-2.0.json
http://www.nationalarchives.gov.uk/doc/open-government-licence/version/2/
**//

(( Open Government Licence v2.0 ))??

You are encouraged to use and re-use the Information that is available under
this licence freely and flexibly, with only a few conditions.

Using Information under this licence

Use of copyright and database right material expressly made available under this
licence (the 'Information') indicates your acceptance of the terms and
conditions below.

The Licensor grants you a worldwide, royalty-free, perpetual, non-exclusive
licence to use the Information subject to the conditions below.

This licence does not affect your freedom under fair dealing or fair use or any
other copyright or database right exceptions and limitations.

You are free to:

   copy, publish, distribute and transmit the Information;

   adapt the Information;

   exploit the Information commercially and non-commercially for example, by
   combining it with other Information, or by including it in your own product
   or application.

You must, where you do any of the above:

   acknowledge the source of the Information by including any attribution
   statement specified by the Information Provider(s) and, where possible,
   provide a link to this licence;

   If the Information Provider does not provide a specific attribution
   statement, or if you are using Information from several Information Providers
   and multiple attributions are not practical in your product or application,
   you may use the following:

   Contains public sector information licensed under the Open Government Licence
   v2.0.

   These are important conditions of this licence and if you fail to comply with
   them the rights granted to you under this licence, or any similar licence
   granted by the Licensor, will end automatically.

Exemptions

This licence does not cover:

   personal data in the Information;

   information that has neither been published nor disclosed under information
   access legislation (including the Freedom of Information Acts for the UK and
   Scotland) by or with the consent of the Information Provider;

   departmental or public sector organisation logos, crests and the Royal Arms
   except where they form an integral part of a document or dataset;

   military insignia;

   third party rights the Information Provider is not authorised to license;

   other intellectual property rights, including patents, trade marks, and
   design rights; and

   identity documents such as the British Passport

Non-endorsement

This licence does not grant you any right to use the Information in a way that
suggests any official status or that the Information Provider endorses you or
your use of the Information.

Non warranty

The Information is licensed 'as is' and the Information Provider excludes all
representations, warranties, obligations and liabilities in relation to the
Information to the maximum extent permitted by law.

The Information Provider is not liable for any errors or omissions in the
Information and shall not be liable for any loss, injury or damage of any kind
caused by its use. The Information Provider does not guarantee the continued
supply of the Information.

Governing Law

This licence is governed by the laws of the jurisdiction in which the
Information Provider has its principal place of business, unless otherwise
specified by the Information Provider.

Definitions

In this licence, the terms below have the following meanings:

   'Information' means information protected by copyright or by database right
   (for example, literary and artistic works, content, data and source code)
   offered for use under the terms of this licence.

   'Information Provider' means the person or organisation providing the
   Information under this licence.

   'Licensor' means any Information Provider who has the authority to offer
   Information under the terms of this licence. It includes the Controller of
   Her Majesty's Stationery Office, who has the authority to offer Information
   subject to Crown copyright and Crown database rights, and Information subject
   to copyright and database rights which have been assigned to or acquired by
   the Crown, under the terms of this licence.

   'Use' means doing any act which is restricted by copyright or database right,
   whether in the original medium or in any other medium, and includes without
   limitation distributing, copying, adapting, modifying as may be technically
   necessary to use it in a different mode or format.

   'You' means the natural or legal person, or body of persons corporate or
   incorporate, acquiring rights under this licence.

About the Open Government Licence

The Controller of Her Majesty's Stationery Office (HMSO) has developed this
licence as a tool to enable Information Providers in the public sector to
license the use and re-use of their Information under a common open licence. The
Controller invites public sector bodies owning their own copyright and database
rights to permit the use of their Information under this licence.

The Controller of HMSO has authority to license Information subject to copyright
and database right owned by the Crown. The extent of the Controller's offer to
license this Information under the terms of this licence is set out on The
National Archives website.

This is version 2.0 of the Open Government Licence. The Controller of HMSO may,
from time to time, issue new versions of the Open Government Licence. If you are
already using Information under a previous version of the Open Government
Licence, the terms of that licence will continue to apply.

These terms are compatible with the Creative Commons Attribution License 4.0 and
the Open Data Commons Attribution License, both of which license copyright and
database rights. This means that when the Information is adapted and licensed
under either of those licences, you automatically satisfy the conditions of the
OGL when you comply with the other licence. The OGLv2.0 is Open Definition
compliant.

Further context, best practice and guidance can be found in the UK Government
Licensing Framework section on The National Archives website.
`
const license_OGL_UK_3_0_lre = `//**
Open Government Licence v3.0
https://spdx.org/licenses/OGL-UK-3.0.json
http://www.nationalarchives.gov.uk/doc/open-government-licence/version/3/
**//

(( Open Government Licence v3.0 ))??

You are encouraged to use and re-use the Information that is available under
this licence freely and flexibly, with only a few conditions.

Using Information under this licence

Use of copyright and database right material expressly made available under this
licence (the 'Information') indicates your acceptance of the terms and
conditions below.

The Licensor grants you a worldwide, royalty-free, perpetual, non-exclusive
licence to use the Information subject to the conditions below.

This licence does not affect your freedom under fair dealing or fair use or any
other copyright or database right exceptions and limitations.

You are free to:

   copy, publish, distribute and transmit the Information;

   adapt the Information;

   exploit the Information commercially and non-commercially for example, by
   combining it with other Information, or by including it in your own product
   or application.

You must (where you do any of the above):

   acknowledge the source of the Information in your product or application by
   including or linking to any attribution statement specified by the
   Information Provider(s) and, where possible, provide a link to this licence;

   If the Information Provider does not provide a specific attribution
   statement, you must use the following:

   Contains public sector information licensed under the Open Government Licence
   v3.0.

   If you are using Information from several Information Providers and listing
   multiple attributions is not practical in your product or application, you
   may include a URI or hyperlink to a resource that contains the required
   attribution statements.

   These are important conditions of this licence and if you fail to comply with
   them the rights granted to you under this licence, or any similar licence
   granted by the Licensor, will end automatically.

Exemptions

This licence does not cover:

   personal data in the Information;

   Information that has not been accessed by way of publication or disclosure
   under information access legislation (including the Freedom of Information
   Acts for the UK and Scotland) by or with the consent of the Information
   Provider;

   departmental or public sector organisation logos, crests and the Royal Arms
   except where they form an integral part of a document or dataset;

   military insignia;

   third party rights the Information Provider is not authorised to license;

   other intellectual property rights, including patents, trade marks, and
   design rights; and

   identity documents such as the British Passport

Non-endorsement

This licence does not grant you any right to use the Information in a way that
suggests any official status or that the Information Provider and/or Licensor
endorse you or your use of the Information.

No warranty

The Information is licensed 'as is' and the Information Provider and/or Licensor
excludes all representations, warranties, obligations and liabilities in
relation to the Information to the maximum extent permitted by law.

The Information Provider and/or Licensor are not liable for any errors or
omissions in the Information and shall not be liable for any loss, injury or
damage of any kind caused by its use. The Information Provider does not
guarantee the continued supply of the Information.

Governing Law

This licence is governed by the laws of the jurisdiction in which the
Information Provider has its principal place of business, unless otherwise
specified by the Information Provider.

Definitions

In this licence, the terms below have the following meanings:

   'Information' means information protected by copyright or by database right
   (for example, literary and artistic works, content, data and source code)
   offered for use under the terms of this licence.

   'Information Provider' means the person or organisation providing the
   Information under this licence.

   'Licensor' means any Information Provider which has the authority to offer
   Information under the terms of this licence or the Keeper of Public Records,
   who has the authority to offer Information subject to Crown copyright and
   Crown database rights and Information subject to copyright and database right
   that has been assigned to or acquired by the Crown, under the terms of this
   licence.

   'Use' means doing any act which is restricted by copyright or database right,
   whether in the original medium or in any other medium, and includes without
   limitation distributing, copying, adapting, modifying as may be technically
   necessary to use it in a different mode or format.

   'You', 'you' and 'your' means the natural or legal person, or body of persons
   corporate or incorporate, acquiring rights in the Information (whether the
   Information is obtained directly from the Licensor or otherwise) under this
   licence.

About the Open Government Licence

The National Archives has developed this licence as a tool to enable Information
Providers in the public sector to license the use and re-use of their
Information under a common open licence. The National Archives invites public
sector bodies owning their own copyright and database rights to permit the use
of their Information under this licence.

The Keeper of the Public Records has authority to license Information subject to
copyright and database right owned by the Crown. The extent of the offer to
license this Information under the terms of this licence is set out in the UK
Government Licensing Framework.

This is version 3.0 of the Open Government Licence. The National Archives may,
from time to time, issue new versions of the Open Government Licence. If you are
already using Information under a previous version of the Open Government
Licence, the terms of that licence will continue to apply.

These terms are compatible with the Creative Commons Attribution License 4.0 and
the Open Data Commons Attribution License, both of which license copyright and
database rights. This means that when the Information is adapted and licensed
under either of those licences, you automatically satisfy the conditions of the
OGL when you comply with the other licence. The OGLv3.0 is Open Definition
compliant.

Further context, best practice and guidance can be found in the UK Government
Licensing Framework section on The National Archives website.
`
const license_OGTSL_lre = `//**
Open Group Test Suite License
https://spdx.org/licenses/OGTSL.json
http://www.opengroup.org/testing/downloads/The_Open_Group_TSL.txt
https://opensource.org/licenses/OGTSL
**//

(( The Open Group Test Suite License ))??

(( Preamble

The intent of this document is to state the conditions under which a Package may
be copied, such that the Copyright Holder maintains some semblance of artistic
control over the development of the package, while giving the users of the
package the right to use and distribute the Package in a more-or-less customary
fashion, plus the right to make reasonable modifications.

Testing is essential for proper development and maintenance of standards-based
products.

For buyers: adequate conformance testing leads to reduced integration costs and
protection of investments in applications, software and people.

For software developers: conformance testing of platforms and middleware greatly
reduces the cost of developing and maintaining multi-platform application
software.

For suppliers: In-depth testing increases customer satisfaction and keeps
development and support costs in check. API conformance is highly measurable and
suppliers who claim it must be able to substantiate that claim.

As such, since these are benchmark measures of conformance, we feel the
integrity of test tools is of importance. In order to preserve the integrity of
the existing conformance modes of this test package and to permit recipients of
modified versions of this package to run the original test modes, this license
requires that the original test modes be preserved.

If you find a bug in one of the standards mode test cases, please let us know so
we can feed this back into the original, and also raise any specification issues
with the appropriate bodies (for example the POSIX committees). ))??

Definitions:

"Package" refers to the collection of files distributed by the Copyright Holder,
and derivatives of that collection of files created through textual
modification.

"Standard Version" refers to such a Package if it has not been modified, or has
been modified in accordance with the wishes of the Copyright Holder.

"Copyright Holder" is whoever is named in the copyright or copyrights for the
package.

"You" is you, if you're thinking about copying or distributing this Package.

"Reasonable copying fee" is whatever you can justify on the basis of media cost,
duplication charges, time of people involved, and so on. (You will not be
required to justify it to the Copyright Holder, but only to the computing
community at large as a market that must bear the fee.)

"Freely Available" means that no fee is charged for the item itself, though
there may be fees involved in handling the item. It also means that recipients
of the item may redistribute it under the same conditions they received it.

   (( 1. ))??
   You may make and give away verbatim copies of the source form of the Standard
   Version of this Package without restriction, provided that you duplicate all
   of the original copyright notices and associated disclaimers.

   (( 2. ))??
   You may apply bug fixes, portability fixes and other modifications derived
   from the Public Domain or from the Copyright Holder. A Package modified in
   such a way shall still be considered the Standard Version.

   (( 3. ))??
   You may otherwise modify your copy of this Package in any way, provided that
   you insert a prominent notice in each changed file stating how and when you
   changed that file, and provided that you do at least the following:

   rename any non-standard executables and testcases so the names do not
   conflict with standard executables and testcases, which must also be
   provided, and provide a separate manual page for each non-standard executable
   and testcase that clearly documents how it differs from the Standard
   Version.

   (( 4. ))??
   You may distribute the programs of this Package in object code or executable
   form, provided that you do at least the following:

   accompany any non-standard executables and testcases with their corresponding
   Standard Version executables and testcases, giving the non-standard
   executables and testcases non-standard names, and clearly documenting the
   differences in manual pages (or equivalent), together with instructions on
   where to get the Standard Version.

   (( 5. ))??
   You may charge a reasonable copying fee for any distribution of this Package.
   You may charge any fee you choose for support of this Package. You may not
   charge a fee for this Package itself. However, you may distribute this
   Package in aggregate with other (possibly commercial) programs as part of a
   larger (possibly commercial) software distribution provided that you do not
   advertise this Package as a product of your own.

   (( 6. ))??
   The scripts and library files supplied as input to or produced as output from
   the programs of this Package do not automatically fall under the copyright of
   this Package, but belong to whomever generated them, and may be sold
   commercially, and may be aggregated with this Package.

   7.Subroutines supplied by you and linked into this Package shall not be
   considered part of this Package.

   (( 8. ))??
   The name of the Copyright Holder may not be used to endorse or promote
   products derived from this software without specific prior written
   permission.

   (( 9. ))??
   THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
   (( The End ))??
`
const license_OLDAP_1_1_lre = `//**
Open LDAP Public License v1.1
https://spdx.org/licenses/OLDAP-1.1.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=806557a5ad59804ef3a44d5abfbe91d706b0791f
**//

(( The OpenLDAP Public License
Version 1.1, 25 August 1998

(( Copyright __20__ ))??
))??

(( Note: This license is derived from the "Artistic License" as distributed with
the Perl Programming Language. Its terms are different from those of the
"Artistic License."
))??

((
PREAMBLE

The intent of this document is to state the conditions under which a Package may
be copied, such that the Copyright Holder maintains some semblance of artistic
control over the development of the package, while giving the users of the
package the right to use and distribute the Package in a more-or-less customary
fashion, plus the right to make reasonable modifications. ))??

Definitions:

"Package" refers to the collection of files distributed by the Copyright Holder,
and derivatives of that collection of files created through textual
modification.

"Standard Version" refers to such a Package if it has not been modified, or has
been modified in accordance with the wishes of the Copyright Holder.

"Copyright Holder" is whoever is named in the copyright or copyrights for the
package.

"You" is you, if you're thinking about copying or distributing this Package.

"Reasonable copying fee" is whatever you can justify on the basis of media cost,
duplication charges, time of people involved, and so on. (You will not be
required to justify it to the Copyright Holder, but only to the computing
community at large as a market that must bear the fee.)

"Freely Available" means that no fee is charged for the item itself, though
there may be fees involved in handling the item. It also means that recipients
of the item may redistribute it under the same conditions they received it.

   (( 1. ))??
   You may make and give away verbatim copies of the source form of the Standard
   Version of this Package without restriction, provided that you duplicate all
   of the original copyright notices and associated disclaimers.

   (( 2. ))??
   You may apply bug fixes, portability fixes and other modifications derived
   from the Public Domain or from the Copyright Holder. A Package modified in
   such a way shall still be considered the Standard Version.

   (( 3. ))??
   You may otherwise modify your copy of this Package in any way, provided that
   you insert a prominent notice in each changed file stating how and when you
   changed that file, and provided that you do at least ONE of the following:

      (( a) ))??
      place your modifications in the Public Domain or otherwise make them
      Freely Available, such as by posting said modifications to Usenet or an
      equivalent medium, or placing the modifications on a major archive site
      such as uunet.uu.net, or by allowing the Copyright Holder to include your
      modifications in the Standard Version of the Package.

      (( b) ))??
      use the modified Package only within your corporation or organization.

      (( c) ))??
      rename any non-standard executables so the names do not conflict with
      standard executables, which must also be provided, and provide a separate
      manual page for each non-standard executable that clearly documents how it
      differs from the Standard Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 4. ))??
   You may distribute the programs of this Package in object code or executable
   form, provided that you do at least ONE of the following:

      (( a) ))??
      distribute a Standard Version of the executables and library files,
      together with instructions (in the manual page or equivalent) on where to
      get the Standard Version.

      (( b) ))??
      accompany the distribution with the machine-readable source of the Package
      with your modifications.

      (( c) ))??
      accompany any non-standard executables with their corresponding Standard
      Version executables, giving the non-standard executables non-standard
      names, and clearly documenting the differences in manual pages (or
      equivalent), together with instructions on where to get the Standard
      Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 5. ))??
   You may charge a reasonable copying fee for any distribution of this Package.
   You may charge any fee you choose for support of this Package. You may not
   charge a fee for this Package itself. However, you may distribute this
   Package in aggregate with other (possibly commercial) programs as part of a
   larger (possibly commercial) software distribution provided that you do not
   advertise this Package as a product of your own.

   (( 6. ))??
   The scripts and library files supplied as input to or produced as output from
   the programs of this Package do not automatically fall under the copyright of
   this Package, but belong to whomever generated them, and may be sold
   commercially, and may be aggregated with this Package.

   (( 7. ))??
   C subroutines supplied by you and linked into this Package in order to
   emulate subroutines and variables of the language defined by this Package
   shall not be considered part of this Package, but are the equivalent of input
   as in Paragraph 6, provided these subroutines do not change the language in
   any way that would cause it to fail the regression tests for the language.

   (( 8. ))??
   The name of the Copyright Holder may not be used to endorse or promote
   products derived from this software without specific prior written
   permission.

   (( 9. ))??
   THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
   (( The End ))??
`
const license_OLDAP_1_2_lre = `//**
Open LDAP Public License v1.2
https://spdx.org/licenses/OLDAP-1.2.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=42b0383c50c299977b5893ee695cf4e486fb0dc7
**//

(( The OpenLDAP Public License
Version 1.2, 1 September 1998
(( Copyright __20__ ))??
))??

(( Note: This license is derived from the "Artistic License" as distributed with
the Perl Programming Language. As differences may exist, the complete license
should be read.))??

(( PREAMBLE

The intent of this document is to state the conditions under which a Package may
be copied, such that the Copyright Holder maintains some semblance of artistic
control over the development of the package, while giving the users of the
package the right to use and distribute the Package in a more-or-less customary
fashion, plus the right to make reasonable modifications. ))??

Definitions:

"Package" refers to the collection of files distributed by the Copyright Holder,
and derivatives of that collection of files created through textual
modification.

"Standard Version" refers to such a Package if it has not been modified, or has
been modified in accordance with the wishes of the Copyright Holder.

"Copyright Holder" is whoever is named in the copyright or copyrights for the
package.

"You" is you, if you're thinking about copying or distributing this Package.

"Reasonable copying fee" is whatever you can justify on the basis of media cost,
duplication charges, time of people involved, and so on. (You will not be
required to justify it to the Copyright Holder, but only to the computing
community at large as a market that must bear the fee.)

"Freely Available" means that no fee is charged for the item itself, though
there may be fees involved in handling the item. It also means that recipients
of the item may redistribute it under the same conditions they received it.

   (( 1. ))??
   You may make and give away verbatim copies of the source form of the Standard
   Version of this Package without restriction, provided that you duplicate all
   of the original copyright notices and associated disclaimers.

   (( 2. ))??
   You may apply bug fixes, portability fixes and other modifications derived
   from the Public Domain or from the Copyright Holder. A Package modified in
   such a way shall still be considered the Standard Version.

   (( 3. ))??
   You may otherwise modify your copy of this Package in any way, provided that
   you insert a prominent notice in each changed file stating how and when you
   changed that file, and provided that you do at least ONE of the following:

      (( a) ))??
      place your modifications in the Public Domain or otherwise make them
      Freely Available, such as by posting said modifications to Usenet or an
      equivalent medium, or placing the modifications on a major archive site
      such as uunet.uu.net, or by allowing the Copyright Holder to include your
      modifications in the Standard Version of the Package.

      (( b) ))??
      use the modified Package only within your corporation or organization.

      (( c) ))??
      rename any non-standard executables so the names do not conflict with
      standard executables, which must also be provided, and provide a separate
      manual page for each non-standard executable that clearly documents how it
      differs from the Standard Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 4. ))??
   You may distribute the programs of this Package in object code or executable
   form, provided that you do at least ONE of the following:

      (( a) ))??
      distribute a Standard Version of the executables and library files,
      together with instructions (in the manual page or equivalent) on where to
      get the Standard Version.

      (( b) ))??
      accompany the distribution with the machine-readable source of the Package
      with your modifications.

      (( c) ))??
      accompany any non-standard executables with their corresponding Standard
      Version executables, giving the non-standard executables non-standard
      names, and clearly documenting the differences in manual pages (or
      equivalent), together with instructions on where to get the Standard
      Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 5. ))??
   You may charge a reasonable copying fee for any distribution of this Package.
   You may charge any fee you choose for support of this Package. You may not
   charge a fee for this Package itself. However, you may distribute this
   Package in aggregate with other (possibly commercial) programs as part of a
   larger (possibly commercial) software distribution provided that you do not
   advertise this Package as a product of your own.

   (( 6. ))??
   The scripts and library files supplied as input to or produced as output from
   the programs of this Package do not automatically fall under the copyright of
   this Package, but belong to whomever generated them, and may be sold
   commercially, and may be aggregated with this Package.

   (( 7. ))??
   C subroutines supplied by you and linked into this Package in order to
   emulate subroutines and variables of the language defined by this Package
   shall not be considered part of this Package, but are the equivalent of input
   as in Paragraph 6, provided these subroutines do not change the language in
   any way that would cause it to fail the regression tests for the language.

   (( 8. ))??
   The name of the Copyright Holder may not be used to endorse or promote
   products derived from this software without specific prior written
   permission.

   (( 9. ))??
   THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
   (( The End ))??
`
const license_OLDAP_1_3_lre = `//**
Open LDAP Public License v1.3
https://spdx.org/licenses/OLDAP-1.3.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=e5f8117f0ce088d0bd7a8e18ddf37eaa40eb09b1
**//

(( The OpenLDAP Public License
Version 1.3, 17 January 1999
(( Copyright __20__ ))??
))??

(( Note: This license is derived from the "Artistic License" as distributed with
the Perl Programming Language. As significant differences exist, the complete
license should be read. ))??

(( PREAMBLE

The intent of this document is to state the conditions under which a Package may
be copied, such that the Copyright Holder maintains some semblance of artistic
control over the development of the package, while giving the users of the
package the right to use and distribute the Package in a more-or-less customary
fashion, plus the right to make reasonable modifications. ))??

Definitions:

"Package" refers to the collection of files distributed by the Copyright Holder,
and derivatives of that collection of files created through textual
modification.

"Standard Version" refers to such a Package if it has not been modified, or has
been modified in accordance with the wishes of the Copyright Holder.

"Copyright Holder" is whoever is named in the copyright or copyrights for the
package.

"You" is you, if you're thinking about copying or distributing this Package.

"Reasonable copying fee" is whatever you can justify on the basis of media cost,
duplication charges, time of people involved, and so on. (You will not be
required to justify it to the Copyright Holder, but only to the computing
community at large as a market that must bear the fee.)

"Freely Available" means that no fee is charged for the item itself, though
there may be fees involved in handling the item. It also means that recipients
of the item may redistribute it under the same conditions they received it.

   (( 1. ))??
   You may make and give away verbatim copies of the source form of the Standard
   Version of this Package without restriction, provided that you duplicate all
   of the original copyright notices and associated disclaimers.

   (( 2. ))??
   You may apply bug fixes, portability fixes and other modifications derived
   from the Public Domain or from the Copyright Holder. A Package modified in
   such a way shall still be considered the Standard Version.

   (( 3. ))??
   You may otherwise modify your copy of this Package in any way, provided that
   you insert a prominent notice in each changed file stating how and when you
   changed that file, and provided that you do at least ONE of the following:

      (( a) ))??
      place your modifications in the Public Domain or otherwise make them
      Freely Available, such as by posting said modifications to Usenet or an
      equivalent medium, or placing the modifications on a major archive site
      such as uunet.uu.net, or by allowing the Copyright Holder to include your
      modifications in the Standard Version of the Package.

      (( b) ))??
      use the modified Package only within your corporation or organization.

      (( c) ))??
      rename any non-standard executables so the names do not conflict with
      standard executables, which must also be provided, and provide a separate
      manual page for each non-standard executable that clearly documents how it
      differs from the Standard Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 4. ))??
   You may distribute the programs of this Package in object code or executable
   form, provided that you do at least ONE of the following:

      (( a) ))??
      distribute a Standard Version of the executables and library files,
      together with instructions (in the manual page or equivalent) on where to
      get the Standard Version.

      (( b) ))??
      accompany the distribution with the machine-readable source of the Package
      with your modifications.

      (( c) ))??
      accompany any non-standard executables with their corresponding Standard
      Version executables, giving the non-standard executables non-standard
      names, and clearly documenting the differences in manual pages (or
      equivalent), together with instructions on where to get the Standard
      Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 5. ))??
   You may charge a reasonable copying fee for any distribution of this Package.
   You may charge any fee you choose for support of this Package. You may not
   charge a fee for this Package itself. However, you may distribute this
   Package in aggregate with other (possibly commercial) programs as part of a
   larger (possibly commercial) software distribution provided that you do not
   advertise this Package as a product of your own.

   (( 6. ))??
   The scripts and library files supplied as input to or produced as output from
   the programs of this Package do not automatically fall under the copyright of
   this Package, but belong to whomever generated them, and may be sold
   commercially, and may be aggregated with this Package.

   (( 7. ))??
   C subroutines supplied by you and linked into this Package in order to
   emulate subroutines and variables defined by this Package shall not be
   considered part of this Package, but are the equivalent of input as in
   Paragraph 6, provided these subroutines do not change the behavior of the
   Package in any way that would cause it to fail the regression tests for the
   Package.

   (( 8. ))??
   Software supplied by you and linked with this Package in order to use
   subroutines and variables defined by this Package shall not be considered
   part of this Package and do not automatically fall under the copyright of
   this Package, and the executables produced by linking your software with this
   Package may be used and redistributed without restriction and may be sold
   commercially.

   (( 9. ))??
   The name of the Copyright Holder may not be used to endorse or promote
   products derived from this software without specific prior written
   permission.

   (( 10. ))??
   THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
   (( The End ))??
`
const license_OLDAP_1_4_lre = `//**
Open LDAP Public License v1.4
https://spdx.org/licenses/OLDAP-1.4.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=c9f95c2f3f2ffb5e0ae55fe7388af75547660941
**//

(( The OpenLDAP Public License
Version 1.4, 18 January 1999
(( Copyright __20__ ))??
))??

(( Note: This license is derived from the "Artistic License" as distributed with
the Perl Programming Language. As significant differences exist, the complete
license should be read. ))??

(( PREAMBLE

The intent of this document is to state the conditions under which a Package may
be copied, such that the Copyright Holder maintains some semblance of artistic
control over the development of the package, while giving the users of the
package the right to use and distribute the Package in a more-or-less customary
fashion, plus the right to make reasonable modifications. ))??

Definitions:

"Package" refers to the collection of files distributed by the Copyright Holder,
and derivatives of that collection of files created through textual
modification.

"Standard Version" refers to such a Package if it has not been modified, or has
been modified in accordance with the wishes of the Copyright Holder.

"Copyright Holder" is whoever is named in the copyright or copyrights for the
package.

"You" is you, if you're thinking about copying or distributing this Package.

"Reasonable copying fee" is whatever you can justify on the basis of media cost,
duplication charges, time of people involved, and so on. (You will not be
required to justify it to the Copyright Holder, but only to the computing
community at large as a market that must bear the fee.)

"Freely Available" means that no fee is charged for the item itself, though
there may be fees involved in handling the item. It also means that recipients
of the item may redistribute it under the same conditions they received it.

   (( 1. ))??
   You may make and give away verbatim copies of the source form of the Standard
   Version of this Package without restriction, provided that you duplicate all
   of the original copyright notices and associated disclaimers.

   (( 2. ))??
   You may apply bug fixes, portability fixes and other modifications derived
   from the Public Domain or from the Copyright Holder. A Package modified in
   such a way shall still be considered the Standard Version.

   (( 3. ))??
   You may otherwise modify your copy of this Package in any way, provided that
   you insert a prominent notice in each changed file stating how and when you
   changed that file, and provided that you do at least ONE of the following:

      (( a) ))??
      place your modifications in the Public Domain or otherwise make them
      Freely Available, such as by posting said modifications to Usenet or an
      equivalent medium, or placing the modifications on a major archive site
      such as uunet.uu.net, or by allowing the Copyright Holder to include your
      modifications in the Standard Version of the Package.

      (( b) ))??
      use the modified Package only within your corporation or organization.

      (( c) ))??
      rename any non-standard executables so the names do not conflict with
      standard executables, which must also be provided, and provide a separate
      manual page for each non-standard executable that clearly documents how it
      differs from the Standard Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 4. ))??
   You may distribute the programs of this Package in object code or executable
   form, provided that you do at least ONE of the following:

      (( a) ))??
      distribute a Standard Version of the executables and library files,
      together with instructions (in the manual page or equivalent) on where to
      get the Standard Version.

      (( b) ))??
      accompany the distribution with the machine-readable source of the Package
      with your modifications.

      (( c) ))??
      accompany any non-standard executables with their corresponding Standard
      Version executables, giving the non-standard executables non-standard
      names, and clearly documenting the differences in manual pages (or
      equivalent), together with instructions on where to get the Standard
      Version.

      (( d) ))??
      make other distribution arrangements with the Copyright Holder.

   (( 5. ))??
   You may charge a reasonable copying fee for any distribution of this Package.
   You may charge any fee you choose for support of this Package. You may not
   charge a fee for this Package itself. However, you may distribute this
   Package in aggregate with other (possibly commercial) programs as part of a
   larger (possibly commercial) software distribution provided that you do not
   advertise this Package as a product of your own.

   (( 6. ))??
   The scripts and library files supplied as input to or produced as output from
   the programs of this Package do not automatically fall under the copyright of
   this Package, but belong to whomever generated them, and may be sold
   commercially, and may be aggregated with this Package.

   (( 7. ))??
   C subroutines supplied by you and linked into this Package in order to
   emulate subroutines and variables defined by this Package shall not be
   considered part of this Package, but are the equivalent of input as in
   Paragraph 6, provided these subroutines do not change the behavior of the
   Package in any way that would cause it to fail the regression tests for the
   Package.

   (( 8. ))??
   Software supplied by you and linked with this Package in order to use
   subroutines and variables defined by this Package shall not be considered
   part of this Package and do not automatically fall under the copyright of
   this Package. Executables produced by linking your software with this Package
   may be used and redistributed without restriction and may be sold
   commercially so long as the primary function of your software is different
   than the package itself.

   (( 9. ))??
   The name of the Copyright Holder may not be used to endorse or promote
   products derived from this software without specific prior written
   permission.

   (( 10. ))??
   THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
   (( The End ))??
`
const license_OLDAP_2_0_lre = `//**
Open LDAP Public License v2.0 (or possibly 2.0A and 2.0B)
https://spdx.org/licenses/OLDAP-2.0.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=cbf50f4e1185a21abd4c0a54d3f4341fe28f36ea
**//

(( The OpenLDAP Public License
Version 2.0, 7 June 1999
(( Copyright __20__ ))??
))??

Redistribution and use of this software and associated documentation
("Software"), with or without modification, are permitted provided that the
following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain copyright statements and notices.
   Redistributions must also contain a copy of this document.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The name "OpenLDAP" must not be used to endorse or promote products derived
   from this Software without prior written permission of the OpenLDAP
   Foundation. For written permission, please contact foundation@openldap.org.

   (( 4. ))??
   Products derived from this Software may not be called "OpenLDAP" nor may
   "OpenLDAP" appear in their names without prior written permission of the
   OpenLDAP Foundation. OpenLDAP is a registered trademark of the OpenLDAP
   Foundation.

   (( 5. ))??
   Due credit should be given to the OpenLDAP Project
   (http:/www.openldap.org/).

THIS SOFTWARE IS PROVIDED BY THE OPENLDAP FOUNDATION AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE OPENLDAP FOUNDATION OR ITS CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
`
const license_OLDAP_2_0_1_lre = `//**
Open LDAP Public License v2.0.1
https://spdx.org/licenses/OLDAP-2.0.1.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=b6d68acd14e51ca3aab4428bf26522aa74873f0e
**//

(( The OpenLDAP Public License
Version 2.0.1, 21 December 1999
(( Copyright __20__ ))??
))??

Redistribution and use of this software and associated documentation
("Software"), with or without modification, are permitted provided that the
following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain copyright statements and notices.
   Redistributions must also contain a copy of this document.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The name "OpenLDAP" must not be used to endorse or promote products derived
   from this Software without prior written permission of the OpenLDAP
   Foundation. For written permission, please contact foundation@openldap.org.

   (( 4. ))??
   Products derived from this Software may not be called "OpenLDAP" nor may
   "OpenLDAP" appear in their names without prior written permission of the
   OpenLDAP Foundation. OpenLDAP is a trademark of the OpenLDAP Foundation.

   (( 5. ))??
   Due credit should be given to the OpenLDAP Project
   (http:/www.openldap.org/).

THIS SOFTWARE IS PROVIDED BY THE OPENLDAP FOUNDATION AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE OPENLDAP FOUNDATION OR ITS CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
`
const license_OLDAP_2_1_lre = `//**
Open LDAP Public License v2.1
https://spdx.org/licenses/OLDAP-2.1.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=b0d176738e96a0d3b9f85cb51e140a86f21be715
**//

(( The OpenLDAP Public License
Version 2.1, 29 February 2000
(( Copyright __20__ ))??
))??

Redistribution and use of this software and associated documentation
("Software"), with or without modification, are permitted provided that the
following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain copyright statements and notices.
   Redistributions must also contain a copy of this document.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The name "OpenLDAP" must not be used to endorse or promote products derived
   from this Software without prior written permission of the OpenLDAP
   Foundation. For written permission, please contact foundation@openldap.org.

   (( 4. ))??
   Products derived from this Software may not be called "OpenLDAP" nor may
   "OpenLDAP" appear in their names without prior written permission of the
   OpenLDAP Foundation. OpenLDAP is a trademark of the OpenLDAP Foundation.

   (( 5. ))??
   Due credit should be given to the OpenLDAP Project
   (http:/www.openldap.org/).

   (( 6. ))??
   The OpenLDAP Foundation may revise this license from time to time. Each
   revision is distinguished by a version number. You may use the Software under
   terms of this license revision or under the terms of any subsequent license
   revision.

THIS SOFTWARE IS PROVIDED BY THE OPENLDAP FOUNDATION AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE OPENLDAP FOUNDATION OR ITS CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
`
const license_OLDAP_2_2_lre = `//**
Open LDAP Public License v2.2
https://spdx.org/licenses/OLDAP-2.2.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=470b0c18ec67621c85881b2733057fecf4a1acc3
**//

(( The OpenLDAP Public License

Version 2.2, 1 March 2000 ))??

Redistribution and use of this software and associated documentation
("Software"), with or without modification, are permitted provided that the
following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain copyright statements and notices.
   Redistributions must also contain a copy of this document.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The name "OpenLDAP" must not be used to endorse or promote products derived
   from this Software without prior written permission of the OpenLDAP
   Foundation.

   (( 4. ))??
   Products derived from this Software may not be called "OpenLDAP" nor may
   "OpenLDAP" appear in their names without prior written permission of the
   OpenLDAP Foundation.

   (( 5. ))??
   Due credit should be given to the OpenLDAP Project
   (http:/www.openldap.org/).

   (( 6. ))??
   The OpenLDAP Foundation may revise this license from time to time. Each
   revision is distinguished by a version number. You may use the Software under
   terms of this license revision or under the terms of any subsequent the
   license.

THIS SOFTWARE IS PROVIDED BY THE OPENLDAP FOUNDATION AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE OPENLDAP FOUNDATION OR ITS CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

OpenLDAP is a trademark of the OpenLDAP Foundation.

Copyright __20__.

Permission to copy and distributed verbatim copies of this document is granted.
`
const license_OLDAP_2_2_1_lre = `//**
Open LDAP Public License v2.2.1
https://spdx.org/licenses/OLDAP-2.2.1.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=4bc786f34b50aa301be6f5600f58a980070f481e
**//

(( The OpenLDAP Public License

Version 2.2.1, 1 March 2000 ))??

Redistribution and use of this software and associated documentation
("Software"), with or without modification, are permitted provided that the
following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain copyright statements and notices.
   Redistributions must also contain a copy of this document.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The name "OpenLDAP" must not be used to endorse or promote products derived
   from this Software without prior written permission of the OpenLDAP
   Foundation.

   (( 4. ))??
   Products derived from this Software may not be called "OpenLDAP" nor may
   "OpenLDAP" appear in their names without prior written permission of the
   OpenLDAP Foundation.

   (( 5. ))??
   Due credit should be given to the OpenLDAP Project
   (http:/www.openldap.org/).

   (( 6. ))??
   The OpenLDAP Foundation may revise this license from time to time. Each
   revision is distinguished by a version number. You may use the Software under
   terms of this license revision or under the terms of any subsequent revision
   of the license.

THIS SOFTWARE IS PROVIDED BY THE OPENLDAP FOUNDATION AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE OPENLDAP FOUNDATION OR ITS CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

OpenLDAP is a trademark of the OpenLDAP Foundation.

Copyright __20__.

Permission to copy and distributed verbatim copies of this document is granted.
`
const license_OLDAP_2_2_2_lre = `//**
Open LDAP Public License 2.2.2
https://spdx.org/licenses/OLDAP-2.2.2.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=df2cc1e21eb7c160695f5b7cffd6296c151ba188
**//

(( The OpenLDAP Public License

Version 2.2.2, 28 July 2000 ))??

Redistribution and use of this software and associated documentation
("Software"), with or without modification, are permitted provided that the
following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain copyright statements and notices.

   (( 2. ))??
   Redistributions in binary form must reproduce applicable copyright statements
   and notices, this list of conditions, and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

   (( 3. ))??
   Redistributions must contain a verbatim copy of this document.

   (( 4. ))??
   The name "OpenLDAP" must not be used to endorse or promote products derived
   from this Software without prior written permission of the OpenLDAP
   Foundation.

   (( 5. ))??
   Products derived from this Software may not be called "OpenLDAP" nor may
   "OpenLDAP" appear in their names without prior written permission of the
   OpenLDAP Foundation.

   (( 6. ))??
   Due credit should be given to the OpenLDAP Project
   (http:/www.openldap.org/).

   (( 7. ))??
   The OpenLDAP Foundation may revise this license from time to time. Each
   revision is distinguished by a version number. You may use the Software under
   terms of this license revision or under the terms of any subsequent revision
   of the license.

THIS SOFTWARE IS PROVIDED BY THE OPENLDAP FOUNDATION AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE OPENLDAP FOUNDATION OR ITS CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

OpenLDAP is a trademark of the OpenLDAP Foundation.

Copyright __20__.

Permission to copy and distributed verbatim copies of this document is granted.
`
const license_OLDAP_2_3_lre = `//**
Open LDAP Public License v2.3
https://spdx.org/licenses/OLDAP-2.3.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=d32cf54a32d581ab475d23c810b0a7fbaf8d63c3
**//

(( The OpenLDAP Public License

Version 2.3, 28 July 2000 ))??

Redistribution and use of this software and associated documentation
("Software"), with or without modification, are permitted provided that the
following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain copyright statements and notices.

   (( 2. ))??
   Redistributions in binary form must reproduce applicable copyright statements
   and notices, this list of conditions, and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

   (( 3. ))??
   Redistributions must contain a verbatim copy of this document.

   (( 4. ))??
   The name "OpenLDAP" must not be used to endorse or promote products derived
   from this Software without prior written permission of the OpenLDAP
   Foundation.

   (( 5. ))??
   Products derived from this Software may not be called "OpenLDAP" nor may
   "OpenLDAP" appear in their names without prior written permission of the
   OpenLDAP Foundation.

   (( 6. ))??
   Due credit should be given to the OpenLDAP Project
   (http:/www.openldap.org/).

   (( 7. ))??
   The OpenLDAP Foundation may revise this license from time to time. Each
   revision is distinguished by a version number. You may use the Software under
   terms of this license revision or under the terms of any subsequent revision
   of the license.

THIS SOFTWARE IS PROVIDED BY THE OPENLDAP FOUNDATION AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE OPENLDAP FOUNDATION OR ITS CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

OpenLDAP is a trademark of the OpenLDAP Foundation.

Copyright __20__.

Permission to copy and distributed verbatim copies of this document is granted.
`
const license_OLDAP_2_4_lre = `//**
Open LDAP Public License v2.4
https://spdx.org/licenses/OLDAP-2.4.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=cd1284c4a91a8a380d904eee68d1583f989ed386
**//

(( The OpenLDAP Public License

Version 2.4, 8 December 2000 ))??

Redistribution and use of this software and associated documentation
("Software"), with or without modification, are permitted provided that the
following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain copyright statements and notices.

   (( 2. ))??
   Redistributions in binary form must reproduce applicable copyright statements
   and notices, this list of conditions, and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

   (( 3. ))??
   Redistributions must contain a verbatim copy of this document.

   (( 4. ))??
   The names and trademarks of the authors and copyright holders must not be
   used in advertising or otherwise to promote the sale, use or other dealing in
   this Software without specific, written prior permission.

   (( 5. ))??
   Due credit should be given to the OpenLDAP Project.

   (( 6. ))??
   The OpenLDAP Foundation may revise this license from time to time. Each
   revision is distinguished by a version number. You may use the Software under
   terms of this license revision or under the terms of any subsequent revision
   of the license.

THIS SOFTWARE IS PROVIDED BY THE OPENLDAP FOUNDATION AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE OPENLDAP FOUNDATION OR ITS CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

OpenLDAP is a trademark of the OpenLDAP Foundation.

Copyright __20__.

Permission to copy and distributed verbatim copies of this document is granted.
`
const license_OLDAP_2_5_lre = `//**
Open LDAP Public License v2.5
https://spdx.org/licenses/OLDAP-2.5.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=6852b9d90022e8593c98205413380536b1b5a7cf
**//

(( The OpenLDAP Public License

Version 2.5, 11 May 2001 ))??

Redistribution and use of this software and associated documentation
("Software"), with or without modification, are permitted provided that the
following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain copyright statements and notices.

   (( 2. ))??
   Redistributions in binary form must reproduce applicable copyright statements
   and notices, this list of conditions, and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

   (( 3. ))??
   Redistributions must contain a verbatim copy of this document.

   (( 4. ))??
   The names and trademarks of the authors and copyright holders must not be
   used in advertising or otherwise to promote the sale, use or other dealing in
   this Software without specific, written prior permission.

   (( 5. ))??
   Due credit should be given to the authors of the Software.

   (( 6. ))??
   The OpenLDAP Foundation may revise this license from time to time. Each
   revision is distinguished by a version number. You may use the Software under
   terms of this license revision or under the terms of any subsequent revision
   of the license.

THIS SOFTWARE IS PROVIDED BY THE OPENLDAP FOUNDATION AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE OPENLDAP FOUNDATION, ITS CONTRIBUTORS, OR THE
AUTHOR(S) OR OWNER(S) OF THE SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

OpenLDAP is a trademark of the OpenLDAP Foundation.

Copyright __20__.

Permission to copy and distributed verbatim copies of this document is granted.
`
const license_OLDAP_2_6_lre = `//**
Open LDAP Public License v2.6
https://spdx.org/licenses/OLDAP-2.6.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=1cae062821881f41b73012ba816434897abf4205
**//

(( The OpenLDAP Public License

Version 2.6, 14 June 2001 ))??

Redistribution and use of this software and associated documentation
("Software"), with or without modification, are permitted provided that the
following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain copyright statements and notices.

   (( 2. ))??
   Redistributions in binary form must reproduce applicable copyright statements
   and notices, this list of conditions, and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

   (( 3. ))??
   Redistributions must contain a verbatim copy of this document.

   (( 4. ))??
   The names and trademarks of the authors and copyright holders must not be
   used in advertising or otherwise to promote the sale, use or other dealing in
   this Software without specific, written prior permission.

   (( 5. ))??
   The OpenLDAP Foundation may revise this license from time to time. Each
   revision is distinguished by a version number. You may use the Software under
   terms of this license revision or under the terms of any subsequent revision
   of the license.

THIS SOFTWARE IS PROVIDED BY THE OPENLDAP FOUNDATION AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS''
AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE OPENLDAP FOUNDATION, ITS CONTRIBUTORS, OR THE
AUTHOR(S) OR OWNER(S) OF THE SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

OpenLDAP is a trademark of the OpenLDAP Foundation.

Copyright __20__.

Permission to copy and distributed verbatim copies of this document is granted.
`
const license_OLDAP_2_7_lre = `//**
Open LDAP Public License v2.7
https://spdx.org/licenses/OLDAP-2.7.json
http://www.openldap.org/devel/gitweb.cgi?p=openldap.git;a=blob;f=LICENSE;hb=47c2415c1df81556eeb39be6cad458ef87c534a2
**//

(( The OpenLDAP Public License

Version 2.7, 7 September 2001 ))??

Redistribution and use of this software and associated documentation
("Software"), with or without modification, are permitted provided that the
following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain copyright statements and notices,

   (( 2. ))??
   Redistributions in binary form must reproduce applicable copyright statements
   and notices, this list of conditions, and the following disclaimer in the
   documentation and/or other materials provided with the distribution, and

   (( 3. ))??
   Redistributions must contain a verbatim copy of this document.

The OpenLDAP Foundation may revise this license from time to time. Each revision
is distinguished by a version number. You may use this Software under terms of
this license revision or under the terms of any subsequent revision of the
license.

THIS SOFTWARE IS PROVIDED BY THE OPENLDAP FOUNDATION AND ITS CONTRIBUTORS ` + "`" + `` + "`" + `AS
IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE OPENLDAP FOUNDATION, ITS CONTRIBUTORS, OR THE
AUTHOR(S) OR OWNER(S) OF THE SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The names of the authors and copyright holders must not be used in advertising
or otherwise to promote the sale, use or other dealing in this Software without
specific, written prior permission. Title to copyright in this Software shall at
all times remain with copyright holders.

OpenLDAP is a registered trademark of the OpenLDAP Foundation.

Copyright __20__.

Permission to copy and distribute verbatim copies of this document is granted.
`
const license_OLDAP_2_8_lre = `//**
Open LDAP Public License v2.8
https://spdx.org/licenses/OLDAP-2.8.json
http://www.openldap.org/software/release/license.html
**//

(( The OpenLDAP Public License

Version 2.8, 17 August 2003 ))??

Redistribution and use of this software and associated documentation
("Software"), with or without modification, are permitted provided that the
following conditions are met:

   (( 1. ))??
   Redistributions in source form must retain copyright statements and notices,

   (( 2. ))??
   Redistributions in binary form must reproduce applicable copyright statements
   and notices, this list of conditions, and the following disclaimer in the
   documentation and/or other materials provided with the distribution, and

   (( 3. ))??
   Redistributions must contain a verbatim copy of this document.

The OpenLDAP Foundation may revise this license from time to time. Each revision
is distinguished by a version number. You may use this Software under terms of
this license revision or under the terms of any subsequent revision of the
license.

THIS SOFTWARE IS PROVIDED BY THE OPENLDAP FOUNDATION AND ITS CONTRIBUTORS ` + "`" + `` + "`" + `AS
IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE OPENLDAP FOUNDATION, ITS CONTRIBUTORS, OR THE
AUTHOR(S) OR OWNER(S) OF THE SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The names of the authors and copyright holders must not be used in advertising
or otherwise to promote the sale, use or other dealing in this Software without
specific, written prior permission. Title to copyright in this Software shall at
all times remain with copyright holders.

OpenLDAP is a registered trademark of the OpenLDAP Foundation.

Copyright __20__.

Permission to copy and distribute verbatim copies of this document is granted.
`
const license_OML_lre = `//**
Open Market License
https://spdx.org/licenses/OML.json
https://fedoraproject.org/wiki/Licensing/Open_Market_License
**//

This FastCGI application library source and object code (the "Software") and its
documentation (the "Documentation") are copyrighted by Open Market, Inc ("Open
Market"). The following terms apply to all files associated with the Software
and Documentation unless explicitly disclaimed in individual files.

Open Market permits you to use, copy, modify, distribute, and license this
Software and the Documentation for any purpose, provided that existing copyright
notices are retained in all copies and that this notice is included verbatim in
any distributions. No written agreement, license, or royalty fee is required for
any of the authorized uses. Modifications to this Software and Documentation may
be copyrighted by their authors and need not follow the licensing terms
described here. If modifications to this Software and Documentation have new
licensing terms, the new terms must be clearly indicated on the first page of
each file where they apply.

OPEN MARKET MAKES NO EXPRESS OR IMPLIED WARRANTY WITH RESPECT TO THE SOFTWARE OR
THE DOCUMENTATION, INCLUDING WITHOUT LIMITATION ANY WARRANTY OF MERCHANTABILITY
OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL OPEN MARKET BE LIABLE TO
YOU OR ANY THIRD PARTY FOR ANY DAMAGES ARISING FROM OR RELATING TO THIS SOFTWARE
OR THE DOCUMENTATION, INCLUDING, WITHOUT LIMITATION, ANY INDIRECT, SPECIAL OR
CONSEQUENTIAL DAMAGES OR SIMILAR DAMAGES, INCLUDING LOST PROFITS OR LOST DATA,
EVEN IF OPEN MARKET HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. THE
SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS". OPEN MARKET HAS NO LIABILITY IN
CONTRACT, TORT, NEGLIGENCE OR OTHERWISE ARISING OUT OF THIS SOFTWARE OR THE
DOCUMENTATION.
`
const license_OPL_1_0_lre = `//**
Open Public License v1.0
https://spdx.org/licenses/OPL-1.0.json
http://old.koalateam.com/jackaroo/OPL_1_0.TXT
https://fedoraproject.org/wiki/Licensing/Open_Public_License
**//

(( OPEN PUBLIC LICENSE

Version 1.0 ))??

   (( 1. ))??
   Definitions.

      (( 1.1. ))??
      "Contributor" means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      "Covered Code" means the Original Code or Modifications or the combination
      of the Original Code and Modifications, in each case including portions
      thereof.

      (( 1.4. ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      "Executable" means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      "Initial Developer" means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7. ))??
      "Larger Work" means a work, which combines Covered Code or portions
      thereof with code not governed by the terms of this License.

      (( 1.8. ))??
      "License" means this document and the corresponding addendum described in
      section 6.4 below.

      (( 1.9. ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10. ))??
      "Original Code" means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      (( 1.11. ))??
      "Source Code" means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or a list of source code differential
      comparisons against either the Original Code or another well known,
      available Covered Code of the Contributor's choice. The Source Code can be
      in a compressed or archival form, provided the appropriate decompression
      or de-archiving software is widely available for no charge.

      (( 1.12. ))??
      "You" means an individual or a legal entity exercising rights under, and
      complying with all of the terms of, this License or a future version of
      this License issued under Section 6.1. For legal entities, "You'' includes
      any entity which controls, is controlled by, or is under common control
      with You. For purposes of this definition, "control'' means (a) the power,
      direct or indirect, to cause the direction or management of such entity,
      whether by contract or otherwise, or (b) ownership of fifty percent (50%)
      or more of the outstanding shares or beneficial ownership of such entity.

      (( 1.13 ))??
      "License Author" means Lutris Technologies, Inc.

   (( 2. ))??
   Source Code License.

      (( 2.1. ))??
      The Initial Developer Grant. The Initial Developer hereby grants You a
      worldwide, royalty-free, non-exclusive license, subject to third party
      intellectual property claims:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark) to
         use, reproduce, modify, display, perform, sublicense and distribute the
         Original Code (or portions thereof) with or without Modifications, or
         as part of a Larger Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Initial
         Developer, to make, have made, use and sell (` + "`" + `` + "`" + `offer to sell and
         import'') the Original Code (or portions thereof), but solely to the
         extent that any such patent is reasonably necessary to enable You to
         Utilize the Original Code (or portions thereof) and not to any greater
         extent that may be necessary to Utilize further Modifications or
         combinations.

      (( 2.2. ))??
      Contributor Grant. Each Contributor hereby grants You a worldwide,
      royalty-free, non-exclusive license, subject to third party intellectual
      property claims:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark) to
         use, reproduce, modify, display, perform, sublicense and distribute the
         Modifications created by such Contributor (or portions thereof) either
         on an unmodified basis, with other Modifications, as Covered Code or as
         part of a Larger Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Contributor, to
         to make, have made, use and sell (` + "`" + `` + "`" + `offer to sell and import'') the
         Contributor Version (or portions thereof), but solely to the extent
         that any such patent is reasonably necessary to enable You to Utilize
         the Contributor Version (or portions thereof), and not to any greater
         extent that may be necessary to Utilize further Modifications or
         combinations

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Application of License. The Modifications which You create or to which You
      contribute are governed by the terms of this License, including without
      limitation Section 2.2. The Source Code version of Covered Code may be
      distributed only under the terms of this License or a future version of
      this License released under Section 6.1, and You must include a copy of
      this License with every copy of the Source Code You distribute. You may
      not offer or impose any terms on any Source Code version that alters or
      restricts the applicable version of this License or the recipients' rights
      hereunder. However, You may include an additional document offering the
      additional rights described in Section 3.5.

      (( 3.2. ))??
      Availability of Source Code. Any Modification which You create or to which
      You contribute must be made available, prior to any use, except for
      internal development and practice, in Source Code form under the terms of
      this License either on the same media as an Executable version or via an
      accepted Electronic Distribution Mechanism to anyone to whom you made an
      Executable version available; and if made available via Electronic
      Distribution Mechanism, must remain available for at least twelve (12)
      months after the date it initially became available, or at least six (6)
      months after a subsequent version of that particular Modification has been
      made available to such recipients. You shall notify the Initial Developer
      of the Modification and the location of the Source Code via the contact
      means provided for in the Developer Specific license. Initial Developer
      will be acting as maintainer of the Source Code and may provide an
      Electronic Distribution mechanism for the Modification to be made
      available.

      (( 3.3. ))??
      Description of Modifications. You must cause all Covered Code to which you
      contribute to contain a file documenting the changes You made to create
      that Covered Code and the date of any change. You must include a prominent
      statement that the Modification is derived, directly or indirectly, from
      Original Code provided by the Initial Developer and including the name of
      the Initial Developer in (a) the Source Code, and (b) in any notice in an
      Executable version or related documentation in which You describe the
      origin or ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters

         (( (a) ))??
         Third Party Claims. If You have knowledge that a party claims an
         intellectual property right in particular functionality or code (or its
         utilization under this License), you must include a text file with the
         source code distribution titled "LEGAL" which describes the claim and
         the party making the claim in sufficient detail that a recipient will
         know whom to contact. If you obtain such knowledge after You make Your
         Modification available as described in Section 3.2, You shall promptly
         modify the LEGAL file in all copies You make available thereafter and
         shall take other steps (such as notifying appropriate mailing lists or
         newsgroups) reasonably calculated to inform those who received the
         Covered Code that new knowledge has been obtained.

         (( (b) ))??
         Representations. Contributor represents that, except as disclosed
         pursuant to Section 3.4(a) above, Contributor believes that
         Contributor's Modifications are Contributor's original creation(s)
         and/or Contributor has sufficient rights to grant the rights conveyed
         by this License.

      (( 3.5. ))??
      Required Notices. You must duplicate the notice in Exhibit A in each file
      of the Source Code, and this License in any documentation for the Source
      Code, where You describe recipients' rights relating to Covered Code. If
      You created one or more Modification(s), You may add your name as a
      Contributor to the notice described in Exhibit A. If it is not possible to
      put such notice in a particular Source Code file due to its structure,
      then you must include such notice in a location (such as a relevant
      directory file) where a user would be likely to look for such a notice.
      You may choose to offer, and to charge a fee for, warranty, support,
      indemnity or liability obligations to one or more recipients of Covered
      Code. However, You may do so only on Your own behalf, and not on behalf of
      the Initial Developer or any Contributor. You must make it absolutely
      clear that any such warranty, support, indemnity or liability obligation
      is offered by You alone, and You hereby agree to indemnify the Initial
      Developer and every Contributor for any liability incurred by the Initial
      Developer or such Contributor as a result of warranty, support, indemnity
      or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Versions. You may distribute Covered Code in
      Executable form only if the requirements of Section 3.1-3.5 have been met
      for that Covered Code, and if You include a notice stating that the Source
      Code version of the Covered Code is available under the terms of this
      License, including a description of how and where You have fulfilled the
      obligations of Section 3.2. The notice must be conspicuously included in
      any notice in an Executable version, related documentation or collateral
      in which You describe recipients' rights relating to the Covered Code. You
      may distribute the Executable version of Covered Code under a license of
      Your choice, which may contain terms different from this License, provided
      that You are in compliance with the terms of this License and that the
      license for the Executable version does not attempt to limit or alter the
      recipient's rights in the Source Code version from the rights set forth in
      this License. If You distribute the Executable version under a different
      license You must make it absolutely clear that any terms which differ from
      this License are offered by You alone, not by the Initial Developer or any
      Contributor. You hereby agree to indemnify the Initial Developer and every
      Contributor for any liability incurred by the Initial Developer or such
      Contributor as a result of any such terms You offer. If you distribute
      executable versions containing Covered Code, you must reproduce the notice
      in Exhibit B in the documentation and/or other materials provided with the
      product.

      (( 3.7. ))??
      Larger Works. You may create a Larger Work by combining Covered Code with
      other code not governed by the terms of this License and distribute the
      Larger Work as a single product. In such a case, You must make sure the
      requirements of this License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute or regulation
   then You must: (a) comply with the terms of this License to the maximum
   extent possible; and (b) Cite all of the statutes or regulations that
   prohibit you from complying fully with this license. (c) describe the
   limitations and the code they affect. Such description must be included in
   the LEGAL file described in Section 3.4 and must be included with all
   distributions of the Source Code. Except to the extent prohibited by statute
   or regulation, such description must be sufficiently detailed for a recipient
   of ordinary skill to be able to understand it.

   (( 5. ))??
   Application of this License.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A, and to related Covered Code.

   (( 6. ))??
   Versions of the License.

      (( 6.1. ))??
      New Versions. License Author may publish revised and/or new versions of
      the License from time to time. Each version will be given a distinguishing
      version number and shall be submitted to opensource.org for
      certification.

      (( 6.2. ))??
      Effect of New Versions. Once Covered Code has been published under a
      particular version of the License, You may always continue to use it under
      the terms of that version. You may also choose to use such Covered Code
      under the terms of any subsequent version of the License published by
      Initial Developer. No one other than Initial Developer has the right to
      modify the terms applicable to Covered Code created under this License.

      (( 6.3. ))??
      Derivative Works. If you create or use a modified version of this License,
      except in association with the required Developer Specific License
      described in section 6.4, (which you may only do in order to apply it to
      code which is not already Covered Code governed by this License), you must
      (a) rename Your license so that the phrases ` + "`" + `` + "`" + `Open'', ` + "`" + `` + "`" + `OpenPL'', ` + "`" + `` + "`" + `OPL''
      or any confusingly similar phrase do not appear anywhere in your license
      and (b) otherwise make it clear that your version of the license contains
      terms which differ from the Open Public License. (Filling in the name of
      the Initial Developer, Original Code or Contributor in the notice
      described in Exhibit A shall not of themselves be deemed to be
      modifications of this License.)

      (( 6.4. ))??
      Required Additional Developer Specific License

      This license is a union of the following two parts that should be found as
      text files in the same place (directory), in the order of preeminence:

         (( [1] ))??
         A Developer specific license.

         (( [2] ))??
         The contents of this file OPL_1_0.TXT, stating the general licensing
         policy of the software.

      In case of conflicting dispositions in the parts of this license, the
      terms of the lower-numbered part will always be superseded by the terms of
      the higher numbered part.

   (( 7. ))??
   DISCLAIMER OF WARRANTY.

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER
   OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR
   OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF
   THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

   (( 8. ))??
   TERMINATION.

      (( 8.1 ))??
      Termination upon Breach

      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. All sublicenses to the
      Covered Code, which are properly granted, shall survive any termination of
      this License. Provisions that, by their nature, must remain in effect
      beyond the termination of this License shall survive.

      (( 8.2. ))??
      Termination Upon Litigation. If You initiate litigation by asserting a
      patent infringement claim (excluding declatory judgment actions) against
      Initial Developer or a Contributor (the Initial Developer or Contributor
      against whom You file such action is referred to as "Participant")
      alleging that:

         (( (a) ))??
         such Participant's Contributor Version directly or indirectly infringes
         any patent, then any and all rights granted by such Participant to You
         under Sections 2.1 and/or 2.2 of this License shall, upon 60 days
         notice from Participant terminate prospectively, unless if within 60
         days after receipt of notice You either: (i) agree in writing to pay
         Participant a mutually agreeable reasonable royalty for Your past and
         future use of Modifications made by such Participant, or (ii) withdraw
         Your litigation claim with respect to the Contributor Version against
         such Participant. If within 60 days of notice, a reasonable royalty and
         payment arrangement are not mutually agreed upon in writing by the
         parties or the litigation claim is not withdrawn, the rights granted by
         Participant to You under Sections 2.1 and/or 2.2 automatically
         terminate at the expiration of the 60 day notice period specified
         above.

         (( (b) ))??
         any software, hardware, or device, other than such Participant's
         Contributor Version, directly or indirectly infringes any patent, then
         any rights granted to You by such Participant under Sections 2.1(b) and
         2.2(b) are revoked effective as of the date You first made, used, sold,
         distributed, or had made, Modifications made by that Participant.

      (( 8.3. ))??
      If You assert a patent infringement claim against Participant alleging
      that such Participant's Contributor Version directly or indirectly
      infringes any patent where such claim is resolved (such as by license or
      settlement) prior to the initiation of patent infringement litigation,
      then the reasonable value of the licenses granted by such Participant
      under Sections 2.1 or 2.2 shall be taken into account in determining the
      amount or value of any payment or license.

      (( 8.4. ))??
      In the event of termination under Sections 8.1 or 8.2 above, all end user
      license agreements (excluding distributors and resellers) which have been
      validly granted by You or any distributor hereunder prior to termination
      shall survive termination.

   (( 9. ))??
   LIMITATION OF LIABILITY.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE INITIAL DEVELOPER, ANY OTHER
   CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE, OR ANY SUPPLIER OF ANY OF
   SUCH PARTIES, BE LIABLE TO YOU OR ANY OTHER PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THAT EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. GOVERNMENT END USERS.

   The Covered Code is a ` + "`" + `` + "`" + `commercial item,'' as that term is defined in 48
   C.F.R. 2.101 (Oct. 1995), consisting of ` + "`" + `` + "`" + `commercial computer software'' and
   ` + "`" + `` + "`" + `commercial computer software documentation,'' as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Covered Code with only those rights set forth herein.

   (( 11. ))??
   MISCELLANEOUS.

   his section was intentionally left blank. The contents of this section are
   found in the corresponding addendum described above.

   (( 12. ))??
   RESPONSIBILITY FOR CLAIMS.

Except in cases where another Contributor has failed to comply with Section 3.4,
You are responsible for damages arising, directly or indirectly, out of Your
utilization of rights under this License, based on the number of copies of
Covered Code you made available, the revenues you received from utilizing such
rights, and other relevant factors. You agree to work with affected parties to
distribute with Initial Developer responsibility on an equitable basis.
(( EXHIBIT A.

Text for this Exhibit A is found in the corresponding addendum, described in
section 6.4 above, text file provided by the Initial Developer. This license is
not valid or complete with out that file.

EXHIBIT B.

Text for this Exhibit B is found in the corresponding addendum, described in
section 6.4 above, text file provided by the Initial Developer. This license is
not valid or complete with out that file. ))??
`
const license_OSET_PL_2_1_lre = `//**
OSET Public License version 2.1
https://spdx.org/licenses/OSET-PL-2.1.json
http://www.osetfoundation.org/public-license
https://opensource.org/licenses/OPL-2.1
**//

(( OSET Public License
(( Copyright __20__ ))??
))??

(( THIS LICENSE DEFINES THE RIGHTS OF USE, REPRODUCTION, DISTRIBUTION,
MODIFICATION, AND REDISTRIBUTION OF CERTAIN COVERED SOFTWARE (AS DEFINED BELOW)
ORIGINALLY RELEASED BY THE OPEN SOURCE ELECTION TECHNOLOGY FOUNDATION (FORMERLY
"THE OSDV FOUNDATION"). ))??

ANYONE WHO USES, REPRODUCES, DISTRIBUTES, MODIFIES, OR REDISTRIBUTES THE COVERED
SOFTWARE, OR ANY PART THEREOF, IS BY THAT ACTION, ACCEPTING IN FULL THE TERMS
CONTAINED IN THIS AGREEMENT. IF YOU DO NOT AGREE TO SUCH TERMS, YOU ARE NOT
PERMITTED TO USE THE COVERED SOFTWARE.
(( This license was prepared based on the Mozilla Public License ("MPL"),
version 2.0. For annotation of the differences between this license and MPL 2.0,
please see the OSET Foundation web site at
www.OSETFoundation.org/public-license.

The text of the license begins here: ))??

   (( 1. ))??
   Definitions

      (( 1.1 ))??
      "Contributor" means each individual or legal entity that creates,
      contributes to the creation of, or owns Covered Software.

      (( 1.2 ))??
      "Contributor Version" means the combination of the Contributions of others
      (if any) used by a Contributor and that particular Contributor's
      Contribution.

      (( 1.3 ))??
      "Contribution" means Covered Software of a particular Contributor.

      (( 1.4 ))??
      "Covered Software" means Source Code Form to which the initial Contributor
      has attached the notice in Exhibit A, the Executable Form of such Source
      Code Form, and Modifications of such Source Code Form, in each case
      including portions thereof.

      (( 1.5 ))??
      "Incompatible With Secondary Licenses" means:

         (( a. ))??
         That the initial Contributor has attached the notice described in
         Exhibit B to the Covered Software; or

         (( b. ))??
         that the Covered Software was made available under the terms of version
         1.x or earlier of the License, but not also under the terms of a
         Secondary License.

      (( 1.6 ))??
      "Executable Form" means any form of the work other than Source Code Form.

      (( 1.7 ))??
      "Larger Work" means a work that combines Covered Software with other
      material, in a separate file (or files) that is not Covered Software.

      (( 1.8 ))??
      "License" means this document.

      (( 1.9 ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently, any
      and all of the rights conveyed by this License.

      (( 1.10 ))??
      "Modifications" means any of the following:

         (( a. ))??
         any file in Source Code Form that results from an addition to, deletion
         from, or modification of the contents of Covered Software; or

         (( b. ))??
         any new file in Source Code Form that contains any Covered Software.

      (( 1.11 ))??
      "Patent Claims" of a Contributor means any patent claim(s), including
      without limitation, method, process, and apparatus claims, in any patent
      Licensable by such Contributor that would be infringed, but for the grant
      of the License, by the making, using, selling, offering for sale, having
      made, import, or transfer of either its Contributions or its Contributor
      Version.

      (( 1.12 ))??
      "Secondary License" means one of: the GNU General Public License, Version
      2.0, the GNU Lesser General Public License, Version 2.1, the GNU Affero
      General Public License, Version 3.0, or any later versions of those
      licenses.

      (( 1.13 ))??
      "Source Code Form" means the form of the work preferred for making
      modifications.

      (( 1.14 ))??
      "You" (or "Your") means an individual or a legal entity exercising rights
      under this License. For legal entities, "You" includes any entity that
      controls, is controlled by, or is under common control with You. For
      purposes of this definition, "control" means:

         (( (a) ))??
         the power, direct or indirect, to cause the direction or management of
         such entity, whether by contract or otherwise, or

         (( (b) ))??
         ownership of more than fifty percent (50%) of the outstanding shares or
         beneficial ownership of such entity.

   (( 2. ))??
   License Grants and Conditions

      (( 2.1 ))??
      Grants

      Each Contributor hereby grants You a world-wide, royalty-free,
      non-exclusive license:

         (( a. ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by such Contributor to use, reproduce, make available,
         modify, display, perform, distribute, and otherwise exploit its
         Contributions, either on an unmodified basis, with Modifications, or as
         part of a Larger Work; and

         (( b. ))??
         under Patent Claims of such Contributor to make, use, sell, offer for
         sale, have made, import, and otherwise transfer either its
         Contributions or its Contributor Version.

      (( 2.2 ))??
      Effective Date

      The licenses granted in Section 2.1 with respect to any Contribution
      become effective for each Contribution on the date the Contributor first
      distributes such Contribution.

      (( 2.3 ))??
      Limitations on Grant Scope

      The licenses granted in this Section 2 are the only rights granted under
      this License. No additional rights or licenses will be implied from the
      distribution or licensing of Covered Software under this License.
      Notwithstanding Section 2.1(b) above, no patent license is granted by a
      Contributor:

         (( a. ))??
         for any code that a Contributor has removed from Covered Software; or

         (( b. ))??
         for infringements caused by: (i) Your and any other third party's
         modifications of Covered Software, or (ii) the combination of its
         Contributions with other software (except as part of its Contributor
         Version); or

         (( c. ))??
         under Patent Claims infringed by Covered Software in the absence of its
         Contributions.

      This License does not grant any rights in the trademarks, service marks,
      or logos of any Contributor (except as may be necessary to comply with the
      notice requirements in Section 3.4).

      (( 2.4 ))??
      Subsequent Licenses

      No Contributor makes additional grants as a result of Your choice to
      distribute the Covered Software under a subsequent version of this License
      (see Section 10.2) or under the terms of a Secondary License (if permitted
      under the terms of Section 3.3).

      (( 2.5 ))??
      Representation

      Each Contributor represents that the Contributor believes its
      Contributions are its original creation(s) or it has sufficient rights to
      grant the rights to its Contributions conveyed by this License.

      (( 2.6 ))??
      Fair Use

      This License is not intended to limit any rights You have under applicable
      copyright doctrines of fair use, fair dealing, or other equivalents.

      (( 2.7 ))??
      Conditions

      Sections 3.1, 3.2, 3.3, and 3.4 are conditions of the licenses granted in
      Section 2.1.

   (( 3. ))??
   Responsibilities

      (( 3.1 ))??
      Distribution of Source Form

      All distribution of Covered Software in Source Code Form, including any
      Modifications that You create or to which You contribute, must be under
      the terms of this License. You must inform recipients that the Source Code
      Form of the Covered Software is governed by the terms of this License, and
      how they can obtain a copy of this License. You must cause any of Your
      Modifications to carry prominent notices stating that You changed the
      files. You may not attempt to alter or restrict the recipients' rights in
      the Source Code Form.

      (( 3.2 ))??
      Distribution of Executable Form

      If You distribute Covered Software in Executable Form then:

         (( a. ))??
         such Covered Software must also be made available in Source Code Form,
         as described in Section 3.1, and You must inform recipients of the
         Executable Form how they can obtain a copy of such Source Code Form by
         reasonable means in a timely manner, at a charge no more than the cost
         of distribution to the recipient; and

         (( b. ))??
         You may distribute such Executable Form under the terms of this
         License, or sublicense it under different terms, provided that the
         license for the Executable Form does not attempt to limit or alter the
         recipients' rights in the Source Code Form under this License.

      (( 3.3 ))??
      Distribution of a Larger Work

      You may create and distribute a Larger Work under terms of Your choice,
      provided that You also comply with the requirements of this License for
      the Covered Software. If the Larger Work is a combination of Covered
      Software with a work governed by one or more Secondary Licenses, and the
      Covered Software is not Incompatible With Secondary Licenses, this License
      permits You to additionally distribute such Covered Software under the
      terms of such Secondary License(s), so that the recipient of the Larger
      Work may, at their option, further distribute the Covered Software under
      the terms of either this License or such Secondary License(s).

      (( 3.4 ))??
      Notices

      You may not remove or alter the substance of any license notices
      (including copyright notices, patent notices, disclaimers of warranty, or
      limitations of liability) contained within the Source Code Form of the
      Covered Software, except that You may alter any license notices to the
      extent required to remedy known factual inaccuracies.

      (( 3.5 ))??
      Application of Additional Terms

         (( 3.5.1 ))??
         You may choose to offer, and to charge a fee for, warranty, support,
         indemnity or liability obligations to one or more recipients of Covered
         Software. However, You may do so only on Your own behalf, and not on
         behalf of any Contributor. You must make it absolutely clear that any
         such warranty, support, indemnity, or liability obligation is offered
         by You alone, and You hereby agree to indemnify every Contributor for
         any liability incurred by such Contributor as a result of warranty,
         support, indemnity or liability terms You offer. You may include
         additional disclaimers of warranty and limitations of liability
         specific to any jurisdiction.

         (( 3.5.2 ))??
         You may place additional conditions upon the rights granted in this
         License to the extent necessary due to statute, judicial order,
         regulation (including without limitation state and federal procurement
         regulation), national security, or public interest. Any such additional
         conditions must be clearly described in the notice provisions required
         under Section 3.4. Any alteration of the terms of this License will
         apply to all copies of the Covered Software distributed by You or by
         any downstream recipients that receive the Covered Software from You.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Software due to statute, judicial
   order, or regulation, then You must: (a) comply with the terms of this
   License to the maximum extent possible; and (b) describe the limitations and
   the code they affect. Such description must be included in the notices
   required under Section 3.4. Except to the extent prohibited by statute or
   regulation, such description must be sufficiently detailed for a recipient of
   ordinary skill to be able to understand it.

   (( 5. ))??
   Termination

      (( 5.1 ))??
      Failure to Comply

      The rights granted under this License will terminate automatically if You
      fail to comply with any of its terms. However, if You become compliant,
      then the rights granted under this License from a particular Contributor
      are reinstated (a) provisionally, unless and until such Contributor
      explicitly and finally terminates Your grants, and (b) on an ongoing
      basis, if such Contributor fails to notify You of the non-compliance by
      some reasonable means prior to 60-days after You have come back into
      compliance. Moreover, Your grants from a particular Contributor are
      reinstated on an ongoing basis if such Contributor notifies You of the
      non-compliance by some reasonable means, this is the first time You have
      received notice of non-compliance with this License from such Contributor,
      and You become compliant prior to 30-days after Your receipt of the
      notice.

      (( 5.2 ))??
      Patent Infringement Claims

      If You initiate litigation against any entity by asserting a patent
      infringement claim (excluding declaratory judgment actions,
      counter-claims, and
      //** cross-claims **//
      __5__
      ) alleging that a Contributor Version directly or indirectly infringes any
      patent, then the rights granted to You by any and all Contributors for the
      Covered Software under Section 2.1 of this License shall terminate.

      (( 5.3 ))??
      Additional Compliance Terms

      Notwithstanding the foregoing in this Section 5, for purposes of this
      Section, if You breach Section 3.1 (Distribution of Source Form), Section
      3.2 (Distribution of Executable Form), Section 3.3 (Distribution of a
      Larger Work), or Section 3.4 (Notices), then becoming compliant as
      described in Section 5.1 must also include, no later than 30 days after
      receipt by You of notice of such violation by a Contributor, making the
      Covered Software available in Source Code Form as required by this License
      on a publicly available computer network for a period of no less than
      three (3) years.

      (( 5.4 ))??
      Contributor Remedies

      If You fail to comply with the terms of this License and do not thereafter
      become compliant in accordance with Section 5.1 and, if applicable,
      Section 5.3, then each Contributor reserves its right, in addition to any
      other rights it may have in law or in equity, to bring an action seeking
      injunctive relief, or damages for willful copyright or patent infringement
      (including without limitation damages for unjust enrichment, where
      available under law), for all actions in violation of rights that would
      otherwise have been granted under the terms of this License.

      (( 5.5 ))??
      End User License Agreements

      In the event of termination under this Section 5, all end user license
      agreements (excluding distributors and resellers), which have been validly
      granted by You or Your distributors under this License prior to
      termination shall survive termination.

   (( 6. ))??
   Disclaimer of Warranty

   Covered Software is provided under this License on an "as is" basis, without
   warranty of any kind, either expressed, implied, or statutory, including,
   without limitation, warranties that the Covered Software is free of defects,
   merchantable, fit for a particular purpose or non-infringing. The entire risk
   as to the quality and performance of the Covered Software is with You. Should
   any Covered Software prove defective in any respect, You (not any
   Contributor) assume the cost of any necessary servicing, repair, or
   correction. This disclaimer of warranty constitutes an essential part of this
   License. No use of any Covered Software is authorized under this License
   except under this disclaimer.

   (( 7. ))??
   Limitation of Liability

   Under no circumstances and under no legal theory, whether tort (including
   negligence), contract, or otherwise, shall any Contributor, or anyone who
   distributes Covered Software as permitted above, be liable to You for any
   direct, indirect, special, incidental, or consequential damages of any
   character including, without limitation, damages for lost profits, loss of
   goodwill, work stoppage, computer failure or malfunction, or any and all
   other commercial damages or losses, even if such party shall have been
   informed of the possibility of such damages. This limitation of liability
   shall not apply to liability for death or personal injury resulting from such
   party's negligence to the extent applicable law prohibits such limitation.
   Some jurisdictions do not allow the exclusion or limitation of incidental or
   consequential damages, so this exclusion and limitation may not apply to
   You.

   (( 8. ))??
   Litigation

   Any litigation relating to this License may be brought only in the courts of
   a jurisdiction where the defendant maintains its principal place of business
   and such litigation shall be governed by laws of that jurisdiction, without
   reference to its conflict-of-law provisions. Nothing in this Section shall
   prevent a party's ability to bring cross-claims or counter-claims.

   (( 9. ))??
   Government Terms

      (( 9.1 ))??
      Commercial Item

      The Covered Software is a "commercial item," as that term is defined in 48
      C.F.R. 2.101 (Oct. 1995), consisting of "commercial computer software" and
      "commercial computer software documentation," as such terms are used in 48
      C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
      227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
      acquire Covered Software with only those rights set forth herein.

      (( 9.2 ))??
      No Sovereign Immunity

      The U.S. federal government and states that use or distribute Covered
      Software hereby waive their sovereign immunity with respect to enforcement
      of the provisions of this License.

      (( 9.3 ))??
      Choice of Law and Venue

         (( 9.3.1 ))??
         If You are a government of a state of the United States, or Your use of
         the Covered Software is pursuant to a procurement contract with such a
         state government, this License shall be governed by the law of such
         state, excluding its conflict-of-law provisions, and the adjudication
         of disputes relating to this License will be subject to the exclusive
         jurisdiction of the state and federal courts located in such state.

         (( 9.3.2 ))??
         If You are an agency of the United States federal government, or Your
         use of the Covered Software is pursuant to a procurement contract with
         such an agency, this License shall be governed by federal law for all
         purposes, and the adjudication of disputes relating to this License
         will be subject to the exclusive jurisdiction of the federal courts
         located in Washington, D.C.

         (( 9.3.3 ))??
         You may alter the terms of this Section 9.3 for this License as
         described in Section 3.5.2.

      (( 9.4 ))??
      Supremacy

      This Section 9 is in lieu of, and supersedes, any other Federal
      Acquisition Regulation, Defense Federal Acquisition Regulation, or other
      clause or provision that addresses government rights in computer software
      under this License.

   (( 10. ))??
   Miscellaneous

   This License represents the complete agreement concerning the subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. Any law or regulation, which provides that the language of a
   contract shall be construed against the drafter, shall not be used to
   construe this License against a Contributor.

   (( 11. ))??
   Versions of the License

      (( 11.1 ))??
      New Versions

      The Open Source Election Technology Foundation ("OSET") (formerly known as
      the Open Source Digital Voting Foundation) is the steward of this License.
      Except as provided in Section 11.3, no one other than the license steward
      has the right to modify or publish new versions of this License. Each
      version will be given a distinguishing version number.

      (( 11.2 ))??
      Effects of New Versions

      You may distribute the Covered Software under the terms of the version of
      the License under which You originally received the Covered Software, or
      under the terms of any subsequent version published by the license
      steward.

      (( 11.3 ))??
      Modified Versions

      If You create software not governed by this License, and You want to
      create a new license for such software, You may create and use a modified
      version of this License if You rename the license and remove any
      references to the name of the license steward (except to note that such
      modified license differs from this License).

      (( 11.4 ))??
      Distributing Source Code Form That is Incompatible With Secondary
      Licenses

      If You choose to distribute Source Code Form that is Incompatible With
      Secondary Licenses under the terms of this version of the License, the
      notice described in Exhibit B of this License must be attached.
      (( EXHIBIT A - Source Code Form License Notice

This Source Code Form is subject to the terms of the OSET Public License, v.2.1
("OPL"). If a copy of the OPL was not distributed with this file, You can obtain
one at: www.OSETFoundation.org/public-license.

If it is not possible or desirable to put the Notice in a particular file, then
You may include the Notice in a location (e.g., such as a LICENSE file in a
relevant directory) where a recipient would be likely to look for such a notice.
You may add additional accurate notices of copyright ownership. ))??

(( EXHIBIT B - "Incompatible With Secondary License" Notice

This Source Code Form is "Incompatible With Secondary Licenses", as defined by
the OSET Public License, v.2.1. ))??
`
const license_OSL_1_0_lre = `//**
Open Software License 1.0
https://spdx.org/licenses/OSL-1.0.json
https://opensource.org/licenses/OSL-1.0
**//

(( The Open Software License v. 1.0 ))??

This Open Software License (the "License") applies to any original work of
authorship (the "Original Work") whose owner (the "Licensor") has placed the
following notice immediately following the copyright notice for the Original
Work:

Licensed under the Open Software License version 1.0

License Terms

   (( 1) ))??
   Grant of Copyright License. Licensor hereby grants You a world-wide,
   royalty-free, non-exclusive, perpetual, non-sublicenseable license to do the
   following:

      (( a) ))??
      to reproduce the Original Work in copies;

      (( b) ))??
      to prepare derivative works ("Derivative Works") based upon the Original
      Work;

      (( c) ))??
      to distribute copies of the Original Work and Derivative Works to the
      public, with the proviso that copies of Original Work or Derivative Works
      that You distribute shall be licensed under the Open Software License;

      (( d) ))??
      to perform the Original Work publicly; and

      (( e) ))??
      to display the Original Work publicly.

   (( 2) ))??
   Grant of Patent License. Licensor hereby grants You a world-wide,
   royalty-free, non-exclusive, perpetual, non-sublicenseable license, under
   patent claims owned or controlled by the Licensor that are embodied in the
   Original Work as furnished by the Licensor ("Licensed Claims") to make, use,
   sell and offer for sale the Original Work. Licensor hereby grants You a
   world-wide, royalty-free, non-exclusive, perpetual, non-sublicenseable
   license under the Licensed Claims to make, use, sell and offer for sale
   Derivative Works.

   (( 3) ))??
   Grant of Source Code License. The term "Source Code" means the preferred form
   of the Original Work for making modifications to it and all available
   documentation describing how to access and modify the Original Work. Licensor
   hereby agrees to provide a machine-readable copy of the Source Code of the
   Original Work along with each copy of the Original Work that Licensor
   distributes. Licensor reserves the right to satisfy this obligation by
   placing a machine-readable copy of the Source Code in an information
   repository reasonably calculated to permit inexpensive and convenient access
   by You for as long as Licensor continues to distribute the Original Work, and
   by publishing the address of that information repository in a notice
   immediately following the copyright notice that applies to the Original
   Work.

   (( 4) ))??
   Exclusions From License Grant. Nothing in this License shall be deemed to
   grant any rights to trademarks, copyrights, patents, trade secrets or any
   other intellectual property of Licensor except as expressly stated herein. No
   patent license is granted to make, use, sell or offer to sell embodiments of
   any patent claims other than the Licensed Claims defined in Section 2. No
   right is granted to the trademarks of Licensor even if such marks are
   included in the Original Work. Nothing in this License shall be interpreted
   to prohibit Licensor from licensing under different terms from this License
   any Original Work that Licensor otherwise would have a right to license.

   (( 5) ))??
   External Deployment. The term "External Deployment" means the use or
   distribution of the Original Work or Derivative Works in any way such that
   the Original Work or Derivative Works may be accessed or used by anyone other
   than You, whether the Original Work or Derivative Works are distributed to
   those persons, made available as an application intended for use over a
   computer network, or used to provide services or otherwise deliver content to
   anyone other than You. As an express condition for the grants of license
   hereunder, You agree that any External Deployment by You shall be deemed a
   distribution and shall be licensed to all under the terms of this License, as
   prescribed in section 1(c) herein.

   (( 6) ))??
   Warranty and Disclaimer of Warranty. LICENSOR WARRANTS THAT THE COPYRIGHT IN
   AND TO THE ORIGINAL WORK IS OWNED BY THE LICENSOR OR THAT THE ORIGINAL WORK
   IS DISTRIBUTED BY LICENSOR UNDER A VALID CURRENT LICENSE FROM THE COPYRIGHT
   OWNER. EXCEPT AS EXPRESSLY STATED IN THE IMMEDIATELY PRECEEDING SENTENCE, THE
   ORIGINAL WORK IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, THE
   WARRANTY OF NON-INFRINGEMENT AND WARRANTIES THAT THE ORIGINAL WORK IS
   MERCHANTABLE OR FIT FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE
   QUALITY OF THE ORIGINAL WORK IS WITH YOU. THIS DISCLAIMER OF WARRANTY
   CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE. NO LICENSE TO ORIGINAL WORK IS
   GRANTED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.

   (( 7) ))??
   Limitation of Liability. UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY,
   WHETHER TORT (INCLUDING NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE
   LICENSOR BE LIABLE TO ANY PERSON FOR ANY DIRECT, INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER ARISING AS A RESULT OF
   THIS LICENSE OR THE USE OF THE ORIGINAL WORK INCLUDING, WITHOUT LIMITATION,
   DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR MALFUNCTION,
   OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH PERSON SHALL
   HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS LIMITATION OF
   LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL INJURY RESULTING
   FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW PROHIBITS SUCH
   LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR LIMITATION OF
   INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS EXCLUSION AND LIMITATION MAY NOT
   APPLY TO YOU.

   (( 8) ))??
   Acceptance and Termination. Nothing else but this License (or another written
   agreement between Licensor and You) grants You permission to create
   Derivative Works based upon the Original Work, and any attempt to do so
   except under the terms of this License (or another written agreement between
   Licensor and You) is expressly prohibited by U.S. copyright law, the
   equivalent laws of other countries, and by international treaty. Therefore,
   by exercising any of the rights granted to You in Sections 1 and 2 herein,
   You indicate Your acceptance of this License and all of its terms and
   conditions. This license shall terminate immediately and you may no longer
   exercise any of the rights granted to You by this License upon Your failure
   to honor the proviso in Section 1(c) herein.

   (( 9) ))??
   Mutual Termination for Patent Action. This License shall terminate
   automatically and You may no longer exercise any of the rights granted to You
   by this License if You file a lawsuit in any court alleging that any OSI
   Certified open source software that is licensed under any license containing
   this "Mutual Termination for Patent Action" clause infringes any patent
   claims that are essential to use that software.

   (( 10) ))??
   Jurisdiction, Venue and Governing Law. You agree that any lawsuit arising
   under or relating to this License shall be maintained in the courts of the
   jurisdiction wherein the Licensor resides or in which Licensor conducts its
   primary business, and under the laws of that jurisdiction excluding its
   conflict-of-law provisions. The application of the United Nations Convention
   on Contracts for the International Sale of Goods is expressly excluded. Any
   use of the Original Work outside the scope of this License or after its
   termination shall be subject to the requirements and penalties of the U.S.
   Copyright Act, 17 U.S.C. § 101 et seq., the equivalent laws of other
   countries, and international treaty. This section shall survive the
   termination of this License.

   (( 11) ))??
   Attorneys Fees. In any action to enforce the terms of this License or seeking
   damages relating thereto, the prevailing party shall be entitled to recover
   its costs and expenses, including, without limitation, reasonable attorneys'
   fees and costs incurred in connection with such action, including any appeal
   of such action. This section shall survive the termination of this License.

   (( 12) ))??
   Miscellaneous. This License represents the complete agreement concerning the
   subject matter hereof. If any provision of this License is held to be
   unenforceable, such provision shall be reformed only to the extent necessary
   to make it enforceable.

   (( 13) ))??
   Definition of "You" in This License. "You" throughout this License, whether
   in upper or lower case, means an individual or a legal entity exercising
   rights under, and complying with all of the terms of, this License. For legal
   entities, "You" includes any entity that controls, is controlled by, or is
   under common control with you. For purposes of this definition, "control"
   means (i) the power, direct or indirect, to cause the direction or management
   of such entity, whether by contract or otherwise, or (ii) ownership of fifty
   percent (50%) or more of the outstanding shares, or (iii) beneficial
   ownership of such entity.

This license is Copyright (C) 2002 Lawrence E. Rosen. All rights reserved.
Permission is hereby granted to copy and distribute this license without
modification. This license may not be modified without the express written
permission of its copyright owner.
`
const license_OSL_1_1_lre = `//**
Open Software License 1.1
https://spdx.org/licenses/OSL-1.1.json
https://fedoraproject.org/wiki/Licensing/OSL1.1
**//

(( The Open Software License v. 1.1 ))??

This Open Software License (the "License") applies to any original work of
authorship (the "Original Work") whose owner (the "Licensor") has placed the
following notice immediately following the copyright notice for the Original
Work:

Licensed under the Open Software License version 1.1

   (( 1) ))??
   Grant of Copyright License. Licensor hereby grants You a world-wide,
   royalty-free, non-exclusive, perpetual, non-sublicenseable license to do the
   following:

      (( a) ))??
      to reproduce the Original Work in copies;

      (( b) ))??
      to prepare derivative works ("Derivative Works") based upon the Original
      Work;

      (( c) ))??
      to distribute copies of the Original Work and Derivative Works to the
      public, with the proviso that copies of Original Work or Derivative Works
      that You distribute shall be licensed under the Open Software License;

      (( d) ))??
      to perform the Original Work publicly; and

      (( e) ))??
      to display the Original Work publicly.

   (( 2) ))??
   Grant of Patent License. Licensor hereby grants You a world-wide,
   royalty-free, non-exclusive, perpetual, non-sublicenseable license, under
   patent claims owned or controlled by the Licensor that are embodied in the
   Original Work as furnished by the Licensor ("Licensed Claims") to make, use,
   sell and offer for sale the Original Work. Licensor hereby grants You a
   world-wide, royalty-free, non-exclusive, perpetual, non-sublicenseable
   license under the Licensed Claims to make, use, sell and offer for sale
   Derivative Works.

   (( 3) ))??
   Grant of Source Code License. The term "Source Code" means the preferred form
   of the Original Work for making modifications to it and all available
   documentation describing how to modify the Original Work. Licensor hereby
   agrees to provide a machine-readable copy of the Source Code of the Original
   Work along with each copy of the Original Work that Licensor distributes.
   Licensor reserves the right to satisfy this obligation by placing a
   machine-readable copy of the Source Code in an information repository
   reasonably calculated to permit inexpensive and convenient access by You for
   as long as Licensor continues to distribute the Original Work, and by
   publishing the address of that information repository in a notice immediately
   following the copyright notice that applies to the Original Work.

   (( 4) ))??
   Exclusions From License Grant. Nothing in this License shall be deemed to
   grant any rights to trademarks, copyrights, patents, trade secrets or any
   other intellectual property of Licensor except as expressly stated herein. No
   patent license is granted to make, use, sell or offer to sell embodiments of
   any patent claims other than the Licensed Claims defined in Section 2. No
   right is granted to the trademarks of Licensor even if such marks are
   included in the Original Work. Nothing in this License shall be interpreted
   to prohibit Licensor from licensing under different terms from this License
   any Original Work that Licensor otherwise would have a right to license.

   (( 5) ))??
   External Deployment. The term "External Deployment" means the use or
   distribution of the Original Work or Derivative Works in any way such that
   the Original Work or Derivative Works may be used by anyone other than You,
   whether the Original Work or Derivative Works are distributed to those
   persons or made available as an application intended for use over a computer
   network. As an express condition for the grants of license hereunder, You
   agree that any External Deployment by You of a Derivative Work shall be
   deemed a distribution and shall be licensed to all under the terms of this
   License, as prescribed in section 1(c) herein.

   (( 6) ))??
   Attribution Rights. You must retain, in the Source Code of any Derivative
   Works that You create, all copyright, patent or trademark notices from the
   Source Code of the Original Work, as well as any notices of licensing and any
   descriptive text identified therein as an "Attribution Notice." You must
   cause the Source Code for any Derivative Works that You create to carry a
   prominent Attribution Notice reasonably calculated to inform recipients that
   You have modified the Original Work.

   (( 7) ))??
   Warranty and Disclaimer of Warranty. Licensor warrants that the copyright in
   and to the Original Work is owned by the Licensor or that the Original Work
   is distributed by Licensor under a valid current license from the copyright
   owner. Except as expressly stated in the immediately proceeding sentence, the
   Original Work is provided under this License on an "AS IS" BASIS and WITHOUT
   WARRANTY, either express or implied, including, without limitation, the
   warranties of NON-INFRINGEMENT, MERCHANTABILITY or FITNESS FOR A PARTICULAR
   PURPOSE. THE ENTIRE RISK AS TO THE QUALITY OF THE ORIGINAL WORK IS WITH YOU.
   This DISCLAIMER OF WARRANTY constitutes an essential part of this License. No
   license to Original Work is granted hereunder except under this disclaimer.

   (( 8) ))??
   Limitation of Liability. Under no circumstances and under no legal theory,
   whether in tort (including negligence), contract, or otherwise, shall the
   Licensor be liable to any person for any direct, indirect, special,
   incidental, or consequential damages of any character arising as a result of
   this License or the use of the Original Work including, without limitation,
   damages for loss of goodwill, work stoppage, computer failure or malfunction,
   or any and all other commercial damages or losses. This limitation of
   liability shall not apply to liability for death or personal injury resulting
   from Licensor's negligence to the extent applicable law prohibits such
   limitation. Some jurisdictions do not allow the exclusion or limitation of
   incidental or consequential damages, so this exclusion and limitation may not
   apply to You.

   (( 9) ))??
   Acceptance and Termination. If You distribute copies of the Original Work or
   a Derivative Work, You must make a reasonable effort under the circumstances
   to obtain the express and volitional assent of recipients to the terms of
   this License. Nothing else but this License (or another written agreement
   between Licensor and You) grants You permission to create Derivative Works
   based upon the Original Work or to exercise any of the rights granted in
   Sections 1 herein, and any attempt to do so except under the terms of this
   License (or another written agreement between Licensor and You) is expressly
   prohibited by U.S. copyright law, the equivalent laws of other countries, and
   by international treaty. Therefore, by exercising any of the rights granted
   to You in Sections 1 herein, You indicate Your acceptance of this License and
   all of its terms and conditions. This License shall terminate immediately and
   you may no longer exercise any of the rights granted to You by this License
   upon Your failure to honor the proviso in Section 1(c) herein.

   (( 10) ))??
   Mutual Termination for Patent Action. This License shall terminate
   automatically and You may no longer exercise any of the rights granted to You
   by this License if You file a lawsuit in any court alleging that any OSI
   Certified open source software that is licensed under any license containing
   this "Mutual Termination for Patent Action" clause infringes any patent
   claims that are essential to use that software.

   (( 11) ))??
   Jurisdiction, Venue and Governing Law. Any action or suit relating to this
   License may be brought only in the courts of a jurisdiction wherein the
   Licensor resides or in which Licensor conducts its primary business, and
   under the laws of that jurisdiction excluding its conflict-of-law provisions.
   The application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any use of the Original
   Work outside the scope of this License or after its termination shall be
   subject to the requirements and penalties of the U.S. Copyright Act, 17
   U.S.C. å¤ 101 et seq., the equivalent laws of other countries, and
   international treaty. This section shall survive the termination of this
   License.

   (( 12) ))??
   Attorneys Fees. In any action to enforce the terms of this License or seeking
   damages relating thereto, the prevailing party shall be entitled to recover
   its costs and expenses, including, without limitation, reasonable attorneys'
   fees and costs incurred in connection with such action, including any appeal
   of such action. This section shall survive the termination of this License.

   (( 13) ))??
   Miscellaneous. This License represents the complete agreement concerning the
   subject matter hereof. If any provision of this License is held to be
   unenforceable, such provision shall be reformed only to the extent necessary
   to make it enforceable.

   (( 14) ))??
   Definition of "You" in This License. "You" throughout this License, whether
   in upper or lower case, means an individual or a legal entity exercising
   rights under, and complying with all of the terms of, this License. For legal
   entities, "You" includes any entity that controls, is controlled by, or is
   under common control with you. For purposes of this definition, "control"
   means (i) the power, direct or indirect, to cause the direction or management
   of such entity, whether by contract or otherwise, or (ii) ownership of fifty
   percent (50%) or more of the outstanding shares, or (iii) beneficial
   ownership of such entity.

   (( 15) ))??
   Right to Use. You may use the Original Work in all ways not otherwise
   restricted or conditioned by this License or by law, and Licensor promises
   not to interfere with or be responsible for such uses by You.

This license is Copyright (C) 2002 Lawrence E. Rosen. All rights reserved.
Permission is hereby granted to copy and distribute this license without
modification. This license may not be modified without the express written
permission of its copyright owner.
`
const license_OSL_2_0_lre = `//**
Open Software License 2.0
https://spdx.org/licenses/OSL-2.0.json
http://web.archive.org/web/20041020171434/http://www.rosenlaw.com/osl2.0.html
**//

((
(( The ))??
Open Software License v. 2.0 ))??

This Open Software License (the "License") applies to any original work of
authorship (the "Original Work") whose owner (the "Licensor") has placed the
following notice immediately following the copyright notice for the Original
Work:

Licensed under the Open Software License version 2.0

   (( 1) ))??
   Grant of Copyright License. Licensor hereby grants You a world-wide,
   royalty-free, non-exclusive, perpetual, sublicenseable license to do the
   following:

      (( a) ))??
      to reproduce the Original Work in copies;

      (( b) ))??
      to prepare derivative works ("Derivative Works") based upon the Original
      Work;

      (( c) ))??
      to distribute copies of the Original Work and Derivative Works to the
      public, with the proviso that copies of Original Work or Derivative Works
      that You distribute shall be licensed under the Open Software License;

      (( d) ))??
      to perform the Original Work publicly; and

      (( e) ))??
      to display the Original Work publicly.

   (( 2) ))??
   Grant of Patent License. Licensor hereby grants You a world-wide,
   royalty-free, non-exclusive, perpetual, sublicenseable license, under patent
   claims owned or controlled by the Licensor that are embodied in the Original
   Work as furnished by the Licensor, to make, use, sell and offer for sale the
   Original Work and Derivative Works.

   (( 3) ))??
   Grant of Source Code License. The term "Source Code" means the preferred form
   of the Original Work for making modifications to it and all available
   documentation describing how to modify the Original Work. Licensor hereby
   agrees to provide a machine-readable copy of the Source Code of the Original
   Work along with each copy of the Original Work that Licensor distributes.
   Licensor reserves the right to satisfy this obligation by placing a
   machine-readable copy of the Source Code in an information repository
   reasonably calculated to permit inexpensive and convenient access by You for
   as long as Licensor continues to distribute the Original Work, and by
   publishing the address of that information repository in a notice immediately
   following the copyright notice that applies to the Original Work.

   (( 4) ))??
   Exclusions From License Grant. Neither the names of Licensor, nor the names
   of any contributors to the Original Work, nor any of their trademarks or
   service marks, may be used to endorse or promote products derived from this
   Original Work without express prior written permission of the Licensor.
   Nothing in this License shall be deemed to grant any rights to trademarks,
   copyrights, patents, trade secrets or any other intellectual property of
   Licensor except as expressly stated herein. No patent license is granted to
   make, use, sell or offer to sell embodiments of any patent claims other than
   the licensed claims defined in Section 2. No right is granted to the
   trademarks of Licensor even if such marks are included in the Original Work.
   Nothing in this License shall be interpreted to prohibit Licensor from
   licensing under different terms from this License any Original Work that
   Licensor otherwise would have a right to license.

   (( 5) ))??
   External Deployment. The term "External Deployment" means the use or
   distribution of the Original Work or Derivative Works in any way such that
   the Original Work or Derivative Works may be used by anyone other than You,
   whether the Original Work or Derivative Works are distributed to those
   persons or made available as an application intended for use over a computer
   network. As an express condition for the grants of license hereunder, You
   agree that any External Deployment by You of a Derivative Work shall be
   deemed a distribution and shall be licensed to all under the terms of this
   License, as prescribed in section 1(c) herein.

   (( 6) ))??
   Attribution Rights. You must retain, in the Source Code of any Derivative
   Works that You create, all copyright, patent or trademark notices from the
   Source Code of the Original Work, as well as any notices of licensing and any
   descriptive text identified therein as an "Attribution Notice." You must
   cause the Source Code for any Derivative Works that You create to carry a
   prominent Attribution Notice reasonably calculated to inform recipients that
   You have modified the Original Work.

   (( 7) ))??
   Warranty of Provenance and Disclaimer of Warranty. Licensor warrants that the
   copyright in and to the Original Work and the patent rights granted herein by
   Licensor are owned by the Licensor or are sublicensed to You under the terms
   of this License with the permission of the contributor(s) of those copyrights
   and patent rights. Except as expressly stated in the immediately proceeding
   sentence, the Original Work is provided under this License on an "AS IS"
   BASIS and WITHOUT WARRANTY, either express or implied, including, without
   limitation, the warranties of NON-INFRINGEMENT, MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY OF THE ORIGINAL
   WORK IS WITH YOU. This DISCLAIMER OF WARRANTY constitutes an essential part
   of this License. No license to Original Work is granted hereunder except
   under this disclaimer.

   (( 8) ))??
   Limitation of Liability. Under no circumstances and under no legal theory,
   whether in tort (including negligence), contract, or otherwise, shall the
   Licensor be liable to any person for any direct, indirect, special,
   incidental, or consequential damages of any character arising as a result of
   this License or the use of the Original Work including, without limitation,
   damages for loss of goodwill, work stoppage, computer failure or malfunction,
   or any and all other commercial damages or losses. This limitation of
   liability shall not apply to liability for death or personal injury resulting
   from Licensor's negligence to the extent applicable law prohibits such
   limitation. Some jurisdictions do not allow the exclusion or limitation of
   incidental or consequential damages, so this exclusion and limitation may not
   apply to You.

   (( 9) ))??
   Acceptance and Termination. If You distribute copies of the Original Work or
   a Derivative Work, You must make a reasonable effort under the circumstances
   to obtain the express assent of recipients to the terms of this License.
   Nothing else but this License (or another written agreement between Licensor
   and You) grants You permission to create Derivative Works based upon the
   Original Work or to exercise any of the rights granted in Section 1 herein,
   and any attempt to do so except under the terms of this License (or another
   written agreement between Licensor and You) is expressly prohibited by U.S.
   copyright law, the equivalent laws of other countries, and by international
   treaty. Therefore, by exercising any of the rights granted to You in Section
   1 herein, You indicate Your acceptance of this License and all of its terms
   and conditions. This License shall terminate immediately and you may no
   longer exercise any of the rights granted to You by this License upon Your
   failure to honor the proviso in Section 1(c) herein.

   (( 10) ))??
   Termination for Patent Action. This License shall terminate automatically and
   You may no longer exercise any of the rights granted to You by this License
   as of the date You commence an action, including a cross-claim or
   counterclaim, for patent infringement (i) against Licensor with respect to a
   patent applicable to software or (ii) against any entity with respect to a
   patent applicable to the Original Work (but excluding combinations of the
   Original Work with other software or hardware).

   (( 11) ))??
   Jurisdiction, Venue and Governing Law. Any action or suit relating to this
   License may be brought only in the courts of a jurisdiction wherein the
   Licensor resides or in which Licensor conducts its primary business, and
   under the laws of that jurisdiction excluding its conflict-of-law provisions.
   The application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any use of the Original
   Work outside the scope of this License or after its termination shall be
   subject to the requirements and penalties of the U.S. Copyright Act, 17
   U.S.C. 101 et seq., the equivalent laws of other countries, and international
   treaty. This section shall survive the termination of this License.

   (( 12) ))??
   Attorneys Fees. In any action to enforce the terms of this License or seeking
   damages relating thereto, the prevailing party shall be entitled to recover
   its costs and expenses, including, without limitation, reasonable attorneys'
   fees and costs incurred in connection with such action, including any appeal
   of such action. This section shall survive the termination of this License.

   (( 13) ))??
   Miscellaneous. This License represents the complete agreement concerning the
   subject matter hereof. If any provision of this License is held to be
   unenforceable, such provision shall be reformed only to the extent necessary
   to make it enforceable.

   (( 14) ))??
   Definition of "You" in This License. "You" throughout this License, whether
   in upper or lower case, means an individual or a legal entity exercising
   rights under, and complying with all of the terms of, this License. For legal
   entities, "You" includes any entity that controls, is controlled by, or is
   under common control with you. For purposes of this definition, "control"
   means (i) the power, direct or indirect, to cause the direction or management
   of such entity, whether by contract or otherwise, or (ii) ownership of fifty
   percent (50%) or more of the outstanding shares, or (iii) beneficial
   ownership of such entity.

   (( 15) ))??
   Right to Use. You may use the Original Work in all ways not otherwise
   restricted or conditioned by this License or by law, and Licensor promises
   not to interfere with or be responsible for such uses by You.

This license is Copyright (C) 2003 Lawrence E. Rosen. All rights reserved.
Permission is hereby granted to copy and distribute this license without
modification. This license may not be modified without the express written
permission of its copyright owner.
`
const license_OSL_2_1_lre = `//**
Open Software License 2.1
https://spdx.org/licenses/OSL-2.1.json
http://web.archive.org/web/20050212003940/http://www.rosenlaw.com/osl21.htm
https://opensource.org/licenses/OSL-2.1
**//

(( The Open Software Licensev. 2.1 ))??

This Open Software License (the "License") applies to any original work of
authorship (the "Original Work") whose owner (the "Licensor") has placed the
following notice immediately following the copyright notice for the Original
Work:

Licensed under the Open Software License version 2.1

   (( 1) ))??
   Grant of Copyright License. Licensor hereby grants You a world-wide,
   royalty-free, non-exclusive, perpetual, sublicenseable license to do the
   following:

      (( a) ))??
      to reproduce the Original Work in copies;

      (( b) ))??
      to prepare derivative works ("Derivative Works") based upon the Original
      Work;

      (( c) ))??
      to distribute copies of the Original Work and Derivative Works to the
      public, with the proviso that copies of Original Work or Derivative Works
      that You distribute shall be licensed under the Open Software License;

      (( d) ))??
      to perform the Original Work publicly; and

      (( e) ))??
      to display the Original Work publicly.

   (( 2) ))??
   Grant of Patent License. Licensor hereby grants You a world-wide,
   royalty-free, non-exclusive, perpetual, sublicenseable license, under patent
   claims owned or controlled by the Licensor that are embodied in the Original
   Work as furnished by the Licensor, to make, use, sell and offer for sale the
   Original Work and Derivative Works.

   (( 3) ))??
   Grant of Source Code License. The term "Source Code" means the preferred form
   of the Original Work for making modifications to it and all available
   documentation describing how to modify the Original Work. Licensor hereby
   agrees to provide a machine-readable copy of the Source Code of the Original
   Work along with each copy of the Original Work that Licensor distributes.
   Licensor reserves the right to satisfy this obligation by placing a
   machine-readable copy of the Source Code in an information repository
   reasonably calculated to permit inexpensive and convenient access by You for
   as long as Licensor continues to distribute the Original Work, and by
   publishing the address of that information repository in a notice immediately
   following the copyright notice that applies to the Original Work.

   (( 4) ))??
   Exclusions From License Grant. Neither the names of Licensor, nor the names
   of any contributors to the Original Work, nor any of their trademarks or
   service marks, may be used to endorse or promote products derived from this
   Original Work without express prior written permission of the Licensor.
   Nothing in this License shall be deemed to grant any rights to trademarks,
   copyrights, patents, trade secrets or any other intellectual property of
   Licensor except as expressly stated herein. No patent license is granted to
   make, use, sell or offer to sell embodiments of any patent claims other than
   the licensed claims defined in Section 2. No right is granted to the
   trademarks of Licensor even if such marks are included in the Original Work.
   Nothing in this License shall be interpreted to prohibit Licensor from
   licensing under different terms from this License any Original Work that
   Licensor otherwise would have a right to license.

   (( 5) ))??
   External Deployment. The term "External Deployment" means the use or
   distribution of the Original Work or Derivative Works in any way such that
   the Original Work or Derivative Works may be used by anyone other than You,
   whether the Original Work or Derivative Works are distributed to those
   persons or made available as an application intended for use over a computer
   network. As an express condition for the grants of license hereunder, You
   agree that any External Deployment by You of a Derivative Work shall be
   deemed a distribution and shall be licensed to all under the terms of this
   License, as prescribed in section 1(c) herein.

   (( 6) ))??
   Attribution Rights. You must retain, in the Source Code of any Derivative
   Works that You create, all copyright, patent or trademark notices from the
   Source Code of the Original Work, as well as any notices of licensing and any
   descriptive text identified therein as an "Attribution Notice." You must
   cause the Source Code for any Derivative Works that You create to carry a
   prominent Attribution Notice reasonably calculated to inform recipients that
   You have modified the Original Work.

   (( 7) ))??
   Warranty of Provenance and Disclaimer of Warranty. Licensor warrants that the
   copyright in and to the Original Work and the patent rights granted herein by
   Licensor are owned by the Licensor or are sublicensed to You under the terms
   of this License with the permission of the contributor(s) of those copyrights
   and patent rights. Except as expressly stated in the immediately proceeding
   sentence, the Original Work is provided under this License on an "AS IS"
   BASIS and WITHOUT WARRANTY, either express or implied, including, without
   limitation, the warranties of NON-INFRINGEMENT, MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY OF THE ORIGINAL
   WORK IS WITH YOU. This DISCLAIMER OF WARRANTY constitutes an essential part
   of this License. No license to Original Work is granted hereunder except
   under this disclaimer.

   (( 8) ))??
   Limitation of Liability. Under no circumstances and under no legal theory,
   whether in tort (including negligence), contract, or otherwise, shall the
   Licensor be liable to any person for any direct, indirect, special,
   incidental, or consequential damages of any character arising as a result of
   this License or the use of the Original Work including, without limitation,
   damages for loss of goodwill, work stoppage, computer failure or malfunction,
   or any and all other commercial damages or losses. This limitation of
   liability shall not apply to liability for death or personal injury resulting
   from Licensor's negligence to the extent applicable law prohibits such
   limitation. Some jurisdictions do not allow the exclusion or limitation of
   incidental or consequential damages, so this exclusion and limitation may not
   apply to You.

   (( 9) ))??
   Acceptance and Termination. If You distribute copies of the Original Work or
   a Derivative Work, You must make a reasonable effort under the circumstances
   to obtain the express assent of recipients to the terms of this License.
   Nothing else but this License (or another written agreement between Licensor
   and You) grants You permission to create Derivative Works based upon the
   Original Work or to exercise any of the rights granted in Section 1 herein,
   and any attempt to do so except under the terms of this License (or another
   written agreement between Licensor and You) is expressly prohibited by U.S.
   copyright law, the equivalent laws of other countries, and by international
   treaty. Therefore, by exercising any of the rights granted to You in Section
   1 herein, You indicate Your acceptance of this License and all of its terms
   and conditions. This License shall terminate immediately and you may no
   longer exercise any of the rights granted to You by this License upon Your
   failure to honor the proviso in Section 1(c) herein.

   (( 10) ))??
   Termination for Patent Action. This License shall terminate automatically and
   You may no longer exercise any of the rights granted to You by this License
   as of the date You commence an action, including a cross-claim or
   counterclaim, against Licensor or any licensee alleging that the Original
   Work infringes a patent. This termination provision shall not apply for an
   action alleging patent infringement by combinations of the Original Work with
   other software or hardware.

   (( 11) ))??
   Jurisdiction, Venue and Governing Law. Any action or suit relating to this
   License may be brought only in the courts of a jurisdiction wherein the
   Licensor resides or in which Licensor conducts its primary business, and
   under the laws of that jurisdiction excluding its conflict-of-law provisions.
   The application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any use of the Original
   Work outside the scope of this License or after its termination shall be
   subject to the requirements and penalties of the U.S. Copyright Act, 17
   U.S.C. § 101 et seq., the equivalent laws of other countries, and
   international treaty. This section shall survive the termination of this
   License.

   (( 12) ))??
   Attorneys Fees. In any action to enforce the terms of this License or seeking
   damages relating thereto, the prevailing party shall be entitled to recover
   its costs and expenses, including, without limitation, reasonable attorneys'
   fees and costs incurred in connection with such action, including any appeal
   of such action. This section shall survive the termination of this License.

   (( 13) ))??
   Miscellaneous. This License represents the complete agreement concerning the
   subject matter hereof. If any provision of this License is held to be
   unenforceable, such provision shall be reformed only to the extent necessary
   to make it enforceable.

   (( 14) ))??
   Definition of "You" in This License. "You" throughout this License, whether
   in upper or lower case, means an individual or a legal entity exercising
   rights under, and complying with all of the terms of, this License. For legal
   entities, "You" includes any entity that controls, is controlled by, or is
   under common control with you. For purposes of this definition, "control"
   means (i) the power, direct or indirect, to cause the direction or management
   of such entity, whether by contract or otherwise, or (ii) ownership of fifty
   percent (50%) or more of the outstanding shares, or (iii) beneficial
   ownership of such entity.

   (( 15) ))??
   Right to Use. You may use the Original Work in all ways not otherwise
   restricted or conditioned by this License or by law, and Licensor promises
   not to interfere with or be responsible for such uses by You.

This license is Copyright (C) 2003-2004 Lawrence E. Rosen. All rights reserved.
Permission is hereby granted to copy and distribute this license without
modification. This license may not be modified without the express written
permission of its copyright owner.
`
const license_OSL_3_0_lre = `//**
Open Software License 3.0
https://spdx.org/licenses/OSL-3.0.json
https://web.archive.org/web/20120101081418/http://rosenlaw.com:80/OSL3.0.htm
https://opensource.org/licenses/OSL-3.0
**//

((
	((The))??
	Open Software License
	((v.))??
	3.0
	(OSL-3.0)
))??

This Open Software License (the "License") applies to any original work of
authorship (the "Original Work") whose owner (the "Licensor") has placed the
following licensing notice adjacent to the copyright notice for the Original
Work:

Licensed under the Open Software License version 3.0

   (( 1) ))??
   Grant of Copyright License. Licensor grants You a worldwide, royalty-free,
   non-exclusive, sublicensable license, for the duration of the copyright, to
   do the following:

      (( a) || 1 ))??
      to reproduce the Original Work in copies, either alone or as part of a
      collective work;

      (( b) || 2 ))??
      to translate, adapt, alter, transform, modify, or arrange the Original
      Work, thereby creating derivative works ("Derivative Works") based upon
      the Original Work;

      (( c) || 3 ))??
      to distribute or communicate copies of the Original Work and Derivative
      Works to the public, with the proviso that copies of Original Work or
      Derivative Works that You distribute or communicate shall be licensed
      under this Open Software License;

      (( d) || 4 ))??
      to perform the Original Work publicly; and

      (( e) || 5 ))??
      to display the Original Work publicly.

   (( 2) ))??
   Grant of Patent License. Licensor grants You a worldwide, royalty-free,
   non-exclusive, sublicensable license, under patent claims owned or controlled
   by the Licensor that are embodied in the Original Work as furnished by the
   Licensor, for the duration of the patents, to make, use, sell, offer for
   sale, have made, and import the Original Work and Derivative Works.

   (( 3) ))??
   Grant of Source Code License. The term "Source Code" means the preferred form
   of the Original Work for making modifications to it and all available
   documentation describing how to modify the Original Work. Licensor agrees to
   provide a machine-readable copy of the Source Code of the Original Work along
   with each copy of the Original Work that Licensor distributes. Licensor
   reserves the right to satisfy this obligation by placing a machine-readable
   copy of the Source Code in an information repository reasonably calculated to
   permit inexpensive and convenient access by You for as long as Licensor
   continues to distribute the Original Work.

   (( 4) ))??
   Exclusions From License Grant. Neither the names of Licensor, nor the names
   of any contributors to the Original Work, nor any of their trademarks or
   service marks, may be used to endorse or promote products derived from this
   Original Work without express prior permission of the Licensor. Except as
   expressly stated herein, nothing in this License grants any license to
   Licensor's trademarks, copyrights, patents, trade secrets or any other
   intellectual property. No patent license is granted to make, use, sell, offer
   for sale, have made, or import embodiments of any patent claims other than
   the licensed claims defined in Section 2. No license is granted to the
   trademarks of Licensor even if such marks are included in the Original Work.
   Nothing in this License shall be interpreted to prohibit Licensor from
   licensing under terms different from this License any Original Work that
   Licensor otherwise would have a right to license.

   (( 5) ))??
   External Deployment. The term "External Deployment" means the use,
   distribution, or communication of the Original Work or Derivative Works in
   any way such that the Original Work or Derivative Works may be used by anyone
   other than You, whether those works are distributed or communicated to those
   persons or made available as an application intended for use over a network.
   As an express condition for the grants of license hereunder, You must treat
   any External Deployment by You of the Original Work or a Derivative Work as a
   distribution under section 1(c).

   (( 6) ))??
   Attribution Rights. You must retain, in the Source Code of any Derivative
   Works that You create, all copyright, patent, or trademark notices from the
   Source Code of the Original Work, as well as any notices of licensing and any
   descriptive text identified therein as an "Attribution Notice." You must
   cause the Source Code for any Derivative Works that You create to carry a
   prominent Attribution Notice reasonably calculated to inform recipients that
   You have modified the Original Work.

   (( 7) ))??
   Warranty of Provenance and Disclaimer of Warranty. Licensor warrants that the
   copyright in and to the Original Work and the patent rights granted herein by
   Licensor are owned by the Licensor or are sublicensed to You under the terms
   of this License with the permission of the contributor(s) of those copyrights
   and patent rights. Except as expressly stated in the immediately preceding
   sentence, the Original Work is provided under this License on an "AS IS"
   BASIS and WITHOUT WARRANTY, either express or implied, including, without
   limitation, the warranties of non-infringement, merchantability or fitness
   for a particular purpose. THE ENTIRE RISK AS TO THE QUALITY OF THE ORIGINAL
   WORK IS WITH YOU. This DISCLAIMER OF WARRANTY constitutes an essential part
   of this License. No license to the Original Work is granted by this License
   except under this disclaimer.

   (( 8) ))??
   Limitation of Liability. Under no circumstances and under no legal theory,
   whether in tort (including negligence), contract, or otherwise, shall the
   Licensor be liable to anyone for any indirect, special, incidental, or
   consequential damages of any character arising as a result of this License or
   the use of the Original Work including, without limitation, damages for loss
   of goodwill, work stoppage, computer failure or malfunction, or any and all
   other commercial damages or losses. This limitation of liability shall not
   apply to the extent applicable law prohibits such limitation.

   (( 9) ))??
   Acceptance and Termination. If, at any time, You expressly assented to this
   License, that assent indicates your clear and irrevocable acceptance of this
   License and all of its terms and conditions. If You distribute or communicate
   copies of the Original Work or a Derivative Work, You must make a reasonable
   effort under the circumstances to obtain the express assent of recipients to
   the terms of this License. This License conditions your rights to undertake
   the activities listed in Section 1, including your right to create Derivative
   Works based upon the Original Work, and doing so without honoring these terms
   and conditions is prohibited by copyright law and international treaty.
   Nothing in this License is intended to affect copyright exceptions and
   limitations (including "fair use" or "fair dealing"). This License shall
   terminate immediately and You may no longer exercise any of the rights
   granted to You by this License upon your failure to honor the conditions in
   Section 1(c).

   (( 10) ))??
   Termination for Patent Action. This License shall terminate automatically and
   You may no longer exercise any of the rights granted to You by this License
   as of the date You commence an action, including a cross-claim or
   counterclaim, against Licensor or any licensee alleging that the Original
   Work infringes a patent. This termination provision shall not apply for an
   action alleging patent infringement by combinations of the Original Work with
   other software or hardware.

   (( 11) ))??
   Jurisdiction, Venue and Governing Law. Any action or suit relating to this
   License may be brought only in the courts of a jurisdiction wherein the
   Licensor resides or in which Licensor conducts its primary business, and
   under the laws of that jurisdiction excluding its conflict-of-law provisions.
   The application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any use of the Original
   Work outside the scope of this License or after its termination shall be
   subject to the requirements and penalties of copyright or patent law in the
   appropriate jurisdiction. This section shall survive the termination of this
   License.

   (( 12) ))??
   Attorneys' Fees. In any action to enforce the terms of this License or
   seeking damages relating thereto, the prevailing party shall be entitled to
   recover its costs and expenses, including, without limitation, reasonable
   attorneys' fees and costs incurred in connection with such action, including
   any appeal of such action. This section shall survive the termination of this
   License.

   (( 13) ))??
   Miscellaneous. If any provision of this License is held to be unenforceable,
   such provision shall be reformed only to the extent necessary to make it
   enforceable.

   (( 14) ))??
   Definition of "You" in This License. "You" throughout this License, whether
   in upper or lower case, means an individual or a legal entity exercising
   rights under, and complying with all of the terms of, this License. For legal
   entities, "You" includes any entity that controls, is controlled by, or is
   under common control with you. For purposes of this definition, "control"
   means (i) the power, direct or indirect, to cause the direction or management
   of such entity, whether by contract or otherwise, or (ii) ownership of fifty
   percent (50%) or more of the outstanding shares, or (iii) beneficial
   ownership of such entity.

   (( 15) ))??
   Right to Use. You may use the Original Work in all ways not otherwise
   restricted or conditioned by this License or by law, and Licensor promises
   not to interfere with or be responsible for such uses by You.

   (( 16) ))??
   Modification of This License. This License is Copyright (c) 2005 Lawrence
   Rosen. Permission is granted to copy, distribute, or communicate this License
   without modification. Nothing in this License permits You to modify this
   License as applied to the Original Work or to Derivative Works. However, You
   may modify the text of this License and copy, distribute or communicate your
   modified version (the "Modified License") and apply it to other original
   works of authorship subject to the following conditions: (i) You may not
   indicate in any way that your Modified License is the "Open Software License"
   or "OSL" and you may not use those names in the name of your Modified
   License; (ii) You must replace the notice specified in the first paragraph
   above with the notice "Licensed under <insert your license name here>" or
   with a notice of your own that is not confusingly similar to the notice in
   this License; and (iii) You may not claim that your original works are open
   source software unless your Modified License has been approved by Open Source
   Initiative (OSI) and You comply with its license review and certification
   process.
`
const license_OpenSSL_lre = `//**
OpenSSL License
https://spdx.org/licenses/OpenSSL.json
http://www.openssl.org/source/license.html
**//

(( OpenSSL License
(( Copyright __20__ ))??
))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   All advertising materials mentioning features or use of this software must
   display the following acknowledgment: "This product includes software
   developed by the OpenSSL Project for use in the OpenSSL Toolkit.
   (http:/www.openssl.org/)"

   (( 4. ))??
   The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to endorse
   or promote products derived from this software without prior written
   permission. For written permission, please contact openssl-core@openssl.org.

   (( 5. ))??
   Products derived from this software may not be called "OpenSSL" nor may
   "OpenSSL" appear in their names without prior written permission of the
   OpenSSL Project.

   (( 6. ))??
   Redistributions of any form whatsoever must retain the following
   acknowledgment: "This product includes software developed by the OpenSSL
   Project for use in the OpenSSL Toolkit (http:/www.openssl.org/)"

THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESSED OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE OpenSSL PROJECT OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

This product includes cryptographic software written by Eric Young
(eay@cryptsoft.com). This product includes software written by Tim Hudson
(tjh@cryptsoft.com).

((
(( Original SSLeay License ))??

Copyright __5__ Eric Young  (eay@cryptsoft.com)
All rights reserved.

This package is an SSL implementation written by Eric Young (eay@cryptsoft.com).
The implementation was written so as to conform with Netscapes SSL.

This library is free for commercial and non-commercial use as long as the
following conditions are aheared to. The following conditions apply to all code
found in this distribution, be it the RC4, RSA, lhash, DES, etc., code; not just
the SSL code. The SSL documentation included with this distribution is covered
by the same copyright terms except that the holder is Tim Hudson
(tjh@cryptsoft.com).

Copyright remains Eric Young's, and as such any Copyright notices in the code
are not to be removed. If this package is used in a product, Eric Young should
be given attribution as the author of the parts of the library used. This can be
in the form of a textual message at program startup or in documentation (online
or textual) provided with the package.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain the copyright notice, this list of
   conditions and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   All advertising materials mentioning features or use of this software must
   display the following acknowledgement:

   "This product includes cryptographic software written by Eric Young
   (eay@cryptsoft.com)"

   The word 'cryptographic' can be left out if the rouines from the library
   being used are not cryptographic related :-).

   (( 4. ))??
   If you include any Windows specific code (or a derivative thereof) from the
   apps directory (application code) you must include an acknowledgement: "This
   product includes software written by Tim Hudson (tjh@cryptsoft.com)"

THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
))??

((
The licence and distribution terms for any publically available version or
derivative of this code cannot be changed. i.e. this code cannot simply be
copied and put under another distribution licence [including the GNU Public
Licence.]
))??
`
const license_PDDL_1_0_lre = `//**
ODC Public Domain Dedication & License 1.0
https://spdx.org/licenses/PDDL-1.0.json
http://opendatacommons.org/licenses/pddl/1.0/
**//

(( Open Data Commons - Public Domain Dedication & License (PDDL) ))??

Preamble

The Open Data Commons - Public Domain Dedication & Licence is a document
intended to allow you to freely share, modify, and use this work for any purpose
and without any restrictions. This licence is intended for use on databases or
their contents ("data"), either together or individually.

Many databases are covered by copyright. Some jurisdictions, mainly in Europe,
have specific special rights that cover databases called the "sui generis"
database right. Both of these sets of rights, as well as other legal rights used
to protect databases and data, can create uncertainty or practical difficulty
for those wishing to share databases and their underlying data but retain a
limited amount of rights under a "some rights reserved" approach to licensing as
outlined in the Science Commons Protocol for Implementing Open Access Data. As a
result, this waiver and licence tries to the fullest extent possible to
eliminate or fully license any rights that cover this database and data. Any
Community Norms or similar statements of use of the database or data do not form
a part of this document, and do not act as a contract for access or other terms
of use for the database or data.

The position of the recipient of the work

Because this document places the database and its contents in or as close as
possible within the public domain, there are no restrictions or requirements
placed on the recipient by this document. Recipients may use this work
commercially, use technical protection measures, combine this data or database
with other databases or data, and share their changes and additions or keep them
secret. It is not a requirement that recipients provide further users with a
copy of this licence or attribute the original creator of the data or database
as a source. The goal is to eliminate restrictions held by the original creator
of the data and database on the use of it by others.

The position of the dedicator of the work

Copyright law, as with most other law under the banner of "intellectual
property", is inherently national law. This means that there exists several
differences in how copyright and other IP rights can be relinquished, waived or
licensed in the many legal jurisdictions of the world. This is despite much
harmonisation of minimum levels of protection. The internet and other
communication technologies span these many disparate legal jurisdictions and
thus pose special difficulties for a document relinquishing and waiving
intellectual property rights, including copyright and database rights, for use
by the global community. Because of this feature of intellectual property law,
this document first relinquishes the rights and waives the relevant rights and
claims. It then goes on to license these same rights for jurisdictions or areas
of law that may make it difficult to relinquish or waive rights or claims.

The purpose of this document is to enable rightsholders to place their work into
the public domain. Unlike licences for free and open source software, free
cultural works, or open content licences, rightsholders will not be able to
"dual license" their work by releasing the same work under different licences.
This is because they have allowed anyone to use the work in whatever way they
choose. Rightsholders therefore can't re-license it under copyright or database
rights on different terms because they have nothing left to license. Doing so
creates truly accessible data to build rich applications and advance the
progress of science and the arts.

This document can cover either or both of the database and its contents (the
data). Because databases can have a wide variety of content - not just factual
data - rightsholders should use the Open Data Commons - Public Domain Dedication
& Licence for an entire database and its contents only if everything can be
placed under the terms of this document. Because even factual data can sometimes
have intellectual property rights, rightsholders should use this licence to
cover both the database and its factual data when making material available
under this document; even if it is likely that the data would not be covered by
copyright or database rights.

Rightsholders can also use this document to cover any copyright or database
rights claims over only a database, and leave the contents to be covered by
other licences or documents. They can do this because this document refers to
the "Work", which can be either - or both - the database and its contents. As a
result, rightsholders need to clearly state what they are dedicating under this
document when they dedicate it.

Just like any licence or other document dealing with intellectual property,
rightsholders should be aware that one can only license what one owns. Please
ensure that the rights have been cleared to make this material available under
this document.

This document permanently and irrevocably makes the Work available to the public
for any use of any kind, and it should not be used unless the rightsholder is
prepared for this to happen.

Part I: Introduction

The Rightsholder (the Person holding rights or claims over the Work) agrees as
follows:

   (( 1.0 ))??
   Definitions of Capitalised Words

   "Copyright" - Includes rights under copyright and under neighbouring rights
   and similarly related sets of rights under the law of the relevant
   jurisdiction under Section 6.4.

   "Data" - The contents of the Database, which includes the information,
   independent works, or other material collected into the Database offered
   under the terms of this Document.

   "Database" - A collection of Data arranged in a systematic or methodical way
   and individually accessible by electronic or other means offered under the
   terms of this Document.

   "Database Right" - Means rights over Data resulting from the Chapter III
   ("sui generis") rights in the Database Directive (Directive 96/9/EC of the
   European Parliament and of the Council of 11 March 1996 on the legal
   protection of databases) and any future updates as well as any similar rights
   available in the relevant jurisdiction under Section 6.4.

   "Document" - means this relinquishment and waiver of rights and claims and
   back up licence agreement.

   "Person" - Means a natural or legal person or a body of persons corporate or
   incorporate.

   "Use" - As a verb, means doing any act that is restricted by Copyright or
   Database Rights whether in the original medium or any other; and includes
   modifying the Work as may be technically necessary to use it in a different
   mode or format. This includes the right to sublicense the Work.

   "Work" - Means either or both of the Database and Data offered under the
   terms of this Document.

   "You" - the Person acquiring rights under the licence elements of this
   Document.

   Words in the singular include the plural and vice versa.

   (( 2.0 ))??
   What this document covers

   (( 2.1. ))??
   Legal effect of this Document. This Document is:

      (( a. ))??
      A dedication to the public domain and waiver of Copyright and Database
      Rights over the Work; and

      (( b. ))??
      A licence of Copyright and Database Rights over the Work in jurisdictions
      that do not allow for relinquishment or waiver.

   (( 2.2. ))??
   Legal rights covered.

      (( a. ))??
      Copyright. Any copyright or neighbouring rights in the Work. Copyright law
      varies between jurisdictions, but is likely to cover: the Database model
      or schema, which is the structure, arrangement, and organisation of the
      Database, and can also include the Database tables and table indexes; the
      data entry and output sheets; and the Field names of Data stored in the
      Database. Copyright may also cover the Data depending on the jurisdiction
      and type of Data; and

      (( b. ))??
      Database Rights. Database Rights only extend to the extraction and
      re-utilisation of the whole or a substantial part of the Data. Database
      Rights can apply even when there is no copyright over the Database.
      Database Rights can also apply when the Data is removed from the Database
      and is selected and arranged in a way that would not infringe any
      applicable copyright.

   (( 2.2 ))??
   Rights not covered.

      (( a. ))??
      This Document does not apply to computer programs used in the making or
      operation of the Database;

      (( b. ))??
      This Document does not cover any patents over the Data or the Database.
      Please see Section 4.2 later in this Document for further details; and

      (( c. ))??
      This Document does not cover any trade marks associated with the Database.
      Please see Section 4.3 later in this Document for further details.

      Users of this Database are cautioned that they may have to clear other
      rights or consult other licences.

   (( 2.3 ))??
   Facts are free. The Rightsholder takes the position that factual information
   is not covered by Copyright. This Document however covers the Work in
   jurisdictions that may protect the factual information in the Work by
   Copyright, and to cover any information protected by Copyright that is
   contained in the Work.

Part II: Dedication to the public domain

   (( 3.0 ))??
   Dedication, waiver, and licence of Copyright and Database Rights

   (( 3.1 ))??
   Dedication of Copyright and Database Rights to the public domain. The
   Rightsholder by using this Document, dedicates the Work to the public domain
   for the benefit of the public and relinquishes all rights in Copyright and
   Database Rights over the Work.

      (( a. ))??
      The Rightsholder realises that once these rights are relinquished, that
      the Rightsholder has no further rights in Copyright and Database Rights
      over the Work, and that the Work is free and open for others to Use.

      (( b. ))??
      The Rightsholder intends for their relinquishment to cover all present and
      future rights in the Work under Copyright and Database Rights, whether
      they are vested or contingent rights, and that this relinquishment of
      rights covers all their heirs and successors.

      The above relinquishment of rights applies worldwide and includes media
      and formats now known or created in the future.

   (( 3.2 ))??
   Waiver of rights and claims in Copyright and Database Rights when Section 3.1
   dedication inapplicable. If the dedication in Section 3.1 does not apply in
   the relevant jurisdiction under Section 6.4, the Rightsholder waives any
   rights and claims that the Rightsholder may have or acquire in the future
   over the Work in:

      (( a. ))??
      Copyright; and

      (( b. ))??
      Database Rights.

      To the extent possible in the relevant jurisdiction, the above waiver of
      rights and claims applies worldwide and includes media and formats now
      known or created in the future. The Rightsholder agrees not to assert the
      above rights and waives the right to enforce them over the Work.

   (( 3.3 ))??
   Licence of Copyright and Database Rights when Sections 3.1 and 3.2
   inapplicable. If the dedication and waiver in Sections 3.1 and 3.2 does not
   apply in the relevant jurisdiction under Section 6.4, the Rightsholder and
   You agree as follows:

      (( a. ))??
      The Licensor grants to You a worldwide, royalty-free, non-exclusive,
      licence to Use the Work for the duration of any applicable Copyright and
      Database Rights. These rights explicitly include commercial use, and do
      not exclude any field of endeavour. To the extent possible in the relevant
      jurisdiction, these rights may be exercised in all media and formats
      whether now known or created in the future.

   (( 3.4 ))??
   Moral rights. This section covers moral rights, including the right to be
   identified as the author of the Work or to object to treatment that would
   otherwise prejudice the author's honour and reputation, or any other
   derogatory treatment:

      (( a. ))??
      For jurisdictions allowing waiver of moral rights, Licensor waives all
      moral rights that Licensor may have in the Work to the fullest extent
      possible by the law of the relevant jurisdiction under Section 6.4;

      (( b. ))??
      If waiver of moral rights under Section 3.4 a in the relevant jurisdiction
      is not possible, Licensor agrees not to assert any moral rights over the
      Work and waives all claims in moral rights to the fullest extent possible
      by the law of the relevant jurisdiction under Section 6.4; and

      (( c. ))??
      For jurisdictions not allowing waiver or an agreement not to assert moral
      rights under Section 3.4 a and b, the author may retain their moral rights
      over the copyrighted aspects of the Work.

      Please note that some jurisdictions do not allow for the waiver of moral
      rights, and so moral rights may still subsist over the work in some
      jurisdictions.

   (( 4.0 ))??
   Relationship to other rights

   (( 4.1 ))??
   No other contractual conditions. The Rightsholder makes this Work available
   to You without any other contractual obligations, either express or implied.
   Any Community Norms statement associated with the Work is not a contract and
   does not form part of this Document.

   (( 4.2 ))??
   Relationship to patents. This Document does not grant You a licence for any
   patents that the Rightsholder may own. Users of this Database are cautioned
   that they may have to clear other rights or consult other licences.

   (( 4.3 ))??
   Relationship to trade marks. This Document does not grant You a licence for
   any trade marks that the Rightsholder may own or that the Rightsholder may
   use to cover the Work. Users of this Database are cautioned that they may
   have to clear other rights or consult other licences.

Part III: General provisions

   (( 5.0 ))??
   Warranties, disclaimer, and limitation of liability

   (( 5.1 ))??
   The Work is provided by the Rightsholder "as is" and without any warranty of
   any kind, either express or implied, whether of title, of accuracy or
   completeness, of the presence of absence of errors, of fitness for purpose,
   or otherwise. Some jurisdictions do not allow the exclusion of implied
   warranties, so this exclusion may not apply to You.

   (( 5.2 ))??
   Subject to any liability that may not be excluded or limited by law, the
   Rightsholder is not liable for, and expressly excludes, all liability for
   loss or damage however and whenever caused to anyone by any use under this
   Document, whether by You or by anyone else, and whether caused by any fault
   on the part of the Rightsholder or not. This exclusion of liability includes,
   but is not limited to, any special, incidental, consequential, punitive, or
   exemplary damages. This exclusion applies even if the Rightsholder has been
   advised of the possibility of such damages.

   (( 5.3 ))??
   If liability may not be excluded by law, it is limited to actual and direct
   financial loss to the extent it is caused by proved negligence on the part of
   the Rightsholder.

   (( 6.0 ))??
   General

   (( 6.1 ))??
   If any provision of this Document is held to be invalid or unenforceable,
   that must not affect the cvalidity or enforceability of the remainder of the
   terms of this Document.

   (( 6.2 ))??
   This Document is the entire agreement between the parties with respect to the
   Work covered here. It replaces any earlier understandings, agreements or
   representations with respect to the Work not specified here.

   (( 6.3 ))??
   This Document does not affect any rights that You or anyone else may
   independently have under any applicable law to make any use of this Work,
   including (for jurisdictions where this Document is a licence) fair dealing,
   fair use, database exceptions, or any other legally recognised limitation or
   exception to infringement of copyright or other applicable laws.

   (( 6.4 ))??
   This Document takes effect in the relevant jurisdiction in which the Document
   terms are sought to be enforced. If the rights waived or granted under
   applicable law in the relevant jurisdiction includes additional rights not
   waived or granted under this Document, these additional rights are included
   in this Document in order to meet the intent of this Document.
`
const license_PHP_3_0_lre = `//**
PHP License v3.0
https://spdx.org/licenses/PHP-3.0.json
http://www.php.net/license/3_0.txt
https://opensource.org/licenses/PHP-3.0
**//

(( The PHP License, version 3.0
(( Copyright __20__ ))??
))??

Redistribution and use in source and binary forms, with or without modification,
is permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The name "PHP" must not be used to endorse or promote products derived from
   this software without prior written permission. For written permission,
   please contact group@php.net.

   (( 4. ))??
   Products derived from this software may not be called "PHP", nor may "PHP"
   appear in their name, without prior written permission from group@php.net.
   You may indicate that your software works in conjunction with PHP by saying
   "Foo for PHP" instead of calling it "PHP Foo" or "phpfoo"

   (( 5. ))??
   The PHP Group may publish revised and/or new versions of the license from
   time to time. Each version will be given a distinguishing version number.
   Once covered code has been published under a particular version of the
   license, you may always continue to use it under the terms of that version.
   You may also choose to use such covered code under the terms of any
   subsequent version of the license published by the PHP Group. No one other
   than the PHP Group has the right to modify the terms applicable to covered
   code created under this License.

   (( 6. ))??
   Redistributions of any form whatsoever must retain the following
   acknowledgment: "This product includes PHP, freely available from
   <http:/www.php.net/>".

THIS SOFTWARE IS PROVIDED BY THE PHP DEVELOPMENT TEAM ` + "`" + `` + "`" + `AS IS'' AND ANY
EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE PHP DEVELOPMENT TEAM OR ITS CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

((
This software consists of voluntary contributions made by many individuals on
behalf of the PHP Group.

The PHP Group can be contacted via Email at group@php.net.

For more information on the PHP Group and the PHP project, please see
<http:/www.php.net>.

This product includes the Zend Engine, freely available at <http:/www.zend.com>.
))??
`
const license_PHP_3_01_lre = `//**
PHP License v3.01
https://spdx.org/licenses/PHP-3.01.json
http://www.php.net/license/3_01.txt
**//

(( The PHP License, version 3.01
(( Copyright __20__ ))??
))??

Redistribution and use in source and binary forms, with or without modification,
is permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The name "PHP" must not be used to endorse or promote products derived from
   this software without prior written permission. For written permission,
   please contact group@php.net.

   (( 4. ))??
   Products derived from this software may not be called "PHP", nor may "PHP"
   appear in their name, without prior written permission from group@php.net.
   You may indicate that your software works in conjunction with PHP by saying
   "Foo for PHP" instead of calling it "PHP Foo" or "phpfoo"

   (( 5. ))??
   The PHP Group may publish revised and/or new versions of the license from
   time to time. Each version will be given a distinguishing version number.
   Once covered code has been published under a particular version of the
   license, you may always continue to use it under the terms of that version.
   You may also choose to use such covered code under the terms of any
   subsequent version of the license published by the PHP Group. No one other
   than the PHP Group has the right to modify the terms applicable to covered
   code created under this License.

   (( 6. ))??
   Redistributions of any form whatsoever must retain the following
   acknowledgment: "This product includes PHP software, freely available from
   <http:/www.php.net/software/>".

THIS SOFTWARE IS PROVIDED BY THE PHP DEVELOPMENT TEAM ` + "`" + `` + "`" + `AS IS'' AND ANY
EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE PHP DEVELOPMENT TEAM OR ITS CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

((
This software consists of voluntary contributions made by many individuals on
behalf of the PHP Group.

The PHP Group can be contacted via Email at group@php.net.

For more information on the PHP Group and the PHP project, please see
<http:/www.php.net>.

PHP includes the Zend Engine, freely available at <http:/www.zend.com>.
))??
`
const license_PSF_2_0_lre = `//**
Python Software Foundation License 2.0
https://spdx.org/licenses/PSF-2.0.json
https://opensource.org/licenses/Python-2.0
**//

(( PYTHON SOFTWARE FOUNDATION LICENSE VERSION 2 ))??

   (( 1. ))??
   This LICENSE AGREEMENT is between the Python Software Foundation ("PSF"), and
   the Individual or Organization ("Licensee") accessing and otherwise using
   this software ("Python") in source or binary form and its associated
   documentation.

   (( 2. ))??
   Subject to the terms and conditions of this License Agreement, PSF hereby
   grants Licensee a nonexclusive, royalty-free, world-wide license to
   reproduce, analyze, test, perform and/or display publicly, prepare derivative
   works, distribute, and otherwise use Python alone or in any derivative
   version, provided, however, that PSF's License Agreement and PSF's notice of
   copyright, i.e., "Copyright (c)
   //** 2001, 2002, 2003, 2004, 2005, 2006 **//
   __6__
   Python Software Foundation; All Rights Reserved" are retained in Python alone
   or in any derivative version prepared by Licensee.

   (( 3. ))??
   In the event Licensee prepares a derivative work that is based on or
   incorporates Python or any part thereof, and wants to make the derivative
   work available to others as provided herein, then Licensee hereby agrees to
   include in any such work a brief summary of the changes made to Python.

   (( 4. ))??
   PSF is making Python available to Licensee on an "AS IS" basis. PSF MAKES NO
   REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED. BY WAY OF EXAMPLE, BUT NOT
   LIMITATION, PSF MAKES NO AND DISCLAIMS ANY REPRESENTATION OR WARRANTY OF
   MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF
   PYTHON WILL NOT INFRINGE ANY THIRD PARTY RIGHTS.

   (( 5. ))??
   PSF SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF PYTHON FOR ANY
   INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS A RESULT OF
   MODIFYING, DISTRIBUTING, OR OTHERWISE USING PYTHON, OR ANY DERIVATIVE
   THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

   (( 6. ))??
   This License Agreement will automatically terminate upon a material breach of
   its terms and conditions.

   (( 7. ))??
   Nothing in this License Agreement shall be deemed to create any relationship
   of agency, partnership, or joint venture between PSF and Licensee. This
   License Agreement does not grant permission to use PSF trademarks or trade
   name in a trademark sense to endorse or promote products or services of
   Licensee, or any third party.

   (( 8. ))??
   By copying, installing or otherwise using Python, Licensee agrees to be bound
   by the terms and conditions of this License Agreement.
`
const license_Parity_6_0_0_lre = `//**
The Parity Public License 6.0.0
https://spdx.org/licenses/Parity-6.0.0.json
https://paritylicense.com/versions/6.0.0.html
**//

(( The Parity Public License 6.0.0
	((
		((Copyright || Contributor))
		__20__
	))??
))??

Source Code: __20__

This license lets you use and share this software for free, as long as you
contribute software you make with it. Specifically:

If you follow the rules below, you may do everything with this software that
would otherwise infringe either the contributor's copyright in it, any patent
claim the contributor can license, or both.

   (( 1. ))??
   Contribute changes and additions you make to this software.

   (( 2. ))??
   If you combine this software with other software, contribute that other
   software.

   (( 3. ))??
   Contribute software you develop, deploy, monitor, or run with this software.

   (( 4. ))??
   Ensure everyone who gets a copy of this software from you, in source code or
   any other form, gets the text of this license and the contributor and source
   code lines above.

   (( 5. ))??
   Do not make any legal claim against anyone accusing this software, with or
   without changes, alone or with other software, of infringing any patent
   claim.

To contribute software, publish all its source code, in the preferred form for
making changes, through a freely accessible distribution system widely used for
similar source code, and license contributions not already licensed to the
public on terms as permissive as this license accordingly.

You are excused for unknowingly breaking 1, 2, or 3 if you contribute as
required, or stop doing anything requiring this license, within 30 days of
learning you broke the rule.

**As far as the law allows, this software comes as is, without any warranty, and
the contributor will not be liable to anyone for any damages related to this
software or this license, for any kind of legal claim.**
`
const license_Parity_7_0_0_lre = `//**
The Parity Public License 7.0.0
https://spdx.org/licenses/Parity-7.0.0.json
https://paritylicense.com/versions/7.0.0.html
**//

(( The Parity Public License 7.0.0
	((
		((Copyright || Contributor))
		__20__
	))??
))??

Source Code: __20__

Purpose

This license allows you to use and share this software for free, but you have to
share software that builds on it alike.

Agreement

In order to receive this license, you have to agree to its rules. Those rules
are both obligations under that agreement and conditions to your license. Don't
do anything with this software that triggers a rule you can't or won't follow.

Notices

Make sure everyone who gets a copy of any part of this software from you, with
or without changes, also gets the text of this license and the contributor and
source code lines above.

Copyleft

Contribute
software you develop, operate, or analyze with this software, including changes
or additions to this software. When in doubt,
contribute.

Prototypes

You don't have to
contribute
any change, addition, or other software that meets all these criteria:

   (( 1. ))??
   You don't use it for more than thirty days.

   (( 2. ))??
   You don't share it outside the team developing it, other than for
   non-production user testing.

   (( 3. ))??
   You don't develop, operate, or analyze other software with it for anyone
   outside the team developing it.

Reverse Engineering

You may use this software to operate and analyze software you can't
contribute
in order to develop alternatives you can and do
contribute.

Contribute

To
contribute
software:

   (( 1. ))??
   Publish all source code for the software in the preferred form for making
   changes through a freely accessible distribution system widely used for
   similar source code so the contributor and others can find and copy it.

   (( 2. ))??
   Make sure every part of the source code is available under this license or
   another license that allows everything this license does, such as
   //** [the Blue Oak Model License 1.0.0](https://blueoakcouncil.org/license/1.0.0), [the Apache License 2.0](https://www.apache.org/licenses/LICENSE-2.0.html), [the MIT license](https://spdx.org/licenses/MIT.html), or [the two-clause BSD license](https://spdx.org/licenses/BSD-2-Clause.html). **//
   __52__

   (( 3. ))??
   Take these steps within thirty days.

   (( 4. ))??
   Note that this license does not
   allow you to change the license terms for this software. You must follow notices.

Excuse

You're excused for unknowingly breaking
copyleft
if you
contribute
as required, or stop doing anything requiring this license, within thirty days
of learning you broke the rule. You're excused for unknowingly breaking
notices
if you take all practical steps to comply within thirty days of learning you
broke the rule.

Defense

Don't make any legal claim against anyone accusing this software, with or
without changes, alone or with other technology, of infringing any patent.

Copyright

The contributor licenses you to do everything with this software that would
otherwise infringe their copyright in it.

Patent

The contributor licenses you to do everything with this software that would
otherwise infringe any patents they can license or become able to license.

Reliability

The contributor can't revoke this license.

No Liability

***As far as the law allows, this software comes as is, without any warranty or
condition, and the contributor won't be liable to anyone for any damages related
to this software or this license, under any kind of legal claim.***
`
const license_Plexus_lre = `//**
Plexus Classworlds License
https://spdx.org/licenses/Plexus.json
https://fedoraproject.org/wiki/Licensing/Plexus_Classworlds_License
**//

//** Copyright **//

Redistribution and use of this software and associated documentation
("Software"), with or without modification, are permitted provided that the
following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain copyright statements and notices.
   Redistributions must also contain a copy of this document.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The name
   __5__
   must not be used to endorse or promote products derived from this Software
   without prior written permission of
   //** The Codehaus **//
   __5__
   . For written permission, please contact
   //** bob@codehaus.org **//
   __5__

   (( 4. ))??
   Products derived from this Software may not be called
   __5__
   nor may
   __5__
   appear in their names without prior written permission of
   //** The Codehaus **//
   __5__
   .
   __5__
   is a registered trademark of
   //** The Codehaus **//
   __5__

   (( 5. ))??
   Due credit should be given to
   //** The Codehaus. (http://classworlds.codehaus.org/) **//
   __6__

THIS SOFTWARE IS PROVIDED BY
//** THE CODEHAUS **//
__5__
AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
//** THE CODEHAUS **//
__5__
OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
`
const license_PolyForm_Noncommercial_1_0_0_lre = `//**
PolyForm Noncommercial License 1.0.0
https://spdx.org/licenses/PolyForm-Noncommercial-1.0.0.json
https://polyformproject.org/licenses/noncommercial/1.0.0
**//

(( # PolyForm Noncommercial License 1.0.0 ))??

<https:/polyformproject.org/licenses/noncommercial/1.0.0>

## Acceptance

In order to get any license under these terms, you must agree to them as both
strict obligations and conditions to all your licenses.

## Copyright License

The licensor grants you a copyright license for the software to do everything
you might do with the software that would otherwise infringe the licensor's
copyright in it for any permitted purpose. However, you may only distribute the
software according to [Distribution License](#distribution-license) and make
changes or new works based on the software according to [Changes and New Works
License](#changes-and-new-works-license).

## Distribution License

The licensor grants you an additional copyright license to distribute copies of
the software. Your license to distribute covers distributing the software with
changes and new works permitted by [Changes and New Works
License](#changes-and-new-works-license).

## Notices

You must ensure that anyone who gets a copy of any part of the software from you
also gets a copy of these terms or the URL for them above, as well as copies of
any plain-text lines beginning with ` + "`" + `Required Notice:` + "`" + ` that the licensor
provided with the software. For example:

> Required Notice: Copyright Yoyodyne, Inc. (http:/example.com)

## Changes and New Works License

The licensor grants you an additional copyright license to make changes and new
works based on the software for any permitted purpose.

## Patent License

The licensor grants you a patent license for the software that covers patent
claims the licensor can license, or becomes able to license, that you would
infringe by using the software.

## Noncommercial Purposes

Any noncommercial purpose is a permitted purpose.

## Personal Uses

Personal use for research, experiment, and testing for the benefit of public
knowledge, personal study, private entertainment, hobby projects, amateur
pursuits, or religious observance, without any anticipated commercial
application, is use for a permitted purpose.

## Noncommercial Organizations

Use by any charitable organization, educational institution, public research
organization, public safety or health organization, environmental protection
organization, or government institution is use for a permitted purpose
regardless of the source of funding or obligations resulting from the funding.

## Fair Use

You may have "fair use" rights for the software under the law. These terms do
not limit them.

## No Other Rights

These terms do not allow you to sublicense or transfer any of your licenses to
anyone else, or prevent the licensor from granting licenses to anyone else.
These terms do not imply any other licenses.

## Patent Defense

If you make any written claim that the software infringes or contributes to
infringement of any patent, your patent license for the software granted under
these terms ends immediately. If your company makes such a claim, your patent
license ends immediately for work on behalf of your company.

## Violations

The first time you are notified in writing that you have violated any of these
terms, or done anything with the software not covered by your licenses, your
licenses can nonetheless continue if you come into full compliance with these
terms, and take practical steps to correct past violations, within 32 days of
receiving notice. Otherwise, all your licenses end immediately.

## No Liability

***As far as the law allows, the software comes as is, without any warranty or
condition, and the licensor will not be liable to you for any damages arising
out of these terms or the use or nature of the software, under any kind of legal
claim.***

## Definitions

The **licensor** is the individual or entity offering these terms, and the
**software** is the software the licensor makes available under these terms.

**You** refers to the individual or entity agreeing to these terms.

**Your company** is any legal entity, sole proprietorship, or other kind of
organization that you work for, plus all organizations that have control over,
are under the control of, or are under common control with that organization.
**Control** means ownership of substantially all the assets of an entity, or the
power to direct its management and policies by vote, contract, or otherwise.
Control can be direct or indirect.

**Your licenses** are all the licenses granted to you for the software under
these terms.

**Use** means anything you do with the software requiring one of your licenses.
`
const license_PolyForm_Small_Business_1_0_0_lre = `//**
PolyForm Small Business License 1.0.0
https://spdx.org/licenses/PolyForm-Small-Business-1.0.0.json
https://polyformproject.org/licenses/small-business/1.0.0
**//

(( # PolyForm Small Business License 1.0.0 ))??

<https:/polyformproject.org/licenses/small-business/1.0.0>

## Acceptance

In order to get any license under these terms, you must agree to them as both
strict obligations and conditions to all your licenses.

## Copyright License

The licensor grants you a copyright license for the software to do everything
you might do with the software that would otherwise infringe the licensor's
copyright in it for any permitted purpose. However, you may only distribute the
software according to [Distribution License](#distribution-license) and make
changes or new works based on the software according to [Changes and New Works
License](#changes-and-new-works-license).

## Distribution License

The licensor grants you an additional copyright license to distribute copies of
the software. Your license to distribute covers distributing the software with
changes and new works permitted by [Changes and New Works
License](#changes-and-new-works-license).

## Notices

You must ensure that anyone who gets a copy of any part of the software from you
also gets a copy of these terms or the URL for them above, as well as copies of
any plain-text lines beginning with ` + "`" + `Required Notice:` + "`" + ` that the licensor
provided with the software. For example:

> Required Notice: Copyright Yoyodyne, Inc. (http:/example.com)

## Changes and New Works License

The licensor grants you an additional copyright license to make changes and new
works based on the software for any permitted purpose.

## Patent License

The licensor grants you a patent license for the software that covers patent
claims the licensor can license, or becomes able to license, that you would
infringe by using the software.

## Fair Use

You may have "fair use" rights for the software under the law. These terms do
not limit them.

## Small Business

Use of the software for the benefit of your company is use for a permitted
purpose if your company has fewer than 100 total individuals working as
employees and independent contractors, and less than 1,000,000 USD (2019) total
revenue in the prior tax year. Adjust this revenue threshold for inflation
according to the United States Bureau of Labor Statistics' consumer price index
for all urban consumers, U.S. city average, for all items, not seasonally
adjusted, with 1982–1984=100 reference base.

## No Other Rights

These terms do not allow you to sublicense or transfer any of your licenses to
anyone else, or prevent the licensor from granting licenses to anyone else.
These terms do not imply any other licenses.

## Patent Defense

If you make any written claim that the software infringes or contributes to
infringement of any patent, your patent license for the software granted under
these terms ends immediately. If your company makes such a claim, your patent
license ends immediately for work on behalf of your company.

## Violations

The first time you are notified in writing that you have violated any of these
terms, or done anything with the software not covered by your licenses, your
licenses can nonetheless continue if you come into full compliance with these
terms, and take practical steps to correct past violations, within 32 days of
receiving notice. Otherwise, all your licenses end immediately.

## No Liability

***As far as the law allows, the software comes as is, without any warranty or
condition, and the licensor will not be liable to you for any damages arising
out of these terms or the use or nature of the software, under any kind of legal
claim.***

## Definitions

The **licensor** is the individual or entity offering these terms, and the
**software** is the software the licensor makes available under these terms.

**You** refers to the individual or entity agreeing to these terms.

**Your company** is any legal entity, sole proprietorship, or other kind of
organization that you work for, plus all organizations that have control over,
are under the control of, or are under common control with that organization.
**Control** means ownership of substantially all the assets of an entity, or the
power to direct its management and policies by vote, contract, or otherwise.
Control can be direct or indirect.

**Your licenses** are all the licenses granted to you for the software under
these terms.

**Use** means anything you do with the software requiring one of your licenses.
`
const license_PostgreSQL_lre = `//**
PostgreSQL License
https://spdx.org/licenses/PostgreSQL.json
http://www.postgresql.org/about/licence
https://opensource.org/licenses/PostgreSQL
**//

(( PostgreSQL Database Management System

(formerly known as Postgres, then as Postgres95)
(( Portions Copyright __20__ ))??
))??

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose, without fee, and without a written agreement is
hereby granted, provided that the above copyright notice and this paragraph and
the following two paragraphs appear in all copies.

IN NO EVENT SHALL
//** THE UNIVERSITY OF CALIFORNIA **//
__5__
BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE AND ITS DOCUMENTATION, EVEN IF
//** THE UNIVERSITY OF CALIFORNIA **//
__5__
HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

//** THE UNIVERSITY OF CALIFORNIA **//
__5__
SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND
//** THE UNIVERSITY OF CALIFORNIA **//
__5__
HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
MODIFICATIONS.
`
const license_Prosperity_3_0_0_lre = `//**
The Prosperity Public License 3.0.0
https://prosperitylicense.com/versions/3.0.0.html
**//

(( The Prosperity Public License 3.0.0
	((
		((Copyright || Contributor))
		__20__
	))??
))??

Source Code: __20__

Purpose

This license allows you to use and share this software for
noncommercial purposes for free and to try this software for
commercial purposes for thirty days.

Agreement

In order to receive this license, you have to agree to its rules. Those rules
are both obligations under that agreement and conditions to your license. Don't
do anything with this software that triggers a rule you can't or won't follow.

Notices

Make sure everyone who gets a copy of any part of this software from you, with
or without changes, also gets the text of this license and the contributor and
source code lines above.

Commercial Trial

Limit your use of this software for commercial purposes to a
thirty-day trial period. If you use this software for work, your
company gets one trial period for all personnel, not one trial per
person.

Contributions Back

Developing feedback, changes, or additions that you contribute back to
the contributor on the terms of a standardized public software license
such as the Blue Oak Model License 1.0.0, the Apache License 2.0, the
MIT license, or the two-clause BSD license doesn’t count as use for a
commercial purpose.

Personal Uses

Personal use for research, experiment, and testing for the benefit of
public knowledge, personal study, private entertainment, hobby
projects, amateur pursuits, or religious observance, without any
anticipated commercial application, doesn’t count as use for a
commercial purpose.

Noncommercial Organizations

Use by any charitable organization, educational institution, public
research organization, public safety or health organization,
environmental protection organization, or government institution
doesn’t count as use for a commercial purpose regardless of the source
of funding or obligations resulting from the funding.

Defense

Don't make any legal claim against anyone accusing this software, with or
without changes, alone or with other technology, of infringing any patent.

Copyright

The contributor licenses you to do everything with this software that would
otherwise infringe their copyright in it.

Patent

The contributor licenses you to do everything with this software that would
otherwise infringe any patents they can license or become able to license.

Reliability

The contributor can’t revoke this license.

Excuse

You’re excused for unknowingly breaking Notices if you take all
practical steps to comply within thirty days of learning you broke the
rule.

No Liability

***As far as the law allows, this software comes as is, without any warranty or
condition, and the contributor won't be liable to anyone for any damages related
to this software or this license, under any kind of legal claim.***
`
const license_Python_2_0_lre = `//**
Python License 2.0
https://spdx.org/licenses/Python-2.0.json
https://opensource.org/licenses/Python-2.0
**//

(( PYTHON SOFTWARE FOUNDATION LICENSE VERSION 2 ))??

   (( 1. ))??
   This LICENSE AGREEMENT is between the Python Software Foundation ("PSF"), and
   the Individual or Organization ("Licensee") accessing and otherwise using
   this software ("Python") in source or binary form and its associated
   documentation.

   (( 2. ))??
   Subject to the terms and conditions of this License Agreement, PSF hereby
   grants Licensee a nonexclusive, royalty-free, world-wide license to
   reproduce, analyze, test, perform and/or display publicly, prepare derivative
   works, distribute, and otherwise use Python alone or in any derivative
   version, provided, however, that PSF's License Agreement and PSF's notice of
   copyright, i.e., "Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006 Python
   Software Foundation; All Rights Reserved" are retained in Python alone or in
   any derivative version prepared by Licensee.

   (( 3. ))??
   In the event Licensee prepares a derivative work that is based on or
   incorporates Python or any part thereof, and wants to make the derivative
   work available to others as provided herein, then Licensee hereby agrees to
   include in any such work a brief summary of the changes made to Python.

   (( 4. ))??
   PSF is making Python available to Licensee on an "AS IS" basis. PSF MAKES NO
   REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED. BY WAY OF EXAMPLE, BUT NOT
   LIMITATION, PSF MAKES NO AND DISCLAIMS ANY REPRESENTATION OR WARRANTY OF
   MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF
   PYTHON WILL NOT INFRINGE ANY THIRD PARTY RIGHTS.

   (( 5. ))??
   PSF SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF PYTHON FOR ANY
   INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS A RESULT OF
   MODIFYING, DISTRIBUTING, OR OTHERWISE USING PYTHON, OR ANY DERIVATIVE
   THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

   (( 6. ))??
   This License Agreement will automatically terminate upon a material breach of
   its terms and conditions.

   (( 7. ))??
   Nothing in this License Agreement shall be deemed to create any relationship
   of agency, partnership, or joint venture between PSF and Licensee. This
   License Agreement does not grant permission to use PSF trademarks or trade
   name in a trademark sense to endorse or promote products or services of
   Licensee, or any third party.

   (( 8. ))??
   By copying, installing or otherwise using Python, Licensee agrees to be bound
   by the terms and conditions of this License Agreement.


(( BEOPEN.COM LICENSE AGREEMENT FOR PYTHON 2.0

BEOPEN PYTHON OPEN SOURCE LICENSE AGREEMENT VERSION 1 ))??

   (( 1. ))??
   This LICENSE AGREEMENT is between BeOpen.com ("BeOpen"), having an office at
   160 Saratoga Avenue, Santa Clara, CA 95051, and the Individual or
   Organization ("Licensee") accessing and otherwise using this software in
   source or binary form and its associated documentation ("the Software").

   (( 2. ))??
   Subject to the terms and conditions of this BeOpen Python License Agreement,
   BeOpen hereby grants Licensee a non-exclusive, royalty-free, world-wide
   license to reproduce, analyze, test, perform and/or display publicly, prepare
   derivative works, distribute, and otherwise use the Software alone or in any
   derivative version, provided, however, that the BeOpen Python License is
   retained in the Software, alone or in any derivative version prepared by
   Licensee.

   (( 3. ))??
   BeOpen is making the Software available to Licensee on an "AS IS" basis.
   BEOPEN MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED. BY WAY OF
   EXAMPLE, BUT NOT LIMITATION, BEOPEN MAKES NO AND DISCLAIMS ANY REPRESENTATION
   OR WARRANTY OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT
   THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY THIRD PARTY RIGHTS.

   (( 4. ))??
   BEOPEN SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF THE SOFTWARE FOR
   ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS A RESULT OF
   USING, MODIFYING OR DISTRIBUTING THE SOFTWARE, OR ANY DERIVATIVE THEREOF,
   EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

   (( 5. ))??
   This License Agreement will automatically terminate upon a material breach of
   its terms and conditions.

   (( 6. ))??
   This License Agreement shall be governed by and interpreted in all respects
   by the law of the State of California, excluding conflict of law provisions.
   Nothing in this License Agreement shall be deemed to create any relationship
   of agency, partnership, or joint venture between BeOpen and Licensee. This
   License Agreement does not grant permission to use BeOpen trademarks or trade
   names in a trademark sense to endorse or promote products or services of
   Licensee, or any third party. As an exception, the "BeOpen Python" logos
   available at http:/www.pythonlabs.com/logos.html may be used according to the
   permissions granted on that web page.

   (( 7. ))??
   By copying, installing or otherwise using the software, Licensee agrees to be
   bound by the terms and conditions of this License Agreement.


(( CNRI OPEN SOURCE LICENSE AGREEMENT (for Python 1.6b1) ))??

   (( IMPORTANT: PLEASE READ THE FOLLOWING AGREEMENT CAREFULLY.

BY CLICKING ON "ACCEPT" WHERE INDICATED BELOW, OR BY COPYING, INSTALLING OR
OTHERWISE USING PYTHON 1.6, beta 1 SOFTWARE, YOU ARE DEEMED TO HAVE AGREED TO
THE TERMS AND CONDITIONS OF THIS LICENSE AGREEMENT. ))??

   (( 1. ))??
   This LICENSE AGREEMENT is between the Corporation for National Research
   Initiatives, having an office at 1895 Preston White Drive, Reston, VA 20191
   ("CNRI"), and the Individual or Organization ("Licensee") accessing and
   otherwise using Python 1.6, beta 1 software in source or binary form and its
   associated documentation, as released at the www.python.org Internet site on
   August 4, 2000 ("Python 1.6b1").

   (( 2. ))??
   Subject to the terms and conditions of this License Agreement, CNRI hereby
   grants Licensee a non-exclusive, royalty-free, world-wide license to
   reproduce, analyze, test, perform and/or display publicly, prepare derivative
   works, distribute, and otherwise use Python 1.6b1 alone or in any derivative
   version, provided, however, that CNRIs License Agreement is retained in
   Python 1.6b1, alone or in any derivative version prepared by Licensee.

   Alternately, in lieu of CNRIs License Agreement, Licensee may substitute the
   following text (omitting the quotes): "Python 1.6, beta 1, is made available
   subject to the terms and conditions in CNRIs License Agreement. This
   Agreement may be located on the Internet using the following unique,
   persistent identifier (known as a handle): 1895.22/1011. This Agreement may
   also be obtained from a proxy server on the Internet using the
   URL:http:/hdl.handle.net/1895.22/1011".

   (( 3. ))??
   In the event Licensee prepares a derivative work that is based on or
   incorporates Python 1.6b1 or any part thereof, and wants to make the
   derivative work available to the public as provided herein, then Licensee
   hereby agrees to indicate in any such work the nature of the modifications
   made to Python 1.6b1.

   (( 4. ))??
   CNRI is making Python 1.6b1 available to Licensee on an "AS IS" basis. CNRI
   MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED. BY WAY OF
   EXAMPLE, BUT NOT LIMITATION, CNRI MAKES NO AND DISCLAIMS ANY REPRESENTATION
   OR WARRANTY OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT
   THE USE OF PYTHON 1.6b1 WILL NOT INFRINGE ANY THIRD PARTY RIGHTS.

   (( 5. ))??
   CNRI SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF THE SOFTWARE FOR
   ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS A RESULT OF
   USING, MODIFYING OR DISTRIBUTING PYTHON 1.6b1, OR ANY DERIVATIVE THEREOF,
   EVEN IF ADVISED OF THE POSSIBILITY THEREOF.

   (( 6. ))??
   This License Agreement will automatically terminate upon a material breach of
   its terms and conditions.

   (( 7. ))??
   This License Agreement shall be governed by and interpreted in all respects
   by the law of the State of Virginia, excluding conflict of law provisions.
   Nothing in this License Agreement shall be deemed to create any relationship
   of agency, partnership, or joint venture between CNRI and Licensee. This
   License Agreement does not grant permission to use CNRI trademarks or trade
   name in a trademark sense to endorse or promote products or services of
   Licensee, or any third party.

   (( 8. ))??
   By clicking on the "ACCEPT" button where indicated, or by copying, installing
   or otherwise using Python 1.6b1, Licensee agrees to be bound by the terms and
   conditions of this License Agreement.
   (( ACCEPT ))??

   (( CWI LICENSE AGREEMENT FOR PYTHON 0.9.0 THROUGH 1.2 ))??

   Copyright (c) 1991 - 1995, Stichting Mathematisch Centrum Amsterdam, The Netherlands. All rights reserved.

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted, provided that
the above copyright notice appear in all copies and that both that copyright
notice and this permission notice appear in supporting documentation, and that
the name of Stichting Mathematisch Centrum or CWI not be used in advertising or
publicity pertaining to distribution of the software without specific, written
prior permission.

STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE FOR ANY SPECIAL, INDIRECT
OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
`
const license_QPL_1_0_lre = `//**
Q Public License 1.0
https://spdx.org/licenses/QPL-1.0.json
http://doc.qt.nokia.com/3.3/license.html
https://opensource.org/licenses/QPL-1.0
**//

(( THE Q PUBLIC LICENSE version 1.0
(( Copyright __20__ ))??
))??

Everyone is permitted to copy and distribute this license document.

The intent of this license is to establish freedom to share and change the
software regulated by this license under the open source model.

This license applies to any software containing a notice placed by the copyright
holder saying that it may be distributed under the terms of the Q Public License
version 1.0. Such software is herein referred to as the Software. This license
covers modification and distribution of the Software, use of third-party
application programs based on the Software, and development of free software
which uses the Software.

Granted Rights

   (( 1. ))??
   You are granted the non-exclusive rights set forth in this license provided
   you agree to and comply with any and all conditions in this license. Whole or
   partial distribution of the Software, or software items that link with the
   Software, in any form signifies acceptance of this license.

   (( 2. ))??
   You may copy and distribute the Software in unmodified form provided that the
   entire package, including - but not restricted to - copyright, trademark
   notices and disclaimers, as released by the initial developer of the
   Software, is distributed.

   (( 3. ))??
   You may make modifications to the Software and distribute your modifications,
   in a form that is separate from the Software, such as patches. The following
   restrictions apply to modifications:

      (( a. ))??
      Modifications must not alter or remove any copyright notices in the
      Software.

      (( b. ))??
      When modifications to the Software are released under this license, a
      non-exclusive royalty-free right is granted to the initial developer of
      the Software to distribute your modification in future versions of the
      Software provided such versions remain available under these terms in
      addition to any other license(s) of the initial developer.

   (( 4. ))??
   You may distribute machine-executable forms of the Software or
   machine-executable forms of modified versions of the Software, provided that
   you meet these restrictions:

      (( a. ))??
      You must include this license document in the distribution.

      (( b. ))??
      You must ensure that all recipients of the machine-executable forms are
      also able to receive the complete machine-readable source code to the
      distributed Software, including all modifications, without any charge
      beyond the costs of data transfer, and place prominent notices in the
      distribution explaining this.

      (( c. ))??
      You must ensure that all modifications included in the machine-executable
      forms are available under the terms of this license.

   (( 5. ))??
   You may use the original or modified versions of the Software to compile,
   link and run application programs legally developed by you or by others.

   (( 6. ))??
   You may develop application programs, reusable components and other software
   items that link with the original or modified versions of the Software. These
   items, when distributed, are subject to the following requirements:

      (( a. ))??
      You must ensure that all recipients of machine-executable forms of these
      items are also able to receive and use the complete machine-readable
      source code to the items without any charge beyond the costs of data
      transfer.

      (( b. ))??
      You must explicitly license all recipients of your items to use and
      re-distribute original and modified versions of the items in both
      machine-executable and source code forms. The recipients must be able to
      do so without any charges whatsoever, and they must be able to
      re-distribute to anyone they choose.

      (( c. ))??
      If the items are not available to the general public, and the initial
      developer of the Software requests a copy of the items, then you must
      supply one.

Limitations of Liability

In no event shall the initial developers or copyright holders be liable for any
damages whatsoever, including - but not restricted to - lost revenue or profits
or other direct, indirect, special, incidental or consequential damages, even if
they have been advised of the possibility of such damages, except to the extent
invariable law, if any, provides otherwise.

No Warranty

The Software and this license document are provided AS IS with NO WARRANTY OF
ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE.

Choice of Law

This license is governed by the Laws of Norway. Disputes shall be settled by
Oslo City Court.
`
const license_Qhull_lre = `//**
Qhull License
https://spdx.org/licenses/Qhull.json
https://fedoraproject.org/wiki/Licensing/Qhull
**//

((Qhull
((Copyright __30__))??
))??

This software includes Qhull from The Geometry Center. Qhull is copyrighted as
noted above. Qhull is free software and may be obtained via http from
www.qhull.org. It may be freely copied, modified, and redistributed under the
following conditions:

   (( 1. ))??
   All copyright notices must remain intact in all files.

   (( 2. ))??
   A copy of this text file must be distributed along with any copies of Qhull
   that you redistribute; this includes copies that you have modified, or copies
   of programs or other software products that include Qhull.

   (( 3. ))??
   If you modify Qhull, you must include a notice giving the name of the person
   performing the modification, the date of modification, and the reason for
   such modification.

   (( 4. ))??
   When distributing modified versions of Qhull, or other software products that
   include Qhull, you must provide notice that the original source code may be
   obtained as noted above.

   (( 5. ))??
   There is no warranty or other guarantee of fitness for Qhull, it is provided
   solely "as is". Bug reports or fixes may be sent to qhull_bug@qhull.org; the
   authors may or may not act on them as they desire.
`
const license_RHeCos_1_1_lre = `//**
Red Hat eCos Public License v1.1
https://spdx.org/licenses/RHeCos-1.1.json
http://ecos.sourceware.org/old-license.html
**//

(( Red Hat eCos Public License v1.1 ))??

   (( 1. ))??
   DEFINITIONS

      (( 1.1. ))??
      "Contributor" means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      "Covered Code" means the Original Code or Modifications or the combination
      of the Original Code and Modifications, in each case including portions
      thereof.

      (( 1.4. ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      "Executable" means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      "Initial Developer" means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7. ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.8. ))??
      "License" means this document.

      (( 1.9. ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10. ))??
      "Original Code" means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      (( 1.11. ))??
      "Source Code" means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or a list of source code differential
      comparisons against either the Original Code or another well known,
      available Covered Code of the Contributor's choice. The Source Code can be
      in a compressed or archival form, provided the appropriate decompression
      or de-archiving software is widely available for no charge.

      (( 1.12. ))??
      "You" means an individual or a legal entity exercising rights under, and
      complying with all of the terms of, this License or a future version of
      this License issued under Section 6.1. For legal entities, "You" includes
      any entity which controls, is controlled by, or is under common control
      with You. For purposes of this definition, "control" means (a) the power,
      direct or indirect, to cause the direction or management of such entity,
      whether by contract or otherwise, or (b) ownership of fifty percent (50%)
      or more of the outstanding shares or beneficial ownership of such entity.

      (( 1.13. ))??
      "Red Hat Branded Code" is code that Red Hat distributes and/or permits
      others to distribute under different terms than the Red Hat eCos Public
      License. Red Hat's Branded Code may contain part or all of the Covered
      Code.

   (( 2. ))??
   SOURCE CODE LICENSE

      (( 2.1. ))??
      The Initial Developer Grant. The Initial Developer hereby grants You a
      world-wide, royalty-free, non-exclusive license, subject to third party
      intellectual property claims:

         (( (a) ))??
         to use, reproduce, modify, display, perform, sublicense and distribute
         the Original Code (or portions thereof) with or without Modifications,
         or as part of a Larger Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Initial
         Developer, to make, have made, use and sell ("Utilize") the Original
         Code (or portions thereof), but solely to the extent that any such
         patent is reasonably necessary to enable You to Utilize the Original
         Code (or portions thereof) and not to any greater extent that may be
         necessary to Utilize further Modifications or combinations.

      (( 2.2. ))??
      Contributor Grant. Each Contributor hereby grants You a world-wide,
      royalty-free, non-exclusive license, subject to third party intellectual
      property claims:

         (( (a) ))??
         to use, reproduce, modify, display, perform, sublicense and distribute
         the Modifications created by such Contributor (or portions thereof)
         either on an unmodified basis, with other Modifications, as Covered
         Code or as part of a Larger Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Contributor, to
         Utilize the Contributor Version (or portions thereof), but solely to
         the extent that any such patent is reasonably necessary to enable You
         to Utilize the Contributor Version (or portions thereof), and not to
         any greater extent that may be necessary to Utilize further
         Modifications or combinations.

   (( 3. ))??
   DISTRIBUTION OBLIGATIONS

      (( 3.1. ))??
      Application of License. The Modifications which You create or to which You
      contribute are governed by the terms of this License, including without
      limitation Section 2.2. The Source Code version of Covered Code may be
      distributed only under the terms of this License or a future version of
      this License released under Section 6.1, and You must include a copy of
      this License with every copy of the Source Code You distribute. You may
      not offer or impose any terms on any Source Code version that alters or
      restricts the applicable version of this License or the recipients' rights
      hereunder. However, You may include an additional document offering the
      additional rights described in Section 3.5.

      (( 3.2. ))??
      Availability of Source Code. Any Modification which You create or to which
      You contribute must be made available in Source Code form under the terms
      of this License via an accepted Electronic Distribution Mechanism to
      anyone to whom you made an Executable version available and to the Initial
      Developer; and if made available via Electronic Distribution Mechanism,
      must remain available for at least twelve (12) months after the date it
      initially became available, or at least six (6) months after a subsequent
      version of that particular Modification has been made available to such
      recipients. You are responsible for ensuring that the Source Code version
      remains available even if the Electronic Distribution Mechanism is
      maintained by a third party. You are responsible for notifying the Initial
      Developer of the Modification and the location of the Source if a contact
      means is provided. Red Hat will be acting as maintainer of the Source and
      may provide an Electronic Distribution mechanism for the Modification to
      be made available. You can contact Red Hat to make the Modification
      available and to notify the Initial Developer.
      (http:/sourceware.cygnus.com/ecos/)

      (( 3.3. ))??
      Description of Modifications. You must cause all Covered Code to which you
      contribute to contain a file documenting the changes You made to create
      that Covered Code and the date of any change. You must include a prominent
      statement that the Modification is derived, directly or indirectly, from
      Original Code provided by the Initial Developer and including the name of
      the Initial Developer in (a) the Source Code, and (b) in any notice in an
      Executable version or related documentation in which You describe the
      origin or ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters

         (( (a) ))??
         Third Party Claims. If You have knowledge that a party claims an
         intellectual property right in particular functionality or code (or its
         utilization under this License), you must include a text file with the
         source code distribution titled "LEGAL" which describes the claim and
         the party making the claim in sufficient detail that a recipient will
         know whom to contact. If you obtain such knowledge after You make Your
         Modification available as described in Section 3.2, You shall promptly
         modify the LEGAL file in all copies You make available thereafter and
         shall take other steps (such as notifying appropriate mailing lists or
         newsgroups) reasonably calculated to inform those who received the
         Covered Code that new knowledge has been obtained.

         (( (b) ))??
         Contributor APIs. If Your Modification is an application programming
         interface and You own or control patents which are reasonably necessary
         to implement that API, you must also include this information in the
         LEGAL file.

      (( 3.5. ))??
      Required Notices. You must duplicate the notice in Exhibit A in each file
      of the Source Code, and this License in any documentation for the Source
      Code, where You describe recipients' rights relating to Covered Code. If
      You created one or more Modification(s), You may add your name as a
      Contributor to the Source Code. If it is not possible to put such notice
      in a particular Source Code file due to its structure, then you must
      include such notice in a location (such as a relevant directory file)
      where a user would be likely to look for such a notice. You may choose to
      offer, and to charge a fee for, warranty, support, indemnity or liability
      obligations to one or more recipients of Covered Code.

      However, You may do so only on Your own behalf, and not on behalf of the
      Initial Developer or any Contributor. You must make it absolutely clear
      that any such warranty, support, indemnity or liability obligation is
      offered by You alone, and You hereby agree to indemnify the Initial
      Developer and every Contributor for any liability incurred by the Initial
      Developer or such Contributor as a result of warranty, support, indemnity
      or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Versions. You may distribute Covered Code in
      Executable form only if the requirements of Section 3.1-3.5 have been met
      for that Covered Code, and if You include a notice stating that the Source
      Code version of the Covered Code is available under the terms of this
      License, including a description of how and where You have fulfilled the
      obligations of Section 3.2. The notice must be conspicuously included in
      any notice in an Executable version, related documentation or collateral
      in which You describe recipients' rights relating to the Covered Code. You
      may distribute the Executable version of Covered Code under a license of
      Your choice, which may contain terms different from this License, provided
      that You are in compliance with the terms of this License and that the
      license for the Executable version does not attempt to limit or alter the
      recipient's rights in the Source Code version from the rights set forth in
      this License. If You distribute the Executable version under a different
      license You must make it absolutely clear that any terms which differ from
      this License are offered by You alone, not by the Initial Developer or any
      Contributor. You hereby agree to indemnify the Initial Developer and every
      Contributor for any liability incurred by the Initial Developer or such
      Contributor as a result of any such terms You offer.

      If you distribute executable versions containing Covered Code, you must
      reproduce the notice in Exhibit B in the documentation and/or other
      materials provided with the product.

      (( 3.7. ))??
      Larger Works. You may create a Larger Work by combining Covered Code with
      other code not governed by the terms of this License and distribute the
      Larger Work as a single product. In such a case, You must make sure the
      requirements of this License are fulfilled for the Covered Code.

   (( 4. ))??
   INABILITY TO COMPLY DUE TO STATUTE OR REGULATION

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute or regulation
   then You must: (a) comply with the terms of this License to the maximum
   extent possible; (b) cite the statute or regulation that prohibits you from
   adhering to the license; and (c) describe the limitations and the code they
   affect. Such description must be included in the LEGAL file described in
   Section 3.4 and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill to be able to
   understand it. You must submit this LEGAL file to Red Hat for review, and You
   will not be able use the covered code in any means until permission is
   granted from Red Hat to allow for the inability to comply due to statute or
   regulation.

   (( 5. ))??
   APPLICATION OF THIS LICENSE

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A, and to related Covered Code.

   Red Hat may include Covered Code in products without such additional products
   becoming subject to the terms of this License, and may license such
   additional products on different terms from those contained in this License.

   Red Hat may license the Source Code of Red Hat Branded Code without Red Hat
   Branded Code becoming subject to the terms of this License, and may license
   Red Hat Branded Code on different terms from those contained in this License.
   Contact Red Hat for details of alternate licensing terms available.

   (( 6. ))??
   VERSIONS OF THE LICENSE

      (( 6.1. ))??
      New Versions. Red Hat may publish revised and/or new versions of the
      License from time to time. Each version will be given a distinguishing
      version number.

      (( 6.2. ))??
      Effect of New Versions. Once Covered Code has been published under a
      particular version of the License, You may always continue to use it under
      the terms of that version. You may also choose to use such Covered Code
      under the terms of any subsequent version of the License published by Red
      Hat. No one other than Red Hat has the right to modify the terms
      applicable to Covered Code beyond what is granted under this and
      subsequent Licenses.

      (( 6.3. ))??
      Derivative Works. If you create or use a modified version of this License
      (which you may only do in order to apply it to code which is not already
      Covered Code governed by this License), you must (a) rename Your license
      so that the phrases "ECOS", "eCos", "Red Hat", "RHEPL" or any confusingly
      similar phrase do not appear anywhere in your license and (b) otherwise
      make it clear that your version of the license contains terms which differ
      from the Red Hat eCos Public License. (Filling in the name of the Initial
      Developer, Original Code or Contributor in the notice described in Exhibit
      A shall not of themselves be deemed to be modifications of this License.)

   (( 7. ))??
   DISCLAIMER OF WARRANTY

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER
   OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR
   OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF
   THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

   (( 8. ))??
   TERMINATION

   This License and the rights granted hereunder will terminate automatically if
   You fail to comply with terms herein and fail to cure such breach within 30
   days of becoming aware of the breach. All sublicenses to the Covered Code
   which are properly granted shall survive any termination of this License.
   Provisions which, by their nature, must remain in effect beyond the
   termination of this License shall survive.

   (( 9. ))??
   LIMITATION OF LIABILITY

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE INITIAL DEVELOPER, ANY OTHER
   CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE, OR ANY SUPPLIER OF ANY OF
   SUCH PARTIES, BE LIABLE TO YOU OR ANY OTHER PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THAT EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. GOVERNMENT END USERS

   The Covered Code is a "commercial item," as that term is defined in 48 C.F.R.
   2.101 (Oct. 1995), consisting of "commercial computer software" and
   "commercial computer software documentation," as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Covered Code with only those rights set forth herein.

   (( 11. ))??
   MISCELLANEOUS

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by California law provisions
   (except to the extent applicable law, if any, provides otherwise), excluding
   its conflict-of-law provisions. With respect to disputes in which at least
   one party is a citizen of, or an entity chartered or registered to do
   business in, the United States of America: (a) unless otherwise agreed in
   writing, all disputes relating to this License (excepting any dispute
   relating to intellectual property rights) shall be subject to final and
   binding arbitration, with the losing party paying all costs of arbitration;
   (b) any arbitration relating to this Agreement shall be held in Santa Clara
   County, California, under the auspices of JAMS/EndDispute; and (c) any
   litigation relating to this Agreement shall be subject to the jurisdiction of
   the Federal Courts of the Northern District of California, with venue lying
   in Santa Clara County, California, with the losing party responsible for
   costs, including without limitation, court costs and reasonable attorneys
   fees and expenses. The application of the United Nations Convention on
   Contracts for the International Sale of Goods is expressly excluded. Any law
   or regulation which provides that the language of a contract shall be
   construed against the drafter shall not apply to this License.

   (( 12. ))??
   RESPONSIBILITY FOR CLAIMS

   Except in cases where another Contributor has failed to comply with Section
   3.4, You are responsible for damages arising, directly or indirectly, out of
   Your utilization of rights under this License, based on the number of copies
   of Covered Code you made available, the revenues you received from utilizing
   such rights, and other relevant factors. You agree to work with affected
   parties to distribute responsibility on an equitable basis.

   (( 13. ))??
   ADDITIONAL TERMS APPLICABLE TO THE RED HAT ECOS PUBLIC LICENSE

   Nothing in this License shall be interpreted to prohibit Red Hat from
   licensing under different terms than this License any code which Red Hat
   otherwise would have a right to license.

   Red Hat and logo - This License does not grant any rights to use the
   trademark Red Hat, the Red Hat logo, eCos logo, even if such marks are
   included in the Original Code. You may contact Red Hat for permission to
   display the Red Hat and eCos marks in either the documentation or the
   Executable version beyond that required in Exhibit B.

   Inability to Comply Due to Contractual Obligation - To the extent that Red
   Hat is limited contractually from making third party code available under
   this License, Red Hat may choose to integrate such third party code into
   Covered Code without being required to distribute such third party code in
   Source Code form, even if such third party code would otherwise be considered
   "Modifications" under this License.
   (( EXHIBIT A

"The contents of this file are subject to the Red Hat eCos Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http:/www.redhat.com/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

The Original Code is eCos - Embedded Configurable Operating System, released
September 30, 1998. The Initial Developer of the Original Code is Red Hat.
Portions created by Red Hat are Copyright (C) 1998, 1999, 2000 Red Hat, Inc. All
Rights Reserved."

EXHIBIT B

Part of the software embedded in this product is eCos - Embedded Configurable
Operating System, a trademark of Red Hat. Portions created by Red Hat are
Copyright (C) 1998, 1999, 2000 Red Hat, Inc. (http:/www.redhat.com/). All Rights
Reserved.

THE SOFTWARE IN THIS PRODUCT WAS IN PART PROVIDED BY RED HAT AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE. ))??
`
const license_RPL_1_1_lre = `//**
Reciprocal Public License 1.1
https://spdx.org/licenses/RPL-1.1.json
https://opensource.org/licenses/RPL-1.1
**//

(( Reciprocal Public License, version 1.1
(( Copyright __20__ ))??
))??

(( PREAMBLE

This Preamble is intended to describe, in plain English, the nature, intent, and
scope of this License. However, this Preamble is not a part of this License. The
legal effect of this License is dependent only upon the terms of the License and
not this Preamble.

This License is based on the concept of reciprocity. In exchange for being
granted certain rights under the terms of this License to Licensor's Software,
whose Source Code You have access to, You are required to reciprocate by
providing equal access and rights to all third parties to the Source Code of any
Modifications, Derivative Works, and Required Components for execution of same
(collectively defined as Extensions) that You Deploy by Deploying Your
Extensions under the terms of this License. In this fashion the available Source
Code related to the original Licensed Software is enlarged for the benefit of
everyone.

Under the terms of this License You may:

   (( a. ))??
   Distribute the Licensed Software exactly as You received it under the terms
   of this License either alone or as a component of an aggregate software
   distribution containing programs from several different sources without
   payment of a royalty or other fee.

   (( b. ))??
   Use the Licensed Software for any purpose consistent with the rights granted
   by this License, but the Licensor is not providing You any warranty
   whatsoever, nor is the Licensor accepting any liability in the event that the
   Licensed Software doesn't work properly or causes You any injury or damages.

   (( c. ))??
   Create Extensions to the Licensed Software consistent with the rights granted
   by this License, provided that You make the Source Code to any Extensions You
   Deploy available to all third parties under the terms of this License,
   document Your Modifications clearly, and title all Extensions distinctly from
   the Licensed Software.

   (( d. ))??
   Charge a fee for warranty or support, or for accepting indemnity or liability
   obligations for Your customers.

Under the terms of this License You may not:

   (( a. ))??
   Charge for the Source Code to the Licensed Software, or Your Extensions,
   other than a nominal fee not to exceed Your cost for reproduction and
   distribution where such reproduction and distribution involve physical
   media.

   (( b. ))??
   Modify or delete any pre-existing copyright notices, change notices, or
   License text in the Licensed Software.

   (( c. ))??
   Assert any patent claims against the Licensor or Contributors, or which would
   in any way restrict the ability of any third party to use the Licensed
   Software or portions thereof in any form under the terms of this License, or
   Your rights to the Licensed Software under this License automatically
   terminate.

   (( d. ))??
   Represent either expressly or by implication, appearance, or otherwise that
   You represent Licensor or

   Contributors in any capacity or that You have any form of legal association
   by virtue of this License.

Under the terms of this License You must:

   (( a. ))??
   Document any Modifications You make to the Licensed Software including the
   nature of the change, the authors of the change, and the date of the change.
   This documentation must appear both in the Source Code and in a text file
   titled "CHANGES" distributed with the Licensed Software and Your Extensions.

   (( b. ))??
   Make the Source Code for any Extensions You Deploy available in a timely
   fashion via an Electronic Distribution Mechanism such as FTP or HTTP
   download.

   (( c. ))??
   Notify the Licensor of the availability of Source Code to Your Extensions in
   a timely fashion and include in such notice a brief description of the
   Extensions, the distinctive title used, and instructions on how to acquire
   the Source Code and future updates.

   (( d. ))??
   Grant Licensor and all third parties a world-wide, non-exclusive,
   royalty-free license under any intellectual property rights owned or
   controlled by You to use, reproduce, display, perform, modify, sublicense,
   and distribute Your Extensions, in any form, under the terms of this License. ))??

LICENSE TERMS

   (( 1.0 ))??
   General; Applicability & Definitions. This Reciprocal Public License Version
   1.1 ("License") applies to any programs or other works as well as any and all
   updates or maintenance releases of said programs or works ("Software") not
   already covered by this License which the Software copyright holder
   ("Licensor") makes publicly available containing a Notice (hereinafter
   defined) from the Licensor specifying or allowing use or distribution under
   the terms of this License. As used in this License and Preamble:

      (( 1.1 ))??
      "Contributor" means any person or entity who created or contributed to the
      creation of an Extension.

      (( 1.2 ))??
      "Deploy" means to use, Serve, sublicense or distribute Licensed Software
      other than for Your internal Research and/or Personal Use, and includes
      without limitation, any and all internal use or distribution of Licensed
      Software within Your business or organization other than for Research
      and/or Personal Use, as well as direct or indirect sublicensing or
      distribution of Licensed Software by You to any third party in any form or
      manner.

      (( 1.3 ))??
      "Derivative Works" as used in this License is defined under U.S. copyright
      law.

      (( 1.4 ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of data
      such as download from an FTP or web site, where such mechanism is publicly
      accessible.

      (( 1.5 ))??
      "Extensions" means any Modifications, Derivative Works, or Required
      Components as those terms are defined in this License.

      (( 1.6 ))??
      "License" means this Reciprocal Public License.

      (( 1.7 ))??
      "Licensed Software" means any Software licensed pursuant to this License.
      Licensed Software also includes all previous Extensions from any
      Contributor that You receive.

      (( 1.8 ))??
      "Licensor" means the copyright holder of any Software previously uncovered
      by this License who releases the Software under the terms of this
      License.

      (( 1.9 ))??
      "Modifications" means any additions to or deletions from the substance or
      structure of (i) a file or other storage containing Licensed Software, or
      (ii) any new file or storage that contains any part of Licensed Software,
      or (iii) any file or storage which replaces or otherwise alters the
      original functionality of Licensed Software at runtime.

      (( 1.10 ))??
      "Notice" means the notice contained in EXHIBIT A.

      (( 1.11 ))??
      "Personal Use" means use of Licensed Software by an individual solely for
      his or her personal, private and non-commercial purposes. An individual's
      use of Licensed Software in his or her capacity as an officer, employee,
      member, independent contractor or agent of a corporation, business or
      organization (commercial or non-commercial) does not qualify as Personal
      Use.

      (( 1.12 ))??
      "Required Components" means any text, programs, scripts, schema, interface
      definitions, control files, or other works created by You which are
      required by a third party of average skill to successfully install and run
      Licensed Software containing Your Modifications, or to install and run
      Your Derivative Works.

      (( 1.13 ))??
      "Research" means investigation or experimentation for the purpose of
      understanding the nature and limits of the Licensed Software and its
      potential uses.

      (( 1.14 ))??
      "Serve" means to deliver Licensed Software and/or Your Extensions by means
      of a computer network to one or more computers for purposes of execution
      of Licensed Software and/or Your Extensions.

      (( 1.15 ))??
      "Software" means any computer programs or other works as well as any
      updates or maintenance releases of those programs or works which are
      distributed publicly by Licensor.

      (( 1.16 ))??
      "Source Code" means the preferred form for making modifications to the
      Licensed Software and/or Your Extensions, including all modules contained
      therein, plus any associated text, interface definition files, scripts
      used to control compilation and installation of an executable program or
      other components required by a third party of average skill to build a
      running version of the Licensed Software or Your Extensions.

      (( 1.17 ))??
      "You" or "Your" means an individual or a legal entity exercising rights
      under this License. For legal entities, "You" or "Your" includes any
      entity which controls, is controlled by, or is under common control with,
      You, where "control" means (a) the power, direct or indirect, to cause the
      direction or management of such entity, whether by contract or otherwise,
      or (b) ownership of fifty percent (50%) or more of the outstanding shares
      or beneficial ownership of such entity.

   (( 2.0 ))??
   Acceptance Of License. You are not required to accept this License since you
   have not signed it, however nothing else grants you permission to use, copy,
   distribute, modify, or create derivatives of either the Software or any
   Extensions created by a Contributor. These actions are prohibited by law if
   you do not accept this License. Therefore, by performing any of these actions
   You indicate Your acceptance of this License and Your agreement to be bound
   by all its terms and conditions. IF YOU DO NOT AGREE WITH ALL THE TERMS AND
   CONDITIONS OF THIS LICENSE DO NOT USE, MODIFY, CREATE DERIVATIVES, OR
   DISTRIBUTE THE SOFTWARE. IF IT IS IMPOSSIBLE FOR YOU TO COMPLY WITH ALL THE
   TERMS AND CONDITIONS OF THIS LICENSE THEN YOU CAN NOT USE, MODIFY, CREATE
   DERIVATIVES, OR DISTRIBUTE THE SOFTWARE.

   (( 3.0 ))??
   Grant of License From Licensor. Subject to the terms and conditions of this
   License, Licensor hereby grants You a world-wide, royalty-free, non-exclusive
   license, subject to Licensor's intellectual property rights, and any third
   party intellectual property claims derived from the Licensed Software under
   this License, to do the following:

      (( 3.1 ))??
      Use, reproduce, modify, display, perform, sublicense and distribute
      Licensed Software and Your Extensions in both Source Code form or as an
      executable program.

      (( 3.2 ))??
      Create Derivative Works (as that term is defined under U.S. copyright law)
      of Licensed Software by adding to or deleting from the substance or
      structure of said Licensed Software.

      (( 3.3 ))??
      Under claims of patents now or hereafter owned or controlled by Licensor,
      to make, use, have made, and/or otherwise dispose of Licensed Software or
      portions thereof, but solely to the extent that any such claim is
      necessary to enable You to make, use, have made, and/or otherwise dispose
      of Licensed Software or portions thereof.

      (( 3.4 ))??
      Licensor reserves the right to release new versions of the Software with
      different features, specifications, capabilities, functions, licensing
      terms, general availability or other characteristics. Title, ownership
      rights, and intellectual property rights in and to the Licensed Software
      shall remain in Licensor and/or its Contributors.

   (( 4.0 ))??
   Grant of License From Contributor. By application of the provisions in
   Section 6 below, each Contributor hereby grants You a world-wide,
   royalty-free, non-exclusive license, subject to said Contributor's
   intellectual property rights, and any third party intellectual property
   claims derived from the Licensed Software under this License, to do the
   following:

      (( 4.1 ))??
      Use, reproduce, modify, display, perform, sublicense and distribute any
      Extensions Deployed by such Contributor or portions thereof, in both
      Source Code form or as an executable program, either on an unmodified
      basis or as part of Derivative Works.

      (( 4.2 ))??
      Under claims of patents now or hereafter owned or controlled by
      Contributor, to make, use, have made, and/or otherwise dispose of
      Extensions or portions thereof, but solely to the extent that any such
      claim is necessary to enable You to make, use, have made, and/or otherwise
      dispose of Contributor's Extensions or portions thereof.

   (( 5.0 ))??
   Exclusions From License Grant. Nothing in this License shall be deemed to
   grant any rights to trademarks, copyrights, patents, trade secrets or any
   other intellectual property of Licensor or any Contributor except as
   expressly stated herein. Except as expressly stated in Sections 3 and 4, no
   other patent rights, express or implied, are granted herein. Your Extensions
   may require additional patent licenses from Licensor or Contributors which
   each may grant in its sole discretion. No right is granted to the trademarks
   of Licensor or any Contributor even if such marks are included in the
   Licensed Software. Nothing in this License shall be interpreted to prohibit
   Licensor from licensing under different terms from this License any code that
   Licensor otherwise would have a right to license.

      (( 5.1 ))??
      You expressly acknowledge and agree that although Licensor and each
      Contributor grants the licenses to their respective portions of the
      Licensed Software set forth herein, no assurances are provided by Licensor
      or any Contributor that the Licensed Software does not infringe the patent
      or other intellectual property rights of any other entity. Licensor and
      each Contributor disclaim any liability to You for claims brought by any
      other entity based on infringement of intellectual property rights or
      otherwise. As a condition to exercising the rights and licenses granted
      hereunder, You hereby assume sole responsibility to secure any other
      intellectual property rights needed, if any. For example, if a third party
      patent license is required to allow You to distribute the Licensed
      Software, it is Your responsibility to acquire that license before
      distributing the Licensed Software.

   (( 6.0 ))??
   Your Obligations And Grants. In consideration of, and as an express condition
   to, the licenses granted to You under this License You hereby agree that any
   Modifications, Derivative Works, or Required Components (collectively
   Extensions) that You create or to which You contribute are governed by the
   terms of this License including, without limitation, Section 4. Any
   Extensions that You create or to which You contribute must be Deployed under
   the terms of this License or a future version of this License released under
   Section 7. You hereby grant to Licensor and all third parties a world-wide,
   non-exclusive, royalty-free license under those intellectual property rights
   You own or control to use, reproduce, display, perform, modify, create
   derivatives, sublicense, and distribute Your Extensions, in any form. Any
   Extensions You make and Deploy must have a distinct title so as to readily
   tell any subsequent user or Contributor that the Extensions are by You. You
   must include a copy of this License with every copy of the Extensions You
   distribute. You agree not to offer or impose any terms on any Source Code or
   executable version of the Licensed Software, or its Extensions that alter or
   restrict the applicable version of this License or the recipients' rights
   hereunder.

      (( 6.1 ))??
      Availability of Source Code. You must make available, under the terms of
      this License, the Source Code of the Licensed Software and any Extensions
      that You Deploy, either on the same media as You distribute any executable
      or other form of the Licensed Software, or via an Electronic Distribution
      Mechanism. The Source Code for any version of Licensed Software, or its
      Extensions that You Deploy must be made available at the time of
      Deployment and must remain available for as long as You Deploy the
      Extensions or at least twelve (12) months after the date You Deploy,
      whichever is longer. You are responsible for ensuring that the Source Code
      version remains available even if the Electronic Distribution Mechanism is
      maintained by a third party. You may not charge a fee for the Source Code
      distributed under this Section in excess of Your actual cost of
      duplication and distribution where such duplication and distribution
      involve physical media.

      (( 6.2 ))??
      Description of Modifications. You must cause any Modifications that You
      create or to which You contribute, to update the file titled "CHANGES"
      distributed with Licensed Software documenting the additions, changes or
      deletions You made, the authors of such Modifications, and the dates of
      any such additions, changes or deletions. You must also cause a
      cross-reference to appear in the Source Code at the location of each
      change. You must include a prominent statement that the Modifications are
      derived, directly or indirectly, from the Licensed Software and include
      the names of the Licensor and any Contributor to the Licensed Software in
      (i) the Source Code and (ii) in any notice displayed by the Licensed
      Software You distribute or in related documentation in which You describe
      the origin or ownership of the Licensed Software. You may not modify or
      delete any pre-existing copyright notices, change notices or License text
      in the Licensed Software.

      (( 6.3 ))??
      Intellectual Property Matters.

         (( a. ))??
         Third Party Claims. If You have knowledge that a license to a third
         party's intellectual property right is required to exercise the rights
         granted by this License, You must include a text file with the Source
         Code distribution titled "LEGAL" that describes the claim and the party
         making the claim in sufficient detail that a recipient will know whom
         to contact. If You obtain such knowledge after You make any Extensions
         available as described in Section 6.1, You shall promptly modify the
         LEGAL file in all copies You make available thereafter and shall take
         other steps (such as notifying appropriate mailing lists or newsgroups)
         reasonably calculated to inform those who received the Licensed
         Software from You that new knowledge has been obtained.

         (( b. ))??
         Contributor APIs. If Your Extensions include an application programming
         interface ("API") and You have knowledge of patent licenses that are
         reasonably necessary to implement that API, You must also include this
         information in the LEGAL file.

         (( c. ))??
         Representations. You represent that, except as disclosed pursuant to
         6.3(a) above, You believe that any Extensions You distribute are Your
         original creations and that You have sufficient rights to grant the
         rights conveyed by this License.

      (( 6.4 ))??
      Required Notices.

         (( a. ))??
         License Text. You must duplicate this License in any documentation You
         provide along with the Source Code of any Extensions You create or to
         which You contribute, wherever You describe recipients' rights relating
         to Licensed Software. You must duplicate the notice contained in
         EXHIBIT A (the "Notice") in each file of the Source Code of any copy
         You distribute of the Licensed Software and Your Extensions. If You
         create an Extension, You may add Your name as a Contributor to the text
         file titled "CONTRIB" distributed with the Licensed Software along with
         a description of the contribution. If it is not possible to put the
         Notice in a particular Source Code file due to its structure, then You
         must include such Notice in a location (such as a relevant directory
         file) where a user would be likely to look for such a notice.

         (( b. ))??
         Source Code Availability. You must notify Licensor within one (1) month
         of the date You initially Deploy of the availability of Source Code to
         Your Extensions and include in such notification the name under which
         you Deployed Your Extensions, a description of the Extensions, and
         instructions on how to acquire the Source Code, including instructions
         on how to acquire updates over time. Should such instructions change
         you must provide Licensor with revised instructions within one (1)
         month of the date of change. Should you be unable to notify Licensor
         directly, you must provide notification by posting to appropriate news
         groups, mailing lists, or web sites where a search engine would
         reasonably be expected to index them.

      (( 6.5 ))??
      Additional Terms. You may choose to offer, and charge a fee for, warranty,
      support, indemnity or liability obligations to one or more recipients of
      Licensed Software. However, You may do so only on Your own behalf, and not
      on behalf of the Licensor or any Contributor. You must make it clear that
      any such warranty, support, indemnity or liability obligation is offered
      by You alone, and You hereby agree to indemnify the Licensor and every
      Contributor for any liability plus attorney fees, costs, and related
      expenses due to any such action or claim incurred by the Licensor or such
      Contributor as a result of warranty, support, indemnity or liability terms
      You offer.

      (( 6.6 ))??
      Conflicts With Other Licenses. Where any portion of Your Extensions, by
      virtue of being Derivative Works of another product or similar
      circumstance, fall under the terms of another license, the terms of that
      license should be honored however You must also make Your Extensions
      available under this License. If the terms of this License continue to
      conflict with the terms of the other license you may write the Licensor
      for permission to resolve the conflict in a fashion that remains
      consistent with the intent of this License. Such permission will be
      granted at the sole discretion of the Licensor.

   (( 7.0 ))??
   Versions of This License. Licensor may publish from time to time revised
   and/or new versions of the License. Once Licensed Software has been published
   under a particular version of the License, You may always continue to use it
   under the terms of that version. You may also choose to use such Licensed
   Software under the terms of any subsequent version of the License published
   by Licensor. No one other than Licensor has the right to modify the terms
   applicable to Licensed Software created under this License.

      (( 7.1 ))??
      If You create or use a modified version of this License, which You may do
      only in order to apply it to software that is not already Licensed
      Software under this License, You must rename Your license so that it is
      not confusingly similar to this License, and must make it clear that Your
      license contains terms that differ from this License. In so naming Your
      license, You may not use any trademark of Licensor or of any Contributor.
      Should Your modifications to this License be limited to alteration of
      EXHIBIT A purely for purposes of adjusting the Notice You require of
      licensees, You may continue to refer to Your License as the Reciprocal
      Public License or simply the RPL.

   (( 8.0 ))??
   Disclaimer of Warranty. LICENSED SOFTWARE IS PROVIDED UNDER THIS LICENSE ON
   AN "AS IS" BASIS, WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
   INCLUDING, WITHOUT LIMITATION, WARRANTIES THAT THE LICENSED SOFTWARE IS FREE
   OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING.
   FURTHER THERE IS NO WARRANTY MADE AND ALL IMPLIED WARRANTIES ARE DISCLAIMED
   THAT THE LICENSED SOFTWARE MEETS OR COMPLIES WITH ANY DESCRIPTION OF
   PERFORMANCE OR OPERATION, SAID COMPATIBILITY AND SUITABILITY BEING YOUR
   RESPONSIBILITY. LICENSOR DISCLAIMS ANY WARRANTY, IMPLIED OR EXPRESSED, THAT
   ANY CONTRIBUTOR'S EXTENSIONS MEET ANY STANDARD OF COMPATIBILITY OR
   DESCRIPTION OF PERFORMANCE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE
   OF THE LICENSED SOFTWARE IS WITH YOU. SHOULD LICENSED SOFTWARE PROVE
   DEFECTIVE IN ANY RESPECT, YOU (AND NOT THE LICENSOR OR ANY OTHER CONTRIBUTOR)
   ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR OR CORRECTION. UNDER THE
   TERMS OF THIS LICENSOR WILL NOT SUPPORT THIS SOFTWARE AND IS UNDER NO
   OBLIGATION TO ISSUE UPDATES TO THIS SOFTWARE. LICENSOR HAS NO KNOWLEDGE OF
   ERRANT CODE OR VIRUS IN THIS SOFTWARE, BUT DOES NOT WARRANT THAT THE SOFTWARE
   IS FREE FROM SUCH ERRORS OR VIRUSES. THIS DISCLAIMER OF WARRANTY CONSTITUTES
   AN ESSENTIAL PART OF THIS LICENSE. NO USE OF LICENSED SOFTWARE IS AUTHORIZED
   HEREUNDER EXCEPT UNDER THIS DISCLAIMER.

   (( 9.0 ))??
   Limitation of Liability. UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY,
   WHETHER TORT (INCLUDING NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE
   LICENSOR, ANY CONTRIBUTOR, OR ANY DISTRIBUTOR OF LICENSED SOFTWARE, OR ANY
   SUPPLIER OF ANY OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT,
   SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING,
   WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER
   FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES,
   EVEN IF SUCH PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH
   DAMAGES. THIS LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH
   OR PERSONAL INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT
   APPLICABLE LAW PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE
   EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS
   EXCLUSION AND LIMITATION MAY NOT APPLY TO YOU.

   (( 10.0 ))??
   High Risk Activities. THE LICENSED SOFTWARE IS NOT FAULT-TOLERANT AND IS NOT
   DESIGNED, MANUFACTURED, OR INTENDED FOR USE OR DISTRIBUTION AS ON-LINE
   CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE PERFORMANCE,
   SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT NAVIGATION OR
   COMMUNICATIONS SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE SUPPORT MACHINES, OR
   WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE LICENSED SOFTWARE COULD LEAD
   DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE PHYSICAL OR ENVIRONMENTAL
   DAMAGE ("HIGH RISK ACTIVITIES"). LICENSOR AND CONTRIBUTORS SPECIFICALLY
   DISCLAIM ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR HIGH RISK
   ACTIVITIES.

   (( 11.0 ))??
   Responsibility for Claims. As between Licensor and Contributors, each party
   is responsible for claims and damages arising, directly or indirectly, out of
   its utilization of rights under this License which specifically disclaims
   warranties and limits any liability of the Licensor. This paragraph is to be
   used in conjunction with and controlled by the Disclaimer Of Warranties of
   Section 8, the Limitation Of Damages in Section 9, and the disclaimer against
   use for High Risk Activities in Section 10. The Licensor has thereby
   disclaimed all warranties and limited any damages that it is or may be liable
   for. You agree to work with Licensor and Contributors to distribute such
   responsibility on an equitable basis consistent with the terms of this
   License including Sections 8, 9, and 10. Nothing herein is intended or shall
   be deemed to constitute any admission of liability.

   (( 12.0 ))??
   Termination. This License and all rights granted hereunder will terminate
   immediately in the event of the circumstances described in Section 13.6 or if
   applicable law prohibits or restricts You from fully and or specifically
   complying with Sections 3, 4 and/or 6, or prevents the enforceability of any
   of those Sections, and You must immediately discontinue any use of Licensed
   Software.

      (( 12.1 ))??
      Automatic Termination Upon Breach. This License and the rights granted
      hereunder will terminate automatically if You fail to comply with the
      terms herein and fail to cure such breach within thirty (30) days of
      becoming aware of the breach. All sublicenses to the Licensed Software
      that are properly granted shall survive any termination of this License.
      Provisions that, by their nature, must remain in effect beyond the
      termination of this License, shall survive.

      (( 12.2 ))??
      Termination Upon Assertion of Patent Infringement. If You initiate
      litigation by asserting a patent infringement claim (excluding declaratory
      judgment actions) against Licensor or a Contributor (Licensor or
      Contributor against whom You file such an action is referred to herein as
      "Respondent") alleging that Licensed Software directly or indirectly
      infringes any patent, then any and all rights granted by such Respondent
      to You under Sections 3 or 4 of this License shall terminate prospectively
      upon sixty (60) days notice from Respondent (the "Notice Period") unless
      within that Notice Period You either agree in writing (i) to pay
      Respondent a mutually agreeable reasonably royalty for Your past or future
      use of Licensed Software made by such Respondent, or (ii) withdraw Your
      litigation claim with respect to Licensed Software against such
      Respondent. If within said Notice Period a reasonable royalty and payment
      arrangement are not mutually agreed upon in writing by the parties or the
      litigation claim is not withdrawn, the rights granted by Licensor to You
      under Sections 3 and 4 automatically terminate at the expiration of said
      Notice Period.

      (( 12.3 ))??
      Reasonable Value of This License. If You assert a patent infringement
      claim against Respondent alleging that Licensed Software directly or
      indirectly infringes any patent where such claim is resolved (such as by
      license or settlement) prior to the initiation of patent infringement
      litigation, then the reasonable value of the licenses granted by said
      Respondent under Sections 3 and 4 shall be taken into account in
      determining the amount or value of any payment or license.

      (( 12.4 ))??
      No Retroactive Effect of Termination. In the event of termination under
      this Section all end user license agreements (excluding licenses to
      distributors and resellers) that have been validly granted by You or any
      distributor hereunder prior to termination shall survive termination.

   (( 13.0 ))??
   Miscellaneous.

      (( 13.1 ))??
      U.S. Government End Users. The Licensed Software is a "commercial item,"
      as that term is defined in 48 C.F.R. 2.101 (Oct. 1995), consisting of
      "commercial computer software" and "commercial computer software
      documentation," as such terms are used in 48 C.F.R. 12.212 (Sept. 1995).
      Consistent with 48 C.F.R. 12.212 and 48 C.F.R. 227.7202-1 through
      227.7202-4 (June 1995), all U.S. Government End Users acquire Licensed
      Software with only those rights set forth herein.

      (( 13.2 ))??
      Relationship of Parties. This License will not be construed as creating an
      agency, partnership, joint venture, or any other form of legal association
      between or among You, Licensor, or any Contributor, and You will not
      represent to the contrary, whether expressly, by implication, appearance,
      or otherwise.

      (( 13.3 ))??
      Independent Development. Nothing in this License will impair Licensor's
      right to acquire, license, develop, subcontract, market, or distribute
      technology or products that perform the same or similar functions as, or
      otherwise compete with, Extensions that You may develop, produce, market,
      or distribute.

      (( 13.4 ))??
      Consent To Breach Not Waiver. Failure by Licensor or Contributor to
      enforce any provision of this License will not be deemed a waiver of
      future enforcement of that or any other provision.

      (( 13.5 ))??
      Severability. This License represents the complete agreement concerning
      the subject matter hereof. If any provision of this License is held to be
      unenforceable, such provision shall be reformed only to the extent
      necessary to make it enforceable.

      (( 13.6 ))??
      Inability to Comply Due to Statute or Regulation. If it is impossible for
      You to comply with any of the terms of this License with respect to some
      or all of the Licensed Software due to statute, judicial order, or
      regulation, then You cannot use, modify, or distribute the software.

      (( 13.7 ))??
      Export Restrictions. You may be restricted with respect to downloading or
      otherwise acquiring, exporting, or reexporting the Licensed Software or
      any underlying information or technology by United States and other
      applicable laws and regulations. By downloading or by otherwise obtaining
      the Licensed Software, You are agreeing to be responsible for compliance
      with all applicable laws and regulations.

      (( 13.8 ))??
      Arbitration, Jurisdiction & Venue. This License shall be governed by
      Colorado law provisions (except to the extent applicable law, if any,
      provides otherwise), excluding its conflict-of-law provisions. You
      expressly agree that any dispute relating to this License shall be
      submitted to binding arbitration under the rules then prevailing of the
      American Arbitration Association. You further agree that Adams County,
      Colorado USA is proper venue and grant such arbitration proceeding
      jurisdiction as may be appropriate for purposes of resolving any dispute
      under this License. Judgement upon any award made in arbitration may be
      entered and enforced in any court of competent jurisdiction. The
      arbitrator shall award attorney's fees and costs of arbitration to the
      prevailing party. Should either party find it necessary to enforce its
      arbitration award or seek specific performance of such award in a civil
      court of competent jurisdiction, the prevailing party shall be entitled to
      reasonable attorney's fees and costs. The application of the United
      Nations Convention on Contracts for the International Sale of Goods is
      expressly excluded. You and Licensor expressly waive any rights to a jury
      trial in any litigation concerning Licensed Software or this License. Any
      law or regulation that provides that the language of a contract shall be
      construed against the drafter shall not apply to this License.

      (( 13.9 ))??
      Entire Agreement. This License constitutes the entire agreement between
      the parties with respect to the subject matter hereof.
      (( EXHIBIT A

The Notice below must appear in each file of the Source Code of any copy You
distribute of the Licensed Software or any Extensions thereto, except as may be
modified as allowed under the terms of Section 7.1

Copyright (C) 1999-2002 Technical Pursuit Inc., All Rights Reserved. Patent
Pending, Technical Pursuit Inc.

Unless explicitly acquired and licensed from Licensor under the Technical
Pursuit License ("TPL") Version 1.0 or greater, the contents of this file are
subject to the Reciprocal Public License ("RPL") Version 1.1, or subsequent
versions as allowed by the RPL, and You may not copy or use this file in either
source code or executable form, except in compliance with the terms and
conditions of the RPL.

You may obtain a copy of both the TPL and the RPL (the "Licenses") from
Technical Pursuit Inc. at http:/www.technicalpursuit.com.

All software distributed under the Licenses is provided strictly on an "AS IS"
basis, WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND TECHNICAL
PURSUIT INC. HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION,
ANY WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
ENJOYMENT, OR NON-INFRINGEMENT. See the Licenses for specific language governing
rights and limitations under the Licenses. ))??
`
const license_RPL_1_5_lre = `//**
Reciprocal Public License 1.5
https://spdx.org/licenses/RPL-1.5.json
https://opensource.org/licenses/RPL-1.5
**//

(( Reciprocal Public License 1.5 (RPL1.5)
Version 1.5, July 15, 2007

Copyright __20__. All Rights Reserved.
))??

PREAMBLE

The Reciprocal Public License (RPL) is based on the concept of reciprocity or,
if you prefer, fairness.

In short, this license grew out of a desire to close loopholes in previous open
source licenses, loopholes that allowed parties to acquire open source software
and derive financial benefit from it without having to release their
improvements or derivatives to the community which enabled them. This occurred
any time an entity did not release their application to a "third party".

While there is a certain freedom in this model of licensing, it struck the
authors of the RPL as being unfair to the open source community at large and to
the original authors of the works in particular. After all, bug fixes,
extensions, and meaningful and valuable derivatives were not consistently
finding their way back into the community where they could fuel further, and
faster, growth and expansion of the overall open source software base.

While you should clearly read and understand the entire license, the essence of
the RPL is found in two definitions: "Deploy" and "Required Components".

Regarding deployment, under the RPL your changes, bug fixes, extensions, etc.
must be made available to the open source community at large when you Deploy in
any form -- either internally or to an outside party. Once you start running the
software you have to start sharing the software.

Further, under the RPL all components you author including schemas, scripts,
source code, etc. -- regardless of whether they're compiled into a single binary
or used as two halves of client/server application -- must be shared. You have
to share the whole pie, not an isolated slice of it.

In addition to these goals, the RPL was authored to meet the requirements of the
Open Source Definition as maintained by the Open Source Initiative (OSI).

The specific terms and conditions of the license are defined in the remainder of
this document.

LICENSE TERMS

   (( 1.0 ))??
   General; Applicability & Definitions. This Reciprocal Public License Version
   1.5 ("License") applies to any programs or other works as well as any and all
   updates or maintenance releases of said programs or works ("Software") not
   already covered by this License which the Software copyright holder
   ("Licensor") makes available containing a License Notice (hereinafter
   defined) from the Licensor specifying or allowing use or distribution under
   the terms of this License. As used in this License:

      (( 1.1 ))??
      "Contributor" means any person or entity who created or contributed to the
      creation of an Extension.

      (( 1.2 ))??
      "Deploy" means to use, Serve, sublicense or distribute Licensed Software
      other than for Your internal Research and/or Personal Use, and includes
      without limitation, any and all internal use or distribution of Licensed
      Software within Your business or organization other than for Research
      and/or Personal Use, as well as direct or indirect sublicensing or
      distribution of Licensed Software by You to any third party in any form or
      manner.

      (( 1.3 ))??
      "Derivative Works" as used in this License is defined under U.S. copyright
      law.

      (( 1.4 ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of data
      such as download from an FTP server or web site, where such mechanism is
      publicly accessible.

      (( 1.5 ))??
      "Extensions" means any Modifications, Derivative Works, or Required
      Components as those terms are defined in this License.

      (( 1.6 ))??
      "License" means this Reciprocal Public License.

      (( 1.7 ))??
      "License Notice" means any notice contained in EXHIBIT A.

      (( 1.8 ))??
      "Licensed Software" means any Software licensed pursuant to this License.
      Licensed Software also includes all previous Extensions from any
      Contributor that You receive.

      (( 1.9 ))??
      "Licensor" means the copyright holder of any Software previously not
      covered by this License who releases the Software under the terms of this
      License.

      (( 1.10 ))??
      "Modifications" means any additions to or deletions from the substance or
      structure of (i) a file or other storage containing Licensed Software, or
      (ii) any new file or storage that contains any part of Licensed Software,
      or (iii) any file or storage which replaces or otherwise alters the
      original functionality of Licensed Software at runtime.

      (( 1.11 ))??
      "Personal Use" means use of Licensed Software by an individual solely for
      his or her personal, private and non-commercial purposes. An individual's
      use of Licensed Software in his or her capacity as an officer, employee,
      member, independent contractor or agent of a corporation, business or
      organization (commercial or non-commercial) does not qualify as Personal
      Use.

      (( 1.12 ))??
      "Required Components" means any text, programs, scripts, schema, interface
      definitions, control files, or other works created by You which are
      required by a third party of average skill to successfully install and run
      Licensed Software containing Your Modifications, or to install and run
      Your Derivative Works.

      (( 1.13 ))??
      "Research" means investigation or experimentation for the purpose of
      understanding the nature and limits of the Licensed Software and its
      potential uses.

      (( 1.14 ))??
      "Serve" means to deliver Licensed Software and/or Your Extensions by means
      of a computer network to one or more computers for purposes of execution
      of Licensed Software and/or Your Extensions.

      (( 1.15 ))??
      "Software" means any computer programs or other works as well as any
      updates or maintenance releases of those programs or works which are
      distributed publicly by Licensor.

      (( 1.16 ))??
      "Source Code" means the preferred form for making modifications to the
      Licensed Software and/or Your Extensions, including all modules contained
      therein, plus any associated text, interface definition files, scripts
      used to control compilation and installation of an executable program or
      other components required by a third party of average skill to build a
      running version of the Licensed Software or Your Extensions.

      (( 1.17 ))??
      "User-Visible Attribution Notice" means any notice contained in EXHIBIT
      B.

      (( 1.18 ))??
      "You" or "Your" means an individual or a legal entity exercising rights
      under this License. For legal entities, "You" or "Your" includes any
      entity which controls, is controlled by, or is under common control with,
      You, where "control" means (a) the power, direct or indirect, to cause the
      direction or management of such entity, whether by contract or otherwise,
      or (b) ownership of fifty percent (50%) or more of the outstanding shares
      or beneficial ownership of such entity.

   (( 2.0 ))??
   Acceptance Of License. You are not required to accept this License since you
   have not signed it, however nothing else grants you permission to use, copy,
   distribute, modify, or create derivatives of either the Software or any
   Extensions created by a Contributor. These actions are prohibited by law if
   you do not accept this License. Therefore, by performing any of these actions
   You indicate Your acceptance of this License and Your agreement to be bound
   by all its terms and conditions. IF YOU DO NOT AGREE WITH ALL THE TERMS AND
   CONDITIONS OF THIS LICENSE DO NOT USE, MODIFY, CREATE DERIVATIVES, OR
   DISTRIBUTE THE SOFTWARE. IF IT IS IMPOSSIBLE FOR YOU TO COMPLY WITH ALL THE
   TERMS AND CONDITIONS OF THIS LICENSE THEN YOU CAN NOT USE, MODIFY, CREATE
   DERIVATIVES, OR DISTRIBUTE THE SOFTWARE.

   (( 3.0 ))??
   Grant of License From Licensor. Subject to the terms and conditions of this
   License, Licensor hereby grants You a world-wide, royalty-free,
   //** non-exclusive **//
   __5__
   license, subject to Licensor's intellectual property rights, and any third
   party intellectual property claims derived from the Licensed Software under
   this License, to do the following:

      (( 3.1 ))??
      Use, reproduce, modify, display, perform, sublicense and distribute
      Licensed Software and Your Extensions in both Source Code form or as an
      executable program.

      (( 3.2 ))??
      Create Derivative Works (as that term is defined under U.S. copyright law)
      of Licensed Software by adding to or deleting from the substance or
      structure of said Licensed Software.

      (( 3.3 ))??
      Under claims of patents now or hereafter owned or controlled by Licensor,
      to make, use, have made, and/or otherwise dispose of Licensed Software or
      portions thereof, but solely to the extent that any such claim is
      necessary to enable You to make, use, have made, and/or otherwise dispose
      of Licensed Software or portions thereof.

      (( 3.4 ))??
      Licensor reserves the right to release new versions of the Software with
      different features, specifications, capabilities, functions, licensing
      terms, general availability or other characteristics. Title, ownership
      rights, and intellectual property rights in and to the Licensed Software
      shall remain in Licensor and/or its Contributors.

   (( 4.0 ))??
   Grant of License From Contributor. By application of the provisions in
   Section 6 below, each Contributor hereby grants You a world-wide,
   //** royalty-free **//
   __5__
   , non-exclusive license, subject to said Contributor's intellectual property
   rights, and any third party intellectual property claims derived from the
   Licensed Software under this License, to do the following:

      (( 4.1 ))??
      Use, reproduce, modify, display, perform, sublicense and distribute any
      Extensions Deployed by such Contributor or portions thereof, in both
      Source Code form or as an executable program, either on an unmodified
      basis or as part of Derivative Works.

      (( 4.2 ))??
      Under claims of patents now or hereafter owned or controlled by
      Contributor, to make, use, have made, and/or otherwise dispose of
      Extensions or portions thereof, but solely to the extent that any such
      claim is necessary to enable You to make, use, have made, and/or otherwise
      dispose of Licensed Software or portions thereof.

   (( 5.0 ))??
   Exclusions From License Grant. Nothing in this License shall be deemed to
   grant any rights to trademarks, copyrights, patents, trade secrets or any
   other intellectual property of Licensor or any Contributor except as
   expressly stated herein. Except as expressly stated in Sections 3 and 4, no
   other patent rights, express or implied, are granted herein. Your Extensions
   may require additional patent licenses from Licensor or Contributors which
   each may grant in its sole discretion. No right is granted to the trademarks
   of Licensor or any Contributor even if such marks are included in the
   Licensed Software. Nothing in this License shall be interpreted to prohibit
   Licensor from licensing under different terms from this License any code that
   Licensor otherwise would have a right to license.

      (( 5.1 ))??
      You expressly acknowledge and agree that although Licensor and each
      Contributor grants the licenses to their respective portions of the
      Licensed Software set forth herein, no assurances are provided by Licensor
      or any Contributor that the Licensed Software does not infringe the patent
      or other intellectual property rights of any other entity. Licensor and
      each Contributor disclaim any liability to You for claims brought by any
      other entity based on infringement of intellectual property rights or
      otherwise. As a condition to exercising the rights and licenses granted
      hereunder, You hereby assume sole responsibility to secure any other
      intellectual property rights needed, if any. For example, if a third party
      patent license is required to allow You to distribute the Licensed
      Software, it is Your responsibility to acquire that license before
      distributing the Licensed Software.

   (( 6.0 ))??
   Your Obligations And Grants. In consideration of, and as an express condition
   to, the licenses granted to You under this License You hereby agree that any
   Modifications, Derivative Works, or Required Components (collectively
   Extensions) that You create or to which You contribute are governed by the
   terms of this License including, without limitation, Section 4. Any
   Extensions that You create or to which You contribute must be Deployed under
   the terms of this License or a future version of this License released under
   Section 7. You hereby grant to Licensor and all third parties a world-wide,
   non-exclusive, royalty-free license under those intellectual property rights
   You own or control to use, reproduce, display, perform, modify, create
   derivatives, sublicense, and distribute Licensed Software, in any form. Any
   Extensions You make and Deploy must have a distinct title so as to readily
   tell any subsequent user or Contributor that the Extensions are by You. You
   must include a copy of this License or directions on how to obtain a copy
   with every copy of the Extensions You distribute. You agree not to offer or
   impose any terms on any Source Code or executable version of the Licensed
   Software, or its Extensions that alter or restrict the applicable version of
   this License or the recipients' rights hereunder.

      (( 6.1 ))??
      Availability of Source Code. You must make available, under the terms of
      this License, the Source Code of any Extensions that You Deploy, via an
      Electronic Distribution Mechanism. The Source Code for any version that
      You Deploy must be made available within one (1) month of when you Deploy
      and must remain available for no less than twelve (12) months after the
      date You cease to Deploy. You are responsible for ensuring that the Source
      Code to each version You Deploy remains available even if the Electronic
      Distribution Mechanism is maintained by a third party. You may not charge
      a fee for any copy of the Source Code distributed under this Section in
      excess of Your actual cost of duplication and distribution of said copy.

      (( 6.2 ))??
      Description of Modifications. You must cause any Modifications that You
      create or to which You contribute to be documented in the Source Code,
      clearly describing the additions, changes or deletions You made. You must
      include a prominent statement that the Modifications are derived, directly
      or indirectly, from the Licensed Software and include the names of the
      Licensor and any Contributor to the Licensed Software in (i) the Source
      Code and (ii) in any notice displayed by the Licensed Software You
      distribute or in related documentation in which You describe the origin or
      ownership of the Licensed Software. You may not modify or delete any
      pre-existing copyright notices, change notices or License text in the
      Licensed Software without written permission of the respective Licensor or
      Contributor.

      (( 6.3 ))??
      Intellectual Property Matters.

         (( a. ))??
         Third Party Claims. If You have knowledge that a license to a third
         party's intellectual property right is required to exercise the rights
         granted by this License, You must include a human-readable file with
         Your distribution that describes the claim and the party making the
         claim in sufficient detail that a recipient will know whom to contact.

         (( b. ))??
         Contributor APIs. If Your Extensions include an application programming
         interface ("API") and You have knowledge of patent licenses that are
         reasonably necessary to implement that API, You must also include this
         information in a human-readable file supplied with Your distribution.

         (( c. ))??
         Representations. You represent that, except as disclosed pursuant to
         6.3(a) above, You believe that any Extensions You distribute are Your
         original creations and that You have sufficient rights to grant the
         rights conveyed by this License.

      (( 6.4 ))??
      Required Notices.

         (( a. ))??
         License Text. You must duplicate this License or instructions on how to
         acquire a copy in any documentation You provide along with the Source
         Code of any Extensions You create or to which You contribute, wherever
         You describe recipients' rights relating to Licensed Software.

         (( b. ))??
         License Notice. You must duplicate any notice contained in EXHIBIT A
         (the "License Notice") in each file of the Source Code of any copy You
         distribute of the Licensed Software and Your Extensions. If You create
         an Extension, You may add Your name as a Contributor to the Source Code
         and accompanying documentation along with a description of the
         contribution. If it is not possible to put the License Notice in a
         particular Source Code file due to its structure, then You must include
         such License Notice in a location where a user would be likely to look
         for such a notice.

         (( c. ))??
         Source Code Availability. You must notify the software community of the
         availability of Source Code to Your Extensions within one (1) month of
         the date You initially Deploy and include in such notification a
         description of the Extensions, and instructions on how to acquire the
         Source Code. Should such instructions change you must notify the
         software community of revised instructions within one (1) month of the
         date of change. You must provide notification by posting to appropriate
         news groups, mailing lists, weblogs, or other sites where a publicly
         accessible search engine would reasonably be expected to index your
         post in relationship to queries regarding the Licensed Software and/or
         Your Extensions.

         (( d. ))??
         User-Visible Attribution. You must duplicate any notice contained in
         EXHIBIT B (the "User-Visible Attribution Notice") in each user-visible
         display of the Licensed Software and Your Extensions which delineates
         copyright, ownership, or similar attribution information. If You create
         an Extension, You may add Your name as a Contributor, and add Your
         attribution notice, as an equally visible and functional element of any
         User-Visible Attribution Notice content. To ensure proper attribution,
         You must also include such User-Visible Attribution Notice in at least
         one location in the Software documentation where a user would be likely
         to look for such notice.

      (( 6.5 ))??
      Additional Terms. You may choose to offer, and charge a fee for, warranty,
      support, indemnity or liability obligations to one or more recipients of
      Licensed Software. However, You may do so only on Your own behalf, and not
      on behalf of the Licensor or any Contributor except as permitted under
      other agreements between you and Licensor or Contributor. You must make it
      clear that any such warranty, support, indemnity or liability obligation
      is offered by You alone, and You hereby agree to indemnify the Licensor
      and every Contributor for any liability plus attorney fees, costs, and
      related expenses due to any such action or claim incurred by the Licensor
      or such Contributor as a result of warranty, support, indemnity or
      liability terms You offer.

      (( 6.6 ))??
      Conflicts With Other Licenses. Where any portion of Your Extensions, by
      virtue of being Derivative Works of another product or similar
      circumstance, fall under the terms of another license, the terms of that
      license should be honored however You must also make Your Extensions
      available under this License. If the terms of this License continue to
      conflict with the terms of the other license you may write the Licensor
      for permission to resolve the conflict in a fashion that remains
      consistent with the intent of this License. Such permission will be
      granted at the sole discretion of the Licensor.

   (( 7.0 ))??
   Versions of This License. Licensor may publish from time to time revised
   versions of the License. Once Licensed Software has been published under a
   particular version of the License, You may always continue to use it under
   the terms of that version. You may also choose to use such Licensed Software
   under the terms of any subsequent version of the License published by
   Licensor. No one other than Licensor has the right to modify the terms
   applicable to Licensed Software created under this License.

      (( 7.1 ))??
      If You create or use a modified version of this License, which You may do
      only in order to apply it to software that is not already Licensed
      Software under this License, You must rename Your license so that it is
      not confusingly similar to this License, and must make it clear that Your
      license contains terms that differ from this License. In so naming Your
      license, You may not use any trademark of Licensor or of any Contributor.
      Should Your modifications to this License be limited to alteration of a)
      Section 13.8 solely to modify the legal Jurisdiction or Venue for
      disputes, b) EXHIBIT A solely to define License Notice text, or c) to
      EXHIBIT B solely to define a User-Visible Attribution Notice, You may
      continue to refer to Your License as the Reciprocal Public License or
      simply the RPL.

   (( 8.0 ))??
   Disclaimer of Warranty. LICENSED SOFTWARE IS PROVIDED UNDER THIS LICENSE ON
   AN "AS IS" BASIS, WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
   INCLUDING, WITHOUT LIMITATION, WARRANTIES THAT THE LICENSED SOFTWARE IS FREE
   OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING.
   FURTHER THERE IS NO WARRANTY MADE AND ALL IMPLIED WARRANTIES ARE DISCLAIMED
   THAT THE LICENSED SOFTWARE MEETS OR COMPLIES WITH ANY DESCRIPTION OF
   PERFORMANCE OR OPERATION, SAID COMPATIBILITY AND SUITABILITY BEING YOUR
   RESPONSIBILITY. LICENSOR DISCLAIMS ANY WARRANTY, IMPLIED OR EXPRESSED, THAT
   ANY CONTRIBUTOR'S EXTENSIONS MEET ANY STANDARD OF COMPATIBILITY OR
   DESCRIPTION OF PERFORMANCE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE
   OF THE LICENSED SOFTWARE IS WITH YOU. SHOULD LICENSED SOFTWARE PROVE
   DEFECTIVE IN ANY RESPECT, YOU (AND NOT THE LICENSOR OR ANY OTHER CONTRIBUTOR)
   ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR OR CORRECTION. UNDER THE
   TERMS OF THIS LICENSOR WILL NOT SUPPORT THIS SOFTWARE AND IS UNDER NO
   OBLIGATION TO ISSUE UPDATES TO THIS SOFTWARE. LICENSOR HAS NO KNOWLEDGE OF
   ERRANT CODE OR VIRUS IN THIS SOFTWARE, BUT DOES NOT WARRANT THAT THE SOFTWARE
   IS FREE FROM SUCH ERRORS OR VIRUSES. THIS DISCLAIMER OF WARRANTY CONSTITUTES
   AN ESSENTIAL PART OF THIS LICENSE. NO USE OF LICENSED SOFTWARE IS AUTHORIZED
   HEREUNDER EXCEPT UNDER THIS DISCLAIMER.

   (( 9.0 ))??
   Limitation of Liability. UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY,
   WHETHER TORT (INCLUDING NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL THE
   LICENSOR, ANY CONTRIBUTOR, OR ANY DISTRIBUTOR OF LICENSED SOFTWARE, OR ANY
   SUPPLIER OF ANY OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT,
   SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING,
   WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER
   FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES,
   EVEN IF SUCH PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH
   DAMAGES. THIS LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH
   OR PERSONAL INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT
   APPLICABLE LAW PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE
   EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS
   EXCLUSION AND LIMITATION MAY NOT APPLY TO YOU.

   (( 10.0 ))??
   High Risk Activities. THE LICENSED SOFTWARE IS NOT FAULT-TOLERANT AND IS NOT
   DESIGNED, MANUFACTURED, OR INTENDED FOR USE OR DISTRIBUTION AS ON-LINE
   CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE PERFORMANCE,
   SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT NAVIGATION OR
   COMMUNICATIONS SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE SUPPORT MACHINES, OR
   WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE LICENSED SOFTWARE COULD LEAD
   DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE PHYSICAL OR ENVIRONMENTAL
   DAMAGE ("HIGH RISK ACTIVITIES"). LICENSOR AND CONTRIBUTORS SPECIFICALLY
   DISCLAIM ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR HIGH RISK
   ACTIVITIES.

   (( 11.0 ))??
   Responsibility for Claims. As between Licensor and Contributors, each party
   is responsible for claims and damages arising, directly or indirectly, out of
   its utilization of rights under this License which specifically disclaims
   warranties and limits any liability of the Licensor. This paragraph is to be
   used in conjunction with and controlled by the Disclaimer Of Warranties of
   Section 8, the Limitation Of Damages in Section 9, and the disclaimer against
   use for High Risk Activities in Section 10. The Licensor has thereby
   disclaimed all warranties and limited any damages that it is or may be liable
   for. You agree to work with Licensor and Contributors to distribute such
   responsibility on an equitable basis consistent with the terms of this
   License including Sections 8, 9, and 10. Nothing herein is intended or shall
   be deemed to constitute any admission of liability.

   (( 12.0 ))??
   Termination. This License and all rights granted hereunder will terminate
   immediately in the event of the circumstances described in Section 13.6 or if
   applicable law prohibits or restricts You from fully and or specifically
   complying with Sections 3, 4 and/or 6, or prevents the enforceability of any
   of those Sections, and You must immediately discontinue any use of Licensed
   Software.

      (( 12.1 ))??
      Automatic Termination Upon Breach. This License and the rights granted
      hereunder will terminate automatically if You fail to comply with the
      terms herein and fail to cure such breach within thirty (30) days of
      becoming aware of the breach. All sublicenses to the Licensed Software
      that are properly granted shall survive any termination of this License.
      Provisions that, by their nature, must remain in effect beyond the
      termination of this License, shall survive.

      (( 12.2 ))??
      Termination Upon Assertion of Patent Infringement. If You initiate
      litigation by asserting a patent infringement claim (excluding declaratory
      judgment actions) against Licensor or a Contributor (Licensor or
      Contributor against whom You file such an action is referred to herein as
      "Respondent") alleging that Licensed Software directly or indirectly
      infringes any patent, then any and all rights granted by such Respondent
      to You under Sections 3 or 4 of this License shall terminate prospectively
      upon sixty (60) days notice from Respondent (the "Notice Period") unless
      within that Notice Period You either agree in writing (i) to pay
      Respondent a mutually agreeable reasonably royalty for Your past or future
      use of Licensed Software made by such Respondent, or (ii) withdraw Your
      litigation claim with respect to Licensed Software against such
      Respondent. If within said Notice Period a reasonable royalty and payment
      arrangement are not mutually agreed upon in writing by the parties or the
      litigation claim is not withdrawn, the rights granted by Licensor to You
      under Sections 3 and 4 automatically terminate at the expiration of said
      Notice Period.

      (( 12.3 ))??
      Reasonable Value of This License. If You assert a patent infringement
      claim against Respondent alleging that Licensed Software directly or
      indirectly infringes any patent where such claim is resolved (such as by
      license or settlement) prior to the initiation of patent infringement
      litigation, then the reasonable value of the licenses granted by said
      Respondent under Sections 3 and 4 shall be taken into account in
      determining the amount or value of any payment or license.

      (( 12.4 ))??
      No Retroactive Effect of Termination. In the event of termination under
      this Section all end user license agreements (excluding licenses to
      distributors and resellers) that have been validly granted by You or any
      distributor hereunder prior to termination shall survive termination.

   (( 13.0 ))??
   Miscellaneous.

      (( 13.1 ))??
      U.S. Government End Users. The Licensed Software is a "commercial item,"
      as that term is defined in 48 C.F.R. 2.101 (Oct. 1995), consisting of
      "commercial computer software" and "commercial computer software
      documentation," as such terms are used in 48 C.F.R. 12.212 (Sept. 1995).
      Consistent with 48 C.F.R. 12.212 and 48 C.F.R. 227.7202-1 through
      227.7202-4 (June 1995), all U.S. Government End Users acquire Licensed
      Software with only those rights set forth herein.

      (( 13.2 ))??
      Relationship of Parties. This License will not be construed as creating an
      agency, partnership, joint venture, or any other form of legal association
      between or among You, Licensor, or any Contributor, and You will not
      represent to the contrary, whether expressly, by implication, appearance,
      or otherwise.

      (( 13.3 ))??
      Independent Development. Nothing in this License will impair Licensor's
      right to acquire, license, develop, subcontract, market, or distribute
      technology or products that perform the same or similar functions as, or
      otherwise compete with, Extensions that You may develop, produce, market,
      or distribute.

      (( 13.4 ))??
      Consent To Breach Not Waiver. Failure by Licensor or Contributor to
      enforce any provision of this License will not be deemed a waiver of
      future enforcement of that or any other provision.

      (( 13.5 ))??
      Severability. This License represents the complete agreement concerning
      the subject matter hereof. If any provision of this License is held to be
      unenforceable, such provision shall be reformed only to the extent
      necessary to make it enforceable.

      (( 13.6 ))??
      Inability to Comply Due to Statute or Regulation. If it is impossible for
      You to comply with any of the terms of this License with respect to some
      or all of the Licensed Software due to statute, judicial order, or
      regulation, then You cannot use, modify, or distribute the software.

      (( 13.7 ))??
      Export Restrictions. You may be restricted with respect to downloading or
      otherwise acquiring, exporting, or reexporting the Licensed Software or
      any underlying information or technology by United States and other
      applicable laws and regulations. By downloading or by otherwise obtaining
      the Licensed Software, You are agreeing to be responsible for compliance
      with all applicable laws and regulations.

      (( 13.8 ))??
      Arbitration, Jurisdiction & Venue. This License shall be governed by
      Colorado law provisions (except to the extent applicable law, if any,
      provides otherwise), excluding its conflict-of-law provisions. You
      expressly agree that any dispute relating to this License shall be
      submitted to binding arbitration under the rules then prevailing of the
      American Arbitration Association. You further agree that Adams County,
      Colorado USA is proper venue and grant such arbitration proceeding
      jurisdiction as may be appropriate for purposes of resolving any dispute
      under this License. Judgement upon any award made in arbitration may be
      entered and enforced in any court of competent jurisdiction. The
      arbitrator shall award attorney's fees and costs of arbitration to the
      prevailing party. Should either party find it necessary to enforce its
      arbitration award or seek specific performance of such award in a civil
      court of competent jurisdiction, the prevailing party shall be entitled to
      reasonable attorney's fees and costs. The application of the United
      Nations Convention on Contracts for the International Sale of Goods is
      expressly excluded. You and Licensor expressly waive any rights to a jury
      trial in any litigation concerning Licensed Software or this License. Any
      law or regulation that provides that the language of a contract shall be
      construed against the drafter shall not apply to this License.

      (( 13.9 ))??
      Entire Agreement. This License constitutes the entire agreement between
      the parties with respect to the subject matter hereof.
      (( EXHIBIT A

The License Notice below must appear in each file of the Source Code of any copy
You distribute of the Licensed Software or any Extensions thereto:

Unless explicitly acquired and licensed from Licensor under another license, the
contents of this file are subject to the Reciprocal Public License ("RPL")
Version 1.5, or subsequent versions as allowed by the RPL, and You may not copy
or use this file in either source code or executable form, except in compliance
with the terms and conditions of the RPL.

All software distributed under the RPL is provided strictly on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LICENSOR HEREBY
DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT, OR
NON-INFRINGEMENT. See the RPL for specific language governing rights and
limitations under the RPL.

EXHIBIT B

The User-Visible Attribution Notice below, when provided, must appear in each
user-visible display as defined in Section 6.4 (d): ))??
`
const license_RPSL_1_0_lre = `//**
RealNetworks Public Source License v1.0
https://spdx.org/licenses/RPSL-1.0.json
https://helixcommunity.org/content/rpsl
https://opensource.org/licenses/RPSL-1.0
**//

(( RealNetworks Public Source License Version 1.0

(Rev. Date October 28, 2002) ))??

   (( 1. ))??
   General Definitions. This License applies to any program or other work which
   RealNetworks, Inc., or any other entity that elects to use this license,
   ("Licensor") makes publicly available and which contains a notice placed by
   Licensor identifying such program or work as "Original Code" and stating that
   it is subject to the terms of this RealNetworks Public Source License version
   1.0 (or subsequent version thereof) ("License"). You are not required to
   accept this License. However, nothing else grants You permission to use,
   copy, modify or distribute the software or its derivative works. These
   actions are prohibited by law if You do not accept this License. Therefore,
   by modifying, copying or distributing the software (or any work based on the
   software), You indicate your acceptance of this License to do so, and all its
   terms and conditions. In addition, you agree to the terms of this License by
   clicking the Accept button or downloading the software. As used in this
   License:

      (( 1.1 ))??
      "Applicable Patent Rights" mean: (a) in the case where Licensor is the
      grantor of rights, claims of patents that (i) are now or hereafter
      acquired, owned by or assigned to Licensor and (ii) are necessarily
      infringed by using or making the Original Code alone and not in
      combination with other software or hardware; and (b) in the case where You
      are the grantor of rights, claims of patents that (i) are now or hereafter
      acquired, owned by or assigned to You and (ii) are infringed (directly or
      indirectly) by using or making Your Modifications, taken alone or in
      combination with Original Code.

      (( 1.2 ))??
      "Compatible Source License" means any one of the licenses listed on
      Exhibit B or at https:/www.helixcommunity.org/content/complicense or other
      licenses specifically identified by Licensor in writing. Notwithstanding
      any term to the contrary in any Compatible Source License, any code
      covered by any Compatible Source License that is used with Covered Code
      must be made readily available in Source Code format for royalty-free use
      under the terms of the Compatible Source License or this License.

      (( 1.3 ))??
      "Contributor" means any person or entity that creates or contributes to
      the creation of Modifications.

      (( 1.4 ))??
      "Covered Code" means the Original Code, Modifications, the combination of
      Original Code and any Modifications, and/or any respective portions
      thereof.

      (( 1.5 ))??
      "Deploy" means to use, sublicense or distribute Covered Code other than
      for Your internal research and development (R&D) and/or Personal Use, and
      includes without limitation, any and all internal use or distribution of
      Covered Code within Your business or organization except for R&D use
      and/or Personal Use, as well as direct or indirect sublicensing or
      distribution of Covered Code by You to any third party in any form or
      manner.

      (( 1.6 ))??
      "Derivative Work" means either the Covered Code or any derivative work
      under United States copyright law, and including any work containing or
      including any portion of the Covered Code or Modifications, either
      verbatim or with modifications and/or translated into another language.
      Derivative Work also includes any work which combines any portion of
      Covered Code or Modifications with code not otherwise governed by the
      terms of this License.

      (( 1.7 ))??
      "Externally Deploy" means to Deploy the Covered Code in any way that may
      be accessed or used by anyone other than You, used to provide any services
      to anyone other than You, or used in any way to deliver any content to
      anyone other than You, whether the Covered Code is distributed to those
      parties, made available as an application intended for use over a computer
      network, or used to provide services or otherwise deliver content to
      anyone other than You.

      (( 1.8. ))??
      "Interface" means interfaces, functions, properties, class definitions,
      APIs, header files, GUIDs, V-Tables, and/or protocols allowing one piece
      of software, firmware or hardware to communicate or interoperate with
      another piece of software, firmware or hardware.

      (( 1.9 ))??
      "Modifications" mean any addition to, deletion from, and/or change to, the
      substance and/or structure of the Original Code, any previous
      Modifications, the combination of Original Code and any previous
      Modifications, and/or any respective portions thereof. When code is
      released as a series of files, a Modification is: (a) any addition to or
      deletion from the contents of a file containing Covered Code; and/or (b)
      any new file or other representation of computer program statements that
      contains any part of Covered Code.

      (( 1.10 ))??
      "Original Code" means (a) the Source Code of a program or other work as
      originally made available by Licensor under this License, including the
      Source Code of any updates or upgrades to such programs or works made
      available by Licensor under this License, and that has been expressly
      identified by Licensor as such in the header file(s) of such work; and (b)
      the object code compiled from such Source Code and originally made
      available by Licensor under this License.

      (( 1.11 ))??
      "Personal Use" means use of Covered Code by an individual solely for his
      or her personal, private and non-commercial purposes. An individual's use
      of Covered Code in his or her capacity as an officer, employee, member,
      independent contractor or agent of a corporation, business or organization
      (commercial or non-commercial) does not qualify as Personal Use.

      (( 1.12 ))??
      "Source Code" means the human readable form of a program or other work
      that is suitable for making modifications to it, including all modules it
      contains, plus any associated interface definition files, scripts used to
      control compilation and installation of an executable (object code).

      (( 1.13 ))??
      "You" or "Your" means an individual or a legal entity exercising rights
      under this License. For legal entities, "You" or "Your" includes any
      entity which controls, is controlled by, or is under common control with,
      You, where "control" means (a) the power, direct or indirect, to cause the
      direction or management of such entity, whether by contract or otherwise,
      or (b) ownership of fifty percent (50%) or more of the outstanding shares
      or beneficial ownership of such entity.

   (( 2. ))??
   Permitted Uses; Conditions & Restrictions. Subject to the terms and
   conditions of this License, Licensor hereby grants You, effective on the date
   You accept this License (via downloading or using Covered Code or otherwise
   indicating your acceptance of this License), a worldwide, royalty-free,
   non-exclusive copyright license, to the extent of Licensor's copyrights cover
   the Original Code, to do the following:

      (( 2.1 ))??
      You may reproduce, display, perform, modify and Deploy Covered Code,
      provided that in each instance:

         (( (a) ))??
         You must retain and reproduce in all copies of Original Code the
         copyright and other proprietary notices and disclaimers of Licensor as
         they appear in the Original Code, and keep intact all notices in the
         Original Code that refer to this License;

         (( (b) ))??
         You must include a copy of this License with every copy of Source Code
         of Covered Code and documentation You distribute, and You may not offer
         or impose any terms on such Source Code that alter or restrict this
         License or the recipients' rights hereunder, except as permitted under
         Section 6;

         (( c ))??
         You must duplicate, to the extent it does not already exist, the notice
         in Exhibit A in each file of the Source Code of all Your Modifications,
         and cause the modified files to carry prominent notices stating that
         You changed the files and the date of any change;

         (( (d) ))??
         You must make Source Code of all Your Externally Deployed Modifications
         publicly available under the terms of this License, including the
         license grants set forth in Section 3 below, for as long as you Deploy
         the Covered Code or twelve (12) months from the date of initial
         Deployment, whichever is longer. You should preferably distribute the
         Source Code of Your Deployed Modifications electronically (e.g.
         download from a web site); and

         (( (e) ))??
         if You Deploy Covered Code in object code, executable form only, You
         must include a prominent notice, in the code itself as well as in
         related documentation, stating that Source Code of the Covered Code is
         available under the terms of this License with information on how and
         where to obtain such Source Code. You must also include the Object Code
         Notice set forth in Exhibit A in the "about" box or other appropriate
         place where other copyright notices are placed, including any packaging
         materials.

      (( 2.2 ))??
      You expressly acknowledge and agree that although Licensor and each
      Contributor grants the licenses to their respective portions of the
      Covered Code set forth herein, no assurances are provided by Licensor or
      any Contributor that the Covered Code does not infringe the patent or
      other intellectual property rights of any other entity. Licensor and each
      Contributor disclaim any liability to You for claims brought by any other
      entity based on infringement of intellectual property rights or otherwise.
      As a condition to exercising the rights and licenses granted hereunder,
      You hereby assume sole responsibility to secure any other intellectual
      property rights needed, if any. For example, if a third party patent
      license is required to allow You to make, use, sell, import or offer for
      sale the Covered Code, it is Your responsibility to acquire such
      license(s).

      (( 2.3 ))??
      Subject to the terms and conditions of this License, Licensor hereby
      grants You, effective on the date You accept this License (via downloading
      or using Covered Code or otherwise indicating your acceptance of this
      License), a worldwide, royalty-free, perpetual, non-exclusive patent
      license under Licensor's Applicable Patent Rights to make, use, sell,
      offer for sale and import the Covered Code, provided that in each instance
      you comply with the terms of this License.

   (( 3. ))??
   Your Grants. In consideration of, and as a condition to, the licenses granted
   to You under this License:

      (( (a) ))??
      You grant to Licensor and all third parties a non-exclusive, perpetual,
      irrevocable, royalty free license under Your Applicable Patent Rights and
      other intellectual property rights owned or controlled by You, to make,
      sell, offer for sale, use, import, reproduce, display, perform, modify,
      distribute and Deploy Your Modifications of the same scope and extent as
      Licensor's licenses under Sections 2.1 and 2.2; and

      (( (b) ))??
      You grant to Licensor and its subsidiaries a non-exclusive, worldwide,
      royalty-free, perpetual and irrevocable license, under Your Applicable
      Patent Rights and other intellectual property rights owned or controlled
      by You, to make, use, sell, offer for sale, import, reproduce, display,
      perform, distribute, modify or have modified (for Licensor and/or its
      subsidiaries), sublicense and distribute Your Modifications, in any form
      and for any purpose, through multiple tiers of distribution.

      (( c ))??
      You agree not use any information derived from Your use and review of the
      Covered Code, including but not limited to any algorithms or inventions
      that may be contained in the Covered Code, for the purpose of asserting
      any of Your patent rights, or assisting a third party to assert any of its
      patent rights, against Licensor or any Contributor.

   (( 4. ))??
   Derivative Works. You may create a Derivative Work by combining Covered Code
   with other code not otherwise governed by the terms of this License and
   distribute the Derivative Work as an integrated product. In each such
   instance, You must make sure the requirements of this License are fulfilled
   for the Covered Code or any portion thereof, including all Modifications.

      (( 4.1 ))??
      You must cause any Derivative Work that you distribute, publish or
      Externally Deploy, that in whole or in part contains or is derived from
      the Covered Code or any part thereof, to be licensed as a whole at no
      charge to all third parties under the terms of this License and no other
      license except as provided in Section 4.2. You also must make Source Code
      available for the Derivative Work under the same terms as Modifications,
      described in Sections 2 and 3, above.

      (( 4.2 ))??
      Compatible Source Licenses. Software modules that have been independently
      developed without any use of Covered Code and which contain no portion of
      the Covered Code, Modifications or other Derivative Works, but are used or
      combined in any way
      //** with **//
      __5__
      the Covered Code or any Derivative Work to form a larger Derivative Work,
      are exempt from the conditions described in Section 4.1 but only to the
      extent that: the software module, including any software that is linked
      to, integrated with, or part of the same applications as, the software
      module by any method must be wholly subject to one of the Compatible
      Source Licenses. Notwithstanding the foregoing, all Covered Code must be
      subject to the terms of this License. Thus, the entire Derivative Work
      must be licensed under a combination of the RPSL (for Covered Code) and a
      Compatible Source License for any independently developed software modules
      within the Derivative Work. The foregoing requirement applies even if the
      Compatible Source License would ordinarily allow the software module to
      link with, or form larger works with, other software that is not subject
      to the Compatible Source License. For example, although the Mozilla Public
      License v1.1 allows Mozilla code to be combined with proprietary software
      that is not subject to the MPL, if MPL-licensed code is used with Covered
      Code the MPL-licensed code could not be combined or linked with any code
      not governed by the MPL. The general intent of this section 4.2 is to
      enable use of Covered Code with applications that are wholly subject to an
      acceptable open source license. You are responsible for determining
      whether your use of software with Covered Code is allowed under Your
      license to such software.

      (( 4.3 ))??
      Mere aggregation of another work not based on the Covered Code with the
      Covered Code (or with a work based on the Covered Code) on a volume of a
      storage or distribution medium does not bring the other work under the
      scope of this License. If You deliver the Covered Code for combination
      and/or integration with an application previously provided by You (for
      example, via automatic updating technology), such combination and/or
      integration constitutes a Derivative Work subject to the terms of this
      License.

   (( 5. ))??
   Exclusions From License Grant. Nothing in this License shall be deemed to
   grant any rights to trademarks, copyrights, patents, trade secrets or any
   other intellectual property of Licensor or any Contributor except as
   expressly stated herein. No right is granted to the trademarks of Licensor or
   any Contributor even if such marks are included in the Covered Code. Nothing
   in this License shall be interpreted to prohibit Licensor from licensing
   under different terms from this License any code that Licensor otherwise
   would have a right to license. Modifications, Derivative Works and/or any use
   or combination of Covered Code with other technology provided by Licensor or
   third parties may require additional patent licenses from Licensor which
   Licensor may grant in its sole discretion. No patent license is granted
   separate from the Original Code or combinations of the Original Code with
   other software or hardware.

      (( 5.1. ))??
      Trademarks. This License does not grant any rights to use the trademarks
      or trade names owned by Licensor ("Licensor Marks" defined in Exhibit C)
      or to any trademark or trade name belonging to any Contributor. No
      Licensor Marks may be used to endorse or promote products derived from the
      Original Code other than as permitted by the Licensor Trademark Policy
      defined in Exhibit C.

   (( 6. ))??
   Additional Terms. You may choose to offer, and to charge a fee for, warranty,
   support, indemnity or liability obligations and/or other rights consistent
   with the scope of the license granted herein ("Additional Terms") to one or
   more recipients of Covered Code. However, You may do so only on Your own
   behalf and as Your sole responsibility, and not on behalf of Licensor or any
   Contributor. You must obtain the recipient's agreement that any such
   Additional Terms are offered by You alone, and You hereby agree to indemnify,
   defend and hold Licensor and every Contributor harmless for any liability
   incurred by or claims asserted against Licensor or such Contributor by reason
   of any such Additional Terms.

   (( 7. ))??
   Versions of the License. Licensor may publish revised and/or new versions of
   this License from time to time. Each version will be given a distinguishing
   version number. Once Original Code has been published under a particular
   version of this License, You may continue to use it under the terms of that
   version. You may also choose to use such Original Code under the terms of any
   subsequent version of this License published by Licensor. No one other than
   Licensor has the right to modify the terms applicable to Covered Code created
   under this License.

   (( 8. ))??
   NO WARRANTY OR SUPPORT. The Covered Code may contain in whole or in part
   pre-release, untested, or not fully tested works. The Covered Code may
   contain errors that could cause failures or loss of data, and may be
   incomplete or contain inaccuracies. You expressly acknowledge and agree that
   use of the Covered Code, or any portion thereof, is at Your sole and entire
   risk. THE COVERED CODE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, UPGRADES OR
   SUPPORT OF ANY KIND AND LICENSOR AND LICENSOR'S LICENSOR(S) (COLLECTIVELY
   REFERRED TO AS "LICENSOR" FOR THE PURPOSES OF SECTIONS 8 AND 9) AND ALL
   CONTRIBUTORS EXPRESSLY DISCLAIM ALL WARRANTIES AND/OR CONDITIONS, EXPRESS OR
   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES AND/OR
   CONDITIONS OF MERCHANTABILITY, OF SATISFACTORY QUALITY, OF FITNESS FOR A
   PARTICULAR PURPOSE, OF ACCURACY, OF QUIET ENJOYMENT, AND NONINFRINGEMENT OF
   THIRD PARTY RIGHTS. LICENSOR AND EACH CONTRIBUTOR DOES NOT WARRANT AGAINST
   INTERFERENCE WITH YOUR ENJOYMENT OF THE COVERED CODE, THAT THE FUNCTIONS
   CONTAINED IN THE COVERED CODE WILL MEET YOUR REQUIREMENTS, THAT THE OPERATION
   OF THE COVERED CODE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN
   THE COVERED CODE WILL BE CORRECTED. NO ORAL OR WRITTEN DOCUMENTATION,
   INFORMATION OR ADVICE GIVEN BY LICENSOR, A LICENSOR AUTHORIZED REPRESENTATIVE
   OR ANY CONTRIBUTOR SHALL CREATE A WARRANTY. You acknowledge that the Covered
   Code is not intended for use in high risk activities, including, but not
   limited to, the design, construction, operation or maintenance of nuclear
   facilities, aircraft navigation, aircraft communication systems, or air
   traffic control machines in which case the failure of the Covered Code could
   lead to death, personal injury, or severe physical or environmental damage.
   Licensor disclaims any express or implied warranty of fitness for such uses.

   (( 9. ))??
   LIMITATION OF LIABILITY. TO THE EXTENT NOT PROHIBITED BY LAW, IN NO EVENT
   SHALL LICENSOR OR ANY CONTRIBUTOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
   INDIRECT OR CONSEQUENTIAL DAMAGES ARISING OUT OF OR RELATING TO THIS LICENSE
   OR YOUR USE OR INABILITY TO USE THE COVERED CODE, OR ANY PORTION THEREOF,
   WHETHER UNDER A THEORY OF CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR
   STRICT LIABILITY), PRODUCTS LIABILITY OR OTHERWISE, EVEN IF LICENSOR OR SUCH
   CONTRIBUTOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES AND
   NOTWITHSTANDING THE FAILURE OF ESSENTIAL PURPOSE OF ANY REMEDY. SOME
   JURISDICTIONS DO NOT ALLOW THE LIMITATION OF LIABILITY OF INCIDENTAL OR
   CONSEQUENTIAL DAMAGES, SO THIS LIMITATION MAY NOT APPLY TO YOU. In no event
   shall Licensor's total liability to You for all damages (other than as may be
   required by applicable law) under this License exceed the amount of ten
   dollars ($10.00).

   (( 10. ))??
   Ownership. Subject to the licenses granted under this License, each
   Contributor retains all rights, title and interest in and to any
   Modifications made by such Contributor. Licensor retains all rights, title
   and interest in and to the Original Code and any Modifications made by or on
   behalf of Licensor ("Licensor Modifications"), and such Licensor
   Modifications will not be automatically subject to this License. Licensor
   may, at its sole discretion, choose to license such Licensor Modifications
   under this License, or on different terms from those contained in this
   License or may choose not to license them at all.

   (( 11. ))??
   Termination.

      (( 11.1 ))??
      Term and Termination. The term of this License is perpetual unless
      terminated as provided below. This License and the rights granted
      hereunder will terminate:

         (( (a) ))??
         automatically without notice from Licensor if You fail to comply with
         any term(s) of this License and fail to cure such breach within 30 days
         of becoming aware of such breach;

         (( (b) ))??
         immediately in the event of the circumstances described in Section
         12.5(b); or

         (( c ))??
         automatically without notice from Licensor if You, at any time during
         the term of this License, commence an action for patent infringement
         against Licensor (including by cross-claim or counter claim in a
         lawsuit);

         (( (d) ))??
         upon written notice from Licensor if You, at any time during the term
         of this License, commence an action for patent infringement against any
         third party alleging that the Covered Code itself (excluding
         combinations with other software or hardware) infringes any patent
         (including by cross-claim or counter claim in a lawsuit).

      (( 11.2 ))??
      Effect of Termination. Upon termination, You agree to immediately stop any
      further use, reproduction, modification, sublicensing and distribution of
      the Covered Code and to destroy all copies of the Covered Code that are in
      your possession or control. All sublicenses to the Covered Code which have
      been properly granted prior to termination shall survive any termination
      of this License. Provisions which, by their nature, should remain in
      effect beyond the termination of this License shall survive, including but
      not limited to Sections 3, 5, 8, 9, 10, 11, 12.2 and 13. No party will be
      liable to any other for compensation, indemnity or damages of any sort
      solely as a result of terminating this License in accordance with its
      terms, and termination of this License will be without prejudice to any
      other right or remedy of any party.

   (( 12. ))??
   Miscellaneous.

      (( 12.1 ))??
      Government End Users. The Covered Code is a "commercial item" as defined
      in FAR 2.101. Government software and technical data rights in the Covered
      Code include only those rights customarily provided to the public as
      defined in this License. This customary commercial license in technical
      data and software is provided in accordance with FAR 12.211 (Technical
      Data) and 12.212 (Computer Software) and, for Department of Defense
      purchases, DFAR 252.227-7015 (Technical Data -- Commercial Items) and
      227.7202-3 (Rights in Commercial Computer Software or Computer Software
      Documentation). Accordingly, all U.S. Government End Users acquire Covered
      Code with only those rights set forth herein.

      (( 12.2 ))??
      Relationship of Parties. This License will not be construed as creating an
      agency, partnership, joint venture or any other form of legal association
      between or among You, Licensor or any Contributor, and You will not
      represent to the contrary, whether expressly, by implication, appearance
      or otherwise.

      (( 12.3 ))??
      Independent Development. Nothing in this License will impair Licensor's
      right to acquire, license, develop, have others develop for it, market
      and/or distribute technology or products that perform the same or similar
      functions as, or otherwise compete with, Modifications, Derivative Works,
      technology or products that You may develop, produce, market or
      distribute.

      (( 12.4 ))??
      Waiver; Construction. Failure by Licensor or any Contributor to enforce
      any provision of this License will not be deemed a waiver of future
      enforcement of that or any other provision. Any law or regulation which
      provides that the language of a contract shall be construed against the
      drafter will not apply to this License.

      (( 12.5 ))??
      Severability. (a) If for any reason a court of competent jurisdiction
      finds any provision of this License, or portion thereof, to be
      unenforceable, that provision of the License will be enforced to the
      maximum extent permissible so as to effect the economic benefits and
      intent of the parties, and the remainder of this License will continue in
      full force and effect. (b) Notwithstanding the foregoing, if applicable
      law prohibits or restricts You from fully and/or specifically complying
      with Sections 2 and/or 3 or prevents the enforceability of either of those
      Sections, this License will immediately terminate and You must immediately
      discontinue any use of the Covered Code and destroy all copies of it that
      are in your possession or control.

      (( 12.6 ))??
      Dispute Resolution. Any litigation or other dispute resolution between You
      and Licensor relating to this License shall take place in the Seattle,
      Washington, and You and Licensor hereby consent to the personal
      jurisdiction of, and venue in, the state and federal courts within that
      District with respect to this License. The application of the United
      Nations Convention on Contracts for the International Sale of Goods is
      expressly excluded.

      (( 12.7 ))??
      Export/Import Laws. This software is subject to all export and import laws
      and restrictions and regulations of the country in which you receive the
      Covered Code and You are solely responsible for ensuring that You do not
      export, re-export or import the Covered Code or any direct product thereof
      in violation of any such restrictions, laws or regulations, or without all
      necessary authorizations.

      (( 12.8 ))??
      Entire Agreement; Governing Law. This License constitutes the entire
      agreement between the parties with respect to the subject matter hereof.
      This License shall be governed by the laws of the United States and the
      State of Washington.

      Where You are located in the province of Quebec, Canada, the following
      clause applies: The parties hereby confirm that they have requested that
      this License and all related documents be drafted in English. Les parties
      ont exigé que le présent contrat et tous les documents connexes soient
      rédigés en anglais.
      (( EXHIBIT A.

Copyright (c) 1995-2002 RealNetworks, Inc. and/or its licensors. All Rights
Reserved.

The contents of this file, and the files included with this file, are subject to
the current version of the RealNetworks Public Source License Version 1.0 (the
"RPSL") available at https:/www.helixcommunity.org/content/rpsl unless you have
licensed the file under the RealNetworks Community Source License Version 1.0
(the "RCSL") available at https:/www.helixcommunity.org/content/rcsl, in which
case the RCSL will apply. You may also obtain the license terms directly from
RealNetworks. You may not use this file except in compliance with the RPSL or,
if you have a valid RCSL with RealNetworks applicable to this file, the RCSL.
Please see the applicable RPSL or RCSL for the rights, obligations and
limitations governing use of the contents of the file.

This file is part of the Helix DNA Technology. RealNetworks is the developer of
the Original code and owns the copyrights in the portions it created.

This file, and the files included with this file, is distributed and made
available on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
IMPLIED, AND REALNETWORKS HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING
WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.

Contributor(s):
//** ____________________________________ **//
__5__

Technology Compatibility Kit Test Suite(s) Location (if licensed under the
RCSL):
//** _____ **//
__5__

Object Code Notice: Helix DNA Client technology included. Copyright (c)
RealNetworks, Inc., 1995-2002. All rights reserved.

EXHIBIT B

Compatible Source Licenses for the RealNetworks Public Source License. The
following list applies to the most recent version of the license as of October
25, 2002, unless otherwise indicated.

Academic Free License

Apache Software License

Apple Public Source License

Artistic license

Attribution Assurance Licenses

BSD license

Common Public License1

Eiffel Forum License

GNU General Public License (GPL)1

GNU Library or "Lesser" General Public License (LGPL)1

IBM Public License

Intel Open Source License

Jabber Open Source License

MIT license

MITRE Collaborative Virtual Workspace License (CVW License)

Motosoto License

Mozilla Public License 1.0 (MPL)

Mozilla Public License 1.1 (MPL)

Nokia Open Source License

Open Group Test Suite License

Python Software Foundation License

Ricoh Source Code Public License

Sun Industry Standards Source License (SISSL)

Sun Public License

University of Illinois/NCSA Open Source License

Vovida Software License v. 1.0

W3C License

X.Net License

Zope Public License

zlib/libpng license

1Note: because this license contains certain reciprocal licensing terms that
purport to extend to independently developed code, You may be prohibited under
the terms of this otherwise compatible license from using code licensed under
its terms with Covered Code because Covered Code may only be licensed under the
RealNetworks Public Source License. Any attempt to apply non RPSL license terms,
including without limitation the GPL, to Covered Code is expressly forbidden.
You are responsible for ensuring that Your use of Compatible Source Licensed
code does not violate either the RPSL or the Compatible Source License.

The latest version of this list can be found at:
https:/www.helixcommunity.org/content/complicense

EXHIBIT C

RealNetworks' Trademark policy.

RealNetworks defines the following trademarks collectively as "Licensor
Trademarks": "RealNetworks", "RealPlayer", "RealJukebox", "RealSystem",
"RealAudio", "RealVideo", "RealOne Player", "RealMedia", "Helix" or any other
trademarks or trade names belonging to RealNetworks.

RealNetworks "Licensor Trademark Policy" forbids any use of Licensor Trademarks
except as permitted by and in strict compliance at all times with RealNetworks'
third party trademark usage guidelines which are posted at
www.realnetworks.com/info/helixlogo.html. ))??
`
const license_RSA_MD_lre = `//**
RSA Message-Digest License
https://spdx.org/licenses/RSA-MD.json
http://www.faqs.org/rfcs/rfc1321.html
**//

//** Copyright **//

License to copy and use this software is granted provided that it is identified
as the "RSA Data Security, Inc.
//** MD5 **//
__5__
Message-Digest Algorithm" in all material mentioning or referencing this
software or this function.

License is also granted to make and use derivative works provided that such
works are identified as "derived from the RSA Data Security, Inc.
//** MD5 **//
__5__
Message-Digest Algorithm" in all material mentioning or referencing the derived
work.

RSA Data Security, Inc. makes no representations concerning either the
merchantability of this software or the suitability of this software for any
particular purpose. It is provided "as is" without express or implied warranty
of any kind.

These notices must be retained in any copies of any part of this documentation
and/or software.
`
const license_RSCPL_lre = `//**
Ricoh Source Code Public License
https://spdx.org/licenses/RSCPL.json
http://wayback.archive.org/web/20060715140826/http://www.risource.org/RPL/RPL-1.0A.shtml
https://opensource.org/licenses/RSCPL
**//

(( Ricoh Source Code Public License

Version 1.0 ))??

   (( 1. ))??
   Definitions.

      (( 1.1. ))??
      "Contributor" means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      "Electronic Distribution Mechanism" means a website or any other mechanism
      generally accepted in the software development community for the
      electronic transfer of data.

      (( 1.4. ))??
      "Executable Code" means Governed Code in any form other than Source Code.

      (( 1.5. ))??
      "Governed Code" means the Original Code or Modifications or the
      combination of the Original Code and Modifications, in each case including
      portions thereof.

      (( 1.6. ))??
      "Larger Work" means a work which combines Governed Code or portions
      thereof with code not governed by the terms of this License.

      (( 1.7. ))??
      "Licensable" means the right to grant, to the maximum extent possible,
      whether at the time of the initial grant or subsequently acquired, any and
      all of the rights conveyed herein.

      (( 1.8. ))??
      "License" means this document.

      (( 1.9. ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Governed Code is released as a series of files, a Modification is:

         (( (a) ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( (b) ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10. ))??
      "Original Code" means the "Platform for Information Applications" Source
      Code as released under this License by RSV.

      (( 1.11 ))??
      "Patent Claims" means any patent claim(s), now owned or hereafter
      acquired, including without limitation, method, process, and apparatus
      claims, in any patent Licensable by the grantor of a license thereto.

      (( 1.12. ))??
      "RSV" means Ricoh Silicon Valley, Inc., a California corporation with
      offices at 2882 Sand Hill Road, Suite 115, Menlo Park, CA 94025-7022.

      (( 1.13. ))??
      "Source Code" means the preferred form of the Governed Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of Executable Code, or a list of source code differential
      comparisons against either the Original Code or another well known,
      available Governed Code of the Contributor's choice. The Source Code can
      be in a compressed or archival form, provided the appropriate
      decompression or de-archiving software is widely available for no charge.

      (( 1.14. ))??
      "You" means an individual or a legal entity exercising rights under, and
      complying with all of the terms of, this License or a future version of
      this License issued under Section 6.1. For legal entities, "You" includes
      any entity which controls, is controlled by, or is under common control
      with You. For purposes of this definition, "control" means (a) the power,
      direct or indirect, to cause the direction or management of such entity,
      whether by contract or otherwise, or (b) ownership of fifty percent (50%)
      or more of the outstanding shares or beneficial ownership of such entity.

   (( 2. ))??
   Source Code License.

      (( 2.1. ))??
      Grant from RSV. RSV hereby grants You a worldwide, royalty-free,
      non-exclusive license, subject to third party intellectual property
      claims:

         (( (a) ))??
         to use, reproduce, modify, create derivative works of, display,
         perform, sublicense and distribute the Original Code (or portions
         thereof) with or without Modifications, or as part of a Larger Work;
         and

         (( (b) ))??
         under Patent Claims infringed by the making, using or selling of
         Original Code, to make, have made, use, practice, sell, and offer for
         sale, and/or otherwise dispose of the Original Code (or portions
         thereof).

      (( 2.2. ))??
      Contributor Grant. Each Contributor hereby grants You a worldwide,
      royalty-free, non-exclusive license, subject to third party intellectual
      property claims:

         (( (a) ))??
         to use, reproduce, modify, create derivative works of, display,
         perform, sublicense and distribute the Modifications created by such
         Contributor (or portions thereof) either on an unmodified basis, with
         other Modifications, as Governed Code or as part of a Larger Work; and

         (( (b) ))??
         under Patent Claims infringed by the making, using, or selling of
         Modifications made by that Contributor either alone and/or in
         combination with its Contributor Version (or portions of such
         combination), to make, use, sell, offer for sale, have made, and/or
         otherwise dispose of: (i) Modifications made by that Contributor (or
         portions thereof); and (ii) the combination of Modifications made by
         that Contributor with its Contributor Version (or portions of such
         combination).

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Application of License. The Modifications which You create or to which You
      contribute are governed by the terms of this License, including without
      limitation Section 2.2. The Source Code version of Governed Code may be
      distributed only under the terms of this License or a future version of
      this License released under Section 6.1, and You must include a copy of
      this License with every copy of the Source Code You distribute. You may
      not offer or impose any terms on any Source Code version that alters or
      restricts the applicable version of this License or the recipients' rights
      hereunder. However, You may include an additional document offering the
      additional rights described in Section 3.5.

      (( 3.2. ))??
      Availability of Source Code. Any Modification which You create or to which
      You contribute must be made available in Source Code form under the terms
      of this License either on the same media as an Executable Code version or
      via an Electronic Distribution Mechanism to anyone to whom you made an
      Executable Code version available; and if made available via an Electronic
      Distribution Mechanism, must remain available for at least twelve (12)
      months after the date it initially became available, or at least six (6)
      months after a subsequent version of that particular Modification has been
      made available to such recipients. You are responsible for ensuring that
      the Source Code version remains available even if the Electronic
      Distribution Mechanism is maintained by a third party.

      (( 3.3. ))??
      Description of Modifications. You must cause all Governed Code to which
      you contribute to contain a file documenting the changes You made to
      create that Governed Code and the date of any change. You must include a
      prominent statement that the Modification is derived, directly or
      indirectly, from Original Code provided by RSV and including the name of
      RSV in (a) the Source Code, and (b) in any notice in an Executable Code
      version or related documentation in which You describe the origin or
      ownership of the Governed Code.

      (( 3.4. ))??
      Intellectual Property Matters.

         (( 3.4.1. ))??
         Third Party Claims. If You have knowledge that a party claims an
         intellectual property right in particular functionality or code (or its
         utilization under this License), you must include a text file with the
         source code distribution titled "LEGAL" which describes the claim and
         the party making the claim in sufficient detail that a recipient will
         know whom to contact. If you obtain such knowledge after You make Your
         Modification available as described in Section 3.2, You shall promptly
         modify the LEGAL file in all copies You make available thereafter and
         shall take other steps (such as notifying RSV and appropriate mailing
         lists or newsgroups) reasonably calculated to inform those who received
         the Governed Code that new knowledge has been obtained. In the event
         that You are a Contributor, You represent that, except as disclosed in
         the LEGAL file, your Modifications are your original creations and, to
         the best of your knowledge, no third party has any claim (including but
         not limited to intellectual property claims) relating to your
         Modifications. You represent that the LEGAL file includes complete
         details of any license or other restriction associated with any part of
         your Modifications.

         (( 3.4.2. ))??
         Contributor APIs. If Your Modification is an application programming
         interface and You own or control patents which are reasonably necessary
         to implement that API, you must also include this information in the
         LEGAL file.

      (( 3.5. ))??
      Required Notices. You must duplicate the notice in Exhibit A in each file
      of the Source Code, and this License in any documentation for the Source
      Code, where You describe recipients' rights relating to Governed Code. If
      You created one or more Modification(s), You may add your name as a
      Contributor to the notice described in Exhibit A. If it is not possible to
      put such notice in a particular Source Code file due to its structure,
      then you must include such notice in a location (such as a relevant
      directory file) where a user would be likely to look for such a notice.
      You may choose to offer, and to charge a fee for, warranty, support,
      indemnity or liability obligations to one or more recipients of Governed
      Code. However, You may do so only on Your own behalf, and not on behalf of
      RSV or any Contributor. You must make it absolutely clear than any such
      warranty, support, indemnity or liability obligation is offered by You
      alone, and You hereby agree to indemnify RSV and every Contributor for any
      liability incurred by RSV or such Contributor as a result of warranty,
      support, indemnity or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Code Versions. You may distribute Governed Code
      in Executable Code form only if the requirements of Section 3.1-3.5 have
      been met for that Governed Code, and if You include a prominent notice
      stating that the Source Code version of the Governed Code is available
      under the terms of this License, including a description of how and where
      You have fulfilled the obligations of Section 3.2. The notice must be
      conspicuously included in any notice in an Executable Code version,
      related documentation or collateral in which You describe recipients'
      rights relating to the Governed Code. You may distribute the Executable
      Code version of Governed Code under a license of Your choice, which may
      contain terms different from this License, provided that You are in
      compliance with the terms of this License and that the license for the
      Executable Code version does not attempt to limit or alter the recipient's
      rights in the Source Code version from the rights set forth in this
      License. If You distribute the Executable Code version under a different
      license You must make it absolutely clear that any terms which differ from
      this License are offered by You alone, not by RSV or any Contributor. You
      hereby agree to indemnify RSV and every Contributor for any liability
      incurred by RSV or such Contributor as a result of any such terms You
      offer.

      (( 3.7. ))??
      Larger Works. You may create a Larger Work by combining Governed Code with
      other code not governed by the terms of this License and distribute the
      Larger Work as a single product. In such a case, You must make sure the
      requirements of this License are fulfilled for the Governed Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for You to comply with any of theterms of this License
   with respect to some or all of the Governed Code due to statute or regulation
   then You must: (a) comply with the terms of this License to the maximum
   extent possible; and (b) describe the limitations and the code they affect.
   Such description must be included in the LEGAL file described in Section 3.4
   and must be included with all distributions of the Source Code. Except to the
   extent prohibited by statute or regulation, such description must be
   sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Trademark Usage.

      (( 5.1. ))??
      Advertising Materials. All advertising materials mentioning features or
      use of the Governed Code must display the following acknowledgement: "This
      product includes software developed by Ricoh Silicon Valley, Inc."

      (( 5.2. ))??
      Endorsements. The names "Ricoh," "Ricoh Silicon Valley," and "RSV" must
      not be used to endorse or promote Contributor Versions or Larger Works
      without the prior written permission of RSV.

      (( 5.3. ))??
      Product Names. Contributor Versions and Larger Works may not be called
      "Ricoh" nor may the word "Ricoh" appear in their names without the prior
      written permission of RSV.

   (( 6. ))??
   Versions of the License.

      (( 6.1. ))??
      New Versions. RSV may publish revised and/or new versions of the License
      from time to time. Each version will be given a distinguishing version
      number.

      (( 6.2. ))??
      Effect of New Versions. Once Governed Code has been published under a
      particular version of the License, You may always continue to use it under
      the terms of that version. You may also choose to use such Governed Code
      under the terms of any subsequent version of the License published by RSV.
      No one other than RSV has the right to modify the terms applicable to
      Governed Code created under this License.

   (( 7. ))??
   Disclaimer of Warranty.

   GOVERNED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE GOVERNED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE GOVERNED CODE IS WITH YOU. SHOULD
   ANY GOVERNED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT RSV OR ANY OTHER
   CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR OR
   CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS
   LICENSE. NO USE OF ANY GOVERNED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

   (( 8. ))??
   Termination.

      (( 8.1. ))??
      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. All sublicenses to the
      Governed Code which are properly granted shall survive any termination of
      this License. Provisions which, by their nature, must remain in effect
      beyond the termination of this License shall survive.

      (( 8.2. ))??
      If You initiate patent infringement litigation against RSV or a
      Contributor (RSV or the Contributor against whom You file such action is
      referred to as "Participant") alleging that:

         (( (a) ))??
         such Participant's Original Code or Contributor Version directly or
         indirectly infringes any patent, then any and all rights granted by
         such Participant to You under Sections 2.1 and/or 2.2 of this License
         shall, upon 60 days notice from Participant terminate prospectively,
         unless if within 60 days after receipt of notice You either: (i) agree
         in writing to pay Participant a mutually agreeable reasonable royalty
         for Your past and future use of the Original Code or the Modifications
         made by such Participant, or (ii) withdraw Your litigation claim with
         respect to the Original Code or the Contributor Version against such
         Participant. If within 60 days of notice, a reasonable royalty and
         payment arrangement are not mutually agreed upon in writing by the
         parties or the litigation claim is not withdrawn, the rights granted by
         Participant to You under Sections 2.1 and/or 2.2 automatically
         terminate at the expiration of the 60 day notice period specified
         above.

         (( (b) ))??
         any software, hardware, or device provided to You by the Participant,
         other than such Participant's Original Code or Contributor Version,
         directly or indirectly infringes any patent, then any rights granted to
         You by such Participant under Sections 2.1(b) and 2.2(b) are revoked
         effective as of the date You first made, used, sold, distributed, or
         had made, Original Code or the Modifications made by that Participant.

      (( 8.3. ))??
      If You assert a patent infringement claim against Participant alleging
      that such Participant's Original Code or Contributor Version directly or
      indirectly infringes any patent where such claim is resolved (such as by
      license or settlement) prior to the initiation of patent infringement
      litigation, then the reasonable value of the licenses granted by such
      Participant under Sections 2.1 or 2.2 shall be taken into account in
      determining the amount or value of any payment or license.

      (( 8.4. ))??
      In the event of termination under Sections 8.1 or 8.2 above, all end user
      license agreements (excluding distributors and resellers) which have been
      validly granted by You or any distributor hereunder prior to termination
      shall survive termination.

   (( 9. ))??
   Limitation of Liability.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL RSV, ANY CONTRIBUTOR, OR ANY
   DISTRIBUTOR OF GOVERNED CODE, OR ANY SUPPLIER OF ANY OF SUCH PARTIES, BE
   LIABLE TO YOU OR ANY OTHER PERSON FOR ANY DIRECT, INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THAT EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU. TO THE EXTENT THAT ANY EXCLUSION OF DAMAGES
   ABOVE IS NOT VALID, YOU AGREE THAT IN NO EVENT WILL RSVS LIABILITY UNDER OR
   RELATED TO THIS AGREEMENT EXCEED FIVE THOUSAND DOLLARS ($5,000). THE GOVERNED
   CODE IS NOT INTENDED FOR USE IN CONNECTION WITH ANY NUCLER, AVIATION, MASS
   TRANSIT OR MEDICAL APPLICATION OR ANY OTHER INHERENTLY DANGEROUS APPLICATION
   THAT COULD RESULT IN DEATH, PERSONAL INJURY, CATASTROPHIC DAMAGE OR MASS
   DESTRUCTION, AND YOU AGREE THAT NEITHER RSV NOR ANY CONTRIBUTOR SHALL HAVE
   ANY LIABILITY OF ANY NATURE AS A RESULT OF ANY SUCH USE OF THE GOVERNED
   CODE.

   (( 10. ))??
   U.S. Government End Users.

   The Governed Code is a "commercial item," as that term is defined in 48
   C.F.R. 2.101 (Oct. 1995), consisting of "commercial computer software" and
   "commercial computer software documentation," as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Governed Code with only those rights set forth herein.

   (( 11. ))??
   Miscellaneous.

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by California law provisions
   (except to the extent applicable law, if any, provides otherwise), excluding
   its conflict-of-law provisions. The parties submit to personal jurisdiction
   in California and further agree that any cause of action arising under or
   related to this Agreement shall be brought in the Federal Courts of the
   Northern District of California, with venue lying in Santa Clara County,
   California. The losing party shall be responsible for costs, including
   without limitation, court costs and reasonable attorneys fees and expenses.
   Notwithstanding anything to the contrary herein, RSV may seek injunctive
   relief related to a breach of this Agreement in any court of competent
   jurisdiction. The application of the United Nations Convention on Contracts
   for the International Sale of Goods is expressly excluded. Any law or
   regulation which provides that the language of a contract shall be construed
   against the drafter shall not apply to this License.

   (( 12. ))??
   Responsibility for Claims.

   Except in cases where another Contributor has failed to comply with Section
   3.4, You are responsible for damages arising, directly or indirectly, out of
   Your utilization of rights under this License, based on the number of copies
   of Governed Code you made available, the revenues you received from utilizing
   such rights, and other relevant factors. You agree to work with affected
   parties to distribute responsibility on an equitable basis.
   (( EXHIBIT A

"The contents of this file are subject to the Ricoh Source Code Public License
Version 1.0 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http:/www.risource.org/RPL

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

This code was initially developed by Ricoh Silicon Valley, Inc. Portions created
by Ricoh Silicon Valley, Inc. are Copyright (C) 1995-1999. All Rights Reserved.

Contributor(s): _." ))??
`
const license_Rdisc_lre = `//**
Rdisc License
https://spdx.org/licenses/Rdisc.json
https://fedoraproject.org/wiki/Licensing/Rdisc_License
**//

Rdisc (this program) was developed by Sun Microsystems, Inc. and is provided for
unrestricted use provided that this legend is included on all tape media and as
a part of the software program in whole or part. Users may copy or modify Rdisc
without charge, and they may freely distribute it.

RDISC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE WARRANTIES
OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE, OR ARISING FROM
A COURSE OF DEALING, USAGE OR TRADE PRACTICE.

Rdisc is provided with no support and without any obligation on the part of Sun
Microsystems, Inc. to assist in its use, correction, modification or
enhancement.

SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE INFRINGEMENT
OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY RDISC OR ANY PART THEREOF.

In no event will Sun Microsystems, Inc. be liable for any lost revenue or
profits or other special, indirect and consequential damages, even if Sun has
been advised of the possibility of such damages.

Sun Microsystems, Inc.

2550 Garcia Avenue

Mountain View, California 94043
`
const license_Ruby_lre = `//**
Ruby License
https://spdx.org/licenses/Ruby.json
http://www.ruby-lang.org/en/LICENSE.txt
**//

   (( 1. ))??
   You may make and give away verbatim copies of the source form of the software
   without restriction, provided that you
   ((duplicate || retain))
   all of the original
   copyright notices and associated disclaimers.

   (( 2. ))??
   You may modify your copy of the software in any way, provided that you do at
   least ONE of the following:

   (( a) ))??
   place your modifications in the Public Domain or otherwise make them Freely
   Available, such as by posting said modifications to Usenet or an equivalent
   medium, or by allowing the
   ((author || owner))
   to include your modifications in the
   software.

   (( b) ))??
   use the modified software only within your corporation or organization.

   (( c) ))??
   ((
   	give non-standard
   	((binaries || executables))
   	non-standard names, with instructions on where to
   	get the original software distribution.
   ||
   	rename any non-standard executables so the names do not conflict with
   	standard executables, which must also be provided.
   ))

   (( d) ))??
   make other distribution arrangements with the
   ((author || owner))

   (( 3. ))??
   You may distribute the software in object code or
   ((binary || executable))
   form, provided that
   you do at least ONE of the following:

   (( a) ))??
   distribute the
   ((binaries || executables))
   and library files of the software, together with
   instructions (in
   ((a || the))
   manual page or equivalent) on where to get the original
   distribution.

   (( b) ))??
   accompany the distribution with the machine-readable source of the software.

   (( c) ))??
   give non-standard
   ((binaries || executables))
    non-standard names, with instructions on where to
   get the original software distribution.

   (( d) ))??
   make other distribution arrangements with the
   ((author || owner))

   (( 4. ))??
   You may modify and include
   ((the part || parts))
   of the software into any other software
   (possibly commercial).
   ((provided you comply with the terms in Sections 1, 2, and 3 above))??

   ((But some files in the distribution are not written by the
   ((author || owner))
   so
   (( that they are not under these terms.
   || they may be made available to you under different terms.))
   ))??

   (( For the list of those files and their copying conditions, see the file LEGAL.
   || __30__
   ))??

   (( 5. ))??
   The scripts and library files supplied as input to or produced as output from
   the software do not automatically fall under the copyright of the software,
   but belong to whomever generated them, and may be sold commercially, and may
   be aggregated with this software.

   (( 6. ))??
   THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
`
const license_SAX_PD_lre = `//**
Sax Public Domain Notice
https://spdx.org/licenses/SAX-PD.json
http://www.saxproject.org/copying.html
**//

Copyright Status

SAX is free!

In fact, it's not possible to own a license to SAX, since it's been placed in
the public domain.

No Warranty

Because SAX is released to the public domain, there is no warranty for the
design or for the software implementation, to the extent permitted by applicable
law. Except when otherwise stated in writing the copyright holders and/or other
parties provide SAX "as is" without warranty of any kind, either expressed or
implied, including, but not limited to, the implied warranties of
merchantability and fitness for a particular purpose. The entire risk as to the
quality and performance of SAX is with you. Should SAX prove defective, you
assume the cost of all necessary servicing, repair or correction.

In no event unless required by applicable law or agreed to in writing will any
copyright holder, or any other party who may modify and/or redistribute SAX, be
liable to you for damages, including any general, special, incidental or
consequential damages arising out of the use or inability to use SAX (including
but not limited to loss of data or data being rendered inaccurate or losses
sustained by you or third parties or a failure of the SAX to operate with any
other programs), even if such holder or other party has been advised of the
possibility of such damages.

Copyright Disclaimers

This page includes statements to that effect by David Megginson, who would have
been able to claim copyright for the original work.

SAX 1.0

Version 1.0 of the Simple API for XML (SAX), created collectively by the
membership of the XML-DEV mailing list, is hereby released into the public
domain.

No one owns SAX: you may use it freely in both commercial and non-commercial
applications, bundle it with your software distribution, include it on a CD-ROM,
list the source code in a book, mirror the documentation at your own web site,
or use it in any other way you see fit.

David Megginson, Megginson Technologies Ltd.

1998-05-11

SAX 2.0

I hereby abandon any property rights to SAX 2.0 (the Simple API for XML), and
release all of the SAX 2.0 source code, compiled code, and documentation
contained in this distribution into the Public Domain. SAX comes with NO
WARRANTY or guarantee of fitness for any purpose.

David Megginson, Megginson Technologies Ltd.

2000-05-05
`
const license_SCEA_lre = `//**
SCEA Shared Source License
https://spdx.org/licenses/SCEA.json
http://research.scea.com/scea_shared_source_license.html
**//

(( SCEA Shared Source License 1.0 ))??

Terms and Conditions:

   (( 1. ))??
   Definitions:

   "Software" shall mean the software and related documentation, whether in
   Source or Object Form, made available under this SCEA Shared Source license
   ("License"), that is indicated by a copyright notice file included in the
   source files or attached or accompanying the source files.

   "Licensor" shall mean Sony Computer Entertainment America, Inc. (herein
   "SCEA")

   "Object Code" or "Object Form" shall mean any form that results from
   translation or transformation of Source Code, including but not limited to
   compiled object code or conversions to other forms intended for machine
   execution.

   "Source Code" or "Source Form" shall have the plain meaning generally
   accepted in the software industry, including but not limited to software
   source code, documentation source, header and configuration files.

   "You" or "Your" shall mean you as an individual or as a company, or whichever
   form under which you are exercising rights under this License.

   (( 2. ))??
   License Grant.

   Licensor hereby grants to You, free of charge subject to the terms and
   conditions of this License, an irrevocable, non-exclusive, worldwide,
   perpetual, and royalty-free license to use, modify, reproduce, distribute,
   publicly perform or display the Software in Object or Source Form .

   (( 3. ))??
   No Right to File for Patent.

   In exchange for the rights that are granted to You free of charge under this
   License, You agree that You will not file for any patent application, seek
   copyright protection or take any other action that might otherwise impair the
   ownership rights in and to the Software that may belong to SCEA or any of the
   other contributors/authors of the Software.

   (( 4. ))??
   Contributions.

   SCEA welcomes contributions in form of modifications, optimizations, tools or
   documentation designed to improve or expand the performance and scope of the
   Software (collectively "Contributions"). Per the terms of this License You
   are free to modify the Software and those modifications would belong to You.
   You may however wish to donate Your Contributions to SCEA for consideration
   for inclusion into the Software. For the avoidance of doubt, if You elect to
   send Your Contributions to SCEA, You are doing so voluntarily and are giving
   the Contributions to SCEA and its parent company Sony Computer Entertainment,
   Inc., free of charge, to use, modify or distribute in any form or in any
   manner. SCEA acknowledges that if You make a donation of Your Contributions
   to SCEA, such Contributions shall not exclusively belong to SCEA or its
   parent company and such donation shall not be to Your exclusion. SCEA, in its
   sole discretion, shall determine whether or not to include Your donated
   Contributions into the Software, in whole, in part, or as modified by SCEA.
   Should SCEA elect to include any such Contributions into the Software, it
   shall do so at its own risk and may elect to give credit or special thanks to
   any such contributors in the attached copyright notice. However, if any of
   Your contributions are included into the Software, they will become part of
   the Software and will be distributed under the terms and conditions of this
   License. Further, if Your donated Contributions are integrated into the
   Software then Sony Computer Entertainment, Inc. shall become the copyright
   owner of the Software now containing Your contributions and SCEA would be the
   Licensor.

   (( 5. ))??
   Redistribution in Source Form

   You may redistribute copies of the Software, modifications or derivatives
   thereof in Source Code Form, provided that You:

      (( a. ))??
      Include a copy of this License and any copyright notices with source

      (( b. ))??
      Identify modifications if any were made to the Software

      (( c. ))??
      Include a copy of all documentation accompanying the Software and
      modifications made by You

   (( 6. ))??
   Redistribution in Object Form

   If You redistribute copies of the Software, modifications or derivatives
   thereof in Object Form only (as incorporated into finished goods, i.e. end
   user applications) then You will not have a duty to include any copies of the
   code, this License, copyright notices, other attributions or documentation.

   (( 7. ))??
   No Warranty

   THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT ANY REPRESENTATIONS OR
   WARRANTIES OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, ANY WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY, OR
   FITNESS FOR A PARTICULAR PURPOSE. YOU ARE SOLELY RESPONSIBLE FOR DETERMINING
   THE APPROPRIATENESS OF USING, MODIFYING OR REDISTRIBUTING THE SOFTWARE AND
   ASSUME ANY RISKS ASSOCIATED WITH YOUR EXERCISE OF PERMISSIONS UNDER THIS
   LICENSE.

   (( 8. ))??
   Limitation of Liability

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY WILL EITHER PARTY BE LIABLE
   TO THE OTHER PARTY OR ANY THIRD PARTY FOR ANY DIRECT, INDIRECT,
   CONSEQUENTIAL, SPECIAL, INCIDENTAL, OR EXEMPLARY DAMAGES WITH RESPECT TO ANY
   INJURY, LOSS, OR DAMAGE, ARISING UNDER OR IN CONNECTION WITH THIS LETTER
   AGREEMENT, WHETHER FORESEEABLE OR UNFORESEEABLE, EVEN IF SUCH PARTY HAS BEEN
   ADVISED OF THE POSSIBILITY OF SUCH INJURY, LOSS, OR DAMAGE. THE LIMITATIONS
   OF LIABILITY SET FORTH IN THIS SECTION SHALL APPLY TO THE FULLEST EXTENT
   PERMISSIBLE AT LAW OR ANY GOVERMENTAL REGULATIONS.

   (( 9. ))??
   Governing Law and Consent to Jurisdiction

   This Agreement shall be governed by and interpreted in accordance with the
   laws of the State of California, excluding that body of law related to choice
   of laws, and of the United States of America. Any action or proceeding
   brought to enforce the terms of this Agreement or to adjudicate any dispute
   arising hereunder shall be brought in the Superior Court of the County of San
   Mateo, State of California or the United States District Court for the
   Northern District of California. Each of the parties hereby submits itself to
   the exclusive jurisdiction and venue of such courts for purposes of any such
   action. In addition, each party hereby waives the right to a jury trial in
   any action or proceeding related to this Agreement.

   (( 10. ))??
   Copyright Notice for Redistribution of Source Code

Copyright __10__

Licensed under the SCEA Shared Source License, Version 1.0 (the "License"); you
may not use this file except in compliance with the License. You may obtain a
copy of the License at:

http:/research.scea.com/scea_shared_source_license.html

Unless required by applicable law or agreed to in writing, software distributed
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
`
const license_SGI_B_1_0_lre = `//**
SGI Free Software License B v1.0
https://spdx.org/licenses/SGI-B-1.0.json
http://oss.sgi.com/projects/FreeB/SGIFreeSWLicB.1.0.html
**//

(( SGI FREE SOFTWARE LICENSE B

(Version 1.0 1/25/2000) ))??

   (( 1. ))??
   Definitions.

      (( 1.1 ))??
      "Additional Notice Provisions" means such additional provisions as appear
      in the Notice in Original Code under the heading "Additional Notice
      Provisions."

      (( 1.2 ))??
      "API" means an application programming interface established by SGI in
      conjunction with the Original Code.

      (( 1.3 ))??
      "Covered Code" means the Original Code or Modifications or the combination
      of the Original Code and Modifications, in each case including portions
      thereof.

      (( 1.4 ))??
      "Hardware" means any physical device that accepts input, processes input,
      stores the results of processing, and/or provides output.

      (( 1.5 ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.6 ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.7 ))??
      "License" means this document.

      (( 1.8 ))??
      "Modifications" means any addition to the substance or structure of the
      Original Code and/or any addition to or deletion from previous
      Modifications. When Covered Code is released as a series of files, a
      Modification is:

         (( A. ))??
         Any addition to the contents of a file containing Original Code and/or
         any addition to or deletion from previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.9 ))??
      "Notice" means any notice in Original Code or Covered Code, as required by
      and in compliance with this License.

      (( 1.10 ))??
      "Original Code" means source code of computer software code which is
      described in the source code Notice required by Exhibit A as Original
      Code, and updates and error corrections specifically thereto.

      (( 1.11 ))??
      "Recipient" means an individual or a legal entity exercising rights under,
      and complying with all of the terms of, this License or a future version
      of this License issued under Section 8. For legal entities, "Recipient"
      includes any entity which controls, is controlled by, or is under common
      control with Recipient. For purposes of this definition, "control" of an
      entity means (a) the power, direct or indirect, to direct or manage such
      entity, or (b) ownership of fifty percent (50%) or more of the outstanding
      shares or beneficial ownership of such entity.

      (( 1.12 ))??
      SGI" means Silicon Graphics, Inc.

   (( 2. ))??
   License Grant and Restrictions.

      (( 2.1 ))??
      License Grant. Subject to the provisions of this License and any third
      party intellectual property claims, for the duration of intellectual
      property protections inherent in the Original Code, SGI hereby grants
      Recipient a worldwide, royalty-free, non-exclusive license, to do the
      following: (i) under copyrights Licensable by SGI, to reproduce,
      distribute, create derivative works from, and, to the extent applicable,
      display and perform the Original Code alone and/or as part of a Larger
      Work; and (ii) under any patent claims Licensable by SGI and embodied in
      the Original Code, to make, have made, use, practice, sell, and offer for
      sale, and/or otherwise dispose of the Original Code. Recipient accepts the
      terms and conditions of this License by undertaking any of the
      aforementioned actions.

      (( 2.2 ))??
      Restriction on Patent License. Notwithstanding the provisions of Section
      2.1(ii), no patent license is granted: 1) separate from the Original Code;
      nor 2) for infringements caused by (i) modification of the Original Code,
      or (ii) the combination of the Original Code with other software or
      Hardware.

      (( 2.3 ))??
      No License For Hardware Implementations. The licenses granted in Section
      2.1 are not applicable to implementation in Hardware of the algorithms
      embodied in the Original Code.

      (( 2.4 ))??
      Modifications License and API Compliance. Modifications are only licensed
      under Section 2.1(i) to the extent such Modifications are fully compliant
      with any API as may be identified in Additional Notice Provisions as
      appear in the Original Code.

   (( 3. ))??
   Redistributions.

      (( A. ))??
      Retention of Notice/Copy of License. The Notice set forth in Exhibit A,
      below, must be conspicuously retained or included in any and all
      redistributions of Covered Code. For distributions of the Covered Code in
      source code form, the Notice must appear in every file that can include a
      text comments field; in executable form, the Notice and a copy of this
      License must appear in related documentation or collateral where the
      Recipient's rights relating to Covered Code are described. Any Additional
      Notice Provisions which actually appears in the Original Code must also be
      retained or included in any and all redistributions of Covered Code.

      (( B. ))??
      Alternative License. Provided that Recipient is in compliance with the
      terms of this License, Recipient may distribute the source code and/or
      executable version(s) of Covered Code under (1) this License; (2) a
      license identical to this License but for only such changes as are
      necessary in order to clarify Recipient's role as licensor of
      Modifications, without derogation of any of SGI's rights; and/or (3) a
      license of Recipient's choosing, containing terms different from this
      License, provided that the license terms include this Section 3 and
      Sections 4, 6, 7, 10, 12, and 13, which terms may not be modified or
      superseded by any other terms of such license. If Recipient elects to use
      any license other than this License, Recipient must make it absolutely
      clear that any of its terms which differ from this License are offered by
      Recipient alone, and not by SGI.

      (( C. ))??
      Indemnity. Recipient hereby agrees to indemnify SGI for any liability
      incurred by SGI as a result of any such alternative license terms
      Recipient offers.

   (( 4. ))??
   Termination. This License and the rights granted hereunder will terminate
   automatically if Recipient breaches any term herein and fails to cure such
   breach within 30 days thereof. Any sublicense to the Covered Code that is
   properly granted shall survive any termination of this License, absent
   termination by the terms of such sublicense. Provisions that, by their
   nature, must remain in effect beyond the termination of this License, shall
   survive.

   (( 5. ))??
   No Trademark Or Other Rights. This License does not grant any rights to: (i)
   any software apart from the Covered Code, nor shall any other rights or
   licenses not expressly granted hereunder arise by implication, estoppel or
   otherwise with respect to the Covered Code; (ii) any trade name, trademark or
   service mark whatsoever, including without limitation any related right for
   purposes of endorsement or promotion of products derived from the Covered
   Code, without prior written permission of SGI; or (iii) any title to or
   ownership of the Original Code, which shall at all times remains with SGI.
   All rights in the Original Code not expressly granted under this License are
   reserved.

   (( 6. ))??
   Compliance with Laws; Non-Infringement. Recipient hereby assures that it
   shall comply with all applicable laws, regulations, and executive orders, in
   connection with any and all dispositions of Covered Code, including but not
   limited to, all export, re-export, and import control laws, regulations, and
   executive orders, of the U.S. government and other countries. Recipient may
   not distribute Covered Code that (i) in any way infringes (directly or
   contributorily) the rights (including patent, copyright, trade secret,
   trademark or other intellectual property rights of any kind) of any other
   person or entity or (ii) breaches any representation or warranty, express,
   implied or statutory, to which, under any applicable law, it might be deemed
   to have been subject.

   (( 7. ))??
   Claims of Infringement. If Recipient learns of any third party claim that any
   disposition of Covered Code and/or functionality wholly or partially
   infringes the third party's intellectual property rights, Recipient will
   promptly notify SGI of such claim.

   (( 8. ))??
   Versions of the License. SGI may publish revised and/or new versions of the
   License from time to time, each with a distinguishing version number. Once
   Covered Code has been published under a particular version of the License,
   Recipient may, for the duration of the license, continue to use it under the
   terms of that version, or choose to use such Covered Code under the terms of
   any subsequent version published by SGI. Subject to the provisions of
   Sections 3 and 4 of this License, only SGI may modify the terms applicable to
   Covered Code created under this License.

   (( 9. ))??
   DISCLAIMER OF WARRANTY. COVERED CODE IS PROVIDED "AS IS." ALL EXPRESS AND
   IMPLIED WARRANTIES AND CONDITIONS ARE DISCLAIMED, INCLUDING, WITHOUT
   LIMITATION, ANY IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY,
   SATISFACTORY QUALITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.
   SGI ASSUMES NO RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE. SHOULD
   THE SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, SGI ASSUMES NO COST OR LIABILITY
   FOR SERVICING, REPAIR OR CORRECTION. THIS DISCLAIMER OF WARRANTY IS AN
   ESSENTIAL PART OF THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED
   HEREUNDER EXCEPT SUBJECT TO THIS DISCLAIMER.

   (( 10. ))??
   LIMITATION OF LIABILITY. UNDER NO CIRCUMSTANCES NOR LEGAL THEORY, WHETHER
   TORT (INCLUDING, WITHOUT LIMITATION, NEGLIGENCE OR STRICT LIABILITY),
   CONTRACT, OR OTHERWISE, SHALL SGI OR ANY SGI LICENSOR BE LIABLE FOR ANY
   DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY
   CHARACTER INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK
   STOPPAGE, LOSS OF DATA, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER
   COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH PARTY SHALL HAVE BEEN INFORMED OF
   THE POSSIBILITY OF SUCH DAMAGES. THIS LIMITATION OF LIABILITY SHALL NOT APPLY
   TO LIABILITY FOR DEATH OR PERSONAL INJURY RESULTING FROM SGI's NEGLIGENCE TO
   THE EXTENT APPLICABLE LAW PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO
   NOT ALLOW THE EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES,
   SO THAT EXCLUSION AND LIMITATION MAY NOT APPLY TO RECIPIENT.

   (( 11. ))??
   Indemnity. Recipient shall be solely responsible for damages arising,
   directly or indirectly, out of its utilization of rights under this License.
   Recipient will defend, indemnify and hold harmless Silicon Graphics, Inc.
   from and against any loss, liability, damages, costs or expenses (including
   the payment of reasonable attorneys fees) arising out of Recipient's use,
   modification, reproduction and distribution of the Covered Code or out of any
   representation or warranty made by Recipient.

   (( 12. ))??
   U.S. Government End Users. The Covered Code is a "commercial item" consisting
   of "commercial computer software" as such terms are defined in title 48 of
   the Code of Federal Regulations and all U.S. Government End Users acquire
   only the rights set forth in this License and are subject to the terms of
   this License.

   (( 13. ))??
   Miscellaneous. This License represents the complete agreement concerning the
   its subject matter. If any provision of this License is held to be
   unenforceable, such provision shall be reformed so as to achieve as nearly as
   possible the same legal and economic effect as the original provision and the
   remainder of this License will remain in effect. This License shall be
   governed by and construed in accordance with the laws of the United States
   and the State of California as applied to agreements entered into and to be
   performed entirely within California between California residents. Any
   litigation relating to this License shall be subject to the exclusive
   jurisdiction of the Federal Courts of the Northern District of California
   (or, absent subject matter jurisdiction in such courts, the courts of the
   State of California), with venue lying exclusively in Santa Clara County,
   California, with the losing party responsible for costs, including without
   limitation, court costs and reasonable attorneys fees and expenses. The
   application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any law or regulation
   which provides that the language of a contract shall be construed against the
   drafter shall not apply to this License.
   (( Exhibit A

License Applicability. Except to the extent portions of this file are made
subject to an alternative license as permitted in the SGI Free Software License
B, Version 1.0 (the "License"), the contents of this file are subject only to
the provisions of the License. You may not use this file except in compliance
with the License. You may obtain a copy of the License at Silicon Graphics,
Inc., attn: Legal Services, 1600 Ampitheatre Parkway, Mountain View, CA
94043-1351, or at:

http:/oss.sgi.com/projects/FreeB

Note that, as provided in the License, the Software is distributed on an "AS IS"
basis, with ALL EXPRESS AND IMPLIED WARRANTIES AND CONDITIONS DISCLAIMED,
INCLUDING, WITHOUT LIMITATION, ANY IMPLIED WARRANTIES AND CONDITIONS OF
MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A PARTICULAR PURPOSE, AND
NON-INFRINGEMENT.

Original Code. The Original Code is:
//** name of software **//
__5__
,
//** version number **//
__5__
, and
//** release date **//
__5__
, developed by Silicon Graphics, Inc. The Original Code is Copyright (c)
//** dates of first publication, as appearing in the Notice in the Original Code **//
__13__
Silicon Graphics, Inc. Copyright in any portions created by third parties is as
indicated elsewhere herein. All Rights Reserved.

Additional Notice Provisions:
))??
`
const license_SGI_B_1_1_lre = `//**
SGI Free Software License B v1.1
https://spdx.org/licenses/SGI-B-1.1.json
http://oss.sgi.com/projects/FreeB/
**//

(( SGI FREE SOFTWARE LICENSE B

(Version 1.1 02/22/2000) ))??

   (( 1. ))??
   Definitions.

      (( 1.1 ))??
      "Additional Notice Provisions" means such additional provisions as appear
      in the Notice in Original Code under the heading "Additional Notice
      Provisions."

      (( 1.2 ))??
      "Covered Code" means the Original Code or Modifications, or any
      combination thereof.

      (( 1.3 ))??
      "Hardware" means any physical device that accepts input, processes input,
      stores the results of processing, and/or provides output.

      (( 1.4 ))??
      "Larger Work" means a work that combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.5 ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.6 ))??
      "License" means this document.

      (( 1.7 ))??
      "Licensed Patents" means patent claims Licensable by SGI that are
      infringed by the use or sale of Original Code or any Modifications
      provided by SGI, or any combination thereof.

      (( 1.8 ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of the Original Code or any previous Modifications. When Covered
      Code is released as a series of files, a Modification is:

         (( A. ))??
         Any addition to the contents of a file containing Original Code and/or
         addition to or deletion from the contents of a file containing previous
         Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.9 ))??
      "Notice" means any notice in Original Code or Covered Code, as required by
      and in compliance with this License.

      (( 1.10 ))??
      "Original Code" means source code of computer software code that is
      described in the source code Notice required by Exhibit A as Original
      Code, and updates and error corrections specifically thereto.

      (( 1.11 ))??
      "Recipient" means an individual or a legal entity exercising rights under,
      and complying with all of the terms of, this License or a future version
      of this License issued under Section 8. For legal entities, "Recipient"
      includes any entity that controls, is controlled by, or is under common
      control with Recipient. For purposes of this definition, "control" of an
      entity means (a) the power, direct or indirect, to direct or manage such
      entity, or (b) ownership of fifty percent (50%) or more of the outstanding
      shares or beneficial ownership of such entity.

      (( 1.12 ))??
      "Recipient Patents" means patent claims Licensable by a Recipient that are
      infringed by the use or sale of Original Code or any Modifications
      provided by SGI, or any combination thereof.

      (( 1.13 ))??
      "SGI" means Silicon Graphics, Inc.

      (( 1.14 ))??
      "SGI Patents" means patent claims Licensable by SGI other than the
      Licensed Patents.

   (( 2. ))??
   License Grant and Restrictions.

      (( 2.1 ))??
      SGI License Grant. Subject to the terms of this License and any third
      party intellectual property claims, for the duration of intellectual
      property protections inherent in the Original Code, SGI hereby grants
      Recipient a worldwide, royalty-free, non-exclusive license, to do the
      following: (i) under copyrights Licensable by SGI, to reproduce,
      distribute, create derivative works from, and, to the extent applicable,
      display and perform the Original Code and/or any Modifications provided by
      SGI alone and/or as part of a Larger Work; and (ii) under any Licensable
      Patents, to make, have made, use, sell, offer for sale, import and/or
      otherwise transfer the Original Code and/or any Modifications provided by
      SGI. Recipient accepts the terms and conditions of this License by
      undertaking any of the aforementioned actions. The patent license shall
      apply to the Covered Code if, at the time any related Modification is
      added, such addition of the Modification causes such combination to be
      covered by the Licensed Patents. The patent license in Section 2.1(ii)
      shall not apply to any other combinations that include the Modification.
      No patent license is provided under SGI Patents for infringements of SGI
      Patents by Modifications not provided by SGI or combinations of Original
      Code and Modifications not provided by SGI.

      (( 2.2 ))??
      Recipient License Grant. Subject to the terms of this License and any
      third party intellectual property claims, Recipient hereby grants SGI and
      any other Recipients a worldwide, royalty-free, non-exclusive license,
      under any Recipient Patents, to make, have made, use, sell, offer for
      sale, import and/or otherwise transfer the Original Code and/or any
      Modifications provided by SGI.

      (( 2.3 ))??
      No License For Hardware Implementations. The licenses granted in Section
      2.1 and 2.2 are not applicable to implementation in Hardware of the
      algorithms embodied in the Original Code or any Modifications provided by
      SGI .

   (( 3. ))??
   Redistributions.

      (( 3.1 ))??
      Retention of Notice/Copy of License. The Notice set forth in Exhibit A,
      below, must be conspicuously retained or included in any and all
      redistributions of Covered Code. For distributions of the Covered Code in
      source code form, the Notice must appear in every file that can include a
      text comments field; in executable form, the Notice and a copy of this
      License must appear in related documentation or collateral where the
      Recipient's rights relating to Covered Code are described. Any Additional
      Notice Provisions which actually appears in the Original Code must also be
      retained or included in any and all redistributions of Covered Code.

      (( 3.2 ))??
      Alternative License. Provided that Recipient is in compliance with the
      terms of this License, Recipient may, so long as without derogation of any
      of SGI's rights in and to the Original Code, distribute the source code
      and/or executable version(s) of Covered Code under (1) this License; (2) a
      license identical to this License but for only such changes as are
      necessary in order to clarify Recipient's role as licensor of
      Modifications; and/or (3) a license of Recipient's choosing, containing
      terms different from this License, provided that the license terms include
      this Section 3 and Sections 4, 6, 7, 10, 12, and 13, which terms may not
      be modified or superseded by any other terms of such license. If Recipient
      elects to use any license other than this License, Recipient must make it
      absolutely clear that any of its terms which differ from this License are
      offered by Recipient alone, and not by SGI. It is emphasized that this
      License is a limited license, and, regardless of the license form employed
      by Recipient in accordance with this Section 3.2, Recipient may relicense
      only such rights, in Original Code and Modifications by SGI, as it has
      actually been granted by SGI in this License.

      (( 3.3 ))??
      Indemnity. Recipient hereby agrees to indemnify SGI for any liability
      incurred by SGI as a result of any such alternative license terms
      Recipient offers.

   (( 4. ))??
   Termination. This License and the rights granted hereunder will terminate
   automatically if Recipient breaches any term herein and fails to cure such
   breach within 30 days thereof. Any sublicense to the Covered Code that is
   properly granted shall survive any termination of this License, absent
   termination by the terms of such sublicense. Provisions that, by their
   nature, must remain in effect beyond the termination of this License, shall
   survive.

   (( 5. ))??
   No Trademark Or Other Rights. This License does not grant any rights to: (i)
   any software apart from the Covered Code, nor shall any other rights or
   licenses not expressly granted hereunder arise by implication, estoppel or
   otherwise with respect to the Covered Code; (ii) any trade name, trademark or
   service mark whatsoever, including without limitation any related right for
   purposes of endorsement or promotion of products derived from the Covered
   Code, without prior written permission of SGI; or (iii) any title to or
   ownership of the Original Code, which shall at all times remains with SGI.
   All rights in the Original Code not expressly granted under this License are
   reserved.

   (( 6. ))??
   Compliance with Laws; Non-Infringement. There are various worldwide laws,
   regulations, and executive orders applicable to dispositions of Covered Code,
   including without limitation export, re-export, and import control laws,
   regulations, and executive orders, of the U.S. government and other
   countries, and Recipient is reminded it is obliged to obey such laws,
   regulations, and executive orders. Recipient may not distribute Covered Code
   that (i) in any way infringes (directly or contributorily) any intellectual
   property rights of any kind of any other person or entity or (ii) breaches
   any representation or warranty, express, implied or statutory, to which,
   under any applicable law, it might be deemed to have been subject.

   (( 7. ))??
   Claims of Infringement. If Recipient learns of any third party claim that any
   disposition of Covered Code and/or functionality wholly or partially
   infringes the third party's intellectual property rights, Recipient will
   promptly notify SGI of such claim.

   (( 8. ))??
   Versions of the License. SGI may publish revised and/or new versions of the
   License from time to time, each with a distinguishing version number. Once
   Covered Code has been published under a particular version of the License,
   Recipient may, for the duration of the license, continue to use it under the
   terms of that version, or choose to use such Covered Code under the terms of
   any subsequent version published by SGI. Subject to the provisions of
   Sections 3 and 4 of this License, only SGI may modify the terms applicable to
   Covered Code created under this License.

   (( 9. ))??
   DISCLAIMER OF WARRANTY. COVERED CODE IS PROVIDED "AS IS." ALL EXPRESS AND
   IMPLIED WARRANTIES AND CONDITIONS ARE DISCLAIMED, INCLUDING, WITHOUT
   LIMITATION, ANY IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY,
   SATISFACTORY QUALITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.
   SGI ASSUMES NO RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE. SHOULD
   THE SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, SGI ASSUMES NO COST OR LIABILITY
   FOR SERVICING, REPAIR OR CORRECTION. THIS DISCLAIMER OF WARRANTY IS AN
   ESSENTIAL PART OF THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED
   HEREUNDER EXCEPT SUBJECT TO THIS DISCLAIMER.

   (( 10. ))??
   LIMITATION OF LIABILITY. UNDER NO CIRCUMSTANCES NOR LEGAL THEORY, WHETHER
   TORT (INCLUDING, WITHOUT LIMITATION, NEGLIGENCE OR STRICT LIABILITY),
   CONTRACT, OR OTHERWISE, SHALL SGI OR ANY SGI LICENSOR BE LIABLE FOR ANY
   DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY
   CHARACTER INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK
   STOPPAGE, LOSS OF DATA, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER
   COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH PARTY SHALL HAVE BEEN INFORMED OF
   THE POSSIBILITY OF SUCH DAMAGES. THIS LIMITATION OF LIABILITY SHALL NOT APPLY
   TO LIABILITY FOR DEATH OR PERSONAL INJURY RESULTING FROM SGI's NEGLIGENCE TO
   THE EXTENT APPLICABLE LAW PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO
   NOT ALLOW THE EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES,
   SO THAT EXCLUSION AND LIMITATION MAY NOT APPLY TO RECIPIENT.

   (( 11. ))??
   Indemnity. Recipient shall be solely responsible for damages arising,
   directly or indirectly, out of its utilization of rights under this License.
   Recipient will defend, indemnify and hold harmless Silicon Graphics, Inc.
   from and against any loss, liability, damages, costs or expenses (including
   the payment of reasonable attorneys fees) arising out of Recipient's use,
   modification, reproduction and distribution of the Covered Code or out of any
   representation or warranty made by Recipient.

   (( 12. ))??
   U.S. Government End Users. The Covered Code is a "commercial item" consisting
   of "commercial computer software" as such terms are defined in title 48 of
   the Code of Federal Regulations and all U.S. Government End Users acquire
   only the rights set forth in this License and are subject to the terms of
   this License.

   (( 13. ))??
   Miscellaneous. This License represents the complete agreement concerning the
   its subject matter. If any provision of this License is held to be
   unenforceable, such provision shall be reformed so as to achieve as nearly as
   possible the same legal and economic effect as the original provision and the
   remainder of this License will remain in effect. This License shall be
   governed by and construed in accordance with the laws of the United States
   and the State of California as applied to agreements entered into and to be
   performed entirely within California between California residents. Any
   litigation relating to this License shall be subject to the exclusive
   jurisdiction of the Federal Courts of the Northern District of California
   (or, absent subject matter jurisdiction in such courts, the courts of the
   State of California), with venue lying exclusively in Santa Clara County,
   California, with the losing party responsible for costs, including without
   limitation, court costs and reasonable attorneys fees and expenses. The
   application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any law or regulation that
   provides that the language of a contract shall be construed against the
   drafter shall not apply to this License.
   (( Exhibit A

License Applicability. Except to the extent portions of this file are made
subject to an alternative license as permitted in the SGI Free Software License
B, Version 1.1 (the "License"), the contents of this file are subject only to
the provisions of the License. You may not use this file except in compliance
with the License. You may obtain a copy of the License at Silicon Graphics,
Inc., attn: Legal Services, 1600 Amphitheatre Parkway, Mountain View, CA
94043-1351, or at:

http:/oss.sgi.com/projects/FreeB

Note that, as provided in the License, the Software is distributed on an "AS IS"
basis, with ALL EXPRESS AND IMPLIED WARRANTIES AND CONDITIONS DISCLAIMED,
INCLUDING, WITHOUT LIMITATION, ANY IMPLIED WARRANTIES AND CONDITIONS OF
MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A PARTICULAR PURPOSE, AND
NON-INFRINGEMENT.

Original Code. The Original Code is:
//** name of software **//
__5__
,
//** version number **//
__5__
, and
//** release date **//
__5__
, developed by Silicon Graphics, Inc. The Original Code is Copyright (c)
//** dates of first publication, as appearing in the Notice in the Original Code **//
__13__
Silicon Graphics, Inc. Copyright in any portions created by third parties is as
indicated elsewhere herein. All Rights Reserved.
(( Additional Notice Provisions:
//** such additional provisions, if any, as appear in the Notice in the Original Code under the heading  **//
 ))??
  ))??
`
const license_SGI_B_2_0_lre = `//**
SGI Free Software License B v2.0
https://spdx.org/licenses/SGI-B-2.0.json
http://oss.sgi.com/projects/FreeB/SGIFreeSWLicB.2.0.pdf
**//

(( SGI FREE SOFTWARE LICENSE B

(Version 2.0, Sept. 18, 2008)
(( Copyright __20__ ))??
))??

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice including the dates of first publication and either
this permission notice or a reference to http:/oss.sgi.com/projects/FreeB/ shall
be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL SILICON
GRAPHICS, INC. BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of Silicon Graphics, Inc. shall not
be used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from Silicon Graphics, Inc.
`
const license_SHL_0_5_lre = `//**
Solderpad Hardware License v0.5
https://spdx.org/licenses/SHL-0.5.json
https://solderpad.org/licenses/SHL-0.5/
**//

SOLDERPAD HARDWARE LICENSE version 0.5

This license is based closely on the Apache License Version 2.0, but is not
approved or endorsed by the Apache Foundation. A copy of the non-modified Apache
License 2.0 can be found at http:/www.apache.org/licenses/LICENSE-2.0.

As this license is not currently OSI or FSF approved, the Licensor permits any
Work licensed under this License, at the option of the Licensee, to be treated
as licensed under the Apache License Version 2.0 (which is so approved).

This License is licensed under the terms of this License and in particular
clause 7 below (Disclaimer of Warranties) applies in relation to its use.

TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION

1. Definitions.

"License" shall mean the terms and conditions for use, reproduction, and
distribution as defined by Sections 1 through 9 of this document.

"Licensor" shall mean the Rights owner or entity authorized by the Rights owner
that is granting the License.

"Legal Entity" shall mean the union of the acting entity and all other entities
that control, are controlled by, or are under common control with that entity.
For the purposes of this definition, "control" means (i) the power, direct or
indirect, to cause the direction or management of such entity, whether by
contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the
outstanding shares, or (iii) beneficial ownership of such entity.

"You" (or "Your") shall mean an individual or Legal Entity exercising
permissions granted by this License.

"Rights" means copyright and any similar right including design right (whether
registered or unregistered), semiconductor topography (mask) rights and database
extraction rights (but excluding Patents and Trademarks).

"Source" form shall mean the preferred form for making modifications, including
but not limited to source code, net lists, board layouts, CAD files,
documentation source, and configuration files.

"Object" form shall mean any form resulting from mechanical transformation or
translation of a Source form, including but not limited to compiled object code,
generated documentation, the instantiation of a hardware design and conversions
to other media types, including intermediate forms such as bytecodes, FPGA
bitstreams, artwork and semiconductor topographies (mask works).

"Work" shall mean the work of authorship, whether in Source form or other Object
form, made available under the License, as indicated by a Rights notice that is
included in or attached to the work (an example is provided in the Appendix
below).

"Derivative Works" shall mean any work, whether in Source or Object form, that
is based on (or derived from) the Work and for which the editorial revisions,
annotations, elaborations, or other modifications represent, as a whole, an
original work of authorship. For the purposes of this License, Derivative Works
shall not include works that remain separable from, or merely link (or bind by
name) or physically connect to or interoperate with the interfaces of, the Work
and Derivative Works thereof.

"Contribution" shall mean any design or work of authorship, including the
original version of the Work and any modifications or additions to that Work or
Derivative Works thereof, that is intentionally submitted to Licensor for
inclusion in the Work by the Rights owner or by an individual or Legal Entity
authorized to submit on behalf of the Rights owner. For the purposes of this
definition, "submitted" means any form of electronic, verbal, or written
communication sent to the Licensor or its representatives, including but not
limited to communication on electronic mailing lists, source code control
systems, and issue tracking systems that are managed by, or on behalf of, the
Licensor for the purpose of discussing and improving the Work, but excluding
communication that is conspicuously marked or otherwise designated in writing by
the Rights owner as "Not a Contribution."

"Contributor" shall mean Licensor and any individual or Legal Entity on behalf
of whom a Contribution has been received by Licensor and subsequently
incorporated within the Work.

2. Grant of License. Subject to the terms and conditions of this License, each
Contributor hereby grants to You a perpetual, worldwide, non-exclusive,
no-charge, royalty-free, irrevocable license under the Rights to reproduce,
prepare Derivative Works of, publicly display, publicly perform, sublicense, and
distribute the Work and such Derivative Works in Source or Object form and do
anything in relation to the Work as if the Rights did not exist.

3. Grant of Patent License. Subject to the terms and conditions of this License,
each Contributor hereby grants to You a perpetual, worldwide, non-exclusive,
no-charge, royalty-free, irrevocable (except as stated in this section) patent
license to make, have made, use, offer to sell, sell, import, and otherwise
transfer the Work, where such license applies only to those patent claims
licensable by such Contributor that are necessarily infringed by their
Contribution(s) alone or by combination of their Contribution(s) with the Work
to which such Contribution(s) was submitted. If You institute patent litigation
against any entity (including a cross-claim or counterclaim in a lawsuit)
alleging that the Work or a Contribution incorporated within the Work
constitutes direct or contributory patent infringement, then any patent licenses
granted to You under this License for that Work shall terminate as of the date
such litigation is filed.

4. Redistribution. You may reproduce and distribute copies of the Work or
Derivative Works thereof in any medium, with or without modifications, and in
Source or Object form, provided that You meet the following conditions:

You must give any other recipients of the Work or Derivative Works a copy of
this License; and

You must cause any modified files to carry prominent notices stating that You
changed the files; and

You must retain, in the Source form of any Derivative Works that You distribute,
all copyright, patent, trademark, and attribution notices from the Source form
of the Work, excluding those notices that do not pertain to any part of the
Derivative Works; and

If the Work includes a "NOTICE" text file as part of its distribution, then any
Derivative Works that You distribute must include a readable copy of the
attribution notices contained within such NOTICE file, excluding those notices
that do not pertain to any part of the Derivative Works, in at least one of the
following places: within a NOTICE text file distributed as part of the
Derivative Works; within the Source form or documentation, if provided along
with the Derivative Works; or, within a display generated by the Derivative
Works, if and wherever such third-party notices normally appear. The contents of
the NOTICE file are for informational purposes only and do not modify the
License. You may add Your own attribution notices within Derivative Works that
You distribute, alongside or as an addendum to the NOTICE text from the Work,
provided that such additional attribution notices cannot be construed as
modifying the License. You may add Your own copyright statement to Your
modifications and may provide additional or different license terms and
conditions for use, reproduction, or distribution of Your modifications, or for
any such Derivative Works as a whole, provided Your use, reproduction, and
distribution of the Work otherwise complies with the conditions stated in this
License.

5. Submission of Contributions. Unless You explicitly state otherwise, any
Contribution intentionally submitted for inclusion in the Work by You to the
Licensor shall be under the terms and conditions of this License, without any
additional terms or conditions. Notwithstanding the above, nothing herein shall
supersede or modify the terms of any separate license agreement you may have
executed with Licensor regarding such Contributions.

6. Trademarks. This License does not grant permission to use the trade names,
trademarks, service marks, or product names of the Licensor, except as required
for reasonable and customary use in describing the origin of the Work and
reproducing the content of the NOTICE file.

7. Disclaimer of Warranty. Unless required by applicable law or agreed to in
writing, Licensor provides the Work (and each Contributor provides its
Contributions) on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied, including, without limitation, any warranties
or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
PARTICULAR PURPOSE. You are solely responsible for determining the
appropriateness of using or redistributing the Work and assume any risks
associated with Your exercise of permissions under this License.

8. Limitation of Liability. In no event and under no legal theory, whether in
tort (including negligence), contract, or otherwise, unless required by
applicable law (such as deliberate and grossly negligent acts) or agreed to in
writing, shall any Contributor be liable to You for damages, including any
direct, indirect, special, incidental, or consequential damages of any character
arising as a result of this License or out of the use or inability to use the
Work (including but not limited to damages for loss of goodwill, work stoppage,
computer failure or malfunction, or any and all other commercial damages or
losses), even if such Contributor has been advised of the possibility of such
damages.

9. Accepting Warranty or Additional Liability. While redistributing the Work or
Derivative Works thereof, You may choose to offer, and charge a fee for,
acceptance of support, warranty, indemnity, or other liability obligations
and/or rights consistent with this License. However, in accepting such
obligations, You may act only on Your own behalf and on Your sole
responsibility, not on behalf of any other Contributor, and only if You agree to
indemnify, defend, and hold each Contributor harmless for any liability incurred
by, or claims asserted against, such Contributor by reason of your accepting any
such warranty or additional liability.

END OF TERMS AND CONDITIONS

APPENDIX: How to apply this license to your work

To apply this license to your work, attach the following boilerplate notice,
with the fields enclosed by brackets "[]" replaced with your own identifying
information. (Don't include the brackets!) The text should be enclosed in the
appropriate comment syntax for the file format. We also recommend that a file or
class name and description of purpose be included on the same "printed page" as
the copyright notice for easier identification within third-party archives.

Copyright [yyyy] [name of copyright owner] Copyright and related rights are
licensed under the Solderpad Hardware License, Version 0.5 (the "License"); you
may not use this file except in compliance with the License. You may obtain a
copy of the License at http:/solderpad.org/licenses/SHL-0.5. Unless required by
applicable law or agreed to in writing, software, hardware and materials
distributed under this License is distributed on an "AS IS" BASIS, WITHOUT
WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License
for the specific language governing permissions and limitations under the
License.
`
const license_SHL_0_51_lre = `//**
Solderpad Hardware License, Version 0.51
https://spdx.org/licenses/SHL-0.51.json
https://solderpad.org/licenses/SHL-0.51/
**//

(( SOLDERPAD HARDWARE LICENSE version 0.51 ))??

This license is based closely on the Apache License Version 2.0, but is not
approved or endorsed by the Apache Foundation. A copy of the non-modified Apache
License 2.0 can be found at http:/www.apache.org/licenses/LICENSE-2.0.

As this license is not currently OSI or FSF approved, the Licensor permits any
Work licensed under this License, at the option of the Licensee, to be treated
as licensed under the Apache License Version 2.0 (which is so approved).

This License is licensed under the terms of this License and in particular
clause 7 below (Disclaimer of Warranties) applies in relation to its use.
(( TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION ))??

   (( 1. ))??
   Definitions.

      "License" shall mean the terms and conditions for use, reproduction, and
      distribution as defined by Sections 1 through 9 of this document.

      "Licensor" shall mean the Rights owner or entity authorized by the Rights
      owner that is granting the License.

      "Legal Entity" shall mean the union of the acting entity and all other
      entities that control, are controlled by, or are under common control with
      that entity. For the purposes of this definition, "control" means (i) the
      power, direct or indirect, to cause the direction or management of such
      entity, whether by contract or otherwise, or (ii) ownership of fifty
      percent (50%) or more of the outstanding shares, or (iii) beneficial
      ownership of such entity.

      "You" (or "Your") shall mean an individual or Legal Entity exercising
      permissions granted by this License.

      "Rights" means copyright and any similar right including design right
      (whether registered or unregistered), semiconductor topography (mask)
      rights and database rights (but excluding Patents and Trademarks).

      "Source" form shall mean the preferred form for making modifications,
      including but not limited to source code, net lists, board layouts, CAD
      files, documentation source, and configuration files.

      "Object" form shall mean any form resulting from mechanical transformation
      or translation of a Source form, including but not limited to compiled
      object code, generated documentation, the instantiation of a hardware
      design and conversions to other media types, including intermediate forms
      such as bytecodes, FPGA bitstreams, artwork and semiconductor topographies
      (mask works).

      "Work" shall mean the work of authorship, whether in Source form or other
      Object form, made available under the License, as indicated by a Rights
      notice that is included in or attached to the work (an example is provided
      in the Appendix below).

      "Derivative Works" shall mean any work, whether in Source or Object form,
      that is based on (or derived from) the Work and for which the editorial
      revisions, annotations, elaborations, or other modifications represent, as
      a whole, an original work of authorship. For the purposes of this License,
      Derivative Works shall not include works that remain separable from, or
      merely link (or bind by name) or physically connect to or interoperate
      with the interfaces of, the Work and Derivative Works thereof.

      "Contribution" shall mean any design or work of authorship, including the
      original version of the Work and any modifications or additions to that
      Work or Derivative Works thereof, that is intentionally submitted to
      Licensor for inclusion in the Work by the Rights owner or by an individual
      or Legal Entity authorized to submit on behalf of the Rights owner. For
      the purposes of this definition, "submitted" means any form of electronic,
      verbal, or written communication sent to the Licensor or its
      representatives, including but not limited to communication on electronic
      mailing lists, source code control systems, and issue tracking systems
      that are managed by, or on behalf of, the Licensor for the purpose of
      discussing and improving the Work, but excluding communication that is
      conspicuously marked or otherwise designated in writing by the Rights
      owner as "Not a Contribution."

      "Contributor" shall mean Licensor and any individual or Legal Entity on
      behalf of whom a Contribution has been received by Licensor and
      subsequently incorporated within the Work.

   (( 2. ))??
   Grant of License. Subject to the terms and conditions of this License, each
   Contributor hereby grants to You a perpetual, worldwide, non-exclusive,
   no-charge, royalty-free, irrevocable license under the Rights to reproduce,
   prepare Derivative Works of, publicly display, publicly perform, sublicense,
   and distribute the Work and such Derivative Works in Source or Object form
   and do anything in relation to the Work as if the Rights did not exist.

   (( 3. ))??
   Grant of Patent License. Subject to the terms and conditions of this License,
   each Contributor hereby grants to You a perpetual, worldwide, non-exclusive,
   no-charge, royalty-free, irrevocable (except as stated in this section)
   patent license to make, have made, use, offer to sell, sell, import, and
   otherwise transfer the Work, where such license applies only to those patent
   claims licensable by such Contributor that are necessarily infringed by their
   Contribution(s) alone or by combination of their Contribution(s) with the
   Work to which such Contribution(s) was submitted. If You institute patent
   litigation against any entity (including a cross-claim or counterclaim in a
   lawsuit) alleging that the Work or a Contribution incorporated within the
   Work constitutes direct or contributory patent infringement, then any patent
   licenses granted to You under this License for that Work shall terminate as
   of the date such litigation is filed.

   (( 4. ))??
   Redistribution. You may reproduce and distribute copies of the Work or
   Derivative Works thereof in any medium, with or without modifications, and in
   Source or Object form, provided that You meet the following conditions:

      (( 1. ))??
      You must give any other recipients of the Work or Derivative Works a copy
      of this License; and

      (( 2. ))??
      You must cause any modified files to carry prominent notices stating that
      You changed the files; and

      (( 3. ))??
      You must retain, in the Source form of any Derivative Works that You
      distribute, all copyright, patent, trademark, and attribution notices from
      the Source form of the Work, excluding those notices that do not pertain
      to any part of the Derivative Works; and

      (( 4. ))??
      If the Work includes a "NOTICE" text file as part of its distribution,
      then any Derivative Works that You distribute must include a readable copy
      of the attribution notices contained within such NOTICE file, excluding
      those notices that do not pertain to any part of the Derivative Works, in
      at least one of the following places: within a NOTICE text file
      distributed as part of the Derivative Works; within the Source form or
      documentation, if provided along with the Derivative Works; or, within a
      display generated by the Derivative Works, if and wherever such
      third-party notices normally appear. The contents of the NOTICE file are
      for informational purposes only and do not modify the License. You may add
      Your own attribution notices within Derivative Works that You distribute,
      alongside or as an addendum to the NOTICE text from the Work, provided
      that such additional attribution notices cannot be construed as modifying
      the License. You may add Your own copyright statement to Your
      modifications and may provide additional or different license terms and
      conditions for use, reproduction, or distribution of Your modifications,
      or for any such Derivative Works as a whole, provided Your use,
      reproduction, and distribution of the Work otherwise complies with the
      conditions stated in this License.

   (( 5. ))??
   Submission of Contributions. Unless You explicitly state otherwise, any
   Contribution intentionally submitted for inclusion in the Work by You to the
   Licensor shall be under the terms and conditions of this License, without any
   additional terms or conditions. Notwithstanding the above, nothing herein
   shall supersede or modify the terms of any separate license agreement you may
   have executed with Licensor regarding such Contributions.

   (( 6. ))??
   Trademarks. This License does not grant permission to use the trade names,
   trademarks, service marks, or product names of the Licensor, except as
   required for reasonable and customary use in describing the origin of the
   Work and reproducing the content of the NOTICE file.

   (( 7. ))??
   Disclaimer of Warranty. Unless required by applicable law or agreed to in
   writing, Licensor provides the Work (and each Contributor provides its
   Contributions) on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
   KIND, either express or implied, including, without limitation, any
   warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or
   FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining
   the appropriateness of using or redistributing the Work and assume any risks
   associated with Your exercise of permissions under this License.

   (( 8. ))??
   Limitation of Liability. In no event and under no legal theory, whether in
   tort (including negligence), contract, or otherwise, unless required by
   applicable law (such as deliberate and grossly negligent acts) or agreed to
   in writing, shall any Contributor be liable to You for damages, including any
   direct, indirect, special, incidental, or consequential damages of any
   character arising as a result of this License or out of the use or inability
   to use the Work (including but not limited to damages for loss of goodwill,
   work stoppage, computer failure or malfunction, or any and all other
   commercial damages or losses), even if such Contributor has been advised of
   the possibility of such damages.

   (( 9. ))??
   Accepting Warranty or Additional Liability. While redistributing the Work or
   Derivative Works thereof, You may choose to offer, and charge a fee for,
   acceptance of support, warranty, indemnity, or other liability obligations
   and/or rights consistent with this License. However, in accepting such
   obligations, You may act only on Your own behalf and on Your sole
   responsibility, not on behalf of any other Contributor, and only if You agree
   to indemnify, defend, and hold each Contributor harmless for any liability
   incurred by, or claims asserted against, such Contributor by reason of your
   accepting any such warranty or additional liability.
   (( END OF TERMS AND CONDITIONS

APPENDIX: How to apply this license to your work

To apply this license to your work, attach the following boilerplate notice,
with the fields enclosed by brackets "[]" replaced with your own identifying
information. (Don't include the brackets!) The text should be enclosed in the
appropriate comment syntax for the file format. We also recommend that a file or
class name and description of purpose be included on the same "printed page" as
the copyright notice for easier identification within third-party archives.

Copyright __20__

Copyright and related rights are licensed under the Solderpad Hardware License,
Version 0.51 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http:/solderpad.org/licenses/SHL-0.51. Unless required by applicable law or
agreed to in writing, software, hardware and materials distributed under this
License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
ANY KIND, either express or implied. See the License for the specific language
governing permissions and limitations under the License. ))??
`
const license_SISSL_lre = `//**
Sun Industry Standards Source License v1.1
https://spdx.org/licenses/SISSL.json
http://www.openoffice.org/licenses/sissl_license.html
https://opensource.org/licenses/SISSL
**//

(( Sun Industry Standards Source License - Version 1.1 ))??

   (( 1.0 ))??
   DEFINITIONS

      (( 1.1 ))??
      "Commercial Use" means distribution or otherwise making the Original Code
      available to a third party.

      (( 1.2 ))??
      "Contributor Version" means the combination of the Original Code, and the
      Modifications made by that particular Contributor.

      (( 1.3 ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.4 ))??
      "Executable" means Original Code in any form other than Source Code.

      (( 1.5 ))??
      "Initial Developer" means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.6 ))??
      "Larger Work" means a work which combines Original Code or portions
      thereof with code not governed by the terms of this License.

      (( 1.7 ))??
      "License" means this document.

      (( 1.8 ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.9 ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. A
      Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10 ))??
      "Original Code" means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code.

      (( 1.11 ))??
      "Patent Claims" means any patent claim(s), now owned or hereafter
      acquired, including without limitation, method, process, and apparatus
      claims, in any patent Licensable by grantor.

      (( 1.12 ))??
      "Source Code" means the preferred form of the Original Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, or scripts used to control
      compilation and installation of an Executable.

      (( 1.13 ))??
      "Standards" means the standards identified in Exhibit B.

      (( 1.14 ))??
      "You" (or "Your") means an individual or a legal entity exercising rights
      under, and complying with all of the terms of, this License or a future
      version of this License issued under Section 6.1. For legal entities,
      "You'' includes any entity which controls, is controlled by, or is under
      common control with You. For purposes of this definition, "control'' means
      (a) the power, direct or indirect, to cause the direction or management of
      such entity, whether by contract or otherwise, or (b) ownership of more
      than fifty percent (50%) of the outstanding shares or beneficial ownership
      of such entity.

   (( 2.0 ))??
   SOURCE CODE LICENSE

      (( 2.1 ))??
      The Initial Developer Grant The Initial Developer hereby grants You a
      world-wide, royalty-free, non-exclusive license, subject to third party
      intellectual property claims:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Initial Developer to use, reproduce, modify, display,
         perform, sublicense and distribute the Original Code (or portions
         thereof) with or without Modifications, and/or as part of a Larger
         Work; and

         (( (b) ))??
         under Patents Claims infringed by the making, using or selling of
         Original Code, to make, have made, use, practice, sell, and offer for
         sale, and/or otherwise dispose of the Original Code (or portions
         thereof).

         (( c ))??
         the licenses granted in this Section 2.1(a) and (b) are effective on
         the date Initial Developer first distributes Original Code under the
         terms of this License.

         (( (d) ))??
         Notwithstanding Section 2.1(b) above, no patent license is granted: 1)
         for code that You delete from the Original Code; 2) separate from the
         Original Code; or 3) for infringements caused by: i) the modification
         of the Original Code or ii) the combination of the Original Code with
         other software or devices, including but not limited to Modifications.

   (( 3.0 ))??
   DISTRIBUTION OBLIGATIONS

      (( 3.1 ))??
      Application of License. The Source Code version of Original Code may be
      distributed only under the terms of this License or a future version of
      this License released under Section 6.1, and You must include a copy of
      this License with every copy of the Source Code You distribute. You may
      not offer or impose any terms on any Source Code version that alters or
      restricts the applicable version of this License or the recipients' rights
      hereunder. Your license for shipment of the Contributor Version is
      conditioned upon Your full compliance with this Section. The Modifications
      which You create must comply with all requirements set out by the
      Standards body in effect one hundred twenty (120) days before You ship the
      Contributor Version. In the event that the Modifications do not meet such
      requirements, You agree to publish either (i) any deviation from the
      Standards protocol resulting from implementation of Your Modifications and
      a reference implementation of Your Modifications or (ii) Your
      Modifications in Source Code form, and to make any such deviation and
      reference implementation or Modifications available to all third parties
      under the same terms as this license on a royalty free basis within thirty
      (30) days of Your first customer shipment of Your Modifications.

      (( 3.2 ))??
      Required Notices. You must duplicate the notice in Exhibit A in each file
      of the Source Code. If it is not possible to put such notice in a
      particular Source Code file due to its structure, then You must include
      such notice in a location (such as a relevant directory) where a user
      would be likely to look for such a notice. If You created one or more
      Modification(s) You may add Your name as a Contributor to the notice
      described in Exhibit A. You must also duplicate this License in any
      documentation for the Source Code where You describe recipients' rights or
      ownership rights relating to Initial Code. You may choose to offer, and to
      charge a fee for, warranty, support, indemnity or liability obligations to
      one or more recipients of Your version of the Code. However, You may do so
      only on Your own behalf, and not on behalf of the Initial Developer. You
      must make it absolutely clear than any such warranty, support, indemnity
      or liability obligation is offered by You alone, and You hereby agree to
      indemnify the Initial Developer for any liability incurred by the Initial
      Developer as a result of warranty, support, indemnity or liability terms
      You offer.

      (( 3.3 ))??
      Distribution of Executable Versions. You may distribute Original Code in
      Executable and Source form only if the requirements of Sections 3.1 and
      3.2 have been met for that Original Code, and if You include a notice
      stating that the Source Code version of the Original Code is available
      under the terms of this License. The notice must be conspicuously included
      in any notice in an Executable or Source versions, related documentation
      or collateral in which You describe recipients' rights relating to the
      Original Code. You may distribute the Executable and Source versions of
      Your version of the Code or ownership rights under a license of Your
      choice, which may contain terms different from this License, provided that
      You are in compliance with the terms of this License. If You distribute
      the Executable and Source versions under a different license You must make
      it absolutely clear that any terms which differ from this License are
      offered by You alone, not by the Initial Developer. You hereby agree to
      indemnify the Initial Developer for any liability incurred by the Initial
      Developer as a result of any such terms You offer.

      (( 3.4 ))??
      Larger Works. You may create a Larger Work by combining Original Code with
      other code not governed by the terms of this License and distribute the
      Larger Work as a single product. In such a case, You must make sure the
      requirements of this License are fulfilled for the Original Code.

   (( 4.0 ))??
   INABILITY TO COMPLY DUE TO STATUTE OR REGULATION

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Original Code due to statute, judicial
   order, or regulation then You must: (a) comply with the terms of this License
   to the maximum extent possible; and (b) describe the limitations and the code
   they affect. Such description must be included in the LEGAL file described in
   Section 3.2 and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5.0 ))??
   APPLICATION OF THIS LICENSE

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A and to related Modifications as set out in Section 3.1.

   (( 6.0 ))??
   VERSIONS OF THE LICENSE

      (( 6.1 ))??
      New Versions. Sun may publish revised and/or new versions of the License
      from time to time. Each version will be given a distinguishing version
      number.

      (( 6.2 ))??
      Effect of New Versions. Once Original Code has been published under a
      particular version of the License, You may always continue to use it under
      the terms of that version. You may also choose to use such Original Code
      under the terms of any subsequent version of the License published by Sun.
      No one other than Sun has the right to modify the terms applicable to
      Original Code.

   (( 7.0 ))??
   DISCLAIMER OF WARRANTY

   ORIGINAL CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE ORIGINAL CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE ORIGINAL CODE IS WITH YOU. SHOULD
   ANY ORIGINAL CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL
   DEVELOPER) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR OR CORRECTION.
   THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE. NO
   USE OF ANY ORIGINAL CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS
   DISCLAIMER.

   (( 8.0 ))??
   TERMINATION

      (( 8.1 ))??
      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. All sublicenses to the
      Original Code which are properly granted shall survive any termination of
      this License. Provisions which, by their nature, must remain in effect
      beyond the termination of this License shall survive.

      (( 8.2 ))??
      In the event of termination under Section 8.1 above, all end user license
      agreements (excluding distributors and resellers) which have been validly
      granted by You or any distributor hereunder prior to termination shall
      survive termination.

   (( 9.0 ))??
   LIMIT OF LIABILITY

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, THE INITIAL DEVELOPER, ANY
   OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF ORIGINAL CODE, OR ANY SUPPLIER OF
   ANY OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10.0 ))??
   U.S. GOVERNMENT END USERS

   U.S. Government: If this Software is being acquired by or on behalf of the
   U.S. Government or by a U.S. Government prime contractor or subcontractor (at
   any tier), then the Government's rights in the Software and accompanying
   documentation shall be only as set forth in this license; this is in
   accordance with 48 C.F.R. 227.7201 through 227.7202-4 (for Department of
   Defense (DoD) acquisitions) and with 48 C.F.R. 2.101 and 12.212 (for non-DoD
   acquisitions).

   (( 11.0 ))??
   MISCELLANEOUS

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by California law provisions
   (except to the extent applicable law, if any, provides otherwise), excluding
   its conflict-of-law provisions. With respect to disputes in which at least
   one party is a citizen of, or an entity chartered or registered to do
   business in the United States of America, any litigation relating to this
   License shall be subject to the jurisdiction of the Federal Courts of the
   Northern District of California, with venue lying in Santa Clara County,
   California, with the losing party responsible for costs, including without
   limitation, court costs and reasonable attorneys' fees and expenses. The
   application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any law or regulation
   which provides that the language of a contract shall be construed against the
   drafter shall not apply to this License.
   (( EXHIBIT A - Sun Standards License

The contents of this file are subject to the Sun Standards License Version 1.1
(the "License"); You may not use this file except in compliance with the
License. You may obtain a copy of the License at
__5__

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either

express or implied. See the License for the specific language governing rights
and limitations under the License.

The Original Code is __5__.
The Initial Developer of the Original Code is __5__.
Portions created by __5__ are Copyright (C) __5__ . All Rights Reserved.
Contributor(s): __5__ ."

EXHIBIT B - Standards

The Standard is defined as the following:

OpenOffice.org XML File Format Specification, located at
http:/xml.openoffice.org

OpenOffice.org Application Programming Interface Specification, located at

http:/api.openoffice.org ))??
`
const license_SISSL_1_2_lre = `//**
Sun Industry Standards Source License v1.2
https://spdx.org/licenses/SISSL-1.2.json
http://gridscheduler.sourceforge.net/Gridengine_SISSL_license.html
**//

((

SUN INDUSTRY STANDARDS SOURCE LICENSE

Version 1.2 ))??

   (( 1.0 ))??
   DEFINITIONS

      (( 1.1 ))??
      Commercial Use means distribution or otherwise making the Original Code
      available to a third party.

      (( 1.2 ))??
      Contributor Version means the combination of the Original Code, and the
      Modifications made by that particular Contributor.

      (( 1.3 ))??
      Electronic Distribution Mechanism means a mechanism generally accepted in
      the software development community for the electronic transfer of data.

      (( 1.4 ))??
      Executable means Original Code in any form other than Source Code.

      (( 1.5 ))??
      Initial Developer means the individual or entity identified as the Initial
      Developer in the Source Code notice required by Exhibit A.

      (( 1.6 ))??
      Larger Work means a work which combines Original Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.7 ))??
      License means this document.

      (( 1.8 ))??
      Licensable means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.9 ))??
      Modifications means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. A
      Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10 ))??
      Original Code means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code.

      (( 1.11 ))??
      Patent Claims means any patent claim(s), now owned or hereafter acquired,
      including without limitation, method, process, and apparatus claims, in
      any patent Licensable by grantor.

      (( 1.12 ))??
      Source Code means the preferred form of the Original Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, or scripts used to control
      compilation and installation of an Executable.

      (( 1.13 ))??
      Standards means the standards identified in Exhibit B.

      (( 1.14 ))??
      You (or Your) means an individual or a legal entity exercising rights
      under, and complying with all of the terms of, this License or a future
      version of this License issued under Section 6.1. For legal entities, You
      includes any entity which controls, is controlled by, or is under common
      control with You. For purposes of this definition, control means (a) the
      power, direct or indirect, to cause the direction or management of such
      entity, whether by contract or otherwise, or (b) ownership of more than
      fifty percent (50%) of the outstanding shares or beneficial ownership of
      such entity.

   (( 2.0 ))??
   SOURCE CODE LICENSE

      (( 2.1 ))??
      The Initial Developer Grant The Initial Developer hereby grants You a
      world-wide, royalty-free, non-exclusive license, subject to third party
      intellectual property claims:

         (a)under intellectual property rights (other than patent or trademark)
         Licensable by Initial Developer to use, reproduce, modify, display,
         perform, sublicense and distribute the Original Code (or portions
         thereof) with or without Modifications, and/or as part of a Larger
         Work; and

         (( (b) ))??
         under Patents Claims infringed by the making, using or selling of
         Original Code, to make, have made, use, practice, sell, and offer for
         sale, and/or otherwise dispose of the Original Code (or portions
         thereof).

         (( c ))??
         the licenses granted in this Section 2.1(a) and (b) are effective on
         the date Initial Developer first distributes Original Code under the
         terms of this License.

         (( (d) ))??
         Notwithstanding Section 2.1(b) above, no patent license is granted: 1)
         for code that You delete from the Original Code; 2) separate from the
         Original Code; or 3) for infringements caused by: i) the modification
         of the Original Code or ii) the combination of the Original Code with
         other software or devices, including but not limited to Modifications.

   (( 3.0 ))??
   DISTRIBUTION OBLIGATIONS

      (( 3.1 ))??
      Application of License. The Source Code version of Original Code may be
      distributed only under the terms of this License or a future version of
      this License released under Section 6.1, and You must include a copy of
      this License with every copy of the Source Code You distribute. You may
      not offer or impose any terms on any Source Code version that alters or
      restricts the applicable version of this License or the recipients rights
      hereunder. Your license for shipment of the Contributor Version is
      conditioned upon Your full compliance with this Section. The Modifications
      which You create must comply with all requirements set out by the
      Standards body in effect one hundred twenty (120) days before You ship the
      Contributor Version. In the event that the Modifications do not meet such
      requirements, You agree to publish either (i) any deviation from the
      Standards protocol resulting from implementation of Your Modifications and
      a reference implementation of Your Modifications or (ii) Your
      Modifications in Source Code form, and to make any such deviation and
      reference implementation or Modifications available to all third parties
      under the same terms a this license on a royalty free basis within thirty
      (30) days of Your first customer shipment of Your Modifications.
      Additionally, in the event that the Modifications you create do not meet
      the requirements set out in this Section, You agree to comply with the
      Standards requirements set out in Exhibit B.

      (( 3.2 ))??
      Required Notices. You must duplicate the notice in Exhibit A in each file
      of the Source Code. If it is not possible to put such notice in a
      particular Source Code file due to its structure, then You must include
      such notice in a location (such as a relevant directory) where a user
      would be likely to look for such a notice. If You created one or more
      Modification(s) You may add Your name as a Contributor to the notice
      described in Exhibit A. You must also duplicate this License in any
      documentation for the Source Code where You describe recipients rights or
      ownership rights relating to Initial Code.

      You may choose to offer, and to charge a fee for, warranty, support,
      indemnity or liability obligations to one or more recipients of Your
      version of the Code. However, You may do so only on Your own behalf, and
      not on behalf of the Initial Developer. You must make it absolutely clear
      than any such warranty, support, indemnity or liability obligation is
      offered by You alone, and You hereby agree to indemnify the Initial
      Developer for any liability incurred by the Initial Developer as a result
      of warranty, support, indemnity or liability terms You offer.

      (( 3.3 ))??
      Distribution of Executable Versions. You may distribute Original Code in
      Executable and Source form only if the requirements of Sections 3.1 and
      3.2 have been met for that Original Code, and if You include a notice
      stating that the Source Code version of the Original Code is available
      under the terms of this License. The notice must be conspicuously included
      in any notice in an Executable or Source versions, related documentation
      or collateral in which You describe recipients rights relating to the
      Original Code. You may distribute the Executable and Source versions of
      Your version of the Code or ownership rights under a license of Your
      choice, which may contain terms different from this License, provided that
      You are in compliance with the terms of this License. If You distribute
      the Executable and Source versions under a different license You must make
      it absolutely clear that any terms which differ from this License are
      offered by You alone, not by the Initial Developer. You hereby agree to
      indemnify the Initial Developer for any liability incurred by the Initial
      Developer as a result of any such terms You offer.

      (( 3.4 ))??
      Larger Works. You may create a Larger Work by combining Original Code with
      other code not governed by the terms of this License and distribute the
      Larger Work as a single product. In such a case, You must make sure the
      requirements of this License are fulfilled for the Original Code.

   (( 4.0 ))??
   INABILITY TO COMPLY DUE TO STATUTE OR REGULATION

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Original Code due to statute, judicial
   order, or regulation then You must: (a) comply with the terms of this License
   to the maximum extent possible; and (b) describe the limitations and the code
   they affect. Such description must be included in the LEGAL file described in
   Section 3.2 and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5.0 ))??
   APPLICATION OF THIS LICENSE

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A and to related Modifications as set out in Section 3.1.

   (( 6.0 ))??
   VERSIONS OF THE LICENSE

      (( 6.1 ))??
      New Versions. Sun may publish revised and/or new versions of the License
      from time to time. Each version will be given a distinguishing version
      number.

      (( 6.2 ))??
      Effect of New Versions. Once Original Code has been published under a
      particular version of the License, You may always continue to use it under
      the terms of that version. You may also choose to use such Original Code
      under the terms of any subsequent version of the License published by Sun.
      No one other than Sun has the right to modify the terms applicable to
      Original Code.

   (( 7.0 ))??
   DISCLAIMER OF WARRANTY

   ORIGINAL CODE IS PROVIDED UNDER THIS LICENSE ON AN AS IS BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE ORIGINAL CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE ORIGINAL CODE IS WITH YOU. SHOULD
   ANY ORIGINAL CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL
   DEVELOPER) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR OR CORRECTION.
   THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE. NO
   USE OF ANY ORIGINAL CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS
   DISCLAIMER.

   (( 8.0 ))??
   TERMINATION

      (( 8.1 ))??
      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. All sublicenses to the
      Original Code which are properly granted shall survive any termination of
      this License. Provisions which, by their nature, must remain in effect
      beyond the termination of this License shall survive. 8.2 In the event of
      termination under Section 8.1 above, all end user license agreements
      (excluding distributors and resellers) which have been validly granted by
      You or any distributor hereunder prior to termination shall survive
      termination.
      (( EXHIBIT A - Sun Industry Standards Source License

The contents of this file are subject to the Sun Industry Standards Source
License Version 1.2 (the License); You

may not use this file except in compliance with the License.

You may obtain a copy of the License at gridengine.sunsource.net/license.html

Software distributed under the License is distributed on an AS IS basis, WITHOUT
WARRANTY OF ANY KIND, either express or

implied. See the License for the specific language governing rights and
limitations under the License.

The Original Code is Grid Engine.

The Initial Developer of the Original Code is:

Sun Microsystems, Inc.

Portions created by: Sun Microsystems, Inc. are Copyright (C) 2001 Sun
Microsystems, Inc.

All Rights Reserved.

Contributor(s):
//** __________________________________ **//
__5__

EXHIBIT B - Standards

   (( 1.0 ))??
   Requirements for project Standards. The requirements for project Standards
   are version-dependent and are defined at: Grid Engine standards.

   (( 2.0 ))??
   Additional requirements. The additional requirements pursuant to Section 3.1
   are defined as:

      (( 2.1 ))??
      Naming Conventions. If any of your Modifications do not meet the
      requirements of the Standard, then you must change the product name so
      that Grid Engine, gridengine, gridengine.sunsource, and similar naming
      conventions are not used.

      (( 2.2 ))??
      Compliance Claims. If any of your Modifications do not meet the
      requirements of the Standards you may not claim, directly or indirectly,
      that your implementation of the Standards is compliant. ))??
`
const license_SMLNJ_lre = `//**
Standard ML of New Jersey License
https://spdx.org/licenses/SMLNJ.json
https://www.smlnj.org/license.html
**//

(( STANDARD ML OF NEW JERSEY COPYRIGHT NOTICE, LICENSE AND DISCLAIMER.
(( Copyright __20__ ))??
))??

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted, provided that
the above copyright notice appear in all copies and that both the copyright
notice and this permission notice and warranty disclaimer appear in supporting
documentation, and that the name of Lucent Technologies, Bell Labs or any Lucent
entity not be used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

Lucent disclaims all warranties with regard to this software, including all
implied warranties of merchantability and fitness. In no event shall Lucent be
liable for any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortious action, arising out of or in connection
with the use or performance of this software.
`
const license_SMPPL_lre = `//**
Secure Messaging Protocol Public License
https://spdx.org/licenses/SMPPL.json
https://github.com/dcblake/SMP/blob/master/Documentation/License.txt
**//

(( Secure Messaging Protocol (SMP) Libraries [ACL, CML, SFL] ))??

Distribution Rights

All source code for the SMP is being provided at no cost and with no financial
limitations regarding its use and distribution. Organizations can use the SMP
without paying any royalties or licensing fees. The SMP was originally developed
by the U.S. Government. BAE Systems is enhancing and supporting the SMP under
contract to the U.S. Government. The U.S. Government is furnishing the SMP
software at no cost to the vendor subject to the conditions of the SMP Public
License provided with the SMP software.

29 May 2002

Secure Messaging Protocol (SMP) Public License

The United States Government/Department of Defense/National Security
Agency/Office of Network Security (collectively "the U.S. Government") hereby
grants permission to any person obtaining a copy of the SMP source and object
files (the "SMP Software") and associated documentation files (the "SMP
Documentation"), or any portions thereof, to do the following, subject to the
following license conditions:

You may, free of charge and without additional permission from the U.S.
Government, use, copy, modify, sublicense and otherwise distribute the SMP
Software or components of the SMP Software, with or without modifications
developed by you and/or by others.

You may, free of charge and without additional permission from the U.S.
Government, distribute copies of the SMP Documentation, with or without
modifications developed by you and/or by others, at no charge or at a charge
that covers the cost of reproducing such copies, provided that this SMP Public
License is retained.

Furthermore, if you distribute the SMP Software or parts of the SMP Software,
with or without modifications developed by you and/or others, then you must
either make available the source code to all portions of the SMP Software
(exclusive of any modifications made by you and/or by others) upon request, or
instead you may notify anyone requesting the SMP Software source code that it is
freely available from the U.S. Government.

Transmission of this SMP Public License must accompany whatever portions of the
SMP Software you redistribute.

The SMP Software is provided without warranty or guarantee of any nature,
express or implied, including without limitation the warranties of
merchantability and fitness for a particular purpose.

The U.S. Government cannot be held liable for any damages either directly or
indirectly caused by the use of the SMP Software.

It is not permitted to copy, sublicense, distribute or transfer any of the SMP
Software except as expressly indicated herein. Any attempts to do otherwise will
be considered a violation of this License and your rights to the SMP Software
will be voided.

The SMP uses the Enhanced SNACC (eSNACC) Abstract Syntax Notation One (ASN.1)
C++ Library to ASN.1 encode and decode security-related data objects. The eSNACC
ASN.1 C++ Library is covered by the ENHANCED SNACC SOFTWARE PUBLIC LICENSE. None
of the GNU public licenses apply to the eSNACC ASN.1 C++ Library. The eSNACC
Compiler is not distributed as part of the SMP.

((Copyright © __3__ National Security Agency))??
`
const license_SNIA_lre = `//**
SNIA Public License 1.1
https://spdx.org/licenses/SNIA.json
https://fedoraproject.org/wiki/Licensing/SNIA_Public_License
**//

(( STORAGE NETWORKING INDUSTRY ASSOCIATION

PUBLIC LICENSE

Version 1.1 ))??

   (( 1. ))??
   Definitions.

   (( 1.1 ))??
   "Commercial Use" means distribution or otherwise making the Covered Code
   available to a third party.

   (( 1.2 ))??
   "Contributor" means each entity that creates or contributes to the creation
   of Modifications.

   (( 1.3 ))??
   "Contributor Version" means the combination of the Original Code, prior
   Modifications used by a Contributor, and the Modifications made by that
   particular Contributor.

   (( 1.4 ))??
   "Covered Code" means the Original Code or Modifications or the combination of
   the Original Code and Modifications, in each case including portions
   thereof.

   (( 1.5 ))??
   "Electronic Distribution Mechanism" means a mechanism generally accepted in
   the software development community for the electronic transfer of data.

   (( 1.6 ))??
   "Executable" means Covered Code in any form other than Source Code.

   (( 1.7 ))??
   "Initial Developer" means the individual or entity identified as the Initial
   Developer in the Source Code notice required by Exhibit A.

   (( 1.8 ))??
   "Larger Work" means a work which combines Covered Code or portions thereof
   with code not governed by the terms of this License.

   (( 1.9 ))??
   "License" means this document.

   (( 1.10 ))??
   "Licensable" means having the right to grant, to the maximum extent possible,
   whether at the time of the initial grant or subsequently acquired, any and
   all of the rights conveyed herein.

   (( 1.11 ))??
   "Modifications" means any addition to or deletion from the substance or
   structure of either the Original Code or any previous Modifications. When
   Covered Code is released as a series of files, a Modification is:

   (( A. ))??
   Any addition to or deletion from the contents of a file containing Original
   Code or previous Modifications.

   (( B. ))??
   Any new file that contains any part of the Original Code or previous
   Modifications.

   (( 1.12 ))??
   "Original Code" means Source Code of computer software code which is
   described in the Source Code notice required by Exhibit A as Original Code,
   and which, at the time of its release under this License is not already
   Covered Code governed by this License.

   (( 1.13 ))??
   "Patent Claims" means any patent claim(s), now owned or hereafter acquired,
   including without limitation, method, process, and apparatus claims, in any
   patent Licensable by grantor.

   (( 1.14 ))??
   "Source Code" means the preferred form of the Covered Code for making
   modifications to it, including all modules it contains, plus any associated
   interface definition files, scripts used to control compilation and
   installation of an Executable, or source code differential comparisons
   against either the Original Code or another well known, available Covered
   Code of the Contributor's choice. The Source Code can be in a compressed or
   archival form, provided the appropriate decompression or de-archiving
   software is widely available for no charge.

   (( 1.15 ))??
   "You" (or "Your") means an individual or a legal entity exercising rights
   under, and complying with all of the terms of, this License or a future
   version of this License issued under Section 6.1. For legal entities, "You"
   includes any entity which controls, is controlled by, or is under common
   control with You. For purposes of this definition, "control" means (a) the
   power, direct or indirect, to cause the direction or management of such
   entity, whether by contract or otherwise, or (b) ownership of more than fifty
   percent (50%) of the outstanding shares or beneficial ownership of such
   entity

   (( 2. ))??
   Source Code License.

   (( 2.1 ))??
   The Initial Developer Grant. The Initial Developer hereby grants You a
   world-wide, royalty-free, non-exclusive license, subject to third party
   intellectual property claims:

   (( (a) ))??
   under intellectual property rights (other than patent or trademark)
   Licensable by Initial Developer to use, reproduce, modify, display, perform,
   sublicense and distribute the Original Code (or portions thereof) with or
   without Modifications, and/or as part of a Larger Work; and

   (( (b) ))??
   under Patents Claims infringed by the making, using or selling of Original
   Code, to make, have made, use, practice, sell, and offer for sale, and/or
   otherwise dispose of the Original Code (or portions thereof).

   (( c ))??
   the licenses granted in this Section 2.1(a) and (b) are effective on the date
   Initial Developer first distributes Original Code under the terms of this
   License.

   (( (d) ))??
   Notwithstanding Section 2.1(b) above, no patent license is granted: 1) for
   code that You delete from the Original Code; 2) separate from the Original
   Code; or 3) for infringements caused by: i) the modification of the Original
   Code or ii) the combination of the Original Code with other software or
   devices.

   (( 2.2 ))??
   Contributor Grant. Subject to third party intellectual property claims, each
   Contributor hereby grants You a world-wide, royalty-free, non-exclusive
   license

   (( (a) ))??
   under intellectual property rights (other than patent or trademark)
   Licensable by Contributor, to use, reproduce, modify, display, perform,
   sublicense and distribute the Modifications created by such Contributor (or
   portions thereof) either on an unmodified basis, with other Modifications, as
   Covered Code and/or as part of a Larger Work; and

   (( (b) ))??
   under Patent Claims infringed by the making, using, or selling of
   Modifications made by that Contributor either alone and/or in combination
   with its Contributor Version (or portions of such combination), to make, use,
   sell, offer for sale, have made, and/or otherwise dispose of: 1)
   Modifications made by that Contributor (or portions thereof); and 2) the
   combination of Modifications made by that Contributor with its Contributor
   Version (or portions of such combination).

   (( c ))??
   the licenses granted in Sections 2.2(a) and 2.2(b) are effective on the date
   Contributor first makes Commercial Use of the Covered Code.

   (( (d) ))??
   Notwithstanding Section 2.2(b) above, no patent license is granted: 1) for
   any code that Contributor has deleted from the Contributor Version; 2)
   separate from the Contributor Version; 3) for infringements caused by: i)
   third party modifications of Contributor Version or ii) the combination of
   Modifications made by that Contributor with other software (except as part of
   the Contributor Version) or other devices; or 4) under Patent Claims
   infringed by Covered Code in the absence of Modifications made by that
   Contributor.

   (( 3. ))??
   Distribution Obligations.

   (( 3.1 ))??
   Application of License. The Modifications which You create or to which You
   contribute are governed by the terms of this License, including without
   limitation Section 2.2. The Source Code version of Covered Code may be
   distributed only under the terms of this License or a future version of this
   License released under Section 6.1, and You must include a copy of this
   License with every copy of the Source Code You distribute. You may not offer
   or impose any terms on any Source Code version that alters or restricts the
   applicable version of this License or the recipients' rights hereunder.
   However, You may include an additional document offering the additional
   rights described in Section 3.5.

   (( 3.2 ))??
   Availability of Source Code. Any Modification which You create or to which
   You contribute must be made available in Source Code form under the terms of
   this License either on the same media as an Executable version or via an
   accepted Electronic Distribution Mechanism to anyone to whom you made an
   Executable version available; and if made available via Electronic
   Distribution Mechanism, must remain available for at least twelve (12) months
   after the date it initially became available, or at least six (6) months
   after a subsequent version of that particular Modification has been made
   available to such recipients. You are responsible for ensuring that the
   Source Code version remains available even if the Electronic Distribution
   Mechanism is maintained by a third party.

   (( 3.3 ))??
   Description of Modifications. You must cause all Covered Code to which You
   contribute to contain a file documenting the changes You made to create that
   Covered Code and the date of any change. You must include a prominent
   statement that the Modification is derived, directly or indirectly, from
   Original Code provided by the Initial Developer and including the name of the
   Initial Developer in (a) the Source Code, and (b) in any notice in an
   Executable version or related documentation in which You describe the origin
   or ownership of the Covered Code.

   (( 3.4 ))??
   Intellectual Property Matters.

   (( (a) ))??
   Third Party Claims. If Contributor has actual knowledge that a license under
   a third party's intellectual property rights is required to exercise the
   rights granted by such Contributor under Sections 2.1 or 2.2, Contributor
   must include a text file with the Source Code distribution titled "LEGAL"
   which describes the claim and the party making the claim in sufficient detail
   that a recipient will know whom to contact. If Contributor obtains such
   knowledge after the Modification is made available as described in Section
   3.2, Contributor shall promptly modify the LEGAL file in all copies
   Contributor makes available thereafter.

   (( (b) ))??
   Contributor API's. If Contributor's Modifications include an application
   programming interface and Contributor has actual knowledge of patent licenses
   which are reasonably necessary to implement that API, Contributor must also
   include this information in the LEGAL file.

   (( c ))??
   Representations. Contributor represents that, except as disclosed pursuant to
   Section 3.4(a) above, Contributor believes that Contributor's Modifications
   are Contributor's original creation(s) and/or Contributor has sufficient
   rights to grant the rights conveyed by this License.

   (( 3.5 ))??
   Required Notices. You must duplicate the notice in Exhibit A in each file of
   the Source Code. If it is not possible to put such notice in a particular
   Source Code file due to its structure, then You must include such notice in a
   location (such as a relevant directory) where a user would be most likely to
   look for such a notice. If You created one or more Modification(s) You may
   add your name as a Contributor to the notice described in Exhibit A. You must
   also duplicate this License in any documentation for the Source Code where
   You describe recipients' rights or ownership rights relating to Covered Code.
   You may choose to offer, and to charge a fee for, warranty, support,
   indemnity or liability obligations to one or more recipients of Covered Code.
   However, You may do so only on Your own behalf, and not on behalf of the
   Initial Developer or any Contributor. You must make it absolutely clear that
   any such warranty, support, indemnity or liability obligation is offered by
   You alone, and You hereby agree to indemnify the Initial Developer and every
   Contributor for any liability (excluding any liability arising from
   intellectual property claims relating to the Covered Code) incurred by the
   Initial Developer or such Contributor as a result of warranty, support,
   indemnity or liability terms You offer.

   (( 3.6 ))??
   Distribution of Executable Versions. You may distribute Covered Code in
   Executable form only if the requirements of Section 3.1-3.5 have been met for
   that Covered Code, and if You include a notice stating that the Source Code
   version of the Covered Code is available under the terms of this License,
   including a description of how and where You have fulfilled the obligation of
   Section 3.2. The notice must be conspicuously included in any notice in an
   Executable version, related documentation or collateral in which You describe
   recipients' rights relating to the Covered Code. You may distribute the
   Executable version of Covered Code or ownership rights under a license of
   Your choice, which may contain terms different from this License, provided
   that You are in compliance with the terms of this License and that the
   license for the Executable version does not attempt to limit or alter the
   recipient's rights in the Source Code version from the rights set forth in
   this License. If You distribute the Executable version under a different
   license You must make it absolutely clear that any terms which differ from
   this License are offered by You alone, not by the Initial Developer or any
   Contributor. You hereby agree to indemnify the Initial Developer and every
   Contributor for any liability (excluding any liability arising from
   intellectual property claims relating to the Covered Code) incurred by the
   Initial Developer or such Contributor as a result of any such terms You
   offer.

   (( 3.7 ))??
   Larger Works. You may create a Larger Work by combining Covered Code with
   other code not governed by the terms of this License and distribute the
   Larger Work as a single product. In such a case, You must make sure the
   requirements of this License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation. If it is impossible for You
   to comply with any of the terms of this License with respect to some or all
   of the Covered Code due to statute, judicial order, or regulation then You
   must: (a) comply with the terms of this License to the maximum extent
   possible; and (b) describe the limitations and the code they affect. Such
   description must be included in the LEGAL file described in Section 3.4 and
   must be included with all distributions of the Source Code. Except to the
   extent prohibited by statute or regulation, such description must be
   sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Application of this License. This License applies to code to which the
   Initial Developer has attached the notice in Exhibit A and to related Covered
   Code.

   (( 6. ))??
   Versions of the License.

   (( 6.1 ))??
   New Versions. The Storage Networking Industry Association (the "SNIA") may
   publish revised and/or new versions of the License from time to time. Each
   version will be given a distinguishing version number.

   (( 6.2 ))??
   Effect of New Versions. Once Covered Code has been published under a
   particular version of the License, You may always continue to use it under
   the terms of that version. You may also choose to use such Covered Code under
   the terms of any subsequent version of the License published by the SNIA. No
   one other than the SNIA has the right to modify the terms applicable to
   Covered Code created under this License.

   (( 6.3 ))??
   Derivative Works. If You create or use a modified version of this License
   (which you may only do in order to apply it to code which is not already
   Covered Code governed by this License), You must (a) rename Your license so
   that the phrases "Storage Networking Industry Association," "SNIA," or any
   confusingly similar phrase do not appear in your license (except to note that
   your license differs from this License) and (b) otherwise make it clear that
   Your version of the license contains terms which differ from the SNIA Public
   License. (Filling in the name of the Initial Developer, Original Code or
   Contributor in the notice described in Exhibit A shall not of themselves be
   deemed to be modifications of this License.)

   (( 7. ))??
   DISCLAIMER OF WARRANTY. COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS
   IS" BASIS, WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,
   INCLUDING, WITHOUT LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF
   DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE
   ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH
   YOU. SHOULD ANY COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE
   INITIAL DEVELOPER OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY
   SERVICING, REPAIR OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN
   ESSENTIAL PART OF THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED
   HEREUNDER EXCEPT UNDER THIS DISCLAIMER.

   (( 8. ))??
   TERMINATION.

   (( 8.1 ))??
   This License and the rights granted hereunder will terminate automatically if
   You fail to comply with terms herein and fail to cure such breach within a
   reasonable time after becoming aware of the breach. All sublicenses to the
   Covered Code which are properly granted shall survive any termination of this
   License. Provisions which, by their nature, must remain in effect beyond the
   termination of this License shall survive.

   (( 8.2 ))??
   If You initiate litigation by asserting a patent infringement claim
   (excluding declaratory judgment actions) against Initial Developer or a
   Contributor (the Initial Developer or Contributor against whom You file such
   action is referred to as "Participant") alleging that: o (a) such
   Participant's Contributor Version directly or indirectly infringes any
   patent, then any and all rights granted by such Participant to You under
   Sections 2.1 and/or 2.2 of this License shall, upon 60 days notice from
   Participant terminate prospectively, unless if within 60 days after receipt
   of notice You either: (i) agree in writing to pay Participant a mutually
   agreeable reasonable royalty for Your past and future use of Modifications
   made by such Participant, or (ii) withdraw Your litigation claim with respect
   to the Contributor Version against such Participant. If within 60 days of
   notice, a reasonable royalty and payment arrangement are not mutually agreed
   upon in writing by the parties or the litigation claim is not withdrawn, the
   rights granted by Participant to You under Sections 2.1 and/or 2.2
   automatically terminate at the expiration of the 60 day notice period
   specified above.

   (( 8.3 ))??
   If You assert a patent infringement claim against Participant alleging that
   such Participant's Contributor Version directly or indirectly infringes any
   patent where such claim is resolved (such as by license or settlement) prior
   to the initiation of patent infringement litigation, then the reasonable
   value of the licenses granted by such Participant under Sections 2.1 or 2.2
   shall be taken into account in determining the amount or value of any payment
   or license.

   (( 8.4 ))??
   In the event of termination under Sections 8.1 or 8.2 above, all end user
   license agreements (excluding distributors and resellers) which have been
   validly granted by You or any distributor hereunder prior to termination
   shall survive termination.

   (( 9. ))??
   LIMITATION OF LIABILITY. UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY,
   WHETHER TORT (INCLUDING NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, THE
   INITIAL DEVELOPER, ANY OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE,
   OR ANY SUPPLIER OF ANY OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY
   INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER
   INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE,
   COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR
   LOSSES, EVEN IF SUCH PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF
   SUCH DAMAGES. THIS LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR
   DEATH OR PERSONAL INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT
   APPLICABLE LAW PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE
   EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS
   EXCLUSION AND LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. GOVERNMENT END USERS. The Covered Code is a "commercial item," as that
   term is defined in 48 C.F.R. 2.101 (Oct. 1995), consisting of "commercial
   computer software" and "commercial computer software documentation," as such
   terms are used in 48 C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R.
   12.212 and 48 C.F.R. 227.7202-1 through 227.7202-4 (June 1995), all U.S.
   Government End Users acquire Covered Code with only those rights set forth
   herein.

   (( 11. ))??
   MISCELLANEOUS This License represents the complete agreement concerning
   subject matter hereof. If any provision of this License is held to be
   unenforceable, such provision shall be reformed only to the extent necessary
   to make it enforceable. This License shall be governed by California law
   provisions (except to the extent applicable law, if any, provides otherwise),
   excluding its conflict-of-law provisions. The application of the United
   Nations Convention on Contracts for the International Sale of Goods is
   expressly excluded. Any law or regulation which provides that the language of
   a contract shall be construed against the drafter shall not apply to this
   License.

   (( 12. ))??
   RESPONSIBILITY FOR CLAIMS. As between Initial Developer and the Contributors,
   each party is responsible for claims and damages arising, directly or
   indirectly, out of its utilization of rights under this License and You agree
   to work with Initial Developer and Contributors to distribute such
   responsibility on an equitable basis. Nothing herein is intended or shall be
   deemed to constitute any admission of liability.

   (( 13. ))??
   MULTIPLE-LICENSED CODE. Initial Developer may designate portions of the
   Covered Code as "Multiple-Licensed". "Multiple-Licensed" means that the
   Initial Developer permits you to utilize portions of the Covered Code under
   Your choice of this License or the alternative licenses, if any, specified by
   the Initial Developer in the file described in Exhibit A.

   (( 14. ))??
   ACCEPTANCE. This License is accepted by You if You retain, use, or distribute
   the Covered Code for any purpose.
   (( EXHIBIT A The SNIA Public License.

The contents of this file are subject to the SNIA Public License Version 1.0
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at

www.snia.org/smi/developers/cim/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

The Original Code is .

The Initial Developer of the Original Code is [COMPLETE THIS] .

Contributor(s): _.

Read more about this license at http:/www.snia.org/smi/developers/open_source/ ))??
`
const license_SPL_1_0_lre = `//**
Sun Public License v1.0
https://spdx.org/licenses/SPL-1.0.json
https://opensource.org/licenses/SPL-1.0
**//

(( SUN PUBLIC LICENSE Version 1.0 ))??

   (( 1. ))??
   Definitions.

      (( 1.0.1. ))??
      "Commercial Use" means distribution or otherwise making the Covered Code
      available to a third party.

      (( 1.1. ))??
      "Contributor" means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      "Covered Code" means the Original Code or Modifications or the combination
      of the Original Code and Modifications, in each case including portions
      thereof and corresponding documentation released with the source code.

      (( 1.4. ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      "Executable" means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      "Initial Developer" means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7. ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.8. ))??
      "License" means this document.

      (( 1.8.1. ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.9. ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

      (( 1.10. ))??
      "Original Code"../ means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      (( 1.10.1. ))??
      "Patent Claims" means any patent claim(s), now owned or hereafter
      acquired, including without limitation, method, process, and apparatus
      claims, in any patent Licensable by grantor.

      (( 1.11. ))??
      "Source Code"../ means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated documentation, interface definition files, scripts used to
      control compilation and installation of an Executable, or source code
      differential comparisons against either the Original Code or another well
      known, available Covered Code of the Contributor's choice. The Source Code
      can be in a compressed or archival form, provided the appropriate
      decompression or de-archiving software is widely available for no charge.

      (( 1.12. ))??
      "You" (or "Your") means an individual or a legal entity exercising rights
      under, and complying with all of the terms of, this License or a future
      version of this License issued under Section 6.1. For legal entities,
      "You" includes any entity which controls, is controlled by, or is under
      common control with You. For purposes of this definition, "control"../
      means (a) the power, direct or indirect, to cause the direction or
      management of such entity, whether by contract or otherwise, or (b)
      ownership of more than fifty percent (50%) of the outstanding shares or
      beneficial ownership of such entity.

   (( 2. ))??
   Source Code License.

      (( 2.1 ))??
      The Initial Developer Grant. The Initial Developer hereby grants You a
      world-wide, royalty-free, non-exclusive license, subject to third party
      intellectual property claims:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Initial Developer to use, reproduce, modify, display,
         perform, sublicense and distribute the Original Code (or portions
         thereof) with or without Modifications, and/or as part of a Larger
         Work; and

         (( (b) ))??
         under Patent Claims infringed by the making, using or selling of
         Original Code, to make, have made, use, practice, sell, and offer for
         sale, and/or otherwise dispose of the Original Code (or portions
         thereof).

         (( c ))??
         the licenses granted in this Section 2.1(a) and (b) are effective on
         the date Initial Developer first distributes Original Code under the
         terms of this License.

         (( (d) ))??
         Notwithstanding Section 2.1(b) above, no patent license is granted: 1)
         for code that You delete from the Original Code; 2) separate from the
         Original Code; or 3) for infringements caused by:

            (( i) ))??
            the modification of the Original Code or

            (( ii) ))??
            the combination of the Original Code with other software or
            devices.

      (( 2.2. ))??
      Contributor Grant. Subject to third party intellectual property claims,
      each Contributor hereby grants You a world-wide, royalty-free,
      non-exclusive license

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Contributor, to use, reproduce, modify, display, perform,
         sublicense and distribute the Modifications created by such Contributor
         (or portions thereof) either on an unmodified basis, with other
         Modifications, as Covered Code and/or as part of a Larger Work; and

         (( b) ))??
         under Patent Claims infringed by the making, using, or selling of
         Modifications made by that Contributor either alone and/or in
         combination with its Contributor Version (or portions of such
         combination), to make, use, sell, offer for sale, have made, and/or
         otherwise dispose of: 1) Modifications made by that Contributor (or
         portions thereof); and 2) the combination of Modifications made by that
         Contributor with its Contributor Version (or portions of such
         combination).

         (( c ))??
         the licenses granted in Sections 2.2(a) and 2.2(b) are effective on the
         date Contributor first makes Commercial Use of the Covered Code.

         (( (d) ))??
         notwithstanding Section 2.2(b) above, no patent license is granted: 1)
         for any code that Contributor has deleted from the Contributor Version;
         2) separate from the Contributor Version; 3) for infringements caused
         by: i) third party modifications of Contributor Version or ii) the
         combination of Modifications made by that Contributor with other
         software (except as part of the Contributor Version) or other devices;
         or 4) under Patent Claims infringed by Covered Code in the absence of
         Modifications made by that Contributor.

   (( 3. ))??
   Distribution Obligations.

      (( 3.1. ))??
      Application of License. The Modifications which You create or to which You
      contribute are governed by the terms of this License, including without
      limitation Section 2.2. The Source Code version of Covered Code may be
      distributed only under the terms of this License or a future version of
      this License released under Section 6.1, and You must include a copy of
      this License with every copy of the Source Code You distribute. You may
      not offer or impose any terms on any Source Code version that alters or
      restricts the applicable version of this License or the recipients' rights
      hereunder. However, You may include an additional document offering the
      additional rights described in Section 3.5.

      (( 3.2. ))??
      Availability of Source Code. Any Modification which You create or to which
      You contribute must be made available in Source Code form under the terms
      of this License either on the same media as an Executable version or via
      an accepted Electronic Distribution Mechanism to anyone to whom you made
      an Executable version available; and if made available via Electronic
      Distribution Mechanism, must remain available for at least twelve (12)
      months after the date it initially became available, or at least six (6)
      months after a subsequent version of that particular Modification has been
      made available to such recipients. You are responsible for ensuring that
      the Source Code version remains available even if the Electronic
      Distribution Mechanism is maintained by a third party.

      (( 3.3. ))??
      Description of Modifications. You must cause all Covered Code to which You
      contribute to contain a file documenting the changes You made to create
      that Covered Code and the date of any change. You must include a prominent
      statement that the Modification is derived, directly or indirectly, from
      Original Code provided by the Initial Developer and including the name of
      the Initial Developer in (a) the Source Code, and (b) in any notice in an
      Executable version or related documentation in which You describe the
      origin or ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters.

         (( (a) ))??
         Third Party Claims. If Contributor has knowledge that a license under a
         third party's intellectual property rights is required to exercise the
         rights granted by such Contributor under Sections 2.1 or 2.2,
         Contributor must include a text file with the Source Code distribution
         titled "../LEGAL'' which describes the claim and the party making the
         claim in sufficient detail that a recipient will know whom to contact.
         If Contributor obtains such knowledge after the Modification is made
         available as described in Section 3.2, Contributor shall promptly
         modify the LEGAL file in all copies Contributor makes available
         thereafter and shall take other steps (such as notifying appropriate
         mailing lists or newsgroups) reasonably calculated to inform those who
         received the Covered Code that new knowledge has been obtained.

         (( (b) ))??
         Contributor APIs. If Contributor's Modifications include an application
         programming interface ("API"../) and Contributor has knowledge of
         patent licenses which are reasonably necessary to implement that API,
         Contributor must also include this information in the LEGAL file.

         (( c ))??
         Representations. Contributor represents that, except as disclosed
         pursuant to Section 3.4(a) above, Contributor believes that
         Contributor's Modifications are Contributor's original creation(s)
         and/or Contributor has sufficient rights to grant the rights conveyed
         by this License.

      (( 3.5. ))??
      Required Notices. You must duplicate the notice in Exhibit A in each file
      of the Source Code. If it is not possible to put such notice in a
      particular Source Code file due to its structure, then You must include
      such notice in a location (such as a relevant directory) where a user
      would be likely to look for such a notice. If You created one or more
      Modification(s) You may add your name as a Contributor to the notice
      described in Exhibit A. You must also duplicate this License in any
      documentation for the Source Code where You describe recipients' rights or
      ownership rights relating to Covered Code. You may choose to offer, and to
      charge a fee for, warranty, support, indemnity or liability obligations to
      one or more recipients of Covered Code. However, You may do so only on
      Your own behalf, and not on behalf of the Initial Developer or any
      Contributor. You must make it absolutely clear than any such warranty,
      support, indemnity or liability obligation is offered by You alone, and
      You hereby agree to indemnify the Initial Developer and every Contributor
      for any liability incurred by the Initial Developer or such Contributor as
      a result of warranty, support, indemnity or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Versions. You may distribute Covered Code in
      Executable form only if the requirements of Section 3.1-3.5 have been met
      for that Covered Code, and if You include a notice stating that the Source
      Code version of the Covered Code is available under the terms of this
      License, including a description of how and where You have fulfilled the
      obligations of Section 3.2. The notice must be conspicuously included in
      any notice in an Executable version, related documentation or collateral
      in which You describe recipients' rights relating to the Covered Code. You
      may distribute the Executable version of Covered Code or ownership rights
      under a license of Your choice, which may contain terms different from
      this License, provided that You are in compliance with the terms of this
      License and that the license for the Executable version does not attempt
      to limit or alter the recipient's rights in the Source Code version from
      the rights set forth in this License. If You distribute the Executable
      version under a different license You must make it absolutely clear that
      any terms which differ from this License are offered by You alone, not by
      the Initial Developer or any Contributor. You hereby agree to indemnify
      the Initial Developer and every Contributor for any liability incurred by
      the Initial Developer or such Contributor as a result of any such terms
      You offer.

      (( 3.7. ))??
      Larger Works. You may create a Larger Work by combining Covered Code with
      other code not governed by the terms of this License and distribute the
      Larger Work as a single product. In such a case, You must make sure the
      requirements of this License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute, judicial
   order, or regulation then You must: (a) comply with the terms of this License
   to the maximum extent possible; and (b) describe the limitations and the code
   they affect. Such description must be included in the LEGAL file described in
   Section 3.4 and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Application of this License.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A and to related Covered Code.

   (( 6. ))??
   Versions of the License.

      (( 6.1. ))??
      New Versions. Sun Microsystems, Inc. ("Sun") may publish revised and/or
      new versions of the License from time to time. Each version will be given
      a distinguishing version number.

      (( 6.2. ))??
      Effect of New Versions. Once Covered Code has been published under a
      particular version of the License, You may always continue to use it under
      the terms of that version. You may also choose to use such Covered Code
      under the terms of any subsequent version of the License published by Sun.
      No one other than Sun has the right to modify the terms applicable to
      Covered Code created under this License.

      (( 6.3. ))??
      Derivative Works. If You create or use a modified version of this License
      (which you may only do in order to apply it to code which is not already
      Covered Code governed by this License), You must: (a) rename Your license
      so that the phrases "Sun," "Sun Public License," or "SPL"../ or any
      confusingly similar phrase do not appear in your license (except to note
      that your license differs from this License) and (b) otherwise make it
      clear that Your version of the license contains terms which differ from
      the Sun Public License. (Filling in the name of the Initial Developer,
      Original Code or Contributor in the notice described in Exhibit A shall
      not of themselves be deemed to be modifications of this License.)

   (( 7. ))??
   DISCLAIMER OF WARRANTY.

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "../AS IS'' BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER
   OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR
   OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF
   THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

   (( 8. ))??
   TERMINATION.

      (( 8.1. ))??
      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. All sublicenses to the
      Covered Code which are properly granted shall survive any termination of
      this License. Provisions which, by their nature, must remain in effect
      beyond the termination of this License shall survive.

      (( 8.2. ))??
      If You initiate litigation by asserting a patent infringement claim
      (excluding declaratory judgment actions) against Initial Developer or a
      Contributor (the Initial Developer or Contributor against whom You file
      such action is referred to as "Participant") alleging that:

         (( (a) ))??
         such Participant's Contributor Version directly or indirectly infringes
         any patent, then any and all rights granted by such Participant to You
         under Sections 2.1 and/or 2.2 of this License shall, upon 60 days
         notice from Participant terminate prospectively, unless if within 60
         days after receipt of notice You either: (i) agree in writing to pay
         Participant a mutually agreeable reasonable royalty for Your past and
         future use of Modifications made by such Participant, or (ii) withdraw
         Your litigation claim with respect to the Contributor Version against
         such Participant. If within 60 days of notice, a reasonable royalty and
         payment arrangement are not mutually agreed upon in writing by the
         parties or the litigation claim is not withdrawn, the rights granted by
         Participant to You under Sections 2.1 and/or 2.2 automatically
         terminate at the expiration of the 60 day notice period specified
         above.

         (( (b) ))??
         any software, hardware, or device, other than such Participant's
         Contributor Version, directly or indirectly infringes any patent, then
         any rights granted to You by such Participant under Sections 2.1(b) and
         2.2(b) are revoked effective as of the date You first made, used, sold,
         distributed, or had made, Modifications made by that Participant.

      (( 8.3. ))??
      If You assert a patent infringement claim against Participant alleging
      that such Participant's Contributor Version directly or indirectly
      infringes any patent where such claim is resolved (such as by license or
      settlement) prior to the initiation of patent infringement litigation,
      then the reasonable value of the licenses granted by such Participant
      under Sections 2.1 or 2.2 shall be taken into account in determining the
      amount or value of any payment or license.

      (( 8.4. ))??
      In the event of termination under Sections 8.1 or 8.2 above, all end user
      license agreements (excluding distributors and resellers) which have been
      validly granted by You or any distributor hereunder prior to termination
      shall survive termination.

   (( 9. ))??
   LIMITATION OF LIABILITY.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, THE INITIAL DEVELOPER, ANY
   OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE, OR ANY SUPPLIER OF ANY
   OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. GOVERNMENT END USERS.

   The Covered Code is a "commercial item," as that term is defined in 48 C.F.R.
   2.101 (Oct. 1995), consisting of "commercial computer software" and
   "commercial computer software documentation,"../ as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Covered Code with only those rights set forth herein.

   (( 11. ))??
   MISCELLANEOUS.

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by California law provisions
   (except to the extent applicable law, if any, provides otherwise), excluding
   its conflict-of-law provisions. With respect to disputes in which at least
   one party is a citizen of, or an entity chartered or registered to do
   business in the United States of America, any litigation relating to this
   License shall be subject to the jurisdiction of the Federal Courts of the
   Northern District of California, with venue lying in Santa Clara County,
   California, with the losing party responsible for costs, including without
   limitation, court costs and reasonable attorneys' fees and expenses. The
   application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any law or regulation
   which provides that the language of a contract shall be construed against the
   drafter shall not apply to this License.

   (( 12. ))??
   RESPONSIBILITY FOR CLAIMS.

   As between Initial Developer and the Contributors, each party is responsible
   for claims and damages arising, directly or indirectly, out of its
   utilization of rights under this License and You agree to work with Initial
   Developer and Contributors to distribute such responsibility on an equitable
   basis. Nothing herein is intended or shall be deemed to constitute any
   admission of liability.

   (( 13. ))??
   MULTIPLE-LICENSED CODE.

   Initial Developer may designate portions of the Covered Code as
   ?Multiple-Licensed?. ?Multiple-Licensed? means that the Initial Developer
   permits you to utilize portions of the Covered Code under Your choice of the
   alternative licenses, if any, specified by the Initial Developer in the file
   described in Exhibit A.
   (( Exhibit A -Sun Public License Notice.

The contents of this file are subject to the Sun Public License Version 1.0 (the
License); you may not use this file except in compliance with the License. A
copy of the License is available at http:/www.sun.com/

The Original Code is _____ .
The Initial Developer of the Original Code is _____ .
Portions created by _____ are Copyright (C) _____ . All Rights Reserved.
Contributor(s): _____ .

Alternatively, the contents of this file may be used under the terms of the
__10__ in which case the provisions of
__5__
License are applicable instead of those above. If you wish to allow use of your
version of this file only under the terms of the
__5__
License and not to allow others to use your version of this file under the SPL,
indicate your decision by deleting the provisions above and replace them with
the notice and other provisions required by the
__5__
License. If you do not delete the provisions above, a recipient may use your
version of this file under either the SPL or the
__5__
License.

[NOTE: The text of this Exhibit A may differ slightly from the text of the
notices in the Source Code files of the Original Code. You should use the text
of this Exhibit A rather than the text found in the Original Code Source Code
for Your Modifications.] ))??
`
const license_SSH_OpenSSH_lre = `//**
SSH OpenSSH license
https://spdx.org/licenses/SSH-OpenSSH.json
https://github.com/openssh/openssh-portable/blob/1b11ea7c58cd5c59838b5fa574cd456d6047b2d4/LICENCE#L10
**//

//** Copyright **//

As far as I am concerned, the code I have written for this software can be used
freely for any purpose. Any derived versions of this software must be clearly
marked as such, and if the derived work is incompatible with the protocol
description in the RFC file, it must be called by a name other than "ssh" or
"Secure Shell".

[Tatu continues]

However, I am not implying to give any licenses to any patents or copyrights
held by third parties, and the software includes parts that are not under my
direct control. As far as I know, all included source code is used in accordance
with the relevant license agreements and can be used freely for any purpose (the
GNU license being the most restrictive); see below for details.

[However, none of that term is relevant at this point in time. All of these
restrictively licenced software components which he talks about have been
removed from OpenSSH, i.e.,

   -
   RSA is no longer included, found in the OpenSSL library

   -
   IDEA is no longer included, its use is deprecated

   -
   DES is now external, in the OpenSSL library

   -
   GMP is no longer used, and instead we call BN code from OpenSSL

   -
   Zlib is now external, in a library

   -
   The make-ssh-known-hosts script is no longer included

   -
   TSS has been removed

   -
   MD5 is now external, in the OpenSSL library

   -
   RC4 support has been replaced with ARC4 support from OpenSSL

   -
   Blowfish is now external, in the OpenSSL library

[The licence continues]

Note that any information and cryptographic algorithms used in this software are
publicly available on the Internet and at any major bookstore, scientific
library, and patent office worldwide. More information can be found e.g. at
"http:/www.cs.hut.fi/crypto".

The legal status of this program is some combination of all these permissions
and restrictions. Use only at your own responsibility. You will be responsible
for any legal consequences yourself; I am not making any claims whether
possessing or using this is legal or not in your country, and I am not taking
any responsibility on your behalf.

NO WARRANTY

BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE
PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED
IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS
IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT
NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE
PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY
COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE THE
PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL,
SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY
TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF
THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER
PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
`
const license_SSH_short_lre = `//**
SSH short notice
https://spdx.org/licenses/SSH-short.json
https://github.com/openssh/openssh-portable/blob/1b11ea7c58cd5c59838b5fa574cd456d6047b2d4/pathnames.h
http://web.mit.edu/kolya/.f/root/athena.mit.edu/sipb.mit.edu/project/openssh/OldFiles/src/openssh-2.9.9p2/ssh-add.1
https://joinup.ec.europa.eu/svn/lesoll/trunk/italc/lib/src/dsa_key.cpp
**//

As far as I am concerned, the code I have written for this software can be used
freely for any purpose. Any derived versions of this software must be clearly
marked as such, and if the derived work is incompatible with the protocol
description in the RFC file, it must be called by a name other than "ssh" or
"Secure Shell".
`
const license_SSPL_1_0_lre = `//**
Server Side Public License, v 1
https://spdx.org/licenses/SSPL-1.0.json
https://www.mongodb.com/licensing/server-side-public-license
**//

((
Server Side Public License

VERSION 1, OCTOBER 16, 2018

Copyright © 2018 MongoDB, Inc.

Everyone is permitted to copy and distribute verbatim copies of this license
document, but changing it is not allowed.
))??

TERMS AND CONDITIONS

   (( 0. ))??
   Definitions.

   "This License" refers to Server Side Public License.

   "Copyright" also means copyright-like laws that apply to other kinds of
   works, such as semiconductor masks.

   "The Program" refers to any copyrightable work licensed under this License.
   Each licensee is addressed as "you". "Licensees" and "recipients" may be
   individuals or organizations.

   To "modify" a work means to copy from or adapt all or part of the work in a
   fashion requiring copyright permission, other than the making of an exact
   copy. The resulting work is called a "modified version" of the earlier work
   or a work "based on" the earlier work.

   A "covered work" means either the unmodified Program or a work based on the
   Program.

   To "propagate" a work means to do anything with it that, without permission,
   would make you directly or secondarily liable for infringement under
   applicable copyright law, except executing it on a computer or modifying a
   private copy. Propagation includes copying, distribution (with or without
   modification), making available to the public, and in some countries other
   activities as well.

   To "convey" a work means any kind of propagation that enables other parties
   to make or receive copies. Mere interaction with a user through a computer
   network, with no transfer of a copy, is not conveying.

   An interactive user interface displays "Appropriate Legal Notices" to the
   extent that it includes a convenient and prominently visible feature that (1)
   displays an appropriate copyright notice, and (2) tells the user that there
   is no warranty for the work (except to the extent that warranties are
   provided), that licensees may convey the work under this License, and how to
   view a copy of this License. If the interface presents a list of user
   commands or options, such as a menu, a prominent item in the list meets this
   criterion.

   (( 1. ))??
   Source Code.

   The "source code" for a work means the preferred form of the work for making
   modifications to it. "Object code" means any non-source form of a work.

   A "Standard Interface" means an interface that either is an official standard
   defined by a recognized standards body, or, in the case of interfaces
   specified for a particular programming language, one that is widely used
   among developers working in that language.

   The "System Libraries" of an executable work include anything, other than the
   work as a whole, that (a) is included in the normal form of packaging a Major
   Component, but which is not part of that Major Component, and (b) serves only
   to enable use of the work with that Major Component, or to implement a
   Standard Interface for which an implementation is available to the public in
   source code form. A "Major Component", in this context, means a major
   essential component (kernel, window system, and so on) of the specific
   operating system (if any) on which the executable work runs, or a compiler
   used to produce the work, or an object code interpreter used to run it.

   The "Corresponding Source" for a work in object code form means all the
   source code needed to generate, install, and (for an executable work) run the
   object code and to modify the work, including scripts to control those
   activities. However, it does not include the work's System Libraries, or
   general-purpose tools or generally available free programs which are used
   unmodified in performing those activities but which are not part of the work.
   For example, Corresponding Source includes interface definition files
   associated with source files for the work, and the source code for shared
   libraries and dynamically linked subprograms that the work is specifically
   designed to require, such as by intimate data communication or control flow
   between those subprograms and other parts of the work.

   The Corresponding Source need not include anything that users can regenerate
   automatically from other parts of the Corresponding Source.

   The Corresponding Source for a work in source code form is that same work.

   (( 2. ))??
   Basic Permissions.

   All rights granted under this License are granted for the term of copyright
   on the Program, and are irrevocable provided the stated conditions are met.
   This License explicitly affirms your unlimited permission to run the
   unmodified Program, subject to section 13. The output from running a covered
   work is covered by this License only if the output, given its content,
   constitutes a covered work. This License acknowledges your rights of fair use
   or other equivalent, as provided by copyright law.

   Subject to section 13, you may make, run and propagate covered works that you
   do not convey, without conditions so long as your license otherwise remains
   in force. You may convey covered works to others for the sole purpose of
   having them make modifications exclusively for you, or provide you with
   facilities for running those works, provided that you comply with the terms
   of this License in conveying all material for which you do not control
   copyright. Those thus making or running the covered works for you must do so
   exclusively on your behalf, under your direction and control, on terms that
   prohibit them from making any copies of your copyrighted material outside
   their relationship with you.

   Conveying under any other circumstances is permitted solely under the
   conditions stated below. Sublicensing is not allowed; section 10 makes it
   unnecessary.

   (( 3. ))??
   Protecting Users' Legal Rights From Anti-Circumvention Law.

   No covered work shall be deemed part of an effective technological measure
   under any applicable law fulfilling obligations under article 11 of the WIPO
   copyright treaty adopted on 20 December 1996, or similar laws prohibiting or
   restricting circumvention of such measures.

   When you convey a covered work, you waive any legal power to forbid
   circumvention of technological measures to the extent such circumvention is
   effected by exercising rights under this License with respect to the covered
   work, and you disclaim any intention to limit operation or modification of
   the work as a means of enforcing, against the work's users, your or third
   parties' legal rights to forbid circumvention of technological measures.

   (( 4. ))??
   Conveying Verbatim Copies.

   You may convey verbatim copies of the Program's source code as you receive
   it, in any medium, provided that you conspicuously and appropriately publish
   on each copy an appropriate copyright notice; keep intact all notices stating
   that this License and any non-permissive terms added in accord with section 7
   apply to the code; keep intact all notices of the absence of any warranty;
   and give all recipients a copy of this License along with the Program.

   You may charge any price or no price for each copy that you convey, and you
   may offer support or warranty protection for a fee.

   (( 5. ))??
   Conveying Modified Source Versions.

   You may convey a work based on the Program, or the modifications to produce
   it from the Program, in the form of source code under the terms of section 4,
   provided that you also meet all of these conditions:

      (( a) ))??
      The work must carry prominent notices stating that you modified it, and
      giving a relevant date.

      (( b) ))??
      The work must carry prominent notices stating that it is released under
      this License and any conditions added under section 7. This requirement
      modifies the requirement in section 4 to "keep intact all notices".

      (( c) ))??
      You must license the entire work, as a whole, under this License to anyone
      who comes into possession of a copy. This License will therefore apply,
      along with any applicable section 7 additional terms, to the whole of the
      work, and all its parts, regardless of how they are packaged. This License
      gives no permission to license the work in any other way, but it does not
      invalidate such permission if you have separately received it.

      (( d) ))??
      If the work has interactive user interfaces, each must display Appropriate
      Legal Notices; however, if the Program has interactive interfaces that do
      not display Appropriate Legal Notices, your work need not make them do
      so.

   A compilation of a covered work with other separate and independent works,
   which are not by their nature extensions of the covered work, and which are
   not combined with it such as to form a larger program, in or on a volume of a
   storage or distribution medium, is called an "aggregate" if the compilation
   and its resulting copyright are not used to limit the access or legal rights
   of the compilation's users beyond what the individual works permit. Inclusion
   of a covered work in an aggregate does not cause this License to apply to the
   other parts of the aggregate.

   (( 6. ))??
   Conveying Non-Source Forms.

   You may convey a covered work in object code form under the terms of sections
   4 and 5, provided that you also convey the machine-readable Corresponding
   Source under the terms of this License, in one of these ways:

      (( a) ))??
      Convey the object code in, or embodied in, a physical product (including a
      physical distribution medium), accompanied by the Corresponding Source
      fixed on a durable physical medium customarily used for software
      interchange.

      (( b) ))??
      Convey the object code in, or embodied in, a physical product (including a
      physical distribution medium), accompanied by a written offer, valid for
      at least three years and valid for as long as you offer spare parts or
      customer support for that product model, to give anyone who possesses the
      object code either (1) a copy of the Corresponding Source for all the
      software in the product that is covered by this License, on a durable
      physical medium customarily used for software interchange, for a price no
      more than your reasonable cost of physically performing this conveying of
      source, or (2) access to copy the Corresponding Source from a network
      server at no charge.

      (( c) ))??
      Convey individual copies of the object code with a copy of the written
      offer to provide the Corresponding Source. This alternative is allowed
      only occasionally and noncommercially, and only if you received the object
      code with such an offer, in accord with subsection 6b.

      (( d) ))??
      Convey the object code by offering access from a designated place (gratis
      or for a charge), and offer equivalent access to the Corresponding Source
      in the same way through the same place at no further charge. You need not
      require recipients to copy the Corresponding Source along with the object
      code. If the place to copy the object code is a network server, the
      Corresponding Source may be on a different server (operated by you or a
      third party) that supports equivalent copying facilities, provided you
      maintain clear directions next to the object code saying where to find the
      Corresponding Source. Regardless of what server hosts the Corresponding
      Source, you remain obligated to ensure that it is available for as long as
      needed to satisfy these requirements.

      (( e) ))??
      Convey the object code using peer-to-peer transmission, provided you
      inform other peers where the object code and Corresponding Source of the
      work are being offered to the general public at no charge under subsection
      6d.

   A separable portion of the object code, whose source code is excluded from
   the Corresponding Source as a System Library, need not be included in
   conveying the object code work.

   A "User Product" is either (1) a "consumer product", which means any tangible
   personal property which is normally used for personal, family, or household
   purposes, or (2) anything designed or sold for incorporation into a dwelling.
   In determining whether a product is a consumer product, doubtful cases shall
   be resolved in favor of coverage. For a particular product received by a
   particular user, "normally used" refers to a typical or common use of that
   class of product, regardless of the status of the particular user or of the
   way in which the particular user actually uses, or expects or is expected to
   use, the product. A product is a consumer product regardless of whether the
   product has substantial commercial, industrial or non-consumer uses, unless
   such uses represent the only significant mode of use of the product.

   "Installation Information" for a User Product means any methods, procedures,
   authorization keys, or other information required to install and execute
   modified versions of a covered work in that User Product from a modified
   version of its Corresponding Source. The information must suffice to ensure
   that the continued functioning of the modified object code is in no case
   prevented or interfered with solely because modification has been made.

   If you convey an object code work under this section in, or with, or
   specifically for use in, a User Product, and the conveying occurs as part of
   a transaction in which the right of possession and use of the User Product is
   transferred to the recipient in perpetuity or for a fixed term (regardless of
   how the transaction is characterized), the Corresponding Source conveyed
   under this section must be accompanied by the Installation Information. But
   this requirement does not apply if neither you nor any third party retains
   the ability to install modified object code on the User Product (for example,
   the work has been installed in ROM).

   The requirement to provide Installation Information does not include a
   requirement to continue to provide support service, warranty, or updates for
   a work that has been modified or installed by the recipient, or for the User
   Product in which it has been modified or installed. Access to a network may
   be denied when the modification itself materially and adversely affects the
   operation of the network or violates the rules and protocols for
   communication across the network.

   Corresponding Source conveyed, and Installation Information provided, in
   accord with this section must be in a format that is publicly documented (and
   with an implementation available to the public in source code form), and must
   require no special password or key for unpacking, reading or copying.

   (( 7. ))??
   Additional Terms.

   "Additional permissions" are terms that supplement the terms of this License
   by making exceptions from one or more of its conditions. Additional
   permissions that are applicable to the entire Program shall be treated as
   though they were included in this License, to the extent that they are valid
   under applicable law. If additional permissions apply only to part of the
   Program, that part may be used separately under those permissions, but the
   entire Program remains governed by this License without regard to the
   additional permissions.

   When you convey a copy of a covered work, you may at your option remove any
   additional permissions from that copy, or from any part of it. (Additional
   permissions may be written to require their own removal in certain cases when
   you modify the work.) You may place additional permissions on material, added
   by you to a covered work, for which you have or can give appropriate
   copyright permission.

   Notwithstanding any other provision of this License, for material you add to
   a covered work, you may (if authorized by the copyright holders of that
   material) supplement the terms of this License with terms:

      (( a) ))??
      Disclaiming warranty or limiting liability differently from the terms of
      sections 15 and 16 of this License; or

      (( b) ))??
      Requiring preservation of specified reasonable legal notices or author
      attributions in that material or in the Appropriate Legal Notices
      displayed by works containing it; or

      (( c) ))??
      Prohibiting misrepresentation of the origin of that material, or requiring
      that modified versions of such material be marked in reasonable ways as
      different from the original version; or

      (( d) ))??
      Limiting the use for publicity purposes of names of licensors or authors
      of the material; or

      (( e) ))??
      Declining to grant rights under trademark law for use of some trade names,
      trademarks, or service marks; or

      (( f) ))??
      Requiring indemnification of licensors and authors of that material by
      anyone who conveys the material (or modified versions of it) with
      contractual assumptions of liability to the recipient, for any liability
      that these contractual assumptions directly impose on those licensors and
      authors.

   All other non-permissive additional terms are considered "further
   restrictions" within the meaning of section 10. If the Program as you
   received it, or any part of it, contains a notice stating that it is governed
   by this License along with a term that is a further restriction, you may
   remove that term. If a license document contains a further restriction but
   permits relicensing or conveying under this License, you may add to a covered
   work material governed by the terms of that license document, provided that
   the further restriction does not survive such relicensing or conveying.

   If you add terms to a covered work in accord with this section, you must
   place, in the relevant source files, a statement of the additional terms that
   apply to those files, or a notice indicating where to find the applicable
   terms.

   Additional terms, permissive or non-permissive, may be stated in the form of
   a separately written license, or stated as exceptions; the above requirements
   apply either way.

   (( 8. ))??
   Termination.

   You may not propagate or modify a covered work except as expressly provided
   under this License. Any attempt otherwise to propagate or modify it is void,
   and will automatically terminate your rights under this License (including
   any patent licenses granted under the third paragraph of section 11).

   However, if you cease all violation of this License, then your license from a
   particular copyright holder is reinstated (a) provisionally, unless and until
   the copyright holder explicitly and finally terminates your license, and (b)
   permanently, if the copyright holder fails to notify you of the violation by
   some reasonable means prior to 60 days after the cessation.

   Moreover, your license from a particular copyright holder is reinstated
   permanently if the copyright holder notifies you of the violation by some
   reasonable means, this is the first time you have received notice of
   violation of this License (for any work) from that copyright holder, and you
   cure the violation prior to 30 days after your receipt of the notice.

   Termination of your rights under this section does not terminate the licenses
   of parties who have received copies or rights from you under this License. If
   your rights have been terminated and not permanently reinstated, you do not
   qualify to receive new licenses for the same material under section 10.

   (( 9. ))??
   Acceptance Not Required for Having Copies.

   You are not required to accept this License in order to receive or run a copy
   of the Program. Ancillary propagation of a covered work occurring solely as a
   consequence of using peer-to-peer transmission to receive a copy likewise
   does not require acceptance. However, nothing other than this License grants
   you permission to propagate or modify any covered work. These actions
   infringe copyright if you do not accept this License. Therefore, by modifying
   or propagating a covered work, you indicate your acceptance of this License
   to do so.

   (( 10. ))??
   Automatic Licensing of Downstream Recipients.

   Each time you convey a covered work, the recipient automatically receives a
   license from the original licensors, to run, modify and propagate that work,
   subject to this License. You are not responsible for enforcing compliance by
   third parties with this License.

   An "entity transaction" is a transaction transferring control of an
   organization, or substantially all assets of one, or subdividing an
   organization, or merging organizations. If propagation of a covered work
   results from an entity transaction, each party to that transaction who
   receives a copy of the work also receives whatever licenses to the work the
   party's predecessor in interest had or could give under the previous
   paragraph, plus a right to possession of the Corresponding Source of the work
   from the predecessor in interest, if the predecessor has it or can get it
   with reasonable efforts.

   You may not impose any further restrictions on the exercise of the rights
   granted or affirmed under this License. For example, you may not impose a
   license fee, royalty, or other charge for exercise of rights granted under
   this License, and you may not initiate litigation (including a cross-claim or
   counterclaim in a lawsuit) alleging that any patent claim is infringed by
   making, using, selling, offering for sale, or importing the Program or any
   portion of it.

   (( 11. ))??
   Patents.

   A "contributor" is a copyright holder who authorizes use under this License
   of the Program or a work on which the Program is based. The work thus
   licensed is called the contributor's "contributor version".

   A contributor's "essential patent claims" are all patent claims owned or
   controlled by the contributor, whether already acquired or hereafter
   acquired, that would be infringed by some manner, permitted by this License,
   of making, using, or selling its contributor version, but do not include
   claims that would be infringed only as a consequence of further modification
   of the contributor version. For purposes of this definition, "control"
   includes the right to grant patent sublicenses in a manner consistent with
   the requirements of this License.

   Each contributor grants you a non-exclusive, worldwide, royalty-free patent
   license under the contributor's essential patent claims, to make, use, sell,
   offer for sale, import and otherwise run, modify and propagate the contents
   of its contributor version.

   In the following three paragraphs, a "patent license" is any express
   agreement or commitment, however denominated, not to enforce a patent (such
   as an express permission to practice a patent or covenant not to sue for
   patent infringement). To "grant" such a patent license to a party means to
   make such an agreement or commitment not to enforce a patent against the
   party.

   If you convey a covered work, knowingly relying on a patent license, and the
   Corresponding Source of the work is not available for anyone to copy, free of
   charge and under the terms of this License, through a publicly available
   network server or other readily accessible means, then you must either (1)
   cause the Corresponding Source to be so available, or (2) arrange to deprive
   yourself of the benefit of the patent license for this particular work, or
   (3) arrange, in a manner consistent with the requirements of this License, to
   extend the patent license to downstream recipients. "Knowingly relying" means
   you have actual knowledge that, but for the patent license, your conveying
   the covered work in a country, or your recipient's use of the covered work in
   a country, would infringe one or more identifiable patents in that country
   that you have reason to believe are valid.

   If, pursuant to or in connection with a single transaction or arrangement,
   you convey, or propagate by procuring conveyance of, a covered work, and
   grant a patent license to some of the parties receiving the covered work
   authorizing them to use, propagate, modify or convey a specific copy of the
   covered work, then the patent license you grant is automatically extended to
   all recipients of the covered work and works based on it.

   A patent license is "discriminatory" if it does not include within the scope
   of its coverage, prohibits the exercise of, or is conditioned on the
   non-exercise of one or more of the rights that are specifically granted under
   this License. You may not convey a covered work if you are a party to an
   arrangement with a third party that is in the business of distributing
   software, under which you make payment to the third party based on the extent
   of your activity of conveying the work, and under which the third party
   grants, to any of the parties who would receive the covered work from you, a
   discriminatory patent license (a) in connection with copies of the covered
   work conveyed by you (or copies made from those copies), or (b) primarily for
   and in connection with specific products or compilations that contain the
   covered work, unless you entered into that arrangement, or that patent
   license was granted, prior to 28 March 2007.

   Nothing in this License shall be construed as excluding or limiting any
   implied license or other defenses to infringement that may otherwise be
   available to you under applicable patent law.

   (( 12. ))??
   No Surrender of Others' Freedom.

   If conditions are imposed on you (whether by court order, agreement or
   otherwise) that contradict the conditions of this License, they do not excuse
   you from the conditions of this License. If you cannot use, propagate or
   convey a covered work so as to satisfy simultaneously your obligations under
   this License and any other pertinent obligations, then as a consequence you
   may not use, propagate or convey it at all. For example, if you agree to
   terms that obligate you to collect a royalty for further conveying from those
   to whom you convey the Program, the only way you could satisfy both those
   terms and this License would be to refrain entirely from conveying the
   Program.

   (( 13. ))??
   Offering the Program as a Service.

   If you make the functionality of the Program or a modified version available
   to third parties as a service, you must make the Service Source Code
   available via network download to everyone at no charge, under the terms of
   this License. Making the functionality of the Program or modified version
   available to third parties as a service includes, without limitation,
   enabling third parties to interact with the functionality of the Program or
   modified version remotely through a computer network, offering a service the
   value of which entirely or primarily derives from the value of the Program or
   modified version, or offering a service that accomplishes for users the
   primary purpose of the Program or modified version.

   "Service Source Code" means the Corresponding Source for the Program or the
   modified version, and the Corresponding Source for all programs that you use
   to make the Program or modified version available as a service, including,
   without limitation, management software, user interfaces, application program
   interfaces, automation software, monitoring software, backup software,
   storage software and hosting software, all such that a user could run an
   instance of the service using the Service Source Code you make available.

   (( 14. ))??
   Revised Versions of this License.

   MongoDB, Inc. may publish revised and/or new versions of the Server Side
   Public License from time to time. Such new versions will be similar in spirit
   to the present version, but may differ in detail to address new problems or
   concerns.

   Each version is given a distinguishing version number. If the Program
   specifies that a certain numbered version of the Server Side Public License
   "or any later version" applies to it, you have the option of following the
   terms and conditions either of that numbered version or of any later version
   published by MongoDB, Inc. If the Program does not specify a version number
   of the Server Side Public License, you may choose any version ever published
   by MongoDB, Inc.

   If the Program specifies that a proxy can decide which future versions of the
   Server Side Public License can be used, that proxy's public statement of
   acceptance of a version permanently authorizes you to choose that version for
   the Program.

   Later license versions may give you additional or different permissions.
   However, no additional obligations are imposed on any author or copyright
   holder as a result of your choosing to follow a later version.

   (( 15. ))??
   Disclaimer of Warranty.

   THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE
   LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR
   OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND,
   EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
   ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
   SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY
   SERVICING, REPAIR OR CORRECTION.

   (( 16. ))??
   Limitation of Liability.

   IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
   ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS THE
   PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
   GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE
   OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR
   DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR
   A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH
   HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

   (( 17. ))??
   Interpretation of Sections 15 and 16.

   If the disclaimer of warranty and limitation of liability provided above
   cannot be given local legal effect according to their terms, reviewing courts
   shall apply local law that most closely approximates an absolute waiver of
   all civil liability in connection with the Program, unless a warranty or
   assumption of liability accompanies a copy of the Program in return for a
   fee.

((END OF TERMS AND CONDITIONS))??
`
const license_SWL_lre = `//**
Scheme Widget Library (SWL) Software License Agreement
https://spdx.org/licenses/SWL.json
https://fedoraproject.org/wiki/Licensing/SWL
**//

The authors hereby grant permission to use, copy, modify, distribute, and
license this software and its documentation for any purpose, provided that
existing copyright notices are retained in all copies and that this notice is
included verbatim in any distributions. No written agreement, license, or
royalty fee is required for any of the authorized uses. Modifications to this
software may be copyrighted by their authors and need not follow the licensing
terms described here, provided that the new terms are clearly indicated on the
first page of each file where they apply.

IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY FOR DIRECT,
INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE
OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY DERIVATIVES THEREOF, EVEN IF THE
AUTHORS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. THE AUTHORS AND
DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
AND NON-INFRINGEMENT. THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, AND THE
AUTHORS AND DISTRIBUTORS HAVE NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

GOVERNMENT USE: If you are acquiring this software on behalf of the U.S.
government, the Government shall have only "Restricted Rights" in the software
and related documentation as defined in the Federal Acquisition Regulations
(FARs) in Clause 52.227.19 (c) (2). If you are acquiring the software on behalf
of the Department of Defense, the software shall be classified as "Commercial
Computer Software" and the Government shall have only "Restricted Rights" as
defined in Clause 252.227-7013 (c) (1) of DFARs. Notwithstanding the foregoing,
the authors grant the U.S. Government and others acting in its behalf permission
to use and distribute the software in accordance with the terms specified in
this license.

BY INSTALLING THIS SOFTWARE, YOU ACKNOWLEDGE THAT YOU HAVE READ THIS AGREEMENT,
THAT YOU UNDERSTAND IT, AND THAT YOU AGREE TO BE BOUND BY ITS TERMS AND
CONDITIONS.
`
const license_Saxpath_lre = `//**
Saxpath License
https://spdx.org/licenses/Saxpath.json
https://fedoraproject.org/wiki/Licensing/Saxpath_License
**//

//** Copyright **//

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions, and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions, and the disclaimer that follows these conditions in
   the documentation and/or other materials provided with the distribution.

   (( 3. ))??
   The name "SAXPath" must not be used to endorse or promote products derived
   from this software without prior written permission. For written permission,
   please contact license@saxpath.org.

   (( 4. ))??
   Products derived from this software may not be called "SAXPath", nor may
   "SAXPath" appear in their name, without prior written permission from the
   SAXPath Project Management (pm@saxpath.org).

In addition, we request (but do not require) that you include in the end-user
documentation provided with the redistribution and/or in the software itself an
acknowledgement equivalent to the following:

"This product includes software developed by the SAXPath Project
(http:/www.saxpath.org/)."

Alternatively, the acknowledgment may be graphical using the logos available at
http:/www.saxpath.org/

THIS SOFTWARE IS PROVIDED ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE SAXPath
AUTHORS OR THE PROJECT CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
`
const license_Sendmail_lre = `//**
Sendmail License
https://spdx.org/licenses/Sendmail.json
http://www.sendmail.com/pdfs/open_source/sendmail_license.pdf
https://web.archive.org/web/20160322142305/https://www.sendmail.com/pdfs/open_source/sendmail_license.pdf
**//

(( SENDMAIL LICENSE ))??

(( The following license terms and conditions apply, unless a redistribution
agreement or other license is obtained from Sendmail, Inc., 6475 Christie Ave,
Third Floor, Emeryville, CA 94608, USA, or by electronic mail at
license@sendmail.com.

License Terms: ))??

Use, Modification and Redistribution (including distribution of any modified or
derived work) in source and binary forms is permitted only if each of the
following conditions is met:

   (( 1. ))??
   Redistributions qualify as "freeware" or "Open Source Software" under one of
   the following terms:

   (( (a) ))??
   Redistributions are made at no charge beyond the reasonable cost of materials
   and delivery.

   (( (b) ))??
   Redistributions are accompanied by a copy of the Source Code or by an
   irrevocable offer to provide a copy of the Source Code for up to three years
   at the cost of materials and delivery. Such redistributions must allow
   further use, modification, and redistribution of the Source Code under
   substantially the same terms as this license. For the purposes of
   redistribution "Source Code" means the complete compilable and linkable
   source code of sendmail including all modifications.

   (( 2. ))??
   Redistributions of Source Code must retain the copyright notices as they
   appear in each Source Code file, these license terms, and the
   disclaimer/limitation of liability set forth as paragraph 6 below.

   (( 3. ))??
   Redistributions in binary form must reproduce the Copyright Notice, these
   license terms, and the disclaimer/limitation of liability set forth as
   paragraph 6 below, in the documentation and/or other materials provided with
   the distribution. For the purposes of binary distribution the "Copyright
   Notice" refers to the following language:

   "Copyright (c) 1998-2010 Sendmail, Inc. All rights reserved."

   (( 4. ))??
   Neither the name of Sendmail, Inc. nor the University of California nor names
   of their contributors may be used to endorse or promote products derived from
   this software without specific prior written permission. The name "sendmail"
   is a trademark of Sendmail, Inc.

   (( 5. ))??
   All redistributions must comply with the conditions imposed by the University
   of California on certain embedded code, which copyright Notice and conditions
   for redistribution are as follows:

   (( (a) ))??
   Copyright (c) 1988, 1993 The Regents of the University of California. All
   rights reserved.

   (( (b) ))??
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   (( (i) ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   (( (ii) ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( (iii) ))??
   Neither the name of the University nor the names of its contributors may be
   used to endorse or promote products derived from this software without
   specific prior written permission.

   (( 6. ))??
   Disclaimer/Limitation of Liability: THIS SOFTWARE IS PROVIDED BY SENDMAIL,
   INC. AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
   INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SENDMAIL,
   INC., THE REGENTS OF THE UNIVERSITY OF CALIFORNIA OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
   (( $Revision: 8.16 $, Last updated $Date: 2010/10/25 23:11:19 $, Document
   139848.1 ))??
`
const license_Sendmail_8_23_lre = `//**
Sendmail License 8.23
https://spdx.org/licenses/Sendmail-8.23.json
https://www.proofpoint.com/sites/default/files/sendmail-license.pdf
https://web.archive.org/web/20181003101040/https://www.proofpoint.com/sites/default/files/sendmail-license.pdf
**//

(( SENDMAIL LICENSE ))??

(( The following license terms and conditions apply, unless a redistribution
agreement or other license is obtained from Proofpoint, Inc., 892 Ross Street,
Sunnyvale, CA, 94089, USA, or by electronic mail at
sendmail-license@proofpoint.com.

License Terms: ))??

Use, Modification and Redistribution (including distribution of any modified or
derived work) in source and binary forms is permitted only if each of the
following conditions is met:

   (( 1. ))??
   Redistributions qualify as "freeware" or "Open Source Software" under one of
   the following terms:

   (( (a) ))??
   Redistributions are made at no charge beyond the reasonable cost of materials
   and delivery.

   (( (b) ))??
   Redistributions are accompanied by a copy of the Source Code or by an
   irrevocable offer to provide a copy of the Source Code for up to three years
   at the cost of materials and delivery. Such redistributions must allow
   further use, modification, and redistribution of the Source Code under
   substantially the same terms as this license. For the purposes of
   redistribution "Source Code" means the complete compilable and linkable
   source code of sendmail and associated libraries and utilities in the
   sendmail distribution including all modifications.

   (( 2. ))??
   Redistributions of Source Code must retain the copyright notices as they
   appear in each Source Code file, these license terms, and the
   disclaimer/limitation of liability set forth as paragraph 6 below.

   (( 3. ))??
   Redistributions in binary form must reproduce the Copyright Notice, these
   license terms, and the disclaimer/limitation of liability set forth as
   paragraph 6 below, in the documentation and/or other materials provided with
   the distribution. For the purposes of binary distribution the "Copyright
   Notice" refers to the following language:

   "Copyright (c) 1998-2014 Proofpoint, Inc. All rights reserved."

   (( 4. ))??
   Neither the name of Proofpoint, Inc. nor the University of California nor
   names of their contributors may be used to endorse or promote products
   derived from this software without specific prior written permission. The
   name "sendmail" is a trademark of Proofpoint, Inc.

   (( 5. ))??
   All redistributions must comply with the conditions imposed by the University
   of California on certain embedded code, which copyright Notice and conditions
   for redistribution are as follows:

   (( (a) ))??
   Copyright (c) 1988, 1993 The Regents of the University of California. All
   rights reserved.

   (( (b) ))??
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   (( (i) ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   (( (ii) ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( (iii) ))??
   Neither the name of the University nor the names of its contributors may be
   used to endorse or promote products derived from this software without
   specific prior written permission.

   (( 6. ))??
   Disclaimer/Limitation of Liability: THIS SOFTWARE IS PROVIDED BY SENDMAIL,
   INC. AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
   INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SENDMAIL,
   INC., THE REGENTS OF THE UNIVERSITY OF CALIFORNIA OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
   (( $Revision: 8.23 $, Last updated $Date: 2014-01-26 20:10:01 $, Document
   139848.1 ))??
`
const license_SimPL_2_0_lre = `//**
Simple Public License 2.0
https://spdx.org/licenses/SimPL-2.0.json
https://opensource.org/licenses/SimPL-2.0
**//

(( Simple Public License (SimPL) ))??

(( Preamble

This Simple Public License 2.0 (SimPL 2.0 for short) is a plain language
implementation of GPL 2.0. The words are different, but the goal is the same -
to guarantee for all users the freedom to share and change software. If anyone
wonders about the meaning of the SimPL, they should interpret it as consistent
with GPL 2.0. ))??

(( Simple Public License (SimPL) 2.0 ))??

The SimPL applies to the software's source and object code and comes with any
rights that I have in it (other than trademarks). You agree to the SimPL by
copying, distributing, or making a derivative work of the software.

You get the royalty free right to:

   -
   Use the software for any purpose;

   -
   Make derivative works of it (this is called a "Derived Work");

   -
   Copy and distribute it and any Derived Work.

If you distribute the software or a Derived Work, you must give back to the
community by:

   -
   Prominently noting the date of any changes you make;

   -
   Leaving other people's copyright notices, warranty disclaimers, and license
   terms in place;

   -
   Providing the source code, build scripts, installation scripts, and interface
   definitions in a form that is easy to get and best to modify;

   -
   Licensing it to everyone under SimPL, or substantially similar terms (such as
   GPL 2.0), without adding further restrictions to the rights provided;

   -
   Conspicuously announcing that it is available under that license.

There are some things that you must shoulder:

   -
   You get NO WARRANTIES. None of any kind;

   -
   If the software damages you in any way, you may only recover direct damages
   up to the amount you paid for it (that is zero if you did not pay anything).
   You may not recover any other damages, including those called "consequential
   damages." (The state or country where you live may not allow you to limit
   your liability in this way, so this may not apply to you);

The SimPL continues perpetually, except that your license rights end
automatically if:

   -
   You do not abide by the "give back to the community" terms (your licensees
   get to keep their rights if they abide);

   -
   Anyone prevents you from distributing the software under the terms of the
   SimPL.

License for the License

You may do anything that you want with the SimPL text; it's a license form to
use in any way that you find helpful. To avoid confusion, however, if you change
the terms in any way then you may not call your license the Simple Public
License or the SimPL (but feel free to acknowledge that your license is "based
on the Simple Public License").
`
const license_Sleepycat_lre = `//**
Sleepycat License
https://spdx.org/licenses/Sleepycat.json
https://opensource.org/licenses/Sleepycat
**//

(( The Sleepycat License
(( Copyright __20__ ))??
))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   -
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   -
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   -
   Redistributions in any form must be accompanied by information on how to
   obtain complete source code for the DB software and any accompanying software
   that uses the DB software. The source code must either be included in the
   distribution or be available for no more than the cost of distribution plus a
   nominal fee, and must be freely redistributable under reasonable conditions.
   For an executable file, complete source code means the source code for all
   modules it contains. It does not include source code for modules or files
   that typically accompany the major components of the operating system on
   which the executable file runs.

THIS SOFTWARE IS PROVIDED BY SLEEPYCAT SOFTWARE ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT, ARE
DISCLAIMED. IN NO EVENT SHALL SLEEPYCAT SOFTWARE BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Copyright (c) 1990, 1993, 1994, 1995 The Regents of the University of California. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   -
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   -
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   -
   Neither the name of the University nor the names of its contributors may be
   used to endorse or promote products derived from this software without
   specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ` + "`" + `` + "`" + `AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Copyright (c) 1995, 1996 The President and Fellows of Harvard University. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   -
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   -
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   -
   Neither the name of the University nor the names of its contributors may be
   used to endorse or promote products derived from this software without
   specific prior written permission.

THIS SOFTWARE IS PROVIDED BY HARVARD AND ITS CONTRIBUTORS ` + "`" + `` + "`" + `AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL HARVARD OR ITS CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
`
const license_Spencer_86_lre = `//**
Spencer License 86
https://spdx.org/licenses/Spencer-86.json
https://fedoraproject.org/wiki/Licensing/Henry_Spencer_Reg-Ex_Library_License
**//

//** Copyright **//

Permission is granted to anyone to use this software for any purpose on any
computer system, and to redistribute it freely, subject to the following
restrictions:

   (( 1. ))??
   The author is not responsible for the consequences of use of this software,
   no matter how awful, even if they arise from defects in it.

   (( 2. ))??
   The origin of this software must not be misrepresented, either by explicit
   claim or by omission.

   (( 3. ))??
   Altered versions must be plainly marked as such, and must not be
   misrepresented as being the original software.
`
const license_Spencer_94_lre = `//**
Spencer License 94
https://spdx.org/licenses/Spencer-94.json
https://fedoraproject.org/wiki/Licensing/Henry_Spencer_Reg-Ex_Library_License
**//

(( Copyright 1992, 1993, 1994 Henry Spencer. All rights reserved. ))??

This software is not subject to any license of the American Telephone and
Telegraph Company or of the Regents of the University of California.

Permission is granted to anyone to use this software for any purpose on any
computer system, and to alter it and redistribute it, subject to the following
restrictions:

   (( 1. ))??
   The author is not responsible for the consequences of use of this software,
   no matter how awful, even if they arise from flaws in it.

   (( 2. ))??
   The origin of this software must not be misrepresented, either by explicit
   claim or by omission. Since few users ever read sources, credits must appear
   in the documentation.

   (( 3. ))??
   Altered versions must be plainly marked as such, and must not be
   misrepresented as being the original software. Since few users ever read
   sources, credits must appear in the documentation.

   (( 4. ))??
   This notice may not be removed or altered.
`
const license_Spencer_99_lre = `//**
Spencer License 99
https://spdx.org/licenses/Spencer-99.json
http://www.opensource.apple.com/source/tcl/tcl-5/tcl/generic/regfronts.c
**//

//** Copyright **//

Development of this software was funded, in part, by Cray Research Inc., UUNET
Communications Services Inc., Sun Microsystems Inc., and Scriptics Corporation,
none of whom are responsible for the results. The author thanks all of them.

Redistribution and use in source and binary forms - with or without modification
- are permitted for any purpose, provided that redistributions in source form
retain this entire copyright notice and indicate the origin and nature of any
modifications.

I'd appreciate being given credit for this package in the documentation of
software which uses it, but that is not a requirement.

THIS SOFTWARE IS PROVIDED ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL HENRY SPENCER
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
`
const license_SugarCRM_1_1_3_lre = `//**
SugarCRM Public License v1.1.3
https://spdx.org/licenses/SugarCRM-1.1.3.json
http://www.sugarcrm.com/crm/SPL
**//

(( SUGARCRM PUBLIC LICENSE ))??

(( Applies to Sugar Open Source Edition v1 through v4. Please note that these
releases are no longer supported or distributed.

Version 1.1.3

The SugarCRM Public License Version ("SPL") consists of the Mozilla Public
License Version 1.1, modified to be specific to SugarCRM, with the Additional
Terms in Exhibit B. The original Mozilla Public License 1.1 can be found at:
http:/www.mozilla.org/MPL/MPL-1.1.html ))??

   (( 1. ))??
   Definitions.

      (( 1.1. ))??
      "Contributor" means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      "Covered Code" means the Original Code or Modifications or the combination
      of the Original Code and Modifications, in each case including portions
      thereof.

      (( 1.4. ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      "Executable" means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      "Initial Developer" means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7. ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.8. ))??
      "License" means this document.

      (( 1.8.1. ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.9. ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

         (( B. ))??
         Any new file that contains any part of the Original Code or previous
         Modifications.

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

      (( 1.10. ))??
      "Original Code" means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      1.10.1. "Patent Claims" means any patent claim(s), now owned or hereafter
      acquired, including without limitation, method, process, and apparatus
      claims, in any patent Licensable by grantor.

      (( 1.11. ))??
      "Source Code" means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or source code differential comparisons
      against either the Original Code or another well known, available Covered
      Code of the Contributor's choice. The Source Code can be in a compressed
      or archival form, provided the appropriate decompression or de-archiving
      software is widely available for no charge.

      (( 1.12. ))??
      "You" (or "Your") means an individual or a legal entity exercising rights
      under, and complying with all of the terms of, this License or a future
      version of this License issued under Section 6.1. For legal entities,
      "You" includes any entity which controls, is controlled by, or is under
      common control with You. For purposes of this definition, "control" means
      (a) the power, direct or indirect, to cause the direction or management of
      such entity, whether by contract or otherwise, or (b) ownership of more
      than fifty percent (50%) of the outstanding shares or beneficial ownership
      of such entity.

   (( 2. ))??
   Source Code License.

      (( 2.2. ))??
      Contributor Grant.

      Subject to third party intellectual property claims, each Contributor
      hereby grants You a world-wide, royalty-free, non-exclusive license

         (( (b) ))??
         under Patent Claims infringed by the making, using, or selling of
         Modifications made by that Contributor either alone and/or in
         combination with its Contributor Version (or portions of such
         combination), to make, use, sell, offer for sale, have made, and/or
         otherwise dispose of: 1) Modifications made by that Contributor (or
         portions thereof); and 2) the combination of Modifications made by that
         Contributor with its Contributor Version (or portions of such
         combination).

         (( c ))??
         the licenses granted in Sections 2.2(a) and 2.2(b) are effective on the
         date Contributor first makes Commercial Use of the Covered Code.

         (( (d) ))??
         Notwithstanding Section 2.2(b) above, no patent license is granted: 1)
         for any code that Contributor has deleted from the Contributor Version;
         2) separate from the Contributor Version; 3) for infringements caused
         by: i) third party modifications of Contributor Version or ii) the
         combination of Modifications made by that Contributor with other
         software (except as part of the Contributor Version) or other devices;
         or 4) under Patent Claims infringed by Covered Code in the absence of
         Modifications made by that Contributor.

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Contributor, to use, reproduce, modify, display, perform,
         sublicense and distribute the Modifications created by such Contributor
         (or portions thereof) either on an unmodified basis, with other
         Modifications, as Covered Code and/or as part of a Larger Work; and

   (( 3. ))??
   Distribution Obligations.

      (( 3.2. ))??
      Availability of Source Code. Any Modification which You create or to which
      You contribute must be made available in Source Code form under the terms
      of this License either on the same media as an Executable version or via
      an accepted Electronic Distribution Mechanism to anyone to whom you made
      an Executable version available; and if made available via Electronic
      Distribution Mechanism, must remain available for at least twelve (12)
      months after the date it initially became available, or at least six (6)
      months after a subsequent version of that particular Modification has been
      made available to such recipients. You are responsible for ensuring that
      the Source Code version remains available even if the Electronic
      Distribution Mechanism is maintained by a third party.

      (( 3.3. ))??
      Description of Modifications. You must cause all Covered Code to which You
      contribute to contain a file documenting the changes You made to create
      that Covered Code and the date of any change. You must include a prominent
      statement that the Modification is derived, directly or indirectly, from
      Original Code provided by the Initial Developer and including the name of
      the Initial Developer in (a) the Source Code, and (b) in any notice in an
      Executable version or related documentation in which You describe the
      origin or ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters

         (( (b) ))??
         Contributor APIs. If Contributor's Modifications include an application
         programming interface and Contributor has knowledge of patent licenses
         which are reasonably necessary to implement that API, Contributor must
         also include this information in the LEGAL file.

         (( (a) ))??
         Third Party Claims. If Contributor has knowledge that a license under a
         third party's intellectual property rights is required to exercise the
         rights granted by such Contributor under Sections 2.1 or 2.2,
         Contributor must include a text file with the Source Code distribution
         titled "LEGAL" which describes the claim and the party making the claim
         in sufficient detail that a recipient will know whom to contact. If
         Contributor obtains such knowledge after the Modification is made
         available as described in Section 3.2, Contributor shall promptly
         modify the LEGAL file in all copies Contributor makes available
         thereafter and shall take other steps (such as notifying appropriate
         mailing lists or newsgroups) reasonably calculated to inform those who
         received the Covered Code that new knowledge has been obtained.

         (( c ))??
         Representations. Contributor represents that, except as disclosed
         pursuant to Section 3.4(a) above, Contributor believes that
         Contributor's Modifications are Contributor's original creation(s)
         and/or Contributor has sufficient rights to grant the rights conveyed
         by this License.

      (( 3.5. ))??
      Required Notices. You must duplicate the notice in Exhibit A in each file
      of the Source Code. If it is not possible to put such notice in a
      particular Source Code file due to its structure, then You must include
      such notice in a location (such as a relevant directory) where a user
      would be likely to look for such a notice. If You created one or more
      Modification(s) You may add your name as a Contributor to the notice
      described in Exhibit A. You must also duplicate this License in any
      documentation for the Source Code where You describe recipients' rights or
      ownership rights relating to Covered Code. You may choose to offer, and to
      charge a fee for, warranty, support, indemnity or liability obligations to
      one or more recipients of Covered Code. However, You may do so only on
      Your own behalf, and not on behalf of the Initial Developer or any
      Contributor. You must make it absolutely clear than any such warranty,
      support, indemnity or liability obligation is offered by You alone, and
      You hereby agree to indemnify the Initial Developer and every Contributor
      for any liability incurred by the Initial Developer or such Contributor as
      a result of warranty, support, indemnity or liability terms You offer.

      (( 3.6. ))??
      Distribution of Executable Versions. You may distribute Covered Code in
      Executable form only if the requirements of Section 3.1-3.5 have been met
      for that Covered Code, and if You include a notice stating that the Source
      Code version of the Covered Code is available under the terms of this
      License, including a description of how and where You have fulfilled the
      obligations of Section 3.2. The notice must be conspicuously included in
      any notice in an Executable version, related documentation or collateral
      in which You describe recipients' rights relating to the Covered Code. You
      may distribute the Executable version of Covered Code or ownership rights
      under a license of Your choice, which may contain terms different from
      this License, provided that You are in compliance with the terms of this
      License and that the license for the Executable version does not attempt
      to limit or alter the recipient's rights in the Source Code version from
      the rights set forth in this License. If You distribute the Executable
      version under a different license You must make it absolutely clear that
      any terms which differ from this License are offered by You alone, not by
      the Initial Developer or any Contributor. You hereby agree to indemnify
      the Initial Developer and every Contributor for any liability incurred by
      the Initial Developer or such Contributor as a result of any such terms
      You offer.

      (( 3.7. ))??
      Larger Works. You may create a Larger Work by combining Covered Code with
      other code not governed by the terms of this License and distribute the
      Larger Work as a single product. In such a case, You must make sure the
      requirements of this License are fulfilled for the Covered Code.

   (( 4. ))??
   Inability to Comply Due to Statute or Regulation.

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute, judicial
   order, or regulation then You must: (a) comply with the terms of this License
   to the maximum extent possible; and (b) describe the limitations and the code
   they affect. Such description must be included in the LEGAL file described in
   Section 3.4 and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5. ))??
   Application of this License.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A and to related Covered Code.

   (( 6. ))??
   Versions of the License.

      (( 6.2. ))??
      Effect of New Versions. Once Covered Code has been published under a
      particular version of the License, You may always continue to use it under
      the terms of that version. You may also choose to use such Covered Code
      under the terms of any subsequent version of the License published by
      SugarCRM. No one other than SugarCRM has the right to modify the terms
      applicable to Covered Code created under this License.

      (( 6.3. ))??
      Derivative Works. If You create or use a modified version of this License
      (which you may only do in order to apply it to code which is not already
      Covered Code governed by this License), You must (a) rename Your license
      so that the phrases "SugarCRM", "SPL" or any confusingly similar phrase do
      not appear in your license (except to note that your license differs from
      this License) and (b) otherwise make it clear that Your version of the
      license contains terms which differ from the SugarCRM Public License.
      (Filling in the name of the Initial Developer, Original Code or
      Contributor in the notice described in Exhibit A shall not of themselves
      be deemed to be modifications of this License.)

   (( 7. ))??
   DISCLAIMER OF WARRANTY.

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT
   LIMITATION, WARRANTIES THAT THE COVERED CODE IS FREE OF DEFECTS,
   MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE OR NON-INFRINGING. THE ENTIRE RISK
   AS TO THE QUALITY AND PERFORMANCE OF THE COVERED CODE IS WITH YOU. SHOULD ANY
   COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT THE INITIAL DEVELOPER
   OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY SERVICING, REPAIR
   OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF
   THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
   THIS DISCLAIMER.

   (( 8. ))??
   TERMINATION.

      (( 8.2. ))??
      If You initiate litigation by asserting a patent infringement claim
      (excluding declatory judgment actions) against Initial Developer or a
      Contributor (the Initial Developer or Contributor against whom You file
      such action is referred to as "Participant") alleging that:

         (( (a) ))??
         such Participant's Contributor Version directly or indirectly infringes
         any patent, then any and all rights granted by such Participant to You
         under Sections 2.1 and/or 2.2 of this License shall, upon 60 days
         notice from Participant terminate prospectively, unless if within 60
         days after receipt of notice You either: (i) agree in writing to pay
         Participant a mutually agreeable reasonable royalty for Your past and
         future use of Modifications made by such Participant, or (ii) withdraw
         Your litigation claim with respect to the Contributor Version against
         such Participant. If within 60 days of notice, a reasonable royalty and
         payment arrangement are not mutually agreed upon in writing by the
         parties or the litigation claim is not withdrawn, the rights granted by
         Participant to You under Sections 2.1 and/or 2.2 automatically
         terminate at the expiration of the 60 day notice period specified
         above.

         (( (b) ))??
         any software, hardware, or device, other than such Participant's
         Contributor Version, directly or indirectly infringes any patent, then
         any rights granted to You by such Participant under Sections 2.1(b) and
         2.2(b) are revoked effective as of the date You first made, used, sold,
         distributed, or had made, Modifications made by that Participant.

      (( 8.3. ))??
      If You assert a patent infringement claim against Participant alleging
      that such Participant's Contributor Version directly or indirectly
      infringes any patent where such claim is resolved (such as by license or
      settlement) prior to the initiation of patent infringement litigation,
      then the reasonable value of the licenses granted by such Participant
      under Sections 2.1 or 2.2 shall be taken into account in determining the
      amount or value of any payment or license.

      (( 8.4. ))??
      In the event of termination under Sections 8.1 or 8.2 above, all end user
      license agreements (excluding distributors and resellers) which have been
      validly granted by You or any distributor hereunder prior to termination
      shall survive termination.

   (( 9. ))??
   LIMITATION OF LIABILITY.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, THE INITIAL DEVELOPER, ANY
   OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE, OR ANY SUPPLIER OF ANY
   OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10. ))??
   U.S. GOVERNMENT END USERS.

   The Covered Code is a "commercial item," as that term is defined in 48 C.F.R.
   2.101 (Oct. 1995), consisting of "commercial computer software" and
   "commercial computer software documentation," as such terms are used in 48
   C.F.R. 12.212 (Sept. 1995). Consistent with 48 C.F.R. 12.212 and 48 C.F.R.
   227.7202-1 through 227.7202-4 (June 1995), all U.S. Government End Users
   acquire Covered Code with only those rights set forth herein.

   (( 11. ))??
   MISCELLANEOUS.

   This License represents the complete agreement concerning subject matter
   hereof. If any provision of this License is held to be unenforceable, such
   provision shall be reformed only to the extent necessary to make it
   enforceable. This License shall be governed by California law provisions
   (except to the extent applicable law, if any, provides otherwise), excluding
   its conflict-of-law provisions. With respect to disputes in which at least
   one party is a citizen of, or an entity chartered or registered to do
   business in the United States of America, any litigation relating to this
   License shall be subject to the jurisdiction of the Federal Courts of the
   Northern District of California, with venue lying in Santa Clara County,
   California, with the losing party responsible for costs, including without
   limitation, court costs and reasonable attorneys' fees and expenses. The
   application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any law or regulation
   which provides that the language of a contract shall be construed against the
   drafter shall not apply to this License.

   (( 12. ))??
   RESPONSIBILITY FOR CLAIMS.

   As between Initial Developer and the Contributors, each party is responsible
   for claims and damages arising, directly or indirectly, out of its
   utilization of rights under this License and You agree to work with Initial
   Developer and Contributors to distribute such responsibility on an equitable
   basis. Nothing herein is intended or shall be deemed to constitute any
   admission of liability.

   (( 13. ))??
   MULTIPLE-LICENSED CODE.

   Initial Developer may designate portions of the Covered Code as
   "Multiple-Licensed". "Multiple-Licensed" means that the Initial Developer
   permits you to utilize portions of the Covered Code under Your choice of the
   SPL or the alternative licenses, if any, specified by the Initial Developer
   in the file described in Exhibit A.
   (( SugarCRM Public License 1.1.3 - Exhibit A

The contents of this file are subject to the SugarCRM Public License Version
1.1.3 ("License"); You may not use this file except in compliance with the
License. You may obtain a copy of the License at http:/www.sugarcrm.com/SPL
Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
specific language governing rights and limitations under the License.

The Original Code is: SugarCRM Open Source

The Initial Developer of the Original Code is SugarCRM, Inc.

Portions created by SugarCRM are Copyright (C) 2004 SugarCRM, Inc.;

All Rights Reserved.

Contributor(s): _.

[NOTE: The text of this Exhibit A may differ slightly from the text of the
notices in the Source Code files of the Original Code. You should use the text
of this Exhibit A rather than the text found in the Original Code Source Code
for Your Modifications.]

SugarCRM Public License 1.1.3 - Exhibit B

Additional Terms applicable to the SugarCRM Public License.

   (( I. ))??
   Effect.,

   These additional terms described in this SugarCRM Public License - Additional
   Terms shall apply to the Covered Code under this License.

   (( II. ))??
   SugarCRM and logo.

   This License does not grant any rights to use the trademarks "SugarCRM" and
   the "SugarCRM" logos even if such marks are included in the Original Code or
   Modifications.

   However, in addition to the other notice obligations, all copies of the
   Covered Code in Executable and Source Code form distributed must, as a form
   of attribution of the original author, include on each user interface screen
   (i) the "Powered by SugarCRM" logo and (ii) the copyright notice in the same
   form as the latest version of the Covered Code distributed by SugarCRM, Inc.
   at the time of distribution of such copy. In addition, the "Powered by
   SugarCRM" logo must be visible to all users and be located at the very bottom
   center of each user interface screen. Notwithstanding the above, the
   dimensions of the "Powered By SugarCRM" logo must be at least 106 x 23
   pixels. When users click on the "Powered by SugarCRM" logo it must direct
   them back to http:/www.sugarforge.org. In addition, the copyright notice must
   remain visible to all users at all times at the bottom of the user interface
   screen. When users click on the copyright notice, it must direct them back to
   http:/www.sugarcrm.com ))??
`
const license_TAPR_OHL_1_0_lre = `//**
TAPR Open Hardware License v1.0
https://spdx.org/licenses/TAPR-OHL-1.0.json
https://www.tapr.org/OHL
**//

(( The TAPR Open Hardware License Version 1.0 (May 25, 2007)
(( Copyright __20__ ))??
))??

PREAMBLE

Open Hardware is a thing - a physical artifact, either electrical or mechanical
- whose design information is available to, and usable by, the public in a way
that allows anyone to make, modify, distribute, and use that thing. In this
preface, design information is called "documentation" and things created from it
are called "products."

The TAPR Open Hardware License ("OHL") agreement provides a legal framework for
Open Hardware projects. It may be used for any kind of product, be it a hammer
or a computer motherboard, and is TAPR's contribution to the community; anyone
may use the OHL for their Open Hardware project.

Like the GNU General Public License, the OHL is designed to guarantee your
freedom to share and to create. It forbids anyone who receives rights under the
OHL to deny any other licensee those same rights to copy, modify, and distribute
documentation, and to make, use and distribute products based on that
documentation.

Unlike the GPL, the OHL is not primarily a copyright license. While copyright
protects documentation from unauthorized copying, modification, and
distribution, it has little to do with your right to make, distribute, or use a
product based on that documentation. For better or worse, patents play a
significant role in those activities. Although it does not prohibit anyone from
patenting inventions embodied in an Open Hardware design, and of course cannot
prevent a third party from enforcing their patent rights, those who benefit from
an OHL design may not bring lawsuits claiming that design infringes their
patents or other intellectual property.

The OHL addresses unique issues involved in the creation of tangible, physical
things, but does not cover software, firmware, or code loaded into programmable
devices. A copyright-oriented license such as the GPL better suits these
creations.

How can you use the OHL, or a design based upon it? While the terms and
conditions below take precedence over this preamble, here is a summary:

   *
   You may modify the documentation and make products based upon it.

   *
   You may use products for any legal purpose without limitation.

   *
   You may distribute unmodified documentation, but you must include the
   complete package as you received it.

   *
   You may distribute products you make to third parties, if you either include
   the documentation on which the product is based, or make it available without
   charge for at least three years to anyone who requests it.

   *
   You may distribute modified documentation or products based on it, if you:

      *
      License your modifications under the OHL.

      *
      Include those modifications, following the requirements stated below.

      *
      Attempt to send the modified documentation by email to any of the
      developers who have provided their email address. This is a good faith
      obligation - if the email fails, you need do nothing more and may go on
      with your distribution.

   *
   If you create a design that you want to license under the OHL, you should:

      *
      Include this document in a file named LICENSE (with the appropriate
      extension) that is included in the documentation package.

      *
      If the file format allows, include a notice like "Licensed under the TAPR
      Open Hardware License (www.tapr.org/OHL)" in each documentation file.
      While not required, you should also include this notice on printed circuit
      board artwork and the product itself; if space is limited the notice can
      be shortened or abbreviated.

      *
      Include a copyright notice in each file and on printed circuit board
      artwork.

      *
      If you wish to be notified of modifications that others may make, include
      your email address in a file named "CONTRIB.TXT" or something similar.

   *
   Any time the OHL requires you to make documentation available to others, you
   must include all the materials you received from the upstream licensors. In
   addition, if you have modified the documentation:

      *
      You must identify the modifications in a text file (preferably named
      "CHANGES.TXT") that you include with the documentation. That file must
      also include a statement like "These modifications are licensed under the
      TAPR Open Hardware License."

      *
      You must include any new files you created, including any manufacturing
      files (such as Gerber files) you create in the course of making products.

      *
      You must include both "before" and "after" versions of all files you
      modified.

      *
      You may include files in proprietary formats, but you must also include
      open format versions (such as Gerber, ASCII, Postscript, or PDF) if your
      tools can create them.

TERMS AND CONDITIONS

1. Introduction

   (( 1.1 ))??
   This Agreement governs how you may use, copy, modify, and distribute
   Documentation, and how you may make, have made, and distribute Products based
   on that Documentation. As used in this Agreement, to "distribute"
   Documentation means to directly or indirectly make copies available to a
   third party, and to "distribute" Products means to directly or indirectly
   give, loan, sell or otherwise transfer them to a third party.

   (( 1.2 ))??
   "Documentation" includes:

      (( (a) ))??
      schematic diagrams;

      (( (b) ))??
      circuit or circuit board layouts, including Gerber and other data files
      used for manufacture;

      (( c ))??
      mechanical drawings, including CAD, CAM, and other data files used for
      manufacture;

      (( (d) ))??
      flow charts and descriptive text; and

      (( (e) ))??
      other explanatory material.

   Documentation may be in any tangible or intangible form of expression,
   including but not limited to computer files in open or proprietary formats
   and representations on paper, film, or other media.

   (( 1.3 ))??
   "Products" include:

      (( (a) ))??
      circuit boards, mechanical assemblies, and other physical parts and
      components;

      (( (b) ))??
      assembled or partially assembled units (including components and
      subassemblies); and

      (( c ))??
      parts and components combined into kits intended for assembly by others;
      which are based in whole or in part on the Documentation.

   (( 1.4 ))??
   This Agreement applies to any Documentation which contains a notice stating
   it is subject to the TAPR Open Hardware License, and to all Products based in
   whole or in part on that Documentation. If Documentation is distributed in an
   archive (such as a "zip" file) which includes this document, all files in
   that archive are subject to this Agreement unless they are specifically
   excluded. Each person who contributes content to the Documentation is
   referred to in this Agreement as a "Licensor."

   (( 1.5 ))??
   By (a) using, copying, modifying, or distributing the Documentation, or (b)
   making or having Products made or distributing them, you accept this
   Agreement, agree to comply with its terms, and become a "Licensee." Any
   activity inconsistent with this Agreement will automatically terminate your
   rights under it (including the immunities from suit granted in Section 2),
   but the rights of others who have received Documentation, or have obtained
   Products, directly or indirectly from you will not be affected so long as
   they fully comply with it themselves.

   (( 1.6 ))??
   This Agreement does not apply to software, firmware, or code loaded into
   programmable devices which may be used in conjunction with Documentation or
   Products. Such software is subject to the license terms established by its
   copyright holder(s).

2. Patents

   (( 2.1 ))??
   Each Licensor grants you, every other Licensee, and every possessor or user
   of Products a perpetual, worldwide, and royalty-free immunity from suit under
   any patent, patent application, or other intellectual property right which he
   or she controls, to the extent necessary to make, have made, possess, use,
   and distribute Products. This immunity does not extend to infringement
   arising from modifications subsequently made by others.

   (( 2.2 ))??
   If you make or have Products made, or distribute Documentation that you have
   modified, you grant every Licensor, every other Licensee, and every possessor
   or user of Products a perpetual, worldwide, and royalty-free immunity from
   suit under any patent, patent application, or other intellectual property
   right which you control, to the extent necessary to make, have made, possess,
   use, and distribute Products. This immunity does not extend to infringement
   arising from modifications subsequently made by others.

   (( 2.3 ))??
   To avoid doubt, providing Documentation to a third party for the sole purpose
   of having that party make Products on your behalf is not considered
   "distribution,"\" and a third party's act of making Products solely on your
   behalf does not cause that party to grant the immunity described in the
   preceding paragraph.

   (( 2.4 ))??
   These grants of immunity are a material part of this Agreement, and form a
   portion of the consideration given by each party to the other. If any court
   judgment or legal agreement prevents you from granting the immunity required
   by this Section, your rights under this Agreement will terminate and you may
   no longer use, copy, modify or distribute the Documentation, or make, have
   made, or distribute Products.

3. Modifications

You may modify the Documentation, and those modifications will become part of
the Documentation. They are subject to this Agreement, as are Products based in
whole or in part on them. If you distribute the modified Documentation, or
Products based in whole or in part upon it, you must email the modified
Documentation in a form compliant with Section 4 to each Licensor who has
provided an email address with the Documentation. Attempting to send the email
completes your obligations under this Section and you need take no further
action if any address fails.

4. Distributing Documentation

   (( 4.1 ))??
   You may distribute unmodified copies of the Documentation in its entirety in
   any medium, provided that you retain all copyright and other notices
   (including references to this Agreement) included by each Licensor, and
   include an unaltered copy of this Agreement.

   (( 4.2 ))??
   You may distribute modified copies of the Documentation if you comply with
   all the requirements of the preceding paragraph and:

      (( (a) ))??
      include a prominent notice in an ASCII or other open format file
      identifying those elements of the Documentation that you changed, and
      stating that the modifications are licensed under the terms of this
      Agreement;

      (( (b) ))??
      include all new documentation files that you create, as well as both the
      original and modified versions of each file you change (files may be in
      your development tool's native file format, but if reasonably possible,
      you must also include open format, such as Gerber, ASCII, Postscript, or
      PDF, versions);

      (( c ))??
      do not change the terms of this Agreement with respect to subsequent
      licensees; and

      (( (d) ))??
      if you make or have Products made, include in the Documentation all
      elements reasonably required to permit others to make Products, including
      Gerber, CAD/CAM and other files used for manufacture.

5. Making Products

   (( 5.1 ))??
   You may use the Documentation to make or have Products made, provided that
   each Product retains any notices included by the Licensor (including, but not
   limited to, copyright notices on circuit boards).

   (( 5.2 ))??
   You may distribute Products you make or have made, provided that you include
   with each unit a copy of the Documentation in a form consistent with Section
   4. Alternatively, you may include either (i) an offer valid for at least
   three years to provide that Documentation, at no charge other than the
   reasonable cost of media and postage, to any person who requests it; or (ii)
   a URL where that Documentation may be downloaded, available for at least
   three years after you last distribute the Product.

6. NEW LICENSE VERSIONS

TAPR may publish updated versions of the OHL which retain the same general
provisions as the present version, but differ in detail to address new problems
or concerns, and carry a distinguishing version number. If the Documentation
specifies a version number which applies to it and "any later version", you may
choose either that version or any later version published by TAPR. If the
Documentation does not specify a version number, you may choose any version ever
published by TAPR. TAPR owns the copyright to the OHL, but grants permission to
any person to copy, distribute, and use it in unmodified form.

7. WARRANTY AND LIABILITY LIMITATIONS

   (( 7.1 ))??
   THE DOCUMENTATION IS PROVIDED ON AN"AS-IS" BASIS WITHOUT WARRANTY OF ANY
   KIND, TO THE EXTENT PERMITTED BY APPLICABLE LAW. ALL WARRANTIES, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE, AND TITLE, ARE HEREBY EXPRESSLY
   DISCLAIMED.

   (( 7.2 ))??
   IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL ANY LICENSOR BE LIABLE TO
   YOU OR ANY THIRD PARTY FOR ANY DIRECT, INDIRECT, INCIDENTAL, CONSEQUENTIAL,
   PUNITIVE, OR EXEMPLARY DAMAGES ARISING OUT OF THE USE OF, OR INABILITY TO
   USE, THE DOCUMENTATION OR PRODUCTS, INCLUDING BUT NOT LIMITED TO CLAIMS OF
   INTELLECTUAL PROPERTY INFRINGEMENT OR LOSS OF DATA, EVEN IF THAT PARTY HAS
   BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

   (( 7.3 ))??
   You agree that the foregoing limitations are reasonable due to the
   non-financial nature of the transaction represented by this Agreement, and
   acknowledge that were it not for these limitations, the Licensor(s) would not
   be willing to make the Documentation available to you.

   (( 7.4 ))??
   You agree to defend, indemnify, and hold each Licensor harmless from any
   claim brought by a third party alleging any defect in the design,
   manufacture, or operation of any Product which you make, have made, or
   distribute pursuant to this Agreement.

####
`
const license_TCL_lre = `//**
TCL/TK License
https://spdx.org/licenses/TCL.json
http://www.tcl.tk/software/tcltk/license.html
https://fedoraproject.org/wiki/Licensing/TCL
**//

//** Copyright **//

((
This software is copyrighted by the Regents of the University of California,
Sun Microsystems, Inc., Scriptics Corporation, ActiveState Corporation and
other parties.
))??

The following terms apply to all files associated with the software unless
explicitly disclaimed in individual files.

The authors hereby grant permission to use, copy, modify, distribute, and
license this software and its documentation for any purpose, provided that
existing copyright notices are retained in all copies and that this notice is
included verbatim in any distributions. No written agreement, license, or
royalty fee is required for any of the authorized uses. Modifications to this
software may be copyrighted by their authors and need not follow the licensing
terms described here, provided that the new terms are clearly indicated on the
first page of each file where they apply.

IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY FOR DIRECT,
INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE
OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY DERIVATIVES THEREOF, EVEN IF THE
AUTHORS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE, AND NON-INFRINGEMENT. THIS SOFTWARE IS PROVIDED ON AN "AS
IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

GOVERNMENT USE: If you are acquiring this software on behalf of the U.S.
government, the Government shall have only "Restricted Rights" in the software
and related documentation as defined in the Federal Acquisition Regulations
(FARs) in Clause 52.227.19 (c) (2). If you are acquiring the software on behalf
of the Department of Defense, the software shall be classified as "Commercial
Computer Software" and the Government shall have only "Restricted Rights" as
defined in Clause 252.227-7013 (c) (1) of DFARs. Notwithstanding the foregoing,
the authors grant the U.S. Government and others acting in its behalf permission
to use and distribute the software in accordance with the terms specified in
this license.
`
const license_TCP_wrappers_lre = `//**
TCP Wrappers License
https://spdx.org/licenses/TCP-wrappers.json
http://rc.quest.com/topics/openssh/license.php#tcpwrappers
**//

//** Copyright **//

This material was originally written and compiled by Wietse Venema at Eindhoven
University of Technology, The Netherlands, in 1990, 1991, 1992, 1993, 1994 and
1995.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that this entire copyright notice is duplicated in all
such copies.

This software is provided "as is" and without any expressed or implied
warranties, including, without limitation, the implied warranties of
merchantibility and fitness for any particular purpose.
`
const license_TMate_lre = `//**
TMate Open Source License
https://spdx.org/licenses/TMate.json
http://svnkit.com/license.html
**//

(( The TMate Open Source License. ))??

(( This license applies to all portions of TMate SVNKit library, which are not
externally-maintained libraries (e.g. Ganymed SSH library).

All the source code and compiled classes in package org.tigris.subversion.javahl
except SvnClient class are covered by the license in JAVAHL-LICENSE file

Copyright __5__ TMate Software. All rights reserved.
))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   *
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   *
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   *
   Redistributions in any form must be accompanied by information on how to
   obtain complete source code for the software that uses SVNKit and any
   accompanying software that uses the software that uses SVNKit. The source
   code must either be included in the distribution or be available for no more
   than the cost of distribution plus a nominal fee, and must be freely
   redistributable under reasonable conditions. For an executable file, complete
   source code means the source code for all modules it contains. It does not
   include source code for modules or files that typically accompany the major
   components of the operating system on which the executable file runs.

   *
   Redistribution in any form without redistributing source code for software
   that uses SVNKit is possible only when such redistribution is explictly
   permitted by TMate Software. Please, contact TMate Software at
   support@svnkit.com to get such permission.

THIS SOFTWARE IS PROVIDED BY TMATE SOFTWARE ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT, ARE
DISCLAIMED.

IN NO EVENT SHALL TMATE SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
`
const license_TORQUE_1_1_lre = `//**
TORQUE v2.5+ Software License v1.1
https://spdx.org/licenses/TORQUE-1.1.json
https://fedoraproject.org/wiki/Licensing/TORQUEv1.1
**//

(( TORQUE v2.5+ Software License v1.1
((Copyright __20__))??
))??

(( Use this license to use or redistribute the TORQUE software v2.5+ and later
versions. For free support for TORQUE users, questions should be emailed to the
community of TORQUE users at torqueusers@supercluster.org. Users can also
subscribe to the user mailing list at
http:/www.supercluster.org/mailman/listinfo/torqueusers. Customers using TORQUE
that also are licensed users of Moab branded software from Adaptive Computing
Inc. can get TORQUE support from Adaptive Computing via:

Email: torque-support@adaptivecomputing.com.

Phone: (801) 717-3700

Web: www.adaptivecomputing.com www.clusterresources.com ))??

This license covers use of the TORQUE v2.5 software (the "Software") at your
site or location, and, for certain users, redistribution of the Software to
other sites and locations1. Later versions of TORQUE are also covered by this
license. Use and redistribution of TORQUE v2.5 in source and binary forms, with
or without modification, are permitted provided that all of the following
conditions are met.

   (( 1. ))??
   Any Redistribution of source code must retain the above copyright notice and
   the acknowledgment contained in paragraph 5, this list of conditions and the
   disclaimer contained in paragraph 5.

   (( 2. ))??
   Any Redistribution in binary form must reproduce the above copyright notice
   and the acknowledgment contained in paragraph 4, this list of conditions and
   the disclaimer contained in paragraph 5 in the documentation and/or other
   materials provided with the distribution.

   (( 3. ))??
   Redistributions in any form must be accompanied by information on how to
   obtain complete source code for TORQUE and any modifications and/or additions
   to TORQUE. The source code must either be included in the distribution or be
   available for no more than the cost of distribution plus a nominal fee, and
   all modifications and additions to the Software must be freely
   redistributable by any party (including Licensor) without restriction.

   (( 4. ))??
   All advertising materials mentioning features or use of the Software must
   display the following acknowledgment:

   "TORQUE is a modification of OpenPBS which was developed by NASA Ames
   Research Center, Lawrence Livermore National Laboratory, and Veridian TORQUE
   Open Source License v1.1. 1 Information Solutions, Inc. Visit
   www.clusterresources.com/products/ for more information about TORQUE and to
   download TORQUE. For information about Moab branded products and so receive
   support from Adaptive Computing for TORQUE, see www.adaptivecomputing.com."

   (( 5. ))??
   DISCLAIMER OF WARRANTY THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF
   ANY KIND. ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
   THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
   AND NON-INFRINGEMENT ARE EXPRESSLY DISCLAIMED. IN NO EVENT SHALL ADAPTIVE
   COMPUTING ENTERPRISES, INC. CORPORATION, ITS AFFILIATED COMPANIES, OR THE
   U.S. GOVERNMENT OR ANY OF ITS AGENCIES BE LIABLE FOR ANY DIRECT OR INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
   OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
   EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

This license will be governed by the laws of Utah, without reference to its
choice of law rules.
(( Note 1: TORQUE is developed from an earlier version v2.3 of OpenPBS. TORQUE
has been developed beyond OpenPBS v2.3. The OpenPBS v2.3 license and OpenPBS
software can be obtained at:

http:/www.pbsworks.com/ResLibSearchResult.aspx?keywords=openpbs&industry=All&pro
duct_service=All&category=Free%20Software%20Downloads&order_by=title. Users of
TORQUE should comply with the TORQUE license as well as the OpenPBS license. ))??
`
const license_TOSL_lre = `//**
Trusster Open Source License
https://spdx.org/licenses/TOSL.json
https://fedoraproject.org/wiki/Licensing/TOSL
**//

((
Trusster Open Source License version 1.0a (TRUST)
((Copyright __20__))??
))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   *
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   *
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   *
   Redistributions in any form must be accompanied by information on how to
   obtain complete source code for this software and any accompanying software
   that uses this software. The source code must either be included in the
   distribution or be available in a timely fashion for no more than the cost of
   distribution plus a nominal fee, and must be freely redistributable under
   reasonable and no more restrictive conditions. For an executable file,
   complete source code means the source code for all modules it contains. It
   does not include source code for modules or files that typically accompany
   the major components of the operating system on which the executable file
   runs.

THIS SOFTWARE IS PROVIDED BY MIKE MINTZ AND ROBERT EKENDAHL ` + "`" + `` + "`" + `AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
NON-INFRINGEMENT, ARE DISCLAIMED. IN NO EVENT SHALL MIKE MINTZ AND ROBERT
EKENDAHL OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
`
const license_TU_Berlin_1_0_lre = `//**
Technische Universitaet Berlin License 1.0
https://spdx.org/licenses/TU-Berlin-1.0.json
https://github.com/swh/ladspa/blob/7bf6f3799fdba70fda297c2d8fd9f526803d9680/gsm/COPYRIGHT
**//

//** Copyright **//

Any use of this software is permitted provided that this notice is not removed
and that neither the authors nor the Technische Universitaet Berlin are deemed
to have made any representations as to the suitability of this software for any
purpose nor are held responsible for any defects of this software. THERE IS
ABSOLUTELY NO WARRANTY FOR THIS SOFTWARE.

As a matter of courtesy, the authors request to be informed about uses this
software has found, about bugs in this software, and about any improvements that
may be of general interest.
(( Berlin,
//** 28.11.1994 **//
__5__

Jutta Degener

Carsten Bormann ))??
`
const license_TU_Berlin_2_0_lre = `//**
Technische Universitaet Berlin License 2.0
https://spdx.org/licenses/TU-Berlin-2.0.json
https://github.com/CorsixTH/deps/blob/fd339a9f526d1d9c9f01ccf39e438a015da50035/licences/libgsm.txt
**//

//** Copyright **//

Any use of this software is permitted provided that this notice is not removed
and that neither the authors nor the Technische Universitaet Berlin are deemed
to have made any representations as to the suitability of this software for any
purpose nor are held responsible for any defects of this software. THERE IS
ABSOLUTELY NO WARRANTY FOR THIS SOFTWARE.

As a matter of courtesy, the authors request to be informed about uses this
software has found, about bugs in this software, and about any improvements that
may be of general interest.
(( Berlin,
//** 28.11.1994 **//
__5__

Jutta Degener

Carsten Bormann ))??

(( oOo ))??

Since the original terms of 15 years ago maybe do not make our intentions
completely clear given today's refined usage of the legal terms, we append this
additional permission:

Permission to use, copy, modify, and distribute this software for any purpose
with or without fee is hereby granted, provided that this notice is not removed
and that neither the authors nor the Technische Universitaet Berlin are deemed
to have made any representations as to the suitability of this software for any
purpose nor are held responsible for any defects of this software. THERE IS
ABSOLUTELY NO WARRANTY FOR THIS SOFTWARE.
(( Berkeley/Bremen,
//** 05.04.2009 **//
__5__

Jutta Degener

Carsten Bormann ))??
`
const license_UCL_1_0_lre = `//**
Upstream Compatibility License v1.0
https://spdx.org/licenses/UCL-1.0.json
https://opensource.org/licenses/UCL-1.0
**//

(( Upstream Compatibility License v. 1.0 (UCL-1.0) ))??

This Upstream Compatibility License (the "License") applies to any original work
of authorship (the "Original Work") whose owner (the "Licensor") has placed the
following licensing notice adjacent to the copyright notice for the Original
Work:

Licensed under the Upstream Compatibility License 1.0

   (( 1) ))??
   Grant of Copyright License. Licensor grants You a worldwide, royalty-free,
   non-exclusive, sublicensable license, for the duration of the copyright, to
   do the following:

      (( a) ))??
      to reproduce the Original Work in copies, either alone or as part of a
      collective work;

      (( b) ))??
      to translate, adapt, alter, transform, modify, or arrange the Original
      Work, thereby creating derivative works ("Derivative Works") based upon
      the Original Work;

      (( c) ))??
      to distribute or communicate copies of the Original Work and Derivative
      Works to the public, with the proviso that copies of Original Work You
      distribute or communicate shall be licensed under this Upstream
      Compatibility License and all Derivative Work You distribute or
      communicate shall be licensed under both this Upstream Compatibility
      License and the Apache License 2.0 or later;

      (( d) ))??
      to perform the Original Work publicly; and

      (( e) ))??
      to display the Original Work publicly.

   (( 2) ))??
   Grant of Patent License. Licensor grants You a worldwide, royalty-free,
   non-exclusive, sublicensable license, under patent claims owned or controlled
   by the Licensor that are embodied in the Original Work as furnished by the
   Licensor, for the duration of the patents, to make, use, sell, offer for
   sale, have made, and import the Original Work and Derivative Works.

   (( 3) ))??
   Grant of Source Code License. The term "Source Code" means the preferred form
   of the Original Work for making modifications to it and all available
   documentation describing how to modify the Original Work. Licensor agrees to
   provide a machine-readable copy of the Source Code of the Original Work along
   with each copy of the Original Work that Licensor distributes. Licensor
   reserves the right to satisfy this obligation by placing a machine-readable
   copy of the Source Code in an information repository reasonably calculated to
   permit inexpensive and convenient access by You for as long as Licensor
   continues to distribute the Original Work.

   (( 4) ))??
   Exclusions From License Grant. Neither the names of Licensor, nor the names
   of any contributors to the Original Work, nor any of their trademarks or
   service marks, may be used to endorse or promote products derived from this
   Original Work without express prior permission of the Licensor. Except as
   expressly stated herein, nothing in this License grants any license to
   Licensor's trademarks, copyrights, patents, trade secrets or any other
   intellectual property. No patent license is granted to make, use, sell, offer
   for sale, have made, or import embodiments of any patent claims other than
   the licensed claims defined in Section 2. No license is granted to the
   trademarks of Licensor even if such marks are included in the Original Work.
   Nothing in this License shall be interpreted to prohibit Licensor from
   licensing under terms different from this License any Original Work that
   Licensor otherwise would have a right to license.

   (( 5) ))??
   External Deployment. The term "External Deployment" means the use,
   distribution, or communication of the Original Work or Derivative Works in
   any way such that the Original Work or Derivative Works may be used by anyone
   other than You, whether those works are distributed or communicated to those
   persons or made available as an application intended for use over a network.
   As an express condition for the grants of license hereunder, You must treat
   any External Deployment by You of the Original Work or a Derivative Work as a
   distribution under section 1(c).

   (( 6) ))??
   Attribution Rights. You must retain, in the Source Code of any Derivative
   Works that You create, all copyright, patent, or trademark notices from the
   Source Code of the Original Work, as well as any notices of licensing and any
   descriptive text identified therein as an "Attribution Notice." You must
   cause the Source Code for any Derivative Works that You create to carry a
   prominent Attribution Notice reasonably calculated to inform recipients that
   You have modified the Original Work.

   (( 7) ))??
   Warranty of Provenance and Disclaimer of Warranty. Licensor warrants that the
   copyright in and to the Original Work and the patent rights granted herein by
   Licensor are owned by the Licensor or are sublicensed to You under the terms
   of this License with the permission of the contributor(s) of those copyrights
   and patent rights. Except as expressly stated in the immediately preceding
   sentence, the Original Work is provided under this License on an "AS IS"
   BASIS and WITHOUT WARRANTY, either express or implied, including, without
   limitation, the warranties of non-infringement, merchantability or fitness
   for a particular purpose. THE ENTIRE RISK AS TO THE QUALITY OF THE ORIGINAL
   WORK IS WITH YOU. This DISCLAIMER OF WARRANTY constitutes an essential part
   of this License. No license to the Original Work is granted by this License
   except under this disclaimer.

   (( 8) ))??
   Limitation of Liability. Under no circumstances and under no legal theory,
   whether in tort (including negligence), contract, or otherwise, shall the
   Licensor be liable to anyone for any indirect, special, incidental, or
   consequential damages of any character arising as a result of this License or
   the use of the Original Work including, without limitation, damages for loss
   of goodwill, work stoppage, computer failure or malfunction, or any and all
   other commercial damages or losses. This limitation of liability shall not
   apply to the extent applicable law prohibits such limitation.

   (( 9) ))??
   Acceptance and Termination. If, at any time, You expressly assented to this
   License, that assent indicates your clear and irrevocable acceptance of this
   License and all of its terms and conditions. If You distribute or communicate
   copies of the Original Work or a Derivative Work, You must make a reasonable
   effort under the circumstances to obtain the express assent of recipients to
   the terms of this License. This License conditions your rights to undertake
   the activities listed in Section 1, including your right to create Derivative
   Works based upon the Original Work, and doing so without honoring these terms
   and conditions is prohibited by copyright law and international treaty.
   Nothing in this License is intended to affect copyright exceptions and
   limitations (including "fair use" or "fair dealing"). This License shall
   terminate immediately and You may no longer exercise any of the rights
   granted to You by this License upon your failure to honor the conditions in
   Section 1(c).

   (( 10) ))??
   Termination for Patent Action. This License shall terminate automatically and
   You may no longer exercise any of the rights granted to You by this License
   as of the date You commence an action, including a cross-claim or
   counterclaim, against Licensor or any licensee alleging that the Original
   Work infringes a patent. This termination provision shall not apply for an
   action alleging patent infringement by combinations of the Original Work with
   other software or hardware.

   (( 11) ))??
   Jurisdiction, Venue and Governing Law. Any action or suit relating to this
   License may be brought only in the courts of a jurisdiction wherein the
   Licensor resides or in which Licensor conducts its primary business, and
   under the laws of that jurisdiction excluding its conflict-of-law provisions.
   The application of the United Nations Convention on Contracts for the
   International Sale of Goods is expressly excluded. Any use of the Original
   Work outside the scope of this License or after its termination shall be
   subject to the requirements and penalties of copyright or patent law in the
   appropriate jurisdiction. This section shall survive the termination of this
   License.

   (( 12) ))??
   Attorneys' Fees. In any action to enforce the terms of this License or
   seeking damages relating thereto, the prevailing party shall be entitled to
   recover its costs and expenses, including, without limitation, reasonable
   attorneys' fees and costs incurred in connection with such action, including
   any appeal of such action. This section shall survive the termination of this
   License.

   (( 13) ))??
   Miscellaneous. If any provision of this License is held to be unenforceable,
   such provision shall be reformed only to the extent necessary to make it
   enforceable.

   (( 14) ))??
   Definition of "You" in This License. "You" throughout this License, whether
   in upper or lower case, means an individual or a legal entity exercising
   rights under, and complying with all of the terms of, this License. For legal
   entities, "You" includes any entity that controls, is controlled by, or is
   under common control with you. For purposes of this definition, "control"
   means (i) the power, direct or indirect, to cause the direction or management
   of such entity, whether by contract or otherwise, or (ii) ownership of fifty
   percent (50%) or more of the outstanding shares, or (iii) beneficial
   ownership of such entity.

   (( 15) ))??
   Right to Use. You may use the Original Work in all ways not otherwise
   restricted or conditioned by this License or by law, and Licensor promises
   not to interfere with or be responsible for such uses by You.

   (( 16) ))??
   Modification of This License. This License is Copyright (c) 2005 Lawrence
   Rosen and Copyright (c) 2017 Nigel Tzeng. Permission is granted to copy,
   distribute, or communicate this License without modification. Nothing in this
   License permits You to modify this License as applied to the Original Work or
   to Derivative Works. However, You may modify the text of this License and
   copy, distribute or communicate your modified version (the "Modified
   License") and apply it to other original works of authorship subject to the
   following conditions: (i) You may not indicate in any way that your Modified
   License is the "Open Software License" or "OSL" or the "Upstream
   Compatibility License" or "UCL" and you may not use those names in the name
   of your Modified License; (ii) You must replace the notice specified in the
   first paragraph above with the notice "Licensed under
   (( <insert your license name here> ))??
   " or with a notice of your own that is not confusingly similar to the notice
   in this License; and (iii) You may not claim that your original works are
   open source software unless your Modified License has been approved by Open
   Source Initiative (OSI) and You comply with its license review and
   certification process.
`
const license_UPL_1_0_lre = `//**
Universal Permissive License v1.0
https://spdx.org/licenses/UPL-1.0.json
https://opensource.org/licenses/UPL
**//

//** Copyright **//

(( The Universal Permissive License (UPL), Version 1.0 ))??

Subject to the condition set forth below, permission is hereby granted to any
person obtaining a copy of this software, associated documentation and/or data
(collectively the "Software"), free of charge and under any and all copyright
rights in the Software, and any and all patent rights owned or freely licensable
by each licensor hereunder covering either (i) the unmodified Software as
contributed to or provided by such licensor, or (ii) the Larger Works (as
defined below), to deal in both

   (( (a) ))??
   the Software, and

   (( (b) ))??
   any piece of software and/or hardware listed in the lrgrwrks.txt file if one
   is included with the Software (each a "Larger Work" to which the Software is
   contributed by such licensors),

without restriction, including without limitation the rights to copy, create
derivative works of, display, perform, and distribute the Software and make,
use, sell, offer for sale, import, export, have made, and have sold the Software
and the Larger Work(s), and to sublicense the foregoing rights on either these
or other terms.

This license is subject to the following condition:

The above copyright notice and either this complete permission notice or at a
minimum a reference to the UPL must be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
`
const license_Unicode_DFS_2015_lre = `//**
Unicode License Agreement - Data Files and Software (2015)
https://spdx.org/licenses/Unicode-DFS-2015.json
https://web.archive.org/web/20151224134844/http://unicode.org/copyright.html
**//

(( UNICODE, INC. LICENSE AGREEMENT - DATA FILES AND SOFTWARE ))??

Unicode Data Files include all data files under the directories
http:/www.unicode.org/Public/, http:/www.unicode.org/reports/, and
http:/www.unicode.org/cldr/data/. Unicode Data Files do not include PDF online
code charts under the directory http:/www.unicode.org/Public/. Software includes
any source code published in the Unicode Standard or under the directories
http:/www.unicode.org/Public/, http:/www.unicode.org/reports/, and
http:/www.unicode.org/cldr/data/.

NOTICE TO USER: Carefully read the following legal agreement. BY DOWNLOADING,
INSTALLING, COPYING OR OTHERWISE USING UNICODE INC.'S DATA FILES ("DATA FILES"),
AND/OR SOFTWARE ("SOFTWARE"), YOU UNEQUIVOCALLY ACCEPT, AND AGREE TO BE BOUND
BY, ALL OF THE TERMS AND CONDITIONS OF THIS AGREEMENT. IF YOU DO NOT AGREE, DO
NOT DOWNLOAD, INSTALL, COPY, DISTRIBUTE OR USE THE DATA FILES OR SOFTWARE.

COPYRIGHT AND PERMISSION NOTICE

Copyright © 1991-2015 Unicode, Inc. All rights reserved. Distributed under the
Terms of Use in http:/www.unicode.org/copyright.html.

Permission is hereby granted, free of charge, to any person obtaining a copy of
the Unicode data files and any associated documentation (the "Data Files") or
Unicode software and any associated documentation (the "Software") to deal in
the Data Files or Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, and/or sell copies of
the Data Files or Software, and to permit persons to whom the Data Files or
Software are furnished to do so, provided that

   (( (a) ))??
   ((this || the above))
   copyright
   ((notices))??
   and
   ((this))??
   permission notice appear with all copies of the Data Files
   or Software,

   (( (b) ))??
   ((both))??
   ((this || the above))
   copyright
   ((notices))??
   and
   ((this))??
   permission notice appear in associated documentation, and

   (( c ))??
   there is clear notice in each modified Data File or in the Software as well
   as in the documentation associated with the Data File(s) or Software that the
   data or software has been modified.

THE DATA FILES AND SOFTWARE ARE PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD
PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR HOLDERS INCLUDED IN THIS
NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL
DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THE DATA FILES OR
SOFTWARE.

Except as contained in this notice, the name of a copyright holder shall not be
used in advertising or otherwise to promote the sale, use or other dealings in
these Data Files or Software without prior written authorization of the
copyright holder.
`
const license_Unicode_DFS_2016_lre = `//**
Unicode License Agreement - Data Files and Software (2016)
https://spdx.org/licenses/Unicode-DFS-2016.json
http://www.unicode.org/copyright.html
**//

(( UNICODE, INC. LICENSE AGREEMENT - DATA FILES AND SOFTWARE ))??

Unicode Data Files include all data files under the directories
http:/www.unicode.org/Public/, http:/www.unicode.org/reports/,
http:/www.unicode.org/cldr/data/, http:/source.icu-project.org/repos/icu/,
(( http:/www.unicode.org/ivd/data/, ))??
and http:/www.unicode.org/utility/trac/browser/.

Unicode Data Files do not include PDF online code charts under the directory
http:/www.unicode.org/Public/.

Software includes any source code published in the Unicode Standard or under the
directories http:/www.unicode.org/Public/, http:/www.unicode.org/reports/,
http:/www.unicode.org/cldr/data/, http:/source.icu-project.org/repos/icu/, and
http:/www.unicode.org/utility/trac/browser/.

NOTICE TO USER: Carefully read the following legal agreement. BY DOWNLOADING,
INSTALLING, COPYING OR OTHERWISE USING UNICODE INC.'S DATA FILES ("DATA FILES"),
AND/OR SOFTWARE ("SOFTWARE"), YOU UNEQUIVOCALLY ACCEPT, AND AGREE TO BE BOUND
BY, ALL OF THE TERMS AND CONDITIONS OF THIS AGREEMENT. IF YOU DO NOT AGREE, DO
NOT DOWNLOAD, INSTALL, COPY, DISTRIBUTE OR USE THE DATA FILES OR SOFTWARE.

COPYRIGHT AND PERMISSION NOTICE
Copyright __5__ Unicode, Inc.
All rights reserved. Distributed under the Terms of Use in http://www.unicode.org/copyright.html.

Permission is hereby granted, free of charge, to any person obtaining a copy of
the Unicode data files and any associated documentation (the "Data Files") or
Unicode software and any associated documentation (the "Software") to deal in
the Data Files or Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, and/or sell copies of
the Data Files or Software, and to permit persons to whom the Data Files or
Software are furnished to do so, provided that either

   (( (a) ))??
   this copyright and permission notice appear with all copies of the Data Files
   or Software, or

   (( (b) ))??
   this copyright and permission notice appear in associated Documentation.

THE DATA FILES AND SOFTWARE ARE PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD
PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR HOLDERS INCLUDED IN THIS
NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL
DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THE DATA FILES OR
SOFTWARE.

Except as contained in this notice, the name of a copyright holder shall not be
used in advertising or otherwise to promote the sale, use or other dealings in
these Data Files or Software without prior written authorization of the
copyright holder.
`
const license_Unicode_TOU_lre = `//**
Unicode Terms of Use
https://spdx.org/licenses/Unicode-TOU.json
http://www.unicode.org/copyright.html
**//

(( Unicode Terms of Use ))??

For the general privacy policy governing access to this site, see the Unicode
Privacy Policy. For trademark usage, see the Unicode® Consortium Name and
Trademark Usage Policy.

   (( A. ))??
   Unicode Copyright.

      (( 1. ))??
      Copyright © 1991-2014 Unicode, Inc. All rights reserved.

      (( 2. ))??
      Certain documents and files on this website contain a legend indicating
      that "Modification is permitted." Any person is hereby authorized, without
      fee, to modify such documents and files to create derivative works
      conforming to the Unicode® Standard, subject to Terms and Conditions
      herein.

      (( 3. ))??
      Any person is hereby authorized, without fee, to view, use, reproduce, and
      distribute all documents and files solely for informational purposes in
      the creation of products supporting the Unicode Standard, subject to the
      Terms and Conditions herein.

      (( 4. ))??
      Further specifications of rights and restrictions pertaining to the use of
      the particular set of data files known as the "Unicode Character Database"
      can be found in Exhibit 1.

      (( 5. ))??
      Each version of the Unicode Standard has further specifications of rights
      and restrictions of use. For the book editions (Unicode 5.0 and earlier),
      these are found on the back of the title page. The online code charts
      carry specific restrictions. All other files, including online
      documentation of the core specification for Unicode 6.0 and later, are
      covered under these general Terms of Use.

      (( 6. ))??
      No license is granted to "mirror" the Unicode website where a fee is
      charged for access to the "mirror" site.

      (( 7. ))??
      Modification is not permitted with respect to this document. All copies of
      this document must be verbatim.

   (( B. ))??
   Restricted Rights Legend. Any technical data or software which is licensed to
   the United States of America, its agencies and/or instrumentalities under
   this Agreement is commercial technical data or commercial computer software
   developed exclusively at private expense as defined in FAR 2.101, or DFARS
   252.227-7014 (June 1995), as applicable. For technical data, use,
   duplication, or disclosure by the Government is subject to restrictions as
   set forth in DFARS 202.227-7015 Technical Data, Commercial and Items (Nov
   1995) and this Agreement. For Software, in accordance with FAR 12-212 or
   DFARS 227-7202, as applicable, use, duplication or disclosure by the
   Government is subject to the restrictions set forth in this Agreement.

   (( C. ))??
   Warranties and Disclaimers.

      (( 1. ))??
      This publication and/or website may include technical or typographical
      errors or other inaccuracies . Changes are periodically added to the
      information herein; these changes will be incorporated in new editions of
      the publication and/or website. Unicode may make improvements and/or
      changes in the product(s) and/or program(s) described in this publication
      and/or website at any time.

      (( 2. ))??
      If this file has been purchased on magnetic or optical media from Unicode,
      Inc. the sole and exclusive remedy for any claim will be exchange of the
      defective media within ninety (90) days of original purchase.

      (( 3. ))??
      EXCEPT AS PROVIDED IN SECTION C.2, THIS PUBLICATION AND/OR SOFTWARE IS
      PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESS, IMPLIED, OR
      STATUTORY, INCLUDING, BUT NOT LIMITED TO, ANY WARRANTIES OF
      MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
      UNICODE AND ITS LICENSORS ASSUME NO RESPONSIBILITY FOR ERRORS OR OMISSIONS
      IN THIS PUBLICATION AND/OR SOFTWARE OR OTHER DOCUMENTS WHICH ARE
      REFERENCED BY OR LINKED TO THIS PUBLICATION OR THE UNICODE WEBSITE.

   (( D. ))??
   Waiver of Damages. In no event shall Unicode or its licensors be liable for
   any special, incidental, indirect or consequential damages of any kind, or
   any damages whatsoever, whether or not Unicode was advised of the possibility
   of the damage, including, without limitation, those resulting from the
   following: loss of use, data or profits, in connection with the use,
   modification or distribution of this information or its derivatives.

   (( E. ))??
   Trademarks & Logos.

      (( 1. ))??
      The Unicode Word Mark and the Unicode Logo are trademarks of Unicode, Inc.
      "The Unicode Consortium" and "Unicode, Inc." are trade names of Unicode,
      Inc. Use of the information and materials found on this website indicates
      your acknowledgement of Unicode, Inc.'s exclusive worldwide rights in the
      Unicode Word Mark, the Unicode Logo, and the Unicode trade names.

      (( 2. ))??
      The Unicode Consortium Name and Trademark Usage Policy ("Trademark
      Policy") are incorporated herein by reference and you agree to abide by
      the provisions of the Trademark Policy, which may be changed from time to
      time in the sole discretion of Unicode, Inc.

      (( 3. ))??
      All third party trademarks referenced herein are the property of their
      respective owners.

   (( F. ))??
   Miscellaneous.

      (( 1. ))??
      Jurisdiction and Venue. This server is operated from a location in the
      State of California, United States of America. Unicode makes no
      representation that the materials are appropriate for use in other
      locations. If you access this server from other locations, you are
      responsible for compliance with local laws. This Agreement, all use of
      this site and any claims and damages resulting from use of this site are
      governed solely by the laws of the State of California without regard to
      any principles which would apply the laws of a different jurisdiction. The
      user agrees that any disputes regarding this site shall be resolved solely
      in the courts located in Santa Clara County, California. The user agrees
      said courts have personal jurisdiction and agree to waive any right to
      transfer the dispute to any other forum.

      (( 2. ))??
      Modification by Unicode Unicode shall have the right to modify this
      Agreement at any time by posting it to this site. The user may not assign
      any part of this Agreement without Unicode's prior written consent.

      (( 3. ))??
      Taxes. The user agrees to pay any taxes arising from access to this
      website or use of the information herein, except for those based on
      Unicode's net income.

      (( 4. ))??
      Severability. If any provision of this Agreement is declared invalid or
      unenforceable, the remaining provisions of this Agreement shall remain in
      effect.

      (( 5. ))??
      Entire Agreement. This Agreement constitutes the entire agreement between
      the parties.
`
const license_Unlicense_lre = `//**
The Unlicense
https://spdx.org/licenses/Unlicense.json
https://unlicense.org/
**//

((This))??
is free and unencumbered software released into the public domain.

Anyone is free to copy, modify, publish, use, compile, sell, or distribute this
software, either in source code form or as a compiled binary, for any purpose,
commercial or non-commercial, and by any means.

In jurisdictions that recognize copyright laws, the author or authors of this
software dedicate any and all copyright interest in the software to the public
domain. We make this dedication for the benefit of the public at large and to
the detriment of our heirs and successors. We intend this dedication to be an
overt act of relinquishment in perpetuity of all present and future rights to
this software under copyright law.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

(( For more information, please refer to <https:/unlicense.org/> ))??
`
const license_VOSTROM_lre = `//**
VOSTROM Public License for Open Source
https://spdx.org/licenses/VOSTROM.json
https://fedoraproject.org/wiki/Licensing/VOSTROM
**//

(( VOSTROM Public License for Open Source
(( Copyright __20__ ))??
))??

This VOSTROM Holdings, Inc. (VOSTROM) Distribution (code and documentation) is
made available to the open source community as a public service by VOSTROM.
Contact VOSTROM at license@vostrom.com for information on other licensing
arrangements (e.g. for use in proprietary applications).

Under this license, this Distribution may be modified and the original version
and modified versions may be copied, distributed, publicly displayed and
performed provided that the following conditions are met:

   (( 1. ))??
   Modified versions are distributed with source code and documentation and with
   permission for others to use any code and documentation (whether in original
   or modified versions) as granted under this license;

   (( 2. ))??
   if modified, the source code, documentation, and user run-time elements
   should be clearly labeled by placing an identifier of origin (such as a name,
   initial, or other tag) after the version number;

   (( 3. ))??
   users, modifiers, distributors, and others coming into possession or using
   the Distribution in original or modified form accept the entire risk as to
   the possession, use, and performance of the Distribution;

   (( 4. ))??
   this copyright management information (software identifier and version
   number, copyright notice and license) shall be retained in all versions of
   the Distribution;

   (( 5. ))??
   VOSTROM may make modifications to the Distribution that are substantially
   similar to modified versions of the Distribution, and may make, use, sell,
   copy, distribute, publicly display, and perform such modifications, including
   making such modifications available under this or other licenses, without
   obligation or restriction;

   (( 6. ))??
   modifications incorporating code, libraries, and/or documentation subject to
   any other open source license may be made, and the resulting work may be
   distributed under the terms of such open source license if required by that
   open source license, but doing so will not affect this Distribution, other
   modifications made under this license or modifications made under other
   VOSTROM licensing arrangements;

   (( 7. ))??
   no permission is granted to distribute, publicly display, or publicly perform
   modifications to the Distribution made using proprietary materials that
   cannot be released in source format under conditions of this license;

   (( 8. ))??
   the name of VOSTROM may not be used in advertising or publicity pertaining to
   Distribution of the software without specific, prior written permission.

This software is made available "as is", and

VOSTROM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, WITH REGARD TO THIS
SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE, AND IN NO EVENT SHALL VOSTROM BE LIABLE
FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
`
const license_VSL_1_0_lre = `//**
Vovida Software License v1.0
https://spdx.org/licenses/VSL-1.0.json
https://opensource.org/licenses/VSL-1.0
**//

(( Vovida Software License v. 1.0 ))??

(( This license applies to all software incorporated in the "Vovida Open
Communication Application Library" except for those portions incorporating third
party software specifically identified as being licensed under separate license. ))??

(( The Vovida Software License, Version 1.0
(( Copyright __20__ ))??
))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The names "VOCAL", "Vovida Open Communication Application Library", and
   "Vovida Open Communication Application Library (VOCAL)" must not be used to
   endorse or promote products derived from this software without prior written
   permission. For written permission, please contact vocal@vovida.org.

   (( 4. ))??
   Products derived from this software may not be called "VOCAL", nor may
   "VOCAL" appear in their name, without prior written permission.

THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED. IN
NO EVENT SHALL VOVIDA NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY
DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
`
const license_Vim_lre = `//**
Vim License
https://spdx.org/licenses/Vim.json
http://vimdoc.sourceforge.net/htmldoc/uganda.html
**//

(( VIM LICENSE ))??

   (( I) ))??
   There are no restrictions on distributing unmodified copies of
   //** Vim **//
   __5__
   except that they must include this license text. You can also distribute
   unmodified parts of
   //** Vim **//
   __5__
   , likewise unrestricted except that they must include this license text. You
   are also allowed to include executables that you made from the unmodified
   //** Vim **//
   __5__
   sources, plus your own usage examples and Vim scripts.

   (( II) ))??
   It is allowed to distribute a modified (or extended) version of
   //** Vim **//
   __5__
   , including executables and/or source code, when the following four
   conditions are met:

      (( 1) ))??
      This license text must be included unmodified.

      (( 2) ))??
      The modified
      //** Vim **//
      __5__
      must be distributed in one of the following five ways:

         (( a) ))??
         If you make changes to
         //** Vim **//
         __5__
         yourself, you must clearly describe in the distribution how to contact
         you. When the maintainer asks you (in any way) for a copy of the
         modified
         //** Vim **//
         __5__
         you distributed, you must make your changes, including source code,
         available to the maintainer without fee. The maintainer reserves the
         right to include your changes in the official version of
         //** Vim **//
         __5__
         . What the maintainer will do with your changes and under what license
         they will be distributed is negotiable. If there has been no
         negotiation then this license, or a later version, also applies to your
         changes. The current maintainer is Bram Moolenaar <Bram@vim.org>. If
         this changes it will be announced in appropriate places (most likely
         vim.sf.net, www.vim.org and/or comp.editors). When it is completely
         impossible to contact the maintainer, the obligation to send him your
         changes ceases. Once the maintainer has confirmed that he has received
         your changes they will not have to be sent again.

         (( b) ))??
         If you have received a modified
         //** Vim **//
         __5__
         that was distributed as mentioned under a) you are allowed to further
         distribute it unmodified, as mentioned at I). If you make additional
         changes the text under a) applies to those changes.

         (( c) ))??
         Provide all the changes, including source code, with every copy of the
         modified
         //** Vim **//
         __5__
         you distribute. This may be done in the form of a context diff. You can
         choose what license to use for new code you add. The changes and their
         license must not restrict others from making their own changes to the
         official version of
         //** Vim **//
         __5__

         (( d) ))??
         When you have a modified
         //** Vim **//
         __5__
         which includes changes as mentioned under c), you can distribute it
         without the source code for the changes if the following three
         conditions are met:

            -
            The license that applies to the changes permits you to distribute
            the changes to the Vim maintainer without fee or restriction, and
            permits the Vim maintainer to include the changes in the official
            version of
            //** Vim **//
            __5__
            without fee or restriction.

            -
            You keep the changes for at least three years after last
            distributing the corresponding modified
            //** Vim **//
            __5__
            . When the maintainer or someone who you distributed the modified
            //** Vim **//
            __5__
            to asks you (in any way) for the changes within this period, you
            must make them available to him.

            -
            You clearly describe in the distribution how to contact you. This
            contact information must remain valid for at least three years after
            last distributing the corresponding modified
            //** Vim **//
            __5__
            , or as long as possible.

         (( e) ))??
         When the GNU General Public License (GPL) applies to the changes, you
         can distribute the modified
         //** Vim **//
         __5__
         under the GNU GPL version 2 or any later version.

      (( 3) ))??
      A message must be added, at least in the output of the ":version" command
      and in the intro screen, such that the user of the modified
      //** Vim **//
      __5__
      is able to see that it was modified. When distributing as mentioned under
      2)e) adding the message is only required for as far as this does not
      conflict with the license used for the changes.

      (( 4) ))??
      The contact information as required under 2)a) and 2)d) must not be
      removed or changed, except that the person himself can make corrections.

   (( III) ))??
   If you distribute a modified version of
   //** Vim **//
   __5__
   , you are encouraged to use the Vim license for your changes and make them
   available to the maintainer, including the source code. The preferred way to
   do this is by e-mail or by uploading the files to a server and e-mailing the
   URL. If the number of changes is small (e.g., a modified Makefile) e-mailing
   a context diff will do. The e-mail address to be used is
   <maintainer@vim.org>

   (( IV) ))??
   It is not allowed to remove this license from the distribution of the
   //** Vim **//
   __5__
   sources, parts of it or from a modified version. You may use this license for
   previous
   //** Vim **//
   __5__
   releases instead of the license that they came with, at your option.
`
const license_W3C_lre = `//**
W3C Software Notice and License (2002-12-31)
https://spdx.org/licenses/W3C.json
http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231.html
https://opensource.org/licenses/W3C
**//

(( W3C SOFTWARE NOTICE AND LICENSE ))??

This work (and included software, documentation such as READMEs, or other
related items) is being provided by the copyright holders under the following
license.

License

By obtaining, using and/or copying this work, you (the licensee) agree that you
have read, understood, and will comply with the following terms and conditions.

Permission to copy, modify, and distribute this software and its documentation,
with or without modification, for any purpose and without fee or royalty is
hereby granted, provided that you include the following on ALL copies of the
software and documentation or portions thereof, including modifications:

The full text of this NOTICE in a location viewable to users of the
redistributed or derivative work.

Any pre-existing intellectual property disclaimers, notices, or terms and
conditions. If none exist, the W3C Software Short Notice should be included
(hypertext is preferred, text is permitted) within the body of any redistributed
or derivative code.

Notice of any changes or modifications to the files, including the date changes
were made. (We recommend you provide URIs to the location from which the code is
derived.)

Disclaimers

THIS SOFTWARE AND DOCUMENTATION IS PROVIDED "AS IS," AND COPYRIGHT HOLDERS MAKE
NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO, WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT
THE USE OF THE SOFTWARE OR DOCUMENTATION WILL NOT INFRINGE ANY THIRD PARTY
PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS.

COPYRIGHT HOLDERS WILL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL OR
CONSEQUENTIAL DAMAGES ARISING OUT OF ANY USE OF THE SOFTWARE OR DOCUMENTATION.

The name and trademarks of copyright holders may NOT be used in advertising or
publicity pertaining to the software without specific, written prior permission.
Title to copyright in this software and any associated documentation will at all
times remain with copyright holders.
(( Notes

This version:
http:/www.w3.org/Consortium/Legal/2002/copyright-software-20021231

This formulation of W3C's notice and license became active on December 31 2002.
This version removes the copyright ownership notice such that this license can
be used with materials other than those owned by the W3C, reflects that ERCIM is
now a host of the W3C, includes references to this specific dated version of the
license, and removes the ambiguous grant of "use". Otherwise, this version is
the same as the previous version and is written so as to preserve the Free
Software Foundation's assessment of GPL compatibility and OSI's certification
under the Open Source Definition. ))??
`
const license_W3C_19980720_lre = `//**
W3C Software Notice and License (1998-07-20)
https://spdx.org/licenses/W3C-19980720.json
http://www.w3.org/Consortium/Legal/copyright-software-19980720.html
**//

(( W3C® SOFTWARE NOTICE AND LICENSE
(( Copyright __30__ ))??
))??

This W3C work (including software, documents, or other related items) is being
provided by the copyright holders under the following license. By obtaining,
using and/or copying this work, you (the licensee) agree that you have read,
understood, and will comply with the following terms and conditions:

Permission to use, copy, modify, and distribute this software and its
documentation, with or without modification, for any purpose and without fee or
royalty is hereby granted, provided that you include the following on ALL copies
of the software and documentation or portions thereof, including modifications,
that you make:

   (( 1. ))??
   The full text of this NOTICE in a location viewable to users of the
   redistributed or derivative work.

   (( 2. ))??
   Any pre-existing intellectual property disclaimers, notices, or terms and
   conditions. If none exist, a short notice of the following form (hypertext is
   preferred, text is permitted) should be used within the body of any
   redistributed or derivative code: "Copyright © [$date-of-software] World
   Wide Web Consortium, (Massachusetts Institute of Technology, Institut
   National de Recherche en Informatique et en Automatique, Keio University).
   All Rights Reserved. http:/www.w3.org/Consortium/Legal/"

   (( 3. ))??
   Notice of any changes or modifications to the W3C files, including the date
   changes were made. (We recommend you provide URIs to the location from which
   the code is derived.)

THIS SOFTWARE AND DOCUMENTATION IS PROVIDED "AS IS," AND COPYRIGHT HOLDERS MAKE
NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO, WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT
THE USE OF THE SOFTWARE OR DOCUMENTATION WILL NOT INFRINGE ANY THIRD PARTY
PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS.

COPYRIGHT HOLDERS WILL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL OR
CONSEQUENTIAL DAMAGES ARISING OUT OF ANY USE OF THE SOFTWARE OR DOCUMENTATION.

The name and trademarks of copyright holders may NOT be used in advertising or
publicity pertaining to the software without specific, written prior permission.
Title to copyright in this software and any associated documentation will at all
times remain with copyright holders.
(( _

This formulation of W3C's notice and license became active on August 14 1998 so
as to improve compatibility with GPL. This version ensures that W3C software
licensing terms are no more restrictive than GPL and consequently W3C software
may be distributed in GPL packages. See the older formulation for the policy
prior to this date. Please see our Copyright FAQ for common questions about
using materials from our site, including specific terms and conditions for
packages like libwww, Amaya, and Jigsaw. Other questions about this notice can
be directed to site-policy@w3.org. ))??
`
const license_W3C_20150513_lre = `//**
W3C Software Notice and Document License (2015-05-13)
https://spdx.org/licenses/W3C-20150513.json
https://www.w3.org/Consortium/Legal/2015/copyright-software-and-document
**//

This work is being provided by the copyright holders under the following
license.

License

By obtaining and/or copying this work, you (the licensee) agree that you have
read, understood, and will comply with the following terms and conditions.

Permission to copy, modify, and distribute this work, with or without
modification, for any purpose and without fee or royalty is hereby granted,
provided that you include the following on ALL copies of the work or portions
thereof, including modifications:

   •
   The full text of this NOTICE in a location viewable to users of the
   redistributed or derivative work.

   •
   Any pre-existing intellectual property disclaimers, notices, or terms and
   conditions. If none exist, the W3C Software and Document Short Notice should
   be included.

   •
   Notice of any changes or modifications, through a copyright statement on the
   new code or document such as "This software or document includes material
   copied from or derived from [title and URI of the W3C document]. Copyright
   (c) [YEAR] W3C® (MIT, ERCIM, Keio, Beihang)."

Disclaimers

THIS WORK IS PROVIDED "AS IS," AND COPYRIGHT HOLDERS MAKE NO REPRESENTATIONS OR
WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO, WARRANTIES OF
MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF THE
SOFTWARE OR DOCUMENT WILL NOT INFRINGE ANY THIRD PARTY PATENTS, COPYRIGHTS,
TRADEMARKS OR OTHER RIGHTS.

COPYRIGHT HOLDERS WILL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL OR
CONSEQUENTIAL DAMAGES ARISING OUT OF ANY USE OF THE SOFTWARE OR DOCUMENT.

The name and trademarks of copyright holders may NOT be used in advertising or
publicity pertaining to the work without specific, written prior permission.
Title to copyright in this work will at all times remain with copyright holders.
`
const license_WTFPL_lre = `//**
Do What The F*ck You Want To Public License
https://spdx.org/licenses/WTFPL.json
http://www.wtfpl.net/about/
http://sam.zoy.org/wtfpl/COPYING
**//


((
	DO WHAT THE
	((FUCK || F*** || F*CK))
	YOU WANT TO PUBLIC LICENSE

	Version 2, December 2004

	//** Many people put their own name here! **//
	((Copyright __20__))??

	((
		Everyone is permitted to copy and distribute verbatim or modified copies of this
		license document, and changing it is allowed as long as the name is changed.
	))??
))??

DO WHAT THE
((FUCK || F*** || F*CK))
YOU WANT TO PUBLIC LICENSE

TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

   (( 0. ))??
   You just DO WHAT THE
   ((FUCK || F*** || F*CK))
   YOU WANT TO.
`
const license_Watcom_1_0_lre = `//**
Sybase Open Watcom Public License 1.0
https://spdx.org/licenses/Watcom-1.0.json
https://opensource.org/licenses/Watcom-1.0
**//

(( Sybase Open Watcom Public License version 1.0 ))??

USE OF THE SYBASE OPEN WATCOM SOFTWARE DESCRIBED BELOW ("SOFTWARE") IS SUBJECT
TO THE TERMS AND CONDITIONS SET FORTH IN THE SYBASE OPEN WATCOM PUBLIC LICENSE
SET FORTH BELOW ("LICENSE"). YOU MAY NOT USE THE SOFTWARE IN ANY MANNER UNLESS
YOU ACCEPT THE TERMS AND CONDITIONS OF THE LICENSE. YOU INDICATE YOUR ACCEPTANCE
BY IN ANY MANNER USING (INCLUDING WITHOUT LIMITATION BY REPRODUCING, MODIFYING
OR DISTRIBUTING) THE SOFTWARE. IF YOU DO NOT ACCEPT ALL OF THE TERMS AND
CONDITIONS OF THE LICENSE, DO NOT USE THE SOFTWARE IN ANY MANNER.

Sybase Open Watcom Public License version 1.0

   (( 1. ))??
   General; Definitions. This License applies only to the following software
   programs: the open source versions of Sybase's Watcom C/C++ and Fortran
   compiler products ("Software"), which are modified versions of, with
   significant changes from, the last versions made commercially available by
   Sybase. As used in this License:

      (( 1.1 ))??
      "Applicable Patent Rights" mean: (a) in the case where Sybase is the
      grantor of rights, (i) claims of patents that are now or hereafter
      acquired, owned by or assigned to Sybase and (ii) that cover subject
      matter contained in the Original Code, but only to the extent necessary to
      use, reproduce and/or distribute the Original Code without infringement;
      and (b) in the case where You are the grantor of rights, (i) claims of
      patents that are now or hereafter acquired, owned by or assigned to You
      and (ii) that cover subject matter in Your Modifications, taken alone or
      in combination with Original Code.

      (( 1.2 ))??
      "Contributor" means any person or entity that creates or contributes to
      the creation of Modifications.

      (( 1.3 ))??
      "Covered Code" means the Original Code, Modifications, the combination of
      Original Code and any Modifications, and/or any respective portions
      thereof.

      (( 1.4 ))??
      "Deploy" means to use, sublicense or distribute Covered Code other than
      for Your internal research and development (R&D) and/or Personal Use, and
      includes without limitation, any and all internal use or distribution of
      Covered Code within Your business or organization except for R&D use
      and/or Personal Use, as well as direct or indirect sublicensing or
      distribution of Covered Code by You to any third party in any form or
      manner.

      (( 1.5 ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.6 ))??
      "Modifications" mean any addition to, deletion from, and/or change to, the
      substance and/or structure of the Original Code, any previous
      Modifications, the combination of Original Code and any previous
      Modifications, and/or any respective portions thereof. When code is
      released as a series of files, a Modification is: (a) any addition to or
      deletion from the contents of a file containing Covered Code; and/or (b)
      any new file or other representation of computer program statements that
      contains any part of Covered Code.

      (( 1.7 ))??
      "Original Code" means (a) the Source Code of a program or other work as
      originally made available by Sybase under this License, including the
      Source Code of any updates or upgrades to such programs or works made
      available by Sybase under this License, and that has been expressly
      identified by Sybase as such in the header file(s) of such work; and (b)
      the object code compiled from such Source Code and originally made
      available by Sybase under this License.

      (( 1.8 ))??
      "Personal Use" means use of Covered Code by an individual solely for his
      or her personal, private and non-commercial purposes. An individual's use
      of Covered Code in his or her capacity as an officer, employee, member,
      independent contractor or agent of a corporation, business or organization
      (commercial or non-commercial) does not qualify as Personal Use.

      (( 1.9 ))??
      "Source Code" means the human readable form of a program or other work
      that is suitable for making modifications to it, including all modules it
      contains, plus any associated interface definition files, scripts used to
      control compilation and installation of an executable (object code).

      (( 1.10 ))??
      "You" or "Your" means an individual or a legal entity exercising rights
      under this License. For legal entities, "You" or "Your" includes any
      entity which controls, is controlled by, or is under common control with,
      You, where "control" means (a) the power, direct or indirect, to cause the
      direction or management of such entity, whether by contract or otherwise,
      or (b) ownership of fifty percent (50%) or more of the outstanding shares
      or beneficial ownership of such entity.

   (( 2. ))??
   Permitted Uses; Conditions & Restrictions.Subject to the terms and conditions
   of this License, Sybase hereby grants You, effective on the date You accept
   this License and download the Original Code, a world-wide, royalty-free,
   non-exclusive license, to the extent of Sybase's Applicable Patent Rights and
   copyrights covering the Original Code, to do the following:

      (( 2.1 ))??
      You may use, reproduce, display, perform, modify and distribute Original
      Code, with or without Modifications, solely for Your internal research and
      development and/or Personal Use, provided that in each instance:

         (( (a) ))??
         You must retain and reproduce in all copies of Original Code the
         copyright and other proprietary notices and disclaimers of Sybase as
         they appear in the Original Code, and keep intact all notices in the
         Original Code that refer to this License; and

         (( (b) ))??
         You must retain and reproduce a copy of this License with every copy of
         Source Code of Covered Code and documentation You distribute, and You
         may not offer or impose any terms on such Source Code that alter or
         restrict this License or the recipients' rights hereunder, except as
         permitted under Section 6.

         (( c ))??
         Whenever reasonably feasible you should include the copy of this
         License in a click-wrap format, which requires affirmative acceptance
         by clicking on an "I accept" button or similar mechanism. If a
         click-wrap format is not included, you must include a statement that
         any use (including without limitation reproduction, modification or
         distribution) of the Software, and any other affirmative act that you
         define, constitutes acceptance of the License, and instructing the user
         not to use the Covered Code in any manner if the user does not accept
         all of the terms and conditions of the License.

      (( 2.2 ))??
      You may use, reproduce, display, perform, modify and Deploy Covered Code,
      provided that in each instance:

         (( (a) ))??
         You must satisfy all the conditions of Section 2.1 with respect to the
         Source Code of the Covered Code;

         (( (b) ))??
         You must duplicate, to the extent it does not already exist, the notice
         in Exhibit A in each file of the Source Code of all Your Modifications,
         and cause the modified files to carry prominent notices stating that
         You changed the files and the date of any change;

         (( c ))??
         You must make Source Code of all Your Deployed Modifications publicly
         available under the terms of this License, including the license grants
         set forth in Section 3 below, for as long as you Deploy the Covered
         Code or twelve (12) months from the date of initial Deployment,
         whichever is longer. You should preferably distribute the Source Code
         of Your Deployed Modifications electronically (e.g. download from a web
         site);

         (( (d) ))??
         if You Deploy Covered Code in object code, executable form only, You
         must include a prominent notice, in the code itself as well as in
         related documentation, stating that Source Code of the Covered Code is
         available under the terms of this License with information on how and
         where to obtain such Source Code; and

         (( (e) ))??
         the object code form of the Covered Code may be distributed under Your
         own license agreement, provided that such license agreement contains
         terms no less protective of Sybase and each Contributor than the terms
         of this License, and stating that any provisions which differ from this
         License are offered by You alone and not by any other party.

      (( 2.3 ))??
      You expressly acknowledge and agree that although Sybase and each
      Contributor grants the licenses to their respective portions of the
      Covered Code set forth herein, no assurances are provided by Sybase or any
      Contributor that the Covered Code does not infringe the patent or other
      intellectual property rights of any other entity. Sybase and each
      Contributor disclaim any liability to You for claims brought by any other
      entity based on infringement of intellectual property rights or otherwise.
      As a condition to exercising the rights and licenses granted hereunder,
      You hereby assume sole responsibility to secure any other intellectual
      property rights needed, if any. For example, if a third party patent
      license is required to allow You to distribute the Covered Code, it is
      Your responsibility to acquire that license before distributing the
      Covered Code.

   (( 3. ))??
   Your Grants. In consideration of, and as a condition to, the licenses granted
   to You under this License, You hereby grant to Sybase and all third parties a
   non-exclusive, royalty-free license, under Your Applicable Patent Rights and
   other intellectual property rights (other than patent) owned or controlled by
   You, to use, reproduce, display, perform, modify, distribute and Deploy Your
   Modifications of the same scope and extent as Sybase's licenses under
   Sections 2.1 and 2.2.

   (( 4. ))??
   Larger Works. You may create a Larger Work by combining Covered Code with
   other code not governed by the terms of this License and distribute the
   Larger Work as a single product. In each such instance, You must make sure
   the requirements of this License are fulfilled for the Covered Code or any
   portion thereof.

   (( 5. ))??
   Limitations on Patent License. Except as expressly stated in Section 2, no
   other patent rights, express or implied, are granted by Sybase herein.
   Modifications and/or Larger Works may require additional patent licenses from
   Sybase which Sybase may grant in its sole discretion.

   (( 6. ))??
   Additional Terms. You may choose to offer, and to charge a fee for, warranty,
   support, indemnity or liability obligations and/or other rights consistent
   with this License ("Additional Terms") to one or more recipients of Covered
   Code. However, You may do so only on Your own behalf and as Your sole
   responsibility, and not on behalf of Sybase or any Contributor. You must
   obtain the recipient's agreement that any such Additional Terms are offered
   by You alone, and You hereby agree to indemnify, defend and hold Sybase and
   every Contributor harmless for any liability incurred by or claims asserted
   against Sybase or such Contributor by reason of any such Additional Terms.

   (( 7. ))??
   Versions of the License. Sybase may publish revised and/or new versions of
   this License from time to time. Each version will be given a distinguishing
   version number. Once Original Code has been published under a particular
   version of this License, You may continue to use it under the terms of that
   version. You may also choose to use such Original Code under the terms of any
   subsequent version of this License published by Sybase. No one other than
   Sybase has the right to modify the terms applicable to Covered Code created
   under this License.

   (( 8. ))??
   NO WARRANTY OR SUPPORT. The Covered Code may contain in whole or in part
   pre-release, untested, or not fully tested works. The Covered Code may
   contain errors that could cause failures or loss of data, and may be
   incomplete or contain inaccuracies. You expressly acknowledge and agree that
   use of the Covered Code, or any portion thereof, is at Your sole and entire
   risk. THE COVERED CODE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, UPGRADES OR
   SUPPORT OF ANY KIND AND SYBASE AND SYBASE'S LICENSOR(S) (COLLECTIVELY
   REFERRED TO AS "SYBASE" FOR THE PURPOSES OF SECTIONS 8 AND 9) AND ALL
   CONTRIBUTORS EXPRESSLY DISCLAIM ALL WARRANTIES AND/OR CONDITIONS, EXPRESS OR
   IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES AND/OR
   CONDITIONS OF MERCHANTABILITY, OF SATISFACTORY QUALITY, OF FITNESS FOR A
   PARTICULAR PURPOSE, OF ACCURACY, OF QUIET ENJOYMENT, AND NONINFRINGEMENT OF
   THIRD PARTY RIGHTS. SYBASE AND EACH CONTRIBUTOR DOES NOT WARRANT AGAINST
   INTERFERENCE WITH YOUR ENJOYMENT OF THE COVERED CODE, THAT THE FUNCTIONS
   CONTAINED IN THE COVERED CODE WILL MEET YOUR REQUIREMENTS, THAT THE OPERATION
   OF THE COVERED CODE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT DEFECTS IN
   THE COVERED CODE WILL BE CORRECTED. NO ORAL OR WRITTEN INFORMATION OR ADVICE
   GIVEN BY SYBASE, A SYBASE AUTHORIZED REPRESENTATIVE OR ANY CONTRIBUTOR SHALL
   CREATE A WARRANTY. You acknowledge that the Covered Code is not intended for
   use in the operation of nuclear facilities, aircraft navigation,
   communication systems, or air traffic control machines in which case the
   failure of the Covered Code could lead to death, personal injury, or severe
   physical or environmental damage.

   (( 9. ))??
   LIMITATION OF LIABILITY. TO THE EXTENT NOT PROHIBITED BY LAW, IN NO EVENT
   SHALL SYBASE OR ANY CONTRIBUTOR BE LIABLE FOR ANY DIRECT, INCIDENTAL,
   SPECIAL, INDIRECT, CONSEQUENTIAL OR OTHER DAMAGES OF ANY KIND ARISING OUT OF
   OR RELATING TO THIS LICENSE OR YOUR USE OR INABILITY TO USE THE COVERED CODE,
   OR ANY PORTION THEREOF, WHETHER UNDER A THEORY OF CONTRACT, WARRANTY, TORT
   (INCLUDING NEGLIGENCE), PRODUCTS LIABILITY OR OTHERWISE, EVEN IF SYBASE OR
   SUCH CONTRIBUTOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, AND
   NOTWITHSTANDING THE FAILURE OF ESSENTIAL PURPOSE OF ANY REMEDY. SOME
   JURISDICTIONS DO NOT ALLOW THE LIMITATION OF LIABILITY OF INCIDENTAL OR
   CONSEQUENTIAL OR OTHER DAMAGES OF ANY KIND, SO THIS LIMITATION MAY NOT APPLY
   TO YOU. In no event shall Sybase's or any Contributor's total liability to
   You for all damages (other than as may be required by applicable law) under
   this License exceed the amount of five hundred dollars ($500.00).

   (( 10. ))??
   Trademarks. This License does not grant any rights to use the trademarks or
   trade names "Sybase" or any other trademarks or trade names belonging to
   Sybase (collectively "Sybase Marks") or to any trademark or trade name
   belonging to any Contributor("Contributor Marks"). No Sybase Marks or
   Contributor Marks may be used to endorse or promote products derived from the
   Original Code or Covered Code other than with the prior written consent of
   Sybase or the Contributor, as applicable.

   (( 11. ))??
   Ownership. Subject to the licenses granted under this License, each
   Contributor retains all rights, title and interest in and to any
   Modifications made by such Contributor. Sybase retains all rights, title and
   interest in and to the Original Code and any Modifications made by or on
   behalf of Sybase ("Sybase Modifications"), and such Sybase Modifications will
   not be automatically subject to this License. Sybase may, at its sole
   discretion, choose to license such Sybase Modifications under this License,
   or on different terms from those contained in this License or may choose not
   to license them at all.

   (( 12. ))??
   Termination.

      (( 12.1 ))??
      Termination. This License and the rights granted hereunder will
      terminate:

         (( (a) ))??
         automatically without notice if You fail to comply with any term(s) of
         this License and fail to cure such breach within 30 days of becoming
         aware of such breach;

         (( (b) ))??
         immediately in the event of the circumstances described in Section
         13.5(b); or

         (( c ))??
         automatically without notice if You, at any time during the term of
         this License, commence an action for patent infringement (including as
         a cross claim or counterclaim) against Sybase or any Contributor.

      (( 12.2 ))??
      Effect of Termination. Upon termination, You agree to immediately stop any
      further use, reproduction, modification, sublicensing and distribution of
      the Covered Code and to destroy all copies of the Covered Code that are in
      your possession or control. All sublicenses to the Covered Code that have
      been properly granted prior to termination shall survive any termination
      of this License. Provisions which, by their nature, should remain in
      effect beyond the termination of this License shall survive, including but
      not limited to Sections 3, 5, 8, 9, 10, 11, 12.2 and 13. No party will be
      liable to any other for compensation, indemnity or damages of any sort
      solely as a result of terminating this License in accordance with its
      terms, and termination of this License will be without prejudice to any
      other right or remedy of any party.

   (( 13. ))??
   Miscellaneous.

      (( 13.1 ))??
      Government End Users. The Covered Code is a "commercial item" as defined
      in FAR 2.101. Government software and technical data rights in the Covered
      Code include only those rights customarily provided to the public as
      defined in this License. This customary commercial license in technical
      data and software is provided in accordance with FAR 12.211 (Technical
      Data) and 12.212 (Computer Software) and, for Department of Defense
      purchases, DFAR 252.227-7015 (Technical Data -- Commercial Items) and
      227.7202-3 (Rights in Commercial Computer Software or Computer Software
      Documentation). Accordingly, all U.S. Government End Users acquire Covered
      Code with only those rights set forth herein.

      (( 13.2 ))??
      Relationship of Parties. This License will not be construed as creating an
      agency, partnership, joint venture or any other form of legal association
      between or among you, Sybase or any Contributor, and You will not
      represent to the contrary, whether expressly, by implication, appearance
      or otherwise.

      (( 13.3 ))??
      Independent Development. Nothing in this License will impair Sybase's or
      any Contributor's right to acquire, license, develop, have others develop
      for it, market and/or distribute technology or products that perform the
      same or similar functions as, or otherwise compete with, Modifications,
      Larger Works, technology or products that You may develop, produce, market
      or distribute.

      (( 13.4 ))??
      Waiver; Construction. Failure by Sybase or any Contributor to enforce any
      provision of this License will not be deemed a waiver of future
      enforcement of that or any other provision. Any law or regulation which
      provides that the language of a contract shall be construed against the
      drafter will not apply to this License.

      (( 13.5 ))??
      Severability. (a) If for any reason a court of competent jurisdiction
      finds any provision of this License, or portion thereof, to be
      unenforceable, that provision of the License will be enforced to the
      maximum extent permissible so as to effect the economic benefits and
      intent of the parties, and the remainder of this License will continue in
      full force and effect. (b) Notwithstanding the foregoing, if applicable
      law prohibits or restricts You from fully and/or specifically complying
      with Sections 2 and/or 3 or prevents the enforceability of either of those
      Sections, this License will immediately terminate and You must immediately
      discontinue any use of the Covered Code and destroy all copies of it that
      are in your possession or control.

      (( 13.6 ))??
      Dispute Resolution. Any litigation or other dispute resolution between You
      and Sybase relating to this License shall take place in the Northern
      District of California, and You and Sybase hereby consent to the personal
      jurisdiction of, and venue in, the state and federal courts within that
      District with respect to this License. The application of the United
      Nations Convention on Contracts for the International Sale of Goods is
      expressly excluded.

      (( 13.7 ))??
      Entire Agreement; Governing Law. This License constitutes the entire
      agreement between the parties with respect to the subject matter hereof.
      This License shall be governed by the laws of the United States and the
      State of California, except that body of California law concerning
      conflicts of law.

Where You are located in the province of Quebec, Canada, the following clause
applies: The parties hereby confirm that they have requested that this License
and all related documents be drafted in English. Les parties ont exigè que le
prèsent contrat et tous les documents connexes soient rèdiès en anglais.
(( EXHIBIT A.

"Portions Copyright (c) 1983-2002 Sybase, Inc. All Rights Reserved.

This file contains Original Code and/or Modifications of Original Code as
defined in and that are subject to the Sybase Open Watcom Public License version
1.0 (the 'License'). You may not use this file except in compliance with the
License. BY USING THIS FILE YOU AGREE TO ALL TERMS AND CONDITIONS OF THE
LICENSE. A copy of the License is provided with the Original Code and
Modifications, and is also available at www.sybase.com/developer/opensource.

The Original Code and all software distributed under the License are distributed
on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
AND SYBASE AND ALL CONTRIBUTORS HEREBY DISCLAIM ALL SUCH WARRANTIES, INCLUDING
WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see the License for the
specific language governing rights and limitations under the License." ))??
`
const license_Wsuipa_lre = `//**
Wsuipa License
https://spdx.org/licenses/Wsuipa.json
https://fedoraproject.org/wiki/Licensing/Wsuipa
**//

(( This file was added by Clea F. Rees on 2008/11/30 with the permission of Dean
Guenther and pointers to this file were added to all source files. ))??

Unlimited copying and redistribution of each of the files is permitted as long
as the file is not modified. Modifications, and redistribution of modified
versions, are also permitted, but only if the resulting file is renamed.
The copyright holder is Washington State University. The original author of the
fonts is Janene Winter. The primary contact
(( (as of 2008) is Dean Guenther. ))??
`
const license_X11_lre = `//**
X11 License
https://spdx.org/licenses/X11.json
http://www.xfree86.org/3.3.6/COPYRIGHT2.html#3
**//

(( X11 License
(( Copyright __20__ ))??
))??

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE X CONSORTIUM
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the X Consortium shall not be
used in advertising or otherwise to promote the sale, use or other dealings in
this Software without prior written authorization from the X Consortium.

X Window System is a trademark of X Consortium, Inc.
`
const license_XFree86_1_1_lre = `//**
XFree86 License 1.1
https://spdx.org/licenses/XFree86-1.1.json
http://www.xfree86.org/current/LICENSE4.html
**//

(( XFree86 License (version 1.1)
(( Copyright __20__ ))??
))??

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

   (( 1. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions, and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution, and in the same place
   and form as other copyright, license and disclaimer information.

   (( 3. ))??
   The end-user documentation included with the redistribution, if any, must
   include the following acknowledgment: "This product includes software
   developed by The XFree86 Project, Inc (http:/www.xfree86.org/) and its
   contributors", in the same place and form as other third-party
   acknowledgments. Alternately, this acknowledgment may appear in the software
   itself, in the same form and location as other such third-party
   acknowledgments.

   (( 4. ))??
   Except as contained in this notice, the name of The XFree86 Project, Inc
   shall not be used in advertising or otherwise to promote the sale, use or
   other dealings in this Software without prior written authorization from The
   XFree86 Project, Inc.

THIS SOFTWARE IS PROVIDED ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE XFREE86
PROJECT, INC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
`
const license_XSkat_lre = `//**
XSkat License
https://spdx.org/licenses/XSkat.json
https://fedoraproject.org/wiki/Licensing/XSkat_License
**//

This program is free software; you can redistribute it freely.

Use it at your own risk; there is NO WARRANTY.

Redistribution of modified versions is permitted provided that the following
conditions are met:

   (( 1. ))??
   All copyright & permission notices are preserved.

   (( 2.a) ))??
   Only changes required for packaging or porting are made.

   or

   (( 2.b) ))??
   It is clearly stated who last changed the program. The program is renamed or
   the version number is of the form x.y.z, where x.y is the version of the
   original program and z is an arbitrary suffix.
`
const license_Xerox_lre = `//**
Xerox License
https://spdx.org/licenses/Xerox.json
https://fedoraproject.org/wiki/Licensing/Xerox
**//

//** Copyright **//

Use and copying of this software and preparation of derivative works based upon
this software are permitted. Any copy of this software or of any derivative work
must include the above copyright notice of Xerox Corporation, this paragraph and
the one after it. Any distribution of this software or derivative works must
comply with all applicable United States export control laws.

This software is made available AS IS, and XEROX CORPORATION DISCLAIMS ALL
WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND
NOTWITHSTANDING ANY OTHER PROVISION CONTAINED HEREIN, ANY LIABILITY FOR DAMAGES
RESULTING FROM THE SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, WHETHER ARISING
IN CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, EVEN IF XEROX
CORPORATION IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
`
const license_Xnet_lre = `//**
X.Net License
https://spdx.org/licenses/Xnet.json
https://opensource.org/licenses/Xnet
**//

(( The X.Net, Inc. License
(( Copyright __20__ ))??
))??

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

This agreement shall be governed in all respects by the laws of the State of
California and by the laws of the United States of America.
`
const license_YPL_1_0_lre = `//**
Yahoo! Public License v1.0
https://spdx.org/licenses/YPL-1.0.json
http://www.zimbra.com/license/yahoo_public_license_1.0.html
**//

(( Yahoo! Public License, Version 1.0 (YPL) ))??

This Yahoo! Public License (this "Agreement") is a legal agreement that
describes the terms under which Yahoo! Inc., a Delaware corporation having its
principal place of business at 701 First Avenue, Sunnyvale, California 94089
("Yahoo!") will provide software to you via download or otherwise ("Software").
By using the Software, you, an individual or an entity ("You") agree to the
terms of this Agreement.

In consideration of the mutual promises and upon the terms and conditions set
forth below, the parties agree as follows:

   (( 1. ))??
   Grant of Copyright License

      (( 1.1 ))??
      - Subject to the terms and conditions of this Agreement, Yahoo! hereby
      grants to You, under any and all of its copyright interest in and to the
      Software, a royalty-free, non-exclusive, non-transferable license to copy,
      modify, compile, execute, and distribute the Software and Modifications.
      For the purposes of this Agreement, any change to, addition to, or
      abridgement of the Software made by You is a "Modification;" however, any
      file You add to the Software that does not contain any part of the
      Software is not a "Modification."

      (( 1.2 ))??
      - If You are an individual acting on behalf of a corporation or other
      entity, Your use of the Software or any Modification is subject to Your
      having the authority to bind such corporation or entity to this Agreement.
      Providing copies to persons within such corporation or entity is not
      considered distribution for purposes of this Agreement.

      (( 1.3 ))??
      - For the Software or any Modification You distribute in source code
      format, You must do so only under the terms of this Agreement, and You
      must include a complete copy of this Agreement with Your distribution.
      With respect to any Modification You distribute in source code format, the
      terms of this Agreement will apply to You in the same way those terms
      apply to Yahoo! with respect to the Software. In other words, when You are
      distributing Modifications under this Agreement, You "stand in the shoes"
      of Yahoo! in terms of the rights You grant and how the terms and
      conditions apply to You and the licensees of Your Modifications.
      Notwithstanding the foregoing, when You "stand in the shoes" of Yahoo!,
      You are not subject to the jurisdiction provision under Section 7, which
      requires all disputes under this Agreement to be subject to the
      jurisdiction of federal or state courts of northern California.

      (( 1.4 ))??
      - For the Software or any Modification You distribute in compiled or
      object code format, You must also provide recipients with access to the
      Software or Modification in source code format along with a complete copy
      of this Agreement. The distribution of the Software or Modifications in
      compiled or object code format may be under a license of Your choice,
      provided that You are in compliance with the terms of this Agreement. In
      addition, You must make absolutely clear that any license terms applying
      to such Software or Modification that differ from this Agreement are
      offered by You alone and not by Yahoo!, and that such license does not
      restrict recipients from exercising rights in the source code to the
      Software granted by Yahoo! under this Agreement or rights in the source
      code to any Modification granted by You as described in Section 1.3.

      (( 1.5 ))??
      - This Agreement does not limit Your right to distribute files that are
      entirely Your own work (i.e., which do not incorporate any portion of the
      Software and are not Modifications) under any terms You choose.

   (( 2. ))??
   Support

   Yahoo! has no obligation to provide technical support or updates to You.
   Nothing in this Agreement requires Yahoo! to enter into any license with You
   for any other edition of the Software.

   (( 3. ))??
   Intellectual Property Rights

      (( 3.1 ))??
      - Except for the license expressly granted under copyright in Section 1.1,
      no rights, licenses or forbearances are granted or may arise in relation
      to this Agreement whether expressly, by implication, exhaustion, estoppel
      or otherwise. All rights, including all intellectual property rights, that
      are not expressly granted under this Agreement are hereby reserved.

      (( 3.2 ))??
      - In any copy of the Software or in any Modification you create, You must
      retain and reproduce, any and all copyright, patent, trademark, and
      attribution notices that are included in the Software in the same form as
      they appear in the Software. This includes the preservation of attribution
      notices in the form of trademarks or logos that exist within a user
      interface of the Software.

      (( 3.3 ))??
      - This license does not grant You rights to use any party's name, logo, or
      trademarks, except solely as necessary to comply with Section 3.2.

   (( 4. ))??
   Disclaimer of Warranties

   THE SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND. YAHOO!
   MAKES NO WARRANTIES, WHETHER EXPRESS, IMPLIED, OR STATUTORY REGARDING OR
   RELATING TO THE SOFTWARE. SPECIFICALLY, YAHOO! DOES NOT WARRANT THAT THE
   SOFTWARE WILL BE ERROR FREE OR WILL PERFORM IN AN UNINTERRUPTED MANNER. TO
   THE GREATEST EXTENT ALLOWED BY LAW, YAHOO! SPECIFICALLY DISCLAIMS ALL IMPLIED
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE (EVEN IF
   YAHOO! HAD BEEN INFORMED OF SUCH PURPOSE), AND NONINFRINGEMENT WITH RESPECT
   TO THE SOFTWARE, ANY MODIFICATIONS THERETO AND WITH RESPECT TO THE USE OF THE
   FOREGOING.

   (( 5. ))??
   Limitation of Liability

   IN NO EVENT WILL YAHOO! BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES OF ANY KIND (INCLUDING WITHOUT
   LIMITATION LOSS OF PROFITS, LOSS OF USE, BUSINESS INTERRUPTION, LOSS OF DATA,
   COST OF COVER) IN CONNECTION WITH OR ARISING OUT OF OR RELATING TO THE
   FURNISHING, PERFORMANCE OR USE OF THE SOFTWARE OR ANY OTHER RIGHTS GRANTED
   HEREUNDER, WHETHER ALLEGED AS A BREACH OF CONTRACT OR TORTIOUS CONDUCT,
   INCLUDING NEGLIGENCE, AND EVEN IF YAHOO! HAS BEEN ADVISED OF THE POSSIBILITY
   OF SUCH DAMAGES.

   (( 6. ))??
   Term and Termination

      (( 6.1 ))??
      - This Agreement will continue in effect unless and until terminated
      earlier pursuant to this Section 6.

      (( 6.2 ))??
      - In the event Yahoo! determines that You have breached this Agreement,
      Yahoo! may terminate this Agreement.

      (( 6.3 ))??
      - All licenses granted hereunder shall terminate upon the termination of
      this Agreement. Termination will be in addition to any rights and remedies
      available to Yahoo! at law or equity or under this Agreement.

      (( 6.4 ))??
      - Termination of this Agreement will not affect the provisions regarding
      reservation of rights (Section 3.1), provisions disclaiming or limiting
      Yahoo!'s liability (Sections 4 and 5), Termination (Section 6) or
      Miscellaneous (Section 7), which provisions will survive termination of
      this Agreement.

   (( 7. ))??
   Miscellaneous

   This Agreement contains the entire agreement of the parties with respect to
   the subject matter of this Agreement and supersedes all previous
   communications, representations, understandings and agreements, either oral
   or written, between the parties with respect to said subject matter. The
   relationship of the parties hereunder is that of independent contractors, and
   this Agreement will not be construed as creating an agency, partnership,
   joint venture or any other form of legal association between the parties. If
   any term, condition, or provision in this Agreement is found to be invalid,
   unlawful or unenforceable to any extent, this Agreement will be construed in
   a manner that most closely effectuates the intent of this Agreement. Such
   invalid term, condition or provision will be severed from the remaining
   terms, conditions and provisions, which will continue to be valid and
   enforceable to the fullest extent permitted by law. This Agreement will be
   interpreted and construed in accordance with the laws of the State of
   California and the United States of America, without regard to conflict of
   law principles. The U.N. Convention on Contracts for the International Sale
   of Goods shall not apply to this Agreement. All disputes arising out of this
   Agreement involving Yahoo! or any of its subsidiaries shall be subject to the
   jurisdiction of the federal or state courts of northern California, with
   venue lying in Santa Clara County, California. No rights may be assigned, no
   obligations may be delegated, and this Agreement may not be transferred by
   You, in whole or in part, whether voluntary or by operation of law, including
   by way of sale of assets, merger or consolidation, without the prior written
   consent of Yahoo!, and any purported assignment, delegation or transfer
   without such consent shall be void ab initio. Any waiver of the provisions of
   this Agreement or of a party's rights or remedies under this Agreement must
   be in writing to be effective. Failure, neglect or delay by a party to
   enforce the provisions of this Agreement or its rights or remedies at any
   time, will not be construed or be deemed to be a waiver of such party's
   rights under this Agreement and will not in any way affect the validity of
   the whole or any part of this Agreement or prejudice such party's right to
   take subsequent action.
`
const license_YPL_1_1_lre = `//**
Yahoo! Public License v1.1
https://spdx.org/licenses/YPL-1.1.json
http://www.zimbra.com/license/yahoo_public_license_1.1.html
**//

(( Yahoo! Public License, Version 1.1 (YPL) ))??

This Yahoo! Public License (this "Agreement") is a legal agreement that
describes the terms under which Yahoo! Inc., a Delaware corporation having its
principal place of business at 701 First Avenue, Sunnyvale, California 94089
("Yahoo!") will provide software to you via download or otherwise ("Software").
By using the Software, you, an individual or an entity ("You") agree to the
terms of this Agreement.

In consideration of the mutual promises and upon the terms and conditions set
forth below, the parties agree as follows:

   (( 1. ))??
   Grant of Copyright License

      (( 1.1 ))??
      - Subject to the terms and conditions of this Agreement, Yahoo! hereby
      grants to You, under any and all of its copyright interest in and to the
      Software, a royalty-free, non-exclusive, non-transferable license to copy,
      modify, compile, execute, and distribute the Software and Modifications.
      For the purposes of this Agreement, any change to, addition to, or
      abridgement of the Software made by You is a "Modification;" however, any
      file You add to the Software that does not contain any part of the
      Software is not a "Modification."

      (( 1.2 ))??
      - If You are an individual acting on behalf of a corporation or other
      entity, Your use of the Software or any Modification is subject to Your
      having the authority to bind such corporation or entity to this Agreement.
      Providing copies to persons within such corporation or entity is not
      considered distribution for purposes of this Agreement.

      (( 1.3 ))??
      - For the Software or any Modification You distribute in source code
      format, You must do so only under the terms of this Agreement, and You
      must include a complete copy of this Agreement with Your distribution.
      With respect to any Modification You distribute in source code format, the
      terms of this Agreement will apply to You in the same way those terms
      apply to Yahoo! with respect to the Software. In other words, when You are
      distributing Modifications under this Agreement, You "stand in the shoes"
      of Yahoo! in terms of the rights You grant and how the terms and
      conditions apply to You and the licensees of Your Modifications.
      Notwithstanding the foregoing, when You "stand in the shoes" of Yahoo!,
      You are not subject to the jurisdiction provision under Section 7, which
      requires all disputes under this Agreement to be subject to the
      jurisdiction of federal or state courts of northern California.

      (( 1.4 ))??
      - For the Software or any Modification You distribute in compiled or
      object code format, You must also provide recipients with access to the
      Software or Modification in source code format along with a complete copy
      of this Agreement. The distribution of the Software or Modifications in
      compiled or object code format may be under a license of Your choice,
      provided that You are in compliance with the terms of this Agreement. In
      addition, You must make absolutely clear that any license terms applying
      to such Software or Modification that differ from this Agreement are
      offered by You alone and not by Yahoo!, and that such license does not
      restrict recipients from exercising rights in the source code to the
      Software granted by Yahoo! under this Agreement or rights in the source
      code to any Modification granted by You as described in Section 1.3.

      (( 1.5 ))??
      - This Agreement does not limit Your right to distribute files that are
      entirely Your own work (i.e., which do not incorporate any portion of the
      Software and are not Modifications) under any terms You choose.

   (( 2. ))??
   Support

   Yahoo! has no obligation to provide technical support or updates to You.
   Nothing in this Agreement requires Yahoo! to enter into any license with You
   for any other edition of the Software.

   (( 3. ))??
   Intellectual Property Rights

      (( 3.1 ))??
      - Except for the license expressly granted under copyright in Section 1.1,
      no rights, licenses or forbearances are granted or may arise in relation
      to this Agreement whether expressly, by implication, exhaustion, estoppel
      or otherwise. All rights, including all intellectual property rights, that
      are not expressly granted under this Agreement are hereby reserved.

      (( 3.2 ))??
      - In any copy of the Software or in any Modification you create, You must
      retain and reproduce, any and all copyright, patent, trademark, and
      attribution notices that are included in the Software in the same form as
      they appear in the Software. This includes the preservation of attribution
      notices in the form of trademarks or logos that exist within a user
      interface of the Software.

      (( 3.3 ))??
      - This license does not grant You rights to use any party's name, logo, or
      trademarks, except solely as necessary to comply with Section 3.2.

   (( 4. ))??
   Disclaimer of Warranties

   THE SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND. YAHOO!
   MAKES NO WARRANTIES, WHETHER EXPRESS, IMPLIED, OR STATUTORY REGARDING OR
   RELATING TO THE SOFTWARE. SPECIFICALLY, YAHOO! DOES NOT WARRANT THAT THE
   SOFTWARE WILL BE ERROR FREE OR WILL PERFORM IN AN UNINTERRUPTED MANNER. TO
   THE GREATEST EXTENT ALLOWED BY LAW, YAHOO! SPECIFICALLY DISCLAIMS ALL IMPLIED
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE (EVEN IF
   YAHOO! HAD BEEN INFORMED OF SUCH PURPOSE), AND NONINFRINGEMENT WITH RESPECT
   TO THE SOFTWARE, ANY MODIFICATIONS THERETO AND WITH RESPECT TO THE USE OF THE
   FOREGOING.

   (( 5. ))??
   Limitation of Liability

   IN NO EVENT WILL YAHOO! BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES OF ANY KIND (INCLUDING WITHOUT
   LIMITATION LOSS OF PROFITS, LOSS OF USE, BUSINESS INTERRUPTION, LOSS OF DATA,
   COST OF COVER) IN CONNECTION WITH OR ARISING OUT OF OR RELATING TO THE
   FURNISHING, PERFORMANCE OR USE OF THE SOFTWARE OR ANY OTHER RIGHTS GRANTED
   HEREUNDER, WHETHER ALLEGED AS A BREACH OF CONTRACT OR TORTIOUS CONDUCT,
   INCLUDING NEGLIGENCE, AND EVEN IF YAHOO! HAS BEEN ADVISED OF THE POSSIBILITY
   OF SUCH DAMAGES.

   (( 6. ))??
   Term and Termination

      (( 6.1 ))??
      - This Agreement will continue in effect unless and until terminated
      earlier pursuant to this Section 6.

      (( 6.2 ))??
      - In the event You violate the terms of this Agreement, Yahoo! may
      terminate this Agreement.

      (( 6.3 ))??
      - All licenses granted hereunder shall terminate upon the termination of
      this Agreement. Termination will be in addition to any rights and remedies
      available to Yahoo! at law or equity or under this Agreement.

      (( 6.4 ))??
      - Termination of this Agreement will not affect the provisions regarding
      reservation of rights (Section 3.1), provisions disclaiming or limiting
      Yahoo!'s liability (Sections 4 and 5), Termination (Section 6) or
      Miscellaneous (Section 7), which provisions will survive termination of
      this Agreement.

   (( 7. ))??
   Miscellaneous

   This Agreement contains the entire agreement of the parties with respect to
   the subject matter of this Agreement and supersedes all previous
   communications, representations, understandings and agreements, either oral
   or written, between the parties with respect to said subject matter. The
   relationship of the parties hereunder is that of independent contractors, and
   this Agreement will not be construed as creating an agency, partnership,
   joint venture or any other form of legal association between the parties. If
   any term, condition, or provision in this Agreement is found to be invalid,
   unlawful or unenforceable to any extent, this Agreement will be construed in
   a manner that most closely effectuates the intent of this Agreement. Such
   invalid term, condition or provision will be severed from the remaining
   terms, conditions and provisions, which will continue to be valid and
   enforceable to the fullest extent permitted by law. This Agreement will be
   interpreted and construed in accordance with the laws of the State of
   California and the United States of America, without regard to conflict of
   law principles. The U.N. Convention on Contracts for the International Sale
   of Goods shall not apply to this Agreement. All disputes arising out of this
   Agreement involving Yahoo! or any of its subsidiaries shall be subject to the
   jurisdiction of the federal or state courts of northern California, with
   venue lying in Santa Clara County, California. No rights may be assigned, no
   obligations may be delegated, and this Agreement may not be transferred by
   You, in whole or in part, whether voluntary or by operation of law, including
   by way of sale of assets, merger or consolidation, without the prior written
   consent of Yahoo!, and any purported assignment, delegation or transfer
   without such consent shall be void ab initio. Any waiver of the provisions of
   this Agreement or of a party's rights or remedies under this Agreement must
   be in writing to be effective. Failure, neglect or delay by a party to
   enforce the provisions of this Agreement or its rights or remedies at any
   time, will not be construed or be deemed to be a waiver of such party's
   rights under this Agreement and will not in any way affect the validity of
   the whole or any part of this Agreement or prejudice such party's right to
   take subsequent action.
`
const license_ZPL_1_1_lre = `//**
Zope Public License 1.1
https://spdx.org/licenses/ZPL-1.1.json
http://old.zope.org/Resources/License/ZPL-1.1
**//

(( Zope Public License (ZPL) Version 1.1
(( Copyright __20__ ))??
))??

(( This license has been certified as open source. ))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions in source code must retain the above copyright notice, this
   list of conditions, and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions, and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   All advertising materials and documentation mentioning features derived from
   or use of this software must display the following acknowledgement:

   "This product includes software developed by Zope Corporation for use in the
   Z Object Publishing Environment (http:/www.zope.com/)."

   In the event that the product being advertised includes an intact Zope
   distribution (with copyright and license included) then this clause is
   waived.

   (( 4. ))??
   Names associated with Zope or Zope Corporation must not be used to endorse or
   promote products derived from this software without prior written permission
   from Zope Corporation.

   (( 5. ))??
   Modified redistributions of any form whatsoever must retain the following
   acknowledgment:

   "This product includes software developed by Zope Corporation for use in the
   Z Object Publishing Environment (http:/www.zope.com/)."

   Intact (re-)distributions of any official Zope release do not require an
   external acknowledgement.

   (( 6. ))??
   Modifications are encouraged but must be packaged separately as patches to
   official Zope releases. Distributions that do not clearly separate the
   patches from the original work must be clearly labeled as unofficial
   distributions. Modifications which do not carry the name Zope may be packaged
   in any form, as long as they conform to all of the clauses above.

Disclaimer

THIS SOFTWARE IS PROVIDED BY ZOPE CORPORATION ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESSED OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL ZOPE CORPORATION OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
(( This software consists of contributions made by Zope Corporation and many
individuals on behalf of Zope Corporation. Specific attributions are listed in
the accompanying credits file. ))??
`
const license_ZPL_2_0_lre = `//**
Zope Public License 2.0
https://spdx.org/licenses/ZPL-2.0.json
http://old.zope.org/Resources/License/ZPL-2.0
https://opensource.org/licenses/ZPL-2.0
**//

(( Zope Public License (ZPL) Version 2.0 ))??

(( This software is Copyright (c) Zope Corporation (tm) and Contributors.
All rights reserved. ))??

(( This license has been certified as open source. It has also been designated
as GPL compatible by the Free Software Foundation (FSF). ))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions in source code must retain the above copyright notice, this
   list of conditions, and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions, and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The name Zope Corporation (tm) must not be used to endorse or promote
   products derived from this software without prior written permission from
   Zope Corporation.

   (( 4. ))??
   The right to distribute this software or to use it for any purpose does not
   give you the right to use Servicemarks (sm) or Trademarks (tm) of Zope
   Corporation. Use of them is covered in a separate agreement (see
   http:/www.zope.com/Marks).

   (( 5. ))??
   If any files are modified, you must cause the modified files to carry
   prominent notices stating that you changed the files and the date of any
   change.

Disclaimer

THIS SOFTWARE IS PROVIDED BY ZOPE CORPORATION ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESSED OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL ZOPE CORPORATION OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
(( This software consists of contributions made by Zope Corporation and many
individuals on behalf of Zope Corporation. Specific attributions are listed in
the accompanying credits file. ))??
`
const license_ZPL_2_1_lre = `//**
Zope Public License 2.1
https://spdx.org/licenses/ZPL-2.1.json
http://old.zope.org/Resources/ZPL/
**//

(( Zope Public License (ZPL) Version 2.1 ))??

(( A copyright notice accompanies this license document that identifies the
copyright holders.

This license has been certified as open source. It has also been designated as
GPL compatible by the Free Software Foundation (FSF). ))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions in source code must retain the accompanying copyright notice,
   this list of conditions, and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the accompanying copyright
   notice, this list of conditions, and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

   (( 3. ))??
   Names of the copyright holders must not be used to endorse or promote
   products derived from this software without prior written permission from the
   copyright holders.

   (( 4. ))??
   The right to distribute this software or to use it for any purpose does not
   give you the right to use Servicemarks (sm) or Trademarks (tm) of the
   copyright holders. Use of them is covered by separate agreement with the
   copyright holders.

   (( 5. ))??
   If any files are modified, you must cause the modified files to carry
   prominent notices stating that you changed the files and the date of any
   change.

Disclaimer

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESSED
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
`
const license_Zed_lre = `//**
Zed License
https://spdx.org/licenses/Zed.json
https://fedoraproject.org/wiki/Licensing/Zed
**//

//** Copyright **//

You may copy and distribute this file freely. Any queries and complaints should
be forwarded to Jim.Davies@comlab.ox.ac.uk.

If you make any changes to this file, please do not distribute the results under
the name ` + "`" + `zed-csp.sty'.
`
const license_Zend_2_0_lre = `//**
Zend License v2.0
https://spdx.org/licenses/Zend-2.0.json
https://web.archive.org/web/20130517195954/http://www.zend.com/license/2_00.txt
**//

(( The Zend Engine License, version 2.00
(( Copyright __20__ ))??
))??

Redistribution and use in source and binary forms, with or without modification,
is permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   (( 2. ))??
   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   (( 3. ))??
   The names "Zend" and "Zend Engine" must not be used to endorse or promote
   products derived from this software without prior permission from Zend
   Technologies Ltd. For written permission, please contact license@zend.com.

   (( 4. ))??
   Zend Technologies Ltd. may publish revised and/or new versions of the license
   from time to time. Each version will be given a distinguishing version
   number. Once covered code has been published under a particular version of
   the license, you may always continue to use it under the terms of that
   version. You may also choose to use such covered code under the terms of any
   subsequent version of the license published by Zend Technologies Ltd. No one
   other than Zend Technologies Ltd. has the right to modify the terms
   applicable to covered code created under this License.

   (( 5. ))??
   Redistributions of any form whatsoever must retain the following
   acknowledgment: "This product includes the Zend Engine, freely available at
   http:/www.zend.com"

   (( 6. ))??
   All advertising materials mentioning features or use of this software must
   display the following acknowledgment: "The Zend Engine is freely available at
   http:/www.zend.com"

THIS SOFTWARE IS PROVIDED BY ZEND TECHNOLOGIES LTD. ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESSED
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL ZEND TECHNOLOGIES LTD. BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
`
const license_Zimbra_1_3_lre = `//**
Zimbra Public License v1.3
https://spdx.org/licenses/Zimbra-1.3.json
http://web.archive.org/web/20100302225219/http://www.zimbra.com/license/zimbra-public-license-1-3.html
**//

(( Zimbra Public License, Version 1.3 (ZPL) ))??

This Zimbra Public License (this "Agreement") is a legal agreement that
describes the terms under which VMware, Inc., a Delaware corporation having its
principal place of business at 3401 Hillview Avenue, Palo Alto, California 94304
("VMware") will provide software to you via download or otherwise ("Software").
By using the Software, you, an individual or an entity ("You") agree to the
terms of this Agreement.

In consideration of the mutual promises and upon the terms and conditions set
forth below, the parties agree as follows:

   (( 1. ))??
   Grant of Copyright License

      (( 1.1 - ))??
      Subject to the terms and conditions of this Agreement, VMware hereby
      grants to You, under any and all of its copyright interest in and to the
      Software, a royalty-free, non-exclusive, non-transferable license to copy,
      modify, compile, execute, and distribute the Software and Modifications.
      For the purposes of this Agreement, any change to, addition to, or
      abridgement of the Software made by You is a "Modification;" however, any
      file You add to the Software that does not contain any part of the
      Software is not a "Modification."

      (( 1.2 - ))??
      If You are an individual acting on behalf of a corporation or other
      entity, Your use of the Software or any Modification is subject to Your
      having the authority to bind such corporation or entity to this Agreement.
      Providing copies to persons within such corporation or entity is not
      considered distribution for purposes of this Agreement.

      (( 1.3 - ))??
      For the Software or any Modification You distribute in source code format,
      You must do so only under the terms of this Agreement, and You must
      include a complete copy of this Agreement with Your distribution. With
      respect to any Modification You distribute in source code format, the
      terms of this Agreement will apply to You in the same way those terms
      apply to VMware with respect to the Software. In other words, when You are
      distributing Modifications under this Agreement, You "stand in the shoes"
      of VMware in terms of the rights You grant and how the terms and
      conditions apply to You and the licensees of Your Modifications.
      Notwithstanding the foregoing, when You "stand in the shoes" of VMware,
      You are not subject to the jurisdiction provision under Section 7, which
      requires all disputes under this Agreement to be subject to the
      jurisdiction of federal or state courts of northern California.

      (( 1.4 - ))??
      For the Software or any Modification You distribute in compiled or object
      code format, You must also provide recipients with access to the Software
      or Modification in source code format along with a complete copy of this
      Agreement. The distribution of the Software or Modifications in compiled
      or object code format may be under a license of Your choice, provided that
      You are in compliance with the terms of this Agreement. In addition, You
      must make absolutely clear that any license terms applying to such
      Software or Modification that differ from this Agreement are offered by
      You alone and not by VMware, and that such license does not restrict
      recipients from exercising rights in the source code to the Software
      granted by VMware under this Agreement or rights in the source code to any
      Modification granted by You as described in Section 1.3.

      (( 1.5 - ))??
      This Agreement does not limit Your right to distribute files that are
      entirely Your own work (i.e., which do not incorporate any portion of the
      Software and are not Modifications) under any terms You choose.

   (( 2. ))??
   Support

   VMware has no obligation to provide technical support or updates to You.
   Nothing in this Agreement requires VMware to enter into any license with You
   for any other edition of the Software.

   (( 3. ))??
   Intellectual Property Rights

      (( 3.1 - ))??
      Except for the license expressly granted under copyright in Section 1.1,
      no rights, licenses or forbearances are granted or may arise in relation
      to this Agreement whether expressly, by implication, exhaustion, estoppel
      or otherwise. All rights, including all intellectual property rights, that
      are not expressly granted under this Agreement are hereby reserved.

      (( 3.2 - ))??
      In any copy of the Software or in any Modification you create, You must
      retain and reproduce, any and all copyright, patent, trademark, and
      attribution notices that are included in the Software in the same form as
      they appear in the Software. This includes the preservation of attribution
      notices in the form of trademarks or logos that exist within a user
      interface of the Software.

      (( 3.3 - ))??
      This license does not grant You rights to use any party's name, logo, or
      trademarks, except solely as necessary to comply with Section 3.2.

   (( 4. ))??
   Disclaimer of Warranties

   THE SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND. VMWARE
   MAKES NO WARRANTIES, WHETHER EXPRESS, IMPLIED, OR STATUTORY REGARDING OR
   RELATING TO THE SOFTWARE. SPECIFICALLY, VMWARE DOES NOT WARRANT THAT THE
   SOFTWARE WILL BE ERROR FREE OR WILL PERFORM IN AN UNINTERRUPTED MANNER. TO
   THE GREATEST EXTENT ALLOWED BY LAW, VMWARE SPECIFICALLY DISCLAIMS ALL IMPLIED
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE (EVEN IF
   VMWARE HAD BEEN INFORMED OF SUCH PURPOSE), AND NONINFRINGEMENT WITH RESPECT
   TO THE SOFTWARE, ANY MODIFICATIONS THERETO AND WITH RESPECT TO THE USE OF THE
   FOREGOING.

   (( 5. ))??
   Limitation of Liability

   IN NO EVENT WILL VMWARE BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES OF ANY KIND (INCLUDING WITHOUT
   LIMITATION LOSS OF PROFITS, LOSS OF USE, BUSINESS INTERRUPTION, LOSS OF DATA,
   COST OF COVER) IN CONNECTION WITH OR ARISING OUT OF OR RELATING TO THE
   FURNISHING, PERFORMANCE OR USE OF THE SOFTWARE OR ANY OTHER RIGHTS GRANTED
   HEREUNDER, WHETHER ALLEGED AS A BREACH OF CONTRACT OR TORTIOUS CONDUCT,
   INCLUDING NEGLIGENCE, AND EVEN IF VMWARE HAS BEEN ADVISED OF THE POSSIBILITY
   OF SUCH DAMAGES.

   (( 6. ))??
   Term and Termination

      (( 6.1 - ))??
      This Agreement will continue in effect unless and until terminated earlier
      pursuant to this Section 6.

      (( 6.2 - ))??
      In the event You violate the terms of this Agreement, VMware may terminate
      this Agreement.

      (( 6.3 - ))??
      All licenses granted hereunder shall terminate upon the termination of
      this Agreement. Termination will be in addition to any rights and remedies
      available to VMware at law or equity or under this Agreement.

      (( 6.4 - ))??
      Termination of this Agreement will not affect the provisions regarding
      reservation of rights (Section 3.1), provisions disclaiming or limiting
      VMware's liability (Sections 4 and 5), Termination (Section 6) or
      Miscellaneous (Section 7), which provisions will survive termination of
      this Agreement.

   (( 7. ))??
   Miscellaneous

   This Agreement contains the entire agreement of the parties with respect to
   the subject matter of this Agreement and supersedes all previous
   communications, representations, understandings and agreements, either oral
   or written, between the parties with respect to said subject matter. The
   relationship of the parties hereunder is that of independent contractors, and
   this Agreement will not be construed as creating an agency, partnership,
   joint venture or any other form of legal association between the parties. If
   any term, condition, or provision in this Agreement is found to be invalid,
   unlawful or unenforceable to any extent, this Agreement will be construed in
   a manner that most closely effectuates the intent of this Agreement. Such
   invalid term, condition or provision will be severed from the remaining
   terms, conditions and provisions, which will continue to be valid and
   enforceable to the fullest extent permitted by law. This Agreement will be
   interpreted and construed in accordance with the laws of the State of
   California and the United States of America, without regard to conflict of
   law principles. The U.N. Convention on Contracts for the International Sale
   of Goods shall not apply to this Agreement. All disputes arising out of this
   Agreement involving VMware or any of its subsidiaries shall be subject to the
   jurisdiction of the federal or state courts of northern California, with
   venue lying in Santa Clara County, California. No rights may be assigned, no
   obligations may be delegated, and this Agreement may not be transferred by
   You, in whole or in part, whether voluntary or by operation of law, including
   by way of sale of assets, merger or consolidation, without the prior written
   consent of VMware, and any purported assignment, delegation or transfer
   without such consent shall be void ab initio. Any waiver of the provisions of
   this Agreement or of a party's rights or remedies under this Agreement must
   be in writing to be effective. Failure, neglect or delay by a party to
   enforce the provisions of this Agreement or its rights or remedies at any
   time, will not be construed or be deemed to be a waiver of such party's
   rights under this Agreement and will not in any way affect the validity of
   the whole or any part of this Agreement or prejudice such party's right to
   take subsequent action.
`
const license_Zimbra_1_4_lre = `//**
Zimbra Public License v1.4
https://spdx.org/licenses/Zimbra-1.4.json
http://www.zimbra.com/legal/zimbra-public-license-1-4
**//

(( Zimbra Public License, Version 1.4 (ZPL) ))??

This Zimbra Public License (this "Agreement") is a legal agreement that
describes the terms under which Zimbra, Inc., a Texas corporation ("Zimbra")
will provide software to you via download or otherwise ("Software"). By using
the Software, you, an individual or an entity ("You") agree to the terms of this
Agreement.

In consideration of the mutual promises and upon the terms and conditions set
forth below, the parties agree as follows:

   (( 1. ))??
   Grant of Copyright License

      (( 1.1 ))??
      - Subject to the terms and conditions of this Agreement, Zimbra hereby
      grants to You, under any and all of its copyright interest in and to the
      Software, a royalty-free, non-exclusive, non-transferable license to copy,
      modify, compile, execute, and distribute the Software and Modifications.
      For the purposes of this Agreement, any change to, addition to, or
      abridgement of the Software made by You is a "Modification;" however, any
      file You add to the Software that does not contain any part of the
      Software is not a "Modification."

      (( 1.2 ))??
      - If You are an individual acting on behalf of a corporation or other
      entity, Your use of the Software or any Modification is subject to Your
      having the authority to bind such corporation or entity to this Agreement.
      Providing copies to persons within such corporation or entity is not
      considered distribution for purposes of this Agreement.

      (( 1.3 ))??
      - For the Software or any Modification You distribute in source code
      format, You must do so only under the terms of this Agreement, and You
      must include a complete copy of this Agreement with Your distribution.
      With respect to any Modification You distribute in source code format, the
      terms of this Agreement will apply to You in the same way those terms
      apply to Zimbra with respect to the Software. In other words, when You are
      distributing Modifications under this Agreement, You "stand in the shoes"
      of Zimbra in terms of the rights You grant and how the terms and
      conditions apply to You and the licensees of Your Modifications.
      Notwithstanding the foregoing, when You "stand in the shoes" of Zimbra,
      You are not subject to the jurisdiction provision under Section 7, which
      requires all disputes under this Agreement to be subject to the
      jurisdiction of federal or state courts of Northern Texas.

      (( 1.4 ))??
      - For the Software or any Modification You distribute in compiled or
      object code format, You must also provide recipients with access to the
      Software or Modification in source code format along with a complete copy
      of this Agreement. The distribution of the Software or Modifications in
      compiled or object code format may be under a license of Your choice,
      provided that You are in compliance with the terms of this Agreement. In
      addition, You must make absolutely clear that any license terms applying
      to such Software or Modification that differ from this Agreement are
      offered by You alone and not by Zimbra, and that such license does not
      restrict recipients from exercising rights in the source code to the
      Software granted by Zimbra under this Agreement or rights in the source
      code to any Modification granted by You as described in Section 1.3.

      (( 1.5 ))??
      - This Agreement does not limit Your right to distribute files that are
      entirely Your own work (i.e., which do not incorporate any portion of the
      Software and are not Modifications) under any terms You choose.

   (( 2. ))??
   Support

   Zimbra has no obligation to provide technical support or updates to You.
   Nothing in this Agreement requires Zimbra to enter into any license with You
   for any other edition of the Software.

   (( 3. ))??
   Intellectual Property Rights

      (( 3.1 - ))??
      Except for the license expressly granted under copyright in Section 1.1,
      no rights, licenses or forbearances are granted or may arise in relation
      to this Agreement whether expressly, by implication, exhaustion, estoppel
      or otherwise. All rights, including all intellectual property rights, that
      are not expressly granted under this Agreement are hereby reserved.

      (( 3.2 - ))??
      In any copy of the Software or in any Modification you create, You must
      retain and reproduce any and all copyright, patent, trademark, and
      attribution notices that are included in the Software in the same form as
      they appear in the Software. This includes the preservation of attribution
      notices in the form of trademarks or logos that exist within a user
      interface of the Software.

      (( 3.3 - ))??
      This license does not grant You rights to use any party's name, logo, or
      trademarks, except solely as necessary to comply with Section 3.2.

   (( 4. ))??
   Disclaimer of Warranties

   THE SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND. ZIMBRA
   MAKES NO WARRANTIES, WHETHER EXPRESS, IMPLIED, OR STATUTORY, REGARDING OR
   RELATING TO THE SOFTWARE. SPECIFICALLY, ZIMBRA DOES NOT WARRANT THAT THE
   SOFTWARE WILL BE ERROR FREE OR WILL PERFORM IN AN UNINTERRUPTED MANNER. TO
   THE GREATEST EXTENT ALLOWED BY LAW, ZIMBRA SPECIFICALLY DISCLAIMS ALL IMPLIED
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE (EVEN IF
   ZIMBRA HAD BEEN INFORMED OF SUCH PURPOSE), AND NONINFRINGEMENT WITH RESPECT
   TO THE SOFTWARE, ANY MODIFICATIONS THERETO, AND WITH RESPECT TO THE USE OF
   THE FOREGOING.

   (( 5. ))??
   Limitation of Liability

   IN NO EVENT WILL ZIMBRA BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES OF ANY KIND (INCLUDING WITHOUT
   LIMITATION LOSS OF PROFITS, LOSS OF USE, BUSINESS INTERRUPTION, LOSS OF DATA,
   AND COST OF COVER) IN CONNECTION WITH OR ARISING OUT OF OR RELATING TO THE
   FURNISHING, PERFORMANCE, OR USE OF THE SOFTWARE OR ANY OTHER RIGHTS GRANTED
   HEREUNDER, WHETHER ALLEGED AS A BREACH OF CONTRACT OR TORTIOUS CONDUCT,
   INCLUDING NEGLIGENCE, AND EVEN IF ZIMBRA HAS BEEN ADVISED OF THE POSSIBILITY
   OF SUCH DAMAGES.

   (( 6. ))??
   Term and Termination

      (( 6.1 - ))??
      This Agreement will continue in effect unless and until terminated earlier
      pursuant to this Section 6.

      (( 6.2 - ))??
      In the event You violate the terms of this Agreement, Zimbra may terminate
      this Agreement.

      (( 6.3 - ))??
      All licenses granted hereunder shall terminate upon the termination of
      this Agreement. Termination will be in addition to any rights and remedies
      available to Zimbra at law or equity or under this Agreement.

      (( 6.4 - ))??
      Termination of this Agreement will not affect the provisions regarding
      reservation of rights (Section 3.1), provisions disclaiming or limiting
      Zimbra's liability (Sections 4 and 5), Termination (Section 6), or
      Miscellaneous (Section 7), which provisions will survive termination of
      this Agreement.

   (( 7. ))??
   Miscellaneous

   This Agreement contains the entire agreement of the parties with respect to
   the subject matter of this Agreement and supersedes all previous
   communications, representations, understandings, and agreements, either oral
   or written, between the parties with respect to said subject matter. The
   relationship of the parties hereunder is that of independent contractors, and
   this Agreement will not be construed as creating an agency, partnership,
   joint venture, or any other form of legal association between the parties. If
   any term, condition, or provision in this Agreement is found to be invalid,
   unlawful, or unenforceable to any extent, this Agreement will be construed in
   a manner that most closely effectuates the intent of this Agreement. Such
   invalid term, condition or provision will be severed from the remaining
   terms, conditions, and provisions, which will continue to be valid and
   enforceable to the fullest extent permitted by law. This Agreement will be
   interpreted and construed in accordance with the laws of the State of
   Delaware and the United States of America, without regard to conflict of law
   principles. The U.N. Convention on Contracts for the International Sale of
   Goods shall not apply to this Agreement. All disputes arising out of this
   Agreement involving Zimbra or any of its parents or subsidiaries shall be
   subject to the jurisdiction of the federal or state courts of Northern Texas,
   with venue lying in Dallas County, Texas. No rights may be assigned, no
   obligations may be delegated, and this Agreement may not be transferred by
   You, in whole or in part, whether voluntary or by operation of law, including
   by way of sale of assets, merger, or consolidation, without the prior written
   consent of Zimbra, and any purported assignment, delegation, or transfer
   without such consent shall be void ab initio. Any waiver of the provisions of
   this Agreement or of a party's rights or remedies under this Agreement must
   be in writing to be effective. Failure, neglect, or delay by a party to
   enforce the provisions of this Agreement or its rights or remedies at any
   time will not be construed or be deemed to be a waiver of such party's rights
   under this Agreement and will not in any way affect the validity of the whole
   or any part of this Agreement or prejudice such party's right to take
   subsequent action.
`
const license_Zlib_lre = `//**
zlib License
https://spdx.org/licenses/Zlib.json
http://www.zlib.net/zlib_license.html
https://opensource.org/licenses/Zlib
**//

(( zlib License
(( Copyright __20__ ))??
))??

This software is provided 'as-is', without any express or implied warranty. In
no event will the authors be held liable for any damages arising from the use of
this software.

Permission is granted to anyone to use this software for any purpose, including
commercial applications, and to alter it and redistribute it freely, subject to
the following restrictions:

   (( 1. ))??
   The origin of this software must not be misrepresented; you must not claim
   that you wrote the original software. If you use this software in a product,
   an acknowledgment in the product documentation would be appreciated but is
   not required.

   (( 2. ))??
   Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   (( 3. ))??
   This notice may not be removed or altered from any source distribution.
`
const license_blessing_lre = `//**
SQLite Blessing
https://spdx.org/licenses/blessing.json
https://www.sqlite.org/src/artifact/e33a4df7e32d742a?ln=4-9
https://sqlite.org/src/artifact/df5091916dbb40e6
**//

The author disclaims copyright to this source code. In place of a legal notice,
here is a blessing:

   May you do good and not evil.

   May you find forgiveness for yourself and forgive others.

   May you share freely, never taking more than you give.
`
const license_bzip2_1_0_5_lre = `//**
bzip2 and libbzip2 License v1.0.5
https://spdx.org/licenses/bzip2-1.0.5.json
https://sourceware.org/bzip2/1.0.5/bzip2-manual-1.0.5.html
http://bzip.org/1.0.5/bzip2-manual-1.0.5.html
**//

(( Version 1.0.5 of 10 December 2007
(( Copyright __20__ ))??
))??

This program, bzip2, the associated library libbzip2, and all documentation, are
copyright © 1996-2007 Julian Seward. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   •
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   •
   The origin of this software must not be misrepresented; you must not claim
   that you wrote the original software. If you use this software in a product,
   an acknowledgment in the product documentation would be appreciated but is
   not required.

   •
   Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   •
   The name of the author may not be used to endorse or promote products derived
   from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

PATENTS: To the best of my knowledge, bzip2 and libbzip2 do not use any patented
algorithms. However, I do not have the resources to carry out a patent search.
Therefore I cannot give any guarantee of the above statement.
`
const license_bzip2_1_0_6_lre = `//**
bzip2 and libbzip2 License v1.0.6
https://spdx.org/licenses/bzip2-1.0.6.json
https://sourceware.org/git/?p=bzip2.git;a=blob;f=LICENSE;hb=bzip2-1.0.6
http://bzip.org/1.0.5/bzip2-manual-1.0.5.html
**//

//** Copyright **//

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1. ))??
   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   (( 2. ))??
   The origin of this software must not be misrepresented; you must not claim
   that you wrote the original software. If you use this software in a product,
   an acknowledgment in the product documentation would be appreciated but is
   not required.

   (( 3. ))??
   Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   (( 4. ))??
   The name of the author may not be used to endorse or promote products derived
   from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ` + "`" + `` + "`" + `AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

((
Julian Seward,
((jseward@bzip.org || jseward@acm.org))
((bzip2/libbzip2 version 1.0.6 of 6 September 2010))??
))??`
const license_copyleft_next_0_3_0_lre = `//**
copyleft-next 0.3.0
https://spdx.org/licenses/copyleft-next-0.3.0.json
https://github.com/copyleft-next/copyleft-next/blob/master/Releases/copyleft-next-0.3.0
**//

(( copyleft-next 0.3.0 ("this License")

Release date: 2013-05-16 ))??

   (( 1. ))??
   License Grants; No Trademark License

   Subject to the terms of this License, I grant You:

      (( a) ))??
      A non-exclusive, worldwide, perpetual, royalty-free, irrevocable copyright
      license, to reproduce, Distribute, prepare derivative works of, publicly
      perform and publicly display My Work.

      (( b) ))??
      A non-exclusive, worldwide, perpetual, royalty-free, irrevocable patent
      license under Licensed Patents to make, have made, use, sell, offer for
      sale, and import Covered Works.

   This License does not grant any rights in My name, trademarks, service marks,
   or logos.

   (( 2. ))??
   Distribution: General Conditions

   You may Distribute Covered Works, provided that You (i) inform recipients how
   they can obtain a copy of this License; (ii) satisfy the applicable
   conditions of sections 3 through 6; and (iii) preserve all Legal Notices
   contained in My Work (to the extent they remain pertinent). "Legal Notices"
   means copyright notices, license notices, license texts, and author
   attributions, but does not include logos, other graphical images, trademarks
   or trademark legends.

   (( 3. ))??
   Conditions for Distributing Derived Works; Outbound GPL Compatibility

   If You Distribute a Derived Work, You must license the entire Derived Work as
   a whole under this License, with prominent notice of such licensing. This
   condition may not be avoided through such means as separate Distribution of
   portions of the Derived Work. You may additionally license the Derived Work
   under the GPL, so that the recipient may further Distribute the Derived Work
   under either this License or the GPL.

   (( 4. ))??
   Condition Against Further Restrictions; Inbound License Compatibility

   When Distributing a Covered Work, You may not impose further restrictions on
   the exercise of rights in the Covered Work granted under this License. This
   condition is not excused merely because such restrictions result from Your
   compliance with conditions or obligations extrinsic to this License (such as
   a court order or an agreement with a third party).

   However, You may Distribute a Covered Work incorporating material governed by
   a license that is both OSI-Approved and FSF-Free as of the release date of
   this License, provided that Your Distribution complies with such other
   license.

   (( 5. ))??
   Conditions for Distributing Object Code

   You may Distribute an Object Code form of a Covered Work, provided that you
   accompany the Object Code with a URL through which the Corresponding Source
   is made available, at no charge, by some standard or customary means of
   providing network access to source code.

   If you Distribute the Object Code in a physical product or tangible storage
   medium ("Product"), the Corresponding Source must be available through such
   URL for two years from the date of Your most recent Distribution of the
   Object Code in the Product. However, if the Product itself contains or is
   accompanied by the Corresponding Source (made available in a customarily
   accessible manner), You need not also comply with the first paragraph of this
   section.

   Each recipient of the Covered Work from You is an intended third-party
   beneficiary of this License solely as to this section 5, with the right to
   enforce its terms.

   (( 6. ))??
   Symmetrical Licensing Condition for Upstream Contributions

   If You Distribute a work to Me specifically for inclusion in or modification
   of a Covered Work (a "Patch"), and no explicit licensing terms apply to the
   Patch, You license the Patch under this License, to the extent of Your
   copyright in the Patch. This condition does not negate the other conditions
   of this License, if applicable to the Patch.

   (( 7. ))??
   Nullification of Copyleft/Proprietary Dual Licensing

   If I offer to license, for a fee, a Covered Work under terms other than a
   license that is OSI-Approved or FSF-Free as of the release date of this
   License or a numbered version of copyleft-next released by the Copyleft-Next
   Project, then the license I grant You under section 1 is no longer subject to
   the conditions in sections 2 through 5.

   (( 8. ))??
   Copyleft Sunset

   The conditions in sections 2 through 5 no longer apply once fifteen years
   have elapsed from the date of My first Distribution of My Work under this
   License.

   (( 9. ))??
   Pass-Through

   When You Distribute a Covered Work, the recipient automatically receives a
   license to My Work from Me, subject to the terms of this License.

   (( 10. ))??
   Termination

   Your license grants under section 1 are automatically terminated if You

      (( a) ))??
      fail to comply with the conditions of this License, unless You cure such
      noncompliance within thirty days after becoming aware of it, or

      (( b) ))??
      initiate a patent infringement litigation claim (excluding declaratory
      judgment actions, counterclaims, and cross-claims) alleging that any part
      of My Work directly or indirectly infringes any patent.

   Termination of Your license grants extends to all copies of Covered Works You
   subsequently obtain. Termination does not terminate the rights of those who
   have received copies or rights from You subject to this License.

   To the extent permission to make copies of a Covered Work is necessary merely
   for running it, such permission is not terminable.

   (( 11. ))??
   Later License Versions

   The Copyleft-Next Project may release new versions of copyleft-next,
   designated by a distinguishing version number ("Later Versions"). Unless I
   explicitly remove the option of Distributing Covered Works under Later
   Versions, You may Distribute Covered Works under any Later Version.

   (( 12. ))??
   No Warranty

   My Work is provided "as-is", without warranty. You bear the risk of using it.
   To the extent permitted by applicable law, each Distributor of My Work
   excludes the implied warranties of title, merchantability, fitness for a
   particular purpose and non-infringement.

   (( 13. ))??
   Limitation of Liability

   To the extent permitted by applicable law, in no event will any Distributor
   of My Work be liable to You for any damages whatsoever, whether direct,
   indirect, special, incidental, or consequential damages, whether arising
   under contract, tort (including negligence), or otherwise, even where the
   Distributor knew or should have known about the possibility of such damages.

   (( 14. ))??
   Severability

   The invalidity or unenforceability of any provision of this License does not
   affect the validity or enforceability of the remainder of this License. Such
   provision is to be reformed to the minimum extent necessary to make it valid
   and enforceable.

   (( 15. ))??
   Definitions

   "Copyleft-Next Project" means the project that maintains the source code
   repository at <https:/gitorious.org/copyleft-next/> as of the release date of
   this License.

   "Corresponding Source" of a Covered Work in Object Code form means (i) the
   Source Code form of the Covered Work; (ii) all scripts, instructions and
   similar information that are reasonably necessary for a skilled developer to
   generate such Object Code from the Source Code provided under (i); and (iii)
   a list clearly identifying all Separate Works (other than those provided in
   compliance with (ii) that were specifically used in building and (if
   applicable) installing the Covered Work (for example, a specified proprietary
   compiler including its version number). Corresponding Source must be
   machine-readable.

   "Covered Work" means My Work or a Derived Work.

   "Derived Work" means a work of authorship that copies from, modifies, adapts,
   is based on, is a derivative work of, transforms, translates or contains all
   or part of My Work, such that copyright permission is required. The following
   are not Derived Works: (i) Mere Aggregation; (ii) a mere reproduction of My
   Work; and (iii) if My Work fails to explicitly state an expectation
   otherwise, a work that merely makes reference to My Work.

   "Distribute" means to distribute, transfer or make a copy available to
   someone else, such that copyright permission is required.

   "Distributor" means Me and anyone else who Distributes a Covered Work.

   "FSF-Free" means classified as 'free' by the Free Software Foundation.

   "GPL" means a version of the GNU General Public License or the GNU Affero
   General Public License.

   "I"/"Me"/"My" refers to the individual or legal entity that places My Work
   under this License. "You"/"Your" refers to the individual or legal entity
   exercising rights in My Work under this License. A legal entity includes each
   entity that controls, is controlled by, or is under common control with such
   legal entity. "Control" means (a) the power to direct the actions of such
   legal entity, whether by contract or otherwise, or (b) ownership of more than
   fifty percent of the outstanding shares or beneficial ownership of such legal
   entity.

   "Licensed Patents" means all patent claims licensable royalty-free by Me, now
   or in the future, that are necessarily infringed by making, using, or selling
   My Work, and excludes claims that would be infringed only as a consequence of
   further modification of My Work.

   "Mere Aggregation" means an aggregation of a Covered Work with a Separate
   Work.

   "My Work" means the particular work of authorship I license to You under this
   License.

   "Object Code" means any form of a work that is not Source Code.

   "OSI-Approved" means approved as 'Open Source' by the Open Source
   Initiative.

   "Separate Work" means a work that is separate from and independent of a
   particular Covered Work and is not by its nature an extension or enhancement
   of the Covered Work, and/or a runtime library, standard library or similar
   component that is used to generate an Object Code form of a Covered Work.

   "Source Code" means the preferred form of a work for making modifications to
   it.
`
const license_copyleft_next_0_3_1_lre = `//**
copyleft-next 0.3.1
https://spdx.org/licenses/copyleft-next-0.3.1.json
https://github.com/copyleft-next/copyleft-next/blob/master/Releases/copyleft-next-0.3.1
**//

(( copyleft-next 0.3.1 ("this License")

Release date: 2016-04-29 ))??

   (( 1. ))??
   License Grants; No Trademark License

   Subject to the terms of this License, I grant You:

      (( a) ))??
      A non-exclusive, worldwide, perpetual, royalty-free, irrevocable copyright
      license, to reproduce, Distribute, prepare derivative works of, publicly
      perform and publicly display My Work.

      (( b) ))??
      A non-exclusive, worldwide, perpetual, royalty-free, irrevocable patent
      license under Licensed Patents to make, have made, use, sell, offer for
      sale, and import Covered Works.

   This License does not grant any rights in My name, trademarks, service marks,
   or logos.

   (( 2. ))??
   Distribution: General Conditions

   You may Distribute Covered Works, provided that You (i) inform recipients how
   they can obtain a copy of this License; (ii) satisfy the applicable
   conditions of sections 3 through 6; and (iii) preserve all Legal Notices
   contained in My Work (to the extent they remain pertinent). "Legal Notices"
   means copyright notices, license notices, license texts, and author
   attributions, but does not include logos, other graphical images, trademarks
   or trademark legends.

   (( 3. ))??
   Conditions for Distributing Derived Works; Outbound GPL Compatibility

   If You Distribute a Derived Work, You must license the entire Derived Work as
   a whole under this License, with prominent notice of such licensing. This
   condition may not be avoided through such means as separate Distribution of
   portions of the Derived Work.

   If the Derived Work includes material licensed under the GPL, You may instead
   license the Derived Work under the GPL.

   (( 4. ))??
   Condition Against Further Restrictions; Inbound License Compatibility

   When Distributing a Covered Work, You may not impose further restrictions on
   the exercise of rights in the Covered Work granted under this License. This
   condition is not excused merely because such restrictions result from Your
   compliance with conditions or obligations extrinsic to this License (such as
   a court order or an agreement with a third party).

   However, You may Distribute a Covered Work incorporating material governed by
   a license that is both OSI-Approved and FSF-Free as of the release date of
   this License, provided that compliance with such other license would not
   conflict with any conditions stated in other sections of this License.

   (( 5. ))??
   Conditions for Distributing Object Code

   You may Distribute an Object Code form of a Covered Work, provided that you
   accompany the Object Code with a URL through which the Corresponding Source
   is made available, at no charge, by some standard or customary means of
   providing network access to source code.

   If you Distribute the Object Code in a physical product or tangible storage
   medium ("Product"), the Corresponding Source must be available through such
   URL for two years from the date of Your most recent Distribution of the
   Object Code in the Product. However, if the Product itself contains or is
   accompanied by the Corresponding Source (made available in a customarily
   accessible manner), You need not also comply with the first paragraph of this
   section.

   Each direct and indirect recipient of the Covered Work from You is an
   intended third-party beneficiary of this License solely as to this section 5,
   with the right to enforce its terms.

   (( 6. ))??
   Symmetrical Licensing Condition for Upstream Contributions

   If You Distribute a work to Me specifically for inclusion in or modification
   of a Covered Work (a "Patch"), and no explicit licensing terms apply to the
   Patch, You license the Patch under this License, to the extent of Your
   copyright in the Patch. This condition does not negate the other conditions
   of this License, if applicable to the Patch.

   (( 7. ))??
   Nullification of Copyleft/Proprietary Dual Licensing

   If I offer to license, for a fee, a Covered Work under terms other than a
   license that is OSI-Approved or FSF-Free as of the release date of this
   License or a numbered version of copyleft-next released by the Copyleft-Next
   Project, then the license I grant You under section 1 is no longer subject to
   the conditions in sections 3 through 5.

   (( 8. ))??
   Copyleft Sunset

   The conditions in sections 3 through 5 no longer apply once fifteen years
   have elapsed from the date of My first Distribution of My Work under this
   License.

   (( 9. ))??
   Pass-Through

   When You Distribute a Covered Work, the recipient automatically receives a
   license to My Work from Me, subject to the terms of this License.

   (( 10. ))??
   Termination

   Your license grants under section 1 are automatically terminated if You

      (( a) ))??
      fail to comply with the conditions of this License, unless You cure such
      noncompliance within thirty days after becoming aware of it, or

      (( b) ))??
      initiate a patent infringement litigation claim (excluding declaratory
      judgment actions, counterclaims, and cross-claims) alleging that any part
      of My Work directly or indirectly infringes any patent.

   Termination of Your license grants extends to all copies of Covered Works You
   subsequently obtain. Termination does not terminate the rights of those who
   have received copies or rights from You subject to this License.

   To the extent permission to make copies of a Covered Work is necessary merely
   for running it, such permission is not terminable.

   (( 11. ))??
   Later License Versions

   The Copyleft-Next Project may release new versions of copyleft-next,
   designated by a distinguishing version number ("Later Versions"). Unless I
   explicitly remove the option of Distributing Covered Works under Later
   Versions, You may Distribute Covered Works under any Later Version.

   (( 12. ))??
   No Warranty

   My Work is provided "as-is", without warranty. You bear the risk of using it.
   To the extent permitted by applicable law, each Distributor of My Work
   excludes the implied warranties of title, merchantability, fitness for a
   particular purpose and non-infringement.

   (( 13. ))??
   Limitation of Liability

   To the extent permitted by applicable law, in no event will any Distributor
   of My Work be liable to You for any damages whatsoever, whether direct,
   indirect, special, incidental, or consequential damages, whether arising
   under contract, tort (including negligence), or otherwise, even where the
   Distributor knew or should have known about the possibility of such damages.

   (( 14. ))??
   Severability

   The invalidity or unenforceability of any provision of this License does not
   affect the validity or enforceability of the remainder of this License. Such
   provision is to be reformed to the minimum extent necessary to make it valid
   and enforceable.

   (( 15. ))??
   Definitions

   "Copyleft-Next Project" means the project that maintains the source code
   repository at <https:/github.com/copyleft-next/copyleft-next.git/> as of the
   release date of this License.

   "Corresponding Source" of a Covered Work in Object Code form means (i) the
   Source Code form of the Covered Work; (ii) all scripts, instructions and
   similar information that are reasonably necessary for a skilled developer to
   generate such Object Code from the Source Code provided under (i); and (iii)
   a list clearly identifying all Separate Works (other than those provided in
   compliance with (ii) that were specifically used in building and (if
   applicable) installing the Covered Work (for example, a specified proprietary
   compiler including its version number). Corresponding Source must be
   machine-readable.

   "Covered Work" means My Work or a Derived Work.

   "Derived Work" means a work of authorship that copies from, modifies, adapts,
   is based on, is a derivative work of, transforms, translates or contains all
   or part of My Work, such that copyright permission is required. The following
   are not Derived Works: (i) Mere Aggregation; (ii) a mere reproduction of My
   Work; and (iii) if My Work fails to explicitly state an expectation
   otherwise, a work that merely makes reference to My Work.

   "Distribute" means to distribute, transfer or make a copy available to
   someone else, such that copyright permission is required.

   "Distributor" means Me and anyone else who Distributes a Covered Work.

   "FSF-Free" means classified as 'free' by the Free Software Foundation.

   "GPL" means a version of the GNU General Public License or the GNU Affero
   General Public License.

   "I"/"Me"/"My" refers to the individual or legal entity that places My Work
   under this License. "You"/"Your" refers to the individual or legal entity
   exercising rights in My Work under this License. A legal entity includes each
   entity that controls, is controlled by, or is under common control with such
   legal entity. "Control" means (a) the power to direct the actions of such
   legal entity, whether by contract or otherwise, or (b) ownership of more than
   fifty percent of the outstanding shares or beneficial ownership of such legal
   entity.

   "Licensed Patents" means all patent claims licensable royalty-free by Me, now
   or in the future, that are necessarily infringed by making, using, or selling
   My Work, and excludes claims that would be infringed only as a consequence of
   further modification of My Work.

   "Mere Aggregation" means an aggregation of a Covered Work with a Separate
   Work.

   "My Work" means the particular work of authorship I license to You under this
   License.

   "Object Code" means any form of a work that is not Source Code.

   "OSI-Approved" means approved as 'Open Source' by the Open Source
   Initiative.

   "Separate Work" means a work that is separate from and independent of a
   particular Covered Work and is not by its nature an extension or enhancement
   of the Covered Work, and/or a runtime library, standard library or similar
   component that is used to generate an Object Code form of a Covered Work.

   "Source Code" means the preferred form of a work for making modifications to
   it.
`
const license_curl_lre = `//**
curl License
https://spdx.org/licenses/curl.json
https://github.com/bagder/curl/blob/master/COPYING
**//

(( COPYRIGHT AND PERMISSION NOTICE
(( Copyright __20__ ))??
))??

Permission to use, copy, modify, and distribute this software for any purpose
with or without fee is hereby granted, provided that the above copyright notice
and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT
SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

Except as contained in this notice, the name of a copyright holder shall not be
used in advertising or otherwise to promote the sale, use or other dealings in
this Software without prior written authorization of the copyright holder.
`
const license_diffmark_lre = `//**
diffmark license
https://spdx.org/licenses/diffmark.json
https://fedoraproject.org/wiki/Licensing/diffmark
**//

   (( 1. ))??
   you can do what you want with it

   (( 2. ))??
   I refuse any responsibility for the consequences
`
const license_dvipdfm_lre = `//**
dvipdfm License
https://spdx.org/licenses/dvipdfm.json
https://fedoraproject.org/wiki/Licensing/dvipdfm
**//

A modified version of this file may be distributed, but it should be distributed
with a *different* name. Changed files must be distributed *together with a
complete and unchanged* distribution of these files.
`
const license_eGenix_lre = `//**
eGenix.com Public License 1.1.0
https://spdx.org/licenses/eGenix.json
http://www.egenix.com/products/eGenix.com-Public-License-1.1.0.pdf
https://fedoraproject.org/wiki/Licensing/eGenix.com_Public_License_1.1.0
**//

(( EGENIX.COM PUBLIC LICENSE AGREEMENT

Version 1.1.0 ))??

(( This license agreement is based on the Python CNRI License Agreement, a
widely accepted open- source license. ))??

   (( 1. ))??
   Introduction

   This "License Agreement" is between eGenix.com Software, Skills and Services
   GmbH ("eGenix.com"), having an office at Pastor-Loeh-Str. 48, D-40764
   Langenfeld, Germany, and the Individual or Organization ("Licensee")
   accessing and otherwise using this software in source or binary form and its
   associated documentation ("the Software").

   (( 2. ))??
   License

   Subject to the terms and conditions of this eGenix.com Public License
   Agreement, eGenix.com hereby grants Licensee a non-exclusive, royalty-free,
   world-wide license to reproduce, analyze, test, perform and/or display
   publicly, prepare derivative works, distribute, and otherwise use the
   Software alone or in any derivative version, provided, however, that the
   eGenix.com Public License Agreement is retained in the Software, or in any
   derivative version of the Software prepared by Licensee.

   (( 3. ))??
   NO WARRANTY

   eGenix.com is making the Software available to Licensee on an "AS IS" basis.
   SUBJECT TO ANY STATUTORY WARRANTIES WHICH CAN NOT BE EXCLUDED, EGENIX.COM
   MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED. BY WAY OF
   EXAMPLE, BUT NOT LIMITATION, EGENIX.COM MAKES NO AND DISCLAIMS ANY
   REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
   PURPOSE OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE ANY THIRD PARTY
   RIGHTS.

   (( 4. ))??
   LIMITATION OF LIABILITY

   EGENIX.COM SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF THE SOFTWARE
   FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS (INCLUDING,
   WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS
   INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) AS A
   RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE, OR ANY DERIVATIVE
   THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF. SOME JURISDICTIONS DO
   NOT ALLOW THE EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES,
   SO THE ABOVE EXCLUSION OR LIMITATION MAY NOT APPLY TO LICENSEE.

   (( 5. ))??
   Termination

   This License Agreement will automatically terminate upon a material breach of
   its terms and conditions.

   (( 6. ))??
   Third Party Rights

   Any software or documentation in source or binary form provided along with
   the Software that is associated with a separate license agreement is licensed
   to Licensee under the terms of that license agreement. This License Agreement
   does not apply to those portions of the Software. Copies of the third party
   licenses are included in the Software Distribution.

   (( 7. ))??
   General

   Nothing in this License Agreement affects any statutory rights of consumers
   that cannot be waived or limited by contract.

   Nothing in this License Agreement shall be deemed to create any relationship
   of agency, partnership, or joint venture between eGenix.com and Licensee.

   If any provision of this License Agreement shall be unlawful, void, or for
   any reason unenforceable, such provision shall be modified to the extent
   necessary to render it enforceable without losing its intent, or, if no such
   modification is possible, be severed from this License Agreement and shall
   not affect the validity and enforceability of the remaining provisions of
   this License Agreement.

   This License Agreement shall be governed by and interpreted in all respects
   by the law of Germany, excluding conflict of law provisions. It shall not be
   governed by the United Nations Convention on Contracts for International Sale
   of Goods. This License Agreement does not grant permission to use eGenix.com
   trademarks or trade names in a trademark sense to endorse or promote products
   or services of Licensee, or any third party.

   The controlling language of this License Agreement is English. If Licensee
   has received a translation into another language, it has been provided for
   Licensee's convenience only.

   (( 8. ))??
   Agreement

By downloading, copying, installing or otherwise using the Software, Licensee
agrees to be bound by the terms and conditions of this License Agreement. For
question regarding this License Agreement, please write to:

eGenix.com Software, Skills and Services GmbH

Pastor-Loeh-Str. 48

D-40764 Langenfeld

Germany
`
const license_etalab_2_0_lre = `//**
Etalab Open License 2.0
https://spdx.org/licenses/etalab-2.0.json
https://github.com/DISIC/politique-de-contribution-open-source/blob/master/LICENSE.pdf
https://raw.githubusercontent.com/DISIC/politique-de-contribution-open-source/master/LICENSE
**//

(( LICENCE OUVERTE / OPEN LICENCE

- Version 2.0

- Avril 2017 ))??

« RÉUTILISATION » DE L'« INFORMATION » SOUS CETTE LICENCE

Le « Concédant » concède au « Réutilisateur » un droit non exclusif et
gratuit de libre « Réutilisation » de l'« Information » objet de la
présente licence, à des fins commerciales ou non, dans le monde entier et pour
une durée illimitée, dans les conditions exprimées ci-dessous.

Le « Réutilisateur » est libre de réutiliser l'« Information » :

   -
   de la reproduire, la copier,

   -
   de l'adapter, la modifier, l'extraire et la transformer, pour créer des «
   Informations dérivées », des produits ou des services,

   -
   de la communiquer, la diffuser, la redistribuer, la publier et la
   transmettre,

   -
   de l'exploiter à titre commercial, par exemple en la combinant avec d'autres
   informations, ou en l'incluant dans son propre produit ou application.

Sous réserve de :

   -
   mentionner la paternité de l'« Information » : sa source (au moins le nom
   du « Concédant ») et la date de dernière mise à jour de l'« Information
   » réutilisée.

Le « Réutilisateur » peut notamment s'acquitter de cette condition en
renvoyant, par un lien hypertexte, vers la source de « l'Information » et
assurant une mention effective de sa paternité.

Par exemple :

« Ministère de xxx - Données originales téléchargées sur
http:/www.data.gouv.fr/fr/datasets/xxx/, mise à jour du 14 février 2017 ».

Cette mention de paternité ne confère aucun caractère officiel à la «
Réutilisation » de l'« Information », et ne doit pas suggérer une
quelconque reconnaissance ou caution par le « Concédant », ou par toute autre
entité publique, du « Réutilisateur » ou de sa « Réutilisation ».

« DONNÉES À CARACTÈRE PERSONNEL »

L'« Information » mise à disposition peut contenir des « Données à
caractère personnel » pouvant faire l'objet d'une « Réutilisation ». Si tel
est le cas, le « Concédant » informe le « Réutilisateur » de leur
présence.

L'« Information » peut être librement réutilisée, dans le cadre des droits
accordés par la présente licence, à condition de respecter le cadre légal
relatif à la protection des données à caractère personnel.

« DROITS DE PROPRIÉTÉ INTELLECTUELLE »

Il est garanti au « Réutilisateur » que les éventuels « Droits de
propriété intellectuelle » détenus par des tiers ou par le « Concédant »
sur l'« Information » ne font pas obstacle aux droits accordés par la
présente licence.

Lorsque le « Concédant » détient des « Droits de propriété intellectuelle
» cessibles sur l'« Information », il les cède au « Réutilisateur » de
façon non exclusive, à titre gracieux, pour le monde entier, pour toute la
durée des « Droits de propriété intellectuelle », et le « Réutilisateur
» peut faire tout usage de l'« Information » conformément aux libertés et
aux conditions définies par la présente licence.

RESPONSABILITÉ

L'« Information » est mise à disposition telle que produite ou reçue par le
« Concédant », sans autre garantie expresse ou tacite que celles prévues par
la présente licence. L'absence de défauts ou d'erreurs éventuellement
contenues dans l'« Information », comme la fourniture continue de l'«
Information » n'est pas garantie par le « Concédant ». Il ne peut être tenu
pour responsable de toute perte, préjudice ou dommage de quelque sorte causé
à des tiers du fait de la « Réutilisation ».

Le « Réutilisateur » est seul responsable de la « Réutilisation » de l'«
Information ».

La « Réutilisation » ne doit pas induire en erreur des tiers quant au contenu
de l'« Information », sa source et sa date de mise à jour.

DROIT APPLICABLE

La présente licence est régie par le droit français.

COMPATIBILITÉ DE LA PRÉSENTE LICENCE

La présente licence a été conçue pour être compatible avec toute licence
libre qui exige au moins la mention de paternité et notamment avec la version
antérieure de la présente licence ainsi qu'avec les licences :

   -
   « Open Government Licence » (OGL) du Royaume-Uni,

   -
   « Creative Commons Attribution » (CC-BY) de Creative Commons et

   -
   « Open Data Commons Attribution » (ODC-BY) de l'Open Knowledge Foundation.

DÉFINITIONS

Sont considérés, au sens de la présente licence comme :

Le « Concédant » : toute personne concédant un droit de « Réutilisation »
sur l'« Information » dans les libertés et les conditions prévues par la
présente licence

L'« Information » :

   -
   toute information publique figurant dans des documents communiqués ou
   publiés par une administration mentionnée au premier alinéa de l'article
   L.300-2 du CRPA;

   -
   toute information mise à disposition par toute personne selon les termes et
   conditions de la présente licence.

La « Réutilisation » : l'utilisation de l'« Information » à d'autres fins
que celles pour lesquelles elle a été produite ou reçue.

Le « Réutilisateur »: toute personne qui réutilise les « Informations »
conformément aux conditions de la présente licence.

Des « Données à caractère personnel » : toute information se rapportant à
une personne physique identifiée ou identifiable, pouvant être identifiée
directement ou indirectement. Leur « Réutilisation » est subordonnée au
respect du cadre juridique en vigueur.

Une « Information dérivée » : toute nouvelle donnée ou information créées
directement à partir de l'« Information » ou à partir d'une combinaison de
l'« Information » et d'autres données ou informations non soumises à cette
licence.

Les « Droits de propriété intellectuelle » : tous droits identifiés comme
tels par le Code de la propriété intellectuelle (notamment le droit d'auteur,
droits voisins au droit d'auteur, droit sui generis des producteurs de bases de
données…).

À PROPOS DE CETTE LICENCE

La présente licence a vocation à être utilisée par les administrations pour
la réutilisation de leurs informations publiques. Elle peut également être
utilisée par toute personne souhaitant mettre à disposition de l'«
Information » dans les conditions définies par la présente licence.

La France est dotée d'un cadre juridique global visant à une diffusion
spontanée par les administrations de leurs informations publiques afin d'en
permettre la plus large réutilisation.

Le droit de la « Réutilisation » de l'« Information » des administrations
est régi par le code des relations entre le public et l'administration (CRPA).

Cette licence facilite la réutilisation libre et gratuite des informations
publiques et figure parmi les licences qui peuvent être utilisées par
l'administration en vertu du décret pris en application de l'article L.323-2 du
CRPA.

Etalab est la mission chargée, sous l'autorité du Premier ministre, d'ouvrir
le plus grand nombre de données publiques des administrations de l'Etat et de
ses établissements publics. Elle a réalisé la Licence Ouverte pour faciliter
la réutilisation libre et gratuite de ces informations publiques, telles que
définies par l'article L321-1 du CRPA.

Cette licence est la version 2.0 de la Licence Ouverte.

Etalab se réserve la faculté de proposer de nouvelles versions de la Licence
Ouverte. Cependant, les « Réutilisateurs » pourront continuer à réutiliser
les informations qu'ils ont obtenues sous cette licence s'ils le souhaitent.
`
const license_gSOAP_1_3b_lre = `//**
gSOAP Public License v1.3b
https://spdx.org/licenses/gSOAP-1.3b.json
http://www.cs.fsu.edu/~engelen/license.html
**//

(( gSOAP Public License

Version 1.3b ))??

(( The gSOAP public license is derived from the Mozilla Public License (MPL1.1).
The sections that were deleted from the original MPL1.1 text are 1.0.1,
2.1.(c),(d), 2.2.(c),(d), 8.2.(b), 10, and 11. Section 3.8 was added. The
modified sections are 2.1.(b), 2.2.(b), 3.2 (simplified), 3.5 (deleted the last
sentence), and 3.6 (simplified).

This license applies to the gSOAP software package, with the exception of the
soapcpp2 and wsdl2h source code located in gsoap/src and gsoap/wsdl, all code
generated by soapcpp2 and wsdl2h, the UDDI source code gsoap/uddi2, and the Web
server sample source code samples/webserver. To use any of these software tools
and components commercially, a commercial license is required and can be
obtained from www.genivia.com. ))??

   (( 1 ))??
   DEFINITIONS.

      (( 1.0.1. ))??

      (( 1.1. ))??
      "Contributor" means each entity that creates or contributes to the
      creation of Modifications.

      (( 1.2. ))??
      "Contributor Version" means the combination of the Original Code, prior
      Modifications used by a Contributor, and the Modifications made by that
      particular Contributor.

      (( 1.3. ))??
      "Covered Code" means the Original Code, or Modifications or the
      combination of the Original Code, and Modifications, in each case
      including portions thereof.

      (( 1.4. ))??
      "Electronic Distribution Mechanism" means a mechanism generally accepted
      in the software development community for the electronic transfer of
      data.

      (( 1.5. ))??
      "Executable" means Covered Code in any form other than Source Code.

      (( 1.6. ))??
      "Initial Developer" means the individual or entity identified as the
      Initial Developer in the Source Code notice required by Exhibit A.

      (( 1.7. ))??
      "Larger Work" means a work which combines Covered Code or portions thereof
      with code not governed by the terms of this License.

      (( 1.8. ))??
      "License" means this document.

      (( 1.8.1. ))??
      "Licensable" means having the right to grant, to the maximum extent
      possible, whether at the time of the initial grant or subsequently
      acquired, any and all of the rights conveyed herein.

      (( 1.9. ))??
      "Modifications" means any addition to or deletion from the substance or
      structure of either the Original Code or any previous Modifications. When
      Covered Code is released as a series of files, a Modification is:

         (( A. ))??
         Any addition to or deletion from the contents of a file containing
         Original Code or previous Modifications.

         (( B. ))??
         Any new file that contains any part of the Original Code, or previous
         Modifications.

      (( 1.10. ))??
      "Original Code" means Source Code of computer software code which is
      described in the Source Code notice required by Exhibit A as Original
      Code, and which, at the time of its release under this License is not
      already Covered Code governed by this License.

      (( 1.10.1. ))??
      "Patent Claims" means any patent claim(s), now owned or hereafter
      acquired, including without limitation, method, process, and apparatus
      claims, in any patent Licensable by grantor.

      (( 1.11. ))??
      "Source Code" means the preferred form of the Covered Code for making
      modifications to it, including all modules it contains, plus any
      associated interface definition files, scripts used to control compilation
      and installation of an Executable, or source code differential comparisons
      against either the Original Code or another well known, available Covered
      Code of the Contributor's choice. The Source Code can be in a compressed
      or archival form, provided the appropriate decompression or de-archiving
      software is widely available for no charge.

      (( 1.12. ))??
      "You" (or "Your") means an individual or a legal entity exercising rights
      under, and complying with all of the terms of, this License or a future
      version of this License issued under Section 6.1. For legal entities,
      "You" includes any entity which controls, is controlled by, or is under
      common control with You. For purposes of this definition, "control" means
      (a) the power, direct or indirect, to cause the direction or management of
      such entity, whether by contract or otherwise, or (b) ownership of more
      than fifty percent (50%) of the outstanding shares or beneficial ownership
      of such entity.

   (( 2 ))??
   SOURCE CODE LICENSE.

      (( 2.1. ))??
      The Initial Developer Grant.

      The Initial Developer hereby grants You a world-wide, royalty-free,
      non-exclusive license, subject to third party intellectual property
      claims:

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Initial Developer to use, reproduce, modify, display,
         perform, sublicense and distribute the Original Code (or portions
         thereof) with or without Modifications, and/or as part of a Larger
         Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Initial
         Developer, to make, have made, use and sell ("offer to sell and
         import") the Original Code, Modifications, or portions thereof, but
         solely to the extent that any such patent is reasonably necessary to
         enable You to utilize, alone or in combination with other software, the
         Original Code, Modifications, or any combination or portions thereof.

         (( c ))??

         (( (d) ))??

      (( 2.2. ))??
      Contributor Grant.

      Subject to third party intellectual property claims, each Contributor
      hereby grants You a world-wide, royalty-free, non-exclusive license

         (( (a) ))??
         under intellectual property rights (other than patent or trademark)
         Licensable by Contributor, to use, reproduce, modify, display, perform,
         sublicense and distribute the Modifications created by such Contributor
         (or portions thereof) either on an unmodified basis, with other
         Modifications, as Covered Code and/or as part of a Larger Work; and

         (( (b) ))??
         under patents now or hereafter owned or controlled by Contributor, to
         make, have made, use and sell ("offer to sell and import") the
         Contributor Version (or portions thereof), but solely to the extent
         that any such patent is reasonably necessary to enable You to utilize,
         alone or in combination with other software, the Contributor Version
         (or portions thereof).

         (( c ))??

         (( (d) ))??

   (( 3 ))??
   DISTRIBUTION OBLIGATIONS.

      (( 3.1. ))??
      Application of License.

      The Modifications which You create or to which You contribute are governed
      by the terms of this License, including without limitation Section 2.2.
      The Source Code version of Covered Code may be distributed only under the
      terms of this License or a future version of this License released under
      Section 6.1, and You must include a copy of this License with every copy
      of the Source Code You distribute. You may not offer or impose any terms
      on any Source Code version that alters or restricts the applicable version
      of this License or the recipients' rights hereunder. However, You may
      include an additional document offering the additional rights described in
      Section 3.5.

      (( 3.2. ))??
      Availability of Source Code.

      Any Modification created by You
      //** shall **//
      __5__
      be provided to the Initial Developer in Source Code form and are subject
      to the terms of the License.

      (( 3.3. ))??
      Description of Modifications.

      You must cause all Covered Code to which You contribute to contain a file
      documenting the changes You made to create that Covered Code and the date
      of any change. You must include a prominent statement that the
      Modification is derived, directly or indirectly, from Original Code
      provided by the Initial Developer and including the name of the Initial
      Developer in (a) the Source Code, and (b) in any notice in an Executable
      version or related documentation in which You describe the origin or
      ownership of the Covered Code.

      (( 3.4. ))??
      Intellectual Property Matters.

         (( (a) ))??
         Third Party Claims. If Contributor has knowledge that a license under a
         third party's intellectual property rights is required to exercise the
         rights granted by such Contributor under Sections 2.1 or 2.2,
         Contributor must include a text file with the Source Code distribution
         titled "LEGAL" which describes the claim and the party making the claim
         in sufficient detail that a recipient will know whom to contact. If
         Contributor obtains such knowledge after the Modification is made
         available as described in Section 3.2, Contributor shall promptly
         modify the LEGAL file in all copies Contributor makes available
         thereafter and shall take other steps (such as notifying appropriate
         mailing lists or newsgroups) reasonably calculated to inform those who
         received the Covered Code that new knowledge has been obtained.

         (( (b) ))??
         Contributor APIs. If Contributor's Modifications include an application
         programming interface and Contributor has knowledge of patent licenses
         which are reasonably necessary to implement that API, Contributor must
         also include this information in the LEGAL file.

         (( c ))??
         Representations. Contributor represents that, except as disclosed
         pursuant to Section 3.4(a) above, Contributor believes that
         Contributor's Modifications are Contributor's original creation(s)
         and/or Contributor has sufficient rights to grant the rights conveyed
         by this License.

      (( 3.5. ))??
      Required Notices.

      You must duplicate the notice in Exhibit A in each file of the Source
      Code. If it is not possible to put such notice in a particular Source Code
      file due to its structure, then You must include such notice in a location
      (such as a relevant directory) where a user would be likely to look for
      such a notice. If You created one or more Modification(s) You may add your
      name as a Contributor to the notice described in Exhibit A. You must also
      duplicate this License in any documentation for the Source Code where You
      describe recipients' rights or ownership rights relating to Covered Code.
      You may choose to offer, and to charge a fee for, warranty, support,
      indemnity or liability obligations to one or more recipients of Covered
      Code. However, You may do so only on Your own behalf, and not on behalf of
      the Initial Developer or any Contributor.

      (( 3.6. ))??
      Distribution of Executable Versions.

      You may distribute Covered Code in Executable form only if the
      requirements of Section 3.1-3.5 have been met for that Covered Code. You
      may distribute the Executable version of Covered Code or ownership rights
      under a license of Your choice, which may contain terms different from
      this License, provided that You are in compliance with the terms of this
      License and that the license for the Executable version does not attempt
      to limit or alter the recipient's rights in the Source Code version from
      the rights set forth in this License. If You distribute the Executable
      version under a different license You must make it absolutely clear that
      any terms which differ from this License are offered by You alone, not by
      the Initial Developer or any Contributor. If you distribute executable
      versions containing Covered Code as part of a product, you must reproduce
      the notice in Exhibit B in the documentation and/or other materials
      provided with the product.

      (( 3.7. ))??
      Larger Works.

      You may create a Larger Work by combining Covered Code with other code not
      governed by the terms of this License and distribute the Larger Work as a
      single product. In such a case, You must make sure the requirements of
      this License are fulfilled for the Covered Code.

      (( 3.8. ))??
      Restrictions.

      You may not remove any product identification, copyright, proprietary
      notices or labels from gSOAP.

   (( 4 ))??
   INABILITY TO COMPLY DUE TO STATUTE OR REGULATION.

   If it is impossible for You to comply with any of the terms of this License
   with respect to some or all of the Covered Code due to statute, judicial
   order, or regulation then You must: (a) comply with the terms of this License
   to the maximum extent possible; and (b) describe the limitations and the code
   they affect. Such description must be included in the LEGAL file described in
   Section 3.4 and must be included with all distributions of the Source Code.
   Except to the extent prohibited by statute or regulation, such description
   must be sufficiently detailed for a recipient of ordinary skill to be able to
   understand it.

   (( 5 ))??
   APPLICATION OF THIS LICENSE.

   This License applies to code to which the Initial Developer has attached the
   notice in Exhibit A and to related Covered Code.

   (( 6 ))??
   VERSIONS OF THE LICENSE.

      (( 6.1. ))??
      New Versions.

      Grantor may publish revised and/or new versions of the License from time
      to time. Each version will be given a distinguishing version number.

      (( 6.2. ))??
      Effect of New Versions.

      Once Covered Code has been published under a particular version of the
      License, You may always continue to use it under the terms of that
      version. You may also choose to use such Covered Code under the terms of
      any subsequent version of the License.

      (( 6.3. ))??
      Derivative Works.

      If You create or use a modified version of this License (which you may
      only do in order to apply it to code which is not already Covered Code
      governed by this License), You must (a) rename Your license so that the
      phrase "gSOAP" or any confusingly similar phrase do not appear in your
      license (except to note that you license differs from this License) and
      (b) otherwise make it clear that Your version of the license contains
      terms which differ from the gSOAP Public License. (Filling in the name of
      the Initial Developer, Original Code or Contributor in the notice
      described in Exhibit A shall not of themselves be deemed to be
      modifications of this License.)

   (( 7 ))??
   DISCLAIMER OF WARRANTY.

   COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT
   WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,
   WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY, OF FITNESS FOR
   A PARTICULAR PURPOSE, NONINFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
   RIGHTS, AND ANY WARRANTY THAT MAY ARISE BY REASON OF TRADE USAGE, CUSTOM, OR
   COURSE OF DEALING. WITHOUT LIMITING THE FOREGOING, YOU ACKNOWLEDGE THAT THE
   SOFTWARE IS PROVIDED "AS IS" AND THAT THE AUTHORS DO NOT WARRANT THE SOFTWARE
   WILL RUN UNINTERRUPTED OR ERROR FREE. LIMITED LIABILITY THE ENTIRE RISK AS TO
   RESULTS AND PERFORMANCE OF THE SOFTWARE IS ASSUMED BY YOU. UNDER NO
   CIRCUMSTANCES WILL THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT,
   INCIDENTAL, EXEMPLARY OR CONSEQUENTIAL DAMAGES OF ANY KIND OR NATURE
   WHATSOEVER, WHETHER BASED ON CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE),
   STRICT LIABILITY OR OTHERWISE, ARISING OUT OF OR IN ANY WAY RELATED TO THE
   SOFTWARE, EVEN IF THE AUTHORS HAVE BEEN ADVISED ON THE POSSIBILITY OF SUCH
   DAMAGE OR IF SUCH DAMAGE COULD HAVE BEEN REASONABLY FORESEEN, AND
   NOTWITHSTANDING ANY FAILURE OF ESSENTIAL PURPOSE OF ANY EXCLUSIVE REMEDY
   PROVIDED. SUCH LIMITATION ON DAMAGES INCLUDES, BUT IS NOT LIMITED TO, DAMAGES
   FOR LOSS OF GOODWILL, LOST PROFITS, LOSS OF DATA OR SOFTWARE, WORK STOPPAGE,
   COMPUTER FAILURE OR MALFUNCTION OR IMPAIRMENT OF OTHER GOODS. IN NO EVENT
   WILL THE AUTHORS BE LIABLE FOR THE COSTS OF PROCUREMENT OF SUBSTITUTE
   SOFTWARE OR SERVICES. YOU ACKNOWLEDGE THAT THIS SOFTWARE IS NOT DESIGNED FOR
   USE IN ON-LINE EQUIPMENT IN HAZARDOUS ENVIRONMENTS SUCH AS OPERATION OF
   NUCLEAR FACILITIES, AIRCRAFT NAVIGATION OR CONTROL, OR LIFE-CRITICAL
   APPLICATIONS. THE AUTHORS EXPRESSLY DISCLAIM ANY LIABILITY RESULTING FROM USE
   OF THE SOFTWARE IN ANY SUCH ON-LINE EQUIPMENT IN HAZARDOUS ENVIRONMENTS AND
   ACCEPTS NO LIABILITY IN RESPECT OF ANY ACTIONS OR CLAIMS BASED ON THE USE OF
   THE SOFTWARE IN ANY SUCH ON-LINE EQUIPMENT IN HAZARDOUS ENVIRONMENTS BY YOU.
   FOR PURPOSES OF THIS PARAGRAPH, THE TERM "LIFE-CRITICAL APPLICATION" MEANS AN
   APPLICATION IN WHICH THE FUNCTIONING OR MALFUNCTIONING OF THE SOFTWARE MAY
   RESULT DIRECTLY OR INDIRECTLY IN PHYSICAL INJURY OR LOSS OF HUMAN LIFE. THIS
   DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF THIS LICENSE. NO USE
   OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER THIS DISCLAIMER.

   (( 8 ))??
   TERMINATION.

      (( 8.1. ))??
      This License and the rights granted hereunder will terminate automatically
      if You fail to comply with terms herein and fail to cure such breach
      within 30 days of becoming aware of the breach. All sublicenses to the
      Covered Code which are properly granted shall survive any termination of
      this License. Provisions which, by their nature, must remain in effect
      beyond the termination of this License shall survive.

      (( 8.2. ))??

      (( 8.3. ))??
      If You assert a patent infringement claim against Participant alleging
      that such Participant's Contributor Version directly or indirectly
      infringes any patent where such claim is resolved (such as by license or
      settlement) prior to the initiation of patent infringement litigation,
      then the reasonable value of the licenses granted by such Participant
      under Sections 2.1 or 2.2 shall be taken into account in determining the
      amount or value of any payment or license.

      (( 8.4. ))??
      In the event of termination under Sections 8.1 or 8.2 above, all end user
      license agreements (excluding distributors and resellers) which have been
      validly granted by You or any distributor hereunder prior to termination
      shall survive termination.

   (( 9 ))??
   LIMITATION OF LIABILITY.

   UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY, WHETHER TORT (INCLUDING
   NEGLIGENCE), CONTRACT, OR OTHERWISE, SHALL YOU, THE INITIAL DEVELOPER, ANY
   OTHER CONTRIBUTOR, OR ANY DISTRIBUTOR OF COVERED CODE, OR ANY SUPPLIER OF ANY
   OF SUCH PARTIES, BE LIABLE TO ANY PERSON FOR ANY INDIRECT, SPECIAL,
   INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER INCLUDING, WITHOUT
   LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE, COMPUTER FAILURE OR
   MALFUNCTION, OR ANY AND ALL OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF SUCH
   PARTY SHALL HAVE BEEN INFORMED OF THE POSSIBILITY OF SUCH DAMAGES. THIS
   LIMITATION OF LIABILITY SHALL NOT APPLY TO LIABILITY FOR DEATH OR PERSONAL
   INJURY RESULTING FROM SUCH PARTY'S NEGLIGENCE TO THE EXTENT APPLICABLE LAW
   PROHIBITS SUCH LIMITATION. SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OR
   LIMITATION OF INCIDENTAL OR CONSEQUENTIAL DAMAGES, SO THIS EXCLUSION AND
   LIMITATION MAY NOT APPLY TO YOU.

   (( 10 ))??
   U.S. GOVERNMENT END USERS.

   (( 11 ))??
   MISCELLANEOUS.

   (( 12 ))??
   RESPONSIBILITY FOR CLAIMS.

   As between Initial Developer and the Contributors, each party is responsible
   for claims and damages arising, directly or indirectly, out of its
   utilization of rights under this License and You agree to work with Initial
   Developer and Contributors to distribute such responsibility on an equitable
   basis. Nothing herein is intended or shall be deemed to constitute any
   admission of liability.
   ((

   EXHIBIT A.

   "The contents of this file are subject to the gSOAP Public License Version
   1.3 (the "License"); you may not use this file except in compliance with the
   License. You may obtain a copy of the License at

   http:/www.cs.fsu.edu/ engelen/soaplicense.html

   Software distributed under the License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   The Original Code of the gSOAP Software is: stdsoap.h, stdsoap2.h, stdsoap.c,
   stdsoap2.c, stdsoap.cpp, stdsoap2.cpp, soapcpp2.h, soapcpp2.c,
   soapcpp2_lex.l, soapcpp2_yacc.y, error2.h, error2.c, symbol2.c, init2.c,
   soapdoc2.html, and soapdoc2.pdf, httpget.h, httpget.c, stl.h, stldeque.h,
   stllist.h, stlvector.h, stlset.h.

   The Initial Developer of the Original Code is Robert A. van Engelen. Portions
   created by Robert A. van Engelen are Copyright (C) 2001-2004 Robert A. van
   Engelen, Genivia inc. All Rights Reserved.

   Contributor(s):

   "_."

   [Note: The text of this Exhibit A may differ slightly form the text of the
   notices in the Source Code files of the Original code. You should use the
   text of this Exhibit A rather than the text found in the Original Code Source
   Code for Your Modifications.]

   EXHIBIT B.

   "Part of the software embedded in this product is gSOAP software.

   Portions created by gSOAP are Copyright (C) 2001-2009 Robert A. van Engelen,
   Genivia inc. All Rights Reserved.

   THE SOFTWARE IN THIS PRODUCT WAS IN PART PROVIDED BY GENIVIA INC AND ANY
   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
   OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
   EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE." ))??
`
const license_gnuplot_lre = `//**
gnuplot License
https://spdx.org/licenses/gnuplot.json
https://fedoraproject.org/wiki/Licensing/Gnuplot
**//

//** Copyright **//

Permission to use, copy, and distribute this software and its documentation for
any purpose with or without fee is hereby granted, provided that the above
copyright notice appear in all copies and that both that copyright notice and
this permission notice appear in supporting documentation.

Permission to modify the software is granted, but not the right to distribute
the complete modified source code. Modifications are to be distributed as
patches to the released version. Permission to distribute binaries produced by
compiling modified sources is granted, provided you

   (( 1. ))??
   distribute the corresponding source modifications from the released version
   in the form of a patch file along with the binaries,

   (( 2. ))??
   add special version identification to distinguish your version in addition to
   the base release version number,

   (( 3. ))??
   provide your name and address as the primary contact for the support of your
   modified version, and

   (( 4. ))??
   retain our contact information in regard to use of the base software.

Permission to distribute the released version of the source code along with
corresponding source modifications in the form of a patch file is granted with
same provisions 2 through 4 for binary distributions.

This software is provided "as is" without express or implied warranty to the
extent permitted by applicable law.
`
const license_iMatix_lre = `//**
iMatix Standard Function Library Agreement
https://spdx.org/licenses/iMatix.json
http://legacy.imatix.com/html/sfl/sfl4.htm#license
**//

(( The SFL License Agreement ))??

This license agreement covers your use of the iMatix STANDARD FUNCTION LIBRARY
(SFL), its source code, documentation, and executable files, hereinafter
referred to as "the Product".

The Product is Copyright (c) 1991-2000 iMatix Corporation. You may use it and
distribute it according to this following License Agreement. If you do not agree
with these terms, please remove the Product from your system. By incorporating
the Product in your work or distributing the Product to others you implicitly
agree to these license terms.

Statement Of Copyright

The Product is, and remains, Copyright © 1991-2000 iMatix Corporation, with
exception of specific copyrights as noted in the individual source files.

Conditions Of Use

   You do not need to provide the source code for the Product as part of your
   product. However, you must do one of these things to comply with the Product
   License Agreement:

      (( 1. ))??
      Provide the source code for Product modules that you use, or

      (( 2. ))??
      Make your product freely available according to a license similar to the
      GNU General Public License, or the Perl Artistic License, or

      (( 3. ))??
      Add this phrase to the documentation for your product: "This product uses
      parts of the iMatix SFL, Copyright © 1991-2000 iMatix Corporation
      <http:/www.imatix.com>".

Rights Of Usage

   You may freely and at no cost use the Product in any project, commercial,
   academic, military, or private, so long as you respect the License Agreement.
   The License Agreement does not affect any software except the Product. In
   particular, any application that uses the Product does not itself fall under
   the License Agreement.

   You may modify any part of the Product, including sources and documentation,
   except this License Agreement, which you may not modify.

   You must clearly indicate any modifications at the start of each source file.
   The user of any modified Product code must know that the source file is not
   original.

   At your discretion, you may rewrite or reuse any part of the Product so that
   your derived code is not obviously part of the Product. This derived code
   does not fall under the Product License Agreement directly, but you must
   include a credit at the start of each source file indicating the original
   authorship and source of the code, and a statement of copyright as follows:

   "Parts copyright (c) 1991-2000 iMatix Corporation."

Rights Of Distribution

   You may freely distribute the Product, or any subset of the Product, by any
   means. The License, in the form of the file called "LICENSE.TXT" must
   accompany any such distribution.

   You may charge a fee for distributing the Product, for providing a warranty
   on the Product, for making modifications to the Product, or for any other
   service provided in relation to the Product. You are not required to ask our
   permission for any of these activities.

   At no time will iMatix associate itself with any distribution of the Product
   except that supplied from the Internet site http:/www.imatix.com.

Disclaimer Of Warranty

   The Product is provided as free software, in the hope that it will be useful.
   It is provided "as-is", without warranty of any kind, either expressed or
   implied, including, but not limited to, the implied warranties of
   merchantability and fitness for a particular purpose. The entire risk as to
   the quality and performance of the Product is with you. Should the Product
   prove defective, the full cost of repair, servicing, or correction lies with
   you.
`
const license_libpng_2_0_lre = `//**
PNG Reference Library version 2
https://spdx.org/licenses/libpng-2.0.json
http://www.libpng.org/pub/png/src/libpng-LICENSE.txt
**//

(( PNG Reference Library License version 2
(( Copyright __50__ ))??
))??

The software is supplied "as is", without warranty of any kind, express or
implied, including, without limitation, the warranties of merchantability,
fitness for a particular purpose, title, and non-infringement. In no even shall
the Copyright owners, or anyone distributing the software, be liable for any
damages or other liability, whether in contract, tort or otherwise, arising
from, out of, or in connection with the software, or the use or other dealings
in the software, even if advised of the possibility of such damage.

Permission is hereby granted to use, copy, modify, and distribute this software,
or portions hereof, for any purpose, without fee, subject to the following
restrictions:

   (( 1. ))??
   The origin of this software must not be misrepresented; you must not claim
   that you wrote the original software. If you use this software in a product,
   an acknowledgment in the product documentation would be appreciated, but is
   not required.

   (( 2. ))??
   Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   (( 3. ))??
   This Copyright notice may not be removed or altered from any source or
   altered source distribution.
`
const license_libselinux_1_0_lre = `//**
libselinux public domain notice
https://spdx.org/licenses/libselinux-1.0.json
https://github.com/SELinuxProject/selinux/blob/master/libselinux/LICENSE
**//

This library (libselinux) is public domain software, i.e. not copyrighted.

Warranty Exclusion

You agree that this software is a non-commercially developed program that may
contain "bugs" (as that term is used in the industry) and that it may not
function as intended. The software is licensed "as is". NSA makes no, and hereby
expressly disclaims all, warranties, express, implied, statutory, or otherwise
with respect to the software, including noninfringement and the implied
warranties of merchantability and fitness for a particular purpose.

Limitation of Liability

In no event will NSA be liable for any damages, including loss of data, lost
profits, cost of cover, or other special, incidental, consequential, direct or
indirect damages arising from the software or the use thereof, however caused
and on any theory of liability. This limitation will apply even if NSA has been
advised of the possibility of such damage. You acknowledge that this is a
reasonable allocation of risk.
`
const license_libtiff_lre = `//**
libtiff License
https://spdx.org/licenses/libtiff.json
https://fedoraproject.org/wiki/Licensing/libtiff
**//

//** Copyright **//

Permission to use, copy, modify, distribute, and sell this software and its
documentation for any purpose is hereby granted without fee, provided that (i)
the above copyright notices and this permission notice appear in all copies of
the software and related documentation, and (ii) the names of Sam Leffler and
Silicon Graphics may not be used in any advertising or publicity relating to the
software without the specific, prior written permission of Sam Leffler and
Silicon Graphics.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL SAM LEFFLER OR SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL,
INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED
OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
`
const license_mpich2_lre = `//**
mpich2 License
https://spdx.org/licenses/mpich2.json
https://fedoraproject.org/wiki/Licensing/MIT
**//

//** Copyright **//

The following is a notice of limited availability of the code, and disclaimer
which must be included in the prologue of the code and in all source listings of
the code.

Copyright Notice

+ 2002 University of Chicago

Permission is hereby granted to use, reproduce, prepare derivative works, and to
redistribute to others. This software was authored by:

Argonne National Laboratory Group W. Gropp: (630) 252-4318; FAX: (630) 252-5986;
e-mail: gropp@mcs.anl.gov E. Lusk: (630) 252-7852; FAX: (630) 252-5986; e-mail:
lusk@mcs.anl.gov Mathematics and Computer Science Division Argonne National
Laboratory, Argonne IL 60439

GOVERNMENT LICENSE

Portions of this material resulted from work developed under a U.S. Government
Contract and are subject to the following license: the Government is granted for
itself and others acting on its behalf a paid-up, nonexclusive, irrevocable
worldwide license in this computer software to reproduce, prepare derivative
works, and perform publicly and display publicly.

DISCLAIMER

This computer code material was prepared, in part, as an account of work
sponsored by an agency of the United States Government. Neither the United
States, nor the University of Chicago, nor any of their employees, makes any
warranty express or implied, or assumes any legal liability or responsibility
for the accuracy, completeness, or usefulness of any information, apparatus,
product, or process disclosed, or represents that its use would not infringe
privately owned rights.
`
const license_psfrag_lre = `//**
psfrag License
https://spdx.org/licenses/psfrag.json
https://fedoraproject.org/wiki/Licensing/psfrag
**//

(( psfrag.dtx
(( Copyright __20__ ))??
))??

This system is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. Don't come complaining to us if you modify this file and it
doesn't work! If this file is modified by anyone but the authors, those changes
and their authors must be explicitly stated HERE.
`
const license_psutils_lre = `//**
psutils License
https://spdx.org/licenses/psutils.json
https://fedoraproject.org/wiki/Licensing/psutils
**//

(( PS Utilities Package
(( The constituent files of this package listed below are Copyright __120__ ))??
))??

They may be copied and used for any purpose (including distribution as part of a
for-profit product), provided:

   (( 1) ))??
   The original attribution of the programs is clearly displayed in the product
   and/or documentation, even if the programs are modified and/or renamed as
   part of the product.

   (( 2) ))??
   The original source code of the programs is provided free of charge (except
   for reasonable distribution costs). For a definition of reasonable
   distribution costs, see the Gnu General Public License or Larry Wall's
   Artistic License (provided with the Perl 4 kit). The GPL and Artistic License
   in NO WAY affect this license; they are merely used as examples of the spirit
   in which it is intended.

   (( 3) ))??
   These programs are provided "as-is". No warranty or guarantee of their
   fitness for any particular task is provided. Use of these programs is
   completely at your own risk.

Basically, I don't mind how you use the programs so long as you acknowledge the
author, and give people the originals if they want them.
`
const license_xinetd_lre = `//**
xinetd License
https://spdx.org/licenses/xinetd.json
https://fedoraproject.org/wiki/Licensing/Xinetd_License
**//

(( ORIGINAL LICENSE: This software is
(( Copyright __20__ ))??
))??

The author (Panagiotis Tsirigotis) grants permission to use, copy, and
distribute this software and its documentation for any purpose and without fee,
provided that the above copyright notice extant in files in this distribution is
not removed from files included in any redistribution and that this copyright
notice is also included in any redistribution.

Modifications to this software may be distributed, either by distributing the
modified software or by distributing patches to the original software, under the
following additional terms:

   (( 1. ))??
   The version number will be modified as follows:

      (( a. ))??
      The first 3 components of the version number (i.e
      <number>.<number>.<number>) will remain unchanged.

      (( b. ))??
      A new component will be appended to the version number to indicate the
      modification level. The form of this component is up to the author of the
      modifications.

   (( 2. ))??
   The author of the modifications will include his/her name by appending it
   along with the new version number to this file and will be responsible for
   any wrong behavior of the modified software.

The author makes no representations about the suitability of this software for
any purpose. It is provided "as is" without any express or implied warranty.

Modifications: Version: 2.1.8.7-current Copyright 1998-2001 by Rob Braun

Sensor Addition Version: 2.1.8.9pre14a Copyright 2001 by Steve Grubb

This is an exerpt from an email I recieved from the original author, allowing
xinetd as maintained by me (Rob Braun), to use the higher version numbers:

I appreciate your maintaining the version string guidelines as specified in the
copyright. But I did not mean them to last as long as they did.

So, if you want, you may use any 2.N.* (N >= 3) version string for future xinetd
versions that you release. Note that I am excluding the 2.2.* line; using that
would only create confusion. Naming the next release 2.3.0 would put to rest the
confusion about 2.2.1 and 2.1.8.*.
`
const license_xpp_lre = `//**
XPP License
https://spdx.org/licenses/xpp.json
https://fedoraproject.org/wiki/Licensing/xpp
**//

(( LICENSE FOR THE Extreme! Lab PullParser
(( Copyright __20__ ))??
))??

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   (( 1) ))??
   All redistributions of source code must retain the above copyright notice,
   the list of authors in the original source code, this list of conditions and
   the disclaimer listed in this license;

   (( 2) ))??
   All redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the disclaimer listed in this license in the
   documentation and/or other materials provided with the distribution;

   (( 3) ))??
   Any documentation included with all redistributions must include the
   following acknowledgement:

   "This product includes software developed by the Indiana University Extreme!
   Lab. For further information please visit http:/www.extreme.indiana.edu/"

   Alternatively, this acknowledgment may appear in the software itself, and
   wherever such third-party acknowledgments normally appear.

   (( 4) ))??
   The name "Indiana Univeristy" and "Indiana Univeristy Extreme! Lab" shall not
   be used to endorse or promote products derived from this software without
   prior written permission from Indiana University. For written permission,
   please contact http:/www.extreme.indiana.edu/.

   (( 5) ))??
   Products derived from this software may not use "Indiana Univeristy" name nor
   may "Indiana Univeristy" appear in their name, without prior written
   permission of the Indiana University. Indiana University provides no
   reassurances that the source code provided does not infringe the patent or
   any other intellectual property rights of any other entity. Indiana
   University disclaims any liability to any recipient for claims brought by any
   other entity based on infringement of intellectual property rights or
   otherwise.

LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO WARRANTIES
AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES NO WARRANTIES
AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY
PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS. INDIANA UNIVERSITY MAKES NO
WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
DOORS", "WORMS", OR OTHER HARMFUL CODE. LICENSEE ASSUMES THE ENTIRE RISK AS TO
THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS, AND TO THE PERFORMANCE
AND VALIDITY OF INFORMATION GENERATED USING SOFTWARE.
`
const license_zlib_acknowledgement_lre = `//**
zlib/libpng License with Acknowledgement
https://spdx.org/licenses/zlib-acknowledgement.json
https://fedoraproject.org/wiki/Licensing/ZlibWithAcknowledgement
**//

//** Copyright **//

This software is provided 'as-is', without any express or implied warranty. In
no event will the authors be held liable for any damages arising from the use of
this software.

Permission is granted to anyone to use this software for any purpose, including
commercial applications, and to alter it and redistribute it freely, subject to
the following restrictions:

   (( 1. ))??
   The origin of this software must not be misrepresented; you must not claim
   that you wrote the original software. If you use this software in a product,
   an acknowledgment (see the following) in the product documentation is
   required.

   Portions Copyright (c) 2002-2007 Charlie Poole or Copyright (c) 2002-2004
   James W. Newkirk, Michael C. Two, Alexei A. Vorontsov or Copyright (c)
   2000-2002 Philip A. Craig

   (( 2. ))??
   Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   (( 3. ))??
   This notice may not be removed or altered from any source distribution.
`
